{{! 
    Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
    See https://llvm.org/LICENSE.txt for license information.
    SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
    
    This file defines the template for enums
}}
<div id="{{USR}}" class="delimiter-container">
    <div>
        <pre><code class="language-cpp code-clang-doc">enum {{Name}}</code></pre>
    </div>
    {{! Enum Values }}
    <table class="table-wrapper">
        <tbody>
            <tr>
                <th>Name</th>
                <th>Value</th>
            </tr>
            {{#Members}}
            <tr>
                <td>{{Name}}</td>
                {{! A ValueExpr is an explicitly assigned enum value }}
                {{#Value}}
                <td>{{Value}}</td>
                {{/Value}}
                {{^Value}}
                <td>{{ValueExpr}}</td>
                {{/Value}}
            </tr>
            {{/Members}}
        </tbody>
    </table>
    {{#Description}}
    <div class="doc-card">
        {{>Comments}}
    </div>
    {{/Description}}
    {{#Location}}
    <p>Defined at line {{LineNumber}} of file {{^FileURL}}{{Filename}}{{/FileURL}}{{#FileURL}}<a href="{{FileURL}}">{{Filename}}</a>{{/FileURL}}</p>
    {{/Location}}
</div>
