<nav class="navbar">
    <div class="navbar__container">
        {{#ProjectName}}
        <div class="navbar__logo">
            {{ProjectName}}
        </div>
        {{/ProjectName}}
        <div class="navbar__menu">
            <ul class="navbar__links">
                <li class="navbar__item">
                    <a href="{{Homepage}}" class="navbar__link">Home</a>
                </li>
            </ul>
        </div>
    </div>
    {{#HasContexts}}
    <div class="navbar-breadcrumb-container">
        {{#Contexts}}
        <a href="{{RelativePath}}{{DocumentationFileName}}.html"><div class="navbar-breadcrumb-item">{{Name}}</div></a>{{^End}}::{{/End}}
        {{/Contexts}}
    </div>
    {{/HasContexts}}
</nav>
