// RISCV Vector builtin enumerators
#ifdef GET_RISCVV_BUILTIN_ENUMERATORS
  BI__builtin_rvv_nds_vfpmadt_vf,
  BI__builtin_rvv_nds_vfpmadt_vf_tu,
  BI__builtin_rvv_nds_vfpmadt_vf_m,
  BI__builtin_rvv_nds_vfpmadt_vf_tum,
  BI__builtin_rvv_nds_vfpmadt_vf_tumu,
  BI__builtin_rvv_nds_vfpmadt_vf_mu,
  BI__builtin_rvv_nds_vln8_v,
  BI__builtin_rvv_nds_vln8_v_tu,
  BI__builtin_rvv_nds_vln8_v_m,
  BI__builtin_rvv_nds_vln8_v_tum,
  BI__builtin_rvv_nds_vln8_v_tumu,
  BI__builtin_rvv_nds_vln8_v_mu,
  BI__builtin_rvv_nds_vlnu8_v,
  BI__builtin_rvv_nds_vlnu8_v_tu,
  BI__builtin_rvv_nds_vlnu8_v_m,
  BI__builtin_rvv_nds_vlnu8_v_tum,
  BI__builtin_rvv_nds_vlnu8_v_tumu,
  BI__builtin_rvv_nds_vlnu8_v_mu,
  BI__builtin_rvv_nds_vfpmadt_vf_rm,
  BI__builtin_rvv_nds_vfpmadt_vf_rm_tu,
  BI__builtin_rvv_nds_vfpmadt_vf_rm_m,
  BI__builtin_rvv_nds_vfpmadt_vf_rm_tum,
  BI__builtin_rvv_nds_vfpmadt_vf_rm_tumu,
  BI__builtin_rvv_nds_vfpmadt_vf_rm_mu,
  BI__builtin_rvv_nds_vfpmadb_vf,
  BI__builtin_rvv_nds_vfpmadb_vf_tu,
  BI__builtin_rvv_nds_vfpmadb_vf_m,
  BI__builtin_rvv_nds_vfpmadb_vf_tum,
  BI__builtin_rvv_nds_vfpmadb_vf_tumu,
  BI__builtin_rvv_nds_vfpmadb_vf_mu,
  BI__builtin_rvv_nds_vfpmadb_vf_rm,
  BI__builtin_rvv_nds_vfpmadb_vf_rm_tu,
  BI__builtin_rvv_nds_vfpmadb_vf_rm_m,
  BI__builtin_rvv_nds_vfpmadb_vf_rm_tum,
  BI__builtin_rvv_nds_vfpmadb_vf_rm_tumu,
  BI__builtin_rvv_nds_vfpmadb_vf_rm_mu,
  BI__builtin_rvv_nds_vd4dots_vv,
  BI__builtin_rvv_nds_vd4dots_vv_tu,
  BI__builtin_rvv_nds_vd4dots_vv_m,
  BI__builtin_rvv_nds_vd4dots_vv_tum,
  BI__builtin_rvv_nds_vd4dots_vv_tumu,
  BI__builtin_rvv_nds_vd4dots_vv_mu,
  BI__builtin_rvv_nds_vd4dotu_vv,
  BI__builtin_rvv_nds_vd4dotu_vv_tu,
  BI__builtin_rvv_nds_vd4dotu_vv_m,
  BI__builtin_rvv_nds_vd4dotu_vv_tum,
  BI__builtin_rvv_nds_vd4dotu_vv_tumu,
  BI__builtin_rvv_nds_vd4dotu_vv_mu,
  BI__builtin_rvv_nds_vd4dotsu_vv,
  BI__builtin_rvv_nds_vd4dotsu_vv_tu,
  BI__builtin_rvv_nds_vd4dotsu_vv_m,
  BI__builtin_rvv_nds_vd4dotsu_vv_tum,
  BI__builtin_rvv_nds_vd4dotsu_vv_tumu,
  BI__builtin_rvv_nds_vd4dotsu_vv_mu,
  BI__builtin_rvv_nds_vfncvt_bf16_s,
  BI__builtin_rvv_nds_vfncvt_bf16_s_tu,
  BI__builtin_rvv_nds_vfncvt_bf16_s_rm,
  BI__builtin_rvv_nds_vfncvt_bf16_s_rm_tu,
  BI__builtin_rvv_nds_vfwcvt_s_bf16,
  BI__builtin_rvv_nds_vfwcvt_s_bf16_tu,
#endif // GET_RISCVV_BUILTIN_ENUMERATORS

// RISCV Vector builtin enumerators
#ifdef GET_RISCVV_BUILTIN_STR_TABLE

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinStringsStorage[] =
  "\0"
  "n\0"
  "zve32x\0"
  "nds_vfpmadt_vf\0"
  "nds_vfpmadt_vf_tu\0"
  "nds_vfpmadt_vf_m\0"
  "nds_vfpmadt_vf_tum\0"
  "nds_vfpmadt_vf_tumu\0"
  "nds_vfpmadt_vf_mu\0"
  "nds_vln8_v\0"
  "nds_vln8_v_tu\0"
  "nds_vln8_v_m\0"
  "nds_vln8_v_tum\0"
  "nds_vln8_v_tumu\0"
  "nds_vln8_v_mu\0"
  "nds_vlnu8_v\0"
  "nds_vlnu8_v_tu\0"
  "nds_vlnu8_v_m\0"
  "nds_vlnu8_v_tum\0"
  "nds_vlnu8_v_tumu\0"
  "nds_vlnu8_v_mu\0"
  "nds_vfpmadt_vf_rm\0"
  "nds_vfpmadt_vf_rm_tu\0"
  "nds_vfpmadt_vf_rm_m\0"
  "nds_vfpmadt_vf_rm_tum\0"
  "nds_vfpmadt_vf_rm_tumu\0"
  "nds_vfpmadt_vf_rm_mu\0"
  "nds_vfpmadb_vf\0"
  "nds_vfpmadb_vf_tu\0"
  "nds_vfpmadb_vf_m\0"
  "nds_vfpmadb_vf_tum\0"
  "nds_vfpmadb_vf_tumu\0"
  "nds_vfpmadb_vf_mu\0"
  "nds_vfpmadb_vf_rm\0"
  "nds_vfpmadb_vf_rm_tu\0"
  "nds_vfpmadb_vf_rm_m\0"
  "nds_vfpmadb_vf_rm_tum\0"
  "nds_vfpmadb_vf_rm_tumu\0"
  "nds_vfpmadb_vf_rm_mu\0"
  "nds_vd4dots_vv\0"
  "nds_vd4dots_vv_tu\0"
  "nds_vd4dots_vv_m\0"
  "nds_vd4dots_vv_tum\0"
  "nds_vd4dots_vv_tumu\0"
  "nds_vd4dots_vv_mu\0"
  "nds_vd4dotu_vv\0"
  "nds_vd4dotu_vv_tu\0"
  "nds_vd4dotu_vv_m\0"
  "nds_vd4dotu_vv_tum\0"
  "nds_vd4dotu_vv_tumu\0"
  "nds_vd4dotu_vv_mu\0"
  "nds_vd4dotsu_vv\0"
  "nds_vd4dotsu_vv_tu\0"
  "nds_vd4dotsu_vv_m\0"
  "nds_vd4dotsu_vv_tum\0"
  "nds_vd4dotsu_vv_tumu\0"
  "nds_vd4dotsu_vv_mu\0"
  "nds_vfncvt_bf16_s\0"
  "nds_vfncvt_bf16_s_tu\0"
  "nds_vfncvt_bf16_s_rm\0"
  "nds_vfncvt_bf16_s_rm_tu\0"
  "nds_vfwcvt_s_bf16\0"
  "nds_vfwcvt_s_bf16_tu\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinStrings = BuiltinStringsStorage;
#endif // GET_RISCVV_BUILTIN_STR_TABLE

// RISCV Vector builtin infos
#ifdef GET_RISCVV_BUILTIN_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{10 /* nds_vfpmadt_vf */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25 /* nds_vfpmadt_vf_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43 /* nds_vfpmadt_vf_m */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60 /* nds_vfpmadt_vf_tum */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79 /* nds_vfpmadt_vf_tumu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99 /* nds_vfpmadt_vf_mu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117 /* nds_vln8_v */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128 /* nds_vln8_v_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142 /* nds_vln8_v_m */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155 /* nds_vln8_v_tum */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{170 /* nds_vln8_v_tumu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{186 /* nds_vln8_v_mu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{200 /* nds_vlnu8_v */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{212 /* nds_vlnu8_v_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{227 /* nds_vlnu8_v_m */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{241 /* nds_vlnu8_v_tum */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{257 /* nds_vlnu8_v_tumu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{274 /* nds_vlnu8_v_mu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{289 /* nds_vfpmadt_vf_rm */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{307 /* nds_vfpmadt_vf_rm_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{328 /* nds_vfpmadt_vf_rm_m */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{348 /* nds_vfpmadt_vf_rm_tum */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{370 /* nds_vfpmadt_vf_rm_tumu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{393 /* nds_vfpmadt_vf_rm_mu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{414 /* nds_vfpmadb_vf */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{429 /* nds_vfpmadb_vf_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{447 /* nds_vfpmadb_vf_m */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{464 /* nds_vfpmadb_vf_tum */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{483 /* nds_vfpmadb_vf_tumu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{503 /* nds_vfpmadb_vf_mu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{521 /* nds_vfpmadb_vf_rm */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{539 /* nds_vfpmadb_vf_rm_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{560 /* nds_vfpmadb_vf_rm_m */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{580 /* nds_vfpmadb_vf_rm_tum */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{602 /* nds_vfpmadb_vf_rm_tumu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{625 /* nds_vfpmadb_vf_rm_mu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{646 /* nds_vd4dots_vv */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{661 /* nds_vd4dots_vv_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{679 /* nds_vd4dots_vv_m */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{696 /* nds_vd4dots_vv_tum */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{715 /* nds_vd4dots_vv_tumu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{735 /* nds_vd4dots_vv_mu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{753 /* nds_vd4dotu_vv */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{768 /* nds_vd4dotu_vv_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{786 /* nds_vd4dotu_vv_m */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{803 /* nds_vd4dotu_vv_tum */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{822 /* nds_vd4dotu_vv_tumu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{842 /* nds_vd4dotu_vv_mu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{860 /* nds_vd4dotsu_vv */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{876 /* nds_vd4dotsu_vv_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{895 /* nds_vd4dotsu_vv_m */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{913 /* nds_vd4dotsu_vv_tum */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{933 /* nds_vd4dotsu_vv_tumu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{954 /* nds_vd4dotsu_vv_mu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{973 /* nds_vfncvt_bf16_s */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{991 /* nds_vfncvt_bf16_s_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1012 /* nds_vfncvt_bf16_s_rm */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1033 /* nds_vfncvt_bf16_s_rm_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1057 /* nds_vfwcvt_s_bf16 */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1075 /* nds_vfwcvt_s_bf16_tu */, 0, 1 /* n */, 3 /* zve32x */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
#endif // GET_RISCVV_BUILTIN_INFOS

