#define CLANG_VERSION 22.1.6
#define CLANG_VERSION_STRING "22.1.6"
#define CLANG_VERSION_MAJOR 22
#define CLANG_VERSION_MAJOR_STRING "22"
#define CLANG_VERSION_MINOR 1
#define CLANG_VERSION_PATCHLEVEL 6
#define MAX_CLANG_ABI_COMPAT_VERSION 22
