#ifdef GET_SVE_BUILTIN_ENUMERATORS
  BI__builtin_sve_svaba_n_s16,
  BI__builtin_sve_svaba_n_s32,
  BI__builtin_sve_svaba_n_s64,
  BI__builtin_sve_svaba_n_s8,
  BI__builtin_sve_svaba_n_u16,
  BI__builtin_sve_svaba_n_u32,
  BI__builtin_sve_svaba_n_u64,
  BI__builtin_sve_svaba_n_u8,
  BI__builtin_sve_svaba_s16,
  BI__builtin_sve_svaba_s32,
  BI__builtin_sve_svaba_s64,
  BI__builtin_sve_svaba_s8,
  BI__builtin_sve_svaba_u16,
  BI__builtin_sve_svaba_u32,
  BI__builtin_sve_svaba_u64,
  BI__builtin_sve_svaba_u8,
  BI__builtin_sve_svabalb_n_s16,
  BI__builtin_sve_svabalb_n_s32,
  BI__builtin_sve_svabalb_n_s64,
  BI__builtin_sve_svabalb_n_u16,
  BI__builtin_sve_svabalb_n_u32,
  BI__builtin_sve_svabalb_n_u64,
  BI__builtin_sve_svabalb_s16,
  BI__builtin_sve_svabalb_s32,
  BI__builtin_sve_svabalb_s64,
  BI__builtin_sve_svabalb_u16,
  BI__builtin_sve_svabalb_u32,
  BI__builtin_sve_svabalb_u64,
  BI__builtin_sve_svabalt_n_s16,
  BI__builtin_sve_svabalt_n_s32,
  BI__builtin_sve_svabalt_n_s64,
  BI__builtin_sve_svabalt_n_u16,
  BI__builtin_sve_svabalt_n_u32,
  BI__builtin_sve_svabalt_n_u64,
  BI__builtin_sve_svabalt_s16,
  BI__builtin_sve_svabalt_s32,
  BI__builtin_sve_svabalt_s64,
  BI__builtin_sve_svabalt_u16,
  BI__builtin_sve_svabalt_u32,
  BI__builtin_sve_svabalt_u64,
  BI__builtin_sve_svabd_f16_m,
  BI__builtin_sve_svabd_f16_x,
  BI__builtin_sve_svabd_f16_z,
  BI__builtin_sve_svabd_f32_m,
  BI__builtin_sve_svabd_f32_x,
  BI__builtin_sve_svabd_f32_z,
  BI__builtin_sve_svabd_f64_m,
  BI__builtin_sve_svabd_f64_x,
  BI__builtin_sve_svabd_f64_z,
  BI__builtin_sve_svabd_n_f16_m,
  BI__builtin_sve_svabd_n_f16_x,
  BI__builtin_sve_svabd_n_f16_z,
  BI__builtin_sve_svabd_n_f32_m,
  BI__builtin_sve_svabd_n_f32_x,
  BI__builtin_sve_svabd_n_f32_z,
  BI__builtin_sve_svabd_n_f64_m,
  BI__builtin_sve_svabd_n_f64_x,
  BI__builtin_sve_svabd_n_f64_z,
  BI__builtin_sve_svabd_n_s16_m,
  BI__builtin_sve_svabd_n_s16_x,
  BI__builtin_sve_svabd_n_s16_z,
  BI__builtin_sve_svabd_n_s32_m,
  BI__builtin_sve_svabd_n_s32_x,
  BI__builtin_sve_svabd_n_s32_z,
  BI__builtin_sve_svabd_n_s64_m,
  BI__builtin_sve_svabd_n_s64_x,
  BI__builtin_sve_svabd_n_s64_z,
  BI__builtin_sve_svabd_n_s8_m,
  BI__builtin_sve_svabd_n_s8_x,
  BI__builtin_sve_svabd_n_s8_z,
  BI__builtin_sve_svabd_n_u16_m,
  BI__builtin_sve_svabd_n_u16_x,
  BI__builtin_sve_svabd_n_u16_z,
  BI__builtin_sve_svabd_n_u32_m,
  BI__builtin_sve_svabd_n_u32_x,
  BI__builtin_sve_svabd_n_u32_z,
  BI__builtin_sve_svabd_n_u64_m,
  BI__builtin_sve_svabd_n_u64_x,
  BI__builtin_sve_svabd_n_u64_z,
  BI__builtin_sve_svabd_n_u8_m,
  BI__builtin_sve_svabd_n_u8_x,
  BI__builtin_sve_svabd_n_u8_z,
  BI__builtin_sve_svabd_s16_m,
  BI__builtin_sve_svabd_s16_x,
  BI__builtin_sve_svabd_s16_z,
  BI__builtin_sve_svabd_s32_m,
  BI__builtin_sve_svabd_s32_x,
  BI__builtin_sve_svabd_s32_z,
  BI__builtin_sve_svabd_s64_m,
  BI__builtin_sve_svabd_s64_x,
  BI__builtin_sve_svabd_s64_z,
  BI__builtin_sve_svabd_s8_m,
  BI__builtin_sve_svabd_s8_x,
  BI__builtin_sve_svabd_s8_z,
  BI__builtin_sve_svabd_u16_m,
  BI__builtin_sve_svabd_u16_x,
  BI__builtin_sve_svabd_u16_z,
  BI__builtin_sve_svabd_u32_m,
  BI__builtin_sve_svabd_u32_x,
  BI__builtin_sve_svabd_u32_z,
  BI__builtin_sve_svabd_u64_m,
  BI__builtin_sve_svabd_u64_x,
  BI__builtin_sve_svabd_u64_z,
  BI__builtin_sve_svabd_u8_m,
  BI__builtin_sve_svabd_u8_x,
  BI__builtin_sve_svabd_u8_z,
  BI__builtin_sve_svabdlb_n_s16,
  BI__builtin_sve_svabdlb_n_s32,
  BI__builtin_sve_svabdlb_n_s64,
  BI__builtin_sve_svabdlb_n_u16,
  BI__builtin_sve_svabdlb_n_u32,
  BI__builtin_sve_svabdlb_n_u64,
  BI__builtin_sve_svabdlb_s16,
  BI__builtin_sve_svabdlb_s32,
  BI__builtin_sve_svabdlb_s64,
  BI__builtin_sve_svabdlb_u16,
  BI__builtin_sve_svabdlb_u32,
  BI__builtin_sve_svabdlb_u64,
  BI__builtin_sve_svabdlt_n_s16,
  BI__builtin_sve_svabdlt_n_s32,
  BI__builtin_sve_svabdlt_n_s64,
  BI__builtin_sve_svabdlt_n_u16,
  BI__builtin_sve_svabdlt_n_u32,
  BI__builtin_sve_svabdlt_n_u64,
  BI__builtin_sve_svabdlt_s16,
  BI__builtin_sve_svabdlt_s32,
  BI__builtin_sve_svabdlt_s64,
  BI__builtin_sve_svabdlt_u16,
  BI__builtin_sve_svabdlt_u32,
  BI__builtin_sve_svabdlt_u64,
  BI__builtin_sve_svabs_f16_m,
  BI__builtin_sve_svabs_f16_x,
  BI__builtin_sve_svabs_f16_z,
  BI__builtin_sve_svabs_f32_m,
  BI__builtin_sve_svabs_f32_x,
  BI__builtin_sve_svabs_f32_z,
  BI__builtin_sve_svabs_f64_m,
  BI__builtin_sve_svabs_f64_x,
  BI__builtin_sve_svabs_f64_z,
  BI__builtin_sve_svabs_s16_m,
  BI__builtin_sve_svabs_s16_x,
  BI__builtin_sve_svabs_s16_z,
  BI__builtin_sve_svabs_s32_m,
  BI__builtin_sve_svabs_s32_x,
  BI__builtin_sve_svabs_s32_z,
  BI__builtin_sve_svabs_s64_m,
  BI__builtin_sve_svabs_s64_x,
  BI__builtin_sve_svabs_s64_z,
  BI__builtin_sve_svabs_s8_m,
  BI__builtin_sve_svabs_s8_x,
  BI__builtin_sve_svabs_s8_z,
  BI__builtin_sve_svacge_f16,
  BI__builtin_sve_svacge_f32,
  BI__builtin_sve_svacge_f64,
  BI__builtin_sve_svacge_n_f16,
  BI__builtin_sve_svacge_n_f32,
  BI__builtin_sve_svacge_n_f64,
  BI__builtin_sve_svacgt_f16,
  BI__builtin_sve_svacgt_f32,
  BI__builtin_sve_svacgt_f64,
  BI__builtin_sve_svacgt_n_f16,
  BI__builtin_sve_svacgt_n_f32,
  BI__builtin_sve_svacgt_n_f64,
  BI__builtin_sve_svacle_f16,
  BI__builtin_sve_svacle_f32,
  BI__builtin_sve_svacle_f64,
  BI__builtin_sve_svacle_n_f16,
  BI__builtin_sve_svacle_n_f32,
  BI__builtin_sve_svacle_n_f64,
  BI__builtin_sve_svaclt_f16,
  BI__builtin_sve_svaclt_f32,
  BI__builtin_sve_svaclt_f64,
  BI__builtin_sve_svaclt_n_f16,
  BI__builtin_sve_svaclt_n_f32,
  BI__builtin_sve_svaclt_n_f64,
  BI__builtin_sve_svadalp_s16_m,
  BI__builtin_sve_svadalp_s16_x,
  BI__builtin_sve_svadalp_s16_z,
  BI__builtin_sve_svadalp_s32_m,
  BI__builtin_sve_svadalp_s32_x,
  BI__builtin_sve_svadalp_s32_z,
  BI__builtin_sve_svadalp_s64_m,
  BI__builtin_sve_svadalp_s64_x,
  BI__builtin_sve_svadalp_s64_z,
  BI__builtin_sve_svadalp_u16_m,
  BI__builtin_sve_svadalp_u16_x,
  BI__builtin_sve_svadalp_u16_z,
  BI__builtin_sve_svadalp_u32_m,
  BI__builtin_sve_svadalp_u32_x,
  BI__builtin_sve_svadalp_u32_z,
  BI__builtin_sve_svadalp_u64_m,
  BI__builtin_sve_svadalp_u64_x,
  BI__builtin_sve_svadalp_u64_z,
  BI__builtin_sve_svadclb_n_u32,
  BI__builtin_sve_svadclb_n_u64,
  BI__builtin_sve_svadclb_u32,
  BI__builtin_sve_svadclb_u64,
  BI__builtin_sve_svadclt_n_u32,
  BI__builtin_sve_svadclt_n_u64,
  BI__builtin_sve_svadclt_u32,
  BI__builtin_sve_svadclt_u64,
  BI__builtin_sve_svadd_bf16_m,
  BI__builtin_sve_svadd_bf16_x,
  BI__builtin_sve_svadd_bf16_z,
  BI__builtin_sve_svadd_f16_m,
  BI__builtin_sve_svadd_f16_x,
  BI__builtin_sve_svadd_f16_z,
  BI__builtin_sve_svadd_f32_m,
  BI__builtin_sve_svadd_f32_x,
  BI__builtin_sve_svadd_f32_z,
  BI__builtin_sve_svadd_f64_m,
  BI__builtin_sve_svadd_f64_x,
  BI__builtin_sve_svadd_f64_z,
  BI__builtin_sve_svadd_n_bf16_m,
  BI__builtin_sve_svadd_n_bf16_x,
  BI__builtin_sve_svadd_n_bf16_z,
  BI__builtin_sve_svadd_n_f16_m,
  BI__builtin_sve_svadd_n_f16_x,
  BI__builtin_sve_svadd_n_f16_z,
  BI__builtin_sve_svadd_n_f32_m,
  BI__builtin_sve_svadd_n_f32_x,
  BI__builtin_sve_svadd_n_f32_z,
  BI__builtin_sve_svadd_n_f64_m,
  BI__builtin_sve_svadd_n_f64_x,
  BI__builtin_sve_svadd_n_f64_z,
  BI__builtin_sve_svadd_n_s16_m,
  BI__builtin_sve_svadd_n_s16_x,
  BI__builtin_sve_svadd_n_s16_z,
  BI__builtin_sve_svadd_n_s32_m,
  BI__builtin_sve_svadd_n_s32_x,
  BI__builtin_sve_svadd_n_s32_z,
  BI__builtin_sve_svadd_n_s64_m,
  BI__builtin_sve_svadd_n_s64_x,
  BI__builtin_sve_svadd_n_s64_z,
  BI__builtin_sve_svadd_n_s8_m,
  BI__builtin_sve_svadd_n_s8_x,
  BI__builtin_sve_svadd_n_s8_z,
  BI__builtin_sve_svadd_n_u16_m,
  BI__builtin_sve_svadd_n_u16_x,
  BI__builtin_sve_svadd_n_u16_z,
  BI__builtin_sve_svadd_n_u32_m,
  BI__builtin_sve_svadd_n_u32_x,
  BI__builtin_sve_svadd_n_u32_z,
  BI__builtin_sve_svadd_n_u64_m,
  BI__builtin_sve_svadd_n_u64_x,
  BI__builtin_sve_svadd_n_u64_z,
  BI__builtin_sve_svadd_n_u8_m,
  BI__builtin_sve_svadd_n_u8_x,
  BI__builtin_sve_svadd_n_u8_z,
  BI__builtin_sve_svadd_s16_m,
  BI__builtin_sve_svadd_s16_x,
  BI__builtin_sve_svadd_s16_z,
  BI__builtin_sve_svadd_s32_m,
  BI__builtin_sve_svadd_s32_x,
  BI__builtin_sve_svadd_s32_z,
  BI__builtin_sve_svadd_s64_m,
  BI__builtin_sve_svadd_s64_x,
  BI__builtin_sve_svadd_s64_z,
  BI__builtin_sve_svadd_s8_m,
  BI__builtin_sve_svadd_s8_x,
  BI__builtin_sve_svadd_s8_z,
  BI__builtin_sve_svadd_single_s16_x2,
  BI__builtin_sve_svadd_single_s16_x4,
  BI__builtin_sve_svadd_single_s32_x2,
  BI__builtin_sve_svadd_single_s32_x4,
  BI__builtin_sve_svadd_single_s64_x2,
  BI__builtin_sve_svadd_single_s64_x4,
  BI__builtin_sve_svadd_single_s8_x2,
  BI__builtin_sve_svadd_single_s8_x4,
  BI__builtin_sve_svadd_single_u16_x2,
  BI__builtin_sve_svadd_single_u16_x4,
  BI__builtin_sve_svadd_single_u32_x2,
  BI__builtin_sve_svadd_single_u32_x4,
  BI__builtin_sve_svadd_single_u64_x2,
  BI__builtin_sve_svadd_single_u64_x4,
  BI__builtin_sve_svadd_single_u8_x2,
  BI__builtin_sve_svadd_single_u8_x4,
  BI__builtin_sve_svadd_u16_m,
  BI__builtin_sve_svadd_u16_x,
  BI__builtin_sve_svadd_u16_z,
  BI__builtin_sve_svadd_u32_m,
  BI__builtin_sve_svadd_u32_x,
  BI__builtin_sve_svadd_u32_z,
  BI__builtin_sve_svadd_u64_m,
  BI__builtin_sve_svadd_u64_x,
  BI__builtin_sve_svadd_u64_z,
  BI__builtin_sve_svadd_u8_m,
  BI__builtin_sve_svadd_u8_x,
  BI__builtin_sve_svadd_u8_z,
  BI__builtin_sve_svadda_f16,
  BI__builtin_sve_svadda_f32,
  BI__builtin_sve_svadda_f64,
  BI__builtin_sve_svaddhnb_n_s16,
  BI__builtin_sve_svaddhnb_n_s32,
  BI__builtin_sve_svaddhnb_n_s64,
  BI__builtin_sve_svaddhnb_n_u16,
  BI__builtin_sve_svaddhnb_n_u32,
  BI__builtin_sve_svaddhnb_n_u64,
  BI__builtin_sve_svaddhnb_s16,
  BI__builtin_sve_svaddhnb_s32,
  BI__builtin_sve_svaddhnb_s64,
  BI__builtin_sve_svaddhnb_u16,
  BI__builtin_sve_svaddhnb_u32,
  BI__builtin_sve_svaddhnb_u64,
  BI__builtin_sve_svaddhnt_n_s16,
  BI__builtin_sve_svaddhnt_n_s32,
  BI__builtin_sve_svaddhnt_n_s64,
  BI__builtin_sve_svaddhnt_n_u16,
  BI__builtin_sve_svaddhnt_n_u32,
  BI__builtin_sve_svaddhnt_n_u64,
  BI__builtin_sve_svaddhnt_s16,
  BI__builtin_sve_svaddhnt_s32,
  BI__builtin_sve_svaddhnt_s64,
  BI__builtin_sve_svaddhnt_u16,
  BI__builtin_sve_svaddhnt_u32,
  BI__builtin_sve_svaddhnt_u64,
  BI__builtin_sve_svaddlb_n_s16,
  BI__builtin_sve_svaddlb_n_s32,
  BI__builtin_sve_svaddlb_n_s64,
  BI__builtin_sve_svaddlb_n_u16,
  BI__builtin_sve_svaddlb_n_u32,
  BI__builtin_sve_svaddlb_n_u64,
  BI__builtin_sve_svaddlb_s16,
  BI__builtin_sve_svaddlb_s32,
  BI__builtin_sve_svaddlb_s64,
  BI__builtin_sve_svaddlb_u16,
  BI__builtin_sve_svaddlb_u32,
  BI__builtin_sve_svaddlb_u64,
  BI__builtin_sve_svaddlbt_n_s16,
  BI__builtin_sve_svaddlbt_n_s32,
  BI__builtin_sve_svaddlbt_n_s64,
  BI__builtin_sve_svaddlbt_s16,
  BI__builtin_sve_svaddlbt_s32,
  BI__builtin_sve_svaddlbt_s64,
  BI__builtin_sve_svaddlt_n_s16,
  BI__builtin_sve_svaddlt_n_s32,
  BI__builtin_sve_svaddlt_n_s64,
  BI__builtin_sve_svaddlt_n_u16,
  BI__builtin_sve_svaddlt_n_u32,
  BI__builtin_sve_svaddlt_n_u64,
  BI__builtin_sve_svaddlt_s16,
  BI__builtin_sve_svaddlt_s32,
  BI__builtin_sve_svaddlt_s64,
  BI__builtin_sve_svaddlt_u16,
  BI__builtin_sve_svaddlt_u32,
  BI__builtin_sve_svaddlt_u64,
  BI__builtin_sve_svaddp_f16_m,
  BI__builtin_sve_svaddp_f16_x,
  BI__builtin_sve_svaddp_f32_m,
  BI__builtin_sve_svaddp_f32_x,
  BI__builtin_sve_svaddp_f64_m,
  BI__builtin_sve_svaddp_f64_x,
  BI__builtin_sve_svaddp_s16_m,
  BI__builtin_sve_svaddp_s16_x,
  BI__builtin_sve_svaddp_s32_m,
  BI__builtin_sve_svaddp_s32_x,
  BI__builtin_sve_svaddp_s64_m,
  BI__builtin_sve_svaddp_s64_x,
  BI__builtin_sve_svaddp_s8_m,
  BI__builtin_sve_svaddp_s8_x,
  BI__builtin_sve_svaddp_u16_m,
  BI__builtin_sve_svaddp_u16_x,
  BI__builtin_sve_svaddp_u32_m,
  BI__builtin_sve_svaddp_u32_x,
  BI__builtin_sve_svaddp_u64_m,
  BI__builtin_sve_svaddp_u64_x,
  BI__builtin_sve_svaddp_u8_m,
  BI__builtin_sve_svaddp_u8_x,
  BI__builtin_sve_svaddqv_f16,
  BI__builtin_sve_svaddqv_f32,
  BI__builtin_sve_svaddqv_f64,
  BI__builtin_sve_svaddqv_s16,
  BI__builtin_sve_svaddqv_s32,
  BI__builtin_sve_svaddqv_s64,
  BI__builtin_sve_svaddqv_s8,
  BI__builtin_sve_svaddqv_u16,
  BI__builtin_sve_svaddqv_u32,
  BI__builtin_sve_svaddqv_u64,
  BI__builtin_sve_svaddqv_u8,
  BI__builtin_sve_svaddv_f16,
  BI__builtin_sve_svaddv_f32,
  BI__builtin_sve_svaddv_f64,
  BI__builtin_sve_svaddv_s16,
  BI__builtin_sve_svaddv_s32,
  BI__builtin_sve_svaddv_s64,
  BI__builtin_sve_svaddv_s8,
  BI__builtin_sve_svaddv_u16,
  BI__builtin_sve_svaddv_u32,
  BI__builtin_sve_svaddv_u64,
  BI__builtin_sve_svaddv_u8,
  BI__builtin_sve_svaddwb_n_s16,
  BI__builtin_sve_svaddwb_n_s32,
  BI__builtin_sve_svaddwb_n_s64,
  BI__builtin_sve_svaddwb_n_u16,
  BI__builtin_sve_svaddwb_n_u32,
  BI__builtin_sve_svaddwb_n_u64,
  BI__builtin_sve_svaddwb_s16,
  BI__builtin_sve_svaddwb_s32,
  BI__builtin_sve_svaddwb_s64,
  BI__builtin_sve_svaddwb_u16,
  BI__builtin_sve_svaddwb_u32,
  BI__builtin_sve_svaddwb_u64,
  BI__builtin_sve_svaddwt_n_s16,
  BI__builtin_sve_svaddwt_n_s32,
  BI__builtin_sve_svaddwt_n_s64,
  BI__builtin_sve_svaddwt_n_u16,
  BI__builtin_sve_svaddwt_n_u32,
  BI__builtin_sve_svaddwt_n_u64,
  BI__builtin_sve_svaddwt_s16,
  BI__builtin_sve_svaddwt_s32,
  BI__builtin_sve_svaddwt_s64,
  BI__builtin_sve_svaddwt_u16,
  BI__builtin_sve_svaddwt_u32,
  BI__builtin_sve_svaddwt_u64,
  BI__builtin_sve_svadrb_u32base_s32offset,
  BI__builtin_sve_svadrb_u32base_u32offset,
  BI__builtin_sve_svadrb_u64base_s64offset,
  BI__builtin_sve_svadrb_u64base_u64offset,
  BI__builtin_sve_svadrd_u32base_s32index,
  BI__builtin_sve_svadrd_u32base_u32index,
  BI__builtin_sve_svadrd_u64base_s64index,
  BI__builtin_sve_svadrd_u64base_u64index,
  BI__builtin_sve_svadrh_u32base_s32index,
  BI__builtin_sve_svadrh_u32base_u32index,
  BI__builtin_sve_svadrh_u64base_s64index,
  BI__builtin_sve_svadrh_u64base_u64index,
  BI__builtin_sve_svadrw_u32base_s32index,
  BI__builtin_sve_svadrw_u32base_u32index,
  BI__builtin_sve_svadrw_u64base_s64index,
  BI__builtin_sve_svadrw_u64base_u64index,
  BI__builtin_sve_svaesd_lane_u8_x2,
  BI__builtin_sve_svaesd_lane_u8_x4,
  BI__builtin_sve_svaesd_u8,
  BI__builtin_sve_svaesdimc_lane_u8_x2,
  BI__builtin_sve_svaesdimc_lane_u8_x4,
  BI__builtin_sve_svaese_lane_u8_x2,
  BI__builtin_sve_svaese_lane_u8_x4,
  BI__builtin_sve_svaese_u8,
  BI__builtin_sve_svaesemc_lane_u8_x2,
  BI__builtin_sve_svaesemc_lane_u8_x4,
  BI__builtin_sve_svaesimc_u8,
  BI__builtin_sve_svaesmc_u8,
  BI__builtin_sve_svamax_f16_m,
  BI__builtin_sve_svamax_f16_x,
  BI__builtin_sve_svamax_f16_x2,
  BI__builtin_sve_svamax_f16_x4,
  BI__builtin_sve_svamax_f16_z,
  BI__builtin_sve_svamax_f32_m,
  BI__builtin_sve_svamax_f32_x,
  BI__builtin_sve_svamax_f32_x2,
  BI__builtin_sve_svamax_f32_x4,
  BI__builtin_sve_svamax_f32_z,
  BI__builtin_sve_svamax_f64_m,
  BI__builtin_sve_svamax_f64_x,
  BI__builtin_sve_svamax_f64_x2,
  BI__builtin_sve_svamax_f64_x4,
  BI__builtin_sve_svamax_f64_z,
  BI__builtin_sve_svamax_n_f16_m,
  BI__builtin_sve_svamax_n_f16_x,
  BI__builtin_sve_svamax_n_f16_z,
  BI__builtin_sve_svamax_n_f32_m,
  BI__builtin_sve_svamax_n_f32_x,
  BI__builtin_sve_svamax_n_f32_z,
  BI__builtin_sve_svamax_n_f64_m,
  BI__builtin_sve_svamax_n_f64_x,
  BI__builtin_sve_svamax_n_f64_z,
  BI__builtin_sve_svamin_f16_m,
  BI__builtin_sve_svamin_f16_x,
  BI__builtin_sve_svamin_f16_x2,
  BI__builtin_sve_svamin_f16_x4,
  BI__builtin_sve_svamin_f16_z,
  BI__builtin_sve_svamin_f32_m,
  BI__builtin_sve_svamin_f32_x,
  BI__builtin_sve_svamin_f32_x2,
  BI__builtin_sve_svamin_f32_x4,
  BI__builtin_sve_svamin_f32_z,
  BI__builtin_sve_svamin_f64_m,
  BI__builtin_sve_svamin_f64_x,
  BI__builtin_sve_svamin_f64_x2,
  BI__builtin_sve_svamin_f64_x4,
  BI__builtin_sve_svamin_f64_z,
  BI__builtin_sve_svamin_n_f16_m,
  BI__builtin_sve_svamin_n_f16_x,
  BI__builtin_sve_svamin_n_f16_z,
  BI__builtin_sve_svamin_n_f32_m,
  BI__builtin_sve_svamin_n_f32_x,
  BI__builtin_sve_svamin_n_f32_z,
  BI__builtin_sve_svamin_n_f64_m,
  BI__builtin_sve_svamin_n_f64_x,
  BI__builtin_sve_svamin_n_f64_z,
  BI__builtin_sve_svand_b_z,
  BI__builtin_sve_svand_n_s16_m,
  BI__builtin_sve_svand_n_s16_x,
  BI__builtin_sve_svand_n_s16_z,
  BI__builtin_sve_svand_n_s32_m,
  BI__builtin_sve_svand_n_s32_x,
  BI__builtin_sve_svand_n_s32_z,
  BI__builtin_sve_svand_n_s64_m,
  BI__builtin_sve_svand_n_s64_x,
  BI__builtin_sve_svand_n_s64_z,
  BI__builtin_sve_svand_n_s8_m,
  BI__builtin_sve_svand_n_s8_x,
  BI__builtin_sve_svand_n_s8_z,
  BI__builtin_sve_svand_n_u16_m,
  BI__builtin_sve_svand_n_u16_x,
  BI__builtin_sve_svand_n_u16_z,
  BI__builtin_sve_svand_n_u32_m,
  BI__builtin_sve_svand_n_u32_x,
  BI__builtin_sve_svand_n_u32_z,
  BI__builtin_sve_svand_n_u64_m,
  BI__builtin_sve_svand_n_u64_x,
  BI__builtin_sve_svand_n_u64_z,
  BI__builtin_sve_svand_n_u8_m,
  BI__builtin_sve_svand_n_u8_x,
  BI__builtin_sve_svand_n_u8_z,
  BI__builtin_sve_svand_s16_m,
  BI__builtin_sve_svand_s16_x,
  BI__builtin_sve_svand_s16_z,
  BI__builtin_sve_svand_s32_m,
  BI__builtin_sve_svand_s32_x,
  BI__builtin_sve_svand_s32_z,
  BI__builtin_sve_svand_s64_m,
  BI__builtin_sve_svand_s64_x,
  BI__builtin_sve_svand_s64_z,
  BI__builtin_sve_svand_s8_m,
  BI__builtin_sve_svand_s8_x,
  BI__builtin_sve_svand_s8_z,
  BI__builtin_sve_svand_u16_m,
  BI__builtin_sve_svand_u16_x,
  BI__builtin_sve_svand_u16_z,
  BI__builtin_sve_svand_u32_m,
  BI__builtin_sve_svand_u32_x,
  BI__builtin_sve_svand_u32_z,
  BI__builtin_sve_svand_u64_m,
  BI__builtin_sve_svand_u64_x,
  BI__builtin_sve_svand_u64_z,
  BI__builtin_sve_svand_u8_m,
  BI__builtin_sve_svand_u8_x,
  BI__builtin_sve_svand_u8_z,
  BI__builtin_sve_svandqv_s16,
  BI__builtin_sve_svandqv_s32,
  BI__builtin_sve_svandqv_s64,
  BI__builtin_sve_svandqv_s8,
  BI__builtin_sve_svandqv_u16,
  BI__builtin_sve_svandqv_u32,
  BI__builtin_sve_svandqv_u64,
  BI__builtin_sve_svandqv_u8,
  BI__builtin_sve_svandv_s16,
  BI__builtin_sve_svandv_s32,
  BI__builtin_sve_svandv_s64,
  BI__builtin_sve_svandv_s8,
  BI__builtin_sve_svandv_u16,
  BI__builtin_sve_svandv_u32,
  BI__builtin_sve_svandv_u64,
  BI__builtin_sve_svandv_u8,
  BI__builtin_sve_svasr_n_s16_m,
  BI__builtin_sve_svasr_n_s16_x,
  BI__builtin_sve_svasr_n_s16_z,
  BI__builtin_sve_svasr_n_s32_m,
  BI__builtin_sve_svasr_n_s32_x,
  BI__builtin_sve_svasr_n_s32_z,
  BI__builtin_sve_svasr_n_s64_m,
  BI__builtin_sve_svasr_n_s64_x,
  BI__builtin_sve_svasr_n_s64_z,
  BI__builtin_sve_svasr_n_s8_m,
  BI__builtin_sve_svasr_n_s8_x,
  BI__builtin_sve_svasr_n_s8_z,
  BI__builtin_sve_svasr_s16_m,
  BI__builtin_sve_svasr_s16_x,
  BI__builtin_sve_svasr_s16_z,
  BI__builtin_sve_svasr_s32_m,
  BI__builtin_sve_svasr_s32_x,
  BI__builtin_sve_svasr_s32_z,
  BI__builtin_sve_svasr_s64_m,
  BI__builtin_sve_svasr_s64_x,
  BI__builtin_sve_svasr_s64_z,
  BI__builtin_sve_svasr_s8_m,
  BI__builtin_sve_svasr_s8_x,
  BI__builtin_sve_svasr_s8_z,
  BI__builtin_sve_svasr_wide_n_s16_m,
  BI__builtin_sve_svasr_wide_n_s16_x,
  BI__builtin_sve_svasr_wide_n_s16_z,
  BI__builtin_sve_svasr_wide_n_s32_m,
  BI__builtin_sve_svasr_wide_n_s32_x,
  BI__builtin_sve_svasr_wide_n_s32_z,
  BI__builtin_sve_svasr_wide_n_s8_m,
  BI__builtin_sve_svasr_wide_n_s8_x,
  BI__builtin_sve_svasr_wide_n_s8_z,
  BI__builtin_sve_svasr_wide_s16_m,
  BI__builtin_sve_svasr_wide_s16_x,
  BI__builtin_sve_svasr_wide_s16_z,
  BI__builtin_sve_svasr_wide_s32_m,
  BI__builtin_sve_svasr_wide_s32_x,
  BI__builtin_sve_svasr_wide_s32_z,
  BI__builtin_sve_svasr_wide_s8_m,
  BI__builtin_sve_svasr_wide_s8_x,
  BI__builtin_sve_svasr_wide_s8_z,
  BI__builtin_sve_svasrd_n_s16_m,
  BI__builtin_sve_svasrd_n_s16_x,
  BI__builtin_sve_svasrd_n_s16_z,
  BI__builtin_sve_svasrd_n_s32_m,
  BI__builtin_sve_svasrd_n_s32_x,
  BI__builtin_sve_svasrd_n_s32_z,
  BI__builtin_sve_svasrd_n_s64_m,
  BI__builtin_sve_svasrd_n_s64_x,
  BI__builtin_sve_svasrd_n_s64_z,
  BI__builtin_sve_svasrd_n_s8_m,
  BI__builtin_sve_svasrd_n_s8_x,
  BI__builtin_sve_svasrd_n_s8_z,
  BI__builtin_sve_svbcax_n_s16,
  BI__builtin_sve_svbcax_n_s32,
  BI__builtin_sve_svbcax_n_s64,
  BI__builtin_sve_svbcax_n_s8,
  BI__builtin_sve_svbcax_n_u16,
  BI__builtin_sve_svbcax_n_u32,
  BI__builtin_sve_svbcax_n_u64,
  BI__builtin_sve_svbcax_n_u8,
  BI__builtin_sve_svbcax_s16,
  BI__builtin_sve_svbcax_s32,
  BI__builtin_sve_svbcax_s64,
  BI__builtin_sve_svbcax_s8,
  BI__builtin_sve_svbcax_u16,
  BI__builtin_sve_svbcax_u32,
  BI__builtin_sve_svbcax_u64,
  BI__builtin_sve_svbcax_u8,
  BI__builtin_sve_svbdep_n_u16,
  BI__builtin_sve_svbdep_n_u32,
  BI__builtin_sve_svbdep_n_u64,
  BI__builtin_sve_svbdep_n_u8,
  BI__builtin_sve_svbdep_u16,
  BI__builtin_sve_svbdep_u32,
  BI__builtin_sve_svbdep_u64,
  BI__builtin_sve_svbdep_u8,
  BI__builtin_sve_svbext_n_u16,
  BI__builtin_sve_svbext_n_u32,
  BI__builtin_sve_svbext_n_u64,
  BI__builtin_sve_svbext_n_u8,
  BI__builtin_sve_svbext_u16,
  BI__builtin_sve_svbext_u32,
  BI__builtin_sve_svbext_u64,
  BI__builtin_sve_svbext_u8,
  BI__builtin_sve_svbfdot_f32,
  BI__builtin_sve_svbfdot_lane_f32,
  BI__builtin_sve_svbfdot_n_f32,
  BI__builtin_sve_svbfmlalb_f32,
  BI__builtin_sve_svbfmlalb_lane_f32,
  BI__builtin_sve_svbfmlalb_n_f32,
  BI__builtin_sve_svbfmlalt_f32,
  BI__builtin_sve_svbfmlalt_lane_f32,
  BI__builtin_sve_svbfmlalt_n_f32,
  BI__builtin_sve_svbfmlslb_f32,
  BI__builtin_sve_svbfmlslb_lane_f32,
  BI__builtin_sve_svbfmlslt_f32,
  BI__builtin_sve_svbfmlslt_lane_f32,
  BI__builtin_sve_svbfmmla_f32,
  BI__builtin_sve_svbgrp_n_u16,
  BI__builtin_sve_svbgrp_n_u32,
  BI__builtin_sve_svbgrp_n_u64,
  BI__builtin_sve_svbgrp_n_u8,
  BI__builtin_sve_svbgrp_u16,
  BI__builtin_sve_svbgrp_u32,
  BI__builtin_sve_svbgrp_u64,
  BI__builtin_sve_svbgrp_u8,
  BI__builtin_sve_svbic_b_z,
  BI__builtin_sve_svbic_n_s16_m,
  BI__builtin_sve_svbic_n_s16_x,
  BI__builtin_sve_svbic_n_s16_z,
  BI__builtin_sve_svbic_n_s32_m,
  BI__builtin_sve_svbic_n_s32_x,
  BI__builtin_sve_svbic_n_s32_z,
  BI__builtin_sve_svbic_n_s64_m,
  BI__builtin_sve_svbic_n_s64_x,
  BI__builtin_sve_svbic_n_s64_z,
  BI__builtin_sve_svbic_n_s8_m,
  BI__builtin_sve_svbic_n_s8_x,
  BI__builtin_sve_svbic_n_s8_z,
  BI__builtin_sve_svbic_n_u16_m,
  BI__builtin_sve_svbic_n_u16_x,
  BI__builtin_sve_svbic_n_u16_z,
  BI__builtin_sve_svbic_n_u32_m,
  BI__builtin_sve_svbic_n_u32_x,
  BI__builtin_sve_svbic_n_u32_z,
  BI__builtin_sve_svbic_n_u64_m,
  BI__builtin_sve_svbic_n_u64_x,
  BI__builtin_sve_svbic_n_u64_z,
  BI__builtin_sve_svbic_n_u8_m,
  BI__builtin_sve_svbic_n_u8_x,
  BI__builtin_sve_svbic_n_u8_z,
  BI__builtin_sve_svbic_s16_m,
  BI__builtin_sve_svbic_s16_x,
  BI__builtin_sve_svbic_s16_z,
  BI__builtin_sve_svbic_s32_m,
  BI__builtin_sve_svbic_s32_x,
  BI__builtin_sve_svbic_s32_z,
  BI__builtin_sve_svbic_s64_m,
  BI__builtin_sve_svbic_s64_x,
  BI__builtin_sve_svbic_s64_z,
  BI__builtin_sve_svbic_s8_m,
  BI__builtin_sve_svbic_s8_x,
  BI__builtin_sve_svbic_s8_z,
  BI__builtin_sve_svbic_u16_m,
  BI__builtin_sve_svbic_u16_x,
  BI__builtin_sve_svbic_u16_z,
  BI__builtin_sve_svbic_u32_m,
  BI__builtin_sve_svbic_u32_x,
  BI__builtin_sve_svbic_u32_z,
  BI__builtin_sve_svbic_u64_m,
  BI__builtin_sve_svbic_u64_x,
  BI__builtin_sve_svbic_u64_z,
  BI__builtin_sve_svbic_u8_m,
  BI__builtin_sve_svbic_u8_x,
  BI__builtin_sve_svbic_u8_z,
  BI__builtin_sve_svbrka_b_m,
  BI__builtin_sve_svbrka_b_z,
  BI__builtin_sve_svbrkb_b_m,
  BI__builtin_sve_svbrkb_b_z,
  BI__builtin_sve_svbrkn_b_z,
  BI__builtin_sve_svbrkpa_b_z,
  BI__builtin_sve_svbrkpb_b_z,
  BI__builtin_sve_svbsl1n_n_s16,
  BI__builtin_sve_svbsl1n_n_s32,
  BI__builtin_sve_svbsl1n_n_s64,
  BI__builtin_sve_svbsl1n_n_s8,
  BI__builtin_sve_svbsl1n_n_u16,
  BI__builtin_sve_svbsl1n_n_u32,
  BI__builtin_sve_svbsl1n_n_u64,
  BI__builtin_sve_svbsl1n_n_u8,
  BI__builtin_sve_svbsl1n_s16,
  BI__builtin_sve_svbsl1n_s32,
  BI__builtin_sve_svbsl1n_s64,
  BI__builtin_sve_svbsl1n_s8,
  BI__builtin_sve_svbsl1n_u16,
  BI__builtin_sve_svbsl1n_u32,
  BI__builtin_sve_svbsl1n_u64,
  BI__builtin_sve_svbsl1n_u8,
  BI__builtin_sve_svbsl2n_n_s16,
  BI__builtin_sve_svbsl2n_n_s32,
  BI__builtin_sve_svbsl2n_n_s64,
  BI__builtin_sve_svbsl2n_n_s8,
  BI__builtin_sve_svbsl2n_n_u16,
  BI__builtin_sve_svbsl2n_n_u32,
  BI__builtin_sve_svbsl2n_n_u64,
  BI__builtin_sve_svbsl2n_n_u8,
  BI__builtin_sve_svbsl2n_s16,
  BI__builtin_sve_svbsl2n_s32,
  BI__builtin_sve_svbsl2n_s64,
  BI__builtin_sve_svbsl2n_s8,
  BI__builtin_sve_svbsl2n_u16,
  BI__builtin_sve_svbsl2n_u32,
  BI__builtin_sve_svbsl2n_u64,
  BI__builtin_sve_svbsl2n_u8,
  BI__builtin_sve_svbsl_n_s16,
  BI__builtin_sve_svbsl_n_s32,
  BI__builtin_sve_svbsl_n_s64,
  BI__builtin_sve_svbsl_n_s8,
  BI__builtin_sve_svbsl_n_u16,
  BI__builtin_sve_svbsl_n_u32,
  BI__builtin_sve_svbsl_n_u64,
  BI__builtin_sve_svbsl_n_u8,
  BI__builtin_sve_svbsl_s16,
  BI__builtin_sve_svbsl_s32,
  BI__builtin_sve_svbsl_s64,
  BI__builtin_sve_svbsl_s8,
  BI__builtin_sve_svbsl_u16,
  BI__builtin_sve_svbsl_u32,
  BI__builtin_sve_svbsl_u64,
  BI__builtin_sve_svbsl_u8,
  BI__builtin_sve_svcadd_f16_m,
  BI__builtin_sve_svcadd_f16_x,
  BI__builtin_sve_svcadd_f16_z,
  BI__builtin_sve_svcadd_f32_m,
  BI__builtin_sve_svcadd_f32_x,
  BI__builtin_sve_svcadd_f32_z,
  BI__builtin_sve_svcadd_f64_m,
  BI__builtin_sve_svcadd_f64_x,
  BI__builtin_sve_svcadd_f64_z,
  BI__builtin_sve_svcadd_s16,
  BI__builtin_sve_svcadd_s32,
  BI__builtin_sve_svcadd_s64,
  BI__builtin_sve_svcadd_s8,
  BI__builtin_sve_svcadd_u16,
  BI__builtin_sve_svcadd_u32,
  BI__builtin_sve_svcadd_u64,
  BI__builtin_sve_svcadd_u8,
  BI__builtin_sve_svcdot_lane_s32,
  BI__builtin_sve_svcdot_lane_s64,
  BI__builtin_sve_svcdot_s32,
  BI__builtin_sve_svcdot_s64,
  BI__builtin_sve_svclamp_bf16,
  BI__builtin_sve_svclamp_f16,
  BI__builtin_sve_svclamp_f32,
  BI__builtin_sve_svclamp_f64,
  BI__builtin_sve_svclamp_s16,
  BI__builtin_sve_svclamp_s32,
  BI__builtin_sve_svclamp_s64,
  BI__builtin_sve_svclamp_s8,
  BI__builtin_sve_svclamp_single_bf16_x2,
  BI__builtin_sve_svclamp_single_bf16_x4,
  BI__builtin_sve_svclamp_single_f16_x2,
  BI__builtin_sve_svclamp_single_f16_x4,
  BI__builtin_sve_svclamp_single_f32_x2,
  BI__builtin_sve_svclamp_single_f32_x4,
  BI__builtin_sve_svclamp_single_f64_x2,
  BI__builtin_sve_svclamp_single_f64_x4,
  BI__builtin_sve_svclamp_single_s16_x2,
  BI__builtin_sve_svclamp_single_s16_x4,
  BI__builtin_sve_svclamp_single_s32_x2,
  BI__builtin_sve_svclamp_single_s32_x4,
  BI__builtin_sve_svclamp_single_s64_x2,
  BI__builtin_sve_svclamp_single_s64_x4,
  BI__builtin_sve_svclamp_single_s8_x2,
  BI__builtin_sve_svclamp_single_s8_x4,
  BI__builtin_sve_svclamp_single_u16_x2,
  BI__builtin_sve_svclamp_single_u16_x4,
  BI__builtin_sve_svclamp_single_u32_x2,
  BI__builtin_sve_svclamp_single_u32_x4,
  BI__builtin_sve_svclamp_single_u64_x2,
  BI__builtin_sve_svclamp_single_u64_x4,
  BI__builtin_sve_svclamp_single_u8_x2,
  BI__builtin_sve_svclamp_single_u8_x4,
  BI__builtin_sve_svclamp_u16,
  BI__builtin_sve_svclamp_u32,
  BI__builtin_sve_svclamp_u64,
  BI__builtin_sve_svclamp_u8,
  BI__builtin_sve_svclasta_bf16,
  BI__builtin_sve_svclasta_f16,
  BI__builtin_sve_svclasta_f32,
  BI__builtin_sve_svclasta_f64,
  BI__builtin_sve_svclasta_n_bf16,
  BI__builtin_sve_svclasta_n_f16,
  BI__builtin_sve_svclasta_n_f32,
  BI__builtin_sve_svclasta_n_f64,
  BI__builtin_sve_svclasta_n_s16,
  BI__builtin_sve_svclasta_n_s32,
  BI__builtin_sve_svclasta_n_s64,
  BI__builtin_sve_svclasta_n_s8,
  BI__builtin_sve_svclasta_n_u16,
  BI__builtin_sve_svclasta_n_u32,
  BI__builtin_sve_svclasta_n_u64,
  BI__builtin_sve_svclasta_n_u8,
  BI__builtin_sve_svclasta_s16,
  BI__builtin_sve_svclasta_s32,
  BI__builtin_sve_svclasta_s64,
  BI__builtin_sve_svclasta_s8,
  BI__builtin_sve_svclasta_u16,
  BI__builtin_sve_svclasta_u32,
  BI__builtin_sve_svclasta_u64,
  BI__builtin_sve_svclasta_u8,
  BI__builtin_sve_svclastb_bf16,
  BI__builtin_sve_svclastb_f16,
  BI__builtin_sve_svclastb_f32,
  BI__builtin_sve_svclastb_f64,
  BI__builtin_sve_svclastb_n_bf16,
  BI__builtin_sve_svclastb_n_f16,
  BI__builtin_sve_svclastb_n_f32,
  BI__builtin_sve_svclastb_n_f64,
  BI__builtin_sve_svclastb_n_s16,
  BI__builtin_sve_svclastb_n_s32,
  BI__builtin_sve_svclastb_n_s64,
  BI__builtin_sve_svclastb_n_s8,
  BI__builtin_sve_svclastb_n_u16,
  BI__builtin_sve_svclastb_n_u32,
  BI__builtin_sve_svclastb_n_u64,
  BI__builtin_sve_svclastb_n_u8,
  BI__builtin_sve_svclastb_s16,
  BI__builtin_sve_svclastb_s32,
  BI__builtin_sve_svclastb_s64,
  BI__builtin_sve_svclastb_s8,
  BI__builtin_sve_svclastb_u16,
  BI__builtin_sve_svclastb_u32,
  BI__builtin_sve_svclastb_u64,
  BI__builtin_sve_svclastb_u8,
  BI__builtin_sve_svcls_s16_m,
  BI__builtin_sve_svcls_s16_x,
  BI__builtin_sve_svcls_s16_z,
  BI__builtin_sve_svcls_s32_m,
  BI__builtin_sve_svcls_s32_x,
  BI__builtin_sve_svcls_s32_z,
  BI__builtin_sve_svcls_s64_m,
  BI__builtin_sve_svcls_s64_x,
  BI__builtin_sve_svcls_s64_z,
  BI__builtin_sve_svcls_s8_m,
  BI__builtin_sve_svcls_s8_x,
  BI__builtin_sve_svcls_s8_z,
  BI__builtin_sve_svclz_s16_m,
  BI__builtin_sve_svclz_s16_x,
  BI__builtin_sve_svclz_s16_z,
  BI__builtin_sve_svclz_s32_m,
  BI__builtin_sve_svclz_s32_x,
  BI__builtin_sve_svclz_s32_z,
  BI__builtin_sve_svclz_s64_m,
  BI__builtin_sve_svclz_s64_x,
  BI__builtin_sve_svclz_s64_z,
  BI__builtin_sve_svclz_s8_m,
  BI__builtin_sve_svclz_s8_x,
  BI__builtin_sve_svclz_s8_z,
  BI__builtin_sve_svclz_u16_m,
  BI__builtin_sve_svclz_u16_x,
  BI__builtin_sve_svclz_u16_z,
  BI__builtin_sve_svclz_u32_m,
  BI__builtin_sve_svclz_u32_x,
  BI__builtin_sve_svclz_u32_z,
  BI__builtin_sve_svclz_u64_m,
  BI__builtin_sve_svclz_u64_x,
  BI__builtin_sve_svclz_u64_z,
  BI__builtin_sve_svclz_u8_m,
  BI__builtin_sve_svclz_u8_x,
  BI__builtin_sve_svclz_u8_z,
  BI__builtin_sve_svcmla_f16_m,
  BI__builtin_sve_svcmla_f16_x,
  BI__builtin_sve_svcmla_f16_z,
  BI__builtin_sve_svcmla_f32_m,
  BI__builtin_sve_svcmla_f32_x,
  BI__builtin_sve_svcmla_f32_z,
  BI__builtin_sve_svcmla_f64_m,
  BI__builtin_sve_svcmla_f64_x,
  BI__builtin_sve_svcmla_f64_z,
  BI__builtin_sve_svcmla_lane_f16,
  BI__builtin_sve_svcmla_lane_f32,
  BI__builtin_sve_svcmla_lane_s16,
  BI__builtin_sve_svcmla_lane_s32,
  BI__builtin_sve_svcmla_lane_u16,
  BI__builtin_sve_svcmla_lane_u32,
  BI__builtin_sve_svcmla_s16,
  BI__builtin_sve_svcmla_s32,
  BI__builtin_sve_svcmla_s64,
  BI__builtin_sve_svcmla_s8,
  BI__builtin_sve_svcmla_u16,
  BI__builtin_sve_svcmla_u32,
  BI__builtin_sve_svcmla_u64,
  BI__builtin_sve_svcmla_u8,
  BI__builtin_sve_svcmpeq_f16,
  BI__builtin_sve_svcmpeq_f32,
  BI__builtin_sve_svcmpeq_f64,
  BI__builtin_sve_svcmpeq_n_f16,
  BI__builtin_sve_svcmpeq_n_f32,
  BI__builtin_sve_svcmpeq_n_f64,
  BI__builtin_sve_svcmpeq_n_s16,
  BI__builtin_sve_svcmpeq_n_s32,
  BI__builtin_sve_svcmpeq_n_s64,
  BI__builtin_sve_svcmpeq_n_s8,
  BI__builtin_sve_svcmpeq_n_u16,
  BI__builtin_sve_svcmpeq_n_u32,
  BI__builtin_sve_svcmpeq_n_u64,
  BI__builtin_sve_svcmpeq_n_u8,
  BI__builtin_sve_svcmpeq_s16,
  BI__builtin_sve_svcmpeq_s32,
  BI__builtin_sve_svcmpeq_s64,
  BI__builtin_sve_svcmpeq_s8,
  BI__builtin_sve_svcmpeq_u16,
  BI__builtin_sve_svcmpeq_u32,
  BI__builtin_sve_svcmpeq_u64,
  BI__builtin_sve_svcmpeq_u8,
  BI__builtin_sve_svcmpeq_wide_n_s16,
  BI__builtin_sve_svcmpeq_wide_n_s32,
  BI__builtin_sve_svcmpeq_wide_n_s8,
  BI__builtin_sve_svcmpeq_wide_s16,
  BI__builtin_sve_svcmpeq_wide_s32,
  BI__builtin_sve_svcmpeq_wide_s8,
  BI__builtin_sve_svcmpge_f16,
  BI__builtin_sve_svcmpge_f32,
  BI__builtin_sve_svcmpge_f64,
  BI__builtin_sve_svcmpge_n_f16,
  BI__builtin_sve_svcmpge_n_f32,
  BI__builtin_sve_svcmpge_n_f64,
  BI__builtin_sve_svcmpge_n_s16,
  BI__builtin_sve_svcmpge_n_s32,
  BI__builtin_sve_svcmpge_n_s64,
  BI__builtin_sve_svcmpge_n_s8,
  BI__builtin_sve_svcmpge_n_u16,
  BI__builtin_sve_svcmpge_n_u32,
  BI__builtin_sve_svcmpge_n_u64,
  BI__builtin_sve_svcmpge_n_u8,
  BI__builtin_sve_svcmpge_s16,
  BI__builtin_sve_svcmpge_s32,
  BI__builtin_sve_svcmpge_s64,
  BI__builtin_sve_svcmpge_s8,
  BI__builtin_sve_svcmpge_u16,
  BI__builtin_sve_svcmpge_u32,
  BI__builtin_sve_svcmpge_u64,
  BI__builtin_sve_svcmpge_u8,
  BI__builtin_sve_svcmpge_wide_n_s16,
  BI__builtin_sve_svcmpge_wide_n_s32,
  BI__builtin_sve_svcmpge_wide_n_s8,
  BI__builtin_sve_svcmpge_wide_n_u16,
  BI__builtin_sve_svcmpge_wide_n_u32,
  BI__builtin_sve_svcmpge_wide_n_u8,
  BI__builtin_sve_svcmpge_wide_s16,
  BI__builtin_sve_svcmpge_wide_s32,
  BI__builtin_sve_svcmpge_wide_s8,
  BI__builtin_sve_svcmpge_wide_u16,
  BI__builtin_sve_svcmpge_wide_u32,
  BI__builtin_sve_svcmpge_wide_u8,
  BI__builtin_sve_svcmpgt_f16,
  BI__builtin_sve_svcmpgt_f32,
  BI__builtin_sve_svcmpgt_f64,
  BI__builtin_sve_svcmpgt_n_f16,
  BI__builtin_sve_svcmpgt_n_f32,
  BI__builtin_sve_svcmpgt_n_f64,
  BI__builtin_sve_svcmpgt_n_s16,
  BI__builtin_sve_svcmpgt_n_s32,
  BI__builtin_sve_svcmpgt_n_s64,
  BI__builtin_sve_svcmpgt_n_s8,
  BI__builtin_sve_svcmpgt_n_u16,
  BI__builtin_sve_svcmpgt_n_u32,
  BI__builtin_sve_svcmpgt_n_u64,
  BI__builtin_sve_svcmpgt_n_u8,
  BI__builtin_sve_svcmpgt_s16,
  BI__builtin_sve_svcmpgt_s32,
  BI__builtin_sve_svcmpgt_s64,
  BI__builtin_sve_svcmpgt_s8,
  BI__builtin_sve_svcmpgt_u16,
  BI__builtin_sve_svcmpgt_u32,
  BI__builtin_sve_svcmpgt_u64,
  BI__builtin_sve_svcmpgt_u8,
  BI__builtin_sve_svcmpgt_wide_n_s16,
  BI__builtin_sve_svcmpgt_wide_n_s32,
  BI__builtin_sve_svcmpgt_wide_n_s8,
  BI__builtin_sve_svcmpgt_wide_n_u16,
  BI__builtin_sve_svcmpgt_wide_n_u32,
  BI__builtin_sve_svcmpgt_wide_n_u8,
  BI__builtin_sve_svcmpgt_wide_s16,
  BI__builtin_sve_svcmpgt_wide_s32,
  BI__builtin_sve_svcmpgt_wide_s8,
  BI__builtin_sve_svcmpgt_wide_u16,
  BI__builtin_sve_svcmpgt_wide_u32,
  BI__builtin_sve_svcmpgt_wide_u8,
  BI__builtin_sve_svcmple_f16,
  BI__builtin_sve_svcmple_f32,
  BI__builtin_sve_svcmple_f64,
  BI__builtin_sve_svcmple_n_f16,
  BI__builtin_sve_svcmple_n_f32,
  BI__builtin_sve_svcmple_n_f64,
  BI__builtin_sve_svcmple_n_s16,
  BI__builtin_sve_svcmple_n_s32,
  BI__builtin_sve_svcmple_n_s64,
  BI__builtin_sve_svcmple_n_s8,
  BI__builtin_sve_svcmple_n_u16,
  BI__builtin_sve_svcmple_n_u32,
  BI__builtin_sve_svcmple_n_u64,
  BI__builtin_sve_svcmple_n_u8,
  BI__builtin_sve_svcmple_s16,
  BI__builtin_sve_svcmple_s32,
  BI__builtin_sve_svcmple_s64,
  BI__builtin_sve_svcmple_s8,
  BI__builtin_sve_svcmple_u16,
  BI__builtin_sve_svcmple_u32,
  BI__builtin_sve_svcmple_u64,
  BI__builtin_sve_svcmple_u8,
  BI__builtin_sve_svcmple_wide_n_s16,
  BI__builtin_sve_svcmple_wide_n_s32,
  BI__builtin_sve_svcmple_wide_n_s8,
  BI__builtin_sve_svcmple_wide_n_u16,
  BI__builtin_sve_svcmple_wide_n_u32,
  BI__builtin_sve_svcmple_wide_n_u8,
  BI__builtin_sve_svcmple_wide_s16,
  BI__builtin_sve_svcmple_wide_s32,
  BI__builtin_sve_svcmple_wide_s8,
  BI__builtin_sve_svcmple_wide_u16,
  BI__builtin_sve_svcmple_wide_u32,
  BI__builtin_sve_svcmple_wide_u8,
  BI__builtin_sve_svcmplt_f16,
  BI__builtin_sve_svcmplt_f32,
  BI__builtin_sve_svcmplt_f64,
  BI__builtin_sve_svcmplt_n_f16,
  BI__builtin_sve_svcmplt_n_f32,
  BI__builtin_sve_svcmplt_n_f64,
  BI__builtin_sve_svcmplt_n_s16,
  BI__builtin_sve_svcmplt_n_s32,
  BI__builtin_sve_svcmplt_n_s64,
  BI__builtin_sve_svcmplt_n_s8,
  BI__builtin_sve_svcmplt_n_u16,
  BI__builtin_sve_svcmplt_n_u32,
  BI__builtin_sve_svcmplt_n_u64,
  BI__builtin_sve_svcmplt_n_u8,
  BI__builtin_sve_svcmplt_s16,
  BI__builtin_sve_svcmplt_s32,
  BI__builtin_sve_svcmplt_s64,
  BI__builtin_sve_svcmplt_s8,
  BI__builtin_sve_svcmplt_u16,
  BI__builtin_sve_svcmplt_u32,
  BI__builtin_sve_svcmplt_u64,
  BI__builtin_sve_svcmplt_u8,
  BI__builtin_sve_svcmplt_wide_n_s16,
  BI__builtin_sve_svcmplt_wide_n_s32,
  BI__builtin_sve_svcmplt_wide_n_s8,
  BI__builtin_sve_svcmplt_wide_n_u16,
  BI__builtin_sve_svcmplt_wide_n_u32,
  BI__builtin_sve_svcmplt_wide_n_u8,
  BI__builtin_sve_svcmplt_wide_s16,
  BI__builtin_sve_svcmplt_wide_s32,
  BI__builtin_sve_svcmplt_wide_s8,
  BI__builtin_sve_svcmplt_wide_u16,
  BI__builtin_sve_svcmplt_wide_u32,
  BI__builtin_sve_svcmplt_wide_u8,
  BI__builtin_sve_svcmpne_f16,
  BI__builtin_sve_svcmpne_f32,
  BI__builtin_sve_svcmpne_f64,
  BI__builtin_sve_svcmpne_n_f16,
  BI__builtin_sve_svcmpne_n_f32,
  BI__builtin_sve_svcmpne_n_f64,
  BI__builtin_sve_svcmpne_n_s16,
  BI__builtin_sve_svcmpne_n_s32,
  BI__builtin_sve_svcmpne_n_s64,
  BI__builtin_sve_svcmpne_n_s8,
  BI__builtin_sve_svcmpne_n_u16,
  BI__builtin_sve_svcmpne_n_u32,
  BI__builtin_sve_svcmpne_n_u64,
  BI__builtin_sve_svcmpne_n_u8,
  BI__builtin_sve_svcmpne_s16,
  BI__builtin_sve_svcmpne_s32,
  BI__builtin_sve_svcmpne_s64,
  BI__builtin_sve_svcmpne_s8,
  BI__builtin_sve_svcmpne_u16,
  BI__builtin_sve_svcmpne_u32,
  BI__builtin_sve_svcmpne_u64,
  BI__builtin_sve_svcmpne_u8,
  BI__builtin_sve_svcmpne_wide_n_s16,
  BI__builtin_sve_svcmpne_wide_n_s32,
  BI__builtin_sve_svcmpne_wide_n_s8,
  BI__builtin_sve_svcmpne_wide_s16,
  BI__builtin_sve_svcmpne_wide_s32,
  BI__builtin_sve_svcmpne_wide_s8,
  BI__builtin_sve_svcmpuo_f16,
  BI__builtin_sve_svcmpuo_f32,
  BI__builtin_sve_svcmpuo_f64,
  BI__builtin_sve_svcmpuo_n_f16,
  BI__builtin_sve_svcmpuo_n_f32,
  BI__builtin_sve_svcmpuo_n_f64,
  BI__builtin_sve_svcnot_s16_m,
  BI__builtin_sve_svcnot_s16_x,
  BI__builtin_sve_svcnot_s16_z,
  BI__builtin_sve_svcnot_s32_m,
  BI__builtin_sve_svcnot_s32_x,
  BI__builtin_sve_svcnot_s32_z,
  BI__builtin_sve_svcnot_s64_m,
  BI__builtin_sve_svcnot_s64_x,
  BI__builtin_sve_svcnot_s64_z,
  BI__builtin_sve_svcnot_s8_m,
  BI__builtin_sve_svcnot_s8_x,
  BI__builtin_sve_svcnot_s8_z,
  BI__builtin_sve_svcnot_u16_m,
  BI__builtin_sve_svcnot_u16_x,
  BI__builtin_sve_svcnot_u16_z,
  BI__builtin_sve_svcnot_u32_m,
  BI__builtin_sve_svcnot_u32_x,
  BI__builtin_sve_svcnot_u32_z,
  BI__builtin_sve_svcnot_u64_m,
  BI__builtin_sve_svcnot_u64_x,
  BI__builtin_sve_svcnot_u64_z,
  BI__builtin_sve_svcnot_u8_m,
  BI__builtin_sve_svcnot_u8_x,
  BI__builtin_sve_svcnot_u8_z,
  BI__builtin_sve_svcnt_bf16_m,
  BI__builtin_sve_svcnt_bf16_x,
  BI__builtin_sve_svcnt_bf16_z,
  BI__builtin_sve_svcnt_f16_m,
  BI__builtin_sve_svcnt_f16_x,
  BI__builtin_sve_svcnt_f16_z,
  BI__builtin_sve_svcnt_f32_m,
  BI__builtin_sve_svcnt_f32_x,
  BI__builtin_sve_svcnt_f32_z,
  BI__builtin_sve_svcnt_f64_m,
  BI__builtin_sve_svcnt_f64_x,
  BI__builtin_sve_svcnt_f64_z,
  BI__builtin_sve_svcnt_s16_m,
  BI__builtin_sve_svcnt_s16_x,
  BI__builtin_sve_svcnt_s16_z,
  BI__builtin_sve_svcnt_s32_m,
  BI__builtin_sve_svcnt_s32_x,
  BI__builtin_sve_svcnt_s32_z,
  BI__builtin_sve_svcnt_s64_m,
  BI__builtin_sve_svcnt_s64_x,
  BI__builtin_sve_svcnt_s64_z,
  BI__builtin_sve_svcnt_s8_m,
  BI__builtin_sve_svcnt_s8_x,
  BI__builtin_sve_svcnt_s8_z,
  BI__builtin_sve_svcnt_u16_m,
  BI__builtin_sve_svcnt_u16_x,
  BI__builtin_sve_svcnt_u16_z,
  BI__builtin_sve_svcnt_u32_m,
  BI__builtin_sve_svcnt_u32_x,
  BI__builtin_sve_svcnt_u32_z,
  BI__builtin_sve_svcnt_u64_m,
  BI__builtin_sve_svcnt_u64_x,
  BI__builtin_sve_svcnt_u64_z,
  BI__builtin_sve_svcnt_u8_m,
  BI__builtin_sve_svcnt_u8_x,
  BI__builtin_sve_svcnt_u8_z,
  BI__builtin_sve_svcntb,
  BI__builtin_sve_svcntb_pat,
  BI__builtin_sve_svcntd,
  BI__builtin_sve_svcntd_pat,
  BI__builtin_sve_svcnth,
  BI__builtin_sve_svcnth_pat,
  BI__builtin_sve_svcntp_b16,
  BI__builtin_sve_svcntp_b32,
  BI__builtin_sve_svcntp_b64,
  BI__builtin_sve_svcntp_b8,
  BI__builtin_sve_svcntp_c16,
  BI__builtin_sve_svcntp_c32,
  BI__builtin_sve_svcntp_c64,
  BI__builtin_sve_svcntp_c8,
  BI__builtin_sve_svcntw,
  BI__builtin_sve_svcntw_pat,
  BI__builtin_sve_svcompact_bf16,
  BI__builtin_sve_svcompact_f16,
  BI__builtin_sve_svcompact_f32,
  BI__builtin_sve_svcompact_f64,
  BI__builtin_sve_svcompact_mf8,
  BI__builtin_sve_svcompact_s16,
  BI__builtin_sve_svcompact_s32,
  BI__builtin_sve_svcompact_s64,
  BI__builtin_sve_svcompact_s8,
  BI__builtin_sve_svcompact_u16,
  BI__builtin_sve_svcompact_u32,
  BI__builtin_sve_svcompact_u64,
  BI__builtin_sve_svcompact_u8,
  BI__builtin_sve_svcreate2_b,
  BI__builtin_sve_svcreate2_bf16,
  BI__builtin_sve_svcreate2_f16,
  BI__builtin_sve_svcreate2_f32,
  BI__builtin_sve_svcreate2_f64,
  BI__builtin_sve_svcreate2_mf8,
  BI__builtin_sve_svcreate2_s16,
  BI__builtin_sve_svcreate2_s32,
  BI__builtin_sve_svcreate2_s64,
  BI__builtin_sve_svcreate2_s8,
  BI__builtin_sve_svcreate2_u16,
  BI__builtin_sve_svcreate2_u32,
  BI__builtin_sve_svcreate2_u64,
  BI__builtin_sve_svcreate2_u8,
  BI__builtin_sve_svcreate3_bf16,
  BI__builtin_sve_svcreate3_f16,
  BI__builtin_sve_svcreate3_f32,
  BI__builtin_sve_svcreate3_f64,
  BI__builtin_sve_svcreate3_mf8,
  BI__builtin_sve_svcreate3_s16,
  BI__builtin_sve_svcreate3_s32,
  BI__builtin_sve_svcreate3_s64,
  BI__builtin_sve_svcreate3_s8,
  BI__builtin_sve_svcreate3_u16,
  BI__builtin_sve_svcreate3_u32,
  BI__builtin_sve_svcreate3_u64,
  BI__builtin_sve_svcreate3_u8,
  BI__builtin_sve_svcreate4_b,
  BI__builtin_sve_svcreate4_bf16,
  BI__builtin_sve_svcreate4_f16,
  BI__builtin_sve_svcreate4_f32,
  BI__builtin_sve_svcreate4_f64,
  BI__builtin_sve_svcreate4_mf8,
  BI__builtin_sve_svcreate4_s16,
  BI__builtin_sve_svcreate4_s32,
  BI__builtin_sve_svcreate4_s64,
  BI__builtin_sve_svcreate4_s8,
  BI__builtin_sve_svcreate4_u16,
  BI__builtin_sve_svcreate4_u32,
  BI__builtin_sve_svcreate4_u64,
  BI__builtin_sve_svcreate4_u8,
  BI__builtin_sve_svcvt1_bf16_mf8_fpm,
  BI__builtin_sve_svcvt1_bf16_mf8_x2_fpm,
  BI__builtin_sve_svcvt1_f16_mf8_fpm,
  BI__builtin_sve_svcvt1_f16_mf8_x2_fpm,
  BI__builtin_sve_svcvt2_bf16_mf8_fpm,
  BI__builtin_sve_svcvt2_bf16_mf8_x2_fpm,
  BI__builtin_sve_svcvt2_f16_mf8_fpm,
  BI__builtin_sve_svcvt2_f16_mf8_x2_fpm,
  BI__builtin_sve_svcvt_bf16_f32_m,
  BI__builtin_sve_svcvt_bf16_f32_x,
  BI__builtin_sve_svcvt_bf16_f32_x2,
  BI__builtin_sve_svcvt_bf16_f32_z,
  BI__builtin_sve_svcvt_f16_f32_m,
  BI__builtin_sve_svcvt_f16_f32_x,
  BI__builtin_sve_svcvt_f16_f32_x2,
  BI__builtin_sve_svcvt_f16_f32_z,
  BI__builtin_sve_svcvt_f16_f64_m,
  BI__builtin_sve_svcvt_f16_f64_x,
  BI__builtin_sve_svcvt_f16_f64_z,
  BI__builtin_sve_svcvt_f16_s16_m,
  BI__builtin_sve_svcvt_f16_s16_x,
  BI__builtin_sve_svcvt_f16_s16_z,
  BI__builtin_sve_svcvt_f16_s32_m,
  BI__builtin_sve_svcvt_f16_s32_x,
  BI__builtin_sve_svcvt_f16_s32_z,
  BI__builtin_sve_svcvt_f16_s64_m,
  BI__builtin_sve_svcvt_f16_s64_x,
  BI__builtin_sve_svcvt_f16_s64_z,
  BI__builtin_sve_svcvt_f16_u16_m,
  BI__builtin_sve_svcvt_f16_u16_x,
  BI__builtin_sve_svcvt_f16_u16_z,
  BI__builtin_sve_svcvt_f16_u32_m,
  BI__builtin_sve_svcvt_f16_u32_x,
  BI__builtin_sve_svcvt_f16_u32_z,
  BI__builtin_sve_svcvt_f16_u64_m,
  BI__builtin_sve_svcvt_f16_u64_x,
  BI__builtin_sve_svcvt_f16_u64_z,
  BI__builtin_sve_svcvt_f32_f16_m,
  BI__builtin_sve_svcvt_f32_f16_x,
  BI__builtin_sve_svcvt_f32_f16_x2,
  BI__builtin_sve_svcvt_f32_f16_z,
  BI__builtin_sve_svcvt_f32_f64_m,
  BI__builtin_sve_svcvt_f32_f64_x,
  BI__builtin_sve_svcvt_f32_f64_z,
  BI__builtin_sve_svcvt_f32_s32_m,
  BI__builtin_sve_svcvt_f32_s32_x,
  BI__builtin_sve_svcvt_f32_s32_x2,
  BI__builtin_sve_svcvt_f32_s32_x4,
  BI__builtin_sve_svcvt_f32_s32_z,
  BI__builtin_sve_svcvt_f32_s64_m,
  BI__builtin_sve_svcvt_f32_s64_x,
  BI__builtin_sve_svcvt_f32_s64_z,
  BI__builtin_sve_svcvt_f32_u32_m,
  BI__builtin_sve_svcvt_f32_u32_x,
  BI__builtin_sve_svcvt_f32_u32_x2,
  BI__builtin_sve_svcvt_f32_u32_x4,
  BI__builtin_sve_svcvt_f32_u32_z,
  BI__builtin_sve_svcvt_f32_u64_m,
  BI__builtin_sve_svcvt_f32_u64_x,
  BI__builtin_sve_svcvt_f32_u64_z,
  BI__builtin_sve_svcvt_f64_f16_m,
  BI__builtin_sve_svcvt_f64_f16_x,
  BI__builtin_sve_svcvt_f64_f16_z,
  BI__builtin_sve_svcvt_f64_f32_m,
  BI__builtin_sve_svcvt_f64_f32_x,
  BI__builtin_sve_svcvt_f64_f32_z,
  BI__builtin_sve_svcvt_f64_s32_m,
  BI__builtin_sve_svcvt_f64_s32_x,
  BI__builtin_sve_svcvt_f64_s32_z,
  BI__builtin_sve_svcvt_f64_s64_m,
  BI__builtin_sve_svcvt_f64_s64_x,
  BI__builtin_sve_svcvt_f64_s64_z,
  BI__builtin_sve_svcvt_f64_u32_m,
  BI__builtin_sve_svcvt_f64_u32_x,
  BI__builtin_sve_svcvt_f64_u32_z,
  BI__builtin_sve_svcvt_f64_u64_m,
  BI__builtin_sve_svcvt_f64_u64_x,
  BI__builtin_sve_svcvt_f64_u64_z,
  BI__builtin_sve_svcvt_mf8_bf16_x2_fpm,
  BI__builtin_sve_svcvt_mf8_f16_x2_fpm,
  BI__builtin_sve_svcvt_mf8_f32_x4_fpm,
  BI__builtin_sve_svcvt_s16_f16_m,
  BI__builtin_sve_svcvt_s16_f16_x,
  BI__builtin_sve_svcvt_s16_f16_z,
  BI__builtin_sve_svcvt_s32_f16_m,
  BI__builtin_sve_svcvt_s32_f16_x,
  BI__builtin_sve_svcvt_s32_f16_z,
  BI__builtin_sve_svcvt_s32_f32_m,
  BI__builtin_sve_svcvt_s32_f32_x,
  BI__builtin_sve_svcvt_s32_f32_x2,
  BI__builtin_sve_svcvt_s32_f32_x4,
  BI__builtin_sve_svcvt_s32_f32_z,
  BI__builtin_sve_svcvt_s32_f64_m,
  BI__builtin_sve_svcvt_s32_f64_x,
  BI__builtin_sve_svcvt_s32_f64_z,
  BI__builtin_sve_svcvt_s64_f16_m,
  BI__builtin_sve_svcvt_s64_f16_x,
  BI__builtin_sve_svcvt_s64_f16_z,
  BI__builtin_sve_svcvt_s64_f32_m,
  BI__builtin_sve_svcvt_s64_f32_x,
  BI__builtin_sve_svcvt_s64_f32_z,
  BI__builtin_sve_svcvt_s64_f64_m,
  BI__builtin_sve_svcvt_s64_f64_x,
  BI__builtin_sve_svcvt_s64_f64_z,
  BI__builtin_sve_svcvt_u16_f16_m,
  BI__builtin_sve_svcvt_u16_f16_x,
  BI__builtin_sve_svcvt_u16_f16_z,
  BI__builtin_sve_svcvt_u32_f16_m,
  BI__builtin_sve_svcvt_u32_f16_x,
  BI__builtin_sve_svcvt_u32_f16_z,
  BI__builtin_sve_svcvt_u32_f32_m,
  BI__builtin_sve_svcvt_u32_f32_x,
  BI__builtin_sve_svcvt_u32_f32_x2,
  BI__builtin_sve_svcvt_u32_f32_x4,
  BI__builtin_sve_svcvt_u32_f32_z,
  BI__builtin_sve_svcvt_u32_f64_m,
  BI__builtin_sve_svcvt_u32_f64_x,
  BI__builtin_sve_svcvt_u32_f64_z,
  BI__builtin_sve_svcvt_u64_f16_m,
  BI__builtin_sve_svcvt_u64_f16_x,
  BI__builtin_sve_svcvt_u64_f16_z,
  BI__builtin_sve_svcvt_u64_f32_m,
  BI__builtin_sve_svcvt_u64_f32_x,
  BI__builtin_sve_svcvt_u64_f32_z,
  BI__builtin_sve_svcvt_u64_f64_m,
  BI__builtin_sve_svcvt_u64_f64_x,
  BI__builtin_sve_svcvt_u64_f64_z,
  BI__builtin_sve_svcvtl1_bf16_mf8_x2_fpm,
  BI__builtin_sve_svcvtl1_f16_mf8_x2_fpm,
  BI__builtin_sve_svcvtl2_bf16_mf8_x2_fpm,
  BI__builtin_sve_svcvtl2_f16_mf8_x2_fpm,
  BI__builtin_sve_svcvtl_f32_f16_x2,
  BI__builtin_sve_svcvtlt1_bf16_mf8_fpm,
  BI__builtin_sve_svcvtlt1_f16_mf8_fpm,
  BI__builtin_sve_svcvtlt2_bf16_mf8_fpm,
  BI__builtin_sve_svcvtlt2_f16_mf8_fpm,
  BI__builtin_sve_svcvtlt_f32_f16_m,
  BI__builtin_sve_svcvtlt_f32_f16_x,
  BI__builtin_sve_svcvtlt_f32_f16_z,
  BI__builtin_sve_svcvtlt_f64_f32_m,
  BI__builtin_sve_svcvtlt_f64_f32_x,
  BI__builtin_sve_svcvtlt_f64_f32_z,
  BI__builtin_sve_svcvtn_bf16_f32_x2,
  BI__builtin_sve_svcvtn_f16_f32_x2,
  BI__builtin_sve_svcvtn_mf8_bf16_x2_fpm,
  BI__builtin_sve_svcvtn_mf8_f16_x2_fpm,
  BI__builtin_sve_svcvtn_mf8_f32_x4_fpm,
  BI__builtin_sve_svcvtnb_mf8_f32_x2_fpm,
  BI__builtin_sve_svcvtnt_bf16_f32_m,
  BI__builtin_sve_svcvtnt_bf16_f32_z,
  BI__builtin_sve_svcvtnt_f16_f32_m,
  BI__builtin_sve_svcvtnt_f16_f32_z,
  BI__builtin_sve_svcvtnt_f32_f64_m,
  BI__builtin_sve_svcvtnt_f32_f64_z,
  BI__builtin_sve_svcvtnt_mf8_f32_x2_fpm,
  BI__builtin_sve_svcvtx_f32_f64_m,
  BI__builtin_sve_svcvtx_f32_f64_x,
  BI__builtin_sve_svcvtx_f32_f64_z,
  BI__builtin_sve_svcvtxnt_f32_f64_m,
  BI__builtin_sve_svcvtxnt_f32_f64_z,
  BI__builtin_sve_svdiv_f16_m,
  BI__builtin_sve_svdiv_f16_x,
  BI__builtin_sve_svdiv_f16_z,
  BI__builtin_sve_svdiv_f32_m,
  BI__builtin_sve_svdiv_f32_x,
  BI__builtin_sve_svdiv_f32_z,
  BI__builtin_sve_svdiv_f64_m,
  BI__builtin_sve_svdiv_f64_x,
  BI__builtin_sve_svdiv_f64_z,
  BI__builtin_sve_svdiv_n_f16_m,
  BI__builtin_sve_svdiv_n_f16_x,
  BI__builtin_sve_svdiv_n_f16_z,
  BI__builtin_sve_svdiv_n_f32_m,
  BI__builtin_sve_svdiv_n_f32_x,
  BI__builtin_sve_svdiv_n_f32_z,
  BI__builtin_sve_svdiv_n_f64_m,
  BI__builtin_sve_svdiv_n_f64_x,
  BI__builtin_sve_svdiv_n_f64_z,
  BI__builtin_sve_svdiv_n_s32_m,
  BI__builtin_sve_svdiv_n_s32_x,
  BI__builtin_sve_svdiv_n_s32_z,
  BI__builtin_sve_svdiv_n_s64_m,
  BI__builtin_sve_svdiv_n_s64_x,
  BI__builtin_sve_svdiv_n_s64_z,
  BI__builtin_sve_svdiv_n_u32_m,
  BI__builtin_sve_svdiv_n_u32_x,
  BI__builtin_sve_svdiv_n_u32_z,
  BI__builtin_sve_svdiv_n_u64_m,
  BI__builtin_sve_svdiv_n_u64_x,
  BI__builtin_sve_svdiv_n_u64_z,
  BI__builtin_sve_svdiv_s32_m,
  BI__builtin_sve_svdiv_s32_x,
  BI__builtin_sve_svdiv_s32_z,
  BI__builtin_sve_svdiv_s64_m,
  BI__builtin_sve_svdiv_s64_x,
  BI__builtin_sve_svdiv_s64_z,
  BI__builtin_sve_svdiv_u32_m,
  BI__builtin_sve_svdiv_u32_x,
  BI__builtin_sve_svdiv_u32_z,
  BI__builtin_sve_svdiv_u64_m,
  BI__builtin_sve_svdiv_u64_x,
  BI__builtin_sve_svdiv_u64_z,
  BI__builtin_sve_svdivr_f16_m,
  BI__builtin_sve_svdivr_f16_x,
  BI__builtin_sve_svdivr_f16_z,
  BI__builtin_sve_svdivr_f32_m,
  BI__builtin_sve_svdivr_f32_x,
  BI__builtin_sve_svdivr_f32_z,
  BI__builtin_sve_svdivr_f64_m,
  BI__builtin_sve_svdivr_f64_x,
  BI__builtin_sve_svdivr_f64_z,
  BI__builtin_sve_svdivr_n_f16_m,
  BI__builtin_sve_svdivr_n_f16_x,
  BI__builtin_sve_svdivr_n_f16_z,
  BI__builtin_sve_svdivr_n_f32_m,
  BI__builtin_sve_svdivr_n_f32_x,
  BI__builtin_sve_svdivr_n_f32_z,
  BI__builtin_sve_svdivr_n_f64_m,
  BI__builtin_sve_svdivr_n_f64_x,
  BI__builtin_sve_svdivr_n_f64_z,
  BI__builtin_sve_svdivr_n_s32_m,
  BI__builtin_sve_svdivr_n_s32_x,
  BI__builtin_sve_svdivr_n_s32_z,
  BI__builtin_sve_svdivr_n_s64_m,
  BI__builtin_sve_svdivr_n_s64_x,
  BI__builtin_sve_svdivr_n_s64_z,
  BI__builtin_sve_svdivr_n_u32_m,
  BI__builtin_sve_svdivr_n_u32_x,
  BI__builtin_sve_svdivr_n_u32_z,
  BI__builtin_sve_svdivr_n_u64_m,
  BI__builtin_sve_svdivr_n_u64_x,
  BI__builtin_sve_svdivr_n_u64_z,
  BI__builtin_sve_svdivr_s32_m,
  BI__builtin_sve_svdivr_s32_x,
  BI__builtin_sve_svdivr_s32_z,
  BI__builtin_sve_svdivr_s64_m,
  BI__builtin_sve_svdivr_s64_x,
  BI__builtin_sve_svdivr_s64_z,
  BI__builtin_sve_svdivr_u32_m,
  BI__builtin_sve_svdivr_u32_x,
  BI__builtin_sve_svdivr_u32_z,
  BI__builtin_sve_svdivr_u64_m,
  BI__builtin_sve_svdivr_u64_x,
  BI__builtin_sve_svdivr_u64_z,
  BI__builtin_sve_svdot_f16_mf8_fpm,
  BI__builtin_sve_svdot_f32_f16,
  BI__builtin_sve_svdot_f32_mf8_fpm,
  BI__builtin_sve_svdot_lane_f16_mf8_fpm,
  BI__builtin_sve_svdot_lane_f32_f16,
  BI__builtin_sve_svdot_lane_f32_mf8_fpm,
  BI__builtin_sve_svdot_lane_s32,
  BI__builtin_sve_svdot_lane_s32_s16,
  BI__builtin_sve_svdot_lane_s64,
  BI__builtin_sve_svdot_lane_u32,
  BI__builtin_sve_svdot_lane_u32_u16,
  BI__builtin_sve_svdot_lane_u64,
  BI__builtin_sve_svdot_n_f16_mf8_fpm,
  BI__builtin_sve_svdot_n_f32_mf8_fpm,
  BI__builtin_sve_svdot_n_s32,
  BI__builtin_sve_svdot_n_s64,
  BI__builtin_sve_svdot_n_u32,
  BI__builtin_sve_svdot_n_u64,
  BI__builtin_sve_svdot_s32,
  BI__builtin_sve_svdot_s32_s16,
  BI__builtin_sve_svdot_s64,
  BI__builtin_sve_svdot_u32,
  BI__builtin_sve_svdot_u32_u16,
  BI__builtin_sve_svdot_u64,
  BI__builtin_sve_svdup_lane_bf16,
  BI__builtin_sve_svdup_lane_f16,
  BI__builtin_sve_svdup_lane_f32,
  BI__builtin_sve_svdup_lane_f64,
  BI__builtin_sve_svdup_lane_s16,
  BI__builtin_sve_svdup_lane_s32,
  BI__builtin_sve_svdup_lane_s64,
  BI__builtin_sve_svdup_lane_s8,
  BI__builtin_sve_svdup_lane_u16,
  BI__builtin_sve_svdup_lane_u32,
  BI__builtin_sve_svdup_lane_u64,
  BI__builtin_sve_svdup_lane_u8,
  BI__builtin_sve_svdup_laneq_bf16,
  BI__builtin_sve_svdup_laneq_f16,
  BI__builtin_sve_svdup_laneq_f32,
  BI__builtin_sve_svdup_laneq_f64,
  BI__builtin_sve_svdup_laneq_mf8,
  BI__builtin_sve_svdup_laneq_s16,
  BI__builtin_sve_svdup_laneq_s32,
  BI__builtin_sve_svdup_laneq_s64,
  BI__builtin_sve_svdup_laneq_s8,
  BI__builtin_sve_svdup_laneq_u16,
  BI__builtin_sve_svdup_laneq_u32,
  BI__builtin_sve_svdup_laneq_u64,
  BI__builtin_sve_svdup_laneq_u8,
  BI__builtin_sve_svdup_n_b16,
  BI__builtin_sve_svdup_n_b32,
  BI__builtin_sve_svdup_n_b64,
  BI__builtin_sve_svdup_n_b8,
  BI__builtin_sve_svdup_n_bf16,
  BI__builtin_sve_svdup_n_bf16_m,
  BI__builtin_sve_svdup_n_bf16_x,
  BI__builtin_sve_svdup_n_bf16_z,
  BI__builtin_sve_svdup_n_f16,
  BI__builtin_sve_svdup_n_f16_m,
  BI__builtin_sve_svdup_n_f16_x,
  BI__builtin_sve_svdup_n_f16_z,
  BI__builtin_sve_svdup_n_f32,
  BI__builtin_sve_svdup_n_f32_m,
  BI__builtin_sve_svdup_n_f32_x,
  BI__builtin_sve_svdup_n_f32_z,
  BI__builtin_sve_svdup_n_f64,
  BI__builtin_sve_svdup_n_f64_m,
  BI__builtin_sve_svdup_n_f64_x,
  BI__builtin_sve_svdup_n_f64_z,
  BI__builtin_sve_svdup_n_s16,
  BI__builtin_sve_svdup_n_s16_m,
  BI__builtin_sve_svdup_n_s16_x,
  BI__builtin_sve_svdup_n_s16_z,
  BI__builtin_sve_svdup_n_s32,
  BI__builtin_sve_svdup_n_s32_m,
  BI__builtin_sve_svdup_n_s32_x,
  BI__builtin_sve_svdup_n_s32_z,
  BI__builtin_sve_svdup_n_s64,
  BI__builtin_sve_svdup_n_s64_m,
  BI__builtin_sve_svdup_n_s64_x,
  BI__builtin_sve_svdup_n_s64_z,
  BI__builtin_sve_svdup_n_s8,
  BI__builtin_sve_svdup_n_s8_m,
  BI__builtin_sve_svdup_n_s8_x,
  BI__builtin_sve_svdup_n_s8_z,
  BI__builtin_sve_svdup_n_u16,
  BI__builtin_sve_svdup_n_u16_m,
  BI__builtin_sve_svdup_n_u16_x,
  BI__builtin_sve_svdup_n_u16_z,
  BI__builtin_sve_svdup_n_u32,
  BI__builtin_sve_svdup_n_u32_m,
  BI__builtin_sve_svdup_n_u32_x,
  BI__builtin_sve_svdup_n_u32_z,
  BI__builtin_sve_svdup_n_u64,
  BI__builtin_sve_svdup_n_u64_m,
  BI__builtin_sve_svdup_n_u64_x,
  BI__builtin_sve_svdup_n_u64_z,
  BI__builtin_sve_svdup_n_u8,
  BI__builtin_sve_svdup_n_u8_m,
  BI__builtin_sve_svdup_n_u8_x,
  BI__builtin_sve_svdup_n_u8_z,
  BI__builtin_sve_svdupq_lane_bf16,
  BI__builtin_sve_svdupq_lane_f16,
  BI__builtin_sve_svdupq_lane_f32,
  BI__builtin_sve_svdupq_lane_f64,
  BI__builtin_sve_svdupq_lane_s16,
  BI__builtin_sve_svdupq_lane_s32,
  BI__builtin_sve_svdupq_lane_s64,
  BI__builtin_sve_svdupq_lane_s8,
  BI__builtin_sve_svdupq_lane_u16,
  BI__builtin_sve_svdupq_lane_u32,
  BI__builtin_sve_svdupq_lane_u64,
  BI__builtin_sve_svdupq_lane_u8,
  BI__builtin_sve_svdupq_n_b16,
  BI__builtin_sve_svdupq_n_b32,
  BI__builtin_sve_svdupq_n_b64,
  BI__builtin_sve_svdupq_n_b8,
  BI__builtin_sve_svdupq_n_bf16,
  BI__builtin_sve_svdupq_n_f16,
  BI__builtin_sve_svdupq_n_f32,
  BI__builtin_sve_svdupq_n_f64,
  BI__builtin_sve_svdupq_n_s16,
  BI__builtin_sve_svdupq_n_s32,
  BI__builtin_sve_svdupq_n_s64,
  BI__builtin_sve_svdupq_n_s8,
  BI__builtin_sve_svdupq_n_u16,
  BI__builtin_sve_svdupq_n_u32,
  BI__builtin_sve_svdupq_n_u64,
  BI__builtin_sve_svdupq_n_u8,
  BI__builtin_sve_sveor3_n_s16,
  BI__builtin_sve_sveor3_n_s32,
  BI__builtin_sve_sveor3_n_s64,
  BI__builtin_sve_sveor3_n_s8,
  BI__builtin_sve_sveor3_n_u16,
  BI__builtin_sve_sveor3_n_u32,
  BI__builtin_sve_sveor3_n_u64,
  BI__builtin_sve_sveor3_n_u8,
  BI__builtin_sve_sveor3_s16,
  BI__builtin_sve_sveor3_s32,
  BI__builtin_sve_sveor3_s64,
  BI__builtin_sve_sveor3_s8,
  BI__builtin_sve_sveor3_u16,
  BI__builtin_sve_sveor3_u32,
  BI__builtin_sve_sveor3_u64,
  BI__builtin_sve_sveor3_u8,
  BI__builtin_sve_sveor_b_z,
  BI__builtin_sve_sveor_n_s16_m,
  BI__builtin_sve_sveor_n_s16_x,
  BI__builtin_sve_sveor_n_s16_z,
  BI__builtin_sve_sveor_n_s32_m,
  BI__builtin_sve_sveor_n_s32_x,
  BI__builtin_sve_sveor_n_s32_z,
  BI__builtin_sve_sveor_n_s64_m,
  BI__builtin_sve_sveor_n_s64_x,
  BI__builtin_sve_sveor_n_s64_z,
  BI__builtin_sve_sveor_n_s8_m,
  BI__builtin_sve_sveor_n_s8_x,
  BI__builtin_sve_sveor_n_s8_z,
  BI__builtin_sve_sveor_n_u16_m,
  BI__builtin_sve_sveor_n_u16_x,
  BI__builtin_sve_sveor_n_u16_z,
  BI__builtin_sve_sveor_n_u32_m,
  BI__builtin_sve_sveor_n_u32_x,
  BI__builtin_sve_sveor_n_u32_z,
  BI__builtin_sve_sveor_n_u64_m,
  BI__builtin_sve_sveor_n_u64_x,
  BI__builtin_sve_sveor_n_u64_z,
  BI__builtin_sve_sveor_n_u8_m,
  BI__builtin_sve_sveor_n_u8_x,
  BI__builtin_sve_sveor_n_u8_z,
  BI__builtin_sve_sveor_s16_m,
  BI__builtin_sve_sveor_s16_x,
  BI__builtin_sve_sveor_s16_z,
  BI__builtin_sve_sveor_s32_m,
  BI__builtin_sve_sveor_s32_x,
  BI__builtin_sve_sveor_s32_z,
  BI__builtin_sve_sveor_s64_m,
  BI__builtin_sve_sveor_s64_x,
  BI__builtin_sve_sveor_s64_z,
  BI__builtin_sve_sveor_s8_m,
  BI__builtin_sve_sveor_s8_x,
  BI__builtin_sve_sveor_s8_z,
  BI__builtin_sve_sveor_u16_m,
  BI__builtin_sve_sveor_u16_x,
  BI__builtin_sve_sveor_u16_z,
  BI__builtin_sve_sveor_u32_m,
  BI__builtin_sve_sveor_u32_x,
  BI__builtin_sve_sveor_u32_z,
  BI__builtin_sve_sveor_u64_m,
  BI__builtin_sve_sveor_u64_x,
  BI__builtin_sve_sveor_u64_z,
  BI__builtin_sve_sveor_u8_m,
  BI__builtin_sve_sveor_u8_x,
  BI__builtin_sve_sveor_u8_z,
  BI__builtin_sve_sveorbt_n_s16,
  BI__builtin_sve_sveorbt_n_s32,
  BI__builtin_sve_sveorbt_n_s64,
  BI__builtin_sve_sveorbt_n_s8,
  BI__builtin_sve_sveorbt_n_u16,
  BI__builtin_sve_sveorbt_n_u32,
  BI__builtin_sve_sveorbt_n_u64,
  BI__builtin_sve_sveorbt_n_u8,
  BI__builtin_sve_sveorbt_s16,
  BI__builtin_sve_sveorbt_s32,
  BI__builtin_sve_sveorbt_s64,
  BI__builtin_sve_sveorbt_s8,
  BI__builtin_sve_sveorbt_u16,
  BI__builtin_sve_sveorbt_u32,
  BI__builtin_sve_sveorbt_u64,
  BI__builtin_sve_sveorbt_u8,
  BI__builtin_sve_sveorqv_s16,
  BI__builtin_sve_sveorqv_s32,
  BI__builtin_sve_sveorqv_s64,
  BI__builtin_sve_sveorqv_s8,
  BI__builtin_sve_sveorqv_u16,
  BI__builtin_sve_sveorqv_u32,
  BI__builtin_sve_sveorqv_u64,
  BI__builtin_sve_sveorqv_u8,
  BI__builtin_sve_sveortb_n_s16,
  BI__builtin_sve_sveortb_n_s32,
  BI__builtin_sve_sveortb_n_s64,
  BI__builtin_sve_sveortb_n_s8,
  BI__builtin_sve_sveortb_n_u16,
  BI__builtin_sve_sveortb_n_u32,
  BI__builtin_sve_sveortb_n_u64,
  BI__builtin_sve_sveortb_n_u8,
  BI__builtin_sve_sveortb_s16,
  BI__builtin_sve_sveortb_s32,
  BI__builtin_sve_sveortb_s64,
  BI__builtin_sve_sveortb_s8,
  BI__builtin_sve_sveortb_u16,
  BI__builtin_sve_sveortb_u32,
  BI__builtin_sve_sveortb_u64,
  BI__builtin_sve_sveortb_u8,
  BI__builtin_sve_sveorv_s16,
  BI__builtin_sve_sveorv_s32,
  BI__builtin_sve_sveorv_s64,
  BI__builtin_sve_sveorv_s8,
  BI__builtin_sve_sveorv_u16,
  BI__builtin_sve_sveorv_u32,
  BI__builtin_sve_sveorv_u64,
  BI__builtin_sve_sveorv_u8,
  BI__builtin_sve_svexpa_f16,
  BI__builtin_sve_svexpa_f32,
  BI__builtin_sve_svexpa_f64,
  BI__builtin_sve_svexpand_bf16,
  BI__builtin_sve_svexpand_f16,
  BI__builtin_sve_svexpand_f32,
  BI__builtin_sve_svexpand_f64,
  BI__builtin_sve_svexpand_mf8,
  BI__builtin_sve_svexpand_s16,
  BI__builtin_sve_svexpand_s32,
  BI__builtin_sve_svexpand_s64,
  BI__builtin_sve_svexpand_s8,
  BI__builtin_sve_svexpand_u16,
  BI__builtin_sve_svexpand_u32,
  BI__builtin_sve_svexpand_u64,
  BI__builtin_sve_svexpand_u8,
  BI__builtin_sve_svext_bf16,
  BI__builtin_sve_svext_f16,
  BI__builtin_sve_svext_f32,
  BI__builtin_sve_svext_f64,
  BI__builtin_sve_svext_s16,
  BI__builtin_sve_svext_s32,
  BI__builtin_sve_svext_s64,
  BI__builtin_sve_svext_s8,
  BI__builtin_sve_svext_u16,
  BI__builtin_sve_svext_u32,
  BI__builtin_sve_svext_u64,
  BI__builtin_sve_svext_u8,
  BI__builtin_sve_svextb_s16_m,
  BI__builtin_sve_svextb_s16_x,
  BI__builtin_sve_svextb_s16_z,
  BI__builtin_sve_svextb_s32_m,
  BI__builtin_sve_svextb_s32_x,
  BI__builtin_sve_svextb_s32_z,
  BI__builtin_sve_svextb_s64_m,
  BI__builtin_sve_svextb_s64_x,
  BI__builtin_sve_svextb_s64_z,
  BI__builtin_sve_svextb_u16_m,
  BI__builtin_sve_svextb_u16_x,
  BI__builtin_sve_svextb_u16_z,
  BI__builtin_sve_svextb_u32_m,
  BI__builtin_sve_svextb_u32_x,
  BI__builtin_sve_svextb_u32_z,
  BI__builtin_sve_svextb_u64_m,
  BI__builtin_sve_svextb_u64_x,
  BI__builtin_sve_svextb_u64_z,
  BI__builtin_sve_svexth_s32_m,
  BI__builtin_sve_svexth_s32_x,
  BI__builtin_sve_svexth_s32_z,
  BI__builtin_sve_svexth_s64_m,
  BI__builtin_sve_svexth_s64_x,
  BI__builtin_sve_svexth_s64_z,
  BI__builtin_sve_svexth_u32_m,
  BI__builtin_sve_svexth_u32_x,
  BI__builtin_sve_svexth_u32_z,
  BI__builtin_sve_svexth_u64_m,
  BI__builtin_sve_svexth_u64_x,
  BI__builtin_sve_svexth_u64_z,
  BI__builtin_sve_svextq_bf16,
  BI__builtin_sve_svextq_f16,
  BI__builtin_sve_svextq_f32,
  BI__builtin_sve_svextq_f64,
  BI__builtin_sve_svextq_mf8,
  BI__builtin_sve_svextq_s16,
  BI__builtin_sve_svextq_s32,
  BI__builtin_sve_svextq_s64,
  BI__builtin_sve_svextq_s8,
  BI__builtin_sve_svextq_u16,
  BI__builtin_sve_svextq_u32,
  BI__builtin_sve_svextq_u64,
  BI__builtin_sve_svextq_u8,
  BI__builtin_sve_svextw_s64_m,
  BI__builtin_sve_svextw_s64_x,
  BI__builtin_sve_svextw_s64_z,
  BI__builtin_sve_svextw_u64_m,
  BI__builtin_sve_svextw_u64_x,
  BI__builtin_sve_svextw_u64_z,
  BI__builtin_sve_svfirstp_b16,
  BI__builtin_sve_svfirstp_b32,
  BI__builtin_sve_svfirstp_b64,
  BI__builtin_sve_svfirstp_b8,
  BI__builtin_sve_svget2_b,
  BI__builtin_sve_svget2_bf16,
  BI__builtin_sve_svget2_f16,
  BI__builtin_sve_svget2_f32,
  BI__builtin_sve_svget2_f64,
  BI__builtin_sve_svget2_mf8,
  BI__builtin_sve_svget2_s16,
  BI__builtin_sve_svget2_s32,
  BI__builtin_sve_svget2_s64,
  BI__builtin_sve_svget2_s8,
  BI__builtin_sve_svget2_u16,
  BI__builtin_sve_svget2_u32,
  BI__builtin_sve_svget2_u64,
  BI__builtin_sve_svget2_u8,
  BI__builtin_sve_svget3_bf16,
  BI__builtin_sve_svget3_f16,
  BI__builtin_sve_svget3_f32,
  BI__builtin_sve_svget3_f64,
  BI__builtin_sve_svget3_mf8,
  BI__builtin_sve_svget3_s16,
  BI__builtin_sve_svget3_s32,
  BI__builtin_sve_svget3_s64,
  BI__builtin_sve_svget3_s8,
  BI__builtin_sve_svget3_u16,
  BI__builtin_sve_svget3_u32,
  BI__builtin_sve_svget3_u64,
  BI__builtin_sve_svget3_u8,
  BI__builtin_sve_svget4_b,
  BI__builtin_sve_svget4_bf16,
  BI__builtin_sve_svget4_f16,
  BI__builtin_sve_svget4_f32,
  BI__builtin_sve_svget4_f64,
  BI__builtin_sve_svget4_mf8,
  BI__builtin_sve_svget4_s16,
  BI__builtin_sve_svget4_s32,
  BI__builtin_sve_svget4_s64,
  BI__builtin_sve_svget4_s8,
  BI__builtin_sve_svget4_u16,
  BI__builtin_sve_svget4_u32,
  BI__builtin_sve_svget4_u64,
  BI__builtin_sve_svget4_u8,
  BI__builtin_sve_svhadd_n_s16_m,
  BI__builtin_sve_svhadd_n_s16_x,
  BI__builtin_sve_svhadd_n_s16_z,
  BI__builtin_sve_svhadd_n_s32_m,
  BI__builtin_sve_svhadd_n_s32_x,
  BI__builtin_sve_svhadd_n_s32_z,
  BI__builtin_sve_svhadd_n_s64_m,
  BI__builtin_sve_svhadd_n_s64_x,
  BI__builtin_sve_svhadd_n_s64_z,
  BI__builtin_sve_svhadd_n_s8_m,
  BI__builtin_sve_svhadd_n_s8_x,
  BI__builtin_sve_svhadd_n_s8_z,
  BI__builtin_sve_svhadd_n_u16_m,
  BI__builtin_sve_svhadd_n_u16_x,
  BI__builtin_sve_svhadd_n_u16_z,
  BI__builtin_sve_svhadd_n_u32_m,
  BI__builtin_sve_svhadd_n_u32_x,
  BI__builtin_sve_svhadd_n_u32_z,
  BI__builtin_sve_svhadd_n_u64_m,
  BI__builtin_sve_svhadd_n_u64_x,
  BI__builtin_sve_svhadd_n_u64_z,
  BI__builtin_sve_svhadd_n_u8_m,
  BI__builtin_sve_svhadd_n_u8_x,
  BI__builtin_sve_svhadd_n_u8_z,
  BI__builtin_sve_svhadd_s16_m,
  BI__builtin_sve_svhadd_s16_x,
  BI__builtin_sve_svhadd_s16_z,
  BI__builtin_sve_svhadd_s32_m,
  BI__builtin_sve_svhadd_s32_x,
  BI__builtin_sve_svhadd_s32_z,
  BI__builtin_sve_svhadd_s64_m,
  BI__builtin_sve_svhadd_s64_x,
  BI__builtin_sve_svhadd_s64_z,
  BI__builtin_sve_svhadd_s8_m,
  BI__builtin_sve_svhadd_s8_x,
  BI__builtin_sve_svhadd_s8_z,
  BI__builtin_sve_svhadd_u16_m,
  BI__builtin_sve_svhadd_u16_x,
  BI__builtin_sve_svhadd_u16_z,
  BI__builtin_sve_svhadd_u32_m,
  BI__builtin_sve_svhadd_u32_x,
  BI__builtin_sve_svhadd_u32_z,
  BI__builtin_sve_svhadd_u64_m,
  BI__builtin_sve_svhadd_u64_x,
  BI__builtin_sve_svhadd_u64_z,
  BI__builtin_sve_svhadd_u8_m,
  BI__builtin_sve_svhadd_u8_x,
  BI__builtin_sve_svhadd_u8_z,
  BI__builtin_sve_svhistcnt_s32_z,
  BI__builtin_sve_svhistcnt_s64_z,
  BI__builtin_sve_svhistcnt_u32_z,
  BI__builtin_sve_svhistcnt_u64_z,
  BI__builtin_sve_svhistseg_s8,
  BI__builtin_sve_svhistseg_u8,
  BI__builtin_sve_svhsub_n_s16_m,
  BI__builtin_sve_svhsub_n_s16_x,
  BI__builtin_sve_svhsub_n_s16_z,
  BI__builtin_sve_svhsub_n_s32_m,
  BI__builtin_sve_svhsub_n_s32_x,
  BI__builtin_sve_svhsub_n_s32_z,
  BI__builtin_sve_svhsub_n_s64_m,
  BI__builtin_sve_svhsub_n_s64_x,
  BI__builtin_sve_svhsub_n_s64_z,
  BI__builtin_sve_svhsub_n_s8_m,
  BI__builtin_sve_svhsub_n_s8_x,
  BI__builtin_sve_svhsub_n_s8_z,
  BI__builtin_sve_svhsub_n_u16_m,
  BI__builtin_sve_svhsub_n_u16_x,
  BI__builtin_sve_svhsub_n_u16_z,
  BI__builtin_sve_svhsub_n_u32_m,
  BI__builtin_sve_svhsub_n_u32_x,
  BI__builtin_sve_svhsub_n_u32_z,
  BI__builtin_sve_svhsub_n_u64_m,
  BI__builtin_sve_svhsub_n_u64_x,
  BI__builtin_sve_svhsub_n_u64_z,
  BI__builtin_sve_svhsub_n_u8_m,
  BI__builtin_sve_svhsub_n_u8_x,
  BI__builtin_sve_svhsub_n_u8_z,
  BI__builtin_sve_svhsub_s16_m,
  BI__builtin_sve_svhsub_s16_x,
  BI__builtin_sve_svhsub_s16_z,
  BI__builtin_sve_svhsub_s32_m,
  BI__builtin_sve_svhsub_s32_x,
  BI__builtin_sve_svhsub_s32_z,
  BI__builtin_sve_svhsub_s64_m,
  BI__builtin_sve_svhsub_s64_x,
  BI__builtin_sve_svhsub_s64_z,
  BI__builtin_sve_svhsub_s8_m,
  BI__builtin_sve_svhsub_s8_x,
  BI__builtin_sve_svhsub_s8_z,
  BI__builtin_sve_svhsub_u16_m,
  BI__builtin_sve_svhsub_u16_x,
  BI__builtin_sve_svhsub_u16_z,
  BI__builtin_sve_svhsub_u32_m,
  BI__builtin_sve_svhsub_u32_x,
  BI__builtin_sve_svhsub_u32_z,
  BI__builtin_sve_svhsub_u64_m,
  BI__builtin_sve_svhsub_u64_x,
  BI__builtin_sve_svhsub_u64_z,
  BI__builtin_sve_svhsub_u8_m,
  BI__builtin_sve_svhsub_u8_x,
  BI__builtin_sve_svhsub_u8_z,
  BI__builtin_sve_svhsubr_n_s16_m,
  BI__builtin_sve_svhsubr_n_s16_x,
  BI__builtin_sve_svhsubr_n_s16_z,
  BI__builtin_sve_svhsubr_n_s32_m,
  BI__builtin_sve_svhsubr_n_s32_x,
  BI__builtin_sve_svhsubr_n_s32_z,
  BI__builtin_sve_svhsubr_n_s64_m,
  BI__builtin_sve_svhsubr_n_s64_x,
  BI__builtin_sve_svhsubr_n_s64_z,
  BI__builtin_sve_svhsubr_n_s8_m,
  BI__builtin_sve_svhsubr_n_s8_x,
  BI__builtin_sve_svhsubr_n_s8_z,
  BI__builtin_sve_svhsubr_n_u16_m,
  BI__builtin_sve_svhsubr_n_u16_x,
  BI__builtin_sve_svhsubr_n_u16_z,
  BI__builtin_sve_svhsubr_n_u32_m,
  BI__builtin_sve_svhsubr_n_u32_x,
  BI__builtin_sve_svhsubr_n_u32_z,
  BI__builtin_sve_svhsubr_n_u64_m,
  BI__builtin_sve_svhsubr_n_u64_x,
  BI__builtin_sve_svhsubr_n_u64_z,
  BI__builtin_sve_svhsubr_n_u8_m,
  BI__builtin_sve_svhsubr_n_u8_x,
  BI__builtin_sve_svhsubr_n_u8_z,
  BI__builtin_sve_svhsubr_s16_m,
  BI__builtin_sve_svhsubr_s16_x,
  BI__builtin_sve_svhsubr_s16_z,
  BI__builtin_sve_svhsubr_s32_m,
  BI__builtin_sve_svhsubr_s32_x,
  BI__builtin_sve_svhsubr_s32_z,
  BI__builtin_sve_svhsubr_s64_m,
  BI__builtin_sve_svhsubr_s64_x,
  BI__builtin_sve_svhsubr_s64_z,
  BI__builtin_sve_svhsubr_s8_m,
  BI__builtin_sve_svhsubr_s8_x,
  BI__builtin_sve_svhsubr_s8_z,
  BI__builtin_sve_svhsubr_u16_m,
  BI__builtin_sve_svhsubr_u16_x,
  BI__builtin_sve_svhsubr_u16_z,
  BI__builtin_sve_svhsubr_u32_m,
  BI__builtin_sve_svhsubr_u32_x,
  BI__builtin_sve_svhsubr_u32_z,
  BI__builtin_sve_svhsubr_u64_m,
  BI__builtin_sve_svhsubr_u64_x,
  BI__builtin_sve_svhsubr_u64_z,
  BI__builtin_sve_svhsubr_u8_m,
  BI__builtin_sve_svhsubr_u8_x,
  BI__builtin_sve_svhsubr_u8_z,
  BI__builtin_sve_svindex_s16,
  BI__builtin_sve_svindex_s32,
  BI__builtin_sve_svindex_s64,
  BI__builtin_sve_svindex_s8,
  BI__builtin_sve_svindex_u16,
  BI__builtin_sve_svindex_u32,
  BI__builtin_sve_svindex_u64,
  BI__builtin_sve_svindex_u8,
  BI__builtin_sve_svinsr_n_bf16,
  BI__builtin_sve_svinsr_n_f16,
  BI__builtin_sve_svinsr_n_f32,
  BI__builtin_sve_svinsr_n_f64,
  BI__builtin_sve_svinsr_n_s16,
  BI__builtin_sve_svinsr_n_s32,
  BI__builtin_sve_svinsr_n_s64,
  BI__builtin_sve_svinsr_n_s8,
  BI__builtin_sve_svinsr_n_u16,
  BI__builtin_sve_svinsr_n_u32,
  BI__builtin_sve_svinsr_n_u64,
  BI__builtin_sve_svinsr_n_u8,
  BI__builtin_sve_svlasta_bf16,
  BI__builtin_sve_svlasta_f16,
  BI__builtin_sve_svlasta_f32,
  BI__builtin_sve_svlasta_f64,
  BI__builtin_sve_svlasta_s16,
  BI__builtin_sve_svlasta_s32,
  BI__builtin_sve_svlasta_s64,
  BI__builtin_sve_svlasta_s8,
  BI__builtin_sve_svlasta_u16,
  BI__builtin_sve_svlasta_u32,
  BI__builtin_sve_svlasta_u64,
  BI__builtin_sve_svlasta_u8,
  BI__builtin_sve_svlastb_bf16,
  BI__builtin_sve_svlastb_f16,
  BI__builtin_sve_svlastb_f32,
  BI__builtin_sve_svlastb_f64,
  BI__builtin_sve_svlastb_s16,
  BI__builtin_sve_svlastb_s32,
  BI__builtin_sve_svlastb_s64,
  BI__builtin_sve_svlastb_s8,
  BI__builtin_sve_svlastb_u16,
  BI__builtin_sve_svlastb_u32,
  BI__builtin_sve_svlastb_u64,
  BI__builtin_sve_svlastb_u8,
  BI__builtin_sve_svlastp_b16,
  BI__builtin_sve_svlastp_b32,
  BI__builtin_sve_svlastp_b64,
  BI__builtin_sve_svlastp_b8,
  BI__builtin_sve_svld1_bf16,
  BI__builtin_sve_svld1_bf16_x2,
  BI__builtin_sve_svld1_bf16_x4,
  BI__builtin_sve_svld1_f16,
  BI__builtin_sve_svld1_f16_x2,
  BI__builtin_sve_svld1_f16_x4,
  BI__builtin_sve_svld1_f32,
  BI__builtin_sve_svld1_f32_x2,
  BI__builtin_sve_svld1_f32_x4,
  BI__builtin_sve_svld1_f64,
  BI__builtin_sve_svld1_f64_x2,
  BI__builtin_sve_svld1_f64_x4,
  BI__builtin_sve_svld1_gather_s32index_f32,
  BI__builtin_sve_svld1_gather_s32index_s32,
  BI__builtin_sve_svld1_gather_s32index_u32,
  BI__builtin_sve_svld1_gather_s32offset_f32,
  BI__builtin_sve_svld1_gather_s32offset_s32,
  BI__builtin_sve_svld1_gather_s32offset_u32,
  BI__builtin_sve_svld1_gather_s64index_f64,
  BI__builtin_sve_svld1_gather_s64index_s64,
  BI__builtin_sve_svld1_gather_s64index_u64,
  BI__builtin_sve_svld1_gather_s64offset_f64,
  BI__builtin_sve_svld1_gather_s64offset_s64,
  BI__builtin_sve_svld1_gather_s64offset_u64,
  BI__builtin_sve_svld1_gather_u32base_f32,
  BI__builtin_sve_svld1_gather_u32base_index_f32,
  BI__builtin_sve_svld1_gather_u32base_index_s32,
  BI__builtin_sve_svld1_gather_u32base_index_u32,
  BI__builtin_sve_svld1_gather_u32base_offset_f32,
  BI__builtin_sve_svld1_gather_u32base_offset_s32,
  BI__builtin_sve_svld1_gather_u32base_offset_u32,
  BI__builtin_sve_svld1_gather_u32base_s32,
  BI__builtin_sve_svld1_gather_u32base_u32,
  BI__builtin_sve_svld1_gather_u32index_f32,
  BI__builtin_sve_svld1_gather_u32index_s32,
  BI__builtin_sve_svld1_gather_u32index_u32,
  BI__builtin_sve_svld1_gather_u32offset_f32,
  BI__builtin_sve_svld1_gather_u32offset_s32,
  BI__builtin_sve_svld1_gather_u32offset_u32,
  BI__builtin_sve_svld1_gather_u64base_f64,
  BI__builtin_sve_svld1_gather_u64base_index_f64,
  BI__builtin_sve_svld1_gather_u64base_index_s64,
  BI__builtin_sve_svld1_gather_u64base_index_u64,
  BI__builtin_sve_svld1_gather_u64base_offset_f64,
  BI__builtin_sve_svld1_gather_u64base_offset_s64,
  BI__builtin_sve_svld1_gather_u64base_offset_u64,
  BI__builtin_sve_svld1_gather_u64base_s64,
  BI__builtin_sve_svld1_gather_u64base_u64,
  BI__builtin_sve_svld1_gather_u64index_f64,
  BI__builtin_sve_svld1_gather_u64index_s64,
  BI__builtin_sve_svld1_gather_u64index_u64,
  BI__builtin_sve_svld1_gather_u64offset_f64,
  BI__builtin_sve_svld1_gather_u64offset_s64,
  BI__builtin_sve_svld1_gather_u64offset_u64,
  BI__builtin_sve_svld1_mf8,
  BI__builtin_sve_svld1_mf8_x2,
  BI__builtin_sve_svld1_mf8_x4,
  BI__builtin_sve_svld1_s16,
  BI__builtin_sve_svld1_s16_x2,
  BI__builtin_sve_svld1_s16_x4,
  BI__builtin_sve_svld1_s32,
  BI__builtin_sve_svld1_s32_x2,
  BI__builtin_sve_svld1_s32_x4,
  BI__builtin_sve_svld1_s64,
  BI__builtin_sve_svld1_s64_x2,
  BI__builtin_sve_svld1_s64_x4,
  BI__builtin_sve_svld1_s8,
  BI__builtin_sve_svld1_s8_x2,
  BI__builtin_sve_svld1_s8_x4,
  BI__builtin_sve_svld1_u16,
  BI__builtin_sve_svld1_u16_x2,
  BI__builtin_sve_svld1_u16_x4,
  BI__builtin_sve_svld1_u32,
  BI__builtin_sve_svld1_u32_x2,
  BI__builtin_sve_svld1_u32_x4,
  BI__builtin_sve_svld1_u64,
  BI__builtin_sve_svld1_u64_x2,
  BI__builtin_sve_svld1_u64_x4,
  BI__builtin_sve_svld1_u8,
  BI__builtin_sve_svld1_u8_x2,
  BI__builtin_sve_svld1_u8_x4,
  BI__builtin_sve_svld1_vnum_bf16,
  BI__builtin_sve_svld1_vnum_bf16_x2,
  BI__builtin_sve_svld1_vnum_bf16_x4,
  BI__builtin_sve_svld1_vnum_f16,
  BI__builtin_sve_svld1_vnum_f16_x2,
  BI__builtin_sve_svld1_vnum_f16_x4,
  BI__builtin_sve_svld1_vnum_f32,
  BI__builtin_sve_svld1_vnum_f32_x2,
  BI__builtin_sve_svld1_vnum_f32_x4,
  BI__builtin_sve_svld1_vnum_f64,
  BI__builtin_sve_svld1_vnum_f64_x2,
  BI__builtin_sve_svld1_vnum_f64_x4,
  BI__builtin_sve_svld1_vnum_mf8,
  BI__builtin_sve_svld1_vnum_mf8_x2,
  BI__builtin_sve_svld1_vnum_mf8_x4,
  BI__builtin_sve_svld1_vnum_s16,
  BI__builtin_sve_svld1_vnum_s16_x2,
  BI__builtin_sve_svld1_vnum_s16_x4,
  BI__builtin_sve_svld1_vnum_s32,
  BI__builtin_sve_svld1_vnum_s32_x2,
  BI__builtin_sve_svld1_vnum_s32_x4,
  BI__builtin_sve_svld1_vnum_s64,
  BI__builtin_sve_svld1_vnum_s64_x2,
  BI__builtin_sve_svld1_vnum_s64_x4,
  BI__builtin_sve_svld1_vnum_s8,
  BI__builtin_sve_svld1_vnum_s8_x2,
  BI__builtin_sve_svld1_vnum_s8_x4,
  BI__builtin_sve_svld1_vnum_u16,
  BI__builtin_sve_svld1_vnum_u16_x2,
  BI__builtin_sve_svld1_vnum_u16_x4,
  BI__builtin_sve_svld1_vnum_u32,
  BI__builtin_sve_svld1_vnum_u32_x2,
  BI__builtin_sve_svld1_vnum_u32_x4,
  BI__builtin_sve_svld1_vnum_u64,
  BI__builtin_sve_svld1_vnum_u64_x2,
  BI__builtin_sve_svld1_vnum_u64_x4,
  BI__builtin_sve_svld1_vnum_u8,
  BI__builtin_sve_svld1_vnum_u8_x2,
  BI__builtin_sve_svld1_vnum_u8_x4,
  BI__builtin_sve_svld1q_gather_u64base_bf16,
  BI__builtin_sve_svld1q_gather_u64base_f16,
  BI__builtin_sve_svld1q_gather_u64base_f32,
  BI__builtin_sve_svld1q_gather_u64base_f64,
  BI__builtin_sve_svld1q_gather_u64base_index_bf16,
  BI__builtin_sve_svld1q_gather_u64base_index_f16,
  BI__builtin_sve_svld1q_gather_u64base_index_f32,
  BI__builtin_sve_svld1q_gather_u64base_index_f64,
  BI__builtin_sve_svld1q_gather_u64base_index_s16,
  BI__builtin_sve_svld1q_gather_u64base_index_s32,
  BI__builtin_sve_svld1q_gather_u64base_index_s64,
  BI__builtin_sve_svld1q_gather_u64base_index_u16,
  BI__builtin_sve_svld1q_gather_u64base_index_u32,
  BI__builtin_sve_svld1q_gather_u64base_index_u64,
  BI__builtin_sve_svld1q_gather_u64base_mf8,
  BI__builtin_sve_svld1q_gather_u64base_offset_bf16,
  BI__builtin_sve_svld1q_gather_u64base_offset_f16,
  BI__builtin_sve_svld1q_gather_u64base_offset_f32,
  BI__builtin_sve_svld1q_gather_u64base_offset_f64,
  BI__builtin_sve_svld1q_gather_u64base_offset_mf8,
  BI__builtin_sve_svld1q_gather_u64base_offset_s16,
  BI__builtin_sve_svld1q_gather_u64base_offset_s32,
  BI__builtin_sve_svld1q_gather_u64base_offset_s64,
  BI__builtin_sve_svld1q_gather_u64base_offset_s8,
  BI__builtin_sve_svld1q_gather_u64base_offset_u16,
  BI__builtin_sve_svld1q_gather_u64base_offset_u32,
  BI__builtin_sve_svld1q_gather_u64base_offset_u64,
  BI__builtin_sve_svld1q_gather_u64base_offset_u8,
  BI__builtin_sve_svld1q_gather_u64base_s16,
  BI__builtin_sve_svld1q_gather_u64base_s32,
  BI__builtin_sve_svld1q_gather_u64base_s64,
  BI__builtin_sve_svld1q_gather_u64base_s8,
  BI__builtin_sve_svld1q_gather_u64base_u16,
  BI__builtin_sve_svld1q_gather_u64base_u32,
  BI__builtin_sve_svld1q_gather_u64base_u64,
  BI__builtin_sve_svld1q_gather_u64base_u8,
  BI__builtin_sve_svld1q_gather_u64index_bf16,
  BI__builtin_sve_svld1q_gather_u64index_f16,
  BI__builtin_sve_svld1q_gather_u64index_f32,
  BI__builtin_sve_svld1q_gather_u64index_f64,
  BI__builtin_sve_svld1q_gather_u64index_s16,
  BI__builtin_sve_svld1q_gather_u64index_s32,
  BI__builtin_sve_svld1q_gather_u64index_s64,
  BI__builtin_sve_svld1q_gather_u64index_u16,
  BI__builtin_sve_svld1q_gather_u64index_u32,
  BI__builtin_sve_svld1q_gather_u64index_u64,
  BI__builtin_sve_svld1q_gather_u64offset_bf16,
  BI__builtin_sve_svld1q_gather_u64offset_f16,
  BI__builtin_sve_svld1q_gather_u64offset_f32,
  BI__builtin_sve_svld1q_gather_u64offset_f64,
  BI__builtin_sve_svld1q_gather_u64offset_mf8,
  BI__builtin_sve_svld1q_gather_u64offset_s16,
  BI__builtin_sve_svld1q_gather_u64offset_s32,
  BI__builtin_sve_svld1q_gather_u64offset_s64,
  BI__builtin_sve_svld1q_gather_u64offset_s8,
  BI__builtin_sve_svld1q_gather_u64offset_u16,
  BI__builtin_sve_svld1q_gather_u64offset_u32,
  BI__builtin_sve_svld1q_gather_u64offset_u64,
  BI__builtin_sve_svld1q_gather_u64offset_u8,
  BI__builtin_sve_svld1ro_bf16,
  BI__builtin_sve_svld1ro_f16,
  BI__builtin_sve_svld1ro_f32,
  BI__builtin_sve_svld1ro_f64,
  BI__builtin_sve_svld1ro_mf8,
  BI__builtin_sve_svld1ro_s16,
  BI__builtin_sve_svld1ro_s32,
  BI__builtin_sve_svld1ro_s64,
  BI__builtin_sve_svld1ro_s8,
  BI__builtin_sve_svld1ro_u16,
  BI__builtin_sve_svld1ro_u32,
  BI__builtin_sve_svld1ro_u64,
  BI__builtin_sve_svld1ro_u8,
  BI__builtin_sve_svld1rq_bf16,
  BI__builtin_sve_svld1rq_f16,
  BI__builtin_sve_svld1rq_f32,
  BI__builtin_sve_svld1rq_f64,
  BI__builtin_sve_svld1rq_mf8,
  BI__builtin_sve_svld1rq_s16,
  BI__builtin_sve_svld1rq_s32,
  BI__builtin_sve_svld1rq_s64,
  BI__builtin_sve_svld1rq_s8,
  BI__builtin_sve_svld1rq_u16,
  BI__builtin_sve_svld1rq_u32,
  BI__builtin_sve_svld1rq_u64,
  BI__builtin_sve_svld1rq_u8,
  BI__builtin_sve_svld1sb_gather_s32offset_s32,
  BI__builtin_sve_svld1sb_gather_s32offset_u32,
  BI__builtin_sve_svld1sb_gather_s64offset_s64,
  BI__builtin_sve_svld1sb_gather_s64offset_u64,
  BI__builtin_sve_svld1sb_gather_u32base_offset_s32,
  BI__builtin_sve_svld1sb_gather_u32base_offset_u32,
  BI__builtin_sve_svld1sb_gather_u32base_s32,
  BI__builtin_sve_svld1sb_gather_u32base_u32,
  BI__builtin_sve_svld1sb_gather_u32offset_s32,
  BI__builtin_sve_svld1sb_gather_u32offset_u32,
  BI__builtin_sve_svld1sb_gather_u64base_offset_s64,
  BI__builtin_sve_svld1sb_gather_u64base_offset_u64,
  BI__builtin_sve_svld1sb_gather_u64base_s64,
  BI__builtin_sve_svld1sb_gather_u64base_u64,
  BI__builtin_sve_svld1sb_gather_u64offset_s64,
  BI__builtin_sve_svld1sb_gather_u64offset_u64,
  BI__builtin_sve_svld1sb_s16,
  BI__builtin_sve_svld1sb_s32,
  BI__builtin_sve_svld1sb_s64,
  BI__builtin_sve_svld1sb_u16,
  BI__builtin_sve_svld1sb_u32,
  BI__builtin_sve_svld1sb_u64,
  BI__builtin_sve_svld1sb_vnum_s16,
  BI__builtin_sve_svld1sb_vnum_s32,
  BI__builtin_sve_svld1sb_vnum_s64,
  BI__builtin_sve_svld1sb_vnum_u16,
  BI__builtin_sve_svld1sb_vnum_u32,
  BI__builtin_sve_svld1sb_vnum_u64,
  BI__builtin_sve_svld1sh_gather_s32index_s32,
  BI__builtin_sve_svld1sh_gather_s32index_u32,
  BI__builtin_sve_svld1sh_gather_s32offset_s32,
  BI__builtin_sve_svld1sh_gather_s32offset_u32,
  BI__builtin_sve_svld1sh_gather_s64index_s64,
  BI__builtin_sve_svld1sh_gather_s64index_u64,
  BI__builtin_sve_svld1sh_gather_s64offset_s64,
  BI__builtin_sve_svld1sh_gather_s64offset_u64,
  BI__builtin_sve_svld1sh_gather_u32base_index_s32,
  BI__builtin_sve_svld1sh_gather_u32base_index_u32,
  BI__builtin_sve_svld1sh_gather_u32base_offset_s32,
  BI__builtin_sve_svld1sh_gather_u32base_offset_u32,
  BI__builtin_sve_svld1sh_gather_u32base_s32,
  BI__builtin_sve_svld1sh_gather_u32base_u32,
  BI__builtin_sve_svld1sh_gather_u32index_s32,
  BI__builtin_sve_svld1sh_gather_u32index_u32,
  BI__builtin_sve_svld1sh_gather_u32offset_s32,
  BI__builtin_sve_svld1sh_gather_u32offset_u32,
  BI__builtin_sve_svld1sh_gather_u64base_index_s64,
  BI__builtin_sve_svld1sh_gather_u64base_index_u64,
  BI__builtin_sve_svld1sh_gather_u64base_offset_s64,
  BI__builtin_sve_svld1sh_gather_u64base_offset_u64,
  BI__builtin_sve_svld1sh_gather_u64base_s64,
  BI__builtin_sve_svld1sh_gather_u64base_u64,
  BI__builtin_sve_svld1sh_gather_u64index_s64,
  BI__builtin_sve_svld1sh_gather_u64index_u64,
  BI__builtin_sve_svld1sh_gather_u64offset_s64,
  BI__builtin_sve_svld1sh_gather_u64offset_u64,
  BI__builtin_sve_svld1sh_s32,
  BI__builtin_sve_svld1sh_s64,
  BI__builtin_sve_svld1sh_u32,
  BI__builtin_sve_svld1sh_u64,
  BI__builtin_sve_svld1sh_vnum_s32,
  BI__builtin_sve_svld1sh_vnum_s64,
  BI__builtin_sve_svld1sh_vnum_u32,
  BI__builtin_sve_svld1sh_vnum_u64,
  BI__builtin_sve_svld1sw_gather_s64index_s64,
  BI__builtin_sve_svld1sw_gather_s64index_u64,
  BI__builtin_sve_svld1sw_gather_s64offset_s64,
  BI__builtin_sve_svld1sw_gather_s64offset_u64,
  BI__builtin_sve_svld1sw_gather_u64base_index_s64,
  BI__builtin_sve_svld1sw_gather_u64base_index_u64,
  BI__builtin_sve_svld1sw_gather_u64base_offset_s64,
  BI__builtin_sve_svld1sw_gather_u64base_offset_u64,
  BI__builtin_sve_svld1sw_gather_u64base_s64,
  BI__builtin_sve_svld1sw_gather_u64base_u64,
  BI__builtin_sve_svld1sw_gather_u64index_s64,
  BI__builtin_sve_svld1sw_gather_u64index_u64,
  BI__builtin_sve_svld1sw_gather_u64offset_s64,
  BI__builtin_sve_svld1sw_gather_u64offset_u64,
  BI__builtin_sve_svld1sw_s64,
  BI__builtin_sve_svld1sw_u64,
  BI__builtin_sve_svld1sw_vnum_s64,
  BI__builtin_sve_svld1sw_vnum_u64,
  BI__builtin_sve_svld1ub_gather_s32offset_s32,
  BI__builtin_sve_svld1ub_gather_s32offset_u32,
  BI__builtin_sve_svld1ub_gather_s64offset_s64,
  BI__builtin_sve_svld1ub_gather_s64offset_u64,
  BI__builtin_sve_svld1ub_gather_u32base_offset_s32,
  BI__builtin_sve_svld1ub_gather_u32base_offset_u32,
  BI__builtin_sve_svld1ub_gather_u32base_s32,
  BI__builtin_sve_svld1ub_gather_u32base_u32,
  BI__builtin_sve_svld1ub_gather_u32offset_s32,
  BI__builtin_sve_svld1ub_gather_u32offset_u32,
  BI__builtin_sve_svld1ub_gather_u64base_offset_s64,
  BI__builtin_sve_svld1ub_gather_u64base_offset_u64,
  BI__builtin_sve_svld1ub_gather_u64base_s64,
  BI__builtin_sve_svld1ub_gather_u64base_u64,
  BI__builtin_sve_svld1ub_gather_u64offset_s64,
  BI__builtin_sve_svld1ub_gather_u64offset_u64,
  BI__builtin_sve_svld1ub_s16,
  BI__builtin_sve_svld1ub_s32,
  BI__builtin_sve_svld1ub_s64,
  BI__builtin_sve_svld1ub_u16,
  BI__builtin_sve_svld1ub_u32,
  BI__builtin_sve_svld1ub_u64,
  BI__builtin_sve_svld1ub_vnum_s16,
  BI__builtin_sve_svld1ub_vnum_s32,
  BI__builtin_sve_svld1ub_vnum_s64,
  BI__builtin_sve_svld1ub_vnum_u16,
  BI__builtin_sve_svld1ub_vnum_u32,
  BI__builtin_sve_svld1ub_vnum_u64,
  BI__builtin_sve_svld1udq_f64,
  BI__builtin_sve_svld1udq_s64,
  BI__builtin_sve_svld1udq_u64,
  BI__builtin_sve_svld1udq_vnum_f64,
  BI__builtin_sve_svld1udq_vnum_s64,
  BI__builtin_sve_svld1udq_vnum_u64,
  BI__builtin_sve_svld1uh_gather_s32index_s32,
  BI__builtin_sve_svld1uh_gather_s32index_u32,
  BI__builtin_sve_svld1uh_gather_s32offset_s32,
  BI__builtin_sve_svld1uh_gather_s32offset_u32,
  BI__builtin_sve_svld1uh_gather_s64index_s64,
  BI__builtin_sve_svld1uh_gather_s64index_u64,
  BI__builtin_sve_svld1uh_gather_s64offset_s64,
  BI__builtin_sve_svld1uh_gather_s64offset_u64,
  BI__builtin_sve_svld1uh_gather_u32base_index_s32,
  BI__builtin_sve_svld1uh_gather_u32base_index_u32,
  BI__builtin_sve_svld1uh_gather_u32base_offset_s32,
  BI__builtin_sve_svld1uh_gather_u32base_offset_u32,
  BI__builtin_sve_svld1uh_gather_u32base_s32,
  BI__builtin_sve_svld1uh_gather_u32base_u32,
  BI__builtin_sve_svld1uh_gather_u32index_s32,
  BI__builtin_sve_svld1uh_gather_u32index_u32,
  BI__builtin_sve_svld1uh_gather_u32offset_s32,
  BI__builtin_sve_svld1uh_gather_u32offset_u32,
  BI__builtin_sve_svld1uh_gather_u64base_index_s64,
  BI__builtin_sve_svld1uh_gather_u64base_index_u64,
  BI__builtin_sve_svld1uh_gather_u64base_offset_s64,
  BI__builtin_sve_svld1uh_gather_u64base_offset_u64,
  BI__builtin_sve_svld1uh_gather_u64base_s64,
  BI__builtin_sve_svld1uh_gather_u64base_u64,
  BI__builtin_sve_svld1uh_gather_u64index_s64,
  BI__builtin_sve_svld1uh_gather_u64index_u64,
  BI__builtin_sve_svld1uh_gather_u64offset_s64,
  BI__builtin_sve_svld1uh_gather_u64offset_u64,
  BI__builtin_sve_svld1uh_s32,
  BI__builtin_sve_svld1uh_s64,
  BI__builtin_sve_svld1uh_u32,
  BI__builtin_sve_svld1uh_u64,
  BI__builtin_sve_svld1uh_vnum_s32,
  BI__builtin_sve_svld1uh_vnum_s64,
  BI__builtin_sve_svld1uh_vnum_u32,
  BI__builtin_sve_svld1uh_vnum_u64,
  BI__builtin_sve_svld1uw_gather_s64index_s64,
  BI__builtin_sve_svld1uw_gather_s64index_u64,
  BI__builtin_sve_svld1uw_gather_s64offset_s64,
  BI__builtin_sve_svld1uw_gather_s64offset_u64,
  BI__builtin_sve_svld1uw_gather_u64base_index_s64,
  BI__builtin_sve_svld1uw_gather_u64base_index_u64,
  BI__builtin_sve_svld1uw_gather_u64base_offset_s64,
  BI__builtin_sve_svld1uw_gather_u64base_offset_u64,
  BI__builtin_sve_svld1uw_gather_u64base_s64,
  BI__builtin_sve_svld1uw_gather_u64base_u64,
  BI__builtin_sve_svld1uw_gather_u64index_s64,
  BI__builtin_sve_svld1uw_gather_u64index_u64,
  BI__builtin_sve_svld1uw_gather_u64offset_s64,
  BI__builtin_sve_svld1uw_gather_u64offset_u64,
  BI__builtin_sve_svld1uw_s64,
  BI__builtin_sve_svld1uw_u64,
  BI__builtin_sve_svld1uw_vnum_s64,
  BI__builtin_sve_svld1uw_vnum_u64,
  BI__builtin_sve_svld1uwq_f32,
  BI__builtin_sve_svld1uwq_s32,
  BI__builtin_sve_svld1uwq_u32,
  BI__builtin_sve_svld1uwq_vnum_f32,
  BI__builtin_sve_svld1uwq_vnum_s32,
  BI__builtin_sve_svld1uwq_vnum_u32,
  BI__builtin_sve_svld2_bf16,
  BI__builtin_sve_svld2_f16,
  BI__builtin_sve_svld2_f32,
  BI__builtin_sve_svld2_f64,
  BI__builtin_sve_svld2_mf8,
  BI__builtin_sve_svld2_s16,
  BI__builtin_sve_svld2_s32,
  BI__builtin_sve_svld2_s64,
  BI__builtin_sve_svld2_s8,
  BI__builtin_sve_svld2_u16,
  BI__builtin_sve_svld2_u32,
  BI__builtin_sve_svld2_u64,
  BI__builtin_sve_svld2_u8,
  BI__builtin_sve_svld2_vnum_bf16,
  BI__builtin_sve_svld2_vnum_f16,
  BI__builtin_sve_svld2_vnum_f32,
  BI__builtin_sve_svld2_vnum_f64,
  BI__builtin_sve_svld2_vnum_mf8,
  BI__builtin_sve_svld2_vnum_s16,
  BI__builtin_sve_svld2_vnum_s32,
  BI__builtin_sve_svld2_vnum_s64,
  BI__builtin_sve_svld2_vnum_s8,
  BI__builtin_sve_svld2_vnum_u16,
  BI__builtin_sve_svld2_vnum_u32,
  BI__builtin_sve_svld2_vnum_u64,
  BI__builtin_sve_svld2_vnum_u8,
  BI__builtin_sve_svld2q_bf16,
  BI__builtin_sve_svld2q_f16,
  BI__builtin_sve_svld2q_f32,
  BI__builtin_sve_svld2q_f64,
  BI__builtin_sve_svld2q_mf8,
  BI__builtin_sve_svld2q_s16,
  BI__builtin_sve_svld2q_s32,
  BI__builtin_sve_svld2q_s64,
  BI__builtin_sve_svld2q_s8,
  BI__builtin_sve_svld2q_u16,
  BI__builtin_sve_svld2q_u32,
  BI__builtin_sve_svld2q_u64,
  BI__builtin_sve_svld2q_u8,
  BI__builtin_sve_svld2q_vnum_bf16,
  BI__builtin_sve_svld2q_vnum_f16,
  BI__builtin_sve_svld2q_vnum_f32,
  BI__builtin_sve_svld2q_vnum_f64,
  BI__builtin_sve_svld2q_vnum_mf8,
  BI__builtin_sve_svld2q_vnum_s16,
  BI__builtin_sve_svld2q_vnum_s32,
  BI__builtin_sve_svld2q_vnum_s64,
  BI__builtin_sve_svld2q_vnum_s8,
  BI__builtin_sve_svld2q_vnum_u16,
  BI__builtin_sve_svld2q_vnum_u32,
  BI__builtin_sve_svld2q_vnum_u64,
  BI__builtin_sve_svld2q_vnum_u8,
  BI__builtin_sve_svld3_bf16,
  BI__builtin_sve_svld3_f16,
  BI__builtin_sve_svld3_f32,
  BI__builtin_sve_svld3_f64,
  BI__builtin_sve_svld3_mf8,
  BI__builtin_sve_svld3_s16,
  BI__builtin_sve_svld3_s32,
  BI__builtin_sve_svld3_s64,
  BI__builtin_sve_svld3_s8,
  BI__builtin_sve_svld3_u16,
  BI__builtin_sve_svld3_u32,
  BI__builtin_sve_svld3_u64,
  BI__builtin_sve_svld3_u8,
  BI__builtin_sve_svld3_vnum_bf16,
  BI__builtin_sve_svld3_vnum_f16,
  BI__builtin_sve_svld3_vnum_f32,
  BI__builtin_sve_svld3_vnum_f64,
  BI__builtin_sve_svld3_vnum_mf8,
  BI__builtin_sve_svld3_vnum_s16,
  BI__builtin_sve_svld3_vnum_s32,
  BI__builtin_sve_svld3_vnum_s64,
  BI__builtin_sve_svld3_vnum_s8,
  BI__builtin_sve_svld3_vnum_u16,
  BI__builtin_sve_svld3_vnum_u32,
  BI__builtin_sve_svld3_vnum_u64,
  BI__builtin_sve_svld3_vnum_u8,
  BI__builtin_sve_svld3q_bf16,
  BI__builtin_sve_svld3q_f16,
  BI__builtin_sve_svld3q_f32,
  BI__builtin_sve_svld3q_f64,
  BI__builtin_sve_svld3q_mf8,
  BI__builtin_sve_svld3q_s16,
  BI__builtin_sve_svld3q_s32,
  BI__builtin_sve_svld3q_s64,
  BI__builtin_sve_svld3q_s8,
  BI__builtin_sve_svld3q_u16,
  BI__builtin_sve_svld3q_u32,
  BI__builtin_sve_svld3q_u64,
  BI__builtin_sve_svld3q_u8,
  BI__builtin_sve_svld3q_vnum_bf16,
  BI__builtin_sve_svld3q_vnum_f16,
  BI__builtin_sve_svld3q_vnum_f32,
  BI__builtin_sve_svld3q_vnum_f64,
  BI__builtin_sve_svld3q_vnum_mf8,
  BI__builtin_sve_svld3q_vnum_s16,
  BI__builtin_sve_svld3q_vnum_s32,
  BI__builtin_sve_svld3q_vnum_s64,
  BI__builtin_sve_svld3q_vnum_s8,
  BI__builtin_sve_svld3q_vnum_u16,
  BI__builtin_sve_svld3q_vnum_u32,
  BI__builtin_sve_svld3q_vnum_u64,
  BI__builtin_sve_svld3q_vnum_u8,
  BI__builtin_sve_svld4_bf16,
  BI__builtin_sve_svld4_f16,
  BI__builtin_sve_svld4_f32,
  BI__builtin_sve_svld4_f64,
  BI__builtin_sve_svld4_mf8,
  BI__builtin_sve_svld4_s16,
  BI__builtin_sve_svld4_s32,
  BI__builtin_sve_svld4_s64,
  BI__builtin_sve_svld4_s8,
  BI__builtin_sve_svld4_u16,
  BI__builtin_sve_svld4_u32,
  BI__builtin_sve_svld4_u64,
  BI__builtin_sve_svld4_u8,
  BI__builtin_sve_svld4_vnum_bf16,
  BI__builtin_sve_svld4_vnum_f16,
  BI__builtin_sve_svld4_vnum_f32,
  BI__builtin_sve_svld4_vnum_f64,
  BI__builtin_sve_svld4_vnum_mf8,
  BI__builtin_sve_svld4_vnum_s16,
  BI__builtin_sve_svld4_vnum_s32,
  BI__builtin_sve_svld4_vnum_s64,
  BI__builtin_sve_svld4_vnum_s8,
  BI__builtin_sve_svld4_vnum_u16,
  BI__builtin_sve_svld4_vnum_u32,
  BI__builtin_sve_svld4_vnum_u64,
  BI__builtin_sve_svld4_vnum_u8,
  BI__builtin_sve_svld4q_bf16,
  BI__builtin_sve_svld4q_f16,
  BI__builtin_sve_svld4q_f32,
  BI__builtin_sve_svld4q_f64,
  BI__builtin_sve_svld4q_mf8,
  BI__builtin_sve_svld4q_s16,
  BI__builtin_sve_svld4q_s32,
  BI__builtin_sve_svld4q_s64,
  BI__builtin_sve_svld4q_s8,
  BI__builtin_sve_svld4q_u16,
  BI__builtin_sve_svld4q_u32,
  BI__builtin_sve_svld4q_u64,
  BI__builtin_sve_svld4q_u8,
  BI__builtin_sve_svld4q_vnum_bf16,
  BI__builtin_sve_svld4q_vnum_f16,
  BI__builtin_sve_svld4q_vnum_f32,
  BI__builtin_sve_svld4q_vnum_f64,
  BI__builtin_sve_svld4q_vnum_mf8,
  BI__builtin_sve_svld4q_vnum_s16,
  BI__builtin_sve_svld4q_vnum_s32,
  BI__builtin_sve_svld4q_vnum_s64,
  BI__builtin_sve_svld4q_vnum_s8,
  BI__builtin_sve_svld4q_vnum_u16,
  BI__builtin_sve_svld4q_vnum_u32,
  BI__builtin_sve_svld4q_vnum_u64,
  BI__builtin_sve_svld4q_vnum_u8,
  BI__builtin_sve_svldff1_bf16,
  BI__builtin_sve_svldff1_f16,
  BI__builtin_sve_svldff1_f32,
  BI__builtin_sve_svldff1_f64,
  BI__builtin_sve_svldff1_gather_s32index_f32,
  BI__builtin_sve_svldff1_gather_s32index_s32,
  BI__builtin_sve_svldff1_gather_s32index_u32,
  BI__builtin_sve_svldff1_gather_s32offset_f32,
  BI__builtin_sve_svldff1_gather_s32offset_s32,
  BI__builtin_sve_svldff1_gather_s32offset_u32,
  BI__builtin_sve_svldff1_gather_s64index_f64,
  BI__builtin_sve_svldff1_gather_s64index_s64,
  BI__builtin_sve_svldff1_gather_s64index_u64,
  BI__builtin_sve_svldff1_gather_s64offset_f64,
  BI__builtin_sve_svldff1_gather_s64offset_s64,
  BI__builtin_sve_svldff1_gather_s64offset_u64,
  BI__builtin_sve_svldff1_gather_u32base_f32,
  BI__builtin_sve_svldff1_gather_u32base_index_f32,
  BI__builtin_sve_svldff1_gather_u32base_index_s32,
  BI__builtin_sve_svldff1_gather_u32base_index_u32,
  BI__builtin_sve_svldff1_gather_u32base_offset_f32,
  BI__builtin_sve_svldff1_gather_u32base_offset_s32,
  BI__builtin_sve_svldff1_gather_u32base_offset_u32,
  BI__builtin_sve_svldff1_gather_u32base_s32,
  BI__builtin_sve_svldff1_gather_u32base_u32,
  BI__builtin_sve_svldff1_gather_u32index_f32,
  BI__builtin_sve_svldff1_gather_u32index_s32,
  BI__builtin_sve_svldff1_gather_u32index_u32,
  BI__builtin_sve_svldff1_gather_u32offset_f32,
  BI__builtin_sve_svldff1_gather_u32offset_s32,
  BI__builtin_sve_svldff1_gather_u32offset_u32,
  BI__builtin_sve_svldff1_gather_u64base_f64,
  BI__builtin_sve_svldff1_gather_u64base_index_f64,
  BI__builtin_sve_svldff1_gather_u64base_index_s64,
  BI__builtin_sve_svldff1_gather_u64base_index_u64,
  BI__builtin_sve_svldff1_gather_u64base_offset_f64,
  BI__builtin_sve_svldff1_gather_u64base_offset_s64,
  BI__builtin_sve_svldff1_gather_u64base_offset_u64,
  BI__builtin_sve_svldff1_gather_u64base_s64,
  BI__builtin_sve_svldff1_gather_u64base_u64,
  BI__builtin_sve_svldff1_gather_u64index_f64,
  BI__builtin_sve_svldff1_gather_u64index_s64,
  BI__builtin_sve_svldff1_gather_u64index_u64,
  BI__builtin_sve_svldff1_gather_u64offset_f64,
  BI__builtin_sve_svldff1_gather_u64offset_s64,
  BI__builtin_sve_svldff1_gather_u64offset_u64,
  BI__builtin_sve_svldff1_mf8,
  BI__builtin_sve_svldff1_s16,
  BI__builtin_sve_svldff1_s32,
  BI__builtin_sve_svldff1_s64,
  BI__builtin_sve_svldff1_s8,
  BI__builtin_sve_svldff1_u16,
  BI__builtin_sve_svldff1_u32,
  BI__builtin_sve_svldff1_u64,
  BI__builtin_sve_svldff1_u8,
  BI__builtin_sve_svldff1_vnum_bf16,
  BI__builtin_sve_svldff1_vnum_f16,
  BI__builtin_sve_svldff1_vnum_f32,
  BI__builtin_sve_svldff1_vnum_f64,
  BI__builtin_sve_svldff1_vnum_mf8,
  BI__builtin_sve_svldff1_vnum_s16,
  BI__builtin_sve_svldff1_vnum_s32,
  BI__builtin_sve_svldff1_vnum_s64,
  BI__builtin_sve_svldff1_vnum_s8,
  BI__builtin_sve_svldff1_vnum_u16,
  BI__builtin_sve_svldff1_vnum_u32,
  BI__builtin_sve_svldff1_vnum_u64,
  BI__builtin_sve_svldff1_vnum_u8,
  BI__builtin_sve_svldff1sb_gather_s32offset_s32,
  BI__builtin_sve_svldff1sb_gather_s32offset_u32,
  BI__builtin_sve_svldff1sb_gather_s64offset_s64,
  BI__builtin_sve_svldff1sb_gather_s64offset_u64,
  BI__builtin_sve_svldff1sb_gather_u32base_offset_s32,
  BI__builtin_sve_svldff1sb_gather_u32base_offset_u32,
  BI__builtin_sve_svldff1sb_gather_u32base_s32,
  BI__builtin_sve_svldff1sb_gather_u32base_u32,
  BI__builtin_sve_svldff1sb_gather_u32offset_s32,
  BI__builtin_sve_svldff1sb_gather_u32offset_u32,
  BI__builtin_sve_svldff1sb_gather_u64base_offset_s64,
  BI__builtin_sve_svldff1sb_gather_u64base_offset_u64,
  BI__builtin_sve_svldff1sb_gather_u64base_s64,
  BI__builtin_sve_svldff1sb_gather_u64base_u64,
  BI__builtin_sve_svldff1sb_gather_u64offset_s64,
  BI__builtin_sve_svldff1sb_gather_u64offset_u64,
  BI__builtin_sve_svldff1sb_s16,
  BI__builtin_sve_svldff1sb_s32,
  BI__builtin_sve_svldff1sb_s64,
  BI__builtin_sve_svldff1sb_u16,
  BI__builtin_sve_svldff1sb_u32,
  BI__builtin_sve_svldff1sb_u64,
  BI__builtin_sve_svldff1sb_vnum_s16,
  BI__builtin_sve_svldff1sb_vnum_s32,
  BI__builtin_sve_svldff1sb_vnum_s64,
  BI__builtin_sve_svldff1sb_vnum_u16,
  BI__builtin_sve_svldff1sb_vnum_u32,
  BI__builtin_sve_svldff1sb_vnum_u64,
  BI__builtin_sve_svldff1sh_gather_s32index_s32,
  BI__builtin_sve_svldff1sh_gather_s32index_u32,
  BI__builtin_sve_svldff1sh_gather_s32offset_s32,
  BI__builtin_sve_svldff1sh_gather_s32offset_u32,
  BI__builtin_sve_svldff1sh_gather_s64index_s64,
  BI__builtin_sve_svldff1sh_gather_s64index_u64,
  BI__builtin_sve_svldff1sh_gather_s64offset_s64,
  BI__builtin_sve_svldff1sh_gather_s64offset_u64,
  BI__builtin_sve_svldff1sh_gather_u32base_index_s32,
  BI__builtin_sve_svldff1sh_gather_u32base_index_u32,
  BI__builtin_sve_svldff1sh_gather_u32base_offset_s32,
  BI__builtin_sve_svldff1sh_gather_u32base_offset_u32,
  BI__builtin_sve_svldff1sh_gather_u32base_s32,
  BI__builtin_sve_svldff1sh_gather_u32base_u32,
  BI__builtin_sve_svldff1sh_gather_u32index_s32,
  BI__builtin_sve_svldff1sh_gather_u32index_u32,
  BI__builtin_sve_svldff1sh_gather_u32offset_s32,
  BI__builtin_sve_svldff1sh_gather_u32offset_u32,
  BI__builtin_sve_svldff1sh_gather_u64base_index_s64,
  BI__builtin_sve_svldff1sh_gather_u64base_index_u64,
  BI__builtin_sve_svldff1sh_gather_u64base_offset_s64,
  BI__builtin_sve_svldff1sh_gather_u64base_offset_u64,
  BI__builtin_sve_svldff1sh_gather_u64base_s64,
  BI__builtin_sve_svldff1sh_gather_u64base_u64,
  BI__builtin_sve_svldff1sh_gather_u64index_s64,
  BI__builtin_sve_svldff1sh_gather_u64index_u64,
  BI__builtin_sve_svldff1sh_gather_u64offset_s64,
  BI__builtin_sve_svldff1sh_gather_u64offset_u64,
  BI__builtin_sve_svldff1sh_s32,
  BI__builtin_sve_svldff1sh_s64,
  BI__builtin_sve_svldff1sh_u32,
  BI__builtin_sve_svldff1sh_u64,
  BI__builtin_sve_svldff1sh_vnum_s32,
  BI__builtin_sve_svldff1sh_vnum_s64,
  BI__builtin_sve_svldff1sh_vnum_u32,
  BI__builtin_sve_svldff1sh_vnum_u64,
  BI__builtin_sve_svldff1sw_gather_s64index_s64,
  BI__builtin_sve_svldff1sw_gather_s64index_u64,
  BI__builtin_sve_svldff1sw_gather_s64offset_s64,
  BI__builtin_sve_svldff1sw_gather_s64offset_u64,
  BI__builtin_sve_svldff1sw_gather_u64base_index_s64,
  BI__builtin_sve_svldff1sw_gather_u64base_index_u64,
  BI__builtin_sve_svldff1sw_gather_u64base_offset_s64,
  BI__builtin_sve_svldff1sw_gather_u64base_offset_u64,
  BI__builtin_sve_svldff1sw_gather_u64base_s64,
  BI__builtin_sve_svldff1sw_gather_u64base_u64,
  BI__builtin_sve_svldff1sw_gather_u64index_s64,
  BI__builtin_sve_svldff1sw_gather_u64index_u64,
  BI__builtin_sve_svldff1sw_gather_u64offset_s64,
  BI__builtin_sve_svldff1sw_gather_u64offset_u64,
  BI__builtin_sve_svldff1sw_s64,
  BI__builtin_sve_svldff1sw_u64,
  BI__builtin_sve_svldff1sw_vnum_s64,
  BI__builtin_sve_svldff1sw_vnum_u64,
  BI__builtin_sve_svldff1ub_gather_s32offset_s32,
  BI__builtin_sve_svldff1ub_gather_s32offset_u32,
  BI__builtin_sve_svldff1ub_gather_s64offset_s64,
  BI__builtin_sve_svldff1ub_gather_s64offset_u64,
  BI__builtin_sve_svldff1ub_gather_u32base_offset_s32,
  BI__builtin_sve_svldff1ub_gather_u32base_offset_u32,
  BI__builtin_sve_svldff1ub_gather_u32base_s32,
  BI__builtin_sve_svldff1ub_gather_u32base_u32,
  BI__builtin_sve_svldff1ub_gather_u32offset_s32,
  BI__builtin_sve_svldff1ub_gather_u32offset_u32,
  BI__builtin_sve_svldff1ub_gather_u64base_offset_s64,
  BI__builtin_sve_svldff1ub_gather_u64base_offset_u64,
  BI__builtin_sve_svldff1ub_gather_u64base_s64,
  BI__builtin_sve_svldff1ub_gather_u64base_u64,
  BI__builtin_sve_svldff1ub_gather_u64offset_s64,
  BI__builtin_sve_svldff1ub_gather_u64offset_u64,
  BI__builtin_sve_svldff1ub_s16,
  BI__builtin_sve_svldff1ub_s32,
  BI__builtin_sve_svldff1ub_s64,
  BI__builtin_sve_svldff1ub_u16,
  BI__builtin_sve_svldff1ub_u32,
  BI__builtin_sve_svldff1ub_u64,
  BI__builtin_sve_svldff1ub_vnum_s16,
  BI__builtin_sve_svldff1ub_vnum_s32,
  BI__builtin_sve_svldff1ub_vnum_s64,
  BI__builtin_sve_svldff1ub_vnum_u16,
  BI__builtin_sve_svldff1ub_vnum_u32,
  BI__builtin_sve_svldff1ub_vnum_u64,
  BI__builtin_sve_svldff1uh_gather_s32index_s32,
  BI__builtin_sve_svldff1uh_gather_s32index_u32,
  BI__builtin_sve_svldff1uh_gather_s32offset_s32,
  BI__builtin_sve_svldff1uh_gather_s32offset_u32,
  BI__builtin_sve_svldff1uh_gather_s64index_s64,
  BI__builtin_sve_svldff1uh_gather_s64index_u64,
  BI__builtin_sve_svldff1uh_gather_s64offset_s64,
  BI__builtin_sve_svldff1uh_gather_s64offset_u64,
  BI__builtin_sve_svldff1uh_gather_u32base_index_s32,
  BI__builtin_sve_svldff1uh_gather_u32base_index_u32,
  BI__builtin_sve_svldff1uh_gather_u32base_offset_s32,
  BI__builtin_sve_svldff1uh_gather_u32base_offset_u32,
  BI__builtin_sve_svldff1uh_gather_u32base_s32,
  BI__builtin_sve_svldff1uh_gather_u32base_u32,
  BI__builtin_sve_svldff1uh_gather_u32index_s32,
  BI__builtin_sve_svldff1uh_gather_u32index_u32,
  BI__builtin_sve_svldff1uh_gather_u32offset_s32,
  BI__builtin_sve_svldff1uh_gather_u32offset_u32,
  BI__builtin_sve_svldff1uh_gather_u64base_index_s64,
  BI__builtin_sve_svldff1uh_gather_u64base_index_u64,
  BI__builtin_sve_svldff1uh_gather_u64base_offset_s64,
  BI__builtin_sve_svldff1uh_gather_u64base_offset_u64,
  BI__builtin_sve_svldff1uh_gather_u64base_s64,
  BI__builtin_sve_svldff1uh_gather_u64base_u64,
  BI__builtin_sve_svldff1uh_gather_u64index_s64,
  BI__builtin_sve_svldff1uh_gather_u64index_u64,
  BI__builtin_sve_svldff1uh_gather_u64offset_s64,
  BI__builtin_sve_svldff1uh_gather_u64offset_u64,
  BI__builtin_sve_svldff1uh_s32,
  BI__builtin_sve_svldff1uh_s64,
  BI__builtin_sve_svldff1uh_u32,
  BI__builtin_sve_svldff1uh_u64,
  BI__builtin_sve_svldff1uh_vnum_s32,
  BI__builtin_sve_svldff1uh_vnum_s64,
  BI__builtin_sve_svldff1uh_vnum_u32,
  BI__builtin_sve_svldff1uh_vnum_u64,
  BI__builtin_sve_svldff1uw_gather_s64index_s64,
  BI__builtin_sve_svldff1uw_gather_s64index_u64,
  BI__builtin_sve_svldff1uw_gather_s64offset_s64,
  BI__builtin_sve_svldff1uw_gather_s64offset_u64,
  BI__builtin_sve_svldff1uw_gather_u64base_index_s64,
  BI__builtin_sve_svldff1uw_gather_u64base_index_u64,
  BI__builtin_sve_svldff1uw_gather_u64base_offset_s64,
  BI__builtin_sve_svldff1uw_gather_u64base_offset_u64,
  BI__builtin_sve_svldff1uw_gather_u64base_s64,
  BI__builtin_sve_svldff1uw_gather_u64base_u64,
  BI__builtin_sve_svldff1uw_gather_u64index_s64,
  BI__builtin_sve_svldff1uw_gather_u64index_u64,
  BI__builtin_sve_svldff1uw_gather_u64offset_s64,
  BI__builtin_sve_svldff1uw_gather_u64offset_u64,
  BI__builtin_sve_svldff1uw_s64,
  BI__builtin_sve_svldff1uw_u64,
  BI__builtin_sve_svldff1uw_vnum_s64,
  BI__builtin_sve_svldff1uw_vnum_u64,
  BI__builtin_sve_svldnf1_bf16,
  BI__builtin_sve_svldnf1_f16,
  BI__builtin_sve_svldnf1_f32,
  BI__builtin_sve_svldnf1_f64,
  BI__builtin_sve_svldnf1_mf8,
  BI__builtin_sve_svldnf1_s16,
  BI__builtin_sve_svldnf1_s32,
  BI__builtin_sve_svldnf1_s64,
  BI__builtin_sve_svldnf1_s8,
  BI__builtin_sve_svldnf1_u16,
  BI__builtin_sve_svldnf1_u32,
  BI__builtin_sve_svldnf1_u64,
  BI__builtin_sve_svldnf1_u8,
  BI__builtin_sve_svldnf1_vnum_bf16,
  BI__builtin_sve_svldnf1_vnum_f16,
  BI__builtin_sve_svldnf1_vnum_f32,
  BI__builtin_sve_svldnf1_vnum_f64,
  BI__builtin_sve_svldnf1_vnum_mf8,
  BI__builtin_sve_svldnf1_vnum_s16,
  BI__builtin_sve_svldnf1_vnum_s32,
  BI__builtin_sve_svldnf1_vnum_s64,
  BI__builtin_sve_svldnf1_vnum_s8,
  BI__builtin_sve_svldnf1_vnum_u16,
  BI__builtin_sve_svldnf1_vnum_u32,
  BI__builtin_sve_svldnf1_vnum_u64,
  BI__builtin_sve_svldnf1_vnum_u8,
  BI__builtin_sve_svldnf1sb_s16,
  BI__builtin_sve_svldnf1sb_s32,
  BI__builtin_sve_svldnf1sb_s64,
  BI__builtin_sve_svldnf1sb_u16,
  BI__builtin_sve_svldnf1sb_u32,
  BI__builtin_sve_svldnf1sb_u64,
  BI__builtin_sve_svldnf1sb_vnum_s16,
  BI__builtin_sve_svldnf1sb_vnum_s32,
  BI__builtin_sve_svldnf1sb_vnum_s64,
  BI__builtin_sve_svldnf1sb_vnum_u16,
  BI__builtin_sve_svldnf1sb_vnum_u32,
  BI__builtin_sve_svldnf1sb_vnum_u64,
  BI__builtin_sve_svldnf1sh_s32,
  BI__builtin_sve_svldnf1sh_s64,
  BI__builtin_sve_svldnf1sh_u32,
  BI__builtin_sve_svldnf1sh_u64,
  BI__builtin_sve_svldnf1sh_vnum_s32,
  BI__builtin_sve_svldnf1sh_vnum_s64,
  BI__builtin_sve_svldnf1sh_vnum_u32,
  BI__builtin_sve_svldnf1sh_vnum_u64,
  BI__builtin_sve_svldnf1sw_s64,
  BI__builtin_sve_svldnf1sw_u64,
  BI__builtin_sve_svldnf1sw_vnum_s64,
  BI__builtin_sve_svldnf1sw_vnum_u64,
  BI__builtin_sve_svldnf1ub_s16,
  BI__builtin_sve_svldnf1ub_s32,
  BI__builtin_sve_svldnf1ub_s64,
  BI__builtin_sve_svldnf1ub_u16,
  BI__builtin_sve_svldnf1ub_u32,
  BI__builtin_sve_svldnf1ub_u64,
  BI__builtin_sve_svldnf1ub_vnum_s16,
  BI__builtin_sve_svldnf1ub_vnum_s32,
  BI__builtin_sve_svldnf1ub_vnum_s64,
  BI__builtin_sve_svldnf1ub_vnum_u16,
  BI__builtin_sve_svldnf1ub_vnum_u32,
  BI__builtin_sve_svldnf1ub_vnum_u64,
  BI__builtin_sve_svldnf1uh_s32,
  BI__builtin_sve_svldnf1uh_s64,
  BI__builtin_sve_svldnf1uh_u32,
  BI__builtin_sve_svldnf1uh_u64,
  BI__builtin_sve_svldnf1uh_vnum_s32,
  BI__builtin_sve_svldnf1uh_vnum_s64,
  BI__builtin_sve_svldnf1uh_vnum_u32,
  BI__builtin_sve_svldnf1uh_vnum_u64,
  BI__builtin_sve_svldnf1uw_s64,
  BI__builtin_sve_svldnf1uw_u64,
  BI__builtin_sve_svldnf1uw_vnum_s64,
  BI__builtin_sve_svldnf1uw_vnum_u64,
  BI__builtin_sve_svldnt1_bf16,
  BI__builtin_sve_svldnt1_bf16_x2,
  BI__builtin_sve_svldnt1_bf16_x4,
  BI__builtin_sve_svldnt1_f16,
  BI__builtin_sve_svldnt1_f16_x2,
  BI__builtin_sve_svldnt1_f16_x4,
  BI__builtin_sve_svldnt1_f32,
  BI__builtin_sve_svldnt1_f32_x2,
  BI__builtin_sve_svldnt1_f32_x4,
  BI__builtin_sve_svldnt1_f64,
  BI__builtin_sve_svldnt1_f64_x2,
  BI__builtin_sve_svldnt1_f64_x4,
  BI__builtin_sve_svldnt1_gather_s64index_f64,
  BI__builtin_sve_svldnt1_gather_s64index_s64,
  BI__builtin_sve_svldnt1_gather_s64index_u64,
  BI__builtin_sve_svldnt1_gather_s64offset_f64,
  BI__builtin_sve_svldnt1_gather_s64offset_s64,
  BI__builtin_sve_svldnt1_gather_s64offset_u64,
  BI__builtin_sve_svldnt1_gather_u32base_f32,
  BI__builtin_sve_svldnt1_gather_u32base_index_f32,
  BI__builtin_sve_svldnt1_gather_u32base_index_s32,
  BI__builtin_sve_svldnt1_gather_u32base_index_u32,
  BI__builtin_sve_svldnt1_gather_u32base_offset_f32,
  BI__builtin_sve_svldnt1_gather_u32base_offset_s32,
  BI__builtin_sve_svldnt1_gather_u32base_offset_u32,
  BI__builtin_sve_svldnt1_gather_u32base_s32,
  BI__builtin_sve_svldnt1_gather_u32base_u32,
  BI__builtin_sve_svldnt1_gather_u32offset_f32,
  BI__builtin_sve_svldnt1_gather_u32offset_s32,
  BI__builtin_sve_svldnt1_gather_u32offset_u32,
  BI__builtin_sve_svldnt1_gather_u64base_f64,
  BI__builtin_sve_svldnt1_gather_u64base_index_f64,
  BI__builtin_sve_svldnt1_gather_u64base_index_s64,
  BI__builtin_sve_svldnt1_gather_u64base_index_u64,
  BI__builtin_sve_svldnt1_gather_u64base_offset_f64,
  BI__builtin_sve_svldnt1_gather_u64base_offset_s64,
  BI__builtin_sve_svldnt1_gather_u64base_offset_u64,
  BI__builtin_sve_svldnt1_gather_u64base_s64,
  BI__builtin_sve_svldnt1_gather_u64base_u64,
  BI__builtin_sve_svldnt1_gather_u64index_f64,
  BI__builtin_sve_svldnt1_gather_u64index_s64,
  BI__builtin_sve_svldnt1_gather_u64index_u64,
  BI__builtin_sve_svldnt1_gather_u64offset_f64,
  BI__builtin_sve_svldnt1_gather_u64offset_s64,
  BI__builtin_sve_svldnt1_gather_u64offset_u64,
  BI__builtin_sve_svldnt1_mf8,
  BI__builtin_sve_svldnt1_mf8_x2,
  BI__builtin_sve_svldnt1_mf8_x4,
  BI__builtin_sve_svldnt1_s16,
  BI__builtin_sve_svldnt1_s16_x2,
  BI__builtin_sve_svldnt1_s16_x4,
  BI__builtin_sve_svldnt1_s32,
  BI__builtin_sve_svldnt1_s32_x2,
  BI__builtin_sve_svldnt1_s32_x4,
  BI__builtin_sve_svldnt1_s64,
  BI__builtin_sve_svldnt1_s64_x2,
  BI__builtin_sve_svldnt1_s64_x4,
  BI__builtin_sve_svldnt1_s8,
  BI__builtin_sve_svldnt1_s8_x2,
  BI__builtin_sve_svldnt1_s8_x4,
  BI__builtin_sve_svldnt1_u16,
  BI__builtin_sve_svldnt1_u16_x2,
  BI__builtin_sve_svldnt1_u16_x4,
  BI__builtin_sve_svldnt1_u32,
  BI__builtin_sve_svldnt1_u32_x2,
  BI__builtin_sve_svldnt1_u32_x4,
  BI__builtin_sve_svldnt1_u64,
  BI__builtin_sve_svldnt1_u64_x2,
  BI__builtin_sve_svldnt1_u64_x4,
  BI__builtin_sve_svldnt1_u8,
  BI__builtin_sve_svldnt1_u8_x2,
  BI__builtin_sve_svldnt1_u8_x4,
  BI__builtin_sve_svldnt1_vnum_bf16,
  BI__builtin_sve_svldnt1_vnum_bf16_x2,
  BI__builtin_sve_svldnt1_vnum_bf16_x4,
  BI__builtin_sve_svldnt1_vnum_f16,
  BI__builtin_sve_svldnt1_vnum_f16_x2,
  BI__builtin_sve_svldnt1_vnum_f16_x4,
  BI__builtin_sve_svldnt1_vnum_f32,
  BI__builtin_sve_svldnt1_vnum_f32_x2,
  BI__builtin_sve_svldnt1_vnum_f32_x4,
  BI__builtin_sve_svldnt1_vnum_f64,
  BI__builtin_sve_svldnt1_vnum_f64_x2,
  BI__builtin_sve_svldnt1_vnum_f64_x4,
  BI__builtin_sve_svldnt1_vnum_mf8,
  BI__builtin_sve_svldnt1_vnum_mf8_x2,
  BI__builtin_sve_svldnt1_vnum_mf8_x4,
  BI__builtin_sve_svldnt1_vnum_s16,
  BI__builtin_sve_svldnt1_vnum_s16_x2,
  BI__builtin_sve_svldnt1_vnum_s16_x4,
  BI__builtin_sve_svldnt1_vnum_s32,
  BI__builtin_sve_svldnt1_vnum_s32_x2,
  BI__builtin_sve_svldnt1_vnum_s32_x4,
  BI__builtin_sve_svldnt1_vnum_s64,
  BI__builtin_sve_svldnt1_vnum_s64_x2,
  BI__builtin_sve_svldnt1_vnum_s64_x4,
  BI__builtin_sve_svldnt1_vnum_s8,
  BI__builtin_sve_svldnt1_vnum_s8_x2,
  BI__builtin_sve_svldnt1_vnum_s8_x4,
  BI__builtin_sve_svldnt1_vnum_u16,
  BI__builtin_sve_svldnt1_vnum_u16_x2,
  BI__builtin_sve_svldnt1_vnum_u16_x4,
  BI__builtin_sve_svldnt1_vnum_u32,
  BI__builtin_sve_svldnt1_vnum_u32_x2,
  BI__builtin_sve_svldnt1_vnum_u32_x4,
  BI__builtin_sve_svldnt1_vnum_u64,
  BI__builtin_sve_svldnt1_vnum_u64_x2,
  BI__builtin_sve_svldnt1_vnum_u64_x4,
  BI__builtin_sve_svldnt1_vnum_u8,
  BI__builtin_sve_svldnt1_vnum_u8_x2,
  BI__builtin_sve_svldnt1_vnum_u8_x4,
  BI__builtin_sve_svldnt1sb_gather_s64offset_s64,
  BI__builtin_sve_svldnt1sb_gather_s64offset_u64,
  BI__builtin_sve_svldnt1sb_gather_u32base_offset_s32,
  BI__builtin_sve_svldnt1sb_gather_u32base_offset_u32,
  BI__builtin_sve_svldnt1sb_gather_u32base_s32,
  BI__builtin_sve_svldnt1sb_gather_u32base_u32,
  BI__builtin_sve_svldnt1sb_gather_u32offset_s32,
  BI__builtin_sve_svldnt1sb_gather_u32offset_u32,
  BI__builtin_sve_svldnt1sb_gather_u64base_offset_s64,
  BI__builtin_sve_svldnt1sb_gather_u64base_offset_u64,
  BI__builtin_sve_svldnt1sb_gather_u64base_s64,
  BI__builtin_sve_svldnt1sb_gather_u64base_u64,
  BI__builtin_sve_svldnt1sb_gather_u64offset_s64,
  BI__builtin_sve_svldnt1sb_gather_u64offset_u64,
  BI__builtin_sve_svldnt1sh_gather_s64index_s64,
  BI__builtin_sve_svldnt1sh_gather_s64index_u64,
  BI__builtin_sve_svldnt1sh_gather_s64offset_s64,
  BI__builtin_sve_svldnt1sh_gather_s64offset_u64,
  BI__builtin_sve_svldnt1sh_gather_u32base_index_s32,
  BI__builtin_sve_svldnt1sh_gather_u32base_index_u32,
  BI__builtin_sve_svldnt1sh_gather_u32base_offset_s32,
  BI__builtin_sve_svldnt1sh_gather_u32base_offset_u32,
  BI__builtin_sve_svldnt1sh_gather_u32base_s32,
  BI__builtin_sve_svldnt1sh_gather_u32base_u32,
  BI__builtin_sve_svldnt1sh_gather_u32offset_s32,
  BI__builtin_sve_svldnt1sh_gather_u32offset_u32,
  BI__builtin_sve_svldnt1sh_gather_u64base_index_s64,
  BI__builtin_sve_svldnt1sh_gather_u64base_index_u64,
  BI__builtin_sve_svldnt1sh_gather_u64base_offset_s64,
  BI__builtin_sve_svldnt1sh_gather_u64base_offset_u64,
  BI__builtin_sve_svldnt1sh_gather_u64base_s64,
  BI__builtin_sve_svldnt1sh_gather_u64base_u64,
  BI__builtin_sve_svldnt1sh_gather_u64index_s64,
  BI__builtin_sve_svldnt1sh_gather_u64index_u64,
  BI__builtin_sve_svldnt1sh_gather_u64offset_s64,
  BI__builtin_sve_svldnt1sh_gather_u64offset_u64,
  BI__builtin_sve_svldnt1sw_gather_s64index_s64,
  BI__builtin_sve_svldnt1sw_gather_s64index_u64,
  BI__builtin_sve_svldnt1sw_gather_s64offset_s64,
  BI__builtin_sve_svldnt1sw_gather_s64offset_u64,
  BI__builtin_sve_svldnt1sw_gather_u64base_index_s64,
  BI__builtin_sve_svldnt1sw_gather_u64base_index_u64,
  BI__builtin_sve_svldnt1sw_gather_u64base_offset_s64,
  BI__builtin_sve_svldnt1sw_gather_u64base_offset_u64,
  BI__builtin_sve_svldnt1sw_gather_u64base_s64,
  BI__builtin_sve_svldnt1sw_gather_u64base_u64,
  BI__builtin_sve_svldnt1sw_gather_u64index_s64,
  BI__builtin_sve_svldnt1sw_gather_u64index_u64,
  BI__builtin_sve_svldnt1sw_gather_u64offset_s64,
  BI__builtin_sve_svldnt1sw_gather_u64offset_u64,
  BI__builtin_sve_svldnt1ub_gather_s64offset_s64,
  BI__builtin_sve_svldnt1ub_gather_s64offset_u64,
  BI__builtin_sve_svldnt1ub_gather_u32base_offset_s32,
  BI__builtin_sve_svldnt1ub_gather_u32base_offset_u32,
  BI__builtin_sve_svldnt1ub_gather_u32base_s32,
  BI__builtin_sve_svldnt1ub_gather_u32base_u32,
  BI__builtin_sve_svldnt1ub_gather_u32offset_s32,
  BI__builtin_sve_svldnt1ub_gather_u32offset_u32,
  BI__builtin_sve_svldnt1ub_gather_u64base_offset_s64,
  BI__builtin_sve_svldnt1ub_gather_u64base_offset_u64,
  BI__builtin_sve_svldnt1ub_gather_u64base_s64,
  BI__builtin_sve_svldnt1ub_gather_u64base_u64,
  BI__builtin_sve_svldnt1ub_gather_u64offset_s64,
  BI__builtin_sve_svldnt1ub_gather_u64offset_u64,
  BI__builtin_sve_svldnt1uh_gather_s64index_s64,
  BI__builtin_sve_svldnt1uh_gather_s64index_u64,
  BI__builtin_sve_svldnt1uh_gather_s64offset_s64,
  BI__builtin_sve_svldnt1uh_gather_s64offset_u64,
  BI__builtin_sve_svldnt1uh_gather_u32base_index_s32,
  BI__builtin_sve_svldnt1uh_gather_u32base_index_u32,
  BI__builtin_sve_svldnt1uh_gather_u32base_offset_s32,
  BI__builtin_sve_svldnt1uh_gather_u32base_offset_u32,
  BI__builtin_sve_svldnt1uh_gather_u32base_s32,
  BI__builtin_sve_svldnt1uh_gather_u32base_u32,
  BI__builtin_sve_svldnt1uh_gather_u32offset_s32,
  BI__builtin_sve_svldnt1uh_gather_u32offset_u32,
  BI__builtin_sve_svldnt1uh_gather_u64base_index_s64,
  BI__builtin_sve_svldnt1uh_gather_u64base_index_u64,
  BI__builtin_sve_svldnt1uh_gather_u64base_offset_s64,
  BI__builtin_sve_svldnt1uh_gather_u64base_offset_u64,
  BI__builtin_sve_svldnt1uh_gather_u64base_s64,
  BI__builtin_sve_svldnt1uh_gather_u64base_u64,
  BI__builtin_sve_svldnt1uh_gather_u64index_s64,
  BI__builtin_sve_svldnt1uh_gather_u64index_u64,
  BI__builtin_sve_svldnt1uh_gather_u64offset_s64,
  BI__builtin_sve_svldnt1uh_gather_u64offset_u64,
  BI__builtin_sve_svldnt1uw_gather_s64index_s64,
  BI__builtin_sve_svldnt1uw_gather_s64index_u64,
  BI__builtin_sve_svldnt1uw_gather_s64offset_s64,
  BI__builtin_sve_svldnt1uw_gather_s64offset_u64,
  BI__builtin_sve_svldnt1uw_gather_u64base_index_s64,
  BI__builtin_sve_svldnt1uw_gather_u64base_index_u64,
  BI__builtin_sve_svldnt1uw_gather_u64base_offset_s64,
  BI__builtin_sve_svldnt1uw_gather_u64base_offset_u64,
  BI__builtin_sve_svldnt1uw_gather_u64base_s64,
  BI__builtin_sve_svldnt1uw_gather_u64base_u64,
  BI__builtin_sve_svldnt1uw_gather_u64index_s64,
  BI__builtin_sve_svldnt1uw_gather_u64index_u64,
  BI__builtin_sve_svldnt1uw_gather_u64offset_s64,
  BI__builtin_sve_svldnt1uw_gather_u64offset_u64,
  BI__builtin_sve_svlen_bf16,
  BI__builtin_sve_svlen_f16,
  BI__builtin_sve_svlen_f32,
  BI__builtin_sve_svlen_f64,
  BI__builtin_sve_svlen_s16,
  BI__builtin_sve_svlen_s32,
  BI__builtin_sve_svlen_s64,
  BI__builtin_sve_svlen_s8,
  BI__builtin_sve_svlen_u16,
  BI__builtin_sve_svlen_u32,
  BI__builtin_sve_svlen_u64,
  BI__builtin_sve_svlen_u8,
  BI__builtin_sve_svlogb_f16_m,
  BI__builtin_sve_svlogb_f16_x,
  BI__builtin_sve_svlogb_f16_z,
  BI__builtin_sve_svlogb_f32_m,
  BI__builtin_sve_svlogb_f32_x,
  BI__builtin_sve_svlogb_f32_z,
  BI__builtin_sve_svlogb_f64_m,
  BI__builtin_sve_svlogb_f64_x,
  BI__builtin_sve_svlogb_f64_z,
  BI__builtin_sve_svlsl_n_s16_m,
  BI__builtin_sve_svlsl_n_s16_x,
  BI__builtin_sve_svlsl_n_s16_z,
  BI__builtin_sve_svlsl_n_s32_m,
  BI__builtin_sve_svlsl_n_s32_x,
  BI__builtin_sve_svlsl_n_s32_z,
  BI__builtin_sve_svlsl_n_s64_m,
  BI__builtin_sve_svlsl_n_s64_x,
  BI__builtin_sve_svlsl_n_s64_z,
  BI__builtin_sve_svlsl_n_s8_m,
  BI__builtin_sve_svlsl_n_s8_x,
  BI__builtin_sve_svlsl_n_s8_z,
  BI__builtin_sve_svlsl_n_u16_m,
  BI__builtin_sve_svlsl_n_u16_x,
  BI__builtin_sve_svlsl_n_u16_z,
  BI__builtin_sve_svlsl_n_u32_m,
  BI__builtin_sve_svlsl_n_u32_x,
  BI__builtin_sve_svlsl_n_u32_z,
  BI__builtin_sve_svlsl_n_u64_m,
  BI__builtin_sve_svlsl_n_u64_x,
  BI__builtin_sve_svlsl_n_u64_z,
  BI__builtin_sve_svlsl_n_u8_m,
  BI__builtin_sve_svlsl_n_u8_x,
  BI__builtin_sve_svlsl_n_u8_z,
  BI__builtin_sve_svlsl_s16_m,
  BI__builtin_sve_svlsl_s16_x,
  BI__builtin_sve_svlsl_s16_z,
  BI__builtin_sve_svlsl_s32_m,
  BI__builtin_sve_svlsl_s32_x,
  BI__builtin_sve_svlsl_s32_z,
  BI__builtin_sve_svlsl_s64_m,
  BI__builtin_sve_svlsl_s64_x,
  BI__builtin_sve_svlsl_s64_z,
  BI__builtin_sve_svlsl_s8_m,
  BI__builtin_sve_svlsl_s8_x,
  BI__builtin_sve_svlsl_s8_z,
  BI__builtin_sve_svlsl_u16_m,
  BI__builtin_sve_svlsl_u16_x,
  BI__builtin_sve_svlsl_u16_z,
  BI__builtin_sve_svlsl_u32_m,
  BI__builtin_sve_svlsl_u32_x,
  BI__builtin_sve_svlsl_u32_z,
  BI__builtin_sve_svlsl_u64_m,
  BI__builtin_sve_svlsl_u64_x,
  BI__builtin_sve_svlsl_u64_z,
  BI__builtin_sve_svlsl_u8_m,
  BI__builtin_sve_svlsl_u8_x,
  BI__builtin_sve_svlsl_u8_z,
  BI__builtin_sve_svlsl_wide_n_s16_m,
  BI__builtin_sve_svlsl_wide_n_s16_x,
  BI__builtin_sve_svlsl_wide_n_s16_z,
  BI__builtin_sve_svlsl_wide_n_s32_m,
  BI__builtin_sve_svlsl_wide_n_s32_x,
  BI__builtin_sve_svlsl_wide_n_s32_z,
  BI__builtin_sve_svlsl_wide_n_s8_m,
  BI__builtin_sve_svlsl_wide_n_s8_x,
  BI__builtin_sve_svlsl_wide_n_s8_z,
  BI__builtin_sve_svlsl_wide_n_u16_m,
  BI__builtin_sve_svlsl_wide_n_u16_x,
  BI__builtin_sve_svlsl_wide_n_u16_z,
  BI__builtin_sve_svlsl_wide_n_u32_m,
  BI__builtin_sve_svlsl_wide_n_u32_x,
  BI__builtin_sve_svlsl_wide_n_u32_z,
  BI__builtin_sve_svlsl_wide_n_u8_m,
  BI__builtin_sve_svlsl_wide_n_u8_x,
  BI__builtin_sve_svlsl_wide_n_u8_z,
  BI__builtin_sve_svlsl_wide_s16_m,
  BI__builtin_sve_svlsl_wide_s16_x,
  BI__builtin_sve_svlsl_wide_s16_z,
  BI__builtin_sve_svlsl_wide_s32_m,
  BI__builtin_sve_svlsl_wide_s32_x,
  BI__builtin_sve_svlsl_wide_s32_z,
  BI__builtin_sve_svlsl_wide_s8_m,
  BI__builtin_sve_svlsl_wide_s8_x,
  BI__builtin_sve_svlsl_wide_s8_z,
  BI__builtin_sve_svlsl_wide_u16_m,
  BI__builtin_sve_svlsl_wide_u16_x,
  BI__builtin_sve_svlsl_wide_u16_z,
  BI__builtin_sve_svlsl_wide_u32_m,
  BI__builtin_sve_svlsl_wide_u32_x,
  BI__builtin_sve_svlsl_wide_u32_z,
  BI__builtin_sve_svlsl_wide_u8_m,
  BI__builtin_sve_svlsl_wide_u8_x,
  BI__builtin_sve_svlsl_wide_u8_z,
  BI__builtin_sve_svlsr_n_u16_m,
  BI__builtin_sve_svlsr_n_u16_x,
  BI__builtin_sve_svlsr_n_u16_z,
  BI__builtin_sve_svlsr_n_u32_m,
  BI__builtin_sve_svlsr_n_u32_x,
  BI__builtin_sve_svlsr_n_u32_z,
  BI__builtin_sve_svlsr_n_u64_m,
  BI__builtin_sve_svlsr_n_u64_x,
  BI__builtin_sve_svlsr_n_u64_z,
  BI__builtin_sve_svlsr_n_u8_m,
  BI__builtin_sve_svlsr_n_u8_x,
  BI__builtin_sve_svlsr_n_u8_z,
  BI__builtin_sve_svlsr_u16_m,
  BI__builtin_sve_svlsr_u16_x,
  BI__builtin_sve_svlsr_u16_z,
  BI__builtin_sve_svlsr_u32_m,
  BI__builtin_sve_svlsr_u32_x,
  BI__builtin_sve_svlsr_u32_z,
  BI__builtin_sve_svlsr_u64_m,
  BI__builtin_sve_svlsr_u64_x,
  BI__builtin_sve_svlsr_u64_z,
  BI__builtin_sve_svlsr_u8_m,
  BI__builtin_sve_svlsr_u8_x,
  BI__builtin_sve_svlsr_u8_z,
  BI__builtin_sve_svlsr_wide_n_u16_m,
  BI__builtin_sve_svlsr_wide_n_u16_x,
  BI__builtin_sve_svlsr_wide_n_u16_z,
  BI__builtin_sve_svlsr_wide_n_u32_m,
  BI__builtin_sve_svlsr_wide_n_u32_x,
  BI__builtin_sve_svlsr_wide_n_u32_z,
  BI__builtin_sve_svlsr_wide_n_u8_m,
  BI__builtin_sve_svlsr_wide_n_u8_x,
  BI__builtin_sve_svlsr_wide_n_u8_z,
  BI__builtin_sve_svlsr_wide_u16_m,
  BI__builtin_sve_svlsr_wide_u16_x,
  BI__builtin_sve_svlsr_wide_u16_z,
  BI__builtin_sve_svlsr_wide_u32_m,
  BI__builtin_sve_svlsr_wide_u32_x,
  BI__builtin_sve_svlsr_wide_u32_z,
  BI__builtin_sve_svlsr_wide_u8_m,
  BI__builtin_sve_svlsr_wide_u8_x,
  BI__builtin_sve_svlsr_wide_u8_z,
  BI__builtin_sve_svluti2_lane_bf16,
  BI__builtin_sve_svluti2_lane_f16,
  BI__builtin_sve_svluti2_lane_s16,
  BI__builtin_sve_svluti2_lane_s8,
  BI__builtin_sve_svluti2_lane_u16,
  BI__builtin_sve_svluti2_lane_u8,
  BI__builtin_sve_svluti4_lane_bf16,
  BI__builtin_sve_svluti4_lane_bf16_x2,
  BI__builtin_sve_svluti4_lane_f16,
  BI__builtin_sve_svluti4_lane_f16_x2,
  BI__builtin_sve_svluti4_lane_s16,
  BI__builtin_sve_svluti4_lane_s16_x2,
  BI__builtin_sve_svluti4_lane_s8,
  BI__builtin_sve_svluti4_lane_u16,
  BI__builtin_sve_svluti4_lane_u16_x2,
  BI__builtin_sve_svluti4_lane_u8,
  BI__builtin_sve_svmad_f16_m,
  BI__builtin_sve_svmad_f16_x,
  BI__builtin_sve_svmad_f16_z,
  BI__builtin_sve_svmad_f32_m,
  BI__builtin_sve_svmad_f32_x,
  BI__builtin_sve_svmad_f32_z,
  BI__builtin_sve_svmad_f64_m,
  BI__builtin_sve_svmad_f64_x,
  BI__builtin_sve_svmad_f64_z,
  BI__builtin_sve_svmad_n_f16_m,
  BI__builtin_sve_svmad_n_f16_x,
  BI__builtin_sve_svmad_n_f16_z,
  BI__builtin_sve_svmad_n_f32_m,
  BI__builtin_sve_svmad_n_f32_x,
  BI__builtin_sve_svmad_n_f32_z,
  BI__builtin_sve_svmad_n_f64_m,
  BI__builtin_sve_svmad_n_f64_x,
  BI__builtin_sve_svmad_n_f64_z,
  BI__builtin_sve_svmad_n_s16_m,
  BI__builtin_sve_svmad_n_s16_x,
  BI__builtin_sve_svmad_n_s16_z,
  BI__builtin_sve_svmad_n_s32_m,
  BI__builtin_sve_svmad_n_s32_x,
  BI__builtin_sve_svmad_n_s32_z,
  BI__builtin_sve_svmad_n_s64_m,
  BI__builtin_sve_svmad_n_s64_x,
  BI__builtin_sve_svmad_n_s64_z,
  BI__builtin_sve_svmad_n_s8_m,
  BI__builtin_sve_svmad_n_s8_x,
  BI__builtin_sve_svmad_n_s8_z,
  BI__builtin_sve_svmad_n_u16_m,
  BI__builtin_sve_svmad_n_u16_x,
  BI__builtin_sve_svmad_n_u16_z,
  BI__builtin_sve_svmad_n_u32_m,
  BI__builtin_sve_svmad_n_u32_x,
  BI__builtin_sve_svmad_n_u32_z,
  BI__builtin_sve_svmad_n_u64_m,
  BI__builtin_sve_svmad_n_u64_x,
  BI__builtin_sve_svmad_n_u64_z,
  BI__builtin_sve_svmad_n_u8_m,
  BI__builtin_sve_svmad_n_u8_x,
  BI__builtin_sve_svmad_n_u8_z,
  BI__builtin_sve_svmad_s16_m,
  BI__builtin_sve_svmad_s16_x,
  BI__builtin_sve_svmad_s16_z,
  BI__builtin_sve_svmad_s32_m,
  BI__builtin_sve_svmad_s32_x,
  BI__builtin_sve_svmad_s32_z,
  BI__builtin_sve_svmad_s64_m,
  BI__builtin_sve_svmad_s64_x,
  BI__builtin_sve_svmad_s64_z,
  BI__builtin_sve_svmad_s8_m,
  BI__builtin_sve_svmad_s8_x,
  BI__builtin_sve_svmad_s8_z,
  BI__builtin_sve_svmad_u16_m,
  BI__builtin_sve_svmad_u16_x,
  BI__builtin_sve_svmad_u16_z,
  BI__builtin_sve_svmad_u32_m,
  BI__builtin_sve_svmad_u32_x,
  BI__builtin_sve_svmad_u32_z,
  BI__builtin_sve_svmad_u64_m,
  BI__builtin_sve_svmad_u64_x,
  BI__builtin_sve_svmad_u64_z,
  BI__builtin_sve_svmad_u8_m,
  BI__builtin_sve_svmad_u8_x,
  BI__builtin_sve_svmad_u8_z,
  BI__builtin_sve_svmatch_s16,
  BI__builtin_sve_svmatch_s8,
  BI__builtin_sve_svmatch_u16,
  BI__builtin_sve_svmatch_u8,
  BI__builtin_sve_svmax_bf16_m,
  BI__builtin_sve_svmax_bf16_x,
  BI__builtin_sve_svmax_bf16_x2,
  BI__builtin_sve_svmax_bf16_x4,
  BI__builtin_sve_svmax_bf16_z,
  BI__builtin_sve_svmax_f16_m,
  BI__builtin_sve_svmax_f16_x,
  BI__builtin_sve_svmax_f16_x2,
  BI__builtin_sve_svmax_f16_x4,
  BI__builtin_sve_svmax_f16_z,
  BI__builtin_sve_svmax_f32_m,
  BI__builtin_sve_svmax_f32_x,
  BI__builtin_sve_svmax_f32_x2,
  BI__builtin_sve_svmax_f32_x4,
  BI__builtin_sve_svmax_f32_z,
  BI__builtin_sve_svmax_f64_m,
  BI__builtin_sve_svmax_f64_x,
  BI__builtin_sve_svmax_f64_x2,
  BI__builtin_sve_svmax_f64_x4,
  BI__builtin_sve_svmax_f64_z,
  BI__builtin_sve_svmax_n_bf16_m,
  BI__builtin_sve_svmax_n_bf16_x,
  BI__builtin_sve_svmax_n_bf16_z,
  BI__builtin_sve_svmax_n_f16_m,
  BI__builtin_sve_svmax_n_f16_x,
  BI__builtin_sve_svmax_n_f16_z,
  BI__builtin_sve_svmax_n_f32_m,
  BI__builtin_sve_svmax_n_f32_x,
  BI__builtin_sve_svmax_n_f32_z,
  BI__builtin_sve_svmax_n_f64_m,
  BI__builtin_sve_svmax_n_f64_x,
  BI__builtin_sve_svmax_n_f64_z,
  BI__builtin_sve_svmax_n_s16_m,
  BI__builtin_sve_svmax_n_s16_x,
  BI__builtin_sve_svmax_n_s16_z,
  BI__builtin_sve_svmax_n_s32_m,
  BI__builtin_sve_svmax_n_s32_x,
  BI__builtin_sve_svmax_n_s32_z,
  BI__builtin_sve_svmax_n_s64_m,
  BI__builtin_sve_svmax_n_s64_x,
  BI__builtin_sve_svmax_n_s64_z,
  BI__builtin_sve_svmax_n_s8_m,
  BI__builtin_sve_svmax_n_s8_x,
  BI__builtin_sve_svmax_n_s8_z,
  BI__builtin_sve_svmax_n_u16_m,
  BI__builtin_sve_svmax_n_u16_x,
  BI__builtin_sve_svmax_n_u16_z,
  BI__builtin_sve_svmax_n_u32_m,
  BI__builtin_sve_svmax_n_u32_x,
  BI__builtin_sve_svmax_n_u32_z,
  BI__builtin_sve_svmax_n_u64_m,
  BI__builtin_sve_svmax_n_u64_x,
  BI__builtin_sve_svmax_n_u64_z,
  BI__builtin_sve_svmax_n_u8_m,
  BI__builtin_sve_svmax_n_u8_x,
  BI__builtin_sve_svmax_n_u8_z,
  BI__builtin_sve_svmax_s16_m,
  BI__builtin_sve_svmax_s16_x,
  BI__builtin_sve_svmax_s16_x2,
  BI__builtin_sve_svmax_s16_x4,
  BI__builtin_sve_svmax_s16_z,
  BI__builtin_sve_svmax_s32_m,
  BI__builtin_sve_svmax_s32_x,
  BI__builtin_sve_svmax_s32_x2,
  BI__builtin_sve_svmax_s32_x4,
  BI__builtin_sve_svmax_s32_z,
  BI__builtin_sve_svmax_s64_m,
  BI__builtin_sve_svmax_s64_x,
  BI__builtin_sve_svmax_s64_x2,
  BI__builtin_sve_svmax_s64_x4,
  BI__builtin_sve_svmax_s64_z,
  BI__builtin_sve_svmax_s8_m,
  BI__builtin_sve_svmax_s8_x,
  BI__builtin_sve_svmax_s8_x2,
  BI__builtin_sve_svmax_s8_x4,
  BI__builtin_sve_svmax_s8_z,
  BI__builtin_sve_svmax_single_bf16_x2,
  BI__builtin_sve_svmax_single_bf16_x4,
  BI__builtin_sve_svmax_single_f16_x2,
  BI__builtin_sve_svmax_single_f16_x4,
  BI__builtin_sve_svmax_single_f32_x2,
  BI__builtin_sve_svmax_single_f32_x4,
  BI__builtin_sve_svmax_single_f64_x2,
  BI__builtin_sve_svmax_single_f64_x4,
  BI__builtin_sve_svmax_single_s16_x2,
  BI__builtin_sve_svmax_single_s16_x4,
  BI__builtin_sve_svmax_single_s32_x2,
  BI__builtin_sve_svmax_single_s32_x4,
  BI__builtin_sve_svmax_single_s64_x2,
  BI__builtin_sve_svmax_single_s64_x4,
  BI__builtin_sve_svmax_single_s8_x2,
  BI__builtin_sve_svmax_single_s8_x4,
  BI__builtin_sve_svmax_single_u16_x2,
  BI__builtin_sve_svmax_single_u16_x4,
  BI__builtin_sve_svmax_single_u32_x2,
  BI__builtin_sve_svmax_single_u32_x4,
  BI__builtin_sve_svmax_single_u64_x2,
  BI__builtin_sve_svmax_single_u64_x4,
  BI__builtin_sve_svmax_single_u8_x2,
  BI__builtin_sve_svmax_single_u8_x4,
  BI__builtin_sve_svmax_u16_m,
  BI__builtin_sve_svmax_u16_x,
  BI__builtin_sve_svmax_u16_x2,
  BI__builtin_sve_svmax_u16_x4,
  BI__builtin_sve_svmax_u16_z,
  BI__builtin_sve_svmax_u32_m,
  BI__builtin_sve_svmax_u32_x,
  BI__builtin_sve_svmax_u32_x2,
  BI__builtin_sve_svmax_u32_x4,
  BI__builtin_sve_svmax_u32_z,
  BI__builtin_sve_svmax_u64_m,
  BI__builtin_sve_svmax_u64_x,
  BI__builtin_sve_svmax_u64_x2,
  BI__builtin_sve_svmax_u64_x4,
  BI__builtin_sve_svmax_u64_z,
  BI__builtin_sve_svmax_u8_m,
  BI__builtin_sve_svmax_u8_x,
  BI__builtin_sve_svmax_u8_x2,
  BI__builtin_sve_svmax_u8_x4,
  BI__builtin_sve_svmax_u8_z,
  BI__builtin_sve_svmaxnm_bf16_m,
  BI__builtin_sve_svmaxnm_bf16_x,
  BI__builtin_sve_svmaxnm_bf16_x2,
  BI__builtin_sve_svmaxnm_bf16_x4,
  BI__builtin_sve_svmaxnm_bf16_z,
  BI__builtin_sve_svmaxnm_f16_m,
  BI__builtin_sve_svmaxnm_f16_x,
  BI__builtin_sve_svmaxnm_f16_x2,
  BI__builtin_sve_svmaxnm_f16_x4,
  BI__builtin_sve_svmaxnm_f16_z,
  BI__builtin_sve_svmaxnm_f32_m,
  BI__builtin_sve_svmaxnm_f32_x,
  BI__builtin_sve_svmaxnm_f32_x2,
  BI__builtin_sve_svmaxnm_f32_x4,
  BI__builtin_sve_svmaxnm_f32_z,
  BI__builtin_sve_svmaxnm_f64_m,
  BI__builtin_sve_svmaxnm_f64_x,
  BI__builtin_sve_svmaxnm_f64_x2,
  BI__builtin_sve_svmaxnm_f64_x4,
  BI__builtin_sve_svmaxnm_f64_z,
  BI__builtin_sve_svmaxnm_n_bf16_m,
  BI__builtin_sve_svmaxnm_n_bf16_x,
  BI__builtin_sve_svmaxnm_n_bf16_z,
  BI__builtin_sve_svmaxnm_n_f16_m,
  BI__builtin_sve_svmaxnm_n_f16_x,
  BI__builtin_sve_svmaxnm_n_f16_z,
  BI__builtin_sve_svmaxnm_n_f32_m,
  BI__builtin_sve_svmaxnm_n_f32_x,
  BI__builtin_sve_svmaxnm_n_f32_z,
  BI__builtin_sve_svmaxnm_n_f64_m,
  BI__builtin_sve_svmaxnm_n_f64_x,
  BI__builtin_sve_svmaxnm_n_f64_z,
  BI__builtin_sve_svmaxnm_single_bf16_x2,
  BI__builtin_sve_svmaxnm_single_bf16_x4,
  BI__builtin_sve_svmaxnm_single_f16_x2,
  BI__builtin_sve_svmaxnm_single_f16_x4,
  BI__builtin_sve_svmaxnm_single_f32_x2,
  BI__builtin_sve_svmaxnm_single_f32_x4,
  BI__builtin_sve_svmaxnm_single_f64_x2,
  BI__builtin_sve_svmaxnm_single_f64_x4,
  BI__builtin_sve_svmaxnmp_f16_m,
  BI__builtin_sve_svmaxnmp_f16_x,
  BI__builtin_sve_svmaxnmp_f32_m,
  BI__builtin_sve_svmaxnmp_f32_x,
  BI__builtin_sve_svmaxnmp_f64_m,
  BI__builtin_sve_svmaxnmp_f64_x,
  BI__builtin_sve_svmaxnmqv_f16,
  BI__builtin_sve_svmaxnmqv_f32,
  BI__builtin_sve_svmaxnmqv_f64,
  BI__builtin_sve_svmaxnmv_f16,
  BI__builtin_sve_svmaxnmv_f32,
  BI__builtin_sve_svmaxnmv_f64,
  BI__builtin_sve_svmaxp_f16_m,
  BI__builtin_sve_svmaxp_f16_x,
  BI__builtin_sve_svmaxp_f32_m,
  BI__builtin_sve_svmaxp_f32_x,
  BI__builtin_sve_svmaxp_f64_m,
  BI__builtin_sve_svmaxp_f64_x,
  BI__builtin_sve_svmaxp_s16_m,
  BI__builtin_sve_svmaxp_s16_x,
  BI__builtin_sve_svmaxp_s32_m,
  BI__builtin_sve_svmaxp_s32_x,
  BI__builtin_sve_svmaxp_s64_m,
  BI__builtin_sve_svmaxp_s64_x,
  BI__builtin_sve_svmaxp_s8_m,
  BI__builtin_sve_svmaxp_s8_x,
  BI__builtin_sve_svmaxp_u16_m,
  BI__builtin_sve_svmaxp_u16_x,
  BI__builtin_sve_svmaxp_u32_m,
  BI__builtin_sve_svmaxp_u32_x,
  BI__builtin_sve_svmaxp_u64_m,
  BI__builtin_sve_svmaxp_u64_x,
  BI__builtin_sve_svmaxp_u8_m,
  BI__builtin_sve_svmaxp_u8_x,
  BI__builtin_sve_svmaxqv_f16,
  BI__builtin_sve_svmaxqv_f32,
  BI__builtin_sve_svmaxqv_f64,
  BI__builtin_sve_svmaxqv_s16,
  BI__builtin_sve_svmaxqv_s32,
  BI__builtin_sve_svmaxqv_s64,
  BI__builtin_sve_svmaxqv_s8,
  BI__builtin_sve_svmaxqv_u16,
  BI__builtin_sve_svmaxqv_u32,
  BI__builtin_sve_svmaxqv_u64,
  BI__builtin_sve_svmaxqv_u8,
  BI__builtin_sve_svmaxv_f16,
  BI__builtin_sve_svmaxv_f32,
  BI__builtin_sve_svmaxv_f64,
  BI__builtin_sve_svmaxv_s16,
  BI__builtin_sve_svmaxv_s32,
  BI__builtin_sve_svmaxv_s64,
  BI__builtin_sve_svmaxv_s8,
  BI__builtin_sve_svmaxv_u16,
  BI__builtin_sve_svmaxv_u32,
  BI__builtin_sve_svmaxv_u64,
  BI__builtin_sve_svmaxv_u8,
  BI__builtin_sve_svmin_bf16_m,
  BI__builtin_sve_svmin_bf16_x,
  BI__builtin_sve_svmin_bf16_x2,
  BI__builtin_sve_svmin_bf16_x4,
  BI__builtin_sve_svmin_bf16_z,
  BI__builtin_sve_svmin_f16_m,
  BI__builtin_sve_svmin_f16_x,
  BI__builtin_sve_svmin_f16_x2,
  BI__builtin_sve_svmin_f16_x4,
  BI__builtin_sve_svmin_f16_z,
  BI__builtin_sve_svmin_f32_m,
  BI__builtin_sve_svmin_f32_x,
  BI__builtin_sve_svmin_f32_x2,
  BI__builtin_sve_svmin_f32_x4,
  BI__builtin_sve_svmin_f32_z,
  BI__builtin_sve_svmin_f64_m,
  BI__builtin_sve_svmin_f64_x,
  BI__builtin_sve_svmin_f64_x2,
  BI__builtin_sve_svmin_f64_x4,
  BI__builtin_sve_svmin_f64_z,
  BI__builtin_sve_svmin_n_bf16_m,
  BI__builtin_sve_svmin_n_bf16_x,
  BI__builtin_sve_svmin_n_bf16_z,
  BI__builtin_sve_svmin_n_f16_m,
  BI__builtin_sve_svmin_n_f16_x,
  BI__builtin_sve_svmin_n_f16_z,
  BI__builtin_sve_svmin_n_f32_m,
  BI__builtin_sve_svmin_n_f32_x,
  BI__builtin_sve_svmin_n_f32_z,
  BI__builtin_sve_svmin_n_f64_m,
  BI__builtin_sve_svmin_n_f64_x,
  BI__builtin_sve_svmin_n_f64_z,
  BI__builtin_sve_svmin_n_s16_m,
  BI__builtin_sve_svmin_n_s16_x,
  BI__builtin_sve_svmin_n_s16_z,
  BI__builtin_sve_svmin_n_s32_m,
  BI__builtin_sve_svmin_n_s32_x,
  BI__builtin_sve_svmin_n_s32_z,
  BI__builtin_sve_svmin_n_s64_m,
  BI__builtin_sve_svmin_n_s64_x,
  BI__builtin_sve_svmin_n_s64_z,
  BI__builtin_sve_svmin_n_s8_m,
  BI__builtin_sve_svmin_n_s8_x,
  BI__builtin_sve_svmin_n_s8_z,
  BI__builtin_sve_svmin_n_u16_m,
  BI__builtin_sve_svmin_n_u16_x,
  BI__builtin_sve_svmin_n_u16_z,
  BI__builtin_sve_svmin_n_u32_m,
  BI__builtin_sve_svmin_n_u32_x,
  BI__builtin_sve_svmin_n_u32_z,
  BI__builtin_sve_svmin_n_u64_m,
  BI__builtin_sve_svmin_n_u64_x,
  BI__builtin_sve_svmin_n_u64_z,
  BI__builtin_sve_svmin_n_u8_m,
  BI__builtin_sve_svmin_n_u8_x,
  BI__builtin_sve_svmin_n_u8_z,
  BI__builtin_sve_svmin_s16_m,
  BI__builtin_sve_svmin_s16_x,
  BI__builtin_sve_svmin_s16_x2,
  BI__builtin_sve_svmin_s16_x4,
  BI__builtin_sve_svmin_s16_z,
  BI__builtin_sve_svmin_s32_m,
  BI__builtin_sve_svmin_s32_x,
  BI__builtin_sve_svmin_s32_x2,
  BI__builtin_sve_svmin_s32_x4,
  BI__builtin_sve_svmin_s32_z,
  BI__builtin_sve_svmin_s64_m,
  BI__builtin_sve_svmin_s64_x,
  BI__builtin_sve_svmin_s64_x2,
  BI__builtin_sve_svmin_s64_x4,
  BI__builtin_sve_svmin_s64_z,
  BI__builtin_sve_svmin_s8_m,
  BI__builtin_sve_svmin_s8_x,
  BI__builtin_sve_svmin_s8_x2,
  BI__builtin_sve_svmin_s8_x4,
  BI__builtin_sve_svmin_s8_z,
  BI__builtin_sve_svmin_single_bf16_x2,
  BI__builtin_sve_svmin_single_bf16_x4,
  BI__builtin_sve_svmin_single_f16_x2,
  BI__builtin_sve_svmin_single_f16_x4,
  BI__builtin_sve_svmin_single_f32_x2,
  BI__builtin_sve_svmin_single_f32_x4,
  BI__builtin_sve_svmin_single_f64_x2,
  BI__builtin_sve_svmin_single_f64_x4,
  BI__builtin_sve_svmin_single_s16_x2,
  BI__builtin_sve_svmin_single_s16_x4,
  BI__builtin_sve_svmin_single_s32_x2,
  BI__builtin_sve_svmin_single_s32_x4,
  BI__builtin_sve_svmin_single_s64_x2,
  BI__builtin_sve_svmin_single_s64_x4,
  BI__builtin_sve_svmin_single_s8_x2,
  BI__builtin_sve_svmin_single_s8_x4,
  BI__builtin_sve_svmin_single_u16_x2,
  BI__builtin_sve_svmin_single_u16_x4,
  BI__builtin_sve_svmin_single_u32_x2,
  BI__builtin_sve_svmin_single_u32_x4,
  BI__builtin_sve_svmin_single_u64_x2,
  BI__builtin_sve_svmin_single_u64_x4,
  BI__builtin_sve_svmin_single_u8_x2,
  BI__builtin_sve_svmin_single_u8_x4,
  BI__builtin_sve_svmin_u16_m,
  BI__builtin_sve_svmin_u16_x,
  BI__builtin_sve_svmin_u16_x2,
  BI__builtin_sve_svmin_u16_x4,
  BI__builtin_sve_svmin_u16_z,
  BI__builtin_sve_svmin_u32_m,
  BI__builtin_sve_svmin_u32_x,
  BI__builtin_sve_svmin_u32_x2,
  BI__builtin_sve_svmin_u32_x4,
  BI__builtin_sve_svmin_u32_z,
  BI__builtin_sve_svmin_u64_m,
  BI__builtin_sve_svmin_u64_x,
  BI__builtin_sve_svmin_u64_x2,
  BI__builtin_sve_svmin_u64_x4,
  BI__builtin_sve_svmin_u64_z,
  BI__builtin_sve_svmin_u8_m,
  BI__builtin_sve_svmin_u8_x,
  BI__builtin_sve_svmin_u8_x2,
  BI__builtin_sve_svmin_u8_x4,
  BI__builtin_sve_svmin_u8_z,
  BI__builtin_sve_svminnm_bf16_m,
  BI__builtin_sve_svminnm_bf16_x,
  BI__builtin_sve_svminnm_bf16_x2,
  BI__builtin_sve_svminnm_bf16_x4,
  BI__builtin_sve_svminnm_bf16_z,
  BI__builtin_sve_svminnm_f16_m,
  BI__builtin_sve_svminnm_f16_x,
  BI__builtin_sve_svminnm_f16_x2,
  BI__builtin_sve_svminnm_f16_x4,
  BI__builtin_sve_svminnm_f16_z,
  BI__builtin_sve_svminnm_f32_m,
  BI__builtin_sve_svminnm_f32_x,
  BI__builtin_sve_svminnm_f32_x2,
  BI__builtin_sve_svminnm_f32_x4,
  BI__builtin_sve_svminnm_f32_z,
  BI__builtin_sve_svminnm_f64_m,
  BI__builtin_sve_svminnm_f64_x,
  BI__builtin_sve_svminnm_f64_x2,
  BI__builtin_sve_svminnm_f64_x4,
  BI__builtin_sve_svminnm_f64_z,
  BI__builtin_sve_svminnm_n_bf16_m,
  BI__builtin_sve_svminnm_n_bf16_x,
  BI__builtin_sve_svminnm_n_bf16_z,
  BI__builtin_sve_svminnm_n_f16_m,
  BI__builtin_sve_svminnm_n_f16_x,
  BI__builtin_sve_svminnm_n_f16_z,
  BI__builtin_sve_svminnm_n_f32_m,
  BI__builtin_sve_svminnm_n_f32_x,
  BI__builtin_sve_svminnm_n_f32_z,
  BI__builtin_sve_svminnm_n_f64_m,
  BI__builtin_sve_svminnm_n_f64_x,
  BI__builtin_sve_svminnm_n_f64_z,
  BI__builtin_sve_svminnm_single_bf16_x2,
  BI__builtin_sve_svminnm_single_bf16_x4,
  BI__builtin_sve_svminnm_single_f16_x2,
  BI__builtin_sve_svminnm_single_f16_x4,
  BI__builtin_sve_svminnm_single_f32_x2,
  BI__builtin_sve_svminnm_single_f32_x4,
  BI__builtin_sve_svminnm_single_f64_x2,
  BI__builtin_sve_svminnm_single_f64_x4,
  BI__builtin_sve_svminnmp_f16_m,
  BI__builtin_sve_svminnmp_f16_x,
  BI__builtin_sve_svminnmp_f32_m,
  BI__builtin_sve_svminnmp_f32_x,
  BI__builtin_sve_svminnmp_f64_m,
  BI__builtin_sve_svminnmp_f64_x,
  BI__builtin_sve_svminnmqv_f16,
  BI__builtin_sve_svminnmqv_f32,
  BI__builtin_sve_svminnmqv_f64,
  BI__builtin_sve_svminnmv_f16,
  BI__builtin_sve_svminnmv_f32,
  BI__builtin_sve_svminnmv_f64,
  BI__builtin_sve_svminp_f16_m,
  BI__builtin_sve_svminp_f16_x,
  BI__builtin_sve_svminp_f32_m,
  BI__builtin_sve_svminp_f32_x,
  BI__builtin_sve_svminp_f64_m,
  BI__builtin_sve_svminp_f64_x,
  BI__builtin_sve_svminp_s16_m,
  BI__builtin_sve_svminp_s16_x,
  BI__builtin_sve_svminp_s32_m,
  BI__builtin_sve_svminp_s32_x,
  BI__builtin_sve_svminp_s64_m,
  BI__builtin_sve_svminp_s64_x,
  BI__builtin_sve_svminp_s8_m,
  BI__builtin_sve_svminp_s8_x,
  BI__builtin_sve_svminp_u16_m,
  BI__builtin_sve_svminp_u16_x,
  BI__builtin_sve_svminp_u32_m,
  BI__builtin_sve_svminp_u32_x,
  BI__builtin_sve_svminp_u64_m,
  BI__builtin_sve_svminp_u64_x,
  BI__builtin_sve_svminp_u8_m,
  BI__builtin_sve_svminp_u8_x,
  BI__builtin_sve_svminqv_f16,
  BI__builtin_sve_svminqv_f32,
  BI__builtin_sve_svminqv_f64,
  BI__builtin_sve_svminqv_s16,
  BI__builtin_sve_svminqv_s32,
  BI__builtin_sve_svminqv_s64,
  BI__builtin_sve_svminqv_s8,
  BI__builtin_sve_svminqv_u16,
  BI__builtin_sve_svminqv_u32,
  BI__builtin_sve_svminqv_u64,
  BI__builtin_sve_svminqv_u8,
  BI__builtin_sve_svminv_f16,
  BI__builtin_sve_svminv_f32,
  BI__builtin_sve_svminv_f64,
  BI__builtin_sve_svminv_s16,
  BI__builtin_sve_svminv_s32,
  BI__builtin_sve_svminv_s64,
  BI__builtin_sve_svminv_s8,
  BI__builtin_sve_svminv_u16,
  BI__builtin_sve_svminv_u32,
  BI__builtin_sve_svminv_u64,
  BI__builtin_sve_svminv_u8,
  BI__builtin_sve_svmla_bf16_m,
  BI__builtin_sve_svmla_bf16_x,
  BI__builtin_sve_svmla_bf16_z,
  BI__builtin_sve_svmla_f16_m,
  BI__builtin_sve_svmla_f16_x,
  BI__builtin_sve_svmla_f16_z,
  BI__builtin_sve_svmla_f32_m,
  BI__builtin_sve_svmla_f32_x,
  BI__builtin_sve_svmla_f32_z,
  BI__builtin_sve_svmla_f64_m,
  BI__builtin_sve_svmla_f64_x,
  BI__builtin_sve_svmla_f64_z,
  BI__builtin_sve_svmla_lane_bf16,
  BI__builtin_sve_svmla_lane_f16,
  BI__builtin_sve_svmla_lane_f32,
  BI__builtin_sve_svmla_lane_f64,
  BI__builtin_sve_svmla_lane_s16,
  BI__builtin_sve_svmla_lane_s32,
  BI__builtin_sve_svmla_lane_s64,
  BI__builtin_sve_svmla_lane_u16,
  BI__builtin_sve_svmla_lane_u32,
  BI__builtin_sve_svmla_lane_u64,
  BI__builtin_sve_svmla_n_bf16_m,
  BI__builtin_sve_svmla_n_bf16_x,
  BI__builtin_sve_svmla_n_bf16_z,
  BI__builtin_sve_svmla_n_f16_m,
  BI__builtin_sve_svmla_n_f16_x,
  BI__builtin_sve_svmla_n_f16_z,
  BI__builtin_sve_svmla_n_f32_m,
  BI__builtin_sve_svmla_n_f32_x,
  BI__builtin_sve_svmla_n_f32_z,
  BI__builtin_sve_svmla_n_f64_m,
  BI__builtin_sve_svmla_n_f64_x,
  BI__builtin_sve_svmla_n_f64_z,
  BI__builtin_sve_svmla_n_s16_m,
  BI__builtin_sve_svmla_n_s16_x,
  BI__builtin_sve_svmla_n_s16_z,
  BI__builtin_sve_svmla_n_s32_m,
  BI__builtin_sve_svmla_n_s32_x,
  BI__builtin_sve_svmla_n_s32_z,
  BI__builtin_sve_svmla_n_s64_m,
  BI__builtin_sve_svmla_n_s64_x,
  BI__builtin_sve_svmla_n_s64_z,
  BI__builtin_sve_svmla_n_s8_m,
  BI__builtin_sve_svmla_n_s8_x,
  BI__builtin_sve_svmla_n_s8_z,
  BI__builtin_sve_svmla_n_u16_m,
  BI__builtin_sve_svmla_n_u16_x,
  BI__builtin_sve_svmla_n_u16_z,
  BI__builtin_sve_svmla_n_u32_m,
  BI__builtin_sve_svmla_n_u32_x,
  BI__builtin_sve_svmla_n_u32_z,
  BI__builtin_sve_svmla_n_u64_m,
  BI__builtin_sve_svmla_n_u64_x,
  BI__builtin_sve_svmla_n_u64_z,
  BI__builtin_sve_svmla_n_u8_m,
  BI__builtin_sve_svmla_n_u8_x,
  BI__builtin_sve_svmla_n_u8_z,
  BI__builtin_sve_svmla_s16_m,
  BI__builtin_sve_svmla_s16_x,
  BI__builtin_sve_svmla_s16_z,
  BI__builtin_sve_svmla_s32_m,
  BI__builtin_sve_svmla_s32_x,
  BI__builtin_sve_svmla_s32_z,
  BI__builtin_sve_svmla_s64_m,
  BI__builtin_sve_svmla_s64_x,
  BI__builtin_sve_svmla_s64_z,
  BI__builtin_sve_svmla_s8_m,
  BI__builtin_sve_svmla_s8_x,
  BI__builtin_sve_svmla_s8_z,
  BI__builtin_sve_svmla_u16_m,
  BI__builtin_sve_svmla_u16_x,
  BI__builtin_sve_svmla_u16_z,
  BI__builtin_sve_svmla_u32_m,
  BI__builtin_sve_svmla_u32_x,
  BI__builtin_sve_svmla_u32_z,
  BI__builtin_sve_svmla_u64_m,
  BI__builtin_sve_svmla_u64_x,
  BI__builtin_sve_svmla_u64_z,
  BI__builtin_sve_svmla_u8_m,
  BI__builtin_sve_svmla_u8_x,
  BI__builtin_sve_svmla_u8_z,
  BI__builtin_sve_svmlalb_f16_mf8_fpm,
  BI__builtin_sve_svmlalb_f32,
  BI__builtin_sve_svmlalb_lane_f16_mf8_fpm,
  BI__builtin_sve_svmlalb_lane_f32,
  BI__builtin_sve_svmlalb_lane_s32,
  BI__builtin_sve_svmlalb_lane_s64,
  BI__builtin_sve_svmlalb_lane_u32,
  BI__builtin_sve_svmlalb_lane_u64,
  BI__builtin_sve_svmlalb_n_f16_mf8_fpm,
  BI__builtin_sve_svmlalb_n_f32,
  BI__builtin_sve_svmlalb_n_s16,
  BI__builtin_sve_svmlalb_n_s32,
  BI__builtin_sve_svmlalb_n_s64,
  BI__builtin_sve_svmlalb_n_u16,
  BI__builtin_sve_svmlalb_n_u32,
  BI__builtin_sve_svmlalb_n_u64,
  BI__builtin_sve_svmlalb_s16,
  BI__builtin_sve_svmlalb_s32,
  BI__builtin_sve_svmlalb_s64,
  BI__builtin_sve_svmlalb_u16,
  BI__builtin_sve_svmlalb_u32,
  BI__builtin_sve_svmlalb_u64,
  BI__builtin_sve_svmlallbb_f32_mf8_fpm,
  BI__builtin_sve_svmlallbb_lane_f32_mf8_fpm,
  BI__builtin_sve_svmlallbb_n_f32_mf8_fpm,
  BI__builtin_sve_svmlallbt_f32_mf8_fpm,
  BI__builtin_sve_svmlallbt_lane_f32_mf8_fpm,
  BI__builtin_sve_svmlallbt_n_f32_mf8_fpm,
  BI__builtin_sve_svmlalltb_f32_mf8_fpm,
  BI__builtin_sve_svmlalltb_lane_f32_mf8_fpm,
  BI__builtin_sve_svmlalltb_n_f32_mf8_fpm,
  BI__builtin_sve_svmlalltt_f32_mf8_fpm,
  BI__builtin_sve_svmlalltt_lane_f32_mf8_fpm,
  BI__builtin_sve_svmlalltt_n_f32_mf8_fpm,
  BI__builtin_sve_svmlalt_f16_mf8_fpm,
  BI__builtin_sve_svmlalt_f32,
  BI__builtin_sve_svmlalt_lane_f16_mf8_fpm,
  BI__builtin_sve_svmlalt_lane_f32,
  BI__builtin_sve_svmlalt_lane_s32,
  BI__builtin_sve_svmlalt_lane_s64,
  BI__builtin_sve_svmlalt_lane_u32,
  BI__builtin_sve_svmlalt_lane_u64,
  BI__builtin_sve_svmlalt_n_f16_mf8_fpm,
  BI__builtin_sve_svmlalt_n_f32,
  BI__builtin_sve_svmlalt_n_s16,
  BI__builtin_sve_svmlalt_n_s32,
  BI__builtin_sve_svmlalt_n_s64,
  BI__builtin_sve_svmlalt_n_u16,
  BI__builtin_sve_svmlalt_n_u32,
  BI__builtin_sve_svmlalt_n_u64,
  BI__builtin_sve_svmlalt_s16,
  BI__builtin_sve_svmlalt_s32,
  BI__builtin_sve_svmlalt_s64,
  BI__builtin_sve_svmlalt_u16,
  BI__builtin_sve_svmlalt_u32,
  BI__builtin_sve_svmlalt_u64,
  BI__builtin_sve_svmls_bf16_m,
  BI__builtin_sve_svmls_bf16_x,
  BI__builtin_sve_svmls_bf16_z,
  BI__builtin_sve_svmls_f16_m,
  BI__builtin_sve_svmls_f16_x,
  BI__builtin_sve_svmls_f16_z,
  BI__builtin_sve_svmls_f32_m,
  BI__builtin_sve_svmls_f32_x,
  BI__builtin_sve_svmls_f32_z,
  BI__builtin_sve_svmls_f64_m,
  BI__builtin_sve_svmls_f64_x,
  BI__builtin_sve_svmls_f64_z,
  BI__builtin_sve_svmls_lane_bf16,
  BI__builtin_sve_svmls_lane_f16,
  BI__builtin_sve_svmls_lane_f32,
  BI__builtin_sve_svmls_lane_f64,
  BI__builtin_sve_svmls_lane_s16,
  BI__builtin_sve_svmls_lane_s32,
  BI__builtin_sve_svmls_lane_s64,
  BI__builtin_sve_svmls_lane_u16,
  BI__builtin_sve_svmls_lane_u32,
  BI__builtin_sve_svmls_lane_u64,
  BI__builtin_sve_svmls_n_bf16_m,
  BI__builtin_sve_svmls_n_bf16_x,
  BI__builtin_sve_svmls_n_bf16_z,
  BI__builtin_sve_svmls_n_f16_m,
  BI__builtin_sve_svmls_n_f16_x,
  BI__builtin_sve_svmls_n_f16_z,
  BI__builtin_sve_svmls_n_f32_m,
  BI__builtin_sve_svmls_n_f32_x,
  BI__builtin_sve_svmls_n_f32_z,
  BI__builtin_sve_svmls_n_f64_m,
  BI__builtin_sve_svmls_n_f64_x,
  BI__builtin_sve_svmls_n_f64_z,
  BI__builtin_sve_svmls_n_s16_m,
  BI__builtin_sve_svmls_n_s16_x,
  BI__builtin_sve_svmls_n_s16_z,
  BI__builtin_sve_svmls_n_s32_m,
  BI__builtin_sve_svmls_n_s32_x,
  BI__builtin_sve_svmls_n_s32_z,
  BI__builtin_sve_svmls_n_s64_m,
  BI__builtin_sve_svmls_n_s64_x,
  BI__builtin_sve_svmls_n_s64_z,
  BI__builtin_sve_svmls_n_s8_m,
  BI__builtin_sve_svmls_n_s8_x,
  BI__builtin_sve_svmls_n_s8_z,
  BI__builtin_sve_svmls_n_u16_m,
  BI__builtin_sve_svmls_n_u16_x,
  BI__builtin_sve_svmls_n_u16_z,
  BI__builtin_sve_svmls_n_u32_m,
  BI__builtin_sve_svmls_n_u32_x,
  BI__builtin_sve_svmls_n_u32_z,
  BI__builtin_sve_svmls_n_u64_m,
  BI__builtin_sve_svmls_n_u64_x,
  BI__builtin_sve_svmls_n_u64_z,
  BI__builtin_sve_svmls_n_u8_m,
  BI__builtin_sve_svmls_n_u8_x,
  BI__builtin_sve_svmls_n_u8_z,
  BI__builtin_sve_svmls_s16_m,
  BI__builtin_sve_svmls_s16_x,
  BI__builtin_sve_svmls_s16_z,
  BI__builtin_sve_svmls_s32_m,
  BI__builtin_sve_svmls_s32_x,
  BI__builtin_sve_svmls_s32_z,
  BI__builtin_sve_svmls_s64_m,
  BI__builtin_sve_svmls_s64_x,
  BI__builtin_sve_svmls_s64_z,
  BI__builtin_sve_svmls_s8_m,
  BI__builtin_sve_svmls_s8_x,
  BI__builtin_sve_svmls_s8_z,
  BI__builtin_sve_svmls_u16_m,
  BI__builtin_sve_svmls_u16_x,
  BI__builtin_sve_svmls_u16_z,
  BI__builtin_sve_svmls_u32_m,
  BI__builtin_sve_svmls_u32_x,
  BI__builtin_sve_svmls_u32_z,
  BI__builtin_sve_svmls_u64_m,
  BI__builtin_sve_svmls_u64_x,
  BI__builtin_sve_svmls_u64_z,
  BI__builtin_sve_svmls_u8_m,
  BI__builtin_sve_svmls_u8_x,
  BI__builtin_sve_svmls_u8_z,
  BI__builtin_sve_svmlslb_f32,
  BI__builtin_sve_svmlslb_lane_f32,
  BI__builtin_sve_svmlslb_lane_s32,
  BI__builtin_sve_svmlslb_lane_s64,
  BI__builtin_sve_svmlslb_lane_u32,
  BI__builtin_sve_svmlslb_lane_u64,
  BI__builtin_sve_svmlslb_n_f32,
  BI__builtin_sve_svmlslb_n_s16,
  BI__builtin_sve_svmlslb_n_s32,
  BI__builtin_sve_svmlslb_n_s64,
  BI__builtin_sve_svmlslb_n_u16,
  BI__builtin_sve_svmlslb_n_u32,
  BI__builtin_sve_svmlslb_n_u64,
  BI__builtin_sve_svmlslb_s16,
  BI__builtin_sve_svmlslb_s32,
  BI__builtin_sve_svmlslb_s64,
  BI__builtin_sve_svmlslb_u16,
  BI__builtin_sve_svmlslb_u32,
  BI__builtin_sve_svmlslb_u64,
  BI__builtin_sve_svmlslt_f32,
  BI__builtin_sve_svmlslt_lane_f32,
  BI__builtin_sve_svmlslt_lane_s32,
  BI__builtin_sve_svmlslt_lane_s64,
  BI__builtin_sve_svmlslt_lane_u32,
  BI__builtin_sve_svmlslt_lane_u64,
  BI__builtin_sve_svmlslt_n_f32,
  BI__builtin_sve_svmlslt_n_s16,
  BI__builtin_sve_svmlslt_n_s32,
  BI__builtin_sve_svmlslt_n_s64,
  BI__builtin_sve_svmlslt_n_u16,
  BI__builtin_sve_svmlslt_n_u32,
  BI__builtin_sve_svmlslt_n_u64,
  BI__builtin_sve_svmlslt_s16,
  BI__builtin_sve_svmlslt_s32,
  BI__builtin_sve_svmlslt_s64,
  BI__builtin_sve_svmlslt_u16,
  BI__builtin_sve_svmlslt_u32,
  BI__builtin_sve_svmlslt_u64,
  BI__builtin_sve_svmmla_f16_mf8_fpm,
  BI__builtin_sve_svmmla_f32,
  BI__builtin_sve_svmmla_f32_f16,
  BI__builtin_sve_svmmla_f32_mf8_fpm,
  BI__builtin_sve_svmmla_f64,
  BI__builtin_sve_svmmla_s32,
  BI__builtin_sve_svmmla_u32,
  BI__builtin_sve_svmov_b_z,
  BI__builtin_sve_svmovlb_s16,
  BI__builtin_sve_svmovlb_s32,
  BI__builtin_sve_svmovlb_s64,
  BI__builtin_sve_svmovlb_u16,
  BI__builtin_sve_svmovlb_u32,
  BI__builtin_sve_svmovlb_u64,
  BI__builtin_sve_svmovlt_s16,
  BI__builtin_sve_svmovlt_s32,
  BI__builtin_sve_svmovlt_s64,
  BI__builtin_sve_svmovlt_u16,
  BI__builtin_sve_svmovlt_u32,
  BI__builtin_sve_svmovlt_u64,
  BI__builtin_sve_svmsb_f16_m,
  BI__builtin_sve_svmsb_f16_x,
  BI__builtin_sve_svmsb_f16_z,
  BI__builtin_sve_svmsb_f32_m,
  BI__builtin_sve_svmsb_f32_x,
  BI__builtin_sve_svmsb_f32_z,
  BI__builtin_sve_svmsb_f64_m,
  BI__builtin_sve_svmsb_f64_x,
  BI__builtin_sve_svmsb_f64_z,
  BI__builtin_sve_svmsb_n_f16_m,
  BI__builtin_sve_svmsb_n_f16_x,
  BI__builtin_sve_svmsb_n_f16_z,
  BI__builtin_sve_svmsb_n_f32_m,
  BI__builtin_sve_svmsb_n_f32_x,
  BI__builtin_sve_svmsb_n_f32_z,
  BI__builtin_sve_svmsb_n_f64_m,
  BI__builtin_sve_svmsb_n_f64_x,
  BI__builtin_sve_svmsb_n_f64_z,
  BI__builtin_sve_svmsb_n_s16_m,
  BI__builtin_sve_svmsb_n_s16_x,
  BI__builtin_sve_svmsb_n_s16_z,
  BI__builtin_sve_svmsb_n_s32_m,
  BI__builtin_sve_svmsb_n_s32_x,
  BI__builtin_sve_svmsb_n_s32_z,
  BI__builtin_sve_svmsb_n_s64_m,
  BI__builtin_sve_svmsb_n_s64_x,
  BI__builtin_sve_svmsb_n_s64_z,
  BI__builtin_sve_svmsb_n_s8_m,
  BI__builtin_sve_svmsb_n_s8_x,
  BI__builtin_sve_svmsb_n_s8_z,
  BI__builtin_sve_svmsb_n_u16_m,
  BI__builtin_sve_svmsb_n_u16_x,
  BI__builtin_sve_svmsb_n_u16_z,
  BI__builtin_sve_svmsb_n_u32_m,
  BI__builtin_sve_svmsb_n_u32_x,
  BI__builtin_sve_svmsb_n_u32_z,
  BI__builtin_sve_svmsb_n_u64_m,
  BI__builtin_sve_svmsb_n_u64_x,
  BI__builtin_sve_svmsb_n_u64_z,
  BI__builtin_sve_svmsb_n_u8_m,
  BI__builtin_sve_svmsb_n_u8_x,
  BI__builtin_sve_svmsb_n_u8_z,
  BI__builtin_sve_svmsb_s16_m,
  BI__builtin_sve_svmsb_s16_x,
  BI__builtin_sve_svmsb_s16_z,
  BI__builtin_sve_svmsb_s32_m,
  BI__builtin_sve_svmsb_s32_x,
  BI__builtin_sve_svmsb_s32_z,
  BI__builtin_sve_svmsb_s64_m,
  BI__builtin_sve_svmsb_s64_x,
  BI__builtin_sve_svmsb_s64_z,
  BI__builtin_sve_svmsb_s8_m,
  BI__builtin_sve_svmsb_s8_x,
  BI__builtin_sve_svmsb_s8_z,
  BI__builtin_sve_svmsb_u16_m,
  BI__builtin_sve_svmsb_u16_x,
  BI__builtin_sve_svmsb_u16_z,
  BI__builtin_sve_svmsb_u32_m,
  BI__builtin_sve_svmsb_u32_x,
  BI__builtin_sve_svmsb_u32_z,
  BI__builtin_sve_svmsb_u64_m,
  BI__builtin_sve_svmsb_u64_x,
  BI__builtin_sve_svmsb_u64_z,
  BI__builtin_sve_svmsb_u8_m,
  BI__builtin_sve_svmsb_u8_x,
  BI__builtin_sve_svmsb_u8_z,
  BI__builtin_sve_svmul_bf16_m,
  BI__builtin_sve_svmul_bf16_x,
  BI__builtin_sve_svmul_bf16_x2,
  BI__builtin_sve_svmul_bf16_x4,
  BI__builtin_sve_svmul_bf16_z,
  BI__builtin_sve_svmul_f16_m,
  BI__builtin_sve_svmul_f16_x,
  BI__builtin_sve_svmul_f16_x2,
  BI__builtin_sve_svmul_f16_x4,
  BI__builtin_sve_svmul_f16_z,
  BI__builtin_sve_svmul_f32_m,
  BI__builtin_sve_svmul_f32_x,
  BI__builtin_sve_svmul_f32_x2,
  BI__builtin_sve_svmul_f32_x4,
  BI__builtin_sve_svmul_f32_z,
  BI__builtin_sve_svmul_f64_m,
  BI__builtin_sve_svmul_f64_x,
  BI__builtin_sve_svmul_f64_x2,
  BI__builtin_sve_svmul_f64_x4,
  BI__builtin_sve_svmul_f64_z,
  BI__builtin_sve_svmul_lane_bf16,
  BI__builtin_sve_svmul_lane_f16,
  BI__builtin_sve_svmul_lane_f32,
  BI__builtin_sve_svmul_lane_f64,
  BI__builtin_sve_svmul_lane_s16,
  BI__builtin_sve_svmul_lane_s32,
  BI__builtin_sve_svmul_lane_s64,
  BI__builtin_sve_svmul_lane_u16,
  BI__builtin_sve_svmul_lane_u32,
  BI__builtin_sve_svmul_lane_u64,
  BI__builtin_sve_svmul_n_bf16_m,
  BI__builtin_sve_svmul_n_bf16_x,
  BI__builtin_sve_svmul_n_bf16_z,
  BI__builtin_sve_svmul_n_f16_m,
  BI__builtin_sve_svmul_n_f16_x,
  BI__builtin_sve_svmul_n_f16_z,
  BI__builtin_sve_svmul_n_f32_m,
  BI__builtin_sve_svmul_n_f32_x,
  BI__builtin_sve_svmul_n_f32_z,
  BI__builtin_sve_svmul_n_f64_m,
  BI__builtin_sve_svmul_n_f64_x,
  BI__builtin_sve_svmul_n_f64_z,
  BI__builtin_sve_svmul_n_s16_m,
  BI__builtin_sve_svmul_n_s16_x,
  BI__builtin_sve_svmul_n_s16_z,
  BI__builtin_sve_svmul_n_s32_m,
  BI__builtin_sve_svmul_n_s32_x,
  BI__builtin_sve_svmul_n_s32_z,
  BI__builtin_sve_svmul_n_s64_m,
  BI__builtin_sve_svmul_n_s64_x,
  BI__builtin_sve_svmul_n_s64_z,
  BI__builtin_sve_svmul_n_s8_m,
  BI__builtin_sve_svmul_n_s8_x,
  BI__builtin_sve_svmul_n_s8_z,
  BI__builtin_sve_svmul_n_u16_m,
  BI__builtin_sve_svmul_n_u16_x,
  BI__builtin_sve_svmul_n_u16_z,
  BI__builtin_sve_svmul_n_u32_m,
  BI__builtin_sve_svmul_n_u32_x,
  BI__builtin_sve_svmul_n_u32_z,
  BI__builtin_sve_svmul_n_u64_m,
  BI__builtin_sve_svmul_n_u64_x,
  BI__builtin_sve_svmul_n_u64_z,
  BI__builtin_sve_svmul_n_u8_m,
  BI__builtin_sve_svmul_n_u8_x,
  BI__builtin_sve_svmul_n_u8_z,
  BI__builtin_sve_svmul_s16_m,
  BI__builtin_sve_svmul_s16_x,
  BI__builtin_sve_svmul_s16_z,
  BI__builtin_sve_svmul_s32_m,
  BI__builtin_sve_svmul_s32_x,
  BI__builtin_sve_svmul_s32_z,
  BI__builtin_sve_svmul_s64_m,
  BI__builtin_sve_svmul_s64_x,
  BI__builtin_sve_svmul_s64_z,
  BI__builtin_sve_svmul_s8_m,
  BI__builtin_sve_svmul_s8_x,
  BI__builtin_sve_svmul_s8_z,
  BI__builtin_sve_svmul_single_bf16_x2,
  BI__builtin_sve_svmul_single_bf16_x4,
  BI__builtin_sve_svmul_single_f16_x2,
  BI__builtin_sve_svmul_single_f16_x4,
  BI__builtin_sve_svmul_single_f32_x2,
  BI__builtin_sve_svmul_single_f32_x4,
  BI__builtin_sve_svmul_single_f64_x2,
  BI__builtin_sve_svmul_single_f64_x4,
  BI__builtin_sve_svmul_u16_m,
  BI__builtin_sve_svmul_u16_x,
  BI__builtin_sve_svmul_u16_z,
  BI__builtin_sve_svmul_u32_m,
  BI__builtin_sve_svmul_u32_x,
  BI__builtin_sve_svmul_u32_z,
  BI__builtin_sve_svmul_u64_m,
  BI__builtin_sve_svmul_u64_x,
  BI__builtin_sve_svmul_u64_z,
  BI__builtin_sve_svmul_u8_m,
  BI__builtin_sve_svmul_u8_x,
  BI__builtin_sve_svmul_u8_z,
  BI__builtin_sve_svmulh_n_s16_m,
  BI__builtin_sve_svmulh_n_s16_x,
  BI__builtin_sve_svmulh_n_s16_z,
  BI__builtin_sve_svmulh_n_s32_m,
  BI__builtin_sve_svmulh_n_s32_x,
  BI__builtin_sve_svmulh_n_s32_z,
  BI__builtin_sve_svmulh_n_s64_m,
  BI__builtin_sve_svmulh_n_s64_x,
  BI__builtin_sve_svmulh_n_s64_z,
  BI__builtin_sve_svmulh_n_s8_m,
  BI__builtin_sve_svmulh_n_s8_x,
  BI__builtin_sve_svmulh_n_s8_z,
  BI__builtin_sve_svmulh_n_u16_m,
  BI__builtin_sve_svmulh_n_u16_x,
  BI__builtin_sve_svmulh_n_u16_z,
  BI__builtin_sve_svmulh_n_u32_m,
  BI__builtin_sve_svmulh_n_u32_x,
  BI__builtin_sve_svmulh_n_u32_z,
  BI__builtin_sve_svmulh_n_u64_m,
  BI__builtin_sve_svmulh_n_u64_x,
  BI__builtin_sve_svmulh_n_u64_z,
  BI__builtin_sve_svmulh_n_u8_m,
  BI__builtin_sve_svmulh_n_u8_x,
  BI__builtin_sve_svmulh_n_u8_z,
  BI__builtin_sve_svmulh_s16_m,
  BI__builtin_sve_svmulh_s16_x,
  BI__builtin_sve_svmulh_s16_z,
  BI__builtin_sve_svmulh_s32_m,
  BI__builtin_sve_svmulh_s32_x,
  BI__builtin_sve_svmulh_s32_z,
  BI__builtin_sve_svmulh_s64_m,
  BI__builtin_sve_svmulh_s64_x,
  BI__builtin_sve_svmulh_s64_z,
  BI__builtin_sve_svmulh_s8_m,
  BI__builtin_sve_svmulh_s8_x,
  BI__builtin_sve_svmulh_s8_z,
  BI__builtin_sve_svmulh_u16_m,
  BI__builtin_sve_svmulh_u16_x,
  BI__builtin_sve_svmulh_u16_z,
  BI__builtin_sve_svmulh_u32_m,
  BI__builtin_sve_svmulh_u32_x,
  BI__builtin_sve_svmulh_u32_z,
  BI__builtin_sve_svmulh_u64_m,
  BI__builtin_sve_svmulh_u64_x,
  BI__builtin_sve_svmulh_u64_z,
  BI__builtin_sve_svmulh_u8_m,
  BI__builtin_sve_svmulh_u8_x,
  BI__builtin_sve_svmulh_u8_z,
  BI__builtin_sve_svmullb_lane_s32,
  BI__builtin_sve_svmullb_lane_s64,
  BI__builtin_sve_svmullb_lane_u32,
  BI__builtin_sve_svmullb_lane_u64,
  BI__builtin_sve_svmullb_n_s16,
  BI__builtin_sve_svmullb_n_s32,
  BI__builtin_sve_svmullb_n_s64,
  BI__builtin_sve_svmullb_n_u16,
  BI__builtin_sve_svmullb_n_u32,
  BI__builtin_sve_svmullb_n_u64,
  BI__builtin_sve_svmullb_s16,
  BI__builtin_sve_svmullb_s32,
  BI__builtin_sve_svmullb_s64,
  BI__builtin_sve_svmullb_u16,
  BI__builtin_sve_svmullb_u32,
  BI__builtin_sve_svmullb_u64,
  BI__builtin_sve_svmullt_lane_s32,
  BI__builtin_sve_svmullt_lane_s64,
  BI__builtin_sve_svmullt_lane_u32,
  BI__builtin_sve_svmullt_lane_u64,
  BI__builtin_sve_svmullt_n_s16,
  BI__builtin_sve_svmullt_n_s32,
  BI__builtin_sve_svmullt_n_s64,
  BI__builtin_sve_svmullt_n_u16,
  BI__builtin_sve_svmullt_n_u32,
  BI__builtin_sve_svmullt_n_u64,
  BI__builtin_sve_svmullt_s16,
  BI__builtin_sve_svmullt_s32,
  BI__builtin_sve_svmullt_s64,
  BI__builtin_sve_svmullt_u16,
  BI__builtin_sve_svmullt_u32,
  BI__builtin_sve_svmullt_u64,
  BI__builtin_sve_svmulx_f16_m,
  BI__builtin_sve_svmulx_f16_x,
  BI__builtin_sve_svmulx_f16_z,
  BI__builtin_sve_svmulx_f32_m,
  BI__builtin_sve_svmulx_f32_x,
  BI__builtin_sve_svmulx_f32_z,
  BI__builtin_sve_svmulx_f64_m,
  BI__builtin_sve_svmulx_f64_x,
  BI__builtin_sve_svmulx_f64_z,
  BI__builtin_sve_svmulx_n_f16_m,
  BI__builtin_sve_svmulx_n_f16_x,
  BI__builtin_sve_svmulx_n_f16_z,
  BI__builtin_sve_svmulx_n_f32_m,
  BI__builtin_sve_svmulx_n_f32_x,
  BI__builtin_sve_svmulx_n_f32_z,
  BI__builtin_sve_svmulx_n_f64_m,
  BI__builtin_sve_svmulx_n_f64_x,
  BI__builtin_sve_svmulx_n_f64_z,
  BI__builtin_sve_svnand_b_z,
  BI__builtin_sve_svnbsl_n_s16,
  BI__builtin_sve_svnbsl_n_s32,
  BI__builtin_sve_svnbsl_n_s64,
  BI__builtin_sve_svnbsl_n_s8,
  BI__builtin_sve_svnbsl_n_u16,
  BI__builtin_sve_svnbsl_n_u32,
  BI__builtin_sve_svnbsl_n_u64,
  BI__builtin_sve_svnbsl_n_u8,
  BI__builtin_sve_svnbsl_s16,
  BI__builtin_sve_svnbsl_s32,
  BI__builtin_sve_svnbsl_s64,
  BI__builtin_sve_svnbsl_s8,
  BI__builtin_sve_svnbsl_u16,
  BI__builtin_sve_svnbsl_u32,
  BI__builtin_sve_svnbsl_u64,
  BI__builtin_sve_svnbsl_u8,
  BI__builtin_sve_svneg_f16_m,
  BI__builtin_sve_svneg_f16_x,
  BI__builtin_sve_svneg_f16_z,
  BI__builtin_sve_svneg_f32_m,
  BI__builtin_sve_svneg_f32_x,
  BI__builtin_sve_svneg_f32_z,
  BI__builtin_sve_svneg_f64_m,
  BI__builtin_sve_svneg_f64_x,
  BI__builtin_sve_svneg_f64_z,
  BI__builtin_sve_svneg_s16_m,
  BI__builtin_sve_svneg_s16_x,
  BI__builtin_sve_svneg_s16_z,
  BI__builtin_sve_svneg_s32_m,
  BI__builtin_sve_svneg_s32_x,
  BI__builtin_sve_svneg_s32_z,
  BI__builtin_sve_svneg_s64_m,
  BI__builtin_sve_svneg_s64_x,
  BI__builtin_sve_svneg_s64_z,
  BI__builtin_sve_svneg_s8_m,
  BI__builtin_sve_svneg_s8_x,
  BI__builtin_sve_svneg_s8_z,
  BI__builtin_sve_svnmad_f16_m,
  BI__builtin_sve_svnmad_f16_x,
  BI__builtin_sve_svnmad_f16_z,
  BI__builtin_sve_svnmad_f32_m,
  BI__builtin_sve_svnmad_f32_x,
  BI__builtin_sve_svnmad_f32_z,
  BI__builtin_sve_svnmad_f64_m,
  BI__builtin_sve_svnmad_f64_x,
  BI__builtin_sve_svnmad_f64_z,
  BI__builtin_sve_svnmad_n_f16_m,
  BI__builtin_sve_svnmad_n_f16_x,
  BI__builtin_sve_svnmad_n_f16_z,
  BI__builtin_sve_svnmad_n_f32_m,
  BI__builtin_sve_svnmad_n_f32_x,
  BI__builtin_sve_svnmad_n_f32_z,
  BI__builtin_sve_svnmad_n_f64_m,
  BI__builtin_sve_svnmad_n_f64_x,
  BI__builtin_sve_svnmad_n_f64_z,
  BI__builtin_sve_svnmatch_s16,
  BI__builtin_sve_svnmatch_s8,
  BI__builtin_sve_svnmatch_u16,
  BI__builtin_sve_svnmatch_u8,
  BI__builtin_sve_svnmla_f16_m,
  BI__builtin_sve_svnmla_f16_x,
  BI__builtin_sve_svnmla_f16_z,
  BI__builtin_sve_svnmla_f32_m,
  BI__builtin_sve_svnmla_f32_x,
  BI__builtin_sve_svnmla_f32_z,
  BI__builtin_sve_svnmla_f64_m,
  BI__builtin_sve_svnmla_f64_x,
  BI__builtin_sve_svnmla_f64_z,
  BI__builtin_sve_svnmla_n_f16_m,
  BI__builtin_sve_svnmla_n_f16_x,
  BI__builtin_sve_svnmla_n_f16_z,
  BI__builtin_sve_svnmla_n_f32_m,
  BI__builtin_sve_svnmla_n_f32_x,
  BI__builtin_sve_svnmla_n_f32_z,
  BI__builtin_sve_svnmla_n_f64_m,
  BI__builtin_sve_svnmla_n_f64_x,
  BI__builtin_sve_svnmla_n_f64_z,
  BI__builtin_sve_svnmls_f16_m,
  BI__builtin_sve_svnmls_f16_x,
  BI__builtin_sve_svnmls_f16_z,
  BI__builtin_sve_svnmls_f32_m,
  BI__builtin_sve_svnmls_f32_x,
  BI__builtin_sve_svnmls_f32_z,
  BI__builtin_sve_svnmls_f64_m,
  BI__builtin_sve_svnmls_f64_x,
  BI__builtin_sve_svnmls_f64_z,
  BI__builtin_sve_svnmls_n_f16_m,
  BI__builtin_sve_svnmls_n_f16_x,
  BI__builtin_sve_svnmls_n_f16_z,
  BI__builtin_sve_svnmls_n_f32_m,
  BI__builtin_sve_svnmls_n_f32_x,
  BI__builtin_sve_svnmls_n_f32_z,
  BI__builtin_sve_svnmls_n_f64_m,
  BI__builtin_sve_svnmls_n_f64_x,
  BI__builtin_sve_svnmls_n_f64_z,
  BI__builtin_sve_svnmsb_f16_m,
  BI__builtin_sve_svnmsb_f16_x,
  BI__builtin_sve_svnmsb_f16_z,
  BI__builtin_sve_svnmsb_f32_m,
  BI__builtin_sve_svnmsb_f32_x,
  BI__builtin_sve_svnmsb_f32_z,
  BI__builtin_sve_svnmsb_f64_m,
  BI__builtin_sve_svnmsb_f64_x,
  BI__builtin_sve_svnmsb_f64_z,
  BI__builtin_sve_svnmsb_n_f16_m,
  BI__builtin_sve_svnmsb_n_f16_x,
  BI__builtin_sve_svnmsb_n_f16_z,
  BI__builtin_sve_svnmsb_n_f32_m,
  BI__builtin_sve_svnmsb_n_f32_x,
  BI__builtin_sve_svnmsb_n_f32_z,
  BI__builtin_sve_svnmsb_n_f64_m,
  BI__builtin_sve_svnmsb_n_f64_x,
  BI__builtin_sve_svnmsb_n_f64_z,
  BI__builtin_sve_svnor_b_z,
  BI__builtin_sve_svnot_b_z,
  BI__builtin_sve_svnot_s16_m,
  BI__builtin_sve_svnot_s16_x,
  BI__builtin_sve_svnot_s16_z,
  BI__builtin_sve_svnot_s32_m,
  BI__builtin_sve_svnot_s32_x,
  BI__builtin_sve_svnot_s32_z,
  BI__builtin_sve_svnot_s64_m,
  BI__builtin_sve_svnot_s64_x,
  BI__builtin_sve_svnot_s64_z,
  BI__builtin_sve_svnot_s8_m,
  BI__builtin_sve_svnot_s8_x,
  BI__builtin_sve_svnot_s8_z,
  BI__builtin_sve_svnot_u16_m,
  BI__builtin_sve_svnot_u16_x,
  BI__builtin_sve_svnot_u16_z,
  BI__builtin_sve_svnot_u32_m,
  BI__builtin_sve_svnot_u32_x,
  BI__builtin_sve_svnot_u32_z,
  BI__builtin_sve_svnot_u64_m,
  BI__builtin_sve_svnot_u64_x,
  BI__builtin_sve_svnot_u64_z,
  BI__builtin_sve_svnot_u8_m,
  BI__builtin_sve_svnot_u8_x,
  BI__builtin_sve_svnot_u8_z,
  BI__builtin_sve_svorn_b_z,
  BI__builtin_sve_svorqv_s16,
  BI__builtin_sve_svorqv_s32,
  BI__builtin_sve_svorqv_s64,
  BI__builtin_sve_svorqv_s8,
  BI__builtin_sve_svorqv_u16,
  BI__builtin_sve_svorqv_u32,
  BI__builtin_sve_svorqv_u64,
  BI__builtin_sve_svorqv_u8,
  BI__builtin_sve_svorr_b_z,
  BI__builtin_sve_svorr_n_s16_m,
  BI__builtin_sve_svorr_n_s16_x,
  BI__builtin_sve_svorr_n_s16_z,
  BI__builtin_sve_svorr_n_s32_m,
  BI__builtin_sve_svorr_n_s32_x,
  BI__builtin_sve_svorr_n_s32_z,
  BI__builtin_sve_svorr_n_s64_m,
  BI__builtin_sve_svorr_n_s64_x,
  BI__builtin_sve_svorr_n_s64_z,
  BI__builtin_sve_svorr_n_s8_m,
  BI__builtin_sve_svorr_n_s8_x,
  BI__builtin_sve_svorr_n_s8_z,
  BI__builtin_sve_svorr_n_u16_m,
  BI__builtin_sve_svorr_n_u16_x,
  BI__builtin_sve_svorr_n_u16_z,
  BI__builtin_sve_svorr_n_u32_m,
  BI__builtin_sve_svorr_n_u32_x,
  BI__builtin_sve_svorr_n_u32_z,
  BI__builtin_sve_svorr_n_u64_m,
  BI__builtin_sve_svorr_n_u64_x,
  BI__builtin_sve_svorr_n_u64_z,
  BI__builtin_sve_svorr_n_u8_m,
  BI__builtin_sve_svorr_n_u8_x,
  BI__builtin_sve_svorr_n_u8_z,
  BI__builtin_sve_svorr_s16_m,
  BI__builtin_sve_svorr_s16_x,
  BI__builtin_sve_svorr_s16_z,
  BI__builtin_sve_svorr_s32_m,
  BI__builtin_sve_svorr_s32_x,
  BI__builtin_sve_svorr_s32_z,
  BI__builtin_sve_svorr_s64_m,
  BI__builtin_sve_svorr_s64_x,
  BI__builtin_sve_svorr_s64_z,
  BI__builtin_sve_svorr_s8_m,
  BI__builtin_sve_svorr_s8_x,
  BI__builtin_sve_svorr_s8_z,
  BI__builtin_sve_svorr_u16_m,
  BI__builtin_sve_svorr_u16_x,
  BI__builtin_sve_svorr_u16_z,
  BI__builtin_sve_svorr_u32_m,
  BI__builtin_sve_svorr_u32_x,
  BI__builtin_sve_svorr_u32_z,
  BI__builtin_sve_svorr_u64_m,
  BI__builtin_sve_svorr_u64_x,
  BI__builtin_sve_svorr_u64_z,
  BI__builtin_sve_svorr_u8_m,
  BI__builtin_sve_svorr_u8_x,
  BI__builtin_sve_svorr_u8_z,
  BI__builtin_sve_svorv_s16,
  BI__builtin_sve_svorv_s32,
  BI__builtin_sve_svorv_s64,
  BI__builtin_sve_svorv_s8,
  BI__builtin_sve_svorv_u16,
  BI__builtin_sve_svorv_u32,
  BI__builtin_sve_svorv_u64,
  BI__builtin_sve_svorv_u8,
  BI__builtin_sve_svpext_lane_c16,
  BI__builtin_sve_svpext_lane_c16_x2,
  BI__builtin_sve_svpext_lane_c32,
  BI__builtin_sve_svpext_lane_c32_x2,
  BI__builtin_sve_svpext_lane_c64,
  BI__builtin_sve_svpext_lane_c64_x2,
  BI__builtin_sve_svpext_lane_c8,
  BI__builtin_sve_svpext_lane_c8_x2,
  BI__builtin_sve_svpfalse_b,
  BI__builtin_sve_svpfalse_c,
  BI__builtin_sve_svpfirst_b,
  BI__builtin_sve_svpmlal_pair_n_u64_x2,
  BI__builtin_sve_svpmlal_pair_u64_x2,
  BI__builtin_sve_svpmov_lane_s16,
  BI__builtin_sve_svpmov_lane_s16_m,
  BI__builtin_sve_svpmov_lane_s32,
  BI__builtin_sve_svpmov_lane_s32_m,
  BI__builtin_sve_svpmov_lane_s64,
  BI__builtin_sve_svpmov_lane_s64_m,
  BI__builtin_sve_svpmov_lane_s8,
  BI__builtin_sve_svpmov_lane_u16,
  BI__builtin_sve_svpmov_lane_u16_m,
  BI__builtin_sve_svpmov_lane_u32,
  BI__builtin_sve_svpmov_lane_u32_m,
  BI__builtin_sve_svpmov_lane_u64,
  BI__builtin_sve_svpmov_lane_u64_m,
  BI__builtin_sve_svpmov_lane_u8,
  BI__builtin_sve_svpmov_s16,
  BI__builtin_sve_svpmov_s16_z,
  BI__builtin_sve_svpmov_s32,
  BI__builtin_sve_svpmov_s32_z,
  BI__builtin_sve_svpmov_s64,
  BI__builtin_sve_svpmov_s64_z,
  BI__builtin_sve_svpmov_s8,
  BI__builtin_sve_svpmov_s8_z,
  BI__builtin_sve_svpmov_u16,
  BI__builtin_sve_svpmov_u16_z,
  BI__builtin_sve_svpmov_u32,
  BI__builtin_sve_svpmov_u32_z,
  BI__builtin_sve_svpmov_u64,
  BI__builtin_sve_svpmov_u64_z,
  BI__builtin_sve_svpmov_u8,
  BI__builtin_sve_svpmov_u8_z,
  BI__builtin_sve_svpmul_n_u8,
  BI__builtin_sve_svpmul_u8,
  BI__builtin_sve_svpmull_pair_n_u64_x2,
  BI__builtin_sve_svpmull_pair_u64_x2,
  BI__builtin_sve_svpmullb_n_u16,
  BI__builtin_sve_svpmullb_n_u64,
  BI__builtin_sve_svpmullb_pair_n_u32,
  BI__builtin_sve_svpmullb_pair_n_u64,
  BI__builtin_sve_svpmullb_pair_n_u8,
  BI__builtin_sve_svpmullb_pair_u32,
  BI__builtin_sve_svpmullb_pair_u64,
  BI__builtin_sve_svpmullb_pair_u8,
  BI__builtin_sve_svpmullb_u16,
  BI__builtin_sve_svpmullb_u64,
  BI__builtin_sve_svpmullt_n_u16,
  BI__builtin_sve_svpmullt_n_u64,
  BI__builtin_sve_svpmullt_pair_n_u32,
  BI__builtin_sve_svpmullt_pair_n_u64,
  BI__builtin_sve_svpmullt_pair_n_u8,
  BI__builtin_sve_svpmullt_pair_u32,
  BI__builtin_sve_svpmullt_pair_u64,
  BI__builtin_sve_svpmullt_pair_u8,
  BI__builtin_sve_svpmullt_u16,
  BI__builtin_sve_svpmullt_u64,
  BI__builtin_sve_svpnext_b16,
  BI__builtin_sve_svpnext_b32,
  BI__builtin_sve_svpnext_b64,
  BI__builtin_sve_svpnext_b8,
  BI__builtin_sve_svprfb,
  BI__builtin_sve_svprfb_gather_s32offset,
  BI__builtin_sve_svprfb_gather_s64offset,
  BI__builtin_sve_svprfb_gather_u32base,
  BI__builtin_sve_svprfb_gather_u32base_offset,
  BI__builtin_sve_svprfb_gather_u32offset,
  BI__builtin_sve_svprfb_gather_u64base,
  BI__builtin_sve_svprfb_gather_u64base_offset,
  BI__builtin_sve_svprfb_gather_u64offset,
  BI__builtin_sve_svprfb_vnum,
  BI__builtin_sve_svprfd,
  BI__builtin_sve_svprfd_gather_s32index,
  BI__builtin_sve_svprfd_gather_s64index,
  BI__builtin_sve_svprfd_gather_u32base,
  BI__builtin_sve_svprfd_gather_u32base_index,
  BI__builtin_sve_svprfd_gather_u32index,
  BI__builtin_sve_svprfd_gather_u64base,
  BI__builtin_sve_svprfd_gather_u64base_index,
  BI__builtin_sve_svprfd_gather_u64index,
  BI__builtin_sve_svprfd_vnum,
  BI__builtin_sve_svprfh,
  BI__builtin_sve_svprfh_gather_s32index,
  BI__builtin_sve_svprfh_gather_s64index,
  BI__builtin_sve_svprfh_gather_u32base,
  BI__builtin_sve_svprfh_gather_u32base_index,
  BI__builtin_sve_svprfh_gather_u32index,
  BI__builtin_sve_svprfh_gather_u64base,
  BI__builtin_sve_svprfh_gather_u64base_index,
  BI__builtin_sve_svprfh_gather_u64index,
  BI__builtin_sve_svprfh_vnum,
  BI__builtin_sve_svprfw,
  BI__builtin_sve_svprfw_gather_s32index,
  BI__builtin_sve_svprfw_gather_s64index,
  BI__builtin_sve_svprfw_gather_u32base,
  BI__builtin_sve_svprfw_gather_u32base_index,
  BI__builtin_sve_svprfw_gather_u32index,
  BI__builtin_sve_svprfw_gather_u64base,
  BI__builtin_sve_svprfw_gather_u64base_index,
  BI__builtin_sve_svprfw_gather_u64index,
  BI__builtin_sve_svprfw_vnum,
  BI__builtin_sve_svpsel_lane_b16,
  BI__builtin_sve_svpsel_lane_b32,
  BI__builtin_sve_svpsel_lane_b64,
  BI__builtin_sve_svpsel_lane_b8,
  BI__builtin_sve_svpsel_lane_c16,
  BI__builtin_sve_svpsel_lane_c32,
  BI__builtin_sve_svpsel_lane_c64,
  BI__builtin_sve_svpsel_lane_c8,
  BI__builtin_sve_svptest_any,
  BI__builtin_sve_svptest_first,
  BI__builtin_sve_svptest_last,
  BI__builtin_sve_svptrue_b16,
  BI__builtin_sve_svptrue_b32,
  BI__builtin_sve_svptrue_b64,
  BI__builtin_sve_svptrue_b8,
  BI__builtin_sve_svptrue_c16,
  BI__builtin_sve_svptrue_c32,
  BI__builtin_sve_svptrue_c64,
  BI__builtin_sve_svptrue_c8,
  BI__builtin_sve_svptrue_pat_b16,
  BI__builtin_sve_svptrue_pat_b32,
  BI__builtin_sve_svptrue_pat_b64,
  BI__builtin_sve_svptrue_pat_b8,
  BI__builtin_sve_svqabs_s16_m,
  BI__builtin_sve_svqabs_s16_x,
  BI__builtin_sve_svqabs_s16_z,
  BI__builtin_sve_svqabs_s32_m,
  BI__builtin_sve_svqabs_s32_x,
  BI__builtin_sve_svqabs_s32_z,
  BI__builtin_sve_svqabs_s64_m,
  BI__builtin_sve_svqabs_s64_x,
  BI__builtin_sve_svqabs_s64_z,
  BI__builtin_sve_svqabs_s8_m,
  BI__builtin_sve_svqabs_s8_x,
  BI__builtin_sve_svqabs_s8_z,
  BI__builtin_sve_svqadd_n_s16,
  BI__builtin_sve_svqadd_n_s16_m,
  BI__builtin_sve_svqadd_n_s16_x,
  BI__builtin_sve_svqadd_n_s16_z,
  BI__builtin_sve_svqadd_n_s32,
  BI__builtin_sve_svqadd_n_s32_m,
  BI__builtin_sve_svqadd_n_s32_x,
  BI__builtin_sve_svqadd_n_s32_z,
  BI__builtin_sve_svqadd_n_s64,
  BI__builtin_sve_svqadd_n_s64_m,
  BI__builtin_sve_svqadd_n_s64_x,
  BI__builtin_sve_svqadd_n_s64_z,
  BI__builtin_sve_svqadd_n_s8,
  BI__builtin_sve_svqadd_n_s8_m,
  BI__builtin_sve_svqadd_n_s8_x,
  BI__builtin_sve_svqadd_n_s8_z,
  BI__builtin_sve_svqadd_n_u16,
  BI__builtin_sve_svqadd_n_u16_m,
  BI__builtin_sve_svqadd_n_u16_x,
  BI__builtin_sve_svqadd_n_u16_z,
  BI__builtin_sve_svqadd_n_u32,
  BI__builtin_sve_svqadd_n_u32_m,
  BI__builtin_sve_svqadd_n_u32_x,
  BI__builtin_sve_svqadd_n_u32_z,
  BI__builtin_sve_svqadd_n_u64,
  BI__builtin_sve_svqadd_n_u64_m,
  BI__builtin_sve_svqadd_n_u64_x,
  BI__builtin_sve_svqadd_n_u64_z,
  BI__builtin_sve_svqadd_n_u8,
  BI__builtin_sve_svqadd_n_u8_m,
  BI__builtin_sve_svqadd_n_u8_x,
  BI__builtin_sve_svqadd_n_u8_z,
  BI__builtin_sve_svqadd_s16,
  BI__builtin_sve_svqadd_s16_m,
  BI__builtin_sve_svqadd_s16_x,
  BI__builtin_sve_svqadd_s16_z,
  BI__builtin_sve_svqadd_s32,
  BI__builtin_sve_svqadd_s32_m,
  BI__builtin_sve_svqadd_s32_x,
  BI__builtin_sve_svqadd_s32_z,
  BI__builtin_sve_svqadd_s64,
  BI__builtin_sve_svqadd_s64_m,
  BI__builtin_sve_svqadd_s64_x,
  BI__builtin_sve_svqadd_s64_z,
  BI__builtin_sve_svqadd_s8,
  BI__builtin_sve_svqadd_s8_m,
  BI__builtin_sve_svqadd_s8_x,
  BI__builtin_sve_svqadd_s8_z,
  BI__builtin_sve_svqadd_u16,
  BI__builtin_sve_svqadd_u16_m,
  BI__builtin_sve_svqadd_u16_x,
  BI__builtin_sve_svqadd_u16_z,
  BI__builtin_sve_svqadd_u32,
  BI__builtin_sve_svqadd_u32_m,
  BI__builtin_sve_svqadd_u32_x,
  BI__builtin_sve_svqadd_u32_z,
  BI__builtin_sve_svqadd_u64,
  BI__builtin_sve_svqadd_u64_m,
  BI__builtin_sve_svqadd_u64_x,
  BI__builtin_sve_svqadd_u64_z,
  BI__builtin_sve_svqadd_u8,
  BI__builtin_sve_svqadd_u8_m,
  BI__builtin_sve_svqadd_u8_x,
  BI__builtin_sve_svqadd_u8_z,
  BI__builtin_sve_svqcadd_s16,
  BI__builtin_sve_svqcadd_s32,
  BI__builtin_sve_svqcadd_s64,
  BI__builtin_sve_svqcadd_s8,
  BI__builtin_sve_svqcvt_s16_s32_x2,
  BI__builtin_sve_svqcvt_s16_s64_x4,
  BI__builtin_sve_svqcvt_s8_s32_x4,
  BI__builtin_sve_svqcvt_u16_s32_x2,
  BI__builtin_sve_svqcvt_u16_s64_x4,
  BI__builtin_sve_svqcvt_u16_u32_x2,
  BI__builtin_sve_svqcvt_u16_u64_x4,
  BI__builtin_sve_svqcvt_u8_s32_x4,
  BI__builtin_sve_svqcvt_u8_u32_x4,
  BI__builtin_sve_svqcvtn_s16_s32_x2,
  BI__builtin_sve_svqcvtn_s16_s64_x4,
  BI__builtin_sve_svqcvtn_s8_s32_x4,
  BI__builtin_sve_svqcvtn_u16_s32_x2,
  BI__builtin_sve_svqcvtn_u16_s64_x4,
  BI__builtin_sve_svqcvtn_u16_u32_x2,
  BI__builtin_sve_svqcvtn_u16_u64_x4,
  BI__builtin_sve_svqcvtn_u8_s32_x4,
  BI__builtin_sve_svqcvtn_u8_u32_x4,
  BI__builtin_sve_svqdecb_n_s32,
  BI__builtin_sve_svqdecb_n_s64,
  BI__builtin_sve_svqdecb_n_u32,
  BI__builtin_sve_svqdecb_n_u64,
  BI__builtin_sve_svqdecb_pat_n_s32,
  BI__builtin_sve_svqdecb_pat_n_s64,
  BI__builtin_sve_svqdecb_pat_n_u32,
  BI__builtin_sve_svqdecb_pat_n_u64,
  BI__builtin_sve_svqdecd_n_s32,
  BI__builtin_sve_svqdecd_n_s64,
  BI__builtin_sve_svqdecd_n_u32,
  BI__builtin_sve_svqdecd_n_u64,
  BI__builtin_sve_svqdecd_pat_n_s32,
  BI__builtin_sve_svqdecd_pat_n_s64,
  BI__builtin_sve_svqdecd_pat_n_u32,
  BI__builtin_sve_svqdecd_pat_n_u64,
  BI__builtin_sve_svqdecd_pat_s64,
  BI__builtin_sve_svqdecd_pat_u64,
  BI__builtin_sve_svqdecd_s64,
  BI__builtin_sve_svqdecd_u64,
  BI__builtin_sve_svqdech_n_s32,
  BI__builtin_sve_svqdech_n_s64,
  BI__builtin_sve_svqdech_n_u32,
  BI__builtin_sve_svqdech_n_u64,
  BI__builtin_sve_svqdech_pat_n_s32,
  BI__builtin_sve_svqdech_pat_n_s64,
  BI__builtin_sve_svqdech_pat_n_u32,
  BI__builtin_sve_svqdech_pat_n_u64,
  BI__builtin_sve_svqdech_pat_s16,
  BI__builtin_sve_svqdech_pat_u16,
  BI__builtin_sve_svqdech_s16,
  BI__builtin_sve_svqdech_u16,
  BI__builtin_sve_svqdecp_n_s32_b16,
  BI__builtin_sve_svqdecp_n_s32_b32,
  BI__builtin_sve_svqdecp_n_s32_b64,
  BI__builtin_sve_svqdecp_n_s32_b8,
  BI__builtin_sve_svqdecp_n_s64_b16,
  BI__builtin_sve_svqdecp_n_s64_b32,
  BI__builtin_sve_svqdecp_n_s64_b64,
  BI__builtin_sve_svqdecp_n_s64_b8,
  BI__builtin_sve_svqdecp_n_u32_b16,
  BI__builtin_sve_svqdecp_n_u32_b32,
  BI__builtin_sve_svqdecp_n_u32_b64,
  BI__builtin_sve_svqdecp_n_u32_b8,
  BI__builtin_sve_svqdecp_n_u64_b16,
  BI__builtin_sve_svqdecp_n_u64_b32,
  BI__builtin_sve_svqdecp_n_u64_b64,
  BI__builtin_sve_svqdecp_n_u64_b8,
  BI__builtin_sve_svqdecp_s16,
  BI__builtin_sve_svqdecp_s32,
  BI__builtin_sve_svqdecp_s64,
  BI__builtin_sve_svqdecp_u16,
  BI__builtin_sve_svqdecp_u32,
  BI__builtin_sve_svqdecp_u64,
  BI__builtin_sve_svqdecw_n_s32,
  BI__builtin_sve_svqdecw_n_s64,
  BI__builtin_sve_svqdecw_n_u32,
  BI__builtin_sve_svqdecw_n_u64,
  BI__builtin_sve_svqdecw_pat_n_s32,
  BI__builtin_sve_svqdecw_pat_n_s64,
  BI__builtin_sve_svqdecw_pat_n_u32,
  BI__builtin_sve_svqdecw_pat_n_u64,
  BI__builtin_sve_svqdecw_pat_s32,
  BI__builtin_sve_svqdecw_pat_u32,
  BI__builtin_sve_svqdecw_s32,
  BI__builtin_sve_svqdecw_u32,
  BI__builtin_sve_svqdmlalb_lane_s32,
  BI__builtin_sve_svqdmlalb_lane_s64,
  BI__builtin_sve_svqdmlalb_n_s16,
  BI__builtin_sve_svqdmlalb_n_s32,
  BI__builtin_sve_svqdmlalb_n_s64,
  BI__builtin_sve_svqdmlalb_s16,
  BI__builtin_sve_svqdmlalb_s32,
  BI__builtin_sve_svqdmlalb_s64,
  BI__builtin_sve_svqdmlalbt_n_s16,
  BI__builtin_sve_svqdmlalbt_n_s32,
  BI__builtin_sve_svqdmlalbt_n_s64,
  BI__builtin_sve_svqdmlalbt_s16,
  BI__builtin_sve_svqdmlalbt_s32,
  BI__builtin_sve_svqdmlalbt_s64,
  BI__builtin_sve_svqdmlalt_lane_s32,
  BI__builtin_sve_svqdmlalt_lane_s64,
  BI__builtin_sve_svqdmlalt_n_s16,
  BI__builtin_sve_svqdmlalt_n_s32,
  BI__builtin_sve_svqdmlalt_n_s64,
  BI__builtin_sve_svqdmlalt_s16,
  BI__builtin_sve_svqdmlalt_s32,
  BI__builtin_sve_svqdmlalt_s64,
  BI__builtin_sve_svqdmlslb_lane_s32,
  BI__builtin_sve_svqdmlslb_lane_s64,
  BI__builtin_sve_svqdmlslb_n_s16,
  BI__builtin_sve_svqdmlslb_n_s32,
  BI__builtin_sve_svqdmlslb_n_s64,
  BI__builtin_sve_svqdmlslb_s16,
  BI__builtin_sve_svqdmlslb_s32,
  BI__builtin_sve_svqdmlslb_s64,
  BI__builtin_sve_svqdmlslbt_n_s16,
  BI__builtin_sve_svqdmlslbt_n_s32,
  BI__builtin_sve_svqdmlslbt_n_s64,
  BI__builtin_sve_svqdmlslbt_s16,
  BI__builtin_sve_svqdmlslbt_s32,
  BI__builtin_sve_svqdmlslbt_s64,
  BI__builtin_sve_svqdmlslt_lane_s32,
  BI__builtin_sve_svqdmlslt_lane_s64,
  BI__builtin_sve_svqdmlslt_n_s16,
  BI__builtin_sve_svqdmlslt_n_s32,
  BI__builtin_sve_svqdmlslt_n_s64,
  BI__builtin_sve_svqdmlslt_s16,
  BI__builtin_sve_svqdmlslt_s32,
  BI__builtin_sve_svqdmlslt_s64,
  BI__builtin_sve_svqdmulh_lane_s16,
  BI__builtin_sve_svqdmulh_lane_s32,
  BI__builtin_sve_svqdmulh_lane_s64,
  BI__builtin_sve_svqdmulh_n_s16,
  BI__builtin_sve_svqdmulh_n_s32,
  BI__builtin_sve_svqdmulh_n_s64,
  BI__builtin_sve_svqdmulh_n_s8,
  BI__builtin_sve_svqdmulh_s16,
  BI__builtin_sve_svqdmulh_s16_x2,
  BI__builtin_sve_svqdmulh_s16_x4,
  BI__builtin_sve_svqdmulh_s32,
  BI__builtin_sve_svqdmulh_s32_x2,
  BI__builtin_sve_svqdmulh_s32_x4,
  BI__builtin_sve_svqdmulh_s64,
  BI__builtin_sve_svqdmulh_s64_x2,
  BI__builtin_sve_svqdmulh_s64_x4,
  BI__builtin_sve_svqdmulh_s8,
  BI__builtin_sve_svqdmulh_s8_x2,
  BI__builtin_sve_svqdmulh_s8_x4,
  BI__builtin_sve_svqdmulh_single_s16_x2,
  BI__builtin_sve_svqdmulh_single_s16_x4,
  BI__builtin_sve_svqdmulh_single_s32_x2,
  BI__builtin_sve_svqdmulh_single_s32_x4,
  BI__builtin_sve_svqdmulh_single_s64_x2,
  BI__builtin_sve_svqdmulh_single_s64_x4,
  BI__builtin_sve_svqdmulh_single_s8_x2,
  BI__builtin_sve_svqdmulh_single_s8_x4,
  BI__builtin_sve_svqdmullb_lane_s32,
  BI__builtin_sve_svqdmullb_lane_s64,
  BI__builtin_sve_svqdmullb_n_s16,
  BI__builtin_sve_svqdmullb_n_s32,
  BI__builtin_sve_svqdmullb_n_s64,
  BI__builtin_sve_svqdmullb_s16,
  BI__builtin_sve_svqdmullb_s32,
  BI__builtin_sve_svqdmullb_s64,
  BI__builtin_sve_svqdmullt_lane_s32,
  BI__builtin_sve_svqdmullt_lane_s64,
  BI__builtin_sve_svqdmullt_n_s16,
  BI__builtin_sve_svqdmullt_n_s32,
  BI__builtin_sve_svqdmullt_n_s64,
  BI__builtin_sve_svqdmullt_s16,
  BI__builtin_sve_svqdmullt_s32,
  BI__builtin_sve_svqdmullt_s64,
  BI__builtin_sve_svqincb_n_s32,
  BI__builtin_sve_svqincb_n_s64,
  BI__builtin_sve_svqincb_n_u32,
  BI__builtin_sve_svqincb_n_u64,
  BI__builtin_sve_svqincb_pat_n_s32,
  BI__builtin_sve_svqincb_pat_n_s64,
  BI__builtin_sve_svqincb_pat_n_u32,
  BI__builtin_sve_svqincb_pat_n_u64,
  BI__builtin_sve_svqincd_n_s32,
  BI__builtin_sve_svqincd_n_s64,
  BI__builtin_sve_svqincd_n_u32,
  BI__builtin_sve_svqincd_n_u64,
  BI__builtin_sve_svqincd_pat_n_s32,
  BI__builtin_sve_svqincd_pat_n_s64,
  BI__builtin_sve_svqincd_pat_n_u32,
  BI__builtin_sve_svqincd_pat_n_u64,
  BI__builtin_sve_svqincd_pat_s64,
  BI__builtin_sve_svqincd_pat_u64,
  BI__builtin_sve_svqincd_s64,
  BI__builtin_sve_svqincd_u64,
  BI__builtin_sve_svqinch_n_s32,
  BI__builtin_sve_svqinch_n_s64,
  BI__builtin_sve_svqinch_n_u32,
  BI__builtin_sve_svqinch_n_u64,
  BI__builtin_sve_svqinch_pat_n_s32,
  BI__builtin_sve_svqinch_pat_n_s64,
  BI__builtin_sve_svqinch_pat_n_u32,
  BI__builtin_sve_svqinch_pat_n_u64,
  BI__builtin_sve_svqinch_pat_s16,
  BI__builtin_sve_svqinch_pat_u16,
  BI__builtin_sve_svqinch_s16,
  BI__builtin_sve_svqinch_u16,
  BI__builtin_sve_svqincp_n_s32_b16,
  BI__builtin_sve_svqincp_n_s32_b32,
  BI__builtin_sve_svqincp_n_s32_b64,
  BI__builtin_sve_svqincp_n_s32_b8,
  BI__builtin_sve_svqincp_n_s64_b16,
  BI__builtin_sve_svqincp_n_s64_b32,
  BI__builtin_sve_svqincp_n_s64_b64,
  BI__builtin_sve_svqincp_n_s64_b8,
  BI__builtin_sve_svqincp_n_u32_b16,
  BI__builtin_sve_svqincp_n_u32_b32,
  BI__builtin_sve_svqincp_n_u32_b64,
  BI__builtin_sve_svqincp_n_u32_b8,
  BI__builtin_sve_svqincp_n_u64_b16,
  BI__builtin_sve_svqincp_n_u64_b32,
  BI__builtin_sve_svqincp_n_u64_b64,
  BI__builtin_sve_svqincp_n_u64_b8,
  BI__builtin_sve_svqincp_s16,
  BI__builtin_sve_svqincp_s32,
  BI__builtin_sve_svqincp_s64,
  BI__builtin_sve_svqincp_u16,
  BI__builtin_sve_svqincp_u32,
  BI__builtin_sve_svqincp_u64,
  BI__builtin_sve_svqincw_n_s32,
  BI__builtin_sve_svqincw_n_s64,
  BI__builtin_sve_svqincw_n_u32,
  BI__builtin_sve_svqincw_n_u64,
  BI__builtin_sve_svqincw_pat_n_s32,
  BI__builtin_sve_svqincw_pat_n_s64,
  BI__builtin_sve_svqincw_pat_n_u32,
  BI__builtin_sve_svqincw_pat_n_u64,
  BI__builtin_sve_svqincw_pat_s32,
  BI__builtin_sve_svqincw_pat_u32,
  BI__builtin_sve_svqincw_s32,
  BI__builtin_sve_svqincw_u32,
  BI__builtin_sve_svqneg_s16_m,
  BI__builtin_sve_svqneg_s16_x,
  BI__builtin_sve_svqneg_s16_z,
  BI__builtin_sve_svqneg_s32_m,
  BI__builtin_sve_svqneg_s32_x,
  BI__builtin_sve_svqneg_s32_z,
  BI__builtin_sve_svqneg_s64_m,
  BI__builtin_sve_svqneg_s64_x,
  BI__builtin_sve_svqneg_s64_z,
  BI__builtin_sve_svqneg_s8_m,
  BI__builtin_sve_svqneg_s8_x,
  BI__builtin_sve_svqneg_s8_z,
  BI__builtin_sve_svqrdcmlah_lane_s16,
  BI__builtin_sve_svqrdcmlah_lane_s32,
  BI__builtin_sve_svqrdcmlah_s16,
  BI__builtin_sve_svqrdcmlah_s32,
  BI__builtin_sve_svqrdcmlah_s64,
  BI__builtin_sve_svqrdcmlah_s8,
  BI__builtin_sve_svqrdmlah_lane_s16,
  BI__builtin_sve_svqrdmlah_lane_s32,
  BI__builtin_sve_svqrdmlah_lane_s64,
  BI__builtin_sve_svqrdmlah_n_s16,
  BI__builtin_sve_svqrdmlah_n_s32,
  BI__builtin_sve_svqrdmlah_n_s64,
  BI__builtin_sve_svqrdmlah_n_s8,
  BI__builtin_sve_svqrdmlah_s16,
  BI__builtin_sve_svqrdmlah_s32,
  BI__builtin_sve_svqrdmlah_s64,
  BI__builtin_sve_svqrdmlah_s8,
  BI__builtin_sve_svqrdmlsh_lane_s16,
  BI__builtin_sve_svqrdmlsh_lane_s32,
  BI__builtin_sve_svqrdmlsh_lane_s64,
  BI__builtin_sve_svqrdmlsh_n_s16,
  BI__builtin_sve_svqrdmlsh_n_s32,
  BI__builtin_sve_svqrdmlsh_n_s64,
  BI__builtin_sve_svqrdmlsh_n_s8,
  BI__builtin_sve_svqrdmlsh_s16,
  BI__builtin_sve_svqrdmlsh_s32,
  BI__builtin_sve_svqrdmlsh_s64,
  BI__builtin_sve_svqrdmlsh_s8,
  BI__builtin_sve_svqrdmulh_lane_s16,
  BI__builtin_sve_svqrdmulh_lane_s32,
  BI__builtin_sve_svqrdmulh_lane_s64,
  BI__builtin_sve_svqrdmulh_n_s16,
  BI__builtin_sve_svqrdmulh_n_s32,
  BI__builtin_sve_svqrdmulh_n_s64,
  BI__builtin_sve_svqrdmulh_n_s8,
  BI__builtin_sve_svqrdmulh_s16,
  BI__builtin_sve_svqrdmulh_s32,
  BI__builtin_sve_svqrdmulh_s64,
  BI__builtin_sve_svqrdmulh_s8,
  BI__builtin_sve_svqrshl_n_s16_m,
  BI__builtin_sve_svqrshl_n_s16_x,
  BI__builtin_sve_svqrshl_n_s16_z,
  BI__builtin_sve_svqrshl_n_s32_m,
  BI__builtin_sve_svqrshl_n_s32_x,
  BI__builtin_sve_svqrshl_n_s32_z,
  BI__builtin_sve_svqrshl_n_s64_m,
  BI__builtin_sve_svqrshl_n_s64_x,
  BI__builtin_sve_svqrshl_n_s64_z,
  BI__builtin_sve_svqrshl_n_s8_m,
  BI__builtin_sve_svqrshl_n_s8_x,
  BI__builtin_sve_svqrshl_n_s8_z,
  BI__builtin_sve_svqrshl_n_u16_m,
  BI__builtin_sve_svqrshl_n_u16_x,
  BI__builtin_sve_svqrshl_n_u16_z,
  BI__builtin_sve_svqrshl_n_u32_m,
  BI__builtin_sve_svqrshl_n_u32_x,
  BI__builtin_sve_svqrshl_n_u32_z,
  BI__builtin_sve_svqrshl_n_u64_m,
  BI__builtin_sve_svqrshl_n_u64_x,
  BI__builtin_sve_svqrshl_n_u64_z,
  BI__builtin_sve_svqrshl_n_u8_m,
  BI__builtin_sve_svqrshl_n_u8_x,
  BI__builtin_sve_svqrshl_n_u8_z,
  BI__builtin_sve_svqrshl_s16_m,
  BI__builtin_sve_svqrshl_s16_x,
  BI__builtin_sve_svqrshl_s16_z,
  BI__builtin_sve_svqrshl_s32_m,
  BI__builtin_sve_svqrshl_s32_x,
  BI__builtin_sve_svqrshl_s32_z,
  BI__builtin_sve_svqrshl_s64_m,
  BI__builtin_sve_svqrshl_s64_x,
  BI__builtin_sve_svqrshl_s64_z,
  BI__builtin_sve_svqrshl_s8_m,
  BI__builtin_sve_svqrshl_s8_x,
  BI__builtin_sve_svqrshl_s8_z,
  BI__builtin_sve_svqrshl_u16_m,
  BI__builtin_sve_svqrshl_u16_x,
  BI__builtin_sve_svqrshl_u16_z,
  BI__builtin_sve_svqrshl_u32_m,
  BI__builtin_sve_svqrshl_u32_x,
  BI__builtin_sve_svqrshl_u32_z,
  BI__builtin_sve_svqrshl_u64_m,
  BI__builtin_sve_svqrshl_u64_x,
  BI__builtin_sve_svqrshl_u64_z,
  BI__builtin_sve_svqrshl_u8_m,
  BI__builtin_sve_svqrshl_u8_x,
  BI__builtin_sve_svqrshl_u8_z,
  BI__builtin_sve_svqrshr_n_s16_s32_x2,
  BI__builtin_sve_svqrshr_n_s16_s64_x4,
  BI__builtin_sve_svqrshr_n_s8_s32_x4,
  BI__builtin_sve_svqrshr_n_u16_u32_x2,
  BI__builtin_sve_svqrshr_n_u16_u64_x4,
  BI__builtin_sve_svqrshr_n_u8_u32_x4,
  BI__builtin_sve_svqrshrn_n_s16_s32_x2,
  BI__builtin_sve_svqrshrn_n_s16_s64_x4,
  BI__builtin_sve_svqrshrn_n_s8_s32_x4,
  BI__builtin_sve_svqrshrn_n_u16_u32_x2,
  BI__builtin_sve_svqrshrn_n_u16_u64_x4,
  BI__builtin_sve_svqrshrn_n_u8_u32_x4,
  BI__builtin_sve_svqrshrnb_n_s16,
  BI__builtin_sve_svqrshrnb_n_s32,
  BI__builtin_sve_svqrshrnb_n_s64,
  BI__builtin_sve_svqrshrnb_n_u16,
  BI__builtin_sve_svqrshrnb_n_u32,
  BI__builtin_sve_svqrshrnb_n_u64,
  BI__builtin_sve_svqrshrnt_n_s16,
  BI__builtin_sve_svqrshrnt_n_s32,
  BI__builtin_sve_svqrshrnt_n_s64,
  BI__builtin_sve_svqrshrnt_n_u16,
  BI__builtin_sve_svqrshrnt_n_u32,
  BI__builtin_sve_svqrshrnt_n_u64,
  BI__builtin_sve_svqrshru_n_u16_s32_x2,
  BI__builtin_sve_svqrshru_n_u16_s64_x4,
  BI__builtin_sve_svqrshru_n_u8_s32_x4,
  BI__builtin_sve_svqrshrun_n_u16_s32_x2,
  BI__builtin_sve_svqrshrun_n_u16_s64_x4,
  BI__builtin_sve_svqrshrun_n_u8_s32_x4,
  BI__builtin_sve_svqrshrunb_n_s16,
  BI__builtin_sve_svqrshrunb_n_s32,
  BI__builtin_sve_svqrshrunb_n_s64,
  BI__builtin_sve_svqrshrunt_n_s16,
  BI__builtin_sve_svqrshrunt_n_s32,
  BI__builtin_sve_svqrshrunt_n_s64,
  BI__builtin_sve_svqshl_n_s16_m,
  BI__builtin_sve_svqshl_n_s16_x,
  BI__builtin_sve_svqshl_n_s16_z,
  BI__builtin_sve_svqshl_n_s32_m,
  BI__builtin_sve_svqshl_n_s32_x,
  BI__builtin_sve_svqshl_n_s32_z,
  BI__builtin_sve_svqshl_n_s64_m,
  BI__builtin_sve_svqshl_n_s64_x,
  BI__builtin_sve_svqshl_n_s64_z,
  BI__builtin_sve_svqshl_n_s8_m,
  BI__builtin_sve_svqshl_n_s8_x,
  BI__builtin_sve_svqshl_n_s8_z,
  BI__builtin_sve_svqshl_n_u16_m,
  BI__builtin_sve_svqshl_n_u16_x,
  BI__builtin_sve_svqshl_n_u16_z,
  BI__builtin_sve_svqshl_n_u32_m,
  BI__builtin_sve_svqshl_n_u32_x,
  BI__builtin_sve_svqshl_n_u32_z,
  BI__builtin_sve_svqshl_n_u64_m,
  BI__builtin_sve_svqshl_n_u64_x,
  BI__builtin_sve_svqshl_n_u64_z,
  BI__builtin_sve_svqshl_n_u8_m,
  BI__builtin_sve_svqshl_n_u8_x,
  BI__builtin_sve_svqshl_n_u8_z,
  BI__builtin_sve_svqshl_s16_m,
  BI__builtin_sve_svqshl_s16_x,
  BI__builtin_sve_svqshl_s16_z,
  BI__builtin_sve_svqshl_s32_m,
  BI__builtin_sve_svqshl_s32_x,
  BI__builtin_sve_svqshl_s32_z,
  BI__builtin_sve_svqshl_s64_m,
  BI__builtin_sve_svqshl_s64_x,
  BI__builtin_sve_svqshl_s64_z,
  BI__builtin_sve_svqshl_s8_m,
  BI__builtin_sve_svqshl_s8_x,
  BI__builtin_sve_svqshl_s8_z,
  BI__builtin_sve_svqshl_u16_m,
  BI__builtin_sve_svqshl_u16_x,
  BI__builtin_sve_svqshl_u16_z,
  BI__builtin_sve_svqshl_u32_m,
  BI__builtin_sve_svqshl_u32_x,
  BI__builtin_sve_svqshl_u32_z,
  BI__builtin_sve_svqshl_u64_m,
  BI__builtin_sve_svqshl_u64_x,
  BI__builtin_sve_svqshl_u64_z,
  BI__builtin_sve_svqshl_u8_m,
  BI__builtin_sve_svqshl_u8_x,
  BI__builtin_sve_svqshl_u8_z,
  BI__builtin_sve_svqshlu_n_s16_m,
  BI__builtin_sve_svqshlu_n_s16_x,
  BI__builtin_sve_svqshlu_n_s16_z,
  BI__builtin_sve_svqshlu_n_s32_m,
  BI__builtin_sve_svqshlu_n_s32_x,
  BI__builtin_sve_svqshlu_n_s32_z,
  BI__builtin_sve_svqshlu_n_s64_m,
  BI__builtin_sve_svqshlu_n_s64_x,
  BI__builtin_sve_svqshlu_n_s64_z,
  BI__builtin_sve_svqshlu_n_s8_m,
  BI__builtin_sve_svqshlu_n_s8_x,
  BI__builtin_sve_svqshlu_n_s8_z,
  BI__builtin_sve_svqshrnb_n_s16,
  BI__builtin_sve_svqshrnb_n_s32,
  BI__builtin_sve_svqshrnb_n_s64,
  BI__builtin_sve_svqshrnb_n_u16,
  BI__builtin_sve_svqshrnb_n_u32,
  BI__builtin_sve_svqshrnb_n_u64,
  BI__builtin_sve_svqshrnt_n_s16,
  BI__builtin_sve_svqshrnt_n_s32,
  BI__builtin_sve_svqshrnt_n_s64,
  BI__builtin_sve_svqshrnt_n_u16,
  BI__builtin_sve_svqshrnt_n_u32,
  BI__builtin_sve_svqshrnt_n_u64,
  BI__builtin_sve_svqshrunb_n_s16,
  BI__builtin_sve_svqshrunb_n_s32,
  BI__builtin_sve_svqshrunb_n_s64,
  BI__builtin_sve_svqshrunt_n_s16,
  BI__builtin_sve_svqshrunt_n_s32,
  BI__builtin_sve_svqshrunt_n_s64,
  BI__builtin_sve_svqsub_n_s16,
  BI__builtin_sve_svqsub_n_s16_m,
  BI__builtin_sve_svqsub_n_s16_x,
  BI__builtin_sve_svqsub_n_s16_z,
  BI__builtin_sve_svqsub_n_s32,
  BI__builtin_sve_svqsub_n_s32_m,
  BI__builtin_sve_svqsub_n_s32_x,
  BI__builtin_sve_svqsub_n_s32_z,
  BI__builtin_sve_svqsub_n_s64,
  BI__builtin_sve_svqsub_n_s64_m,
  BI__builtin_sve_svqsub_n_s64_x,
  BI__builtin_sve_svqsub_n_s64_z,
  BI__builtin_sve_svqsub_n_s8,
  BI__builtin_sve_svqsub_n_s8_m,
  BI__builtin_sve_svqsub_n_s8_x,
  BI__builtin_sve_svqsub_n_s8_z,
  BI__builtin_sve_svqsub_n_u16,
  BI__builtin_sve_svqsub_n_u16_m,
  BI__builtin_sve_svqsub_n_u16_x,
  BI__builtin_sve_svqsub_n_u16_z,
  BI__builtin_sve_svqsub_n_u32,
  BI__builtin_sve_svqsub_n_u32_m,
  BI__builtin_sve_svqsub_n_u32_x,
  BI__builtin_sve_svqsub_n_u32_z,
  BI__builtin_sve_svqsub_n_u64,
  BI__builtin_sve_svqsub_n_u64_m,
  BI__builtin_sve_svqsub_n_u64_x,
  BI__builtin_sve_svqsub_n_u64_z,
  BI__builtin_sve_svqsub_n_u8,
  BI__builtin_sve_svqsub_n_u8_m,
  BI__builtin_sve_svqsub_n_u8_x,
  BI__builtin_sve_svqsub_n_u8_z,
  BI__builtin_sve_svqsub_s16,
  BI__builtin_sve_svqsub_s16_m,
  BI__builtin_sve_svqsub_s16_x,
  BI__builtin_sve_svqsub_s16_z,
  BI__builtin_sve_svqsub_s32,
  BI__builtin_sve_svqsub_s32_m,
  BI__builtin_sve_svqsub_s32_x,
  BI__builtin_sve_svqsub_s32_z,
  BI__builtin_sve_svqsub_s64,
  BI__builtin_sve_svqsub_s64_m,
  BI__builtin_sve_svqsub_s64_x,
  BI__builtin_sve_svqsub_s64_z,
  BI__builtin_sve_svqsub_s8,
  BI__builtin_sve_svqsub_s8_m,
  BI__builtin_sve_svqsub_s8_x,
  BI__builtin_sve_svqsub_s8_z,
  BI__builtin_sve_svqsub_u16,
  BI__builtin_sve_svqsub_u16_m,
  BI__builtin_sve_svqsub_u16_x,
  BI__builtin_sve_svqsub_u16_z,
  BI__builtin_sve_svqsub_u32,
  BI__builtin_sve_svqsub_u32_m,
  BI__builtin_sve_svqsub_u32_x,
  BI__builtin_sve_svqsub_u32_z,
  BI__builtin_sve_svqsub_u64,
  BI__builtin_sve_svqsub_u64_m,
  BI__builtin_sve_svqsub_u64_x,
  BI__builtin_sve_svqsub_u64_z,
  BI__builtin_sve_svqsub_u8,
  BI__builtin_sve_svqsub_u8_m,
  BI__builtin_sve_svqsub_u8_x,
  BI__builtin_sve_svqsub_u8_z,
  BI__builtin_sve_svqsubr_n_s16_m,
  BI__builtin_sve_svqsubr_n_s16_x,
  BI__builtin_sve_svqsubr_n_s16_z,
  BI__builtin_sve_svqsubr_n_s32_m,
  BI__builtin_sve_svqsubr_n_s32_x,
  BI__builtin_sve_svqsubr_n_s32_z,
  BI__builtin_sve_svqsubr_n_s64_m,
  BI__builtin_sve_svqsubr_n_s64_x,
  BI__builtin_sve_svqsubr_n_s64_z,
  BI__builtin_sve_svqsubr_n_s8_m,
  BI__builtin_sve_svqsubr_n_s8_x,
  BI__builtin_sve_svqsubr_n_s8_z,
  BI__builtin_sve_svqsubr_n_u16_m,
  BI__builtin_sve_svqsubr_n_u16_x,
  BI__builtin_sve_svqsubr_n_u16_z,
  BI__builtin_sve_svqsubr_n_u32_m,
  BI__builtin_sve_svqsubr_n_u32_x,
  BI__builtin_sve_svqsubr_n_u32_z,
  BI__builtin_sve_svqsubr_n_u64_m,
  BI__builtin_sve_svqsubr_n_u64_x,
  BI__builtin_sve_svqsubr_n_u64_z,
  BI__builtin_sve_svqsubr_n_u8_m,
  BI__builtin_sve_svqsubr_n_u8_x,
  BI__builtin_sve_svqsubr_n_u8_z,
  BI__builtin_sve_svqsubr_s16_m,
  BI__builtin_sve_svqsubr_s16_x,
  BI__builtin_sve_svqsubr_s16_z,
  BI__builtin_sve_svqsubr_s32_m,
  BI__builtin_sve_svqsubr_s32_x,
  BI__builtin_sve_svqsubr_s32_z,
  BI__builtin_sve_svqsubr_s64_m,
  BI__builtin_sve_svqsubr_s64_x,
  BI__builtin_sve_svqsubr_s64_z,
  BI__builtin_sve_svqsubr_s8_m,
  BI__builtin_sve_svqsubr_s8_x,
  BI__builtin_sve_svqsubr_s8_z,
  BI__builtin_sve_svqsubr_u16_m,
  BI__builtin_sve_svqsubr_u16_x,
  BI__builtin_sve_svqsubr_u16_z,
  BI__builtin_sve_svqsubr_u32_m,
  BI__builtin_sve_svqsubr_u32_x,
  BI__builtin_sve_svqsubr_u32_z,
  BI__builtin_sve_svqsubr_u64_m,
  BI__builtin_sve_svqsubr_u64_x,
  BI__builtin_sve_svqsubr_u64_z,
  BI__builtin_sve_svqsubr_u8_m,
  BI__builtin_sve_svqsubr_u8_x,
  BI__builtin_sve_svqsubr_u8_z,
  BI__builtin_sve_svqxtnb_s16,
  BI__builtin_sve_svqxtnb_s32,
  BI__builtin_sve_svqxtnb_s64,
  BI__builtin_sve_svqxtnb_u16,
  BI__builtin_sve_svqxtnb_u32,
  BI__builtin_sve_svqxtnb_u64,
  BI__builtin_sve_svqxtnt_s16,
  BI__builtin_sve_svqxtnt_s32,
  BI__builtin_sve_svqxtnt_s64,
  BI__builtin_sve_svqxtnt_u16,
  BI__builtin_sve_svqxtnt_u32,
  BI__builtin_sve_svqxtnt_u64,
  BI__builtin_sve_svqxtunb_s16,
  BI__builtin_sve_svqxtunb_s32,
  BI__builtin_sve_svqxtunb_s64,
  BI__builtin_sve_svqxtunt_s16,
  BI__builtin_sve_svqxtunt_s32,
  BI__builtin_sve_svqxtunt_s64,
  BI__builtin_sve_svraddhnb_n_s16,
  BI__builtin_sve_svraddhnb_n_s32,
  BI__builtin_sve_svraddhnb_n_s64,
  BI__builtin_sve_svraddhnb_n_u16,
  BI__builtin_sve_svraddhnb_n_u32,
  BI__builtin_sve_svraddhnb_n_u64,
  BI__builtin_sve_svraddhnb_s16,
  BI__builtin_sve_svraddhnb_s32,
  BI__builtin_sve_svraddhnb_s64,
  BI__builtin_sve_svraddhnb_u16,
  BI__builtin_sve_svraddhnb_u32,
  BI__builtin_sve_svraddhnb_u64,
  BI__builtin_sve_svraddhnt_n_s16,
  BI__builtin_sve_svraddhnt_n_s32,
  BI__builtin_sve_svraddhnt_n_s64,
  BI__builtin_sve_svraddhnt_n_u16,
  BI__builtin_sve_svraddhnt_n_u32,
  BI__builtin_sve_svraddhnt_n_u64,
  BI__builtin_sve_svraddhnt_s16,
  BI__builtin_sve_svraddhnt_s32,
  BI__builtin_sve_svraddhnt_s64,
  BI__builtin_sve_svraddhnt_u16,
  BI__builtin_sve_svraddhnt_u32,
  BI__builtin_sve_svraddhnt_u64,
  BI__builtin_sve_svrax1_s64,
  BI__builtin_sve_svrax1_u64,
  BI__builtin_sve_svrbit_s16_m,
  BI__builtin_sve_svrbit_s16_x,
  BI__builtin_sve_svrbit_s16_z,
  BI__builtin_sve_svrbit_s32_m,
  BI__builtin_sve_svrbit_s32_x,
  BI__builtin_sve_svrbit_s32_z,
  BI__builtin_sve_svrbit_s64_m,
  BI__builtin_sve_svrbit_s64_x,
  BI__builtin_sve_svrbit_s64_z,
  BI__builtin_sve_svrbit_s8_m,
  BI__builtin_sve_svrbit_s8_x,
  BI__builtin_sve_svrbit_s8_z,
  BI__builtin_sve_svrbit_u16_m,
  BI__builtin_sve_svrbit_u16_x,
  BI__builtin_sve_svrbit_u16_z,
  BI__builtin_sve_svrbit_u32_m,
  BI__builtin_sve_svrbit_u32_x,
  BI__builtin_sve_svrbit_u32_z,
  BI__builtin_sve_svrbit_u64_m,
  BI__builtin_sve_svrbit_u64_x,
  BI__builtin_sve_svrbit_u64_z,
  BI__builtin_sve_svrbit_u8_m,
  BI__builtin_sve_svrbit_u8_x,
  BI__builtin_sve_svrbit_u8_z,
  BI__builtin_sve_svrdffr,
  BI__builtin_sve_svrdffr_z,
  BI__builtin_sve_svrecpe_f16,
  BI__builtin_sve_svrecpe_f32,
  BI__builtin_sve_svrecpe_f64,
  BI__builtin_sve_svrecpe_u32_m,
  BI__builtin_sve_svrecpe_u32_x,
  BI__builtin_sve_svrecpe_u32_z,
  BI__builtin_sve_svrecps_f16,
  BI__builtin_sve_svrecps_f32,
  BI__builtin_sve_svrecps_f64,
  BI__builtin_sve_svrecpx_f16_m,
  BI__builtin_sve_svrecpx_f16_x,
  BI__builtin_sve_svrecpx_f16_z,
  BI__builtin_sve_svrecpx_f32_m,
  BI__builtin_sve_svrecpx_f32_x,
  BI__builtin_sve_svrecpx_f32_z,
  BI__builtin_sve_svrecpx_f64_m,
  BI__builtin_sve_svrecpx_f64_x,
  BI__builtin_sve_svrecpx_f64_z,
  BI__builtin_sve_svreinterpret_b,
  BI__builtin_sve_svreinterpret_c,
  BI__builtin_sve_svrev_b16,
  BI__builtin_sve_svrev_b32,
  BI__builtin_sve_svrev_b64,
  BI__builtin_sve_svrev_b8,
  BI__builtin_sve_svrev_bf16,
  BI__builtin_sve_svrev_f16,
  BI__builtin_sve_svrev_f32,
  BI__builtin_sve_svrev_f64,
  BI__builtin_sve_svrev_s16,
  BI__builtin_sve_svrev_s32,
  BI__builtin_sve_svrev_s64,
  BI__builtin_sve_svrev_s8,
  BI__builtin_sve_svrev_u16,
  BI__builtin_sve_svrev_u32,
  BI__builtin_sve_svrev_u64,
  BI__builtin_sve_svrev_u8,
  BI__builtin_sve_svrevb_s16_m,
  BI__builtin_sve_svrevb_s16_x,
  BI__builtin_sve_svrevb_s16_z,
  BI__builtin_sve_svrevb_s32_m,
  BI__builtin_sve_svrevb_s32_x,
  BI__builtin_sve_svrevb_s32_z,
  BI__builtin_sve_svrevb_s64_m,
  BI__builtin_sve_svrevb_s64_x,
  BI__builtin_sve_svrevb_s64_z,
  BI__builtin_sve_svrevb_u16_m,
  BI__builtin_sve_svrevb_u16_x,
  BI__builtin_sve_svrevb_u16_z,
  BI__builtin_sve_svrevb_u32_m,
  BI__builtin_sve_svrevb_u32_x,
  BI__builtin_sve_svrevb_u32_z,
  BI__builtin_sve_svrevb_u64_m,
  BI__builtin_sve_svrevb_u64_x,
  BI__builtin_sve_svrevb_u64_z,
  BI__builtin_sve_svrevd_bf16_m,
  BI__builtin_sve_svrevd_bf16_x,
  BI__builtin_sve_svrevd_bf16_z,
  BI__builtin_sve_svrevd_f16_m,
  BI__builtin_sve_svrevd_f16_x,
  BI__builtin_sve_svrevd_f16_z,
  BI__builtin_sve_svrevd_f32_m,
  BI__builtin_sve_svrevd_f32_x,
  BI__builtin_sve_svrevd_f32_z,
  BI__builtin_sve_svrevd_f64_m,
  BI__builtin_sve_svrevd_f64_x,
  BI__builtin_sve_svrevd_f64_z,
  BI__builtin_sve_svrevd_mf8_m,
  BI__builtin_sve_svrevd_mf8_x,
  BI__builtin_sve_svrevd_mf8_z,
  BI__builtin_sve_svrevd_s16_m,
  BI__builtin_sve_svrevd_s16_x,
  BI__builtin_sve_svrevd_s16_z,
  BI__builtin_sve_svrevd_s32_m,
  BI__builtin_sve_svrevd_s32_x,
  BI__builtin_sve_svrevd_s32_z,
  BI__builtin_sve_svrevd_s64_m,
  BI__builtin_sve_svrevd_s64_x,
  BI__builtin_sve_svrevd_s64_z,
  BI__builtin_sve_svrevd_s8_m,
  BI__builtin_sve_svrevd_s8_x,
  BI__builtin_sve_svrevd_s8_z,
  BI__builtin_sve_svrevd_u16_m,
  BI__builtin_sve_svrevd_u16_x,
  BI__builtin_sve_svrevd_u16_z,
  BI__builtin_sve_svrevd_u32_m,
  BI__builtin_sve_svrevd_u32_x,
  BI__builtin_sve_svrevd_u32_z,
  BI__builtin_sve_svrevd_u64_m,
  BI__builtin_sve_svrevd_u64_x,
  BI__builtin_sve_svrevd_u64_z,
  BI__builtin_sve_svrevd_u8_m,
  BI__builtin_sve_svrevd_u8_x,
  BI__builtin_sve_svrevd_u8_z,
  BI__builtin_sve_svrevh_s32_m,
  BI__builtin_sve_svrevh_s32_x,
  BI__builtin_sve_svrevh_s32_z,
  BI__builtin_sve_svrevh_s64_m,
  BI__builtin_sve_svrevh_s64_x,
  BI__builtin_sve_svrevh_s64_z,
  BI__builtin_sve_svrevh_u32_m,
  BI__builtin_sve_svrevh_u32_x,
  BI__builtin_sve_svrevh_u32_z,
  BI__builtin_sve_svrevh_u64_m,
  BI__builtin_sve_svrevh_u64_x,
  BI__builtin_sve_svrevh_u64_z,
  BI__builtin_sve_svrevw_s64_m,
  BI__builtin_sve_svrevw_s64_x,
  BI__builtin_sve_svrevw_s64_z,
  BI__builtin_sve_svrevw_u64_m,
  BI__builtin_sve_svrevw_u64_x,
  BI__builtin_sve_svrevw_u64_z,
  BI__builtin_sve_svrhadd_n_s16_m,
  BI__builtin_sve_svrhadd_n_s16_x,
  BI__builtin_sve_svrhadd_n_s16_z,
  BI__builtin_sve_svrhadd_n_s32_m,
  BI__builtin_sve_svrhadd_n_s32_x,
  BI__builtin_sve_svrhadd_n_s32_z,
  BI__builtin_sve_svrhadd_n_s64_m,
  BI__builtin_sve_svrhadd_n_s64_x,
  BI__builtin_sve_svrhadd_n_s64_z,
  BI__builtin_sve_svrhadd_n_s8_m,
  BI__builtin_sve_svrhadd_n_s8_x,
  BI__builtin_sve_svrhadd_n_s8_z,
  BI__builtin_sve_svrhadd_n_u16_m,
  BI__builtin_sve_svrhadd_n_u16_x,
  BI__builtin_sve_svrhadd_n_u16_z,
  BI__builtin_sve_svrhadd_n_u32_m,
  BI__builtin_sve_svrhadd_n_u32_x,
  BI__builtin_sve_svrhadd_n_u32_z,
  BI__builtin_sve_svrhadd_n_u64_m,
  BI__builtin_sve_svrhadd_n_u64_x,
  BI__builtin_sve_svrhadd_n_u64_z,
  BI__builtin_sve_svrhadd_n_u8_m,
  BI__builtin_sve_svrhadd_n_u8_x,
  BI__builtin_sve_svrhadd_n_u8_z,
  BI__builtin_sve_svrhadd_s16_m,
  BI__builtin_sve_svrhadd_s16_x,
  BI__builtin_sve_svrhadd_s16_z,
  BI__builtin_sve_svrhadd_s32_m,
  BI__builtin_sve_svrhadd_s32_x,
  BI__builtin_sve_svrhadd_s32_z,
  BI__builtin_sve_svrhadd_s64_m,
  BI__builtin_sve_svrhadd_s64_x,
  BI__builtin_sve_svrhadd_s64_z,
  BI__builtin_sve_svrhadd_s8_m,
  BI__builtin_sve_svrhadd_s8_x,
  BI__builtin_sve_svrhadd_s8_z,
  BI__builtin_sve_svrhadd_u16_m,
  BI__builtin_sve_svrhadd_u16_x,
  BI__builtin_sve_svrhadd_u16_z,
  BI__builtin_sve_svrhadd_u32_m,
  BI__builtin_sve_svrhadd_u32_x,
  BI__builtin_sve_svrhadd_u32_z,
  BI__builtin_sve_svrhadd_u64_m,
  BI__builtin_sve_svrhadd_u64_x,
  BI__builtin_sve_svrhadd_u64_z,
  BI__builtin_sve_svrhadd_u8_m,
  BI__builtin_sve_svrhadd_u8_x,
  BI__builtin_sve_svrhadd_u8_z,
  BI__builtin_sve_svrint32x_f32_m,
  BI__builtin_sve_svrint32x_f32_x,
  BI__builtin_sve_svrint32x_f32_z,
  BI__builtin_sve_svrint32x_f64_m,
  BI__builtin_sve_svrint32x_f64_x,
  BI__builtin_sve_svrint32x_f64_z,
  BI__builtin_sve_svrint32z_f32_m,
  BI__builtin_sve_svrint32z_f32_x,
  BI__builtin_sve_svrint32z_f32_z,
  BI__builtin_sve_svrint32z_f64_m,
  BI__builtin_sve_svrint32z_f64_x,
  BI__builtin_sve_svrint32z_f64_z,
  BI__builtin_sve_svrint64x_f32_m,
  BI__builtin_sve_svrint64x_f32_x,
  BI__builtin_sve_svrint64x_f32_z,
  BI__builtin_sve_svrint64x_f64_m,
  BI__builtin_sve_svrint64x_f64_x,
  BI__builtin_sve_svrint64x_f64_z,
  BI__builtin_sve_svrint64z_f32_m,
  BI__builtin_sve_svrint64z_f32_x,
  BI__builtin_sve_svrint64z_f32_z,
  BI__builtin_sve_svrint64z_f64_m,
  BI__builtin_sve_svrint64z_f64_x,
  BI__builtin_sve_svrint64z_f64_z,
  BI__builtin_sve_svrinta_f16_m,
  BI__builtin_sve_svrinta_f16_x,
  BI__builtin_sve_svrinta_f16_z,
  BI__builtin_sve_svrinta_f32_m,
  BI__builtin_sve_svrinta_f32_x,
  BI__builtin_sve_svrinta_f32_x2,
  BI__builtin_sve_svrinta_f32_x4,
  BI__builtin_sve_svrinta_f32_z,
  BI__builtin_sve_svrinta_f64_m,
  BI__builtin_sve_svrinta_f64_x,
  BI__builtin_sve_svrinta_f64_z,
  BI__builtin_sve_svrinti_f16_m,
  BI__builtin_sve_svrinti_f16_x,
  BI__builtin_sve_svrinti_f16_z,
  BI__builtin_sve_svrinti_f32_m,
  BI__builtin_sve_svrinti_f32_x,
  BI__builtin_sve_svrinti_f32_z,
  BI__builtin_sve_svrinti_f64_m,
  BI__builtin_sve_svrinti_f64_x,
  BI__builtin_sve_svrinti_f64_z,
  BI__builtin_sve_svrintm_f16_m,
  BI__builtin_sve_svrintm_f16_x,
  BI__builtin_sve_svrintm_f16_z,
  BI__builtin_sve_svrintm_f32_m,
  BI__builtin_sve_svrintm_f32_x,
  BI__builtin_sve_svrintm_f32_x2,
  BI__builtin_sve_svrintm_f32_x4,
  BI__builtin_sve_svrintm_f32_z,
  BI__builtin_sve_svrintm_f64_m,
  BI__builtin_sve_svrintm_f64_x,
  BI__builtin_sve_svrintm_f64_z,
  BI__builtin_sve_svrintn_f16_m,
  BI__builtin_sve_svrintn_f16_x,
  BI__builtin_sve_svrintn_f16_z,
  BI__builtin_sve_svrintn_f32_m,
  BI__builtin_sve_svrintn_f32_x,
  BI__builtin_sve_svrintn_f32_x2,
  BI__builtin_sve_svrintn_f32_x4,
  BI__builtin_sve_svrintn_f32_z,
  BI__builtin_sve_svrintn_f64_m,
  BI__builtin_sve_svrintn_f64_x,
  BI__builtin_sve_svrintn_f64_z,
  BI__builtin_sve_svrintp_f16_m,
  BI__builtin_sve_svrintp_f16_x,
  BI__builtin_sve_svrintp_f16_z,
  BI__builtin_sve_svrintp_f32_m,
  BI__builtin_sve_svrintp_f32_x,
  BI__builtin_sve_svrintp_f32_x2,
  BI__builtin_sve_svrintp_f32_x4,
  BI__builtin_sve_svrintp_f32_z,
  BI__builtin_sve_svrintp_f64_m,
  BI__builtin_sve_svrintp_f64_x,
  BI__builtin_sve_svrintp_f64_z,
  BI__builtin_sve_svrintx_f16_m,
  BI__builtin_sve_svrintx_f16_x,
  BI__builtin_sve_svrintx_f16_z,
  BI__builtin_sve_svrintx_f32_m,
  BI__builtin_sve_svrintx_f32_x,
  BI__builtin_sve_svrintx_f32_z,
  BI__builtin_sve_svrintx_f64_m,
  BI__builtin_sve_svrintx_f64_x,
  BI__builtin_sve_svrintx_f64_z,
  BI__builtin_sve_svrintz_f16_m,
  BI__builtin_sve_svrintz_f16_x,
  BI__builtin_sve_svrintz_f16_z,
  BI__builtin_sve_svrintz_f32_m,
  BI__builtin_sve_svrintz_f32_x,
  BI__builtin_sve_svrintz_f32_z,
  BI__builtin_sve_svrintz_f64_m,
  BI__builtin_sve_svrintz_f64_x,
  BI__builtin_sve_svrintz_f64_z,
  BI__builtin_sve_svrshl_n_s16_m,
  BI__builtin_sve_svrshl_n_s16_x,
  BI__builtin_sve_svrshl_n_s16_z,
  BI__builtin_sve_svrshl_n_s32_m,
  BI__builtin_sve_svrshl_n_s32_x,
  BI__builtin_sve_svrshl_n_s32_z,
  BI__builtin_sve_svrshl_n_s64_m,
  BI__builtin_sve_svrshl_n_s64_x,
  BI__builtin_sve_svrshl_n_s64_z,
  BI__builtin_sve_svrshl_n_s8_m,
  BI__builtin_sve_svrshl_n_s8_x,
  BI__builtin_sve_svrshl_n_s8_z,
  BI__builtin_sve_svrshl_n_u16_m,
  BI__builtin_sve_svrshl_n_u16_x,
  BI__builtin_sve_svrshl_n_u16_z,
  BI__builtin_sve_svrshl_n_u32_m,
  BI__builtin_sve_svrshl_n_u32_x,
  BI__builtin_sve_svrshl_n_u32_z,
  BI__builtin_sve_svrshl_n_u64_m,
  BI__builtin_sve_svrshl_n_u64_x,
  BI__builtin_sve_svrshl_n_u64_z,
  BI__builtin_sve_svrshl_n_u8_m,
  BI__builtin_sve_svrshl_n_u8_x,
  BI__builtin_sve_svrshl_n_u8_z,
  BI__builtin_sve_svrshl_s16_m,
  BI__builtin_sve_svrshl_s16_x,
  BI__builtin_sve_svrshl_s16_x2,
  BI__builtin_sve_svrshl_s16_x4,
  BI__builtin_sve_svrshl_s16_z,
  BI__builtin_sve_svrshl_s32_m,
  BI__builtin_sve_svrshl_s32_x,
  BI__builtin_sve_svrshl_s32_x2,
  BI__builtin_sve_svrshl_s32_x4,
  BI__builtin_sve_svrshl_s32_z,
  BI__builtin_sve_svrshl_s64_m,
  BI__builtin_sve_svrshl_s64_x,
  BI__builtin_sve_svrshl_s64_x2,
  BI__builtin_sve_svrshl_s64_x4,
  BI__builtin_sve_svrshl_s64_z,
  BI__builtin_sve_svrshl_s8_m,
  BI__builtin_sve_svrshl_s8_x,
  BI__builtin_sve_svrshl_s8_x2,
  BI__builtin_sve_svrshl_s8_x4,
  BI__builtin_sve_svrshl_s8_z,
  BI__builtin_sve_svrshl_single_s16_x2,
  BI__builtin_sve_svrshl_single_s16_x4,
  BI__builtin_sve_svrshl_single_s32_x2,
  BI__builtin_sve_svrshl_single_s32_x4,
  BI__builtin_sve_svrshl_single_s64_x2,
  BI__builtin_sve_svrshl_single_s64_x4,
  BI__builtin_sve_svrshl_single_s8_x2,
  BI__builtin_sve_svrshl_single_s8_x4,
  BI__builtin_sve_svrshl_single_u16_x2,
  BI__builtin_sve_svrshl_single_u16_x4,
  BI__builtin_sve_svrshl_single_u32_x2,
  BI__builtin_sve_svrshl_single_u32_x4,
  BI__builtin_sve_svrshl_single_u64_x2,
  BI__builtin_sve_svrshl_single_u64_x4,
  BI__builtin_sve_svrshl_single_u8_x2,
  BI__builtin_sve_svrshl_single_u8_x4,
  BI__builtin_sve_svrshl_u16_m,
  BI__builtin_sve_svrshl_u16_x,
  BI__builtin_sve_svrshl_u16_x2,
  BI__builtin_sve_svrshl_u16_x4,
  BI__builtin_sve_svrshl_u16_z,
  BI__builtin_sve_svrshl_u32_m,
  BI__builtin_sve_svrshl_u32_x,
  BI__builtin_sve_svrshl_u32_x2,
  BI__builtin_sve_svrshl_u32_x4,
  BI__builtin_sve_svrshl_u32_z,
  BI__builtin_sve_svrshl_u64_m,
  BI__builtin_sve_svrshl_u64_x,
  BI__builtin_sve_svrshl_u64_x2,
  BI__builtin_sve_svrshl_u64_x4,
  BI__builtin_sve_svrshl_u64_z,
  BI__builtin_sve_svrshl_u8_m,
  BI__builtin_sve_svrshl_u8_x,
  BI__builtin_sve_svrshl_u8_x2,
  BI__builtin_sve_svrshl_u8_x4,
  BI__builtin_sve_svrshl_u8_z,
  BI__builtin_sve_svrshr_n_s16_m,
  BI__builtin_sve_svrshr_n_s16_x,
  BI__builtin_sve_svrshr_n_s16_z,
  BI__builtin_sve_svrshr_n_s32_m,
  BI__builtin_sve_svrshr_n_s32_x,
  BI__builtin_sve_svrshr_n_s32_z,
  BI__builtin_sve_svrshr_n_s64_m,
  BI__builtin_sve_svrshr_n_s64_x,
  BI__builtin_sve_svrshr_n_s64_z,
  BI__builtin_sve_svrshr_n_s8_m,
  BI__builtin_sve_svrshr_n_s8_x,
  BI__builtin_sve_svrshr_n_s8_z,
  BI__builtin_sve_svrshr_n_u16_m,
  BI__builtin_sve_svrshr_n_u16_x,
  BI__builtin_sve_svrshr_n_u16_z,
  BI__builtin_sve_svrshr_n_u32_m,
  BI__builtin_sve_svrshr_n_u32_x,
  BI__builtin_sve_svrshr_n_u32_z,
  BI__builtin_sve_svrshr_n_u64_m,
  BI__builtin_sve_svrshr_n_u64_x,
  BI__builtin_sve_svrshr_n_u64_z,
  BI__builtin_sve_svrshr_n_u8_m,
  BI__builtin_sve_svrshr_n_u8_x,
  BI__builtin_sve_svrshr_n_u8_z,
  BI__builtin_sve_svrshrnb_n_s16,
  BI__builtin_sve_svrshrnb_n_s32,
  BI__builtin_sve_svrshrnb_n_s64,
  BI__builtin_sve_svrshrnb_n_u16,
  BI__builtin_sve_svrshrnb_n_u32,
  BI__builtin_sve_svrshrnb_n_u64,
  BI__builtin_sve_svrshrnt_n_s16,
  BI__builtin_sve_svrshrnt_n_s32,
  BI__builtin_sve_svrshrnt_n_s64,
  BI__builtin_sve_svrshrnt_n_u16,
  BI__builtin_sve_svrshrnt_n_u32,
  BI__builtin_sve_svrshrnt_n_u64,
  BI__builtin_sve_svrsqrte_f16,
  BI__builtin_sve_svrsqrte_f32,
  BI__builtin_sve_svrsqrte_f64,
  BI__builtin_sve_svrsqrte_u32_m,
  BI__builtin_sve_svrsqrte_u32_x,
  BI__builtin_sve_svrsqrte_u32_z,
  BI__builtin_sve_svrsqrts_f16,
  BI__builtin_sve_svrsqrts_f32,
  BI__builtin_sve_svrsqrts_f64,
  BI__builtin_sve_svrsra_n_s16,
  BI__builtin_sve_svrsra_n_s32,
  BI__builtin_sve_svrsra_n_s64,
  BI__builtin_sve_svrsra_n_s8,
  BI__builtin_sve_svrsra_n_u16,
  BI__builtin_sve_svrsra_n_u32,
  BI__builtin_sve_svrsra_n_u64,
  BI__builtin_sve_svrsra_n_u8,
  BI__builtin_sve_svrsubhnb_n_s16,
  BI__builtin_sve_svrsubhnb_n_s32,
  BI__builtin_sve_svrsubhnb_n_s64,
  BI__builtin_sve_svrsubhnb_n_u16,
  BI__builtin_sve_svrsubhnb_n_u32,
  BI__builtin_sve_svrsubhnb_n_u64,
  BI__builtin_sve_svrsubhnb_s16,
  BI__builtin_sve_svrsubhnb_s32,
  BI__builtin_sve_svrsubhnb_s64,
  BI__builtin_sve_svrsubhnb_u16,
  BI__builtin_sve_svrsubhnb_u32,
  BI__builtin_sve_svrsubhnb_u64,
  BI__builtin_sve_svrsubhnt_n_s16,
  BI__builtin_sve_svrsubhnt_n_s32,
  BI__builtin_sve_svrsubhnt_n_s64,
  BI__builtin_sve_svrsubhnt_n_u16,
  BI__builtin_sve_svrsubhnt_n_u32,
  BI__builtin_sve_svrsubhnt_n_u64,
  BI__builtin_sve_svrsubhnt_s16,
  BI__builtin_sve_svrsubhnt_s32,
  BI__builtin_sve_svrsubhnt_s64,
  BI__builtin_sve_svrsubhnt_u16,
  BI__builtin_sve_svrsubhnt_u32,
  BI__builtin_sve_svrsubhnt_u64,
  BI__builtin_sve_svsbclb_n_u32,
  BI__builtin_sve_svsbclb_n_u64,
  BI__builtin_sve_svsbclb_u32,
  BI__builtin_sve_svsbclb_u64,
  BI__builtin_sve_svsbclt_n_u32,
  BI__builtin_sve_svsbclt_n_u64,
  BI__builtin_sve_svsbclt_u32,
  BI__builtin_sve_svsbclt_u64,
  BI__builtin_sve_svscale_bf16_m,
  BI__builtin_sve_svscale_bf16_x,
  BI__builtin_sve_svscale_bf16_x2,
  BI__builtin_sve_svscale_bf16_x4,
  BI__builtin_sve_svscale_bf16_z,
  BI__builtin_sve_svscale_f16_m,
  BI__builtin_sve_svscale_f16_x,
  BI__builtin_sve_svscale_f16_x2,
  BI__builtin_sve_svscale_f16_x4,
  BI__builtin_sve_svscale_f16_z,
  BI__builtin_sve_svscale_f32_m,
  BI__builtin_sve_svscale_f32_x,
  BI__builtin_sve_svscale_f32_x2,
  BI__builtin_sve_svscale_f32_x4,
  BI__builtin_sve_svscale_f32_z,
  BI__builtin_sve_svscale_f64_m,
  BI__builtin_sve_svscale_f64_x,
  BI__builtin_sve_svscale_f64_x2,
  BI__builtin_sve_svscale_f64_x4,
  BI__builtin_sve_svscale_f64_z,
  BI__builtin_sve_svscale_n_bf16_m,
  BI__builtin_sve_svscale_n_bf16_x,
  BI__builtin_sve_svscale_n_bf16_z,
  BI__builtin_sve_svscale_n_f16_m,
  BI__builtin_sve_svscale_n_f16_x,
  BI__builtin_sve_svscale_n_f16_z,
  BI__builtin_sve_svscale_n_f32_m,
  BI__builtin_sve_svscale_n_f32_x,
  BI__builtin_sve_svscale_n_f32_z,
  BI__builtin_sve_svscale_n_f64_m,
  BI__builtin_sve_svscale_n_f64_x,
  BI__builtin_sve_svscale_n_f64_z,
  BI__builtin_sve_svscale_single_bf16_x2,
  BI__builtin_sve_svscale_single_bf16_x4,
  BI__builtin_sve_svscale_single_f16_x2,
  BI__builtin_sve_svscale_single_f16_x4,
  BI__builtin_sve_svscale_single_f32_x2,
  BI__builtin_sve_svscale_single_f32_x4,
  BI__builtin_sve_svscale_single_f64_x2,
  BI__builtin_sve_svscale_single_f64_x4,
  BI__builtin_sve_svsel_b,
  BI__builtin_sve_svsel_bf16,
  BI__builtin_sve_svsel_bf16_x2,
  BI__builtin_sve_svsel_bf16_x4,
  BI__builtin_sve_svsel_f16,
  BI__builtin_sve_svsel_f16_x2,
  BI__builtin_sve_svsel_f16_x4,
  BI__builtin_sve_svsel_f32,
  BI__builtin_sve_svsel_f32_x2,
  BI__builtin_sve_svsel_f32_x4,
  BI__builtin_sve_svsel_f64,
  BI__builtin_sve_svsel_f64_x2,
  BI__builtin_sve_svsel_f64_x4,
  BI__builtin_sve_svsel_mf8_x2,
  BI__builtin_sve_svsel_mf8_x4,
  BI__builtin_sve_svsel_s16,
  BI__builtin_sve_svsel_s16_x2,
  BI__builtin_sve_svsel_s16_x4,
  BI__builtin_sve_svsel_s32,
  BI__builtin_sve_svsel_s32_x2,
  BI__builtin_sve_svsel_s32_x4,
  BI__builtin_sve_svsel_s64,
  BI__builtin_sve_svsel_s64_x2,
  BI__builtin_sve_svsel_s64_x4,
  BI__builtin_sve_svsel_s8,
  BI__builtin_sve_svsel_s8_x2,
  BI__builtin_sve_svsel_s8_x4,
  BI__builtin_sve_svsel_u16,
  BI__builtin_sve_svsel_u16_x2,
  BI__builtin_sve_svsel_u16_x4,
  BI__builtin_sve_svsel_u32,
  BI__builtin_sve_svsel_u32_x2,
  BI__builtin_sve_svsel_u32_x4,
  BI__builtin_sve_svsel_u64,
  BI__builtin_sve_svsel_u64_x2,
  BI__builtin_sve_svsel_u64_x4,
  BI__builtin_sve_svsel_u8,
  BI__builtin_sve_svsel_u8_x2,
  BI__builtin_sve_svsel_u8_x4,
  BI__builtin_sve_svset2_b,
  BI__builtin_sve_svset2_bf16,
  BI__builtin_sve_svset2_f16,
  BI__builtin_sve_svset2_f32,
  BI__builtin_sve_svset2_f64,
  BI__builtin_sve_svset2_mf8,
  BI__builtin_sve_svset2_s16,
  BI__builtin_sve_svset2_s32,
  BI__builtin_sve_svset2_s64,
  BI__builtin_sve_svset2_s8,
  BI__builtin_sve_svset2_u16,
  BI__builtin_sve_svset2_u32,
  BI__builtin_sve_svset2_u64,
  BI__builtin_sve_svset2_u8,
  BI__builtin_sve_svset3_bf16,
  BI__builtin_sve_svset3_f16,
  BI__builtin_sve_svset3_f32,
  BI__builtin_sve_svset3_f64,
  BI__builtin_sve_svset3_mf8,
  BI__builtin_sve_svset3_s16,
  BI__builtin_sve_svset3_s32,
  BI__builtin_sve_svset3_s64,
  BI__builtin_sve_svset3_s8,
  BI__builtin_sve_svset3_u16,
  BI__builtin_sve_svset3_u32,
  BI__builtin_sve_svset3_u64,
  BI__builtin_sve_svset3_u8,
  BI__builtin_sve_svset4_b,
  BI__builtin_sve_svset4_bf16,
  BI__builtin_sve_svset4_f16,
  BI__builtin_sve_svset4_f32,
  BI__builtin_sve_svset4_f64,
  BI__builtin_sve_svset4_mf8,
  BI__builtin_sve_svset4_s16,
  BI__builtin_sve_svset4_s32,
  BI__builtin_sve_svset4_s64,
  BI__builtin_sve_svset4_s8,
  BI__builtin_sve_svset4_u16,
  BI__builtin_sve_svset4_u32,
  BI__builtin_sve_svset4_u64,
  BI__builtin_sve_svset4_u8,
  BI__builtin_sve_svsetffr,
  BI__builtin_sve_svshllb_n_s16,
  BI__builtin_sve_svshllb_n_s32,
  BI__builtin_sve_svshllb_n_s64,
  BI__builtin_sve_svshllb_n_u16,
  BI__builtin_sve_svshllb_n_u32,
  BI__builtin_sve_svshllb_n_u64,
  BI__builtin_sve_svshllt_n_s16,
  BI__builtin_sve_svshllt_n_s32,
  BI__builtin_sve_svshllt_n_s64,
  BI__builtin_sve_svshllt_n_u16,
  BI__builtin_sve_svshllt_n_u32,
  BI__builtin_sve_svshllt_n_u64,
  BI__builtin_sve_svshrnb_n_s16,
  BI__builtin_sve_svshrnb_n_s32,
  BI__builtin_sve_svshrnb_n_s64,
  BI__builtin_sve_svshrnb_n_u16,
  BI__builtin_sve_svshrnb_n_u32,
  BI__builtin_sve_svshrnb_n_u64,
  BI__builtin_sve_svshrnt_n_s16,
  BI__builtin_sve_svshrnt_n_s32,
  BI__builtin_sve_svshrnt_n_s64,
  BI__builtin_sve_svshrnt_n_u16,
  BI__builtin_sve_svshrnt_n_u32,
  BI__builtin_sve_svshrnt_n_u64,
  BI__builtin_sve_svsli_n_s16,
  BI__builtin_sve_svsli_n_s32,
  BI__builtin_sve_svsli_n_s64,
  BI__builtin_sve_svsli_n_s8,
  BI__builtin_sve_svsli_n_u16,
  BI__builtin_sve_svsli_n_u32,
  BI__builtin_sve_svsli_n_u64,
  BI__builtin_sve_svsli_n_u8,
  BI__builtin_sve_svsm4e_u32,
  BI__builtin_sve_svsm4ekey_u32,
  BI__builtin_sve_svsplice_bf16,
  BI__builtin_sve_svsplice_f16,
  BI__builtin_sve_svsplice_f32,
  BI__builtin_sve_svsplice_f64,
  BI__builtin_sve_svsplice_s16,
  BI__builtin_sve_svsplice_s32,
  BI__builtin_sve_svsplice_s64,
  BI__builtin_sve_svsplice_s8,
  BI__builtin_sve_svsplice_u16,
  BI__builtin_sve_svsplice_u32,
  BI__builtin_sve_svsplice_u64,
  BI__builtin_sve_svsplice_u8,
  BI__builtin_sve_svsqadd_n_u16_m,
  BI__builtin_sve_svsqadd_n_u16_x,
  BI__builtin_sve_svsqadd_n_u16_z,
  BI__builtin_sve_svsqadd_n_u32_m,
  BI__builtin_sve_svsqadd_n_u32_x,
  BI__builtin_sve_svsqadd_n_u32_z,
  BI__builtin_sve_svsqadd_n_u64_m,
  BI__builtin_sve_svsqadd_n_u64_x,
  BI__builtin_sve_svsqadd_n_u64_z,
  BI__builtin_sve_svsqadd_n_u8_m,
  BI__builtin_sve_svsqadd_n_u8_x,
  BI__builtin_sve_svsqadd_n_u8_z,
  BI__builtin_sve_svsqadd_u16_m,
  BI__builtin_sve_svsqadd_u16_x,
  BI__builtin_sve_svsqadd_u16_z,
  BI__builtin_sve_svsqadd_u32_m,
  BI__builtin_sve_svsqadd_u32_x,
  BI__builtin_sve_svsqadd_u32_z,
  BI__builtin_sve_svsqadd_u64_m,
  BI__builtin_sve_svsqadd_u64_x,
  BI__builtin_sve_svsqadd_u64_z,
  BI__builtin_sve_svsqadd_u8_m,
  BI__builtin_sve_svsqadd_u8_x,
  BI__builtin_sve_svsqadd_u8_z,
  BI__builtin_sve_svsqrt_f16_m,
  BI__builtin_sve_svsqrt_f16_x,
  BI__builtin_sve_svsqrt_f16_z,
  BI__builtin_sve_svsqrt_f32_m,
  BI__builtin_sve_svsqrt_f32_x,
  BI__builtin_sve_svsqrt_f32_z,
  BI__builtin_sve_svsqrt_f64_m,
  BI__builtin_sve_svsqrt_f64_x,
  BI__builtin_sve_svsqrt_f64_z,
  BI__builtin_sve_svsra_n_s16,
  BI__builtin_sve_svsra_n_s32,
  BI__builtin_sve_svsra_n_s64,
  BI__builtin_sve_svsra_n_s8,
  BI__builtin_sve_svsra_n_u16,
  BI__builtin_sve_svsra_n_u32,
  BI__builtin_sve_svsra_n_u64,
  BI__builtin_sve_svsra_n_u8,
  BI__builtin_sve_svsri_n_s16,
  BI__builtin_sve_svsri_n_s32,
  BI__builtin_sve_svsri_n_s64,
  BI__builtin_sve_svsri_n_s8,
  BI__builtin_sve_svsri_n_u16,
  BI__builtin_sve_svsri_n_u32,
  BI__builtin_sve_svsri_n_u64,
  BI__builtin_sve_svsri_n_u8,
  BI__builtin_sve_svst1_bf16,
  BI__builtin_sve_svst1_bf16_x2,
  BI__builtin_sve_svst1_bf16_x4,
  BI__builtin_sve_svst1_f16,
  BI__builtin_sve_svst1_f16_x2,
  BI__builtin_sve_svst1_f16_x4,
  BI__builtin_sve_svst1_f32,
  BI__builtin_sve_svst1_f32_x2,
  BI__builtin_sve_svst1_f32_x4,
  BI__builtin_sve_svst1_f64,
  BI__builtin_sve_svst1_f64_x2,
  BI__builtin_sve_svst1_f64_x4,
  BI__builtin_sve_svst1_mf8,
  BI__builtin_sve_svst1_mf8_x2,
  BI__builtin_sve_svst1_mf8_x4,
  BI__builtin_sve_svst1_s16,
  BI__builtin_sve_svst1_s16_x2,
  BI__builtin_sve_svst1_s16_x4,
  BI__builtin_sve_svst1_s32,
  BI__builtin_sve_svst1_s32_x2,
  BI__builtin_sve_svst1_s32_x4,
  BI__builtin_sve_svst1_s64,
  BI__builtin_sve_svst1_s64_x2,
  BI__builtin_sve_svst1_s64_x4,
  BI__builtin_sve_svst1_s8,
  BI__builtin_sve_svst1_s8_x2,
  BI__builtin_sve_svst1_s8_x4,
  BI__builtin_sve_svst1_scatter_s32index_f32,
  BI__builtin_sve_svst1_scatter_s32index_s32,
  BI__builtin_sve_svst1_scatter_s32index_u32,
  BI__builtin_sve_svst1_scatter_s32offset_f32,
  BI__builtin_sve_svst1_scatter_s32offset_s32,
  BI__builtin_sve_svst1_scatter_s32offset_u32,
  BI__builtin_sve_svst1_scatter_s64index_f64,
  BI__builtin_sve_svst1_scatter_s64index_s64,
  BI__builtin_sve_svst1_scatter_s64index_u64,
  BI__builtin_sve_svst1_scatter_s64offset_f64,
  BI__builtin_sve_svst1_scatter_s64offset_s64,
  BI__builtin_sve_svst1_scatter_s64offset_u64,
  BI__builtin_sve_svst1_scatter_u32base_f32,
  BI__builtin_sve_svst1_scatter_u32base_index_f32,
  BI__builtin_sve_svst1_scatter_u32base_index_s32,
  BI__builtin_sve_svst1_scatter_u32base_index_u32,
  BI__builtin_sve_svst1_scatter_u32base_offset_f32,
  BI__builtin_sve_svst1_scatter_u32base_offset_s32,
  BI__builtin_sve_svst1_scatter_u32base_offset_u32,
  BI__builtin_sve_svst1_scatter_u32base_s32,
  BI__builtin_sve_svst1_scatter_u32base_u32,
  BI__builtin_sve_svst1_scatter_u32index_f32,
  BI__builtin_sve_svst1_scatter_u32index_s32,
  BI__builtin_sve_svst1_scatter_u32index_u32,
  BI__builtin_sve_svst1_scatter_u32offset_f32,
  BI__builtin_sve_svst1_scatter_u32offset_s32,
  BI__builtin_sve_svst1_scatter_u32offset_u32,
  BI__builtin_sve_svst1_scatter_u64base_f64,
  BI__builtin_sve_svst1_scatter_u64base_index_f64,
  BI__builtin_sve_svst1_scatter_u64base_index_s64,
  BI__builtin_sve_svst1_scatter_u64base_index_u64,
  BI__builtin_sve_svst1_scatter_u64base_offset_f64,
  BI__builtin_sve_svst1_scatter_u64base_offset_s64,
  BI__builtin_sve_svst1_scatter_u64base_offset_u64,
  BI__builtin_sve_svst1_scatter_u64base_s64,
  BI__builtin_sve_svst1_scatter_u64base_u64,
  BI__builtin_sve_svst1_scatter_u64index_f64,
  BI__builtin_sve_svst1_scatter_u64index_s64,
  BI__builtin_sve_svst1_scatter_u64index_u64,
  BI__builtin_sve_svst1_scatter_u64offset_f64,
  BI__builtin_sve_svst1_scatter_u64offset_s64,
  BI__builtin_sve_svst1_scatter_u64offset_u64,
  BI__builtin_sve_svst1_u16,
  BI__builtin_sve_svst1_u16_x2,
  BI__builtin_sve_svst1_u16_x4,
  BI__builtin_sve_svst1_u32,
  BI__builtin_sve_svst1_u32_x2,
  BI__builtin_sve_svst1_u32_x4,
  BI__builtin_sve_svst1_u64,
  BI__builtin_sve_svst1_u64_x2,
  BI__builtin_sve_svst1_u64_x4,
  BI__builtin_sve_svst1_u8,
  BI__builtin_sve_svst1_u8_x2,
  BI__builtin_sve_svst1_u8_x4,
  BI__builtin_sve_svst1_vnum_bf16,
  BI__builtin_sve_svst1_vnum_bf16_x2,
  BI__builtin_sve_svst1_vnum_bf16_x4,
  BI__builtin_sve_svst1_vnum_f16,
  BI__builtin_sve_svst1_vnum_f16_x2,
  BI__builtin_sve_svst1_vnum_f16_x4,
  BI__builtin_sve_svst1_vnum_f32,
  BI__builtin_sve_svst1_vnum_f32_x2,
  BI__builtin_sve_svst1_vnum_f32_x4,
  BI__builtin_sve_svst1_vnum_f64,
  BI__builtin_sve_svst1_vnum_f64_x2,
  BI__builtin_sve_svst1_vnum_f64_x4,
  BI__builtin_sve_svst1_vnum_mf8,
  BI__builtin_sve_svst1_vnum_mf8_x2,
  BI__builtin_sve_svst1_vnum_mf8_x4,
  BI__builtin_sve_svst1_vnum_s16,
  BI__builtin_sve_svst1_vnum_s16_x2,
  BI__builtin_sve_svst1_vnum_s16_x4,
  BI__builtin_sve_svst1_vnum_s32,
  BI__builtin_sve_svst1_vnum_s32_x2,
  BI__builtin_sve_svst1_vnum_s32_x4,
  BI__builtin_sve_svst1_vnum_s64,
  BI__builtin_sve_svst1_vnum_s64_x2,
  BI__builtin_sve_svst1_vnum_s64_x4,
  BI__builtin_sve_svst1_vnum_s8,
  BI__builtin_sve_svst1_vnum_s8_x2,
  BI__builtin_sve_svst1_vnum_s8_x4,
  BI__builtin_sve_svst1_vnum_u16,
  BI__builtin_sve_svst1_vnum_u16_x2,
  BI__builtin_sve_svst1_vnum_u16_x4,
  BI__builtin_sve_svst1_vnum_u32,
  BI__builtin_sve_svst1_vnum_u32_x2,
  BI__builtin_sve_svst1_vnum_u32_x4,
  BI__builtin_sve_svst1_vnum_u64,
  BI__builtin_sve_svst1_vnum_u64_x2,
  BI__builtin_sve_svst1_vnum_u64_x4,
  BI__builtin_sve_svst1_vnum_u8,
  BI__builtin_sve_svst1_vnum_u8_x2,
  BI__builtin_sve_svst1_vnum_u8_x4,
  BI__builtin_sve_svst1b_s16,
  BI__builtin_sve_svst1b_s32,
  BI__builtin_sve_svst1b_s64,
  BI__builtin_sve_svst1b_scatter_s32offset_s32,
  BI__builtin_sve_svst1b_scatter_s32offset_u32,
  BI__builtin_sve_svst1b_scatter_s64offset_s64,
  BI__builtin_sve_svst1b_scatter_s64offset_u64,
  BI__builtin_sve_svst1b_scatter_u32base_offset_s32,
  BI__builtin_sve_svst1b_scatter_u32base_offset_u32,
  BI__builtin_sve_svst1b_scatter_u32base_s32,
  BI__builtin_sve_svst1b_scatter_u32base_u32,
  BI__builtin_sve_svst1b_scatter_u32offset_s32,
  BI__builtin_sve_svst1b_scatter_u32offset_u32,
  BI__builtin_sve_svst1b_scatter_u64base_offset_s64,
  BI__builtin_sve_svst1b_scatter_u64base_offset_u64,
  BI__builtin_sve_svst1b_scatter_u64base_s64,
  BI__builtin_sve_svst1b_scatter_u64base_u64,
  BI__builtin_sve_svst1b_scatter_u64offset_s64,
  BI__builtin_sve_svst1b_scatter_u64offset_u64,
  BI__builtin_sve_svst1b_u16,
  BI__builtin_sve_svst1b_u32,
  BI__builtin_sve_svst1b_u64,
  BI__builtin_sve_svst1b_vnum_s16,
  BI__builtin_sve_svst1b_vnum_s32,
  BI__builtin_sve_svst1b_vnum_s64,
  BI__builtin_sve_svst1b_vnum_u16,
  BI__builtin_sve_svst1b_vnum_u32,
  BI__builtin_sve_svst1b_vnum_u64,
  BI__builtin_sve_svst1dq_f64,
  BI__builtin_sve_svst1dq_s64,
  BI__builtin_sve_svst1dq_u64,
  BI__builtin_sve_svst1dq_vnum_f64,
  BI__builtin_sve_svst1dq_vnum_s64,
  BI__builtin_sve_svst1dq_vnum_u64,
  BI__builtin_sve_svst1h_s32,
  BI__builtin_sve_svst1h_s64,
  BI__builtin_sve_svst1h_scatter_s32index_s32,
  BI__builtin_sve_svst1h_scatter_s32index_u32,
  BI__builtin_sve_svst1h_scatter_s32offset_s32,
  BI__builtin_sve_svst1h_scatter_s32offset_u32,
  BI__builtin_sve_svst1h_scatter_s64index_s64,
  BI__builtin_sve_svst1h_scatter_s64index_u64,
  BI__builtin_sve_svst1h_scatter_s64offset_s64,
  BI__builtin_sve_svst1h_scatter_s64offset_u64,
  BI__builtin_sve_svst1h_scatter_u32base_index_s32,
  BI__builtin_sve_svst1h_scatter_u32base_index_u32,
  BI__builtin_sve_svst1h_scatter_u32base_offset_s32,
  BI__builtin_sve_svst1h_scatter_u32base_offset_u32,
  BI__builtin_sve_svst1h_scatter_u32base_s32,
  BI__builtin_sve_svst1h_scatter_u32base_u32,
  BI__builtin_sve_svst1h_scatter_u32index_s32,
  BI__builtin_sve_svst1h_scatter_u32index_u32,
  BI__builtin_sve_svst1h_scatter_u32offset_s32,
  BI__builtin_sve_svst1h_scatter_u32offset_u32,
  BI__builtin_sve_svst1h_scatter_u64base_index_s64,
  BI__builtin_sve_svst1h_scatter_u64base_index_u64,
  BI__builtin_sve_svst1h_scatter_u64base_offset_s64,
  BI__builtin_sve_svst1h_scatter_u64base_offset_u64,
  BI__builtin_sve_svst1h_scatter_u64base_s64,
  BI__builtin_sve_svst1h_scatter_u64base_u64,
  BI__builtin_sve_svst1h_scatter_u64index_s64,
  BI__builtin_sve_svst1h_scatter_u64index_u64,
  BI__builtin_sve_svst1h_scatter_u64offset_s64,
  BI__builtin_sve_svst1h_scatter_u64offset_u64,
  BI__builtin_sve_svst1h_u32,
  BI__builtin_sve_svst1h_u64,
  BI__builtin_sve_svst1h_vnum_s32,
  BI__builtin_sve_svst1h_vnum_s64,
  BI__builtin_sve_svst1h_vnum_u32,
  BI__builtin_sve_svst1h_vnum_u64,
  BI__builtin_sve_svst1q_scatter_s64index_bf16,
  BI__builtin_sve_svst1q_scatter_s64index_f16,
  BI__builtin_sve_svst1q_scatter_s64index_f32,
  BI__builtin_sve_svst1q_scatter_s64index_f64,
  BI__builtin_sve_svst1q_scatter_s64index_s16,
  BI__builtin_sve_svst1q_scatter_s64index_s32,
  BI__builtin_sve_svst1q_scatter_s64index_s64,
  BI__builtin_sve_svst1q_scatter_s64index_u16,
  BI__builtin_sve_svst1q_scatter_s64index_u32,
  BI__builtin_sve_svst1q_scatter_s64index_u64,
  BI__builtin_sve_svst1q_scatter_s64offset_bf16,
  BI__builtin_sve_svst1q_scatter_s64offset_f16,
  BI__builtin_sve_svst1q_scatter_s64offset_f32,
  BI__builtin_sve_svst1q_scatter_s64offset_f64,
  BI__builtin_sve_svst1q_scatter_s64offset_mf8,
  BI__builtin_sve_svst1q_scatter_s64offset_s16,
  BI__builtin_sve_svst1q_scatter_s64offset_s32,
  BI__builtin_sve_svst1q_scatter_s64offset_s64,
  BI__builtin_sve_svst1q_scatter_s64offset_s8,
  BI__builtin_sve_svst1q_scatter_s64offset_u16,
  BI__builtin_sve_svst1q_scatter_s64offset_u32,
  BI__builtin_sve_svst1q_scatter_s64offset_u64,
  BI__builtin_sve_svst1q_scatter_s64offset_u8,
  BI__builtin_sve_svst1q_scatter_u64base_bf16,
  BI__builtin_sve_svst1q_scatter_u64base_f16,
  BI__builtin_sve_svst1q_scatter_u64base_f32,
  BI__builtin_sve_svst1q_scatter_u64base_f64,
  BI__builtin_sve_svst1q_scatter_u64base_index_bf16,
  BI__builtin_sve_svst1q_scatter_u64base_index_f16,
  BI__builtin_sve_svst1q_scatter_u64base_index_f32,
  BI__builtin_sve_svst1q_scatter_u64base_index_f64,
  BI__builtin_sve_svst1q_scatter_u64base_index_s16,
  BI__builtin_sve_svst1q_scatter_u64base_index_s32,
  BI__builtin_sve_svst1q_scatter_u64base_index_s64,
  BI__builtin_sve_svst1q_scatter_u64base_index_u16,
  BI__builtin_sve_svst1q_scatter_u64base_index_u32,
  BI__builtin_sve_svst1q_scatter_u64base_index_u64,
  BI__builtin_sve_svst1q_scatter_u64base_mf8,
  BI__builtin_sve_svst1q_scatter_u64base_offset_bf16,
  BI__builtin_sve_svst1q_scatter_u64base_offset_f16,
  BI__builtin_sve_svst1q_scatter_u64base_offset_f32,
  BI__builtin_sve_svst1q_scatter_u64base_offset_f64,
  BI__builtin_sve_svst1q_scatter_u64base_offset_mf8,
  BI__builtin_sve_svst1q_scatter_u64base_offset_s16,
  BI__builtin_sve_svst1q_scatter_u64base_offset_s32,
  BI__builtin_sve_svst1q_scatter_u64base_offset_s64,
  BI__builtin_sve_svst1q_scatter_u64base_offset_s8,
  BI__builtin_sve_svst1q_scatter_u64base_offset_u16,
  BI__builtin_sve_svst1q_scatter_u64base_offset_u32,
  BI__builtin_sve_svst1q_scatter_u64base_offset_u64,
  BI__builtin_sve_svst1q_scatter_u64base_offset_u8,
  BI__builtin_sve_svst1q_scatter_u64base_s16,
  BI__builtin_sve_svst1q_scatter_u64base_s32,
  BI__builtin_sve_svst1q_scatter_u64base_s64,
  BI__builtin_sve_svst1q_scatter_u64base_s8,
  BI__builtin_sve_svst1q_scatter_u64base_u16,
  BI__builtin_sve_svst1q_scatter_u64base_u32,
  BI__builtin_sve_svst1q_scatter_u64base_u64,
  BI__builtin_sve_svst1q_scatter_u64base_u8,
  BI__builtin_sve_svst1q_scatter_u64index_bf16,
  BI__builtin_sve_svst1q_scatter_u64index_f16,
  BI__builtin_sve_svst1q_scatter_u64index_f32,
  BI__builtin_sve_svst1q_scatter_u64index_f64,
  BI__builtin_sve_svst1q_scatter_u64index_s16,
  BI__builtin_sve_svst1q_scatter_u64index_s32,
  BI__builtin_sve_svst1q_scatter_u64index_s64,
  BI__builtin_sve_svst1q_scatter_u64index_u16,
  BI__builtin_sve_svst1q_scatter_u64index_u32,
  BI__builtin_sve_svst1q_scatter_u64index_u64,
  BI__builtin_sve_svst1q_scatter_u64offset_bf16,
  BI__builtin_sve_svst1q_scatter_u64offset_f16,
  BI__builtin_sve_svst1q_scatter_u64offset_f32,
  BI__builtin_sve_svst1q_scatter_u64offset_f64,
  BI__builtin_sve_svst1q_scatter_u64offset_mf8,
  BI__builtin_sve_svst1q_scatter_u64offset_s16,
  BI__builtin_sve_svst1q_scatter_u64offset_s32,
  BI__builtin_sve_svst1q_scatter_u64offset_s64,
  BI__builtin_sve_svst1q_scatter_u64offset_s8,
  BI__builtin_sve_svst1q_scatter_u64offset_u16,
  BI__builtin_sve_svst1q_scatter_u64offset_u32,
  BI__builtin_sve_svst1q_scatter_u64offset_u64,
  BI__builtin_sve_svst1q_scatter_u64offset_u8,
  BI__builtin_sve_svst1w_s64,
  BI__builtin_sve_svst1w_scatter_s64index_s64,
  BI__builtin_sve_svst1w_scatter_s64index_u64,
  BI__builtin_sve_svst1w_scatter_s64offset_s64,
  BI__builtin_sve_svst1w_scatter_s64offset_u64,
  BI__builtin_sve_svst1w_scatter_u64base_index_s64,
  BI__builtin_sve_svst1w_scatter_u64base_index_u64,
  BI__builtin_sve_svst1w_scatter_u64base_offset_s64,
  BI__builtin_sve_svst1w_scatter_u64base_offset_u64,
  BI__builtin_sve_svst1w_scatter_u64base_s64,
  BI__builtin_sve_svst1w_scatter_u64base_u64,
  BI__builtin_sve_svst1w_scatter_u64index_s64,
  BI__builtin_sve_svst1w_scatter_u64index_u64,
  BI__builtin_sve_svst1w_scatter_u64offset_s64,
  BI__builtin_sve_svst1w_scatter_u64offset_u64,
  BI__builtin_sve_svst1w_u64,
  BI__builtin_sve_svst1w_vnum_s64,
  BI__builtin_sve_svst1w_vnum_u64,
  BI__builtin_sve_svst1wq_f32,
  BI__builtin_sve_svst1wq_s32,
  BI__builtin_sve_svst1wq_u32,
  BI__builtin_sve_svst1wq_vnum_f32,
  BI__builtin_sve_svst1wq_vnum_s32,
  BI__builtin_sve_svst1wq_vnum_u32,
  BI__builtin_sve_svst2_bf16,
  BI__builtin_sve_svst2_f16,
  BI__builtin_sve_svst2_f32,
  BI__builtin_sve_svst2_f64,
  BI__builtin_sve_svst2_mf8,
  BI__builtin_sve_svst2_s16,
  BI__builtin_sve_svst2_s32,
  BI__builtin_sve_svst2_s64,
  BI__builtin_sve_svst2_s8,
  BI__builtin_sve_svst2_u16,
  BI__builtin_sve_svst2_u32,
  BI__builtin_sve_svst2_u64,
  BI__builtin_sve_svst2_u8,
  BI__builtin_sve_svst2_vnum_bf16,
  BI__builtin_sve_svst2_vnum_f16,
  BI__builtin_sve_svst2_vnum_f32,
  BI__builtin_sve_svst2_vnum_f64,
  BI__builtin_sve_svst2_vnum_mf8,
  BI__builtin_sve_svst2_vnum_s16,
  BI__builtin_sve_svst2_vnum_s32,
  BI__builtin_sve_svst2_vnum_s64,
  BI__builtin_sve_svst2_vnum_s8,
  BI__builtin_sve_svst2_vnum_u16,
  BI__builtin_sve_svst2_vnum_u32,
  BI__builtin_sve_svst2_vnum_u64,
  BI__builtin_sve_svst2_vnum_u8,
  BI__builtin_sve_svst2q_bf16,
  BI__builtin_sve_svst2q_f16,
  BI__builtin_sve_svst2q_f32,
  BI__builtin_sve_svst2q_f64,
  BI__builtin_sve_svst2q_mf8,
  BI__builtin_sve_svst2q_s16,
  BI__builtin_sve_svst2q_s32,
  BI__builtin_sve_svst2q_s64,
  BI__builtin_sve_svst2q_s8,
  BI__builtin_sve_svst2q_u16,
  BI__builtin_sve_svst2q_u32,
  BI__builtin_sve_svst2q_u64,
  BI__builtin_sve_svst2q_u8,
  BI__builtin_sve_svst2q_vnum_bf16,
  BI__builtin_sve_svst2q_vnum_f16,
  BI__builtin_sve_svst2q_vnum_f32,
  BI__builtin_sve_svst2q_vnum_f64,
  BI__builtin_sve_svst2q_vnum_mf8,
  BI__builtin_sve_svst2q_vnum_s16,
  BI__builtin_sve_svst2q_vnum_s32,
  BI__builtin_sve_svst2q_vnum_s64,
  BI__builtin_sve_svst2q_vnum_s8,
  BI__builtin_sve_svst2q_vnum_u16,
  BI__builtin_sve_svst2q_vnum_u32,
  BI__builtin_sve_svst2q_vnum_u64,
  BI__builtin_sve_svst2q_vnum_u8,
  BI__builtin_sve_svst3_bf16,
  BI__builtin_sve_svst3_f16,
  BI__builtin_sve_svst3_f32,
  BI__builtin_sve_svst3_f64,
  BI__builtin_sve_svst3_mf8,
  BI__builtin_sve_svst3_s16,
  BI__builtin_sve_svst3_s32,
  BI__builtin_sve_svst3_s64,
  BI__builtin_sve_svst3_s8,
  BI__builtin_sve_svst3_u16,
  BI__builtin_sve_svst3_u32,
  BI__builtin_sve_svst3_u64,
  BI__builtin_sve_svst3_u8,
  BI__builtin_sve_svst3_vnum_bf16,
  BI__builtin_sve_svst3_vnum_f16,
  BI__builtin_sve_svst3_vnum_f32,
  BI__builtin_sve_svst3_vnum_f64,
  BI__builtin_sve_svst3_vnum_mf8,
  BI__builtin_sve_svst3_vnum_s16,
  BI__builtin_sve_svst3_vnum_s32,
  BI__builtin_sve_svst3_vnum_s64,
  BI__builtin_sve_svst3_vnum_s8,
  BI__builtin_sve_svst3_vnum_u16,
  BI__builtin_sve_svst3_vnum_u32,
  BI__builtin_sve_svst3_vnum_u64,
  BI__builtin_sve_svst3_vnum_u8,
  BI__builtin_sve_svst3q_bf16,
  BI__builtin_sve_svst3q_f16,
  BI__builtin_sve_svst3q_f32,
  BI__builtin_sve_svst3q_f64,
  BI__builtin_sve_svst3q_mf8,
  BI__builtin_sve_svst3q_s16,
  BI__builtin_sve_svst3q_s32,
  BI__builtin_sve_svst3q_s64,
  BI__builtin_sve_svst3q_s8,
  BI__builtin_sve_svst3q_u16,
  BI__builtin_sve_svst3q_u32,
  BI__builtin_sve_svst3q_u64,
  BI__builtin_sve_svst3q_u8,
  BI__builtin_sve_svst3q_vnum_bf16,
  BI__builtin_sve_svst3q_vnum_f16,
  BI__builtin_sve_svst3q_vnum_f32,
  BI__builtin_sve_svst3q_vnum_f64,
  BI__builtin_sve_svst3q_vnum_mf8,
  BI__builtin_sve_svst3q_vnum_s16,
  BI__builtin_sve_svst3q_vnum_s32,
  BI__builtin_sve_svst3q_vnum_s64,
  BI__builtin_sve_svst3q_vnum_s8,
  BI__builtin_sve_svst3q_vnum_u16,
  BI__builtin_sve_svst3q_vnum_u32,
  BI__builtin_sve_svst3q_vnum_u64,
  BI__builtin_sve_svst3q_vnum_u8,
  BI__builtin_sve_svst4_bf16,
  BI__builtin_sve_svst4_f16,
  BI__builtin_sve_svst4_f32,
  BI__builtin_sve_svst4_f64,
  BI__builtin_sve_svst4_mf8,
  BI__builtin_sve_svst4_s16,
  BI__builtin_sve_svst4_s32,
  BI__builtin_sve_svst4_s64,
  BI__builtin_sve_svst4_s8,
  BI__builtin_sve_svst4_u16,
  BI__builtin_sve_svst4_u32,
  BI__builtin_sve_svst4_u64,
  BI__builtin_sve_svst4_u8,
  BI__builtin_sve_svst4_vnum_bf16,
  BI__builtin_sve_svst4_vnum_f16,
  BI__builtin_sve_svst4_vnum_f32,
  BI__builtin_sve_svst4_vnum_f64,
  BI__builtin_sve_svst4_vnum_mf8,
  BI__builtin_sve_svst4_vnum_s16,
  BI__builtin_sve_svst4_vnum_s32,
  BI__builtin_sve_svst4_vnum_s64,
  BI__builtin_sve_svst4_vnum_s8,
  BI__builtin_sve_svst4_vnum_u16,
  BI__builtin_sve_svst4_vnum_u32,
  BI__builtin_sve_svst4_vnum_u64,
  BI__builtin_sve_svst4_vnum_u8,
  BI__builtin_sve_svst4q_bf16,
  BI__builtin_sve_svst4q_f16,
  BI__builtin_sve_svst4q_f32,
  BI__builtin_sve_svst4q_f64,
  BI__builtin_sve_svst4q_mf8,
  BI__builtin_sve_svst4q_s16,
  BI__builtin_sve_svst4q_s32,
  BI__builtin_sve_svst4q_s64,
  BI__builtin_sve_svst4q_s8,
  BI__builtin_sve_svst4q_u16,
  BI__builtin_sve_svst4q_u32,
  BI__builtin_sve_svst4q_u64,
  BI__builtin_sve_svst4q_u8,
  BI__builtin_sve_svst4q_vnum_bf16,
  BI__builtin_sve_svst4q_vnum_f16,
  BI__builtin_sve_svst4q_vnum_f32,
  BI__builtin_sve_svst4q_vnum_f64,
  BI__builtin_sve_svst4q_vnum_mf8,
  BI__builtin_sve_svst4q_vnum_s16,
  BI__builtin_sve_svst4q_vnum_s32,
  BI__builtin_sve_svst4q_vnum_s64,
  BI__builtin_sve_svst4q_vnum_s8,
  BI__builtin_sve_svst4q_vnum_u16,
  BI__builtin_sve_svst4q_vnum_u32,
  BI__builtin_sve_svst4q_vnum_u64,
  BI__builtin_sve_svst4q_vnum_u8,
  BI__builtin_sve_svstnt1_bf16,
  BI__builtin_sve_svstnt1_bf16_x2,
  BI__builtin_sve_svstnt1_bf16_x4,
  BI__builtin_sve_svstnt1_f16,
  BI__builtin_sve_svstnt1_f16_x2,
  BI__builtin_sve_svstnt1_f16_x4,
  BI__builtin_sve_svstnt1_f32,
  BI__builtin_sve_svstnt1_f32_x2,
  BI__builtin_sve_svstnt1_f32_x4,
  BI__builtin_sve_svstnt1_f64,
  BI__builtin_sve_svstnt1_f64_x2,
  BI__builtin_sve_svstnt1_f64_x4,
  BI__builtin_sve_svstnt1_mf8,
  BI__builtin_sve_svstnt1_mf8_x2,
  BI__builtin_sve_svstnt1_mf8_x4,
  BI__builtin_sve_svstnt1_s16,
  BI__builtin_sve_svstnt1_s16_x2,
  BI__builtin_sve_svstnt1_s16_x4,
  BI__builtin_sve_svstnt1_s32,
  BI__builtin_sve_svstnt1_s32_x2,
  BI__builtin_sve_svstnt1_s32_x4,
  BI__builtin_sve_svstnt1_s64,
  BI__builtin_sve_svstnt1_s64_x2,
  BI__builtin_sve_svstnt1_s64_x4,
  BI__builtin_sve_svstnt1_s8,
  BI__builtin_sve_svstnt1_s8_x2,
  BI__builtin_sve_svstnt1_s8_x4,
  BI__builtin_sve_svstnt1_scatter_s64index_f64,
  BI__builtin_sve_svstnt1_scatter_s64index_s64,
  BI__builtin_sve_svstnt1_scatter_s64index_u64,
  BI__builtin_sve_svstnt1_scatter_s64offset_f64,
  BI__builtin_sve_svstnt1_scatter_s64offset_s64,
  BI__builtin_sve_svstnt1_scatter_s64offset_u64,
  BI__builtin_sve_svstnt1_scatter_u32base_f32,
  BI__builtin_sve_svstnt1_scatter_u32base_index_f32,
  BI__builtin_sve_svstnt1_scatter_u32base_index_s32,
  BI__builtin_sve_svstnt1_scatter_u32base_index_u32,
  BI__builtin_sve_svstnt1_scatter_u32base_offset_f32,
  BI__builtin_sve_svstnt1_scatter_u32base_offset_s32,
  BI__builtin_sve_svstnt1_scatter_u32base_offset_u32,
  BI__builtin_sve_svstnt1_scatter_u32base_s32,
  BI__builtin_sve_svstnt1_scatter_u32base_u32,
  BI__builtin_sve_svstnt1_scatter_u32offset_f32,
  BI__builtin_sve_svstnt1_scatter_u32offset_s32,
  BI__builtin_sve_svstnt1_scatter_u32offset_u32,
  BI__builtin_sve_svstnt1_scatter_u64base_f64,
  BI__builtin_sve_svstnt1_scatter_u64base_index_f64,
  BI__builtin_sve_svstnt1_scatter_u64base_index_s64,
  BI__builtin_sve_svstnt1_scatter_u64base_index_u64,
  BI__builtin_sve_svstnt1_scatter_u64base_offset_f64,
  BI__builtin_sve_svstnt1_scatter_u64base_offset_s64,
  BI__builtin_sve_svstnt1_scatter_u64base_offset_u64,
  BI__builtin_sve_svstnt1_scatter_u64base_s64,
  BI__builtin_sve_svstnt1_scatter_u64base_u64,
  BI__builtin_sve_svstnt1_scatter_u64index_f64,
  BI__builtin_sve_svstnt1_scatter_u64index_s64,
  BI__builtin_sve_svstnt1_scatter_u64index_u64,
  BI__builtin_sve_svstnt1_scatter_u64offset_f64,
  BI__builtin_sve_svstnt1_scatter_u64offset_s64,
  BI__builtin_sve_svstnt1_scatter_u64offset_u64,
  BI__builtin_sve_svstnt1_u16,
  BI__builtin_sve_svstnt1_u16_x2,
  BI__builtin_sve_svstnt1_u16_x4,
  BI__builtin_sve_svstnt1_u32,
  BI__builtin_sve_svstnt1_u32_x2,
  BI__builtin_sve_svstnt1_u32_x4,
  BI__builtin_sve_svstnt1_u64,
  BI__builtin_sve_svstnt1_u64_x2,
  BI__builtin_sve_svstnt1_u64_x4,
  BI__builtin_sve_svstnt1_u8,
  BI__builtin_sve_svstnt1_u8_x2,
  BI__builtin_sve_svstnt1_u8_x4,
  BI__builtin_sve_svstnt1_vnum_bf16,
  BI__builtin_sve_svstnt1_vnum_bf16_x2,
  BI__builtin_sve_svstnt1_vnum_bf16_x4,
  BI__builtin_sve_svstnt1_vnum_f16,
  BI__builtin_sve_svstnt1_vnum_f16_x2,
  BI__builtin_sve_svstnt1_vnum_f16_x4,
  BI__builtin_sve_svstnt1_vnum_f32,
  BI__builtin_sve_svstnt1_vnum_f32_x2,
  BI__builtin_sve_svstnt1_vnum_f32_x4,
  BI__builtin_sve_svstnt1_vnum_f64,
  BI__builtin_sve_svstnt1_vnum_f64_x2,
  BI__builtin_sve_svstnt1_vnum_f64_x4,
  BI__builtin_sve_svstnt1_vnum_mf8,
  BI__builtin_sve_svstnt1_vnum_mf8_x2,
  BI__builtin_sve_svstnt1_vnum_mf8_x4,
  BI__builtin_sve_svstnt1_vnum_s16,
  BI__builtin_sve_svstnt1_vnum_s16_x2,
  BI__builtin_sve_svstnt1_vnum_s16_x4,
  BI__builtin_sve_svstnt1_vnum_s32,
  BI__builtin_sve_svstnt1_vnum_s32_x2,
  BI__builtin_sve_svstnt1_vnum_s32_x4,
  BI__builtin_sve_svstnt1_vnum_s64,
  BI__builtin_sve_svstnt1_vnum_s64_x2,
  BI__builtin_sve_svstnt1_vnum_s64_x4,
  BI__builtin_sve_svstnt1_vnum_s8,
  BI__builtin_sve_svstnt1_vnum_s8_x2,
  BI__builtin_sve_svstnt1_vnum_s8_x4,
  BI__builtin_sve_svstnt1_vnum_u16,
  BI__builtin_sve_svstnt1_vnum_u16_x2,
  BI__builtin_sve_svstnt1_vnum_u16_x4,
  BI__builtin_sve_svstnt1_vnum_u32,
  BI__builtin_sve_svstnt1_vnum_u32_x2,
  BI__builtin_sve_svstnt1_vnum_u32_x4,
  BI__builtin_sve_svstnt1_vnum_u64,
  BI__builtin_sve_svstnt1_vnum_u64_x2,
  BI__builtin_sve_svstnt1_vnum_u64_x4,
  BI__builtin_sve_svstnt1_vnum_u8,
  BI__builtin_sve_svstnt1_vnum_u8_x2,
  BI__builtin_sve_svstnt1_vnum_u8_x4,
  BI__builtin_sve_svstnt1b_scatter_s64offset_s64,
  BI__builtin_sve_svstnt1b_scatter_s64offset_u64,
  BI__builtin_sve_svstnt1b_scatter_u32base_offset_s32,
  BI__builtin_sve_svstnt1b_scatter_u32base_offset_u32,
  BI__builtin_sve_svstnt1b_scatter_u32base_s32,
  BI__builtin_sve_svstnt1b_scatter_u32base_u32,
  BI__builtin_sve_svstnt1b_scatter_u32offset_s32,
  BI__builtin_sve_svstnt1b_scatter_u32offset_u32,
  BI__builtin_sve_svstnt1b_scatter_u64base_offset_s64,
  BI__builtin_sve_svstnt1b_scatter_u64base_offset_u64,
  BI__builtin_sve_svstnt1b_scatter_u64base_s64,
  BI__builtin_sve_svstnt1b_scatter_u64base_u64,
  BI__builtin_sve_svstnt1b_scatter_u64offset_s64,
  BI__builtin_sve_svstnt1b_scatter_u64offset_u64,
  BI__builtin_sve_svstnt1h_scatter_s64index_s64,
  BI__builtin_sve_svstnt1h_scatter_s64index_u64,
  BI__builtin_sve_svstnt1h_scatter_s64offset_s64,
  BI__builtin_sve_svstnt1h_scatter_s64offset_u64,
  BI__builtin_sve_svstnt1h_scatter_u32base_index_s32,
  BI__builtin_sve_svstnt1h_scatter_u32base_index_u32,
  BI__builtin_sve_svstnt1h_scatter_u32base_offset_s32,
  BI__builtin_sve_svstnt1h_scatter_u32base_offset_u32,
  BI__builtin_sve_svstnt1h_scatter_u32base_s32,
  BI__builtin_sve_svstnt1h_scatter_u32base_u32,
  BI__builtin_sve_svstnt1h_scatter_u32offset_s32,
  BI__builtin_sve_svstnt1h_scatter_u32offset_u32,
  BI__builtin_sve_svstnt1h_scatter_u64base_index_s64,
  BI__builtin_sve_svstnt1h_scatter_u64base_index_u64,
  BI__builtin_sve_svstnt1h_scatter_u64base_offset_s64,
  BI__builtin_sve_svstnt1h_scatter_u64base_offset_u64,
  BI__builtin_sve_svstnt1h_scatter_u64base_s64,
  BI__builtin_sve_svstnt1h_scatter_u64base_u64,
  BI__builtin_sve_svstnt1h_scatter_u64index_s64,
  BI__builtin_sve_svstnt1h_scatter_u64index_u64,
  BI__builtin_sve_svstnt1h_scatter_u64offset_s64,
  BI__builtin_sve_svstnt1h_scatter_u64offset_u64,
  BI__builtin_sve_svstnt1w_scatter_s64index_s64,
  BI__builtin_sve_svstnt1w_scatter_s64index_u64,
  BI__builtin_sve_svstnt1w_scatter_s64offset_s64,
  BI__builtin_sve_svstnt1w_scatter_s64offset_u64,
  BI__builtin_sve_svstnt1w_scatter_u64base_index_s64,
  BI__builtin_sve_svstnt1w_scatter_u64base_index_u64,
  BI__builtin_sve_svstnt1w_scatter_u64base_offset_s64,
  BI__builtin_sve_svstnt1w_scatter_u64base_offset_u64,
  BI__builtin_sve_svstnt1w_scatter_u64base_s64,
  BI__builtin_sve_svstnt1w_scatter_u64base_u64,
  BI__builtin_sve_svstnt1w_scatter_u64index_s64,
  BI__builtin_sve_svstnt1w_scatter_u64index_u64,
  BI__builtin_sve_svstnt1w_scatter_u64offset_s64,
  BI__builtin_sve_svstnt1w_scatter_u64offset_u64,
  BI__builtin_sve_svsub_bf16_m,
  BI__builtin_sve_svsub_bf16_x,
  BI__builtin_sve_svsub_bf16_z,
  BI__builtin_sve_svsub_f16_m,
  BI__builtin_sve_svsub_f16_x,
  BI__builtin_sve_svsub_f16_z,
  BI__builtin_sve_svsub_f32_m,
  BI__builtin_sve_svsub_f32_x,
  BI__builtin_sve_svsub_f32_z,
  BI__builtin_sve_svsub_f64_m,
  BI__builtin_sve_svsub_f64_x,
  BI__builtin_sve_svsub_f64_z,
  BI__builtin_sve_svsub_n_bf16_m,
  BI__builtin_sve_svsub_n_bf16_x,
  BI__builtin_sve_svsub_n_bf16_z,
  BI__builtin_sve_svsub_n_f16_m,
  BI__builtin_sve_svsub_n_f16_x,
  BI__builtin_sve_svsub_n_f16_z,
  BI__builtin_sve_svsub_n_f32_m,
  BI__builtin_sve_svsub_n_f32_x,
  BI__builtin_sve_svsub_n_f32_z,
  BI__builtin_sve_svsub_n_f64_m,
  BI__builtin_sve_svsub_n_f64_x,
  BI__builtin_sve_svsub_n_f64_z,
  BI__builtin_sve_svsub_n_s16_m,
  BI__builtin_sve_svsub_n_s16_x,
  BI__builtin_sve_svsub_n_s16_z,
  BI__builtin_sve_svsub_n_s32_m,
  BI__builtin_sve_svsub_n_s32_x,
  BI__builtin_sve_svsub_n_s32_z,
  BI__builtin_sve_svsub_n_s64_m,
  BI__builtin_sve_svsub_n_s64_x,
  BI__builtin_sve_svsub_n_s64_z,
  BI__builtin_sve_svsub_n_s8_m,
  BI__builtin_sve_svsub_n_s8_x,
  BI__builtin_sve_svsub_n_s8_z,
  BI__builtin_sve_svsub_n_u16_m,
  BI__builtin_sve_svsub_n_u16_x,
  BI__builtin_sve_svsub_n_u16_z,
  BI__builtin_sve_svsub_n_u32_m,
  BI__builtin_sve_svsub_n_u32_x,
  BI__builtin_sve_svsub_n_u32_z,
  BI__builtin_sve_svsub_n_u64_m,
  BI__builtin_sve_svsub_n_u64_x,
  BI__builtin_sve_svsub_n_u64_z,
  BI__builtin_sve_svsub_n_u8_m,
  BI__builtin_sve_svsub_n_u8_x,
  BI__builtin_sve_svsub_n_u8_z,
  BI__builtin_sve_svsub_s16_m,
  BI__builtin_sve_svsub_s16_x,
  BI__builtin_sve_svsub_s16_z,
  BI__builtin_sve_svsub_s32_m,
  BI__builtin_sve_svsub_s32_x,
  BI__builtin_sve_svsub_s32_z,
  BI__builtin_sve_svsub_s64_m,
  BI__builtin_sve_svsub_s64_x,
  BI__builtin_sve_svsub_s64_z,
  BI__builtin_sve_svsub_s8_m,
  BI__builtin_sve_svsub_s8_x,
  BI__builtin_sve_svsub_s8_z,
  BI__builtin_sve_svsub_u16_m,
  BI__builtin_sve_svsub_u16_x,
  BI__builtin_sve_svsub_u16_z,
  BI__builtin_sve_svsub_u32_m,
  BI__builtin_sve_svsub_u32_x,
  BI__builtin_sve_svsub_u32_z,
  BI__builtin_sve_svsub_u64_m,
  BI__builtin_sve_svsub_u64_x,
  BI__builtin_sve_svsub_u64_z,
  BI__builtin_sve_svsub_u8_m,
  BI__builtin_sve_svsub_u8_x,
  BI__builtin_sve_svsub_u8_z,
  BI__builtin_sve_svsubhnb_n_s16,
  BI__builtin_sve_svsubhnb_n_s32,
  BI__builtin_sve_svsubhnb_n_s64,
  BI__builtin_sve_svsubhnb_n_u16,
  BI__builtin_sve_svsubhnb_n_u32,
  BI__builtin_sve_svsubhnb_n_u64,
  BI__builtin_sve_svsubhnb_s16,
  BI__builtin_sve_svsubhnb_s32,
  BI__builtin_sve_svsubhnb_s64,
  BI__builtin_sve_svsubhnb_u16,
  BI__builtin_sve_svsubhnb_u32,
  BI__builtin_sve_svsubhnb_u64,
  BI__builtin_sve_svsubhnt_n_s16,
  BI__builtin_sve_svsubhnt_n_s32,
  BI__builtin_sve_svsubhnt_n_s64,
  BI__builtin_sve_svsubhnt_n_u16,
  BI__builtin_sve_svsubhnt_n_u32,
  BI__builtin_sve_svsubhnt_n_u64,
  BI__builtin_sve_svsubhnt_s16,
  BI__builtin_sve_svsubhnt_s32,
  BI__builtin_sve_svsubhnt_s64,
  BI__builtin_sve_svsubhnt_u16,
  BI__builtin_sve_svsubhnt_u32,
  BI__builtin_sve_svsubhnt_u64,
  BI__builtin_sve_svsublb_n_s16,
  BI__builtin_sve_svsublb_n_s32,
  BI__builtin_sve_svsublb_n_s64,
  BI__builtin_sve_svsublb_n_u16,
  BI__builtin_sve_svsublb_n_u32,
  BI__builtin_sve_svsublb_n_u64,
  BI__builtin_sve_svsublb_s16,
  BI__builtin_sve_svsublb_s32,
  BI__builtin_sve_svsublb_s64,
  BI__builtin_sve_svsublb_u16,
  BI__builtin_sve_svsublb_u32,
  BI__builtin_sve_svsublb_u64,
  BI__builtin_sve_svsublbt_n_s16,
  BI__builtin_sve_svsublbt_n_s32,
  BI__builtin_sve_svsublbt_n_s64,
  BI__builtin_sve_svsublbt_s16,
  BI__builtin_sve_svsublbt_s32,
  BI__builtin_sve_svsublbt_s64,
  BI__builtin_sve_svsublt_n_s16,
  BI__builtin_sve_svsublt_n_s32,
  BI__builtin_sve_svsublt_n_s64,
  BI__builtin_sve_svsublt_n_u16,
  BI__builtin_sve_svsublt_n_u32,
  BI__builtin_sve_svsublt_n_u64,
  BI__builtin_sve_svsublt_s16,
  BI__builtin_sve_svsublt_s32,
  BI__builtin_sve_svsublt_s64,
  BI__builtin_sve_svsublt_u16,
  BI__builtin_sve_svsublt_u32,
  BI__builtin_sve_svsublt_u64,
  BI__builtin_sve_svsubltb_n_s16,
  BI__builtin_sve_svsubltb_n_s32,
  BI__builtin_sve_svsubltb_n_s64,
  BI__builtin_sve_svsubltb_s16,
  BI__builtin_sve_svsubltb_s32,
  BI__builtin_sve_svsubltb_s64,
  BI__builtin_sve_svsubr_f16_m,
  BI__builtin_sve_svsubr_f16_x,
  BI__builtin_sve_svsubr_f16_z,
  BI__builtin_sve_svsubr_f32_m,
  BI__builtin_sve_svsubr_f32_x,
  BI__builtin_sve_svsubr_f32_z,
  BI__builtin_sve_svsubr_f64_m,
  BI__builtin_sve_svsubr_f64_x,
  BI__builtin_sve_svsubr_f64_z,
  BI__builtin_sve_svsubr_n_f16_m,
  BI__builtin_sve_svsubr_n_f16_x,
  BI__builtin_sve_svsubr_n_f16_z,
  BI__builtin_sve_svsubr_n_f32_m,
  BI__builtin_sve_svsubr_n_f32_x,
  BI__builtin_sve_svsubr_n_f32_z,
  BI__builtin_sve_svsubr_n_f64_m,
  BI__builtin_sve_svsubr_n_f64_x,
  BI__builtin_sve_svsubr_n_f64_z,
  BI__builtin_sve_svsubr_n_s16_m,
  BI__builtin_sve_svsubr_n_s16_x,
  BI__builtin_sve_svsubr_n_s16_z,
  BI__builtin_sve_svsubr_n_s32_m,
  BI__builtin_sve_svsubr_n_s32_x,
  BI__builtin_sve_svsubr_n_s32_z,
  BI__builtin_sve_svsubr_n_s64_m,
  BI__builtin_sve_svsubr_n_s64_x,
  BI__builtin_sve_svsubr_n_s64_z,
  BI__builtin_sve_svsubr_n_s8_m,
  BI__builtin_sve_svsubr_n_s8_x,
  BI__builtin_sve_svsubr_n_s8_z,
  BI__builtin_sve_svsubr_n_u16_m,
  BI__builtin_sve_svsubr_n_u16_x,
  BI__builtin_sve_svsubr_n_u16_z,
  BI__builtin_sve_svsubr_n_u32_m,
  BI__builtin_sve_svsubr_n_u32_x,
  BI__builtin_sve_svsubr_n_u32_z,
  BI__builtin_sve_svsubr_n_u64_m,
  BI__builtin_sve_svsubr_n_u64_x,
  BI__builtin_sve_svsubr_n_u64_z,
  BI__builtin_sve_svsubr_n_u8_m,
  BI__builtin_sve_svsubr_n_u8_x,
  BI__builtin_sve_svsubr_n_u8_z,
  BI__builtin_sve_svsubr_s16_m,
  BI__builtin_sve_svsubr_s16_x,
  BI__builtin_sve_svsubr_s16_z,
  BI__builtin_sve_svsubr_s32_m,
  BI__builtin_sve_svsubr_s32_x,
  BI__builtin_sve_svsubr_s32_z,
  BI__builtin_sve_svsubr_s64_m,
  BI__builtin_sve_svsubr_s64_x,
  BI__builtin_sve_svsubr_s64_z,
  BI__builtin_sve_svsubr_s8_m,
  BI__builtin_sve_svsubr_s8_x,
  BI__builtin_sve_svsubr_s8_z,
  BI__builtin_sve_svsubr_u16_m,
  BI__builtin_sve_svsubr_u16_x,
  BI__builtin_sve_svsubr_u16_z,
  BI__builtin_sve_svsubr_u32_m,
  BI__builtin_sve_svsubr_u32_x,
  BI__builtin_sve_svsubr_u32_z,
  BI__builtin_sve_svsubr_u64_m,
  BI__builtin_sve_svsubr_u64_x,
  BI__builtin_sve_svsubr_u64_z,
  BI__builtin_sve_svsubr_u8_m,
  BI__builtin_sve_svsubr_u8_x,
  BI__builtin_sve_svsubr_u8_z,
  BI__builtin_sve_svsubwb_n_s16,
  BI__builtin_sve_svsubwb_n_s32,
  BI__builtin_sve_svsubwb_n_s64,
  BI__builtin_sve_svsubwb_n_u16,
  BI__builtin_sve_svsubwb_n_u32,
  BI__builtin_sve_svsubwb_n_u64,
  BI__builtin_sve_svsubwb_s16,
  BI__builtin_sve_svsubwb_s32,
  BI__builtin_sve_svsubwb_s64,
  BI__builtin_sve_svsubwb_u16,
  BI__builtin_sve_svsubwb_u32,
  BI__builtin_sve_svsubwb_u64,
  BI__builtin_sve_svsubwt_n_s16,
  BI__builtin_sve_svsubwt_n_s32,
  BI__builtin_sve_svsubwt_n_s64,
  BI__builtin_sve_svsubwt_n_u16,
  BI__builtin_sve_svsubwt_n_u32,
  BI__builtin_sve_svsubwt_n_u64,
  BI__builtin_sve_svsubwt_s16,
  BI__builtin_sve_svsubwt_s32,
  BI__builtin_sve_svsubwt_s64,
  BI__builtin_sve_svsubwt_u16,
  BI__builtin_sve_svsubwt_u32,
  BI__builtin_sve_svsubwt_u64,
  BI__builtin_sve_svsudot_lane_s32,
  BI__builtin_sve_svsudot_n_s32,
  BI__builtin_sve_svsudot_s32,
  BI__builtin_sve_svtbl2_bf16,
  BI__builtin_sve_svtbl2_f16,
  BI__builtin_sve_svtbl2_f32,
  BI__builtin_sve_svtbl2_f64,
  BI__builtin_sve_svtbl2_s16,
  BI__builtin_sve_svtbl2_s32,
  BI__builtin_sve_svtbl2_s64,
  BI__builtin_sve_svtbl2_s8,
  BI__builtin_sve_svtbl2_u16,
  BI__builtin_sve_svtbl2_u32,
  BI__builtin_sve_svtbl2_u64,
  BI__builtin_sve_svtbl2_u8,
  BI__builtin_sve_svtbl_bf16,
  BI__builtin_sve_svtbl_f16,
  BI__builtin_sve_svtbl_f32,
  BI__builtin_sve_svtbl_f64,
  BI__builtin_sve_svtbl_s16,
  BI__builtin_sve_svtbl_s32,
  BI__builtin_sve_svtbl_s64,
  BI__builtin_sve_svtbl_s8,
  BI__builtin_sve_svtbl_u16,
  BI__builtin_sve_svtbl_u32,
  BI__builtin_sve_svtbl_u64,
  BI__builtin_sve_svtbl_u8,
  BI__builtin_sve_svtblq_bf16,
  BI__builtin_sve_svtblq_f16,
  BI__builtin_sve_svtblq_f32,
  BI__builtin_sve_svtblq_f64,
  BI__builtin_sve_svtblq_mf8,
  BI__builtin_sve_svtblq_s16,
  BI__builtin_sve_svtblq_s32,
  BI__builtin_sve_svtblq_s64,
  BI__builtin_sve_svtblq_s8,
  BI__builtin_sve_svtblq_u16,
  BI__builtin_sve_svtblq_u32,
  BI__builtin_sve_svtblq_u64,
  BI__builtin_sve_svtblq_u8,
  BI__builtin_sve_svtbx_bf16,
  BI__builtin_sve_svtbx_f16,
  BI__builtin_sve_svtbx_f32,
  BI__builtin_sve_svtbx_f64,
  BI__builtin_sve_svtbx_s16,
  BI__builtin_sve_svtbx_s32,
  BI__builtin_sve_svtbx_s64,
  BI__builtin_sve_svtbx_s8,
  BI__builtin_sve_svtbx_u16,
  BI__builtin_sve_svtbx_u32,
  BI__builtin_sve_svtbx_u64,
  BI__builtin_sve_svtbx_u8,
  BI__builtin_sve_svtbxq_bf16,
  BI__builtin_sve_svtbxq_f16,
  BI__builtin_sve_svtbxq_f32,
  BI__builtin_sve_svtbxq_f64,
  BI__builtin_sve_svtbxq_mf8,
  BI__builtin_sve_svtbxq_s16,
  BI__builtin_sve_svtbxq_s32,
  BI__builtin_sve_svtbxq_s64,
  BI__builtin_sve_svtbxq_s8,
  BI__builtin_sve_svtbxq_u16,
  BI__builtin_sve_svtbxq_u32,
  BI__builtin_sve_svtbxq_u64,
  BI__builtin_sve_svtbxq_u8,
  BI__builtin_sve_svtmad_f16,
  BI__builtin_sve_svtmad_f32,
  BI__builtin_sve_svtmad_f64,
  BI__builtin_sve_svtrn1_b16,
  BI__builtin_sve_svtrn1_b32,
  BI__builtin_sve_svtrn1_b64,
  BI__builtin_sve_svtrn1_b8,
  BI__builtin_sve_svtrn1_bf16,
  BI__builtin_sve_svtrn1_f16,
  BI__builtin_sve_svtrn1_f32,
  BI__builtin_sve_svtrn1_f64,
  BI__builtin_sve_svtrn1_s16,
  BI__builtin_sve_svtrn1_s32,
  BI__builtin_sve_svtrn1_s64,
  BI__builtin_sve_svtrn1_s8,
  BI__builtin_sve_svtrn1_u16,
  BI__builtin_sve_svtrn1_u32,
  BI__builtin_sve_svtrn1_u64,
  BI__builtin_sve_svtrn1_u8,
  BI__builtin_sve_svtrn1q_bf16,
  BI__builtin_sve_svtrn1q_f16,
  BI__builtin_sve_svtrn1q_f32,
  BI__builtin_sve_svtrn1q_f64,
  BI__builtin_sve_svtrn1q_s16,
  BI__builtin_sve_svtrn1q_s32,
  BI__builtin_sve_svtrn1q_s64,
  BI__builtin_sve_svtrn1q_s8,
  BI__builtin_sve_svtrn1q_u16,
  BI__builtin_sve_svtrn1q_u32,
  BI__builtin_sve_svtrn1q_u64,
  BI__builtin_sve_svtrn1q_u8,
  BI__builtin_sve_svtrn2_b16,
  BI__builtin_sve_svtrn2_b32,
  BI__builtin_sve_svtrn2_b64,
  BI__builtin_sve_svtrn2_b8,
  BI__builtin_sve_svtrn2_bf16,
  BI__builtin_sve_svtrn2_f16,
  BI__builtin_sve_svtrn2_f32,
  BI__builtin_sve_svtrn2_f64,
  BI__builtin_sve_svtrn2_s16,
  BI__builtin_sve_svtrn2_s32,
  BI__builtin_sve_svtrn2_s64,
  BI__builtin_sve_svtrn2_s8,
  BI__builtin_sve_svtrn2_u16,
  BI__builtin_sve_svtrn2_u32,
  BI__builtin_sve_svtrn2_u64,
  BI__builtin_sve_svtrn2_u8,
  BI__builtin_sve_svtrn2q_bf16,
  BI__builtin_sve_svtrn2q_f16,
  BI__builtin_sve_svtrn2q_f32,
  BI__builtin_sve_svtrn2q_f64,
  BI__builtin_sve_svtrn2q_s16,
  BI__builtin_sve_svtrn2q_s32,
  BI__builtin_sve_svtrn2q_s64,
  BI__builtin_sve_svtrn2q_s8,
  BI__builtin_sve_svtrn2q_u16,
  BI__builtin_sve_svtrn2q_u32,
  BI__builtin_sve_svtrn2q_u64,
  BI__builtin_sve_svtrn2q_u8,
  BI__builtin_sve_svtsmul_f16,
  BI__builtin_sve_svtsmul_f32,
  BI__builtin_sve_svtsmul_f64,
  BI__builtin_sve_svtssel_f16,
  BI__builtin_sve_svtssel_f32,
  BI__builtin_sve_svtssel_f64,
  BI__builtin_sve_svundef2_b,
  BI__builtin_sve_svundef2_bf16,
  BI__builtin_sve_svundef2_f16,
  BI__builtin_sve_svundef2_f32,
  BI__builtin_sve_svundef2_f64,
  BI__builtin_sve_svundef2_mf8,
  BI__builtin_sve_svundef2_s16,
  BI__builtin_sve_svundef2_s32,
  BI__builtin_sve_svundef2_s64,
  BI__builtin_sve_svundef2_s8,
  BI__builtin_sve_svundef2_u16,
  BI__builtin_sve_svundef2_u32,
  BI__builtin_sve_svundef2_u64,
  BI__builtin_sve_svundef2_u8,
  BI__builtin_sve_svundef3_bf16,
  BI__builtin_sve_svundef3_f16,
  BI__builtin_sve_svundef3_f32,
  BI__builtin_sve_svundef3_f64,
  BI__builtin_sve_svundef3_mf8,
  BI__builtin_sve_svundef3_s16,
  BI__builtin_sve_svundef3_s32,
  BI__builtin_sve_svundef3_s64,
  BI__builtin_sve_svundef3_s8,
  BI__builtin_sve_svundef3_u16,
  BI__builtin_sve_svundef3_u32,
  BI__builtin_sve_svundef3_u64,
  BI__builtin_sve_svundef3_u8,
  BI__builtin_sve_svundef4_b,
  BI__builtin_sve_svundef4_bf16,
  BI__builtin_sve_svundef4_f16,
  BI__builtin_sve_svundef4_f32,
  BI__builtin_sve_svundef4_f64,
  BI__builtin_sve_svundef4_mf8,
  BI__builtin_sve_svundef4_s16,
  BI__builtin_sve_svundef4_s32,
  BI__builtin_sve_svundef4_s64,
  BI__builtin_sve_svundef4_s8,
  BI__builtin_sve_svundef4_u16,
  BI__builtin_sve_svundef4_u32,
  BI__builtin_sve_svundef4_u64,
  BI__builtin_sve_svundef4_u8,
  BI__builtin_sve_svundef_bf16,
  BI__builtin_sve_svundef_f16,
  BI__builtin_sve_svundef_f32,
  BI__builtin_sve_svundef_f64,
  BI__builtin_sve_svundef_mf8,
  BI__builtin_sve_svundef_s16,
  BI__builtin_sve_svundef_s32,
  BI__builtin_sve_svundef_s64,
  BI__builtin_sve_svundef_s8,
  BI__builtin_sve_svundef_u16,
  BI__builtin_sve_svundef_u32,
  BI__builtin_sve_svundef_u64,
  BI__builtin_sve_svundef_u8,
  BI__builtin_sve_svunpk_s16_s8_x2,
  BI__builtin_sve_svunpk_s16_s8_x4,
  BI__builtin_sve_svunpk_s32_s16_x2,
  BI__builtin_sve_svunpk_s32_s16_x4,
  BI__builtin_sve_svunpk_s64_s32_x2,
  BI__builtin_sve_svunpk_s64_s32_x4,
  BI__builtin_sve_svunpk_u16_u8_x2,
  BI__builtin_sve_svunpk_u16_u8_x4,
  BI__builtin_sve_svunpk_u32_u16_x2,
  BI__builtin_sve_svunpk_u32_u16_x4,
  BI__builtin_sve_svunpk_u64_u32_x2,
  BI__builtin_sve_svunpk_u64_u32_x4,
  BI__builtin_sve_svunpkhi_b,
  BI__builtin_sve_svunpkhi_s16,
  BI__builtin_sve_svunpkhi_s32,
  BI__builtin_sve_svunpkhi_s64,
  BI__builtin_sve_svunpkhi_u16,
  BI__builtin_sve_svunpkhi_u32,
  BI__builtin_sve_svunpkhi_u64,
  BI__builtin_sve_svunpklo_b,
  BI__builtin_sve_svunpklo_s16,
  BI__builtin_sve_svunpklo_s32,
  BI__builtin_sve_svunpklo_s64,
  BI__builtin_sve_svunpklo_u16,
  BI__builtin_sve_svunpklo_u32,
  BI__builtin_sve_svunpklo_u64,
  BI__builtin_sve_svuqadd_n_s16_m,
  BI__builtin_sve_svuqadd_n_s16_x,
  BI__builtin_sve_svuqadd_n_s16_z,
  BI__builtin_sve_svuqadd_n_s32_m,
  BI__builtin_sve_svuqadd_n_s32_x,
  BI__builtin_sve_svuqadd_n_s32_z,
  BI__builtin_sve_svuqadd_n_s64_m,
  BI__builtin_sve_svuqadd_n_s64_x,
  BI__builtin_sve_svuqadd_n_s64_z,
  BI__builtin_sve_svuqadd_n_s8_m,
  BI__builtin_sve_svuqadd_n_s8_x,
  BI__builtin_sve_svuqadd_n_s8_z,
  BI__builtin_sve_svuqadd_s16_m,
  BI__builtin_sve_svuqadd_s16_x,
  BI__builtin_sve_svuqadd_s16_z,
  BI__builtin_sve_svuqadd_s32_m,
  BI__builtin_sve_svuqadd_s32_x,
  BI__builtin_sve_svuqadd_s32_z,
  BI__builtin_sve_svuqadd_s64_m,
  BI__builtin_sve_svuqadd_s64_x,
  BI__builtin_sve_svuqadd_s64_z,
  BI__builtin_sve_svuqadd_s8_m,
  BI__builtin_sve_svuqadd_s8_x,
  BI__builtin_sve_svuqadd_s8_z,
  BI__builtin_sve_svusdot_lane_s32,
  BI__builtin_sve_svusdot_n_s32,
  BI__builtin_sve_svusdot_s32,
  BI__builtin_sve_svusmmla_s32,
  BI__builtin_sve_svuzp1_b16,
  BI__builtin_sve_svuzp1_b32,
  BI__builtin_sve_svuzp1_b64,
  BI__builtin_sve_svuzp1_b8,
  BI__builtin_sve_svuzp1_bf16,
  BI__builtin_sve_svuzp1_f16,
  BI__builtin_sve_svuzp1_f32,
  BI__builtin_sve_svuzp1_f64,
  BI__builtin_sve_svuzp1_s16,
  BI__builtin_sve_svuzp1_s32,
  BI__builtin_sve_svuzp1_s64,
  BI__builtin_sve_svuzp1_s8,
  BI__builtin_sve_svuzp1_u16,
  BI__builtin_sve_svuzp1_u32,
  BI__builtin_sve_svuzp1_u64,
  BI__builtin_sve_svuzp1_u8,
  BI__builtin_sve_svuzp1q_bf16,
  BI__builtin_sve_svuzp1q_f16,
  BI__builtin_sve_svuzp1q_f32,
  BI__builtin_sve_svuzp1q_f64,
  BI__builtin_sve_svuzp1q_s16,
  BI__builtin_sve_svuzp1q_s32,
  BI__builtin_sve_svuzp1q_s64,
  BI__builtin_sve_svuzp1q_s8,
  BI__builtin_sve_svuzp1q_u16,
  BI__builtin_sve_svuzp1q_u32,
  BI__builtin_sve_svuzp1q_u64,
  BI__builtin_sve_svuzp1q_u8,
  BI__builtin_sve_svuzp2_b16,
  BI__builtin_sve_svuzp2_b32,
  BI__builtin_sve_svuzp2_b64,
  BI__builtin_sve_svuzp2_b8,
  BI__builtin_sve_svuzp2_bf16,
  BI__builtin_sve_svuzp2_f16,
  BI__builtin_sve_svuzp2_f32,
  BI__builtin_sve_svuzp2_f64,
  BI__builtin_sve_svuzp2_s16,
  BI__builtin_sve_svuzp2_s32,
  BI__builtin_sve_svuzp2_s64,
  BI__builtin_sve_svuzp2_s8,
  BI__builtin_sve_svuzp2_u16,
  BI__builtin_sve_svuzp2_u32,
  BI__builtin_sve_svuzp2_u64,
  BI__builtin_sve_svuzp2_u8,
  BI__builtin_sve_svuzp2q_bf16,
  BI__builtin_sve_svuzp2q_f16,
  BI__builtin_sve_svuzp2q_f32,
  BI__builtin_sve_svuzp2q_f64,
  BI__builtin_sve_svuzp2q_s16,
  BI__builtin_sve_svuzp2q_s32,
  BI__builtin_sve_svuzp2q_s64,
  BI__builtin_sve_svuzp2q_s8,
  BI__builtin_sve_svuzp2q_u16,
  BI__builtin_sve_svuzp2q_u32,
  BI__builtin_sve_svuzp2q_u64,
  BI__builtin_sve_svuzp2q_u8,
  BI__builtin_sve_svuzp_bf16_x2,
  BI__builtin_sve_svuzp_bf16_x4,
  BI__builtin_sve_svuzp_f16_x2,
  BI__builtin_sve_svuzp_f16_x4,
  BI__builtin_sve_svuzp_f32_x2,
  BI__builtin_sve_svuzp_f32_x4,
  BI__builtin_sve_svuzp_f64_x2,
  BI__builtin_sve_svuzp_f64_x4,
  BI__builtin_sve_svuzp_mf8_x2,
  BI__builtin_sve_svuzp_mf8_x4,
  BI__builtin_sve_svuzp_s16_x2,
  BI__builtin_sve_svuzp_s16_x4,
  BI__builtin_sve_svuzp_s32_x2,
  BI__builtin_sve_svuzp_s32_x4,
  BI__builtin_sve_svuzp_s64_x2,
  BI__builtin_sve_svuzp_s64_x4,
  BI__builtin_sve_svuzp_s8_x2,
  BI__builtin_sve_svuzp_s8_x4,
  BI__builtin_sve_svuzp_u16_x2,
  BI__builtin_sve_svuzp_u16_x4,
  BI__builtin_sve_svuzp_u32_x2,
  BI__builtin_sve_svuzp_u32_x4,
  BI__builtin_sve_svuzp_u64_x2,
  BI__builtin_sve_svuzp_u64_x4,
  BI__builtin_sve_svuzp_u8_x2,
  BI__builtin_sve_svuzp_u8_x4,
  BI__builtin_sve_svuzpq1_bf16,
  BI__builtin_sve_svuzpq1_f16,
  BI__builtin_sve_svuzpq1_f32,
  BI__builtin_sve_svuzpq1_f64,
  BI__builtin_sve_svuzpq1_mf8,
  BI__builtin_sve_svuzpq1_s16,
  BI__builtin_sve_svuzpq1_s32,
  BI__builtin_sve_svuzpq1_s64,
  BI__builtin_sve_svuzpq1_s8,
  BI__builtin_sve_svuzpq1_u16,
  BI__builtin_sve_svuzpq1_u32,
  BI__builtin_sve_svuzpq1_u64,
  BI__builtin_sve_svuzpq1_u8,
  BI__builtin_sve_svuzpq2_bf16,
  BI__builtin_sve_svuzpq2_f16,
  BI__builtin_sve_svuzpq2_f32,
  BI__builtin_sve_svuzpq2_f64,
  BI__builtin_sve_svuzpq2_mf8,
  BI__builtin_sve_svuzpq2_s16,
  BI__builtin_sve_svuzpq2_s32,
  BI__builtin_sve_svuzpq2_s64,
  BI__builtin_sve_svuzpq2_s8,
  BI__builtin_sve_svuzpq2_u16,
  BI__builtin_sve_svuzpq2_u32,
  BI__builtin_sve_svuzpq2_u64,
  BI__builtin_sve_svuzpq2_u8,
  BI__builtin_sve_svuzpq_bf16_x2,
  BI__builtin_sve_svuzpq_bf16_x4,
  BI__builtin_sve_svuzpq_f16_x2,
  BI__builtin_sve_svuzpq_f16_x4,
  BI__builtin_sve_svuzpq_f32_x2,
  BI__builtin_sve_svuzpq_f32_x4,
  BI__builtin_sve_svuzpq_f64_x2,
  BI__builtin_sve_svuzpq_f64_x4,
  BI__builtin_sve_svuzpq_mf8_x2,
  BI__builtin_sve_svuzpq_mf8_x4,
  BI__builtin_sve_svuzpq_s16_x2,
  BI__builtin_sve_svuzpq_s16_x4,
  BI__builtin_sve_svuzpq_s32_x2,
  BI__builtin_sve_svuzpq_s32_x4,
  BI__builtin_sve_svuzpq_s64_x2,
  BI__builtin_sve_svuzpq_s64_x4,
  BI__builtin_sve_svuzpq_s8_x2,
  BI__builtin_sve_svuzpq_s8_x4,
  BI__builtin_sve_svuzpq_u16_x2,
  BI__builtin_sve_svuzpq_u16_x4,
  BI__builtin_sve_svuzpq_u32_x2,
  BI__builtin_sve_svuzpq_u32_x4,
  BI__builtin_sve_svuzpq_u64_x2,
  BI__builtin_sve_svuzpq_u64_x4,
  BI__builtin_sve_svuzpq_u8_x2,
  BI__builtin_sve_svuzpq_u8_x4,
  BI__builtin_sve_svwhilege_b16_s32,
  BI__builtin_sve_svwhilege_b16_s64,
  BI__builtin_sve_svwhilege_b16_s64_x2,
  BI__builtin_sve_svwhilege_b16_u32,
  BI__builtin_sve_svwhilege_b16_u64,
  BI__builtin_sve_svwhilege_b16_u64_x2,
  BI__builtin_sve_svwhilege_b32_s32,
  BI__builtin_sve_svwhilege_b32_s64,
  BI__builtin_sve_svwhilege_b32_s64_x2,
  BI__builtin_sve_svwhilege_b32_u32,
  BI__builtin_sve_svwhilege_b32_u64,
  BI__builtin_sve_svwhilege_b32_u64_x2,
  BI__builtin_sve_svwhilege_b64_s32,
  BI__builtin_sve_svwhilege_b64_s64,
  BI__builtin_sve_svwhilege_b64_s64_x2,
  BI__builtin_sve_svwhilege_b64_u32,
  BI__builtin_sve_svwhilege_b64_u64,
  BI__builtin_sve_svwhilege_b64_u64_x2,
  BI__builtin_sve_svwhilege_b8_s32,
  BI__builtin_sve_svwhilege_b8_s64,
  BI__builtin_sve_svwhilege_b8_s64_x2,
  BI__builtin_sve_svwhilege_b8_u32,
  BI__builtin_sve_svwhilege_b8_u64,
  BI__builtin_sve_svwhilege_b8_u64_x2,
  BI__builtin_sve_svwhilege_c16_s64,
  BI__builtin_sve_svwhilege_c16_u64,
  BI__builtin_sve_svwhilege_c32_s64,
  BI__builtin_sve_svwhilege_c32_u64,
  BI__builtin_sve_svwhilege_c64_s64,
  BI__builtin_sve_svwhilege_c64_u64,
  BI__builtin_sve_svwhilege_c8_s64,
  BI__builtin_sve_svwhilege_c8_u64,
  BI__builtin_sve_svwhilegt_b16_s32,
  BI__builtin_sve_svwhilegt_b16_s64,
  BI__builtin_sve_svwhilegt_b16_s64_x2,
  BI__builtin_sve_svwhilegt_b16_u32,
  BI__builtin_sve_svwhilegt_b16_u64,
  BI__builtin_sve_svwhilegt_b16_u64_x2,
  BI__builtin_sve_svwhilegt_b32_s32,
  BI__builtin_sve_svwhilegt_b32_s64,
  BI__builtin_sve_svwhilegt_b32_s64_x2,
  BI__builtin_sve_svwhilegt_b32_u32,
  BI__builtin_sve_svwhilegt_b32_u64,
  BI__builtin_sve_svwhilegt_b32_u64_x2,
  BI__builtin_sve_svwhilegt_b64_s32,
  BI__builtin_sve_svwhilegt_b64_s64,
  BI__builtin_sve_svwhilegt_b64_s64_x2,
  BI__builtin_sve_svwhilegt_b64_u32,
  BI__builtin_sve_svwhilegt_b64_u64,
  BI__builtin_sve_svwhilegt_b64_u64_x2,
  BI__builtin_sve_svwhilegt_b8_s32,
  BI__builtin_sve_svwhilegt_b8_s64,
  BI__builtin_sve_svwhilegt_b8_s64_x2,
  BI__builtin_sve_svwhilegt_b8_u32,
  BI__builtin_sve_svwhilegt_b8_u64,
  BI__builtin_sve_svwhilegt_b8_u64_x2,
  BI__builtin_sve_svwhilegt_c16_s64,
  BI__builtin_sve_svwhilegt_c16_u64,
  BI__builtin_sve_svwhilegt_c32_s64,
  BI__builtin_sve_svwhilegt_c32_u64,
  BI__builtin_sve_svwhilegt_c64_s64,
  BI__builtin_sve_svwhilegt_c64_u64,
  BI__builtin_sve_svwhilegt_c8_s64,
  BI__builtin_sve_svwhilegt_c8_u64,
  BI__builtin_sve_svwhilele_b16_s32,
  BI__builtin_sve_svwhilele_b16_s64,
  BI__builtin_sve_svwhilele_b16_s64_x2,
  BI__builtin_sve_svwhilele_b16_u32,
  BI__builtin_sve_svwhilele_b16_u64,
  BI__builtin_sve_svwhilele_b16_u64_x2,
  BI__builtin_sve_svwhilele_b32_s32,
  BI__builtin_sve_svwhilele_b32_s64,
  BI__builtin_sve_svwhilele_b32_s64_x2,
  BI__builtin_sve_svwhilele_b32_u32,
  BI__builtin_sve_svwhilele_b32_u64,
  BI__builtin_sve_svwhilele_b32_u64_x2,
  BI__builtin_sve_svwhilele_b64_s32,
  BI__builtin_sve_svwhilele_b64_s64,
  BI__builtin_sve_svwhilele_b64_s64_x2,
  BI__builtin_sve_svwhilele_b64_u32,
  BI__builtin_sve_svwhilele_b64_u64,
  BI__builtin_sve_svwhilele_b64_u64_x2,
  BI__builtin_sve_svwhilele_b8_s32,
  BI__builtin_sve_svwhilele_b8_s64,
  BI__builtin_sve_svwhilele_b8_s64_x2,
  BI__builtin_sve_svwhilele_b8_u32,
  BI__builtin_sve_svwhilele_b8_u64,
  BI__builtin_sve_svwhilele_b8_u64_x2,
  BI__builtin_sve_svwhilele_c16_s64,
  BI__builtin_sve_svwhilele_c16_u64,
  BI__builtin_sve_svwhilele_c32_s64,
  BI__builtin_sve_svwhilele_c32_u64,
  BI__builtin_sve_svwhilele_c64_s64,
  BI__builtin_sve_svwhilele_c64_u64,
  BI__builtin_sve_svwhilele_c8_s64,
  BI__builtin_sve_svwhilele_c8_u64,
  BI__builtin_sve_svwhilelt_b16_s32,
  BI__builtin_sve_svwhilelt_b16_s64,
  BI__builtin_sve_svwhilelt_b16_s64_x2,
  BI__builtin_sve_svwhilelt_b16_u32,
  BI__builtin_sve_svwhilelt_b16_u64,
  BI__builtin_sve_svwhilelt_b16_u64_x2,
  BI__builtin_sve_svwhilelt_b32_s32,
  BI__builtin_sve_svwhilelt_b32_s64,
  BI__builtin_sve_svwhilelt_b32_s64_x2,
  BI__builtin_sve_svwhilelt_b32_u32,
  BI__builtin_sve_svwhilelt_b32_u64,
  BI__builtin_sve_svwhilelt_b32_u64_x2,
  BI__builtin_sve_svwhilelt_b64_s32,
  BI__builtin_sve_svwhilelt_b64_s64,
  BI__builtin_sve_svwhilelt_b64_s64_x2,
  BI__builtin_sve_svwhilelt_b64_u32,
  BI__builtin_sve_svwhilelt_b64_u64,
  BI__builtin_sve_svwhilelt_b64_u64_x2,
  BI__builtin_sve_svwhilelt_b8_s32,
  BI__builtin_sve_svwhilelt_b8_s64,
  BI__builtin_sve_svwhilelt_b8_s64_x2,
  BI__builtin_sve_svwhilelt_b8_u32,
  BI__builtin_sve_svwhilelt_b8_u64,
  BI__builtin_sve_svwhilelt_b8_u64_x2,
  BI__builtin_sve_svwhilelt_c16_s64,
  BI__builtin_sve_svwhilelt_c16_u64,
  BI__builtin_sve_svwhilelt_c32_s64,
  BI__builtin_sve_svwhilelt_c32_u64,
  BI__builtin_sve_svwhilelt_c64_s64,
  BI__builtin_sve_svwhilelt_c64_u64,
  BI__builtin_sve_svwhilelt_c8_s64,
  BI__builtin_sve_svwhilelt_c8_u64,
  BI__builtin_sve_svwhilerw_bf16,
  BI__builtin_sve_svwhilerw_f16,
  BI__builtin_sve_svwhilerw_f32,
  BI__builtin_sve_svwhilerw_f64,
  BI__builtin_sve_svwhilerw_s16,
  BI__builtin_sve_svwhilerw_s32,
  BI__builtin_sve_svwhilerw_s64,
  BI__builtin_sve_svwhilerw_s8,
  BI__builtin_sve_svwhilerw_u16,
  BI__builtin_sve_svwhilerw_u32,
  BI__builtin_sve_svwhilerw_u64,
  BI__builtin_sve_svwhilerw_u8,
  BI__builtin_sve_svwhilewr_bf16,
  BI__builtin_sve_svwhilewr_f16,
  BI__builtin_sve_svwhilewr_f32,
  BI__builtin_sve_svwhilewr_f64,
  BI__builtin_sve_svwhilewr_s16,
  BI__builtin_sve_svwhilewr_s32,
  BI__builtin_sve_svwhilewr_s64,
  BI__builtin_sve_svwhilewr_s8,
  BI__builtin_sve_svwhilewr_u16,
  BI__builtin_sve_svwhilewr_u32,
  BI__builtin_sve_svwhilewr_u64,
  BI__builtin_sve_svwhilewr_u8,
  BI__builtin_sve_svwrffr,
  BI__builtin_sve_svxar_n_s16,
  BI__builtin_sve_svxar_n_s32,
  BI__builtin_sve_svxar_n_s64,
  BI__builtin_sve_svxar_n_s8,
  BI__builtin_sve_svxar_n_u16,
  BI__builtin_sve_svxar_n_u32,
  BI__builtin_sve_svxar_n_u64,
  BI__builtin_sve_svxar_n_u8,
  BI__builtin_sve_svzip1_b16,
  BI__builtin_sve_svzip1_b32,
  BI__builtin_sve_svzip1_b64,
  BI__builtin_sve_svzip1_b8,
  BI__builtin_sve_svzip1_bf16,
  BI__builtin_sve_svzip1_f16,
  BI__builtin_sve_svzip1_f32,
  BI__builtin_sve_svzip1_f64,
  BI__builtin_sve_svzip1_s16,
  BI__builtin_sve_svzip1_s32,
  BI__builtin_sve_svzip1_s64,
  BI__builtin_sve_svzip1_s8,
  BI__builtin_sve_svzip1_u16,
  BI__builtin_sve_svzip1_u32,
  BI__builtin_sve_svzip1_u64,
  BI__builtin_sve_svzip1_u8,
  BI__builtin_sve_svzip1q_bf16,
  BI__builtin_sve_svzip1q_f16,
  BI__builtin_sve_svzip1q_f32,
  BI__builtin_sve_svzip1q_f64,
  BI__builtin_sve_svzip1q_s16,
  BI__builtin_sve_svzip1q_s32,
  BI__builtin_sve_svzip1q_s64,
  BI__builtin_sve_svzip1q_s8,
  BI__builtin_sve_svzip1q_u16,
  BI__builtin_sve_svzip1q_u32,
  BI__builtin_sve_svzip1q_u64,
  BI__builtin_sve_svzip1q_u8,
  BI__builtin_sve_svzip2_b16,
  BI__builtin_sve_svzip2_b32,
  BI__builtin_sve_svzip2_b64,
  BI__builtin_sve_svzip2_b8,
  BI__builtin_sve_svzip2_bf16,
  BI__builtin_sve_svzip2_f16,
  BI__builtin_sve_svzip2_f32,
  BI__builtin_sve_svzip2_f64,
  BI__builtin_sve_svzip2_s16,
  BI__builtin_sve_svzip2_s32,
  BI__builtin_sve_svzip2_s64,
  BI__builtin_sve_svzip2_s8,
  BI__builtin_sve_svzip2_u16,
  BI__builtin_sve_svzip2_u32,
  BI__builtin_sve_svzip2_u64,
  BI__builtin_sve_svzip2_u8,
  BI__builtin_sve_svzip2q_bf16,
  BI__builtin_sve_svzip2q_f16,
  BI__builtin_sve_svzip2q_f32,
  BI__builtin_sve_svzip2q_f64,
  BI__builtin_sve_svzip2q_s16,
  BI__builtin_sve_svzip2q_s32,
  BI__builtin_sve_svzip2q_s64,
  BI__builtin_sve_svzip2q_s8,
  BI__builtin_sve_svzip2q_u16,
  BI__builtin_sve_svzip2q_u32,
  BI__builtin_sve_svzip2q_u64,
  BI__builtin_sve_svzip2q_u8,
  BI__builtin_sve_svzip_bf16_x2,
  BI__builtin_sve_svzip_bf16_x4,
  BI__builtin_sve_svzip_f16_x2,
  BI__builtin_sve_svzip_f16_x4,
  BI__builtin_sve_svzip_f32_x2,
  BI__builtin_sve_svzip_f32_x4,
  BI__builtin_sve_svzip_f64_x2,
  BI__builtin_sve_svzip_f64_x4,
  BI__builtin_sve_svzip_mf8_x2,
  BI__builtin_sve_svzip_mf8_x4,
  BI__builtin_sve_svzip_s16_x2,
  BI__builtin_sve_svzip_s16_x4,
  BI__builtin_sve_svzip_s32_x2,
  BI__builtin_sve_svzip_s32_x4,
  BI__builtin_sve_svzip_s64_x2,
  BI__builtin_sve_svzip_s64_x4,
  BI__builtin_sve_svzip_s8_x2,
  BI__builtin_sve_svzip_s8_x4,
  BI__builtin_sve_svzip_u16_x2,
  BI__builtin_sve_svzip_u16_x4,
  BI__builtin_sve_svzip_u32_x2,
  BI__builtin_sve_svzip_u32_x4,
  BI__builtin_sve_svzip_u64_x2,
  BI__builtin_sve_svzip_u64_x4,
  BI__builtin_sve_svzip_u8_x2,
  BI__builtin_sve_svzip_u8_x4,
  BI__builtin_sve_svzipq1_bf16,
  BI__builtin_sve_svzipq1_f16,
  BI__builtin_sve_svzipq1_f32,
  BI__builtin_sve_svzipq1_f64,
  BI__builtin_sve_svzipq1_mf8,
  BI__builtin_sve_svzipq1_s16,
  BI__builtin_sve_svzipq1_s32,
  BI__builtin_sve_svzipq1_s64,
  BI__builtin_sve_svzipq1_s8,
  BI__builtin_sve_svzipq1_u16,
  BI__builtin_sve_svzipq1_u32,
  BI__builtin_sve_svzipq1_u64,
  BI__builtin_sve_svzipq1_u8,
  BI__builtin_sve_svzipq2_bf16,
  BI__builtin_sve_svzipq2_f16,
  BI__builtin_sve_svzipq2_f32,
  BI__builtin_sve_svzipq2_f64,
  BI__builtin_sve_svzipq2_mf8,
  BI__builtin_sve_svzipq2_s16,
  BI__builtin_sve_svzipq2_s32,
  BI__builtin_sve_svzipq2_s64,
  BI__builtin_sve_svzipq2_s8,
  BI__builtin_sve_svzipq2_u16,
  BI__builtin_sve_svzipq2_u32,
  BI__builtin_sve_svzipq2_u64,
  BI__builtin_sve_svzipq2_u8,
  BI__builtin_sve_svzipq_bf16_x2,
  BI__builtin_sve_svzipq_bf16_x4,
  BI__builtin_sve_svzipq_f16_x2,
  BI__builtin_sve_svzipq_f16_x4,
  BI__builtin_sve_svzipq_f32_x2,
  BI__builtin_sve_svzipq_f32_x4,
  BI__builtin_sve_svzipq_f64_x2,
  BI__builtin_sve_svzipq_f64_x4,
  BI__builtin_sve_svzipq_mf8_x2,
  BI__builtin_sve_svzipq_mf8_x4,
  BI__builtin_sve_svzipq_s16_x2,
  BI__builtin_sve_svzipq_s16_x4,
  BI__builtin_sve_svzipq_s32_x2,
  BI__builtin_sve_svzipq_s32_x4,
  BI__builtin_sve_svzipq_s64_x2,
  BI__builtin_sve_svzipq_s64_x4,
  BI__builtin_sve_svzipq_s8_x2,
  BI__builtin_sve_svzipq_s8_x4,
  BI__builtin_sve_svzipq_u16_x2,
  BI__builtin_sve_svzipq_u16_x4,
  BI__builtin_sve_svzipq_u32_x2,
  BI__builtin_sve_svzipq_u32_x4,
  BI__builtin_sve_svzipq_u64_x2,
  BI__builtin_sve_svzipq_u64_x4,
  BI__builtin_sve_svzipq_u8_x2,
  BI__builtin_sve_svzipq_u8_x4,
  BI__builtin_sve_reinterpret_s8_s8,
  BI__builtin_sve_reinterpret_s8_u8,
  BI__builtin_sve_reinterpret_s8_mf8,
  BI__builtin_sve_reinterpret_s8_s16,
  BI__builtin_sve_reinterpret_s8_u16,
  BI__builtin_sve_reinterpret_s8_s32,
  BI__builtin_sve_reinterpret_s8_u32,
  BI__builtin_sve_reinterpret_s8_s64,
  BI__builtin_sve_reinterpret_s8_u64,
  BI__builtin_sve_reinterpret_s8_f16,
  BI__builtin_sve_reinterpret_s8_bf16,
  BI__builtin_sve_reinterpret_s8_f32,
  BI__builtin_sve_reinterpret_s8_f64,
  BI__builtin_sve_reinterpret_u8_s8,
  BI__builtin_sve_reinterpret_u8_u8,
  BI__builtin_sve_reinterpret_u8_mf8,
  BI__builtin_sve_reinterpret_u8_s16,
  BI__builtin_sve_reinterpret_u8_u16,
  BI__builtin_sve_reinterpret_u8_s32,
  BI__builtin_sve_reinterpret_u8_u32,
  BI__builtin_sve_reinterpret_u8_s64,
  BI__builtin_sve_reinterpret_u8_u64,
  BI__builtin_sve_reinterpret_u8_f16,
  BI__builtin_sve_reinterpret_u8_bf16,
  BI__builtin_sve_reinterpret_u8_f32,
  BI__builtin_sve_reinterpret_u8_f64,
  BI__builtin_sve_reinterpret_mf8_s8,
  BI__builtin_sve_reinterpret_mf8_u8,
  BI__builtin_sve_reinterpret_mf8_mf8,
  BI__builtin_sve_reinterpret_mf8_s16,
  BI__builtin_sve_reinterpret_mf8_u16,
  BI__builtin_sve_reinterpret_mf8_s32,
  BI__builtin_sve_reinterpret_mf8_u32,
  BI__builtin_sve_reinterpret_mf8_s64,
  BI__builtin_sve_reinterpret_mf8_u64,
  BI__builtin_sve_reinterpret_mf8_f16,
  BI__builtin_sve_reinterpret_mf8_bf16,
  BI__builtin_sve_reinterpret_mf8_f32,
  BI__builtin_sve_reinterpret_mf8_f64,
  BI__builtin_sve_reinterpret_s16_s8,
  BI__builtin_sve_reinterpret_s16_u8,
  BI__builtin_sve_reinterpret_s16_mf8,
  BI__builtin_sve_reinterpret_s16_s16,
  BI__builtin_sve_reinterpret_s16_u16,
  BI__builtin_sve_reinterpret_s16_s32,
  BI__builtin_sve_reinterpret_s16_u32,
  BI__builtin_sve_reinterpret_s16_s64,
  BI__builtin_sve_reinterpret_s16_u64,
  BI__builtin_sve_reinterpret_s16_f16,
  BI__builtin_sve_reinterpret_s16_bf16,
  BI__builtin_sve_reinterpret_s16_f32,
  BI__builtin_sve_reinterpret_s16_f64,
  BI__builtin_sve_reinterpret_u16_s8,
  BI__builtin_sve_reinterpret_u16_u8,
  BI__builtin_sve_reinterpret_u16_mf8,
  BI__builtin_sve_reinterpret_u16_s16,
  BI__builtin_sve_reinterpret_u16_u16,
  BI__builtin_sve_reinterpret_u16_s32,
  BI__builtin_sve_reinterpret_u16_u32,
  BI__builtin_sve_reinterpret_u16_s64,
  BI__builtin_sve_reinterpret_u16_u64,
  BI__builtin_sve_reinterpret_u16_f16,
  BI__builtin_sve_reinterpret_u16_bf16,
  BI__builtin_sve_reinterpret_u16_f32,
  BI__builtin_sve_reinterpret_u16_f64,
  BI__builtin_sve_reinterpret_s32_s8,
  BI__builtin_sve_reinterpret_s32_u8,
  BI__builtin_sve_reinterpret_s32_mf8,
  BI__builtin_sve_reinterpret_s32_s16,
  BI__builtin_sve_reinterpret_s32_u16,
  BI__builtin_sve_reinterpret_s32_s32,
  BI__builtin_sve_reinterpret_s32_u32,
  BI__builtin_sve_reinterpret_s32_s64,
  BI__builtin_sve_reinterpret_s32_u64,
  BI__builtin_sve_reinterpret_s32_f16,
  BI__builtin_sve_reinterpret_s32_bf16,
  BI__builtin_sve_reinterpret_s32_f32,
  BI__builtin_sve_reinterpret_s32_f64,
  BI__builtin_sve_reinterpret_u32_s8,
  BI__builtin_sve_reinterpret_u32_u8,
  BI__builtin_sve_reinterpret_u32_mf8,
  BI__builtin_sve_reinterpret_u32_s16,
  BI__builtin_sve_reinterpret_u32_u16,
  BI__builtin_sve_reinterpret_u32_s32,
  BI__builtin_sve_reinterpret_u32_u32,
  BI__builtin_sve_reinterpret_u32_s64,
  BI__builtin_sve_reinterpret_u32_u64,
  BI__builtin_sve_reinterpret_u32_f16,
  BI__builtin_sve_reinterpret_u32_bf16,
  BI__builtin_sve_reinterpret_u32_f32,
  BI__builtin_sve_reinterpret_u32_f64,
  BI__builtin_sve_reinterpret_s64_s8,
  BI__builtin_sve_reinterpret_s64_u8,
  BI__builtin_sve_reinterpret_s64_mf8,
  BI__builtin_sve_reinterpret_s64_s16,
  BI__builtin_sve_reinterpret_s64_u16,
  BI__builtin_sve_reinterpret_s64_s32,
  BI__builtin_sve_reinterpret_s64_u32,
  BI__builtin_sve_reinterpret_s64_s64,
  BI__builtin_sve_reinterpret_s64_u64,
  BI__builtin_sve_reinterpret_s64_f16,
  BI__builtin_sve_reinterpret_s64_bf16,
  BI__builtin_sve_reinterpret_s64_f32,
  BI__builtin_sve_reinterpret_s64_f64,
  BI__builtin_sve_reinterpret_u64_s8,
  BI__builtin_sve_reinterpret_u64_u8,
  BI__builtin_sve_reinterpret_u64_mf8,
  BI__builtin_sve_reinterpret_u64_s16,
  BI__builtin_sve_reinterpret_u64_u16,
  BI__builtin_sve_reinterpret_u64_s32,
  BI__builtin_sve_reinterpret_u64_u32,
  BI__builtin_sve_reinterpret_u64_s64,
  BI__builtin_sve_reinterpret_u64_u64,
  BI__builtin_sve_reinterpret_u64_f16,
  BI__builtin_sve_reinterpret_u64_bf16,
  BI__builtin_sve_reinterpret_u64_f32,
  BI__builtin_sve_reinterpret_u64_f64,
  BI__builtin_sve_reinterpret_f16_s8,
  BI__builtin_sve_reinterpret_f16_u8,
  BI__builtin_sve_reinterpret_f16_mf8,
  BI__builtin_sve_reinterpret_f16_s16,
  BI__builtin_sve_reinterpret_f16_u16,
  BI__builtin_sve_reinterpret_f16_s32,
  BI__builtin_sve_reinterpret_f16_u32,
  BI__builtin_sve_reinterpret_f16_s64,
  BI__builtin_sve_reinterpret_f16_u64,
  BI__builtin_sve_reinterpret_f16_f16,
  BI__builtin_sve_reinterpret_f16_bf16,
  BI__builtin_sve_reinterpret_f16_f32,
  BI__builtin_sve_reinterpret_f16_f64,
  BI__builtin_sve_reinterpret_bf16_s8,
  BI__builtin_sve_reinterpret_bf16_u8,
  BI__builtin_sve_reinterpret_bf16_mf8,
  BI__builtin_sve_reinterpret_bf16_s16,
  BI__builtin_sve_reinterpret_bf16_u16,
  BI__builtin_sve_reinterpret_bf16_s32,
  BI__builtin_sve_reinterpret_bf16_u32,
  BI__builtin_sve_reinterpret_bf16_s64,
  BI__builtin_sve_reinterpret_bf16_u64,
  BI__builtin_sve_reinterpret_bf16_f16,
  BI__builtin_sve_reinterpret_bf16_bf16,
  BI__builtin_sve_reinterpret_bf16_f32,
  BI__builtin_sve_reinterpret_bf16_f64,
  BI__builtin_sve_reinterpret_f32_s8,
  BI__builtin_sve_reinterpret_f32_u8,
  BI__builtin_sve_reinterpret_f32_mf8,
  BI__builtin_sve_reinterpret_f32_s16,
  BI__builtin_sve_reinterpret_f32_u16,
  BI__builtin_sve_reinterpret_f32_s32,
  BI__builtin_sve_reinterpret_f32_u32,
  BI__builtin_sve_reinterpret_f32_s64,
  BI__builtin_sve_reinterpret_f32_u64,
  BI__builtin_sve_reinterpret_f32_f16,
  BI__builtin_sve_reinterpret_f32_bf16,
  BI__builtin_sve_reinterpret_f32_f32,
  BI__builtin_sve_reinterpret_f32_f64,
  BI__builtin_sve_reinterpret_f64_s8,
  BI__builtin_sve_reinterpret_f64_u8,
  BI__builtin_sve_reinterpret_f64_mf8,
  BI__builtin_sve_reinterpret_f64_s16,
  BI__builtin_sve_reinterpret_f64_u16,
  BI__builtin_sve_reinterpret_f64_s32,
  BI__builtin_sve_reinterpret_f64_u32,
  BI__builtin_sve_reinterpret_f64_s64,
  BI__builtin_sve_reinterpret_f64_u64,
  BI__builtin_sve_reinterpret_f64_f16,
  BI__builtin_sve_reinterpret_f64_bf16,
  BI__builtin_sve_reinterpret_f64_f32,
  BI__builtin_sve_reinterpret_f64_f64,
  BI__builtin_sve_reinterpret_s8_s8_x2,
  BI__builtin_sve_reinterpret_s8_u8_x2,
  BI__builtin_sve_reinterpret_s8_mf8_x2,
  BI__builtin_sve_reinterpret_s8_s16_x2,
  BI__builtin_sve_reinterpret_s8_u16_x2,
  BI__builtin_sve_reinterpret_s8_s32_x2,
  BI__builtin_sve_reinterpret_s8_u32_x2,
  BI__builtin_sve_reinterpret_s8_s64_x2,
  BI__builtin_sve_reinterpret_s8_u64_x2,
  BI__builtin_sve_reinterpret_s8_f16_x2,
  BI__builtin_sve_reinterpret_s8_bf16_x2,
  BI__builtin_sve_reinterpret_s8_f32_x2,
  BI__builtin_sve_reinterpret_s8_f64_x2,
  BI__builtin_sve_reinterpret_u8_s8_x2,
  BI__builtin_sve_reinterpret_u8_u8_x2,
  BI__builtin_sve_reinterpret_u8_mf8_x2,
  BI__builtin_sve_reinterpret_u8_s16_x2,
  BI__builtin_sve_reinterpret_u8_u16_x2,
  BI__builtin_sve_reinterpret_u8_s32_x2,
  BI__builtin_sve_reinterpret_u8_u32_x2,
  BI__builtin_sve_reinterpret_u8_s64_x2,
  BI__builtin_sve_reinterpret_u8_u64_x2,
  BI__builtin_sve_reinterpret_u8_f16_x2,
  BI__builtin_sve_reinterpret_u8_bf16_x2,
  BI__builtin_sve_reinterpret_u8_f32_x2,
  BI__builtin_sve_reinterpret_u8_f64_x2,
  BI__builtin_sve_reinterpret_mf8_s8_x2,
  BI__builtin_sve_reinterpret_mf8_u8_x2,
  BI__builtin_sve_reinterpret_mf8_mf8_x2,
  BI__builtin_sve_reinterpret_mf8_s16_x2,
  BI__builtin_sve_reinterpret_mf8_u16_x2,
  BI__builtin_sve_reinterpret_mf8_s32_x2,
  BI__builtin_sve_reinterpret_mf8_u32_x2,
  BI__builtin_sve_reinterpret_mf8_s64_x2,
  BI__builtin_sve_reinterpret_mf8_u64_x2,
  BI__builtin_sve_reinterpret_mf8_f16_x2,
  BI__builtin_sve_reinterpret_mf8_bf16_x2,
  BI__builtin_sve_reinterpret_mf8_f32_x2,
  BI__builtin_sve_reinterpret_mf8_f64_x2,
  BI__builtin_sve_reinterpret_s16_s8_x2,
  BI__builtin_sve_reinterpret_s16_u8_x2,
  BI__builtin_sve_reinterpret_s16_mf8_x2,
  BI__builtin_sve_reinterpret_s16_s16_x2,
  BI__builtin_sve_reinterpret_s16_u16_x2,
  BI__builtin_sve_reinterpret_s16_s32_x2,
  BI__builtin_sve_reinterpret_s16_u32_x2,
  BI__builtin_sve_reinterpret_s16_s64_x2,
  BI__builtin_sve_reinterpret_s16_u64_x2,
  BI__builtin_sve_reinterpret_s16_f16_x2,
  BI__builtin_sve_reinterpret_s16_bf16_x2,
  BI__builtin_sve_reinterpret_s16_f32_x2,
  BI__builtin_sve_reinterpret_s16_f64_x2,
  BI__builtin_sve_reinterpret_u16_s8_x2,
  BI__builtin_sve_reinterpret_u16_u8_x2,
  BI__builtin_sve_reinterpret_u16_mf8_x2,
  BI__builtin_sve_reinterpret_u16_s16_x2,
  BI__builtin_sve_reinterpret_u16_u16_x2,
  BI__builtin_sve_reinterpret_u16_s32_x2,
  BI__builtin_sve_reinterpret_u16_u32_x2,
  BI__builtin_sve_reinterpret_u16_s64_x2,
  BI__builtin_sve_reinterpret_u16_u64_x2,
  BI__builtin_sve_reinterpret_u16_f16_x2,
  BI__builtin_sve_reinterpret_u16_bf16_x2,
  BI__builtin_sve_reinterpret_u16_f32_x2,
  BI__builtin_sve_reinterpret_u16_f64_x2,
  BI__builtin_sve_reinterpret_s32_s8_x2,
  BI__builtin_sve_reinterpret_s32_u8_x2,
  BI__builtin_sve_reinterpret_s32_mf8_x2,
  BI__builtin_sve_reinterpret_s32_s16_x2,
  BI__builtin_sve_reinterpret_s32_u16_x2,
  BI__builtin_sve_reinterpret_s32_s32_x2,
  BI__builtin_sve_reinterpret_s32_u32_x2,
  BI__builtin_sve_reinterpret_s32_s64_x2,
  BI__builtin_sve_reinterpret_s32_u64_x2,
  BI__builtin_sve_reinterpret_s32_f16_x2,
  BI__builtin_sve_reinterpret_s32_bf16_x2,
  BI__builtin_sve_reinterpret_s32_f32_x2,
  BI__builtin_sve_reinterpret_s32_f64_x2,
  BI__builtin_sve_reinterpret_u32_s8_x2,
  BI__builtin_sve_reinterpret_u32_u8_x2,
  BI__builtin_sve_reinterpret_u32_mf8_x2,
  BI__builtin_sve_reinterpret_u32_s16_x2,
  BI__builtin_sve_reinterpret_u32_u16_x2,
  BI__builtin_sve_reinterpret_u32_s32_x2,
  BI__builtin_sve_reinterpret_u32_u32_x2,
  BI__builtin_sve_reinterpret_u32_s64_x2,
  BI__builtin_sve_reinterpret_u32_u64_x2,
  BI__builtin_sve_reinterpret_u32_f16_x2,
  BI__builtin_sve_reinterpret_u32_bf16_x2,
  BI__builtin_sve_reinterpret_u32_f32_x2,
  BI__builtin_sve_reinterpret_u32_f64_x2,
  BI__builtin_sve_reinterpret_s64_s8_x2,
  BI__builtin_sve_reinterpret_s64_u8_x2,
  BI__builtin_sve_reinterpret_s64_mf8_x2,
  BI__builtin_sve_reinterpret_s64_s16_x2,
  BI__builtin_sve_reinterpret_s64_u16_x2,
  BI__builtin_sve_reinterpret_s64_s32_x2,
  BI__builtin_sve_reinterpret_s64_u32_x2,
  BI__builtin_sve_reinterpret_s64_s64_x2,
  BI__builtin_sve_reinterpret_s64_u64_x2,
  BI__builtin_sve_reinterpret_s64_f16_x2,
  BI__builtin_sve_reinterpret_s64_bf16_x2,
  BI__builtin_sve_reinterpret_s64_f32_x2,
  BI__builtin_sve_reinterpret_s64_f64_x2,
  BI__builtin_sve_reinterpret_u64_s8_x2,
  BI__builtin_sve_reinterpret_u64_u8_x2,
  BI__builtin_sve_reinterpret_u64_mf8_x2,
  BI__builtin_sve_reinterpret_u64_s16_x2,
  BI__builtin_sve_reinterpret_u64_u16_x2,
  BI__builtin_sve_reinterpret_u64_s32_x2,
  BI__builtin_sve_reinterpret_u64_u32_x2,
  BI__builtin_sve_reinterpret_u64_s64_x2,
  BI__builtin_sve_reinterpret_u64_u64_x2,
  BI__builtin_sve_reinterpret_u64_f16_x2,
  BI__builtin_sve_reinterpret_u64_bf16_x2,
  BI__builtin_sve_reinterpret_u64_f32_x2,
  BI__builtin_sve_reinterpret_u64_f64_x2,
  BI__builtin_sve_reinterpret_f16_s8_x2,
  BI__builtin_sve_reinterpret_f16_u8_x2,
  BI__builtin_sve_reinterpret_f16_mf8_x2,
  BI__builtin_sve_reinterpret_f16_s16_x2,
  BI__builtin_sve_reinterpret_f16_u16_x2,
  BI__builtin_sve_reinterpret_f16_s32_x2,
  BI__builtin_sve_reinterpret_f16_u32_x2,
  BI__builtin_sve_reinterpret_f16_s64_x2,
  BI__builtin_sve_reinterpret_f16_u64_x2,
  BI__builtin_sve_reinterpret_f16_f16_x2,
  BI__builtin_sve_reinterpret_f16_bf16_x2,
  BI__builtin_sve_reinterpret_f16_f32_x2,
  BI__builtin_sve_reinterpret_f16_f64_x2,
  BI__builtin_sve_reinterpret_bf16_s8_x2,
  BI__builtin_sve_reinterpret_bf16_u8_x2,
  BI__builtin_sve_reinterpret_bf16_mf8_x2,
  BI__builtin_sve_reinterpret_bf16_s16_x2,
  BI__builtin_sve_reinterpret_bf16_u16_x2,
  BI__builtin_sve_reinterpret_bf16_s32_x2,
  BI__builtin_sve_reinterpret_bf16_u32_x2,
  BI__builtin_sve_reinterpret_bf16_s64_x2,
  BI__builtin_sve_reinterpret_bf16_u64_x2,
  BI__builtin_sve_reinterpret_bf16_f16_x2,
  BI__builtin_sve_reinterpret_bf16_bf16_x2,
  BI__builtin_sve_reinterpret_bf16_f32_x2,
  BI__builtin_sve_reinterpret_bf16_f64_x2,
  BI__builtin_sve_reinterpret_f32_s8_x2,
  BI__builtin_sve_reinterpret_f32_u8_x2,
  BI__builtin_sve_reinterpret_f32_mf8_x2,
  BI__builtin_sve_reinterpret_f32_s16_x2,
  BI__builtin_sve_reinterpret_f32_u16_x2,
  BI__builtin_sve_reinterpret_f32_s32_x2,
  BI__builtin_sve_reinterpret_f32_u32_x2,
  BI__builtin_sve_reinterpret_f32_s64_x2,
  BI__builtin_sve_reinterpret_f32_u64_x2,
  BI__builtin_sve_reinterpret_f32_f16_x2,
  BI__builtin_sve_reinterpret_f32_bf16_x2,
  BI__builtin_sve_reinterpret_f32_f32_x2,
  BI__builtin_sve_reinterpret_f32_f64_x2,
  BI__builtin_sve_reinterpret_f64_s8_x2,
  BI__builtin_sve_reinterpret_f64_u8_x2,
  BI__builtin_sve_reinterpret_f64_mf8_x2,
  BI__builtin_sve_reinterpret_f64_s16_x2,
  BI__builtin_sve_reinterpret_f64_u16_x2,
  BI__builtin_sve_reinterpret_f64_s32_x2,
  BI__builtin_sve_reinterpret_f64_u32_x2,
  BI__builtin_sve_reinterpret_f64_s64_x2,
  BI__builtin_sve_reinterpret_f64_u64_x2,
  BI__builtin_sve_reinterpret_f64_f16_x2,
  BI__builtin_sve_reinterpret_f64_bf16_x2,
  BI__builtin_sve_reinterpret_f64_f32_x2,
  BI__builtin_sve_reinterpret_f64_f64_x2,
  BI__builtin_sve_reinterpret_s8_s8_x3,
  BI__builtin_sve_reinterpret_s8_u8_x3,
  BI__builtin_sve_reinterpret_s8_mf8_x3,
  BI__builtin_sve_reinterpret_s8_s16_x3,
  BI__builtin_sve_reinterpret_s8_u16_x3,
  BI__builtin_sve_reinterpret_s8_s32_x3,
  BI__builtin_sve_reinterpret_s8_u32_x3,
  BI__builtin_sve_reinterpret_s8_s64_x3,
  BI__builtin_sve_reinterpret_s8_u64_x3,
  BI__builtin_sve_reinterpret_s8_f16_x3,
  BI__builtin_sve_reinterpret_s8_bf16_x3,
  BI__builtin_sve_reinterpret_s8_f32_x3,
  BI__builtin_sve_reinterpret_s8_f64_x3,
  BI__builtin_sve_reinterpret_u8_s8_x3,
  BI__builtin_sve_reinterpret_u8_u8_x3,
  BI__builtin_sve_reinterpret_u8_mf8_x3,
  BI__builtin_sve_reinterpret_u8_s16_x3,
  BI__builtin_sve_reinterpret_u8_u16_x3,
  BI__builtin_sve_reinterpret_u8_s32_x3,
  BI__builtin_sve_reinterpret_u8_u32_x3,
  BI__builtin_sve_reinterpret_u8_s64_x3,
  BI__builtin_sve_reinterpret_u8_u64_x3,
  BI__builtin_sve_reinterpret_u8_f16_x3,
  BI__builtin_sve_reinterpret_u8_bf16_x3,
  BI__builtin_sve_reinterpret_u8_f32_x3,
  BI__builtin_sve_reinterpret_u8_f64_x3,
  BI__builtin_sve_reinterpret_mf8_s8_x3,
  BI__builtin_sve_reinterpret_mf8_u8_x3,
  BI__builtin_sve_reinterpret_mf8_mf8_x3,
  BI__builtin_sve_reinterpret_mf8_s16_x3,
  BI__builtin_sve_reinterpret_mf8_u16_x3,
  BI__builtin_sve_reinterpret_mf8_s32_x3,
  BI__builtin_sve_reinterpret_mf8_u32_x3,
  BI__builtin_sve_reinterpret_mf8_s64_x3,
  BI__builtin_sve_reinterpret_mf8_u64_x3,
  BI__builtin_sve_reinterpret_mf8_f16_x3,
  BI__builtin_sve_reinterpret_mf8_bf16_x3,
  BI__builtin_sve_reinterpret_mf8_f32_x3,
  BI__builtin_sve_reinterpret_mf8_f64_x3,
  BI__builtin_sve_reinterpret_s16_s8_x3,
  BI__builtin_sve_reinterpret_s16_u8_x3,
  BI__builtin_sve_reinterpret_s16_mf8_x3,
  BI__builtin_sve_reinterpret_s16_s16_x3,
  BI__builtin_sve_reinterpret_s16_u16_x3,
  BI__builtin_sve_reinterpret_s16_s32_x3,
  BI__builtin_sve_reinterpret_s16_u32_x3,
  BI__builtin_sve_reinterpret_s16_s64_x3,
  BI__builtin_sve_reinterpret_s16_u64_x3,
  BI__builtin_sve_reinterpret_s16_f16_x3,
  BI__builtin_sve_reinterpret_s16_bf16_x3,
  BI__builtin_sve_reinterpret_s16_f32_x3,
  BI__builtin_sve_reinterpret_s16_f64_x3,
  BI__builtin_sve_reinterpret_u16_s8_x3,
  BI__builtin_sve_reinterpret_u16_u8_x3,
  BI__builtin_sve_reinterpret_u16_mf8_x3,
  BI__builtin_sve_reinterpret_u16_s16_x3,
  BI__builtin_sve_reinterpret_u16_u16_x3,
  BI__builtin_sve_reinterpret_u16_s32_x3,
  BI__builtin_sve_reinterpret_u16_u32_x3,
  BI__builtin_sve_reinterpret_u16_s64_x3,
  BI__builtin_sve_reinterpret_u16_u64_x3,
  BI__builtin_sve_reinterpret_u16_f16_x3,
  BI__builtin_sve_reinterpret_u16_bf16_x3,
  BI__builtin_sve_reinterpret_u16_f32_x3,
  BI__builtin_sve_reinterpret_u16_f64_x3,
  BI__builtin_sve_reinterpret_s32_s8_x3,
  BI__builtin_sve_reinterpret_s32_u8_x3,
  BI__builtin_sve_reinterpret_s32_mf8_x3,
  BI__builtin_sve_reinterpret_s32_s16_x3,
  BI__builtin_sve_reinterpret_s32_u16_x3,
  BI__builtin_sve_reinterpret_s32_s32_x3,
  BI__builtin_sve_reinterpret_s32_u32_x3,
  BI__builtin_sve_reinterpret_s32_s64_x3,
  BI__builtin_sve_reinterpret_s32_u64_x3,
  BI__builtin_sve_reinterpret_s32_f16_x3,
  BI__builtin_sve_reinterpret_s32_bf16_x3,
  BI__builtin_sve_reinterpret_s32_f32_x3,
  BI__builtin_sve_reinterpret_s32_f64_x3,
  BI__builtin_sve_reinterpret_u32_s8_x3,
  BI__builtin_sve_reinterpret_u32_u8_x3,
  BI__builtin_sve_reinterpret_u32_mf8_x3,
  BI__builtin_sve_reinterpret_u32_s16_x3,
  BI__builtin_sve_reinterpret_u32_u16_x3,
  BI__builtin_sve_reinterpret_u32_s32_x3,
  BI__builtin_sve_reinterpret_u32_u32_x3,
  BI__builtin_sve_reinterpret_u32_s64_x3,
  BI__builtin_sve_reinterpret_u32_u64_x3,
  BI__builtin_sve_reinterpret_u32_f16_x3,
  BI__builtin_sve_reinterpret_u32_bf16_x3,
  BI__builtin_sve_reinterpret_u32_f32_x3,
  BI__builtin_sve_reinterpret_u32_f64_x3,
  BI__builtin_sve_reinterpret_s64_s8_x3,
  BI__builtin_sve_reinterpret_s64_u8_x3,
  BI__builtin_sve_reinterpret_s64_mf8_x3,
  BI__builtin_sve_reinterpret_s64_s16_x3,
  BI__builtin_sve_reinterpret_s64_u16_x3,
  BI__builtin_sve_reinterpret_s64_s32_x3,
  BI__builtin_sve_reinterpret_s64_u32_x3,
  BI__builtin_sve_reinterpret_s64_s64_x3,
  BI__builtin_sve_reinterpret_s64_u64_x3,
  BI__builtin_sve_reinterpret_s64_f16_x3,
  BI__builtin_sve_reinterpret_s64_bf16_x3,
  BI__builtin_sve_reinterpret_s64_f32_x3,
  BI__builtin_sve_reinterpret_s64_f64_x3,
  BI__builtin_sve_reinterpret_u64_s8_x3,
  BI__builtin_sve_reinterpret_u64_u8_x3,
  BI__builtin_sve_reinterpret_u64_mf8_x3,
  BI__builtin_sve_reinterpret_u64_s16_x3,
  BI__builtin_sve_reinterpret_u64_u16_x3,
  BI__builtin_sve_reinterpret_u64_s32_x3,
  BI__builtin_sve_reinterpret_u64_u32_x3,
  BI__builtin_sve_reinterpret_u64_s64_x3,
  BI__builtin_sve_reinterpret_u64_u64_x3,
  BI__builtin_sve_reinterpret_u64_f16_x3,
  BI__builtin_sve_reinterpret_u64_bf16_x3,
  BI__builtin_sve_reinterpret_u64_f32_x3,
  BI__builtin_sve_reinterpret_u64_f64_x3,
  BI__builtin_sve_reinterpret_f16_s8_x3,
  BI__builtin_sve_reinterpret_f16_u8_x3,
  BI__builtin_sve_reinterpret_f16_mf8_x3,
  BI__builtin_sve_reinterpret_f16_s16_x3,
  BI__builtin_sve_reinterpret_f16_u16_x3,
  BI__builtin_sve_reinterpret_f16_s32_x3,
  BI__builtin_sve_reinterpret_f16_u32_x3,
  BI__builtin_sve_reinterpret_f16_s64_x3,
  BI__builtin_sve_reinterpret_f16_u64_x3,
  BI__builtin_sve_reinterpret_f16_f16_x3,
  BI__builtin_sve_reinterpret_f16_bf16_x3,
  BI__builtin_sve_reinterpret_f16_f32_x3,
  BI__builtin_sve_reinterpret_f16_f64_x3,
  BI__builtin_sve_reinterpret_bf16_s8_x3,
  BI__builtin_sve_reinterpret_bf16_u8_x3,
  BI__builtin_sve_reinterpret_bf16_mf8_x3,
  BI__builtin_sve_reinterpret_bf16_s16_x3,
  BI__builtin_sve_reinterpret_bf16_u16_x3,
  BI__builtin_sve_reinterpret_bf16_s32_x3,
  BI__builtin_sve_reinterpret_bf16_u32_x3,
  BI__builtin_sve_reinterpret_bf16_s64_x3,
  BI__builtin_sve_reinterpret_bf16_u64_x3,
  BI__builtin_sve_reinterpret_bf16_f16_x3,
  BI__builtin_sve_reinterpret_bf16_bf16_x3,
  BI__builtin_sve_reinterpret_bf16_f32_x3,
  BI__builtin_sve_reinterpret_bf16_f64_x3,
  BI__builtin_sve_reinterpret_f32_s8_x3,
  BI__builtin_sve_reinterpret_f32_u8_x3,
  BI__builtin_sve_reinterpret_f32_mf8_x3,
  BI__builtin_sve_reinterpret_f32_s16_x3,
  BI__builtin_sve_reinterpret_f32_u16_x3,
  BI__builtin_sve_reinterpret_f32_s32_x3,
  BI__builtin_sve_reinterpret_f32_u32_x3,
  BI__builtin_sve_reinterpret_f32_s64_x3,
  BI__builtin_sve_reinterpret_f32_u64_x3,
  BI__builtin_sve_reinterpret_f32_f16_x3,
  BI__builtin_sve_reinterpret_f32_bf16_x3,
  BI__builtin_sve_reinterpret_f32_f32_x3,
  BI__builtin_sve_reinterpret_f32_f64_x3,
  BI__builtin_sve_reinterpret_f64_s8_x3,
  BI__builtin_sve_reinterpret_f64_u8_x3,
  BI__builtin_sve_reinterpret_f64_mf8_x3,
  BI__builtin_sve_reinterpret_f64_s16_x3,
  BI__builtin_sve_reinterpret_f64_u16_x3,
  BI__builtin_sve_reinterpret_f64_s32_x3,
  BI__builtin_sve_reinterpret_f64_u32_x3,
  BI__builtin_sve_reinterpret_f64_s64_x3,
  BI__builtin_sve_reinterpret_f64_u64_x3,
  BI__builtin_sve_reinterpret_f64_f16_x3,
  BI__builtin_sve_reinterpret_f64_bf16_x3,
  BI__builtin_sve_reinterpret_f64_f32_x3,
  BI__builtin_sve_reinterpret_f64_f64_x3,
  BI__builtin_sve_reinterpret_s8_s8_x4,
  BI__builtin_sve_reinterpret_s8_u8_x4,
  BI__builtin_sve_reinterpret_s8_mf8_x4,
  BI__builtin_sve_reinterpret_s8_s16_x4,
  BI__builtin_sve_reinterpret_s8_u16_x4,
  BI__builtin_sve_reinterpret_s8_s32_x4,
  BI__builtin_sve_reinterpret_s8_u32_x4,
  BI__builtin_sve_reinterpret_s8_s64_x4,
  BI__builtin_sve_reinterpret_s8_u64_x4,
  BI__builtin_sve_reinterpret_s8_f16_x4,
  BI__builtin_sve_reinterpret_s8_bf16_x4,
  BI__builtin_sve_reinterpret_s8_f32_x4,
  BI__builtin_sve_reinterpret_s8_f64_x4,
  BI__builtin_sve_reinterpret_u8_s8_x4,
  BI__builtin_sve_reinterpret_u8_u8_x4,
  BI__builtin_sve_reinterpret_u8_mf8_x4,
  BI__builtin_sve_reinterpret_u8_s16_x4,
  BI__builtin_sve_reinterpret_u8_u16_x4,
  BI__builtin_sve_reinterpret_u8_s32_x4,
  BI__builtin_sve_reinterpret_u8_u32_x4,
  BI__builtin_sve_reinterpret_u8_s64_x4,
  BI__builtin_sve_reinterpret_u8_u64_x4,
  BI__builtin_sve_reinterpret_u8_f16_x4,
  BI__builtin_sve_reinterpret_u8_bf16_x4,
  BI__builtin_sve_reinterpret_u8_f32_x4,
  BI__builtin_sve_reinterpret_u8_f64_x4,
  BI__builtin_sve_reinterpret_mf8_s8_x4,
  BI__builtin_sve_reinterpret_mf8_u8_x4,
  BI__builtin_sve_reinterpret_mf8_mf8_x4,
  BI__builtin_sve_reinterpret_mf8_s16_x4,
  BI__builtin_sve_reinterpret_mf8_u16_x4,
  BI__builtin_sve_reinterpret_mf8_s32_x4,
  BI__builtin_sve_reinterpret_mf8_u32_x4,
  BI__builtin_sve_reinterpret_mf8_s64_x4,
  BI__builtin_sve_reinterpret_mf8_u64_x4,
  BI__builtin_sve_reinterpret_mf8_f16_x4,
  BI__builtin_sve_reinterpret_mf8_bf16_x4,
  BI__builtin_sve_reinterpret_mf8_f32_x4,
  BI__builtin_sve_reinterpret_mf8_f64_x4,
  BI__builtin_sve_reinterpret_s16_s8_x4,
  BI__builtin_sve_reinterpret_s16_u8_x4,
  BI__builtin_sve_reinterpret_s16_mf8_x4,
  BI__builtin_sve_reinterpret_s16_s16_x4,
  BI__builtin_sve_reinterpret_s16_u16_x4,
  BI__builtin_sve_reinterpret_s16_s32_x4,
  BI__builtin_sve_reinterpret_s16_u32_x4,
  BI__builtin_sve_reinterpret_s16_s64_x4,
  BI__builtin_sve_reinterpret_s16_u64_x4,
  BI__builtin_sve_reinterpret_s16_f16_x4,
  BI__builtin_sve_reinterpret_s16_bf16_x4,
  BI__builtin_sve_reinterpret_s16_f32_x4,
  BI__builtin_sve_reinterpret_s16_f64_x4,
  BI__builtin_sve_reinterpret_u16_s8_x4,
  BI__builtin_sve_reinterpret_u16_u8_x4,
  BI__builtin_sve_reinterpret_u16_mf8_x4,
  BI__builtin_sve_reinterpret_u16_s16_x4,
  BI__builtin_sve_reinterpret_u16_u16_x4,
  BI__builtin_sve_reinterpret_u16_s32_x4,
  BI__builtin_sve_reinterpret_u16_u32_x4,
  BI__builtin_sve_reinterpret_u16_s64_x4,
  BI__builtin_sve_reinterpret_u16_u64_x4,
  BI__builtin_sve_reinterpret_u16_f16_x4,
  BI__builtin_sve_reinterpret_u16_bf16_x4,
  BI__builtin_sve_reinterpret_u16_f32_x4,
  BI__builtin_sve_reinterpret_u16_f64_x4,
  BI__builtin_sve_reinterpret_s32_s8_x4,
  BI__builtin_sve_reinterpret_s32_u8_x4,
  BI__builtin_sve_reinterpret_s32_mf8_x4,
  BI__builtin_sve_reinterpret_s32_s16_x4,
  BI__builtin_sve_reinterpret_s32_u16_x4,
  BI__builtin_sve_reinterpret_s32_s32_x4,
  BI__builtin_sve_reinterpret_s32_u32_x4,
  BI__builtin_sve_reinterpret_s32_s64_x4,
  BI__builtin_sve_reinterpret_s32_u64_x4,
  BI__builtin_sve_reinterpret_s32_f16_x4,
  BI__builtin_sve_reinterpret_s32_bf16_x4,
  BI__builtin_sve_reinterpret_s32_f32_x4,
  BI__builtin_sve_reinterpret_s32_f64_x4,
  BI__builtin_sve_reinterpret_u32_s8_x4,
  BI__builtin_sve_reinterpret_u32_u8_x4,
  BI__builtin_sve_reinterpret_u32_mf8_x4,
  BI__builtin_sve_reinterpret_u32_s16_x4,
  BI__builtin_sve_reinterpret_u32_u16_x4,
  BI__builtin_sve_reinterpret_u32_s32_x4,
  BI__builtin_sve_reinterpret_u32_u32_x4,
  BI__builtin_sve_reinterpret_u32_s64_x4,
  BI__builtin_sve_reinterpret_u32_u64_x4,
  BI__builtin_sve_reinterpret_u32_f16_x4,
  BI__builtin_sve_reinterpret_u32_bf16_x4,
  BI__builtin_sve_reinterpret_u32_f32_x4,
  BI__builtin_sve_reinterpret_u32_f64_x4,
  BI__builtin_sve_reinterpret_s64_s8_x4,
  BI__builtin_sve_reinterpret_s64_u8_x4,
  BI__builtin_sve_reinterpret_s64_mf8_x4,
  BI__builtin_sve_reinterpret_s64_s16_x4,
  BI__builtin_sve_reinterpret_s64_u16_x4,
  BI__builtin_sve_reinterpret_s64_s32_x4,
  BI__builtin_sve_reinterpret_s64_u32_x4,
  BI__builtin_sve_reinterpret_s64_s64_x4,
  BI__builtin_sve_reinterpret_s64_u64_x4,
  BI__builtin_sve_reinterpret_s64_f16_x4,
  BI__builtin_sve_reinterpret_s64_bf16_x4,
  BI__builtin_sve_reinterpret_s64_f32_x4,
  BI__builtin_sve_reinterpret_s64_f64_x4,
  BI__builtin_sve_reinterpret_u64_s8_x4,
  BI__builtin_sve_reinterpret_u64_u8_x4,
  BI__builtin_sve_reinterpret_u64_mf8_x4,
  BI__builtin_sve_reinterpret_u64_s16_x4,
  BI__builtin_sve_reinterpret_u64_u16_x4,
  BI__builtin_sve_reinterpret_u64_s32_x4,
  BI__builtin_sve_reinterpret_u64_u32_x4,
  BI__builtin_sve_reinterpret_u64_s64_x4,
  BI__builtin_sve_reinterpret_u64_u64_x4,
  BI__builtin_sve_reinterpret_u64_f16_x4,
  BI__builtin_sve_reinterpret_u64_bf16_x4,
  BI__builtin_sve_reinterpret_u64_f32_x4,
  BI__builtin_sve_reinterpret_u64_f64_x4,
  BI__builtin_sve_reinterpret_f16_s8_x4,
  BI__builtin_sve_reinterpret_f16_u8_x4,
  BI__builtin_sve_reinterpret_f16_mf8_x4,
  BI__builtin_sve_reinterpret_f16_s16_x4,
  BI__builtin_sve_reinterpret_f16_u16_x4,
  BI__builtin_sve_reinterpret_f16_s32_x4,
  BI__builtin_sve_reinterpret_f16_u32_x4,
  BI__builtin_sve_reinterpret_f16_s64_x4,
  BI__builtin_sve_reinterpret_f16_u64_x4,
  BI__builtin_sve_reinterpret_f16_f16_x4,
  BI__builtin_sve_reinterpret_f16_bf16_x4,
  BI__builtin_sve_reinterpret_f16_f32_x4,
  BI__builtin_sve_reinterpret_f16_f64_x4,
  BI__builtin_sve_reinterpret_bf16_s8_x4,
  BI__builtin_sve_reinterpret_bf16_u8_x4,
  BI__builtin_sve_reinterpret_bf16_mf8_x4,
  BI__builtin_sve_reinterpret_bf16_s16_x4,
  BI__builtin_sve_reinterpret_bf16_u16_x4,
  BI__builtin_sve_reinterpret_bf16_s32_x4,
  BI__builtin_sve_reinterpret_bf16_u32_x4,
  BI__builtin_sve_reinterpret_bf16_s64_x4,
  BI__builtin_sve_reinterpret_bf16_u64_x4,
  BI__builtin_sve_reinterpret_bf16_f16_x4,
  BI__builtin_sve_reinterpret_bf16_bf16_x4,
  BI__builtin_sve_reinterpret_bf16_f32_x4,
  BI__builtin_sve_reinterpret_bf16_f64_x4,
  BI__builtin_sve_reinterpret_f32_s8_x4,
  BI__builtin_sve_reinterpret_f32_u8_x4,
  BI__builtin_sve_reinterpret_f32_mf8_x4,
  BI__builtin_sve_reinterpret_f32_s16_x4,
  BI__builtin_sve_reinterpret_f32_u16_x4,
  BI__builtin_sve_reinterpret_f32_s32_x4,
  BI__builtin_sve_reinterpret_f32_u32_x4,
  BI__builtin_sve_reinterpret_f32_s64_x4,
  BI__builtin_sve_reinterpret_f32_u64_x4,
  BI__builtin_sve_reinterpret_f32_f16_x4,
  BI__builtin_sve_reinterpret_f32_bf16_x4,
  BI__builtin_sve_reinterpret_f32_f32_x4,
  BI__builtin_sve_reinterpret_f32_f64_x4,
  BI__builtin_sve_reinterpret_f64_s8_x4,
  BI__builtin_sve_reinterpret_f64_u8_x4,
  BI__builtin_sve_reinterpret_f64_mf8_x4,
  BI__builtin_sve_reinterpret_f64_s16_x4,
  BI__builtin_sve_reinterpret_f64_u16_x4,
  BI__builtin_sve_reinterpret_f64_s32_x4,
  BI__builtin_sve_reinterpret_f64_u32_x4,
  BI__builtin_sve_reinterpret_f64_s64_x4,
  BI__builtin_sve_reinterpret_f64_u64_x4,
  BI__builtin_sve_reinterpret_f64_f16_x4,
  BI__builtin_sve_reinterpret_f64_bf16_x4,
  BI__builtin_sve_reinterpret_f64_f32_x4,
  BI__builtin_sve_reinterpret_f64_f64_x4,
#endif // GET_SVE_BUILTIN_ENUMERATORS

#ifdef GET_SVE_BUILTIN_STR_TABLE

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinStringsStorage[] =
  "\0"
  "n\0"
  "svaba_n_s16\0"
  "q8sq8sq8ss\0"
  "(sve,(sve2|sme))|sme\0"
  "svaba_n_s32\0"
  "q4iq4iq4ii\0"
  "svaba_n_s64\0"
  "q2Wiq2Wiq2WiWi\0"
  "svaba_n_s8\0"
  "q16Scq16Scq16ScSc\0"
  "svaba_n_u16\0"
  "q8Usq8Usq8UsUs\0"
  "svaba_n_u32\0"
  "q4Uiq4Uiq4UiUi\0"
  "svaba_n_u64\0"
  "q2UWiq2UWiq2UWiUWi\0"
  "svaba_n_u8\0"
  "q16Ucq16Ucq16UcUc\0"
  "svaba_s16\0"
  "q8sq8sq8sq8s\0"
  "svaba_s32\0"
  "q4iq4iq4iq4i\0"
  "svaba_s64\0"
  "q2Wiq2Wiq2Wiq2Wi\0"
  "svaba_s8\0"
  "q16Scq16Scq16Scq16Sc\0"
  "svaba_u16\0"
  "q8Usq8Usq8Usq8Us\0"
  "svaba_u32\0"
  "q4Uiq4Uiq4Uiq4Ui\0"
  "svaba_u64\0"
  "q2UWiq2UWiq2UWiq2UWi\0"
  "svaba_u8\0"
  "q16Ucq16Ucq16Ucq16Uc\0"
  "svabalb_n_s16\0"
  "q8sq8sq16ScSc\0"
  "svabalb_n_s32\0"
  "q4iq4iq8ss\0"
  "svabalb_n_s64\0"
  "q2Wiq2Wiq4ii\0"
  "svabalb_n_u16\0"
  "q8Usq8Usq16UcUc\0"
  "svabalb_n_u32\0"
  "q4Uiq4Uiq8UsUs\0"
  "svabalb_n_u64\0"
  "q2UWiq2UWiq4UiUi\0"
  "svabalb_s16\0"
  "q8sq8sq16Scq16Sc\0"
  "svabalb_s32\0"
  "q4iq4iq8sq8s\0"
  "svabalb_s64\0"
  "q2Wiq2Wiq4iq4i\0"
  "svabalb_u16\0"
  "q8Usq8Usq16Ucq16Uc\0"
  "svabalb_u32\0"
  "q4Uiq4Uiq8Usq8Us\0"
  "svabalb_u64\0"
  "q2UWiq2UWiq4Uiq4Ui\0"
  "svabalt_n_s16\0"
  "svabalt_n_s32\0"
  "svabalt_n_s64\0"
  "svabalt_n_u16\0"
  "svabalt_n_u32\0"
  "svabalt_n_u64\0"
  "svabalt_s16\0"
  "svabalt_s32\0"
  "svabalt_s64\0"
  "svabalt_u16\0"
  "svabalt_u32\0"
  "svabalt_u64\0"
  "svabd_f16_m\0"
  "q8hq16bq8hq8h\0"
  "sve|sme\0"
  "svabd_f16_x\0"
  "svabd_f16_z\0"
  "svabd_f32_m\0"
  "q4fq16bq4fq4f\0"
  "svabd_f32_x\0"
  "svabd_f32_z\0"
  "svabd_f64_m\0"
  "q2dq16bq2dq2d\0"
  "svabd_f64_x\0"
  "svabd_f64_z\0"
  "svabd_n_f16_m\0"
  "q8hq16bq8hh\0"
  "svabd_n_f16_x\0"
  "svabd_n_f16_z\0"
  "svabd_n_f32_m\0"
  "q4fq16bq4ff\0"
  "svabd_n_f32_x\0"
  "svabd_n_f32_z\0"
  "svabd_n_f64_m\0"
  "q2dq16bq2dd\0"
  "svabd_n_f64_x\0"
  "svabd_n_f64_z\0"
  "svabd_n_s16_m\0"
  "q8sq16bq8ss\0"
  "svabd_n_s16_x\0"
  "svabd_n_s16_z\0"
  "svabd_n_s32_m\0"
  "q4iq16bq4ii\0"
  "svabd_n_s32_x\0"
  "svabd_n_s32_z\0"
  "svabd_n_s64_m\0"
  "q2Wiq16bq2WiWi\0"
  "svabd_n_s64_x\0"
  "svabd_n_s64_z\0"
  "svabd_n_s8_m\0"
  "q16Scq16bq16ScSc\0"
  "svabd_n_s8_x\0"
  "svabd_n_s8_z\0"
  "svabd_n_u16_m\0"
  "q8Usq16bq8UsUs\0"
  "svabd_n_u16_x\0"
  "svabd_n_u16_z\0"
  "svabd_n_u32_m\0"
  "q4Uiq16bq4UiUi\0"
  "svabd_n_u32_x\0"
  "svabd_n_u32_z\0"
  "svabd_n_u64_m\0"
  "q2UWiq16bq2UWiUWi\0"
  "svabd_n_u64_x\0"
  "svabd_n_u64_z\0"
  "svabd_n_u8_m\0"
  "q16Ucq16bq16UcUc\0"
  "svabd_n_u8_x\0"
  "svabd_n_u8_z\0"
  "svabd_s16_m\0"
  "q8sq16bq8sq8s\0"
  "svabd_s16_x\0"
  "svabd_s16_z\0"
  "svabd_s32_m\0"
  "q4iq16bq4iq4i\0"
  "svabd_s32_x\0"
  "svabd_s32_z\0"
  "svabd_s64_m\0"
  "q2Wiq16bq2Wiq2Wi\0"
  "svabd_s64_x\0"
  "svabd_s64_z\0"
  "svabd_s8_m\0"
  "q16Scq16bq16Scq16Sc\0"
  "svabd_s8_x\0"
  "svabd_s8_z\0"
  "svabd_u16_m\0"
  "q8Usq16bq8Usq8Us\0"
  "svabd_u16_x\0"
  "svabd_u16_z\0"
  "svabd_u32_m\0"
  "q4Uiq16bq4Uiq4Ui\0"
  "svabd_u32_x\0"
  "svabd_u32_z\0"
  "svabd_u64_m\0"
  "q2UWiq16bq2UWiq2UWi\0"
  "svabd_u64_x\0"
  "svabd_u64_z\0"
  "svabd_u8_m\0"
  "q16Ucq16bq16Ucq16Uc\0"
  "svabd_u8_x\0"
  "svabd_u8_z\0"
  "svabdlb_n_s16\0"
  "q8sq16ScSc\0"
  "svabdlb_n_s32\0"
  "q4iq8ss\0"
  "svabdlb_n_s64\0"
  "q2Wiq4ii\0"
  "svabdlb_n_u16\0"
  "q8Usq16UcUc\0"
  "svabdlb_n_u32\0"
  "q4Uiq8UsUs\0"
  "svabdlb_n_u64\0"
  "q2UWiq4UiUi\0"
  "svabdlb_s16\0"
  "q8sq16Scq16Sc\0"
  "svabdlb_s32\0"
  "q4iq8sq8s\0"
  "svabdlb_s64\0"
  "q2Wiq4iq4i\0"
  "svabdlb_u16\0"
  "q8Usq16Ucq16Uc\0"
  "svabdlb_u32\0"
  "q4Uiq8Usq8Us\0"
  "svabdlb_u64\0"
  "q2UWiq4Uiq4Ui\0"
  "svabdlt_n_s16\0"
  "svabdlt_n_s32\0"
  "svabdlt_n_s64\0"
  "svabdlt_n_u16\0"
  "svabdlt_n_u32\0"
  "svabdlt_n_u64\0"
  "svabdlt_s16\0"
  "svabdlt_s32\0"
  "svabdlt_s64\0"
  "svabdlt_u16\0"
  "svabdlt_u32\0"
  "svabdlt_u64\0"
  "svabs_f16_m\0"
  "q8hq8hq16bq8h\0"
  "svabs_f16_x\0"
  "q8hq16bq8h\0"
  "svabs_f16_z\0"
  "svabs_f32_m\0"
  "q4fq4fq16bq4f\0"
  "svabs_f32_x\0"
  "q4fq16bq4f\0"
  "svabs_f32_z\0"
  "svabs_f64_m\0"
  "q2dq2dq16bq2d\0"
  "svabs_f64_x\0"
  "q2dq16bq2d\0"
  "svabs_f64_z\0"
  "svabs_s16_m\0"
  "q8sq8sq16bq8s\0"
  "svabs_s16_x\0"
  "q8sq16bq8s\0"
  "svabs_s16_z\0"
  "svabs_s32_m\0"
  "q4iq4iq16bq4i\0"
  "svabs_s32_x\0"
  "q4iq16bq4i\0"
  "svabs_s32_z\0"
  "svabs_s64_m\0"
  "q2Wiq2Wiq16bq2Wi\0"
  "svabs_s64_x\0"
  "q2Wiq16bq2Wi\0"
  "svabs_s64_z\0"
  "svabs_s8_m\0"
  "q16Scq16Scq16bq16Sc\0"
  "svabs_s8_x\0"
  "q16Scq16bq16Sc\0"
  "svabs_s8_z\0"
  "svacge_f16\0"
  "q16bq16bq8hq8h\0"
  "svacge_f32\0"
  "q16bq16bq4fq4f\0"
  "svacge_f64\0"
  "q16bq16bq2dq2d\0"
  "svacge_n_f16\0"
  "q16bq16bq8hh\0"
  "svacge_n_f32\0"
  "q16bq16bq4ff\0"
  "svacge_n_f64\0"
  "q16bq16bq2dd\0"
  "svacgt_f16\0"
  "svacgt_f32\0"
  "svacgt_f64\0"
  "svacgt_n_f16\0"
  "svacgt_n_f32\0"
  "svacgt_n_f64\0"
  "svacle_f16\0"
  "svacle_f32\0"
  "svacle_f64\0"
  "svacle_n_f16\0"
  "svacle_n_f32\0"
  "svacle_n_f64\0"
  "svaclt_f16\0"
  "svaclt_f32\0"
  "svaclt_f64\0"
  "svaclt_n_f16\0"
  "svaclt_n_f32\0"
  "svaclt_n_f64\0"
  "svadalp_s16_m\0"
  "q8sq16bq8sq16Sc\0"
  "svadalp_s16_x\0"
  "svadalp_s16_z\0"
  "svadalp_s32_m\0"
  "q4iq16bq4iq8s\0"
  "svadalp_s32_x\0"
  "svadalp_s32_z\0"
  "svadalp_s64_m\0"
  "q2Wiq16bq2Wiq4i\0"
  "svadalp_s64_x\0"
  "svadalp_s64_z\0"
  "svadalp_u16_m\0"
  "q8Usq16bq8Usq16Uc\0"
  "svadalp_u16_x\0"
  "svadalp_u16_z\0"
  "svadalp_u32_m\0"
  "q4Uiq16bq4Uiq8Us\0"
  "svadalp_u32_x\0"
  "svadalp_u32_z\0"
  "svadalp_u64_m\0"
  "q2UWiq16bq2UWiq4Ui\0"
  "svadalp_u64_x\0"
  "svadalp_u64_z\0"
  "svadclb_n_u32\0"
  "svadclb_n_u64\0"
  "svadclb_u32\0"
  "svadclb_u64\0"
  "svadclt_n_u32\0"
  "svadclt_n_u64\0"
  "svadclt_u32\0"
  "svadclt_u64\0"
  "svadd_bf16_m\0"
  "q8yq16bq8yq8y\0"
  "(sve,sve-b16b16)|(sme,sme2,sve-b16b16)\0"
  "svadd_bf16_x\0"
  "svadd_bf16_z\0"
  "svadd_f16_m\0"
  "svadd_f16_x\0"
  "svadd_f16_z\0"
  "svadd_f32_m\0"
  "svadd_f32_x\0"
  "svadd_f32_z\0"
  "svadd_f64_m\0"
  "svadd_f64_x\0"
  "svadd_f64_z\0"
  "svadd_n_bf16_m\0"
  "q8yq16bq8yy\0"
  "svadd_n_bf16_x\0"
  "svadd_n_bf16_z\0"
  "svadd_n_f16_m\0"
  "svadd_n_f16_x\0"
  "svadd_n_f16_z\0"
  "svadd_n_f32_m\0"
  "svadd_n_f32_x\0"
  "svadd_n_f32_z\0"
  "svadd_n_f64_m\0"
  "svadd_n_f64_x\0"
  "svadd_n_f64_z\0"
  "svadd_n_s16_m\0"
  "svadd_n_s16_x\0"
  "svadd_n_s16_z\0"
  "svadd_n_s32_m\0"
  "svadd_n_s32_x\0"
  "svadd_n_s32_z\0"
  "svadd_n_s64_m\0"
  "svadd_n_s64_x\0"
  "svadd_n_s64_z\0"
  "svadd_n_s8_m\0"
  "svadd_n_s8_x\0"
  "svadd_n_s8_z\0"
  "svadd_n_u16_m\0"
  "svadd_n_u16_x\0"
  "svadd_n_u16_z\0"
  "svadd_n_u32_m\0"
  "svadd_n_u32_x\0"
  "svadd_n_u32_z\0"
  "svadd_n_u64_m\0"
  "svadd_n_u64_x\0"
  "svadd_n_u64_z\0"
  "svadd_n_u8_m\0"
  "svadd_n_u8_x\0"
  "svadd_n_u8_z\0"
  "svadd_s16_m\0"
  "svadd_s16_x\0"
  "svadd_s16_z\0"
  "svadd_s32_m\0"
  "svadd_s32_x\0"
  "svadd_s32_z\0"
  "svadd_s64_m\0"
  "svadd_s64_x\0"
  "svadd_s64_z\0"
  "svadd_s8_m\0"
  "svadd_s8_x\0"
  "svadd_s8_z\0"
  "svadd_single_s16_x2\0"
  "q16sq16sq8s\0"
  "sme,sme2\0"
  "svadd_single_s16_x4\0"
  "q32sq32sq8s\0"
  "svadd_single_s32_x2\0"
  "q8iq8iq4i\0"
  "svadd_single_s32_x4\0"
  "q16iq16iq4i\0"
  "svadd_single_s64_x2\0"
  "q4Wiq4Wiq2Wi\0"
  "svadd_single_s64_x4\0"
  "q8Wiq8Wiq2Wi\0"
  "svadd_single_s8_x2\0"
  "q32Scq32Scq16Sc\0"
  "svadd_single_s8_x4\0"
  "q64Scq64Scq16Sc\0"
  "svadd_single_u16_x2\0"
  "q16Usq16Usq8Us\0"
  "svadd_single_u16_x4\0"
  "q32Usq32Usq8Us\0"
  "svadd_single_u32_x2\0"
  "q8Uiq8Uiq4Ui\0"
  "svadd_single_u32_x4\0"
  "q16Uiq16Uiq4Ui\0"
  "svadd_single_u64_x2\0"
  "q4UWiq4UWiq2UWi\0"
  "svadd_single_u64_x4\0"
  "q8UWiq8UWiq2UWi\0"
  "svadd_single_u8_x2\0"
  "q32Ucq32Ucq16Uc\0"
  "svadd_single_u8_x4\0"
  "q64Ucq64Ucq16Uc\0"
  "svadd_u16_m\0"
  "svadd_u16_x\0"
  "svadd_u16_z\0"
  "svadd_u32_m\0"
  "svadd_u32_x\0"
  "svadd_u32_z\0"
  "svadd_u64_m\0"
  "svadd_u64_x\0"
  "svadd_u64_z\0"
  "svadd_u8_m\0"
  "svadd_u8_x\0"
  "svadd_u8_z\0"
  "svadda_f16\0"
  "hq16bhq8h\0"
  "sve\0"
  "svadda_f32\0"
  "fq16bfq4f\0"
  "svadda_f64\0"
  "dq16bdq2d\0"
  "svaddhnb_n_s16\0"
  "q16Scq8ss\0"
  "svaddhnb_n_s32\0"
  "q8sq4ii\0"
  "svaddhnb_n_s64\0"
  "q4iq2WiWi\0"
  "svaddhnb_n_u16\0"
  "q16Ucq8UsUs\0"
  "svaddhnb_n_u32\0"
  "q8Usq4UiUi\0"
  "svaddhnb_n_u64\0"
  "q4Uiq2UWiUWi\0"
  "svaddhnb_s16\0"
  "q16Scq8sq8s\0"
  "svaddhnb_s32\0"
  "q8sq4iq4i\0"
  "svaddhnb_s64\0"
  "q4iq2Wiq2Wi\0"
  "svaddhnb_u16\0"
  "q16Ucq8Usq8Us\0"
  "svaddhnb_u32\0"
  "q8Usq4Uiq4Ui\0"
  "svaddhnb_u64\0"
  "q4Uiq2UWiq2UWi\0"
  "svaddhnt_n_s16\0"
  "q16Scq16Scq8ss\0"
  "svaddhnt_n_s32\0"
  "q8sq8sq4ii\0"
  "svaddhnt_n_s64\0"
  "q4iq4iq2WiWi\0"
  "svaddhnt_n_u16\0"
  "q16Ucq16Ucq8UsUs\0"
  "svaddhnt_n_u32\0"
  "q8Usq8Usq4UiUi\0"
  "svaddhnt_n_u64\0"
  "q4Uiq4Uiq2UWiUWi\0"
  "svaddhnt_s16\0"
  "q16Scq16Scq8sq8s\0"
  "svaddhnt_s32\0"
  "q8sq8sq4iq4i\0"
  "svaddhnt_s64\0"
  "q4iq4iq2Wiq2Wi\0"
  "svaddhnt_u16\0"
  "q16Ucq16Ucq8Usq8Us\0"
  "svaddhnt_u32\0"
  "q8Usq8Usq4Uiq4Ui\0"
  "svaddhnt_u64\0"
  "q4Uiq4Uiq2UWiq2UWi\0"
  "svaddlb_n_s16\0"
  "svaddlb_n_s32\0"
  "svaddlb_n_s64\0"
  "svaddlb_n_u16\0"
  "svaddlb_n_u32\0"
  "svaddlb_n_u64\0"
  "svaddlb_s16\0"
  "svaddlb_s32\0"
  "svaddlb_s64\0"
  "svaddlb_u16\0"
  "svaddlb_u32\0"
  "svaddlb_u64\0"
  "svaddlbt_n_s16\0"
  "svaddlbt_n_s32\0"
  "svaddlbt_n_s64\0"
  "svaddlbt_s16\0"
  "svaddlbt_s32\0"
  "svaddlbt_s64\0"
  "svaddlt_n_s16\0"
  "svaddlt_n_s32\0"
  "svaddlt_n_s64\0"
  "svaddlt_n_u16\0"
  "svaddlt_n_u32\0"
  "svaddlt_n_u64\0"
  "svaddlt_s16\0"
  "svaddlt_s32\0"
  "svaddlt_s64\0"
  "svaddlt_u16\0"
  "svaddlt_u32\0"
  "svaddlt_u64\0"
  "svaddp_f16_m\0"
  "svaddp_f16_x\0"
  "svaddp_f32_m\0"
  "svaddp_f32_x\0"
  "svaddp_f64_m\0"
  "svaddp_f64_x\0"
  "svaddp_s16_m\0"
  "svaddp_s16_x\0"
  "svaddp_s32_m\0"
  "svaddp_s32_x\0"
  "svaddp_s64_m\0"
  "svaddp_s64_x\0"
  "svaddp_s8_m\0"
  "svaddp_s8_x\0"
  "svaddp_u16_m\0"
  "svaddp_u16_x\0"
  "svaddp_u32_m\0"
  "svaddp_u32_x\0"
  "svaddp_u64_m\0"
  "svaddp_u64_x\0"
  "svaddp_u8_m\0"
  "svaddp_u8_x\0"
  "svaddqv_f16\0"
  "V8hq16bq8h\0"
  "(sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1))\0"
  "svaddqv_f32\0"
  "V4fq16bq4f\0"
  "svaddqv_f64\0"
  "V2dq16bq2d\0"
  "svaddqv_s16\0"
  "V8sq16bq8s\0"
  "svaddqv_s32\0"
  "V4iq16bq4i\0"
  "svaddqv_s64\0"
  "V2Wiq16bq2Wi\0"
  "svaddqv_s8\0"
  "V16Scq16bq16Sc\0"
  "svaddqv_u16\0"
  "V8Usq16bq8Us\0"
  "svaddqv_u32\0"
  "V4Uiq16bq4Ui\0"
  "svaddqv_u64\0"
  "V2UWiq16bq2UWi\0"
  "svaddqv_u8\0"
  "V16Ucq16bq16Uc\0"
  "svaddv_f16\0"
  "hq16bq8h\0"
  "svaddv_f32\0"
  "fq16bq4f\0"
  "svaddv_f64\0"
  "dq16bq2d\0"
  "svaddv_s16\0"
  "Wiq16bq8s\0"
  "svaddv_s32\0"
  "Wiq16bq4i\0"
  "svaddv_s64\0"
  "Wiq16bq2Wi\0"
  "svaddv_s8\0"
  "Wiq16bq16Sc\0"
  "svaddv_u16\0"
  "UWiq16bq8Us\0"
  "svaddv_u32\0"
  "UWiq16bq4Ui\0"
  "svaddv_u64\0"
  "UWiq16bq2UWi\0"
  "svaddv_u8\0"
  "UWiq16bq16Uc\0"
  "svaddwb_n_s16\0"
  "q8sq8sSc\0"
  "svaddwb_n_s32\0"
  "q4iq4is\0"
  "svaddwb_n_s64\0"
  "q2Wiq2Wii\0"
  "svaddwb_n_u16\0"
  "q8Usq8UsUc\0"
  "svaddwb_n_u32\0"
  "q4Uiq4UiUs\0"
  "svaddwb_n_u64\0"
  "q2UWiq2UWiUi\0"
  "svaddwb_s16\0"
  "q8sq8sq16Sc\0"
  "svaddwb_s32\0"
  "q4iq4iq8s\0"
  "svaddwb_s64\0"
  "q2Wiq2Wiq4i\0"
  "svaddwb_u16\0"
  "q8Usq8Usq16Uc\0"
  "svaddwb_u32\0"
  "q4Uiq4Uiq8Us\0"
  "svaddwb_u64\0"
  "q2UWiq2UWiq4Ui\0"
  "svaddwt_n_s16\0"
  "svaddwt_n_s32\0"
  "svaddwt_n_s64\0"
  "svaddwt_n_u16\0"
  "svaddwt_n_u32\0"
  "svaddwt_n_u64\0"
  "svaddwt_s16\0"
  "svaddwt_s32\0"
  "svaddwt_s64\0"
  "svaddwt_u16\0"
  "svaddwt_u32\0"
  "svaddwt_u64\0"
  "svadrb_u32base_s32offset\0"
  "q4Uiq4Uiq4i\0"
  "svadrb_u32base_u32offset\0"
  "q4Uiq4Uiq4Ui\0"
  "svadrb_u64base_s64offset\0"
  "q2UWiq2UWiq2Wi\0"
  "svadrb_u64base_u64offset\0"
  "q2UWiq2UWiq2UWi\0"
  "svadrd_u32base_s32index\0"
  "svadrd_u32base_u32index\0"
  "svadrd_u64base_s64index\0"
  "svadrd_u64base_u64index\0"
  "svadrh_u32base_s32index\0"
  "svadrh_u32base_u32index\0"
  "svadrh_u64base_s64index\0"
  "svadrh_u64base_u64index\0"
  "svadrw_u32base_s32index\0"
  "svadrw_u32base_u32index\0"
  "svadrw_u64base_s64index\0"
  "svadrw_u64base_u64index\0"
  "svaesd_lane_u8_x2\0"
  "q32Ucq32Ucq16UcIUWi\0"
  "(sve,sve-aes2)|(sme,sve-aes2,ssve-aes)\0"
  "svaesd_lane_u8_x4\0"
  "q64Ucq64Ucq16UcIUWi\0"
  "svaesd_u8\0"
  "q16Ucq16Ucq16Uc\0"
  "(sve,sve-aes)|(sme,ssve-aes)\0"
  "svaesdimc_lane_u8_x2\0"
  "svaesdimc_lane_u8_x4\0"
  "svaese_lane_u8_x2\0"
  "svaese_lane_u8_x4\0"
  "svaese_u8\0"
  "svaesemc_lane_u8_x2\0"
  "svaesemc_lane_u8_x4\0"
  "svaesimc_u8\0"
  "q16Ucq16Uc\0"
  "svaesmc_u8\0"
  "svamax_f16_m\0"
  "(sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax)\0"
  "svamax_f16_x\0"
  "svamax_f16_x2\0"
  "q16hq16hq16h\0"
  "sme,sme2,faminmax\0"
  "svamax_f16_x4\0"
  "q32hq32hq32h\0"
  "svamax_f16_z\0"
  "svamax_f32_m\0"
  "svamax_f32_x\0"
  "svamax_f32_x2\0"
  "q8fq8fq8f\0"
  "svamax_f32_x4\0"
  "q16fq16fq16f\0"
  "svamax_f32_z\0"
  "svamax_f64_m\0"
  "svamax_f64_x\0"
  "svamax_f64_x2\0"
  "q4dq4dq4d\0"
  "svamax_f64_x4\0"
  "q8dq8dq8d\0"
  "svamax_f64_z\0"
  "svamax_n_f16_m\0"
  "svamax_n_f16_x\0"
  "svamax_n_f16_z\0"
  "svamax_n_f32_m\0"
  "svamax_n_f32_x\0"
  "svamax_n_f32_z\0"
  "svamax_n_f64_m\0"
  "svamax_n_f64_x\0"
  "svamax_n_f64_z\0"
  "svamin_f16_m\0"
  "svamin_f16_x\0"
  "svamin_f16_x2\0"
  "svamin_f16_x4\0"
  "svamin_f16_z\0"
  "svamin_f32_m\0"
  "svamin_f32_x\0"
  "svamin_f32_x2\0"
  "svamin_f32_x4\0"
  "svamin_f32_z\0"
  "svamin_f64_m\0"
  "svamin_f64_x\0"
  "svamin_f64_x2\0"
  "svamin_f64_x4\0"
  "svamin_f64_z\0"
  "svamin_n_f16_m\0"
  "svamin_n_f16_x\0"
  "svamin_n_f16_z\0"
  "svamin_n_f32_m\0"
  "svamin_n_f32_x\0"
  "svamin_n_f32_z\0"
  "svamin_n_f64_m\0"
  "svamin_n_f64_x\0"
  "svamin_n_f64_z\0"
  "svand_b_z\0"
  "q16bq16bq16bq16b\0"
  "svand_n_s16_m\0"
  "svand_n_s16_x\0"
  "svand_n_s16_z\0"
  "svand_n_s32_m\0"
  "svand_n_s32_x\0"
  "svand_n_s32_z\0"
  "svand_n_s64_m\0"
  "svand_n_s64_x\0"
  "svand_n_s64_z\0"
  "svand_n_s8_m\0"
  "svand_n_s8_x\0"
  "svand_n_s8_z\0"
  "svand_n_u16_m\0"
  "svand_n_u16_x\0"
  "svand_n_u16_z\0"
  "svand_n_u32_m\0"
  "svand_n_u32_x\0"
  "svand_n_u32_z\0"
  "svand_n_u64_m\0"
  "svand_n_u64_x\0"
  "svand_n_u64_z\0"
  "svand_n_u8_m\0"
  "svand_n_u8_x\0"
  "svand_n_u8_z\0"
  "svand_s16_m\0"
  "svand_s16_x\0"
  "svand_s16_z\0"
  "svand_s32_m\0"
  "svand_s32_x\0"
  "svand_s32_z\0"
  "svand_s64_m\0"
  "svand_s64_x\0"
  "svand_s64_z\0"
  "svand_s8_m\0"
  "svand_s8_x\0"
  "svand_s8_z\0"
  "svand_u16_m\0"
  "svand_u16_x\0"
  "svand_u16_z\0"
  "svand_u32_m\0"
  "svand_u32_x\0"
  "svand_u32_z\0"
  "svand_u64_m\0"
  "svand_u64_x\0"
  "svand_u64_z\0"
  "svand_u8_m\0"
  "svand_u8_x\0"
  "svand_u8_z\0"
  "svandqv_s16\0"
  "svandqv_s32\0"
  "svandqv_s64\0"
  "svandqv_s8\0"
  "svandqv_u16\0"
  "svandqv_u32\0"
  "svandqv_u64\0"
  "svandqv_u8\0"
  "svandv_s16\0"
  "sq16bq8s\0"
  "svandv_s32\0"
  "iq16bq4i\0"
  "svandv_s64\0"
  "svandv_s8\0"
  "Scq16bq16Sc\0"
  "svandv_u16\0"
  "Usq16bq8Us\0"
  "svandv_u32\0"
  "Uiq16bq4Ui\0"
  "svandv_u64\0"
  "svandv_u8\0"
  "Ucq16bq16Uc\0"
  "svasr_n_s16_m\0"
  "q8sq16bq8sUs\0"
  "svasr_n_s16_x\0"
  "svasr_n_s16_z\0"
  "svasr_n_s32_m\0"
  "q4iq16bq4iUi\0"
  "svasr_n_s32_x\0"
  "svasr_n_s32_z\0"
  "svasr_n_s64_m\0"
  "q2Wiq16bq2WiUWi\0"
  "svasr_n_s64_x\0"
  "svasr_n_s64_z\0"
  "svasr_n_s8_m\0"
  "q16Scq16bq16ScUc\0"
  "svasr_n_s8_x\0"
  "svasr_n_s8_z\0"
  "svasr_s16_m\0"
  "q8sq16bq8sq8Us\0"
  "svasr_s16_x\0"
  "svasr_s16_z\0"
  "svasr_s32_m\0"
  "q4iq16bq4iq4Ui\0"
  "svasr_s32_x\0"
  "svasr_s32_z\0"
  "svasr_s64_m\0"
  "q2Wiq16bq2Wiq2UWi\0"
  "svasr_s64_x\0"
  "svasr_s64_z\0"
  "svasr_s8_m\0"
  "q16Scq16bq16Scq16Uc\0"
  "svasr_s8_x\0"
  "svasr_s8_z\0"
  "svasr_wide_n_s16_m\0"
  "q8sq16bq8sUWi\0"
  "svasr_wide_n_s16_x\0"
  "svasr_wide_n_s16_z\0"
  "svasr_wide_n_s32_m\0"
  "q4iq16bq4iUWi\0"
  "svasr_wide_n_s32_x\0"
  "svasr_wide_n_s32_z\0"
  "svasr_wide_n_s8_m\0"
  "q16Scq16bq16ScUWi\0"
  "svasr_wide_n_s8_x\0"
  "svasr_wide_n_s8_z\0"
  "svasr_wide_s16_m\0"
  "q8sq16bq8sq2UWi\0"
  "svasr_wide_s16_x\0"
  "svasr_wide_s16_z\0"
  "svasr_wide_s32_m\0"
  "q4iq16bq4iq2UWi\0"
  "svasr_wide_s32_x\0"
  "svasr_wide_s32_z\0"
  "svasr_wide_s8_m\0"
  "q16Scq16bq16Scq2UWi\0"
  "svasr_wide_s8_x\0"
  "svasr_wide_s8_z\0"
  "svasrd_n_s16_m\0"
  "q8sq16bq8sIUWi\0"
  "svasrd_n_s16_x\0"
  "svasrd_n_s16_z\0"
  "svasrd_n_s32_m\0"
  "q4iq16bq4iIUWi\0"
  "svasrd_n_s32_x\0"
  "svasrd_n_s32_z\0"
  "svasrd_n_s64_m\0"
  "q2Wiq16bq2WiIUWi\0"
  "svasrd_n_s64_x\0"
  "svasrd_n_s64_z\0"
  "svasrd_n_s8_m\0"
  "q16Scq16bq16ScIUWi\0"
  "svasrd_n_s8_x\0"
  "svasrd_n_s8_z\0"
  "svbcax_n_s16\0"
  "svbcax_n_s32\0"
  "svbcax_n_s64\0"
  "svbcax_n_s8\0"
  "svbcax_n_u16\0"
  "svbcax_n_u32\0"
  "svbcax_n_u64\0"
  "svbcax_n_u8\0"
  "svbcax_s16\0"
  "svbcax_s32\0"
  "svbcax_s64\0"
  "svbcax_s8\0"
  "svbcax_u16\0"
  "svbcax_u32\0"
  "svbcax_u64\0"
  "svbcax_u8\0"
  "svbdep_n_u16\0"
  "q8Usq8UsUs\0"
  "(sve,sve-bitperm)|(sme,ssve-bitperm)\0"
  "svbdep_n_u32\0"
  "q4Uiq4UiUi\0"
  "svbdep_n_u64\0"
  "q2UWiq2UWiUWi\0"
  "svbdep_n_u8\0"
  "q16Ucq16UcUc\0"
  "svbdep_u16\0"
  "q8Usq8Usq8Us\0"
  "svbdep_u32\0"
  "svbdep_u64\0"
  "svbdep_u8\0"
  "svbext_n_u16\0"
  "svbext_n_u32\0"
  "svbext_n_u64\0"
  "svbext_n_u8\0"
  "svbext_u16\0"
  "svbext_u32\0"
  "svbext_u64\0"
  "svbext_u8\0"
  "svbfdot_f32\0"
  "q4fq4fq8yq8y\0"
  "(sve,bf16)|(sme,bf16)\0"
  "svbfdot_lane_f32\0"
  "q4fq4fq8yq8yIUWi\0"
  "svbfdot_n_f32\0"
  "q4fq4fq8yy\0"
  "svbfmlalb_f32\0"
  "svbfmlalb_lane_f32\0"
  "svbfmlalb_n_f32\0"
  "svbfmlalt_f32\0"
  "svbfmlalt_lane_f32\0"
  "svbfmlalt_n_f32\0"
  "svbfmlslb_f32\0"
  "(sve,(sve2p1|sme2))|(sme,(sve2p1|sme2))\0"
  "svbfmlslb_lane_f32\0"
  "svbfmlslt_f32\0"
  "svbfmlslt_lane_f32\0"
  "svbfmmla_f32\0"
  "sve,bf16\0"
  "svbgrp_n_u16\0"
  "svbgrp_n_u32\0"
  "svbgrp_n_u64\0"
  "svbgrp_n_u8\0"
  "svbgrp_u16\0"
  "svbgrp_u32\0"
  "svbgrp_u64\0"
  "svbgrp_u8\0"
  "svbic_b_z\0"
  "svbic_n_s16_m\0"
  "svbic_n_s16_x\0"
  "svbic_n_s16_z\0"
  "svbic_n_s32_m\0"
  "svbic_n_s32_x\0"
  "svbic_n_s32_z\0"
  "svbic_n_s64_m\0"
  "svbic_n_s64_x\0"
  "svbic_n_s64_z\0"
  "svbic_n_s8_m\0"
  "svbic_n_s8_x\0"
  "svbic_n_s8_z\0"
  "svbic_n_u16_m\0"
  "svbic_n_u16_x\0"
  "svbic_n_u16_z\0"
  "svbic_n_u32_m\0"
  "svbic_n_u32_x\0"
  "svbic_n_u32_z\0"
  "svbic_n_u64_m\0"
  "svbic_n_u64_x\0"
  "svbic_n_u64_z\0"
  "svbic_n_u8_m\0"
  "svbic_n_u8_x\0"
  "svbic_n_u8_z\0"
  "svbic_s16_m\0"
  "svbic_s16_x\0"
  "svbic_s16_z\0"
  "svbic_s32_m\0"
  "svbic_s32_x\0"
  "svbic_s32_z\0"
  "svbic_s64_m\0"
  "svbic_s64_x\0"
  "svbic_s64_z\0"
  "svbic_s8_m\0"
  "svbic_s8_x\0"
  "svbic_s8_z\0"
  "svbic_u16_m\0"
  "svbic_u16_x\0"
  "svbic_u16_z\0"
  "svbic_u32_m\0"
  "svbic_u32_x\0"
  "svbic_u32_z\0"
  "svbic_u64_m\0"
  "svbic_u64_x\0"
  "svbic_u64_z\0"
  "svbic_u8_m\0"
  "svbic_u8_x\0"
  "svbic_u8_z\0"
  "svbrka_b_m\0"
  "svbrka_b_z\0"
  "q16bq16bq16b\0"
  "svbrkb_b_m\0"
  "svbrkb_b_z\0"
  "svbrkn_b_z\0"
  "svbrkpa_b_z\0"
  "svbrkpb_b_z\0"
  "svbsl1n_n_s16\0"
  "svbsl1n_n_s32\0"
  "svbsl1n_n_s64\0"
  "svbsl1n_n_s8\0"
  "svbsl1n_n_u16\0"
  "svbsl1n_n_u32\0"
  "svbsl1n_n_u64\0"
  "svbsl1n_n_u8\0"
  "svbsl1n_s16\0"
  "svbsl1n_s32\0"
  "svbsl1n_s64\0"
  "svbsl1n_s8\0"
  "svbsl1n_u16\0"
  "svbsl1n_u32\0"
  "svbsl1n_u64\0"
  "svbsl1n_u8\0"
  "svbsl2n_n_s16\0"
  "svbsl2n_n_s32\0"
  "svbsl2n_n_s64\0"
  "svbsl2n_n_s8\0"
  "svbsl2n_n_u16\0"
  "svbsl2n_n_u32\0"
  "svbsl2n_n_u64\0"
  "svbsl2n_n_u8\0"
  "svbsl2n_s16\0"
  "svbsl2n_s32\0"
  "svbsl2n_s64\0"
  "svbsl2n_s8\0"
  "svbsl2n_u16\0"
  "svbsl2n_u32\0"
  "svbsl2n_u64\0"
  "svbsl2n_u8\0"
  "svbsl_n_s16\0"
  "svbsl_n_s32\0"
  "svbsl_n_s64\0"
  "svbsl_n_s8\0"
  "svbsl_n_u16\0"
  "svbsl_n_u32\0"
  "svbsl_n_u64\0"
  "svbsl_n_u8\0"
  "svbsl_s16\0"
  "svbsl_s32\0"
  "svbsl_s64\0"
  "svbsl_s8\0"
  "svbsl_u16\0"
  "svbsl_u32\0"
  "svbsl_u64\0"
  "svbsl_u8\0"
  "svcadd_f16_m\0"
  "q8hq16bq8hq8hIUWi\0"
  "svcadd_f16_x\0"
  "svcadd_f16_z\0"
  "svcadd_f32_m\0"
  "q4fq16bq4fq4fIUWi\0"
  "svcadd_f32_x\0"
  "svcadd_f32_z\0"
  "svcadd_f64_m\0"
  "q2dq16bq2dq2dIUWi\0"
  "svcadd_f64_x\0"
  "svcadd_f64_z\0"
  "svcadd_s16\0"
  "q8sq8sq8sIUWi\0"
  "svcadd_s32\0"
  "q4iq4iq4iIUWi\0"
  "svcadd_s64\0"
  "q2Wiq2Wiq2WiIUWi\0"
  "svcadd_s8\0"
  "q16Scq16Scq16ScIUWi\0"
  "svcadd_u16\0"
  "q8Usq8Usq8UsIUWi\0"
  "svcadd_u32\0"
  "q4Uiq4Uiq4UiIUWi\0"
  "svcadd_u64\0"
  "q2UWiq2UWiq2UWiIUWi\0"
  "svcadd_u8\0"
  "q16Ucq16Ucq16UcIUWi\0"
  "svcdot_lane_s32\0"
  "q4iq4iq16Scq16ScIUWiIUWi\0"
  "svcdot_lane_s64\0"
  "q2Wiq2Wiq8sq8sIUWiIUWi\0"
  "svcdot_s32\0"
  "q4iq4iq16Scq16ScIUWi\0"
  "svcdot_s64\0"
  "q2Wiq2Wiq8sq8sIUWi\0"
  "svclamp_bf16\0"
  "q8yq8yq8yq8y\0"
  "svclamp_f16\0"
  "q8hq8hq8hq8h\0"
  "svclamp_f32\0"
  "q4fq4fq4fq4f\0"
  "svclamp_f64\0"
  "q2dq2dq2dq2d\0"
  "svclamp_s16\0"
  "(sve,(sve2p1|sme))|sme\0"
  "svclamp_s32\0"
  "svclamp_s64\0"
  "svclamp_s8\0"
  "svclamp_single_bf16_x2\0"
  "q16yq16yq8yq8y\0"
  "sme,sme2,sve-b16b16\0"
  "svclamp_single_bf16_x4\0"
  "q32yq32yq8yq8y\0"
  "svclamp_single_f16_x2\0"
  "q16hq16hq8hq8h\0"
  "svclamp_single_f16_x4\0"
  "q32hq32hq8hq8h\0"
  "svclamp_single_f32_x2\0"
  "q8fq8fq4fq4f\0"
  "svclamp_single_f32_x4\0"
  "q16fq16fq4fq4f\0"
  "svclamp_single_f64_x2\0"
  "q4dq4dq2dq2d\0"
  "svclamp_single_f64_x4\0"
  "q8dq8dq2dq2d\0"
  "svclamp_single_s16_x2\0"
  "q16sq16sq8sq8s\0"
  "svclamp_single_s16_x4\0"
  "q32sq32sq8sq8s\0"
  "svclamp_single_s32_x2\0"
  "q8iq8iq4iq4i\0"
  "svclamp_single_s32_x4\0"
  "q16iq16iq4iq4i\0"
  "svclamp_single_s64_x2\0"
  "q4Wiq4Wiq2Wiq2Wi\0"
  "svclamp_single_s64_x4\0"
  "q8Wiq8Wiq2Wiq2Wi\0"
  "svclamp_single_s8_x2\0"
  "q32Scq32Scq16Scq16Sc\0"
  "svclamp_single_s8_x4\0"
  "q64Scq64Scq16Scq16Sc\0"
  "svclamp_single_u16_x2\0"
  "q16Usq16Usq8Usq8Us\0"
  "svclamp_single_u16_x4\0"
  "q32Usq32Usq8Usq8Us\0"
  "svclamp_single_u32_x2\0"
  "q8Uiq8Uiq4Uiq4Ui\0"
  "svclamp_single_u32_x4\0"
  "q16Uiq16Uiq4Uiq4Ui\0"
  "svclamp_single_u64_x2\0"
  "q4UWiq4UWiq2UWiq2UWi\0"
  "svclamp_single_u64_x4\0"
  "q8UWiq8UWiq2UWiq2UWi\0"
  "svclamp_single_u8_x2\0"
  "q32Ucq32Ucq16Ucq16Uc\0"
  "svclamp_single_u8_x4\0"
  "q64Ucq64Ucq16Ucq16Uc\0"
  "svclamp_u16\0"
  "svclamp_u32\0"
  "svclamp_u64\0"
  "svclamp_u8\0"
  "svclasta_bf16\0"
  "svclasta_f16\0"
  "svclasta_f32\0"
  "svclasta_f64\0"
  "svclasta_n_bf16\0"
  "yq16byq8y\0"
  "svclasta_n_f16\0"
  "svclasta_n_f32\0"
  "svclasta_n_f64\0"
  "svclasta_n_s16\0"
  "sq16bsq8s\0"
  "svclasta_n_s32\0"
  "iq16biq4i\0"
  "svclasta_n_s64\0"
  "Wiq16bWiq2Wi\0"
  "svclasta_n_s8\0"
  "Scq16bScq16Sc\0"
  "svclasta_n_u16\0"
  "Usq16bUsq8Us\0"
  "svclasta_n_u32\0"
  "Uiq16bUiq4Ui\0"
  "svclasta_n_u64\0"
  "UWiq16bUWiq2UWi\0"
  "svclasta_n_u8\0"
  "Ucq16bUcq16Uc\0"
  "svclasta_s16\0"
  "svclasta_s32\0"
  "svclasta_s64\0"
  "svclasta_s8\0"
  "svclasta_u16\0"
  "svclasta_u32\0"
  "svclasta_u64\0"
  "svclasta_u8\0"
  "svclastb_bf16\0"
  "svclastb_f16\0"
  "svclastb_f32\0"
  "svclastb_f64\0"
  "svclastb_n_bf16\0"
  "svclastb_n_f16\0"
  "svclastb_n_f32\0"
  "svclastb_n_f64\0"
  "svclastb_n_s16\0"
  "svclastb_n_s32\0"
  "svclastb_n_s64\0"
  "svclastb_n_s8\0"
  "svclastb_n_u16\0"
  "svclastb_n_u32\0"
  "svclastb_n_u64\0"
  "svclastb_n_u8\0"
  "svclastb_s16\0"
  "svclastb_s32\0"
  "svclastb_s64\0"
  "svclastb_s8\0"
  "svclastb_u16\0"
  "svclastb_u32\0"
  "svclastb_u64\0"
  "svclastb_u8\0"
  "svcls_s16_m\0"
  "q8Usq8Usq16bq8s\0"
  "svcls_s16_x\0"
  "q8Usq16bq8s\0"
  "svcls_s16_z\0"
  "svcls_s32_m\0"
  "q4Uiq4Uiq16bq4i\0"
  "svcls_s32_x\0"
  "q4Uiq16bq4i\0"
  "svcls_s32_z\0"
  "svcls_s64_m\0"
  "q2UWiq2UWiq16bq2Wi\0"
  "svcls_s64_x\0"
  "q2UWiq16bq2Wi\0"
  "svcls_s64_z\0"
  "svcls_s8_m\0"
  "q16Ucq16Ucq16bq16Sc\0"
  "svcls_s8_x\0"
  "q16Ucq16bq16Sc\0"
  "svcls_s8_z\0"
  "svclz_s16_m\0"
  "svclz_s16_x\0"
  "svclz_s16_z\0"
  "svclz_s32_m\0"
  "svclz_s32_x\0"
  "svclz_s32_z\0"
  "svclz_s64_m\0"
  "svclz_s64_x\0"
  "svclz_s64_z\0"
  "svclz_s8_m\0"
  "svclz_s8_x\0"
  "svclz_s8_z\0"
  "svclz_u16_m\0"
  "q8Usq8Usq16bq8Us\0"
  "svclz_u16_x\0"
  "q8Usq16bq8Us\0"
  "svclz_u16_z\0"
  "svclz_u32_m\0"
  "q4Uiq4Uiq16bq4Ui\0"
  "svclz_u32_x\0"
  "q4Uiq16bq4Ui\0"
  "svclz_u32_z\0"
  "svclz_u64_m\0"
  "q2UWiq2UWiq16bq2UWi\0"
  "svclz_u64_x\0"
  "q2UWiq16bq2UWi\0"
  "svclz_u64_z\0"
  "svclz_u8_m\0"
  "q16Ucq16Ucq16bq16Uc\0"
  "svclz_u8_x\0"
  "q16Ucq16bq16Uc\0"
  "svclz_u8_z\0"
  "svcmla_f16_m\0"
  "q8hq16bq8hq8hq8hIUWi\0"
  "svcmla_f16_x\0"
  "svcmla_f16_z\0"
  "svcmla_f32_m\0"
  "q4fq16bq4fq4fq4fIUWi\0"
  "svcmla_f32_x\0"
  "svcmla_f32_z\0"
  "svcmla_f64_m\0"
  "q2dq16bq2dq2dq2dIUWi\0"
  "svcmla_f64_x\0"
  "svcmla_f64_z\0"
  "svcmla_lane_f16\0"
  "q8hq8hq8hq8hIUWiIUWi\0"
  "svcmla_lane_f32\0"
  "q4fq4fq4fq4fIUWiIUWi\0"
  "svcmla_lane_s16\0"
  "q8sq8sq8sq8sIUWiIUWi\0"
  "svcmla_lane_s32\0"
  "q4iq4iq4iq4iIUWiIUWi\0"
  "svcmla_lane_u16\0"
  "q8Usq8Usq8Usq8UsIUWiIUWi\0"
  "svcmla_lane_u32\0"
  "q4Uiq4Uiq4Uiq4UiIUWiIUWi\0"
  "svcmla_s16\0"
  "q8sq8sq8sq8sIUWi\0"
  "svcmla_s32\0"
  "q4iq4iq4iq4iIUWi\0"
  "svcmla_s64\0"
  "q2Wiq2Wiq2Wiq2WiIUWi\0"
  "svcmla_s8\0"
  "q16Scq16Scq16Scq16ScIUWi\0"
  "svcmla_u16\0"
  "q8Usq8Usq8Usq8UsIUWi\0"
  "svcmla_u32\0"
  "q4Uiq4Uiq4Uiq4UiIUWi\0"
  "svcmla_u64\0"
  "q2UWiq2UWiq2UWiq2UWiIUWi\0"
  "svcmla_u8\0"
  "q16Ucq16Ucq16Ucq16UcIUWi\0"
  "svcmpeq_f16\0"
  "svcmpeq_f32\0"
  "svcmpeq_f64\0"
  "svcmpeq_n_f16\0"
  "svcmpeq_n_f32\0"
  "svcmpeq_n_f64\0"
  "svcmpeq_n_s16\0"
  "q16bq16bq8ss\0"
  "svcmpeq_n_s32\0"
  "q16bq16bq4ii\0"
  "svcmpeq_n_s64\0"
  "q16bq16bq2WiWi\0"
  "svcmpeq_n_s8\0"
  "q16bq16bq16ScSc\0"
  "svcmpeq_n_u16\0"
  "q16bq16bq8UsUs\0"
  "svcmpeq_n_u32\0"
  "q16bq16bq4UiUi\0"
  "svcmpeq_n_u64\0"
  "q16bq16bq2UWiUWi\0"
  "svcmpeq_n_u8\0"
  "q16bq16bq16UcUc\0"
  "svcmpeq_s16\0"
  "q16bq16bq8sq8s\0"
  "svcmpeq_s32\0"
  "q16bq16bq4iq4i\0"
  "svcmpeq_s64\0"
  "q16bq16bq2Wiq2Wi\0"
  "svcmpeq_s8\0"
  "q16bq16bq16Scq16Sc\0"
  "svcmpeq_u16\0"
  "q16bq16bq8Usq8Us\0"
  "svcmpeq_u32\0"
  "q16bq16bq4Uiq4Ui\0"
  "svcmpeq_u64\0"
  "q16bq16bq2UWiq2UWi\0"
  "svcmpeq_u8\0"
  "q16bq16bq16Ucq16Uc\0"
  "svcmpeq_wide_n_s16\0"
  "q16bq16bq8sWi\0"
  "svcmpeq_wide_n_s32\0"
  "q16bq16bq4iWi\0"
  "svcmpeq_wide_n_s8\0"
  "q16bq16bq16ScWi\0"
  "svcmpeq_wide_s16\0"
  "q16bq16bq8sq2Wi\0"
  "svcmpeq_wide_s32\0"
  "q16bq16bq4iq2Wi\0"
  "svcmpeq_wide_s8\0"
  "q16bq16bq16Scq2Wi\0"
  "svcmpge_f16\0"
  "svcmpge_f32\0"
  "svcmpge_f64\0"
  "svcmpge_n_f16\0"
  "svcmpge_n_f32\0"
  "svcmpge_n_f64\0"
  "svcmpge_n_s16\0"
  "svcmpge_n_s32\0"
  "svcmpge_n_s64\0"
  "svcmpge_n_s8\0"
  "svcmpge_n_u16\0"
  "svcmpge_n_u32\0"
  "svcmpge_n_u64\0"
  "svcmpge_n_u8\0"
  "svcmpge_s16\0"
  "svcmpge_s32\0"
  "svcmpge_s64\0"
  "svcmpge_s8\0"
  "svcmpge_u16\0"
  "svcmpge_u32\0"
  "svcmpge_u64\0"
  "svcmpge_u8\0"
  "svcmpge_wide_n_s16\0"
  "svcmpge_wide_n_s32\0"
  "svcmpge_wide_n_s8\0"
  "svcmpge_wide_n_u16\0"
  "q16bq16bq8UsUWi\0"
  "svcmpge_wide_n_u32\0"
  "q16bq16bq4UiUWi\0"
  "svcmpge_wide_n_u8\0"
  "q16bq16bq16UcUWi\0"
  "svcmpge_wide_s16\0"
  "svcmpge_wide_s32\0"
  "svcmpge_wide_s8\0"
  "svcmpge_wide_u16\0"
  "q16bq16bq8Usq2UWi\0"
  "svcmpge_wide_u32\0"
  "q16bq16bq4Uiq2UWi\0"
  "svcmpge_wide_u8\0"
  "q16bq16bq16Ucq2UWi\0"
  "svcmpgt_f16\0"
  "svcmpgt_f32\0"
  "svcmpgt_f64\0"
  "svcmpgt_n_f16\0"
  "svcmpgt_n_f32\0"
  "svcmpgt_n_f64\0"
  "svcmpgt_n_s16\0"
  "svcmpgt_n_s32\0"
  "svcmpgt_n_s64\0"
  "svcmpgt_n_s8\0"
  "svcmpgt_n_u16\0"
  "svcmpgt_n_u32\0"
  "svcmpgt_n_u64\0"
  "svcmpgt_n_u8\0"
  "svcmpgt_s16\0"
  "svcmpgt_s32\0"
  "svcmpgt_s64\0"
  "svcmpgt_s8\0"
  "svcmpgt_u16\0"
  "svcmpgt_u32\0"
  "svcmpgt_u64\0"
  "svcmpgt_u8\0"
  "svcmpgt_wide_n_s16\0"
  "svcmpgt_wide_n_s32\0"
  "svcmpgt_wide_n_s8\0"
  "svcmpgt_wide_n_u16\0"
  "svcmpgt_wide_n_u32\0"
  "svcmpgt_wide_n_u8\0"
  "svcmpgt_wide_s16\0"
  "svcmpgt_wide_s32\0"
  "svcmpgt_wide_s8\0"
  "svcmpgt_wide_u16\0"
  "svcmpgt_wide_u32\0"
  "svcmpgt_wide_u8\0"
  "svcmple_f16\0"
  "svcmple_f32\0"
  "svcmple_f64\0"
  "svcmple_n_f16\0"
  "svcmple_n_f32\0"
  "svcmple_n_f64\0"
  "svcmple_n_s16\0"
  "svcmple_n_s32\0"
  "svcmple_n_s64\0"
  "svcmple_n_s8\0"
  "svcmple_n_u16\0"
  "svcmple_n_u32\0"
  "svcmple_n_u64\0"
  "svcmple_n_u8\0"
  "svcmple_s16\0"
  "svcmple_s32\0"
  "svcmple_s64\0"
  "svcmple_s8\0"
  "svcmple_u16\0"
  "svcmple_u32\0"
  "svcmple_u64\0"
  "svcmple_u8\0"
  "svcmple_wide_n_s16\0"
  "svcmple_wide_n_s32\0"
  "svcmple_wide_n_s8\0"
  "svcmple_wide_n_u16\0"
  "svcmple_wide_n_u32\0"
  "svcmple_wide_n_u8\0"
  "svcmple_wide_s16\0"
  "svcmple_wide_s32\0"
  "svcmple_wide_s8\0"
  "svcmple_wide_u16\0"
  "svcmple_wide_u32\0"
  "svcmple_wide_u8\0"
  "svcmplt_f16\0"
  "svcmplt_f32\0"
  "svcmplt_f64\0"
  "svcmplt_n_f16\0"
  "svcmplt_n_f32\0"
  "svcmplt_n_f64\0"
  "svcmplt_n_s16\0"
  "svcmplt_n_s32\0"
  "svcmplt_n_s64\0"
  "svcmplt_n_s8\0"
  "svcmplt_n_u16\0"
  "svcmplt_n_u32\0"
  "svcmplt_n_u64\0"
  "svcmplt_n_u8\0"
  "svcmplt_s16\0"
  "svcmplt_s32\0"
  "svcmplt_s64\0"
  "svcmplt_s8\0"
  "svcmplt_u16\0"
  "svcmplt_u32\0"
  "svcmplt_u64\0"
  "svcmplt_u8\0"
  "svcmplt_wide_n_s16\0"
  "svcmplt_wide_n_s32\0"
  "svcmplt_wide_n_s8\0"
  "svcmplt_wide_n_u16\0"
  "svcmplt_wide_n_u32\0"
  "svcmplt_wide_n_u8\0"
  "svcmplt_wide_s16\0"
  "svcmplt_wide_s32\0"
  "svcmplt_wide_s8\0"
  "svcmplt_wide_u16\0"
  "svcmplt_wide_u32\0"
  "svcmplt_wide_u8\0"
  "svcmpne_f16\0"
  "svcmpne_f32\0"
  "svcmpne_f64\0"
  "svcmpne_n_f16\0"
  "svcmpne_n_f32\0"
  "svcmpne_n_f64\0"
  "svcmpne_n_s16\0"
  "svcmpne_n_s32\0"
  "svcmpne_n_s64\0"
  "svcmpne_n_s8\0"
  "svcmpne_n_u16\0"
  "svcmpne_n_u32\0"
  "svcmpne_n_u64\0"
  "svcmpne_n_u8\0"
  "svcmpne_s16\0"
  "svcmpne_s32\0"
  "svcmpne_s64\0"
  "svcmpne_s8\0"
  "svcmpne_u16\0"
  "svcmpne_u32\0"
  "svcmpne_u64\0"
  "svcmpne_u8\0"
  "svcmpne_wide_n_s16\0"
  "svcmpne_wide_n_s32\0"
  "svcmpne_wide_n_s8\0"
  "svcmpne_wide_s16\0"
  "svcmpne_wide_s32\0"
  "svcmpne_wide_s8\0"
  "svcmpuo_f16\0"
  "svcmpuo_f32\0"
  "svcmpuo_f64\0"
  "svcmpuo_n_f16\0"
  "svcmpuo_n_f32\0"
  "svcmpuo_n_f64\0"
  "svcnot_s16_m\0"
  "svcnot_s16_x\0"
  "svcnot_s16_z\0"
  "svcnot_s32_m\0"
  "svcnot_s32_x\0"
  "svcnot_s32_z\0"
  "svcnot_s64_m\0"
  "svcnot_s64_x\0"
  "svcnot_s64_z\0"
  "svcnot_s8_m\0"
  "svcnot_s8_x\0"
  "svcnot_s8_z\0"
  "svcnot_u16_m\0"
  "svcnot_u16_x\0"
  "svcnot_u16_z\0"
  "svcnot_u32_m\0"
  "svcnot_u32_x\0"
  "svcnot_u32_z\0"
  "svcnot_u64_m\0"
  "svcnot_u64_x\0"
  "svcnot_u64_z\0"
  "svcnot_u8_m\0"
  "svcnot_u8_x\0"
  "svcnot_u8_z\0"
  "svcnt_bf16_m\0"
  "q8Usq8Usq16bq8y\0"
  "svcnt_bf16_x\0"
  "q8Usq16bq8y\0"
  "svcnt_bf16_z\0"
  "svcnt_f16_m\0"
  "q8Usq8Usq16bq8h\0"
  "svcnt_f16_x\0"
  "q8Usq16bq8h\0"
  "svcnt_f16_z\0"
  "svcnt_f32_m\0"
  "q4Uiq4Uiq16bq4f\0"
  "svcnt_f32_x\0"
  "q4Uiq16bq4f\0"
  "svcnt_f32_z\0"
  "svcnt_f64_m\0"
  "q2UWiq2UWiq16bq2d\0"
  "svcnt_f64_x\0"
  "q2UWiq16bq2d\0"
  "svcnt_f64_z\0"
  "svcnt_s16_m\0"
  "svcnt_s16_x\0"
  "svcnt_s16_z\0"
  "svcnt_s32_m\0"
  "svcnt_s32_x\0"
  "svcnt_s32_z\0"
  "svcnt_s64_m\0"
  "svcnt_s64_x\0"
  "svcnt_s64_z\0"
  "svcnt_s8_m\0"
  "svcnt_s8_x\0"
  "svcnt_s8_z\0"
  "svcnt_u16_m\0"
  "svcnt_u16_x\0"
  "svcnt_u16_z\0"
  "svcnt_u32_m\0"
  "svcnt_u32_x\0"
  "svcnt_u32_z\0"
  "svcnt_u64_m\0"
  "svcnt_u64_x\0"
  "svcnt_u64_z\0"
  "svcnt_u8_m\0"
  "svcnt_u8_x\0"
  "svcnt_u8_z\0"
  "svcntb\0"
  "UWiv\0"
  "svcntb_pat\0"
  "UWiIi\0"
  "svcntd\0"
  "svcntd_pat\0"
  "svcnth\0"
  "svcnth_pat\0"
  "svcntp_b16\0"
  "UWiq16bq16b\0"
  "svcntp_b32\0"
  "svcntp_b64\0"
  "svcntp_b8\0"
  "svcntp_c16\0"
  "UWiQaIUWi\0"
  "(sve,sve2p1)|(sme,(sve2p1|sme2))\0"
  "svcntp_c32\0"
  "svcntp_c64\0"
  "svcntp_c8\0"
  "svcntw\0"
  "svcntw_pat\0"
  "svcompact_bf16\0"
  "q8yq16bq8y\0"
  "(sve,(sve2p2|sme2p2))|(sme,sme2p2)\0"
  "svcompact_f16\0"
  "svcompact_f32\0"
  "sve|(sme,sme2p2)\0"
  "svcompact_f64\0"
  "svcompact_mf8\0"
  "q16mq16bq16m\0"
  "svcompact_s16\0"
  "svcompact_s32\0"
  "svcompact_s64\0"
  "svcompact_s8\0"
  "svcompact_u16\0"
  "svcompact_u32\0"
  "svcompact_u64\0"
  "svcompact_u8\0"
  "svcreate2_b\0"
  "q32bq16bq16b\0"
  "svcreate2_bf16\0"
  "q16yq8yq8y\0"
  "svcreate2_f16\0"
  "q16hq8hq8h\0"
  "svcreate2_f32\0"
  "q8fq4fq4f\0"
  "svcreate2_f64\0"
  "q4dq2dq2d\0"
  "svcreate2_mf8\0"
  "q32mq16mq16m\0"
  "svcreate2_s16\0"
  "q16sq8sq8s\0"
  "svcreate2_s32\0"
  "q8iq4iq4i\0"
  "svcreate2_s64\0"
  "q4Wiq2Wiq2Wi\0"
  "svcreate2_s8\0"
  "q32Scq16Scq16Sc\0"
  "svcreate2_u16\0"
  "q16Usq8Usq8Us\0"
  "svcreate2_u32\0"
  "q8Uiq4Uiq4Ui\0"
  "svcreate2_u64\0"
  "q4UWiq2UWiq2UWi\0"
  "svcreate2_u8\0"
  "q32Ucq16Ucq16Uc\0"
  "svcreate3_bf16\0"
  "q24yq8yq8yq8y\0"
  "svcreate3_f16\0"
  "q24hq8hq8hq8h\0"
  "svcreate3_f32\0"
  "q12fq4fq4fq4f\0"
  "svcreate3_f64\0"
  "q6dq2dq2dq2d\0"
  "svcreate3_mf8\0"
  "q48mq16mq16mq16m\0"
  "svcreate3_s16\0"
  "q24sq8sq8sq8s\0"
  "svcreate3_s32\0"
  "q12iq4iq4iq4i\0"
  "svcreate3_s64\0"
  "q6Wiq2Wiq2Wiq2Wi\0"
  "svcreate3_s8\0"
  "q48Scq16Scq16Scq16Sc\0"
  "svcreate3_u16\0"
  "q24Usq8Usq8Usq8Us\0"
  "svcreate3_u32\0"
  "q12Uiq4Uiq4Uiq4Ui\0"
  "svcreate3_u64\0"
  "q6UWiq2UWiq2UWiq2UWi\0"
  "svcreate3_u8\0"
  "q48Ucq16Ucq16Ucq16Uc\0"
  "svcreate4_b\0"
  "q64bq16bq16bq16bq16b\0"
  "svcreate4_bf16\0"
  "q32yq8yq8yq8yq8y\0"
  "svcreate4_f16\0"
  "q32hq8hq8hq8hq8h\0"
  "svcreate4_f32\0"
  "q16fq4fq4fq4fq4f\0"
  "svcreate4_f64\0"
  "q8dq2dq2dq2dq2d\0"
  "svcreate4_mf8\0"
  "q64mq16mq16mq16mq16m\0"
  "svcreate4_s16\0"
  "q32sq8sq8sq8sq8s\0"
  "svcreate4_s32\0"
  "q16iq4iq4iq4iq4i\0"
  "svcreate4_s64\0"
  "q8Wiq2Wiq2Wiq2Wiq2Wi\0"
  "svcreate4_s8\0"
  "q64Scq16Scq16Scq16Scq16Sc\0"
  "svcreate4_u16\0"
  "q32Usq8Usq8Usq8Usq8Us\0"
  "svcreate4_u32\0"
  "q16Uiq4Uiq4Uiq4Uiq4Ui\0"
  "svcreate4_u64\0"
  "q8UWiq2UWiq2UWiq2UWiq2UWi\0"
  "svcreate4_u8\0"
  "q64Ucq16Ucq16Ucq16Ucq16Uc\0"
  "svcvt1_bf16_mf8_fpm\0"
  "q8yq16mUWi\0"
  "(sve,(sve2|sme2),fp8)|(sme,sme2,fp8)\0"
  "svcvt1_bf16_mf8_x2_fpm\0"
  "q16yq16mUWi\0"
  "sme,sme2,fp8\0"
  "svcvt1_f16_mf8_fpm\0"
  "q8hq16mUWi\0"
  "svcvt1_f16_mf8_x2_fpm\0"
  "q16hq16mUWi\0"
  "svcvt2_bf16_mf8_fpm\0"
  "svcvt2_bf16_mf8_x2_fpm\0"
  "svcvt2_f16_mf8_fpm\0"
  "svcvt2_f16_mf8_x2_fpm\0"
  "svcvt_bf16_f32_m\0"
  "q8yq8yq16bq4f\0"
  "svcvt_bf16_f32_x\0"
  "q8yq16bq4f\0"
  "svcvt_bf16_f32_x2\0"
  "q8yq8f\0"
  "svcvt_bf16_f32_z\0"
  "svcvt_f16_f32_m\0"
  "q8hq8hq16bq4f\0"
  "svcvt_f16_f32_x\0"
  "q8hq16bq4f\0"
  "svcvt_f16_f32_x2\0"
  "q8hq8f\0"
  "svcvt_f16_f32_z\0"
  "svcvt_f16_f64_m\0"
  "q8hq8hq16bq2d\0"
  "svcvt_f16_f64_x\0"
  "q8hq16bq2d\0"
  "svcvt_f16_f64_z\0"
  "svcvt_f16_s16_m\0"
  "q8hq8hq16bq8s\0"
  "svcvt_f16_s16_x\0"
  "q8hq16bq8s\0"
  "svcvt_f16_s16_z\0"
  "svcvt_f16_s32_m\0"
  "q8hq8hq16bq4i\0"
  "svcvt_f16_s32_x\0"
  "q8hq16bq4i\0"
  "svcvt_f16_s32_z\0"
  "svcvt_f16_s64_m\0"
  "q8hq8hq16bq2Wi\0"
  "svcvt_f16_s64_x\0"
  "q8hq16bq2Wi\0"
  "svcvt_f16_s64_z\0"
  "svcvt_f16_u16_m\0"
  "q8hq8hq16bq8Us\0"
  "svcvt_f16_u16_x\0"
  "q8hq16bq8Us\0"
  "svcvt_f16_u16_z\0"
  "svcvt_f16_u32_m\0"
  "q8hq8hq16bq4Ui\0"
  "svcvt_f16_u32_x\0"
  "q8hq16bq4Ui\0"
  "svcvt_f16_u32_z\0"
  "svcvt_f16_u64_m\0"
  "q8hq8hq16bq2UWi\0"
  "svcvt_f16_u64_x\0"
  "q8hq16bq2UWi\0"
  "svcvt_f16_u64_z\0"
  "svcvt_f32_f16_m\0"
  "q4fq4fq16bq8h\0"
  "svcvt_f32_f16_x\0"
  "q4fq16bq8h\0"
  "svcvt_f32_f16_x2\0"
  "q8fq8h\0"
  "sme,sme-f16f16\0"
  "svcvt_f32_f16_z\0"
  "svcvt_f32_f64_m\0"
  "q4fq4fq16bq2d\0"
  "svcvt_f32_f64_x\0"
  "q4fq16bq2d\0"
  "svcvt_f32_f64_z\0"
  "svcvt_f32_s32_m\0"
  "q4fq4fq16bq4i\0"
  "svcvt_f32_s32_x\0"
  "q4fq16bq4i\0"
  "svcvt_f32_s32_x2\0"
  "q8fq8i\0"
  "svcvt_f32_s32_x4\0"
  "q16fq16i\0"
  "svcvt_f32_s32_z\0"
  "svcvt_f32_s64_m\0"
  "q4fq4fq16bq2Wi\0"
  "svcvt_f32_s64_x\0"
  "q4fq16bq2Wi\0"
  "svcvt_f32_s64_z\0"
  "svcvt_f32_u32_m\0"
  "q4fq4fq16bq4Ui\0"
  "svcvt_f32_u32_x\0"
  "q4fq16bq4Ui\0"
  "svcvt_f32_u32_x2\0"
  "q8fq8Ui\0"
  "svcvt_f32_u32_x4\0"
  "q16fq16Ui\0"
  "svcvt_f32_u32_z\0"
  "svcvt_f32_u64_m\0"
  "q4fq4fq16bq2UWi\0"
  "svcvt_f32_u64_x\0"
  "q4fq16bq2UWi\0"
  "svcvt_f32_u64_z\0"
  "svcvt_f64_f16_m\0"
  "q2dq2dq16bq8h\0"
  "svcvt_f64_f16_x\0"
  "q2dq16bq8h\0"
  "svcvt_f64_f16_z\0"
  "svcvt_f64_f32_m\0"
  "q2dq2dq16bq4f\0"
  "svcvt_f64_f32_x\0"
  "q2dq16bq4f\0"
  "svcvt_f64_f32_z\0"
  "svcvt_f64_s32_m\0"
  "q2dq2dq16bq4i\0"
  "svcvt_f64_s32_x\0"
  "q2dq16bq4i\0"
  "svcvt_f64_s32_z\0"
  "svcvt_f64_s64_m\0"
  "q2dq2dq16bq2Wi\0"
  "svcvt_f64_s64_x\0"
  "q2dq16bq2Wi\0"
  "svcvt_f64_s64_z\0"
  "svcvt_f64_u32_m\0"
  "q2dq2dq16bq4Ui\0"
  "svcvt_f64_u32_x\0"
  "q2dq16bq4Ui\0"
  "svcvt_f64_u32_z\0"
  "svcvt_f64_u64_m\0"
  "q2dq2dq16bq2UWi\0"
  "svcvt_f64_u64_x\0"
  "q2dq16bq2UWi\0"
  "svcvt_f64_u64_z\0"
  "svcvt_mf8_bf16_x2_fpm\0"
  "q16mq16yUWi\0"
  "svcvt_mf8_f16_x2_fpm\0"
  "q16mq16hUWi\0"
  "svcvt_mf8_f32_x4_fpm\0"
  "q16mq16fUWi\0"
  "svcvt_s16_f16_m\0"
  "q8sq8sq16bq8h\0"
  "svcvt_s16_f16_x\0"
  "q8sq16bq8h\0"
  "svcvt_s16_f16_z\0"
  "svcvt_s32_f16_m\0"
  "q4iq4iq16bq8h\0"
  "svcvt_s32_f16_x\0"
  "q4iq16bq8h\0"
  "svcvt_s32_f16_z\0"
  "svcvt_s32_f32_m\0"
  "q4iq4iq16bq4f\0"
  "svcvt_s32_f32_x\0"
  "q4iq16bq4f\0"
  "svcvt_s32_f32_x2\0"
  "q8iq8f\0"
  "svcvt_s32_f32_x4\0"
  "q16iq16f\0"
  "svcvt_s32_f32_z\0"
  "svcvt_s32_f64_m\0"
  "q4iq4iq16bq2d\0"
  "svcvt_s32_f64_x\0"
  "q4iq16bq2d\0"
  "svcvt_s32_f64_z\0"
  "svcvt_s64_f16_m\0"
  "q2Wiq2Wiq16bq8h\0"
  "svcvt_s64_f16_x\0"
  "q2Wiq16bq8h\0"
  "svcvt_s64_f16_z\0"
  "svcvt_s64_f32_m\0"
  "q2Wiq2Wiq16bq4f\0"
  "svcvt_s64_f32_x\0"
  "q2Wiq16bq4f\0"
  "svcvt_s64_f32_z\0"
  "svcvt_s64_f64_m\0"
  "q2Wiq2Wiq16bq2d\0"
  "svcvt_s64_f64_x\0"
  "q2Wiq16bq2d\0"
  "svcvt_s64_f64_z\0"
  "svcvt_u16_f16_m\0"
  "svcvt_u16_f16_x\0"
  "svcvt_u16_f16_z\0"
  "svcvt_u32_f16_m\0"
  "q4Uiq4Uiq16bq8h\0"
  "svcvt_u32_f16_x\0"
  "q4Uiq16bq8h\0"
  "svcvt_u32_f16_z\0"
  "svcvt_u32_f32_m\0"
  "svcvt_u32_f32_x\0"
  "svcvt_u32_f32_x2\0"
  "q8Uiq8f\0"
  "svcvt_u32_f32_x4\0"
  "q16Uiq16f\0"
  "svcvt_u32_f32_z\0"
  "svcvt_u32_f64_m\0"
  "q4Uiq4Uiq16bq2d\0"
  "svcvt_u32_f64_x\0"
  "q4Uiq16bq2d\0"
  "svcvt_u32_f64_z\0"
  "svcvt_u64_f16_m\0"
  "q2UWiq2UWiq16bq8h\0"
  "svcvt_u64_f16_x\0"
  "q2UWiq16bq8h\0"
  "svcvt_u64_f16_z\0"
  "svcvt_u64_f32_m\0"
  "q2UWiq2UWiq16bq4f\0"
  "svcvt_u64_f32_x\0"
  "q2UWiq16bq4f\0"
  "svcvt_u64_f32_z\0"
  "svcvt_u64_f64_m\0"
  "svcvt_u64_f64_x\0"
  "svcvt_u64_f64_z\0"
  "svcvtl1_bf16_mf8_x2_fpm\0"
  "svcvtl1_f16_mf8_x2_fpm\0"
  "svcvtl2_bf16_mf8_x2_fpm\0"
  "svcvtl2_f16_mf8_x2_fpm\0"
  "svcvtl_f32_f16_x2\0"
  "svcvtlt1_bf16_mf8_fpm\0"
  "svcvtlt1_f16_mf8_fpm\0"
  "svcvtlt2_bf16_mf8_fpm\0"
  "svcvtlt2_f16_mf8_fpm\0"
  "svcvtlt_f32_f16_m\0"
  "svcvtlt_f32_f16_x\0"
  "svcvtlt_f32_f16_z\0"
  "(sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2))\0"
  "svcvtlt_f64_f32_m\0"
  "svcvtlt_f64_f32_x\0"
  "svcvtlt_f64_f32_z\0"
  "svcvtn_bf16_f32_x2\0"
  "svcvtn_f16_f32_x2\0"
  "svcvtn_mf8_bf16_x2_fpm\0"
  "svcvtn_mf8_f16_x2_fpm\0"
  "svcvtn_mf8_f32_x4_fpm\0"
  "svcvtnb_mf8_f32_x2_fpm\0"
  "q16mq8fUWi\0"
  "svcvtnt_bf16_f32_m\0"
  "svcvtnt_bf16_f32_z\0"
  "svcvtnt_f16_f32_m\0"
  "svcvtnt_f16_f32_z\0"
  "svcvtnt_f32_f64_m\0"
  "svcvtnt_f32_f64_z\0"
  "svcvtnt_mf8_f32_x2_fpm\0"
  "q16mq16mq8fUWi\0"
  "svcvtx_f32_f64_m\0"
  "svcvtx_f32_f64_x\0"
  "svcvtx_f32_f64_z\0"
  "svcvtxnt_f32_f64_m\0"
  "svcvtxnt_f32_f64_z\0"
  "svdiv_f16_m\0"
  "svdiv_f16_x\0"
  "svdiv_f16_z\0"
  "svdiv_f32_m\0"
  "svdiv_f32_x\0"
  "svdiv_f32_z\0"
  "svdiv_f64_m\0"
  "svdiv_f64_x\0"
  "svdiv_f64_z\0"
  "svdiv_n_f16_m\0"
  "svdiv_n_f16_x\0"
  "svdiv_n_f16_z\0"
  "svdiv_n_f32_m\0"
  "svdiv_n_f32_x\0"
  "svdiv_n_f32_z\0"
  "svdiv_n_f64_m\0"
  "svdiv_n_f64_x\0"
  "svdiv_n_f64_z\0"
  "svdiv_n_s32_m\0"
  "svdiv_n_s32_x\0"
  "svdiv_n_s32_z\0"
  "svdiv_n_s64_m\0"
  "svdiv_n_s64_x\0"
  "svdiv_n_s64_z\0"
  "svdiv_n_u32_m\0"
  "svdiv_n_u32_x\0"
  "svdiv_n_u32_z\0"
  "svdiv_n_u64_m\0"
  "svdiv_n_u64_x\0"
  "svdiv_n_u64_z\0"
  "svdiv_s32_m\0"
  "svdiv_s32_x\0"
  "svdiv_s32_z\0"
  "svdiv_s64_m\0"
  "svdiv_s64_x\0"
  "svdiv_s64_z\0"
  "svdiv_u32_m\0"
  "svdiv_u32_x\0"
  "svdiv_u32_z\0"
  "svdiv_u64_m\0"
  "svdiv_u64_x\0"
  "svdiv_u64_z\0"
  "svdivr_f16_m\0"
  "svdivr_f16_x\0"
  "svdivr_f16_z\0"
  "svdivr_f32_m\0"
  "svdivr_f32_x\0"
  "svdivr_f32_z\0"
  "svdivr_f64_m\0"
  "svdivr_f64_x\0"
  "svdivr_f64_z\0"
  "svdivr_n_f16_m\0"
  "svdivr_n_f16_x\0"
  "svdivr_n_f16_z\0"
  "svdivr_n_f32_m\0"
  "svdivr_n_f32_x\0"
  "svdivr_n_f32_z\0"
  "svdivr_n_f64_m\0"
  "svdivr_n_f64_x\0"
  "svdivr_n_f64_z\0"
  "svdivr_n_s32_m\0"
  "svdivr_n_s32_x\0"
  "svdivr_n_s32_z\0"
  "svdivr_n_s64_m\0"
  "svdivr_n_s64_x\0"
  "svdivr_n_s64_z\0"
  "svdivr_n_u32_m\0"
  "svdivr_n_u32_x\0"
  "svdivr_n_u32_z\0"
  "svdivr_n_u64_m\0"
  "svdivr_n_u64_x\0"
  "svdivr_n_u64_z\0"
  "svdivr_s32_m\0"
  "svdivr_s32_x\0"
  "svdivr_s32_z\0"
  "svdivr_s64_m\0"
  "svdivr_s64_x\0"
  "svdivr_s64_z\0"
  "svdivr_u32_m\0"
  "svdivr_u32_x\0"
  "svdivr_u32_z\0"
  "svdivr_u64_m\0"
  "svdivr_u64_x\0"
  "svdivr_u64_z\0"
  "svdot_f16_mf8_fpm\0"
  "q8hq8hq16mq16mUWi\0"
  "(sve,sve2,fp8dot2)|(sme,ssve-fp8dot2)\0"
  "svdot_f32_f16\0"
  "q4fq4fq8hq8h\0"
  "svdot_f32_mf8_fpm\0"
  "q4fq4fq16mq16mUWi\0"
  "(sve,sve2,fp8dot4)|(sme,ssve-fp8dot4)\0"
  "svdot_lane_f16_mf8_fpm\0"
  "q8hq8hq16mq16mIUWiUWi\0"
  "svdot_lane_f32_f16\0"
  "q4fq4fq8hq8hIUWi\0"
  "svdot_lane_f32_mf8_fpm\0"
  "q4fq4fq16mq16mIUWiUWi\0"
  "svdot_lane_s32\0"
  "svdot_lane_s32_s16\0"
  "q4iq4iq8sq8sIUWi\0"
  "svdot_lane_s64\0"
  "svdot_lane_u32\0"
  "q4Uiq4Uiq16Ucq16UcIUWi\0"
  "svdot_lane_u32_u16\0"
  "q4Uiq4Uiq8Usq8UsIUWi\0"
  "svdot_lane_u64\0"
  "q2UWiq2UWiq8Usq8UsIUWi\0"
  "svdot_n_f16_mf8_fpm\0"
  "q8hq8hq16mmUWi\0"
  "svdot_n_f32_mf8_fpm\0"
  "q4fq4fq16mmUWi\0"
  "svdot_n_s32\0"
  "q4iq4iq16ScSc\0"
  "svdot_n_s64\0"
  "q2Wiq2Wiq8ss\0"
  "svdot_n_u32\0"
  "q4Uiq4Uiq16UcUc\0"
  "svdot_n_u64\0"
  "q2UWiq2UWiq8UsUs\0"
  "svdot_s32\0"
  "q4iq4iq16Scq16Sc\0"
  "svdot_s32_s16\0"
  "svdot_s64\0"
  "q2Wiq2Wiq8sq8s\0"
  "svdot_u32\0"
  "q4Uiq4Uiq16Ucq16Uc\0"
  "svdot_u32_u16\0"
  "svdot_u64\0"
  "q2UWiq2UWiq8Usq8Us\0"
  "svdup_lane_bf16\0"
  "q8yq8yUs\0"
  "svdup_lane_f16\0"
  "q8hq8hUs\0"
  "svdup_lane_f32\0"
  "q4fq4fUi\0"
  "svdup_lane_f64\0"
  "q2dq2dUWi\0"
  "svdup_lane_s16\0"
  "q8sq8sUs\0"
  "svdup_lane_s32\0"
  "q4iq4iUi\0"
  "svdup_lane_s64\0"
  "q2Wiq2WiUWi\0"
  "svdup_lane_s8\0"
  "q16Scq16ScUc\0"
  "svdup_lane_u16\0"
  "svdup_lane_u32\0"
  "svdup_lane_u64\0"
  "svdup_lane_u8\0"
  "svdup_laneq_bf16\0"
  "q8yq8yIUWi\0"
  "svdup_laneq_f16\0"
  "q8hq8hIUWi\0"
  "svdup_laneq_f32\0"
  "q4fq4fIUWi\0"
  "svdup_laneq_f64\0"
  "q2dq2dIUWi\0"
  "svdup_laneq_mf8\0"
  "q16mq16mIUWi\0"
  "svdup_laneq_s16\0"
  "q8sq8sIUWi\0"
  "svdup_laneq_s32\0"
  "q4iq4iIUWi\0"
  "svdup_laneq_s64\0"
  "q2Wiq2WiIUWi\0"
  "svdup_laneq_s8\0"
  "q16Scq16ScIUWi\0"
  "svdup_laneq_u16\0"
  "q8Usq8UsIUWi\0"
  "svdup_laneq_u32\0"
  "q4Uiq4UiIUWi\0"
  "svdup_laneq_u64\0"
  "q2UWiq2UWiIUWi\0"
  "svdup_laneq_u8\0"
  "q16Ucq16UcIUWi\0"
  "svdup_n_b16\0"
  "q16bb\0"
  "svdup_n_b32\0"
  "svdup_n_b64\0"
  "svdup_n_b8\0"
  "svdup_n_bf16\0"
  "q8yy\0"
  "svdup_n_bf16_m\0"
  "q8yq8yq16by\0"
  "svdup_n_bf16_x\0"
  "q8yq16by\0"
  "svdup_n_bf16_z\0"
  "svdup_n_f16\0"
  "q8hh\0"
  "svdup_n_f16_m\0"
  "q8hq8hq16bh\0"
  "svdup_n_f16_x\0"
  "q8hq16bh\0"
  "svdup_n_f16_z\0"
  "svdup_n_f32\0"
  "q4ff\0"
  "svdup_n_f32_m\0"
  "q4fq4fq16bf\0"
  "svdup_n_f32_x\0"
  "q4fq16bf\0"
  "svdup_n_f32_z\0"
  "svdup_n_f64\0"
  "q2dd\0"
  "svdup_n_f64_m\0"
  "q2dq2dq16bd\0"
  "svdup_n_f64_x\0"
  "q2dq16bd\0"
  "svdup_n_f64_z\0"
  "svdup_n_s16\0"
  "q8ss\0"
  "svdup_n_s16_m\0"
  "q8sq8sq16bs\0"
  "svdup_n_s16_x\0"
  "q8sq16bs\0"
  "svdup_n_s16_z\0"
  "svdup_n_s32\0"
  "q4ii\0"
  "svdup_n_s32_m\0"
  "q4iq4iq16bi\0"
  "svdup_n_s32_x\0"
  "q4iq16bi\0"
  "svdup_n_s32_z\0"
  "svdup_n_s64\0"
  "q2WiWi\0"
  "svdup_n_s64_m\0"
  "q2Wiq2Wiq16bWi\0"
  "svdup_n_s64_x\0"
  "q2Wiq16bWi\0"
  "svdup_n_s64_z\0"
  "svdup_n_s8\0"
  "q16ScSc\0"
  "svdup_n_s8_m\0"
  "q16Scq16Scq16bSc\0"
  "svdup_n_s8_x\0"
  "q16Scq16bSc\0"
  "svdup_n_s8_z\0"
  "svdup_n_u16\0"
  "q8UsUs\0"
  "svdup_n_u16_m\0"
  "q8Usq8Usq16bUs\0"
  "svdup_n_u16_x\0"
  "q8Usq16bUs\0"
  "svdup_n_u16_z\0"
  "svdup_n_u32\0"
  "q4UiUi\0"
  "svdup_n_u32_m\0"
  "q4Uiq4Uiq16bUi\0"
  "svdup_n_u32_x\0"
  "q4Uiq16bUi\0"
  "svdup_n_u32_z\0"
  "svdup_n_u64\0"
  "q2UWiUWi\0"
  "svdup_n_u64_m\0"
  "q2UWiq2UWiq16bUWi\0"
  "svdup_n_u64_x\0"
  "q2UWiq16bUWi\0"
  "svdup_n_u64_z\0"
  "svdup_n_u8\0"
  "q16UcUc\0"
  "svdup_n_u8_m\0"
  "q16Ucq16Ucq16bUc\0"
  "svdup_n_u8_x\0"
  "q16Ucq16bUc\0"
  "svdup_n_u8_z\0"
  "svdupq_lane_bf16\0"
  "q8yq8yUWi\0"
  "svdupq_lane_f16\0"
  "q8hq8hUWi\0"
  "svdupq_lane_f32\0"
  "q4fq4fUWi\0"
  "svdupq_lane_f64\0"
  "svdupq_lane_s16\0"
  "q8sq8sUWi\0"
  "svdupq_lane_s32\0"
  "q4iq4iUWi\0"
  "svdupq_lane_s64\0"
  "svdupq_lane_s8\0"
  "q16Scq16ScUWi\0"
  "svdupq_lane_u16\0"
  "q8Usq8UsUWi\0"
  "svdupq_lane_u32\0"
  "q4Uiq4UiUWi\0"
  "svdupq_lane_u64\0"
  "svdupq_lane_u8\0"
  "q16Ucq16UcUWi\0"
  "svdupq_n_b16\0"
  "q16bbbbbbbbb\0"
  "svdupq_n_b32\0"
  "q16bbbbb\0"
  "svdupq_n_b64\0"
  "q16bbb\0"
  "svdupq_n_b8\0"
  "q16bbbbbbbbbbbbbbbbb\0"
  "svdupq_n_bf16\0"
  "q8yyyyyyyyy\0"
  "svdupq_n_f16\0"
  "q8hhhhhhhhh\0"
  "svdupq_n_f32\0"
  "q4fffff\0"
  "svdupq_n_f64\0"
  "q2ddd\0"
  "svdupq_n_s16\0"
  "q8sssssssss\0"
  "svdupq_n_s32\0"
  "q4iiiii\0"
  "svdupq_n_s64\0"
  "q2WiWiWi\0"
  "svdupq_n_s8\0"
  "q16ScScScScScScScScScScScScScScScScSc\0"
  "svdupq_n_u16\0"
  "q8UsUsUsUsUsUsUsUsUs\0"
  "svdupq_n_u32\0"
  "q4UiUiUiUiUi\0"
  "svdupq_n_u64\0"
  "q2UWiUWiUWi\0"
  "svdupq_n_u8\0"
  "q16UcUcUcUcUcUcUcUcUcUcUcUcUcUcUcUcUc\0"
  "sveor3_n_s16\0"
  "sveor3_n_s32\0"
  "sveor3_n_s64\0"
  "sveor3_n_s8\0"
  "sveor3_n_u16\0"
  "sveor3_n_u32\0"
  "sveor3_n_u64\0"
  "sveor3_n_u8\0"
  "sveor3_s16\0"
  "sveor3_s32\0"
  "sveor3_s64\0"
  "sveor3_s8\0"
  "sveor3_u16\0"
  "sveor3_u32\0"
  "sveor3_u64\0"
  "sveor3_u8\0"
  "sveor_b_z\0"
  "sveor_n_s16_m\0"
  "sveor_n_s16_x\0"
  "sveor_n_s16_z\0"
  "sveor_n_s32_m\0"
  "sveor_n_s32_x\0"
  "sveor_n_s32_z\0"
  "sveor_n_s64_m\0"
  "sveor_n_s64_x\0"
  "sveor_n_s64_z\0"
  "sveor_n_s8_m\0"
  "sveor_n_s8_x\0"
  "sveor_n_s8_z\0"
  "sveor_n_u16_m\0"
  "sveor_n_u16_x\0"
  "sveor_n_u16_z\0"
  "sveor_n_u32_m\0"
  "sveor_n_u32_x\0"
  "sveor_n_u32_z\0"
  "sveor_n_u64_m\0"
  "sveor_n_u64_x\0"
  "sveor_n_u64_z\0"
  "sveor_n_u8_m\0"
  "sveor_n_u8_x\0"
  "sveor_n_u8_z\0"
  "sveor_s16_m\0"
  "sveor_s16_x\0"
  "sveor_s16_z\0"
  "sveor_s32_m\0"
  "sveor_s32_x\0"
  "sveor_s32_z\0"
  "sveor_s64_m\0"
  "sveor_s64_x\0"
  "sveor_s64_z\0"
  "sveor_s8_m\0"
  "sveor_s8_x\0"
  "sveor_s8_z\0"
  "sveor_u16_m\0"
  "sveor_u16_x\0"
  "sveor_u16_z\0"
  "sveor_u32_m\0"
  "sveor_u32_x\0"
  "sveor_u32_z\0"
  "sveor_u64_m\0"
  "sveor_u64_x\0"
  "sveor_u64_z\0"
  "sveor_u8_m\0"
  "sveor_u8_x\0"
  "sveor_u8_z\0"
  "sveorbt_n_s16\0"
  "sveorbt_n_s32\0"
  "sveorbt_n_s64\0"
  "sveorbt_n_s8\0"
  "sveorbt_n_u16\0"
  "sveorbt_n_u32\0"
  "sveorbt_n_u64\0"
  "sveorbt_n_u8\0"
  "sveorbt_s16\0"
  "sveorbt_s32\0"
  "sveorbt_s64\0"
  "sveorbt_s8\0"
  "sveorbt_u16\0"
  "sveorbt_u32\0"
  "sveorbt_u64\0"
  "sveorbt_u8\0"
  "sveorqv_s16\0"
  "sveorqv_s32\0"
  "sveorqv_s64\0"
  "sveorqv_s8\0"
  "sveorqv_u16\0"
  "sveorqv_u32\0"
  "sveorqv_u64\0"
  "sveorqv_u8\0"
  "sveortb_n_s16\0"
  "sveortb_n_s32\0"
  "sveortb_n_s64\0"
  "sveortb_n_s8\0"
  "sveortb_n_u16\0"
  "sveortb_n_u32\0"
  "sveortb_n_u64\0"
  "sveortb_n_u8\0"
  "sveortb_s16\0"
  "sveortb_s32\0"
  "sveortb_s64\0"
  "sveortb_s8\0"
  "sveortb_u16\0"
  "sveortb_u32\0"
  "sveortb_u64\0"
  "sveortb_u8\0"
  "sveorv_s16\0"
  "sveorv_s32\0"
  "sveorv_s64\0"
  "sveorv_s8\0"
  "sveorv_u16\0"
  "sveorv_u32\0"
  "sveorv_u64\0"
  "sveorv_u8\0"
  "svexpa_f16\0"
  "q8hq8Us\0"
  "sve|(sme,sme2,ssve-fexpa)\0"
  "svexpa_f32\0"
  "q4fq4Ui\0"
  "svexpa_f64\0"
  "q2dq2UWi\0"
  "svexpand_bf16\0"
  "svexpand_f16\0"
  "svexpand_f32\0"
  "svexpand_f64\0"
  "svexpand_mf8\0"
  "svexpand_s16\0"
  "svexpand_s32\0"
  "svexpand_s64\0"
  "svexpand_s8\0"
  "svexpand_u16\0"
  "svexpand_u32\0"
  "svexpand_u64\0"
  "svexpand_u8\0"
  "svext_bf16\0"
  "q8yq8yq8yIUWi\0"
  "svext_f16\0"
  "q8hq8hq8hIUWi\0"
  "svext_f32\0"
  "q4fq4fq4fIUWi\0"
  "svext_f64\0"
  "q2dq2dq2dIUWi\0"
  "svext_s16\0"
  "svext_s32\0"
  "svext_s64\0"
  "svext_s8\0"
  "svext_u16\0"
  "svext_u32\0"
  "svext_u64\0"
  "svext_u8\0"
  "svextb_s16_m\0"
  "svextb_s16_x\0"
  "svextb_s16_z\0"
  "svextb_s32_m\0"
  "svextb_s32_x\0"
  "svextb_s32_z\0"
  "svextb_s64_m\0"
  "svextb_s64_x\0"
  "svextb_s64_z\0"
  "svextb_u16_m\0"
  "svextb_u16_x\0"
  "svextb_u16_z\0"
  "svextb_u32_m\0"
  "svextb_u32_x\0"
  "svextb_u32_z\0"
  "svextb_u64_m\0"
  "svextb_u64_x\0"
  "svextb_u64_z\0"
  "svexth_s32_m\0"
  "svexth_s32_x\0"
  "svexth_s32_z\0"
  "svexth_s64_m\0"
  "svexth_s64_x\0"
  "svexth_s64_z\0"
  "svexth_u32_m\0"
  "svexth_u32_x\0"
  "svexth_u32_z\0"
  "svexth_u64_m\0"
  "svexth_u64_x\0"
  "svexth_u64_z\0"
  "svextq_bf16\0"
  "q8yq8yq8yi\0"
  "svextq_f16\0"
  "q8hq8hq8hi\0"
  "svextq_f32\0"
  "q4fq4fq4fi\0"
  "svextq_f64\0"
  "q2dq2dq2di\0"
  "svextq_mf8\0"
  "q16mq16mq16mi\0"
  "svextq_s16\0"
  "q8sq8sq8si\0"
  "svextq_s32\0"
  "svextq_s64\0"
  "q2Wiq2Wiq2Wii\0"
  "svextq_s8\0"
  "q16Scq16Scq16Sci\0"
  "svextq_u16\0"
  "q8Usq8Usq8Usi\0"
  "svextq_u32\0"
  "q4Uiq4Uiq4Uii\0"
  "svextq_u64\0"
  "q2UWiq2UWiq2UWii\0"
  "svextq_u8\0"
  "q16Ucq16Ucq16Uci\0"
  "svextw_s64_m\0"
  "svextw_s64_x\0"
  "svextw_s64_z\0"
  "svextw_u64_m\0"
  "svextw_u64_x\0"
  "svextw_u64_z\0"
  "svfirstp_b16\0"
  "Wiq16bq16b\0"
  "svfirstp_b32\0"
  "svfirstp_b64\0"
  "svfirstp_b8\0"
  "svget2_b\0"
  "q16bq32bIUWi\0"
  "svget2_bf16\0"
  "q8yq16yIUWi\0"
  "svget2_f16\0"
  "q8hq16hIUWi\0"
  "svget2_f32\0"
  "q4fq8fIUWi\0"
  "svget2_f64\0"
  "q2dq4dIUWi\0"
  "svget2_mf8\0"
  "q16mq32mIUWi\0"
  "svget2_s16\0"
  "q8sq16sIUWi\0"
  "svget2_s32\0"
  "q4iq8iIUWi\0"
  "svget2_s64\0"
  "q2Wiq4WiIUWi\0"
  "svget2_s8\0"
  "q16Scq32ScIUWi\0"
  "svget2_u16\0"
  "q8Usq16UsIUWi\0"
  "svget2_u32\0"
  "q4Uiq8UiIUWi\0"
  "svget2_u64\0"
  "q2UWiq4UWiIUWi\0"
  "svget2_u8\0"
  "q16Ucq32UcIUWi\0"
  "svget3_bf16\0"
  "q8yq24yIUWi\0"
  "svget3_f16\0"
  "q8hq24hIUWi\0"
  "svget3_f32\0"
  "q4fq12fIUWi\0"
  "svget3_f64\0"
  "q2dq6dIUWi\0"
  "svget3_mf8\0"
  "q16mq48mIUWi\0"
  "svget3_s16\0"
  "q8sq24sIUWi\0"
  "svget3_s32\0"
  "q4iq12iIUWi\0"
  "svget3_s64\0"
  "q2Wiq6WiIUWi\0"
  "svget3_s8\0"
  "q16Scq48ScIUWi\0"
  "svget3_u16\0"
  "q8Usq24UsIUWi\0"
  "svget3_u32\0"
  "q4Uiq12UiIUWi\0"
  "svget3_u64\0"
  "q2UWiq6UWiIUWi\0"
  "svget3_u8\0"
  "q16Ucq48UcIUWi\0"
  "svget4_b\0"
  "q16bq64bIUWi\0"
  "svget4_bf16\0"
  "q8yq32yIUWi\0"
  "svget4_f16\0"
  "q8hq32hIUWi\0"
  "svget4_f32\0"
  "q4fq16fIUWi\0"
  "svget4_f64\0"
  "q2dq8dIUWi\0"
  "svget4_mf8\0"
  "q16mq64mIUWi\0"
  "svget4_s16\0"
  "q8sq32sIUWi\0"
  "svget4_s32\0"
  "q4iq16iIUWi\0"
  "svget4_s64\0"
  "q2Wiq8WiIUWi\0"
  "svget4_s8\0"
  "q16Scq64ScIUWi\0"
  "svget4_u16\0"
  "q8Usq32UsIUWi\0"
  "svget4_u32\0"
  "q4Uiq16UiIUWi\0"
  "svget4_u64\0"
  "q2UWiq8UWiIUWi\0"
  "svget4_u8\0"
  "q16Ucq64UcIUWi\0"
  "svhadd_n_s16_m\0"
  "svhadd_n_s16_x\0"
  "svhadd_n_s16_z\0"
  "svhadd_n_s32_m\0"
  "svhadd_n_s32_x\0"
  "svhadd_n_s32_z\0"
  "svhadd_n_s64_m\0"
  "svhadd_n_s64_x\0"
  "svhadd_n_s64_z\0"
  "svhadd_n_s8_m\0"
  "svhadd_n_s8_x\0"
  "svhadd_n_s8_z\0"
  "svhadd_n_u16_m\0"
  "svhadd_n_u16_x\0"
  "svhadd_n_u16_z\0"
  "svhadd_n_u32_m\0"
  "svhadd_n_u32_x\0"
  "svhadd_n_u32_z\0"
  "svhadd_n_u64_m\0"
  "svhadd_n_u64_x\0"
  "svhadd_n_u64_z\0"
  "svhadd_n_u8_m\0"
  "svhadd_n_u8_x\0"
  "svhadd_n_u8_z\0"
  "svhadd_s16_m\0"
  "svhadd_s16_x\0"
  "svhadd_s16_z\0"
  "svhadd_s32_m\0"
  "svhadd_s32_x\0"
  "svhadd_s32_z\0"
  "svhadd_s64_m\0"
  "svhadd_s64_x\0"
  "svhadd_s64_z\0"
  "svhadd_s8_m\0"
  "svhadd_s8_x\0"
  "svhadd_s8_z\0"
  "svhadd_u16_m\0"
  "svhadd_u16_x\0"
  "svhadd_u16_z\0"
  "svhadd_u32_m\0"
  "svhadd_u32_x\0"
  "svhadd_u32_z\0"
  "svhadd_u64_m\0"
  "svhadd_u64_x\0"
  "svhadd_u64_z\0"
  "svhadd_u8_m\0"
  "svhadd_u8_x\0"
  "svhadd_u8_z\0"
  "svhistcnt_s32_z\0"
  "q4Uiq16bq4iq4i\0"
  "sve,sve2\0"
  "svhistcnt_s64_z\0"
  "q2UWiq16bq2Wiq2Wi\0"
  "svhistcnt_u32_z\0"
  "svhistcnt_u64_z\0"
  "svhistseg_s8\0"
  "q16Ucq16Scq16Sc\0"
  "svhistseg_u8\0"
  "svhsub_n_s16_m\0"
  "svhsub_n_s16_x\0"
  "svhsub_n_s16_z\0"
  "svhsub_n_s32_m\0"
  "svhsub_n_s32_x\0"
  "svhsub_n_s32_z\0"
  "svhsub_n_s64_m\0"
  "svhsub_n_s64_x\0"
  "svhsub_n_s64_z\0"
  "svhsub_n_s8_m\0"
  "svhsub_n_s8_x\0"
  "svhsub_n_s8_z\0"
  "svhsub_n_u16_m\0"
  "svhsub_n_u16_x\0"
  "svhsub_n_u16_z\0"
  "svhsub_n_u32_m\0"
  "svhsub_n_u32_x\0"
  "svhsub_n_u32_z\0"
  "svhsub_n_u64_m\0"
  "svhsub_n_u64_x\0"
  "svhsub_n_u64_z\0"
  "svhsub_n_u8_m\0"
  "svhsub_n_u8_x\0"
  "svhsub_n_u8_z\0"
  "svhsub_s16_m\0"
  "svhsub_s16_x\0"
  "svhsub_s16_z\0"
  "svhsub_s32_m\0"
  "svhsub_s32_x\0"
  "svhsub_s32_z\0"
  "svhsub_s64_m\0"
  "svhsub_s64_x\0"
  "svhsub_s64_z\0"
  "svhsub_s8_m\0"
  "svhsub_s8_x\0"
  "svhsub_s8_z\0"
  "svhsub_u16_m\0"
  "svhsub_u16_x\0"
  "svhsub_u16_z\0"
  "svhsub_u32_m\0"
  "svhsub_u32_x\0"
  "svhsub_u32_z\0"
  "svhsub_u64_m\0"
  "svhsub_u64_x\0"
  "svhsub_u64_z\0"
  "svhsub_u8_m\0"
  "svhsub_u8_x\0"
  "svhsub_u8_z\0"
  "svhsubr_n_s16_m\0"
  "svhsubr_n_s16_x\0"
  "svhsubr_n_s16_z\0"
  "svhsubr_n_s32_m\0"
  "svhsubr_n_s32_x\0"
  "svhsubr_n_s32_z\0"
  "svhsubr_n_s64_m\0"
  "svhsubr_n_s64_x\0"
  "svhsubr_n_s64_z\0"
  "svhsubr_n_s8_m\0"
  "svhsubr_n_s8_x\0"
  "svhsubr_n_s8_z\0"
  "svhsubr_n_u16_m\0"
  "svhsubr_n_u16_x\0"
  "svhsubr_n_u16_z\0"
  "svhsubr_n_u32_m\0"
  "svhsubr_n_u32_x\0"
  "svhsubr_n_u32_z\0"
  "svhsubr_n_u64_m\0"
  "svhsubr_n_u64_x\0"
  "svhsubr_n_u64_z\0"
  "svhsubr_n_u8_m\0"
  "svhsubr_n_u8_x\0"
  "svhsubr_n_u8_z\0"
  "svhsubr_s16_m\0"
  "svhsubr_s16_x\0"
  "svhsubr_s16_z\0"
  "svhsubr_s32_m\0"
  "svhsubr_s32_x\0"
  "svhsubr_s32_z\0"
  "svhsubr_s64_m\0"
  "svhsubr_s64_x\0"
  "svhsubr_s64_z\0"
  "svhsubr_s8_m\0"
  "svhsubr_s8_x\0"
  "svhsubr_s8_z\0"
  "svhsubr_u16_m\0"
  "svhsubr_u16_x\0"
  "svhsubr_u16_z\0"
  "svhsubr_u32_m\0"
  "svhsubr_u32_x\0"
  "svhsubr_u32_z\0"
  "svhsubr_u64_m\0"
  "svhsubr_u64_x\0"
  "svhsubr_u64_z\0"
  "svhsubr_u8_m\0"
  "svhsubr_u8_x\0"
  "svhsubr_u8_z\0"
  "svindex_s16\0"
  "q8sss\0"
  "svindex_s32\0"
  "q4iii\0"
  "svindex_s64\0"
  "svindex_s8\0"
  "q16ScScSc\0"
  "svindex_u16\0"
  "q8UsUsUs\0"
  "svindex_u32\0"
  "q4UiUiUi\0"
  "svindex_u64\0"
  "svindex_u8\0"
  "q16UcUcUc\0"
  "svinsr_n_bf16\0"
  "q8yq8yy\0"
  "svinsr_n_f16\0"
  "q8hq8hh\0"
  "svinsr_n_f32\0"
  "q4fq4ff\0"
  "svinsr_n_f64\0"
  "q2dq2dd\0"
  "svinsr_n_s16\0"
  "q8sq8ss\0"
  "svinsr_n_s32\0"
  "q4iq4ii\0"
  "svinsr_n_s64\0"
  "q2Wiq2WiWi\0"
  "svinsr_n_s8\0"
  "q16Scq16ScSc\0"
  "svinsr_n_u16\0"
  "svinsr_n_u32\0"
  "svinsr_n_u64\0"
  "svinsr_n_u8\0"
  "svlasta_bf16\0"
  "yq16bq8y\0"
  "svlasta_f16\0"
  "svlasta_f32\0"
  "svlasta_f64\0"
  "svlasta_s16\0"
  "svlasta_s32\0"
  "svlasta_s64\0"
  "svlasta_s8\0"
  "svlasta_u16\0"
  "svlasta_u32\0"
  "svlasta_u64\0"
  "svlasta_u8\0"
  "svlastb_bf16\0"
  "svlastb_f16\0"
  "svlastb_f32\0"
  "svlastb_f64\0"
  "svlastb_s16\0"
  "svlastb_s32\0"
  "svlastb_s64\0"
  "svlastb_s8\0"
  "svlastb_u16\0"
  "svlastb_u32\0"
  "svlastb_u64\0"
  "svlastb_u8\0"
  "svlastp_b16\0"
  "svlastp_b32\0"
  "svlastp_b64\0"
  "svlastp_b8\0"
  "svld1_bf16\0"
  "q8yq16byC*\0"
  "svld1_bf16_x2\0"
  "q16yQayC*\0"
  "svld1_bf16_x4\0"
  "q32yQayC*\0"
  "svld1_f16\0"
  "q8hq16bhC*\0"
  "svld1_f16_x2\0"
  "q16hQahC*\0"
  "svld1_f16_x4\0"
  "q32hQahC*\0"
  "svld1_f32\0"
  "q4fq16bfC*\0"
  "svld1_f32_x2\0"
  "q8fQafC*\0"
  "svld1_f32_x4\0"
  "q16fQafC*\0"
  "svld1_f64\0"
  "q2dq16bdC*\0"
  "svld1_f64_x2\0"
  "q4dQadC*\0"
  "svld1_f64_x4\0"
  "q8dQadC*\0"
  "svld1_gather_s32index_f32\0"
  "q4fq16bfC*q4i\0"
  "svld1_gather_s32index_s32\0"
  "q4iq16bSiC*q4i\0"
  "svld1_gather_s32index_u32\0"
  "q4Uiq16bUiC*q4i\0"
  "svld1_gather_s32offset_f32\0"
  "svld1_gather_s32offset_s32\0"
  "svld1_gather_s32offset_u32\0"
  "svld1_gather_s64index_f64\0"
  "q2dq16bdC*q2Wi\0"
  "svld1_gather_s64index_s64\0"
  "q2Wiq16bSWiC*q2Wi\0"
  "svld1_gather_s64index_u64\0"
  "q2UWiq16bUWiC*q2Wi\0"
  "svld1_gather_s64offset_f64\0"
  "svld1_gather_s64offset_s64\0"
  "svld1_gather_s64offset_u64\0"
  "svld1_gather_u32base_f32\0"
  "svld1_gather_u32base_index_f32\0"
  "q4fq16bq4UiWi\0"
  "svld1_gather_u32base_index_s32\0"
  "q4iq16bq4UiWi\0"
  "svld1_gather_u32base_index_u32\0"
  "q4Uiq16bq4UiWi\0"
  "svld1_gather_u32base_offset_f32\0"
  "svld1_gather_u32base_offset_s32\0"
  "svld1_gather_u32base_offset_u32\0"
  "svld1_gather_u32base_s32\0"
  "q4iq16bq4Ui\0"
  "svld1_gather_u32base_u32\0"
  "svld1_gather_u32index_f32\0"
  "q4fq16bfC*q4Ui\0"
  "svld1_gather_u32index_s32\0"
  "q4iq16bSiC*q4Ui\0"
  "svld1_gather_u32index_u32\0"
  "q4Uiq16bUiC*q4Ui\0"
  "svld1_gather_u32offset_f32\0"
  "svld1_gather_u32offset_s32\0"
  "svld1_gather_u32offset_u32\0"
  "svld1_gather_u64base_f64\0"
  "svld1_gather_u64base_index_f64\0"
  "q2dq16bq2UWiWi\0"
  "svld1_gather_u64base_index_s64\0"
  "q2Wiq16bq2UWiWi\0"
  "svld1_gather_u64base_index_u64\0"
  "q2UWiq16bq2UWiWi\0"
  "svld1_gather_u64base_offset_f64\0"
  "svld1_gather_u64base_offset_s64\0"
  "svld1_gather_u64base_offset_u64\0"
  "svld1_gather_u64base_s64\0"
  "q2Wiq16bq2UWi\0"
  "svld1_gather_u64base_u64\0"
  "svld1_gather_u64index_f64\0"
  "q2dq16bdC*q2UWi\0"
  "svld1_gather_u64index_s64\0"
  "q2Wiq16bSWiC*q2UWi\0"
  "svld1_gather_u64index_u64\0"
  "q2UWiq16bUWiC*q2UWi\0"
  "svld1_gather_u64offset_f64\0"
  "svld1_gather_u64offset_s64\0"
  "svld1_gather_u64offset_u64\0"
  "svld1_mf8\0"
  "q16mq16bmC*\0"
  "svld1_mf8_x2\0"
  "q32mQamC*\0"
  "svld1_mf8_x4\0"
  "q64mQamC*\0"
  "svld1_s16\0"
  "q8sq16bSsC*\0"
  "svld1_s16_x2\0"
  "q16sQaSsC*\0"
  "svld1_s16_x4\0"
  "q32sQaSsC*\0"
  "svld1_s32\0"
  "q4iq16bSiC*\0"
  "svld1_s32_x2\0"
  "q8iQaSiC*\0"
  "svld1_s32_x4\0"
  "q16iQaSiC*\0"
  "svld1_s64\0"
  "q2Wiq16bSWiC*\0"
  "svld1_s64_x2\0"
  "q4WiQaSWiC*\0"
  "svld1_s64_x4\0"
  "q8WiQaSWiC*\0"
  "svld1_s8\0"
  "q16Scq16bScC*\0"
  "svld1_s8_x2\0"
  "q32ScQaScC*\0"
  "svld1_s8_x4\0"
  "q64ScQaScC*\0"
  "svld1_u16\0"
  "q8Usq16bUsC*\0"
  "svld1_u16_x2\0"
  "q16UsQaUsC*\0"
  "svld1_u16_x4\0"
  "q32UsQaUsC*\0"
  "svld1_u32\0"
  "q4Uiq16bUiC*\0"
  "svld1_u32_x2\0"
  "q8UiQaUiC*\0"
  "svld1_u32_x4\0"
  "q16UiQaUiC*\0"
  "svld1_u64\0"
  "q2UWiq16bUWiC*\0"
  "svld1_u64_x2\0"
  "q4UWiQaUWiC*\0"
  "svld1_u64_x4\0"
  "q8UWiQaUWiC*\0"
  "svld1_u8\0"
  "q16Ucq16bUcC*\0"
  "svld1_u8_x2\0"
  "q32UcQaUcC*\0"
  "svld1_u8_x4\0"
  "q64UcQaUcC*\0"
  "svld1_vnum_bf16\0"
  "q8yq16byC*Wi\0"
  "svld1_vnum_bf16_x2\0"
  "q16yQayC*Wi\0"
  "svld1_vnum_bf16_x4\0"
  "q32yQayC*Wi\0"
  "svld1_vnum_f16\0"
  "q8hq16bhC*Wi\0"
  "svld1_vnum_f16_x2\0"
  "q16hQahC*Wi\0"
  "svld1_vnum_f16_x4\0"
  "q32hQahC*Wi\0"
  "svld1_vnum_f32\0"
  "q4fq16bfC*Wi\0"
  "svld1_vnum_f32_x2\0"
  "q8fQafC*Wi\0"
  "svld1_vnum_f32_x4\0"
  "q16fQafC*Wi\0"
  "svld1_vnum_f64\0"
  "q2dq16bdC*Wi\0"
  "svld1_vnum_f64_x2\0"
  "q4dQadC*Wi\0"
  "svld1_vnum_f64_x4\0"
  "q8dQadC*Wi\0"
  "svld1_vnum_mf8\0"
  "q16mq16bmC*Wi\0"
  "svld1_vnum_mf8_x2\0"
  "q32mQamC*Wi\0"
  "svld1_vnum_mf8_x4\0"
  "q64mQamC*Wi\0"
  "svld1_vnum_s16\0"
  "q8sq16bSsC*Wi\0"
  "svld1_vnum_s16_x2\0"
  "q16sQaSsC*Wi\0"
  "svld1_vnum_s16_x4\0"
  "q32sQaSsC*Wi\0"
  "svld1_vnum_s32\0"
  "q4iq16bSiC*Wi\0"
  "svld1_vnum_s32_x2\0"
  "q8iQaSiC*Wi\0"
  "svld1_vnum_s32_x4\0"
  "q16iQaSiC*Wi\0"
  "svld1_vnum_s64\0"
  "q2Wiq16bSWiC*Wi\0"
  "svld1_vnum_s64_x2\0"
  "q4WiQaSWiC*Wi\0"
  "svld1_vnum_s64_x4\0"
  "q8WiQaSWiC*Wi\0"
  "svld1_vnum_s8\0"
  "q16Scq16bScC*Wi\0"
  "svld1_vnum_s8_x2\0"
  "q32ScQaScC*Wi\0"
  "svld1_vnum_s8_x4\0"
  "q64ScQaScC*Wi\0"
  "svld1_vnum_u16\0"
  "q8Usq16bUsC*Wi\0"
  "svld1_vnum_u16_x2\0"
  "q16UsQaUsC*Wi\0"
  "svld1_vnum_u16_x4\0"
  "q32UsQaUsC*Wi\0"
  "svld1_vnum_u32\0"
  "q4Uiq16bUiC*Wi\0"
  "svld1_vnum_u32_x2\0"
  "q8UiQaUiC*Wi\0"
  "svld1_vnum_u32_x4\0"
  "q16UiQaUiC*Wi\0"
  "svld1_vnum_u64\0"
  "q2UWiq16bUWiC*Wi\0"
  "svld1_vnum_u64_x2\0"
  "q4UWiQaUWiC*Wi\0"
  "svld1_vnum_u64_x4\0"
  "q8UWiQaUWiC*Wi\0"
  "svld1_vnum_u8\0"
  "q16Ucq16bUcC*Wi\0"
  "svld1_vnum_u8_x2\0"
  "q32UcQaUcC*Wi\0"
  "svld1_vnum_u8_x4\0"
  "q64UcQaUcC*Wi\0"
  "svld1q_gather_u64base_bf16\0"
  "q8yq16bq2UWi\0"
  "sve,sve2p1\0"
  "svld1q_gather_u64base_f16\0"
  "svld1q_gather_u64base_f32\0"
  "svld1q_gather_u64base_f64\0"
  "svld1q_gather_u64base_index_bf16\0"
  "q8yq16bq2UWiWi\0"
  "svld1q_gather_u64base_index_f16\0"
  "q8hq16bq2UWiWi\0"
  "svld1q_gather_u64base_index_f32\0"
  "q4fq16bq2UWiWi\0"
  "svld1q_gather_u64base_index_f64\0"
  "svld1q_gather_u64base_index_s16\0"
  "q8sq16bq2UWiWi\0"
  "svld1q_gather_u64base_index_s32\0"
  "q4iq16bq2UWiWi\0"
  "svld1q_gather_u64base_index_s64\0"
  "svld1q_gather_u64base_index_u16\0"
  "q8Usq16bq2UWiWi\0"
  "svld1q_gather_u64base_index_u32\0"
  "q4Uiq16bq2UWiWi\0"
  "svld1q_gather_u64base_index_u64\0"
  "svld1q_gather_u64base_mf8\0"
  "q16mq16bq2UWi\0"
  "svld1q_gather_u64base_offset_bf16\0"
  "svld1q_gather_u64base_offset_f16\0"
  "svld1q_gather_u64base_offset_f32\0"
  "svld1q_gather_u64base_offset_f64\0"
  "svld1q_gather_u64base_offset_mf8\0"
  "q16mq16bq2UWiWi\0"
  "svld1q_gather_u64base_offset_s16\0"
  "svld1q_gather_u64base_offset_s32\0"
  "svld1q_gather_u64base_offset_s64\0"
  "svld1q_gather_u64base_offset_s8\0"
  "q16Scq16bq2UWiWi\0"
  "svld1q_gather_u64base_offset_u16\0"
  "svld1q_gather_u64base_offset_u32\0"
  "svld1q_gather_u64base_offset_u64\0"
  "svld1q_gather_u64base_offset_u8\0"
  "q16Ucq16bq2UWiWi\0"
  "svld1q_gather_u64base_s16\0"
  "q8sq16bq2UWi\0"
  "svld1q_gather_u64base_s32\0"
  "q4iq16bq2UWi\0"
  "svld1q_gather_u64base_s64\0"
  "svld1q_gather_u64base_s8\0"
  "q16Scq16bq2UWi\0"
  "svld1q_gather_u64base_u16\0"
  "q8Usq16bq2UWi\0"
  "svld1q_gather_u64base_u32\0"
  "q4Uiq16bq2UWi\0"
  "svld1q_gather_u64base_u64\0"
  "svld1q_gather_u64base_u8\0"
  "q16Ucq16bq2UWi\0"
  "svld1q_gather_u64index_bf16\0"
  "q8yq16byC*q2UWi\0"
  "svld1q_gather_u64index_f16\0"
  "q8hq16bhC*q2UWi\0"
  "svld1q_gather_u64index_f32\0"
  "q4fq16bfC*q2UWi\0"
  "svld1q_gather_u64index_f64\0"
  "svld1q_gather_u64index_s16\0"
  "q8sq16bSsC*q2UWi\0"
  "svld1q_gather_u64index_s32\0"
  "q4iq16bSiC*q2UWi\0"
  "svld1q_gather_u64index_s64\0"
  "svld1q_gather_u64index_u16\0"
  "q8Usq16bUsC*q2UWi\0"
  "svld1q_gather_u64index_u32\0"
  "q4Uiq16bUiC*q2UWi\0"
  "svld1q_gather_u64index_u64\0"
  "svld1q_gather_u64offset_bf16\0"
  "svld1q_gather_u64offset_f16\0"
  "svld1q_gather_u64offset_f32\0"
  "svld1q_gather_u64offset_f64\0"
  "svld1q_gather_u64offset_mf8\0"
  "q16mq16bmC*q2UWi\0"
  "svld1q_gather_u64offset_s16\0"
  "svld1q_gather_u64offset_s32\0"
  "svld1q_gather_u64offset_s64\0"
  "svld1q_gather_u64offset_s8\0"
  "q16Scq16bScC*q2UWi\0"
  "svld1q_gather_u64offset_u16\0"
  "svld1q_gather_u64offset_u32\0"
  "svld1q_gather_u64offset_u64\0"
  "svld1q_gather_u64offset_u8\0"
  "q16Ucq16bUcC*q2UWi\0"
  "svld1ro_bf16\0"
  "sve,f64mm\0"
  "svld1ro_f16\0"
  "svld1ro_f32\0"
  "svld1ro_f64\0"
  "svld1ro_mf8\0"
  "svld1ro_s16\0"
  "svld1ro_s32\0"
  "svld1ro_s64\0"
  "svld1ro_s8\0"
  "svld1ro_u16\0"
  "svld1ro_u32\0"
  "svld1ro_u64\0"
  "svld1ro_u8\0"
  "svld1rq_bf16\0"
  "svld1rq_f16\0"
  "svld1rq_f32\0"
  "svld1rq_f64\0"
  "svld1rq_mf8\0"
  "svld1rq_s16\0"
  "svld1rq_s32\0"
  "svld1rq_s64\0"
  "svld1rq_s8\0"
  "svld1rq_u16\0"
  "svld1rq_u32\0"
  "svld1rq_u64\0"
  "svld1rq_u8\0"
  "svld1sb_gather_s32offset_s32\0"
  "q4iq16bScC*q4i\0"
  "svld1sb_gather_s32offset_u32\0"
  "q4Uiq16bScC*q4i\0"
  "svld1sb_gather_s64offset_s64\0"
  "q2Wiq16bScC*q2Wi\0"
  "svld1sb_gather_s64offset_u64\0"
  "q2UWiq16bScC*q2Wi\0"
  "svld1sb_gather_u32base_offset_s32\0"
  "svld1sb_gather_u32base_offset_u32\0"
  "svld1sb_gather_u32base_s32\0"
  "svld1sb_gather_u32base_u32\0"
  "svld1sb_gather_u32offset_s32\0"
  "q4iq16bScC*q4Ui\0"
  "svld1sb_gather_u32offset_u32\0"
  "q4Uiq16bScC*q4Ui\0"
  "svld1sb_gather_u64base_offset_s64\0"
  "svld1sb_gather_u64base_offset_u64\0"
  "svld1sb_gather_u64base_s64\0"
  "svld1sb_gather_u64base_u64\0"
  "svld1sb_gather_u64offset_s64\0"
  "q2Wiq16bScC*q2UWi\0"
  "svld1sb_gather_u64offset_u64\0"
  "q2UWiq16bScC*q2UWi\0"
  "svld1sb_s16\0"
  "q8sq16bScC*\0"
  "svld1sb_s32\0"
  "q4iq16bScC*\0"
  "svld1sb_s64\0"
  "q2Wiq16bScC*\0"
  "svld1sb_u16\0"
  "q8Usq16bScC*\0"
  "svld1sb_u32\0"
  "q4Uiq16bScC*\0"
  "svld1sb_u64\0"
  "q2UWiq16bScC*\0"
  "svld1sb_vnum_s16\0"
  "q8sq16bScC*Wi\0"
  "svld1sb_vnum_s32\0"
  "q4iq16bScC*Wi\0"
  "svld1sb_vnum_s64\0"
  "q2Wiq16bScC*Wi\0"
  "svld1sb_vnum_u16\0"
  "q8Usq16bScC*Wi\0"
  "svld1sb_vnum_u32\0"
  "q4Uiq16bScC*Wi\0"
  "svld1sb_vnum_u64\0"
  "q2UWiq16bScC*Wi\0"
  "svld1sh_gather_s32index_s32\0"
  "q4iq16bSsC*q4i\0"
  "svld1sh_gather_s32index_u32\0"
  "q4Uiq16bSsC*q4i\0"
  "svld1sh_gather_s32offset_s32\0"
  "svld1sh_gather_s32offset_u32\0"
  "svld1sh_gather_s64index_s64\0"
  "q2Wiq16bSsC*q2Wi\0"
  "svld1sh_gather_s64index_u64\0"
  "q2UWiq16bSsC*q2Wi\0"
  "svld1sh_gather_s64offset_s64\0"
  "svld1sh_gather_s64offset_u64\0"
  "svld1sh_gather_u32base_index_s32\0"
  "svld1sh_gather_u32base_index_u32\0"
  "svld1sh_gather_u32base_offset_s32\0"
  "svld1sh_gather_u32base_offset_u32\0"
  "svld1sh_gather_u32base_s32\0"
  "svld1sh_gather_u32base_u32\0"
  "svld1sh_gather_u32index_s32\0"
  "q4iq16bSsC*q4Ui\0"
  "svld1sh_gather_u32index_u32\0"
  "q4Uiq16bSsC*q4Ui\0"
  "svld1sh_gather_u32offset_s32\0"
  "svld1sh_gather_u32offset_u32\0"
  "svld1sh_gather_u64base_index_s64\0"
  "svld1sh_gather_u64base_index_u64\0"
  "svld1sh_gather_u64base_offset_s64\0"
  "svld1sh_gather_u64base_offset_u64\0"
  "svld1sh_gather_u64base_s64\0"
  "svld1sh_gather_u64base_u64\0"
  "svld1sh_gather_u64index_s64\0"
  "q2Wiq16bSsC*q2UWi\0"
  "svld1sh_gather_u64index_u64\0"
  "q2UWiq16bSsC*q2UWi\0"
  "svld1sh_gather_u64offset_s64\0"
  "svld1sh_gather_u64offset_u64\0"
  "svld1sh_s32\0"
  "q4iq16bSsC*\0"
  "svld1sh_s64\0"
  "q2Wiq16bSsC*\0"
  "svld1sh_u32\0"
  "q4Uiq16bSsC*\0"
  "svld1sh_u64\0"
  "q2UWiq16bSsC*\0"
  "svld1sh_vnum_s32\0"
  "q4iq16bSsC*Wi\0"
  "svld1sh_vnum_s64\0"
  "q2Wiq16bSsC*Wi\0"
  "svld1sh_vnum_u32\0"
  "q4Uiq16bSsC*Wi\0"
  "svld1sh_vnum_u64\0"
  "q2UWiq16bSsC*Wi\0"
  "svld1sw_gather_s64index_s64\0"
  "q2Wiq16bSiC*q2Wi\0"
  "svld1sw_gather_s64index_u64\0"
  "q2UWiq16bSiC*q2Wi\0"
  "svld1sw_gather_s64offset_s64\0"
  "svld1sw_gather_s64offset_u64\0"
  "svld1sw_gather_u64base_index_s64\0"
  "svld1sw_gather_u64base_index_u64\0"
  "svld1sw_gather_u64base_offset_s64\0"
  "svld1sw_gather_u64base_offset_u64\0"
  "svld1sw_gather_u64base_s64\0"
  "svld1sw_gather_u64base_u64\0"
  "svld1sw_gather_u64index_s64\0"
  "q2Wiq16bSiC*q2UWi\0"
  "svld1sw_gather_u64index_u64\0"
  "q2UWiq16bSiC*q2UWi\0"
  "svld1sw_gather_u64offset_s64\0"
  "svld1sw_gather_u64offset_u64\0"
  "svld1sw_s64\0"
  "q2Wiq16bSiC*\0"
  "svld1sw_u64\0"
  "q2UWiq16bSiC*\0"
  "svld1sw_vnum_s64\0"
  "q2Wiq16bSiC*Wi\0"
  "svld1sw_vnum_u64\0"
  "q2UWiq16bSiC*Wi\0"
  "svld1ub_gather_s32offset_s32\0"
  "q4iq16bUcC*q4i\0"
  "svld1ub_gather_s32offset_u32\0"
  "q4Uiq16bUcC*q4i\0"
  "svld1ub_gather_s64offset_s64\0"
  "q2Wiq16bUcC*q2Wi\0"
  "svld1ub_gather_s64offset_u64\0"
  "q2UWiq16bUcC*q2Wi\0"
  "svld1ub_gather_u32base_offset_s32\0"
  "svld1ub_gather_u32base_offset_u32\0"
  "svld1ub_gather_u32base_s32\0"
  "svld1ub_gather_u32base_u32\0"
  "svld1ub_gather_u32offset_s32\0"
  "q4iq16bUcC*q4Ui\0"
  "svld1ub_gather_u32offset_u32\0"
  "q4Uiq16bUcC*q4Ui\0"
  "svld1ub_gather_u64base_offset_s64\0"
  "svld1ub_gather_u64base_offset_u64\0"
  "svld1ub_gather_u64base_s64\0"
  "svld1ub_gather_u64base_u64\0"
  "svld1ub_gather_u64offset_s64\0"
  "q2Wiq16bUcC*q2UWi\0"
  "svld1ub_gather_u64offset_u64\0"
  "q2UWiq16bUcC*q2UWi\0"
  "svld1ub_s16\0"
  "q8sq16bUcC*\0"
  "svld1ub_s32\0"
  "q4iq16bUcC*\0"
  "svld1ub_s64\0"
  "q2Wiq16bUcC*\0"
  "svld1ub_u16\0"
  "q8Usq16bUcC*\0"
  "svld1ub_u32\0"
  "q4Uiq16bUcC*\0"
  "svld1ub_u64\0"
  "q2UWiq16bUcC*\0"
  "svld1ub_vnum_s16\0"
  "q8sq16bUcC*Wi\0"
  "svld1ub_vnum_s32\0"
  "q4iq16bUcC*Wi\0"
  "svld1ub_vnum_s64\0"
  "q2Wiq16bUcC*Wi\0"
  "svld1ub_vnum_u16\0"
  "q8Usq16bUcC*Wi\0"
  "svld1ub_vnum_u32\0"
  "q4Uiq16bUcC*Wi\0"
  "svld1ub_vnum_u64\0"
  "q2UWiq16bUcC*Wi\0"
  "svld1udq_f64\0"
  "svld1udq_s64\0"
  "svld1udq_u64\0"
  "svld1udq_vnum_f64\0"
  "svld1udq_vnum_s64\0"
  "svld1udq_vnum_u64\0"
  "svld1uh_gather_s32index_s32\0"
  "q4iq16bUsC*q4i\0"
  "svld1uh_gather_s32index_u32\0"
  "q4Uiq16bUsC*q4i\0"
  "svld1uh_gather_s32offset_s32\0"
  "svld1uh_gather_s32offset_u32\0"
  "svld1uh_gather_s64index_s64\0"
  "q2Wiq16bUsC*q2Wi\0"
  "svld1uh_gather_s64index_u64\0"
  "q2UWiq16bUsC*q2Wi\0"
  "svld1uh_gather_s64offset_s64\0"
  "svld1uh_gather_s64offset_u64\0"
  "svld1uh_gather_u32base_index_s32\0"
  "svld1uh_gather_u32base_index_u32\0"
  "svld1uh_gather_u32base_offset_s32\0"
  "svld1uh_gather_u32base_offset_u32\0"
  "svld1uh_gather_u32base_s32\0"
  "svld1uh_gather_u32base_u32\0"
  "svld1uh_gather_u32index_s32\0"
  "q4iq16bUsC*q4Ui\0"
  "svld1uh_gather_u32index_u32\0"
  "q4Uiq16bUsC*q4Ui\0"
  "svld1uh_gather_u32offset_s32\0"
  "svld1uh_gather_u32offset_u32\0"
  "svld1uh_gather_u64base_index_s64\0"
  "svld1uh_gather_u64base_index_u64\0"
  "svld1uh_gather_u64base_offset_s64\0"
  "svld1uh_gather_u64base_offset_u64\0"
  "svld1uh_gather_u64base_s64\0"
  "svld1uh_gather_u64base_u64\0"
  "svld1uh_gather_u64index_s64\0"
  "q2Wiq16bUsC*q2UWi\0"
  "svld1uh_gather_u64index_u64\0"
  "q2UWiq16bUsC*q2UWi\0"
  "svld1uh_gather_u64offset_s64\0"
  "svld1uh_gather_u64offset_u64\0"
  "svld1uh_s32\0"
  "q4iq16bUsC*\0"
  "svld1uh_s64\0"
  "q2Wiq16bUsC*\0"
  "svld1uh_u32\0"
  "q4Uiq16bUsC*\0"
  "svld1uh_u64\0"
  "q2UWiq16bUsC*\0"
  "svld1uh_vnum_s32\0"
  "q4iq16bUsC*Wi\0"
  "svld1uh_vnum_s64\0"
  "q2Wiq16bUsC*Wi\0"
  "svld1uh_vnum_u32\0"
  "q4Uiq16bUsC*Wi\0"
  "svld1uh_vnum_u64\0"
  "q2UWiq16bUsC*Wi\0"
  "svld1uw_gather_s64index_s64\0"
  "q2Wiq16bUiC*q2Wi\0"
  "svld1uw_gather_s64index_u64\0"
  "q2UWiq16bUiC*q2Wi\0"
  "svld1uw_gather_s64offset_s64\0"
  "svld1uw_gather_s64offset_u64\0"
  "svld1uw_gather_u64base_index_s64\0"
  "svld1uw_gather_u64base_index_u64\0"
  "svld1uw_gather_u64base_offset_s64\0"
  "svld1uw_gather_u64base_offset_u64\0"
  "svld1uw_gather_u64base_s64\0"
  "svld1uw_gather_u64base_u64\0"
  "svld1uw_gather_u64index_s64\0"
  "q2Wiq16bUiC*q2UWi\0"
  "svld1uw_gather_u64index_u64\0"
  "q2UWiq16bUiC*q2UWi\0"
  "svld1uw_gather_u64offset_s64\0"
  "svld1uw_gather_u64offset_u64\0"
  "svld1uw_s64\0"
  "q2Wiq16bUiC*\0"
  "svld1uw_u64\0"
  "q2UWiq16bUiC*\0"
  "svld1uw_vnum_s64\0"
  "q2Wiq16bUiC*Wi\0"
  "svld1uw_vnum_u64\0"
  "q2UWiq16bUiC*Wi\0"
  "svld1uwq_f32\0"
  "svld1uwq_s32\0"
  "svld1uwq_u32\0"
  "svld1uwq_vnum_f32\0"
  "svld1uwq_vnum_s32\0"
  "svld1uwq_vnum_u32\0"
  "svld2_bf16\0"
  "q16yq16byC*\0"
  "svld2_f16\0"
  "q16hq16bhC*\0"
  "svld2_f32\0"
  "q8fq16bfC*\0"
  "svld2_f64\0"
  "q4dq16bdC*\0"
  "svld2_mf8\0"
  "q32mq16bmC*\0"
  "svld2_s16\0"
  "q16sq16bSsC*\0"
  "svld2_s32\0"
  "q8iq16bSiC*\0"
  "svld2_s64\0"
  "q4Wiq16bSWiC*\0"
  "svld2_s8\0"
  "q32Scq16bScC*\0"
  "svld2_u16\0"
  "q16Usq16bUsC*\0"
  "svld2_u32\0"
  "q8Uiq16bUiC*\0"
  "svld2_u64\0"
  "q4UWiq16bUWiC*\0"
  "svld2_u8\0"
  "q32Ucq16bUcC*\0"
  "svld2_vnum_bf16\0"
  "q16yq16byC*Wi\0"
  "svld2_vnum_f16\0"
  "q16hq16bhC*Wi\0"
  "svld2_vnum_f32\0"
  "q8fq16bfC*Wi\0"
  "svld2_vnum_f64\0"
  "q4dq16bdC*Wi\0"
  "svld2_vnum_mf8\0"
  "q32mq16bmC*Wi\0"
  "svld2_vnum_s16\0"
  "q16sq16bSsC*Wi\0"
  "svld2_vnum_s32\0"
  "q8iq16bSiC*Wi\0"
  "svld2_vnum_s64\0"
  "q4Wiq16bSWiC*Wi\0"
  "svld2_vnum_s8\0"
  "q32Scq16bScC*Wi\0"
  "svld2_vnum_u16\0"
  "q16Usq16bUsC*Wi\0"
  "svld2_vnum_u32\0"
  "q8Uiq16bUiC*Wi\0"
  "svld2_vnum_u64\0"
  "q4UWiq16bUWiC*Wi\0"
  "svld2_vnum_u8\0"
  "q32Ucq16bUcC*Wi\0"
  "svld2q_bf16\0"
  "svld2q_f16\0"
  "svld2q_f32\0"
  "svld2q_f64\0"
  "svld2q_mf8\0"
  "svld2q_s16\0"
  "svld2q_s32\0"
  "svld2q_s64\0"
  "svld2q_s8\0"
  "svld2q_u16\0"
  "svld2q_u32\0"
  "svld2q_u64\0"
  "svld2q_u8\0"
  "svld2q_vnum_bf16\0"
  "svld2q_vnum_f16\0"
  "svld2q_vnum_f32\0"
  "svld2q_vnum_f64\0"
  "svld2q_vnum_mf8\0"
  "svld2q_vnum_s16\0"
  "svld2q_vnum_s32\0"
  "svld2q_vnum_s64\0"
  "svld2q_vnum_s8\0"
  "svld2q_vnum_u16\0"
  "svld2q_vnum_u32\0"
  "svld2q_vnum_u64\0"
  "svld2q_vnum_u8\0"
  "svld3_bf16\0"
  "q24yq16byC*\0"
  "svld3_f16\0"
  "q24hq16bhC*\0"
  "svld3_f32\0"
  "q12fq16bfC*\0"
  "svld3_f64\0"
  "q6dq16bdC*\0"
  "svld3_mf8\0"
  "q48mq16bmC*\0"
  "svld3_s16\0"
  "q24sq16bSsC*\0"
  "svld3_s32\0"
  "q12iq16bSiC*\0"
  "svld3_s64\0"
  "q6Wiq16bSWiC*\0"
  "svld3_s8\0"
  "q48Scq16bScC*\0"
  "svld3_u16\0"
  "q24Usq16bUsC*\0"
  "svld3_u32\0"
  "q12Uiq16bUiC*\0"
  "svld3_u64\0"
  "q6UWiq16bUWiC*\0"
  "svld3_u8\0"
  "q48Ucq16bUcC*\0"
  "svld3_vnum_bf16\0"
  "q24yq16byC*Wi\0"
  "svld3_vnum_f16\0"
  "q24hq16bhC*Wi\0"
  "svld3_vnum_f32\0"
  "q12fq16bfC*Wi\0"
  "svld3_vnum_f64\0"
  "q6dq16bdC*Wi\0"
  "svld3_vnum_mf8\0"
  "q48mq16bmC*Wi\0"
  "svld3_vnum_s16\0"
  "q24sq16bSsC*Wi\0"
  "svld3_vnum_s32\0"
  "q12iq16bSiC*Wi\0"
  "svld3_vnum_s64\0"
  "q6Wiq16bSWiC*Wi\0"
  "svld3_vnum_s8\0"
  "q48Scq16bScC*Wi\0"
  "svld3_vnum_u16\0"
  "q24Usq16bUsC*Wi\0"
  "svld3_vnum_u32\0"
  "q12Uiq16bUiC*Wi\0"
  "svld3_vnum_u64\0"
  "q6UWiq16bUWiC*Wi\0"
  "svld3_vnum_u8\0"
  "q48Ucq16bUcC*Wi\0"
  "svld3q_bf16\0"
  "svld3q_f16\0"
  "svld3q_f32\0"
  "svld3q_f64\0"
  "svld3q_mf8\0"
  "svld3q_s16\0"
  "svld3q_s32\0"
  "svld3q_s64\0"
  "svld3q_s8\0"
  "svld3q_u16\0"
  "svld3q_u32\0"
  "svld3q_u64\0"
  "svld3q_u8\0"
  "svld3q_vnum_bf16\0"
  "svld3q_vnum_f16\0"
  "svld3q_vnum_f32\0"
  "svld3q_vnum_f64\0"
  "svld3q_vnum_mf8\0"
  "svld3q_vnum_s16\0"
  "svld3q_vnum_s32\0"
  "svld3q_vnum_s64\0"
  "svld3q_vnum_s8\0"
  "svld3q_vnum_u16\0"
  "svld3q_vnum_u32\0"
  "svld3q_vnum_u64\0"
  "svld3q_vnum_u8\0"
  "svld4_bf16\0"
  "q32yq16byC*\0"
  "svld4_f16\0"
  "q32hq16bhC*\0"
  "svld4_f32\0"
  "q16fq16bfC*\0"
  "svld4_f64\0"
  "q8dq16bdC*\0"
  "svld4_mf8\0"
  "q64mq16bmC*\0"
  "svld4_s16\0"
  "q32sq16bSsC*\0"
  "svld4_s32\0"
  "q16iq16bSiC*\0"
  "svld4_s64\0"
  "q8Wiq16bSWiC*\0"
  "svld4_s8\0"
  "q64Scq16bScC*\0"
  "svld4_u16\0"
  "q32Usq16bUsC*\0"
  "svld4_u32\0"
  "q16Uiq16bUiC*\0"
  "svld4_u64\0"
  "q8UWiq16bUWiC*\0"
  "svld4_u8\0"
  "q64Ucq16bUcC*\0"
  "svld4_vnum_bf16\0"
  "q32yq16byC*Wi\0"
  "svld4_vnum_f16\0"
  "q32hq16bhC*Wi\0"
  "svld4_vnum_f32\0"
  "q16fq16bfC*Wi\0"
  "svld4_vnum_f64\0"
  "q8dq16bdC*Wi\0"
  "svld4_vnum_mf8\0"
  "q64mq16bmC*Wi\0"
  "svld4_vnum_s16\0"
  "q32sq16bSsC*Wi\0"
  "svld4_vnum_s32\0"
  "q16iq16bSiC*Wi\0"
  "svld4_vnum_s64\0"
  "q8Wiq16bSWiC*Wi\0"
  "svld4_vnum_s8\0"
  "q64Scq16bScC*Wi\0"
  "svld4_vnum_u16\0"
  "q32Usq16bUsC*Wi\0"
  "svld4_vnum_u32\0"
  "q16Uiq16bUiC*Wi\0"
  "svld4_vnum_u64\0"
  "q8UWiq16bUWiC*Wi\0"
  "svld4_vnum_u8\0"
  "q64Ucq16bUcC*Wi\0"
  "svld4q_bf16\0"
  "svld4q_f16\0"
  "svld4q_f32\0"
  "svld4q_f64\0"
  "svld4q_mf8\0"
  "svld4q_s16\0"
  "svld4q_s32\0"
  "svld4q_s64\0"
  "svld4q_s8\0"
  "svld4q_u16\0"
  "svld4q_u32\0"
  "svld4q_u64\0"
  "svld4q_u8\0"
  "svld4q_vnum_bf16\0"
  "svld4q_vnum_f16\0"
  "svld4q_vnum_f32\0"
  "svld4q_vnum_f64\0"
  "svld4q_vnum_mf8\0"
  "svld4q_vnum_s16\0"
  "svld4q_vnum_s32\0"
  "svld4q_vnum_s64\0"
  "svld4q_vnum_s8\0"
  "svld4q_vnum_u16\0"
  "svld4q_vnum_u32\0"
  "svld4q_vnum_u64\0"
  "svld4q_vnum_u8\0"
  "svldff1_bf16\0"
  "svldff1_f16\0"
  "svldff1_f32\0"
  "svldff1_f64\0"
  "svldff1_gather_s32index_f32\0"
  "svldff1_gather_s32index_s32\0"
  "svldff1_gather_s32index_u32\0"
  "svldff1_gather_s32offset_f32\0"
  "svldff1_gather_s32offset_s32\0"
  "svldff1_gather_s32offset_u32\0"
  "svldff1_gather_s64index_f64\0"
  "svldff1_gather_s64index_s64\0"
  "svldff1_gather_s64index_u64\0"
  "svldff1_gather_s64offset_f64\0"
  "svldff1_gather_s64offset_s64\0"
  "svldff1_gather_s64offset_u64\0"
  "svldff1_gather_u32base_f32\0"
  "svldff1_gather_u32base_index_f32\0"
  "svldff1_gather_u32base_index_s32\0"
  "svldff1_gather_u32base_index_u32\0"
  "svldff1_gather_u32base_offset_f32\0"
  "svldff1_gather_u32base_offset_s32\0"
  "svldff1_gather_u32base_offset_u32\0"
  "svldff1_gather_u32base_s32\0"
  "svldff1_gather_u32base_u32\0"
  "svldff1_gather_u32index_f32\0"
  "svldff1_gather_u32index_s32\0"
  "svldff1_gather_u32index_u32\0"
  "svldff1_gather_u32offset_f32\0"
  "svldff1_gather_u32offset_s32\0"
  "svldff1_gather_u32offset_u32\0"
  "svldff1_gather_u64base_f64\0"
  "svldff1_gather_u64base_index_f64\0"
  "svldff1_gather_u64base_index_s64\0"
  "svldff1_gather_u64base_index_u64\0"
  "svldff1_gather_u64base_offset_f64\0"
  "svldff1_gather_u64base_offset_s64\0"
  "svldff1_gather_u64base_offset_u64\0"
  "svldff1_gather_u64base_s64\0"
  "svldff1_gather_u64base_u64\0"
  "svldff1_gather_u64index_f64\0"
  "svldff1_gather_u64index_s64\0"
  "svldff1_gather_u64index_u64\0"
  "svldff1_gather_u64offset_f64\0"
  "svldff1_gather_u64offset_s64\0"
  "svldff1_gather_u64offset_u64\0"
  "svldff1_mf8\0"
  "svldff1_s16\0"
  "svldff1_s32\0"
  "svldff1_s64\0"
  "svldff1_s8\0"
  "svldff1_u16\0"
  "svldff1_u32\0"
  "svldff1_u64\0"
  "svldff1_u8\0"
  "svldff1_vnum_bf16\0"
  "svldff1_vnum_f16\0"
  "svldff1_vnum_f32\0"
  "svldff1_vnum_f64\0"
  "svldff1_vnum_mf8\0"
  "svldff1_vnum_s16\0"
  "svldff1_vnum_s32\0"
  "svldff1_vnum_s64\0"
  "svldff1_vnum_s8\0"
  "svldff1_vnum_u16\0"
  "svldff1_vnum_u32\0"
  "svldff1_vnum_u64\0"
  "svldff1_vnum_u8\0"
  "svldff1sb_gather_s32offset_s32\0"
  "svldff1sb_gather_s32offset_u32\0"
  "svldff1sb_gather_s64offset_s64\0"
  "svldff1sb_gather_s64offset_u64\0"
  "svldff1sb_gather_u32base_offset_s32\0"
  "svldff1sb_gather_u32base_offset_u32\0"
  "svldff1sb_gather_u32base_s32\0"
  "svldff1sb_gather_u32base_u32\0"
  "svldff1sb_gather_u32offset_s32\0"
  "svldff1sb_gather_u32offset_u32\0"
  "svldff1sb_gather_u64base_offset_s64\0"
  "svldff1sb_gather_u64base_offset_u64\0"
  "svldff1sb_gather_u64base_s64\0"
  "svldff1sb_gather_u64base_u64\0"
  "svldff1sb_gather_u64offset_s64\0"
  "svldff1sb_gather_u64offset_u64\0"
  "svldff1sb_s16\0"
  "svldff1sb_s32\0"
  "svldff1sb_s64\0"
  "svldff1sb_u16\0"
  "svldff1sb_u32\0"
  "svldff1sb_u64\0"
  "svldff1sb_vnum_s16\0"
  "svldff1sb_vnum_s32\0"
  "svldff1sb_vnum_s64\0"
  "svldff1sb_vnum_u16\0"
  "svldff1sb_vnum_u32\0"
  "svldff1sb_vnum_u64\0"
  "svldff1sh_gather_s32index_s32\0"
  "svldff1sh_gather_s32index_u32\0"
  "svldff1sh_gather_s32offset_s32\0"
  "svldff1sh_gather_s32offset_u32\0"
  "svldff1sh_gather_s64index_s64\0"
  "svldff1sh_gather_s64index_u64\0"
  "svldff1sh_gather_s64offset_s64\0"
  "svldff1sh_gather_s64offset_u64\0"
  "svldff1sh_gather_u32base_index_s32\0"
  "svldff1sh_gather_u32base_index_u32\0"
  "svldff1sh_gather_u32base_offset_s32\0"
  "svldff1sh_gather_u32base_offset_u32\0"
  "svldff1sh_gather_u32base_s32\0"
  "svldff1sh_gather_u32base_u32\0"
  "svldff1sh_gather_u32index_s32\0"
  "svldff1sh_gather_u32index_u32\0"
  "svldff1sh_gather_u32offset_s32\0"
  "svldff1sh_gather_u32offset_u32\0"
  "svldff1sh_gather_u64base_index_s64\0"
  "svldff1sh_gather_u64base_index_u64\0"
  "svldff1sh_gather_u64base_offset_s64\0"
  "svldff1sh_gather_u64base_offset_u64\0"
  "svldff1sh_gather_u64base_s64\0"
  "svldff1sh_gather_u64base_u64\0"
  "svldff1sh_gather_u64index_s64\0"
  "svldff1sh_gather_u64index_u64\0"
  "svldff1sh_gather_u64offset_s64\0"
  "svldff1sh_gather_u64offset_u64\0"
  "svldff1sh_s32\0"
  "svldff1sh_s64\0"
  "svldff1sh_u32\0"
  "svldff1sh_u64\0"
  "svldff1sh_vnum_s32\0"
  "svldff1sh_vnum_s64\0"
  "svldff1sh_vnum_u32\0"
  "svldff1sh_vnum_u64\0"
  "svldff1sw_gather_s64index_s64\0"
  "svldff1sw_gather_s64index_u64\0"
  "svldff1sw_gather_s64offset_s64\0"
  "svldff1sw_gather_s64offset_u64\0"
  "svldff1sw_gather_u64base_index_s64\0"
  "svldff1sw_gather_u64base_index_u64\0"
  "svldff1sw_gather_u64base_offset_s64\0"
  "svldff1sw_gather_u64base_offset_u64\0"
  "svldff1sw_gather_u64base_s64\0"
  "svldff1sw_gather_u64base_u64\0"
  "svldff1sw_gather_u64index_s64\0"
  "svldff1sw_gather_u64index_u64\0"
  "svldff1sw_gather_u64offset_s64\0"
  "svldff1sw_gather_u64offset_u64\0"
  "svldff1sw_s64\0"
  "svldff1sw_u64\0"
  "svldff1sw_vnum_s64\0"
  "svldff1sw_vnum_u64\0"
  "svldff1ub_gather_s32offset_s32\0"
  "svldff1ub_gather_s32offset_u32\0"
  "svldff1ub_gather_s64offset_s64\0"
  "svldff1ub_gather_s64offset_u64\0"
  "svldff1ub_gather_u32base_offset_s32\0"
  "svldff1ub_gather_u32base_offset_u32\0"
  "svldff1ub_gather_u32base_s32\0"
  "svldff1ub_gather_u32base_u32\0"
  "svldff1ub_gather_u32offset_s32\0"
  "svldff1ub_gather_u32offset_u32\0"
  "svldff1ub_gather_u64base_offset_s64\0"
  "svldff1ub_gather_u64base_offset_u64\0"
  "svldff1ub_gather_u64base_s64\0"
  "svldff1ub_gather_u64base_u64\0"
  "svldff1ub_gather_u64offset_s64\0"
  "svldff1ub_gather_u64offset_u64\0"
  "svldff1ub_s16\0"
  "svldff1ub_s32\0"
  "svldff1ub_s64\0"
  "svldff1ub_u16\0"
  "svldff1ub_u32\0"
  "svldff1ub_u64\0"
  "svldff1ub_vnum_s16\0"
  "svldff1ub_vnum_s32\0"
  "svldff1ub_vnum_s64\0"
  "svldff1ub_vnum_u16\0"
  "svldff1ub_vnum_u32\0"
  "svldff1ub_vnum_u64\0"
  "svldff1uh_gather_s32index_s32\0"
  "svldff1uh_gather_s32index_u32\0"
  "svldff1uh_gather_s32offset_s32\0"
  "svldff1uh_gather_s32offset_u32\0"
  "svldff1uh_gather_s64index_s64\0"
  "svldff1uh_gather_s64index_u64\0"
  "svldff1uh_gather_s64offset_s64\0"
  "svldff1uh_gather_s64offset_u64\0"
  "svldff1uh_gather_u32base_index_s32\0"
  "svldff1uh_gather_u32base_index_u32\0"
  "svldff1uh_gather_u32base_offset_s32\0"
  "svldff1uh_gather_u32base_offset_u32\0"
  "svldff1uh_gather_u32base_s32\0"
  "svldff1uh_gather_u32base_u32\0"
  "svldff1uh_gather_u32index_s32\0"
  "svldff1uh_gather_u32index_u32\0"
  "svldff1uh_gather_u32offset_s32\0"
  "svldff1uh_gather_u32offset_u32\0"
  "svldff1uh_gather_u64base_index_s64\0"
  "svldff1uh_gather_u64base_index_u64\0"
  "svldff1uh_gather_u64base_offset_s64\0"
  "svldff1uh_gather_u64base_offset_u64\0"
  "svldff1uh_gather_u64base_s64\0"
  "svldff1uh_gather_u64base_u64\0"
  "svldff1uh_gather_u64index_s64\0"
  "svldff1uh_gather_u64index_u64\0"
  "svldff1uh_gather_u64offset_s64\0"
  "svldff1uh_gather_u64offset_u64\0"
  "svldff1uh_s32\0"
  "svldff1uh_s64\0"
  "svldff1uh_u32\0"
  "svldff1uh_u64\0"
  "svldff1uh_vnum_s32\0"
  "svldff1uh_vnum_s64\0"
  "svldff1uh_vnum_u32\0"
  "svldff1uh_vnum_u64\0"
  "svldff1uw_gather_s64index_s64\0"
  "svldff1uw_gather_s64index_u64\0"
  "svldff1uw_gather_s64offset_s64\0"
  "svldff1uw_gather_s64offset_u64\0"
  "svldff1uw_gather_u64base_index_s64\0"
  "svldff1uw_gather_u64base_index_u64\0"
  "svldff1uw_gather_u64base_offset_s64\0"
  "svldff1uw_gather_u64base_offset_u64\0"
  "svldff1uw_gather_u64base_s64\0"
  "svldff1uw_gather_u64base_u64\0"
  "svldff1uw_gather_u64index_s64\0"
  "svldff1uw_gather_u64index_u64\0"
  "svldff1uw_gather_u64offset_s64\0"
  "svldff1uw_gather_u64offset_u64\0"
  "svldff1uw_s64\0"
  "svldff1uw_u64\0"
  "svldff1uw_vnum_s64\0"
  "svldff1uw_vnum_u64\0"
  "svldnf1_bf16\0"
  "svldnf1_f16\0"
  "svldnf1_f32\0"
  "svldnf1_f64\0"
  "svldnf1_mf8\0"
  "svldnf1_s16\0"
  "svldnf1_s32\0"
  "svldnf1_s64\0"
  "svldnf1_s8\0"
  "svldnf1_u16\0"
  "svldnf1_u32\0"
  "svldnf1_u64\0"
  "svldnf1_u8\0"
  "svldnf1_vnum_bf16\0"
  "svldnf1_vnum_f16\0"
  "svldnf1_vnum_f32\0"
  "svldnf1_vnum_f64\0"
  "svldnf1_vnum_mf8\0"
  "svldnf1_vnum_s16\0"
  "svldnf1_vnum_s32\0"
  "svldnf1_vnum_s64\0"
  "svldnf1_vnum_s8\0"
  "svldnf1_vnum_u16\0"
  "svldnf1_vnum_u32\0"
  "svldnf1_vnum_u64\0"
  "svldnf1_vnum_u8\0"
  "svldnf1sb_s16\0"
  "svldnf1sb_s32\0"
  "svldnf1sb_s64\0"
  "svldnf1sb_u16\0"
  "svldnf1sb_u32\0"
  "svldnf1sb_u64\0"
  "svldnf1sb_vnum_s16\0"
  "svldnf1sb_vnum_s32\0"
  "svldnf1sb_vnum_s64\0"
  "svldnf1sb_vnum_u16\0"
  "svldnf1sb_vnum_u32\0"
  "svldnf1sb_vnum_u64\0"
  "svldnf1sh_s32\0"
  "svldnf1sh_s64\0"
  "svldnf1sh_u32\0"
  "svldnf1sh_u64\0"
  "svldnf1sh_vnum_s32\0"
  "svldnf1sh_vnum_s64\0"
  "svldnf1sh_vnum_u32\0"
  "svldnf1sh_vnum_u64\0"
  "svldnf1sw_s64\0"
  "svldnf1sw_u64\0"
  "svldnf1sw_vnum_s64\0"
  "svldnf1sw_vnum_u64\0"
  "svldnf1ub_s16\0"
  "svldnf1ub_s32\0"
  "svldnf1ub_s64\0"
  "svldnf1ub_u16\0"
  "svldnf1ub_u32\0"
  "svldnf1ub_u64\0"
  "svldnf1ub_vnum_s16\0"
  "svldnf1ub_vnum_s32\0"
  "svldnf1ub_vnum_s64\0"
  "svldnf1ub_vnum_u16\0"
  "svldnf1ub_vnum_u32\0"
  "svldnf1ub_vnum_u64\0"
  "svldnf1uh_s32\0"
  "svldnf1uh_s64\0"
  "svldnf1uh_u32\0"
  "svldnf1uh_u64\0"
  "svldnf1uh_vnum_s32\0"
  "svldnf1uh_vnum_s64\0"
  "svldnf1uh_vnum_u32\0"
  "svldnf1uh_vnum_u64\0"
  "svldnf1uw_s64\0"
  "svldnf1uw_u64\0"
  "svldnf1uw_vnum_s64\0"
  "svldnf1uw_vnum_u64\0"
  "svldnt1_bf16\0"
  "svldnt1_bf16_x2\0"
  "svldnt1_bf16_x4\0"
  "svldnt1_f16\0"
  "svldnt1_f16_x2\0"
  "svldnt1_f16_x4\0"
  "svldnt1_f32\0"
  "svldnt1_f32_x2\0"
  "svldnt1_f32_x4\0"
  "svldnt1_f64\0"
  "svldnt1_f64_x2\0"
  "svldnt1_f64_x4\0"
  "svldnt1_gather_s64index_f64\0"
  "svldnt1_gather_s64index_s64\0"
  "svldnt1_gather_s64index_u64\0"
  "svldnt1_gather_s64offset_f64\0"
  "svldnt1_gather_s64offset_s64\0"
  "svldnt1_gather_s64offset_u64\0"
  "svldnt1_gather_u32base_f32\0"
  "svldnt1_gather_u32base_index_f32\0"
  "svldnt1_gather_u32base_index_s32\0"
  "svldnt1_gather_u32base_index_u32\0"
  "svldnt1_gather_u32base_offset_f32\0"
  "svldnt1_gather_u32base_offset_s32\0"
  "svldnt1_gather_u32base_offset_u32\0"
  "svldnt1_gather_u32base_s32\0"
  "svldnt1_gather_u32base_u32\0"
  "svldnt1_gather_u32offset_f32\0"
  "svldnt1_gather_u32offset_s32\0"
  "svldnt1_gather_u32offset_u32\0"
  "svldnt1_gather_u64base_f64\0"
  "svldnt1_gather_u64base_index_f64\0"
  "svldnt1_gather_u64base_index_s64\0"
  "svldnt1_gather_u64base_index_u64\0"
  "svldnt1_gather_u64base_offset_f64\0"
  "svldnt1_gather_u64base_offset_s64\0"
  "svldnt1_gather_u64base_offset_u64\0"
  "svldnt1_gather_u64base_s64\0"
  "svldnt1_gather_u64base_u64\0"
  "svldnt1_gather_u64index_f64\0"
  "svldnt1_gather_u64index_s64\0"
  "svldnt1_gather_u64index_u64\0"
  "svldnt1_gather_u64offset_f64\0"
  "svldnt1_gather_u64offset_s64\0"
  "svldnt1_gather_u64offset_u64\0"
  "svldnt1_mf8\0"
  "svldnt1_mf8_x2\0"
  "svldnt1_mf8_x4\0"
  "svldnt1_s16\0"
  "svldnt1_s16_x2\0"
  "svldnt1_s16_x4\0"
  "svldnt1_s32\0"
  "svldnt1_s32_x2\0"
  "svldnt1_s32_x4\0"
  "svldnt1_s64\0"
  "svldnt1_s64_x2\0"
  "svldnt1_s64_x4\0"
  "svldnt1_s8\0"
  "svldnt1_s8_x2\0"
  "svldnt1_s8_x4\0"
  "svldnt1_u16\0"
  "svldnt1_u16_x2\0"
  "svldnt1_u16_x4\0"
  "svldnt1_u32\0"
  "svldnt1_u32_x2\0"
  "svldnt1_u32_x4\0"
  "svldnt1_u64\0"
  "svldnt1_u64_x2\0"
  "svldnt1_u64_x4\0"
  "svldnt1_u8\0"
  "svldnt1_u8_x2\0"
  "svldnt1_u8_x4\0"
  "svldnt1_vnum_bf16\0"
  "svldnt1_vnum_bf16_x2\0"
  "svldnt1_vnum_bf16_x4\0"
  "svldnt1_vnum_f16\0"
  "svldnt1_vnum_f16_x2\0"
  "svldnt1_vnum_f16_x4\0"
  "svldnt1_vnum_f32\0"
  "svldnt1_vnum_f32_x2\0"
  "svldnt1_vnum_f32_x4\0"
  "svldnt1_vnum_f64\0"
  "svldnt1_vnum_f64_x2\0"
  "svldnt1_vnum_f64_x4\0"
  "svldnt1_vnum_mf8\0"
  "svldnt1_vnum_mf8_x2\0"
  "svldnt1_vnum_mf8_x4\0"
  "svldnt1_vnum_s16\0"
  "svldnt1_vnum_s16_x2\0"
  "svldnt1_vnum_s16_x4\0"
  "svldnt1_vnum_s32\0"
  "svldnt1_vnum_s32_x2\0"
  "svldnt1_vnum_s32_x4\0"
  "svldnt1_vnum_s64\0"
  "svldnt1_vnum_s64_x2\0"
  "svldnt1_vnum_s64_x4\0"
  "svldnt1_vnum_s8\0"
  "svldnt1_vnum_s8_x2\0"
  "svldnt1_vnum_s8_x4\0"
  "svldnt1_vnum_u16\0"
  "svldnt1_vnum_u16_x2\0"
  "svldnt1_vnum_u16_x4\0"
  "svldnt1_vnum_u32\0"
  "svldnt1_vnum_u32_x2\0"
  "svldnt1_vnum_u32_x4\0"
  "svldnt1_vnum_u64\0"
  "svldnt1_vnum_u64_x2\0"
  "svldnt1_vnum_u64_x4\0"
  "svldnt1_vnum_u8\0"
  "svldnt1_vnum_u8_x2\0"
  "svldnt1_vnum_u8_x4\0"
  "svldnt1sb_gather_s64offset_s64\0"
  "svldnt1sb_gather_s64offset_u64\0"
  "svldnt1sb_gather_u32base_offset_s32\0"
  "svldnt1sb_gather_u32base_offset_u32\0"
  "svldnt1sb_gather_u32base_s32\0"
  "svldnt1sb_gather_u32base_u32\0"
  "svldnt1sb_gather_u32offset_s32\0"
  "svldnt1sb_gather_u32offset_u32\0"
  "svldnt1sb_gather_u64base_offset_s64\0"
  "svldnt1sb_gather_u64base_offset_u64\0"
  "svldnt1sb_gather_u64base_s64\0"
  "svldnt1sb_gather_u64base_u64\0"
  "svldnt1sb_gather_u64offset_s64\0"
  "svldnt1sb_gather_u64offset_u64\0"
  "svldnt1sh_gather_s64index_s64\0"
  "svldnt1sh_gather_s64index_u64\0"
  "svldnt1sh_gather_s64offset_s64\0"
  "svldnt1sh_gather_s64offset_u64\0"
  "svldnt1sh_gather_u32base_index_s32\0"
  "svldnt1sh_gather_u32base_index_u32\0"
  "svldnt1sh_gather_u32base_offset_s32\0"
  "svldnt1sh_gather_u32base_offset_u32\0"
  "svldnt1sh_gather_u32base_s32\0"
  "svldnt1sh_gather_u32base_u32\0"
  "svldnt1sh_gather_u32offset_s32\0"
  "svldnt1sh_gather_u32offset_u32\0"
  "svldnt1sh_gather_u64base_index_s64\0"
  "svldnt1sh_gather_u64base_index_u64\0"
  "svldnt1sh_gather_u64base_offset_s64\0"
  "svldnt1sh_gather_u64base_offset_u64\0"
  "svldnt1sh_gather_u64base_s64\0"
  "svldnt1sh_gather_u64base_u64\0"
  "svldnt1sh_gather_u64index_s64\0"
  "svldnt1sh_gather_u64index_u64\0"
  "svldnt1sh_gather_u64offset_s64\0"
  "svldnt1sh_gather_u64offset_u64\0"
  "svldnt1sw_gather_s64index_s64\0"
  "svldnt1sw_gather_s64index_u64\0"
  "svldnt1sw_gather_s64offset_s64\0"
  "svldnt1sw_gather_s64offset_u64\0"
  "svldnt1sw_gather_u64base_index_s64\0"
  "svldnt1sw_gather_u64base_index_u64\0"
  "svldnt1sw_gather_u64base_offset_s64\0"
  "svldnt1sw_gather_u64base_offset_u64\0"
  "svldnt1sw_gather_u64base_s64\0"
  "svldnt1sw_gather_u64base_u64\0"
  "svldnt1sw_gather_u64index_s64\0"
  "svldnt1sw_gather_u64index_u64\0"
  "svldnt1sw_gather_u64offset_s64\0"
  "svldnt1sw_gather_u64offset_u64\0"
  "svldnt1ub_gather_s64offset_s64\0"
  "svldnt1ub_gather_s64offset_u64\0"
  "svldnt1ub_gather_u32base_offset_s32\0"
  "svldnt1ub_gather_u32base_offset_u32\0"
  "svldnt1ub_gather_u32base_s32\0"
  "svldnt1ub_gather_u32base_u32\0"
  "svldnt1ub_gather_u32offset_s32\0"
  "svldnt1ub_gather_u32offset_u32\0"
  "svldnt1ub_gather_u64base_offset_s64\0"
  "svldnt1ub_gather_u64base_offset_u64\0"
  "svldnt1ub_gather_u64base_s64\0"
  "svldnt1ub_gather_u64base_u64\0"
  "svldnt1ub_gather_u64offset_s64\0"
  "svldnt1ub_gather_u64offset_u64\0"
  "svldnt1uh_gather_s64index_s64\0"
  "svldnt1uh_gather_s64index_u64\0"
  "svldnt1uh_gather_s64offset_s64\0"
  "svldnt1uh_gather_s64offset_u64\0"
  "svldnt1uh_gather_u32base_index_s32\0"
  "svldnt1uh_gather_u32base_index_u32\0"
  "svldnt1uh_gather_u32base_offset_s32\0"
  "svldnt1uh_gather_u32base_offset_u32\0"
  "svldnt1uh_gather_u32base_s32\0"
  "svldnt1uh_gather_u32base_u32\0"
  "svldnt1uh_gather_u32offset_s32\0"
  "svldnt1uh_gather_u32offset_u32\0"
  "svldnt1uh_gather_u64base_index_s64\0"
  "svldnt1uh_gather_u64base_index_u64\0"
  "svldnt1uh_gather_u64base_offset_s64\0"
  "svldnt1uh_gather_u64base_offset_u64\0"
  "svldnt1uh_gather_u64base_s64\0"
  "svldnt1uh_gather_u64base_u64\0"
  "svldnt1uh_gather_u64index_s64\0"
  "svldnt1uh_gather_u64index_u64\0"
  "svldnt1uh_gather_u64offset_s64\0"
  "svldnt1uh_gather_u64offset_u64\0"
  "svldnt1uw_gather_s64index_s64\0"
  "svldnt1uw_gather_s64index_u64\0"
  "svldnt1uw_gather_s64offset_s64\0"
  "svldnt1uw_gather_s64offset_u64\0"
  "svldnt1uw_gather_u64base_index_s64\0"
  "svldnt1uw_gather_u64base_index_u64\0"
  "svldnt1uw_gather_u64base_offset_s64\0"
  "svldnt1uw_gather_u64base_offset_u64\0"
  "svldnt1uw_gather_u64base_s64\0"
  "svldnt1uw_gather_u64base_u64\0"
  "svldnt1uw_gather_u64index_s64\0"
  "svldnt1uw_gather_u64index_u64\0"
  "svldnt1uw_gather_u64offset_s64\0"
  "svldnt1uw_gather_u64offset_u64\0"
  "svlen_bf16\0"
  "UWiq8y\0"
  "svlen_f16\0"
  "UWiq8h\0"
  "svlen_f32\0"
  "UWiq4f\0"
  "svlen_f64\0"
  "UWiq2d\0"
  "svlen_s16\0"
  "UWiq8s\0"
  "svlen_s32\0"
  "UWiq4i\0"
  "svlen_s64\0"
  "UWiq2Wi\0"
  "svlen_s8\0"
  "UWiq16Sc\0"
  "svlen_u16\0"
  "UWiq8Us\0"
  "svlen_u32\0"
  "UWiq4Ui\0"
  "svlen_u64\0"
  "UWiq2UWi\0"
  "svlen_u8\0"
  "UWiq16Uc\0"
  "svlogb_f16_m\0"
  "svlogb_f16_x\0"
  "svlogb_f16_z\0"
  "svlogb_f32_m\0"
  "svlogb_f32_x\0"
  "svlogb_f32_z\0"
  "svlogb_f64_m\0"
  "svlogb_f64_x\0"
  "svlogb_f64_z\0"
  "svlsl_n_s16_m\0"
  "svlsl_n_s16_x\0"
  "svlsl_n_s16_z\0"
  "svlsl_n_s32_m\0"
  "svlsl_n_s32_x\0"
  "svlsl_n_s32_z\0"
  "svlsl_n_s64_m\0"
  "svlsl_n_s64_x\0"
  "svlsl_n_s64_z\0"
  "svlsl_n_s8_m\0"
  "svlsl_n_s8_x\0"
  "svlsl_n_s8_z\0"
  "svlsl_n_u16_m\0"
  "svlsl_n_u16_x\0"
  "svlsl_n_u16_z\0"
  "svlsl_n_u32_m\0"
  "svlsl_n_u32_x\0"
  "svlsl_n_u32_z\0"
  "svlsl_n_u64_m\0"
  "svlsl_n_u64_x\0"
  "svlsl_n_u64_z\0"
  "svlsl_n_u8_m\0"
  "svlsl_n_u8_x\0"
  "svlsl_n_u8_z\0"
  "svlsl_s16_m\0"
  "svlsl_s16_x\0"
  "svlsl_s16_z\0"
  "svlsl_s32_m\0"
  "svlsl_s32_x\0"
  "svlsl_s32_z\0"
  "svlsl_s64_m\0"
  "svlsl_s64_x\0"
  "svlsl_s64_z\0"
  "svlsl_s8_m\0"
  "svlsl_s8_x\0"
  "svlsl_s8_z\0"
  "svlsl_u16_m\0"
  "svlsl_u16_x\0"
  "svlsl_u16_z\0"
  "svlsl_u32_m\0"
  "svlsl_u32_x\0"
  "svlsl_u32_z\0"
  "svlsl_u64_m\0"
  "svlsl_u64_x\0"
  "svlsl_u64_z\0"
  "svlsl_u8_m\0"
  "svlsl_u8_x\0"
  "svlsl_u8_z\0"
  "svlsl_wide_n_s16_m\0"
  "svlsl_wide_n_s16_x\0"
  "svlsl_wide_n_s16_z\0"
  "svlsl_wide_n_s32_m\0"
  "svlsl_wide_n_s32_x\0"
  "svlsl_wide_n_s32_z\0"
  "svlsl_wide_n_s8_m\0"
  "svlsl_wide_n_s8_x\0"
  "svlsl_wide_n_s8_z\0"
  "svlsl_wide_n_u16_m\0"
  "q8Usq16bq8UsUWi\0"
  "svlsl_wide_n_u16_x\0"
  "svlsl_wide_n_u16_z\0"
  "svlsl_wide_n_u32_m\0"
  "q4Uiq16bq4UiUWi\0"
  "svlsl_wide_n_u32_x\0"
  "svlsl_wide_n_u32_z\0"
  "svlsl_wide_n_u8_m\0"
  "q16Ucq16bq16UcUWi\0"
  "svlsl_wide_n_u8_x\0"
  "svlsl_wide_n_u8_z\0"
  "svlsl_wide_s16_m\0"
  "svlsl_wide_s16_x\0"
  "svlsl_wide_s16_z\0"
  "svlsl_wide_s32_m\0"
  "svlsl_wide_s32_x\0"
  "svlsl_wide_s32_z\0"
  "svlsl_wide_s8_m\0"
  "svlsl_wide_s8_x\0"
  "svlsl_wide_s8_z\0"
  "svlsl_wide_u16_m\0"
  "q8Usq16bq8Usq2UWi\0"
  "svlsl_wide_u16_x\0"
  "svlsl_wide_u16_z\0"
  "svlsl_wide_u32_m\0"
  "q4Uiq16bq4Uiq2UWi\0"
  "svlsl_wide_u32_x\0"
  "svlsl_wide_u32_z\0"
  "svlsl_wide_u8_m\0"
  "q16Ucq16bq16Ucq2UWi\0"
  "svlsl_wide_u8_x\0"
  "svlsl_wide_u8_z\0"
  "svlsr_n_u16_m\0"
  "svlsr_n_u16_x\0"
  "svlsr_n_u16_z\0"
  "svlsr_n_u32_m\0"
  "svlsr_n_u32_x\0"
  "svlsr_n_u32_z\0"
  "svlsr_n_u64_m\0"
  "svlsr_n_u64_x\0"
  "svlsr_n_u64_z\0"
  "svlsr_n_u8_m\0"
  "svlsr_n_u8_x\0"
  "svlsr_n_u8_z\0"
  "svlsr_u16_m\0"
  "svlsr_u16_x\0"
  "svlsr_u16_z\0"
  "svlsr_u32_m\0"
  "svlsr_u32_x\0"
  "svlsr_u32_z\0"
  "svlsr_u64_m\0"
  "svlsr_u64_x\0"
  "svlsr_u64_z\0"
  "svlsr_u8_m\0"
  "svlsr_u8_x\0"
  "svlsr_u8_z\0"
  "svlsr_wide_n_u16_m\0"
  "svlsr_wide_n_u16_x\0"
  "svlsr_wide_n_u16_z\0"
  "svlsr_wide_n_u32_m\0"
  "svlsr_wide_n_u32_x\0"
  "svlsr_wide_n_u32_z\0"
  "svlsr_wide_n_u8_m\0"
  "svlsr_wide_n_u8_x\0"
  "svlsr_wide_n_u8_z\0"
  "svlsr_wide_u16_m\0"
  "svlsr_wide_u16_x\0"
  "svlsr_wide_u16_z\0"
  "svlsr_wide_u32_m\0"
  "svlsr_wide_u32_x\0"
  "svlsr_wide_u32_z\0"
  "svlsr_wide_u8_m\0"
  "svlsr_wide_u8_x\0"
  "svlsr_wide_u8_z\0"
  "svluti2_lane_bf16\0"
  "q8yq8yq16UcIUWi\0"
  "(sve,(sve2|sme2),lut)|(sme,sme2,lut)\0"
  "svluti2_lane_f16\0"
  "q8hq8hq16UcIUWi\0"
  "svluti2_lane_s16\0"
  "q8sq8sq16UcIUWi\0"
  "svluti2_lane_s8\0"
  "q16Scq16Scq16UcIUWi\0"
  "svluti2_lane_u16\0"
  "q8Usq8Usq16UcIUWi\0"
  "svluti2_lane_u8\0"
  "svluti4_lane_bf16\0"
  "svluti4_lane_bf16_x2\0"
  "q8yq16yq16UcIUWi\0"
  "svluti4_lane_f16\0"
  "svluti4_lane_f16_x2\0"
  "q8hq16hq16UcIUWi\0"
  "svluti4_lane_s16\0"
  "svluti4_lane_s16_x2\0"
  "q8sq16sq16UcIUWi\0"
  "svluti4_lane_s8\0"
  "svluti4_lane_u16\0"
  "svluti4_lane_u16_x2\0"
  "q8Usq16Usq16UcIUWi\0"
  "svluti4_lane_u8\0"
  "svmad_f16_m\0"
  "q8hq16bq8hq8hq8h\0"
  "svmad_f16_x\0"
  "svmad_f16_z\0"
  "svmad_f32_m\0"
  "q4fq16bq4fq4fq4f\0"
  "svmad_f32_x\0"
  "svmad_f32_z\0"
  "svmad_f64_m\0"
  "q2dq16bq2dq2dq2d\0"
  "svmad_f64_x\0"
  "svmad_f64_z\0"
  "svmad_n_f16_m\0"
  "q8hq16bq8hq8hh\0"
  "svmad_n_f16_x\0"
  "svmad_n_f16_z\0"
  "svmad_n_f32_m\0"
  "q4fq16bq4fq4ff\0"
  "svmad_n_f32_x\0"
  "svmad_n_f32_z\0"
  "svmad_n_f64_m\0"
  "q2dq16bq2dq2dd\0"
  "svmad_n_f64_x\0"
  "svmad_n_f64_z\0"
  "svmad_n_s16_m\0"
  "q8sq16bq8sq8ss\0"
  "svmad_n_s16_x\0"
  "svmad_n_s16_z\0"
  "svmad_n_s32_m\0"
  "q4iq16bq4iq4ii\0"
  "svmad_n_s32_x\0"
  "svmad_n_s32_z\0"
  "svmad_n_s64_m\0"
  "q2Wiq16bq2Wiq2WiWi\0"
  "svmad_n_s64_x\0"
  "svmad_n_s64_z\0"
  "svmad_n_s8_m\0"
  "q16Scq16bq16Scq16ScSc\0"
  "svmad_n_s8_x\0"
  "svmad_n_s8_z\0"
  "svmad_n_u16_m\0"
  "q8Usq16bq8Usq8UsUs\0"
  "svmad_n_u16_x\0"
  "svmad_n_u16_z\0"
  "svmad_n_u32_m\0"
  "q4Uiq16bq4Uiq4UiUi\0"
  "svmad_n_u32_x\0"
  "svmad_n_u32_z\0"
  "svmad_n_u64_m\0"
  "q2UWiq16bq2UWiq2UWiUWi\0"
  "svmad_n_u64_x\0"
  "svmad_n_u64_z\0"
  "svmad_n_u8_m\0"
  "q16Ucq16bq16Ucq16UcUc\0"
  "svmad_n_u8_x\0"
  "svmad_n_u8_z\0"
  "svmad_s16_m\0"
  "q8sq16bq8sq8sq8s\0"
  "svmad_s16_x\0"
  "svmad_s16_z\0"
  "svmad_s32_m\0"
  "q4iq16bq4iq4iq4i\0"
  "svmad_s32_x\0"
  "svmad_s32_z\0"
  "svmad_s64_m\0"
  "q2Wiq16bq2Wiq2Wiq2Wi\0"
  "svmad_s64_x\0"
  "svmad_s64_z\0"
  "svmad_s8_m\0"
  "q16Scq16bq16Scq16Scq16Sc\0"
  "svmad_s8_x\0"
  "svmad_s8_z\0"
  "svmad_u16_m\0"
  "q8Usq16bq8Usq8Usq8Us\0"
  "svmad_u16_x\0"
  "svmad_u16_z\0"
  "svmad_u32_m\0"
  "q4Uiq16bq4Uiq4Uiq4Ui\0"
  "svmad_u32_x\0"
  "svmad_u32_z\0"
  "svmad_u64_m\0"
  "q2UWiq16bq2UWiq2UWiq2UWi\0"
  "svmad_u64_x\0"
  "svmad_u64_z\0"
  "svmad_u8_m\0"
  "q16Ucq16bq16Ucq16Ucq16Uc\0"
  "svmad_u8_x\0"
  "svmad_u8_z\0"
  "svmatch_s16\0"
  "svmatch_s8\0"
  "svmatch_u16\0"
  "svmatch_u8\0"
  "svmax_bf16_m\0"
  "svmax_bf16_x\0"
  "svmax_bf16_x2\0"
  "q16yq16yq16y\0"
  "svmax_bf16_x4\0"
  "q32yq32yq32y\0"
  "svmax_bf16_z\0"
  "svmax_f16_m\0"
  "svmax_f16_x\0"
  "svmax_f16_x2\0"
  "svmax_f16_x4\0"
  "svmax_f16_z\0"
  "svmax_f32_m\0"
  "svmax_f32_x\0"
  "svmax_f32_x2\0"
  "svmax_f32_x4\0"
  "svmax_f32_z\0"
  "svmax_f64_m\0"
  "svmax_f64_x\0"
  "svmax_f64_x2\0"
  "svmax_f64_x4\0"
  "svmax_f64_z\0"
  "svmax_n_bf16_m\0"
  "svmax_n_bf16_x\0"
  "svmax_n_bf16_z\0"
  "svmax_n_f16_m\0"
  "svmax_n_f16_x\0"
  "svmax_n_f16_z\0"
  "svmax_n_f32_m\0"
  "svmax_n_f32_x\0"
  "svmax_n_f32_z\0"
  "svmax_n_f64_m\0"
  "svmax_n_f64_x\0"
  "svmax_n_f64_z\0"
  "svmax_n_s16_m\0"
  "svmax_n_s16_x\0"
  "svmax_n_s16_z\0"
  "svmax_n_s32_m\0"
  "svmax_n_s32_x\0"
  "svmax_n_s32_z\0"
  "svmax_n_s64_m\0"
  "svmax_n_s64_x\0"
  "svmax_n_s64_z\0"
  "svmax_n_s8_m\0"
  "svmax_n_s8_x\0"
  "svmax_n_s8_z\0"
  "svmax_n_u16_m\0"
  "svmax_n_u16_x\0"
  "svmax_n_u16_z\0"
  "svmax_n_u32_m\0"
  "svmax_n_u32_x\0"
  "svmax_n_u32_z\0"
  "svmax_n_u64_m\0"
  "svmax_n_u64_x\0"
  "svmax_n_u64_z\0"
  "svmax_n_u8_m\0"
  "svmax_n_u8_x\0"
  "svmax_n_u8_z\0"
  "svmax_s16_m\0"
  "svmax_s16_x\0"
  "svmax_s16_x2\0"
  "q16sq16sq16s\0"
  "svmax_s16_x4\0"
  "q32sq32sq32s\0"
  "svmax_s16_z\0"
  "svmax_s32_m\0"
  "svmax_s32_x\0"
  "svmax_s32_x2\0"
  "q8iq8iq8i\0"
  "svmax_s32_x4\0"
  "q16iq16iq16i\0"
  "svmax_s32_z\0"
  "svmax_s64_m\0"
  "svmax_s64_x\0"
  "svmax_s64_x2\0"
  "q4Wiq4Wiq4Wi\0"
  "svmax_s64_x4\0"
  "q8Wiq8Wiq8Wi\0"
  "svmax_s64_z\0"
  "svmax_s8_m\0"
  "svmax_s8_x\0"
  "svmax_s8_x2\0"
  "q32Scq32Scq32Sc\0"
  "svmax_s8_x4\0"
  "q64Scq64Scq64Sc\0"
  "svmax_s8_z\0"
  "svmax_single_bf16_x2\0"
  "q16yq16yq8y\0"
  "svmax_single_bf16_x4\0"
  "q32yq32yq8y\0"
  "svmax_single_f16_x2\0"
  "q16hq16hq8h\0"
  "svmax_single_f16_x4\0"
  "q32hq32hq8h\0"
  "svmax_single_f32_x2\0"
  "q8fq8fq4f\0"
  "svmax_single_f32_x4\0"
  "q16fq16fq4f\0"
  "svmax_single_f64_x2\0"
  "q4dq4dq2d\0"
  "svmax_single_f64_x4\0"
  "q8dq8dq2d\0"
  "svmax_single_s16_x2\0"
  "svmax_single_s16_x4\0"
  "svmax_single_s32_x2\0"
  "svmax_single_s32_x4\0"
  "svmax_single_s64_x2\0"
  "svmax_single_s64_x4\0"
  "svmax_single_s8_x2\0"
  "svmax_single_s8_x4\0"
  "svmax_single_u16_x2\0"
  "svmax_single_u16_x4\0"
  "svmax_single_u32_x2\0"
  "svmax_single_u32_x4\0"
  "svmax_single_u64_x2\0"
  "svmax_single_u64_x4\0"
  "svmax_single_u8_x2\0"
  "svmax_single_u8_x4\0"
  "svmax_u16_m\0"
  "svmax_u16_x\0"
  "svmax_u16_x2\0"
  "q16Usq16Usq16Us\0"
  "svmax_u16_x4\0"
  "q32Usq32Usq32Us\0"
  "svmax_u16_z\0"
  "svmax_u32_m\0"
  "svmax_u32_x\0"
  "svmax_u32_x2\0"
  "q8Uiq8Uiq8Ui\0"
  "svmax_u32_x4\0"
  "q16Uiq16Uiq16Ui\0"
  "svmax_u32_z\0"
  "svmax_u64_m\0"
  "svmax_u64_x\0"
  "svmax_u64_x2\0"
  "q4UWiq4UWiq4UWi\0"
  "svmax_u64_x4\0"
  "q8UWiq8UWiq8UWi\0"
  "svmax_u64_z\0"
  "svmax_u8_m\0"
  "svmax_u8_x\0"
  "svmax_u8_x2\0"
  "q32Ucq32Ucq32Uc\0"
  "svmax_u8_x4\0"
  "q64Ucq64Ucq64Uc\0"
  "svmax_u8_z\0"
  "svmaxnm_bf16_m\0"
  "svmaxnm_bf16_x\0"
  "svmaxnm_bf16_x2\0"
  "svmaxnm_bf16_x4\0"
  "svmaxnm_bf16_z\0"
  "svmaxnm_f16_m\0"
  "svmaxnm_f16_x\0"
  "svmaxnm_f16_x2\0"
  "svmaxnm_f16_x4\0"
  "svmaxnm_f16_z\0"
  "svmaxnm_f32_m\0"
  "svmaxnm_f32_x\0"
  "svmaxnm_f32_x2\0"
  "svmaxnm_f32_x4\0"
  "svmaxnm_f32_z\0"
  "svmaxnm_f64_m\0"
  "svmaxnm_f64_x\0"
  "svmaxnm_f64_x2\0"
  "svmaxnm_f64_x4\0"
  "svmaxnm_f64_z\0"
  "svmaxnm_n_bf16_m\0"
  "svmaxnm_n_bf16_x\0"
  "svmaxnm_n_bf16_z\0"
  "svmaxnm_n_f16_m\0"
  "svmaxnm_n_f16_x\0"
  "svmaxnm_n_f16_z\0"
  "svmaxnm_n_f32_m\0"
  "svmaxnm_n_f32_x\0"
  "svmaxnm_n_f32_z\0"
  "svmaxnm_n_f64_m\0"
  "svmaxnm_n_f64_x\0"
  "svmaxnm_n_f64_z\0"
  "svmaxnm_single_bf16_x2\0"
  "svmaxnm_single_bf16_x4\0"
  "svmaxnm_single_f16_x2\0"
  "svmaxnm_single_f16_x4\0"
  "svmaxnm_single_f32_x2\0"
  "svmaxnm_single_f32_x4\0"
  "svmaxnm_single_f64_x2\0"
  "svmaxnm_single_f64_x4\0"
  "svmaxnmp_f16_m\0"
  "svmaxnmp_f16_x\0"
  "svmaxnmp_f32_m\0"
  "svmaxnmp_f32_x\0"
  "svmaxnmp_f64_m\0"
  "svmaxnmp_f64_x\0"
  "svmaxnmqv_f16\0"
  "svmaxnmqv_f32\0"
  "svmaxnmqv_f64\0"
  "svmaxnmv_f16\0"
  "svmaxnmv_f32\0"
  "svmaxnmv_f64\0"
  "svmaxp_f16_m\0"
  "svmaxp_f16_x\0"
  "svmaxp_f32_m\0"
  "svmaxp_f32_x\0"
  "svmaxp_f64_m\0"
  "svmaxp_f64_x\0"
  "svmaxp_s16_m\0"
  "svmaxp_s16_x\0"
  "svmaxp_s32_m\0"
  "svmaxp_s32_x\0"
  "svmaxp_s64_m\0"
  "svmaxp_s64_x\0"
  "svmaxp_s8_m\0"
  "svmaxp_s8_x\0"
  "svmaxp_u16_m\0"
  "svmaxp_u16_x\0"
  "svmaxp_u32_m\0"
  "svmaxp_u32_x\0"
  "svmaxp_u64_m\0"
  "svmaxp_u64_x\0"
  "svmaxp_u8_m\0"
  "svmaxp_u8_x\0"
  "svmaxqv_f16\0"
  "svmaxqv_f32\0"
  "svmaxqv_f64\0"
  "svmaxqv_s16\0"
  "svmaxqv_s32\0"
  "svmaxqv_s64\0"
  "svmaxqv_s8\0"
  "svmaxqv_u16\0"
  "svmaxqv_u32\0"
  "svmaxqv_u64\0"
  "svmaxqv_u8\0"
  "svmaxv_f16\0"
  "svmaxv_f32\0"
  "svmaxv_f64\0"
  "svmaxv_s16\0"
  "svmaxv_s32\0"
  "svmaxv_s64\0"
  "svmaxv_s8\0"
  "svmaxv_u16\0"
  "svmaxv_u32\0"
  "svmaxv_u64\0"
  "svmaxv_u8\0"
  "svmin_bf16_m\0"
  "svmin_bf16_x\0"
  "svmin_bf16_x2\0"
  "svmin_bf16_x4\0"
  "svmin_bf16_z\0"
  "svmin_f16_m\0"
  "svmin_f16_x\0"
  "svmin_f16_x2\0"
  "svmin_f16_x4\0"
  "svmin_f16_z\0"
  "svmin_f32_m\0"
  "svmin_f32_x\0"
  "svmin_f32_x2\0"
  "svmin_f32_x4\0"
  "svmin_f32_z\0"
  "svmin_f64_m\0"
  "svmin_f64_x\0"
  "svmin_f64_x2\0"
  "svmin_f64_x4\0"
  "svmin_f64_z\0"
  "svmin_n_bf16_m\0"
  "svmin_n_bf16_x\0"
  "svmin_n_bf16_z\0"
  "svmin_n_f16_m\0"
  "svmin_n_f16_x\0"
  "svmin_n_f16_z\0"
  "svmin_n_f32_m\0"
  "svmin_n_f32_x\0"
  "svmin_n_f32_z\0"
  "svmin_n_f64_m\0"
  "svmin_n_f64_x\0"
  "svmin_n_f64_z\0"
  "svmin_n_s16_m\0"
  "svmin_n_s16_x\0"
  "svmin_n_s16_z\0"
  "svmin_n_s32_m\0"
  "svmin_n_s32_x\0"
  "svmin_n_s32_z\0"
  "svmin_n_s64_m\0"
  "svmin_n_s64_x\0"
  "svmin_n_s64_z\0"
  "svmin_n_s8_m\0"
  "svmin_n_s8_x\0"
  "svmin_n_s8_z\0"
  "svmin_n_u16_m\0"
  "svmin_n_u16_x\0"
  "svmin_n_u16_z\0"
  "svmin_n_u32_m\0"
  "svmin_n_u32_x\0"
  "svmin_n_u32_z\0"
  "svmin_n_u64_m\0"
  "svmin_n_u64_x\0"
  "svmin_n_u64_z\0"
  "svmin_n_u8_m\0"
  "svmin_n_u8_x\0"
  "svmin_n_u8_z\0"
  "svmin_s16_m\0"
  "svmin_s16_x\0"
  "svmin_s16_x2\0"
  "svmin_s16_x4\0"
  "svmin_s16_z\0"
  "svmin_s32_m\0"
  "svmin_s32_x\0"
  "svmin_s32_x2\0"
  "svmin_s32_x4\0"
  "svmin_s32_z\0"
  "svmin_s64_m\0"
  "svmin_s64_x\0"
  "svmin_s64_x2\0"
  "svmin_s64_x4\0"
  "svmin_s64_z\0"
  "svmin_s8_m\0"
  "svmin_s8_x\0"
  "svmin_s8_x2\0"
  "svmin_s8_x4\0"
  "svmin_s8_z\0"
  "svmin_single_bf16_x2\0"
  "svmin_single_bf16_x4\0"
  "svmin_single_f16_x2\0"
  "svmin_single_f16_x4\0"
  "svmin_single_f32_x2\0"
  "svmin_single_f32_x4\0"
  "svmin_single_f64_x2\0"
  "svmin_single_f64_x4\0"
  "svmin_single_s16_x2\0"
  "svmin_single_s16_x4\0"
  "svmin_single_s32_x2\0"
  "svmin_single_s32_x4\0"
  "svmin_single_s64_x2\0"
  "svmin_single_s64_x4\0"
  "svmin_single_s8_x2\0"
  "svmin_single_s8_x4\0"
  "svmin_single_u16_x2\0"
  "svmin_single_u16_x4\0"
  "svmin_single_u32_x2\0"
  "svmin_single_u32_x4\0"
  "svmin_single_u64_x2\0"
  "svmin_single_u64_x4\0"
  "svmin_single_u8_x2\0"
  "svmin_single_u8_x4\0"
  "svmin_u16_m\0"
  "svmin_u16_x\0"
  "svmin_u16_x2\0"
  "svmin_u16_x4\0"
  "svmin_u16_z\0"
  "svmin_u32_m\0"
  "svmin_u32_x\0"
  "svmin_u32_x2\0"
  "svmin_u32_x4\0"
  "svmin_u32_z\0"
  "svmin_u64_m\0"
  "svmin_u64_x\0"
  "svmin_u64_x2\0"
  "svmin_u64_x4\0"
  "svmin_u64_z\0"
  "svmin_u8_m\0"
  "svmin_u8_x\0"
  "svmin_u8_x2\0"
  "svmin_u8_x4\0"
  "svmin_u8_z\0"
  "svminnm_bf16_m\0"
  "svminnm_bf16_x\0"
  "svminnm_bf16_x2\0"
  "svminnm_bf16_x4\0"
  "svminnm_bf16_z\0"
  "svminnm_f16_m\0"
  "svminnm_f16_x\0"
  "svminnm_f16_x2\0"
  "svminnm_f16_x4\0"
  "svminnm_f16_z\0"
  "svminnm_f32_m\0"
  "svminnm_f32_x\0"
  "svminnm_f32_x2\0"
  "svminnm_f32_x4\0"
  "svminnm_f32_z\0"
  "svminnm_f64_m\0"
  "svminnm_f64_x\0"
  "svminnm_f64_x2\0"
  "svminnm_f64_x4\0"
  "svminnm_f64_z\0"
  "svminnm_n_bf16_m\0"
  "svminnm_n_bf16_x\0"
  "svminnm_n_bf16_z\0"
  "svminnm_n_f16_m\0"
  "svminnm_n_f16_x\0"
  "svminnm_n_f16_z\0"
  "svminnm_n_f32_m\0"
  "svminnm_n_f32_x\0"
  "svminnm_n_f32_z\0"
  "svminnm_n_f64_m\0"
  "svminnm_n_f64_x\0"
  "svminnm_n_f64_z\0"
  "svminnm_single_bf16_x2\0"
  "svminnm_single_bf16_x4\0"
  "svminnm_single_f16_x2\0"
  "svminnm_single_f16_x4\0"
  "svminnm_single_f32_x2\0"
  "svminnm_single_f32_x4\0"
  "svminnm_single_f64_x2\0"
  "svminnm_single_f64_x4\0"
  "svminnmp_f16_m\0"
  "svminnmp_f16_x\0"
  "svminnmp_f32_m\0"
  "svminnmp_f32_x\0"
  "svminnmp_f64_m\0"
  "svminnmp_f64_x\0"
  "svminnmqv_f16\0"
  "svminnmqv_f32\0"
  "svminnmqv_f64\0"
  "svminnmv_f16\0"
  "svminnmv_f32\0"
  "svminnmv_f64\0"
  "svminp_f16_m\0"
  "svminp_f16_x\0"
  "svminp_f32_m\0"
  "svminp_f32_x\0"
  "svminp_f64_m\0"
  "svminp_f64_x\0"
  "svminp_s16_m\0"
  "svminp_s16_x\0"
  "svminp_s32_m\0"
  "svminp_s32_x\0"
  "svminp_s64_m\0"
  "svminp_s64_x\0"
  "svminp_s8_m\0"
  "svminp_s8_x\0"
  "svminp_u16_m\0"
  "svminp_u16_x\0"
  "svminp_u32_m\0"
  "svminp_u32_x\0"
  "svminp_u64_m\0"
  "svminp_u64_x\0"
  "svminp_u8_m\0"
  "svminp_u8_x\0"
  "svminqv_f16\0"
  "svminqv_f32\0"
  "svminqv_f64\0"
  "svminqv_s16\0"
  "svminqv_s32\0"
  "svminqv_s64\0"
  "svminqv_s8\0"
  "svminqv_u16\0"
  "svminqv_u32\0"
  "svminqv_u64\0"
  "svminqv_u8\0"
  "svminv_f16\0"
  "svminv_f32\0"
  "svminv_f64\0"
  "svminv_s16\0"
  "svminv_s32\0"
  "svminv_s64\0"
  "svminv_s8\0"
  "svminv_u16\0"
  "svminv_u32\0"
  "svminv_u64\0"
  "svminv_u8\0"
  "svmla_bf16_m\0"
  "q8yq16bq8yq8yq8y\0"
  "svmla_bf16_x\0"
  "svmla_bf16_z\0"
  "svmla_f16_m\0"
  "svmla_f16_x\0"
  "svmla_f16_z\0"
  "svmla_f32_m\0"
  "svmla_f32_x\0"
  "svmla_f32_z\0"
  "svmla_f64_m\0"
  "svmla_f64_x\0"
  "svmla_f64_z\0"
  "svmla_lane_bf16\0"
  "q8yq8yq8yq8yIUWi\0"
  "svmla_lane_f16\0"
  "q8hq8hq8hq8hIUWi\0"
  "svmla_lane_f32\0"
  "q4fq4fq4fq4fIUWi\0"
  "svmla_lane_f64\0"
  "q2dq2dq2dq2dIUWi\0"
  "svmla_lane_s16\0"
  "svmla_lane_s32\0"
  "svmla_lane_s64\0"
  "svmla_lane_u16\0"
  "svmla_lane_u32\0"
  "svmla_lane_u64\0"
  "svmla_n_bf16_m\0"
  "q8yq16bq8yq8yy\0"
  "svmla_n_bf16_x\0"
  "svmla_n_bf16_z\0"
  "svmla_n_f16_m\0"
  "svmla_n_f16_x\0"
  "svmla_n_f16_z\0"
  "svmla_n_f32_m\0"
  "svmla_n_f32_x\0"
  "svmla_n_f32_z\0"
  "svmla_n_f64_m\0"
  "svmla_n_f64_x\0"
  "svmla_n_f64_z\0"
  "svmla_n_s16_m\0"
  "svmla_n_s16_x\0"
  "svmla_n_s16_z\0"
  "svmla_n_s32_m\0"
  "svmla_n_s32_x\0"
  "svmla_n_s32_z\0"
  "svmla_n_s64_m\0"
  "svmla_n_s64_x\0"
  "svmla_n_s64_z\0"
  "svmla_n_s8_m\0"
  "svmla_n_s8_x\0"
  "svmla_n_s8_z\0"
  "svmla_n_u16_m\0"
  "svmla_n_u16_x\0"
  "svmla_n_u16_z\0"
  "svmla_n_u32_m\0"
  "svmla_n_u32_x\0"
  "svmla_n_u32_z\0"
  "svmla_n_u64_m\0"
  "svmla_n_u64_x\0"
  "svmla_n_u64_z\0"
  "svmla_n_u8_m\0"
  "svmla_n_u8_x\0"
  "svmla_n_u8_z\0"
  "svmla_s16_m\0"
  "svmla_s16_x\0"
  "svmla_s16_z\0"
  "svmla_s32_m\0"
  "svmla_s32_x\0"
  "svmla_s32_z\0"
  "svmla_s64_m\0"
  "svmla_s64_x\0"
  "svmla_s64_z\0"
  "svmla_s8_m\0"
  "svmla_s8_x\0"
  "svmla_s8_z\0"
  "svmla_u16_m\0"
  "svmla_u16_x\0"
  "svmla_u16_z\0"
  "svmla_u32_m\0"
  "svmla_u32_x\0"
  "svmla_u32_z\0"
  "svmla_u64_m\0"
  "svmla_u64_x\0"
  "svmla_u64_z\0"
  "svmla_u8_m\0"
  "svmla_u8_x\0"
  "svmla_u8_z\0"
  "svmlalb_f16_mf8_fpm\0"
  "(sve,sve2,fp8fma)|(sme,ssve-fp8fma)\0"
  "svmlalb_f32\0"
  "svmlalb_lane_f16_mf8_fpm\0"
  "svmlalb_lane_f32\0"
  "svmlalb_lane_s32\0"
  "svmlalb_lane_s64\0"
  "q2Wiq2Wiq4iq4iIUWi\0"
  "svmlalb_lane_u32\0"
  "svmlalb_lane_u64\0"
  "q2UWiq2UWiq4Uiq4UiIUWi\0"
  "svmlalb_n_f16_mf8_fpm\0"
  "svmlalb_n_f32\0"
  "q4fq4fq8hh\0"
  "svmlalb_n_s16\0"
  "svmlalb_n_s32\0"
  "svmlalb_n_s64\0"
  "svmlalb_n_u16\0"
  "svmlalb_n_u32\0"
  "svmlalb_n_u64\0"
  "svmlalb_s16\0"
  "svmlalb_s32\0"
  "svmlalb_s64\0"
  "svmlalb_u16\0"
  "svmlalb_u32\0"
  "svmlalb_u64\0"
  "svmlallbb_f32_mf8_fpm\0"
  "svmlallbb_lane_f32_mf8_fpm\0"
  "svmlallbb_n_f32_mf8_fpm\0"
  "svmlallbt_f32_mf8_fpm\0"
  "svmlallbt_lane_f32_mf8_fpm\0"
  "svmlallbt_n_f32_mf8_fpm\0"
  "svmlalltb_f32_mf8_fpm\0"
  "svmlalltb_lane_f32_mf8_fpm\0"
  "svmlalltb_n_f32_mf8_fpm\0"
  "svmlalltt_f32_mf8_fpm\0"
  "svmlalltt_lane_f32_mf8_fpm\0"
  "svmlalltt_n_f32_mf8_fpm\0"
  "svmlalt_f16_mf8_fpm\0"
  "svmlalt_f32\0"
  "svmlalt_lane_f16_mf8_fpm\0"
  "svmlalt_lane_f32\0"
  "svmlalt_lane_s32\0"
  "svmlalt_lane_s64\0"
  "svmlalt_lane_u32\0"
  "svmlalt_lane_u64\0"
  "svmlalt_n_f16_mf8_fpm\0"
  "svmlalt_n_f32\0"
  "svmlalt_n_s16\0"
  "svmlalt_n_s32\0"
  "svmlalt_n_s64\0"
  "svmlalt_n_u16\0"
  "svmlalt_n_u32\0"
  "svmlalt_n_u64\0"
  "svmlalt_s16\0"
  "svmlalt_s32\0"
  "svmlalt_s64\0"
  "svmlalt_u16\0"
  "svmlalt_u32\0"
  "svmlalt_u64\0"
  "svmls_bf16_m\0"
  "svmls_bf16_x\0"
  "svmls_bf16_z\0"
  "svmls_f16_m\0"
  "svmls_f16_x\0"
  "svmls_f16_z\0"
  "svmls_f32_m\0"
  "svmls_f32_x\0"
  "svmls_f32_z\0"
  "svmls_f64_m\0"
  "svmls_f64_x\0"
  "svmls_f64_z\0"
  "svmls_lane_bf16\0"
  "svmls_lane_f16\0"
  "svmls_lane_f32\0"
  "svmls_lane_f64\0"
  "svmls_lane_s16\0"
  "svmls_lane_s32\0"
  "svmls_lane_s64\0"
  "svmls_lane_u16\0"
  "svmls_lane_u32\0"
  "svmls_lane_u64\0"
  "svmls_n_bf16_m\0"
  "svmls_n_bf16_x\0"
  "svmls_n_bf16_z\0"
  "svmls_n_f16_m\0"
  "svmls_n_f16_x\0"
  "svmls_n_f16_z\0"
  "svmls_n_f32_m\0"
  "svmls_n_f32_x\0"
  "svmls_n_f32_z\0"
  "svmls_n_f64_m\0"
  "svmls_n_f64_x\0"
  "svmls_n_f64_z\0"
  "svmls_n_s16_m\0"
  "svmls_n_s16_x\0"
  "svmls_n_s16_z\0"
  "svmls_n_s32_m\0"
  "svmls_n_s32_x\0"
  "svmls_n_s32_z\0"
  "svmls_n_s64_m\0"
  "svmls_n_s64_x\0"
  "svmls_n_s64_z\0"
  "svmls_n_s8_m\0"
  "svmls_n_s8_x\0"
  "svmls_n_s8_z\0"
  "svmls_n_u16_m\0"
  "svmls_n_u16_x\0"
  "svmls_n_u16_z\0"
  "svmls_n_u32_m\0"
  "svmls_n_u32_x\0"
  "svmls_n_u32_z\0"
  "svmls_n_u64_m\0"
  "svmls_n_u64_x\0"
  "svmls_n_u64_z\0"
  "svmls_n_u8_m\0"
  "svmls_n_u8_x\0"
  "svmls_n_u8_z\0"
  "svmls_s16_m\0"
  "svmls_s16_x\0"
  "svmls_s16_z\0"
  "svmls_s32_m\0"
  "svmls_s32_x\0"
  "svmls_s32_z\0"
  "svmls_s64_m\0"
  "svmls_s64_x\0"
  "svmls_s64_z\0"
  "svmls_s8_m\0"
  "svmls_s8_x\0"
  "svmls_s8_z\0"
  "svmls_u16_m\0"
  "svmls_u16_x\0"
  "svmls_u16_z\0"
  "svmls_u32_m\0"
  "svmls_u32_x\0"
  "svmls_u32_z\0"
  "svmls_u64_m\0"
  "svmls_u64_x\0"
  "svmls_u64_z\0"
  "svmls_u8_m\0"
  "svmls_u8_x\0"
  "svmls_u8_z\0"
  "svmlslb_f32\0"
  "svmlslb_lane_f32\0"
  "svmlslb_lane_s32\0"
  "svmlslb_lane_s64\0"
  "svmlslb_lane_u32\0"
  "svmlslb_lane_u64\0"
  "svmlslb_n_f32\0"
  "svmlslb_n_s16\0"
  "svmlslb_n_s32\0"
  "svmlslb_n_s64\0"
  "svmlslb_n_u16\0"
  "svmlslb_n_u32\0"
  "svmlslb_n_u64\0"
  "svmlslb_s16\0"
  "svmlslb_s32\0"
  "svmlslb_s64\0"
  "svmlslb_u16\0"
  "svmlslb_u32\0"
  "svmlslb_u64\0"
  "svmlslt_f32\0"
  "svmlslt_lane_f32\0"
  "svmlslt_lane_s32\0"
  "svmlslt_lane_s64\0"
  "svmlslt_lane_u32\0"
  "svmlslt_lane_u64\0"
  "svmlslt_n_f32\0"
  "svmlslt_n_s16\0"
  "svmlslt_n_s32\0"
  "svmlslt_n_s64\0"
  "svmlslt_n_u16\0"
  "svmlslt_n_u32\0"
  "svmlslt_n_u64\0"
  "svmlslt_s16\0"
  "svmlslt_s32\0"
  "svmlslt_s64\0"
  "svmlslt_u16\0"
  "svmlslt_u32\0"
  "svmlslt_u64\0"
  "svmmla_f16_mf8_fpm\0"
  "sve,sve2,f8f16mm\0"
  "svmmla_f32\0"
  "sve,f32mm\0"
  "svmmla_f32_f16\0"
  "sve,sve-f16f32mm\0"
  "svmmla_f32_mf8_fpm\0"
  "sve,sve2,f8f32mm\0"
  "svmmla_f64\0"
  "svmmla_s32\0"
  "sve,i8mm\0"
  "svmmla_u32\0"
  "svmov_b_z\0"
  "svmovlb_s16\0"
  "q8sq16Sc\0"
  "svmovlb_s32\0"
  "q4iq8s\0"
  "svmovlb_s64\0"
  "q2Wiq4i\0"
  "svmovlb_u16\0"
  "q8Usq16Uc\0"
  "svmovlb_u32\0"
  "q4Uiq8Us\0"
  "svmovlb_u64\0"
  "q2UWiq4Ui\0"
  "svmovlt_s16\0"
  "svmovlt_s32\0"
  "svmovlt_s64\0"
  "svmovlt_u16\0"
  "svmovlt_u32\0"
  "svmovlt_u64\0"
  "svmsb_f16_m\0"
  "svmsb_f16_x\0"
  "svmsb_f16_z\0"
  "svmsb_f32_m\0"
  "svmsb_f32_x\0"
  "svmsb_f32_z\0"
  "svmsb_f64_m\0"
  "svmsb_f64_x\0"
  "svmsb_f64_z\0"
  "svmsb_n_f16_m\0"
  "svmsb_n_f16_x\0"
  "svmsb_n_f16_z\0"
  "svmsb_n_f32_m\0"
  "svmsb_n_f32_x\0"
  "svmsb_n_f32_z\0"
  "svmsb_n_f64_m\0"
  "svmsb_n_f64_x\0"
  "svmsb_n_f64_z\0"
  "svmsb_n_s16_m\0"
  "svmsb_n_s16_x\0"
  "svmsb_n_s16_z\0"
  "svmsb_n_s32_m\0"
  "svmsb_n_s32_x\0"
  "svmsb_n_s32_z\0"
  "svmsb_n_s64_m\0"
  "svmsb_n_s64_x\0"
  "svmsb_n_s64_z\0"
  "svmsb_n_s8_m\0"
  "svmsb_n_s8_x\0"
  "svmsb_n_s8_z\0"
  "svmsb_n_u16_m\0"
  "svmsb_n_u16_x\0"
  "svmsb_n_u16_z\0"
  "svmsb_n_u32_m\0"
  "svmsb_n_u32_x\0"
  "svmsb_n_u32_z\0"
  "svmsb_n_u64_m\0"
  "svmsb_n_u64_x\0"
  "svmsb_n_u64_z\0"
  "svmsb_n_u8_m\0"
  "svmsb_n_u8_x\0"
  "svmsb_n_u8_z\0"
  "svmsb_s16_m\0"
  "svmsb_s16_x\0"
  "svmsb_s16_z\0"
  "svmsb_s32_m\0"
  "svmsb_s32_x\0"
  "svmsb_s32_z\0"
  "svmsb_s64_m\0"
  "svmsb_s64_x\0"
  "svmsb_s64_z\0"
  "svmsb_s8_m\0"
  "svmsb_s8_x\0"
  "svmsb_s8_z\0"
  "svmsb_u16_m\0"
  "svmsb_u16_x\0"
  "svmsb_u16_z\0"
  "svmsb_u32_m\0"
  "svmsb_u32_x\0"
  "svmsb_u32_z\0"
  "svmsb_u64_m\0"
  "svmsb_u64_x\0"
  "svmsb_u64_z\0"
  "svmsb_u8_m\0"
  "svmsb_u8_x\0"
  "svmsb_u8_z\0"
  "svmul_bf16_m\0"
  "svmul_bf16_x\0"
  "svmul_bf16_x2\0"
  "sme,sme2,sve-bfscale\0"
  "svmul_bf16_x4\0"
  "svmul_bf16_z\0"
  "svmul_f16_m\0"
  "svmul_f16_x\0"
  "svmul_f16_x2\0"
  "sme,sme2p2\0"
  "svmul_f16_x4\0"
  "svmul_f16_z\0"
  "svmul_f32_m\0"
  "svmul_f32_x\0"
  "svmul_f32_x2\0"
  "svmul_f32_x4\0"
  "svmul_f32_z\0"
  "svmul_f64_m\0"
  "svmul_f64_x\0"
  "svmul_f64_x2\0"
  "svmul_f64_x4\0"
  "svmul_f64_z\0"
  "svmul_lane_bf16\0"
  "svmul_lane_f16\0"
  "svmul_lane_f32\0"
  "svmul_lane_f64\0"
  "svmul_lane_s16\0"
  "svmul_lane_s32\0"
  "svmul_lane_s64\0"
  "svmul_lane_u16\0"
  "svmul_lane_u32\0"
  "svmul_lane_u64\0"
  "svmul_n_bf16_m\0"
  "svmul_n_bf16_x\0"
  "svmul_n_bf16_z\0"
  "svmul_n_f16_m\0"
  "svmul_n_f16_x\0"
  "svmul_n_f16_z\0"
  "svmul_n_f32_m\0"
  "svmul_n_f32_x\0"
  "svmul_n_f32_z\0"
  "svmul_n_f64_m\0"
  "svmul_n_f64_x\0"
  "svmul_n_f64_z\0"
  "svmul_n_s16_m\0"
  "svmul_n_s16_x\0"
  "svmul_n_s16_z\0"
  "svmul_n_s32_m\0"
  "svmul_n_s32_x\0"
  "svmul_n_s32_z\0"
  "svmul_n_s64_m\0"
  "svmul_n_s64_x\0"
  "svmul_n_s64_z\0"
  "svmul_n_s8_m\0"
  "svmul_n_s8_x\0"
  "svmul_n_s8_z\0"
  "svmul_n_u16_m\0"
  "svmul_n_u16_x\0"
  "svmul_n_u16_z\0"
  "svmul_n_u32_m\0"
  "svmul_n_u32_x\0"
  "svmul_n_u32_z\0"
  "svmul_n_u64_m\0"
  "svmul_n_u64_x\0"
  "svmul_n_u64_z\0"
  "svmul_n_u8_m\0"
  "svmul_n_u8_x\0"
  "svmul_n_u8_z\0"
  "svmul_s16_m\0"
  "svmul_s16_x\0"
  "svmul_s16_z\0"
  "svmul_s32_m\0"
  "svmul_s32_x\0"
  "svmul_s32_z\0"
  "svmul_s64_m\0"
  "svmul_s64_x\0"
  "svmul_s64_z\0"
  "svmul_s8_m\0"
  "svmul_s8_x\0"
  "svmul_s8_z\0"
  "svmul_single_bf16_x2\0"
  "svmul_single_bf16_x4\0"
  "svmul_single_f16_x2\0"
  "svmul_single_f16_x4\0"
  "svmul_single_f32_x2\0"
  "svmul_single_f32_x4\0"
  "svmul_single_f64_x2\0"
  "svmul_single_f64_x4\0"
  "svmul_u16_m\0"
  "svmul_u16_x\0"
  "svmul_u16_z\0"
  "svmul_u32_m\0"
  "svmul_u32_x\0"
  "svmul_u32_z\0"
  "svmul_u64_m\0"
  "svmul_u64_x\0"
  "svmul_u64_z\0"
  "svmul_u8_m\0"
  "svmul_u8_x\0"
  "svmul_u8_z\0"
  "svmulh_n_s16_m\0"
  "svmulh_n_s16_x\0"
  "svmulh_n_s16_z\0"
  "svmulh_n_s32_m\0"
  "svmulh_n_s32_x\0"
  "svmulh_n_s32_z\0"
  "svmulh_n_s64_m\0"
  "svmulh_n_s64_x\0"
  "svmulh_n_s64_z\0"
  "svmulh_n_s8_m\0"
  "svmulh_n_s8_x\0"
  "svmulh_n_s8_z\0"
  "svmulh_n_u16_m\0"
  "svmulh_n_u16_x\0"
  "svmulh_n_u16_z\0"
  "svmulh_n_u32_m\0"
  "svmulh_n_u32_x\0"
  "svmulh_n_u32_z\0"
  "svmulh_n_u64_m\0"
  "svmulh_n_u64_x\0"
  "svmulh_n_u64_z\0"
  "svmulh_n_u8_m\0"
  "svmulh_n_u8_x\0"
  "svmulh_n_u8_z\0"
  "svmulh_s16_m\0"
  "svmulh_s16_x\0"
  "svmulh_s16_z\0"
  "svmulh_s32_m\0"
  "svmulh_s32_x\0"
  "svmulh_s32_z\0"
  "svmulh_s64_m\0"
  "svmulh_s64_x\0"
  "svmulh_s64_z\0"
  "svmulh_s8_m\0"
  "svmulh_s8_x\0"
  "svmulh_s8_z\0"
  "svmulh_u16_m\0"
  "svmulh_u16_x\0"
  "svmulh_u16_z\0"
  "svmulh_u32_m\0"
  "svmulh_u32_x\0"
  "svmulh_u32_z\0"
  "svmulh_u64_m\0"
  "svmulh_u64_x\0"
  "svmulh_u64_z\0"
  "svmulh_u8_m\0"
  "svmulh_u8_x\0"
  "svmulh_u8_z\0"
  "svmullb_lane_s32\0"
  "q4iq8sq8sIUWi\0"
  "svmullb_lane_s64\0"
  "q2Wiq4iq4iIUWi\0"
  "svmullb_lane_u32\0"
  "q4Uiq8Usq8UsIUWi\0"
  "svmullb_lane_u64\0"
  "q2UWiq4Uiq4UiIUWi\0"
  "svmullb_n_s16\0"
  "svmullb_n_s32\0"
  "svmullb_n_s64\0"
  "svmullb_n_u16\0"
  "svmullb_n_u32\0"
  "svmullb_n_u64\0"
  "svmullb_s16\0"
  "svmullb_s32\0"
  "svmullb_s64\0"
  "svmullb_u16\0"
  "svmullb_u32\0"
  "svmullb_u64\0"
  "svmullt_lane_s32\0"
  "svmullt_lane_s64\0"
  "svmullt_lane_u32\0"
  "svmullt_lane_u64\0"
  "svmullt_n_s16\0"
  "svmullt_n_s32\0"
  "svmullt_n_s64\0"
  "svmullt_n_u16\0"
  "svmullt_n_u32\0"
  "svmullt_n_u64\0"
  "svmullt_s16\0"
  "svmullt_s32\0"
  "svmullt_s64\0"
  "svmullt_u16\0"
  "svmullt_u32\0"
  "svmullt_u64\0"
  "svmulx_f16_m\0"
  "svmulx_f16_x\0"
  "svmulx_f16_z\0"
  "svmulx_f32_m\0"
  "svmulx_f32_x\0"
  "svmulx_f32_z\0"
  "svmulx_f64_m\0"
  "svmulx_f64_x\0"
  "svmulx_f64_z\0"
  "svmulx_n_f16_m\0"
  "svmulx_n_f16_x\0"
  "svmulx_n_f16_z\0"
  "svmulx_n_f32_m\0"
  "svmulx_n_f32_x\0"
  "svmulx_n_f32_z\0"
  "svmulx_n_f64_m\0"
  "svmulx_n_f64_x\0"
  "svmulx_n_f64_z\0"
  "svnand_b_z\0"
  "svnbsl_n_s16\0"
  "svnbsl_n_s32\0"
  "svnbsl_n_s64\0"
  "svnbsl_n_s8\0"
  "svnbsl_n_u16\0"
  "svnbsl_n_u32\0"
  "svnbsl_n_u64\0"
  "svnbsl_n_u8\0"
  "svnbsl_s16\0"
  "svnbsl_s32\0"
  "svnbsl_s64\0"
  "svnbsl_s8\0"
  "svnbsl_u16\0"
  "svnbsl_u32\0"
  "svnbsl_u64\0"
  "svnbsl_u8\0"
  "svneg_f16_m\0"
  "svneg_f16_x\0"
  "svneg_f16_z\0"
  "svneg_f32_m\0"
  "svneg_f32_x\0"
  "svneg_f32_z\0"
  "svneg_f64_m\0"
  "svneg_f64_x\0"
  "svneg_f64_z\0"
  "svneg_s16_m\0"
  "svneg_s16_x\0"
  "svneg_s16_z\0"
  "svneg_s32_m\0"
  "svneg_s32_x\0"
  "svneg_s32_z\0"
  "svneg_s64_m\0"
  "svneg_s64_x\0"
  "svneg_s64_z\0"
  "svneg_s8_m\0"
  "svneg_s8_x\0"
  "svneg_s8_z\0"
  "svnmad_f16_m\0"
  "svnmad_f16_x\0"
  "svnmad_f16_z\0"
  "svnmad_f32_m\0"
  "svnmad_f32_x\0"
  "svnmad_f32_z\0"
  "svnmad_f64_m\0"
  "svnmad_f64_x\0"
  "svnmad_f64_z\0"
  "svnmad_n_f16_m\0"
  "svnmad_n_f16_x\0"
  "svnmad_n_f16_z\0"
  "svnmad_n_f32_m\0"
  "svnmad_n_f32_x\0"
  "svnmad_n_f32_z\0"
  "svnmad_n_f64_m\0"
  "svnmad_n_f64_x\0"
  "svnmad_n_f64_z\0"
  "svnmatch_s16\0"
  "svnmatch_s8\0"
  "svnmatch_u16\0"
  "svnmatch_u8\0"
  "svnmla_f16_m\0"
  "svnmla_f16_x\0"
  "svnmla_f16_z\0"
  "svnmla_f32_m\0"
  "svnmla_f32_x\0"
  "svnmla_f32_z\0"
  "svnmla_f64_m\0"
  "svnmla_f64_x\0"
  "svnmla_f64_z\0"
  "svnmla_n_f16_m\0"
  "svnmla_n_f16_x\0"
  "svnmla_n_f16_z\0"
  "svnmla_n_f32_m\0"
  "svnmla_n_f32_x\0"
  "svnmla_n_f32_z\0"
  "svnmla_n_f64_m\0"
  "svnmla_n_f64_x\0"
  "svnmla_n_f64_z\0"
  "svnmls_f16_m\0"
  "svnmls_f16_x\0"
  "svnmls_f16_z\0"
  "svnmls_f32_m\0"
  "svnmls_f32_x\0"
  "svnmls_f32_z\0"
  "svnmls_f64_m\0"
  "svnmls_f64_x\0"
  "svnmls_f64_z\0"
  "svnmls_n_f16_m\0"
  "svnmls_n_f16_x\0"
  "svnmls_n_f16_z\0"
  "svnmls_n_f32_m\0"
  "svnmls_n_f32_x\0"
  "svnmls_n_f32_z\0"
  "svnmls_n_f64_m\0"
  "svnmls_n_f64_x\0"
  "svnmls_n_f64_z\0"
  "svnmsb_f16_m\0"
  "svnmsb_f16_x\0"
  "svnmsb_f16_z\0"
  "svnmsb_f32_m\0"
  "svnmsb_f32_x\0"
  "svnmsb_f32_z\0"
  "svnmsb_f64_m\0"
  "svnmsb_f64_x\0"
  "svnmsb_f64_z\0"
  "svnmsb_n_f16_m\0"
  "svnmsb_n_f16_x\0"
  "svnmsb_n_f16_z\0"
  "svnmsb_n_f32_m\0"
  "svnmsb_n_f32_x\0"
  "svnmsb_n_f32_z\0"
  "svnmsb_n_f64_m\0"
  "svnmsb_n_f64_x\0"
  "svnmsb_n_f64_z\0"
  "svnor_b_z\0"
  "svnot_b_z\0"
  "svnot_s16_m\0"
  "svnot_s16_x\0"
  "svnot_s16_z\0"
  "svnot_s32_m\0"
  "svnot_s32_x\0"
  "svnot_s32_z\0"
  "svnot_s64_m\0"
  "svnot_s64_x\0"
  "svnot_s64_z\0"
  "svnot_s8_m\0"
  "svnot_s8_x\0"
  "svnot_s8_z\0"
  "svnot_u16_m\0"
  "svnot_u16_x\0"
  "svnot_u16_z\0"
  "svnot_u32_m\0"
  "svnot_u32_x\0"
  "svnot_u32_z\0"
  "svnot_u64_m\0"
  "svnot_u64_x\0"
  "svnot_u64_z\0"
  "svnot_u8_m\0"
  "svnot_u8_x\0"
  "svnot_u8_z\0"
  "svorn_b_z\0"
  "svorqv_s16\0"
  "svorqv_s32\0"
  "svorqv_s64\0"
  "svorqv_s8\0"
  "svorqv_u16\0"
  "svorqv_u32\0"
  "svorqv_u64\0"
  "svorqv_u8\0"
  "svorr_b_z\0"
  "svorr_n_s16_m\0"
  "svorr_n_s16_x\0"
  "svorr_n_s16_z\0"
  "svorr_n_s32_m\0"
  "svorr_n_s32_x\0"
  "svorr_n_s32_z\0"
  "svorr_n_s64_m\0"
  "svorr_n_s64_x\0"
  "svorr_n_s64_z\0"
  "svorr_n_s8_m\0"
  "svorr_n_s8_x\0"
  "svorr_n_s8_z\0"
  "svorr_n_u16_m\0"
  "svorr_n_u16_x\0"
  "svorr_n_u16_z\0"
  "svorr_n_u32_m\0"
  "svorr_n_u32_x\0"
  "svorr_n_u32_z\0"
  "svorr_n_u64_m\0"
  "svorr_n_u64_x\0"
  "svorr_n_u64_z\0"
  "svorr_n_u8_m\0"
  "svorr_n_u8_x\0"
  "svorr_n_u8_z\0"
  "svorr_s16_m\0"
  "svorr_s16_x\0"
  "svorr_s16_z\0"
  "svorr_s32_m\0"
  "svorr_s32_x\0"
  "svorr_s32_z\0"
  "svorr_s64_m\0"
  "svorr_s64_x\0"
  "svorr_s64_z\0"
  "svorr_s8_m\0"
  "svorr_s8_x\0"
  "svorr_s8_z\0"
  "svorr_u16_m\0"
  "svorr_u16_x\0"
  "svorr_u16_z\0"
  "svorr_u32_m\0"
  "svorr_u32_x\0"
  "svorr_u32_z\0"
  "svorr_u64_m\0"
  "svorr_u64_x\0"
  "svorr_u64_z\0"
  "svorr_u8_m\0"
  "svorr_u8_x\0"
  "svorr_u8_z\0"
  "svorv_s16\0"
  "svorv_s32\0"
  "svorv_s64\0"
  "svorv_s8\0"
  "svorv_u16\0"
  "svorv_u32\0"
  "svorv_u64\0"
  "svorv_u8\0"
  "svpext_lane_c16\0"
  "q16bQaIUWi\0"
  "svpext_lane_c16_x2\0"
  "q32bQaIUWi\0"
  "svpext_lane_c32\0"
  "svpext_lane_c32_x2\0"
  "svpext_lane_c64\0"
  "svpext_lane_c64_x2\0"
  "svpext_lane_c8\0"
  "svpext_lane_c8_x2\0"
  "svpfalse_b\0"
  "q16bv\0"
  "svpfalse_c\0"
  "Qav\0"
  "svpfirst_b\0"
  "svpmlal_pair_n_u64_x2\0"
  "q4UWiq4UWiq2UWiUWi\0"
  "svpmlal_pair_u64_x2\0"
  "svpmov_lane_s16\0"
  "q16bq8sIUWi\0"
  "svpmov_lane_s16_m\0"
  "q8sq8sq16bIUWi\0"
  "svpmov_lane_s32\0"
  "q16bq4iIUWi\0"
  "svpmov_lane_s32_m\0"
  "q4iq4iq16bIUWi\0"
  "svpmov_lane_s64\0"
  "q16bq2WiIUWi\0"
  "svpmov_lane_s64_m\0"
  "q2Wiq2Wiq16bIUWi\0"
  "svpmov_lane_s8\0"
  "q16bq16ScIUWi\0"
  "svpmov_lane_u16\0"
  "q16bq8UsIUWi\0"
  "svpmov_lane_u16_m\0"
  "q8Usq8Usq16bIUWi\0"
  "svpmov_lane_u32\0"
  "q16bq4UiIUWi\0"
  "svpmov_lane_u32_m\0"
  "q4Uiq4Uiq16bIUWi\0"
  "svpmov_lane_u64\0"
  "q16bq2UWiIUWi\0"
  "svpmov_lane_u64_m\0"
  "q2UWiq2UWiq16bIUWi\0"
  "svpmov_lane_u8\0"
  "q16bq16UcIUWi\0"
  "svpmov_s16\0"
  "q16bq8s\0"
  "svpmov_s16_z\0"
  "q8sq16b\0"
  "svpmov_s32\0"
  "q16bq4i\0"
  "svpmov_s32_z\0"
  "q4iq16b\0"
  "svpmov_s64\0"
  "q16bq2Wi\0"
  "svpmov_s64_z\0"
  "q2Wiq16b\0"
  "svpmov_s8\0"
  "q16bq16Sc\0"
  "svpmov_s8_z\0"
  "q16Scq16b\0"
  "svpmov_u16\0"
  "q16bq8Us\0"
  "svpmov_u16_z\0"
  "q8Usq16b\0"
  "svpmov_u32\0"
  "q16bq4Ui\0"
  "svpmov_u32_z\0"
  "q4Uiq16b\0"
  "svpmov_u64\0"
  "q16bq2UWi\0"
  "svpmov_u64_z\0"
  "q2UWiq16b\0"
  "svpmov_u8\0"
  "q16bq16Uc\0"
  "svpmov_u8_z\0"
  "q16Ucq16b\0"
  "svpmul_n_u8\0"
  "svpmul_u8\0"
  "svpmull_pair_n_u64_x2\0"
  "q4UWiq2UWiUWi\0"
  "svpmull_pair_u64_x2\0"
  "svpmullb_n_u16\0"
  "svpmullb_n_u64\0"
  "svpmullb_pair_n_u32\0"
  "svpmullb_pair_n_u64\0"
  "svpmullb_pair_n_u8\0"
  "svpmullb_pair_u32\0"
  "svpmullb_pair_u64\0"
  "svpmullb_pair_u8\0"
  "svpmullb_u16\0"
  "svpmullb_u64\0"
  "svpmullt_n_u16\0"
  "svpmullt_n_u64\0"
  "svpmullt_pair_n_u32\0"
  "svpmullt_pair_n_u64\0"
  "svpmullt_pair_n_u8\0"
  "svpmullt_pair_u32\0"
  "svpmullt_pair_u64\0"
  "svpmullt_pair_u8\0"
  "svpmullt_u16\0"
  "svpmullt_u64\0"
  "svpnext_b16\0"
  "svpnext_b32\0"
  "svpnext_b64\0"
  "svpnext_b8\0"
  "svprfb\0"
  "vq16bvC*Ii\0"
  "svprfb_gather_s32offset\0"
  "vq16bvC*q4iIi\0"
  "svprfb_gather_s64offset\0"
  "vq16bvC*q2WiIi\0"
  "svprfb_gather_u32base\0"
  "vq16bq4UiIi\0"
  "svprfb_gather_u32base_offset\0"
  "vq16bq4UiWiIi\0"
  "svprfb_gather_u32offset\0"
  "vq16bvC*q4UiIi\0"
  "svprfb_gather_u64base\0"
  "vq16bq2UWiIi\0"
  "svprfb_gather_u64base_offset\0"
  "vq16bq2UWiWiIi\0"
  "svprfb_gather_u64offset\0"
  "vq16bvC*q2UWiIi\0"
  "svprfb_vnum\0"
  "vq16bvC*WiIi\0"
  "svprfd\0"
  "svprfd_gather_s32index\0"
  "svprfd_gather_s64index\0"
  "svprfd_gather_u32base\0"
  "svprfd_gather_u32base_index\0"
  "svprfd_gather_u32index\0"
  "svprfd_gather_u64base\0"
  "svprfd_gather_u64base_index\0"
  "svprfd_gather_u64index\0"
  "svprfd_vnum\0"
  "svprfh\0"
  "svprfh_gather_s32index\0"
  "svprfh_gather_s64index\0"
  "svprfh_gather_u32base\0"
  "svprfh_gather_u32base_index\0"
  "svprfh_gather_u32index\0"
  "svprfh_gather_u64base\0"
  "svprfh_gather_u64base_index\0"
  "svprfh_gather_u64index\0"
  "svprfh_vnum\0"
  "svprfw\0"
  "svprfw_gather_s32index\0"
  "svprfw_gather_s64index\0"
  "svprfw_gather_u32base\0"
  "svprfw_gather_u32base_index\0"
  "svprfw_gather_u32index\0"
  "svprfw_gather_u64base\0"
  "svprfw_gather_u64base_index\0"
  "svprfw_gather_u64index\0"
  "svprfw_vnum\0"
  "svpsel_lane_b16\0"
  "q16bq16bq16bUi\0"
  "svpsel_lane_b32\0"
  "svpsel_lane_b64\0"
  "svpsel_lane_b8\0"
  "svpsel_lane_c16\0"
  "QaQaq16bUi\0"
  "svpsel_lane_c32\0"
  "svpsel_lane_c64\0"
  "svpsel_lane_c8\0"
  "svptest_any\0"
  "bq16bq16b\0"
  "svptest_first\0"
  "svptest_last\0"
  "svptrue_b16\0"
  "svptrue_b32\0"
  "svptrue_b64\0"
  "svptrue_b8\0"
  "svptrue_c16\0"
  "svptrue_c32\0"
  "svptrue_c64\0"
  "svptrue_c8\0"
  "svptrue_pat_b16\0"
  "q16bIi\0"
  "svptrue_pat_b32\0"
  "svptrue_pat_b64\0"
  "svptrue_pat_b8\0"
  "svqabs_s16_m\0"
  "svqabs_s16_x\0"
  "svqabs_s16_z\0"
  "svqabs_s32_m\0"
  "svqabs_s32_x\0"
  "svqabs_s32_z\0"
  "svqabs_s64_m\0"
  "svqabs_s64_x\0"
  "svqabs_s64_z\0"
  "svqabs_s8_m\0"
  "svqabs_s8_x\0"
  "svqabs_s8_z\0"
  "svqadd_n_s16\0"
  "svqadd_n_s16_m\0"
  "svqadd_n_s16_x\0"
  "svqadd_n_s16_z\0"
  "svqadd_n_s32\0"
  "svqadd_n_s32_m\0"
  "svqadd_n_s32_x\0"
  "svqadd_n_s32_z\0"
  "svqadd_n_s64\0"
  "svqadd_n_s64_m\0"
  "svqadd_n_s64_x\0"
  "svqadd_n_s64_z\0"
  "svqadd_n_s8\0"
  "svqadd_n_s8_m\0"
  "svqadd_n_s8_x\0"
  "svqadd_n_s8_z\0"
  "svqadd_n_u16\0"
  "svqadd_n_u16_m\0"
  "svqadd_n_u16_x\0"
  "svqadd_n_u16_z\0"
  "svqadd_n_u32\0"
  "svqadd_n_u32_m\0"
  "svqadd_n_u32_x\0"
  "svqadd_n_u32_z\0"
  "svqadd_n_u64\0"
  "svqadd_n_u64_m\0"
  "svqadd_n_u64_x\0"
  "svqadd_n_u64_z\0"
  "svqadd_n_u8\0"
  "svqadd_n_u8_m\0"
  "svqadd_n_u8_x\0"
  "svqadd_n_u8_z\0"
  "svqadd_s16\0"
  "q8sq8sq8s\0"
  "svqadd_s16_m\0"
  "svqadd_s16_x\0"
  "svqadd_s16_z\0"
  "svqadd_s32\0"
  "q4iq4iq4i\0"
  "svqadd_s32_m\0"
  "svqadd_s32_x\0"
  "svqadd_s32_z\0"
  "svqadd_s64\0"
  "q2Wiq2Wiq2Wi\0"
  "svqadd_s64_m\0"
  "svqadd_s64_x\0"
  "svqadd_s64_z\0"
  "svqadd_s8\0"
  "q16Scq16Scq16Sc\0"
  "svqadd_s8_m\0"
  "svqadd_s8_x\0"
  "svqadd_s8_z\0"
  "svqadd_u16\0"
  "svqadd_u16_m\0"
  "svqadd_u16_x\0"
  "svqadd_u16_z\0"
  "svqadd_u32\0"
  "svqadd_u32_m\0"
  "svqadd_u32_x\0"
  "svqadd_u32_z\0"
  "svqadd_u64\0"
  "svqadd_u64_m\0"
  "svqadd_u64_x\0"
  "svqadd_u64_z\0"
  "svqadd_u8\0"
  "svqadd_u8_m\0"
  "svqadd_u8_x\0"
  "svqadd_u8_z\0"
  "svqcadd_s16\0"
  "svqcadd_s32\0"
  "svqcadd_s64\0"
  "svqcadd_s8\0"
  "svqcvt_s16_s32_x2\0"
  "q8sq8i\0"
  "svqcvt_s16_s64_x4\0"
  "q8sq8Wi\0"
  "svqcvt_s8_s32_x4\0"
  "q16Scq16i\0"
  "svqcvt_u16_s32_x2\0"
  "q8Usq8i\0"
  "svqcvt_u16_s64_x4\0"
  "q8Usq8Wi\0"
  "svqcvt_u16_u32_x2\0"
  "q8Usq8Ui\0"
  "svqcvt_u16_u64_x4\0"
  "q8Usq8UWi\0"
  "svqcvt_u8_s32_x4\0"
  "q16Ucq16i\0"
  "svqcvt_u8_u32_x4\0"
  "q16Ucq16Ui\0"
  "svqcvtn_s16_s32_x2\0"
  "svqcvtn_s16_s64_x4\0"
  "svqcvtn_s8_s32_x4\0"
  "svqcvtn_u16_s32_x2\0"
  "svqcvtn_u16_s64_x4\0"
  "svqcvtn_u16_u32_x2\0"
  "svqcvtn_u16_u64_x4\0"
  "svqcvtn_u8_s32_x4\0"
  "svqcvtn_u8_u32_x4\0"
  "svqdecb_n_s32\0"
  "iiIUWi\0"
  "svqdecb_n_s64\0"
  "WiWiIUWi\0"
  "svqdecb_n_u32\0"
  "UiUiIUWi\0"
  "svqdecb_n_u64\0"
  "UWiUWiIUWi\0"
  "svqdecb_pat_n_s32\0"
  "iiIiIUWi\0"
  "svqdecb_pat_n_s64\0"
  "WiWiIiIUWi\0"
  "svqdecb_pat_n_u32\0"
  "UiUiIiIUWi\0"
  "svqdecb_pat_n_u64\0"
  "UWiUWiIiIUWi\0"
  "svqdecd_n_s32\0"
  "svqdecd_n_s64\0"
  "svqdecd_n_u32\0"
  "svqdecd_n_u64\0"
  "svqdecd_pat_n_s32\0"
  "svqdecd_pat_n_s64\0"
  "svqdecd_pat_n_u32\0"
  "svqdecd_pat_n_u64\0"
  "svqdecd_pat_s64\0"
  "q2Wiq2WiIiIUWi\0"
  "svqdecd_pat_u64\0"
  "q2UWiq2UWiIiIUWi\0"
  "svqdecd_s64\0"
  "svqdecd_u64\0"
  "svqdech_n_s32\0"
  "svqdech_n_s64\0"
  "svqdech_n_u32\0"
  "svqdech_n_u64\0"
  "svqdech_pat_n_s32\0"
  "svqdech_pat_n_s64\0"
  "svqdech_pat_n_u32\0"
  "svqdech_pat_n_u64\0"
  "svqdech_pat_s16\0"
  "q8sq8sIiIUWi\0"
  "svqdech_pat_u16\0"
  "q8Usq8UsIiIUWi\0"
  "svqdech_s16\0"
  "svqdech_u16\0"
  "svqdecp_n_s32_b16\0"
  "iiq16b\0"
  "svqdecp_n_s32_b32\0"
  "svqdecp_n_s32_b64\0"
  "svqdecp_n_s32_b8\0"
  "svqdecp_n_s64_b16\0"
  "WiWiq16b\0"
  "svqdecp_n_s64_b32\0"
  "svqdecp_n_s64_b64\0"
  "svqdecp_n_s64_b8\0"
  "svqdecp_n_u32_b16\0"
  "UiUiq16b\0"
  "svqdecp_n_u32_b32\0"
  "svqdecp_n_u32_b64\0"
  "svqdecp_n_u32_b8\0"
  "svqdecp_n_u64_b16\0"
  "UWiUWiq16b\0"
  "svqdecp_n_u64_b32\0"
  "svqdecp_n_u64_b64\0"
  "svqdecp_n_u64_b8\0"
  "svqdecp_s16\0"
  "q8sq8sq16b\0"
  "svqdecp_s32\0"
  "q4iq4iq16b\0"
  "svqdecp_s64\0"
  "q2Wiq2Wiq16b\0"
  "svqdecp_u16\0"
  "q8Usq8Usq16b\0"
  "svqdecp_u32\0"
  "q4Uiq4Uiq16b\0"
  "svqdecp_u64\0"
  "q2UWiq2UWiq16b\0"
  "svqdecw_n_s32\0"
  "svqdecw_n_s64\0"
  "svqdecw_n_u32\0"
  "svqdecw_n_u64\0"
  "svqdecw_pat_n_s32\0"
  "svqdecw_pat_n_s64\0"
  "svqdecw_pat_n_u32\0"
  "svqdecw_pat_n_u64\0"
  "svqdecw_pat_s32\0"
  "q4iq4iIiIUWi\0"
  "svqdecw_pat_u32\0"
  "q4Uiq4UiIiIUWi\0"
  "svqdecw_s32\0"
  "svqdecw_u32\0"
  "svqdmlalb_lane_s32\0"
  "svqdmlalb_lane_s64\0"
  "svqdmlalb_n_s16\0"
  "svqdmlalb_n_s32\0"
  "svqdmlalb_n_s64\0"
  "svqdmlalb_s16\0"
  "svqdmlalb_s32\0"
  "svqdmlalb_s64\0"
  "svqdmlalbt_n_s16\0"
  "svqdmlalbt_n_s32\0"
  "svqdmlalbt_n_s64\0"
  "svqdmlalbt_s16\0"
  "svqdmlalbt_s32\0"
  "svqdmlalbt_s64\0"
  "svqdmlalt_lane_s32\0"
  "svqdmlalt_lane_s64\0"
  "svqdmlalt_n_s16\0"
  "svqdmlalt_n_s32\0"
  "svqdmlalt_n_s64\0"
  "svqdmlalt_s16\0"
  "svqdmlalt_s32\0"
  "svqdmlalt_s64\0"
  "svqdmlslb_lane_s32\0"
  "svqdmlslb_lane_s64\0"
  "svqdmlslb_n_s16\0"
  "svqdmlslb_n_s32\0"
  "svqdmlslb_n_s64\0"
  "svqdmlslb_s16\0"
  "svqdmlslb_s32\0"
  "svqdmlslb_s64\0"
  "svqdmlslbt_n_s16\0"
  "svqdmlslbt_n_s32\0"
  "svqdmlslbt_n_s64\0"
  "svqdmlslbt_s16\0"
  "svqdmlslbt_s32\0"
  "svqdmlslbt_s64\0"
  "svqdmlslt_lane_s32\0"
  "svqdmlslt_lane_s64\0"
  "svqdmlslt_n_s16\0"
  "svqdmlslt_n_s32\0"
  "svqdmlslt_n_s64\0"
  "svqdmlslt_s16\0"
  "svqdmlslt_s32\0"
  "svqdmlslt_s64\0"
  "svqdmulh_lane_s16\0"
  "svqdmulh_lane_s32\0"
  "svqdmulh_lane_s64\0"
  "svqdmulh_n_s16\0"
  "svqdmulh_n_s32\0"
  "svqdmulh_n_s64\0"
  "svqdmulh_n_s8\0"
  "svqdmulh_s16\0"
  "svqdmulh_s16_x2\0"
  "svqdmulh_s16_x4\0"
  "svqdmulh_s32\0"
  "svqdmulh_s32_x2\0"
  "svqdmulh_s32_x4\0"
  "svqdmulh_s64\0"
  "svqdmulh_s64_x2\0"
  "svqdmulh_s64_x4\0"
  "svqdmulh_s8\0"
  "svqdmulh_s8_x2\0"
  "svqdmulh_s8_x4\0"
  "svqdmulh_single_s16_x2\0"
  "svqdmulh_single_s16_x4\0"
  "svqdmulh_single_s32_x2\0"
  "svqdmulh_single_s32_x4\0"
  "svqdmulh_single_s64_x2\0"
  "svqdmulh_single_s64_x4\0"
  "svqdmulh_single_s8_x2\0"
  "svqdmulh_single_s8_x4\0"
  "svqdmullb_lane_s32\0"
  "svqdmullb_lane_s64\0"
  "svqdmullb_n_s16\0"
  "svqdmullb_n_s32\0"
  "svqdmullb_n_s64\0"
  "svqdmullb_s16\0"
  "svqdmullb_s32\0"
  "svqdmullb_s64\0"
  "svqdmullt_lane_s32\0"
  "svqdmullt_lane_s64\0"
  "svqdmullt_n_s16\0"
  "svqdmullt_n_s32\0"
  "svqdmullt_n_s64\0"
  "svqdmullt_s16\0"
  "svqdmullt_s32\0"
  "svqdmullt_s64\0"
  "svqincb_n_s32\0"
  "svqincb_n_s64\0"
  "svqincb_n_u32\0"
  "svqincb_n_u64\0"
  "svqincb_pat_n_s32\0"
  "svqincb_pat_n_s64\0"
  "svqincb_pat_n_u32\0"
  "svqincb_pat_n_u64\0"
  "svqincd_n_s32\0"
  "svqincd_n_s64\0"
  "svqincd_n_u32\0"
  "svqincd_n_u64\0"
  "svqincd_pat_n_s32\0"
  "svqincd_pat_n_s64\0"
  "svqincd_pat_n_u32\0"
  "svqincd_pat_n_u64\0"
  "svqincd_pat_s64\0"
  "svqincd_pat_u64\0"
  "svqincd_s64\0"
  "svqincd_u64\0"
  "svqinch_n_s32\0"
  "svqinch_n_s64\0"
  "svqinch_n_u32\0"
  "svqinch_n_u64\0"
  "svqinch_pat_n_s32\0"
  "svqinch_pat_n_s64\0"
  "svqinch_pat_n_u32\0"
  "svqinch_pat_n_u64\0"
  "svqinch_pat_s16\0"
  "svqinch_pat_u16\0"
  "svqinch_s16\0"
  "svqinch_u16\0"
  "svqincp_n_s32_b16\0"
  "svqincp_n_s32_b32\0"
  "svqincp_n_s32_b64\0"
  "svqincp_n_s32_b8\0"
  "svqincp_n_s64_b16\0"
  "svqincp_n_s64_b32\0"
  "svqincp_n_s64_b64\0"
  "svqincp_n_s64_b8\0"
  "svqincp_n_u32_b16\0"
  "svqincp_n_u32_b32\0"
  "svqincp_n_u32_b64\0"
  "svqincp_n_u32_b8\0"
  "svqincp_n_u64_b16\0"
  "svqincp_n_u64_b32\0"
  "svqincp_n_u64_b64\0"
  "svqincp_n_u64_b8\0"
  "svqincp_s16\0"
  "svqincp_s32\0"
  "svqincp_s64\0"
  "svqincp_u16\0"
  "svqincp_u32\0"
  "svqincp_u64\0"
  "svqincw_n_s32\0"
  "svqincw_n_s64\0"
  "svqincw_n_u32\0"
  "svqincw_n_u64\0"
  "svqincw_pat_n_s32\0"
  "svqincw_pat_n_s64\0"
  "svqincw_pat_n_u32\0"
  "svqincw_pat_n_u64\0"
  "svqincw_pat_s32\0"
  "svqincw_pat_u32\0"
  "svqincw_s32\0"
  "svqincw_u32\0"
  "svqneg_s16_m\0"
  "svqneg_s16_x\0"
  "svqneg_s16_z\0"
  "svqneg_s32_m\0"
  "svqneg_s32_x\0"
  "svqneg_s32_z\0"
  "svqneg_s64_m\0"
  "svqneg_s64_x\0"
  "svqneg_s64_z\0"
  "svqneg_s8_m\0"
  "svqneg_s8_x\0"
  "svqneg_s8_z\0"
  "svqrdcmlah_lane_s16\0"
  "svqrdcmlah_lane_s32\0"
  "svqrdcmlah_s16\0"
  "svqrdcmlah_s32\0"
  "svqrdcmlah_s64\0"
  "svqrdcmlah_s8\0"
  "svqrdmlah_lane_s16\0"
  "svqrdmlah_lane_s32\0"
  "svqrdmlah_lane_s64\0"
  "svqrdmlah_n_s16\0"
  "svqrdmlah_n_s32\0"
  "svqrdmlah_n_s64\0"
  "svqrdmlah_n_s8\0"
  "svqrdmlah_s16\0"
  "svqrdmlah_s32\0"
  "svqrdmlah_s64\0"
  "svqrdmlah_s8\0"
  "svqrdmlsh_lane_s16\0"
  "svqrdmlsh_lane_s32\0"
  "svqrdmlsh_lane_s64\0"
  "svqrdmlsh_n_s16\0"
  "svqrdmlsh_n_s32\0"
  "svqrdmlsh_n_s64\0"
  "svqrdmlsh_n_s8\0"
  "svqrdmlsh_s16\0"
  "svqrdmlsh_s32\0"
  "svqrdmlsh_s64\0"
  "svqrdmlsh_s8\0"
  "svqrdmulh_lane_s16\0"
  "svqrdmulh_lane_s32\0"
  "svqrdmulh_lane_s64\0"
  "svqrdmulh_n_s16\0"
  "svqrdmulh_n_s32\0"
  "svqrdmulh_n_s64\0"
  "svqrdmulh_n_s8\0"
  "svqrdmulh_s16\0"
  "svqrdmulh_s32\0"
  "svqrdmulh_s64\0"
  "svqrdmulh_s8\0"
  "svqrshl_n_s16_m\0"
  "svqrshl_n_s16_x\0"
  "svqrshl_n_s16_z\0"
  "svqrshl_n_s32_m\0"
  "svqrshl_n_s32_x\0"
  "svqrshl_n_s32_z\0"
  "svqrshl_n_s64_m\0"
  "svqrshl_n_s64_x\0"
  "svqrshl_n_s64_z\0"
  "svqrshl_n_s8_m\0"
  "svqrshl_n_s8_x\0"
  "svqrshl_n_s8_z\0"
  "svqrshl_n_u16_m\0"
  "q8Usq16bq8Uss\0"
  "svqrshl_n_u16_x\0"
  "svqrshl_n_u16_z\0"
  "svqrshl_n_u32_m\0"
  "q4Uiq16bq4Uii\0"
  "svqrshl_n_u32_x\0"
  "svqrshl_n_u32_z\0"
  "svqrshl_n_u64_m\0"
  "svqrshl_n_u64_x\0"
  "svqrshl_n_u64_z\0"
  "svqrshl_n_u8_m\0"
  "q16Ucq16bq16UcSc\0"
  "svqrshl_n_u8_x\0"
  "svqrshl_n_u8_z\0"
  "svqrshl_s16_m\0"
  "svqrshl_s16_x\0"
  "svqrshl_s16_z\0"
  "svqrshl_s32_m\0"
  "svqrshl_s32_x\0"
  "svqrshl_s32_z\0"
  "svqrshl_s64_m\0"
  "svqrshl_s64_x\0"
  "svqrshl_s64_z\0"
  "svqrshl_s8_m\0"
  "svqrshl_s8_x\0"
  "svqrshl_s8_z\0"
  "svqrshl_u16_m\0"
  "q8Usq16bq8Usq8s\0"
  "svqrshl_u16_x\0"
  "svqrshl_u16_z\0"
  "svqrshl_u32_m\0"
  "q4Uiq16bq4Uiq4i\0"
  "svqrshl_u32_x\0"
  "svqrshl_u32_z\0"
  "svqrshl_u64_m\0"
  "q2UWiq16bq2UWiq2Wi\0"
  "svqrshl_u64_x\0"
  "svqrshl_u64_z\0"
  "svqrshl_u8_m\0"
  "q16Ucq16bq16Ucq16Sc\0"
  "svqrshl_u8_x\0"
  "svqrshl_u8_z\0"
  "svqrshr_n_s16_s32_x2\0"
  "q8sq8iIUWi\0"
  "svqrshr_n_s16_s64_x4\0"
  "q8sq8WiIUWi\0"
  "svqrshr_n_s8_s32_x4\0"
  "q16Scq16iIUWi\0"
  "svqrshr_n_u16_u32_x2\0"
  "q8Usq8UiIUWi\0"
  "svqrshr_n_u16_u64_x4\0"
  "q8Usq8UWiIUWi\0"
  "svqrshr_n_u8_u32_x4\0"
  "q16Ucq16UiIUWi\0"
  "svqrshrn_n_s16_s32_x2\0"
  "svqrshrn_n_s16_s64_x4\0"
  "svqrshrn_n_s8_s32_x4\0"
  "svqrshrn_n_u16_u32_x2\0"
  "svqrshrn_n_u16_u64_x4\0"
  "svqrshrn_n_u8_u32_x4\0"
  "svqrshrnb_n_s16\0"
  "q16Scq8sIUWi\0"
  "svqrshrnb_n_s32\0"
  "q8sq4iIUWi\0"
  "svqrshrnb_n_s64\0"
  "q4iq2WiIUWi\0"
  "svqrshrnb_n_u16\0"
  "q16Ucq8UsIUWi\0"
  "svqrshrnb_n_u32\0"
  "q8Usq4UiIUWi\0"
  "svqrshrnb_n_u64\0"
  "q4Uiq2UWiIUWi\0"
  "svqrshrnt_n_s16\0"
  "q16Scq16Scq8sIUWi\0"
  "svqrshrnt_n_s32\0"
  "q8sq8sq4iIUWi\0"
  "svqrshrnt_n_s64\0"
  "q4iq4iq2WiIUWi\0"
  "svqrshrnt_n_u16\0"
  "q16Ucq16Ucq8UsIUWi\0"
  "svqrshrnt_n_u32\0"
  "q8Usq8Usq4UiIUWi\0"
  "svqrshrnt_n_u64\0"
  "q4Uiq4Uiq2UWiIUWi\0"
  "svqrshru_n_u16_s32_x2\0"
  "q8Usq8iIUWi\0"
  "svqrshru_n_u16_s64_x4\0"
  "q8Usq8WiIUWi\0"
  "svqrshru_n_u8_s32_x4\0"
  "q16Ucq16iIUWi\0"
  "svqrshrun_n_u16_s32_x2\0"
  "svqrshrun_n_u16_s64_x4\0"
  "svqrshrun_n_u8_s32_x4\0"
  "svqrshrunb_n_s16\0"
  "q16Ucq8sIUWi\0"
  "svqrshrunb_n_s32\0"
  "q8Usq4iIUWi\0"
  "svqrshrunb_n_s64\0"
  "q4Uiq2WiIUWi\0"
  "svqrshrunt_n_s16\0"
  "q16Ucq16Ucq8sIUWi\0"
  "svqrshrunt_n_s32\0"
  "q8Usq8Usq4iIUWi\0"
  "svqrshrunt_n_s64\0"
  "q4Uiq4Uiq2WiIUWi\0"
  "svqshl_n_s16_m\0"
  "svqshl_n_s16_x\0"
  "svqshl_n_s16_z\0"
  "svqshl_n_s32_m\0"
  "svqshl_n_s32_x\0"
  "svqshl_n_s32_z\0"
  "svqshl_n_s64_m\0"
  "svqshl_n_s64_x\0"
  "svqshl_n_s64_z\0"
  "svqshl_n_s8_m\0"
  "svqshl_n_s8_x\0"
  "svqshl_n_s8_z\0"
  "svqshl_n_u16_m\0"
  "svqshl_n_u16_x\0"
  "svqshl_n_u16_z\0"
  "svqshl_n_u32_m\0"
  "svqshl_n_u32_x\0"
  "svqshl_n_u32_z\0"
  "svqshl_n_u64_m\0"
  "svqshl_n_u64_x\0"
  "svqshl_n_u64_z\0"
  "svqshl_n_u8_m\0"
  "svqshl_n_u8_x\0"
  "svqshl_n_u8_z\0"
  "svqshl_s16_m\0"
  "svqshl_s16_x\0"
  "svqshl_s16_z\0"
  "svqshl_s32_m\0"
  "svqshl_s32_x\0"
  "svqshl_s32_z\0"
  "svqshl_s64_m\0"
  "svqshl_s64_x\0"
  "svqshl_s64_z\0"
  "svqshl_s8_m\0"
  "svqshl_s8_x\0"
  "svqshl_s8_z\0"
  "svqshl_u16_m\0"
  "svqshl_u16_x\0"
  "svqshl_u16_z\0"
  "svqshl_u32_m\0"
  "svqshl_u32_x\0"
  "svqshl_u32_z\0"
  "svqshl_u64_m\0"
  "svqshl_u64_x\0"
  "svqshl_u64_z\0"
  "svqshl_u8_m\0"
  "svqshl_u8_x\0"
  "svqshl_u8_z\0"
  "svqshlu_n_s16_m\0"
  "q8Usq16bq8sIUWi\0"
  "svqshlu_n_s16_x\0"
  "svqshlu_n_s16_z\0"
  "svqshlu_n_s32_m\0"
  "q4Uiq16bq4iIUWi\0"
  "svqshlu_n_s32_x\0"
  "svqshlu_n_s32_z\0"
  "svqshlu_n_s64_m\0"
  "q2UWiq16bq2WiIUWi\0"
  "svqshlu_n_s64_x\0"
  "svqshlu_n_s64_z\0"
  "svqshlu_n_s8_m\0"
  "q16Ucq16bq16ScIUWi\0"
  "svqshlu_n_s8_x\0"
  "svqshlu_n_s8_z\0"
  "svqshrnb_n_s16\0"
  "svqshrnb_n_s32\0"
  "svqshrnb_n_s64\0"
  "svqshrnb_n_u16\0"
  "svqshrnb_n_u32\0"
  "svqshrnb_n_u64\0"
  "svqshrnt_n_s16\0"
  "svqshrnt_n_s32\0"
  "svqshrnt_n_s64\0"
  "svqshrnt_n_u16\0"
  "svqshrnt_n_u32\0"
  "svqshrnt_n_u64\0"
  "svqshrunb_n_s16\0"
  "svqshrunb_n_s32\0"
  "svqshrunb_n_s64\0"
  "svqshrunt_n_s16\0"
  "svqshrunt_n_s32\0"
  "svqshrunt_n_s64\0"
  "svqsub_n_s16\0"
  "svqsub_n_s16_m\0"
  "svqsub_n_s16_x\0"
  "svqsub_n_s16_z\0"
  "svqsub_n_s32\0"
  "svqsub_n_s32_m\0"
  "svqsub_n_s32_x\0"
  "svqsub_n_s32_z\0"
  "svqsub_n_s64\0"
  "svqsub_n_s64_m\0"
  "svqsub_n_s64_x\0"
  "svqsub_n_s64_z\0"
  "svqsub_n_s8\0"
  "svqsub_n_s8_m\0"
  "svqsub_n_s8_x\0"
  "svqsub_n_s8_z\0"
  "svqsub_n_u16\0"
  "svqsub_n_u16_m\0"
  "svqsub_n_u16_x\0"
  "svqsub_n_u16_z\0"
  "svqsub_n_u32\0"
  "svqsub_n_u32_m\0"
  "svqsub_n_u32_x\0"
  "svqsub_n_u32_z\0"
  "svqsub_n_u64\0"
  "svqsub_n_u64_m\0"
  "svqsub_n_u64_x\0"
  "svqsub_n_u64_z\0"
  "svqsub_n_u8\0"
  "svqsub_n_u8_m\0"
  "svqsub_n_u8_x\0"
  "svqsub_n_u8_z\0"
  "svqsub_s16\0"
  "svqsub_s16_m\0"
  "svqsub_s16_x\0"
  "svqsub_s16_z\0"
  "svqsub_s32\0"
  "svqsub_s32_m\0"
  "svqsub_s32_x\0"
  "svqsub_s32_z\0"
  "svqsub_s64\0"
  "svqsub_s64_m\0"
  "svqsub_s64_x\0"
  "svqsub_s64_z\0"
  "svqsub_s8\0"
  "svqsub_s8_m\0"
  "svqsub_s8_x\0"
  "svqsub_s8_z\0"
  "svqsub_u16\0"
  "svqsub_u16_m\0"
  "svqsub_u16_x\0"
  "svqsub_u16_z\0"
  "svqsub_u32\0"
  "svqsub_u32_m\0"
  "svqsub_u32_x\0"
  "svqsub_u32_z\0"
  "svqsub_u64\0"
  "svqsub_u64_m\0"
  "svqsub_u64_x\0"
  "svqsub_u64_z\0"
  "svqsub_u8\0"
  "svqsub_u8_m\0"
  "svqsub_u8_x\0"
  "svqsub_u8_z\0"
  "svqsubr_n_s16_m\0"
  "svqsubr_n_s16_x\0"
  "svqsubr_n_s16_z\0"
  "svqsubr_n_s32_m\0"
  "svqsubr_n_s32_x\0"
  "svqsubr_n_s32_z\0"
  "svqsubr_n_s64_m\0"
  "svqsubr_n_s64_x\0"
  "svqsubr_n_s64_z\0"
  "svqsubr_n_s8_m\0"
  "svqsubr_n_s8_x\0"
  "svqsubr_n_s8_z\0"
  "svqsubr_n_u16_m\0"
  "svqsubr_n_u16_x\0"
  "svqsubr_n_u16_z\0"
  "svqsubr_n_u32_m\0"
  "svqsubr_n_u32_x\0"
  "svqsubr_n_u32_z\0"
  "svqsubr_n_u64_m\0"
  "svqsubr_n_u64_x\0"
  "svqsubr_n_u64_z\0"
  "svqsubr_n_u8_m\0"
  "svqsubr_n_u8_x\0"
  "svqsubr_n_u8_z\0"
  "svqsubr_s16_m\0"
  "svqsubr_s16_x\0"
  "svqsubr_s16_z\0"
  "svqsubr_s32_m\0"
  "svqsubr_s32_x\0"
  "svqsubr_s32_z\0"
  "svqsubr_s64_m\0"
  "svqsubr_s64_x\0"
  "svqsubr_s64_z\0"
  "svqsubr_s8_m\0"
  "svqsubr_s8_x\0"
  "svqsubr_s8_z\0"
  "svqsubr_u16_m\0"
  "svqsubr_u16_x\0"
  "svqsubr_u16_z\0"
  "svqsubr_u32_m\0"
  "svqsubr_u32_x\0"
  "svqsubr_u32_z\0"
  "svqsubr_u64_m\0"
  "svqsubr_u64_x\0"
  "svqsubr_u64_z\0"
  "svqsubr_u8_m\0"
  "svqsubr_u8_x\0"
  "svqsubr_u8_z\0"
  "svqxtnb_s16\0"
  "q16Scq8s\0"
  "svqxtnb_s32\0"
  "q8sq4i\0"
  "svqxtnb_s64\0"
  "q4iq2Wi\0"
  "svqxtnb_u16\0"
  "q16Ucq8Us\0"
  "svqxtnb_u32\0"
  "q8Usq4Ui\0"
  "svqxtnb_u64\0"
  "q4Uiq2UWi\0"
  "svqxtnt_s16\0"
  "q16Scq16Scq8s\0"
  "svqxtnt_s32\0"
  "q8sq8sq4i\0"
  "svqxtnt_s64\0"
  "q4iq4iq2Wi\0"
  "svqxtnt_u16\0"
  "q16Ucq16Ucq8Us\0"
  "svqxtnt_u32\0"
  "q8Usq8Usq4Ui\0"
  "svqxtnt_u64\0"
  "q4Uiq4Uiq2UWi\0"
  "svqxtunb_s16\0"
  "q16Ucq8s\0"
  "svqxtunb_s32\0"
  "q8Usq4i\0"
  "svqxtunb_s64\0"
  "q4Uiq2Wi\0"
  "svqxtunt_s16\0"
  "q16Ucq16Ucq8s\0"
  "svqxtunt_s32\0"
  "q8Usq8Usq4i\0"
  "svqxtunt_s64\0"
  "q4Uiq4Uiq2Wi\0"
  "svraddhnb_n_s16\0"
  "svraddhnb_n_s32\0"
  "svraddhnb_n_s64\0"
  "svraddhnb_n_u16\0"
  "svraddhnb_n_u32\0"
  "svraddhnb_n_u64\0"
  "svraddhnb_s16\0"
  "svraddhnb_s32\0"
  "svraddhnb_s64\0"
  "svraddhnb_u16\0"
  "svraddhnb_u32\0"
  "svraddhnb_u64\0"
  "svraddhnt_n_s16\0"
  "svraddhnt_n_s32\0"
  "svraddhnt_n_s64\0"
  "svraddhnt_n_u16\0"
  "svraddhnt_n_u32\0"
  "svraddhnt_n_u64\0"
  "svraddhnt_s16\0"
  "svraddhnt_s32\0"
  "svraddhnt_s64\0"
  "svraddhnt_u16\0"
  "svraddhnt_u32\0"
  "svraddhnt_u64\0"
  "svrax1_s64\0"
  "(sve,sve-sha3)|(sme,sme2p1,sve-sha3)\0"
  "svrax1_u64\0"
  "svrbit_s16_m\0"
  "svrbit_s16_x\0"
  "svrbit_s16_z\0"
  "svrbit_s32_m\0"
  "svrbit_s32_x\0"
  "svrbit_s32_z\0"
  "svrbit_s64_m\0"
  "svrbit_s64_x\0"
  "svrbit_s64_z\0"
  "svrbit_s8_m\0"
  "svrbit_s8_x\0"
  "svrbit_s8_z\0"
  "svrbit_u16_m\0"
  "svrbit_u16_x\0"
  "svrbit_u16_z\0"
  "svrbit_u32_m\0"
  "svrbit_u32_x\0"
  "svrbit_u32_z\0"
  "svrbit_u64_m\0"
  "svrbit_u64_x\0"
  "svrbit_u64_z\0"
  "svrbit_u8_m\0"
  "svrbit_u8_x\0"
  "svrbit_u8_z\0"
  "svrdffr\0"
  "svrdffr_z\0"
  "q16bq16b\0"
  "svrecpe_f16\0"
  "q8hq8h\0"
  "svrecpe_f32\0"
  "q4fq4f\0"
  "svrecpe_f64\0"
  "q2dq2d\0"
  "svrecpe_u32_m\0"
  "svrecpe_u32_x\0"
  "svrecpe_u32_z\0"
  "svrecps_f16\0"
  "q8hq8hq8h\0"
  "svrecps_f32\0"
  "q4fq4fq4f\0"
  "svrecps_f64\0"
  "q2dq2dq2d\0"
  "svrecpx_f16_m\0"
  "svrecpx_f16_x\0"
  "svrecpx_f16_z\0"
  "svrecpx_f32_m\0"
  "svrecpx_f32_x\0"
  "svrecpx_f32_z\0"
  "svrecpx_f64_m\0"
  "svrecpx_f64_x\0"
  "svrecpx_f64_z\0"
  "svreinterpret_b\0"
  "q16bQa\0"
  "svreinterpret_c\0"
  "Qaq16b\0"
  "svrev_b16\0"
  "svrev_b32\0"
  "svrev_b64\0"
  "svrev_b8\0"
  "svrev_bf16\0"
  "q8yq8y\0"
  "svrev_f16\0"
  "svrev_f32\0"
  "svrev_f64\0"
  "svrev_s16\0"
  "q8sq8s\0"
  "svrev_s32\0"
  "q4iq4i\0"
  "svrev_s64\0"
  "q2Wiq2Wi\0"
  "svrev_s8\0"
  "q16Scq16Sc\0"
  "svrev_u16\0"
  "q8Usq8Us\0"
  "svrev_u32\0"
  "q4Uiq4Ui\0"
  "svrev_u64\0"
  "q2UWiq2UWi\0"
  "svrev_u8\0"
  "svrevb_s16_m\0"
  "svrevb_s16_x\0"
  "svrevb_s16_z\0"
  "svrevb_s32_m\0"
  "svrevb_s32_x\0"
  "svrevb_s32_z\0"
  "svrevb_s64_m\0"
  "svrevb_s64_x\0"
  "svrevb_s64_z\0"
  "svrevb_u16_m\0"
  "svrevb_u16_x\0"
  "svrevb_u16_z\0"
  "svrevb_u32_m\0"
  "svrevb_u32_x\0"
  "svrevb_u32_z\0"
  "svrevb_u64_m\0"
  "svrevb_u64_x\0"
  "svrevb_u64_z\0"
  "svrevd_bf16_m\0"
  "q8yq8yq16bq8y\0"
  "svrevd_bf16_x\0"
  "svrevd_bf16_z\0"
  "svrevd_f16_m\0"
  "svrevd_f16_x\0"
  "svrevd_f16_z\0"
  "svrevd_f32_m\0"
  "svrevd_f32_x\0"
  "svrevd_f32_z\0"
  "svrevd_f64_m\0"
  "svrevd_f64_x\0"
  "svrevd_f64_z\0"
  "svrevd_mf8_m\0"
  "q16mq16mq16bq16m\0"
  "svrevd_mf8_x\0"
  "svrevd_mf8_z\0"
  "svrevd_s16_m\0"
  "svrevd_s16_x\0"
  "svrevd_s16_z\0"
  "svrevd_s32_m\0"
  "svrevd_s32_x\0"
  "svrevd_s32_z\0"
  "svrevd_s64_m\0"
  "svrevd_s64_x\0"
  "svrevd_s64_z\0"
  "svrevd_s8_m\0"
  "svrevd_s8_x\0"
  "svrevd_s8_z\0"
  "svrevd_u16_m\0"
  "svrevd_u16_x\0"
  "svrevd_u16_z\0"
  "svrevd_u32_m\0"
  "svrevd_u32_x\0"
  "svrevd_u32_z\0"
  "svrevd_u64_m\0"
  "svrevd_u64_x\0"
  "svrevd_u64_z\0"
  "svrevd_u8_m\0"
  "svrevd_u8_x\0"
  "svrevd_u8_z\0"
  "svrevh_s32_m\0"
  "svrevh_s32_x\0"
  "svrevh_s32_z\0"
  "svrevh_s64_m\0"
  "svrevh_s64_x\0"
  "svrevh_s64_z\0"
  "svrevh_u32_m\0"
  "svrevh_u32_x\0"
  "svrevh_u32_z\0"
  "svrevh_u64_m\0"
  "svrevh_u64_x\0"
  "svrevh_u64_z\0"
  "svrevw_s64_m\0"
  "svrevw_s64_x\0"
  "svrevw_s64_z\0"
  "svrevw_u64_m\0"
  "svrevw_u64_x\0"
  "svrevw_u64_z\0"
  "svrhadd_n_s16_m\0"
  "svrhadd_n_s16_x\0"
  "svrhadd_n_s16_z\0"
  "svrhadd_n_s32_m\0"
  "svrhadd_n_s32_x\0"
  "svrhadd_n_s32_z\0"
  "svrhadd_n_s64_m\0"
  "svrhadd_n_s64_x\0"
  "svrhadd_n_s64_z\0"
  "svrhadd_n_s8_m\0"
  "svrhadd_n_s8_x\0"
  "svrhadd_n_s8_z\0"
  "svrhadd_n_u16_m\0"
  "svrhadd_n_u16_x\0"
  "svrhadd_n_u16_z\0"
  "svrhadd_n_u32_m\0"
  "svrhadd_n_u32_x\0"
  "svrhadd_n_u32_z\0"
  "svrhadd_n_u64_m\0"
  "svrhadd_n_u64_x\0"
  "svrhadd_n_u64_z\0"
  "svrhadd_n_u8_m\0"
  "svrhadd_n_u8_x\0"
  "svrhadd_n_u8_z\0"
  "svrhadd_s16_m\0"
  "svrhadd_s16_x\0"
  "svrhadd_s16_z\0"
  "svrhadd_s32_m\0"
  "svrhadd_s32_x\0"
  "svrhadd_s32_z\0"
  "svrhadd_s64_m\0"
  "svrhadd_s64_x\0"
  "svrhadd_s64_z\0"
  "svrhadd_s8_m\0"
  "svrhadd_s8_x\0"
  "svrhadd_s8_z\0"
  "svrhadd_u16_m\0"
  "svrhadd_u16_x\0"
  "svrhadd_u16_z\0"
  "svrhadd_u32_m\0"
  "svrhadd_u32_x\0"
  "svrhadd_u32_z\0"
  "svrhadd_u64_m\0"
  "svrhadd_u64_x\0"
  "svrhadd_u64_z\0"
  "svrhadd_u8_m\0"
  "svrhadd_u8_x\0"
  "svrhadd_u8_z\0"
  "svrint32x_f32_m\0"
  "svrint32x_f32_x\0"
  "svrint32x_f32_z\0"
  "svrint32x_f64_m\0"
  "svrint32x_f64_x\0"
  "svrint32x_f64_z\0"
  "svrint32z_f32_m\0"
  "svrint32z_f32_x\0"
  "svrint32z_f32_z\0"
  "svrint32z_f64_m\0"
  "svrint32z_f64_x\0"
  "svrint32z_f64_z\0"
  "svrint64x_f32_m\0"
  "svrint64x_f32_x\0"
  "svrint64x_f32_z\0"
  "svrint64x_f64_m\0"
  "svrint64x_f64_x\0"
  "svrint64x_f64_z\0"
  "svrint64z_f32_m\0"
  "svrint64z_f32_x\0"
  "svrint64z_f32_z\0"
  "svrint64z_f64_m\0"
  "svrint64z_f64_x\0"
  "svrint64z_f64_z\0"
  "svrinta_f16_m\0"
  "svrinta_f16_x\0"
  "svrinta_f16_z\0"
  "svrinta_f32_m\0"
  "svrinta_f32_x\0"
  "svrinta_f32_x2\0"
  "q8fq8f\0"
  "svrinta_f32_x4\0"
  "q16fq16f\0"
  "svrinta_f32_z\0"
  "svrinta_f64_m\0"
  "svrinta_f64_x\0"
  "svrinta_f64_z\0"
  "svrinti_f16_m\0"
  "svrinti_f16_x\0"
  "svrinti_f16_z\0"
  "svrinti_f32_m\0"
  "svrinti_f32_x\0"
  "svrinti_f32_z\0"
  "svrinti_f64_m\0"
  "svrinti_f64_x\0"
  "svrinti_f64_z\0"
  "svrintm_f16_m\0"
  "svrintm_f16_x\0"
  "svrintm_f16_z\0"
  "svrintm_f32_m\0"
  "svrintm_f32_x\0"
  "svrintm_f32_x2\0"
  "svrintm_f32_x4\0"
  "svrintm_f32_z\0"
  "svrintm_f64_m\0"
  "svrintm_f64_x\0"
  "svrintm_f64_z\0"
  "svrintn_f16_m\0"
  "svrintn_f16_x\0"
  "svrintn_f16_z\0"
  "svrintn_f32_m\0"
  "svrintn_f32_x\0"
  "svrintn_f32_x2\0"
  "svrintn_f32_x4\0"
  "svrintn_f32_z\0"
  "svrintn_f64_m\0"
  "svrintn_f64_x\0"
  "svrintn_f64_z\0"
  "svrintp_f16_m\0"
  "svrintp_f16_x\0"
  "svrintp_f16_z\0"
  "svrintp_f32_m\0"
  "svrintp_f32_x\0"
  "svrintp_f32_x2\0"
  "svrintp_f32_x4\0"
  "svrintp_f32_z\0"
  "svrintp_f64_m\0"
  "svrintp_f64_x\0"
  "svrintp_f64_z\0"
  "svrintx_f16_m\0"
  "svrintx_f16_x\0"
  "svrintx_f16_z\0"
  "svrintx_f32_m\0"
  "svrintx_f32_x\0"
  "svrintx_f32_z\0"
  "svrintx_f64_m\0"
  "svrintx_f64_x\0"
  "svrintx_f64_z\0"
  "svrintz_f16_m\0"
  "svrintz_f16_x\0"
  "svrintz_f16_z\0"
  "svrintz_f32_m\0"
  "svrintz_f32_x\0"
  "svrintz_f32_z\0"
  "svrintz_f64_m\0"
  "svrintz_f64_x\0"
  "svrintz_f64_z\0"
  "svrshl_n_s16_m\0"
  "svrshl_n_s16_x\0"
  "svrshl_n_s16_z\0"
  "svrshl_n_s32_m\0"
  "svrshl_n_s32_x\0"
  "svrshl_n_s32_z\0"
  "svrshl_n_s64_m\0"
  "svrshl_n_s64_x\0"
  "svrshl_n_s64_z\0"
  "svrshl_n_s8_m\0"
  "svrshl_n_s8_x\0"
  "svrshl_n_s8_z\0"
  "svrshl_n_u16_m\0"
  "svrshl_n_u16_x\0"
  "svrshl_n_u16_z\0"
  "svrshl_n_u32_m\0"
  "svrshl_n_u32_x\0"
  "svrshl_n_u32_z\0"
  "svrshl_n_u64_m\0"
  "svrshl_n_u64_x\0"
  "svrshl_n_u64_z\0"
  "svrshl_n_u8_m\0"
  "svrshl_n_u8_x\0"
  "svrshl_n_u8_z\0"
  "svrshl_s16_m\0"
  "svrshl_s16_x\0"
  "svrshl_s16_x2\0"
  "svrshl_s16_x4\0"
  "svrshl_s16_z\0"
  "svrshl_s32_m\0"
  "svrshl_s32_x\0"
  "svrshl_s32_x2\0"
  "svrshl_s32_x4\0"
  "svrshl_s32_z\0"
  "svrshl_s64_m\0"
  "svrshl_s64_x\0"
  "svrshl_s64_x2\0"
  "svrshl_s64_x4\0"
  "svrshl_s64_z\0"
  "svrshl_s8_m\0"
  "svrshl_s8_x\0"
  "svrshl_s8_x2\0"
  "svrshl_s8_x4\0"
  "svrshl_s8_z\0"
  "svrshl_single_s16_x2\0"
  "svrshl_single_s16_x4\0"
  "svrshl_single_s32_x2\0"
  "svrshl_single_s32_x4\0"
  "svrshl_single_s64_x2\0"
  "svrshl_single_s64_x4\0"
  "svrshl_single_s8_x2\0"
  "svrshl_single_s8_x4\0"
  "svrshl_single_u16_x2\0"
  "svrshl_single_u16_x4\0"
  "svrshl_single_u32_x2\0"
  "svrshl_single_u32_x4\0"
  "svrshl_single_u64_x2\0"
  "svrshl_single_u64_x4\0"
  "svrshl_single_u8_x2\0"
  "svrshl_single_u8_x4\0"
  "svrshl_u16_m\0"
  "svrshl_u16_x\0"
  "svrshl_u16_x2\0"
  "svrshl_u16_x4\0"
  "svrshl_u16_z\0"
  "svrshl_u32_m\0"
  "svrshl_u32_x\0"
  "svrshl_u32_x2\0"
  "svrshl_u32_x4\0"
  "svrshl_u32_z\0"
  "svrshl_u64_m\0"
  "svrshl_u64_x\0"
  "svrshl_u64_x2\0"
  "svrshl_u64_x4\0"
  "svrshl_u64_z\0"
  "svrshl_u8_m\0"
  "svrshl_u8_x\0"
  "svrshl_u8_x2\0"
  "svrshl_u8_x4\0"
  "svrshl_u8_z\0"
  "svrshr_n_s16_m\0"
  "svrshr_n_s16_x\0"
  "svrshr_n_s16_z\0"
  "svrshr_n_s32_m\0"
  "svrshr_n_s32_x\0"
  "svrshr_n_s32_z\0"
  "svrshr_n_s64_m\0"
  "svrshr_n_s64_x\0"
  "svrshr_n_s64_z\0"
  "svrshr_n_s8_m\0"
  "svrshr_n_s8_x\0"
  "svrshr_n_s8_z\0"
  "svrshr_n_u16_m\0"
  "q8Usq16bq8UsIUWi\0"
  "svrshr_n_u16_x\0"
  "svrshr_n_u16_z\0"
  "svrshr_n_u32_m\0"
  "q4Uiq16bq4UiIUWi\0"
  "svrshr_n_u32_x\0"
  "svrshr_n_u32_z\0"
  "svrshr_n_u64_m\0"
  "q2UWiq16bq2UWiIUWi\0"
  "svrshr_n_u64_x\0"
  "svrshr_n_u64_z\0"
  "svrshr_n_u8_m\0"
  "q16Ucq16bq16UcIUWi\0"
  "svrshr_n_u8_x\0"
  "svrshr_n_u8_z\0"
  "svrshrnb_n_s16\0"
  "svrshrnb_n_s32\0"
  "svrshrnb_n_s64\0"
  "svrshrnb_n_u16\0"
  "svrshrnb_n_u32\0"
  "svrshrnb_n_u64\0"
  "svrshrnt_n_s16\0"
  "svrshrnt_n_s32\0"
  "svrshrnt_n_s64\0"
  "svrshrnt_n_u16\0"
  "svrshrnt_n_u32\0"
  "svrshrnt_n_u64\0"
  "svrsqrte_f16\0"
  "svrsqrte_f32\0"
  "svrsqrte_f64\0"
  "svrsqrte_u32_m\0"
  "svrsqrte_u32_x\0"
  "svrsqrte_u32_z\0"
  "svrsqrts_f16\0"
  "svrsqrts_f32\0"
  "svrsqrts_f64\0"
  "svrsra_n_s16\0"
  "svrsra_n_s32\0"
  "svrsra_n_s64\0"
  "svrsra_n_s8\0"
  "svrsra_n_u16\0"
  "svrsra_n_u32\0"
  "svrsra_n_u64\0"
  "svrsra_n_u8\0"
  "svrsubhnb_n_s16\0"
  "svrsubhnb_n_s32\0"
  "svrsubhnb_n_s64\0"
  "svrsubhnb_n_u16\0"
  "svrsubhnb_n_u32\0"
  "svrsubhnb_n_u64\0"
  "svrsubhnb_s16\0"
  "svrsubhnb_s32\0"
  "svrsubhnb_s64\0"
  "svrsubhnb_u16\0"
  "svrsubhnb_u32\0"
  "svrsubhnb_u64\0"
  "svrsubhnt_n_s16\0"
  "svrsubhnt_n_s32\0"
  "svrsubhnt_n_s64\0"
  "svrsubhnt_n_u16\0"
  "svrsubhnt_n_u32\0"
  "svrsubhnt_n_u64\0"
  "svrsubhnt_s16\0"
  "svrsubhnt_s32\0"
  "svrsubhnt_s64\0"
  "svrsubhnt_u16\0"
  "svrsubhnt_u32\0"
  "svrsubhnt_u64\0"
  "svsbclb_n_u32\0"
  "svsbclb_n_u64\0"
  "svsbclb_u32\0"
  "svsbclb_u64\0"
  "svsbclt_n_u32\0"
  "svsbclt_n_u64\0"
  "svsbclt_u32\0"
  "svsbclt_u64\0"
  "svscale_bf16_m\0"
  "q8yq16bq8yq8s\0"
  "(sve,sve-bfscale)|(sme,sve-bfscale,sme2)\0"
  "svscale_bf16_x\0"
  "svscale_bf16_x2\0"
  "q16yq16yq16s\0"
  "svscale_bf16_x4\0"
  "q32yq32yq32s\0"
  "svscale_bf16_z\0"
  "svscale_f16_m\0"
  "q8hq16bq8hq8s\0"
  "svscale_f16_x\0"
  "svscale_f16_x2\0"
  "q16hq16hq16s\0"
  "svscale_f16_x4\0"
  "q32hq32hq32s\0"
  "svscale_f16_z\0"
  "svscale_f32_m\0"
  "q4fq16bq4fq4i\0"
  "svscale_f32_x\0"
  "svscale_f32_x2\0"
  "q8fq8fq8i\0"
  "svscale_f32_x4\0"
  "q16fq16fq16i\0"
  "svscale_f32_z\0"
  "svscale_f64_m\0"
  "q2dq16bq2dq2Wi\0"
  "svscale_f64_x\0"
  "svscale_f64_x2\0"
  "q4dq4dq4Wi\0"
  "svscale_f64_x4\0"
  "q8dq8dq8Wi\0"
  "svscale_f64_z\0"
  "svscale_n_bf16_m\0"
  "q8yq16bq8ys\0"
  "svscale_n_bf16_x\0"
  "svscale_n_bf16_z\0"
  "svscale_n_f16_m\0"
  "q8hq16bq8hs\0"
  "svscale_n_f16_x\0"
  "svscale_n_f16_z\0"
  "svscale_n_f32_m\0"
  "q4fq16bq4fi\0"
  "svscale_n_f32_x\0"
  "svscale_n_f32_z\0"
  "svscale_n_f64_m\0"
  "q2dq16bq2dWi\0"
  "svscale_n_f64_x\0"
  "svscale_n_f64_z\0"
  "svscale_single_bf16_x2\0"
  "q16yq16yq8s\0"
  "svscale_single_bf16_x4\0"
  "q32yq32yq8s\0"
  "svscale_single_f16_x2\0"
  "q16hq16hq8s\0"
  "svscale_single_f16_x4\0"
  "q32hq32hq8s\0"
  "svscale_single_f32_x2\0"
  "q8fq8fq4i\0"
  "svscale_single_f32_x4\0"
  "q16fq16fq4i\0"
  "svscale_single_f64_x2\0"
  "q4dq4dq2Wi\0"
  "svscale_single_f64_x4\0"
  "q8dq8dq2Wi\0"
  "svsel_b\0"
  "svsel_bf16\0"
  "svsel_bf16_x2\0"
  "q16yQaq16yq16y\0"
  "svsel_bf16_x4\0"
  "q32yQaq32yq32y\0"
  "svsel_f16\0"
  "svsel_f16_x2\0"
  "q16hQaq16hq16h\0"
  "svsel_f16_x4\0"
  "q32hQaq32hq32h\0"
  "svsel_f32\0"
  "svsel_f32_x2\0"
  "q8fQaq8fq8f\0"
  "svsel_f32_x4\0"
  "q16fQaq16fq16f\0"
  "svsel_f64\0"
  "svsel_f64_x2\0"
  "q4dQaq4dq4d\0"
  "svsel_f64_x4\0"
  "q8dQaq8dq8d\0"
  "svsel_mf8_x2\0"
  "q32mQaq32mq32m\0"
  "svsel_mf8_x4\0"
  "q64mQaq64mq64m\0"
  "svsel_s16\0"
  "svsel_s16_x2\0"
  "q16sQaq16sq16s\0"
  "svsel_s16_x4\0"
  "q32sQaq32sq32s\0"
  "svsel_s32\0"
  "svsel_s32_x2\0"
  "q8iQaq8iq8i\0"
  "svsel_s32_x4\0"
  "q16iQaq16iq16i\0"
  "svsel_s64\0"
  "svsel_s64_x2\0"
  "q4WiQaq4Wiq4Wi\0"
  "svsel_s64_x4\0"
  "q8WiQaq8Wiq8Wi\0"
  "svsel_s8\0"
  "svsel_s8_x2\0"
  "q32ScQaq32Scq32Sc\0"
  "svsel_s8_x4\0"
  "q64ScQaq64Scq64Sc\0"
  "svsel_u16\0"
  "svsel_u16_x2\0"
  "q16UsQaq16Usq16Us\0"
  "svsel_u16_x4\0"
  "q32UsQaq32Usq32Us\0"
  "svsel_u32\0"
  "svsel_u32_x2\0"
  "q8UiQaq8Uiq8Ui\0"
  "svsel_u32_x4\0"
  "q16UiQaq16Uiq16Ui\0"
  "svsel_u64\0"
  "svsel_u64_x2\0"
  "q4UWiQaq4UWiq4UWi\0"
  "svsel_u64_x4\0"
  "q8UWiQaq8UWiq8UWi\0"
  "svsel_u8\0"
  "svsel_u8_x2\0"
  "q32UcQaq32Ucq32Uc\0"
  "svsel_u8_x4\0"
  "q64UcQaq64Ucq64Uc\0"
  "svset2_b\0"
  "q32bq32bIUWiq16b\0"
  "svset2_bf16\0"
  "q16yq16yIUWiq8y\0"
  "svset2_f16\0"
  "q16hq16hIUWiq8h\0"
  "svset2_f32\0"
  "q8fq8fIUWiq4f\0"
  "svset2_f64\0"
  "q4dq4dIUWiq2d\0"
  "svset2_mf8\0"
  "q32mq32mIUWiq16m\0"
  "svset2_s16\0"
  "q16sq16sIUWiq8s\0"
  "svset2_s32\0"
  "q8iq8iIUWiq4i\0"
  "svset2_s64\0"
  "q4Wiq4WiIUWiq2Wi\0"
  "svset2_s8\0"
  "q32Scq32ScIUWiq16Sc\0"
  "svset2_u16\0"
  "q16Usq16UsIUWiq8Us\0"
  "svset2_u32\0"
  "q8Uiq8UiIUWiq4Ui\0"
  "svset2_u64\0"
  "q4UWiq4UWiIUWiq2UWi\0"
  "svset2_u8\0"
  "q32Ucq32UcIUWiq16Uc\0"
  "svset3_bf16\0"
  "q24yq24yIUWiq8y\0"
  "svset3_f16\0"
  "q24hq24hIUWiq8h\0"
  "svset3_f32\0"
  "q12fq12fIUWiq4f\0"
  "svset3_f64\0"
  "q6dq6dIUWiq2d\0"
  "svset3_mf8\0"
  "q48mq48mIUWiq16m\0"
  "svset3_s16\0"
  "q24sq24sIUWiq8s\0"
  "svset3_s32\0"
  "q12iq12iIUWiq4i\0"
  "svset3_s64\0"
  "q6Wiq6WiIUWiq2Wi\0"
  "svset3_s8\0"
  "q48Scq48ScIUWiq16Sc\0"
  "svset3_u16\0"
  "q24Usq24UsIUWiq8Us\0"
  "svset3_u32\0"
  "q12Uiq12UiIUWiq4Ui\0"
  "svset3_u64\0"
  "q6UWiq6UWiIUWiq2UWi\0"
  "svset3_u8\0"
  "q48Ucq48UcIUWiq16Uc\0"
  "svset4_b\0"
  "q64bq64bIUWiq16b\0"
  "svset4_bf16\0"
  "q32yq32yIUWiq8y\0"
  "svset4_f16\0"
  "q32hq32hIUWiq8h\0"
  "svset4_f32\0"
  "q16fq16fIUWiq4f\0"
  "svset4_f64\0"
  "q8dq8dIUWiq2d\0"
  "svset4_mf8\0"
  "q64mq64mIUWiq16m\0"
  "svset4_s16\0"
  "q32sq32sIUWiq8s\0"
  "svset4_s32\0"
  "q16iq16iIUWiq4i\0"
  "svset4_s64\0"
  "q8Wiq8WiIUWiq2Wi\0"
  "svset4_s8\0"
  "q64Scq64ScIUWiq16Sc\0"
  "svset4_u16\0"
  "q32Usq32UsIUWiq8Us\0"
  "svset4_u32\0"
  "q16Uiq16UiIUWiq4Ui\0"
  "svset4_u64\0"
  "q8UWiq8UWiIUWiq2UWi\0"
  "svset4_u8\0"
  "q64Ucq64UcIUWiq16Uc\0"
  "svsetffr\0"
  "vv\0"
  "svshllb_n_s16\0"
  "q8sq16ScIUWi\0"
  "svshllb_n_s32\0"
  "q4iq8sIUWi\0"
  "svshllb_n_s64\0"
  "q2Wiq4iIUWi\0"
  "svshllb_n_u16\0"
  "q8Usq16UcIUWi\0"
  "svshllb_n_u32\0"
  "q4Uiq8UsIUWi\0"
  "svshllb_n_u64\0"
  "q2UWiq4UiIUWi\0"
  "svshllt_n_s16\0"
  "svshllt_n_s32\0"
  "svshllt_n_s64\0"
  "svshllt_n_u16\0"
  "svshllt_n_u32\0"
  "svshllt_n_u64\0"
  "svshrnb_n_s16\0"
  "svshrnb_n_s32\0"
  "svshrnb_n_s64\0"
  "svshrnb_n_u16\0"
  "svshrnb_n_u32\0"
  "svshrnb_n_u64\0"
  "svshrnt_n_s16\0"
  "svshrnt_n_s32\0"
  "svshrnt_n_s64\0"
  "svshrnt_n_u16\0"
  "svshrnt_n_u32\0"
  "svshrnt_n_u64\0"
  "svsli_n_s16\0"
  "svsli_n_s32\0"
  "svsli_n_s64\0"
  "svsli_n_s8\0"
  "svsli_n_u16\0"
  "svsli_n_u32\0"
  "svsli_n_u64\0"
  "svsli_n_u8\0"
  "svsm4e_u32\0"
  "sve,sve-sm4\0"
  "svsm4ekey_u32\0"
  "svsplice_bf16\0"
  "svsplice_f16\0"
  "svsplice_f32\0"
  "svsplice_f64\0"
  "svsplice_s16\0"
  "svsplice_s32\0"
  "svsplice_s64\0"
  "svsplice_s8\0"
  "svsplice_u16\0"
  "svsplice_u32\0"
  "svsplice_u64\0"
  "svsplice_u8\0"
  "svsqadd_n_u16_m\0"
  "svsqadd_n_u16_x\0"
  "svsqadd_n_u16_z\0"
  "svsqadd_n_u32_m\0"
  "svsqadd_n_u32_x\0"
  "svsqadd_n_u32_z\0"
  "svsqadd_n_u64_m\0"
  "svsqadd_n_u64_x\0"
  "svsqadd_n_u64_z\0"
  "svsqadd_n_u8_m\0"
  "svsqadd_n_u8_x\0"
  "svsqadd_n_u8_z\0"
  "svsqadd_u16_m\0"
  "svsqadd_u16_x\0"
  "svsqadd_u16_z\0"
  "svsqadd_u32_m\0"
  "svsqadd_u32_x\0"
  "svsqadd_u32_z\0"
  "svsqadd_u64_m\0"
  "svsqadd_u64_x\0"
  "svsqadd_u64_z\0"
  "svsqadd_u8_m\0"
  "svsqadd_u8_x\0"
  "svsqadd_u8_z\0"
  "svsqrt_f16_m\0"
  "svsqrt_f16_x\0"
  "svsqrt_f16_z\0"
  "svsqrt_f32_m\0"
  "svsqrt_f32_x\0"
  "svsqrt_f32_z\0"
  "svsqrt_f64_m\0"
  "svsqrt_f64_x\0"
  "svsqrt_f64_z\0"
  "svsra_n_s16\0"
  "svsra_n_s32\0"
  "svsra_n_s64\0"
  "svsra_n_s8\0"
  "svsra_n_u16\0"
  "svsra_n_u32\0"
  "svsra_n_u64\0"
  "svsra_n_u8\0"
  "svsri_n_s16\0"
  "svsri_n_s32\0"
  "svsri_n_s64\0"
  "svsri_n_s8\0"
  "svsri_n_u16\0"
  "svsri_n_u32\0"
  "svsri_n_u64\0"
  "svsri_n_u8\0"
  "svst1_bf16\0"
  "vq16by*q8y\0"
  "svst1_bf16_x2\0"
  "vQay*q16y\0"
  "svst1_bf16_x4\0"
  "vQay*q32y\0"
  "svst1_f16\0"
  "vq16bh*q8h\0"
  "svst1_f16_x2\0"
  "vQah*q16h\0"
  "svst1_f16_x4\0"
  "vQah*q32h\0"
  "svst1_f32\0"
  "vq16bf*q4f\0"
  "svst1_f32_x2\0"
  "vQaf*q8f\0"
  "svst1_f32_x4\0"
  "vQaf*q16f\0"
  "svst1_f64\0"
  "vq16bd*q2d\0"
  "svst1_f64_x2\0"
  "vQad*q4d\0"
  "svst1_f64_x4\0"
  "vQad*q8d\0"
  "svst1_mf8\0"
  "vq16bm*q16m\0"
  "svst1_mf8_x2\0"
  "vQam*q32m\0"
  "svst1_mf8_x4\0"
  "vQam*q64m\0"
  "svst1_s16\0"
  "vq16bSs*q8s\0"
  "svst1_s16_x2\0"
  "vQaSs*q16s\0"
  "svst1_s16_x4\0"
  "vQaSs*q32s\0"
  "svst1_s32\0"
  "vq16bSi*q4i\0"
  "svst1_s32_x2\0"
  "vQaSi*q8i\0"
  "svst1_s32_x4\0"
  "vQaSi*q16i\0"
  "svst1_s64\0"
  "vq16bSWi*q2Wi\0"
  "svst1_s64_x2\0"
  "vQaSWi*q4Wi\0"
  "svst1_s64_x4\0"
  "vQaSWi*q8Wi\0"
  "svst1_s8\0"
  "vq16bSc*q16Sc\0"
  "svst1_s8_x2\0"
  "vQaSc*q32Sc\0"
  "svst1_s8_x4\0"
  "vQaSc*q64Sc\0"
  "svst1_scatter_s32index_f32\0"
  "vq16bf*q4iq4f\0"
  "svst1_scatter_s32index_s32\0"
  "vq16bSi*q4iq4i\0"
  "svst1_scatter_s32index_u32\0"
  "vq16bUi*q4iq4Ui\0"
  "svst1_scatter_s32offset_f32\0"
  "svst1_scatter_s32offset_s32\0"
  "svst1_scatter_s32offset_u32\0"
  "svst1_scatter_s64index_f64\0"
  "vq16bd*q2Wiq2d\0"
  "svst1_scatter_s64index_s64\0"
  "vq16bSWi*q2Wiq2Wi\0"
  "svst1_scatter_s64index_u64\0"
  "vq16bUWi*q2Wiq2UWi\0"
  "svst1_scatter_s64offset_f64\0"
  "svst1_scatter_s64offset_s64\0"
  "svst1_scatter_s64offset_u64\0"
  "svst1_scatter_u32base_f32\0"
  "vq16bq4Uiq4f\0"
  "svst1_scatter_u32base_index_f32\0"
  "vq16bq4UiWiq4f\0"
  "svst1_scatter_u32base_index_s32\0"
  "vq16bq4UiWiq4i\0"
  "svst1_scatter_u32base_index_u32\0"
  "vq16bq4UiWiq4Ui\0"
  "svst1_scatter_u32base_offset_f32\0"
  "svst1_scatter_u32base_offset_s32\0"
  "svst1_scatter_u32base_offset_u32\0"
  "svst1_scatter_u32base_s32\0"
  "vq16bq4Uiq4i\0"
  "svst1_scatter_u32base_u32\0"
  "vq16bq4Uiq4Ui\0"
  "svst1_scatter_u32index_f32\0"
  "vq16bf*q4Uiq4f\0"
  "svst1_scatter_u32index_s32\0"
  "vq16bSi*q4Uiq4i\0"
  "svst1_scatter_u32index_u32\0"
  "vq16bUi*q4Uiq4Ui\0"
  "svst1_scatter_u32offset_f32\0"
  "svst1_scatter_u32offset_s32\0"
  "svst1_scatter_u32offset_u32\0"
  "svst1_scatter_u64base_f64\0"
  "vq16bq2UWiq2d\0"
  "svst1_scatter_u64base_index_f64\0"
  "vq16bq2UWiWiq2d\0"
  "svst1_scatter_u64base_index_s64\0"
  "vq16bq2UWiWiq2Wi\0"
  "svst1_scatter_u64base_index_u64\0"
  "vq16bq2UWiWiq2UWi\0"
  "svst1_scatter_u64base_offset_f64\0"
  "svst1_scatter_u64base_offset_s64\0"
  "svst1_scatter_u64base_offset_u64\0"
  "svst1_scatter_u64base_s64\0"
  "vq16bq2UWiq2Wi\0"
  "svst1_scatter_u64base_u64\0"
  "vq16bq2UWiq2UWi\0"
  "svst1_scatter_u64index_f64\0"
  "vq16bd*q2UWiq2d\0"
  "svst1_scatter_u64index_s64\0"
  "vq16bSWi*q2UWiq2Wi\0"
  "svst1_scatter_u64index_u64\0"
  "vq16bUWi*q2UWiq2UWi\0"
  "svst1_scatter_u64offset_f64\0"
  "svst1_scatter_u64offset_s64\0"
  "svst1_scatter_u64offset_u64\0"
  "svst1_u16\0"
  "vq16bUs*q8Us\0"
  "svst1_u16_x2\0"
  "vQaUs*q16Us\0"
  "svst1_u16_x4\0"
  "vQaUs*q32Us\0"
  "svst1_u32\0"
  "vq16bUi*q4Ui\0"
  "svst1_u32_x2\0"
  "vQaUi*q8Ui\0"
  "svst1_u32_x4\0"
  "vQaUi*q16Ui\0"
  "svst1_u64\0"
  "vq16bUWi*q2UWi\0"
  "svst1_u64_x2\0"
  "vQaUWi*q4UWi\0"
  "svst1_u64_x4\0"
  "vQaUWi*q8UWi\0"
  "svst1_u8\0"
  "vq16bUc*q16Uc\0"
  "svst1_u8_x2\0"
  "vQaUc*q32Uc\0"
  "svst1_u8_x4\0"
  "vQaUc*q64Uc\0"
  "svst1_vnum_bf16\0"
  "vq16by*Wiq8y\0"
  "svst1_vnum_bf16_x2\0"
  "vQay*Wiq16y\0"
  "svst1_vnum_bf16_x4\0"
  "vQay*Wiq32y\0"
  "svst1_vnum_f16\0"
  "vq16bh*Wiq8h\0"
  "svst1_vnum_f16_x2\0"
  "vQah*Wiq16h\0"
  "svst1_vnum_f16_x4\0"
  "vQah*Wiq32h\0"
  "svst1_vnum_f32\0"
  "vq16bf*Wiq4f\0"
  "svst1_vnum_f32_x2\0"
  "vQaf*Wiq8f\0"
  "svst1_vnum_f32_x4\0"
  "vQaf*Wiq16f\0"
  "svst1_vnum_f64\0"
  "vq16bd*Wiq2d\0"
  "svst1_vnum_f64_x2\0"
  "vQad*Wiq4d\0"
  "svst1_vnum_f64_x4\0"
  "vQad*Wiq8d\0"
  "svst1_vnum_mf8\0"
  "vq16bm*Wiq16m\0"
  "svst1_vnum_mf8_x2\0"
  "vQam*Wiq32m\0"
  "svst1_vnum_mf8_x4\0"
  "vQam*Wiq64m\0"
  "svst1_vnum_s16\0"
  "vq16bSs*Wiq8s\0"
  "svst1_vnum_s16_x2\0"
  "vQaSs*Wiq16s\0"
  "svst1_vnum_s16_x4\0"
  "vQaSs*Wiq32s\0"
  "svst1_vnum_s32\0"
  "vq16bSi*Wiq4i\0"
  "svst1_vnum_s32_x2\0"
  "vQaSi*Wiq8i\0"
  "svst1_vnum_s32_x4\0"
  "vQaSi*Wiq16i\0"
  "svst1_vnum_s64\0"
  "vq16bSWi*Wiq2Wi\0"
  "svst1_vnum_s64_x2\0"
  "vQaSWi*Wiq4Wi\0"
  "svst1_vnum_s64_x4\0"
  "vQaSWi*Wiq8Wi\0"
  "svst1_vnum_s8\0"
  "vq16bSc*Wiq16Sc\0"
  "svst1_vnum_s8_x2\0"
  "vQaSc*Wiq32Sc\0"
  "svst1_vnum_s8_x4\0"
  "vQaSc*Wiq64Sc\0"
  "svst1_vnum_u16\0"
  "vq16bUs*Wiq8Us\0"
  "svst1_vnum_u16_x2\0"
  "vQaUs*Wiq16Us\0"
  "svst1_vnum_u16_x4\0"
  "vQaUs*Wiq32Us\0"
  "svst1_vnum_u32\0"
  "vq16bUi*Wiq4Ui\0"
  "svst1_vnum_u32_x2\0"
  "vQaUi*Wiq8Ui\0"
  "svst1_vnum_u32_x4\0"
  "vQaUi*Wiq16Ui\0"
  "svst1_vnum_u64\0"
  "vq16bUWi*Wiq2UWi\0"
  "svst1_vnum_u64_x2\0"
  "vQaUWi*Wiq4UWi\0"
  "svst1_vnum_u64_x4\0"
  "vQaUWi*Wiq8UWi\0"
  "svst1_vnum_u8\0"
  "vq16bUc*Wiq16Uc\0"
  "svst1_vnum_u8_x2\0"
  "vQaUc*Wiq32Uc\0"
  "svst1_vnum_u8_x4\0"
  "vQaUc*Wiq64Uc\0"
  "svst1b_s16\0"
  "vq16bSc*q8s\0"
  "svst1b_s32\0"
  "vq16bSc*q4i\0"
  "svst1b_s64\0"
  "vq16bSc*q2Wi\0"
  "svst1b_scatter_s32offset_s32\0"
  "vq16bSc*q4iq4i\0"
  "svst1b_scatter_s32offset_u32\0"
  "vq16bUc*q4iq4Ui\0"
  "svst1b_scatter_s64offset_s64\0"
  "vq16bSc*q2Wiq2Wi\0"
  "svst1b_scatter_s64offset_u64\0"
  "vq16bUc*q2Wiq2UWi\0"
  "svst1b_scatter_u32base_offset_s32\0"
  "svst1b_scatter_u32base_offset_u32\0"
  "svst1b_scatter_u32base_s32\0"
  "svst1b_scatter_u32base_u32\0"
  "svst1b_scatter_u32offset_s32\0"
  "vq16bSc*q4Uiq4i\0"
  "svst1b_scatter_u32offset_u32\0"
  "vq16bUc*q4Uiq4Ui\0"
  "svst1b_scatter_u64base_offset_s64\0"
  "svst1b_scatter_u64base_offset_u64\0"
  "svst1b_scatter_u64base_s64\0"
  "svst1b_scatter_u64base_u64\0"
  "svst1b_scatter_u64offset_s64\0"
  "vq16bSc*q2UWiq2Wi\0"
  "svst1b_scatter_u64offset_u64\0"
  "vq16bUc*q2UWiq2UWi\0"
  "svst1b_u16\0"
  "vq16bUc*q8Us\0"
  "svst1b_u32\0"
  "vq16bUc*q4Ui\0"
  "svst1b_u64\0"
  "vq16bUc*q2UWi\0"
  "svst1b_vnum_s16\0"
  "vq16bSc*Wiq8s\0"
  "svst1b_vnum_s32\0"
  "vq16bSc*Wiq4i\0"
  "svst1b_vnum_s64\0"
  "vq16bSc*Wiq2Wi\0"
  "svst1b_vnum_u16\0"
  "vq16bUc*Wiq8Us\0"
  "svst1b_vnum_u32\0"
  "vq16bUc*Wiq4Ui\0"
  "svst1b_vnum_u64\0"
  "vq16bUc*Wiq2UWi\0"
  "svst1dq_f64\0"
  "svst1dq_s64\0"
  "svst1dq_u64\0"
  "svst1dq_vnum_f64\0"
  "svst1dq_vnum_s64\0"
  "svst1dq_vnum_u64\0"
  "svst1h_s32\0"
  "vq16bSs*q4i\0"
  "svst1h_s64\0"
  "vq16bSs*q2Wi\0"
  "svst1h_scatter_s32index_s32\0"
  "vq16bSs*q4iq4i\0"
  "svst1h_scatter_s32index_u32\0"
  "vq16bUs*q4iq4Ui\0"
  "svst1h_scatter_s32offset_s32\0"
  "svst1h_scatter_s32offset_u32\0"
  "svst1h_scatter_s64index_s64\0"
  "vq16bSs*q2Wiq2Wi\0"
  "svst1h_scatter_s64index_u64\0"
  "vq16bUs*q2Wiq2UWi\0"
  "svst1h_scatter_s64offset_s64\0"
  "svst1h_scatter_s64offset_u64\0"
  "svst1h_scatter_u32base_index_s32\0"
  "svst1h_scatter_u32base_index_u32\0"
  "svst1h_scatter_u32base_offset_s32\0"
  "svst1h_scatter_u32base_offset_u32\0"
  "svst1h_scatter_u32base_s32\0"
  "svst1h_scatter_u32base_u32\0"
  "svst1h_scatter_u32index_s32\0"
  "vq16bSs*q4Uiq4i\0"
  "svst1h_scatter_u32index_u32\0"
  "vq16bUs*q4Uiq4Ui\0"
  "svst1h_scatter_u32offset_s32\0"
  "svst1h_scatter_u32offset_u32\0"
  "svst1h_scatter_u64base_index_s64\0"
  "svst1h_scatter_u64base_index_u64\0"
  "svst1h_scatter_u64base_offset_s64\0"
  "svst1h_scatter_u64base_offset_u64\0"
  "svst1h_scatter_u64base_s64\0"
  "svst1h_scatter_u64base_u64\0"
  "svst1h_scatter_u64index_s64\0"
  "vq16bSs*q2UWiq2Wi\0"
  "svst1h_scatter_u64index_u64\0"
  "vq16bUs*q2UWiq2UWi\0"
  "svst1h_scatter_u64offset_s64\0"
  "svst1h_scatter_u64offset_u64\0"
  "svst1h_u32\0"
  "vq16bUs*q4Ui\0"
  "svst1h_u64\0"
  "vq16bUs*q2UWi\0"
  "svst1h_vnum_s32\0"
  "vq16bSs*Wiq4i\0"
  "svst1h_vnum_s64\0"
  "vq16bSs*Wiq2Wi\0"
  "svst1h_vnum_u32\0"
  "vq16bUs*Wiq4Ui\0"
  "svst1h_vnum_u64\0"
  "vq16bUs*Wiq2UWi\0"
  "svst1q_scatter_s64index_bf16\0"
  "vq16by*q2Wiq8y\0"
  "svst1q_scatter_s64index_f16\0"
  "vq16bh*q2Wiq8h\0"
  "svst1q_scatter_s64index_f32\0"
  "vq16bf*q2Wiq4f\0"
  "svst1q_scatter_s64index_f64\0"
  "svst1q_scatter_s64index_s16\0"
  "vq16bSs*q2Wiq8s\0"
  "svst1q_scatter_s64index_s32\0"
  "vq16bSi*q2Wiq4i\0"
  "svst1q_scatter_s64index_s64\0"
  "svst1q_scatter_s64index_u16\0"
  "vq16bUs*q2Wiq8Us\0"
  "svst1q_scatter_s64index_u32\0"
  "vq16bUi*q2Wiq4Ui\0"
  "svst1q_scatter_s64index_u64\0"
  "svst1q_scatter_s64offset_bf16\0"
  "svst1q_scatter_s64offset_f16\0"
  "svst1q_scatter_s64offset_f32\0"
  "svst1q_scatter_s64offset_f64\0"
  "svst1q_scatter_s64offset_mf8\0"
  "vq16bm*q2Wiq16m\0"
  "svst1q_scatter_s64offset_s16\0"
  "svst1q_scatter_s64offset_s32\0"
  "svst1q_scatter_s64offset_s64\0"
  "svst1q_scatter_s64offset_s8\0"
  "vq16bSc*q2Wiq16Sc\0"
  "svst1q_scatter_s64offset_u16\0"
  "svst1q_scatter_s64offset_u32\0"
  "svst1q_scatter_s64offset_u64\0"
  "svst1q_scatter_s64offset_u8\0"
  "vq16bUc*q2Wiq16Uc\0"
  "svst1q_scatter_u64base_bf16\0"
  "vq16bq2UWiq8y\0"
  "svst1q_scatter_u64base_f16\0"
  "vq16bq2UWiq8h\0"
  "svst1q_scatter_u64base_f32\0"
  "vq16bq2UWiq4f\0"
  "svst1q_scatter_u64base_f64\0"
  "svst1q_scatter_u64base_index_bf16\0"
  "vq16bq2UWiWiq8y\0"
  "svst1q_scatter_u64base_index_f16\0"
  "vq16bq2UWiWiq8h\0"
  "svst1q_scatter_u64base_index_f32\0"
  "vq16bq2UWiWiq4f\0"
  "svst1q_scatter_u64base_index_f64\0"
  "svst1q_scatter_u64base_index_s16\0"
  "vq16bq2UWiWiq8s\0"
  "svst1q_scatter_u64base_index_s32\0"
  "vq16bq2UWiWiq4i\0"
  "svst1q_scatter_u64base_index_s64\0"
  "svst1q_scatter_u64base_index_u16\0"
  "vq16bq2UWiWiq8Us\0"
  "svst1q_scatter_u64base_index_u32\0"
  "vq16bq2UWiWiq4Ui\0"
  "svst1q_scatter_u64base_index_u64\0"
  "svst1q_scatter_u64base_mf8\0"
  "vq16bq2UWiq16m\0"
  "svst1q_scatter_u64base_offset_bf16\0"
  "svst1q_scatter_u64base_offset_f16\0"
  "svst1q_scatter_u64base_offset_f32\0"
  "svst1q_scatter_u64base_offset_f64\0"
  "svst1q_scatter_u64base_offset_mf8\0"
  "vq16bq2UWiWiq16m\0"
  "svst1q_scatter_u64base_offset_s16\0"
  "svst1q_scatter_u64base_offset_s32\0"
  "svst1q_scatter_u64base_offset_s64\0"
  "svst1q_scatter_u64base_offset_s8\0"
  "vq16bq2UWiWiq16Sc\0"
  "svst1q_scatter_u64base_offset_u16\0"
  "svst1q_scatter_u64base_offset_u32\0"
  "svst1q_scatter_u64base_offset_u64\0"
  "svst1q_scatter_u64base_offset_u8\0"
  "vq16bq2UWiWiq16Uc\0"
  "svst1q_scatter_u64base_s16\0"
  "vq16bq2UWiq8s\0"
  "svst1q_scatter_u64base_s32\0"
  "vq16bq2UWiq4i\0"
  "svst1q_scatter_u64base_s64\0"
  "svst1q_scatter_u64base_s8\0"
  "vq16bq2UWiq16Sc\0"
  "svst1q_scatter_u64base_u16\0"
  "vq16bq2UWiq8Us\0"
  "svst1q_scatter_u64base_u32\0"
  "vq16bq2UWiq4Ui\0"
  "svst1q_scatter_u64base_u64\0"
  "svst1q_scatter_u64base_u8\0"
  "vq16bq2UWiq16Uc\0"
  "svst1q_scatter_u64index_bf16\0"
  "vq16by*q2UWiq8y\0"
  "svst1q_scatter_u64index_f16\0"
  "vq16bh*q2UWiq8h\0"
  "svst1q_scatter_u64index_f32\0"
  "vq16bf*q2UWiq4f\0"
  "svst1q_scatter_u64index_f64\0"
  "svst1q_scatter_u64index_s16\0"
  "vq16bSs*q2UWiq8s\0"
  "svst1q_scatter_u64index_s32\0"
  "vq16bSi*q2UWiq4i\0"
  "svst1q_scatter_u64index_s64\0"
  "svst1q_scatter_u64index_u16\0"
  "vq16bUs*q2UWiq8Us\0"
  "svst1q_scatter_u64index_u32\0"
  "vq16bUi*q2UWiq4Ui\0"
  "svst1q_scatter_u64index_u64\0"
  "svst1q_scatter_u64offset_bf16\0"
  "svst1q_scatter_u64offset_f16\0"
  "svst1q_scatter_u64offset_f32\0"
  "svst1q_scatter_u64offset_f64\0"
  "svst1q_scatter_u64offset_mf8\0"
  "vq16bm*q2UWiq16m\0"
  "svst1q_scatter_u64offset_s16\0"
  "svst1q_scatter_u64offset_s32\0"
  "svst1q_scatter_u64offset_s64\0"
  "svst1q_scatter_u64offset_s8\0"
  "vq16bSc*q2UWiq16Sc\0"
  "svst1q_scatter_u64offset_u16\0"
  "svst1q_scatter_u64offset_u32\0"
  "svst1q_scatter_u64offset_u64\0"
  "svst1q_scatter_u64offset_u8\0"
  "vq16bUc*q2UWiq16Uc\0"
  "svst1w_s64\0"
  "vq16bSi*q2Wi\0"
  "svst1w_scatter_s64index_s64\0"
  "vq16bSi*q2Wiq2Wi\0"
  "svst1w_scatter_s64index_u64\0"
  "vq16bUi*q2Wiq2UWi\0"
  "svst1w_scatter_s64offset_s64\0"
  "svst1w_scatter_s64offset_u64\0"
  "svst1w_scatter_u64base_index_s64\0"
  "svst1w_scatter_u64base_index_u64\0"
  "svst1w_scatter_u64base_offset_s64\0"
  "svst1w_scatter_u64base_offset_u64\0"
  "svst1w_scatter_u64base_s64\0"
  "svst1w_scatter_u64base_u64\0"
  "svst1w_scatter_u64index_s64\0"
  "vq16bSi*q2UWiq2Wi\0"
  "svst1w_scatter_u64index_u64\0"
  "vq16bUi*q2UWiq2UWi\0"
  "svst1w_scatter_u64offset_s64\0"
  "svst1w_scatter_u64offset_u64\0"
  "svst1w_u64\0"
  "vq16bUi*q2UWi\0"
  "svst1w_vnum_s64\0"
  "vq16bSi*Wiq2Wi\0"
  "svst1w_vnum_u64\0"
  "vq16bUi*Wiq2UWi\0"
  "svst1wq_f32\0"
  "svst1wq_s32\0"
  "svst1wq_u32\0"
  "svst1wq_vnum_f32\0"
  "svst1wq_vnum_s32\0"
  "svst1wq_vnum_u32\0"
  "svst2_bf16\0"
  "vq16by*q16y\0"
  "svst2_f16\0"
  "vq16bh*q16h\0"
  "svst2_f32\0"
  "vq16bf*q8f\0"
  "svst2_f64\0"
  "vq16bd*q4d\0"
  "svst2_mf8\0"
  "vq16bm*q32m\0"
  "svst2_s16\0"
  "vq16bSs*q16s\0"
  "svst2_s32\0"
  "vq16bSi*q8i\0"
  "svst2_s64\0"
  "vq16bSWi*q4Wi\0"
  "svst2_s8\0"
  "vq16bSc*q32Sc\0"
  "svst2_u16\0"
  "vq16bUs*q16Us\0"
  "svst2_u32\0"
  "vq16bUi*q8Ui\0"
  "svst2_u64\0"
  "vq16bUWi*q4UWi\0"
  "svst2_u8\0"
  "vq16bUc*q32Uc\0"
  "svst2_vnum_bf16\0"
  "vq16by*Wiq16y\0"
  "svst2_vnum_f16\0"
  "vq16bh*Wiq16h\0"
  "svst2_vnum_f32\0"
  "vq16bf*Wiq8f\0"
  "svst2_vnum_f64\0"
  "vq16bd*Wiq4d\0"
  "svst2_vnum_mf8\0"
  "vq16bm*Wiq32m\0"
  "svst2_vnum_s16\0"
  "vq16bSs*Wiq16s\0"
  "svst2_vnum_s32\0"
  "vq16bSi*Wiq8i\0"
  "svst2_vnum_s64\0"
  "vq16bSWi*Wiq4Wi\0"
  "svst2_vnum_s8\0"
  "vq16bSc*Wiq32Sc\0"
  "svst2_vnum_u16\0"
  "vq16bUs*Wiq16Us\0"
  "svst2_vnum_u32\0"
  "vq16bUi*Wiq8Ui\0"
  "svst2_vnum_u64\0"
  "vq16bUWi*Wiq4UWi\0"
  "svst2_vnum_u8\0"
  "vq16bUc*Wiq32Uc\0"
  "svst2q_bf16\0"
  "vq16byC*q16y\0"
  "svst2q_f16\0"
  "vq16bhC*q16h\0"
  "svst2q_f32\0"
  "vq16bfC*q8f\0"
  "svst2q_f64\0"
  "vq16bdC*q4d\0"
  "svst2q_mf8\0"
  "vq16bmC*q32m\0"
  "svst2q_s16\0"
  "vq16bSsC*q16s\0"
  "svst2q_s32\0"
  "vq16bSiC*q8i\0"
  "svst2q_s64\0"
  "vq16bSWiC*q4Wi\0"
  "svst2q_s8\0"
  "vq16bScC*q32Sc\0"
  "svst2q_u16\0"
  "vq16bUsC*q16Us\0"
  "svst2q_u32\0"
  "vq16bUiC*q8Ui\0"
  "svst2q_u64\0"
  "vq16bUWiC*q4UWi\0"
  "svst2q_u8\0"
  "vq16bUcC*q32Uc\0"
  "svst2q_vnum_bf16\0"
  "vq16byC*Wiq16y\0"
  "svst2q_vnum_f16\0"
  "vq16bhC*Wiq16h\0"
  "svst2q_vnum_f32\0"
  "vq16bfC*Wiq8f\0"
  "svst2q_vnum_f64\0"
  "vq16bdC*Wiq4d\0"
  "svst2q_vnum_mf8\0"
  "vq16bmC*Wiq32m\0"
  "svst2q_vnum_s16\0"
  "vq16bSsC*Wiq16s\0"
  "svst2q_vnum_s32\0"
  "vq16bSiC*Wiq8i\0"
  "svst2q_vnum_s64\0"
  "vq16bSWiC*Wiq4Wi\0"
  "svst2q_vnum_s8\0"
  "vq16bScC*Wiq32Sc\0"
  "svst2q_vnum_u16\0"
  "vq16bUsC*Wiq16Us\0"
  "svst2q_vnum_u32\0"
  "vq16bUiC*Wiq8Ui\0"
  "svst2q_vnum_u64\0"
  "vq16bUWiC*Wiq4UWi\0"
  "svst2q_vnum_u8\0"
  "vq16bUcC*Wiq32Uc\0"
  "svst3_bf16\0"
  "vq16by*q24y\0"
  "svst3_f16\0"
  "vq16bh*q24h\0"
  "svst3_f32\0"
  "vq16bf*q12f\0"
  "svst3_f64\0"
  "vq16bd*q6d\0"
  "svst3_mf8\0"
  "vq16bm*q48m\0"
  "svst3_s16\0"
  "vq16bSs*q24s\0"
  "svst3_s32\0"
  "vq16bSi*q12i\0"
  "svst3_s64\0"
  "vq16bSWi*q6Wi\0"
  "svst3_s8\0"
  "vq16bSc*q48Sc\0"
  "svst3_u16\0"
  "vq16bUs*q24Us\0"
  "svst3_u32\0"
  "vq16bUi*q12Ui\0"
  "svst3_u64\0"
  "vq16bUWi*q6UWi\0"
  "svst3_u8\0"
  "vq16bUc*q48Uc\0"
  "svst3_vnum_bf16\0"
  "vq16by*Wiq24y\0"
  "svst3_vnum_f16\0"
  "vq16bh*Wiq24h\0"
  "svst3_vnum_f32\0"
  "vq16bf*Wiq12f\0"
  "svst3_vnum_f64\0"
  "vq16bd*Wiq6d\0"
  "svst3_vnum_mf8\0"
  "vq16bm*Wiq48m\0"
  "svst3_vnum_s16\0"
  "vq16bSs*Wiq24s\0"
  "svst3_vnum_s32\0"
  "vq16bSi*Wiq12i\0"
  "svst3_vnum_s64\0"
  "vq16bSWi*Wiq6Wi\0"
  "svst3_vnum_s8\0"
  "vq16bSc*Wiq48Sc\0"
  "svst3_vnum_u16\0"
  "vq16bUs*Wiq24Us\0"
  "svst3_vnum_u32\0"
  "vq16bUi*Wiq12Ui\0"
  "svst3_vnum_u64\0"
  "vq16bUWi*Wiq6UWi\0"
  "svst3_vnum_u8\0"
  "vq16bUc*Wiq48Uc\0"
  "svst3q_bf16\0"
  "vq16byC*q24y\0"
  "svst3q_f16\0"
  "vq16bhC*q24h\0"
  "svst3q_f32\0"
  "vq16bfC*q12f\0"
  "svst3q_f64\0"
  "vq16bdC*q6d\0"
  "svst3q_mf8\0"
  "vq16bmC*q48m\0"
  "svst3q_s16\0"
  "vq16bSsC*q24s\0"
  "svst3q_s32\0"
  "vq16bSiC*q12i\0"
  "svst3q_s64\0"
  "vq16bSWiC*q6Wi\0"
  "svst3q_s8\0"
  "vq16bScC*q48Sc\0"
  "svst3q_u16\0"
  "vq16bUsC*q24Us\0"
  "svst3q_u32\0"
  "vq16bUiC*q12Ui\0"
  "svst3q_u64\0"
  "vq16bUWiC*q6UWi\0"
  "svst3q_u8\0"
  "vq16bUcC*q48Uc\0"
  "svst3q_vnum_bf16\0"
  "vq16byC*Wiq24y\0"
  "svst3q_vnum_f16\0"
  "vq16bhC*Wiq24h\0"
  "svst3q_vnum_f32\0"
  "vq16bfC*Wiq12f\0"
  "svst3q_vnum_f64\0"
  "vq16bdC*Wiq6d\0"
  "svst3q_vnum_mf8\0"
  "vq16bmC*Wiq48m\0"
  "svst3q_vnum_s16\0"
  "vq16bSsC*Wiq24s\0"
  "svst3q_vnum_s32\0"
  "vq16bSiC*Wiq12i\0"
  "svst3q_vnum_s64\0"
  "vq16bSWiC*Wiq6Wi\0"
  "svst3q_vnum_s8\0"
  "vq16bScC*Wiq48Sc\0"
  "svst3q_vnum_u16\0"
  "vq16bUsC*Wiq24Us\0"
  "svst3q_vnum_u32\0"
  "vq16bUiC*Wiq12Ui\0"
  "svst3q_vnum_u64\0"
  "vq16bUWiC*Wiq6UWi\0"
  "svst3q_vnum_u8\0"
  "vq16bUcC*Wiq48Uc\0"
  "svst4_bf16\0"
  "vq16by*q32y\0"
  "svst4_f16\0"
  "vq16bh*q32h\0"
  "svst4_f32\0"
  "vq16bf*q16f\0"
  "svst4_f64\0"
  "vq16bd*q8d\0"
  "svst4_mf8\0"
  "vq16bm*q64m\0"
  "svst4_s16\0"
  "vq16bSs*q32s\0"
  "svst4_s32\0"
  "vq16bSi*q16i\0"
  "svst4_s64\0"
  "vq16bSWi*q8Wi\0"
  "svst4_s8\0"
  "vq16bSc*q64Sc\0"
  "svst4_u16\0"
  "vq16bUs*q32Us\0"
  "svst4_u32\0"
  "vq16bUi*q16Ui\0"
  "svst4_u64\0"
  "vq16bUWi*q8UWi\0"
  "svst4_u8\0"
  "vq16bUc*q64Uc\0"
  "svst4_vnum_bf16\0"
  "vq16by*Wiq32y\0"
  "svst4_vnum_f16\0"
  "vq16bh*Wiq32h\0"
  "svst4_vnum_f32\0"
  "vq16bf*Wiq16f\0"
  "svst4_vnum_f64\0"
  "vq16bd*Wiq8d\0"
  "svst4_vnum_mf8\0"
  "vq16bm*Wiq64m\0"
  "svst4_vnum_s16\0"
  "vq16bSs*Wiq32s\0"
  "svst4_vnum_s32\0"
  "vq16bSi*Wiq16i\0"
  "svst4_vnum_s64\0"
  "vq16bSWi*Wiq8Wi\0"
  "svst4_vnum_s8\0"
  "vq16bSc*Wiq64Sc\0"
  "svst4_vnum_u16\0"
  "vq16bUs*Wiq32Us\0"
  "svst4_vnum_u32\0"
  "vq16bUi*Wiq16Ui\0"
  "svst4_vnum_u64\0"
  "vq16bUWi*Wiq8UWi\0"
  "svst4_vnum_u8\0"
  "vq16bUc*Wiq64Uc\0"
  "svst4q_bf16\0"
  "vq16byC*q32y\0"
  "svst4q_f16\0"
  "vq16bhC*q32h\0"
  "svst4q_f32\0"
  "vq16bfC*q16f\0"
  "svst4q_f64\0"
  "vq16bdC*q8d\0"
  "svst4q_mf8\0"
  "vq16bmC*q64m\0"
  "svst4q_s16\0"
  "vq16bSsC*q32s\0"
  "svst4q_s32\0"
  "vq16bSiC*q16i\0"
  "svst4q_s64\0"
  "vq16bSWiC*q8Wi\0"
  "svst4q_s8\0"
  "vq16bScC*q64Sc\0"
  "svst4q_u16\0"
  "vq16bUsC*q32Us\0"
  "svst4q_u32\0"
  "vq16bUiC*q16Ui\0"
  "svst4q_u64\0"
  "vq16bUWiC*q8UWi\0"
  "svst4q_u8\0"
  "vq16bUcC*q64Uc\0"
  "svst4q_vnum_bf16\0"
  "vq16byC*Wiq32y\0"
  "svst4q_vnum_f16\0"
  "vq16bhC*Wiq32h\0"
  "svst4q_vnum_f32\0"
  "vq16bfC*Wiq16f\0"
  "svst4q_vnum_f64\0"
  "vq16bdC*Wiq8d\0"
  "svst4q_vnum_mf8\0"
  "vq16bmC*Wiq64m\0"
  "svst4q_vnum_s16\0"
  "vq16bSsC*Wiq32s\0"
  "svst4q_vnum_s32\0"
  "vq16bSiC*Wiq16i\0"
  "svst4q_vnum_s64\0"
  "vq16bSWiC*Wiq8Wi\0"
  "svst4q_vnum_s8\0"
  "vq16bScC*Wiq64Sc\0"
  "svst4q_vnum_u16\0"
  "vq16bUsC*Wiq32Us\0"
  "svst4q_vnum_u32\0"
  "vq16bUiC*Wiq16Ui\0"
  "svst4q_vnum_u64\0"
  "vq16bUWiC*Wiq8UWi\0"
  "svst4q_vnum_u8\0"
  "vq16bUcC*Wiq64Uc\0"
  "svstnt1_bf16\0"
  "svstnt1_bf16_x2\0"
  "svstnt1_bf16_x4\0"
  "svstnt1_f16\0"
  "svstnt1_f16_x2\0"
  "svstnt1_f16_x4\0"
  "svstnt1_f32\0"
  "svstnt1_f32_x2\0"
  "svstnt1_f32_x4\0"
  "svstnt1_f64\0"
  "svstnt1_f64_x2\0"
  "svstnt1_f64_x4\0"
  "svstnt1_mf8\0"
  "svstnt1_mf8_x2\0"
  "svstnt1_mf8_x4\0"
  "svstnt1_s16\0"
  "svstnt1_s16_x2\0"
  "svstnt1_s16_x4\0"
  "svstnt1_s32\0"
  "svstnt1_s32_x2\0"
  "svstnt1_s32_x4\0"
  "svstnt1_s64\0"
  "svstnt1_s64_x2\0"
  "svstnt1_s64_x4\0"
  "svstnt1_s8\0"
  "svstnt1_s8_x2\0"
  "svstnt1_s8_x4\0"
  "svstnt1_scatter_s64index_f64\0"
  "svstnt1_scatter_s64index_s64\0"
  "svstnt1_scatter_s64index_u64\0"
  "svstnt1_scatter_s64offset_f64\0"
  "svstnt1_scatter_s64offset_s64\0"
  "svstnt1_scatter_s64offset_u64\0"
  "svstnt1_scatter_u32base_f32\0"
  "svstnt1_scatter_u32base_index_f32\0"
  "svstnt1_scatter_u32base_index_s32\0"
  "svstnt1_scatter_u32base_index_u32\0"
  "svstnt1_scatter_u32base_offset_f32\0"
  "svstnt1_scatter_u32base_offset_s32\0"
  "svstnt1_scatter_u32base_offset_u32\0"
  "svstnt1_scatter_u32base_s32\0"
  "svstnt1_scatter_u32base_u32\0"
  "svstnt1_scatter_u32offset_f32\0"
  "svstnt1_scatter_u32offset_s32\0"
  "svstnt1_scatter_u32offset_u32\0"
  "svstnt1_scatter_u64base_f64\0"
  "svstnt1_scatter_u64base_index_f64\0"
  "svstnt1_scatter_u64base_index_s64\0"
  "svstnt1_scatter_u64base_index_u64\0"
  "svstnt1_scatter_u64base_offset_f64\0"
  "svstnt1_scatter_u64base_offset_s64\0"
  "svstnt1_scatter_u64base_offset_u64\0"
  "svstnt1_scatter_u64base_s64\0"
  "svstnt1_scatter_u64base_u64\0"
  "svstnt1_scatter_u64index_f64\0"
  "svstnt1_scatter_u64index_s64\0"
  "svstnt1_scatter_u64index_u64\0"
  "svstnt1_scatter_u64offset_f64\0"
  "svstnt1_scatter_u64offset_s64\0"
  "svstnt1_scatter_u64offset_u64\0"
  "svstnt1_u16\0"
  "svstnt1_u16_x2\0"
  "svstnt1_u16_x4\0"
  "svstnt1_u32\0"
  "svstnt1_u32_x2\0"
  "svstnt1_u32_x4\0"
  "svstnt1_u64\0"
  "svstnt1_u64_x2\0"
  "svstnt1_u64_x4\0"
  "svstnt1_u8\0"
  "svstnt1_u8_x2\0"
  "svstnt1_u8_x4\0"
  "svstnt1_vnum_bf16\0"
  "svstnt1_vnum_bf16_x2\0"
  "svstnt1_vnum_bf16_x4\0"
  "svstnt1_vnum_f16\0"
  "svstnt1_vnum_f16_x2\0"
  "svstnt1_vnum_f16_x4\0"
  "svstnt1_vnum_f32\0"
  "svstnt1_vnum_f32_x2\0"
  "svstnt1_vnum_f32_x4\0"
  "svstnt1_vnum_f64\0"
  "svstnt1_vnum_f64_x2\0"
  "svstnt1_vnum_f64_x4\0"
  "svstnt1_vnum_mf8\0"
  "svstnt1_vnum_mf8_x2\0"
  "svstnt1_vnum_mf8_x4\0"
  "svstnt1_vnum_s16\0"
  "svstnt1_vnum_s16_x2\0"
  "svstnt1_vnum_s16_x4\0"
  "svstnt1_vnum_s32\0"
  "svstnt1_vnum_s32_x2\0"
  "svstnt1_vnum_s32_x4\0"
  "svstnt1_vnum_s64\0"
  "svstnt1_vnum_s64_x2\0"
  "svstnt1_vnum_s64_x4\0"
  "svstnt1_vnum_s8\0"
  "svstnt1_vnum_s8_x2\0"
  "svstnt1_vnum_s8_x4\0"
  "svstnt1_vnum_u16\0"
  "svstnt1_vnum_u16_x2\0"
  "svstnt1_vnum_u16_x4\0"
  "svstnt1_vnum_u32\0"
  "svstnt1_vnum_u32_x2\0"
  "svstnt1_vnum_u32_x4\0"
  "svstnt1_vnum_u64\0"
  "svstnt1_vnum_u64_x2\0"
  "svstnt1_vnum_u64_x4\0"
  "svstnt1_vnum_u8\0"
  "svstnt1_vnum_u8_x2\0"
  "svstnt1_vnum_u8_x4\0"
  "svstnt1b_scatter_s64offset_s64\0"
  "svstnt1b_scatter_s64offset_u64\0"
  "svstnt1b_scatter_u32base_offset_s32\0"
  "svstnt1b_scatter_u32base_offset_u32\0"
  "svstnt1b_scatter_u32base_s32\0"
  "svstnt1b_scatter_u32base_u32\0"
  "svstnt1b_scatter_u32offset_s32\0"
  "svstnt1b_scatter_u32offset_u32\0"
  "svstnt1b_scatter_u64base_offset_s64\0"
  "svstnt1b_scatter_u64base_offset_u64\0"
  "svstnt1b_scatter_u64base_s64\0"
  "svstnt1b_scatter_u64base_u64\0"
  "svstnt1b_scatter_u64offset_s64\0"
  "svstnt1b_scatter_u64offset_u64\0"
  "svstnt1h_scatter_s64index_s64\0"
  "svstnt1h_scatter_s64index_u64\0"
  "svstnt1h_scatter_s64offset_s64\0"
  "svstnt1h_scatter_s64offset_u64\0"
  "svstnt1h_scatter_u32base_index_s32\0"
  "svstnt1h_scatter_u32base_index_u32\0"
  "svstnt1h_scatter_u32base_offset_s32\0"
  "svstnt1h_scatter_u32base_offset_u32\0"
  "svstnt1h_scatter_u32base_s32\0"
  "svstnt1h_scatter_u32base_u32\0"
  "svstnt1h_scatter_u32offset_s32\0"
  "svstnt1h_scatter_u32offset_u32\0"
  "svstnt1h_scatter_u64base_index_s64\0"
  "svstnt1h_scatter_u64base_index_u64\0"
  "svstnt1h_scatter_u64base_offset_s64\0"
  "svstnt1h_scatter_u64base_offset_u64\0"
  "svstnt1h_scatter_u64base_s64\0"
  "svstnt1h_scatter_u64base_u64\0"
  "svstnt1h_scatter_u64index_s64\0"
  "svstnt1h_scatter_u64index_u64\0"
  "svstnt1h_scatter_u64offset_s64\0"
  "svstnt1h_scatter_u64offset_u64\0"
  "svstnt1w_scatter_s64index_s64\0"
  "svstnt1w_scatter_s64index_u64\0"
  "svstnt1w_scatter_s64offset_s64\0"
  "svstnt1w_scatter_s64offset_u64\0"
  "svstnt1w_scatter_u64base_index_s64\0"
  "svstnt1w_scatter_u64base_index_u64\0"
  "svstnt1w_scatter_u64base_offset_s64\0"
  "svstnt1w_scatter_u64base_offset_u64\0"
  "svstnt1w_scatter_u64base_s64\0"
  "svstnt1w_scatter_u64base_u64\0"
  "svstnt1w_scatter_u64index_s64\0"
  "svstnt1w_scatter_u64index_u64\0"
  "svstnt1w_scatter_u64offset_s64\0"
  "svstnt1w_scatter_u64offset_u64\0"
  "svsub_bf16_m\0"
  "svsub_bf16_x\0"
  "svsub_bf16_z\0"
  "svsub_f16_m\0"
  "svsub_f16_x\0"
  "svsub_f16_z\0"
  "svsub_f32_m\0"
  "svsub_f32_x\0"
  "svsub_f32_z\0"
  "svsub_f64_m\0"
  "svsub_f64_x\0"
  "svsub_f64_z\0"
  "svsub_n_bf16_m\0"
  "svsub_n_bf16_x\0"
  "svsub_n_bf16_z\0"
  "svsub_n_f16_m\0"
  "svsub_n_f16_x\0"
  "svsub_n_f16_z\0"
  "svsub_n_f32_m\0"
  "svsub_n_f32_x\0"
  "svsub_n_f32_z\0"
  "svsub_n_f64_m\0"
  "svsub_n_f64_x\0"
  "svsub_n_f64_z\0"
  "svsub_n_s16_m\0"
  "svsub_n_s16_x\0"
  "svsub_n_s16_z\0"
  "svsub_n_s32_m\0"
  "svsub_n_s32_x\0"
  "svsub_n_s32_z\0"
  "svsub_n_s64_m\0"
  "svsub_n_s64_x\0"
  "svsub_n_s64_z\0"
  "svsub_n_s8_m\0"
  "svsub_n_s8_x\0"
  "svsub_n_s8_z\0"
  "svsub_n_u16_m\0"
  "svsub_n_u16_x\0"
  "svsub_n_u16_z\0"
  "svsub_n_u32_m\0"
  "svsub_n_u32_x\0"
  "svsub_n_u32_z\0"
  "svsub_n_u64_m\0"
  "svsub_n_u64_x\0"
  "svsub_n_u64_z\0"
  "svsub_n_u8_m\0"
  "svsub_n_u8_x\0"
  "svsub_n_u8_z\0"
  "svsub_s16_m\0"
  "svsub_s16_x\0"
  "svsub_s16_z\0"
  "svsub_s32_m\0"
  "svsub_s32_x\0"
  "svsub_s32_z\0"
  "svsub_s64_m\0"
  "svsub_s64_x\0"
  "svsub_s64_z\0"
  "svsub_s8_m\0"
  "svsub_s8_x\0"
  "svsub_s8_z\0"
  "svsub_u16_m\0"
  "svsub_u16_x\0"
  "svsub_u16_z\0"
  "svsub_u32_m\0"
  "svsub_u32_x\0"
  "svsub_u32_z\0"
  "svsub_u64_m\0"
  "svsub_u64_x\0"
  "svsub_u64_z\0"
  "svsub_u8_m\0"
  "svsub_u8_x\0"
  "svsub_u8_z\0"
  "svsubhnb_n_s16\0"
  "svsubhnb_n_s32\0"
  "svsubhnb_n_s64\0"
  "svsubhnb_n_u16\0"
  "svsubhnb_n_u32\0"
  "svsubhnb_n_u64\0"
  "svsubhnb_s16\0"
  "svsubhnb_s32\0"
  "svsubhnb_s64\0"
  "svsubhnb_u16\0"
  "svsubhnb_u32\0"
  "svsubhnb_u64\0"
  "svsubhnt_n_s16\0"
  "svsubhnt_n_s32\0"
  "svsubhnt_n_s64\0"
  "svsubhnt_n_u16\0"
  "svsubhnt_n_u32\0"
  "svsubhnt_n_u64\0"
  "svsubhnt_s16\0"
  "svsubhnt_s32\0"
  "svsubhnt_s64\0"
  "svsubhnt_u16\0"
  "svsubhnt_u32\0"
  "svsubhnt_u64\0"
  "svsublb_n_s16\0"
  "svsublb_n_s32\0"
  "svsublb_n_s64\0"
  "svsublb_n_u16\0"
  "svsublb_n_u32\0"
  "svsublb_n_u64\0"
  "svsublb_s16\0"
  "svsublb_s32\0"
  "svsublb_s64\0"
  "svsublb_u16\0"
  "svsublb_u32\0"
  "svsublb_u64\0"
  "svsublbt_n_s16\0"
  "svsublbt_n_s32\0"
  "svsublbt_n_s64\0"
  "svsublbt_s16\0"
  "svsublbt_s32\0"
  "svsublbt_s64\0"
  "svsublt_n_s16\0"
  "svsublt_n_s32\0"
  "svsublt_n_s64\0"
  "svsublt_n_u16\0"
  "svsublt_n_u32\0"
  "svsublt_n_u64\0"
  "svsublt_s16\0"
  "svsublt_s32\0"
  "svsublt_s64\0"
  "svsublt_u16\0"
  "svsublt_u32\0"
  "svsublt_u64\0"
  "svsubltb_n_s16\0"
  "svsubltb_n_s32\0"
  "svsubltb_n_s64\0"
  "svsubltb_s16\0"
  "svsubltb_s32\0"
  "svsubltb_s64\0"
  "svsubr_f16_m\0"
  "svsubr_f16_x\0"
  "svsubr_f16_z\0"
  "svsubr_f32_m\0"
  "svsubr_f32_x\0"
  "svsubr_f32_z\0"
  "svsubr_f64_m\0"
  "svsubr_f64_x\0"
  "svsubr_f64_z\0"
  "svsubr_n_f16_m\0"
  "svsubr_n_f16_x\0"
  "svsubr_n_f16_z\0"
  "svsubr_n_f32_m\0"
  "svsubr_n_f32_x\0"
  "svsubr_n_f32_z\0"
  "svsubr_n_f64_m\0"
  "svsubr_n_f64_x\0"
  "svsubr_n_f64_z\0"
  "svsubr_n_s16_m\0"
  "svsubr_n_s16_x\0"
  "svsubr_n_s16_z\0"
  "svsubr_n_s32_m\0"
  "svsubr_n_s32_x\0"
  "svsubr_n_s32_z\0"
  "svsubr_n_s64_m\0"
  "svsubr_n_s64_x\0"
  "svsubr_n_s64_z\0"
  "svsubr_n_s8_m\0"
  "svsubr_n_s8_x\0"
  "svsubr_n_s8_z\0"
  "svsubr_n_u16_m\0"
  "svsubr_n_u16_x\0"
  "svsubr_n_u16_z\0"
  "svsubr_n_u32_m\0"
  "svsubr_n_u32_x\0"
  "svsubr_n_u32_z\0"
  "svsubr_n_u64_m\0"
  "svsubr_n_u64_x\0"
  "svsubr_n_u64_z\0"
  "svsubr_n_u8_m\0"
  "svsubr_n_u8_x\0"
  "svsubr_n_u8_z\0"
  "svsubr_s16_m\0"
  "svsubr_s16_x\0"
  "svsubr_s16_z\0"
  "svsubr_s32_m\0"
  "svsubr_s32_x\0"
  "svsubr_s32_z\0"
  "svsubr_s64_m\0"
  "svsubr_s64_x\0"
  "svsubr_s64_z\0"
  "svsubr_s8_m\0"
  "svsubr_s8_x\0"
  "svsubr_s8_z\0"
  "svsubr_u16_m\0"
  "svsubr_u16_x\0"
  "svsubr_u16_z\0"
  "svsubr_u32_m\0"
  "svsubr_u32_x\0"
  "svsubr_u32_z\0"
  "svsubr_u64_m\0"
  "svsubr_u64_x\0"
  "svsubr_u64_z\0"
  "svsubr_u8_m\0"
  "svsubr_u8_x\0"
  "svsubr_u8_z\0"
  "svsubwb_n_s16\0"
  "svsubwb_n_s32\0"
  "svsubwb_n_s64\0"
  "svsubwb_n_u16\0"
  "svsubwb_n_u32\0"
  "svsubwb_n_u64\0"
  "svsubwb_s16\0"
  "svsubwb_s32\0"
  "svsubwb_s64\0"
  "svsubwb_u16\0"
  "svsubwb_u32\0"
  "svsubwb_u64\0"
  "svsubwt_n_s16\0"
  "svsubwt_n_s32\0"
  "svsubwt_n_s64\0"
  "svsubwt_n_u16\0"
  "svsubwt_n_u32\0"
  "svsubwt_n_u64\0"
  "svsubwt_s16\0"
  "svsubwt_s32\0"
  "svsubwt_s64\0"
  "svsubwt_u16\0"
  "svsubwt_u32\0"
  "svsubwt_u64\0"
  "svsudot_lane_s32\0"
  "q4iq4iq16Scq16UcIUWi\0"
  "(sve,i8mm)|(sme,i8mm)\0"
  "svsudot_n_s32\0"
  "q4iq4iq16ScUc\0"
  "svsudot_s32\0"
  "q4iq4iq16Scq16Uc\0"
  "svtbl2_bf16\0"
  "q8yq16yq8Us\0"
  "svtbl2_f16\0"
  "q8hq16hq8Us\0"
  "svtbl2_f32\0"
  "q4fq8fq4Ui\0"
  "svtbl2_f64\0"
  "q2dq4dq2UWi\0"
  "svtbl2_s16\0"
  "q8sq16sq8Us\0"
  "svtbl2_s32\0"
  "q4iq8iq4Ui\0"
  "svtbl2_s64\0"
  "q2Wiq4Wiq2UWi\0"
  "svtbl2_s8\0"
  "q16Scq32Scq16Uc\0"
  "svtbl2_u16\0"
  "q8Usq16Usq8Us\0"
  "svtbl2_u32\0"
  "q4Uiq8Uiq4Ui\0"
  "svtbl2_u64\0"
  "q2UWiq4UWiq2UWi\0"
  "svtbl2_u8\0"
  "q16Ucq32Ucq16Uc\0"
  "svtbl_bf16\0"
  "q8yq8yq8Us\0"
  "svtbl_f16\0"
  "q8hq8hq8Us\0"
  "svtbl_f32\0"
  "q4fq4fq4Ui\0"
  "svtbl_f64\0"
  "q2dq2dq2UWi\0"
  "svtbl_s16\0"
  "q8sq8sq8Us\0"
  "svtbl_s32\0"
  "q4iq4iq4Ui\0"
  "svtbl_s64\0"
  "q2Wiq2Wiq2UWi\0"
  "svtbl_s8\0"
  "q16Scq16Scq16Uc\0"
  "svtbl_u16\0"
  "svtbl_u32\0"
  "svtbl_u64\0"
  "svtbl_u8\0"
  "svtblq_bf16\0"
  "svtblq_f16\0"
  "svtblq_f32\0"
  "svtblq_f64\0"
  "svtblq_mf8\0"
  "q16mq16mq16Uc\0"
  "svtblq_s16\0"
  "svtblq_s32\0"
  "svtblq_s64\0"
  "svtblq_s8\0"
  "svtblq_u16\0"
  "svtblq_u32\0"
  "svtblq_u64\0"
  "svtblq_u8\0"
  "svtbx_bf16\0"
  "q8yq8yq8yq8Us\0"
  "svtbx_f16\0"
  "q8hq8hq8hq8Us\0"
  "svtbx_f32\0"
  "q4fq4fq4fq4Ui\0"
  "svtbx_f64\0"
  "q2dq2dq2dq2UWi\0"
  "svtbx_s16\0"
  "q8sq8sq8sq8Us\0"
  "svtbx_s32\0"
  "q4iq4iq4iq4Ui\0"
  "svtbx_s64\0"
  "q2Wiq2Wiq2Wiq2UWi\0"
  "svtbx_s8\0"
  "q16Scq16Scq16Scq16Uc\0"
  "svtbx_u16\0"
  "svtbx_u32\0"
  "svtbx_u64\0"
  "svtbx_u8\0"
  "svtbxq_bf16\0"
  "svtbxq_f16\0"
  "svtbxq_f32\0"
  "svtbxq_f64\0"
  "svtbxq_mf8\0"
  "q16mq16mq16mq16Uc\0"
  "svtbxq_s16\0"
  "svtbxq_s32\0"
  "svtbxq_s64\0"
  "svtbxq_s8\0"
  "svtbxq_u16\0"
  "svtbxq_u32\0"
  "svtbxq_u64\0"
  "svtbxq_u8\0"
  "svtmad_f16\0"
  "svtmad_f32\0"
  "svtmad_f64\0"
  "svtrn1_b16\0"
  "svtrn1_b32\0"
  "svtrn1_b64\0"
  "svtrn1_b8\0"
  "svtrn1_bf16\0"
  "q8yq8yq8y\0"
  "svtrn1_f16\0"
  "svtrn1_f32\0"
  "svtrn1_f64\0"
  "svtrn1_s16\0"
  "svtrn1_s32\0"
  "svtrn1_s64\0"
  "svtrn1_s8\0"
  "svtrn1_u16\0"
  "svtrn1_u32\0"
  "svtrn1_u64\0"
  "svtrn1_u8\0"
  "svtrn1q_bf16\0"
  "svtrn1q_f16\0"
  "svtrn1q_f32\0"
  "svtrn1q_f64\0"
  "svtrn1q_s16\0"
  "svtrn1q_s32\0"
  "svtrn1q_s64\0"
  "svtrn1q_s8\0"
  "svtrn1q_u16\0"
  "svtrn1q_u32\0"
  "svtrn1q_u64\0"
  "svtrn1q_u8\0"
  "svtrn2_b16\0"
  "svtrn2_b32\0"
  "svtrn2_b64\0"
  "svtrn2_b8\0"
  "svtrn2_bf16\0"
  "svtrn2_f16\0"
  "svtrn2_f32\0"
  "svtrn2_f64\0"
  "svtrn2_s16\0"
  "svtrn2_s32\0"
  "svtrn2_s64\0"
  "svtrn2_s8\0"
  "svtrn2_u16\0"
  "svtrn2_u32\0"
  "svtrn2_u64\0"
  "svtrn2_u8\0"
  "svtrn2q_bf16\0"
  "svtrn2q_f16\0"
  "svtrn2q_f32\0"
  "svtrn2q_f64\0"
  "svtrn2q_s16\0"
  "svtrn2q_s32\0"
  "svtrn2q_s64\0"
  "svtrn2q_s8\0"
  "svtrn2q_u16\0"
  "svtrn2q_u32\0"
  "svtrn2q_u64\0"
  "svtrn2q_u8\0"
  "svtsmul_f16\0"
  "svtsmul_f32\0"
  "svtsmul_f64\0"
  "svtssel_f16\0"
  "svtssel_f32\0"
  "svtssel_f64\0"
  "svundef2_b\0"
  "q32b\0"
  "svundef2_bf16\0"
  "q16yv\0"
  "svundef2_f16\0"
  "q16hv\0"
  "svundef2_f32\0"
  "q8fv\0"
  "svundef2_f64\0"
  "q4dv\0"
  "svundef2_mf8\0"
  "q32mv\0"
  "svundef2_s16\0"
  "q16sv\0"
  "svundef2_s32\0"
  "q8iv\0"
  "svundef2_s64\0"
  "q4Wiv\0"
  "svundef2_s8\0"
  "q32Scv\0"
  "svundef2_u16\0"
  "q16Usv\0"
  "svundef2_u32\0"
  "q8Uiv\0"
  "svundef2_u64\0"
  "q4UWiv\0"
  "svundef2_u8\0"
  "q32Ucv\0"
  "svundef3_bf16\0"
  "q24yv\0"
  "svundef3_f16\0"
  "q24hv\0"
  "svundef3_f32\0"
  "q12fv\0"
  "svundef3_f64\0"
  "q6dv\0"
  "svundef3_mf8\0"
  "q48mv\0"
  "svundef3_s16\0"
  "q24sv\0"
  "svundef3_s32\0"
  "q12iv\0"
  "svundef3_s64\0"
  "q6Wiv\0"
  "svundef3_s8\0"
  "q48Scv\0"
  "svundef3_u16\0"
  "q24Usv\0"
  "svundef3_u32\0"
  "q12Uiv\0"
  "svundef3_u64\0"
  "q6UWiv\0"
  "svundef3_u8\0"
  "q48Ucv\0"
  "svundef4_b\0"
  "q64b\0"
  "svundef4_bf16\0"
  "q32yv\0"
  "svundef4_f16\0"
  "q32hv\0"
  "svundef4_f32\0"
  "q16fv\0"
  "svundef4_f64\0"
  "q8dv\0"
  "svundef4_mf8\0"
  "q64mv\0"
  "svundef4_s16\0"
  "q32sv\0"
  "svundef4_s32\0"
  "q16iv\0"
  "svundef4_s64\0"
  "q8Wiv\0"
  "svundef4_s8\0"
  "q64Scv\0"
  "svundef4_u16\0"
  "q32Usv\0"
  "svundef4_u32\0"
  "q16Uiv\0"
  "svundef4_u64\0"
  "q8UWiv\0"
  "svundef4_u8\0"
  "q64Ucv\0"
  "svundef_bf16\0"
  "q8yv\0"
  "svundef_f16\0"
  "q8hv\0"
  "svundef_f32\0"
  "q4fv\0"
  "svundef_f64\0"
  "q2dv\0"
  "svundef_mf8\0"
  "q16mv\0"
  "svundef_s16\0"
  "q8sv\0"
  "svundef_s32\0"
  "q4iv\0"
  "svundef_s64\0"
  "q2Wiv\0"
  "svundef_s8\0"
  "q16Scv\0"
  "svundef_u16\0"
  "q8Usv\0"
  "svundef_u32\0"
  "q4Uiv\0"
  "svundef_u64\0"
  "q2UWiv\0"
  "svundef_u8\0"
  "q16Ucv\0"
  "svunpk_s16_s8_x2\0"
  "q16sq16Sc\0"
  "svunpk_s16_s8_x4\0"
  "q32sq32Sc\0"
  "svunpk_s32_s16_x2\0"
  "q8iq8s\0"
  "svunpk_s32_s16_x4\0"
  "q16iq16s\0"
  "svunpk_s64_s32_x2\0"
  "q4Wiq4i\0"
  "svunpk_s64_s32_x4\0"
  "q8Wiq8i\0"
  "svunpk_u16_u8_x2\0"
  "q16Usq16Uc\0"
  "svunpk_u16_u8_x4\0"
  "q32Usq32Uc\0"
  "svunpk_u32_u16_x2\0"
  "q8Uiq8Us\0"
  "svunpk_u32_u16_x4\0"
  "q16Uiq16Us\0"
  "svunpk_u64_u32_x2\0"
  "q4UWiq4Ui\0"
  "svunpk_u64_u32_x4\0"
  "q8UWiq8Ui\0"
  "svunpkhi_b\0"
  "svunpkhi_s16\0"
  "svunpkhi_s32\0"
  "svunpkhi_s64\0"
  "svunpkhi_u16\0"
  "svunpkhi_u32\0"
  "svunpkhi_u64\0"
  "svunpklo_b\0"
  "svunpklo_s16\0"
  "svunpklo_s32\0"
  "svunpklo_s64\0"
  "svunpklo_u16\0"
  "svunpklo_u32\0"
  "svunpklo_u64\0"
  "svuqadd_n_s16_m\0"
  "svuqadd_n_s16_x\0"
  "svuqadd_n_s16_z\0"
  "svuqadd_n_s32_m\0"
  "svuqadd_n_s32_x\0"
  "svuqadd_n_s32_z\0"
  "svuqadd_n_s64_m\0"
  "svuqadd_n_s64_x\0"
  "svuqadd_n_s64_z\0"
  "svuqadd_n_s8_m\0"
  "svuqadd_n_s8_x\0"
  "svuqadd_n_s8_z\0"
  "svuqadd_s16_m\0"
  "svuqadd_s16_x\0"
  "svuqadd_s16_z\0"
  "svuqadd_s32_m\0"
  "svuqadd_s32_x\0"
  "svuqadd_s32_z\0"
  "svuqadd_s64_m\0"
  "svuqadd_s64_x\0"
  "svuqadd_s64_z\0"
  "svuqadd_s8_m\0"
  "svuqadd_s8_x\0"
  "svuqadd_s8_z\0"
  "svusdot_lane_s32\0"
  "q4iq4iq16Ucq16ScIUWi\0"
  "svusdot_n_s32\0"
  "q4iq4iq16UcSc\0"
  "svusdot_s32\0"
  "q4iq4iq16Ucq16Sc\0"
  "svusmmla_s32\0"
  "svuzp1_b16\0"
  "svuzp1_b32\0"
  "svuzp1_b64\0"
  "svuzp1_b8\0"
  "svuzp1_bf16\0"
  "svuzp1_f16\0"
  "svuzp1_f32\0"
  "svuzp1_f64\0"
  "svuzp1_s16\0"
  "svuzp1_s32\0"
  "svuzp1_s64\0"
  "svuzp1_s8\0"
  "svuzp1_u16\0"
  "svuzp1_u32\0"
  "svuzp1_u64\0"
  "svuzp1_u8\0"
  "svuzp1q_bf16\0"
  "svuzp1q_f16\0"
  "svuzp1q_f32\0"
  "svuzp1q_f64\0"
  "svuzp1q_s16\0"
  "svuzp1q_s32\0"
  "svuzp1q_s64\0"
  "svuzp1q_s8\0"
  "svuzp1q_u16\0"
  "svuzp1q_u32\0"
  "svuzp1q_u64\0"
  "svuzp1q_u8\0"
  "svuzp2_b16\0"
  "svuzp2_b32\0"
  "svuzp2_b64\0"
  "svuzp2_b8\0"
  "svuzp2_bf16\0"
  "svuzp2_f16\0"
  "svuzp2_f32\0"
  "svuzp2_f64\0"
  "svuzp2_s16\0"
  "svuzp2_s32\0"
  "svuzp2_s64\0"
  "svuzp2_s8\0"
  "svuzp2_u16\0"
  "svuzp2_u32\0"
  "svuzp2_u64\0"
  "svuzp2_u8\0"
  "svuzp2q_bf16\0"
  "svuzp2q_f16\0"
  "svuzp2q_f32\0"
  "svuzp2q_f64\0"
  "svuzp2q_s16\0"
  "svuzp2q_s32\0"
  "svuzp2q_s64\0"
  "svuzp2q_s8\0"
  "svuzp2q_u16\0"
  "svuzp2q_u32\0"
  "svuzp2q_u64\0"
  "svuzp2q_u8\0"
  "svuzp_bf16_x2\0"
  "q16yq16y\0"
  "svuzp_bf16_x4\0"
  "q32yq32y\0"
  "svuzp_f16_x2\0"
  "q16hq16h\0"
  "svuzp_f16_x4\0"
  "q32hq32h\0"
  "svuzp_f32_x2\0"
  "svuzp_f32_x4\0"
  "svuzp_f64_x2\0"
  "q4dq4d\0"
  "svuzp_f64_x4\0"
  "q8dq8d\0"
  "svuzp_mf8_x2\0"
  "q32mq32m\0"
  "svuzp_mf8_x4\0"
  "q64mq64m\0"
  "svuzp_s16_x2\0"
  "q16sq16s\0"
  "svuzp_s16_x4\0"
  "q32sq32s\0"
  "svuzp_s32_x2\0"
  "q8iq8i\0"
  "svuzp_s32_x4\0"
  "q16iq16i\0"
  "svuzp_s64_x2\0"
  "q4Wiq4Wi\0"
  "svuzp_s64_x4\0"
  "q8Wiq8Wi\0"
  "svuzp_s8_x2\0"
  "q32Scq32Sc\0"
  "svuzp_s8_x4\0"
  "q64Scq64Sc\0"
  "svuzp_u16_x2\0"
  "q16Usq16Us\0"
  "svuzp_u16_x4\0"
  "q32Usq32Us\0"
  "svuzp_u32_x2\0"
  "q8Uiq8Ui\0"
  "svuzp_u32_x4\0"
  "q16Uiq16Ui\0"
  "svuzp_u64_x2\0"
  "q4UWiq4UWi\0"
  "svuzp_u64_x4\0"
  "q8UWiq8UWi\0"
  "svuzp_u8_x2\0"
  "q32Ucq32Uc\0"
  "svuzp_u8_x4\0"
  "q64Ucq64Uc\0"
  "svuzpq1_bf16\0"
  "svuzpq1_f16\0"
  "svuzpq1_f32\0"
  "svuzpq1_f64\0"
  "svuzpq1_mf8\0"
  "q16mq16mq16m\0"
  "svuzpq1_s16\0"
  "svuzpq1_s32\0"
  "svuzpq1_s64\0"
  "svuzpq1_s8\0"
  "svuzpq1_u16\0"
  "svuzpq1_u32\0"
  "svuzpq1_u64\0"
  "svuzpq1_u8\0"
  "svuzpq2_bf16\0"
  "svuzpq2_f16\0"
  "svuzpq2_f32\0"
  "svuzpq2_f64\0"
  "svuzpq2_mf8\0"
  "svuzpq2_s16\0"
  "svuzpq2_s32\0"
  "svuzpq2_s64\0"
  "svuzpq2_s8\0"
  "svuzpq2_u16\0"
  "svuzpq2_u32\0"
  "svuzpq2_u64\0"
  "svuzpq2_u8\0"
  "svuzpq_bf16_x2\0"
  "svuzpq_bf16_x4\0"
  "svuzpq_f16_x2\0"
  "svuzpq_f16_x4\0"
  "svuzpq_f32_x2\0"
  "svuzpq_f32_x4\0"
  "svuzpq_f64_x2\0"
  "svuzpq_f64_x4\0"
  "svuzpq_mf8_x2\0"
  "svuzpq_mf8_x4\0"
  "svuzpq_s16_x2\0"
  "svuzpq_s16_x4\0"
  "svuzpq_s32_x2\0"
  "svuzpq_s32_x4\0"
  "svuzpq_s64_x2\0"
  "svuzpq_s64_x4\0"
  "svuzpq_s8_x2\0"
  "svuzpq_s8_x4\0"
  "svuzpq_u16_x2\0"
  "svuzpq_u16_x4\0"
  "svuzpq_u32_x2\0"
  "svuzpq_u32_x4\0"
  "svuzpq_u64_x2\0"
  "svuzpq_u64_x4\0"
  "svuzpq_u8_x2\0"
  "svuzpq_u8_x4\0"
  "svwhilege_b16_s32\0"
  "q16bii\0"
  "svwhilege_b16_s64\0"
  "q16bWiWi\0"
  "svwhilege_b16_s64_x2\0"
  "q32bWiWi\0"
  "svwhilege_b16_u32\0"
  "q16bUiUi\0"
  "svwhilege_b16_u64\0"
  "q16bUWiUWi\0"
  "svwhilege_b16_u64_x2\0"
  "q32bUWiUWi\0"
  "svwhilege_b32_s32\0"
  "svwhilege_b32_s64\0"
  "svwhilege_b32_s64_x2\0"
  "svwhilege_b32_u32\0"
  "svwhilege_b32_u64\0"
  "svwhilege_b32_u64_x2\0"
  "svwhilege_b64_s32\0"
  "svwhilege_b64_s64\0"
  "svwhilege_b64_s64_x2\0"
  "svwhilege_b64_u32\0"
  "svwhilege_b64_u64\0"
  "svwhilege_b64_u64_x2\0"
  "svwhilege_b8_s32\0"
  "svwhilege_b8_s64\0"
  "svwhilege_b8_s64_x2\0"
  "svwhilege_b8_u32\0"
  "svwhilege_b8_u64\0"
  "svwhilege_b8_u64_x2\0"
  "svwhilege_c16_s64\0"
  "QaWiWiIUWi\0"
  "svwhilege_c16_u64\0"
  "QaUWiUWiIUWi\0"
  "svwhilege_c32_s64\0"
  "svwhilege_c32_u64\0"
  "svwhilege_c64_s64\0"
  "svwhilege_c64_u64\0"
  "svwhilege_c8_s64\0"
  "svwhilege_c8_u64\0"
  "svwhilegt_b16_s32\0"
  "svwhilegt_b16_s64\0"
  "svwhilegt_b16_s64_x2\0"
  "svwhilegt_b16_u32\0"
  "svwhilegt_b16_u64\0"
  "svwhilegt_b16_u64_x2\0"
  "svwhilegt_b32_s32\0"
  "svwhilegt_b32_s64\0"
  "svwhilegt_b32_s64_x2\0"
  "svwhilegt_b32_u32\0"
  "svwhilegt_b32_u64\0"
  "svwhilegt_b32_u64_x2\0"
  "svwhilegt_b64_s32\0"
  "svwhilegt_b64_s64\0"
  "svwhilegt_b64_s64_x2\0"
  "svwhilegt_b64_u32\0"
  "svwhilegt_b64_u64\0"
  "svwhilegt_b64_u64_x2\0"
  "svwhilegt_b8_s32\0"
  "svwhilegt_b8_s64\0"
  "svwhilegt_b8_s64_x2\0"
  "svwhilegt_b8_u32\0"
  "svwhilegt_b8_u64\0"
  "svwhilegt_b8_u64_x2\0"
  "svwhilegt_c16_s64\0"
  "svwhilegt_c16_u64\0"
  "svwhilegt_c32_s64\0"
  "svwhilegt_c32_u64\0"
  "svwhilegt_c64_s64\0"
  "svwhilegt_c64_u64\0"
  "svwhilegt_c8_s64\0"
  "svwhilegt_c8_u64\0"
  "svwhilele_b16_s32\0"
  "svwhilele_b16_s64\0"
  "svwhilele_b16_s64_x2\0"
  "svwhilele_b16_u32\0"
  "svwhilele_b16_u64\0"
  "svwhilele_b16_u64_x2\0"
  "svwhilele_b32_s32\0"
  "svwhilele_b32_s64\0"
  "svwhilele_b32_s64_x2\0"
  "svwhilele_b32_u32\0"
  "svwhilele_b32_u64\0"
  "svwhilele_b32_u64_x2\0"
  "svwhilele_b64_s32\0"
  "svwhilele_b64_s64\0"
  "svwhilele_b64_s64_x2\0"
  "svwhilele_b64_u32\0"
  "svwhilele_b64_u64\0"
  "svwhilele_b64_u64_x2\0"
  "svwhilele_b8_s32\0"
  "svwhilele_b8_s64\0"
  "svwhilele_b8_s64_x2\0"
  "svwhilele_b8_u32\0"
  "svwhilele_b8_u64\0"
  "svwhilele_b8_u64_x2\0"
  "svwhilele_c16_s64\0"
  "svwhilele_c16_u64\0"
  "svwhilele_c32_s64\0"
  "svwhilele_c32_u64\0"
  "svwhilele_c64_s64\0"
  "svwhilele_c64_u64\0"
  "svwhilele_c8_s64\0"
  "svwhilele_c8_u64\0"
  "svwhilelt_b16_s32\0"
  "svwhilelt_b16_s64\0"
  "svwhilelt_b16_s64_x2\0"
  "svwhilelt_b16_u32\0"
  "svwhilelt_b16_u64\0"
  "svwhilelt_b16_u64_x2\0"
  "svwhilelt_b32_s32\0"
  "svwhilelt_b32_s64\0"
  "svwhilelt_b32_s64_x2\0"
  "svwhilelt_b32_u32\0"
  "svwhilelt_b32_u64\0"
  "svwhilelt_b32_u64_x2\0"
  "svwhilelt_b64_s32\0"
  "svwhilelt_b64_s64\0"
  "svwhilelt_b64_s64_x2\0"
  "svwhilelt_b64_u32\0"
  "svwhilelt_b64_u64\0"
  "svwhilelt_b64_u64_x2\0"
  "svwhilelt_b8_s32\0"
  "svwhilelt_b8_s64\0"
  "svwhilelt_b8_s64_x2\0"
  "svwhilelt_b8_u32\0"
  "svwhilelt_b8_u64\0"
  "svwhilelt_b8_u64_x2\0"
  "svwhilelt_c16_s64\0"
  "svwhilelt_c16_u64\0"
  "svwhilelt_c32_s64\0"
  "svwhilelt_c32_u64\0"
  "svwhilelt_c64_s64\0"
  "svwhilelt_c64_u64\0"
  "svwhilelt_c8_s64\0"
  "svwhilelt_c8_u64\0"
  "svwhilerw_bf16\0"
  "q16byC*yC*\0"
  "svwhilerw_f16\0"
  "q16bhC*hC*\0"
  "svwhilerw_f32\0"
  "q16bfC*fC*\0"
  "svwhilerw_f64\0"
  "q16bdC*dC*\0"
  "svwhilerw_s16\0"
  "q16bSsC*SsC*\0"
  "svwhilerw_s32\0"
  "q16bSiC*SiC*\0"
  "svwhilerw_s64\0"
  "q16bSWiC*SWiC*\0"
  "svwhilerw_s8\0"
  "q16bScC*ScC*\0"
  "svwhilerw_u16\0"
  "q16bUsC*UsC*\0"
  "svwhilerw_u32\0"
  "q16bUiC*UiC*\0"
  "svwhilerw_u64\0"
  "q16bUWiC*UWiC*\0"
  "svwhilerw_u8\0"
  "q16bUcC*UcC*\0"
  "svwhilewr_bf16\0"
  "svwhilewr_f16\0"
  "svwhilewr_f32\0"
  "svwhilewr_f64\0"
  "svwhilewr_s16\0"
  "svwhilewr_s32\0"
  "svwhilewr_s64\0"
  "svwhilewr_s8\0"
  "svwhilewr_u16\0"
  "svwhilewr_u32\0"
  "svwhilewr_u64\0"
  "svwhilewr_u8\0"
  "svwrffr\0"
  "vq16b\0"
  "svxar_n_s16\0"
  "svxar_n_s32\0"
  "svxar_n_s64\0"
  "svxar_n_s8\0"
  "svxar_n_u16\0"
  "svxar_n_u32\0"
  "svxar_n_u64\0"
  "svxar_n_u8\0"
  "svzip1_b16\0"
  "svzip1_b32\0"
  "svzip1_b64\0"
  "svzip1_b8\0"
  "svzip1_bf16\0"
  "svzip1_f16\0"
  "svzip1_f32\0"
  "svzip1_f64\0"
  "svzip1_s16\0"
  "svzip1_s32\0"
  "svzip1_s64\0"
  "svzip1_s8\0"
  "svzip1_u16\0"
  "svzip1_u32\0"
  "svzip1_u64\0"
  "svzip1_u8\0"
  "svzip1q_bf16\0"
  "svzip1q_f16\0"
  "svzip1q_f32\0"
  "svzip1q_f64\0"
  "svzip1q_s16\0"
  "svzip1q_s32\0"
  "svzip1q_s64\0"
  "svzip1q_s8\0"
  "svzip1q_u16\0"
  "svzip1q_u32\0"
  "svzip1q_u64\0"
  "svzip1q_u8\0"
  "svzip2_b16\0"
  "svzip2_b32\0"
  "svzip2_b64\0"
  "svzip2_b8\0"
  "svzip2_bf16\0"
  "svzip2_f16\0"
  "svzip2_f32\0"
  "svzip2_f64\0"
  "svzip2_s16\0"
  "svzip2_s32\0"
  "svzip2_s64\0"
  "svzip2_s8\0"
  "svzip2_u16\0"
  "svzip2_u32\0"
  "svzip2_u64\0"
  "svzip2_u8\0"
  "svzip2q_bf16\0"
  "svzip2q_f16\0"
  "svzip2q_f32\0"
  "svzip2q_f64\0"
  "svzip2q_s16\0"
  "svzip2q_s32\0"
  "svzip2q_s64\0"
  "svzip2q_s8\0"
  "svzip2q_u16\0"
  "svzip2q_u32\0"
  "svzip2q_u64\0"
  "svzip2q_u8\0"
  "svzip_bf16_x2\0"
  "svzip_bf16_x4\0"
  "svzip_f16_x2\0"
  "svzip_f16_x4\0"
  "svzip_f32_x2\0"
  "svzip_f32_x4\0"
  "svzip_f64_x2\0"
  "svzip_f64_x4\0"
  "svzip_mf8_x2\0"
  "svzip_mf8_x4\0"
  "svzip_s16_x2\0"
  "svzip_s16_x4\0"
  "svzip_s32_x2\0"
  "svzip_s32_x4\0"
  "svzip_s64_x2\0"
  "svzip_s64_x4\0"
  "svzip_s8_x2\0"
  "svzip_s8_x4\0"
  "svzip_u16_x2\0"
  "svzip_u16_x4\0"
  "svzip_u32_x2\0"
  "svzip_u32_x4\0"
  "svzip_u64_x2\0"
  "svzip_u64_x4\0"
  "svzip_u8_x2\0"
  "svzip_u8_x4\0"
  "svzipq1_bf16\0"
  "svzipq1_f16\0"
  "svzipq1_f32\0"
  "svzipq1_f64\0"
  "svzipq1_mf8\0"
  "svzipq1_s16\0"
  "svzipq1_s32\0"
  "svzipq1_s64\0"
  "svzipq1_s8\0"
  "svzipq1_u16\0"
  "svzipq1_u32\0"
  "svzipq1_u64\0"
  "svzipq1_u8\0"
  "svzipq2_bf16\0"
  "svzipq2_f16\0"
  "svzipq2_f32\0"
  "svzipq2_f64\0"
  "svzipq2_mf8\0"
  "svzipq2_s16\0"
  "svzipq2_s32\0"
  "svzipq2_s64\0"
  "svzipq2_s8\0"
  "svzipq2_u16\0"
  "svzipq2_u32\0"
  "svzipq2_u64\0"
  "svzipq2_u8\0"
  "svzipq_bf16_x2\0"
  "svzipq_bf16_x4\0"
  "svzipq_f16_x2\0"
  "svzipq_f16_x4\0"
  "svzipq_f32_x2\0"
  "svzipq_f32_x4\0"
  "svzipq_f64_x2\0"
  "svzipq_f64_x4\0"
  "svzipq_mf8_x2\0"
  "svzipq_mf8_x4\0"
  "svzipq_s16_x2\0"
  "svzipq_s16_x4\0"
  "svzipq_s32_x2\0"
  "svzipq_s32_x4\0"
  "svzipq_s64_x2\0"
  "svzipq_s64_x4\0"
  "svzipq_s8_x2\0"
  "svzipq_s8_x4\0"
  "svzipq_u16_x2\0"
  "svzipq_u16_x4\0"
  "svzipq_u32_x2\0"
  "svzipq_u32_x4\0"
  "svzipq_u64_x2\0"
  "svzipq_u64_x4\0"
  "svzipq_u8_x2\0"
  "svzipq_u8_x4\0"
  "sme|sve\0"
  "reinterpret_s8_s8\0"
  "reinterpret_s8_u8\0"
  "q16Scq16Uc\0"
  "reinterpret_s8_mf8\0"
  "q16Scq16m\0"
  "reinterpret_s8_s16\0"
  "reinterpret_s8_u16\0"
  "q16Scq8Us\0"
  "reinterpret_s8_s32\0"
  "q16Scq4i\0"
  "reinterpret_s8_u32\0"
  "q16Scq4Ui\0"
  "reinterpret_s8_s64\0"
  "q16Scq2Wi\0"
  "reinterpret_s8_u64\0"
  "q16Scq2UWi\0"
  "reinterpret_s8_f16\0"
  "q16Scq8h\0"
  "reinterpret_s8_bf16\0"
  "q16Scq8y\0"
  "reinterpret_s8_f32\0"
  "q16Scq4f\0"
  "reinterpret_s8_f64\0"
  "q16Scq2d\0"
  "reinterpret_u8_s8\0"
  "q16Ucq16Sc\0"
  "reinterpret_u8_u8\0"
  "reinterpret_u8_mf8\0"
  "q16Ucq16m\0"
  "reinterpret_u8_s16\0"
  "reinterpret_u8_u16\0"
  "reinterpret_u8_s32\0"
  "q16Ucq4i\0"
  "reinterpret_u8_u32\0"
  "q16Ucq4Ui\0"
  "reinterpret_u8_s64\0"
  "q16Ucq2Wi\0"
  "reinterpret_u8_u64\0"
  "q16Ucq2UWi\0"
  "reinterpret_u8_f16\0"
  "q16Ucq8h\0"
  "reinterpret_u8_bf16\0"
  "q16Ucq8y\0"
  "reinterpret_u8_f32\0"
  "q16Ucq4f\0"
  "reinterpret_u8_f64\0"
  "q16Ucq2d\0"
  "reinterpret_mf8_s8\0"
  "q16mq16Sc\0"
  "reinterpret_mf8_u8\0"
  "q16mq16Uc\0"
  "reinterpret_mf8_mf8\0"
  "q16mq16m\0"
  "reinterpret_mf8_s16\0"
  "q16mq8s\0"
  "reinterpret_mf8_u16\0"
  "q16mq8Us\0"
  "reinterpret_mf8_s32\0"
  "q16mq4i\0"
  "reinterpret_mf8_u32\0"
  "q16mq4Ui\0"
  "reinterpret_mf8_s64\0"
  "q16mq2Wi\0"
  "reinterpret_mf8_u64\0"
  "q16mq2UWi\0"
  "reinterpret_mf8_f16\0"
  "q16mq8h\0"
  "reinterpret_mf8_bf16\0"
  "q16mq8y\0"
  "reinterpret_mf8_f32\0"
  "q16mq4f\0"
  "reinterpret_mf8_f64\0"
  "q16mq2d\0"
  "reinterpret_s16_s8\0"
  "reinterpret_s16_u8\0"
  "q8sq16Uc\0"
  "reinterpret_s16_mf8\0"
  "q8sq16m\0"
  "reinterpret_s16_s16\0"
  "reinterpret_s16_u16\0"
  "q8sq8Us\0"
  "reinterpret_s16_s32\0"
  "reinterpret_s16_u32\0"
  "q8sq4Ui\0"
  "reinterpret_s16_s64\0"
  "q8sq2Wi\0"
  "reinterpret_s16_u64\0"
  "q8sq2UWi\0"
  "reinterpret_s16_f16\0"
  "q8sq8h\0"
  "reinterpret_s16_bf16\0"
  "q8sq8y\0"
  "reinterpret_s16_f32\0"
  "q8sq4f\0"
  "reinterpret_s16_f64\0"
  "q8sq2d\0"
  "reinterpret_u16_s8\0"
  "q8Usq16Sc\0"
  "reinterpret_u16_u8\0"
  "reinterpret_u16_mf8\0"
  "q8Usq16m\0"
  "reinterpret_u16_s16\0"
  "q8Usq8s\0"
  "reinterpret_u16_u16\0"
  "reinterpret_u16_s32\0"
  "reinterpret_u16_u32\0"
  "reinterpret_u16_s64\0"
  "q8Usq2Wi\0"
  "reinterpret_u16_u64\0"
  "q8Usq2UWi\0"
  "reinterpret_u16_f16\0"
  "q8Usq8h\0"
  "reinterpret_u16_bf16\0"
  "q8Usq8y\0"
  "reinterpret_u16_f32\0"
  "q8Usq4f\0"
  "reinterpret_u16_f64\0"
  "q8Usq2d\0"
  "reinterpret_s32_s8\0"
  "q4iq16Sc\0"
  "reinterpret_s32_u8\0"
  "q4iq16Uc\0"
  "reinterpret_s32_mf8\0"
  "q4iq16m\0"
  "reinterpret_s32_s16\0"
  "reinterpret_s32_u16\0"
  "q4iq8Us\0"
  "reinterpret_s32_s32\0"
  "reinterpret_s32_u32\0"
  "q4iq4Ui\0"
  "reinterpret_s32_s64\0"
  "reinterpret_s32_u64\0"
  "q4iq2UWi\0"
  "reinterpret_s32_f16\0"
  "q4iq8h\0"
  "reinterpret_s32_bf16\0"
  "q4iq8y\0"
  "reinterpret_s32_f32\0"
  "q4iq4f\0"
  "reinterpret_s32_f64\0"
  "q4iq2d\0"
  "reinterpret_u32_s8\0"
  "q4Uiq16Sc\0"
  "reinterpret_u32_u8\0"
  "q4Uiq16Uc\0"
  "reinterpret_u32_mf8\0"
  "q4Uiq16m\0"
  "reinterpret_u32_s16\0"
  "q4Uiq8s\0"
  "reinterpret_u32_u16\0"
  "reinterpret_u32_s32\0"
  "q4Uiq4i\0"
  "reinterpret_u32_u32\0"
  "reinterpret_u32_s64\0"
  "reinterpret_u32_u64\0"
  "reinterpret_u32_f16\0"
  "q4Uiq8h\0"
  "reinterpret_u32_bf16\0"
  "q4Uiq8y\0"
  "reinterpret_u32_f32\0"
  "q4Uiq4f\0"
  "reinterpret_u32_f64\0"
  "q4Uiq2d\0"
  "reinterpret_s64_s8\0"
  "q2Wiq16Sc\0"
  "reinterpret_s64_u8\0"
  "q2Wiq16Uc\0"
  "reinterpret_s64_mf8\0"
  "q2Wiq16m\0"
  "reinterpret_s64_s16\0"
  "q2Wiq8s\0"
  "reinterpret_s64_u16\0"
  "q2Wiq8Us\0"
  "reinterpret_s64_s32\0"
  "reinterpret_s64_u32\0"
  "q2Wiq4Ui\0"
  "reinterpret_s64_s64\0"
  "reinterpret_s64_u64\0"
  "q2Wiq2UWi\0"
  "reinterpret_s64_f16\0"
  "q2Wiq8h\0"
  "reinterpret_s64_bf16\0"
  "q2Wiq8y\0"
  "reinterpret_s64_f32\0"
  "q2Wiq4f\0"
  "reinterpret_s64_f64\0"
  "q2Wiq2d\0"
  "reinterpret_u64_s8\0"
  "q2UWiq16Sc\0"
  "reinterpret_u64_u8\0"
  "q2UWiq16Uc\0"
  "reinterpret_u64_mf8\0"
  "q2UWiq16m\0"
  "reinterpret_u64_s16\0"
  "q2UWiq8s\0"
  "reinterpret_u64_u16\0"
  "q2UWiq8Us\0"
  "reinterpret_u64_s32\0"
  "q2UWiq4i\0"
  "reinterpret_u64_u32\0"
  "reinterpret_u64_s64\0"
  "q2UWiq2Wi\0"
  "reinterpret_u64_u64\0"
  "reinterpret_u64_f16\0"
  "q2UWiq8h\0"
  "reinterpret_u64_bf16\0"
  "q2UWiq8y\0"
  "reinterpret_u64_f32\0"
  "q2UWiq4f\0"
  "reinterpret_u64_f64\0"
  "q2UWiq2d\0"
  "reinterpret_f16_s8\0"
  "q8hq16Sc\0"
  "reinterpret_f16_u8\0"
  "q8hq16Uc\0"
  "reinterpret_f16_mf8\0"
  "q8hq16m\0"
  "reinterpret_f16_s16\0"
  "q8hq8s\0"
  "reinterpret_f16_u16\0"
  "reinterpret_f16_s32\0"
  "q8hq4i\0"
  "reinterpret_f16_u32\0"
  "q8hq4Ui\0"
  "reinterpret_f16_s64\0"
  "q8hq2Wi\0"
  "reinterpret_f16_u64\0"
  "q8hq2UWi\0"
  "reinterpret_f16_f16\0"
  "reinterpret_f16_bf16\0"
  "q8hq8y\0"
  "reinterpret_f16_f32\0"
  "q8hq4f\0"
  "reinterpret_f16_f64\0"
  "q8hq2d\0"
  "reinterpret_bf16_s8\0"
  "q8yq16Sc\0"
  "reinterpret_bf16_u8\0"
  "q8yq16Uc\0"
  "reinterpret_bf16_mf8\0"
  "q8yq16m\0"
  "reinterpret_bf16_s16\0"
  "q8yq8s\0"
  "reinterpret_bf16_u16\0"
  "q8yq8Us\0"
  "reinterpret_bf16_s32\0"
  "q8yq4i\0"
  "reinterpret_bf16_u32\0"
  "q8yq4Ui\0"
  "reinterpret_bf16_s64\0"
  "q8yq2Wi\0"
  "reinterpret_bf16_u64\0"
  "q8yq2UWi\0"
  "reinterpret_bf16_f16\0"
  "q8yq8h\0"
  "reinterpret_bf16_bf16\0"
  "reinterpret_bf16_f32\0"
  "q8yq4f\0"
  "reinterpret_bf16_f64\0"
  "q8yq2d\0"
  "reinterpret_f32_s8\0"
  "q4fq16Sc\0"
  "reinterpret_f32_u8\0"
  "q4fq16Uc\0"
  "reinterpret_f32_mf8\0"
  "q4fq16m\0"
  "reinterpret_f32_s16\0"
  "q4fq8s\0"
  "reinterpret_f32_u16\0"
  "q4fq8Us\0"
  "reinterpret_f32_s32\0"
  "q4fq4i\0"
  "reinterpret_f32_u32\0"
  "reinterpret_f32_s64\0"
  "q4fq2Wi\0"
  "reinterpret_f32_u64\0"
  "q4fq2UWi\0"
  "reinterpret_f32_f16\0"
  "q4fq8h\0"
  "reinterpret_f32_bf16\0"
  "q4fq8y\0"
  "reinterpret_f32_f32\0"
  "reinterpret_f32_f64\0"
  "q4fq2d\0"
  "reinterpret_f64_s8\0"
  "q2dq16Sc\0"
  "reinterpret_f64_u8\0"
  "q2dq16Uc\0"
  "reinterpret_f64_mf8\0"
  "q2dq16m\0"
  "reinterpret_f64_s16\0"
  "q2dq8s\0"
  "reinterpret_f64_u16\0"
  "q2dq8Us\0"
  "reinterpret_f64_s32\0"
  "q2dq4i\0"
  "reinterpret_f64_u32\0"
  "q2dq4Ui\0"
  "reinterpret_f64_s64\0"
  "q2dq2Wi\0"
  "reinterpret_f64_u64\0"
  "reinterpret_f64_f16\0"
  "q2dq8h\0"
  "reinterpret_f64_bf16\0"
  "q2dq8y\0"
  "reinterpret_f64_f32\0"
  "q2dq4f\0"
  "reinterpret_f64_f64\0"
  "reinterpret_s8_s8_x2\0"
  "reinterpret_s8_u8_x2\0"
  "q32Scq32Uc\0"
  "reinterpret_s8_mf8_x2\0"
  "q32Scq32m\0"
  "reinterpret_s8_s16_x2\0"
  "q32Scq16s\0"
  "reinterpret_s8_u16_x2\0"
  "q32Scq16Us\0"
  "reinterpret_s8_s32_x2\0"
  "q32Scq8i\0"
  "reinterpret_s8_u32_x2\0"
  "q32Scq8Ui\0"
  "reinterpret_s8_s64_x2\0"
  "q32Scq4Wi\0"
  "reinterpret_s8_u64_x2\0"
  "q32Scq4UWi\0"
  "reinterpret_s8_f16_x2\0"
  "q32Scq16h\0"
  "reinterpret_s8_bf16_x2\0"
  "q32Scq16y\0"
  "reinterpret_s8_f32_x2\0"
  "q32Scq8f\0"
  "reinterpret_s8_f64_x2\0"
  "q32Scq4d\0"
  "reinterpret_u8_s8_x2\0"
  "q32Ucq32Sc\0"
  "reinterpret_u8_u8_x2\0"
  "reinterpret_u8_mf8_x2\0"
  "q32Ucq32m\0"
  "reinterpret_u8_s16_x2\0"
  "q32Ucq16s\0"
  "reinterpret_u8_u16_x2\0"
  "q32Ucq16Us\0"
  "reinterpret_u8_s32_x2\0"
  "q32Ucq8i\0"
  "reinterpret_u8_u32_x2\0"
  "q32Ucq8Ui\0"
  "reinterpret_u8_s64_x2\0"
  "q32Ucq4Wi\0"
  "reinterpret_u8_u64_x2\0"
  "q32Ucq4UWi\0"
  "reinterpret_u8_f16_x2\0"
  "q32Ucq16h\0"
  "reinterpret_u8_bf16_x2\0"
  "q32Ucq16y\0"
  "reinterpret_u8_f32_x2\0"
  "q32Ucq8f\0"
  "reinterpret_u8_f64_x2\0"
  "q32Ucq4d\0"
  "reinterpret_mf8_s8_x2\0"
  "q32mq32Sc\0"
  "reinterpret_mf8_u8_x2\0"
  "q32mq32Uc\0"
  "reinterpret_mf8_mf8_x2\0"
  "reinterpret_mf8_s16_x2\0"
  "q32mq16s\0"
  "reinterpret_mf8_u16_x2\0"
  "q32mq16Us\0"
  "reinterpret_mf8_s32_x2\0"
  "q32mq8i\0"
  "reinterpret_mf8_u32_x2\0"
  "q32mq8Ui\0"
  "reinterpret_mf8_s64_x2\0"
  "q32mq4Wi\0"
  "reinterpret_mf8_u64_x2\0"
  "q32mq4UWi\0"
  "reinterpret_mf8_f16_x2\0"
  "q32mq16h\0"
  "reinterpret_mf8_bf16_x2\0"
  "q32mq16y\0"
  "reinterpret_mf8_f32_x2\0"
  "q32mq8f\0"
  "reinterpret_mf8_f64_x2\0"
  "q32mq4d\0"
  "reinterpret_s16_s8_x2\0"
  "q16sq32Sc\0"
  "reinterpret_s16_u8_x2\0"
  "q16sq32Uc\0"
  "reinterpret_s16_mf8_x2\0"
  "q16sq32m\0"
  "reinterpret_s16_s16_x2\0"
  "reinterpret_s16_u16_x2\0"
  "q16sq16Us\0"
  "reinterpret_s16_s32_x2\0"
  "q16sq8i\0"
  "reinterpret_s16_u32_x2\0"
  "q16sq8Ui\0"
  "reinterpret_s16_s64_x2\0"
  "q16sq4Wi\0"
  "reinterpret_s16_u64_x2\0"
  "q16sq4UWi\0"
  "reinterpret_s16_f16_x2\0"
  "q16sq16h\0"
  "reinterpret_s16_bf16_x2\0"
  "q16sq16y\0"
  "reinterpret_s16_f32_x2\0"
  "q16sq8f\0"
  "reinterpret_s16_f64_x2\0"
  "q16sq4d\0"
  "reinterpret_u16_s8_x2\0"
  "q16Usq32Sc\0"
  "reinterpret_u16_u8_x2\0"
  "q16Usq32Uc\0"
  "reinterpret_u16_mf8_x2\0"
  "q16Usq32m\0"
  "reinterpret_u16_s16_x2\0"
  "q16Usq16s\0"
  "reinterpret_u16_u16_x2\0"
  "reinterpret_u16_s32_x2\0"
  "q16Usq8i\0"
  "reinterpret_u16_u32_x2\0"
  "q16Usq8Ui\0"
  "reinterpret_u16_s64_x2\0"
  "q16Usq4Wi\0"
  "reinterpret_u16_u64_x2\0"
  "q16Usq4UWi\0"
  "reinterpret_u16_f16_x2\0"
  "q16Usq16h\0"
  "reinterpret_u16_bf16_x2\0"
  "q16Usq16y\0"
  "reinterpret_u16_f32_x2\0"
  "q16Usq8f\0"
  "reinterpret_u16_f64_x2\0"
  "q16Usq4d\0"
  "reinterpret_s32_s8_x2\0"
  "q8iq32Sc\0"
  "reinterpret_s32_u8_x2\0"
  "q8iq32Uc\0"
  "reinterpret_s32_mf8_x2\0"
  "q8iq32m\0"
  "reinterpret_s32_s16_x2\0"
  "q8iq16s\0"
  "reinterpret_s32_u16_x2\0"
  "q8iq16Us\0"
  "reinterpret_s32_s32_x2\0"
  "reinterpret_s32_u32_x2\0"
  "q8iq8Ui\0"
  "reinterpret_s32_s64_x2\0"
  "q8iq4Wi\0"
  "reinterpret_s32_u64_x2\0"
  "q8iq4UWi\0"
  "reinterpret_s32_f16_x2\0"
  "q8iq16h\0"
  "reinterpret_s32_bf16_x2\0"
  "q8iq16y\0"
  "reinterpret_s32_f32_x2\0"
  "reinterpret_s32_f64_x2\0"
  "q8iq4d\0"
  "reinterpret_u32_s8_x2\0"
  "q8Uiq32Sc\0"
  "reinterpret_u32_u8_x2\0"
  "q8Uiq32Uc\0"
  "reinterpret_u32_mf8_x2\0"
  "q8Uiq32m\0"
  "reinterpret_u32_s16_x2\0"
  "q8Uiq16s\0"
  "reinterpret_u32_u16_x2\0"
  "q8Uiq16Us\0"
  "reinterpret_u32_s32_x2\0"
  "q8Uiq8i\0"
  "reinterpret_u32_u32_x2\0"
  "reinterpret_u32_s64_x2\0"
  "q8Uiq4Wi\0"
  "reinterpret_u32_u64_x2\0"
  "q8Uiq4UWi\0"
  "reinterpret_u32_f16_x2\0"
  "q8Uiq16h\0"
  "reinterpret_u32_bf16_x2\0"
  "q8Uiq16y\0"
  "reinterpret_u32_f32_x2\0"
  "reinterpret_u32_f64_x2\0"
  "q8Uiq4d\0"
  "reinterpret_s64_s8_x2\0"
  "q4Wiq32Sc\0"
  "reinterpret_s64_u8_x2\0"
  "q4Wiq32Uc\0"
  "reinterpret_s64_mf8_x2\0"
  "q4Wiq32m\0"
  "reinterpret_s64_s16_x2\0"
  "q4Wiq16s\0"
  "reinterpret_s64_u16_x2\0"
  "q4Wiq16Us\0"
  "reinterpret_s64_s32_x2\0"
  "q4Wiq8i\0"
  "reinterpret_s64_u32_x2\0"
  "q4Wiq8Ui\0"
  "reinterpret_s64_s64_x2\0"
  "reinterpret_s64_u64_x2\0"
  "q4Wiq4UWi\0"
  "reinterpret_s64_f16_x2\0"
  "q4Wiq16h\0"
  "reinterpret_s64_bf16_x2\0"
  "q4Wiq16y\0"
  "reinterpret_s64_f32_x2\0"
  "q4Wiq8f\0"
  "reinterpret_s64_f64_x2\0"
  "q4Wiq4d\0"
  "reinterpret_u64_s8_x2\0"
  "q4UWiq32Sc\0"
  "reinterpret_u64_u8_x2\0"
  "q4UWiq32Uc\0"
  "reinterpret_u64_mf8_x2\0"
  "q4UWiq32m\0"
  "reinterpret_u64_s16_x2\0"
  "q4UWiq16s\0"
  "reinterpret_u64_u16_x2\0"
  "q4UWiq16Us\0"
  "reinterpret_u64_s32_x2\0"
  "q4UWiq8i\0"
  "reinterpret_u64_u32_x2\0"
  "q4UWiq8Ui\0"
  "reinterpret_u64_s64_x2\0"
  "q4UWiq4Wi\0"
  "reinterpret_u64_u64_x2\0"
  "reinterpret_u64_f16_x2\0"
  "q4UWiq16h\0"
  "reinterpret_u64_bf16_x2\0"
  "q4UWiq16y\0"
  "reinterpret_u64_f32_x2\0"
  "q4UWiq8f\0"
  "reinterpret_u64_f64_x2\0"
  "q4UWiq4d\0"
  "reinterpret_f16_s8_x2\0"
  "q16hq32Sc\0"
  "reinterpret_f16_u8_x2\0"
  "q16hq32Uc\0"
  "reinterpret_f16_mf8_x2\0"
  "q16hq32m\0"
  "reinterpret_f16_s16_x2\0"
  "q16hq16s\0"
  "reinterpret_f16_u16_x2\0"
  "q16hq16Us\0"
  "reinterpret_f16_s32_x2\0"
  "q16hq8i\0"
  "reinterpret_f16_u32_x2\0"
  "q16hq8Ui\0"
  "reinterpret_f16_s64_x2\0"
  "q16hq4Wi\0"
  "reinterpret_f16_u64_x2\0"
  "q16hq4UWi\0"
  "reinterpret_f16_f16_x2\0"
  "reinterpret_f16_bf16_x2\0"
  "q16hq16y\0"
  "reinterpret_f16_f32_x2\0"
  "q16hq8f\0"
  "reinterpret_f16_f64_x2\0"
  "q16hq4d\0"
  "reinterpret_bf16_s8_x2\0"
  "q16yq32Sc\0"
  "reinterpret_bf16_u8_x2\0"
  "q16yq32Uc\0"
  "reinterpret_bf16_mf8_x2\0"
  "q16yq32m\0"
  "reinterpret_bf16_s16_x2\0"
  "q16yq16s\0"
  "reinterpret_bf16_u16_x2\0"
  "q16yq16Us\0"
  "reinterpret_bf16_s32_x2\0"
  "q16yq8i\0"
  "reinterpret_bf16_u32_x2\0"
  "q16yq8Ui\0"
  "reinterpret_bf16_s64_x2\0"
  "q16yq4Wi\0"
  "reinterpret_bf16_u64_x2\0"
  "q16yq4UWi\0"
  "reinterpret_bf16_f16_x2\0"
  "q16yq16h\0"
  "reinterpret_bf16_bf16_x2\0"
  "reinterpret_bf16_f32_x2\0"
  "q16yq8f\0"
  "reinterpret_bf16_f64_x2\0"
  "q16yq4d\0"
  "reinterpret_f32_s8_x2\0"
  "q8fq32Sc\0"
  "reinterpret_f32_u8_x2\0"
  "q8fq32Uc\0"
  "reinterpret_f32_mf8_x2\0"
  "q8fq32m\0"
  "reinterpret_f32_s16_x2\0"
  "q8fq16s\0"
  "reinterpret_f32_u16_x2\0"
  "q8fq16Us\0"
  "reinterpret_f32_s32_x2\0"
  "reinterpret_f32_u32_x2\0"
  "reinterpret_f32_s64_x2\0"
  "q8fq4Wi\0"
  "reinterpret_f32_u64_x2\0"
  "q8fq4UWi\0"
  "reinterpret_f32_f16_x2\0"
  "q8fq16h\0"
  "reinterpret_f32_bf16_x2\0"
  "q8fq16y\0"
  "reinterpret_f32_f32_x2\0"
  "reinterpret_f32_f64_x2\0"
  "q8fq4d\0"
  "reinterpret_f64_s8_x2\0"
  "q4dq32Sc\0"
  "reinterpret_f64_u8_x2\0"
  "q4dq32Uc\0"
  "reinterpret_f64_mf8_x2\0"
  "q4dq32m\0"
  "reinterpret_f64_s16_x2\0"
  "q4dq16s\0"
  "reinterpret_f64_u16_x2\0"
  "q4dq16Us\0"
  "reinterpret_f64_s32_x2\0"
  "q4dq8i\0"
  "reinterpret_f64_u32_x2\0"
  "q4dq8Ui\0"
  "reinterpret_f64_s64_x2\0"
  "q4dq4Wi\0"
  "reinterpret_f64_u64_x2\0"
  "q4dq4UWi\0"
  "reinterpret_f64_f16_x2\0"
  "q4dq16h\0"
  "reinterpret_f64_bf16_x2\0"
  "q4dq16y\0"
  "reinterpret_f64_f32_x2\0"
  "q4dq8f\0"
  "reinterpret_f64_f64_x2\0"
  "reinterpret_s8_s8_x3\0"
  "q48Scq48Sc\0"
  "reinterpret_s8_u8_x3\0"
  "q48Scq48Uc\0"
  "reinterpret_s8_mf8_x3\0"
  "q48Scq48m\0"
  "reinterpret_s8_s16_x3\0"
  "q48Scq24s\0"
  "reinterpret_s8_u16_x3\0"
  "q48Scq24Us\0"
  "reinterpret_s8_s32_x3\0"
  "q48Scq12i\0"
  "reinterpret_s8_u32_x3\0"
  "q48Scq12Ui\0"
  "reinterpret_s8_s64_x3\0"
  "q48Scq6Wi\0"
  "reinterpret_s8_u64_x3\0"
  "q48Scq6UWi\0"
  "reinterpret_s8_f16_x3\0"
  "q48Scq24h\0"
  "reinterpret_s8_bf16_x3\0"
  "q48Scq24y\0"
  "reinterpret_s8_f32_x3\0"
  "q48Scq12f\0"
  "reinterpret_s8_f64_x3\0"
  "q48Scq6d\0"
  "reinterpret_u8_s8_x3\0"
  "q48Ucq48Sc\0"
  "reinterpret_u8_u8_x3\0"
  "q48Ucq48Uc\0"
  "reinterpret_u8_mf8_x3\0"
  "q48Ucq48m\0"
  "reinterpret_u8_s16_x3\0"
  "q48Ucq24s\0"
  "reinterpret_u8_u16_x3\0"
  "q48Ucq24Us\0"
  "reinterpret_u8_s32_x3\0"
  "q48Ucq12i\0"
  "reinterpret_u8_u32_x3\0"
  "q48Ucq12Ui\0"
  "reinterpret_u8_s64_x3\0"
  "q48Ucq6Wi\0"
  "reinterpret_u8_u64_x3\0"
  "q48Ucq6UWi\0"
  "reinterpret_u8_f16_x3\0"
  "q48Ucq24h\0"
  "reinterpret_u8_bf16_x3\0"
  "q48Ucq24y\0"
  "reinterpret_u8_f32_x3\0"
  "q48Ucq12f\0"
  "reinterpret_u8_f64_x3\0"
  "q48Ucq6d\0"
  "reinterpret_mf8_s8_x3\0"
  "q48mq48Sc\0"
  "reinterpret_mf8_u8_x3\0"
  "q48mq48Uc\0"
  "reinterpret_mf8_mf8_x3\0"
  "q48mq48m\0"
  "reinterpret_mf8_s16_x3\0"
  "q48mq24s\0"
  "reinterpret_mf8_u16_x3\0"
  "q48mq24Us\0"
  "reinterpret_mf8_s32_x3\0"
  "q48mq12i\0"
  "reinterpret_mf8_u32_x3\0"
  "q48mq12Ui\0"
  "reinterpret_mf8_s64_x3\0"
  "q48mq6Wi\0"
  "reinterpret_mf8_u64_x3\0"
  "q48mq6UWi\0"
  "reinterpret_mf8_f16_x3\0"
  "q48mq24h\0"
  "reinterpret_mf8_bf16_x3\0"
  "q48mq24y\0"
  "reinterpret_mf8_f32_x3\0"
  "q48mq12f\0"
  "reinterpret_mf8_f64_x3\0"
  "q48mq6d\0"
  "reinterpret_s16_s8_x3\0"
  "q24sq48Sc\0"
  "reinterpret_s16_u8_x3\0"
  "q24sq48Uc\0"
  "reinterpret_s16_mf8_x3\0"
  "q24sq48m\0"
  "reinterpret_s16_s16_x3\0"
  "q24sq24s\0"
  "reinterpret_s16_u16_x3\0"
  "q24sq24Us\0"
  "reinterpret_s16_s32_x3\0"
  "q24sq12i\0"
  "reinterpret_s16_u32_x3\0"
  "q24sq12Ui\0"
  "reinterpret_s16_s64_x3\0"
  "q24sq6Wi\0"
  "reinterpret_s16_u64_x3\0"
  "q24sq6UWi\0"
  "reinterpret_s16_f16_x3\0"
  "q24sq24h\0"
  "reinterpret_s16_bf16_x3\0"
  "q24sq24y\0"
  "reinterpret_s16_f32_x3\0"
  "q24sq12f\0"
  "reinterpret_s16_f64_x3\0"
  "q24sq6d\0"
  "reinterpret_u16_s8_x3\0"
  "q24Usq48Sc\0"
  "reinterpret_u16_u8_x3\0"
  "q24Usq48Uc\0"
  "reinterpret_u16_mf8_x3\0"
  "q24Usq48m\0"
  "reinterpret_u16_s16_x3\0"
  "q24Usq24s\0"
  "reinterpret_u16_u16_x3\0"
  "q24Usq24Us\0"
  "reinterpret_u16_s32_x3\0"
  "q24Usq12i\0"
  "reinterpret_u16_u32_x3\0"
  "q24Usq12Ui\0"
  "reinterpret_u16_s64_x3\0"
  "q24Usq6Wi\0"
  "reinterpret_u16_u64_x3\0"
  "q24Usq6UWi\0"
  "reinterpret_u16_f16_x3\0"
  "q24Usq24h\0"
  "reinterpret_u16_bf16_x3\0"
  "q24Usq24y\0"
  "reinterpret_u16_f32_x3\0"
  "q24Usq12f\0"
  "reinterpret_u16_f64_x3\0"
  "q24Usq6d\0"
  "reinterpret_s32_s8_x3\0"
  "q12iq48Sc\0"
  "reinterpret_s32_u8_x3\0"
  "q12iq48Uc\0"
  "reinterpret_s32_mf8_x3\0"
  "q12iq48m\0"
  "reinterpret_s32_s16_x3\0"
  "q12iq24s\0"
  "reinterpret_s32_u16_x3\0"
  "q12iq24Us\0"
  "reinterpret_s32_s32_x3\0"
  "q12iq12i\0"
  "reinterpret_s32_u32_x3\0"
  "q12iq12Ui\0"
  "reinterpret_s32_s64_x3\0"
  "q12iq6Wi\0"
  "reinterpret_s32_u64_x3\0"
  "q12iq6UWi\0"
  "reinterpret_s32_f16_x3\0"
  "q12iq24h\0"
  "reinterpret_s32_bf16_x3\0"
  "q12iq24y\0"
  "reinterpret_s32_f32_x3\0"
  "q12iq12f\0"
  "reinterpret_s32_f64_x3\0"
  "q12iq6d\0"
  "reinterpret_u32_s8_x3\0"
  "q12Uiq48Sc\0"
  "reinterpret_u32_u8_x3\0"
  "q12Uiq48Uc\0"
  "reinterpret_u32_mf8_x3\0"
  "q12Uiq48m\0"
  "reinterpret_u32_s16_x3\0"
  "q12Uiq24s\0"
  "reinterpret_u32_u16_x3\0"
  "q12Uiq24Us\0"
  "reinterpret_u32_s32_x3\0"
  "q12Uiq12i\0"
  "reinterpret_u32_u32_x3\0"
  "q12Uiq12Ui\0"
  "reinterpret_u32_s64_x3\0"
  "q12Uiq6Wi\0"
  "reinterpret_u32_u64_x3\0"
  "q12Uiq6UWi\0"
  "reinterpret_u32_f16_x3\0"
  "q12Uiq24h\0"
  "reinterpret_u32_bf16_x3\0"
  "q12Uiq24y\0"
  "reinterpret_u32_f32_x3\0"
  "q12Uiq12f\0"
  "reinterpret_u32_f64_x3\0"
  "q12Uiq6d\0"
  "reinterpret_s64_s8_x3\0"
  "q6Wiq48Sc\0"
  "reinterpret_s64_u8_x3\0"
  "q6Wiq48Uc\0"
  "reinterpret_s64_mf8_x3\0"
  "q6Wiq48m\0"
  "reinterpret_s64_s16_x3\0"
  "q6Wiq24s\0"
  "reinterpret_s64_u16_x3\0"
  "q6Wiq24Us\0"
  "reinterpret_s64_s32_x3\0"
  "q6Wiq12i\0"
  "reinterpret_s64_u32_x3\0"
  "q6Wiq12Ui\0"
  "reinterpret_s64_s64_x3\0"
  "q6Wiq6Wi\0"
  "reinterpret_s64_u64_x3\0"
  "q6Wiq6UWi\0"
  "reinterpret_s64_f16_x3\0"
  "q6Wiq24h\0"
  "reinterpret_s64_bf16_x3\0"
  "q6Wiq24y\0"
  "reinterpret_s64_f32_x3\0"
  "q6Wiq12f\0"
  "reinterpret_s64_f64_x3\0"
  "q6Wiq6d\0"
  "reinterpret_u64_s8_x3\0"
  "q6UWiq48Sc\0"
  "reinterpret_u64_u8_x3\0"
  "q6UWiq48Uc\0"
  "reinterpret_u64_mf8_x3\0"
  "q6UWiq48m\0"
  "reinterpret_u64_s16_x3\0"
  "q6UWiq24s\0"
  "reinterpret_u64_u16_x3\0"
  "q6UWiq24Us\0"
  "reinterpret_u64_s32_x3\0"
  "q6UWiq12i\0"
  "reinterpret_u64_u32_x3\0"
  "q6UWiq12Ui\0"
  "reinterpret_u64_s64_x3\0"
  "q6UWiq6Wi\0"
  "reinterpret_u64_u64_x3\0"
  "q6UWiq6UWi\0"
  "reinterpret_u64_f16_x3\0"
  "q6UWiq24h\0"
  "reinterpret_u64_bf16_x3\0"
  "q6UWiq24y\0"
  "reinterpret_u64_f32_x3\0"
  "q6UWiq12f\0"
  "reinterpret_u64_f64_x3\0"
  "q6UWiq6d\0"
  "reinterpret_f16_s8_x3\0"
  "q24hq48Sc\0"
  "reinterpret_f16_u8_x3\0"
  "q24hq48Uc\0"
  "reinterpret_f16_mf8_x3\0"
  "q24hq48m\0"
  "reinterpret_f16_s16_x3\0"
  "q24hq24s\0"
  "reinterpret_f16_u16_x3\0"
  "q24hq24Us\0"
  "reinterpret_f16_s32_x3\0"
  "q24hq12i\0"
  "reinterpret_f16_u32_x3\0"
  "q24hq12Ui\0"
  "reinterpret_f16_s64_x3\0"
  "q24hq6Wi\0"
  "reinterpret_f16_u64_x3\0"
  "q24hq6UWi\0"
  "reinterpret_f16_f16_x3\0"
  "q24hq24h\0"
  "reinterpret_f16_bf16_x3\0"
  "q24hq24y\0"
  "reinterpret_f16_f32_x3\0"
  "q24hq12f\0"
  "reinterpret_f16_f64_x3\0"
  "q24hq6d\0"
  "reinterpret_bf16_s8_x3\0"
  "q24yq48Sc\0"
  "reinterpret_bf16_u8_x3\0"
  "q24yq48Uc\0"
  "reinterpret_bf16_mf8_x3\0"
  "q24yq48m\0"
  "reinterpret_bf16_s16_x3\0"
  "q24yq24s\0"
  "reinterpret_bf16_u16_x3\0"
  "q24yq24Us\0"
  "reinterpret_bf16_s32_x3\0"
  "q24yq12i\0"
  "reinterpret_bf16_u32_x3\0"
  "q24yq12Ui\0"
  "reinterpret_bf16_s64_x3\0"
  "q24yq6Wi\0"
  "reinterpret_bf16_u64_x3\0"
  "q24yq6UWi\0"
  "reinterpret_bf16_f16_x3\0"
  "q24yq24h\0"
  "reinterpret_bf16_bf16_x3\0"
  "q24yq24y\0"
  "reinterpret_bf16_f32_x3\0"
  "q24yq12f\0"
  "reinterpret_bf16_f64_x3\0"
  "q24yq6d\0"
  "reinterpret_f32_s8_x3\0"
  "q12fq48Sc\0"
  "reinterpret_f32_u8_x3\0"
  "q12fq48Uc\0"
  "reinterpret_f32_mf8_x3\0"
  "q12fq48m\0"
  "reinterpret_f32_s16_x3\0"
  "q12fq24s\0"
  "reinterpret_f32_u16_x3\0"
  "q12fq24Us\0"
  "reinterpret_f32_s32_x3\0"
  "q12fq12i\0"
  "reinterpret_f32_u32_x3\0"
  "q12fq12Ui\0"
  "reinterpret_f32_s64_x3\0"
  "q12fq6Wi\0"
  "reinterpret_f32_u64_x3\0"
  "q12fq6UWi\0"
  "reinterpret_f32_f16_x3\0"
  "q12fq24h\0"
  "reinterpret_f32_bf16_x3\0"
  "q12fq24y\0"
  "reinterpret_f32_f32_x3\0"
  "q12fq12f\0"
  "reinterpret_f32_f64_x3\0"
  "q12fq6d\0"
  "reinterpret_f64_s8_x3\0"
  "q6dq48Sc\0"
  "reinterpret_f64_u8_x3\0"
  "q6dq48Uc\0"
  "reinterpret_f64_mf8_x3\0"
  "q6dq48m\0"
  "reinterpret_f64_s16_x3\0"
  "q6dq24s\0"
  "reinterpret_f64_u16_x3\0"
  "q6dq24Us\0"
  "reinterpret_f64_s32_x3\0"
  "q6dq12i\0"
  "reinterpret_f64_u32_x3\0"
  "q6dq12Ui\0"
  "reinterpret_f64_s64_x3\0"
  "q6dq6Wi\0"
  "reinterpret_f64_u64_x3\0"
  "q6dq6UWi\0"
  "reinterpret_f64_f16_x3\0"
  "q6dq24h\0"
  "reinterpret_f64_bf16_x3\0"
  "q6dq24y\0"
  "reinterpret_f64_f32_x3\0"
  "q6dq12f\0"
  "reinterpret_f64_f64_x3\0"
  "q6dq6d\0"
  "reinterpret_s8_s8_x4\0"
  "reinterpret_s8_u8_x4\0"
  "q64Scq64Uc\0"
  "reinterpret_s8_mf8_x4\0"
  "q64Scq64m\0"
  "reinterpret_s8_s16_x4\0"
  "q64Scq32s\0"
  "reinterpret_s8_u16_x4\0"
  "q64Scq32Us\0"
  "reinterpret_s8_s32_x4\0"
  "q64Scq16i\0"
  "reinterpret_s8_u32_x4\0"
  "q64Scq16Ui\0"
  "reinterpret_s8_s64_x4\0"
  "q64Scq8Wi\0"
  "reinterpret_s8_u64_x4\0"
  "q64Scq8UWi\0"
  "reinterpret_s8_f16_x4\0"
  "q64Scq32h\0"
  "reinterpret_s8_bf16_x4\0"
  "q64Scq32y\0"
  "reinterpret_s8_f32_x4\0"
  "q64Scq16f\0"
  "reinterpret_s8_f64_x4\0"
  "q64Scq8d\0"
  "reinterpret_u8_s8_x4\0"
  "q64Ucq64Sc\0"
  "reinterpret_u8_u8_x4\0"
  "reinterpret_u8_mf8_x4\0"
  "q64Ucq64m\0"
  "reinterpret_u8_s16_x4\0"
  "q64Ucq32s\0"
  "reinterpret_u8_u16_x4\0"
  "q64Ucq32Us\0"
  "reinterpret_u8_s32_x4\0"
  "q64Ucq16i\0"
  "reinterpret_u8_u32_x4\0"
  "q64Ucq16Ui\0"
  "reinterpret_u8_s64_x4\0"
  "q64Ucq8Wi\0"
  "reinterpret_u8_u64_x4\0"
  "q64Ucq8UWi\0"
  "reinterpret_u8_f16_x4\0"
  "q64Ucq32h\0"
  "reinterpret_u8_bf16_x4\0"
  "q64Ucq32y\0"
  "reinterpret_u8_f32_x4\0"
  "q64Ucq16f\0"
  "reinterpret_u8_f64_x4\0"
  "q64Ucq8d\0"
  "reinterpret_mf8_s8_x4\0"
  "q64mq64Sc\0"
  "reinterpret_mf8_u8_x4\0"
  "q64mq64Uc\0"
  "reinterpret_mf8_mf8_x4\0"
  "reinterpret_mf8_s16_x4\0"
  "q64mq32s\0"
  "reinterpret_mf8_u16_x4\0"
  "q64mq32Us\0"
  "reinterpret_mf8_s32_x4\0"
  "q64mq16i\0"
  "reinterpret_mf8_u32_x4\0"
  "q64mq16Ui\0"
  "reinterpret_mf8_s64_x4\0"
  "q64mq8Wi\0"
  "reinterpret_mf8_u64_x4\0"
  "q64mq8UWi\0"
  "reinterpret_mf8_f16_x4\0"
  "q64mq32h\0"
  "reinterpret_mf8_bf16_x4\0"
  "q64mq32y\0"
  "reinterpret_mf8_f32_x4\0"
  "q64mq16f\0"
  "reinterpret_mf8_f64_x4\0"
  "q64mq8d\0"
  "reinterpret_s16_s8_x4\0"
  "q32sq64Sc\0"
  "reinterpret_s16_u8_x4\0"
  "q32sq64Uc\0"
  "reinterpret_s16_mf8_x4\0"
  "q32sq64m\0"
  "reinterpret_s16_s16_x4\0"
  "reinterpret_s16_u16_x4\0"
  "q32sq32Us\0"
  "reinterpret_s16_s32_x4\0"
  "q32sq16i\0"
  "reinterpret_s16_u32_x4\0"
  "q32sq16Ui\0"
  "reinterpret_s16_s64_x4\0"
  "q32sq8Wi\0"
  "reinterpret_s16_u64_x4\0"
  "q32sq8UWi\0"
  "reinterpret_s16_f16_x4\0"
  "q32sq32h\0"
  "reinterpret_s16_bf16_x4\0"
  "q32sq32y\0"
  "reinterpret_s16_f32_x4\0"
  "q32sq16f\0"
  "reinterpret_s16_f64_x4\0"
  "q32sq8d\0"
  "reinterpret_u16_s8_x4\0"
  "q32Usq64Sc\0"
  "reinterpret_u16_u8_x4\0"
  "q32Usq64Uc\0"
  "reinterpret_u16_mf8_x4\0"
  "q32Usq64m\0"
  "reinterpret_u16_s16_x4\0"
  "q32Usq32s\0"
  "reinterpret_u16_u16_x4\0"
  "reinterpret_u16_s32_x4\0"
  "q32Usq16i\0"
  "reinterpret_u16_u32_x4\0"
  "q32Usq16Ui\0"
  "reinterpret_u16_s64_x4\0"
  "q32Usq8Wi\0"
  "reinterpret_u16_u64_x4\0"
  "q32Usq8UWi\0"
  "reinterpret_u16_f16_x4\0"
  "q32Usq32h\0"
  "reinterpret_u16_bf16_x4\0"
  "q32Usq32y\0"
  "reinterpret_u16_f32_x4\0"
  "q32Usq16f\0"
  "reinterpret_u16_f64_x4\0"
  "q32Usq8d\0"
  "reinterpret_s32_s8_x4\0"
  "q16iq64Sc\0"
  "reinterpret_s32_u8_x4\0"
  "q16iq64Uc\0"
  "reinterpret_s32_mf8_x4\0"
  "q16iq64m\0"
  "reinterpret_s32_s16_x4\0"
  "q16iq32s\0"
  "reinterpret_s32_u16_x4\0"
  "q16iq32Us\0"
  "reinterpret_s32_s32_x4\0"
  "reinterpret_s32_u32_x4\0"
  "q16iq16Ui\0"
  "reinterpret_s32_s64_x4\0"
  "q16iq8Wi\0"
  "reinterpret_s32_u64_x4\0"
  "q16iq8UWi\0"
  "reinterpret_s32_f16_x4\0"
  "q16iq32h\0"
  "reinterpret_s32_bf16_x4\0"
  "q16iq32y\0"
  "reinterpret_s32_f32_x4\0"
  "reinterpret_s32_f64_x4\0"
  "q16iq8d\0"
  "reinterpret_u32_s8_x4\0"
  "q16Uiq64Sc\0"
  "reinterpret_u32_u8_x4\0"
  "q16Uiq64Uc\0"
  "reinterpret_u32_mf8_x4\0"
  "q16Uiq64m\0"
  "reinterpret_u32_s16_x4\0"
  "q16Uiq32s\0"
  "reinterpret_u32_u16_x4\0"
  "q16Uiq32Us\0"
  "reinterpret_u32_s32_x4\0"
  "q16Uiq16i\0"
  "reinterpret_u32_u32_x4\0"
  "reinterpret_u32_s64_x4\0"
  "q16Uiq8Wi\0"
  "reinterpret_u32_u64_x4\0"
  "q16Uiq8UWi\0"
  "reinterpret_u32_f16_x4\0"
  "q16Uiq32h\0"
  "reinterpret_u32_bf16_x4\0"
  "q16Uiq32y\0"
  "reinterpret_u32_f32_x4\0"
  "reinterpret_u32_f64_x4\0"
  "q16Uiq8d\0"
  "reinterpret_s64_s8_x4\0"
  "q8Wiq64Sc\0"
  "reinterpret_s64_u8_x4\0"
  "q8Wiq64Uc\0"
  "reinterpret_s64_mf8_x4\0"
  "q8Wiq64m\0"
  "reinterpret_s64_s16_x4\0"
  "q8Wiq32s\0"
  "reinterpret_s64_u16_x4\0"
  "q8Wiq32Us\0"
  "reinterpret_s64_s32_x4\0"
  "q8Wiq16i\0"
  "reinterpret_s64_u32_x4\0"
  "q8Wiq16Ui\0"
  "reinterpret_s64_s64_x4\0"
  "reinterpret_s64_u64_x4\0"
  "q8Wiq8UWi\0"
  "reinterpret_s64_f16_x4\0"
  "q8Wiq32h\0"
  "reinterpret_s64_bf16_x4\0"
  "q8Wiq32y\0"
  "reinterpret_s64_f32_x4\0"
  "q8Wiq16f\0"
  "reinterpret_s64_f64_x4\0"
  "q8Wiq8d\0"
  "reinterpret_u64_s8_x4\0"
  "q8UWiq64Sc\0"
  "reinterpret_u64_u8_x4\0"
  "q8UWiq64Uc\0"
  "reinterpret_u64_mf8_x4\0"
  "q8UWiq64m\0"
  "reinterpret_u64_s16_x4\0"
  "q8UWiq32s\0"
  "reinterpret_u64_u16_x4\0"
  "q8UWiq32Us\0"
  "reinterpret_u64_s32_x4\0"
  "q8UWiq16i\0"
  "reinterpret_u64_u32_x4\0"
  "q8UWiq16Ui\0"
  "reinterpret_u64_s64_x4\0"
  "q8UWiq8Wi\0"
  "reinterpret_u64_u64_x4\0"
  "reinterpret_u64_f16_x4\0"
  "q8UWiq32h\0"
  "reinterpret_u64_bf16_x4\0"
  "q8UWiq32y\0"
  "reinterpret_u64_f32_x4\0"
  "q8UWiq16f\0"
  "reinterpret_u64_f64_x4\0"
  "q8UWiq8d\0"
  "reinterpret_f16_s8_x4\0"
  "q32hq64Sc\0"
  "reinterpret_f16_u8_x4\0"
  "q32hq64Uc\0"
  "reinterpret_f16_mf8_x4\0"
  "q32hq64m\0"
  "reinterpret_f16_s16_x4\0"
  "q32hq32s\0"
  "reinterpret_f16_u16_x4\0"
  "q32hq32Us\0"
  "reinterpret_f16_s32_x4\0"
  "q32hq16i\0"
  "reinterpret_f16_u32_x4\0"
  "q32hq16Ui\0"
  "reinterpret_f16_s64_x4\0"
  "q32hq8Wi\0"
  "reinterpret_f16_u64_x4\0"
  "q32hq8UWi\0"
  "reinterpret_f16_f16_x4\0"
  "reinterpret_f16_bf16_x4\0"
  "q32hq32y\0"
  "reinterpret_f16_f32_x4\0"
  "q32hq16f\0"
  "reinterpret_f16_f64_x4\0"
  "q32hq8d\0"
  "reinterpret_bf16_s8_x4\0"
  "q32yq64Sc\0"
  "reinterpret_bf16_u8_x4\0"
  "q32yq64Uc\0"
  "reinterpret_bf16_mf8_x4\0"
  "q32yq64m\0"
  "reinterpret_bf16_s16_x4\0"
  "q32yq32s\0"
  "reinterpret_bf16_u16_x4\0"
  "q32yq32Us\0"
  "reinterpret_bf16_s32_x4\0"
  "q32yq16i\0"
  "reinterpret_bf16_u32_x4\0"
  "q32yq16Ui\0"
  "reinterpret_bf16_s64_x4\0"
  "q32yq8Wi\0"
  "reinterpret_bf16_u64_x4\0"
  "q32yq8UWi\0"
  "reinterpret_bf16_f16_x4\0"
  "q32yq32h\0"
  "reinterpret_bf16_bf16_x4\0"
  "reinterpret_bf16_f32_x4\0"
  "q32yq16f\0"
  "reinterpret_bf16_f64_x4\0"
  "q32yq8d\0"
  "reinterpret_f32_s8_x4\0"
  "q16fq64Sc\0"
  "reinterpret_f32_u8_x4\0"
  "q16fq64Uc\0"
  "reinterpret_f32_mf8_x4\0"
  "q16fq64m\0"
  "reinterpret_f32_s16_x4\0"
  "q16fq32s\0"
  "reinterpret_f32_u16_x4\0"
  "q16fq32Us\0"
  "reinterpret_f32_s32_x4\0"
  "reinterpret_f32_u32_x4\0"
  "reinterpret_f32_s64_x4\0"
  "q16fq8Wi\0"
  "reinterpret_f32_u64_x4\0"
  "q16fq8UWi\0"
  "reinterpret_f32_f16_x4\0"
  "q16fq32h\0"
  "reinterpret_f32_bf16_x4\0"
  "q16fq32y\0"
  "reinterpret_f32_f32_x4\0"
  "reinterpret_f32_f64_x4\0"
  "q16fq8d\0"
  "reinterpret_f64_s8_x4\0"
  "q8dq64Sc\0"
  "reinterpret_f64_u8_x4\0"
  "q8dq64Uc\0"
  "reinterpret_f64_mf8_x4\0"
  "q8dq64m\0"
  "reinterpret_f64_s16_x4\0"
  "q8dq32s\0"
  "reinterpret_f64_u16_x4\0"
  "q8dq32Us\0"
  "reinterpret_f64_s32_x4\0"
  "q8dq16i\0"
  "reinterpret_f64_u32_x4\0"
  "q8dq16Ui\0"
  "reinterpret_f64_s64_x4\0"
  "q8dq8Wi\0"
  "reinterpret_f64_u64_x4\0"
  "q8dq8UWi\0"
  "reinterpret_f64_f16_x4\0"
  "q8dq32h\0"
  "reinterpret_f64_bf16_x4\0"
  "q8dq32y\0"
  "reinterpret_f64_f32_x4\0"
  "q8dq16f\0"
  "reinterpret_f64_f64_x4\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinStrings = BuiltinStringsStorage;
#endif // GET_SVE_BUILTIN_STR_TABLE

#ifdef GET_SVE_BUILTIN_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{3 /* svaba_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47 /* svaba_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70 /* svaba_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97 /* svaba_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126 /* svaba_n_u16 */, 138 /* q8Usq8Usq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153 /* svaba_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{180 /* svaba_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{211 /* svaba_n_u8 */, 222 /* q16Ucq16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{240 /* svaba_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{263 /* svaba_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{286 /* svaba_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{313 /* svaba_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{343 /* svaba_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{370 /* svaba_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{397 /* svaba_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{428 /* svaba_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{458 /* svabalb_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{486 /* svabalb_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{511 /* svabalb_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{538 /* svabalb_n_u16 */, 552 /* q8Usq8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{568 /* svabalb_n_u32 */, 582 /* q4Uiq4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{597 /* svabalb_n_u64 */, 611 /* q2UWiq2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{628 /* svabalb_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{657 /* svabalb_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{682 /* svabalb_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{709 /* svabalb_u16 */, 721 /* q8Usq8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{740 /* svabalb_u32 */, 752 /* q4Uiq4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{769 /* svabalb_u64 */, 781 /* q2UWiq2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{800 /* svabalt_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{814 /* svabalt_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{828 /* svabalt_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{842 /* svabalt_n_u16 */, 552 /* q8Usq8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{856 /* svabalt_n_u32 */, 582 /* q4Uiq4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{870 /* svabalt_n_u64 */, 611 /* q2UWiq2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{884 /* svabalt_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{896 /* svabalt_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{908 /* svabalt_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{920 /* svabalt_u16 */, 721 /* q8Usq8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{932 /* svabalt_u32 */, 752 /* q4Uiq4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{944 /* svabalt_u64 */, 781 /* q2UWiq2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{956 /* svabd_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{990 /* svabd_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1002 /* svabd_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1014 /* svabd_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1040 /* svabd_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1052 /* svabd_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1064 /* svabd_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1090 /* svabd_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1102 /* svabd_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1114 /* svabd_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1140 /* svabd_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1154 /* svabd_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1168 /* svabd_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1194 /* svabd_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1208 /* svabd_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1222 /* svabd_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1248 /* svabd_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1262 /* svabd_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1276 /* svabd_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1302 /* svabd_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1316 /* svabd_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1330 /* svabd_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1356 /* svabd_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1370 /* svabd_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1384 /* svabd_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1413 /* svabd_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1427 /* svabd_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1441 /* svabd_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1471 /* svabd_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1484 /* svabd_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1497 /* svabd_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1526 /* svabd_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1540 /* svabd_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1554 /* svabd_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1583 /* svabd_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1597 /* svabd_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1611 /* svabd_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1643 /* svabd_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1657 /* svabd_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1671 /* svabd_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1701 /* svabd_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1714 /* svabd_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1727 /* svabd_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1753 /* svabd_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1765 /* svabd_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1777 /* svabd_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1803 /* svabd_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1815 /* svabd_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1827 /* svabd_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1856 /* svabd_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1868 /* svabd_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1880 /* svabd_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1911 /* svabd_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1922 /* svabd_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1933 /* svabd_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1962 /* svabd_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1974 /* svabd_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1986 /* svabd_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2015 /* svabd_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2027 /* svabd_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2039 /* svabd_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2071 /* svabd_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2083 /* svabd_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2095 /* svabd_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2126 /* svabd_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2137 /* svabd_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2148 /* svabdlb_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2173 /* svabdlb_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2195 /* svabdlb_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2218 /* svabdlb_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2244 /* svabdlb_n_u32 */, 2258 /* q4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2269 /* svabdlb_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2295 /* svabdlb_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2321 /* svabdlb_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2343 /* svabdlb_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2366 /* svabdlb_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2393 /* svabdlb_u32 */, 2405 /* q4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2418 /* svabdlb_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2444 /* svabdlt_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2458 /* svabdlt_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2472 /* svabdlt_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2486 /* svabdlt_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2500 /* svabdlt_n_u32 */, 2258 /* q4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2514 /* svabdlt_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2528 /* svabdlt_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2540 /* svabdlt_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2552 /* svabdlt_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2564 /* svabdlt_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2576 /* svabdlt_u32 */, 2405 /* q4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2588 /* svabdlt_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2600 /* svabs_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2626 /* svabs_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2649 /* svabs_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2661 /* svabs_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2687 /* svabs_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2710 /* svabs_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2722 /* svabs_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2748 /* svabs_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2771 /* svabs_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2783 /* svabs_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2809 /* svabs_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2832 /* svabs_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2844 /* svabs_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2870 /* svabs_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2893 /* svabs_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2905 /* svabs_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2934 /* svabs_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2959 /* svabs_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2971 /* svabs_s8_m */, 2982 /* q16Scq16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3002 /* svabs_s8_x */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3028 /* svabs_s8_z */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3039 /* svacge_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3065 /* svacge_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3091 /* svacge_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3117 /* svacge_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3143 /* svacge_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3169 /* svacge_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3195 /* svacgt_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3206 /* svacgt_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3217 /* svacgt_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3228 /* svacgt_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3241 /* svacgt_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3254 /* svacgt_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3267 /* svacle_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3278 /* svacle_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3289 /* svacle_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3300 /* svacle_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3313 /* svacle_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3326 /* svacle_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3339 /* svaclt_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3350 /* svaclt_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3361 /* svaclt_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3372 /* svaclt_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3385 /* svaclt_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3398 /* svaclt_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3411 /* svadalp_s16_m */, 3425 /* q8sq16bq8sq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3441 /* svadalp_s16_x */, 3425 /* q8sq16bq8sq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3455 /* svadalp_s16_z */, 3425 /* q8sq16bq8sq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3469 /* svadalp_s32_m */, 3483 /* q4iq16bq4iq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3497 /* svadalp_s32_x */, 3483 /* q4iq16bq4iq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3511 /* svadalp_s32_z */, 3483 /* q4iq16bq4iq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3525 /* svadalp_s64_m */, 3539 /* q2Wiq16bq2Wiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3555 /* svadalp_s64_x */, 3539 /* q2Wiq16bq2Wiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3569 /* svadalp_s64_z */, 3539 /* q2Wiq16bq2Wiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3583 /* svadalp_u16_m */, 3597 /* q8Usq16bq8Usq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3615 /* svadalp_u16_x */, 3597 /* q8Usq16bq8Usq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3629 /* svadalp_u16_z */, 3597 /* q8Usq16bq8Usq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3643 /* svadalp_u32_m */, 3657 /* q4Uiq16bq4Uiq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3674 /* svadalp_u32_x */, 3657 /* q4Uiq16bq4Uiq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3688 /* svadalp_u32_z */, 3657 /* q4Uiq16bq4Uiq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3702 /* svadalp_u64_m */, 3716 /* q2UWiq16bq2UWiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3735 /* svadalp_u64_x */, 3716 /* q2UWiq16bq2UWiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3749 /* svadalp_u64_z */, 3716 /* q2UWiq16bq2UWiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3763 /* svadclb_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3777 /* svadclb_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3791 /* svadclb_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3803 /* svadclb_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3815 /* svadclt_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3829 /* svadclt_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3843 /* svadclt_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3855 /* svadclt_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3867 /* svadd_bf16_m */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3933 /* svadd_bf16_x */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3946 /* svadd_bf16_z */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3959 /* svadd_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3971 /* svadd_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3983 /* svadd_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3995 /* svadd_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4007 /* svadd_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4019 /* svadd_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4031 /* svadd_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4043 /* svadd_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4055 /* svadd_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4067 /* svadd_n_bf16_m */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4094 /* svadd_n_bf16_x */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4109 /* svadd_n_bf16_z */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4124 /* svadd_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4138 /* svadd_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4152 /* svadd_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4166 /* svadd_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4180 /* svadd_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4194 /* svadd_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4208 /* svadd_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4222 /* svadd_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4236 /* svadd_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4250 /* svadd_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4264 /* svadd_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4278 /* svadd_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4292 /* svadd_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4306 /* svadd_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4320 /* svadd_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4334 /* svadd_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4348 /* svadd_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4362 /* svadd_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4376 /* svadd_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4389 /* svadd_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4402 /* svadd_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4415 /* svadd_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4429 /* svadd_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4443 /* svadd_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4457 /* svadd_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4471 /* svadd_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4485 /* svadd_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4499 /* svadd_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4513 /* svadd_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4527 /* svadd_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4541 /* svadd_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4554 /* svadd_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4567 /* svadd_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4580 /* svadd_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4592 /* svadd_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4604 /* svadd_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4616 /* svadd_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4628 /* svadd_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4640 /* svadd_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4652 /* svadd_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4664 /* svadd_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4676 /* svadd_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4688 /* svadd_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4699 /* svadd_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4710 /* svadd_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4721 /* svadd_single_s16_x2 */, 4741 /* q16sq16sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4762 /* svadd_single_s16_x4 */, 4782 /* q32sq32sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4794 /* svadd_single_s32_x2 */, 4814 /* q8iq8iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4824 /* svadd_single_s32_x4 */, 4844 /* q16iq16iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4856 /* svadd_single_s64_x2 */, 4876 /* q4Wiq4Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4889 /* svadd_single_s64_x4 */, 4909 /* q8Wiq8Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4922 /* svadd_single_s8_x2 */, 4941 /* q32Scq32Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4957 /* svadd_single_s8_x4 */, 4976 /* q64Scq64Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4992 /* svadd_single_u16_x2 */, 5012 /* q16Usq16Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5027 /* svadd_single_u16_x4 */, 5047 /* q32Usq32Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5062 /* svadd_single_u32_x2 */, 5082 /* q8Uiq8Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5095 /* svadd_single_u32_x4 */, 5115 /* q16Uiq16Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5130 /* svadd_single_u64_x2 */, 5150 /* q4UWiq4UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5166 /* svadd_single_u64_x4 */, 5186 /* q8UWiq8UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5202 /* svadd_single_u8_x2 */, 5221 /* q32Ucq32Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5237 /* svadd_single_u8_x4 */, 5256 /* q64Ucq64Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5272 /* svadd_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5284 /* svadd_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5296 /* svadd_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5308 /* svadd_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5320 /* svadd_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5332 /* svadd_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5344 /* svadd_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5356 /* svadd_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5368 /* svadd_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5380 /* svadd_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5391 /* svadd_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5402 /* svadd_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5413 /* svadda_f16 */, 5424 /* hq16bhq8h */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5438 /* svadda_f32 */, 5449 /* fq16bfq4f */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5459 /* svadda_f64 */, 5470 /* dq16bdq2d */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5480 /* svaddhnb_n_s16 */, 5495 /* q16Scq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5505 /* svaddhnb_n_s32 */, 5520 /* q8sq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5528 /* svaddhnb_n_s64 */, 5543 /* q4iq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5553 /* svaddhnb_n_u16 */, 5568 /* q16Ucq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5580 /* svaddhnb_n_u32 */, 5595 /* q8Usq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5606 /* svaddhnb_n_u64 */, 5621 /* q4Uiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5634 /* svaddhnb_s16 */, 5647 /* q16Scq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5659 /* svaddhnb_s32 */, 5672 /* q8sq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5682 /* svaddhnb_s64 */, 5695 /* q4iq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5707 /* svaddhnb_u16 */, 5720 /* q16Ucq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5734 /* svaddhnb_u32 */, 5747 /* q8Usq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5760 /* svaddhnb_u64 */, 5773 /* q4Uiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5788 /* svaddhnt_n_s16 */, 5803 /* q16Scq16Scq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5818 /* svaddhnt_n_s32 */, 5833 /* q8sq8sq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5844 /* svaddhnt_n_s64 */, 5859 /* q4iq4iq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5872 /* svaddhnt_n_u16 */, 5887 /* q16Ucq16Ucq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5904 /* svaddhnt_n_u32 */, 5919 /* q8Usq8Usq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5934 /* svaddhnt_n_u64 */, 5949 /* q4Uiq4Uiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5966 /* svaddhnt_s16 */, 5979 /* q16Scq16Scq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5996 /* svaddhnt_s32 */, 6009 /* q8sq8sq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6022 /* svaddhnt_s64 */, 6035 /* q4iq4iq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6050 /* svaddhnt_u16 */, 6063 /* q16Ucq16Ucq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6082 /* svaddhnt_u32 */, 6095 /* q8Usq8Usq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6112 /* svaddhnt_u64 */, 6125 /* q4Uiq4Uiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6144 /* svaddlb_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6158 /* svaddlb_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6172 /* svaddlb_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6186 /* svaddlb_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6200 /* svaddlb_n_u32 */, 2258 /* q4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6214 /* svaddlb_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6228 /* svaddlb_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6240 /* svaddlb_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6252 /* svaddlb_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6264 /* svaddlb_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6276 /* svaddlb_u32 */, 2405 /* q4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6288 /* svaddlb_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6300 /* svaddlbt_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6315 /* svaddlbt_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6330 /* svaddlbt_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6345 /* svaddlbt_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6358 /* svaddlbt_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6371 /* svaddlbt_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6384 /* svaddlt_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6398 /* svaddlt_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6412 /* svaddlt_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6426 /* svaddlt_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6440 /* svaddlt_n_u32 */, 2258 /* q4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6454 /* svaddlt_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6468 /* svaddlt_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6480 /* svaddlt_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6492 /* svaddlt_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6504 /* svaddlt_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6516 /* svaddlt_u32 */, 2405 /* q4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6528 /* svaddlt_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6540 /* svaddp_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6553 /* svaddp_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6566 /* svaddp_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6579 /* svaddp_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6592 /* svaddp_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6605 /* svaddp_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6618 /* svaddp_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6631 /* svaddp_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6644 /* svaddp_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6657 /* svaddp_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6670 /* svaddp_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6683 /* svaddp_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6696 /* svaddp_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6708 /* svaddp_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6720 /* svaddp_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6733 /* svaddp_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6746 /* svaddp_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6759 /* svaddp_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6772 /* svaddp_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6785 /* svaddp_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6798 /* svaddp_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6810 /* svaddp_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6822 /* svaddqv_f16 */, 6834 /* V8hq16bq8h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6889 /* svaddqv_f32 */, 6901 /* V4fq16bq4f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6912 /* svaddqv_f64 */, 6924 /* V2dq16bq2d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6935 /* svaddqv_s16 */, 6947 /* V8sq16bq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6958 /* svaddqv_s32 */, 6970 /* V4iq16bq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6981 /* svaddqv_s64 */, 6993 /* V2Wiq16bq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7006 /* svaddqv_s8 */, 7017 /* V16Scq16bq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7032 /* svaddqv_u16 */, 7044 /* V8Usq16bq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7057 /* svaddqv_u32 */, 7069 /* V4Uiq16bq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7082 /* svaddqv_u64 */, 7094 /* V2UWiq16bq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7109 /* svaddqv_u8 */, 7120 /* V16Ucq16bq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7135 /* svaddv_f16 */, 7146 /* hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7155 /* svaddv_f32 */, 7166 /* fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7175 /* svaddv_f64 */, 7186 /* dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7195 /* svaddv_s16 */, 7206 /* Wiq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7216 /* svaddv_s32 */, 7227 /* Wiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7237 /* svaddv_s64 */, 7248 /* Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7259 /* svaddv_s8 */, 7269 /* Wiq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7281 /* svaddv_u16 */, 7292 /* UWiq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7304 /* svaddv_u32 */, 7315 /* UWiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7327 /* svaddv_u64 */, 7338 /* UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7351 /* svaddv_u8 */, 7361 /* UWiq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7374 /* svaddwb_n_s16 */, 7388 /* q8sq8sSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7397 /* svaddwb_n_s32 */, 7411 /* q4iq4is */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7419 /* svaddwb_n_s64 */, 7433 /* q2Wiq2Wii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7443 /* svaddwb_n_u16 */, 7457 /* q8Usq8UsUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7468 /* svaddwb_n_u32 */, 7482 /* q4Uiq4UiUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7493 /* svaddwb_n_u64 */, 7507 /* q2UWiq2UWiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7520 /* svaddwb_s16 */, 7532 /* q8sq8sq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7544 /* svaddwb_s32 */, 7556 /* q4iq4iq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7566 /* svaddwb_s64 */, 7578 /* q2Wiq2Wiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7590 /* svaddwb_u16 */, 7602 /* q8Usq8Usq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7616 /* svaddwb_u32 */, 7628 /* q4Uiq4Uiq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7641 /* svaddwb_u64 */, 7653 /* q2UWiq2UWiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7668 /* svaddwt_n_s16 */, 7388 /* q8sq8sSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7682 /* svaddwt_n_s32 */, 7411 /* q4iq4is */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7696 /* svaddwt_n_s64 */, 7433 /* q2Wiq2Wii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7710 /* svaddwt_n_u16 */, 7457 /* q8Usq8UsUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7724 /* svaddwt_n_u32 */, 7482 /* q4Uiq4UiUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7738 /* svaddwt_n_u64 */, 7507 /* q2UWiq2UWiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7752 /* svaddwt_s16 */, 7532 /* q8sq8sq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7764 /* svaddwt_s32 */, 7556 /* q4iq4iq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7776 /* svaddwt_s64 */, 7578 /* q2Wiq2Wiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7788 /* svaddwt_u16 */, 7602 /* q8Usq8Usq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7800 /* svaddwt_u32 */, 7628 /* q4Uiq4Uiq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7812 /* svaddwt_u64 */, 7653 /* q2UWiq2UWiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7824 /* svadrb_u32base_s32offset */, 7849 /* q4Uiq4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7861 /* svadrb_u32base_u32offset */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7899 /* svadrb_u64base_s64offset */, 7924 /* q2UWiq2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7939 /* svadrb_u64base_u64offset */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7980 /* svadrd_u32base_s32index */, 7849 /* q4Uiq4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8004 /* svadrd_u32base_u32index */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8028 /* svadrd_u64base_s64index */, 7924 /* q2UWiq2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8052 /* svadrd_u64base_u64index */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8076 /* svadrh_u32base_s32index */, 7849 /* q4Uiq4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8100 /* svadrh_u32base_u32index */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8124 /* svadrh_u64base_s64index */, 7924 /* q2UWiq2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8148 /* svadrh_u64base_u64index */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8172 /* svadrw_u32base_s32index */, 7849 /* q4Uiq4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8196 /* svadrw_u32base_u32index */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8220 /* svadrw_u64base_s64index */, 7924 /* q2UWiq2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8244 /* svadrw_u64base_u64index */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8268 /* svaesd_lane_u8_x2 */, 8286 /* q32Ucq32Ucq16UcIUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8345 /* svaesd_lane_u8_x4 */, 8363 /* q64Ucq64Ucq16UcIUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8383 /* svaesd_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 8409 /* (sve,sve-aes)|(sme,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8438 /* svaesdimc_lane_u8_x2 */, 8286 /* q32Ucq32Ucq16UcIUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8459 /* svaesdimc_lane_u8_x4 */, 8363 /* q64Ucq64Ucq16UcIUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8480 /* svaese_lane_u8_x2 */, 8286 /* q32Ucq32Ucq16UcIUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8498 /* svaese_lane_u8_x4 */, 8363 /* q64Ucq64Ucq16UcIUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8516 /* svaese_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 8409 /* (sve,sve-aes)|(sme,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8526 /* svaesemc_lane_u8_x2 */, 8286 /* q32Ucq32Ucq16UcIUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8546 /* svaesemc_lane_u8_x4 */, 8363 /* q64Ucq64Ucq16UcIUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8566 /* svaesimc_u8 */, 8578 /* q16Ucq16Uc */, 1 /* n */, 8409 /* (sve,sve-aes)|(sme,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8589 /* svaesmc_u8 */, 8578 /* q16Ucq16Uc */, 1 /* n */, 8409 /* (sve,sve-aes)|(sme,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8600 /* svamax_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8660 /* svamax_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8673 /* svamax_f16_x2 */, 8687 /* q16hq16hq16h */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8718 /* svamax_f16_x4 */, 8732 /* q32hq32hq32h */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8745 /* svamax_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8758 /* svamax_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8771 /* svamax_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8784 /* svamax_f32_x2 */, 8798 /* q8fq8fq8f */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8808 /* svamax_f32_x4 */, 8822 /* q16fq16fq16f */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8835 /* svamax_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8848 /* svamax_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8861 /* svamax_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8874 /* svamax_f64_x2 */, 8888 /* q4dq4dq4d */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8898 /* svamax_f64_x4 */, 8912 /* q8dq8dq8d */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8922 /* svamax_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8935 /* svamax_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8950 /* svamax_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8965 /* svamax_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8980 /* svamax_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8995 /* svamax_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9010 /* svamax_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9025 /* svamax_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9040 /* svamax_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9055 /* svamax_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9070 /* svamin_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9083 /* svamin_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9096 /* svamin_f16_x2 */, 8687 /* q16hq16hq16h */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9110 /* svamin_f16_x4 */, 8732 /* q32hq32hq32h */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9124 /* svamin_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9137 /* svamin_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9150 /* svamin_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9163 /* svamin_f32_x2 */, 8798 /* q8fq8fq8f */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9177 /* svamin_f32_x4 */, 8822 /* q16fq16fq16f */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9191 /* svamin_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9204 /* svamin_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9217 /* svamin_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9230 /* svamin_f64_x2 */, 8888 /* q4dq4dq4d */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9244 /* svamin_f64_x4 */, 8912 /* q8dq8dq8d */, 1 /* n */, 8700 /* sme,sme2,faminmax */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9258 /* svamin_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9271 /* svamin_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9286 /* svamin_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9301 /* svamin_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9316 /* svamin_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9331 /* svamin_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9346 /* svamin_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9361 /* svamin_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9376 /* svamin_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9391 /* svamin_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 8613 /* (sve,(sve2|sme2),faminmax)|(sme,sme2,faminmax) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9406 /* svand_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9433 /* svand_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9447 /* svand_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9461 /* svand_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9475 /* svand_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9489 /* svand_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9503 /* svand_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9517 /* svand_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9531 /* svand_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9545 /* svand_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9559 /* svand_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9572 /* svand_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9585 /* svand_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9598 /* svand_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9612 /* svand_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9626 /* svand_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9640 /* svand_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9654 /* svand_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9668 /* svand_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9682 /* svand_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9696 /* svand_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9710 /* svand_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9724 /* svand_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9737 /* svand_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9750 /* svand_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9763 /* svand_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9775 /* svand_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9787 /* svand_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9799 /* svand_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9811 /* svand_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9823 /* svand_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9835 /* svand_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9847 /* svand_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9859 /* svand_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9871 /* svand_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9882 /* svand_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9893 /* svand_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9904 /* svand_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9916 /* svand_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9928 /* svand_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9940 /* svand_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9952 /* svand_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9964 /* svand_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9976 /* svand_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9988 /* svand_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10000 /* svand_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10012 /* svand_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10023 /* svand_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10034 /* svand_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10045 /* svandqv_s16 */, 6947 /* V8sq16bq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10057 /* svandqv_s32 */, 6970 /* V4iq16bq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10069 /* svandqv_s64 */, 6993 /* V2Wiq16bq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10081 /* svandqv_s8 */, 7017 /* V16Scq16bq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10092 /* svandqv_u16 */, 7044 /* V8Usq16bq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10104 /* svandqv_u32 */, 7069 /* V4Uiq16bq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10116 /* svandqv_u64 */, 7094 /* V2UWiq16bq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10128 /* svandqv_u8 */, 7120 /* V16Ucq16bq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10139 /* svandv_s16 */, 10150 /* sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10159 /* svandv_s32 */, 10170 /* iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10179 /* svandv_s64 */, 7248 /* Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10190 /* svandv_s8 */, 10200 /* Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10212 /* svandv_u16 */, 10223 /* Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10234 /* svandv_u32 */, 10245 /* Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10256 /* svandv_u64 */, 7338 /* UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10267 /* svandv_u8 */, 10277 /* Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10289 /* svasr_n_s16_m */, 10303 /* q8sq16bq8sUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10316 /* svasr_n_s16_x */, 10303 /* q8sq16bq8sUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10330 /* svasr_n_s16_z */, 10303 /* q8sq16bq8sUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10344 /* svasr_n_s32_m */, 10358 /* q4iq16bq4iUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10371 /* svasr_n_s32_x */, 10358 /* q4iq16bq4iUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10385 /* svasr_n_s32_z */, 10358 /* q4iq16bq4iUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10399 /* svasr_n_s64_m */, 10413 /* q2Wiq16bq2WiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10429 /* svasr_n_s64_x */, 10413 /* q2Wiq16bq2WiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10443 /* svasr_n_s64_z */, 10413 /* q2Wiq16bq2WiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10457 /* svasr_n_s8_m */, 10470 /* q16Scq16bq16ScUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10487 /* svasr_n_s8_x */, 10470 /* q16Scq16bq16ScUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10500 /* svasr_n_s8_z */, 10470 /* q16Scq16bq16ScUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10513 /* svasr_s16_m */, 10525 /* q8sq16bq8sq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10540 /* svasr_s16_x */, 10525 /* q8sq16bq8sq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10552 /* svasr_s16_z */, 10525 /* q8sq16bq8sq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10564 /* svasr_s32_m */, 10576 /* q4iq16bq4iq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10591 /* svasr_s32_x */, 10576 /* q4iq16bq4iq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10603 /* svasr_s32_z */, 10576 /* q4iq16bq4iq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10615 /* svasr_s64_m */, 10627 /* q2Wiq16bq2Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10645 /* svasr_s64_x */, 10627 /* q2Wiq16bq2Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10657 /* svasr_s64_z */, 10627 /* q2Wiq16bq2Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10669 /* svasr_s8_m */, 10680 /* q16Scq16bq16Scq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10700 /* svasr_s8_x */, 10680 /* q16Scq16bq16Scq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10711 /* svasr_s8_z */, 10680 /* q16Scq16bq16Scq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10722 /* svasr_wide_n_s16_m */, 10741 /* q8sq16bq8sUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10755 /* svasr_wide_n_s16_x */, 10741 /* q8sq16bq8sUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10774 /* svasr_wide_n_s16_z */, 10741 /* q8sq16bq8sUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10793 /* svasr_wide_n_s32_m */, 10812 /* q4iq16bq4iUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10826 /* svasr_wide_n_s32_x */, 10812 /* q4iq16bq4iUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10845 /* svasr_wide_n_s32_z */, 10812 /* q4iq16bq4iUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10864 /* svasr_wide_n_s8_m */, 10882 /* q16Scq16bq16ScUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10900 /* svasr_wide_n_s8_x */, 10882 /* q16Scq16bq16ScUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10918 /* svasr_wide_n_s8_z */, 10882 /* q16Scq16bq16ScUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10936 /* svasr_wide_s16_m */, 10953 /* q8sq16bq8sq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10969 /* svasr_wide_s16_x */, 10953 /* q8sq16bq8sq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10986 /* svasr_wide_s16_z */, 10953 /* q8sq16bq8sq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11003 /* svasr_wide_s32_m */, 11020 /* q4iq16bq4iq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11036 /* svasr_wide_s32_x */, 11020 /* q4iq16bq4iq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11053 /* svasr_wide_s32_z */, 11020 /* q4iq16bq4iq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11070 /* svasr_wide_s8_m */, 11086 /* q16Scq16bq16Scq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11106 /* svasr_wide_s8_x */, 11086 /* q16Scq16bq16Scq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11122 /* svasr_wide_s8_z */, 11086 /* q16Scq16bq16Scq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11138 /* svasrd_n_s16_m */, 11153 /* q8sq16bq8sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11168 /* svasrd_n_s16_x */, 11153 /* q8sq16bq8sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11183 /* svasrd_n_s16_z */, 11153 /* q8sq16bq8sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11198 /* svasrd_n_s32_m */, 11213 /* q4iq16bq4iIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11228 /* svasrd_n_s32_x */, 11213 /* q4iq16bq4iIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11243 /* svasrd_n_s32_z */, 11213 /* q4iq16bq4iIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11258 /* svasrd_n_s64_m */, 11273 /* q2Wiq16bq2WiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11290 /* svasrd_n_s64_x */, 11273 /* q2Wiq16bq2WiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11305 /* svasrd_n_s64_z */, 11273 /* q2Wiq16bq2WiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11320 /* svasrd_n_s8_m */, 11334 /* q16Scq16bq16ScIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11353 /* svasrd_n_s8_x */, 11334 /* q16Scq16bq16ScIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11367 /* svasrd_n_s8_z */, 11334 /* q16Scq16bq16ScIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11381 /* svbcax_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11394 /* svbcax_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11407 /* svbcax_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11420 /* svbcax_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11432 /* svbcax_n_u16 */, 138 /* q8Usq8Usq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11445 /* svbcax_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11458 /* svbcax_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11471 /* svbcax_n_u8 */, 222 /* q16Ucq16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11483 /* svbcax_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11494 /* svbcax_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11505 /* svbcax_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11516 /* svbcax_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11526 /* svbcax_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11537 /* svbcax_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11548 /* svbcax_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11559 /* svbcax_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11569 /* svbdep_n_u16 */, 11582 /* q8Usq8UsUs */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11630 /* svbdep_n_u32 */, 11643 /* q4Uiq4UiUi */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11654 /* svbdep_n_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11681 /* svbdep_n_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11706 /* svbdep_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11730 /* svbdep_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11741 /* svbdep_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11752 /* svbdep_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11762 /* svbext_n_u16 */, 11582 /* q8Usq8UsUs */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11775 /* svbext_n_u32 */, 11643 /* q4Uiq4UiUi */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11788 /* svbext_n_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11801 /* svbext_n_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11813 /* svbext_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11824 /* svbext_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11835 /* svbext_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11846 /* svbext_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11856 /* svbfdot_f32 */, 11868 /* q4fq4fq8yq8y */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11903 /* svbfdot_lane_f32 */, 11920 /* q4fq4fq8yq8yIUWi */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11937 /* svbfdot_n_f32 */, 11951 /* q4fq4fq8yy */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11962 /* svbfmlalb_f32 */, 11868 /* q4fq4fq8yq8y */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11976 /* svbfmlalb_lane_f32 */, 11920 /* q4fq4fq8yq8yIUWi */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11995 /* svbfmlalb_n_f32 */, 11951 /* q4fq4fq8yy */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12011 /* svbfmlalt_f32 */, 11868 /* q4fq4fq8yq8y */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12025 /* svbfmlalt_lane_f32 */, 11920 /* q4fq4fq8yq8yIUWi */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12044 /* svbfmlalt_n_f32 */, 11951 /* q4fq4fq8yy */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12060 /* svbfmlslb_f32 */, 11868 /* q4fq4fq8yq8y */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12114 /* svbfmlslb_lane_f32 */, 11920 /* q4fq4fq8yq8yIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12133 /* svbfmlslt_f32 */, 11868 /* q4fq4fq8yq8y */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12147 /* svbfmlslt_lane_f32 */, 11920 /* q4fq4fq8yq8yIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12166 /* svbfmmla_f32 */, 11868 /* q4fq4fq8yq8y */, 1 /* n */, 12179 /* sve,bf16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12188 /* svbgrp_n_u16 */, 11582 /* q8Usq8UsUs */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12201 /* svbgrp_n_u32 */, 11643 /* q4Uiq4UiUi */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12214 /* svbgrp_n_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12227 /* svbgrp_n_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12239 /* svbgrp_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12250 /* svbgrp_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12261 /* svbgrp_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12272 /* svbgrp_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 11593 /* (sve,sve-bitperm)|(sme,ssve-bitperm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12282 /* svbic_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12292 /* svbic_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12306 /* svbic_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12320 /* svbic_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12334 /* svbic_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12348 /* svbic_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12362 /* svbic_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12376 /* svbic_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12390 /* svbic_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12404 /* svbic_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12418 /* svbic_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12431 /* svbic_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12444 /* svbic_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12457 /* svbic_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12471 /* svbic_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12485 /* svbic_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12499 /* svbic_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12513 /* svbic_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12527 /* svbic_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12541 /* svbic_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12555 /* svbic_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12569 /* svbic_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12583 /* svbic_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12596 /* svbic_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12609 /* svbic_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12622 /* svbic_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12634 /* svbic_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12646 /* svbic_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12658 /* svbic_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12670 /* svbic_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12682 /* svbic_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12694 /* svbic_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12706 /* svbic_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12718 /* svbic_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12730 /* svbic_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12741 /* svbic_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12752 /* svbic_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12763 /* svbic_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12775 /* svbic_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12787 /* svbic_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12799 /* svbic_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12811 /* svbic_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12823 /* svbic_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12835 /* svbic_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12847 /* svbic_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12859 /* svbic_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12871 /* svbic_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12882 /* svbic_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12893 /* svbic_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12904 /* svbrka_b_m */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12915 /* svbrka_b_z */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12939 /* svbrkb_b_m */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12950 /* svbrkb_b_z */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12961 /* svbrkn_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12972 /* svbrkpa_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12984 /* svbrkpb_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12996 /* svbsl1n_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13010 /* svbsl1n_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13024 /* svbsl1n_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13038 /* svbsl1n_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13051 /* svbsl1n_n_u16 */, 138 /* q8Usq8Usq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13065 /* svbsl1n_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13079 /* svbsl1n_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13093 /* svbsl1n_n_u8 */, 222 /* q16Ucq16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13106 /* svbsl1n_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13118 /* svbsl1n_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13130 /* svbsl1n_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13142 /* svbsl1n_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13153 /* svbsl1n_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13165 /* svbsl1n_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13177 /* svbsl1n_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13189 /* svbsl1n_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13200 /* svbsl2n_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13214 /* svbsl2n_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13228 /* svbsl2n_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13242 /* svbsl2n_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13255 /* svbsl2n_n_u16 */, 138 /* q8Usq8Usq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13269 /* svbsl2n_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13283 /* svbsl2n_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13297 /* svbsl2n_n_u8 */, 222 /* q16Ucq16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13310 /* svbsl2n_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13322 /* svbsl2n_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13334 /* svbsl2n_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13346 /* svbsl2n_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13357 /* svbsl2n_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13369 /* svbsl2n_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13381 /* svbsl2n_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13393 /* svbsl2n_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13404 /* svbsl_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13416 /* svbsl_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13428 /* svbsl_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13440 /* svbsl_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13451 /* svbsl_n_u16 */, 138 /* q8Usq8Usq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13463 /* svbsl_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13475 /* svbsl_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13487 /* svbsl_n_u8 */, 222 /* q16Ucq16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13498 /* svbsl_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13508 /* svbsl_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13518 /* svbsl_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13528 /* svbsl_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13537 /* svbsl_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13547 /* svbsl_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13557 /* svbsl_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13567 /* svbsl_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13576 /* svcadd_f16_m */, 13589 /* q8hq16bq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13607 /* svcadd_f16_x */, 13589 /* q8hq16bq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13620 /* svcadd_f16_z */, 13589 /* q8hq16bq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13633 /* svcadd_f32_m */, 13646 /* q4fq16bq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13664 /* svcadd_f32_x */, 13646 /* q4fq16bq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13677 /* svcadd_f32_z */, 13646 /* q4fq16bq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13690 /* svcadd_f64_m */, 13703 /* q2dq16bq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13721 /* svcadd_f64_x */, 13703 /* q2dq16bq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13734 /* svcadd_f64_z */, 13703 /* q2dq16bq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13747 /* svcadd_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13772 /* svcadd_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13797 /* svcadd_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13825 /* svcadd_s8 */, 13835 /* q16Scq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13855 /* svcadd_u16 */, 13866 /* q8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13883 /* svcadd_u32 */, 13894 /* q4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13911 /* svcadd_u64 */, 13922 /* q2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13942 /* svcadd_u8 */, 13952 /* q16Ucq16Ucq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13972 /* svcdot_lane_s32 */, 13988 /* q4iq4iq16Scq16ScIUWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14013 /* svcdot_lane_s64 */, 14029 /* q2Wiq2Wiq8sq8sIUWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14052 /* svcdot_s32 */, 14063 /* q4iq4iq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14084 /* svcdot_s64 */, 14095 /* q2Wiq2Wiq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14114 /* svclamp_bf16 */, 14127 /* q8yq8yq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14140 /* svclamp_f16 */, 14152 /* q8hq8hq8hq8h */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14165 /* svclamp_f32 */, 14177 /* q4fq4fq4fq4f */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14190 /* svclamp_f64 */, 14202 /* q2dq2dq2dq2d */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14215 /* svclamp_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14250 /* svclamp_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14262 /* svclamp_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14274 /* svclamp_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14285 /* svclamp_single_bf16_x2 */, 14308 /* q16yq16yq8yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14343 /* svclamp_single_bf16_x4 */, 14366 /* q32yq32yq8yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14381 /* svclamp_single_f16_x2 */, 14403 /* q16hq16hq8hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14418 /* svclamp_single_f16_x4 */, 14440 /* q32hq32hq8hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14455 /* svclamp_single_f32_x2 */, 14477 /* q8fq8fq4fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14490 /* svclamp_single_f32_x4 */, 14512 /* q16fq16fq4fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14527 /* svclamp_single_f64_x2 */, 14549 /* q4dq4dq2dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14562 /* svclamp_single_f64_x4 */, 14584 /* q8dq8dq2dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14597 /* svclamp_single_s16_x2 */, 14619 /* q16sq16sq8sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14634 /* svclamp_single_s16_x4 */, 14656 /* q32sq32sq8sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14671 /* svclamp_single_s32_x2 */, 14693 /* q8iq8iq4iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14706 /* svclamp_single_s32_x4 */, 14728 /* q16iq16iq4iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14743 /* svclamp_single_s64_x2 */, 14765 /* q4Wiq4Wiq2Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14782 /* svclamp_single_s64_x4 */, 14804 /* q8Wiq8Wiq2Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14821 /* svclamp_single_s8_x2 */, 14842 /* q32Scq32Scq16Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14863 /* svclamp_single_s8_x4 */, 14884 /* q64Scq64Scq16Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14905 /* svclamp_single_u16_x2 */, 14927 /* q16Usq16Usq8Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14946 /* svclamp_single_u16_x4 */, 14968 /* q32Usq32Usq8Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14987 /* svclamp_single_u32_x2 */, 15009 /* q8Uiq8Uiq4Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15026 /* svclamp_single_u32_x4 */, 15048 /* q16Uiq16Uiq4Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15067 /* svclamp_single_u64_x2 */, 15089 /* q4UWiq4UWiq2UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15110 /* svclamp_single_u64_x4 */, 15132 /* q8UWiq8UWiq2UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15153 /* svclamp_single_u8_x2 */, 15174 /* q32Ucq32Ucq16Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15195 /* svclamp_single_u8_x4 */, 15216 /* q64Ucq64Ucq16Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15237 /* svclamp_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15249 /* svclamp_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15261 /* svclamp_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15273 /* svclamp_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15284 /* svclasta_bf16 */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15298 /* svclasta_f16 */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15311 /* svclasta_f32 */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15324 /* svclasta_f64 */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15337 /* svclasta_n_bf16 */, 15353 /* yq16byq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15363 /* svclasta_n_f16 */, 5424 /* hq16bhq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15378 /* svclasta_n_f32 */, 5449 /* fq16bfq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15393 /* svclasta_n_f64 */, 5470 /* dq16bdq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15408 /* svclasta_n_s16 */, 15423 /* sq16bsq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15433 /* svclasta_n_s32 */, 15448 /* iq16biq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15458 /* svclasta_n_s64 */, 15473 /* Wiq16bWiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15486 /* svclasta_n_s8 */, 15500 /* Scq16bScq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15514 /* svclasta_n_u16 */, 15529 /* Usq16bUsq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15542 /* svclasta_n_u32 */, 15557 /* Uiq16bUiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15570 /* svclasta_n_u64 */, 15585 /* UWiq16bUWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15601 /* svclasta_n_u8 */, 15615 /* Ucq16bUcq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15629 /* svclasta_s16 */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15642 /* svclasta_s32 */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15655 /* svclasta_s64 */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15668 /* svclasta_s8 */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15680 /* svclasta_u16 */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15693 /* svclasta_u32 */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15706 /* svclasta_u64 */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15719 /* svclasta_u8 */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15731 /* svclastb_bf16 */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15745 /* svclastb_f16 */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15758 /* svclastb_f32 */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15771 /* svclastb_f64 */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15784 /* svclastb_n_bf16 */, 15353 /* yq16byq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15800 /* svclastb_n_f16 */, 5424 /* hq16bhq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15815 /* svclastb_n_f32 */, 5449 /* fq16bfq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15830 /* svclastb_n_f64 */, 5470 /* dq16bdq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15845 /* svclastb_n_s16 */, 15423 /* sq16bsq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15860 /* svclastb_n_s32 */, 15448 /* iq16biq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15875 /* svclastb_n_s64 */, 15473 /* Wiq16bWiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15890 /* svclastb_n_s8 */, 15500 /* Scq16bScq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15904 /* svclastb_n_u16 */, 15529 /* Usq16bUsq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15919 /* svclastb_n_u32 */, 15557 /* Uiq16bUiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15934 /* svclastb_n_u64 */, 15585 /* UWiq16bUWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15949 /* svclastb_n_u8 */, 15615 /* Ucq16bUcq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15963 /* svclastb_s16 */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15976 /* svclastb_s32 */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15989 /* svclastb_s64 */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16002 /* svclastb_s8 */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16014 /* svclastb_u16 */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16027 /* svclastb_u32 */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16040 /* svclastb_u64 */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16053 /* svclastb_u8 */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16065 /* svcls_s16_m */, 16077 /* q8Usq8Usq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16093 /* svcls_s16_x */, 16105 /* q8Usq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16117 /* svcls_s16_z */, 16105 /* q8Usq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16129 /* svcls_s32_m */, 16141 /* q4Uiq4Uiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16157 /* svcls_s32_x */, 16169 /* q4Uiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16181 /* svcls_s32_z */, 16169 /* q4Uiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16193 /* svcls_s64_m */, 16205 /* q2UWiq2UWiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16224 /* svcls_s64_x */, 16236 /* q2UWiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16250 /* svcls_s64_z */, 16236 /* q2UWiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16262 /* svcls_s8_m */, 16273 /* q16Ucq16Ucq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16293 /* svcls_s8_x */, 16304 /* q16Ucq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16319 /* svcls_s8_z */, 16304 /* q16Ucq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16330 /* svclz_s16_m */, 16077 /* q8Usq8Usq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16342 /* svclz_s16_x */, 16105 /* q8Usq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16354 /* svclz_s16_z */, 16105 /* q8Usq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16366 /* svclz_s32_m */, 16141 /* q4Uiq4Uiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16378 /* svclz_s32_x */, 16169 /* q4Uiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16390 /* svclz_s32_z */, 16169 /* q4Uiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16402 /* svclz_s64_m */, 16205 /* q2UWiq2UWiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16414 /* svclz_s64_x */, 16236 /* q2UWiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16426 /* svclz_s64_z */, 16236 /* q2UWiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16438 /* svclz_s8_m */, 16273 /* q16Ucq16Ucq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16449 /* svclz_s8_x */, 16304 /* q16Ucq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16460 /* svclz_s8_z */, 16304 /* q16Ucq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16471 /* svclz_u16_m */, 16483 /* q8Usq8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16500 /* svclz_u16_x */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16525 /* svclz_u16_z */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16537 /* svclz_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16566 /* svclz_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16591 /* svclz_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16603 /* svclz_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16635 /* svclz_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16662 /* svclz_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16674 /* svclz_u8_m */, 16685 /* q16Ucq16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16705 /* svclz_u8_x */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16731 /* svclz_u8_z */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16742 /* svcmla_f16_m */, 16755 /* q8hq16bq8hq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16776 /* svcmla_f16_x */, 16755 /* q8hq16bq8hq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16789 /* svcmla_f16_z */, 16755 /* q8hq16bq8hq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16802 /* svcmla_f32_m */, 16815 /* q4fq16bq4fq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16836 /* svcmla_f32_x */, 16815 /* q4fq16bq4fq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16849 /* svcmla_f32_z */, 16815 /* q4fq16bq4fq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16862 /* svcmla_f64_m */, 16875 /* q2dq16bq2dq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16896 /* svcmla_f64_x */, 16875 /* q2dq16bq2dq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16909 /* svcmla_f64_z */, 16875 /* q2dq16bq2dq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16922 /* svcmla_lane_f16 */, 16938 /* q8hq8hq8hq8hIUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16959 /* svcmla_lane_f32 */, 16975 /* q4fq4fq4fq4fIUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16996 /* svcmla_lane_s16 */, 17012 /* q8sq8sq8sq8sIUWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17033 /* svcmla_lane_s32 */, 17049 /* q4iq4iq4iq4iIUWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17070 /* svcmla_lane_u16 */, 17086 /* q8Usq8Usq8Usq8UsIUWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17111 /* svcmla_lane_u32 */, 17127 /* q4Uiq4Uiq4Uiq4UiIUWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17152 /* svcmla_s16 */, 17163 /* q8sq8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17180 /* svcmla_s32 */, 17191 /* q4iq4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17208 /* svcmla_s64 */, 17219 /* q2Wiq2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17240 /* svcmla_s8 */, 17250 /* q16Scq16Scq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17275 /* svcmla_u16 */, 17286 /* q8Usq8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17307 /* svcmla_u32 */, 17318 /* q4Uiq4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17339 /* svcmla_u64 */, 17350 /* q2UWiq2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17375 /* svcmla_u8 */, 17385 /* q16Ucq16Ucq16Ucq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17410 /* svcmpeq_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17422 /* svcmpeq_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17434 /* svcmpeq_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17446 /* svcmpeq_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17460 /* svcmpeq_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17474 /* svcmpeq_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17488 /* svcmpeq_n_s16 */, 17502 /* q16bq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17515 /* svcmpeq_n_s32 */, 17529 /* q16bq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17542 /* svcmpeq_n_s64 */, 17556 /* q16bq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17571 /* svcmpeq_n_s8 */, 17584 /* q16bq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17600 /* svcmpeq_n_u16 */, 17614 /* q16bq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17629 /* svcmpeq_n_u32 */, 17643 /* q16bq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17658 /* svcmpeq_n_u64 */, 17672 /* q16bq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17689 /* svcmpeq_n_u8 */, 17702 /* q16bq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17718 /* svcmpeq_s16 */, 17730 /* q16bq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17745 /* svcmpeq_s32 */, 17757 /* q16bq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17772 /* svcmpeq_s64 */, 17784 /* q16bq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17801 /* svcmpeq_s8 */, 17812 /* q16bq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17831 /* svcmpeq_u16 */, 17843 /* q16bq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17860 /* svcmpeq_u32 */, 17872 /* q16bq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17889 /* svcmpeq_u64 */, 17901 /* q16bq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17920 /* svcmpeq_u8 */, 17931 /* q16bq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17950 /* svcmpeq_wide_n_s16 */, 17969 /* q16bq16bq8sWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17983 /* svcmpeq_wide_n_s32 */, 18002 /* q16bq16bq4iWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18016 /* svcmpeq_wide_n_s8 */, 18034 /* q16bq16bq16ScWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18050 /* svcmpeq_wide_s16 */, 18067 /* q16bq16bq8sq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18083 /* svcmpeq_wide_s32 */, 18100 /* q16bq16bq4iq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18116 /* svcmpeq_wide_s8 */, 18132 /* q16bq16bq16Scq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18150 /* svcmpge_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18162 /* svcmpge_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18174 /* svcmpge_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18186 /* svcmpge_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18200 /* svcmpge_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18214 /* svcmpge_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18228 /* svcmpge_n_s16 */, 17502 /* q16bq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18242 /* svcmpge_n_s32 */, 17529 /* q16bq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18256 /* svcmpge_n_s64 */, 17556 /* q16bq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18270 /* svcmpge_n_s8 */, 17584 /* q16bq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18283 /* svcmpge_n_u16 */, 17614 /* q16bq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18297 /* svcmpge_n_u32 */, 17643 /* q16bq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18311 /* svcmpge_n_u64 */, 17672 /* q16bq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18325 /* svcmpge_n_u8 */, 17702 /* q16bq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18338 /* svcmpge_s16 */, 17730 /* q16bq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18350 /* svcmpge_s32 */, 17757 /* q16bq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18362 /* svcmpge_s64 */, 17784 /* q16bq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18374 /* svcmpge_s8 */, 17812 /* q16bq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18385 /* svcmpge_u16 */, 17843 /* q16bq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18397 /* svcmpge_u32 */, 17872 /* q16bq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18409 /* svcmpge_u64 */, 17901 /* q16bq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18421 /* svcmpge_u8 */, 17931 /* q16bq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18432 /* svcmpge_wide_n_s16 */, 17969 /* q16bq16bq8sWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18451 /* svcmpge_wide_n_s32 */, 18002 /* q16bq16bq4iWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18470 /* svcmpge_wide_n_s8 */, 18034 /* q16bq16bq16ScWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18488 /* svcmpge_wide_n_u16 */, 18507 /* q16bq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18523 /* svcmpge_wide_n_u32 */, 18542 /* q16bq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18558 /* svcmpge_wide_n_u8 */, 18576 /* q16bq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18593 /* svcmpge_wide_s16 */, 18067 /* q16bq16bq8sq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18610 /* svcmpge_wide_s32 */, 18100 /* q16bq16bq4iq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18627 /* svcmpge_wide_s8 */, 18132 /* q16bq16bq16Scq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18643 /* svcmpge_wide_u16 */, 18660 /* q16bq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18678 /* svcmpge_wide_u32 */, 18695 /* q16bq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18713 /* svcmpge_wide_u8 */, 18729 /* q16bq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18748 /* svcmpgt_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18760 /* svcmpgt_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18772 /* svcmpgt_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18784 /* svcmpgt_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18798 /* svcmpgt_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18812 /* svcmpgt_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18826 /* svcmpgt_n_s16 */, 17502 /* q16bq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18840 /* svcmpgt_n_s32 */, 17529 /* q16bq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18854 /* svcmpgt_n_s64 */, 17556 /* q16bq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18868 /* svcmpgt_n_s8 */, 17584 /* q16bq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18881 /* svcmpgt_n_u16 */, 17614 /* q16bq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18895 /* svcmpgt_n_u32 */, 17643 /* q16bq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18909 /* svcmpgt_n_u64 */, 17672 /* q16bq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18923 /* svcmpgt_n_u8 */, 17702 /* q16bq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18936 /* svcmpgt_s16 */, 17730 /* q16bq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18948 /* svcmpgt_s32 */, 17757 /* q16bq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18960 /* svcmpgt_s64 */, 17784 /* q16bq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18972 /* svcmpgt_s8 */, 17812 /* q16bq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18983 /* svcmpgt_u16 */, 17843 /* q16bq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18995 /* svcmpgt_u32 */, 17872 /* q16bq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19007 /* svcmpgt_u64 */, 17901 /* q16bq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19019 /* svcmpgt_u8 */, 17931 /* q16bq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19030 /* svcmpgt_wide_n_s16 */, 17969 /* q16bq16bq8sWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19049 /* svcmpgt_wide_n_s32 */, 18002 /* q16bq16bq4iWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19068 /* svcmpgt_wide_n_s8 */, 18034 /* q16bq16bq16ScWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19086 /* svcmpgt_wide_n_u16 */, 18507 /* q16bq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19105 /* svcmpgt_wide_n_u32 */, 18542 /* q16bq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19124 /* svcmpgt_wide_n_u8 */, 18576 /* q16bq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19142 /* svcmpgt_wide_s16 */, 18067 /* q16bq16bq8sq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19159 /* svcmpgt_wide_s32 */, 18100 /* q16bq16bq4iq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19176 /* svcmpgt_wide_s8 */, 18132 /* q16bq16bq16Scq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19192 /* svcmpgt_wide_u16 */, 18660 /* q16bq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19209 /* svcmpgt_wide_u32 */, 18695 /* q16bq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19226 /* svcmpgt_wide_u8 */, 18729 /* q16bq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19242 /* svcmple_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19254 /* svcmple_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19266 /* svcmple_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19278 /* svcmple_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19292 /* svcmple_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19306 /* svcmple_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19320 /* svcmple_n_s16 */, 17502 /* q16bq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19334 /* svcmple_n_s32 */, 17529 /* q16bq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19348 /* svcmple_n_s64 */, 17556 /* q16bq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19362 /* svcmple_n_s8 */, 17584 /* q16bq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19375 /* svcmple_n_u16 */, 17614 /* q16bq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19389 /* svcmple_n_u32 */, 17643 /* q16bq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19403 /* svcmple_n_u64 */, 17672 /* q16bq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19417 /* svcmple_n_u8 */, 17702 /* q16bq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19430 /* svcmple_s16 */, 17730 /* q16bq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19442 /* svcmple_s32 */, 17757 /* q16bq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19454 /* svcmple_s64 */, 17784 /* q16bq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19466 /* svcmple_s8 */, 17812 /* q16bq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19477 /* svcmple_u16 */, 17843 /* q16bq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19489 /* svcmple_u32 */, 17872 /* q16bq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19501 /* svcmple_u64 */, 17901 /* q16bq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19513 /* svcmple_u8 */, 17931 /* q16bq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19524 /* svcmple_wide_n_s16 */, 17969 /* q16bq16bq8sWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19543 /* svcmple_wide_n_s32 */, 18002 /* q16bq16bq4iWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19562 /* svcmple_wide_n_s8 */, 18034 /* q16bq16bq16ScWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19580 /* svcmple_wide_n_u16 */, 18507 /* q16bq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19599 /* svcmple_wide_n_u32 */, 18542 /* q16bq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19618 /* svcmple_wide_n_u8 */, 18576 /* q16bq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19636 /* svcmple_wide_s16 */, 18067 /* q16bq16bq8sq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19653 /* svcmple_wide_s32 */, 18100 /* q16bq16bq4iq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19670 /* svcmple_wide_s8 */, 18132 /* q16bq16bq16Scq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19686 /* svcmple_wide_u16 */, 18660 /* q16bq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19703 /* svcmple_wide_u32 */, 18695 /* q16bq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19720 /* svcmple_wide_u8 */, 18729 /* q16bq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19736 /* svcmplt_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19748 /* svcmplt_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19760 /* svcmplt_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19772 /* svcmplt_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19786 /* svcmplt_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19800 /* svcmplt_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19814 /* svcmplt_n_s16 */, 17502 /* q16bq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19828 /* svcmplt_n_s32 */, 17529 /* q16bq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19842 /* svcmplt_n_s64 */, 17556 /* q16bq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19856 /* svcmplt_n_s8 */, 17584 /* q16bq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19869 /* svcmplt_n_u16 */, 17614 /* q16bq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19883 /* svcmplt_n_u32 */, 17643 /* q16bq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19897 /* svcmplt_n_u64 */, 17672 /* q16bq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19911 /* svcmplt_n_u8 */, 17702 /* q16bq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19924 /* svcmplt_s16 */, 17730 /* q16bq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19936 /* svcmplt_s32 */, 17757 /* q16bq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19948 /* svcmplt_s64 */, 17784 /* q16bq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19960 /* svcmplt_s8 */, 17812 /* q16bq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19971 /* svcmplt_u16 */, 17843 /* q16bq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19983 /* svcmplt_u32 */, 17872 /* q16bq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19995 /* svcmplt_u64 */, 17901 /* q16bq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20007 /* svcmplt_u8 */, 17931 /* q16bq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20018 /* svcmplt_wide_n_s16 */, 17969 /* q16bq16bq8sWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20037 /* svcmplt_wide_n_s32 */, 18002 /* q16bq16bq4iWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20056 /* svcmplt_wide_n_s8 */, 18034 /* q16bq16bq16ScWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20074 /* svcmplt_wide_n_u16 */, 18507 /* q16bq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20093 /* svcmplt_wide_n_u32 */, 18542 /* q16bq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20112 /* svcmplt_wide_n_u8 */, 18576 /* q16bq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20130 /* svcmplt_wide_s16 */, 18067 /* q16bq16bq8sq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20147 /* svcmplt_wide_s32 */, 18100 /* q16bq16bq4iq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20164 /* svcmplt_wide_s8 */, 18132 /* q16bq16bq16Scq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20180 /* svcmplt_wide_u16 */, 18660 /* q16bq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20197 /* svcmplt_wide_u32 */, 18695 /* q16bq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20214 /* svcmplt_wide_u8 */, 18729 /* q16bq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20230 /* svcmpne_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20242 /* svcmpne_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20254 /* svcmpne_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20266 /* svcmpne_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20280 /* svcmpne_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20294 /* svcmpne_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20308 /* svcmpne_n_s16 */, 17502 /* q16bq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20322 /* svcmpne_n_s32 */, 17529 /* q16bq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20336 /* svcmpne_n_s64 */, 17556 /* q16bq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20350 /* svcmpne_n_s8 */, 17584 /* q16bq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20363 /* svcmpne_n_u16 */, 17614 /* q16bq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20377 /* svcmpne_n_u32 */, 17643 /* q16bq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20391 /* svcmpne_n_u64 */, 17672 /* q16bq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20405 /* svcmpne_n_u8 */, 17702 /* q16bq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20418 /* svcmpne_s16 */, 17730 /* q16bq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20430 /* svcmpne_s32 */, 17757 /* q16bq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20442 /* svcmpne_s64 */, 17784 /* q16bq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20454 /* svcmpne_s8 */, 17812 /* q16bq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20465 /* svcmpne_u16 */, 17843 /* q16bq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20477 /* svcmpne_u32 */, 17872 /* q16bq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20489 /* svcmpne_u64 */, 17901 /* q16bq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20501 /* svcmpne_u8 */, 17931 /* q16bq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20512 /* svcmpne_wide_n_s16 */, 17969 /* q16bq16bq8sWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20531 /* svcmpne_wide_n_s32 */, 18002 /* q16bq16bq4iWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20550 /* svcmpne_wide_n_s8 */, 18034 /* q16bq16bq16ScWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20568 /* svcmpne_wide_s16 */, 18067 /* q16bq16bq8sq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20585 /* svcmpne_wide_s32 */, 18100 /* q16bq16bq4iq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20602 /* svcmpne_wide_s8 */, 18132 /* q16bq16bq16Scq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20618 /* svcmpuo_f16 */, 3050 /* q16bq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20630 /* svcmpuo_f32 */, 3076 /* q16bq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20642 /* svcmpuo_f64 */, 3102 /* q16bq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20654 /* svcmpuo_n_f16 */, 3130 /* q16bq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20668 /* svcmpuo_n_f32 */, 3156 /* q16bq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20682 /* svcmpuo_n_f64 */, 3182 /* q16bq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20696 /* svcnot_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20709 /* svcnot_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20722 /* svcnot_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20735 /* svcnot_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20748 /* svcnot_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20761 /* svcnot_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20774 /* svcnot_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20787 /* svcnot_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20800 /* svcnot_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20813 /* svcnot_s8_m */, 2982 /* q16Scq16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20825 /* svcnot_s8_x */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20837 /* svcnot_s8_z */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20849 /* svcnot_u16_m */, 16483 /* q8Usq8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20862 /* svcnot_u16_x */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20875 /* svcnot_u16_z */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20888 /* svcnot_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20901 /* svcnot_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20914 /* svcnot_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20927 /* svcnot_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20940 /* svcnot_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20953 /* svcnot_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20966 /* svcnot_u8_m */, 16685 /* q16Ucq16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20978 /* svcnot_u8_x */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20990 /* svcnot_u8_z */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21002 /* svcnt_bf16_m */, 21015 /* q8Usq8Usq16bq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21031 /* svcnt_bf16_x */, 21044 /* q8Usq16bq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21056 /* svcnt_bf16_z */, 21044 /* q8Usq16bq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21069 /* svcnt_f16_m */, 21081 /* q8Usq8Usq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21097 /* svcnt_f16_x */, 21109 /* q8Usq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21121 /* svcnt_f16_z */, 21109 /* q8Usq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21133 /* svcnt_f32_m */, 21145 /* q4Uiq4Uiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21161 /* svcnt_f32_x */, 21173 /* q4Uiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21185 /* svcnt_f32_z */, 21173 /* q4Uiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21197 /* svcnt_f64_m */, 21209 /* q2UWiq2UWiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21227 /* svcnt_f64_x */, 21239 /* q2UWiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21252 /* svcnt_f64_z */, 21239 /* q2UWiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21264 /* svcnt_s16_m */, 16077 /* q8Usq8Usq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21276 /* svcnt_s16_x */, 16105 /* q8Usq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21288 /* svcnt_s16_z */, 16105 /* q8Usq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21300 /* svcnt_s32_m */, 16141 /* q4Uiq4Uiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21312 /* svcnt_s32_x */, 16169 /* q4Uiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21324 /* svcnt_s32_z */, 16169 /* q4Uiq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21336 /* svcnt_s64_m */, 16205 /* q2UWiq2UWiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21348 /* svcnt_s64_x */, 16236 /* q2UWiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21360 /* svcnt_s64_z */, 16236 /* q2UWiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21372 /* svcnt_s8_m */, 16273 /* q16Ucq16Ucq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21383 /* svcnt_s8_x */, 16304 /* q16Ucq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21394 /* svcnt_s8_z */, 16304 /* q16Ucq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21405 /* svcnt_u16_m */, 16483 /* q8Usq8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21417 /* svcnt_u16_x */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21429 /* svcnt_u16_z */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21441 /* svcnt_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21453 /* svcnt_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21465 /* svcnt_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21477 /* svcnt_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21489 /* svcnt_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21501 /* svcnt_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21513 /* svcnt_u8_m */, 16685 /* q16Ucq16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21524 /* svcnt_u8_x */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21535 /* svcnt_u8_z */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21546 /* svcntb */, 21553 /* UWiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21558 /* svcntb_pat */, 21569 /* UWiIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21575 /* svcntd */, 21553 /* UWiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21582 /* svcntd_pat */, 21569 /* UWiIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21593 /* svcnth */, 21553 /* UWiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21600 /* svcnth_pat */, 21569 /* UWiIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21611 /* svcntp_b16 */, 21622 /* UWiq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21634 /* svcntp_b32 */, 21622 /* UWiq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21645 /* svcntp_b64 */, 21622 /* UWiq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21656 /* svcntp_b8 */, 21622 /* UWiq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21666 /* svcntp_c16 */, 21677 /* UWiQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21720 /* svcntp_c32 */, 21677 /* UWiQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21731 /* svcntp_c64 */, 21677 /* UWiQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21742 /* svcntp_c8 */, 21677 /* UWiQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21752 /* svcntw */, 21553 /* UWiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21759 /* svcntw_pat */, 21569 /* UWiIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21770 /* svcompact_bf16 */, 21785 /* q8yq16bq8y */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21831 /* svcompact_f16 */, 2638 /* q8hq16bq8h */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21845 /* svcompact_f32 */, 2699 /* q4fq16bq4f */, 1 /* n */, 21859 /* sve|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21876 /* svcompact_f64 */, 2760 /* q2dq16bq2d */, 1 /* n */, 21859 /* sve|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21890 /* svcompact_mf8 */, 21904 /* q16mq16bq16m */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21917 /* svcompact_s16 */, 2821 /* q8sq16bq8s */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21931 /* svcompact_s32 */, 2882 /* q4iq16bq4i */, 1 /* n */, 21859 /* sve|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21945 /* svcompact_s64 */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 21859 /* sve|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21959 /* svcompact_s8 */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21972 /* svcompact_u16 */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21986 /* svcompact_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 21859 /* sve|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22000 /* svcompact_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 21859 /* sve|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22014 /* svcompact_u8 */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22027 /* svcreate2_b */, 22039 /* q32bq16bq16b */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22052 /* svcreate2_bf16 */, 22067 /* q16yq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22078 /* svcreate2_f16 */, 22092 /* q16hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22103 /* svcreate2_f32 */, 22117 /* q8fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22127 /* svcreate2_f64 */, 22141 /* q4dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22151 /* svcreate2_mf8 */, 22165 /* q32mq16mq16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22178 /* svcreate2_s16 */, 22192 /* q16sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22203 /* svcreate2_s32 */, 22217 /* q8iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22227 /* svcreate2_s64 */, 22241 /* q4Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22254 /* svcreate2_s8 */, 22267 /* q32Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22283 /* svcreate2_u16 */, 22297 /* q16Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22311 /* svcreate2_u32 */, 22325 /* q8Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22338 /* svcreate2_u64 */, 22352 /* q4UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22368 /* svcreate2_u8 */, 22381 /* q32Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22397 /* svcreate3_bf16 */, 22412 /* q24yq8yq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22426 /* svcreate3_f16 */, 22440 /* q24hq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22454 /* svcreate3_f32 */, 22468 /* q12fq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22482 /* svcreate3_f64 */, 22496 /* q6dq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22509 /* svcreate3_mf8 */, 22523 /* q48mq16mq16mq16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22540 /* svcreate3_s16 */, 22554 /* q24sq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22568 /* svcreate3_s32 */, 22582 /* q12iq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22596 /* svcreate3_s64 */, 22610 /* q6Wiq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22627 /* svcreate3_s8 */, 22640 /* q48Scq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22661 /* svcreate3_u16 */, 22675 /* q24Usq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22693 /* svcreate3_u32 */, 22707 /* q12Uiq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22725 /* svcreate3_u64 */, 22739 /* q6UWiq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22760 /* svcreate3_u8 */, 22773 /* q48Ucq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22794 /* svcreate4_b */, 22806 /* q64bq16bq16bq16bq16b */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22827 /* svcreate4_bf16 */, 22842 /* q32yq8yq8yq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22859 /* svcreate4_f16 */, 22873 /* q32hq8hq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22890 /* svcreate4_f32 */, 22904 /* q16fq4fq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22921 /* svcreate4_f64 */, 22935 /* q8dq2dq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22951 /* svcreate4_mf8 */, 22965 /* q64mq16mq16mq16mq16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22986 /* svcreate4_s16 */, 23000 /* q32sq8sq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23017 /* svcreate4_s32 */, 23031 /* q16iq4iq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23048 /* svcreate4_s64 */, 23062 /* q8Wiq2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23083 /* svcreate4_s8 */, 23096 /* q64Scq16Scq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23122 /* svcreate4_u16 */, 23136 /* q32Usq8Usq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23158 /* svcreate4_u32 */, 23172 /* q16Uiq4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23194 /* svcreate4_u64 */, 23208 /* q8UWiq2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23234 /* svcreate4_u8 */, 23247 /* q64Ucq16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23273 /* svcvt1_bf16_mf8_fpm */, 23293 /* q8yq16mUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23341 /* svcvt1_bf16_mf8_x2_fpm */, 23364 /* q16yq16mUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23389 /* svcvt1_f16_mf8_fpm */, 23408 /* q8hq16mUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23419 /* svcvt1_f16_mf8_x2_fpm */, 23441 /* q16hq16mUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23453 /* svcvt2_bf16_mf8_fpm */, 23293 /* q8yq16mUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23473 /* svcvt2_bf16_mf8_x2_fpm */, 23364 /* q16yq16mUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23496 /* svcvt2_f16_mf8_fpm */, 23408 /* q8hq16mUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23515 /* svcvt2_f16_mf8_x2_fpm */, 23441 /* q16hq16mUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23537 /* svcvt_bf16_f32_m */, 23554 /* q8yq8yq16bq4f */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23568 /* svcvt_bf16_f32_x */, 23585 /* q8yq16bq4f */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23596 /* svcvt_bf16_f32_x2 */, 23614 /* q8yq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23621 /* svcvt_bf16_f32_z */, 23585 /* q8yq16bq4f */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23638 /* svcvt_f16_f32_m */, 23654 /* q8hq8hq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23668 /* svcvt_f16_f32_x */, 23684 /* q8hq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23695 /* svcvt_f16_f32_x2 */, 23712 /* q8hq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23719 /* svcvt_f16_f32_z */, 23684 /* q8hq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23735 /* svcvt_f16_f64_m */, 23751 /* q8hq8hq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23765 /* svcvt_f16_f64_x */, 23781 /* q8hq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23792 /* svcvt_f16_f64_z */, 23781 /* q8hq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23808 /* svcvt_f16_s16_m */, 23824 /* q8hq8hq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23838 /* svcvt_f16_s16_x */, 23854 /* q8hq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23865 /* svcvt_f16_s16_z */, 23854 /* q8hq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23881 /* svcvt_f16_s32_m */, 23897 /* q8hq8hq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23911 /* svcvt_f16_s32_x */, 23927 /* q8hq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23938 /* svcvt_f16_s32_z */, 23927 /* q8hq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23954 /* svcvt_f16_s64_m */, 23970 /* q8hq8hq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23985 /* svcvt_f16_s64_x */, 24001 /* q8hq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24013 /* svcvt_f16_s64_z */, 24001 /* q8hq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24029 /* svcvt_f16_u16_m */, 24045 /* q8hq8hq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24060 /* svcvt_f16_u16_x */, 24076 /* q8hq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24088 /* svcvt_f16_u16_z */, 24076 /* q8hq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24104 /* svcvt_f16_u32_m */, 24120 /* q8hq8hq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24135 /* svcvt_f16_u32_x */, 24151 /* q8hq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24163 /* svcvt_f16_u32_z */, 24151 /* q8hq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24179 /* svcvt_f16_u64_m */, 24195 /* q8hq8hq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24211 /* svcvt_f16_u64_x */, 24227 /* q8hq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24240 /* svcvt_f16_u64_z */, 24227 /* q8hq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24256 /* svcvt_f32_f16_m */, 24272 /* q4fq4fq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24286 /* svcvt_f32_f16_x */, 24302 /* q4fq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24313 /* svcvt_f32_f16_x2 */, 24330 /* q8fq8h */, 1 /* n */, 24337 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24352 /* svcvt_f32_f16_z */, 24302 /* q4fq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24368 /* svcvt_f32_f64_m */, 24384 /* q4fq4fq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24398 /* svcvt_f32_f64_x */, 24414 /* q4fq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24425 /* svcvt_f32_f64_z */, 24414 /* q4fq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24441 /* svcvt_f32_s32_m */, 24457 /* q4fq4fq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24471 /* svcvt_f32_s32_x */, 24487 /* q4fq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24498 /* svcvt_f32_s32_x2 */, 24515 /* q8fq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24522 /* svcvt_f32_s32_x4 */, 24539 /* q16fq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24548 /* svcvt_f32_s32_z */, 24487 /* q4fq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24564 /* svcvt_f32_s64_m */, 24580 /* q4fq4fq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24595 /* svcvt_f32_s64_x */, 24611 /* q4fq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24623 /* svcvt_f32_s64_z */, 24611 /* q4fq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24639 /* svcvt_f32_u32_m */, 24655 /* q4fq4fq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24670 /* svcvt_f32_u32_x */, 24686 /* q4fq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24698 /* svcvt_f32_u32_x2 */, 24715 /* q8fq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24723 /* svcvt_f32_u32_x4 */, 24740 /* q16fq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24750 /* svcvt_f32_u32_z */, 24686 /* q4fq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24766 /* svcvt_f32_u64_m */, 24782 /* q4fq4fq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24798 /* svcvt_f32_u64_x */, 24814 /* q4fq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24827 /* svcvt_f32_u64_z */, 24814 /* q4fq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24843 /* svcvt_f64_f16_m */, 24859 /* q2dq2dq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24873 /* svcvt_f64_f16_x */, 24889 /* q2dq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24900 /* svcvt_f64_f16_z */, 24889 /* q2dq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24916 /* svcvt_f64_f32_m */, 24932 /* q2dq2dq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24946 /* svcvt_f64_f32_x */, 24962 /* q2dq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24973 /* svcvt_f64_f32_z */, 24962 /* q2dq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24989 /* svcvt_f64_s32_m */, 25005 /* q2dq2dq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25019 /* svcvt_f64_s32_x */, 25035 /* q2dq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25046 /* svcvt_f64_s32_z */, 25035 /* q2dq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25062 /* svcvt_f64_s64_m */, 25078 /* q2dq2dq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25093 /* svcvt_f64_s64_x */, 25109 /* q2dq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25121 /* svcvt_f64_s64_z */, 25109 /* q2dq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25137 /* svcvt_f64_u32_m */, 25153 /* q2dq2dq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25168 /* svcvt_f64_u32_x */, 25184 /* q2dq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25196 /* svcvt_f64_u32_z */, 25184 /* q2dq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25212 /* svcvt_f64_u64_m */, 25228 /* q2dq2dq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25244 /* svcvt_f64_u64_x */, 25260 /* q2dq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25273 /* svcvt_f64_u64_z */, 25260 /* q2dq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25289 /* svcvt_mf8_bf16_x2_fpm */, 25311 /* q16mq16yUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25323 /* svcvt_mf8_f16_x2_fpm */, 25344 /* q16mq16hUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25356 /* svcvt_mf8_f32_x4_fpm */, 25377 /* q16mq16fUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25389 /* svcvt_s16_f16_m */, 25405 /* q8sq8sq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25419 /* svcvt_s16_f16_x */, 25435 /* q8sq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25446 /* svcvt_s16_f16_z */, 25435 /* q8sq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25462 /* svcvt_s32_f16_m */, 25478 /* q4iq4iq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25492 /* svcvt_s32_f16_x */, 25508 /* q4iq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25519 /* svcvt_s32_f16_z */, 25508 /* q4iq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25535 /* svcvt_s32_f32_m */, 25551 /* q4iq4iq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25565 /* svcvt_s32_f32_x */, 25581 /* q4iq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25592 /* svcvt_s32_f32_x2 */, 25609 /* q8iq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25616 /* svcvt_s32_f32_x4 */, 25633 /* q16iq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25642 /* svcvt_s32_f32_z */, 25581 /* q4iq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25658 /* svcvt_s32_f64_m */, 25674 /* q4iq4iq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25688 /* svcvt_s32_f64_x */, 25704 /* q4iq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25715 /* svcvt_s32_f64_z */, 25704 /* q4iq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25731 /* svcvt_s64_f16_m */, 25747 /* q2Wiq2Wiq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25763 /* svcvt_s64_f16_x */, 25779 /* q2Wiq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25791 /* svcvt_s64_f16_z */, 25779 /* q2Wiq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25807 /* svcvt_s64_f32_m */, 25823 /* q2Wiq2Wiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25839 /* svcvt_s64_f32_x */, 25855 /* q2Wiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25867 /* svcvt_s64_f32_z */, 25855 /* q2Wiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25883 /* svcvt_s64_f64_m */, 25899 /* q2Wiq2Wiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25915 /* svcvt_s64_f64_x */, 25931 /* q2Wiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25943 /* svcvt_s64_f64_z */, 25931 /* q2Wiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25959 /* svcvt_u16_f16_m */, 21081 /* q8Usq8Usq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25975 /* svcvt_u16_f16_x */, 21109 /* q8Usq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25991 /* svcvt_u16_f16_z */, 21109 /* q8Usq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26007 /* svcvt_u32_f16_m */, 26023 /* q4Uiq4Uiq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26039 /* svcvt_u32_f16_x */, 26055 /* q4Uiq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26067 /* svcvt_u32_f16_z */, 26055 /* q4Uiq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26083 /* svcvt_u32_f32_m */, 21145 /* q4Uiq4Uiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26099 /* svcvt_u32_f32_x */, 21173 /* q4Uiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26115 /* svcvt_u32_f32_x2 */, 26132 /* q8Uiq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26140 /* svcvt_u32_f32_x4 */, 26157 /* q16Uiq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26167 /* svcvt_u32_f32_z */, 21173 /* q4Uiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26183 /* svcvt_u32_f64_m */, 26199 /* q4Uiq4Uiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26215 /* svcvt_u32_f64_x */, 26231 /* q4Uiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26243 /* svcvt_u32_f64_z */, 26231 /* q4Uiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26259 /* svcvt_u64_f16_m */, 26275 /* q2UWiq2UWiq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26293 /* svcvt_u64_f16_x */, 26309 /* q2UWiq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26322 /* svcvt_u64_f16_z */, 26309 /* q2UWiq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26338 /* svcvt_u64_f32_m */, 26354 /* q2UWiq2UWiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26372 /* svcvt_u64_f32_x */, 26388 /* q2UWiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26401 /* svcvt_u64_f32_z */, 26388 /* q2UWiq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26417 /* svcvt_u64_f64_m */, 21209 /* q2UWiq2UWiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26433 /* svcvt_u64_f64_x */, 21239 /* q2UWiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26449 /* svcvt_u64_f64_z */, 21239 /* q2UWiq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26465 /* svcvtl1_bf16_mf8_x2_fpm */, 23364 /* q16yq16mUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26489 /* svcvtl1_f16_mf8_x2_fpm */, 23441 /* q16hq16mUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26512 /* svcvtl2_bf16_mf8_x2_fpm */, 23364 /* q16yq16mUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26536 /* svcvtl2_f16_mf8_x2_fpm */, 23441 /* q16hq16mUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26559 /* svcvtl_f32_f16_x2 */, 24330 /* q8fq8h */, 1 /* n */, 24337 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26577 /* svcvtlt1_bf16_mf8_fpm */, 23293 /* q8yq16mUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26599 /* svcvtlt1_f16_mf8_fpm */, 23408 /* q8hq16mUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26620 /* svcvtlt2_bf16_mf8_fpm */, 23293 /* q8yq16mUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26642 /* svcvtlt2_f16_mf8_fpm */, 23408 /* q8hq16mUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26663 /* svcvtlt_f32_f16_m */, 24272 /* q4fq4fq16bq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26681 /* svcvtlt_f32_f16_x */, 24302 /* q4fq16bq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26699 /* svcvtlt_f32_f16_z */, 24302 /* q4fq16bq8h */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26761 /* svcvtlt_f64_f32_m */, 24932 /* q2dq2dq16bq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26779 /* svcvtlt_f64_f32_x */, 24962 /* q2dq16bq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26797 /* svcvtlt_f64_f32_z */, 24962 /* q2dq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26815 /* svcvtn_bf16_f32_x2 */, 23614 /* q8yq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26834 /* svcvtn_f16_f32_x2 */, 23712 /* q8hq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26852 /* svcvtn_mf8_bf16_x2_fpm */, 25311 /* q16mq16yUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26875 /* svcvtn_mf8_f16_x2_fpm */, 25344 /* q16mq16hUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26897 /* svcvtn_mf8_f32_x4_fpm */, 25377 /* q16mq16fUWi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26919 /* svcvtnb_mf8_f32_x2_fpm */, 26942 /* q16mq8fUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26953 /* svcvtnt_bf16_f32_m */, 23554 /* q8yq8yq16bq4f */, 1 /* n */, 11881 /* (sve,bf16)|(sme,bf16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26972 /* svcvtnt_bf16_f32_z */, 23554 /* q8yq8yq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26991 /* svcvtnt_f16_f32_m */, 23654 /* q8hq8hq16bq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27009 /* svcvtnt_f16_f32_z */, 23654 /* q8hq8hq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27027 /* svcvtnt_f32_f64_m */, 24384 /* q4fq4fq16bq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27045 /* svcvtnt_f32_f64_z */, 24384 /* q4fq4fq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27063 /* svcvtnt_mf8_f32_x2_fpm */, 27086 /* q16mq16mq8fUWi */, 1 /* n */, 23304 /* (sve,(sve2|sme2),fp8)|(sme,sme2,fp8) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27101 /* svcvtx_f32_f64_m */, 24384 /* q4fq4fq16bq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27118 /* svcvtx_f32_f64_x */, 24414 /* q4fq16bq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27135 /* svcvtx_f32_f64_z */, 24414 /* q4fq16bq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27152 /* svcvtxnt_f32_f64_m */, 24384 /* q4fq4fq16bq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27171 /* svcvtxnt_f32_f64_z */, 24384 /* q4fq4fq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27190 /* svdiv_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27202 /* svdiv_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27214 /* svdiv_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27226 /* svdiv_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27238 /* svdiv_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27250 /* svdiv_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27262 /* svdiv_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27274 /* svdiv_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27286 /* svdiv_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27298 /* svdiv_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27312 /* svdiv_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27326 /* svdiv_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27340 /* svdiv_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27354 /* svdiv_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27368 /* svdiv_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27382 /* svdiv_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27396 /* svdiv_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27410 /* svdiv_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27424 /* svdiv_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27438 /* svdiv_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27452 /* svdiv_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27466 /* svdiv_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27480 /* svdiv_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27494 /* svdiv_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27508 /* svdiv_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27522 /* svdiv_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27536 /* svdiv_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27550 /* svdiv_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27564 /* svdiv_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27578 /* svdiv_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27592 /* svdiv_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27604 /* svdiv_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27616 /* svdiv_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27628 /* svdiv_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27640 /* svdiv_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27652 /* svdiv_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27664 /* svdiv_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27676 /* svdiv_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27688 /* svdiv_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27700 /* svdiv_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27712 /* svdiv_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27724 /* svdiv_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27736 /* svdivr_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27749 /* svdivr_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27762 /* svdivr_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27775 /* svdivr_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27788 /* svdivr_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27801 /* svdivr_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27814 /* svdivr_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27827 /* svdivr_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27840 /* svdivr_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27853 /* svdivr_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27868 /* svdivr_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27883 /* svdivr_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27898 /* svdivr_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27913 /* svdivr_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27928 /* svdivr_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27943 /* svdivr_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27958 /* svdivr_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27973 /* svdivr_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27988 /* svdivr_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28003 /* svdivr_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28018 /* svdivr_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28033 /* svdivr_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28048 /* svdivr_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28063 /* svdivr_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28078 /* svdivr_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28093 /* svdivr_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28108 /* svdivr_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28123 /* svdivr_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28138 /* svdivr_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28153 /* svdivr_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28168 /* svdivr_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28181 /* svdivr_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28194 /* svdivr_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28207 /* svdivr_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28220 /* svdivr_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28233 /* svdivr_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28246 /* svdivr_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28259 /* svdivr_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28272 /* svdivr_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28285 /* svdivr_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28298 /* svdivr_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28311 /* svdivr_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28324 /* svdot_f16_mf8_fpm */, 28342 /* q8hq8hq16mq16mUWi */, 1 /* n */, 28360 /* (sve,sve2,fp8dot2)|(sme,ssve-fp8dot2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28398 /* svdot_f32_f16 */, 28412 /* q4fq4fq8hq8h */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28425 /* svdot_f32_mf8_fpm */, 28443 /* q4fq4fq16mq16mUWi */, 1 /* n */, 28461 /* (sve,sve2,fp8dot4)|(sme,ssve-fp8dot4) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28499 /* svdot_lane_f16_mf8_fpm */, 28522 /* q8hq8hq16mq16mIUWiUWi */, 1 /* n */, 28360 /* (sve,sve2,fp8dot2)|(sme,ssve-fp8dot2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28544 /* svdot_lane_f32_f16 */, 28563 /* q4fq4fq8hq8hIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28580 /* svdot_lane_f32_mf8_fpm */, 28603 /* q4fq4fq16mq16mIUWiUWi */, 1 /* n */, 28461 /* (sve,sve2,fp8dot4)|(sme,ssve-fp8dot4) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28625 /* svdot_lane_s32 */, 14063 /* q4iq4iq16Scq16ScIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28640 /* svdot_lane_s32_s16 */, 28659 /* q4iq4iq8sq8sIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28676 /* svdot_lane_s64 */, 14095 /* q2Wiq2Wiq8sq8sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28691 /* svdot_lane_u32 */, 28706 /* q4Uiq4Uiq16Ucq16UcIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28729 /* svdot_lane_u32_u16 */, 28748 /* q4Uiq4Uiq8Usq8UsIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28769 /* svdot_lane_u64 */, 28784 /* q2UWiq2UWiq8Usq8UsIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28807 /* svdot_n_f16_mf8_fpm */, 28827 /* q8hq8hq16mmUWi */, 1 /* n */, 28360 /* (sve,sve2,fp8dot2)|(sme,ssve-fp8dot2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28842 /* svdot_n_f32_mf8_fpm */, 28862 /* q4fq4fq16mmUWi */, 1 /* n */, 28461 /* (sve,sve2,fp8dot4)|(sme,ssve-fp8dot4) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28877 /* svdot_n_s32 */, 28889 /* q4iq4iq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28903 /* svdot_n_s64 */, 28915 /* q2Wiq2Wiq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28928 /* svdot_n_u32 */, 28940 /* q4Uiq4Uiq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28956 /* svdot_n_u64 */, 28968 /* q2UWiq2UWiq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28985 /* svdot_s32 */, 28995 /* q4iq4iq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29012 /* svdot_s32_s16 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29026 /* svdot_s64 */, 29036 /* q2Wiq2Wiq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29051 /* svdot_u32 */, 29061 /* q4Uiq4Uiq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29080 /* svdot_u32_u16 */, 752 /* q4Uiq4Uiq8Usq8Us */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29094 /* svdot_u64 */, 29104 /* q2UWiq2UWiq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29123 /* svdup_lane_bf16 */, 29139 /* q8yq8yUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29148 /* svdup_lane_f16 */, 29163 /* q8hq8hUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29172 /* svdup_lane_f32 */, 29187 /* q4fq4fUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29196 /* svdup_lane_f64 */, 29211 /* q2dq2dUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29221 /* svdup_lane_s16 */, 29236 /* q8sq8sUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29245 /* svdup_lane_s32 */, 29260 /* q4iq4iUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29269 /* svdup_lane_s64 */, 29284 /* q2Wiq2WiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29296 /* svdup_lane_s8 */, 29310 /* q16Scq16ScUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29323 /* svdup_lane_u16 */, 11582 /* q8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29338 /* svdup_lane_u32 */, 11643 /* q4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29353 /* svdup_lane_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29368 /* svdup_lane_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29382 /* svdup_laneq_bf16 */, 29399 /* q8yq8yIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29410 /* svdup_laneq_f16 */, 29426 /* q8hq8hIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29437 /* svdup_laneq_f32 */, 29453 /* q4fq4fIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29464 /* svdup_laneq_f64 */, 29480 /* q2dq2dIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29491 /* svdup_laneq_mf8 */, 29507 /* q16mq16mIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29520 /* svdup_laneq_s16 */, 29536 /* q8sq8sIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29547 /* svdup_laneq_s32 */, 29563 /* q4iq4iIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29574 /* svdup_laneq_s64 */, 29590 /* q2Wiq2WiIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29603 /* svdup_laneq_s8 */, 29618 /* q16Scq16ScIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29633 /* svdup_laneq_u16 */, 29649 /* q8Usq8UsIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29662 /* svdup_laneq_u32 */, 29678 /* q4Uiq4UiIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29691 /* svdup_laneq_u64 */, 29707 /* q2UWiq2UWiIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29722 /* svdup_laneq_u8 */, 29737 /* q16Ucq16UcIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29752 /* svdup_n_b16 */, 29764 /* q16bb */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29770 /* svdup_n_b32 */, 29764 /* q16bb */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29782 /* svdup_n_b64 */, 29764 /* q16bb */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29794 /* svdup_n_b8 */, 29764 /* q16bb */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29805 /* svdup_n_bf16 */, 29818 /* q8yy */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29823 /* svdup_n_bf16_m */, 29838 /* q8yq8yq16by */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29850 /* svdup_n_bf16_x */, 29865 /* q8yq16by */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29874 /* svdup_n_bf16_z */, 29865 /* q8yq16by */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29889 /* svdup_n_f16 */, 29901 /* q8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29906 /* svdup_n_f16_m */, 29920 /* q8hq8hq16bh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29932 /* svdup_n_f16_x */, 29946 /* q8hq16bh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29955 /* svdup_n_f16_z */, 29946 /* q8hq16bh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29969 /* svdup_n_f32 */, 29981 /* q4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29986 /* svdup_n_f32_m */, 30000 /* q4fq4fq16bf */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30012 /* svdup_n_f32_x */, 30026 /* q4fq16bf */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30035 /* svdup_n_f32_z */, 30026 /* q4fq16bf */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30049 /* svdup_n_f64 */, 30061 /* q2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30066 /* svdup_n_f64_m */, 30080 /* q2dq2dq16bd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30092 /* svdup_n_f64_x */, 30106 /* q2dq16bd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30115 /* svdup_n_f64_z */, 30106 /* q2dq16bd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30129 /* svdup_n_s16 */, 30141 /* q8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30146 /* svdup_n_s16_m */, 30160 /* q8sq8sq16bs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30172 /* svdup_n_s16_x */, 30186 /* q8sq16bs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30195 /* svdup_n_s16_z */, 30186 /* q8sq16bs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30209 /* svdup_n_s32 */, 30221 /* q4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30226 /* svdup_n_s32_m */, 30240 /* q4iq4iq16bi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30252 /* svdup_n_s32_x */, 30266 /* q4iq16bi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30275 /* svdup_n_s32_z */, 30266 /* q4iq16bi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30289 /* svdup_n_s64 */, 30301 /* q2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30308 /* svdup_n_s64_m */, 30322 /* q2Wiq2Wiq16bWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30337 /* svdup_n_s64_x */, 30351 /* q2Wiq16bWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30362 /* svdup_n_s64_z */, 30351 /* q2Wiq16bWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30376 /* svdup_n_s8 */, 30387 /* q16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30395 /* svdup_n_s8_m */, 30408 /* q16Scq16Scq16bSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30425 /* svdup_n_s8_x */, 30438 /* q16Scq16bSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30450 /* svdup_n_s8_z */, 30438 /* q16Scq16bSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30463 /* svdup_n_u16 */, 30475 /* q8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30482 /* svdup_n_u16_m */, 30496 /* q8Usq8Usq16bUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30511 /* svdup_n_u16_x */, 30525 /* q8Usq16bUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30536 /* svdup_n_u16_z */, 30525 /* q8Usq16bUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30550 /* svdup_n_u32 */, 30562 /* q4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30569 /* svdup_n_u32_m */, 30583 /* q4Uiq4Uiq16bUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30598 /* svdup_n_u32_x */, 30612 /* q4Uiq16bUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30623 /* svdup_n_u32_z */, 30612 /* q4Uiq16bUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30637 /* svdup_n_u64 */, 30649 /* q2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30658 /* svdup_n_u64_m */, 30672 /* q2UWiq2UWiq16bUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30690 /* svdup_n_u64_x */, 30704 /* q2UWiq16bUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30717 /* svdup_n_u64_z */, 30704 /* q2UWiq16bUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30731 /* svdup_n_u8 */, 30742 /* q16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30750 /* svdup_n_u8_m */, 30763 /* q16Ucq16Ucq16bUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30780 /* svdup_n_u8_x */, 30793 /* q16Ucq16bUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30805 /* svdup_n_u8_z */, 30793 /* q16Ucq16bUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30818 /* svdupq_lane_bf16 */, 30835 /* q8yq8yUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30845 /* svdupq_lane_f16 */, 30861 /* q8hq8hUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30871 /* svdupq_lane_f32 */, 30887 /* q4fq4fUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30897 /* svdupq_lane_f64 */, 29211 /* q2dq2dUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30913 /* svdupq_lane_s16 */, 30929 /* q8sq8sUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30939 /* svdupq_lane_s32 */, 30955 /* q4iq4iUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30965 /* svdupq_lane_s64 */, 29284 /* q2Wiq2WiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30981 /* svdupq_lane_s8 */, 30996 /* q16Scq16ScUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31010 /* svdupq_lane_u16 */, 31026 /* q8Usq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31038 /* svdupq_lane_u32 */, 31054 /* q4Uiq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31066 /* svdupq_lane_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31082 /* svdupq_lane_u8 */, 31097 /* q16Ucq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31111 /* svdupq_n_b16 */, 31124 /* q16bbbbbbbbb */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31137 /* svdupq_n_b32 */, 31150 /* q16bbbbb */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31159 /* svdupq_n_b64 */, 31172 /* q16bbb */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31179 /* svdupq_n_b8 */, 31191 /* q16bbbbbbbbbbbbbbbbb */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31212 /* svdupq_n_bf16 */, 31226 /* q8yyyyyyyyy */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31238 /* svdupq_n_f16 */, 31251 /* q8hhhhhhhhh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31263 /* svdupq_n_f32 */, 31276 /* q4fffff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31284 /* svdupq_n_f64 */, 31297 /* q2ddd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31303 /* svdupq_n_s16 */, 31316 /* q8sssssssss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31328 /* svdupq_n_s32 */, 31341 /* q4iiiii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31349 /* svdupq_n_s64 */, 31362 /* q2WiWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31371 /* svdupq_n_s8 */, 31383 /* q16ScScScScScScScScScScScScScScScScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31421 /* svdupq_n_u16 */, 31434 /* q8UsUsUsUsUsUsUsUsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31455 /* svdupq_n_u32 */, 31468 /* q4UiUiUiUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31481 /* svdupq_n_u64 */, 31494 /* q2UWiUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31506 /* svdupq_n_u8 */, 31518 /* q16UcUcUcUcUcUcUcUcUcUcUcUcUcUcUcUcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31556 /* sveor3_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31569 /* sveor3_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31582 /* sveor3_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31595 /* sveor3_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31607 /* sveor3_n_u16 */, 138 /* q8Usq8Usq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31620 /* sveor3_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31633 /* sveor3_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31646 /* sveor3_n_u8 */, 222 /* q16Ucq16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31658 /* sveor3_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31669 /* sveor3_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31680 /* sveor3_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31691 /* sveor3_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31701 /* sveor3_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31712 /* sveor3_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31723 /* sveor3_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31734 /* sveor3_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31744 /* sveor_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31754 /* sveor_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31768 /* sveor_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31782 /* sveor_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31796 /* sveor_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31810 /* sveor_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31824 /* sveor_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31838 /* sveor_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31852 /* sveor_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31866 /* sveor_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31880 /* sveor_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31893 /* sveor_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31906 /* sveor_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31919 /* sveor_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31933 /* sveor_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31947 /* sveor_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31961 /* sveor_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31975 /* sveor_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31989 /* sveor_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32003 /* sveor_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32017 /* sveor_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32031 /* sveor_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32045 /* sveor_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32058 /* sveor_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32071 /* sveor_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32084 /* sveor_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32096 /* sveor_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32108 /* sveor_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32120 /* sveor_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32132 /* sveor_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32144 /* sveor_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32156 /* sveor_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32168 /* sveor_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32180 /* sveor_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32192 /* sveor_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32203 /* sveor_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32214 /* sveor_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32225 /* sveor_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32237 /* sveor_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32249 /* sveor_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32261 /* sveor_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32273 /* sveor_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32285 /* sveor_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32297 /* sveor_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32309 /* sveor_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32321 /* sveor_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32333 /* sveor_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32344 /* sveor_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32355 /* sveor_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32366 /* sveorbt_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32380 /* sveorbt_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32394 /* sveorbt_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32408 /* sveorbt_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32421 /* sveorbt_n_u16 */, 138 /* q8Usq8Usq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32435 /* sveorbt_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32449 /* sveorbt_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32463 /* sveorbt_n_u8 */, 222 /* q16Ucq16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32476 /* sveorbt_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32488 /* sveorbt_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32500 /* sveorbt_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32512 /* sveorbt_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32523 /* sveorbt_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32535 /* sveorbt_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32547 /* sveorbt_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32559 /* sveorbt_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32570 /* sveorqv_s16 */, 6947 /* V8sq16bq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32582 /* sveorqv_s32 */, 6970 /* V4iq16bq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32594 /* sveorqv_s64 */, 6993 /* V2Wiq16bq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32606 /* sveorqv_s8 */, 7017 /* V16Scq16bq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32617 /* sveorqv_u16 */, 7044 /* V8Usq16bq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32629 /* sveorqv_u32 */, 7069 /* V4Uiq16bq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32641 /* sveorqv_u64 */, 7094 /* V2UWiq16bq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32653 /* sveorqv_u8 */, 7120 /* V16Ucq16bq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32664 /* sveortb_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32678 /* sveortb_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32692 /* sveortb_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32706 /* sveortb_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32719 /* sveortb_n_u16 */, 138 /* q8Usq8Usq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32733 /* sveortb_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32747 /* sveortb_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32761 /* sveortb_n_u8 */, 222 /* q16Ucq16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32774 /* sveortb_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32786 /* sveortb_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32798 /* sveortb_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32810 /* sveortb_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32821 /* sveortb_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32833 /* sveortb_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32845 /* sveortb_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32857 /* sveortb_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32868 /* sveorv_s16 */, 10150 /* sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32879 /* sveorv_s32 */, 10170 /* iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32890 /* sveorv_s64 */, 7248 /* Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32901 /* sveorv_s8 */, 10200 /* Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32911 /* sveorv_u16 */, 10223 /* Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32922 /* sveorv_u32 */, 10245 /* Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32933 /* sveorv_u64 */, 7338 /* UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32944 /* sveorv_u8 */, 10277 /* Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32954 /* svexpa_f16 */, 32965 /* q8hq8Us */, 1 /* n */, 32973 /* sve|(sme,sme2,ssve-fexpa) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32999 /* svexpa_f32 */, 33010 /* q4fq4Ui */, 1 /* n */, 32973 /* sve|(sme,sme2,ssve-fexpa) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33018 /* svexpa_f64 */, 33029 /* q2dq2UWi */, 1 /* n */, 32973 /* sve|(sme,sme2,ssve-fexpa) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33038 /* svexpand_bf16 */, 21785 /* q8yq16bq8y */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33052 /* svexpand_f16 */, 2638 /* q8hq16bq8h */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33065 /* svexpand_f32 */, 2699 /* q4fq16bq4f */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33078 /* svexpand_f64 */, 2760 /* q2dq16bq2d */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33091 /* svexpand_mf8 */, 21904 /* q16mq16bq16m */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33104 /* svexpand_s16 */, 2821 /* q8sq16bq8s */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33117 /* svexpand_s32 */, 2882 /* q4iq16bq4i */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33130 /* svexpand_s64 */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33143 /* svexpand_s8 */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33155 /* svexpand_u16 */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33168 /* svexpand_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33181 /* svexpand_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33194 /* svexpand_u8 */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 21796 /* (sve,(sve2p2|sme2p2))|(sme,sme2p2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33206 /* svext_bf16 */, 33217 /* q8yq8yq8yIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33231 /* svext_f16 */, 33241 /* q8hq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33255 /* svext_f32 */, 33265 /* q4fq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33279 /* svext_f64 */, 33289 /* q2dq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33303 /* svext_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33313 /* svext_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33323 /* svext_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33333 /* svext_s8 */, 13835 /* q16Scq16Scq16ScIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33342 /* svext_u16 */, 13866 /* q8Usq8Usq8UsIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33352 /* svext_u32 */, 13894 /* q4Uiq4Uiq4UiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33362 /* svext_u64 */, 13922 /* q2UWiq2UWiq2UWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33372 /* svext_u8 */, 13952 /* q16Ucq16Ucq16UcIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33381 /* svextb_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33394 /* svextb_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33407 /* svextb_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33420 /* svextb_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33433 /* svextb_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33446 /* svextb_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33459 /* svextb_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33472 /* svextb_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33485 /* svextb_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33498 /* svextb_u16_m */, 16483 /* q8Usq8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33511 /* svextb_u16_x */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33524 /* svextb_u16_z */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33537 /* svextb_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33550 /* svextb_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33563 /* svextb_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33576 /* svextb_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33589 /* svextb_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33602 /* svextb_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33615 /* svexth_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33628 /* svexth_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33641 /* svexth_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33654 /* svexth_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33667 /* svexth_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33680 /* svexth_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33693 /* svexth_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33706 /* svexth_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33719 /* svexth_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33732 /* svexth_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33745 /* svexth_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33758 /* svexth_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33771 /* svextq_bf16 */, 33783 /* q8yq8yq8yi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33794 /* svextq_f16 */, 33805 /* q8hq8hq8hi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33816 /* svextq_f32 */, 33827 /* q4fq4fq4fi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33838 /* svextq_f64 */, 33849 /* q2dq2dq2di */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33860 /* svextq_mf8 */, 33871 /* q16mq16mq16mi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33885 /* svextq_s16 */, 33896 /* q8sq8sq8si */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33907 /* svextq_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33918 /* svextq_s64 */, 33929 /* q2Wiq2Wiq2Wii */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33943 /* svextq_s8 */, 33953 /* q16Scq16Scq16Sci */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33970 /* svextq_u16 */, 33981 /* q8Usq8Usq8Usi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33995 /* svextq_u32 */, 34006 /* q4Uiq4Uiq4Uii */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34020 /* svextq_u64 */, 34031 /* q2UWiq2UWiq2UWii */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34048 /* svextq_u8 */, 34058 /* q16Ucq16Ucq16Uci */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34075 /* svextw_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34088 /* svextw_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34101 /* svextw_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34114 /* svextw_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34127 /* svextw_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34140 /* svextw_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34153 /* svfirstp_b16 */, 34166 /* Wiq16bq16b */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34177 /* svfirstp_b32 */, 34166 /* Wiq16bq16b */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34190 /* svfirstp_b64 */, 34166 /* Wiq16bq16b */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34203 /* svfirstp_b8 */, 34166 /* Wiq16bq16b */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34215 /* svget2_b */, 34224 /* q16bq32bIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34237 /* svget2_bf16 */, 34249 /* q8yq16yIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34261 /* svget2_f16 */, 34272 /* q8hq16hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34284 /* svget2_f32 */, 34295 /* q4fq8fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34306 /* svget2_f64 */, 34317 /* q2dq4dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34328 /* svget2_mf8 */, 34339 /* q16mq32mIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34352 /* svget2_s16 */, 34363 /* q8sq16sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34375 /* svget2_s32 */, 34386 /* q4iq8iIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34397 /* svget2_s64 */, 34408 /* q2Wiq4WiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34421 /* svget2_s8 */, 34431 /* q16Scq32ScIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34446 /* svget2_u16 */, 34457 /* q8Usq16UsIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34471 /* svget2_u32 */, 34482 /* q4Uiq8UiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34495 /* svget2_u64 */, 34506 /* q2UWiq4UWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34521 /* svget2_u8 */, 34531 /* q16Ucq32UcIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34546 /* svget3_bf16 */, 34558 /* q8yq24yIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34570 /* svget3_f16 */, 34581 /* q8hq24hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34593 /* svget3_f32 */, 34604 /* q4fq12fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34616 /* svget3_f64 */, 34627 /* q2dq6dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34638 /* svget3_mf8 */, 34649 /* q16mq48mIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34662 /* svget3_s16 */, 34673 /* q8sq24sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34685 /* svget3_s32 */, 34696 /* q4iq12iIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34708 /* svget3_s64 */, 34719 /* q2Wiq6WiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34732 /* svget3_s8 */, 34742 /* q16Scq48ScIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34757 /* svget3_u16 */, 34768 /* q8Usq24UsIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34782 /* svget3_u32 */, 34793 /* q4Uiq12UiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34807 /* svget3_u64 */, 34818 /* q2UWiq6UWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34833 /* svget3_u8 */, 34843 /* q16Ucq48UcIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34858 /* svget4_b */, 34867 /* q16bq64bIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34880 /* svget4_bf16 */, 34892 /* q8yq32yIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34904 /* svget4_f16 */, 34915 /* q8hq32hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34927 /* svget4_f32 */, 34938 /* q4fq16fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34950 /* svget4_f64 */, 34961 /* q2dq8dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34972 /* svget4_mf8 */, 34983 /* q16mq64mIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34996 /* svget4_s16 */, 35007 /* q8sq32sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35019 /* svget4_s32 */, 35030 /* q4iq16iIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35042 /* svget4_s64 */, 35053 /* q2Wiq8WiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35066 /* svget4_s8 */, 35076 /* q16Scq64ScIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35091 /* svget4_u16 */, 35102 /* q8Usq32UsIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35116 /* svget4_u32 */, 35127 /* q4Uiq16UiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35141 /* svget4_u64 */, 35152 /* q2UWiq8UWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35167 /* svget4_u8 */, 35177 /* q16Ucq64UcIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35192 /* svhadd_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35207 /* svhadd_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35222 /* svhadd_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35237 /* svhadd_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35252 /* svhadd_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35267 /* svhadd_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35282 /* svhadd_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35297 /* svhadd_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35312 /* svhadd_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35327 /* svhadd_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35341 /* svhadd_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35355 /* svhadd_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35369 /* svhadd_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35384 /* svhadd_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35399 /* svhadd_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35414 /* svhadd_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35429 /* svhadd_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35444 /* svhadd_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35459 /* svhadd_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35474 /* svhadd_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35489 /* svhadd_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35504 /* svhadd_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35518 /* svhadd_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35532 /* svhadd_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35546 /* svhadd_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35559 /* svhadd_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35572 /* svhadd_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35585 /* svhadd_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35598 /* svhadd_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35611 /* svhadd_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35624 /* svhadd_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35637 /* svhadd_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35650 /* svhadd_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35663 /* svhadd_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35675 /* svhadd_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35687 /* svhadd_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35699 /* svhadd_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35712 /* svhadd_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35725 /* svhadd_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35738 /* svhadd_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35751 /* svhadd_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35764 /* svhadd_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35777 /* svhadd_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35790 /* svhadd_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35803 /* svhadd_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35816 /* svhadd_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35828 /* svhadd_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35840 /* svhadd_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35852 /* svhistcnt_s32_z */, 35868 /* q4Uiq16bq4iq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35892 /* svhistcnt_s64_z */, 35908 /* q2UWiq16bq2Wiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35926 /* svhistcnt_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35942 /* svhistcnt_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35958 /* svhistseg_s8 */, 35971 /* q16Ucq16Scq16Sc */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35987 /* svhistseg_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36000 /* svhsub_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36015 /* svhsub_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36030 /* svhsub_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36045 /* svhsub_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36060 /* svhsub_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36075 /* svhsub_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36090 /* svhsub_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36105 /* svhsub_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36120 /* svhsub_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36135 /* svhsub_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36149 /* svhsub_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36163 /* svhsub_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36177 /* svhsub_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36192 /* svhsub_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36207 /* svhsub_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36222 /* svhsub_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36237 /* svhsub_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36252 /* svhsub_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36267 /* svhsub_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36282 /* svhsub_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36297 /* svhsub_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36312 /* svhsub_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36326 /* svhsub_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36340 /* svhsub_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36354 /* svhsub_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36367 /* svhsub_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36380 /* svhsub_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36393 /* svhsub_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36406 /* svhsub_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36419 /* svhsub_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36432 /* svhsub_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36445 /* svhsub_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36458 /* svhsub_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36471 /* svhsub_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36483 /* svhsub_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36495 /* svhsub_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36507 /* svhsub_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36520 /* svhsub_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36533 /* svhsub_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36546 /* svhsub_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36559 /* svhsub_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36572 /* svhsub_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36585 /* svhsub_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36598 /* svhsub_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36611 /* svhsub_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36624 /* svhsub_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36636 /* svhsub_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36648 /* svhsub_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36660 /* svhsubr_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36676 /* svhsubr_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36692 /* svhsubr_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36708 /* svhsubr_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36724 /* svhsubr_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36740 /* svhsubr_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36756 /* svhsubr_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36772 /* svhsubr_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36788 /* svhsubr_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36804 /* svhsubr_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36819 /* svhsubr_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36834 /* svhsubr_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36849 /* svhsubr_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36865 /* svhsubr_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36881 /* svhsubr_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36897 /* svhsubr_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36913 /* svhsubr_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36929 /* svhsubr_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36945 /* svhsubr_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36961 /* svhsubr_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36977 /* svhsubr_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36993 /* svhsubr_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37008 /* svhsubr_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37023 /* svhsubr_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37038 /* svhsubr_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37052 /* svhsubr_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37066 /* svhsubr_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37080 /* svhsubr_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37094 /* svhsubr_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37108 /* svhsubr_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37122 /* svhsubr_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37136 /* svhsubr_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37150 /* svhsubr_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37164 /* svhsubr_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37177 /* svhsubr_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37190 /* svhsubr_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37203 /* svhsubr_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37217 /* svhsubr_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37231 /* svhsubr_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37245 /* svhsubr_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37259 /* svhsubr_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37273 /* svhsubr_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37287 /* svhsubr_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37301 /* svhsubr_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37315 /* svhsubr_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37329 /* svhsubr_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37342 /* svhsubr_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37355 /* svhsubr_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37368 /* svindex_s16 */, 37380 /* q8sss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37386 /* svindex_s32 */, 37398 /* q4iii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37404 /* svindex_s64 */, 31362 /* q2WiWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37416 /* svindex_s8 */, 37427 /* q16ScScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37437 /* svindex_u16 */, 37449 /* q8UsUsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37458 /* svindex_u32 */, 37470 /* q4UiUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37479 /* svindex_u64 */, 31494 /* q2UWiUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37491 /* svindex_u8 */, 37502 /* q16UcUcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37512 /* svinsr_n_bf16 */, 37526 /* q8yq8yy */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37534 /* svinsr_n_f16 */, 37547 /* q8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37555 /* svinsr_n_f32 */, 37568 /* q4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37576 /* svinsr_n_f64 */, 37589 /* q2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37597 /* svinsr_n_s16 */, 37610 /* q8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37618 /* svinsr_n_s32 */, 37631 /* q4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37639 /* svinsr_n_s64 */, 37652 /* q2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37663 /* svinsr_n_s8 */, 37675 /* q16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37688 /* svinsr_n_u16 */, 11582 /* q8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37701 /* svinsr_n_u32 */, 11643 /* q4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37714 /* svinsr_n_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37727 /* svinsr_n_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37739 /* svlasta_bf16 */, 37752 /* yq16bq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37761 /* svlasta_f16 */, 7146 /* hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37773 /* svlasta_f32 */, 7166 /* fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37785 /* svlasta_f64 */, 7186 /* dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37797 /* svlasta_s16 */, 10150 /* sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37809 /* svlasta_s32 */, 10170 /* iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37821 /* svlasta_s64 */, 7248 /* Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37833 /* svlasta_s8 */, 10200 /* Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37844 /* svlasta_u16 */, 10223 /* Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37856 /* svlasta_u32 */, 10245 /* Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37868 /* svlasta_u64 */, 7338 /* UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37880 /* svlasta_u8 */, 10277 /* Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37891 /* svlastb_bf16 */, 37752 /* yq16bq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37904 /* svlastb_f16 */, 7146 /* hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37916 /* svlastb_f32 */, 7166 /* fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37928 /* svlastb_f64 */, 7186 /* dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37940 /* svlastb_s16 */, 10150 /* sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37952 /* svlastb_s32 */, 10170 /* iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37964 /* svlastb_s64 */, 7248 /* Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37976 /* svlastb_s8 */, 10200 /* Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37987 /* svlastb_u16 */, 10223 /* Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37999 /* svlastb_u32 */, 10245 /* Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38011 /* svlastb_u64 */, 7338 /* UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38023 /* svlastb_u8 */, 10277 /* Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38034 /* svlastp_b16 */, 34166 /* Wiq16bq16b */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38046 /* svlastp_b32 */, 34166 /* Wiq16bq16b */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38058 /* svlastp_b64 */, 34166 /* Wiq16bq16b */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38070 /* svlastp_b8 */, 34166 /* Wiq16bq16b */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38081 /* svld1_bf16 */, 38092 /* q8yq16byC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38103 /* svld1_bf16_x2 */, 38117 /* q16yQayC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38127 /* svld1_bf16_x4 */, 38141 /* q32yQayC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38151 /* svld1_f16 */, 38161 /* q8hq16bhC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38172 /* svld1_f16_x2 */, 38185 /* q16hQahC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38195 /* svld1_f16_x4 */, 38208 /* q32hQahC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38218 /* svld1_f32 */, 38228 /* q4fq16bfC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38239 /* svld1_f32_x2 */, 38252 /* q8fQafC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38261 /* svld1_f32_x4 */, 38274 /* q16fQafC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38284 /* svld1_f64 */, 38294 /* q2dq16bdC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38305 /* svld1_f64_x2 */, 38318 /* q4dQadC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38327 /* svld1_f64_x4 */, 38340 /* q8dQadC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38349 /* svld1_gather_s32index_f32 */, 38375 /* q4fq16bfC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38389 /* svld1_gather_s32index_s32 */, 38415 /* q4iq16bSiC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38430 /* svld1_gather_s32index_u32 */, 38456 /* q4Uiq16bUiC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38472 /* svld1_gather_s32offset_f32 */, 38375 /* q4fq16bfC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38499 /* svld1_gather_s32offset_s32 */, 38415 /* q4iq16bSiC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38526 /* svld1_gather_s32offset_u32 */, 38456 /* q4Uiq16bUiC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38553 /* svld1_gather_s64index_f64 */, 38579 /* q2dq16bdC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38594 /* svld1_gather_s64index_s64 */, 38620 /* q2Wiq16bSWiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38638 /* svld1_gather_s64index_u64 */, 38664 /* q2UWiq16bUWiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38683 /* svld1_gather_s64offset_f64 */, 38579 /* q2dq16bdC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38710 /* svld1_gather_s64offset_s64 */, 38620 /* q2Wiq16bSWiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38737 /* svld1_gather_s64offset_u64 */, 38664 /* q2UWiq16bUWiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38764 /* svld1_gather_u32base_f32 */, 24686 /* q4fq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38789 /* svld1_gather_u32base_index_f32 */, 38820 /* q4fq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38834 /* svld1_gather_u32base_index_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38879 /* svld1_gather_u32base_index_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38925 /* svld1_gather_u32base_offset_f32 */, 38820 /* q4fq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38957 /* svld1_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38989 /* svld1_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39021 /* svld1_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39058 /* svld1_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39083 /* svld1_gather_u32index_f32 */, 39109 /* q4fq16bfC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39124 /* svld1_gather_u32index_s32 */, 39150 /* q4iq16bSiC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39166 /* svld1_gather_u32index_u32 */, 39192 /* q4Uiq16bUiC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39209 /* svld1_gather_u32offset_f32 */, 39109 /* q4fq16bfC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39236 /* svld1_gather_u32offset_s32 */, 39150 /* q4iq16bSiC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39263 /* svld1_gather_u32offset_u32 */, 39192 /* q4Uiq16bUiC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39290 /* svld1_gather_u64base_f64 */, 25260 /* q2dq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39315 /* svld1_gather_u64base_index_f64 */, 39346 /* q2dq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39361 /* svld1_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39408 /* svld1_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39456 /* svld1_gather_u64base_offset_f64 */, 39346 /* q2dq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39488 /* svld1_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39520 /* svld1_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39552 /* svld1_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39591 /* svld1_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39616 /* svld1_gather_u64index_f64 */, 39642 /* q2dq16bdC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39658 /* svld1_gather_u64index_s64 */, 39684 /* q2Wiq16bSWiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39703 /* svld1_gather_u64index_u64 */, 39729 /* q2UWiq16bUWiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39749 /* svld1_gather_u64offset_f64 */, 39642 /* q2dq16bdC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39776 /* svld1_gather_u64offset_s64 */, 39684 /* q2Wiq16bSWiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39803 /* svld1_gather_u64offset_u64 */, 39729 /* q2UWiq16bUWiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39830 /* svld1_mf8 */, 39840 /* q16mq16bmC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39852 /* svld1_mf8_x2 */, 39865 /* q32mQamC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39875 /* svld1_mf8_x4 */, 39888 /* q64mQamC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39898 /* svld1_s16 */, 39908 /* q8sq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39920 /* svld1_s16_x2 */, 39933 /* q16sQaSsC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39944 /* svld1_s16_x4 */, 39957 /* q32sQaSsC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39968 /* svld1_s32 */, 39978 /* q4iq16bSiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39990 /* svld1_s32_x2 */, 40003 /* q8iQaSiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40013 /* svld1_s32_x4 */, 40026 /* q16iQaSiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40037 /* svld1_s64 */, 40047 /* q2Wiq16bSWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40061 /* svld1_s64_x2 */, 40074 /* q4WiQaSWiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40086 /* svld1_s64_x4 */, 40099 /* q8WiQaSWiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40111 /* svld1_s8 */, 40120 /* q16Scq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40134 /* svld1_s8_x2 */, 40146 /* q32ScQaScC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40158 /* svld1_s8_x4 */, 40170 /* q64ScQaScC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40182 /* svld1_u16 */, 40192 /* q8Usq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40205 /* svld1_u16_x2 */, 40218 /* q16UsQaUsC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40230 /* svld1_u16_x4 */, 40243 /* q32UsQaUsC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40255 /* svld1_u32 */, 40265 /* q4Uiq16bUiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40278 /* svld1_u32_x2 */, 40291 /* q8UiQaUiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40302 /* svld1_u32_x4 */, 40315 /* q16UiQaUiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40327 /* svld1_u64 */, 40337 /* q2UWiq16bUWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40352 /* svld1_u64_x2 */, 40365 /* q4UWiQaUWiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40378 /* svld1_u64_x4 */, 40391 /* q8UWiQaUWiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40404 /* svld1_u8 */, 40413 /* q16Ucq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40427 /* svld1_u8_x2 */, 40439 /* q32UcQaUcC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40451 /* svld1_u8_x4 */, 40463 /* q64UcQaUcC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40475 /* svld1_vnum_bf16 */, 40491 /* q8yq16byC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40504 /* svld1_vnum_bf16_x2 */, 40523 /* q16yQayC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40535 /* svld1_vnum_bf16_x4 */, 40554 /* q32yQayC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40566 /* svld1_vnum_f16 */, 40581 /* q8hq16bhC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40594 /* svld1_vnum_f16_x2 */, 40612 /* q16hQahC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40624 /* svld1_vnum_f16_x4 */, 40642 /* q32hQahC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40654 /* svld1_vnum_f32 */, 40669 /* q4fq16bfC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40682 /* svld1_vnum_f32_x2 */, 40700 /* q8fQafC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40711 /* svld1_vnum_f32_x4 */, 40729 /* q16fQafC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40741 /* svld1_vnum_f64 */, 40756 /* q2dq16bdC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40769 /* svld1_vnum_f64_x2 */, 40787 /* q4dQadC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40798 /* svld1_vnum_f64_x4 */, 40816 /* q8dQadC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40827 /* svld1_vnum_mf8 */, 40842 /* q16mq16bmC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40856 /* svld1_vnum_mf8_x2 */, 40874 /* q32mQamC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40886 /* svld1_vnum_mf8_x4 */, 40904 /* q64mQamC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40916 /* svld1_vnum_s16 */, 40931 /* q8sq16bSsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40945 /* svld1_vnum_s16_x2 */, 40963 /* q16sQaSsC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{40976 /* svld1_vnum_s16_x4 */, 40994 /* q32sQaSsC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41007 /* svld1_vnum_s32 */, 41022 /* q4iq16bSiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41036 /* svld1_vnum_s32_x2 */, 41054 /* q8iQaSiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41066 /* svld1_vnum_s32_x4 */, 41084 /* q16iQaSiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41097 /* svld1_vnum_s64 */, 41112 /* q2Wiq16bSWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41128 /* svld1_vnum_s64_x2 */, 41146 /* q4WiQaSWiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41160 /* svld1_vnum_s64_x4 */, 41178 /* q8WiQaSWiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41192 /* svld1_vnum_s8 */, 41206 /* q16Scq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41222 /* svld1_vnum_s8_x2 */, 41239 /* q32ScQaScC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41253 /* svld1_vnum_s8_x4 */, 41270 /* q64ScQaScC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41284 /* svld1_vnum_u16 */, 41299 /* q8Usq16bUsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41314 /* svld1_vnum_u16_x2 */, 41332 /* q16UsQaUsC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41346 /* svld1_vnum_u16_x4 */, 41364 /* q32UsQaUsC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41378 /* svld1_vnum_u32 */, 41393 /* q4Uiq16bUiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41408 /* svld1_vnum_u32_x2 */, 41426 /* q8UiQaUiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41439 /* svld1_vnum_u32_x4 */, 41457 /* q16UiQaUiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41471 /* svld1_vnum_u64 */, 41486 /* q2UWiq16bUWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41503 /* svld1_vnum_u64_x2 */, 41521 /* q4UWiQaUWiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41536 /* svld1_vnum_u64_x4 */, 41554 /* q8UWiQaUWiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41569 /* svld1_vnum_u8 */, 41583 /* q16Ucq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41599 /* svld1_vnum_u8_x2 */, 41616 /* q32UcQaUcC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41630 /* svld1_vnum_u8_x4 */, 41647 /* q64UcQaUcC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41661 /* svld1q_gather_u64base_bf16 */, 41688 /* q8yq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41712 /* svld1q_gather_u64base_f16 */, 24227 /* q8hq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41738 /* svld1q_gather_u64base_f32 */, 24814 /* q4fq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41764 /* svld1q_gather_u64base_f64 */, 25260 /* q2dq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41790 /* svld1q_gather_u64base_index_bf16 */, 41823 /* q8yq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41838 /* svld1q_gather_u64base_index_f16 */, 41870 /* q8hq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41885 /* svld1q_gather_u64base_index_f32 */, 41917 /* q4fq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41932 /* svld1q_gather_u64base_index_f64 */, 39346 /* q2dq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{41964 /* svld1q_gather_u64base_index_s16 */, 41996 /* q8sq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42011 /* svld1q_gather_u64base_index_s32 */, 42043 /* q4iq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42058 /* svld1q_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42090 /* svld1q_gather_u64base_index_u16 */, 42122 /* q8Usq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42138 /* svld1q_gather_u64base_index_u32 */, 42170 /* q4Uiq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42186 /* svld1q_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42218 /* svld1q_gather_u64base_mf8 */, 42244 /* q16mq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42258 /* svld1q_gather_u64base_offset_bf16 */, 41823 /* q8yq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42292 /* svld1q_gather_u64base_offset_f16 */, 41870 /* q8hq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42325 /* svld1q_gather_u64base_offset_f32 */, 41917 /* q4fq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42358 /* svld1q_gather_u64base_offset_f64 */, 39346 /* q2dq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42391 /* svld1q_gather_u64base_offset_mf8 */, 42424 /* q16mq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42440 /* svld1q_gather_u64base_offset_s16 */, 41996 /* q8sq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42473 /* svld1q_gather_u64base_offset_s32 */, 42043 /* q4iq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42506 /* svld1q_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42539 /* svld1q_gather_u64base_offset_s8 */, 42571 /* q16Scq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42588 /* svld1q_gather_u64base_offset_u16 */, 42122 /* q8Usq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42621 /* svld1q_gather_u64base_offset_u32 */, 42170 /* q4Uiq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42654 /* svld1q_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42687 /* svld1q_gather_u64base_offset_u8 */, 42719 /* q16Ucq16bq2UWiWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42736 /* svld1q_gather_u64base_s16 */, 42762 /* q8sq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42775 /* svld1q_gather_u64base_s32 */, 42801 /* q4iq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42814 /* svld1q_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42840 /* svld1q_gather_u64base_s8 */, 42865 /* q16Scq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42880 /* svld1q_gather_u64base_u16 */, 42906 /* q8Usq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42920 /* svld1q_gather_u64base_u32 */, 42946 /* q4Uiq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42960 /* svld1q_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{42986 /* svld1q_gather_u64base_u8 */, 43011 /* q16Ucq16bq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43026 /* svld1q_gather_u64index_bf16 */, 43054 /* q8yq16byC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43070 /* svld1q_gather_u64index_f16 */, 43097 /* q8hq16bhC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43113 /* svld1q_gather_u64index_f32 */, 43140 /* q4fq16bfC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43156 /* svld1q_gather_u64index_f64 */, 39642 /* q2dq16bdC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43183 /* svld1q_gather_u64index_s16 */, 43210 /* q8sq16bSsC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43227 /* svld1q_gather_u64index_s32 */, 43254 /* q4iq16bSiC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43271 /* svld1q_gather_u64index_s64 */, 39684 /* q2Wiq16bSWiC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43298 /* svld1q_gather_u64index_u16 */, 43325 /* q8Usq16bUsC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43343 /* svld1q_gather_u64index_u32 */, 43370 /* q4Uiq16bUiC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43388 /* svld1q_gather_u64index_u64 */, 39729 /* q2UWiq16bUWiC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43415 /* svld1q_gather_u64offset_bf16 */, 43054 /* q8yq16byC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43444 /* svld1q_gather_u64offset_f16 */, 43097 /* q8hq16bhC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43472 /* svld1q_gather_u64offset_f32 */, 43140 /* q4fq16bfC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43500 /* svld1q_gather_u64offset_f64 */, 39642 /* q2dq16bdC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43528 /* svld1q_gather_u64offset_mf8 */, 43556 /* q16mq16bmC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43573 /* svld1q_gather_u64offset_s16 */, 43210 /* q8sq16bSsC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43601 /* svld1q_gather_u64offset_s32 */, 43254 /* q4iq16bSiC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43629 /* svld1q_gather_u64offset_s64 */, 39684 /* q2Wiq16bSWiC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43657 /* svld1q_gather_u64offset_s8 */, 43684 /* q16Scq16bScC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43703 /* svld1q_gather_u64offset_u16 */, 43325 /* q8Usq16bUsC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43731 /* svld1q_gather_u64offset_u32 */, 43370 /* q4Uiq16bUiC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43759 /* svld1q_gather_u64offset_u64 */, 39729 /* q2UWiq16bUWiC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43787 /* svld1q_gather_u64offset_u8 */, 43814 /* q16Ucq16bUcC*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43833 /* svld1ro_bf16 */, 38092 /* q8yq16byC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43856 /* svld1ro_f16 */, 38161 /* q8hq16bhC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43868 /* svld1ro_f32 */, 38228 /* q4fq16bfC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43880 /* svld1ro_f64 */, 38294 /* q2dq16bdC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43892 /* svld1ro_mf8 */, 39840 /* q16mq16bmC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43904 /* svld1ro_s16 */, 39908 /* q8sq16bSsC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43916 /* svld1ro_s32 */, 39978 /* q4iq16bSiC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43928 /* svld1ro_s64 */, 40047 /* q2Wiq16bSWiC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43940 /* svld1ro_s8 */, 40120 /* q16Scq16bScC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43951 /* svld1ro_u16 */, 40192 /* q8Usq16bUsC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43963 /* svld1ro_u32 */, 40265 /* q4Uiq16bUiC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43975 /* svld1ro_u64 */, 40337 /* q2UWiq16bUWiC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43987 /* svld1ro_u8 */, 40413 /* q16Ucq16bUcC* */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{43998 /* svld1rq_bf16 */, 38092 /* q8yq16byC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44011 /* svld1rq_f16 */, 38161 /* q8hq16bhC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44023 /* svld1rq_f32 */, 38228 /* q4fq16bfC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44035 /* svld1rq_f64 */, 38294 /* q2dq16bdC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44047 /* svld1rq_mf8 */, 39840 /* q16mq16bmC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44059 /* svld1rq_s16 */, 39908 /* q8sq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44071 /* svld1rq_s32 */, 39978 /* q4iq16bSiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44083 /* svld1rq_s64 */, 40047 /* q2Wiq16bSWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44095 /* svld1rq_s8 */, 40120 /* q16Scq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44106 /* svld1rq_u16 */, 40192 /* q8Usq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44118 /* svld1rq_u32 */, 40265 /* q4Uiq16bUiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44130 /* svld1rq_u64 */, 40337 /* q2UWiq16bUWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44142 /* svld1rq_u8 */, 40413 /* q16Ucq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44153 /* svld1sb_gather_s32offset_s32 */, 44182 /* q4iq16bScC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44197 /* svld1sb_gather_s32offset_u32 */, 44226 /* q4Uiq16bScC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44242 /* svld1sb_gather_s64offset_s64 */, 44271 /* q2Wiq16bScC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44288 /* svld1sb_gather_s64offset_u64 */, 44317 /* q2UWiq16bScC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44335 /* svld1sb_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44369 /* svld1sb_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44403 /* svld1sb_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44430 /* svld1sb_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44457 /* svld1sb_gather_u32offset_s32 */, 44486 /* q4iq16bScC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44502 /* svld1sb_gather_u32offset_u32 */, 44531 /* q4Uiq16bScC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44548 /* svld1sb_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44582 /* svld1sb_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44616 /* svld1sb_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44643 /* svld1sb_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44670 /* svld1sb_gather_u64offset_s64 */, 44699 /* q2Wiq16bScC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44717 /* svld1sb_gather_u64offset_u64 */, 44746 /* q2UWiq16bScC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44765 /* svld1sb_s16 */, 44777 /* q8sq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44789 /* svld1sb_s32 */, 44801 /* q4iq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44813 /* svld1sb_s64 */, 44825 /* q2Wiq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44838 /* svld1sb_u16 */, 44850 /* q8Usq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44863 /* svld1sb_u32 */, 44875 /* q4Uiq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44888 /* svld1sb_u64 */, 44900 /* q2UWiq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44914 /* svld1sb_vnum_s16 */, 44931 /* q8sq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44945 /* svld1sb_vnum_s32 */, 44962 /* q4iq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44976 /* svld1sb_vnum_s64 */, 44993 /* q2Wiq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45008 /* svld1sb_vnum_u16 */, 45025 /* q8Usq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45040 /* svld1sb_vnum_u32 */, 45057 /* q4Uiq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45072 /* svld1sb_vnum_u64 */, 45089 /* q2UWiq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45105 /* svld1sh_gather_s32index_s32 */, 45133 /* q4iq16bSsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45148 /* svld1sh_gather_s32index_u32 */, 45176 /* q4Uiq16bSsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45192 /* svld1sh_gather_s32offset_s32 */, 45133 /* q4iq16bSsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45221 /* svld1sh_gather_s32offset_u32 */, 45176 /* q4Uiq16bSsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45250 /* svld1sh_gather_s64index_s64 */, 45278 /* q2Wiq16bSsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45295 /* svld1sh_gather_s64index_u64 */, 45323 /* q2UWiq16bSsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45341 /* svld1sh_gather_s64offset_s64 */, 45278 /* q2Wiq16bSsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45370 /* svld1sh_gather_s64offset_u64 */, 45323 /* q2UWiq16bSsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45399 /* svld1sh_gather_u32base_index_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45432 /* svld1sh_gather_u32base_index_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45465 /* svld1sh_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45499 /* svld1sh_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45533 /* svld1sh_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45560 /* svld1sh_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45587 /* svld1sh_gather_u32index_s32 */, 45615 /* q4iq16bSsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45631 /* svld1sh_gather_u32index_u32 */, 45659 /* q4Uiq16bSsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45676 /* svld1sh_gather_u32offset_s32 */, 45615 /* q4iq16bSsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45705 /* svld1sh_gather_u32offset_u32 */, 45659 /* q4Uiq16bSsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45734 /* svld1sh_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45767 /* svld1sh_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45800 /* svld1sh_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45834 /* svld1sh_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45868 /* svld1sh_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45895 /* svld1sh_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45922 /* svld1sh_gather_u64index_s64 */, 45950 /* q2Wiq16bSsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{45968 /* svld1sh_gather_u64index_u64 */, 45996 /* q2UWiq16bSsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46015 /* svld1sh_gather_u64offset_s64 */, 45950 /* q2Wiq16bSsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46044 /* svld1sh_gather_u64offset_u64 */, 45996 /* q2UWiq16bSsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46073 /* svld1sh_s32 */, 46085 /* q4iq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46097 /* svld1sh_s64 */, 46109 /* q2Wiq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46122 /* svld1sh_u32 */, 46134 /* q4Uiq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46147 /* svld1sh_u64 */, 46159 /* q2UWiq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46173 /* svld1sh_vnum_s32 */, 46190 /* q4iq16bSsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46204 /* svld1sh_vnum_s64 */, 46221 /* q2Wiq16bSsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46236 /* svld1sh_vnum_u32 */, 46253 /* q4Uiq16bSsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46268 /* svld1sh_vnum_u64 */, 46285 /* q2UWiq16bSsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46301 /* svld1sw_gather_s64index_s64 */, 46329 /* q2Wiq16bSiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46346 /* svld1sw_gather_s64index_u64 */, 46374 /* q2UWiq16bSiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46392 /* svld1sw_gather_s64offset_s64 */, 46329 /* q2Wiq16bSiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46421 /* svld1sw_gather_s64offset_u64 */, 46374 /* q2UWiq16bSiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46450 /* svld1sw_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46483 /* svld1sw_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46516 /* svld1sw_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46550 /* svld1sw_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46584 /* svld1sw_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46611 /* svld1sw_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46638 /* svld1sw_gather_u64index_s64 */, 46666 /* q2Wiq16bSiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46684 /* svld1sw_gather_u64index_u64 */, 46712 /* q2UWiq16bSiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46731 /* svld1sw_gather_u64offset_s64 */, 46666 /* q2Wiq16bSiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46760 /* svld1sw_gather_u64offset_u64 */, 46712 /* q2UWiq16bSiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46789 /* svld1sw_s64 */, 46801 /* q2Wiq16bSiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46814 /* svld1sw_u64 */, 46826 /* q2UWiq16bSiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46840 /* svld1sw_vnum_s64 */, 46857 /* q2Wiq16bSiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46872 /* svld1sw_vnum_u64 */, 46889 /* q2UWiq16bSiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46905 /* svld1ub_gather_s32offset_s32 */, 46934 /* q4iq16bUcC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46949 /* svld1ub_gather_s32offset_u32 */, 46978 /* q4Uiq16bUcC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{46994 /* svld1ub_gather_s64offset_s64 */, 47023 /* q2Wiq16bUcC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47040 /* svld1ub_gather_s64offset_u64 */, 47069 /* q2UWiq16bUcC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47087 /* svld1ub_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47121 /* svld1ub_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47155 /* svld1ub_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47182 /* svld1ub_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47209 /* svld1ub_gather_u32offset_s32 */, 47238 /* q4iq16bUcC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47254 /* svld1ub_gather_u32offset_u32 */, 47283 /* q4Uiq16bUcC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47300 /* svld1ub_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47334 /* svld1ub_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47368 /* svld1ub_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47395 /* svld1ub_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47422 /* svld1ub_gather_u64offset_s64 */, 47451 /* q2Wiq16bUcC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47469 /* svld1ub_gather_u64offset_u64 */, 47498 /* q2UWiq16bUcC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47517 /* svld1ub_s16 */, 47529 /* q8sq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47541 /* svld1ub_s32 */, 47553 /* q4iq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47565 /* svld1ub_s64 */, 47577 /* q2Wiq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47590 /* svld1ub_u16 */, 47602 /* q8Usq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47615 /* svld1ub_u32 */, 47627 /* q4Uiq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47640 /* svld1ub_u64 */, 47652 /* q2UWiq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47666 /* svld1ub_vnum_s16 */, 47683 /* q8sq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47697 /* svld1ub_vnum_s32 */, 47714 /* q4iq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47728 /* svld1ub_vnum_s64 */, 47745 /* q2Wiq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47760 /* svld1ub_vnum_u16 */, 47777 /* q8Usq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47792 /* svld1ub_vnum_u32 */, 47809 /* q4Uiq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47824 /* svld1ub_vnum_u64 */, 47841 /* q2UWiq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47857 /* svld1udq_f64 */, 38294 /* q2dq16bdC* */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47870 /* svld1udq_s64 */, 40047 /* q2Wiq16bSWiC* */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47883 /* svld1udq_u64 */, 40337 /* q2UWiq16bUWiC* */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47896 /* svld1udq_vnum_f64 */, 40756 /* q2dq16bdC*Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47914 /* svld1udq_vnum_s64 */, 41112 /* q2Wiq16bSWiC*Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47932 /* svld1udq_vnum_u64 */, 41486 /* q2UWiq16bUWiC*Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47950 /* svld1uh_gather_s32index_s32 */, 47978 /* q4iq16bUsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{47993 /* svld1uh_gather_s32index_u32 */, 48021 /* q4Uiq16bUsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48037 /* svld1uh_gather_s32offset_s32 */, 47978 /* q4iq16bUsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48066 /* svld1uh_gather_s32offset_u32 */, 48021 /* q4Uiq16bUsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48095 /* svld1uh_gather_s64index_s64 */, 48123 /* q2Wiq16bUsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48140 /* svld1uh_gather_s64index_u64 */, 48168 /* q2UWiq16bUsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48186 /* svld1uh_gather_s64offset_s64 */, 48123 /* q2Wiq16bUsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48215 /* svld1uh_gather_s64offset_u64 */, 48168 /* q2UWiq16bUsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48244 /* svld1uh_gather_u32base_index_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48277 /* svld1uh_gather_u32base_index_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48310 /* svld1uh_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48344 /* svld1uh_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48378 /* svld1uh_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48405 /* svld1uh_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48432 /* svld1uh_gather_u32index_s32 */, 48460 /* q4iq16bUsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48476 /* svld1uh_gather_u32index_u32 */, 48504 /* q4Uiq16bUsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48521 /* svld1uh_gather_u32offset_s32 */, 48460 /* q4iq16bUsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48550 /* svld1uh_gather_u32offset_u32 */, 48504 /* q4Uiq16bUsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48579 /* svld1uh_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48612 /* svld1uh_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48645 /* svld1uh_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48679 /* svld1uh_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48713 /* svld1uh_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48740 /* svld1uh_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48767 /* svld1uh_gather_u64index_s64 */, 48795 /* q2Wiq16bUsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48813 /* svld1uh_gather_u64index_u64 */, 48841 /* q2UWiq16bUsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48860 /* svld1uh_gather_u64offset_s64 */, 48795 /* q2Wiq16bUsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48889 /* svld1uh_gather_u64offset_u64 */, 48841 /* q2UWiq16bUsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48918 /* svld1uh_s32 */, 48930 /* q4iq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48942 /* svld1uh_s64 */, 48954 /* q2Wiq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48967 /* svld1uh_u32 */, 48979 /* q4Uiq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{48992 /* svld1uh_u64 */, 49004 /* q2UWiq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49018 /* svld1uh_vnum_s32 */, 49035 /* q4iq16bUsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49049 /* svld1uh_vnum_s64 */, 49066 /* q2Wiq16bUsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49081 /* svld1uh_vnum_u32 */, 49098 /* q4Uiq16bUsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49113 /* svld1uh_vnum_u64 */, 49130 /* q2UWiq16bUsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49146 /* svld1uw_gather_s64index_s64 */, 49174 /* q2Wiq16bUiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49191 /* svld1uw_gather_s64index_u64 */, 49219 /* q2UWiq16bUiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49237 /* svld1uw_gather_s64offset_s64 */, 49174 /* q2Wiq16bUiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49266 /* svld1uw_gather_s64offset_u64 */, 49219 /* q2UWiq16bUiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49295 /* svld1uw_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49328 /* svld1uw_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49361 /* svld1uw_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49395 /* svld1uw_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49429 /* svld1uw_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49456 /* svld1uw_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49483 /* svld1uw_gather_u64index_s64 */, 49511 /* q2Wiq16bUiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49529 /* svld1uw_gather_u64index_u64 */, 49557 /* q2UWiq16bUiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49576 /* svld1uw_gather_u64offset_s64 */, 49511 /* q2Wiq16bUiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49605 /* svld1uw_gather_u64offset_u64 */, 49557 /* q2UWiq16bUiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49634 /* svld1uw_s64 */, 49646 /* q2Wiq16bUiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49659 /* svld1uw_u64 */, 49671 /* q2UWiq16bUiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49685 /* svld1uw_vnum_s64 */, 49702 /* q2Wiq16bUiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49717 /* svld1uw_vnum_u64 */, 49734 /* q2UWiq16bUiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49750 /* svld1uwq_f32 */, 38228 /* q4fq16bfC* */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49763 /* svld1uwq_s32 */, 39978 /* q4iq16bSiC* */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49776 /* svld1uwq_u32 */, 40265 /* q4Uiq16bUiC* */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49789 /* svld1uwq_vnum_f32 */, 40669 /* q4fq16bfC*Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49807 /* svld1uwq_vnum_s32 */, 41022 /* q4iq16bSiC*Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49825 /* svld1uwq_vnum_u32 */, 41393 /* q4Uiq16bUiC*Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49843 /* svld2_bf16 */, 49854 /* q16yq16byC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49866 /* svld2_f16 */, 49876 /* q16hq16bhC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49888 /* svld2_f32 */, 49898 /* q8fq16bfC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49909 /* svld2_f64 */, 49919 /* q4dq16bdC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49930 /* svld2_mf8 */, 49940 /* q32mq16bmC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49952 /* svld2_s16 */, 49962 /* q16sq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49975 /* svld2_s32 */, 49985 /* q8iq16bSiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{49997 /* svld2_s64 */, 50007 /* q4Wiq16bSWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50021 /* svld2_s8 */, 50030 /* q32Scq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50044 /* svld2_u16 */, 50054 /* q16Usq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50068 /* svld2_u32 */, 50078 /* q8Uiq16bUiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50091 /* svld2_u64 */, 50101 /* q4UWiq16bUWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50116 /* svld2_u8 */, 50125 /* q32Ucq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50139 /* svld2_vnum_bf16 */, 50155 /* q16yq16byC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50169 /* svld2_vnum_f16 */, 50184 /* q16hq16bhC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50198 /* svld2_vnum_f32 */, 50213 /* q8fq16bfC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50226 /* svld2_vnum_f64 */, 50241 /* q4dq16bdC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50254 /* svld2_vnum_mf8 */, 50269 /* q32mq16bmC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50283 /* svld2_vnum_s16 */, 50298 /* q16sq16bSsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50313 /* svld2_vnum_s32 */, 50328 /* q8iq16bSiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50342 /* svld2_vnum_s64 */, 50357 /* q4Wiq16bSWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50373 /* svld2_vnum_s8 */, 50387 /* q32Scq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50403 /* svld2_vnum_u16 */, 50418 /* q16Usq16bUsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50434 /* svld2_vnum_u32 */, 50449 /* q8Uiq16bUiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50464 /* svld2_vnum_u64 */, 50479 /* q4UWiq16bUWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50496 /* svld2_vnum_u8 */, 50510 /* q32Ucq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50526 /* svld2q_bf16 */, 49854 /* q16yq16byC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50538 /* svld2q_f16 */, 49876 /* q16hq16bhC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50549 /* svld2q_f32 */, 49898 /* q8fq16bfC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50560 /* svld2q_f64 */, 49919 /* q4dq16bdC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50571 /* svld2q_mf8 */, 49940 /* q32mq16bmC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50582 /* svld2q_s16 */, 49962 /* q16sq16bSsC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50593 /* svld2q_s32 */, 49985 /* q8iq16bSiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50604 /* svld2q_s64 */, 50007 /* q4Wiq16bSWiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50615 /* svld2q_s8 */, 50030 /* q32Scq16bScC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50625 /* svld2q_u16 */, 50054 /* q16Usq16bUsC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50636 /* svld2q_u32 */, 50078 /* q8Uiq16bUiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50647 /* svld2q_u64 */, 50101 /* q4UWiq16bUWiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50658 /* svld2q_u8 */, 50125 /* q32Ucq16bUcC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50668 /* svld2q_vnum_bf16 */, 50155 /* q16yq16byC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50685 /* svld2q_vnum_f16 */, 50184 /* q16hq16bhC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50701 /* svld2q_vnum_f32 */, 50213 /* q8fq16bfC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50717 /* svld2q_vnum_f64 */, 50241 /* q4dq16bdC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50733 /* svld2q_vnum_mf8 */, 50269 /* q32mq16bmC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50749 /* svld2q_vnum_s16 */, 50298 /* q16sq16bSsC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50765 /* svld2q_vnum_s32 */, 50328 /* q8iq16bSiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50781 /* svld2q_vnum_s64 */, 50357 /* q4Wiq16bSWiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50797 /* svld2q_vnum_s8 */, 50387 /* q32Scq16bScC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50812 /* svld2q_vnum_u16 */, 50418 /* q16Usq16bUsC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50828 /* svld2q_vnum_u32 */, 50449 /* q8Uiq16bUiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50844 /* svld2q_vnum_u64 */, 50479 /* q4UWiq16bUWiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50860 /* svld2q_vnum_u8 */, 50510 /* q32Ucq16bUcC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50875 /* svld3_bf16 */, 50886 /* q24yq16byC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50898 /* svld3_f16 */, 50908 /* q24hq16bhC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50920 /* svld3_f32 */, 50930 /* q12fq16bfC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50942 /* svld3_f64 */, 50952 /* q6dq16bdC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50963 /* svld3_mf8 */, 50973 /* q48mq16bmC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{50985 /* svld3_s16 */, 50995 /* q24sq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51008 /* svld3_s32 */, 51018 /* q12iq16bSiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51031 /* svld3_s64 */, 51041 /* q6Wiq16bSWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51055 /* svld3_s8 */, 51064 /* q48Scq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51078 /* svld3_u16 */, 51088 /* q24Usq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51102 /* svld3_u32 */, 51112 /* q12Uiq16bUiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51126 /* svld3_u64 */, 51136 /* q6UWiq16bUWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51151 /* svld3_u8 */, 51160 /* q48Ucq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51174 /* svld3_vnum_bf16 */, 51190 /* q24yq16byC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51204 /* svld3_vnum_f16 */, 51219 /* q24hq16bhC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51233 /* svld3_vnum_f32 */, 51248 /* q12fq16bfC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51262 /* svld3_vnum_f64 */, 51277 /* q6dq16bdC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51290 /* svld3_vnum_mf8 */, 51305 /* q48mq16bmC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51319 /* svld3_vnum_s16 */, 51334 /* q24sq16bSsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51349 /* svld3_vnum_s32 */, 51364 /* q12iq16bSiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51379 /* svld3_vnum_s64 */, 51394 /* q6Wiq16bSWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51410 /* svld3_vnum_s8 */, 51424 /* q48Scq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51440 /* svld3_vnum_u16 */, 51455 /* q24Usq16bUsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51471 /* svld3_vnum_u32 */, 51486 /* q12Uiq16bUiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51502 /* svld3_vnum_u64 */, 51517 /* q6UWiq16bUWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51534 /* svld3_vnum_u8 */, 51548 /* q48Ucq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51564 /* svld3q_bf16 */, 50886 /* q24yq16byC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51576 /* svld3q_f16 */, 50908 /* q24hq16bhC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51587 /* svld3q_f32 */, 50930 /* q12fq16bfC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51598 /* svld3q_f64 */, 50952 /* q6dq16bdC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51609 /* svld3q_mf8 */, 50973 /* q48mq16bmC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51620 /* svld3q_s16 */, 50995 /* q24sq16bSsC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51631 /* svld3q_s32 */, 51018 /* q12iq16bSiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51642 /* svld3q_s64 */, 51041 /* q6Wiq16bSWiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51653 /* svld3q_s8 */, 51064 /* q48Scq16bScC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51663 /* svld3q_u16 */, 51088 /* q24Usq16bUsC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51674 /* svld3q_u32 */, 51112 /* q12Uiq16bUiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51685 /* svld3q_u64 */, 51136 /* q6UWiq16bUWiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51696 /* svld3q_u8 */, 51160 /* q48Ucq16bUcC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51706 /* svld3q_vnum_bf16 */, 51190 /* q24yq16byC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51723 /* svld3q_vnum_f16 */, 51219 /* q24hq16bhC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51739 /* svld3q_vnum_f32 */, 51248 /* q12fq16bfC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51755 /* svld3q_vnum_f64 */, 51277 /* q6dq16bdC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51771 /* svld3q_vnum_mf8 */, 51305 /* q48mq16bmC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51787 /* svld3q_vnum_s16 */, 51334 /* q24sq16bSsC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51803 /* svld3q_vnum_s32 */, 51364 /* q12iq16bSiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51819 /* svld3q_vnum_s64 */, 51394 /* q6Wiq16bSWiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51835 /* svld3q_vnum_s8 */, 51424 /* q48Scq16bScC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51850 /* svld3q_vnum_u16 */, 51455 /* q24Usq16bUsC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51866 /* svld3q_vnum_u32 */, 51486 /* q12Uiq16bUiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51882 /* svld3q_vnum_u64 */, 51517 /* q6UWiq16bUWiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51898 /* svld3q_vnum_u8 */, 51548 /* q48Ucq16bUcC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51913 /* svld4_bf16 */, 51924 /* q32yq16byC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51936 /* svld4_f16 */, 51946 /* q32hq16bhC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51958 /* svld4_f32 */, 51968 /* q16fq16bfC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{51980 /* svld4_f64 */, 51990 /* q8dq16bdC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52001 /* svld4_mf8 */, 52011 /* q64mq16bmC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52023 /* svld4_s16 */, 52033 /* q32sq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52046 /* svld4_s32 */, 52056 /* q16iq16bSiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52069 /* svld4_s64 */, 52079 /* q8Wiq16bSWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52093 /* svld4_s8 */, 52102 /* q64Scq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52116 /* svld4_u16 */, 52126 /* q32Usq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52140 /* svld4_u32 */, 52150 /* q16Uiq16bUiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52164 /* svld4_u64 */, 52174 /* q8UWiq16bUWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52189 /* svld4_u8 */, 52198 /* q64Ucq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52212 /* svld4_vnum_bf16 */, 52228 /* q32yq16byC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52242 /* svld4_vnum_f16 */, 52257 /* q32hq16bhC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52271 /* svld4_vnum_f32 */, 52286 /* q16fq16bfC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52300 /* svld4_vnum_f64 */, 52315 /* q8dq16bdC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52328 /* svld4_vnum_mf8 */, 52343 /* q64mq16bmC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52357 /* svld4_vnum_s16 */, 52372 /* q32sq16bSsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52387 /* svld4_vnum_s32 */, 52402 /* q16iq16bSiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52417 /* svld4_vnum_s64 */, 52432 /* q8Wiq16bSWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52448 /* svld4_vnum_s8 */, 52462 /* q64Scq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52478 /* svld4_vnum_u16 */, 52493 /* q32Usq16bUsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52509 /* svld4_vnum_u32 */, 52524 /* q16Uiq16bUiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52540 /* svld4_vnum_u64 */, 52555 /* q8UWiq16bUWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52572 /* svld4_vnum_u8 */, 52586 /* q64Ucq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52602 /* svld4q_bf16 */, 51924 /* q32yq16byC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52614 /* svld4q_f16 */, 51946 /* q32hq16bhC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52625 /* svld4q_f32 */, 51968 /* q16fq16bfC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52636 /* svld4q_f64 */, 51990 /* q8dq16bdC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52647 /* svld4q_mf8 */, 52011 /* q64mq16bmC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52658 /* svld4q_s16 */, 52033 /* q32sq16bSsC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52669 /* svld4q_s32 */, 52056 /* q16iq16bSiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52680 /* svld4q_s64 */, 52079 /* q8Wiq16bSWiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52691 /* svld4q_s8 */, 52102 /* q64Scq16bScC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52701 /* svld4q_u16 */, 52126 /* q32Usq16bUsC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52712 /* svld4q_u32 */, 52150 /* q16Uiq16bUiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52723 /* svld4q_u64 */, 52174 /* q8UWiq16bUWiC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52734 /* svld4q_u8 */, 52198 /* q64Ucq16bUcC* */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52744 /* svld4q_vnum_bf16 */, 52228 /* q32yq16byC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52761 /* svld4q_vnum_f16 */, 52257 /* q32hq16bhC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52777 /* svld4q_vnum_f32 */, 52286 /* q16fq16bfC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52793 /* svld4q_vnum_f64 */, 52315 /* q8dq16bdC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52809 /* svld4q_vnum_mf8 */, 52343 /* q64mq16bmC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52825 /* svld4q_vnum_s16 */, 52372 /* q32sq16bSsC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52841 /* svld4q_vnum_s32 */, 52402 /* q16iq16bSiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52857 /* svld4q_vnum_s64 */, 52432 /* q8Wiq16bSWiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52873 /* svld4q_vnum_s8 */, 52462 /* q64Scq16bScC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52888 /* svld4q_vnum_u16 */, 52493 /* q32Usq16bUsC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52904 /* svld4q_vnum_u32 */, 52524 /* q16Uiq16bUiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52920 /* svld4q_vnum_u64 */, 52555 /* q8UWiq16bUWiC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52936 /* svld4q_vnum_u8 */, 52586 /* q64Ucq16bUcC*Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52951 /* svldff1_bf16 */, 38092 /* q8yq16byC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52964 /* svldff1_f16 */, 38161 /* q8hq16bhC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52976 /* svldff1_f32 */, 38228 /* q4fq16bfC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{52988 /* svldff1_f64 */, 38294 /* q2dq16bdC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53000 /* svldff1_gather_s32index_f32 */, 38375 /* q4fq16bfC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53028 /* svldff1_gather_s32index_s32 */, 38415 /* q4iq16bSiC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53056 /* svldff1_gather_s32index_u32 */, 38456 /* q4Uiq16bUiC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53084 /* svldff1_gather_s32offset_f32 */, 38375 /* q4fq16bfC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53113 /* svldff1_gather_s32offset_s32 */, 38415 /* q4iq16bSiC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53142 /* svldff1_gather_s32offset_u32 */, 38456 /* q4Uiq16bUiC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53171 /* svldff1_gather_s64index_f64 */, 38579 /* q2dq16bdC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53199 /* svldff1_gather_s64index_s64 */, 38620 /* q2Wiq16bSWiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53227 /* svldff1_gather_s64index_u64 */, 38664 /* q2UWiq16bUWiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53255 /* svldff1_gather_s64offset_f64 */, 38579 /* q2dq16bdC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53284 /* svldff1_gather_s64offset_s64 */, 38620 /* q2Wiq16bSWiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53313 /* svldff1_gather_s64offset_u64 */, 38664 /* q2UWiq16bUWiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53342 /* svldff1_gather_u32base_f32 */, 24686 /* q4fq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53369 /* svldff1_gather_u32base_index_f32 */, 38820 /* q4fq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53402 /* svldff1_gather_u32base_index_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53435 /* svldff1_gather_u32base_index_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53468 /* svldff1_gather_u32base_offset_f32 */, 38820 /* q4fq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53502 /* svldff1_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53536 /* svldff1_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53570 /* svldff1_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53597 /* svldff1_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53624 /* svldff1_gather_u32index_f32 */, 39109 /* q4fq16bfC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53652 /* svldff1_gather_u32index_s32 */, 39150 /* q4iq16bSiC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53680 /* svldff1_gather_u32index_u32 */, 39192 /* q4Uiq16bUiC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53708 /* svldff1_gather_u32offset_f32 */, 39109 /* q4fq16bfC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53737 /* svldff1_gather_u32offset_s32 */, 39150 /* q4iq16bSiC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53766 /* svldff1_gather_u32offset_u32 */, 39192 /* q4Uiq16bUiC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53795 /* svldff1_gather_u64base_f64 */, 25260 /* q2dq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53822 /* svldff1_gather_u64base_index_f64 */, 39346 /* q2dq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53855 /* svldff1_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53888 /* svldff1_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53921 /* svldff1_gather_u64base_offset_f64 */, 39346 /* q2dq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53955 /* svldff1_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{53989 /* svldff1_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54023 /* svldff1_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54050 /* svldff1_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54077 /* svldff1_gather_u64index_f64 */, 39642 /* q2dq16bdC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54105 /* svldff1_gather_u64index_s64 */, 39684 /* q2Wiq16bSWiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54133 /* svldff1_gather_u64index_u64 */, 39729 /* q2UWiq16bUWiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54161 /* svldff1_gather_u64offset_f64 */, 39642 /* q2dq16bdC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54190 /* svldff1_gather_u64offset_s64 */, 39684 /* q2Wiq16bSWiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54219 /* svldff1_gather_u64offset_u64 */, 39729 /* q2UWiq16bUWiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54248 /* svldff1_mf8 */, 39840 /* q16mq16bmC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54260 /* svldff1_s16 */, 39908 /* q8sq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54272 /* svldff1_s32 */, 39978 /* q4iq16bSiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54284 /* svldff1_s64 */, 40047 /* q2Wiq16bSWiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54296 /* svldff1_s8 */, 40120 /* q16Scq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54307 /* svldff1_u16 */, 40192 /* q8Usq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54319 /* svldff1_u32 */, 40265 /* q4Uiq16bUiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54331 /* svldff1_u64 */, 40337 /* q2UWiq16bUWiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54343 /* svldff1_u8 */, 40413 /* q16Ucq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54354 /* svldff1_vnum_bf16 */, 40491 /* q8yq16byC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54372 /* svldff1_vnum_f16 */, 40581 /* q8hq16bhC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54389 /* svldff1_vnum_f32 */, 40669 /* q4fq16bfC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54406 /* svldff1_vnum_f64 */, 40756 /* q2dq16bdC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54423 /* svldff1_vnum_mf8 */, 40842 /* q16mq16bmC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54440 /* svldff1_vnum_s16 */, 40931 /* q8sq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54457 /* svldff1_vnum_s32 */, 41022 /* q4iq16bSiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54474 /* svldff1_vnum_s64 */, 41112 /* q2Wiq16bSWiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54491 /* svldff1_vnum_s8 */, 41206 /* q16Scq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54507 /* svldff1_vnum_u16 */, 41299 /* q8Usq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54524 /* svldff1_vnum_u32 */, 41393 /* q4Uiq16bUiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54541 /* svldff1_vnum_u64 */, 41486 /* q2UWiq16bUWiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54558 /* svldff1_vnum_u8 */, 41583 /* q16Ucq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54574 /* svldff1sb_gather_s32offset_s32 */, 44182 /* q4iq16bScC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54605 /* svldff1sb_gather_s32offset_u32 */, 44226 /* q4Uiq16bScC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54636 /* svldff1sb_gather_s64offset_s64 */, 44271 /* q2Wiq16bScC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54667 /* svldff1sb_gather_s64offset_u64 */, 44317 /* q2UWiq16bScC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54698 /* svldff1sb_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54734 /* svldff1sb_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54770 /* svldff1sb_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54799 /* svldff1sb_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54828 /* svldff1sb_gather_u32offset_s32 */, 44486 /* q4iq16bScC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54859 /* svldff1sb_gather_u32offset_u32 */, 44531 /* q4Uiq16bScC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54890 /* svldff1sb_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54926 /* svldff1sb_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54962 /* svldff1sb_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{54991 /* svldff1sb_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55020 /* svldff1sb_gather_u64offset_s64 */, 44699 /* q2Wiq16bScC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55051 /* svldff1sb_gather_u64offset_u64 */, 44746 /* q2UWiq16bScC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55082 /* svldff1sb_s16 */, 44777 /* q8sq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55096 /* svldff1sb_s32 */, 44801 /* q4iq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55110 /* svldff1sb_s64 */, 44825 /* q2Wiq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55124 /* svldff1sb_u16 */, 44850 /* q8Usq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55138 /* svldff1sb_u32 */, 44875 /* q4Uiq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55152 /* svldff1sb_u64 */, 44900 /* q2UWiq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55166 /* svldff1sb_vnum_s16 */, 44931 /* q8sq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55185 /* svldff1sb_vnum_s32 */, 44962 /* q4iq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55204 /* svldff1sb_vnum_s64 */, 44993 /* q2Wiq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55223 /* svldff1sb_vnum_u16 */, 45025 /* q8Usq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55242 /* svldff1sb_vnum_u32 */, 45057 /* q4Uiq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55261 /* svldff1sb_vnum_u64 */, 45089 /* q2UWiq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55280 /* svldff1sh_gather_s32index_s32 */, 45133 /* q4iq16bSsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55310 /* svldff1sh_gather_s32index_u32 */, 45176 /* q4Uiq16bSsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55340 /* svldff1sh_gather_s32offset_s32 */, 45133 /* q4iq16bSsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55371 /* svldff1sh_gather_s32offset_u32 */, 45176 /* q4Uiq16bSsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55402 /* svldff1sh_gather_s64index_s64 */, 45278 /* q2Wiq16bSsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55432 /* svldff1sh_gather_s64index_u64 */, 45323 /* q2UWiq16bSsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55462 /* svldff1sh_gather_s64offset_s64 */, 45278 /* q2Wiq16bSsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55493 /* svldff1sh_gather_s64offset_u64 */, 45323 /* q2UWiq16bSsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55524 /* svldff1sh_gather_u32base_index_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55559 /* svldff1sh_gather_u32base_index_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55594 /* svldff1sh_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55630 /* svldff1sh_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55666 /* svldff1sh_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55695 /* svldff1sh_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55724 /* svldff1sh_gather_u32index_s32 */, 45615 /* q4iq16bSsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55754 /* svldff1sh_gather_u32index_u32 */, 45659 /* q4Uiq16bSsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55784 /* svldff1sh_gather_u32offset_s32 */, 45615 /* q4iq16bSsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55815 /* svldff1sh_gather_u32offset_u32 */, 45659 /* q4Uiq16bSsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55846 /* svldff1sh_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55881 /* svldff1sh_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55916 /* svldff1sh_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55952 /* svldff1sh_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{55988 /* svldff1sh_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56017 /* svldff1sh_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56046 /* svldff1sh_gather_u64index_s64 */, 45950 /* q2Wiq16bSsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56076 /* svldff1sh_gather_u64index_u64 */, 45996 /* q2UWiq16bSsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56106 /* svldff1sh_gather_u64offset_s64 */, 45950 /* q2Wiq16bSsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56137 /* svldff1sh_gather_u64offset_u64 */, 45996 /* q2UWiq16bSsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56168 /* svldff1sh_s32 */, 46085 /* q4iq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56182 /* svldff1sh_s64 */, 46109 /* q2Wiq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56196 /* svldff1sh_u32 */, 46134 /* q4Uiq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56210 /* svldff1sh_u64 */, 46159 /* q2UWiq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56224 /* svldff1sh_vnum_s32 */, 46190 /* q4iq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56243 /* svldff1sh_vnum_s64 */, 46221 /* q2Wiq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56262 /* svldff1sh_vnum_u32 */, 46253 /* q4Uiq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56281 /* svldff1sh_vnum_u64 */, 46285 /* q2UWiq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56300 /* svldff1sw_gather_s64index_s64 */, 46329 /* q2Wiq16bSiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56330 /* svldff1sw_gather_s64index_u64 */, 46374 /* q2UWiq16bSiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56360 /* svldff1sw_gather_s64offset_s64 */, 46329 /* q2Wiq16bSiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56391 /* svldff1sw_gather_s64offset_u64 */, 46374 /* q2UWiq16bSiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56422 /* svldff1sw_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56457 /* svldff1sw_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56492 /* svldff1sw_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56528 /* svldff1sw_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56564 /* svldff1sw_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56593 /* svldff1sw_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56622 /* svldff1sw_gather_u64index_s64 */, 46666 /* q2Wiq16bSiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56652 /* svldff1sw_gather_u64index_u64 */, 46712 /* q2UWiq16bSiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56682 /* svldff1sw_gather_u64offset_s64 */, 46666 /* q2Wiq16bSiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56713 /* svldff1sw_gather_u64offset_u64 */, 46712 /* q2UWiq16bSiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56744 /* svldff1sw_s64 */, 46801 /* q2Wiq16bSiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56758 /* svldff1sw_u64 */, 46826 /* q2UWiq16bSiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56772 /* svldff1sw_vnum_s64 */, 46857 /* q2Wiq16bSiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56791 /* svldff1sw_vnum_u64 */, 46889 /* q2UWiq16bSiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56810 /* svldff1ub_gather_s32offset_s32 */, 46934 /* q4iq16bUcC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56841 /* svldff1ub_gather_s32offset_u32 */, 46978 /* q4Uiq16bUcC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56872 /* svldff1ub_gather_s64offset_s64 */, 47023 /* q2Wiq16bUcC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56903 /* svldff1ub_gather_s64offset_u64 */, 47069 /* q2UWiq16bUcC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56934 /* svldff1ub_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{56970 /* svldff1ub_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57006 /* svldff1ub_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57035 /* svldff1ub_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57064 /* svldff1ub_gather_u32offset_s32 */, 47238 /* q4iq16bUcC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57095 /* svldff1ub_gather_u32offset_u32 */, 47283 /* q4Uiq16bUcC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57126 /* svldff1ub_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57162 /* svldff1ub_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57198 /* svldff1ub_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57227 /* svldff1ub_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57256 /* svldff1ub_gather_u64offset_s64 */, 47451 /* q2Wiq16bUcC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57287 /* svldff1ub_gather_u64offset_u64 */, 47498 /* q2UWiq16bUcC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57318 /* svldff1ub_s16 */, 47529 /* q8sq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57332 /* svldff1ub_s32 */, 47553 /* q4iq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57346 /* svldff1ub_s64 */, 47577 /* q2Wiq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57360 /* svldff1ub_u16 */, 47602 /* q8Usq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57374 /* svldff1ub_u32 */, 47627 /* q4Uiq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57388 /* svldff1ub_u64 */, 47652 /* q2UWiq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57402 /* svldff1ub_vnum_s16 */, 47683 /* q8sq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57421 /* svldff1ub_vnum_s32 */, 47714 /* q4iq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57440 /* svldff1ub_vnum_s64 */, 47745 /* q2Wiq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57459 /* svldff1ub_vnum_u16 */, 47777 /* q8Usq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57478 /* svldff1ub_vnum_u32 */, 47809 /* q4Uiq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57497 /* svldff1ub_vnum_u64 */, 47841 /* q2UWiq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57516 /* svldff1uh_gather_s32index_s32 */, 47978 /* q4iq16bUsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57546 /* svldff1uh_gather_s32index_u32 */, 48021 /* q4Uiq16bUsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57576 /* svldff1uh_gather_s32offset_s32 */, 47978 /* q4iq16bUsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57607 /* svldff1uh_gather_s32offset_u32 */, 48021 /* q4Uiq16bUsC*q4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57638 /* svldff1uh_gather_s64index_s64 */, 48123 /* q2Wiq16bUsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57668 /* svldff1uh_gather_s64index_u64 */, 48168 /* q2UWiq16bUsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57698 /* svldff1uh_gather_s64offset_s64 */, 48123 /* q2Wiq16bUsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57729 /* svldff1uh_gather_s64offset_u64 */, 48168 /* q2UWiq16bUsC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57760 /* svldff1uh_gather_u32base_index_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57795 /* svldff1uh_gather_u32base_index_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57830 /* svldff1uh_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57866 /* svldff1uh_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57902 /* svldff1uh_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57931 /* svldff1uh_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57960 /* svldff1uh_gather_u32index_s32 */, 48460 /* q4iq16bUsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57990 /* svldff1uh_gather_u32index_u32 */, 48504 /* q4Uiq16bUsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58020 /* svldff1uh_gather_u32offset_s32 */, 48460 /* q4iq16bUsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58051 /* svldff1uh_gather_u32offset_u32 */, 48504 /* q4Uiq16bUsC*q4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58082 /* svldff1uh_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58117 /* svldff1uh_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58152 /* svldff1uh_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58188 /* svldff1uh_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58224 /* svldff1uh_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58253 /* svldff1uh_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58282 /* svldff1uh_gather_u64index_s64 */, 48795 /* q2Wiq16bUsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58312 /* svldff1uh_gather_u64index_u64 */, 48841 /* q2UWiq16bUsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58342 /* svldff1uh_gather_u64offset_s64 */, 48795 /* q2Wiq16bUsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58373 /* svldff1uh_gather_u64offset_u64 */, 48841 /* q2UWiq16bUsC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58404 /* svldff1uh_s32 */, 48930 /* q4iq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58418 /* svldff1uh_s64 */, 48954 /* q2Wiq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58432 /* svldff1uh_u32 */, 48979 /* q4Uiq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58446 /* svldff1uh_u64 */, 49004 /* q2UWiq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58460 /* svldff1uh_vnum_s32 */, 49035 /* q4iq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58479 /* svldff1uh_vnum_s64 */, 49066 /* q2Wiq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58498 /* svldff1uh_vnum_u32 */, 49098 /* q4Uiq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58517 /* svldff1uh_vnum_u64 */, 49130 /* q2UWiq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58536 /* svldff1uw_gather_s64index_s64 */, 49174 /* q2Wiq16bUiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58566 /* svldff1uw_gather_s64index_u64 */, 49219 /* q2UWiq16bUiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58596 /* svldff1uw_gather_s64offset_s64 */, 49174 /* q2Wiq16bUiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58627 /* svldff1uw_gather_s64offset_u64 */, 49219 /* q2UWiq16bUiC*q2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58658 /* svldff1uw_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58693 /* svldff1uw_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58728 /* svldff1uw_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58764 /* svldff1uw_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58800 /* svldff1uw_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58829 /* svldff1uw_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58858 /* svldff1uw_gather_u64index_s64 */, 49511 /* q2Wiq16bUiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58888 /* svldff1uw_gather_u64index_u64 */, 49557 /* q2UWiq16bUiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58918 /* svldff1uw_gather_u64offset_s64 */, 49511 /* q2Wiq16bUiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58949 /* svldff1uw_gather_u64offset_u64 */, 49557 /* q2UWiq16bUiC*q2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58980 /* svldff1uw_s64 */, 49646 /* q2Wiq16bUiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{58994 /* svldff1uw_u64 */, 49671 /* q2UWiq16bUiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59008 /* svldff1uw_vnum_s64 */, 49702 /* q2Wiq16bUiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59027 /* svldff1uw_vnum_u64 */, 49734 /* q2UWiq16bUiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59046 /* svldnf1_bf16 */, 38092 /* q8yq16byC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59059 /* svldnf1_f16 */, 38161 /* q8hq16bhC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59071 /* svldnf1_f32 */, 38228 /* q4fq16bfC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59083 /* svldnf1_f64 */, 38294 /* q2dq16bdC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59095 /* svldnf1_mf8 */, 39840 /* q16mq16bmC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59107 /* svldnf1_s16 */, 39908 /* q8sq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59119 /* svldnf1_s32 */, 39978 /* q4iq16bSiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59131 /* svldnf1_s64 */, 40047 /* q2Wiq16bSWiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59143 /* svldnf1_s8 */, 40120 /* q16Scq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59154 /* svldnf1_u16 */, 40192 /* q8Usq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59166 /* svldnf1_u32 */, 40265 /* q4Uiq16bUiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59178 /* svldnf1_u64 */, 40337 /* q2UWiq16bUWiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59190 /* svldnf1_u8 */, 40413 /* q16Ucq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59201 /* svldnf1_vnum_bf16 */, 40491 /* q8yq16byC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59219 /* svldnf1_vnum_f16 */, 40581 /* q8hq16bhC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59236 /* svldnf1_vnum_f32 */, 40669 /* q4fq16bfC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59253 /* svldnf1_vnum_f64 */, 40756 /* q2dq16bdC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59270 /* svldnf1_vnum_mf8 */, 40842 /* q16mq16bmC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59287 /* svldnf1_vnum_s16 */, 40931 /* q8sq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59304 /* svldnf1_vnum_s32 */, 41022 /* q4iq16bSiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59321 /* svldnf1_vnum_s64 */, 41112 /* q2Wiq16bSWiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59338 /* svldnf1_vnum_s8 */, 41206 /* q16Scq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59354 /* svldnf1_vnum_u16 */, 41299 /* q8Usq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59371 /* svldnf1_vnum_u32 */, 41393 /* q4Uiq16bUiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59388 /* svldnf1_vnum_u64 */, 41486 /* q2UWiq16bUWiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59405 /* svldnf1_vnum_u8 */, 41583 /* q16Ucq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59421 /* svldnf1sb_s16 */, 44777 /* q8sq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59435 /* svldnf1sb_s32 */, 44801 /* q4iq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59449 /* svldnf1sb_s64 */, 44825 /* q2Wiq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59463 /* svldnf1sb_u16 */, 44850 /* q8Usq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59477 /* svldnf1sb_u32 */, 44875 /* q4Uiq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59491 /* svldnf1sb_u64 */, 44900 /* q2UWiq16bScC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59505 /* svldnf1sb_vnum_s16 */, 44931 /* q8sq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59524 /* svldnf1sb_vnum_s32 */, 44962 /* q4iq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59543 /* svldnf1sb_vnum_s64 */, 44993 /* q2Wiq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59562 /* svldnf1sb_vnum_u16 */, 45025 /* q8Usq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59581 /* svldnf1sb_vnum_u32 */, 45057 /* q4Uiq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59600 /* svldnf1sb_vnum_u64 */, 45089 /* q2UWiq16bScC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59619 /* svldnf1sh_s32 */, 46085 /* q4iq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59633 /* svldnf1sh_s64 */, 46109 /* q2Wiq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59647 /* svldnf1sh_u32 */, 46134 /* q4Uiq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59661 /* svldnf1sh_u64 */, 46159 /* q2UWiq16bSsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59675 /* svldnf1sh_vnum_s32 */, 46190 /* q4iq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59694 /* svldnf1sh_vnum_s64 */, 46221 /* q2Wiq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59713 /* svldnf1sh_vnum_u32 */, 46253 /* q4Uiq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59732 /* svldnf1sh_vnum_u64 */, 46285 /* q2UWiq16bSsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59751 /* svldnf1sw_s64 */, 46801 /* q2Wiq16bSiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59765 /* svldnf1sw_u64 */, 46826 /* q2UWiq16bSiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59779 /* svldnf1sw_vnum_s64 */, 46857 /* q2Wiq16bSiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59798 /* svldnf1sw_vnum_u64 */, 46889 /* q2UWiq16bSiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59817 /* svldnf1ub_s16 */, 47529 /* q8sq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59831 /* svldnf1ub_s32 */, 47553 /* q4iq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59845 /* svldnf1ub_s64 */, 47577 /* q2Wiq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59859 /* svldnf1ub_u16 */, 47602 /* q8Usq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59873 /* svldnf1ub_u32 */, 47627 /* q4Uiq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59887 /* svldnf1ub_u64 */, 47652 /* q2UWiq16bUcC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59901 /* svldnf1ub_vnum_s16 */, 47683 /* q8sq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59920 /* svldnf1ub_vnum_s32 */, 47714 /* q4iq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59939 /* svldnf1ub_vnum_s64 */, 47745 /* q2Wiq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59958 /* svldnf1ub_vnum_u16 */, 47777 /* q8Usq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59977 /* svldnf1ub_vnum_u32 */, 47809 /* q4Uiq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{59996 /* svldnf1ub_vnum_u64 */, 47841 /* q2UWiq16bUcC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60015 /* svldnf1uh_s32 */, 48930 /* q4iq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60029 /* svldnf1uh_s64 */, 48954 /* q2Wiq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60043 /* svldnf1uh_u32 */, 48979 /* q4Uiq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60057 /* svldnf1uh_u64 */, 49004 /* q2UWiq16bUsC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60071 /* svldnf1uh_vnum_s32 */, 49035 /* q4iq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60090 /* svldnf1uh_vnum_s64 */, 49066 /* q2Wiq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60109 /* svldnf1uh_vnum_u32 */, 49098 /* q4Uiq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60128 /* svldnf1uh_vnum_u64 */, 49130 /* q2UWiq16bUsC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60147 /* svldnf1uw_s64 */, 49646 /* q2Wiq16bUiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60161 /* svldnf1uw_u64 */, 49671 /* q2UWiq16bUiC* */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60175 /* svldnf1uw_vnum_s64 */, 49702 /* q2Wiq16bUiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60194 /* svldnf1uw_vnum_u64 */, 49734 /* q2UWiq16bUiC*Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60213 /* svldnt1_bf16 */, 38092 /* q8yq16byC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60226 /* svldnt1_bf16_x2 */, 38117 /* q16yQayC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60242 /* svldnt1_bf16_x4 */, 38141 /* q32yQayC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60258 /* svldnt1_f16 */, 38161 /* q8hq16bhC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60270 /* svldnt1_f16_x2 */, 38185 /* q16hQahC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60285 /* svldnt1_f16_x4 */, 38208 /* q32hQahC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60300 /* svldnt1_f32 */, 38228 /* q4fq16bfC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60312 /* svldnt1_f32_x2 */, 38252 /* q8fQafC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60327 /* svldnt1_f32_x4 */, 38274 /* q16fQafC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60342 /* svldnt1_f64 */, 38294 /* q2dq16bdC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60354 /* svldnt1_f64_x2 */, 38318 /* q4dQadC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60369 /* svldnt1_f64_x4 */, 38340 /* q8dQadC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60384 /* svldnt1_gather_s64index_f64 */, 38579 /* q2dq16bdC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60412 /* svldnt1_gather_s64index_s64 */, 38620 /* q2Wiq16bSWiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60440 /* svldnt1_gather_s64index_u64 */, 38664 /* q2UWiq16bUWiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60468 /* svldnt1_gather_s64offset_f64 */, 38579 /* q2dq16bdC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60497 /* svldnt1_gather_s64offset_s64 */, 38620 /* q2Wiq16bSWiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60526 /* svldnt1_gather_s64offset_u64 */, 38664 /* q2UWiq16bUWiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60555 /* svldnt1_gather_u32base_f32 */, 24686 /* q4fq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60582 /* svldnt1_gather_u32base_index_f32 */, 38820 /* q4fq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60615 /* svldnt1_gather_u32base_index_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60648 /* svldnt1_gather_u32base_index_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60681 /* svldnt1_gather_u32base_offset_f32 */, 38820 /* q4fq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60715 /* svldnt1_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60749 /* svldnt1_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60783 /* svldnt1_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60810 /* svldnt1_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60837 /* svldnt1_gather_u32offset_f32 */, 39109 /* q4fq16bfC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60866 /* svldnt1_gather_u32offset_s32 */, 39150 /* q4iq16bSiC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60895 /* svldnt1_gather_u32offset_u32 */, 39192 /* q4Uiq16bUiC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60924 /* svldnt1_gather_u64base_f64 */, 25260 /* q2dq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60951 /* svldnt1_gather_u64base_index_f64 */, 39346 /* q2dq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{60984 /* svldnt1_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61017 /* svldnt1_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61050 /* svldnt1_gather_u64base_offset_f64 */, 39346 /* q2dq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61084 /* svldnt1_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61118 /* svldnt1_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61152 /* svldnt1_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61179 /* svldnt1_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61206 /* svldnt1_gather_u64index_f64 */, 39642 /* q2dq16bdC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61234 /* svldnt1_gather_u64index_s64 */, 39684 /* q2Wiq16bSWiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61262 /* svldnt1_gather_u64index_u64 */, 39729 /* q2UWiq16bUWiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61290 /* svldnt1_gather_u64offset_f64 */, 39642 /* q2dq16bdC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61319 /* svldnt1_gather_u64offset_s64 */, 39684 /* q2Wiq16bSWiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61348 /* svldnt1_gather_u64offset_u64 */, 39729 /* q2UWiq16bUWiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61377 /* svldnt1_mf8 */, 39840 /* q16mq16bmC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61389 /* svldnt1_mf8_x2 */, 39865 /* q32mQamC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61404 /* svldnt1_mf8_x4 */, 39888 /* q64mQamC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61419 /* svldnt1_s16 */, 39908 /* q8sq16bSsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61431 /* svldnt1_s16_x2 */, 39933 /* q16sQaSsC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61446 /* svldnt1_s16_x4 */, 39957 /* q32sQaSsC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61461 /* svldnt1_s32 */, 39978 /* q4iq16bSiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61473 /* svldnt1_s32_x2 */, 40003 /* q8iQaSiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61488 /* svldnt1_s32_x4 */, 40026 /* q16iQaSiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61503 /* svldnt1_s64 */, 40047 /* q2Wiq16bSWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61515 /* svldnt1_s64_x2 */, 40074 /* q4WiQaSWiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61530 /* svldnt1_s64_x4 */, 40099 /* q8WiQaSWiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61545 /* svldnt1_s8 */, 40120 /* q16Scq16bScC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61556 /* svldnt1_s8_x2 */, 40146 /* q32ScQaScC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61570 /* svldnt1_s8_x4 */, 40170 /* q64ScQaScC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61584 /* svldnt1_u16 */, 40192 /* q8Usq16bUsC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61596 /* svldnt1_u16_x2 */, 40218 /* q16UsQaUsC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61611 /* svldnt1_u16_x4 */, 40243 /* q32UsQaUsC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61626 /* svldnt1_u32 */, 40265 /* q4Uiq16bUiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61638 /* svldnt1_u32_x2 */, 40291 /* q8UiQaUiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61653 /* svldnt1_u32_x4 */, 40315 /* q16UiQaUiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61668 /* svldnt1_u64 */, 40337 /* q2UWiq16bUWiC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61680 /* svldnt1_u64_x2 */, 40365 /* q4UWiQaUWiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61695 /* svldnt1_u64_x4 */, 40391 /* q8UWiQaUWiC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61710 /* svldnt1_u8 */, 40413 /* q16Ucq16bUcC* */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61721 /* svldnt1_u8_x2 */, 40439 /* q32UcQaUcC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61735 /* svldnt1_u8_x4 */, 40463 /* q64UcQaUcC* */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61749 /* svldnt1_vnum_bf16 */, 40491 /* q8yq16byC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61767 /* svldnt1_vnum_bf16_x2 */, 40523 /* q16yQayC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61788 /* svldnt1_vnum_bf16_x4 */, 40554 /* q32yQayC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61809 /* svldnt1_vnum_f16 */, 40581 /* q8hq16bhC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61826 /* svldnt1_vnum_f16_x2 */, 40612 /* q16hQahC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61846 /* svldnt1_vnum_f16_x4 */, 40642 /* q32hQahC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61866 /* svldnt1_vnum_f32 */, 40669 /* q4fq16bfC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61883 /* svldnt1_vnum_f32_x2 */, 40700 /* q8fQafC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61903 /* svldnt1_vnum_f32_x4 */, 40729 /* q16fQafC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61923 /* svldnt1_vnum_f64 */, 40756 /* q2dq16bdC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61940 /* svldnt1_vnum_f64_x2 */, 40787 /* q4dQadC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61960 /* svldnt1_vnum_f64_x4 */, 40816 /* q8dQadC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61980 /* svldnt1_vnum_mf8 */, 40842 /* q16mq16bmC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{61997 /* svldnt1_vnum_mf8_x2 */, 40874 /* q32mQamC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62017 /* svldnt1_vnum_mf8_x4 */, 40904 /* q64mQamC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62037 /* svldnt1_vnum_s16 */, 40931 /* q8sq16bSsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62054 /* svldnt1_vnum_s16_x2 */, 40963 /* q16sQaSsC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62074 /* svldnt1_vnum_s16_x4 */, 40994 /* q32sQaSsC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62094 /* svldnt1_vnum_s32 */, 41022 /* q4iq16bSiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62111 /* svldnt1_vnum_s32_x2 */, 41054 /* q8iQaSiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62131 /* svldnt1_vnum_s32_x4 */, 41084 /* q16iQaSiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62151 /* svldnt1_vnum_s64 */, 41112 /* q2Wiq16bSWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62168 /* svldnt1_vnum_s64_x2 */, 41146 /* q4WiQaSWiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62188 /* svldnt1_vnum_s64_x4 */, 41178 /* q8WiQaSWiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62208 /* svldnt1_vnum_s8 */, 41206 /* q16Scq16bScC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62224 /* svldnt1_vnum_s8_x2 */, 41239 /* q32ScQaScC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62243 /* svldnt1_vnum_s8_x4 */, 41270 /* q64ScQaScC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62262 /* svldnt1_vnum_u16 */, 41299 /* q8Usq16bUsC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62279 /* svldnt1_vnum_u16_x2 */, 41332 /* q16UsQaUsC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62299 /* svldnt1_vnum_u16_x4 */, 41364 /* q32UsQaUsC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62319 /* svldnt1_vnum_u32 */, 41393 /* q4Uiq16bUiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62336 /* svldnt1_vnum_u32_x2 */, 41426 /* q8UiQaUiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62356 /* svldnt1_vnum_u32_x4 */, 41457 /* q16UiQaUiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62376 /* svldnt1_vnum_u64 */, 41486 /* q2UWiq16bUWiC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62393 /* svldnt1_vnum_u64_x2 */, 41521 /* q4UWiQaUWiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62413 /* svldnt1_vnum_u64_x4 */, 41554 /* q8UWiQaUWiC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62433 /* svldnt1_vnum_u8 */, 41583 /* q16Ucq16bUcC*Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62449 /* svldnt1_vnum_u8_x2 */, 41616 /* q32UcQaUcC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62468 /* svldnt1_vnum_u8_x4 */, 41647 /* q64UcQaUcC*Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62487 /* svldnt1sb_gather_s64offset_s64 */, 44271 /* q2Wiq16bScC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62518 /* svldnt1sb_gather_s64offset_u64 */, 44317 /* q2UWiq16bScC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62549 /* svldnt1sb_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62585 /* svldnt1sb_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62621 /* svldnt1sb_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62650 /* svldnt1sb_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62679 /* svldnt1sb_gather_u32offset_s32 */, 44486 /* q4iq16bScC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62710 /* svldnt1sb_gather_u32offset_u32 */, 44531 /* q4Uiq16bScC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62741 /* svldnt1sb_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62777 /* svldnt1sb_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62813 /* svldnt1sb_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62842 /* svldnt1sb_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62871 /* svldnt1sb_gather_u64offset_s64 */, 44699 /* q2Wiq16bScC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62902 /* svldnt1sb_gather_u64offset_u64 */, 44746 /* q2UWiq16bScC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62933 /* svldnt1sh_gather_s64index_s64 */, 45278 /* q2Wiq16bSsC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62963 /* svldnt1sh_gather_s64index_u64 */, 45323 /* q2UWiq16bSsC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62993 /* svldnt1sh_gather_s64offset_s64 */, 45278 /* q2Wiq16bSsC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63024 /* svldnt1sh_gather_s64offset_u64 */, 45323 /* q2UWiq16bSsC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63055 /* svldnt1sh_gather_u32base_index_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63090 /* svldnt1sh_gather_u32base_index_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63125 /* svldnt1sh_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63161 /* svldnt1sh_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63197 /* svldnt1sh_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63226 /* svldnt1sh_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63255 /* svldnt1sh_gather_u32offset_s32 */, 45615 /* q4iq16bSsC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63286 /* svldnt1sh_gather_u32offset_u32 */, 45659 /* q4Uiq16bSsC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63317 /* svldnt1sh_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63352 /* svldnt1sh_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63387 /* svldnt1sh_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63423 /* svldnt1sh_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63459 /* svldnt1sh_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63488 /* svldnt1sh_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63517 /* svldnt1sh_gather_u64index_s64 */, 45950 /* q2Wiq16bSsC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63547 /* svldnt1sh_gather_u64index_u64 */, 45996 /* q2UWiq16bSsC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63577 /* svldnt1sh_gather_u64offset_s64 */, 45950 /* q2Wiq16bSsC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63608 /* svldnt1sh_gather_u64offset_u64 */, 45996 /* q2UWiq16bSsC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63639 /* svldnt1sw_gather_s64index_s64 */, 46329 /* q2Wiq16bSiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63669 /* svldnt1sw_gather_s64index_u64 */, 46374 /* q2UWiq16bSiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63699 /* svldnt1sw_gather_s64offset_s64 */, 46329 /* q2Wiq16bSiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63730 /* svldnt1sw_gather_s64offset_u64 */, 46374 /* q2UWiq16bSiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63761 /* svldnt1sw_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63796 /* svldnt1sw_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63831 /* svldnt1sw_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63867 /* svldnt1sw_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63903 /* svldnt1sw_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63932 /* svldnt1sw_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63961 /* svldnt1sw_gather_u64index_s64 */, 46666 /* q2Wiq16bSiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{63991 /* svldnt1sw_gather_u64index_u64 */, 46712 /* q2UWiq16bSiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64021 /* svldnt1sw_gather_u64offset_s64 */, 46666 /* q2Wiq16bSiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64052 /* svldnt1sw_gather_u64offset_u64 */, 46712 /* q2UWiq16bSiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64083 /* svldnt1ub_gather_s64offset_s64 */, 47023 /* q2Wiq16bUcC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64114 /* svldnt1ub_gather_s64offset_u64 */, 47069 /* q2UWiq16bUcC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64145 /* svldnt1ub_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64181 /* svldnt1ub_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64217 /* svldnt1ub_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64246 /* svldnt1ub_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64275 /* svldnt1ub_gather_u32offset_s32 */, 47238 /* q4iq16bUcC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64306 /* svldnt1ub_gather_u32offset_u32 */, 47283 /* q4Uiq16bUcC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64337 /* svldnt1ub_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64373 /* svldnt1ub_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64409 /* svldnt1ub_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64438 /* svldnt1ub_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64467 /* svldnt1ub_gather_u64offset_s64 */, 47451 /* q2Wiq16bUcC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64498 /* svldnt1ub_gather_u64offset_u64 */, 47498 /* q2UWiq16bUcC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64529 /* svldnt1uh_gather_s64index_s64 */, 48123 /* q2Wiq16bUsC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64559 /* svldnt1uh_gather_s64index_u64 */, 48168 /* q2UWiq16bUsC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64589 /* svldnt1uh_gather_s64offset_s64 */, 48123 /* q2Wiq16bUsC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64620 /* svldnt1uh_gather_s64offset_u64 */, 48168 /* q2UWiq16bUsC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64651 /* svldnt1uh_gather_u32base_index_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64686 /* svldnt1uh_gather_u32base_index_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64721 /* svldnt1uh_gather_u32base_offset_s32 */, 38865 /* q4iq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64757 /* svldnt1uh_gather_u32base_offset_u32 */, 38910 /* q4Uiq16bq4UiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64793 /* svldnt1uh_gather_u32base_s32 */, 39046 /* q4iq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64822 /* svldnt1uh_gather_u32base_u32 */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64851 /* svldnt1uh_gather_u32offset_s32 */, 48460 /* q4iq16bUsC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64882 /* svldnt1uh_gather_u32offset_u32 */, 48504 /* q4Uiq16bUsC*q4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64913 /* svldnt1uh_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64948 /* svldnt1uh_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{64983 /* svldnt1uh_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65019 /* svldnt1uh_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65055 /* svldnt1uh_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65084 /* svldnt1uh_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65113 /* svldnt1uh_gather_u64index_s64 */, 48795 /* q2Wiq16bUsC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65143 /* svldnt1uh_gather_u64index_u64 */, 48841 /* q2UWiq16bUsC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65173 /* svldnt1uh_gather_u64offset_s64 */, 48795 /* q2Wiq16bUsC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65204 /* svldnt1uh_gather_u64offset_u64 */, 48841 /* q2UWiq16bUsC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65235 /* svldnt1uw_gather_s64index_s64 */, 49174 /* q2Wiq16bUiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65265 /* svldnt1uw_gather_s64index_u64 */, 49219 /* q2UWiq16bUiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65295 /* svldnt1uw_gather_s64offset_s64 */, 49174 /* q2Wiq16bUiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65326 /* svldnt1uw_gather_s64offset_u64 */, 49219 /* q2UWiq16bUiC*q2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65357 /* svldnt1uw_gather_u64base_index_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65392 /* svldnt1uw_gather_u64base_index_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65427 /* svldnt1uw_gather_u64base_offset_s64 */, 39392 /* q2Wiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65463 /* svldnt1uw_gather_u64base_offset_u64 */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65499 /* svldnt1uw_gather_u64base_s64 */, 39577 /* q2Wiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65528 /* svldnt1uw_gather_u64base_u64 */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65557 /* svldnt1uw_gather_u64index_s64 */, 49511 /* q2Wiq16bUiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65587 /* svldnt1uw_gather_u64index_u64 */, 49557 /* q2UWiq16bUiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65617 /* svldnt1uw_gather_u64offset_s64 */, 49511 /* q2Wiq16bUiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65648 /* svldnt1uw_gather_u64offset_u64 */, 49557 /* q2UWiq16bUiC*q2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65679 /* svlen_bf16 */, 65690 /* UWiq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65697 /* svlen_f16 */, 65707 /* UWiq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65714 /* svlen_f32 */, 65724 /* UWiq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65731 /* svlen_f64 */, 65741 /* UWiq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65748 /* svlen_s16 */, 65758 /* UWiq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65765 /* svlen_s32 */, 65775 /* UWiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65782 /* svlen_s64 */, 65792 /* UWiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65800 /* svlen_s8 */, 65809 /* UWiq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65818 /* svlen_u16 */, 65828 /* UWiq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65836 /* svlen_u32 */, 65846 /* UWiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65854 /* svlen_u64 */, 65864 /* UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65873 /* svlen_u8 */, 65882 /* UWiq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65891 /* svlogb_f16_m */, 25405 /* q8sq8sq16bq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65904 /* svlogb_f16_x */, 25435 /* q8sq16bq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65917 /* svlogb_f16_z */, 25435 /* q8sq16bq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65930 /* svlogb_f32_m */, 25551 /* q4iq4iq16bq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65943 /* svlogb_f32_x */, 25581 /* q4iq16bq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65956 /* svlogb_f32_z */, 25581 /* q4iq16bq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65969 /* svlogb_f64_m */, 25899 /* q2Wiq2Wiq16bq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65982 /* svlogb_f64_x */, 25931 /* q2Wiq16bq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{65995 /* svlogb_f64_z */, 25931 /* q2Wiq16bq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66008 /* svlsl_n_s16_m */, 10303 /* q8sq16bq8sUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66022 /* svlsl_n_s16_x */, 10303 /* q8sq16bq8sUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66036 /* svlsl_n_s16_z */, 10303 /* q8sq16bq8sUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66050 /* svlsl_n_s32_m */, 10358 /* q4iq16bq4iUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66064 /* svlsl_n_s32_x */, 10358 /* q4iq16bq4iUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66078 /* svlsl_n_s32_z */, 10358 /* q4iq16bq4iUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66092 /* svlsl_n_s64_m */, 10413 /* q2Wiq16bq2WiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66106 /* svlsl_n_s64_x */, 10413 /* q2Wiq16bq2WiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66120 /* svlsl_n_s64_z */, 10413 /* q2Wiq16bq2WiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66134 /* svlsl_n_s8_m */, 10470 /* q16Scq16bq16ScUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66147 /* svlsl_n_s8_x */, 10470 /* q16Scq16bq16ScUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66160 /* svlsl_n_s8_z */, 10470 /* q16Scq16bq16ScUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66173 /* svlsl_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66187 /* svlsl_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66201 /* svlsl_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66215 /* svlsl_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66229 /* svlsl_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66243 /* svlsl_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66257 /* svlsl_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66271 /* svlsl_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66285 /* svlsl_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66299 /* svlsl_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66312 /* svlsl_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66325 /* svlsl_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66338 /* svlsl_s16_m */, 10525 /* q8sq16bq8sq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66350 /* svlsl_s16_x */, 10525 /* q8sq16bq8sq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66362 /* svlsl_s16_z */, 10525 /* q8sq16bq8sq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66374 /* svlsl_s32_m */, 10576 /* q4iq16bq4iq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66386 /* svlsl_s32_x */, 10576 /* q4iq16bq4iq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66398 /* svlsl_s32_z */, 10576 /* q4iq16bq4iq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66410 /* svlsl_s64_m */, 10627 /* q2Wiq16bq2Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66422 /* svlsl_s64_x */, 10627 /* q2Wiq16bq2Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66434 /* svlsl_s64_z */, 10627 /* q2Wiq16bq2Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66446 /* svlsl_s8_m */, 10680 /* q16Scq16bq16Scq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66457 /* svlsl_s8_x */, 10680 /* q16Scq16bq16Scq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66468 /* svlsl_s8_z */, 10680 /* q16Scq16bq16Scq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66479 /* svlsl_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66491 /* svlsl_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66503 /* svlsl_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66515 /* svlsl_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66527 /* svlsl_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66539 /* svlsl_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66551 /* svlsl_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66563 /* svlsl_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66575 /* svlsl_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66587 /* svlsl_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66598 /* svlsl_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66609 /* svlsl_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66620 /* svlsl_wide_n_s16_m */, 10741 /* q8sq16bq8sUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66639 /* svlsl_wide_n_s16_x */, 10741 /* q8sq16bq8sUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66658 /* svlsl_wide_n_s16_z */, 10741 /* q8sq16bq8sUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66677 /* svlsl_wide_n_s32_m */, 10812 /* q4iq16bq4iUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66696 /* svlsl_wide_n_s32_x */, 10812 /* q4iq16bq4iUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66715 /* svlsl_wide_n_s32_z */, 10812 /* q4iq16bq4iUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66734 /* svlsl_wide_n_s8_m */, 10882 /* q16Scq16bq16ScUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66752 /* svlsl_wide_n_s8_x */, 10882 /* q16Scq16bq16ScUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66770 /* svlsl_wide_n_s8_z */, 10882 /* q16Scq16bq16ScUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66788 /* svlsl_wide_n_u16_m */, 66807 /* q8Usq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66823 /* svlsl_wide_n_u16_x */, 66807 /* q8Usq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66842 /* svlsl_wide_n_u16_z */, 66807 /* q8Usq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66861 /* svlsl_wide_n_u32_m */, 66880 /* q4Uiq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66896 /* svlsl_wide_n_u32_x */, 66880 /* q4Uiq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66915 /* svlsl_wide_n_u32_z */, 66880 /* q4Uiq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66934 /* svlsl_wide_n_u8_m */, 66952 /* q16Ucq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66970 /* svlsl_wide_n_u8_x */, 66952 /* q16Ucq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{66988 /* svlsl_wide_n_u8_z */, 66952 /* q16Ucq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67006 /* svlsl_wide_s16_m */, 10953 /* q8sq16bq8sq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67023 /* svlsl_wide_s16_x */, 10953 /* q8sq16bq8sq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67040 /* svlsl_wide_s16_z */, 10953 /* q8sq16bq8sq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67057 /* svlsl_wide_s32_m */, 11020 /* q4iq16bq4iq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67074 /* svlsl_wide_s32_x */, 11020 /* q4iq16bq4iq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67091 /* svlsl_wide_s32_z */, 11020 /* q4iq16bq4iq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67108 /* svlsl_wide_s8_m */, 11086 /* q16Scq16bq16Scq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67124 /* svlsl_wide_s8_x */, 11086 /* q16Scq16bq16Scq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67140 /* svlsl_wide_s8_z */, 11086 /* q16Scq16bq16Scq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67156 /* svlsl_wide_u16_m */, 67173 /* q8Usq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67191 /* svlsl_wide_u16_x */, 67173 /* q8Usq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67208 /* svlsl_wide_u16_z */, 67173 /* q8Usq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67225 /* svlsl_wide_u32_m */, 67242 /* q4Uiq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67260 /* svlsl_wide_u32_x */, 67242 /* q4Uiq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67277 /* svlsl_wide_u32_z */, 67242 /* q4Uiq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67294 /* svlsl_wide_u8_m */, 67310 /* q16Ucq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67330 /* svlsl_wide_u8_x */, 67310 /* q16Ucq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67346 /* svlsl_wide_u8_z */, 67310 /* q16Ucq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67362 /* svlsr_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67376 /* svlsr_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67390 /* svlsr_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67404 /* svlsr_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67418 /* svlsr_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67432 /* svlsr_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67446 /* svlsr_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67460 /* svlsr_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67474 /* svlsr_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67488 /* svlsr_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67501 /* svlsr_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67514 /* svlsr_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67527 /* svlsr_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67539 /* svlsr_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67551 /* svlsr_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67563 /* svlsr_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67575 /* svlsr_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67587 /* svlsr_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67599 /* svlsr_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67611 /* svlsr_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67623 /* svlsr_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67635 /* svlsr_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67646 /* svlsr_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67657 /* svlsr_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67668 /* svlsr_wide_n_u16_m */, 66807 /* q8Usq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67687 /* svlsr_wide_n_u16_x */, 66807 /* q8Usq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67706 /* svlsr_wide_n_u16_z */, 66807 /* q8Usq16bq8UsUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67725 /* svlsr_wide_n_u32_m */, 66880 /* q4Uiq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67744 /* svlsr_wide_n_u32_x */, 66880 /* q4Uiq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67763 /* svlsr_wide_n_u32_z */, 66880 /* q4Uiq16bq4UiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67782 /* svlsr_wide_n_u8_m */, 66952 /* q16Ucq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67800 /* svlsr_wide_n_u8_x */, 66952 /* q16Ucq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67818 /* svlsr_wide_n_u8_z */, 66952 /* q16Ucq16bq16UcUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67836 /* svlsr_wide_u16_m */, 67173 /* q8Usq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67853 /* svlsr_wide_u16_x */, 67173 /* q8Usq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67870 /* svlsr_wide_u16_z */, 67173 /* q8Usq16bq8Usq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67887 /* svlsr_wide_u32_m */, 67242 /* q4Uiq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67904 /* svlsr_wide_u32_x */, 67242 /* q4Uiq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67921 /* svlsr_wide_u32_z */, 67242 /* q4Uiq16bq4Uiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67938 /* svlsr_wide_u8_m */, 67310 /* q16Ucq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67954 /* svlsr_wide_u8_x */, 67310 /* q16Ucq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67970 /* svlsr_wide_u8_z */, 67310 /* q16Ucq16bq16Ucq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{67986 /* svluti2_lane_bf16 */, 68004 /* q8yq8yq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68057 /* svluti2_lane_f16 */, 68074 /* q8hq8hq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68090 /* svluti2_lane_s16 */, 68107 /* q8sq8sq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68123 /* svluti2_lane_s8 */, 68139 /* q16Scq16Scq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68159 /* svluti2_lane_u16 */, 68176 /* q8Usq8Usq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68194 /* svluti2_lane_u8 */, 13952 /* q16Ucq16Ucq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68210 /* svluti4_lane_bf16 */, 68004 /* q8yq8yq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68228 /* svluti4_lane_bf16_x2 */, 68249 /* q8yq16yq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68266 /* svluti4_lane_f16 */, 68074 /* q8hq8hq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68283 /* svluti4_lane_f16_x2 */, 68303 /* q8hq16hq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68320 /* svluti4_lane_s16 */, 68107 /* q8sq8sq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68337 /* svluti4_lane_s16_x2 */, 68357 /* q8sq16sq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68374 /* svluti4_lane_s8 */, 68139 /* q16Scq16Scq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68390 /* svluti4_lane_u16 */, 68176 /* q8Usq8Usq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68407 /* svluti4_lane_u16_x2 */, 68427 /* q8Usq16Usq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68446 /* svluti4_lane_u8 */, 13952 /* q16Ucq16Ucq16UcIUWi */, 1 /* n */, 68020 /* (sve,(sve2|sme2),lut)|(sme,sme2,lut) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68462 /* svmad_f16_m */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68491 /* svmad_f16_x */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68503 /* svmad_f16_z */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68515 /* svmad_f32_m */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68544 /* svmad_f32_x */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68556 /* svmad_f32_z */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68568 /* svmad_f64_m */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68597 /* svmad_f64_x */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68609 /* svmad_f64_z */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68621 /* svmad_n_f16_m */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68650 /* svmad_n_f16_x */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68664 /* svmad_n_f16_z */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68678 /* svmad_n_f32_m */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68707 /* svmad_n_f32_x */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68721 /* svmad_n_f32_z */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68735 /* svmad_n_f64_m */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68764 /* svmad_n_f64_x */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68778 /* svmad_n_f64_z */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68792 /* svmad_n_s16_m */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68821 /* svmad_n_s16_x */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68835 /* svmad_n_s16_z */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68849 /* svmad_n_s32_m */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68878 /* svmad_n_s32_x */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68892 /* svmad_n_s32_z */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68906 /* svmad_n_s64_m */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68939 /* svmad_n_s64_x */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68953 /* svmad_n_s64_z */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{68967 /* svmad_n_s8_m */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69002 /* svmad_n_s8_x */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69015 /* svmad_n_s8_z */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69028 /* svmad_n_u16_m */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69061 /* svmad_n_u16_x */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69075 /* svmad_n_u16_z */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69089 /* svmad_n_u32_m */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69122 /* svmad_n_u32_x */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69136 /* svmad_n_u32_z */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69150 /* svmad_n_u64_m */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69187 /* svmad_n_u64_x */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69201 /* svmad_n_u64_z */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69215 /* svmad_n_u8_m */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69250 /* svmad_n_u8_x */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69263 /* svmad_n_u8_z */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69276 /* svmad_s16_m */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69305 /* svmad_s16_x */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69317 /* svmad_s16_z */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69329 /* svmad_s32_m */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69358 /* svmad_s32_x */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69370 /* svmad_s32_z */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69382 /* svmad_s64_m */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69415 /* svmad_s64_x */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69427 /* svmad_s64_z */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69439 /* svmad_s8_m */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69475 /* svmad_s8_x */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69486 /* svmad_s8_z */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69497 /* svmad_u16_m */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69530 /* svmad_u16_x */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69542 /* svmad_u16_z */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69554 /* svmad_u32_m */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69587 /* svmad_u32_x */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69599 /* svmad_u32_z */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69611 /* svmad_u64_m */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69648 /* svmad_u64_x */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69660 /* svmad_u64_z */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69672 /* svmad_u8_m */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69708 /* svmad_u8_x */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69719 /* svmad_u8_z */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69730 /* svmatch_s16 */, 17730 /* q16bq16bq8sq8s */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69742 /* svmatch_s8 */, 17812 /* q16bq16bq16Scq16Sc */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69753 /* svmatch_u16 */, 17843 /* q16bq16bq8Usq8Us */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69765 /* svmatch_u8 */, 17931 /* q16bq16bq16Ucq16Uc */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69776 /* svmax_bf16_m */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69789 /* svmax_bf16_x */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69802 /* svmax_bf16_x2 */, 69816 /* q16yq16yq16y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69829 /* svmax_bf16_x4 */, 69843 /* q32yq32yq32y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69856 /* svmax_bf16_z */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69869 /* svmax_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69881 /* svmax_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69893 /* svmax_f16_x2 */, 8687 /* q16hq16hq16h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69906 /* svmax_f16_x4 */, 8732 /* q32hq32hq32h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69919 /* svmax_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69931 /* svmax_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69943 /* svmax_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69955 /* svmax_f32_x2 */, 8798 /* q8fq8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69968 /* svmax_f32_x4 */, 8822 /* q16fq16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69981 /* svmax_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{69993 /* svmax_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70005 /* svmax_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70017 /* svmax_f64_x2 */, 8888 /* q4dq4dq4d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70030 /* svmax_f64_x4 */, 8912 /* q8dq8dq8d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70043 /* svmax_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70055 /* svmax_n_bf16_m */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70070 /* svmax_n_bf16_x */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70085 /* svmax_n_bf16_z */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70100 /* svmax_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70114 /* svmax_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70128 /* svmax_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70142 /* svmax_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70156 /* svmax_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70170 /* svmax_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70184 /* svmax_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70198 /* svmax_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70212 /* svmax_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70226 /* svmax_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70240 /* svmax_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70254 /* svmax_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70268 /* svmax_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70282 /* svmax_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70296 /* svmax_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70310 /* svmax_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70324 /* svmax_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70338 /* svmax_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70352 /* svmax_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70365 /* svmax_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70378 /* svmax_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70391 /* svmax_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70405 /* svmax_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70419 /* svmax_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70433 /* svmax_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70447 /* svmax_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70461 /* svmax_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70475 /* svmax_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70489 /* svmax_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70503 /* svmax_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70517 /* svmax_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70530 /* svmax_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70543 /* svmax_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70556 /* svmax_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70568 /* svmax_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70580 /* svmax_s16_x2 */, 70593 /* q16sq16sq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70606 /* svmax_s16_x4 */, 70619 /* q32sq32sq32s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70632 /* svmax_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70644 /* svmax_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70656 /* svmax_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70668 /* svmax_s32_x2 */, 70681 /* q8iq8iq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70691 /* svmax_s32_x4 */, 70704 /* q16iq16iq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70717 /* svmax_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70729 /* svmax_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70741 /* svmax_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70753 /* svmax_s64_x2 */, 70766 /* q4Wiq4Wiq4Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70779 /* svmax_s64_x4 */, 70792 /* q8Wiq8Wiq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70805 /* svmax_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70817 /* svmax_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70828 /* svmax_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70839 /* svmax_s8_x2 */, 70851 /* q32Scq32Scq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70867 /* svmax_s8_x4 */, 70879 /* q64Scq64Scq64Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70895 /* svmax_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70906 /* svmax_single_bf16_x2 */, 70927 /* q16yq16yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70939 /* svmax_single_bf16_x4 */, 70960 /* q32yq32yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{70972 /* svmax_single_f16_x2 */, 70992 /* q16hq16hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71004 /* svmax_single_f16_x4 */, 71024 /* q32hq32hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71036 /* svmax_single_f32_x2 */, 71056 /* q8fq8fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71066 /* svmax_single_f32_x4 */, 71086 /* q16fq16fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71098 /* svmax_single_f64_x2 */, 71118 /* q4dq4dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71128 /* svmax_single_f64_x4 */, 71148 /* q8dq8dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71158 /* svmax_single_s16_x2 */, 4741 /* q16sq16sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71178 /* svmax_single_s16_x4 */, 4782 /* q32sq32sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71198 /* svmax_single_s32_x2 */, 4814 /* q8iq8iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71218 /* svmax_single_s32_x4 */, 4844 /* q16iq16iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71238 /* svmax_single_s64_x2 */, 4876 /* q4Wiq4Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71258 /* svmax_single_s64_x4 */, 4909 /* q8Wiq8Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71278 /* svmax_single_s8_x2 */, 4941 /* q32Scq32Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71297 /* svmax_single_s8_x4 */, 4976 /* q64Scq64Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71316 /* svmax_single_u16_x2 */, 5012 /* q16Usq16Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71336 /* svmax_single_u16_x4 */, 5047 /* q32Usq32Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71356 /* svmax_single_u32_x2 */, 5082 /* q8Uiq8Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71376 /* svmax_single_u32_x4 */, 5115 /* q16Uiq16Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71396 /* svmax_single_u64_x2 */, 5150 /* q4UWiq4UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71416 /* svmax_single_u64_x4 */, 5186 /* q8UWiq8UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71436 /* svmax_single_u8_x2 */, 5221 /* q32Ucq32Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71455 /* svmax_single_u8_x4 */, 5256 /* q64Ucq64Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71474 /* svmax_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71486 /* svmax_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71498 /* svmax_u16_x2 */, 71511 /* q16Usq16Usq16Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71527 /* svmax_u16_x4 */, 71540 /* q32Usq32Usq32Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71556 /* svmax_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71568 /* svmax_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71580 /* svmax_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71592 /* svmax_u32_x2 */, 71605 /* q8Uiq8Uiq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71618 /* svmax_u32_x4 */, 71631 /* q16Uiq16Uiq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71647 /* svmax_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71659 /* svmax_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71671 /* svmax_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71683 /* svmax_u64_x2 */, 71696 /* q4UWiq4UWiq4UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71712 /* svmax_u64_x4 */, 71725 /* q8UWiq8UWiq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71741 /* svmax_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71753 /* svmax_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71764 /* svmax_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71775 /* svmax_u8_x2 */, 71787 /* q32Ucq32Ucq32Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71803 /* svmax_u8_x4 */, 71815 /* q64Ucq64Ucq64Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71831 /* svmax_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71842 /* svmaxnm_bf16_m */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71857 /* svmaxnm_bf16_x */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71872 /* svmaxnm_bf16_x2 */, 69816 /* q16yq16yq16y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71888 /* svmaxnm_bf16_x4 */, 69843 /* q32yq32yq32y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71904 /* svmaxnm_bf16_z */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71919 /* svmaxnm_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71933 /* svmaxnm_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71947 /* svmaxnm_f16_x2 */, 8687 /* q16hq16hq16h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71962 /* svmaxnm_f16_x4 */, 8732 /* q32hq32hq32h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71977 /* svmaxnm_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{71991 /* svmaxnm_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72005 /* svmaxnm_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72019 /* svmaxnm_f32_x2 */, 8798 /* q8fq8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72034 /* svmaxnm_f32_x4 */, 8822 /* q16fq16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72049 /* svmaxnm_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72063 /* svmaxnm_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72077 /* svmaxnm_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72091 /* svmaxnm_f64_x2 */, 8888 /* q4dq4dq4d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72106 /* svmaxnm_f64_x4 */, 8912 /* q8dq8dq8d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72121 /* svmaxnm_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72135 /* svmaxnm_n_bf16_m */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72152 /* svmaxnm_n_bf16_x */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72169 /* svmaxnm_n_bf16_z */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72186 /* svmaxnm_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72202 /* svmaxnm_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72218 /* svmaxnm_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72234 /* svmaxnm_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72250 /* svmaxnm_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72266 /* svmaxnm_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72282 /* svmaxnm_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72298 /* svmaxnm_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72314 /* svmaxnm_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72330 /* svmaxnm_single_bf16_x2 */, 70927 /* q16yq16yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72353 /* svmaxnm_single_bf16_x4 */, 70960 /* q32yq32yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72376 /* svmaxnm_single_f16_x2 */, 70992 /* q16hq16hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72398 /* svmaxnm_single_f16_x4 */, 71024 /* q32hq32hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72420 /* svmaxnm_single_f32_x2 */, 71056 /* q8fq8fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72442 /* svmaxnm_single_f32_x4 */, 71086 /* q16fq16fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72464 /* svmaxnm_single_f64_x2 */, 71118 /* q4dq4dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72486 /* svmaxnm_single_f64_x4 */, 71148 /* q8dq8dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72508 /* svmaxnmp_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72523 /* svmaxnmp_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72538 /* svmaxnmp_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72553 /* svmaxnmp_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72568 /* svmaxnmp_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72583 /* svmaxnmp_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72598 /* svmaxnmqv_f16 */, 6834 /* V8hq16bq8h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72612 /* svmaxnmqv_f32 */, 6901 /* V4fq16bq4f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72626 /* svmaxnmqv_f64 */, 6924 /* V2dq16bq2d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72640 /* svmaxnmv_f16 */, 7146 /* hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72653 /* svmaxnmv_f32 */, 7166 /* fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72666 /* svmaxnmv_f64 */, 7186 /* dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72679 /* svmaxp_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72692 /* svmaxp_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72705 /* svmaxp_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72718 /* svmaxp_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72731 /* svmaxp_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72744 /* svmaxp_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72757 /* svmaxp_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72770 /* svmaxp_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72783 /* svmaxp_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72796 /* svmaxp_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72809 /* svmaxp_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72822 /* svmaxp_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72835 /* svmaxp_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72847 /* svmaxp_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72859 /* svmaxp_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72872 /* svmaxp_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72885 /* svmaxp_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72898 /* svmaxp_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72911 /* svmaxp_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72924 /* svmaxp_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72937 /* svmaxp_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72949 /* svmaxp_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72961 /* svmaxqv_f16 */, 6834 /* V8hq16bq8h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72973 /* svmaxqv_f32 */, 6901 /* V4fq16bq4f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72985 /* svmaxqv_f64 */, 6924 /* V2dq16bq2d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{72997 /* svmaxqv_s16 */, 6947 /* V8sq16bq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73009 /* svmaxqv_s32 */, 6970 /* V4iq16bq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73021 /* svmaxqv_s64 */, 6993 /* V2Wiq16bq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73033 /* svmaxqv_s8 */, 7017 /* V16Scq16bq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73044 /* svmaxqv_u16 */, 7044 /* V8Usq16bq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73056 /* svmaxqv_u32 */, 7069 /* V4Uiq16bq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73068 /* svmaxqv_u64 */, 7094 /* V2UWiq16bq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73080 /* svmaxqv_u8 */, 7120 /* V16Ucq16bq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73091 /* svmaxv_f16 */, 7146 /* hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73102 /* svmaxv_f32 */, 7166 /* fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73113 /* svmaxv_f64 */, 7186 /* dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73124 /* svmaxv_s16 */, 10150 /* sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73135 /* svmaxv_s32 */, 10170 /* iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73146 /* svmaxv_s64 */, 7248 /* Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73157 /* svmaxv_s8 */, 10200 /* Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73167 /* svmaxv_u16 */, 10223 /* Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73178 /* svmaxv_u32 */, 10245 /* Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73189 /* svmaxv_u64 */, 7338 /* UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73200 /* svmaxv_u8 */, 10277 /* Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73210 /* svmin_bf16_m */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73223 /* svmin_bf16_x */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73236 /* svmin_bf16_x2 */, 69816 /* q16yq16yq16y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73250 /* svmin_bf16_x4 */, 69843 /* q32yq32yq32y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73264 /* svmin_bf16_z */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73277 /* svmin_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73289 /* svmin_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73301 /* svmin_f16_x2 */, 8687 /* q16hq16hq16h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73314 /* svmin_f16_x4 */, 8732 /* q32hq32hq32h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73327 /* svmin_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73339 /* svmin_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73351 /* svmin_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73363 /* svmin_f32_x2 */, 8798 /* q8fq8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73376 /* svmin_f32_x4 */, 8822 /* q16fq16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73389 /* svmin_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73401 /* svmin_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73413 /* svmin_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73425 /* svmin_f64_x2 */, 8888 /* q4dq4dq4d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73438 /* svmin_f64_x4 */, 8912 /* q8dq8dq8d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73451 /* svmin_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73463 /* svmin_n_bf16_m */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73478 /* svmin_n_bf16_x */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73493 /* svmin_n_bf16_z */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73508 /* svmin_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73522 /* svmin_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73536 /* svmin_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73550 /* svmin_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73564 /* svmin_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73578 /* svmin_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73592 /* svmin_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73606 /* svmin_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73620 /* svmin_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73634 /* svmin_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73648 /* svmin_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73662 /* svmin_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73676 /* svmin_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73690 /* svmin_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73704 /* svmin_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73718 /* svmin_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73732 /* svmin_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73746 /* svmin_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73760 /* svmin_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73773 /* svmin_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73786 /* svmin_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73799 /* svmin_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73813 /* svmin_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73827 /* svmin_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73841 /* svmin_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73855 /* svmin_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73869 /* svmin_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73883 /* svmin_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73897 /* svmin_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73911 /* svmin_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73925 /* svmin_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73938 /* svmin_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73951 /* svmin_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73964 /* svmin_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73976 /* svmin_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73988 /* svmin_s16_x2 */, 70593 /* q16sq16sq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74001 /* svmin_s16_x4 */, 70619 /* q32sq32sq32s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74014 /* svmin_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74026 /* svmin_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74038 /* svmin_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74050 /* svmin_s32_x2 */, 70681 /* q8iq8iq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74063 /* svmin_s32_x4 */, 70704 /* q16iq16iq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74076 /* svmin_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74088 /* svmin_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74100 /* svmin_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74112 /* svmin_s64_x2 */, 70766 /* q4Wiq4Wiq4Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74125 /* svmin_s64_x4 */, 70792 /* q8Wiq8Wiq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74138 /* svmin_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74150 /* svmin_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74161 /* svmin_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74172 /* svmin_s8_x2 */, 70851 /* q32Scq32Scq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74184 /* svmin_s8_x4 */, 70879 /* q64Scq64Scq64Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74196 /* svmin_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74207 /* svmin_single_bf16_x2 */, 70927 /* q16yq16yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74228 /* svmin_single_bf16_x4 */, 70960 /* q32yq32yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74249 /* svmin_single_f16_x2 */, 70992 /* q16hq16hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74269 /* svmin_single_f16_x4 */, 71024 /* q32hq32hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74289 /* svmin_single_f32_x2 */, 71056 /* q8fq8fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74309 /* svmin_single_f32_x4 */, 71086 /* q16fq16fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74329 /* svmin_single_f64_x2 */, 71118 /* q4dq4dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74349 /* svmin_single_f64_x4 */, 71148 /* q8dq8dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74369 /* svmin_single_s16_x2 */, 4741 /* q16sq16sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74389 /* svmin_single_s16_x4 */, 4782 /* q32sq32sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74409 /* svmin_single_s32_x2 */, 4814 /* q8iq8iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74429 /* svmin_single_s32_x4 */, 4844 /* q16iq16iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74449 /* svmin_single_s64_x2 */, 4876 /* q4Wiq4Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74469 /* svmin_single_s64_x4 */, 4909 /* q8Wiq8Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74489 /* svmin_single_s8_x2 */, 4941 /* q32Scq32Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74508 /* svmin_single_s8_x4 */, 4976 /* q64Scq64Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74527 /* svmin_single_u16_x2 */, 5012 /* q16Usq16Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74547 /* svmin_single_u16_x4 */, 5047 /* q32Usq32Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74567 /* svmin_single_u32_x2 */, 5082 /* q8Uiq8Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74587 /* svmin_single_u32_x4 */, 5115 /* q16Uiq16Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74607 /* svmin_single_u64_x2 */, 5150 /* q4UWiq4UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74627 /* svmin_single_u64_x4 */, 5186 /* q8UWiq8UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74647 /* svmin_single_u8_x2 */, 5221 /* q32Ucq32Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74666 /* svmin_single_u8_x4 */, 5256 /* q64Ucq64Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74685 /* svmin_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74697 /* svmin_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74709 /* svmin_u16_x2 */, 71511 /* q16Usq16Usq16Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74722 /* svmin_u16_x4 */, 71540 /* q32Usq32Usq32Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74735 /* svmin_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74747 /* svmin_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74759 /* svmin_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74771 /* svmin_u32_x2 */, 71605 /* q8Uiq8Uiq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74784 /* svmin_u32_x4 */, 71631 /* q16Uiq16Uiq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74797 /* svmin_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74809 /* svmin_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74821 /* svmin_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74833 /* svmin_u64_x2 */, 71696 /* q4UWiq4UWiq4UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74846 /* svmin_u64_x4 */, 71725 /* q8UWiq8UWiq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74859 /* svmin_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74871 /* svmin_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74882 /* svmin_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74893 /* svmin_u8_x2 */, 71787 /* q32Ucq32Ucq32Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74905 /* svmin_u8_x4 */, 71815 /* q64Ucq64Ucq64Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74917 /* svmin_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74928 /* svminnm_bf16_m */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74943 /* svminnm_bf16_x */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74958 /* svminnm_bf16_x2 */, 69816 /* q16yq16yq16y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74974 /* svminnm_bf16_x4 */, 69843 /* q32yq32yq32y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{74990 /* svminnm_bf16_z */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75005 /* svminnm_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75019 /* svminnm_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75033 /* svminnm_f16_x2 */, 8687 /* q16hq16hq16h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75048 /* svminnm_f16_x4 */, 8732 /* q32hq32hq32h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75063 /* svminnm_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75077 /* svminnm_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75091 /* svminnm_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75105 /* svminnm_f32_x2 */, 8798 /* q8fq8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75120 /* svminnm_f32_x4 */, 8822 /* q16fq16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75135 /* svminnm_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75149 /* svminnm_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75163 /* svminnm_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75177 /* svminnm_f64_x2 */, 8888 /* q4dq4dq4d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75192 /* svminnm_f64_x4 */, 8912 /* q8dq8dq8d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75207 /* svminnm_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75221 /* svminnm_n_bf16_m */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75238 /* svminnm_n_bf16_x */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75255 /* svminnm_n_bf16_z */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75272 /* svminnm_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75288 /* svminnm_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75304 /* svminnm_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75320 /* svminnm_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75336 /* svminnm_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75352 /* svminnm_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75368 /* svminnm_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75384 /* svminnm_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75400 /* svminnm_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75416 /* svminnm_single_bf16_x2 */, 70927 /* q16yq16yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75439 /* svminnm_single_bf16_x4 */, 70960 /* q32yq32yq8y */, 1 /* n */, 14323 /* sme,sme2,sve-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75462 /* svminnm_single_f16_x2 */, 70992 /* q16hq16hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75484 /* svminnm_single_f16_x4 */, 71024 /* q32hq32hq8h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75506 /* svminnm_single_f32_x2 */, 71056 /* q8fq8fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75528 /* svminnm_single_f32_x4 */, 71086 /* q16fq16fq4f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75550 /* svminnm_single_f64_x2 */, 71118 /* q4dq4dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75572 /* svminnm_single_f64_x4 */, 71148 /* q8dq8dq2d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75594 /* svminnmp_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75609 /* svminnmp_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75624 /* svminnmp_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75639 /* svminnmp_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75654 /* svminnmp_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75669 /* svminnmp_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75684 /* svminnmqv_f16 */, 6834 /* V8hq16bq8h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75698 /* svminnmqv_f32 */, 6901 /* V4fq16bq4f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75712 /* svminnmqv_f64 */, 6924 /* V2dq16bq2d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75726 /* svminnmv_f16 */, 7146 /* hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75739 /* svminnmv_f32 */, 7166 /* fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75752 /* svminnmv_f64 */, 7186 /* dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75765 /* svminp_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75778 /* svminp_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75791 /* svminp_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75804 /* svminp_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75817 /* svminp_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75830 /* svminp_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75843 /* svminp_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75856 /* svminp_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75869 /* svminp_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75882 /* svminp_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75895 /* svminp_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75908 /* svminp_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75921 /* svminp_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75933 /* svminp_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75945 /* svminp_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75958 /* svminp_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75971 /* svminp_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75984 /* svminp_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{75997 /* svminp_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76010 /* svminp_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76023 /* svminp_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76035 /* svminp_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76047 /* svminqv_f16 */, 6834 /* V8hq16bq8h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76059 /* svminqv_f32 */, 6901 /* V4fq16bq4f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76071 /* svminqv_f64 */, 6924 /* V2dq16bq2d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76083 /* svminqv_s16 */, 6947 /* V8sq16bq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76095 /* svminqv_s32 */, 6970 /* V4iq16bq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76107 /* svminqv_s64 */, 6993 /* V2Wiq16bq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76119 /* svminqv_s8 */, 7017 /* V16Scq16bq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76130 /* svminqv_u16 */, 7044 /* V8Usq16bq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76142 /* svminqv_u32 */, 7069 /* V4Uiq16bq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76154 /* svminqv_u64 */, 7094 /* V2UWiq16bq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76166 /* svminqv_u8 */, 7120 /* V16Ucq16bq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76177 /* svminv_f16 */, 7146 /* hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76188 /* svminv_f32 */, 7166 /* fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76199 /* svminv_f64 */, 7186 /* dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76210 /* svminv_s16 */, 10150 /* sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76221 /* svminv_s32 */, 10170 /* iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76232 /* svminv_s64 */, 7248 /* Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76243 /* svminv_s8 */, 10200 /* Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76253 /* svminv_u16 */, 10223 /* Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76264 /* svminv_u32 */, 10245 /* Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76275 /* svminv_u64 */, 7338 /* UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76286 /* svminv_u8 */, 10277 /* Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76296 /* svmla_bf16_m */, 76309 /* q8yq16bq8yq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76326 /* svmla_bf16_x */, 76309 /* q8yq16bq8yq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76339 /* svmla_bf16_z */, 76309 /* q8yq16bq8yq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76352 /* svmla_f16_m */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76364 /* svmla_f16_x */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76376 /* svmla_f16_z */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76388 /* svmla_f32_m */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76400 /* svmla_f32_x */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76412 /* svmla_f32_z */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76424 /* svmla_f64_m */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76436 /* svmla_f64_x */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76448 /* svmla_f64_z */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76460 /* svmla_lane_bf16 */, 76476 /* q8yq8yq8yq8yIUWi */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76493 /* svmla_lane_f16 */, 76508 /* q8hq8hq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76525 /* svmla_lane_f32 */, 76540 /* q4fq4fq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76557 /* svmla_lane_f64 */, 76572 /* q2dq2dq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76589 /* svmla_lane_s16 */, 17163 /* q8sq8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76604 /* svmla_lane_s32 */, 17191 /* q4iq4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76619 /* svmla_lane_s64 */, 17219 /* q2Wiq2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76634 /* svmla_lane_u16 */, 17286 /* q8Usq8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76649 /* svmla_lane_u32 */, 17318 /* q4Uiq4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76664 /* svmla_lane_u64 */, 17350 /* q2UWiq2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76679 /* svmla_n_bf16_m */, 76694 /* q8yq16bq8yq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76709 /* svmla_n_bf16_x */, 76694 /* q8yq16bq8yq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76724 /* svmla_n_bf16_z */, 76694 /* q8yq16bq8yq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76739 /* svmla_n_f16_m */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76753 /* svmla_n_f16_x */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76767 /* svmla_n_f16_z */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76781 /* svmla_n_f32_m */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76795 /* svmla_n_f32_x */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76809 /* svmla_n_f32_z */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76823 /* svmla_n_f64_m */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76837 /* svmla_n_f64_x */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76851 /* svmla_n_f64_z */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76865 /* svmla_n_s16_m */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76879 /* svmla_n_s16_x */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76893 /* svmla_n_s16_z */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76907 /* svmla_n_s32_m */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76921 /* svmla_n_s32_x */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76935 /* svmla_n_s32_z */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76949 /* svmla_n_s64_m */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76963 /* svmla_n_s64_x */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76977 /* svmla_n_s64_z */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{76991 /* svmla_n_s8_m */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77004 /* svmla_n_s8_x */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77017 /* svmla_n_s8_z */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77030 /* svmla_n_u16_m */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77044 /* svmla_n_u16_x */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77058 /* svmla_n_u16_z */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77072 /* svmla_n_u32_m */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77086 /* svmla_n_u32_x */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77100 /* svmla_n_u32_z */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77114 /* svmla_n_u64_m */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77128 /* svmla_n_u64_x */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77142 /* svmla_n_u64_z */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77156 /* svmla_n_u8_m */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77169 /* svmla_n_u8_x */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77182 /* svmla_n_u8_z */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77195 /* svmla_s16_m */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77207 /* svmla_s16_x */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77219 /* svmla_s16_z */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77231 /* svmla_s32_m */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77243 /* svmla_s32_x */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77255 /* svmla_s32_z */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77267 /* svmla_s64_m */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77279 /* svmla_s64_x */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77291 /* svmla_s64_z */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77303 /* svmla_s8_m */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77314 /* svmla_s8_x */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77325 /* svmla_s8_z */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77336 /* svmla_u16_m */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77348 /* svmla_u16_x */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77360 /* svmla_u16_z */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77372 /* svmla_u32_m */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77384 /* svmla_u32_x */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77396 /* svmla_u32_z */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77408 /* svmla_u64_m */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77420 /* svmla_u64_x */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77432 /* svmla_u64_z */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77444 /* svmla_u8_m */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77455 /* svmla_u8_x */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77466 /* svmla_u8_z */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77477 /* svmlalb_f16_mf8_fpm */, 28342 /* q8hq8hq16mq16mUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77533 /* svmlalb_f32 */, 28412 /* q4fq4fq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77545 /* svmlalb_lane_f16_mf8_fpm */, 28522 /* q8hq8hq16mq16mIUWiUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77570 /* svmlalb_lane_f32 */, 28563 /* q4fq4fq8hq8hIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77587 /* svmlalb_lane_s32 */, 28659 /* q4iq4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77604 /* svmlalb_lane_s64 */, 77621 /* q2Wiq2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77640 /* svmlalb_lane_u32 */, 28748 /* q4Uiq4Uiq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77657 /* svmlalb_lane_u64 */, 77674 /* q2UWiq2UWiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77697 /* svmlalb_n_f16_mf8_fpm */, 28827 /* q8hq8hq16mmUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77719 /* svmlalb_n_f32 */, 77733 /* q4fq4fq8hh */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77744 /* svmlalb_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77758 /* svmlalb_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77772 /* svmlalb_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77786 /* svmlalb_n_u16 */, 552 /* q8Usq8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77800 /* svmlalb_n_u32 */, 582 /* q4Uiq4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77814 /* svmlalb_n_u64 */, 611 /* q2UWiq2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77828 /* svmlalb_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77840 /* svmlalb_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77852 /* svmlalb_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77864 /* svmlalb_u16 */, 721 /* q8Usq8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77876 /* svmlalb_u32 */, 752 /* q4Uiq4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77888 /* svmlalb_u64 */, 781 /* q2UWiq2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77900 /* svmlallbb_f32_mf8_fpm */, 28443 /* q4fq4fq16mq16mUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77922 /* svmlallbb_lane_f32_mf8_fpm */, 28603 /* q4fq4fq16mq16mIUWiUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77949 /* svmlallbb_n_f32_mf8_fpm */, 28862 /* q4fq4fq16mmUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77973 /* svmlallbt_f32_mf8_fpm */, 28443 /* q4fq4fq16mq16mUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{77995 /* svmlallbt_lane_f32_mf8_fpm */, 28603 /* q4fq4fq16mq16mIUWiUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78022 /* svmlallbt_n_f32_mf8_fpm */, 28862 /* q4fq4fq16mmUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78046 /* svmlalltb_f32_mf8_fpm */, 28443 /* q4fq4fq16mq16mUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78068 /* svmlalltb_lane_f32_mf8_fpm */, 28603 /* q4fq4fq16mq16mIUWiUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78095 /* svmlalltb_n_f32_mf8_fpm */, 28862 /* q4fq4fq16mmUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78119 /* svmlalltt_f32_mf8_fpm */, 28443 /* q4fq4fq16mq16mUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78141 /* svmlalltt_lane_f32_mf8_fpm */, 28603 /* q4fq4fq16mq16mIUWiUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78168 /* svmlalltt_n_f32_mf8_fpm */, 28862 /* q4fq4fq16mmUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78192 /* svmlalt_f16_mf8_fpm */, 28342 /* q8hq8hq16mq16mUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78212 /* svmlalt_f32 */, 28412 /* q4fq4fq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78224 /* svmlalt_lane_f16_mf8_fpm */, 28522 /* q8hq8hq16mq16mIUWiUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78249 /* svmlalt_lane_f32 */, 28563 /* q4fq4fq8hq8hIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78266 /* svmlalt_lane_s32 */, 28659 /* q4iq4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78283 /* svmlalt_lane_s64 */, 77621 /* q2Wiq2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78300 /* svmlalt_lane_u32 */, 28748 /* q4Uiq4Uiq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78317 /* svmlalt_lane_u64 */, 77674 /* q2UWiq2UWiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78334 /* svmlalt_n_f16_mf8_fpm */, 28827 /* q8hq8hq16mmUWi */, 1 /* n */, 77497 /* (sve,sve2,fp8fma)|(sme,ssve-fp8fma) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78356 /* svmlalt_n_f32 */, 77733 /* q4fq4fq8hh */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78370 /* svmlalt_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78384 /* svmlalt_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78398 /* svmlalt_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78412 /* svmlalt_n_u16 */, 552 /* q8Usq8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78426 /* svmlalt_n_u32 */, 582 /* q4Uiq4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78440 /* svmlalt_n_u64 */, 611 /* q2UWiq2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78454 /* svmlalt_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78466 /* svmlalt_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78478 /* svmlalt_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78490 /* svmlalt_u16 */, 721 /* q8Usq8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78502 /* svmlalt_u32 */, 752 /* q4Uiq4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78514 /* svmlalt_u64 */, 781 /* q2UWiq2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78526 /* svmls_bf16_m */, 76309 /* q8yq16bq8yq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78539 /* svmls_bf16_x */, 76309 /* q8yq16bq8yq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78552 /* svmls_bf16_z */, 76309 /* q8yq16bq8yq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78565 /* svmls_f16_m */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78577 /* svmls_f16_x */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78589 /* svmls_f16_z */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78601 /* svmls_f32_m */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78613 /* svmls_f32_x */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78625 /* svmls_f32_z */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78637 /* svmls_f64_m */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78649 /* svmls_f64_x */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78661 /* svmls_f64_z */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78673 /* svmls_lane_bf16 */, 76476 /* q8yq8yq8yq8yIUWi */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78689 /* svmls_lane_f16 */, 76508 /* q8hq8hq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78704 /* svmls_lane_f32 */, 76540 /* q4fq4fq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78719 /* svmls_lane_f64 */, 76572 /* q2dq2dq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78734 /* svmls_lane_s16 */, 17163 /* q8sq8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78749 /* svmls_lane_s32 */, 17191 /* q4iq4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78764 /* svmls_lane_s64 */, 17219 /* q2Wiq2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78779 /* svmls_lane_u16 */, 17286 /* q8Usq8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78794 /* svmls_lane_u32 */, 17318 /* q4Uiq4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78809 /* svmls_lane_u64 */, 17350 /* q2UWiq2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78824 /* svmls_n_bf16_m */, 76694 /* q8yq16bq8yq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78839 /* svmls_n_bf16_x */, 76694 /* q8yq16bq8yq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78854 /* svmls_n_bf16_z */, 76694 /* q8yq16bq8yq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78869 /* svmls_n_f16_m */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78883 /* svmls_n_f16_x */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78897 /* svmls_n_f16_z */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78911 /* svmls_n_f32_m */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78925 /* svmls_n_f32_x */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78939 /* svmls_n_f32_z */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78953 /* svmls_n_f64_m */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78967 /* svmls_n_f64_x */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78981 /* svmls_n_f64_z */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{78995 /* svmls_n_s16_m */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79009 /* svmls_n_s16_x */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79023 /* svmls_n_s16_z */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79037 /* svmls_n_s32_m */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79051 /* svmls_n_s32_x */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79065 /* svmls_n_s32_z */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79079 /* svmls_n_s64_m */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79093 /* svmls_n_s64_x */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79107 /* svmls_n_s64_z */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79121 /* svmls_n_s8_m */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79134 /* svmls_n_s8_x */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79147 /* svmls_n_s8_z */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79160 /* svmls_n_u16_m */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79174 /* svmls_n_u16_x */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79188 /* svmls_n_u16_z */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79202 /* svmls_n_u32_m */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79216 /* svmls_n_u32_x */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79230 /* svmls_n_u32_z */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79244 /* svmls_n_u64_m */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79258 /* svmls_n_u64_x */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79272 /* svmls_n_u64_z */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79286 /* svmls_n_u8_m */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79299 /* svmls_n_u8_x */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79312 /* svmls_n_u8_z */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79325 /* svmls_s16_m */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79337 /* svmls_s16_x */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79349 /* svmls_s16_z */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79361 /* svmls_s32_m */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79373 /* svmls_s32_x */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79385 /* svmls_s32_z */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79397 /* svmls_s64_m */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79409 /* svmls_s64_x */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79421 /* svmls_s64_z */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79433 /* svmls_s8_m */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79444 /* svmls_s8_x */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79455 /* svmls_s8_z */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79466 /* svmls_u16_m */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79478 /* svmls_u16_x */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79490 /* svmls_u16_z */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79502 /* svmls_u32_m */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79514 /* svmls_u32_x */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79526 /* svmls_u32_z */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79538 /* svmls_u64_m */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79550 /* svmls_u64_x */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79562 /* svmls_u64_z */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79574 /* svmls_u8_m */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79585 /* svmls_u8_x */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79596 /* svmls_u8_z */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79607 /* svmlslb_f32 */, 28412 /* q4fq4fq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79619 /* svmlslb_lane_f32 */, 28563 /* q4fq4fq8hq8hIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79636 /* svmlslb_lane_s32 */, 28659 /* q4iq4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79653 /* svmlslb_lane_s64 */, 77621 /* q2Wiq2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79670 /* svmlslb_lane_u32 */, 28748 /* q4Uiq4Uiq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79687 /* svmlslb_lane_u64 */, 77674 /* q2UWiq2UWiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79704 /* svmlslb_n_f32 */, 77733 /* q4fq4fq8hh */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79718 /* svmlslb_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79732 /* svmlslb_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79746 /* svmlslb_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79760 /* svmlslb_n_u16 */, 552 /* q8Usq8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79774 /* svmlslb_n_u32 */, 582 /* q4Uiq4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79788 /* svmlslb_n_u64 */, 611 /* q2UWiq2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79802 /* svmlslb_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79814 /* svmlslb_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79826 /* svmlslb_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79838 /* svmlslb_u16 */, 721 /* q8Usq8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79850 /* svmlslb_u32 */, 752 /* q4Uiq4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79862 /* svmlslb_u64 */, 781 /* q2UWiq2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79874 /* svmlslt_f32 */, 28412 /* q4fq4fq8hq8h */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79886 /* svmlslt_lane_f32 */, 28563 /* q4fq4fq8hq8hIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79903 /* svmlslt_lane_s32 */, 28659 /* q4iq4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79920 /* svmlslt_lane_s64 */, 77621 /* q2Wiq2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79937 /* svmlslt_lane_u32 */, 28748 /* q4Uiq4Uiq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79954 /* svmlslt_lane_u64 */, 77674 /* q2UWiq2UWiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79971 /* svmlslt_n_f32 */, 77733 /* q4fq4fq8hh */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79985 /* svmlslt_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{79999 /* svmlslt_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80013 /* svmlslt_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80027 /* svmlslt_n_u16 */, 552 /* q8Usq8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80041 /* svmlslt_n_u32 */, 582 /* q4Uiq4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80055 /* svmlslt_n_u64 */, 611 /* q2UWiq2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80069 /* svmlslt_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80081 /* svmlslt_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80093 /* svmlslt_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80105 /* svmlslt_u16 */, 721 /* q8Usq8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80117 /* svmlslt_u32 */, 752 /* q4Uiq4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80129 /* svmlslt_u64 */, 781 /* q2UWiq2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80141 /* svmmla_f16_mf8_fpm */, 28342 /* q8hq8hq16mq16mUWi */, 1 /* n */, 80160 /* sve,sve2,f8f16mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80177 /* svmmla_f32 */, 14177 /* q4fq4fq4fq4f */, 1 /* n */, 80188 /* sve,f32mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80198 /* svmmla_f32_f16 */, 28412 /* q4fq4fq8hq8h */, 1 /* n */, 80213 /* sve,sve-f16f32mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80230 /* svmmla_f32_mf8_fpm */, 28443 /* q4fq4fq16mq16mUWi */, 1 /* n */, 80249 /* sve,sve2,f8f32mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80266 /* svmmla_f64 */, 14202 /* q2dq2dq2dq2d */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80277 /* svmmla_s32 */, 28995 /* q4iq4iq16Scq16Sc */, 1 /* n */, 80288 /* sve,i8mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80297 /* svmmla_u32 */, 29061 /* q4Uiq4Uiq16Ucq16Uc */, 1 /* n */, 80288 /* sve,i8mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80308 /* svmov_b_z */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80318 /* svmovlb_s16 */, 80330 /* q8sq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80339 /* svmovlb_s32 */, 80351 /* q4iq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80358 /* svmovlb_s64 */, 80370 /* q2Wiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80378 /* svmovlb_u16 */, 80390 /* q8Usq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80400 /* svmovlb_u32 */, 80412 /* q4Uiq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80421 /* svmovlb_u64 */, 80433 /* q2UWiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80443 /* svmovlt_s16 */, 80330 /* q8sq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80455 /* svmovlt_s32 */, 80351 /* q4iq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80467 /* svmovlt_s64 */, 80370 /* q2Wiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80479 /* svmovlt_u16 */, 80390 /* q8Usq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80491 /* svmovlt_u32 */, 80412 /* q4Uiq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80503 /* svmovlt_u64 */, 80433 /* q2UWiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80515 /* svmsb_f16_m */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80527 /* svmsb_f16_x */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80539 /* svmsb_f16_z */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80551 /* svmsb_f32_m */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80563 /* svmsb_f32_x */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80575 /* svmsb_f32_z */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80587 /* svmsb_f64_m */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80599 /* svmsb_f64_x */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80611 /* svmsb_f64_z */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80623 /* svmsb_n_f16_m */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80637 /* svmsb_n_f16_x */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80651 /* svmsb_n_f16_z */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80665 /* svmsb_n_f32_m */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80679 /* svmsb_n_f32_x */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80693 /* svmsb_n_f32_z */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80707 /* svmsb_n_f64_m */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80721 /* svmsb_n_f64_x */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80735 /* svmsb_n_f64_z */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80749 /* svmsb_n_s16_m */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80763 /* svmsb_n_s16_x */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80777 /* svmsb_n_s16_z */, 68806 /* q8sq16bq8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80791 /* svmsb_n_s32_m */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80805 /* svmsb_n_s32_x */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80819 /* svmsb_n_s32_z */, 68863 /* q4iq16bq4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80833 /* svmsb_n_s64_m */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80847 /* svmsb_n_s64_x */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80861 /* svmsb_n_s64_z */, 68920 /* q2Wiq16bq2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80875 /* svmsb_n_s8_m */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80888 /* svmsb_n_s8_x */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80901 /* svmsb_n_s8_z */, 68980 /* q16Scq16bq16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80914 /* svmsb_n_u16_m */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80928 /* svmsb_n_u16_x */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80942 /* svmsb_n_u16_z */, 69042 /* q8Usq16bq8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80956 /* svmsb_n_u32_m */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80970 /* svmsb_n_u32_x */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80984 /* svmsb_n_u32_z */, 69103 /* q4Uiq16bq4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{80998 /* svmsb_n_u64_m */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81012 /* svmsb_n_u64_x */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81026 /* svmsb_n_u64_z */, 69164 /* q2UWiq16bq2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81040 /* svmsb_n_u8_m */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81053 /* svmsb_n_u8_x */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81066 /* svmsb_n_u8_z */, 69228 /* q16Ucq16bq16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81079 /* svmsb_s16_m */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81091 /* svmsb_s16_x */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81103 /* svmsb_s16_z */, 69288 /* q8sq16bq8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81115 /* svmsb_s32_m */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81127 /* svmsb_s32_x */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81139 /* svmsb_s32_z */, 69341 /* q4iq16bq4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81151 /* svmsb_s64_m */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81163 /* svmsb_s64_x */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81175 /* svmsb_s64_z */, 69394 /* q2Wiq16bq2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81187 /* svmsb_s8_m */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81198 /* svmsb_s8_x */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81209 /* svmsb_s8_z */, 69450 /* q16Scq16bq16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81220 /* svmsb_u16_m */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81232 /* svmsb_u16_x */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81244 /* svmsb_u16_z */, 69509 /* q8Usq16bq8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81256 /* svmsb_u32_m */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81268 /* svmsb_u32_x */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81280 /* svmsb_u32_z */, 69566 /* q4Uiq16bq4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81292 /* svmsb_u64_m */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81304 /* svmsb_u64_x */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81316 /* svmsb_u64_z */, 69623 /* q2UWiq16bq2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81328 /* svmsb_u8_m */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81339 /* svmsb_u8_x */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81350 /* svmsb_u8_z */, 69683 /* q16Ucq16bq16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81361 /* svmul_bf16_m */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81374 /* svmul_bf16_x */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81387 /* svmul_bf16_x2 */, 69816 /* q16yq16yq16y */, 1 /* n */, 81401 /* sme,sme2,sve-bfscale */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81422 /* svmul_bf16_x4 */, 69843 /* q32yq32yq32y */, 1 /* n */, 81401 /* sme,sme2,sve-bfscale */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81436 /* svmul_bf16_z */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81449 /* svmul_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81461 /* svmul_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81473 /* svmul_f16_x2 */, 8687 /* q16hq16hq16h */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81497 /* svmul_f16_x4 */, 8732 /* q32hq32hq32h */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81510 /* svmul_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81522 /* svmul_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81534 /* svmul_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81546 /* svmul_f32_x2 */, 8798 /* q8fq8fq8f */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81559 /* svmul_f32_x4 */, 8822 /* q16fq16fq16f */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81572 /* svmul_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81584 /* svmul_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81596 /* svmul_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81608 /* svmul_f64_x2 */, 8888 /* q4dq4dq4d */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81621 /* svmul_f64_x4 */, 8912 /* q8dq8dq8d */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81634 /* svmul_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81646 /* svmul_lane_bf16 */, 33217 /* q8yq8yq8yIUWi */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81662 /* svmul_lane_f16 */, 33241 /* q8hq8hq8hIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81677 /* svmul_lane_f32 */, 33265 /* q4fq4fq4fIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81692 /* svmul_lane_f64 */, 33289 /* q2dq2dq2dIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81707 /* svmul_lane_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81722 /* svmul_lane_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81737 /* svmul_lane_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81752 /* svmul_lane_u16 */, 13866 /* q8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81767 /* svmul_lane_u32 */, 13894 /* q4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81782 /* svmul_lane_u64 */, 13922 /* q2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81797 /* svmul_n_bf16_m */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81812 /* svmul_n_bf16_x */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81827 /* svmul_n_bf16_z */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81842 /* svmul_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81856 /* svmul_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81870 /* svmul_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81884 /* svmul_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81898 /* svmul_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81912 /* svmul_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81926 /* svmul_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81940 /* svmul_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81954 /* svmul_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81968 /* svmul_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81982 /* svmul_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{81996 /* svmul_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82010 /* svmul_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82024 /* svmul_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82038 /* svmul_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82052 /* svmul_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82066 /* svmul_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82080 /* svmul_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82094 /* svmul_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82107 /* svmul_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82120 /* svmul_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82133 /* svmul_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82147 /* svmul_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82161 /* svmul_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82175 /* svmul_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82189 /* svmul_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82203 /* svmul_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82217 /* svmul_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82231 /* svmul_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82245 /* svmul_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82259 /* svmul_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82272 /* svmul_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82285 /* svmul_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82298 /* svmul_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82310 /* svmul_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82322 /* svmul_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82334 /* svmul_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82346 /* svmul_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82358 /* svmul_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82370 /* svmul_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82382 /* svmul_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82394 /* svmul_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82406 /* svmul_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82417 /* svmul_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82428 /* svmul_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82439 /* svmul_single_bf16_x2 */, 70927 /* q16yq16yq8y */, 1 /* n */, 81401 /* sme,sme2,sve-bfscale */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82460 /* svmul_single_bf16_x4 */, 70960 /* q32yq32yq8y */, 1 /* n */, 81401 /* sme,sme2,sve-bfscale */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82481 /* svmul_single_f16_x2 */, 70992 /* q16hq16hq8h */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82501 /* svmul_single_f16_x4 */, 71024 /* q32hq32hq8h */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82521 /* svmul_single_f32_x2 */, 71056 /* q8fq8fq4f */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82541 /* svmul_single_f32_x4 */, 71086 /* q16fq16fq4f */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82561 /* svmul_single_f64_x2 */, 71118 /* q4dq4dq2d */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82581 /* svmul_single_f64_x4 */, 71148 /* q8dq8dq2d */, 1 /* n */, 81486 /* sme,sme2p2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82601 /* svmul_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82613 /* svmul_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82625 /* svmul_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82637 /* svmul_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82649 /* svmul_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82661 /* svmul_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82673 /* svmul_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82685 /* svmul_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82697 /* svmul_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82709 /* svmul_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82720 /* svmul_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82731 /* svmul_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82742 /* svmulh_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82757 /* svmulh_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82772 /* svmulh_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82787 /* svmulh_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82802 /* svmulh_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82817 /* svmulh_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82832 /* svmulh_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82847 /* svmulh_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82862 /* svmulh_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82877 /* svmulh_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82891 /* svmulh_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82905 /* svmulh_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82919 /* svmulh_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82934 /* svmulh_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82949 /* svmulh_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82964 /* svmulh_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82979 /* svmulh_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{82994 /* svmulh_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83009 /* svmulh_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83024 /* svmulh_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83039 /* svmulh_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83054 /* svmulh_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83068 /* svmulh_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83082 /* svmulh_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83096 /* svmulh_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83109 /* svmulh_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83122 /* svmulh_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83135 /* svmulh_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83148 /* svmulh_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83161 /* svmulh_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83174 /* svmulh_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83187 /* svmulh_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83200 /* svmulh_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83213 /* svmulh_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83225 /* svmulh_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83237 /* svmulh_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83249 /* svmulh_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83262 /* svmulh_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83275 /* svmulh_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83288 /* svmulh_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83301 /* svmulh_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83314 /* svmulh_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83327 /* svmulh_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83340 /* svmulh_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83353 /* svmulh_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83366 /* svmulh_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83378 /* svmulh_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83390 /* svmulh_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83402 /* svmullb_lane_s32 */, 83419 /* q4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83433 /* svmullb_lane_s64 */, 83450 /* q2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83465 /* svmullb_lane_u32 */, 83482 /* q4Uiq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83499 /* svmullb_lane_u64 */, 83516 /* q2UWiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83534 /* svmullb_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83548 /* svmullb_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83562 /* svmullb_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83576 /* svmullb_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83590 /* svmullb_n_u32 */, 2258 /* q4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83604 /* svmullb_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83618 /* svmullb_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83630 /* svmullb_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83642 /* svmullb_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83654 /* svmullb_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83666 /* svmullb_u32 */, 2405 /* q4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83678 /* svmullb_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83690 /* svmullt_lane_s32 */, 83419 /* q4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83707 /* svmullt_lane_s64 */, 83450 /* q2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83724 /* svmullt_lane_u32 */, 83482 /* q4Uiq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83741 /* svmullt_lane_u64 */, 83516 /* q2UWiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83758 /* svmullt_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83772 /* svmullt_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83786 /* svmullt_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83800 /* svmullt_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83814 /* svmullt_n_u32 */, 2258 /* q4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83828 /* svmullt_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83842 /* svmullt_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83854 /* svmullt_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83866 /* svmullt_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83878 /* svmullt_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83890 /* svmullt_u32 */, 2405 /* q4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83902 /* svmullt_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83914 /* svmulx_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83927 /* svmulx_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83940 /* svmulx_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83953 /* svmulx_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83966 /* svmulx_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83979 /* svmulx_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{83992 /* svmulx_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84005 /* svmulx_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84018 /* svmulx_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84031 /* svmulx_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84046 /* svmulx_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84061 /* svmulx_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84076 /* svmulx_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84091 /* svmulx_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84106 /* svmulx_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84121 /* svmulx_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84136 /* svmulx_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84151 /* svmulx_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84166 /* svnand_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84177 /* svnbsl_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84190 /* svnbsl_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84203 /* svnbsl_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84216 /* svnbsl_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84228 /* svnbsl_n_u16 */, 138 /* q8Usq8Usq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84241 /* svnbsl_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84254 /* svnbsl_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84267 /* svnbsl_n_u8 */, 222 /* q16Ucq16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84279 /* svnbsl_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84290 /* svnbsl_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84301 /* svnbsl_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84312 /* svnbsl_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84322 /* svnbsl_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84333 /* svnbsl_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84344 /* svnbsl_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84355 /* svnbsl_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84365 /* svneg_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84377 /* svneg_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84389 /* svneg_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84401 /* svneg_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84413 /* svneg_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84425 /* svneg_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84437 /* svneg_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84449 /* svneg_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84461 /* svneg_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84473 /* svneg_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84485 /* svneg_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84497 /* svneg_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84509 /* svneg_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84521 /* svneg_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84533 /* svneg_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84545 /* svneg_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84557 /* svneg_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84569 /* svneg_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84581 /* svneg_s8_m */, 2982 /* q16Scq16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84592 /* svneg_s8_x */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84603 /* svneg_s8_z */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84614 /* svnmad_f16_m */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84627 /* svnmad_f16_x */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84640 /* svnmad_f16_z */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84653 /* svnmad_f32_m */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84666 /* svnmad_f32_x */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84679 /* svnmad_f32_z */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84692 /* svnmad_f64_m */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84705 /* svnmad_f64_x */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84718 /* svnmad_f64_z */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84731 /* svnmad_n_f16_m */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84746 /* svnmad_n_f16_x */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84761 /* svnmad_n_f16_z */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84776 /* svnmad_n_f32_m */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84791 /* svnmad_n_f32_x */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84806 /* svnmad_n_f32_z */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84821 /* svnmad_n_f64_m */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84836 /* svnmad_n_f64_x */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84851 /* svnmad_n_f64_z */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84866 /* svnmatch_s16 */, 17730 /* q16bq16bq8sq8s */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84879 /* svnmatch_s8 */, 17812 /* q16bq16bq16Scq16Sc */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84891 /* svnmatch_u16 */, 17843 /* q16bq16bq8Usq8Us */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84904 /* svnmatch_u8 */, 17931 /* q16bq16bq16Ucq16Uc */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84916 /* svnmla_f16_m */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84929 /* svnmla_f16_x */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84942 /* svnmla_f16_z */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84955 /* svnmla_f32_m */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84968 /* svnmla_f32_x */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84981 /* svnmla_f32_z */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{84994 /* svnmla_f64_m */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85007 /* svnmla_f64_x */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85020 /* svnmla_f64_z */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85033 /* svnmla_n_f16_m */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85048 /* svnmla_n_f16_x */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85063 /* svnmla_n_f16_z */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85078 /* svnmla_n_f32_m */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85093 /* svnmla_n_f32_x */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85108 /* svnmla_n_f32_z */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85123 /* svnmla_n_f64_m */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85138 /* svnmla_n_f64_x */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85153 /* svnmla_n_f64_z */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85168 /* svnmls_f16_m */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85181 /* svnmls_f16_x */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85194 /* svnmls_f16_z */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85207 /* svnmls_f32_m */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85220 /* svnmls_f32_x */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85233 /* svnmls_f32_z */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85246 /* svnmls_f64_m */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85259 /* svnmls_f64_x */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85272 /* svnmls_f64_z */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85285 /* svnmls_n_f16_m */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85300 /* svnmls_n_f16_x */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85315 /* svnmls_n_f16_z */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85330 /* svnmls_n_f32_m */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85345 /* svnmls_n_f32_x */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85360 /* svnmls_n_f32_z */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85375 /* svnmls_n_f64_m */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85390 /* svnmls_n_f64_x */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85405 /* svnmls_n_f64_z */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85420 /* svnmsb_f16_m */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85433 /* svnmsb_f16_x */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85446 /* svnmsb_f16_z */, 68474 /* q8hq16bq8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85459 /* svnmsb_f32_m */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85472 /* svnmsb_f32_x */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85485 /* svnmsb_f32_z */, 68527 /* q4fq16bq4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85498 /* svnmsb_f64_m */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85511 /* svnmsb_f64_x */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85524 /* svnmsb_f64_z */, 68580 /* q2dq16bq2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85537 /* svnmsb_n_f16_m */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85552 /* svnmsb_n_f16_x */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85567 /* svnmsb_n_f16_z */, 68635 /* q8hq16bq8hq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85582 /* svnmsb_n_f32_m */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85597 /* svnmsb_n_f32_x */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85612 /* svnmsb_n_f32_z */, 68692 /* q4fq16bq4fq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85627 /* svnmsb_n_f64_m */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85642 /* svnmsb_n_f64_x */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85657 /* svnmsb_n_f64_z */, 68749 /* q2dq16bq2dq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85672 /* svnor_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85682 /* svnot_b_z */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85692 /* svnot_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85704 /* svnot_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85716 /* svnot_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85728 /* svnot_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85740 /* svnot_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85752 /* svnot_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85764 /* svnot_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85776 /* svnot_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85788 /* svnot_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85800 /* svnot_s8_m */, 2982 /* q16Scq16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85811 /* svnot_s8_x */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85822 /* svnot_s8_z */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85833 /* svnot_u16_m */, 16483 /* q8Usq8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85845 /* svnot_u16_x */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85857 /* svnot_u16_z */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85869 /* svnot_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85881 /* svnot_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85893 /* svnot_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85905 /* svnot_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85917 /* svnot_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85929 /* svnot_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85941 /* svnot_u8_m */, 16685 /* q16Ucq16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85952 /* svnot_u8_x */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85963 /* svnot_u8_z */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85974 /* svorn_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85984 /* svorqv_s16 */, 6947 /* V8sq16bq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{85995 /* svorqv_s32 */, 6970 /* V4iq16bq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86006 /* svorqv_s64 */, 6993 /* V2Wiq16bq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86017 /* svorqv_s8 */, 7017 /* V16Scq16bq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86027 /* svorqv_u16 */, 7044 /* V8Usq16bq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86038 /* svorqv_u32 */, 7069 /* V4Uiq16bq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86049 /* svorqv_u64 */, 7094 /* V2UWiq16bq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86060 /* svorqv_u8 */, 7120 /* V16Ucq16bq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86070 /* svorr_b_z */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86080 /* svorr_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86094 /* svorr_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86108 /* svorr_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86122 /* svorr_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86136 /* svorr_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86150 /* svorr_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86164 /* svorr_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86178 /* svorr_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86192 /* svorr_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86206 /* svorr_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86219 /* svorr_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86232 /* svorr_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86245 /* svorr_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86259 /* svorr_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86273 /* svorr_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86287 /* svorr_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86301 /* svorr_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86315 /* svorr_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86329 /* svorr_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86343 /* svorr_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86357 /* svorr_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86371 /* svorr_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86384 /* svorr_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86397 /* svorr_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86410 /* svorr_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86422 /* svorr_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86434 /* svorr_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86446 /* svorr_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86458 /* svorr_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86470 /* svorr_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86482 /* svorr_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86494 /* svorr_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86506 /* svorr_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86518 /* svorr_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86529 /* svorr_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86540 /* svorr_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86551 /* svorr_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86563 /* svorr_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86575 /* svorr_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86587 /* svorr_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86599 /* svorr_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86611 /* svorr_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86623 /* svorr_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86635 /* svorr_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86647 /* svorr_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86659 /* svorr_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86670 /* svorr_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86681 /* svorr_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86692 /* svorv_s16 */, 10150 /* sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86702 /* svorv_s32 */, 10170 /* iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86712 /* svorv_s64 */, 7248 /* Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86722 /* svorv_s8 */, 10200 /* Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86731 /* svorv_u16 */, 10223 /* Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86741 /* svorv_u32 */, 10245 /* Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86751 /* svorv_u64 */, 7338 /* UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86761 /* svorv_u8 */, 10277 /* Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86770 /* svpext_lane_c16 */, 86786 /* q16bQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86797 /* svpext_lane_c16_x2 */, 86816 /* q32bQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86827 /* svpext_lane_c32 */, 86786 /* q16bQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86843 /* svpext_lane_c32_x2 */, 86816 /* q32bQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86862 /* svpext_lane_c64 */, 86786 /* q16bQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86878 /* svpext_lane_c64_x2 */, 86816 /* q32bQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86897 /* svpext_lane_c8 */, 86786 /* q16bQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86912 /* svpext_lane_c8_x2 */, 86816 /* q32bQaIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86930 /* svpfalse_b */, 86941 /* q16bv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86947 /* svpfalse_c */, 86958 /* Qav */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86962 /* svpfirst_b */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86973 /* svpmlal_pair_n_u64_x2 */, 86995 /* q4UWiq4UWiq2UWiUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87014 /* svpmlal_pair_u64_x2 */, 15089 /* q4UWiq4UWiq2UWiq2UWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87034 /* svpmov_lane_s16 */, 87050 /* q16bq8sIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87062 /* svpmov_lane_s16_m */, 87080 /* q8sq8sq16bIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87095 /* svpmov_lane_s32 */, 87111 /* q16bq4iIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87123 /* svpmov_lane_s32_m */, 87141 /* q4iq4iq16bIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87156 /* svpmov_lane_s64 */, 87172 /* q16bq2WiIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87185 /* svpmov_lane_s64_m */, 87203 /* q2Wiq2Wiq16bIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87220 /* svpmov_lane_s8 */, 87235 /* q16bq16ScIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87249 /* svpmov_lane_u16 */, 87265 /* q16bq8UsIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87278 /* svpmov_lane_u16_m */, 87296 /* q8Usq8Usq16bIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87313 /* svpmov_lane_u32 */, 87329 /* q16bq4UiIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87342 /* svpmov_lane_u32_m */, 87360 /* q4Uiq4Uiq16bIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87377 /* svpmov_lane_u64 */, 87393 /* q16bq2UWiIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87407 /* svpmov_lane_u64_m */, 87425 /* q2UWiq2UWiq16bIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87444 /* svpmov_lane_u8 */, 87459 /* q16bq16UcIUWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87473 /* svpmov_s16 */, 87484 /* q16bq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87492 /* svpmov_s16_z */, 87505 /* q8sq16b */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87513 /* svpmov_s32 */, 87524 /* q16bq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87532 /* svpmov_s32_z */, 87545 /* q4iq16b */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87553 /* svpmov_s64 */, 87564 /* q16bq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87573 /* svpmov_s64_z */, 87586 /* q2Wiq16b */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87595 /* svpmov_s8 */, 87605 /* q16bq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87615 /* svpmov_s8_z */, 87627 /* q16Scq16b */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87637 /* svpmov_u16 */, 87648 /* q16bq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87657 /* svpmov_u16_z */, 87670 /* q8Usq16b */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87679 /* svpmov_u32 */, 87690 /* q16bq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87699 /* svpmov_u32_z */, 87712 /* q4Uiq16b */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87721 /* svpmov_u64 */, 87732 /* q16bq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87742 /* svpmov_u64_z */, 87755 /* q2UWiq16b */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87765 /* svpmov_u8 */, 87775 /* q16bq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87785 /* svpmov_u8_z */, 87797 /* q16Ucq16b */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87807 /* svpmul_n_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87819 /* svpmul_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87829 /* svpmull_pair_n_u64_x2 */, 87851 /* q4UWiq2UWiUWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87865 /* svpmull_pair_u64_x2 */, 22352 /* q4UWiq2UWiq2UWi */, 1 /* n */, 8306 /* (sve,sve-aes2)|(sme,sve-aes2,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87885 /* svpmullb_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87900 /* svpmullb_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87915 /* svpmullb_pair_n_u32 */, 11643 /* q4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87935 /* svpmullb_pair_n_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 8409 /* (sve,sve-aes)|(sme,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87955 /* svpmullb_pair_n_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87974 /* svpmullb_pair_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87992 /* svpmullb_pair_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 8409 /* (sve,sve-aes)|(sme,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88010 /* svpmullb_pair_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88027 /* svpmullb_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88040 /* svpmullb_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88053 /* svpmullt_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88068 /* svpmullt_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88083 /* svpmullt_pair_n_u32 */, 11643 /* q4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88103 /* svpmullt_pair_n_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 8409 /* (sve,sve-aes)|(sme,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88123 /* svpmullt_pair_n_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88142 /* svpmullt_pair_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88160 /* svpmullt_pair_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 8409 /* (sve,sve-aes)|(sme,ssve-aes) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88178 /* svpmullt_pair_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88195 /* svpmullt_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88208 /* svpmullt_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88221 /* svpnext_b16 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88233 /* svpnext_b32 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88245 /* svpnext_b64 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88257 /* svpnext_b8 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88268 /* svprfb */, 88275 /* vq16bvC*Ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88286 /* svprfb_gather_s32offset */, 88310 /* vq16bvC*q4iIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88324 /* svprfb_gather_s64offset */, 88348 /* vq16bvC*q2WiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88363 /* svprfb_gather_u32base */, 88385 /* vq16bq4UiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88397 /* svprfb_gather_u32base_offset */, 88426 /* vq16bq4UiWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88440 /* svprfb_gather_u32offset */, 88464 /* vq16bvC*q4UiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88479 /* svprfb_gather_u64base */, 88501 /* vq16bq2UWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88514 /* svprfb_gather_u64base_offset */, 88543 /* vq16bq2UWiWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88558 /* svprfb_gather_u64offset */, 88582 /* vq16bvC*q2UWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88598 /* svprfb_vnum */, 88610 /* vq16bvC*WiIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88623 /* svprfd */, 88275 /* vq16bvC*Ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88630 /* svprfd_gather_s32index */, 88310 /* vq16bvC*q4iIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88653 /* svprfd_gather_s64index */, 88348 /* vq16bvC*q2WiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88676 /* svprfd_gather_u32base */, 88385 /* vq16bq4UiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88698 /* svprfd_gather_u32base_index */, 88426 /* vq16bq4UiWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88726 /* svprfd_gather_u32index */, 88464 /* vq16bvC*q4UiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88749 /* svprfd_gather_u64base */, 88501 /* vq16bq2UWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88771 /* svprfd_gather_u64base_index */, 88543 /* vq16bq2UWiWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88799 /* svprfd_gather_u64index */, 88582 /* vq16bvC*q2UWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88822 /* svprfd_vnum */, 88610 /* vq16bvC*WiIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88834 /* svprfh */, 88275 /* vq16bvC*Ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88841 /* svprfh_gather_s32index */, 88310 /* vq16bvC*q4iIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88864 /* svprfh_gather_s64index */, 88348 /* vq16bvC*q2WiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88887 /* svprfh_gather_u32base */, 88385 /* vq16bq4UiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88909 /* svprfh_gather_u32base_index */, 88426 /* vq16bq4UiWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88937 /* svprfh_gather_u32index */, 88464 /* vq16bvC*q4UiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88960 /* svprfh_gather_u64base */, 88501 /* vq16bq2UWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88982 /* svprfh_gather_u64base_index */, 88543 /* vq16bq2UWiWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89010 /* svprfh_gather_u64index */, 88582 /* vq16bvC*q2UWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89033 /* svprfh_vnum */, 88610 /* vq16bvC*WiIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89045 /* svprfw */, 88275 /* vq16bvC*Ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89052 /* svprfw_gather_s32index */, 88310 /* vq16bvC*q4iIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89075 /* svprfw_gather_s64index */, 88348 /* vq16bvC*q2WiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89098 /* svprfw_gather_u32base */, 88385 /* vq16bq4UiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89120 /* svprfw_gather_u32base_index */, 88426 /* vq16bq4UiWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89148 /* svprfw_gather_u32index */, 88464 /* vq16bvC*q4UiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89171 /* svprfw_gather_u64base */, 88501 /* vq16bq2UWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89193 /* svprfw_gather_u64base_index */, 88543 /* vq16bq2UWiWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89221 /* svprfw_gather_u64index */, 88582 /* vq16bvC*q2UWiIi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89244 /* svprfw_vnum */, 88610 /* vq16bvC*WiIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89256 /* svpsel_lane_b16 */, 89272 /* q16bq16bq16bUi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89287 /* svpsel_lane_b32 */, 89272 /* q16bq16bq16bUi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89303 /* svpsel_lane_b64 */, 89272 /* q16bq16bq16bUi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89319 /* svpsel_lane_b8 */, 89272 /* q16bq16bq16bUi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89334 /* svpsel_lane_c16 */, 89350 /* QaQaq16bUi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89361 /* svpsel_lane_c32 */, 89350 /* QaQaq16bUi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89377 /* svpsel_lane_c64 */, 89350 /* QaQaq16bUi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89393 /* svpsel_lane_c8 */, 89350 /* QaQaq16bUi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89408 /* svptest_any */, 89420 /* bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89430 /* svptest_first */, 89420 /* bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89444 /* svptest_last */, 89420 /* bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89457 /* svptrue_b16 */, 86941 /* q16bv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89469 /* svptrue_b32 */, 86941 /* q16bv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89481 /* svptrue_b64 */, 86941 /* q16bv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89493 /* svptrue_b8 */, 86941 /* q16bv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89504 /* svptrue_c16 */, 86958 /* Qav */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89516 /* svptrue_c32 */, 86958 /* Qav */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89528 /* svptrue_c64 */, 86958 /* Qav */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89540 /* svptrue_c8 */, 86958 /* Qav */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89551 /* svptrue_pat_b16 */, 89567 /* q16bIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89574 /* svptrue_pat_b32 */, 89567 /* q16bIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89590 /* svptrue_pat_b64 */, 89567 /* q16bIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89606 /* svptrue_pat_b8 */, 89567 /* q16bIi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89621 /* svqabs_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89634 /* svqabs_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89647 /* svqabs_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89660 /* svqabs_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89673 /* svqabs_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89686 /* svqabs_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89699 /* svqabs_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89712 /* svqabs_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89725 /* svqabs_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89738 /* svqabs_s8_m */, 2982 /* q16Scq16Scq16bq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89750 /* svqabs_s8_x */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89762 /* svqabs_s8_z */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89774 /* svqadd_n_s16 */, 37610 /* q8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89787 /* svqadd_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89802 /* svqadd_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89817 /* svqadd_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89832 /* svqadd_n_s32 */, 37631 /* q4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89845 /* svqadd_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89860 /* svqadd_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89875 /* svqadd_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89890 /* svqadd_n_s64 */, 37652 /* q2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89903 /* svqadd_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89918 /* svqadd_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89933 /* svqadd_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89948 /* svqadd_n_s8 */, 37675 /* q16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89960 /* svqadd_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89974 /* svqadd_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{89988 /* svqadd_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90002 /* svqadd_n_u16 */, 11582 /* q8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90015 /* svqadd_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90030 /* svqadd_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90045 /* svqadd_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90060 /* svqadd_n_u32 */, 11643 /* q4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90073 /* svqadd_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90088 /* svqadd_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90103 /* svqadd_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90118 /* svqadd_n_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90131 /* svqadd_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90146 /* svqadd_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90161 /* svqadd_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90176 /* svqadd_n_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90188 /* svqadd_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90202 /* svqadd_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90216 /* svqadd_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90230 /* svqadd_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90251 /* svqadd_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90264 /* svqadd_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90277 /* svqadd_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90290 /* svqadd_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90311 /* svqadd_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90324 /* svqadd_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90337 /* svqadd_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90350 /* svqadd_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90374 /* svqadd_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90387 /* svqadd_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90400 /* svqadd_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90413 /* svqadd_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90439 /* svqadd_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90451 /* svqadd_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90463 /* svqadd_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90475 /* svqadd_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90486 /* svqadd_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90499 /* svqadd_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90512 /* svqadd_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90525 /* svqadd_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90536 /* svqadd_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90549 /* svqadd_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90562 /* svqadd_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90575 /* svqadd_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90586 /* svqadd_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90599 /* svqadd_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90612 /* svqadd_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90625 /* svqadd_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90635 /* svqadd_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90647 /* svqadd_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90659 /* svqadd_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90671 /* svqcadd_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90683 /* svqcadd_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90695 /* svqcadd_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90707 /* svqcadd_s8 */, 13835 /* q16Scq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90718 /* svqcvt_s16_s32_x2 */, 90736 /* q8sq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90743 /* svqcvt_s16_s64_x4 */, 90761 /* q8sq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90769 /* svqcvt_s8_s32_x4 */, 90786 /* q16Scq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90796 /* svqcvt_u16_s32_x2 */, 90814 /* q8Usq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90822 /* svqcvt_u16_s64_x4 */, 90840 /* q8Usq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90849 /* svqcvt_u16_u32_x2 */, 90867 /* q8Usq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90876 /* svqcvt_u16_u64_x4 */, 90894 /* q8Usq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90904 /* svqcvt_u8_s32_x4 */, 90921 /* q16Ucq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90931 /* svqcvt_u8_u32_x4 */, 90948 /* q16Ucq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90959 /* svqcvtn_s16_s32_x2 */, 90736 /* q8sq8i */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90978 /* svqcvtn_s16_s64_x4 */, 90761 /* q8sq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{90997 /* svqcvtn_s8_s32_x4 */, 90786 /* q16Scq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91015 /* svqcvtn_u16_s32_x2 */, 90814 /* q8Usq8i */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91034 /* svqcvtn_u16_s64_x4 */, 90840 /* q8Usq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91053 /* svqcvtn_u16_u32_x2 */, 90867 /* q8Usq8Ui */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91072 /* svqcvtn_u16_u64_x4 */, 90894 /* q8Usq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91091 /* svqcvtn_u8_s32_x4 */, 90921 /* q16Ucq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91109 /* svqcvtn_u8_u32_x4 */, 90948 /* q16Ucq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91127 /* svqdecb_n_s32 */, 91141 /* iiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91148 /* svqdecb_n_s64 */, 91162 /* WiWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91171 /* svqdecb_n_u32 */, 91185 /* UiUiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91194 /* svqdecb_n_u64 */, 91208 /* UWiUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91219 /* svqdecb_pat_n_s32 */, 91237 /* iiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91246 /* svqdecb_pat_n_s64 */, 91264 /* WiWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91275 /* svqdecb_pat_n_u32 */, 91293 /* UiUiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91304 /* svqdecb_pat_n_u64 */, 91322 /* UWiUWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91335 /* svqdecd_n_s32 */, 91141 /* iiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91349 /* svqdecd_n_s64 */, 91162 /* WiWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91363 /* svqdecd_n_u32 */, 91185 /* UiUiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91377 /* svqdecd_n_u64 */, 91208 /* UWiUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91391 /* svqdecd_pat_n_s32 */, 91237 /* iiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91409 /* svqdecd_pat_n_s64 */, 91264 /* WiWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91427 /* svqdecd_pat_n_u32 */, 91293 /* UiUiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91445 /* svqdecd_pat_n_u64 */, 91322 /* UWiUWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91463 /* svqdecd_pat_s64 */, 91479 /* q2Wiq2WiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91494 /* svqdecd_pat_u64 */, 91510 /* q2UWiq2UWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91527 /* svqdecd_s64 */, 29590 /* q2Wiq2WiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91539 /* svqdecd_u64 */, 29707 /* q2UWiq2UWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91551 /* svqdech_n_s32 */, 91141 /* iiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91565 /* svqdech_n_s64 */, 91162 /* WiWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91579 /* svqdech_n_u32 */, 91185 /* UiUiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91593 /* svqdech_n_u64 */, 91208 /* UWiUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91607 /* svqdech_pat_n_s32 */, 91237 /* iiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91625 /* svqdech_pat_n_s64 */, 91264 /* WiWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91643 /* svqdech_pat_n_u32 */, 91293 /* UiUiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91661 /* svqdech_pat_n_u64 */, 91322 /* UWiUWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91679 /* svqdech_pat_s16 */, 91695 /* q8sq8sIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91708 /* svqdech_pat_u16 */, 91724 /* q8Usq8UsIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91739 /* svqdech_s16 */, 29536 /* q8sq8sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91751 /* svqdech_u16 */, 29649 /* q8Usq8UsIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91763 /* svqdecp_n_s32_b16 */, 91781 /* iiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91788 /* svqdecp_n_s32_b32 */, 91781 /* iiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91806 /* svqdecp_n_s32_b64 */, 91781 /* iiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91824 /* svqdecp_n_s32_b8 */, 91781 /* iiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91841 /* svqdecp_n_s64_b16 */, 91859 /* WiWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91868 /* svqdecp_n_s64_b32 */, 91859 /* WiWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91886 /* svqdecp_n_s64_b64 */, 91859 /* WiWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91904 /* svqdecp_n_s64_b8 */, 91859 /* WiWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91921 /* svqdecp_n_u32_b16 */, 91939 /* UiUiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91948 /* svqdecp_n_u32_b32 */, 91939 /* UiUiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91966 /* svqdecp_n_u32_b64 */, 91939 /* UiUiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{91984 /* svqdecp_n_u32_b8 */, 91939 /* UiUiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92001 /* svqdecp_n_u64_b16 */, 92019 /* UWiUWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92030 /* svqdecp_n_u64_b32 */, 92019 /* UWiUWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92048 /* svqdecp_n_u64_b64 */, 92019 /* UWiUWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92066 /* svqdecp_n_u64_b8 */, 92019 /* UWiUWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92083 /* svqdecp_s16 */, 92095 /* q8sq8sq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92106 /* svqdecp_s32 */, 92118 /* q4iq4iq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92129 /* svqdecp_s64 */, 92141 /* q2Wiq2Wiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92154 /* svqdecp_u16 */, 92166 /* q8Usq8Usq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92179 /* svqdecp_u32 */, 92191 /* q4Uiq4Uiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92204 /* svqdecp_u64 */, 92216 /* q2UWiq2UWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92231 /* svqdecw_n_s32 */, 91141 /* iiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92245 /* svqdecw_n_s64 */, 91162 /* WiWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92259 /* svqdecw_n_u32 */, 91185 /* UiUiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92273 /* svqdecw_n_u64 */, 91208 /* UWiUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92287 /* svqdecw_pat_n_s32 */, 91237 /* iiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92305 /* svqdecw_pat_n_s64 */, 91264 /* WiWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92323 /* svqdecw_pat_n_u32 */, 91293 /* UiUiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92341 /* svqdecw_pat_n_u64 */, 91322 /* UWiUWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92359 /* svqdecw_pat_s32 */, 92375 /* q4iq4iIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92388 /* svqdecw_pat_u32 */, 92404 /* q4Uiq4UiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92419 /* svqdecw_s32 */, 29563 /* q4iq4iIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92431 /* svqdecw_u32 */, 29678 /* q4Uiq4UiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92443 /* svqdmlalb_lane_s32 */, 28659 /* q4iq4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92462 /* svqdmlalb_lane_s64 */, 77621 /* q2Wiq2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92481 /* svqdmlalb_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92497 /* svqdmlalb_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92513 /* svqdmlalb_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92529 /* svqdmlalb_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92543 /* svqdmlalb_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92557 /* svqdmlalb_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92571 /* svqdmlalbt_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92588 /* svqdmlalbt_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92605 /* svqdmlalbt_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92622 /* svqdmlalbt_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92637 /* svqdmlalbt_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92652 /* svqdmlalbt_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92667 /* svqdmlalt_lane_s32 */, 28659 /* q4iq4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92686 /* svqdmlalt_lane_s64 */, 77621 /* q2Wiq2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92705 /* svqdmlalt_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92721 /* svqdmlalt_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92737 /* svqdmlalt_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92753 /* svqdmlalt_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92767 /* svqdmlalt_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92781 /* svqdmlalt_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92795 /* svqdmlslb_lane_s32 */, 28659 /* q4iq4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92814 /* svqdmlslb_lane_s64 */, 77621 /* q2Wiq2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92833 /* svqdmlslb_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92849 /* svqdmlslb_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92865 /* svqdmlslb_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92881 /* svqdmlslb_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92895 /* svqdmlslb_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92909 /* svqdmlslb_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92923 /* svqdmlslbt_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92940 /* svqdmlslbt_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92957 /* svqdmlslbt_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92974 /* svqdmlslbt_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{92989 /* svqdmlslbt_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93004 /* svqdmlslbt_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93019 /* svqdmlslt_lane_s32 */, 28659 /* q4iq4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93038 /* svqdmlslt_lane_s64 */, 77621 /* q2Wiq2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93057 /* svqdmlslt_n_s16 */, 472 /* q8sq8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93073 /* svqdmlslt_n_s32 */, 500 /* q4iq4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93089 /* svqdmlslt_n_s64 */, 525 /* q2Wiq2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93105 /* svqdmlslt_s16 */, 640 /* q8sq8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93119 /* svqdmlslt_s32 */, 669 /* q4iq4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93133 /* svqdmlslt_s64 */, 694 /* q2Wiq2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93147 /* svqdmulh_lane_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93165 /* svqdmulh_lane_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93183 /* svqdmulh_lane_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93201 /* svqdmulh_n_s16 */, 37610 /* q8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93216 /* svqdmulh_n_s32 */, 37631 /* q4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93231 /* svqdmulh_n_s64 */, 37652 /* q2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93246 /* svqdmulh_n_s8 */, 37675 /* q16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93260 /* svqdmulh_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93273 /* svqdmulh_s16_x2 */, 70593 /* q16sq16sq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93289 /* svqdmulh_s16_x4 */, 70619 /* q32sq32sq32s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93305 /* svqdmulh_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93318 /* svqdmulh_s32_x2 */, 70681 /* q8iq8iq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93334 /* svqdmulh_s32_x4 */, 70704 /* q16iq16iq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93350 /* svqdmulh_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93363 /* svqdmulh_s64_x2 */, 70766 /* q4Wiq4Wiq4Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93379 /* svqdmulh_s64_x4 */, 70792 /* q8Wiq8Wiq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93395 /* svqdmulh_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93407 /* svqdmulh_s8_x2 */, 70851 /* q32Scq32Scq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93422 /* svqdmulh_s8_x4 */, 70879 /* q64Scq64Scq64Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93437 /* svqdmulh_single_s16_x2 */, 4741 /* q16sq16sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93460 /* svqdmulh_single_s16_x4 */, 4782 /* q32sq32sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93483 /* svqdmulh_single_s32_x2 */, 4814 /* q8iq8iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93506 /* svqdmulh_single_s32_x4 */, 4844 /* q16iq16iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93529 /* svqdmulh_single_s64_x2 */, 4876 /* q4Wiq4Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93552 /* svqdmulh_single_s64_x4 */, 4909 /* q8Wiq8Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93575 /* svqdmulh_single_s8_x2 */, 4941 /* q32Scq32Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93597 /* svqdmulh_single_s8_x4 */, 4976 /* q64Scq64Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93619 /* svqdmullb_lane_s32 */, 83419 /* q4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93638 /* svqdmullb_lane_s64 */, 83450 /* q2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93657 /* svqdmullb_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93673 /* svqdmullb_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93689 /* svqdmullb_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93705 /* svqdmullb_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93719 /* svqdmullb_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93733 /* svqdmullb_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93747 /* svqdmullt_lane_s32 */, 83419 /* q4iq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93766 /* svqdmullt_lane_s64 */, 83450 /* q2Wiq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93785 /* svqdmullt_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93801 /* svqdmullt_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93817 /* svqdmullt_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93833 /* svqdmullt_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93847 /* svqdmullt_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93861 /* svqdmullt_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93875 /* svqincb_n_s32 */, 91141 /* iiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93889 /* svqincb_n_s64 */, 91162 /* WiWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93903 /* svqincb_n_u32 */, 91185 /* UiUiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93917 /* svqincb_n_u64 */, 91208 /* UWiUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93931 /* svqincb_pat_n_s32 */, 91237 /* iiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93949 /* svqincb_pat_n_s64 */, 91264 /* WiWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93967 /* svqincb_pat_n_u32 */, 91293 /* UiUiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{93985 /* svqincb_pat_n_u64 */, 91322 /* UWiUWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94003 /* svqincd_n_s32 */, 91141 /* iiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94017 /* svqincd_n_s64 */, 91162 /* WiWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94031 /* svqincd_n_u32 */, 91185 /* UiUiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94045 /* svqincd_n_u64 */, 91208 /* UWiUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94059 /* svqincd_pat_n_s32 */, 91237 /* iiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94077 /* svqincd_pat_n_s64 */, 91264 /* WiWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94095 /* svqincd_pat_n_u32 */, 91293 /* UiUiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94113 /* svqincd_pat_n_u64 */, 91322 /* UWiUWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94131 /* svqincd_pat_s64 */, 91479 /* q2Wiq2WiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94147 /* svqincd_pat_u64 */, 91510 /* q2UWiq2UWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94163 /* svqincd_s64 */, 29590 /* q2Wiq2WiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94175 /* svqincd_u64 */, 29707 /* q2UWiq2UWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94187 /* svqinch_n_s32 */, 91141 /* iiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94201 /* svqinch_n_s64 */, 91162 /* WiWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94215 /* svqinch_n_u32 */, 91185 /* UiUiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94229 /* svqinch_n_u64 */, 91208 /* UWiUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94243 /* svqinch_pat_n_s32 */, 91237 /* iiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94261 /* svqinch_pat_n_s64 */, 91264 /* WiWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94279 /* svqinch_pat_n_u32 */, 91293 /* UiUiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94297 /* svqinch_pat_n_u64 */, 91322 /* UWiUWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94315 /* svqinch_pat_s16 */, 91695 /* q8sq8sIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94331 /* svqinch_pat_u16 */, 91724 /* q8Usq8UsIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94347 /* svqinch_s16 */, 29536 /* q8sq8sIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94359 /* svqinch_u16 */, 29649 /* q8Usq8UsIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94371 /* svqincp_n_s32_b16 */, 91781 /* iiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94389 /* svqincp_n_s32_b32 */, 91781 /* iiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94407 /* svqincp_n_s32_b64 */, 91781 /* iiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94425 /* svqincp_n_s32_b8 */, 91781 /* iiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94442 /* svqincp_n_s64_b16 */, 91859 /* WiWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94460 /* svqincp_n_s64_b32 */, 91859 /* WiWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94478 /* svqincp_n_s64_b64 */, 91859 /* WiWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94496 /* svqincp_n_s64_b8 */, 91859 /* WiWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94513 /* svqincp_n_u32_b16 */, 91939 /* UiUiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94531 /* svqincp_n_u32_b32 */, 91939 /* UiUiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94549 /* svqincp_n_u32_b64 */, 91939 /* UiUiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94567 /* svqincp_n_u32_b8 */, 91939 /* UiUiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94584 /* svqincp_n_u64_b16 */, 92019 /* UWiUWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94602 /* svqincp_n_u64_b32 */, 92019 /* UWiUWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94620 /* svqincp_n_u64_b64 */, 92019 /* UWiUWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94638 /* svqincp_n_u64_b8 */, 92019 /* UWiUWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94655 /* svqincp_s16 */, 92095 /* q8sq8sq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94667 /* svqincp_s32 */, 92118 /* q4iq4iq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94679 /* svqincp_s64 */, 92141 /* q2Wiq2Wiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94691 /* svqincp_u16 */, 92166 /* q8Usq8Usq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94703 /* svqincp_u32 */, 92191 /* q4Uiq4Uiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94715 /* svqincp_u64 */, 92216 /* q2UWiq2UWiq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94727 /* svqincw_n_s32 */, 91141 /* iiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94741 /* svqincw_n_s64 */, 91162 /* WiWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94755 /* svqincw_n_u32 */, 91185 /* UiUiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94769 /* svqincw_n_u64 */, 91208 /* UWiUWiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94783 /* svqincw_pat_n_s32 */, 91237 /* iiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94801 /* svqincw_pat_n_s64 */, 91264 /* WiWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94819 /* svqincw_pat_n_u32 */, 91293 /* UiUiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94837 /* svqincw_pat_n_u64 */, 91322 /* UWiUWiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94855 /* svqincw_pat_s32 */, 92375 /* q4iq4iIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94871 /* svqincw_pat_u32 */, 92404 /* q4Uiq4UiIiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94887 /* svqincw_s32 */, 29563 /* q4iq4iIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94899 /* svqincw_u32 */, 29678 /* q4Uiq4UiIUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94911 /* svqneg_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94924 /* svqneg_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94937 /* svqneg_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94950 /* svqneg_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94963 /* svqneg_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94976 /* svqneg_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{94989 /* svqneg_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95002 /* svqneg_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95015 /* svqneg_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95028 /* svqneg_s8_m */, 2982 /* q16Scq16Scq16bq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95040 /* svqneg_s8_x */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95052 /* svqneg_s8_z */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95064 /* svqrdcmlah_lane_s16 */, 17012 /* q8sq8sq8sq8sIUWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95084 /* svqrdcmlah_lane_s32 */, 17049 /* q4iq4iq4iq4iIUWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95104 /* svqrdcmlah_s16 */, 17163 /* q8sq8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95119 /* svqrdcmlah_s32 */, 17191 /* q4iq4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95134 /* svqrdcmlah_s64 */, 17219 /* q2Wiq2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95149 /* svqrdcmlah_s8 */, 17250 /* q16Scq16Scq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95163 /* svqrdmlah_lane_s16 */, 17163 /* q8sq8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95182 /* svqrdmlah_lane_s32 */, 17191 /* q4iq4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95201 /* svqrdmlah_lane_s64 */, 17219 /* q2Wiq2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95220 /* svqrdmlah_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95236 /* svqrdmlah_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95252 /* svqrdmlah_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95268 /* svqrdmlah_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95283 /* svqrdmlah_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95297 /* svqrdmlah_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95311 /* svqrdmlah_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95325 /* svqrdmlah_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95338 /* svqrdmlsh_lane_s16 */, 17163 /* q8sq8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95357 /* svqrdmlsh_lane_s32 */, 17191 /* q4iq4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95376 /* svqrdmlsh_lane_s64 */, 17219 /* q2Wiq2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95395 /* svqrdmlsh_n_s16 */, 15 /* q8sq8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95411 /* svqrdmlsh_n_s32 */, 59 /* q4iq4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95427 /* svqrdmlsh_n_s64 */, 82 /* q2Wiq2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95443 /* svqrdmlsh_n_s8 */, 108 /* q16Scq16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95458 /* svqrdmlsh_s16 */, 250 /* q8sq8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95472 /* svqrdmlsh_s32 */, 273 /* q4iq4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95486 /* svqrdmlsh_s64 */, 296 /* q2Wiq2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95500 /* svqrdmlsh_s8 */, 322 /* q16Scq16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95513 /* svqrdmulh_lane_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95532 /* svqrdmulh_lane_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95551 /* svqrdmulh_lane_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95570 /* svqrdmulh_n_s16 */, 37610 /* q8sq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95586 /* svqrdmulh_n_s32 */, 37631 /* q4iq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95602 /* svqrdmulh_n_s64 */, 37652 /* q2Wiq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95618 /* svqrdmulh_n_s8 */, 37675 /* q16Scq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95633 /* svqrdmulh_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95647 /* svqrdmulh_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95661 /* svqrdmulh_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95675 /* svqrdmulh_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95688 /* svqrshl_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95704 /* svqrshl_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95720 /* svqrshl_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95736 /* svqrshl_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95752 /* svqrshl_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95768 /* svqrshl_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95784 /* svqrshl_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95800 /* svqrshl_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95816 /* svqrshl_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95832 /* svqrshl_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95847 /* svqrshl_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95862 /* svqrshl_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95877 /* svqrshl_n_u16_m */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95907 /* svqrshl_n_u16_x */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95923 /* svqrshl_n_u16_z */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95939 /* svqrshl_n_u32_m */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95969 /* svqrshl_n_u32_x */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{95985 /* svqrshl_n_u32_z */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96001 /* svqrshl_n_u64_m */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96017 /* svqrshl_n_u64_x */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96033 /* svqrshl_n_u64_z */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96049 /* svqrshl_n_u8_m */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96081 /* svqrshl_n_u8_x */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96096 /* svqrshl_n_u8_z */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96111 /* svqrshl_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96125 /* svqrshl_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96139 /* svqrshl_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96153 /* svqrshl_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96167 /* svqrshl_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96181 /* svqrshl_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96195 /* svqrshl_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96209 /* svqrshl_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96223 /* svqrshl_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96237 /* svqrshl_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96250 /* svqrshl_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96263 /* svqrshl_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96276 /* svqrshl_u16_m */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96306 /* svqrshl_u16_x */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96320 /* svqrshl_u16_z */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96334 /* svqrshl_u32_m */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96364 /* svqrshl_u32_x */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96378 /* svqrshl_u32_z */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96392 /* svqrshl_u64_m */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96425 /* svqrshl_u64_x */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96439 /* svqrshl_u64_z */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96453 /* svqrshl_u8_m */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96486 /* svqrshl_u8_x */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96499 /* svqrshl_u8_z */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96512 /* svqrshr_n_s16_s32_x2 */, 96533 /* q8sq8iIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96544 /* svqrshr_n_s16_s64_x4 */, 96565 /* q8sq8WiIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96577 /* svqrshr_n_s8_s32_x4 */, 96597 /* q16Scq16iIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96611 /* svqrshr_n_u16_u32_x2 */, 96632 /* q8Usq8UiIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96645 /* svqrshr_n_u16_u64_x4 */, 96666 /* q8Usq8UWiIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96680 /* svqrshr_n_u8_u32_x4 */, 96700 /* q16Ucq16UiIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96715 /* svqrshrn_n_s16_s32_x2 */, 96533 /* q8sq8iIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96737 /* svqrshrn_n_s16_s64_x4 */, 96565 /* q8sq8WiIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96759 /* svqrshrn_n_s8_s32_x4 */, 96597 /* q16Scq16iIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96780 /* svqrshrn_n_u16_u32_x2 */, 96632 /* q8Usq8UiIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96802 /* svqrshrn_n_u16_u64_x4 */, 96666 /* q8Usq8UWiIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96824 /* svqrshrn_n_u8_u32_x4 */, 96700 /* q16Ucq16UiIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96845 /* svqrshrnb_n_s16 */, 96861 /* q16Scq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96874 /* svqrshrnb_n_s32 */, 96890 /* q8sq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96901 /* svqrshrnb_n_s64 */, 96917 /* q4iq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96929 /* svqrshrnb_n_u16 */, 96945 /* q16Ucq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96959 /* svqrshrnb_n_u32 */, 96975 /* q8Usq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{96988 /* svqrshrnb_n_u64 */, 97004 /* q4Uiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97018 /* svqrshrnt_n_s16 */, 97034 /* q16Scq16Scq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97052 /* svqrshrnt_n_s32 */, 97068 /* q8sq8sq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97082 /* svqrshrnt_n_s64 */, 97098 /* q4iq4iq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97113 /* svqrshrnt_n_u16 */, 97129 /* q16Ucq16Ucq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97148 /* svqrshrnt_n_u32 */, 97164 /* q8Usq8Usq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97181 /* svqrshrnt_n_u64 */, 97197 /* q4Uiq4Uiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97215 /* svqrshru_n_u16_s32_x2 */, 97237 /* q8Usq8iIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97249 /* svqrshru_n_u16_s64_x4 */, 97271 /* q8Usq8WiIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97284 /* svqrshru_n_u8_s32_x4 */, 97305 /* q16Ucq16iIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97319 /* svqrshrun_n_u16_s32_x2 */, 97237 /* q8Usq8iIUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97342 /* svqrshrun_n_u16_s64_x4 */, 97271 /* q8Usq8WiIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97365 /* svqrshrun_n_u8_s32_x4 */, 97305 /* q16Ucq16iIUWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97387 /* svqrshrunb_n_s16 */, 97404 /* q16Ucq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97417 /* svqrshrunb_n_s32 */, 97434 /* q8Usq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97446 /* svqrshrunb_n_s64 */, 97463 /* q4Uiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97476 /* svqrshrunt_n_s16 */, 97493 /* q16Ucq16Ucq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97511 /* svqrshrunt_n_s32 */, 97528 /* q8Usq8Usq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97544 /* svqrshrunt_n_s64 */, 97561 /* q4Uiq4Uiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97578 /* svqshl_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97593 /* svqshl_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97608 /* svqshl_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97623 /* svqshl_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97638 /* svqshl_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97653 /* svqshl_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97668 /* svqshl_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97683 /* svqshl_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97698 /* svqshl_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97713 /* svqshl_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97727 /* svqshl_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97741 /* svqshl_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97755 /* svqshl_n_u16_m */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97770 /* svqshl_n_u16_x */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97785 /* svqshl_n_u16_z */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97800 /* svqshl_n_u32_m */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97815 /* svqshl_n_u32_x */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97830 /* svqshl_n_u32_z */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97845 /* svqshl_n_u64_m */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97860 /* svqshl_n_u64_x */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97875 /* svqshl_n_u64_z */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97890 /* svqshl_n_u8_m */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97904 /* svqshl_n_u8_x */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97918 /* svqshl_n_u8_z */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97932 /* svqshl_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97945 /* svqshl_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97958 /* svqshl_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97971 /* svqshl_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97984 /* svqshl_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{97997 /* svqshl_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98010 /* svqshl_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98023 /* svqshl_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98036 /* svqshl_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98049 /* svqshl_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98061 /* svqshl_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98073 /* svqshl_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98085 /* svqshl_u16_m */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98098 /* svqshl_u16_x */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98111 /* svqshl_u16_z */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98124 /* svqshl_u32_m */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98137 /* svqshl_u32_x */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98150 /* svqshl_u32_z */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98163 /* svqshl_u64_m */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98176 /* svqshl_u64_x */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98189 /* svqshl_u64_z */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98202 /* svqshl_u8_m */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98214 /* svqshl_u8_x */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98226 /* svqshl_u8_z */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98238 /* svqshlu_n_s16_m */, 98254 /* q8Usq16bq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98270 /* svqshlu_n_s16_x */, 98254 /* q8Usq16bq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98286 /* svqshlu_n_s16_z */, 98254 /* q8Usq16bq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98302 /* svqshlu_n_s32_m */, 98318 /* q4Uiq16bq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98334 /* svqshlu_n_s32_x */, 98318 /* q4Uiq16bq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98350 /* svqshlu_n_s32_z */, 98318 /* q4Uiq16bq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98366 /* svqshlu_n_s64_m */, 98382 /* q2UWiq16bq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98400 /* svqshlu_n_s64_x */, 98382 /* q2UWiq16bq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98416 /* svqshlu_n_s64_z */, 98382 /* q2UWiq16bq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98432 /* svqshlu_n_s8_m */, 98447 /* q16Ucq16bq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98466 /* svqshlu_n_s8_x */, 98447 /* q16Ucq16bq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98481 /* svqshlu_n_s8_z */, 98447 /* q16Ucq16bq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98496 /* svqshrnb_n_s16 */, 96861 /* q16Scq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98511 /* svqshrnb_n_s32 */, 96890 /* q8sq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98526 /* svqshrnb_n_s64 */, 96917 /* q4iq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98541 /* svqshrnb_n_u16 */, 96945 /* q16Ucq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98556 /* svqshrnb_n_u32 */, 96975 /* q8Usq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98571 /* svqshrnb_n_u64 */, 97004 /* q4Uiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98586 /* svqshrnt_n_s16 */, 97034 /* q16Scq16Scq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98601 /* svqshrnt_n_s32 */, 97068 /* q8sq8sq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98616 /* svqshrnt_n_s64 */, 97098 /* q4iq4iq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98631 /* svqshrnt_n_u16 */, 97129 /* q16Ucq16Ucq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98646 /* svqshrnt_n_u32 */, 97164 /* q8Usq8Usq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98661 /* svqshrnt_n_u64 */, 97197 /* q4Uiq4Uiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98676 /* svqshrunb_n_s16 */, 97404 /* q16Ucq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98692 /* svqshrunb_n_s32 */, 97434 /* q8Usq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98708 /* svqshrunb_n_s64 */, 97463 /* q4Uiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98724 /* svqshrunt_n_s16 */, 97493 /* q16Ucq16Ucq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98740 /* svqshrunt_n_s32 */, 97528 /* q8Usq8Usq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98756 /* svqshrunt_n_s64 */, 97561 /* q4Uiq4Uiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98772 /* svqsub_n_s16 */, 37610 /* q8sq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98785 /* svqsub_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98800 /* svqsub_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98815 /* svqsub_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98830 /* svqsub_n_s32 */, 37631 /* q4iq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98843 /* svqsub_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98858 /* svqsub_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98873 /* svqsub_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98888 /* svqsub_n_s64 */, 37652 /* q2Wiq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98901 /* svqsub_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98916 /* svqsub_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98931 /* svqsub_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98946 /* svqsub_n_s8 */, 37675 /* q16Scq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98958 /* svqsub_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98972 /* svqsub_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{98986 /* svqsub_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99000 /* svqsub_n_u16 */, 11582 /* q8Usq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99013 /* svqsub_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99028 /* svqsub_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99043 /* svqsub_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99058 /* svqsub_n_u32 */, 11643 /* q4Uiq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99071 /* svqsub_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99086 /* svqsub_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99101 /* svqsub_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99116 /* svqsub_n_u64 */, 11667 /* q2UWiq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99129 /* svqsub_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99144 /* svqsub_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99159 /* svqsub_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99174 /* svqsub_n_u8 */, 11693 /* q16Ucq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99186 /* svqsub_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99200 /* svqsub_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99214 /* svqsub_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99228 /* svqsub_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99239 /* svqsub_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99252 /* svqsub_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99265 /* svqsub_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99278 /* svqsub_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99289 /* svqsub_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99302 /* svqsub_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99315 /* svqsub_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99328 /* svqsub_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99339 /* svqsub_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99352 /* svqsub_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99365 /* svqsub_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99378 /* svqsub_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99388 /* svqsub_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99400 /* svqsub_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99412 /* svqsub_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99424 /* svqsub_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99435 /* svqsub_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99448 /* svqsub_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99461 /* svqsub_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99474 /* svqsub_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99485 /* svqsub_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99498 /* svqsub_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99511 /* svqsub_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99524 /* svqsub_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99535 /* svqsub_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99548 /* svqsub_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99561 /* svqsub_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99574 /* svqsub_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99584 /* svqsub_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99596 /* svqsub_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99608 /* svqsub_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99620 /* svqsubr_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99636 /* svqsubr_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99652 /* svqsubr_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99668 /* svqsubr_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99684 /* svqsubr_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99700 /* svqsubr_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99716 /* svqsubr_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99732 /* svqsubr_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99748 /* svqsubr_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99764 /* svqsubr_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99779 /* svqsubr_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99794 /* svqsubr_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99809 /* svqsubr_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99825 /* svqsubr_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99841 /* svqsubr_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99857 /* svqsubr_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99873 /* svqsubr_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99889 /* svqsubr_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99905 /* svqsubr_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99921 /* svqsubr_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99937 /* svqsubr_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99953 /* svqsubr_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99968 /* svqsubr_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99983 /* svqsubr_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{99998 /* svqsubr_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100012 /* svqsubr_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100026 /* svqsubr_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100040 /* svqsubr_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100054 /* svqsubr_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100068 /* svqsubr_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100082 /* svqsubr_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100096 /* svqsubr_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100110 /* svqsubr_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100124 /* svqsubr_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100137 /* svqsubr_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100150 /* svqsubr_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100163 /* svqsubr_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100177 /* svqsubr_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100191 /* svqsubr_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100205 /* svqsubr_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100219 /* svqsubr_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100233 /* svqsubr_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100247 /* svqsubr_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100261 /* svqsubr_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100275 /* svqsubr_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100289 /* svqsubr_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100302 /* svqsubr_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100315 /* svqsubr_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100328 /* svqxtnb_s16 */, 100340 /* q16Scq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100349 /* svqxtnb_s32 */, 100361 /* q8sq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100368 /* svqxtnb_s64 */, 100380 /* q4iq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100388 /* svqxtnb_u16 */, 100400 /* q16Ucq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100410 /* svqxtnb_u32 */, 100422 /* q8Usq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100431 /* svqxtnb_u64 */, 100443 /* q4Uiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100453 /* svqxtnt_s16 */, 100465 /* q16Scq16Scq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100479 /* svqxtnt_s32 */, 100491 /* q8sq8sq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100501 /* svqxtnt_s64 */, 100513 /* q4iq4iq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100524 /* svqxtnt_u16 */, 100536 /* q16Ucq16Ucq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100551 /* svqxtnt_u32 */, 100563 /* q8Usq8Usq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100576 /* svqxtnt_u64 */, 100588 /* q4Uiq4Uiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100602 /* svqxtunb_s16 */, 100615 /* q16Ucq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100624 /* svqxtunb_s32 */, 100637 /* q8Usq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100645 /* svqxtunb_s64 */, 100658 /* q4Uiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100667 /* svqxtunt_s16 */, 100680 /* q16Ucq16Ucq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100694 /* svqxtunt_s32 */, 100707 /* q8Usq8Usq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100719 /* svqxtunt_s64 */, 100732 /* q4Uiq4Uiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100745 /* svraddhnb_n_s16 */, 5495 /* q16Scq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100761 /* svraddhnb_n_s32 */, 5520 /* q8sq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100777 /* svraddhnb_n_s64 */, 5543 /* q4iq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100793 /* svraddhnb_n_u16 */, 5568 /* q16Ucq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100809 /* svraddhnb_n_u32 */, 5595 /* q8Usq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100825 /* svraddhnb_n_u64 */, 5621 /* q4Uiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100841 /* svraddhnb_s16 */, 5647 /* q16Scq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100855 /* svraddhnb_s32 */, 5672 /* q8sq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100869 /* svraddhnb_s64 */, 5695 /* q4iq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100883 /* svraddhnb_u16 */, 5720 /* q16Ucq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100897 /* svraddhnb_u32 */, 5747 /* q8Usq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100911 /* svraddhnb_u64 */, 5773 /* q4Uiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100925 /* svraddhnt_n_s16 */, 5803 /* q16Scq16Scq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100941 /* svraddhnt_n_s32 */, 5833 /* q8sq8sq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100957 /* svraddhnt_n_s64 */, 5859 /* q4iq4iq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100973 /* svraddhnt_n_u16 */, 5887 /* q16Ucq16Ucq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100989 /* svraddhnt_n_u32 */, 5919 /* q8Usq8Usq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101005 /* svraddhnt_n_u64 */, 5949 /* q4Uiq4Uiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101021 /* svraddhnt_s16 */, 5979 /* q16Scq16Scq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101035 /* svraddhnt_s32 */, 6009 /* q8sq8sq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101049 /* svraddhnt_s64 */, 6035 /* q4iq4iq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101063 /* svraddhnt_u16 */, 6063 /* q16Ucq16Ucq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101077 /* svraddhnt_u32 */, 6095 /* q8Usq8Usq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101091 /* svraddhnt_u64 */, 6125 /* q4Uiq4Uiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101105 /* svrax1_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 101116 /* (sve,sve-sha3)|(sme,sme2p1,sve-sha3) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101153 /* svrax1_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 101116 /* (sve,sve-sha3)|(sme,sme2p1,sve-sha3) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101164 /* svrbit_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101177 /* svrbit_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101190 /* svrbit_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101203 /* svrbit_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101216 /* svrbit_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101229 /* svrbit_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101242 /* svrbit_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101255 /* svrbit_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101268 /* svrbit_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101281 /* svrbit_s8_m */, 2982 /* q16Scq16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101293 /* svrbit_s8_x */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101305 /* svrbit_s8_z */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101317 /* svrbit_u16_m */, 16483 /* q8Usq8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101330 /* svrbit_u16_x */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101343 /* svrbit_u16_z */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101356 /* svrbit_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101369 /* svrbit_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101382 /* svrbit_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101395 /* svrbit_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101408 /* svrbit_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101421 /* svrbit_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101434 /* svrbit_u8_m */, 16685 /* q16Ucq16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101446 /* svrbit_u8_x */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101458 /* svrbit_u8_z */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101470 /* svrdffr */, 86941 /* q16bv */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101478 /* svrdffr_z */, 101488 /* q16bq16b */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101497 /* svrecpe_f16 */, 101509 /* q8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101516 /* svrecpe_f32 */, 101528 /* q4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101535 /* svrecpe_f64 */, 101547 /* q2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101554 /* svrecpe_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101568 /* svrecpe_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101582 /* svrecpe_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101596 /* svrecps_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101618 /* svrecps_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101640 /* svrecps_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101662 /* svrecpx_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101676 /* svrecpx_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101690 /* svrecpx_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101704 /* svrecpx_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101718 /* svrecpx_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101732 /* svrecpx_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101746 /* svrecpx_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101760 /* svrecpx_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101774 /* svrecpx_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101788 /* svreinterpret_b */, 101804 /* q16bQa */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101811 /* svreinterpret_c */, 101827 /* Qaq16b */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101834 /* svrev_b16 */, 101488 /* q16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101844 /* svrev_b32 */, 101488 /* q16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101854 /* svrev_b64 */, 101488 /* q16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101864 /* svrev_b8 */, 101488 /* q16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101873 /* svrev_bf16 */, 101884 /* q8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101891 /* svrev_f16 */, 101509 /* q8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101901 /* svrev_f32 */, 101528 /* q4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101911 /* svrev_f64 */, 101547 /* q2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101921 /* svrev_s16 */, 101931 /* q8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101938 /* svrev_s32 */, 101948 /* q4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101955 /* svrev_s64 */, 101965 /* q2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101974 /* svrev_s8 */, 101983 /* q16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{101994 /* svrev_u16 */, 102004 /* q8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102013 /* svrev_u32 */, 102023 /* q4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102032 /* svrev_u64 */, 102042 /* q2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102053 /* svrev_u8 */, 8578 /* q16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102062 /* svrevb_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102075 /* svrevb_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102088 /* svrevb_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102101 /* svrevb_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102114 /* svrevb_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102127 /* svrevb_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102140 /* svrevb_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102153 /* svrevb_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102166 /* svrevb_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102179 /* svrevb_u16_m */, 16483 /* q8Usq8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102192 /* svrevb_u16_x */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102205 /* svrevb_u16_z */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102218 /* svrevb_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102231 /* svrevb_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102244 /* svrevb_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102257 /* svrevb_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102270 /* svrevb_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102283 /* svrevb_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102296 /* svrevd_bf16_m */, 102310 /* q8yq8yq16bq8y */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102324 /* svrevd_bf16_x */, 21785 /* q8yq16bq8y */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102338 /* svrevd_bf16_z */, 21785 /* q8yq16bq8y */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102352 /* svrevd_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102365 /* svrevd_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102378 /* svrevd_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102391 /* svrevd_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102404 /* svrevd_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102417 /* svrevd_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102430 /* svrevd_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102443 /* svrevd_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102456 /* svrevd_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102469 /* svrevd_mf8_m */, 102482 /* q16mq16mq16bq16m */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102499 /* svrevd_mf8_x */, 21904 /* q16mq16bq16m */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102512 /* svrevd_mf8_z */, 21904 /* q16mq16bq16m */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102525 /* svrevd_s16_m */, 2795 /* q8sq8sq16bq8s */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102538 /* svrevd_s16_x */, 2821 /* q8sq16bq8s */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102551 /* svrevd_s16_z */, 2821 /* q8sq16bq8s */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102564 /* svrevd_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102577 /* svrevd_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102590 /* svrevd_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102603 /* svrevd_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102616 /* svrevd_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102629 /* svrevd_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102642 /* svrevd_s8_m */, 2982 /* q16Scq16Scq16bq16Sc */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102654 /* svrevd_s8_x */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102666 /* svrevd_s8_z */, 3013 /* q16Scq16bq16Sc */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102678 /* svrevd_u16_m */, 16483 /* q8Usq8Usq16bq8Us */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102691 /* svrevd_u16_x */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102704 /* svrevd_u16_z */, 16512 /* q8Usq16bq8Us */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102717 /* svrevd_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102730 /* svrevd_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102743 /* svrevd_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102756 /* svrevd_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102769 /* svrevd_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102782 /* svrevd_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102795 /* svrevd_u8_m */, 16685 /* q16Ucq16Ucq16bq16Uc */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102807 /* svrevd_u8_x */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102819 /* svrevd_u8_z */, 16716 /* q16Ucq16bq16Uc */, 1 /* n */, 14227 /* (sve,(sve2p1|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102831 /* svrevh_s32_m */, 2856 /* q4iq4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102844 /* svrevh_s32_x */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102857 /* svrevh_s32_z */, 2882 /* q4iq16bq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102870 /* svrevh_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102883 /* svrevh_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102896 /* svrevh_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102909 /* svrevh_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102922 /* svrevh_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102935 /* svrevh_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102948 /* svrevh_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102961 /* svrevh_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102974 /* svrevh_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{102987 /* svrevw_s64_m */, 2917 /* q2Wiq2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103000 /* svrevw_s64_x */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103013 /* svrevw_s64_z */, 2946 /* q2Wiq16bq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103026 /* svrevw_u64_m */, 16615 /* q2UWiq2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103039 /* svrevw_u64_x */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103052 /* svrevw_u64_z */, 16647 /* q2UWiq16bq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103065 /* svrhadd_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103081 /* svrhadd_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103097 /* svrhadd_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103113 /* svrhadd_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103129 /* svrhadd_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103145 /* svrhadd_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103161 /* svrhadd_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103177 /* svrhadd_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103193 /* svrhadd_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103209 /* svrhadd_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103224 /* svrhadd_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103239 /* svrhadd_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103254 /* svrhadd_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103270 /* svrhadd_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103286 /* svrhadd_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103302 /* svrhadd_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103318 /* svrhadd_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103334 /* svrhadd_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103350 /* svrhadd_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103366 /* svrhadd_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103382 /* svrhadd_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103398 /* svrhadd_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103413 /* svrhadd_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103428 /* svrhadd_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103443 /* svrhadd_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103457 /* svrhadd_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103471 /* svrhadd_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103485 /* svrhadd_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103499 /* svrhadd_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103513 /* svrhadd_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103527 /* svrhadd_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103541 /* svrhadd_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103555 /* svrhadd_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103569 /* svrhadd_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103582 /* svrhadd_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103595 /* svrhadd_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103608 /* svrhadd_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103622 /* svrhadd_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103636 /* svrhadd_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103650 /* svrhadd_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103664 /* svrhadd_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103678 /* svrhadd_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103692 /* svrhadd_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103706 /* svrhadd_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103720 /* svrhadd_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103734 /* svrhadd_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103747 /* svrhadd_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103760 /* svrhadd_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103773 /* svrint32x_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103789 /* svrint32x_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103805 /* svrint32x_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103821 /* svrint32x_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103837 /* svrint32x_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103853 /* svrint32x_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103869 /* svrint32z_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103885 /* svrint32z_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103901 /* svrint32z_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103917 /* svrint32z_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103933 /* svrint32z_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103949 /* svrint32z_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103965 /* svrint64x_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103981 /* svrint64x_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{103997 /* svrint64x_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104013 /* svrint64x_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104029 /* svrint64x_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104045 /* svrint64x_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104061 /* svrint64z_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104077 /* svrint64z_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104093 /* svrint64z_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104109 /* svrint64z_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104125 /* svrint64z_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104141 /* svrint64z_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 26717 /* (sve,(sve2p2|sme2p2))|(sme,(sve2p2|sme2p2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104157 /* svrinta_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104171 /* svrinta_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104185 /* svrinta_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104199 /* svrinta_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104213 /* svrinta_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104227 /* svrinta_f32_x2 */, 104242 /* q8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104249 /* svrinta_f32_x4 */, 104264 /* q16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104273 /* svrinta_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104287 /* svrinta_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104301 /* svrinta_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104315 /* svrinta_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104329 /* svrinti_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104343 /* svrinti_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104357 /* svrinti_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104371 /* svrinti_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104385 /* svrinti_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104399 /* svrinti_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104413 /* svrinti_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104427 /* svrinti_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104441 /* svrinti_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104455 /* svrintm_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104469 /* svrintm_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104483 /* svrintm_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104497 /* svrintm_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104511 /* svrintm_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104525 /* svrintm_f32_x2 */, 104242 /* q8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104540 /* svrintm_f32_x4 */, 104264 /* q16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104555 /* svrintm_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104569 /* svrintm_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104583 /* svrintm_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104597 /* svrintm_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104611 /* svrintn_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104625 /* svrintn_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104639 /* svrintn_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104653 /* svrintn_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104667 /* svrintn_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104681 /* svrintn_f32_x2 */, 104242 /* q8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104696 /* svrintn_f32_x4 */, 104264 /* q16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104711 /* svrintn_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104725 /* svrintn_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104739 /* svrintn_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104753 /* svrintn_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104767 /* svrintp_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104781 /* svrintp_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104795 /* svrintp_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104809 /* svrintp_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104823 /* svrintp_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104837 /* svrintp_f32_x2 */, 104242 /* q8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104852 /* svrintp_f32_x4 */, 104264 /* q16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104867 /* svrintp_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104881 /* svrintp_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104895 /* svrintp_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104909 /* svrintp_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104923 /* svrintx_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104937 /* svrintx_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104951 /* svrintx_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104965 /* svrintx_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104979 /* svrintx_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{104993 /* svrintx_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105007 /* svrintx_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105021 /* svrintx_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105035 /* svrintx_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105049 /* svrintz_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105063 /* svrintz_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105077 /* svrintz_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105091 /* svrintz_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105105 /* svrintz_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105119 /* svrintz_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105133 /* svrintz_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105147 /* svrintz_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105161 /* svrintz_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105175 /* svrshl_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105190 /* svrshl_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105205 /* svrshl_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105220 /* svrshl_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105235 /* svrshl_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105250 /* svrshl_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105265 /* svrshl_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105280 /* svrshl_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105295 /* svrshl_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105310 /* svrshl_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105324 /* svrshl_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105338 /* svrshl_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105352 /* svrshl_n_u16_m */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105367 /* svrshl_n_u16_x */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105382 /* svrshl_n_u16_z */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105397 /* svrshl_n_u32_m */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105412 /* svrshl_n_u32_x */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105427 /* svrshl_n_u32_z */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105442 /* svrshl_n_u64_m */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105457 /* svrshl_n_u64_x */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105472 /* svrshl_n_u64_z */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105487 /* svrshl_n_u8_m */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105501 /* svrshl_n_u8_x */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105515 /* svrshl_n_u8_z */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105529 /* svrshl_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105542 /* svrshl_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105555 /* svrshl_s16_x2 */, 70593 /* q16sq16sq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105569 /* svrshl_s16_x4 */, 70619 /* q32sq32sq32s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105583 /* svrshl_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105596 /* svrshl_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105609 /* svrshl_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105622 /* svrshl_s32_x2 */, 70681 /* q8iq8iq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105636 /* svrshl_s32_x4 */, 70704 /* q16iq16iq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105650 /* svrshl_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105663 /* svrshl_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105676 /* svrshl_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105689 /* svrshl_s64_x2 */, 70766 /* q4Wiq4Wiq4Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105703 /* svrshl_s64_x4 */, 70792 /* q8Wiq8Wiq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105717 /* svrshl_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105730 /* svrshl_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105742 /* svrshl_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105754 /* svrshl_s8_x2 */, 70851 /* q32Scq32Scq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105767 /* svrshl_s8_x4 */, 70879 /* q64Scq64Scq64Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105780 /* svrshl_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105792 /* svrshl_single_s16_x2 */, 4741 /* q16sq16sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105813 /* svrshl_single_s16_x4 */, 4782 /* q32sq32sq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105834 /* svrshl_single_s32_x2 */, 4814 /* q8iq8iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105855 /* svrshl_single_s32_x4 */, 4844 /* q16iq16iq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105876 /* svrshl_single_s64_x2 */, 4876 /* q4Wiq4Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105897 /* svrshl_single_s64_x4 */, 4909 /* q8Wiq8Wiq2Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105918 /* svrshl_single_s8_x2 */, 4941 /* q32Scq32Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105938 /* svrshl_single_s8_x4 */, 4976 /* q64Scq64Scq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105958 /* svrshl_single_u16_x2 */, 5012 /* q16Usq16Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{105979 /* svrshl_single_u16_x4 */, 5047 /* q32Usq32Usq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106000 /* svrshl_single_u32_x2 */, 5082 /* q8Uiq8Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106021 /* svrshl_single_u32_x4 */, 5115 /* q16Uiq16Uiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106042 /* svrshl_single_u64_x2 */, 5150 /* q4UWiq4UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106063 /* svrshl_single_u64_x4 */, 5186 /* q8UWiq8UWiq2UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106084 /* svrshl_single_u8_x2 */, 5221 /* q32Ucq32Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106104 /* svrshl_single_u8_x4 */, 5256 /* q64Ucq64Ucq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106124 /* svrshl_u16_m */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106137 /* svrshl_u16_x */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106150 /* svrshl_u16_x2 */, 71511 /* q16Usq16Usq16Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106164 /* svrshl_u16_x4 */, 71540 /* q32Usq32Usq32Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106178 /* svrshl_u16_z */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106191 /* svrshl_u32_m */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106204 /* svrshl_u32_x */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106217 /* svrshl_u32_x2 */, 71605 /* q8Uiq8Uiq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106231 /* svrshl_u32_x4 */, 71631 /* q16Uiq16Uiq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106245 /* svrshl_u32_z */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106258 /* svrshl_u64_m */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106271 /* svrshl_u64_x */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106284 /* svrshl_u64_x2 */, 71696 /* q4UWiq4UWiq4UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106298 /* svrshl_u64_x4 */, 71725 /* q8UWiq8UWiq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106312 /* svrshl_u64_z */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106325 /* svrshl_u8_m */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106337 /* svrshl_u8_x */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106349 /* svrshl_u8_x2 */, 71787 /* q32Ucq32Ucq32Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106362 /* svrshl_u8_x4 */, 71815 /* q64Ucq64Ucq64Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106375 /* svrshl_u8_z */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106387 /* svrshr_n_s16_m */, 11153 /* q8sq16bq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106402 /* svrshr_n_s16_x */, 11153 /* q8sq16bq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106417 /* svrshr_n_s16_z */, 11153 /* q8sq16bq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106432 /* svrshr_n_s32_m */, 11213 /* q4iq16bq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106447 /* svrshr_n_s32_x */, 11213 /* q4iq16bq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106462 /* svrshr_n_s32_z */, 11213 /* q4iq16bq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106477 /* svrshr_n_s64_m */, 11273 /* q2Wiq16bq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106492 /* svrshr_n_s64_x */, 11273 /* q2Wiq16bq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106507 /* svrshr_n_s64_z */, 11273 /* q2Wiq16bq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106522 /* svrshr_n_s8_m */, 11334 /* q16Scq16bq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106536 /* svrshr_n_s8_x */, 11334 /* q16Scq16bq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106550 /* svrshr_n_s8_z */, 11334 /* q16Scq16bq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106564 /* svrshr_n_u16_m */, 106579 /* q8Usq16bq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106596 /* svrshr_n_u16_x */, 106579 /* q8Usq16bq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106611 /* svrshr_n_u16_z */, 106579 /* q8Usq16bq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106626 /* svrshr_n_u32_m */, 106641 /* q4Uiq16bq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106658 /* svrshr_n_u32_x */, 106641 /* q4Uiq16bq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106673 /* svrshr_n_u32_z */, 106641 /* q4Uiq16bq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106688 /* svrshr_n_u64_m */, 106703 /* q2UWiq16bq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106722 /* svrshr_n_u64_x */, 106703 /* q2UWiq16bq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106737 /* svrshr_n_u64_z */, 106703 /* q2UWiq16bq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106752 /* svrshr_n_u8_m */, 106766 /* q16Ucq16bq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106785 /* svrshr_n_u8_x */, 106766 /* q16Ucq16bq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106799 /* svrshr_n_u8_z */, 106766 /* q16Ucq16bq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106813 /* svrshrnb_n_s16 */, 96861 /* q16Scq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106828 /* svrshrnb_n_s32 */, 96890 /* q8sq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106843 /* svrshrnb_n_s64 */, 96917 /* q4iq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106858 /* svrshrnb_n_u16 */, 96945 /* q16Ucq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106873 /* svrshrnb_n_u32 */, 96975 /* q8Usq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106888 /* svrshrnb_n_u64 */, 97004 /* q4Uiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106903 /* svrshrnt_n_s16 */, 97034 /* q16Scq16Scq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106918 /* svrshrnt_n_s32 */, 97068 /* q8sq8sq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106933 /* svrshrnt_n_s64 */, 97098 /* q4iq4iq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106948 /* svrshrnt_n_u16 */, 97129 /* q16Ucq16Ucq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106963 /* svrshrnt_n_u32 */, 97164 /* q8Usq8Usq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106978 /* svrshrnt_n_u64 */, 97197 /* q4Uiq4Uiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{106993 /* svrsqrte_f16 */, 101509 /* q8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107006 /* svrsqrte_f32 */, 101528 /* q4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107019 /* svrsqrte_f64 */, 101547 /* q2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107032 /* svrsqrte_u32_m */, 16549 /* q4Uiq4Uiq16bq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107047 /* svrsqrte_u32_x */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107062 /* svrsqrte_u32_z */, 16578 /* q4Uiq16bq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107077 /* svrsqrts_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107090 /* svrsqrts_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107103 /* svrsqrts_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107116 /* svrsra_n_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107129 /* svrsra_n_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107142 /* svrsra_n_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107155 /* svrsra_n_s8 */, 13835 /* q16Scq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107167 /* svrsra_n_u16 */, 13866 /* q8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107180 /* svrsra_n_u32 */, 13894 /* q4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107193 /* svrsra_n_u64 */, 13922 /* q2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107206 /* svrsra_n_u8 */, 13952 /* q16Ucq16Ucq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107218 /* svrsubhnb_n_s16 */, 5495 /* q16Scq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107234 /* svrsubhnb_n_s32 */, 5520 /* q8sq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107250 /* svrsubhnb_n_s64 */, 5543 /* q4iq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107266 /* svrsubhnb_n_u16 */, 5568 /* q16Ucq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107282 /* svrsubhnb_n_u32 */, 5595 /* q8Usq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107298 /* svrsubhnb_n_u64 */, 5621 /* q4Uiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107314 /* svrsubhnb_s16 */, 5647 /* q16Scq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107328 /* svrsubhnb_s32 */, 5672 /* q8sq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107342 /* svrsubhnb_s64 */, 5695 /* q4iq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107356 /* svrsubhnb_u16 */, 5720 /* q16Ucq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107370 /* svrsubhnb_u32 */, 5747 /* q8Usq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107384 /* svrsubhnb_u64 */, 5773 /* q4Uiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107398 /* svrsubhnt_n_s16 */, 5803 /* q16Scq16Scq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107414 /* svrsubhnt_n_s32 */, 5833 /* q8sq8sq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107430 /* svrsubhnt_n_s64 */, 5859 /* q4iq4iq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107446 /* svrsubhnt_n_u16 */, 5887 /* q16Ucq16Ucq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107462 /* svrsubhnt_n_u32 */, 5919 /* q8Usq8Usq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107478 /* svrsubhnt_n_u64 */, 5949 /* q4Uiq4Uiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107494 /* svrsubhnt_s16 */, 5979 /* q16Scq16Scq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107508 /* svrsubhnt_s32 */, 6009 /* q8sq8sq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107522 /* svrsubhnt_s64 */, 6035 /* q4iq4iq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107536 /* svrsubhnt_u16 */, 6063 /* q16Ucq16Ucq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107550 /* svrsubhnt_u32 */, 6095 /* q8Usq8Usq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107564 /* svrsubhnt_u64 */, 6125 /* q4Uiq4Uiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107578 /* svsbclb_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107592 /* svsbclb_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107606 /* svsbclb_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107618 /* svsbclb_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107630 /* svsbclt_n_u32 */, 165 /* q4Uiq4Uiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107644 /* svsbclt_n_u64 */, 192 /* q2UWiq2UWiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107658 /* svsbclt_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107670 /* svsbclt_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107682 /* svscale_bf16_m */, 107697 /* q8yq16bq8yq8s */, 1 /* n */, 107711 /* (sve,sve-bfscale)|(sme,sve-bfscale,sme2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107752 /* svscale_bf16_x */, 107697 /* q8yq16bq8yq8s */, 1 /* n */, 107711 /* (sve,sve-bfscale)|(sme,sve-bfscale,sme2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107767 /* svscale_bf16_x2 */, 107783 /* q16yq16yq16s */, 1 /* n */, 81401 /* sme,sme2,sve-bfscale */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107796 /* svscale_bf16_x4 */, 107812 /* q32yq32yq32s */, 1 /* n */, 81401 /* sme,sme2,sve-bfscale */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107825 /* svscale_bf16_z */, 107697 /* q8yq16bq8yq8s */, 1 /* n */, 107711 /* (sve,sve-bfscale)|(sme,sve-bfscale,sme2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107840 /* svscale_f16_m */, 107854 /* q8hq16bq8hq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107868 /* svscale_f16_x */, 107854 /* q8hq16bq8hq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107882 /* svscale_f16_x2 */, 107897 /* q16hq16hq16s */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107910 /* svscale_f16_x4 */, 107925 /* q32hq32hq32s */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107938 /* svscale_f16_z */, 107854 /* q8hq16bq8hq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107952 /* svscale_f32_m */, 107966 /* q4fq16bq4fq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107980 /* svscale_f32_x */, 107966 /* q4fq16bq4fq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{107994 /* svscale_f32_x2 */, 108009 /* q8fq8fq8i */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108019 /* svscale_f32_x4 */, 108034 /* q16fq16fq16i */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108047 /* svscale_f32_z */, 107966 /* q4fq16bq4fq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108061 /* svscale_f64_m */, 108075 /* q2dq16bq2dq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108090 /* svscale_f64_x */, 108075 /* q2dq16bq2dq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108104 /* svscale_f64_x2 */, 108119 /* q4dq4dq4Wi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108130 /* svscale_f64_x4 */, 108145 /* q8dq8dq8Wi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108156 /* svscale_f64_z */, 108075 /* q2dq16bq2dq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108170 /* svscale_n_bf16_m */, 108187 /* q8yq16bq8ys */, 1 /* n */, 107711 /* (sve,sve-bfscale)|(sme,sve-bfscale,sme2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108199 /* svscale_n_bf16_x */, 108187 /* q8yq16bq8ys */, 1 /* n */, 107711 /* (sve,sve-bfscale)|(sme,sve-bfscale,sme2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108216 /* svscale_n_bf16_z */, 108187 /* q8yq16bq8ys */, 1 /* n */, 107711 /* (sve,sve-bfscale)|(sme,sve-bfscale,sme2) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108233 /* svscale_n_f16_m */, 108249 /* q8hq16bq8hs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108261 /* svscale_n_f16_x */, 108249 /* q8hq16bq8hs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108277 /* svscale_n_f16_z */, 108249 /* q8hq16bq8hs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108293 /* svscale_n_f32_m */, 108309 /* q4fq16bq4fi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108321 /* svscale_n_f32_x */, 108309 /* q4fq16bq4fi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108337 /* svscale_n_f32_z */, 108309 /* q4fq16bq4fi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108353 /* svscale_n_f64_m */, 108369 /* q2dq16bq2dWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108382 /* svscale_n_f64_x */, 108369 /* q2dq16bq2dWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108398 /* svscale_n_f64_z */, 108369 /* q2dq16bq2dWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108414 /* svscale_single_bf16_x2 */, 108437 /* q16yq16yq8s */, 1 /* n */, 81401 /* sme,sme2,sve-bfscale */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108449 /* svscale_single_bf16_x4 */, 108472 /* q32yq32yq8s */, 1 /* n */, 81401 /* sme,sme2,sve-bfscale */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108484 /* svscale_single_f16_x2 */, 108506 /* q16hq16hq8s */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108518 /* svscale_single_f16_x4 */, 108540 /* q32hq32hq8s */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108552 /* svscale_single_f32_x2 */, 108574 /* q8fq8fq4i */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108584 /* svscale_single_f32_x4 */, 108606 /* q16fq16fq4i */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108618 /* svscale_single_f64_x2 */, 108640 /* q4dq4dq2Wi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108651 /* svscale_single_f64_x4 */, 108673 /* q8dq8dq2Wi */, 1 /* n */, 23376 /* sme,sme2,fp8 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108684 /* svsel_b */, 9416 /* q16bq16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108692 /* svsel_bf16 */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108703 /* svsel_bf16_x2 */, 108717 /* q16yQaq16yq16y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108732 /* svsel_bf16_x4 */, 108746 /* q32yQaq32yq32y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108761 /* svsel_f16 */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108771 /* svsel_f16_x2 */, 108784 /* q16hQaq16hq16h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108799 /* svsel_f16_x4 */, 108812 /* q32hQaq32hq32h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108827 /* svsel_f32 */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108837 /* svsel_f32_x2 */, 108850 /* q8fQaq8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108862 /* svsel_f32_x4 */, 108875 /* q16fQaq16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108890 /* svsel_f64 */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108900 /* svsel_f64_x2 */, 108913 /* q4dQaq4dq4d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108925 /* svsel_f64_x4 */, 108938 /* q8dQaq8dq8d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108950 /* svsel_mf8_x2 */, 108963 /* q32mQaq32mq32m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{108978 /* svsel_mf8_x4 */, 108991 /* q64mQaq64mq64m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109006 /* svsel_s16 */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109016 /* svsel_s16_x2 */, 109029 /* q16sQaq16sq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109044 /* svsel_s16_x4 */, 109057 /* q32sQaq32sq32s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109072 /* svsel_s32 */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109082 /* svsel_s32_x2 */, 109095 /* q8iQaq8iq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109107 /* svsel_s32_x4 */, 109120 /* q16iQaq16iq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109135 /* svsel_s64 */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109145 /* svsel_s64_x2 */, 109158 /* q4WiQaq4Wiq4Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109173 /* svsel_s64_x4 */, 109186 /* q8WiQaq8Wiq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109201 /* svsel_s8 */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109210 /* svsel_s8_x2 */, 109222 /* q32ScQaq32Scq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109240 /* svsel_s8_x4 */, 109252 /* q64ScQaq64Scq64Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109270 /* svsel_u16 */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109280 /* svsel_u16_x2 */, 109293 /* q16UsQaq16Usq16Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109311 /* svsel_u16_x4 */, 109324 /* q32UsQaq32Usq32Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109342 /* svsel_u32 */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109352 /* svsel_u32_x2 */, 109365 /* q8UiQaq8Uiq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109380 /* svsel_u32_x4 */, 109393 /* q16UiQaq16Uiq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109411 /* svsel_u64 */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109421 /* svsel_u64_x2 */, 109434 /* q4UWiQaq4UWiq4UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109452 /* svsel_u64_x4 */, 109465 /* q8UWiQaq8UWiq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109483 /* svsel_u8 */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109492 /* svsel_u8_x2 */, 109504 /* q32UcQaq32Ucq32Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109522 /* svsel_u8_x4 */, 109534 /* q64UcQaq64Ucq64Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109552 /* svset2_b */, 109561 /* q32bq32bIUWiq16b */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109578 /* svset2_bf16 */, 109590 /* q16yq16yIUWiq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109606 /* svset2_f16 */, 109617 /* q16hq16hIUWiq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109633 /* svset2_f32 */, 109644 /* q8fq8fIUWiq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109658 /* svset2_f64 */, 109669 /* q4dq4dIUWiq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109683 /* svset2_mf8 */, 109694 /* q32mq32mIUWiq16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109711 /* svset2_s16 */, 109722 /* q16sq16sIUWiq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109738 /* svset2_s32 */, 109749 /* q8iq8iIUWiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109763 /* svset2_s64 */, 109774 /* q4Wiq4WiIUWiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109791 /* svset2_s8 */, 109801 /* q32Scq32ScIUWiq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109821 /* svset2_u16 */, 109832 /* q16Usq16UsIUWiq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109851 /* svset2_u32 */, 109862 /* q8Uiq8UiIUWiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109879 /* svset2_u64 */, 109890 /* q4UWiq4UWiIUWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109910 /* svset2_u8 */, 109920 /* q32Ucq32UcIUWiq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109940 /* svset3_bf16 */, 109952 /* q24yq24yIUWiq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109968 /* svset3_f16 */, 109979 /* q24hq24hIUWiq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{109995 /* svset3_f32 */, 110006 /* q12fq12fIUWiq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110022 /* svset3_f64 */, 110033 /* q6dq6dIUWiq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110047 /* svset3_mf8 */, 110058 /* q48mq48mIUWiq16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110075 /* svset3_s16 */, 110086 /* q24sq24sIUWiq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110102 /* svset3_s32 */, 110113 /* q12iq12iIUWiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110129 /* svset3_s64 */, 110140 /* q6Wiq6WiIUWiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110157 /* svset3_s8 */, 110167 /* q48Scq48ScIUWiq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110187 /* svset3_u16 */, 110198 /* q24Usq24UsIUWiq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110217 /* svset3_u32 */, 110228 /* q12Uiq12UiIUWiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110247 /* svset3_u64 */, 110258 /* q6UWiq6UWiIUWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110278 /* svset3_u8 */, 110288 /* q48Ucq48UcIUWiq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110308 /* svset4_b */, 110317 /* q64bq64bIUWiq16b */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110334 /* svset4_bf16 */, 110346 /* q32yq32yIUWiq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110362 /* svset4_f16 */, 110373 /* q32hq32hIUWiq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110389 /* svset4_f32 */, 110400 /* q16fq16fIUWiq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110416 /* svset4_f64 */, 110427 /* q8dq8dIUWiq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110441 /* svset4_mf8 */, 110452 /* q64mq64mIUWiq16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110469 /* svset4_s16 */, 110480 /* q32sq32sIUWiq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110496 /* svset4_s32 */, 110507 /* q16iq16iIUWiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110523 /* svset4_s64 */, 110534 /* q8Wiq8WiIUWiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110551 /* svset4_s8 */, 110561 /* q64Scq64ScIUWiq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110581 /* svset4_u16 */, 110592 /* q32Usq32UsIUWiq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110611 /* svset4_u32 */, 110622 /* q16Uiq16UiIUWiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110641 /* svset4_u64 */, 110652 /* q8UWiq8UWiIUWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110672 /* svset4_u8 */, 110682 /* q64Ucq64UcIUWiq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110702 /* svsetffr */, 110711 /* vv */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110714 /* svshllb_n_s16 */, 110728 /* q8sq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110741 /* svshllb_n_s32 */, 110755 /* q4iq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110766 /* svshllb_n_s64 */, 110780 /* q2Wiq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110792 /* svshllb_n_u16 */, 110806 /* q8Usq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110820 /* svshllb_n_u32 */, 110834 /* q4Uiq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110847 /* svshllb_n_u64 */, 110861 /* q2UWiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110875 /* svshllt_n_s16 */, 110728 /* q8sq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110889 /* svshllt_n_s32 */, 110755 /* q4iq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110903 /* svshllt_n_s64 */, 110780 /* q2Wiq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110917 /* svshllt_n_u16 */, 110806 /* q8Usq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110931 /* svshllt_n_u32 */, 110834 /* q4Uiq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110945 /* svshllt_n_u64 */, 110861 /* q2UWiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110959 /* svshrnb_n_s16 */, 96861 /* q16Scq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110973 /* svshrnb_n_s32 */, 96890 /* q8sq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{110987 /* svshrnb_n_s64 */, 96917 /* q4iq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111001 /* svshrnb_n_u16 */, 96945 /* q16Ucq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111015 /* svshrnb_n_u32 */, 96975 /* q8Usq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111029 /* svshrnb_n_u64 */, 97004 /* q4Uiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111043 /* svshrnt_n_s16 */, 97034 /* q16Scq16Scq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111057 /* svshrnt_n_s32 */, 97068 /* q8sq8sq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111071 /* svshrnt_n_s64 */, 97098 /* q4iq4iq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111085 /* svshrnt_n_u16 */, 97129 /* q16Ucq16Ucq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111099 /* svshrnt_n_u32 */, 97164 /* q8Usq8Usq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111113 /* svshrnt_n_u64 */, 97197 /* q4Uiq4Uiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111127 /* svsli_n_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111139 /* svsli_n_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111151 /* svsli_n_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111163 /* svsli_n_s8 */, 13835 /* q16Scq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111174 /* svsli_n_u16 */, 13866 /* q8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111186 /* svsli_n_u32 */, 13894 /* q4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111198 /* svsli_n_u64 */, 13922 /* q2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111210 /* svsli_n_u8 */, 13952 /* q16Ucq16Ucq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111221 /* svsm4e_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 111232 /* sve,sve-sm4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111244 /* svsm4ekey_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 111232 /* sve,sve-sm4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111258 /* svsplice_bf16 */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111272 /* svsplice_f16 */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111285 /* svsplice_f32 */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111298 /* svsplice_f64 */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111311 /* svsplice_s16 */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111324 /* svsplice_s32 */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111337 /* svsplice_s64 */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111350 /* svsplice_s8 */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111362 /* svsplice_u16 */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111375 /* svsplice_u32 */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111388 /* svsplice_u64 */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111401 /* svsplice_u8 */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111413 /* svsqadd_n_u16_m */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111429 /* svsqadd_n_u16_x */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111445 /* svsqadd_n_u16_z */, 95893 /* q8Usq16bq8Uss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111461 /* svsqadd_n_u32_m */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111477 /* svsqadd_n_u32_x */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111493 /* svsqadd_n_u32_z */, 95955 /* q4Uiq16bq4Uii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111509 /* svsqadd_n_u64_m */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111525 /* svsqadd_n_u64_x */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111541 /* svsqadd_n_u64_z */, 39439 /* q2UWiq16bq2UWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111557 /* svsqadd_n_u8_m */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111572 /* svsqadd_n_u8_x */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111587 /* svsqadd_n_u8_z */, 96064 /* q16Ucq16bq16UcSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111602 /* svsqadd_u16_m */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111616 /* svsqadd_u16_x */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111630 /* svsqadd_u16_z */, 96290 /* q8Usq16bq8Usq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111644 /* svsqadd_u32_m */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111658 /* svsqadd_u32_x */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111672 /* svsqadd_u32_z */, 96348 /* q4Uiq16bq4Uiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111686 /* svsqadd_u64_m */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111700 /* svsqadd_u64_x */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111714 /* svsqadd_u64_z */, 96406 /* q2UWiq16bq2UWiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111728 /* svsqadd_u8_m */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111741 /* svsqadd_u8_x */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111754 /* svsqadd_u8_z */, 96466 /* q16Ucq16bq16Ucq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111767 /* svsqrt_f16_m */, 2612 /* q8hq8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111780 /* svsqrt_f16_x */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111793 /* svsqrt_f16_z */, 2638 /* q8hq16bq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111806 /* svsqrt_f32_m */, 2673 /* q4fq4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111819 /* svsqrt_f32_x */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111832 /* svsqrt_f32_z */, 2699 /* q4fq16bq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111845 /* svsqrt_f64_m */, 2734 /* q2dq2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111858 /* svsqrt_f64_x */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111871 /* svsqrt_f64_z */, 2760 /* q2dq16bq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111884 /* svsra_n_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111896 /* svsra_n_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111908 /* svsra_n_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111920 /* svsra_n_s8 */, 13835 /* q16Scq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111931 /* svsra_n_u16 */, 13866 /* q8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111943 /* svsra_n_u32 */, 13894 /* q4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111955 /* svsra_n_u64 */, 13922 /* q2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111967 /* svsra_n_u8 */, 13952 /* q16Ucq16Ucq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111978 /* svsri_n_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{111990 /* svsri_n_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112002 /* svsri_n_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112014 /* svsri_n_s8 */, 13835 /* q16Scq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112025 /* svsri_n_u16 */, 13866 /* q8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112037 /* svsri_n_u32 */, 13894 /* q4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112049 /* svsri_n_u64 */, 13922 /* q2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112061 /* svsri_n_u8 */, 13952 /* q16Ucq16Ucq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112072 /* svst1_bf16 */, 112083 /* vq16by*q8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112094 /* svst1_bf16_x2 */, 112108 /* vQay*q16y */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112118 /* svst1_bf16_x4 */, 112132 /* vQay*q32y */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112142 /* svst1_f16 */, 112152 /* vq16bh*q8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112163 /* svst1_f16_x2 */, 112176 /* vQah*q16h */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112186 /* svst1_f16_x4 */, 112199 /* vQah*q32h */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112209 /* svst1_f32 */, 112219 /* vq16bf*q4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112230 /* svst1_f32_x2 */, 112243 /* vQaf*q8f */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112252 /* svst1_f32_x4 */, 112265 /* vQaf*q16f */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112275 /* svst1_f64 */, 112285 /* vq16bd*q2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112296 /* svst1_f64_x2 */, 112309 /* vQad*q4d */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112318 /* svst1_f64_x4 */, 112331 /* vQad*q8d */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112340 /* svst1_mf8 */, 112350 /* vq16bm*q16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112362 /* svst1_mf8_x2 */, 112375 /* vQam*q32m */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112385 /* svst1_mf8_x4 */, 112398 /* vQam*q64m */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112408 /* svst1_s16 */, 112418 /* vq16bSs*q8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112430 /* svst1_s16_x2 */, 112443 /* vQaSs*q16s */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112454 /* svst1_s16_x4 */, 112467 /* vQaSs*q32s */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112478 /* svst1_s32 */, 112488 /* vq16bSi*q4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112500 /* svst1_s32_x2 */, 112513 /* vQaSi*q8i */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112523 /* svst1_s32_x4 */, 112536 /* vQaSi*q16i */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112547 /* svst1_s64 */, 112557 /* vq16bSWi*q2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112571 /* svst1_s64_x2 */, 112584 /* vQaSWi*q4Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112596 /* svst1_s64_x4 */, 112609 /* vQaSWi*q8Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112621 /* svst1_s8 */, 112630 /* vq16bSc*q16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112644 /* svst1_s8_x2 */, 112656 /* vQaSc*q32Sc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112668 /* svst1_s8_x4 */, 112680 /* vQaSc*q64Sc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112692 /* svst1_scatter_s32index_f32 */, 112719 /* vq16bf*q4iq4f */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112733 /* svst1_scatter_s32index_s32 */, 112760 /* vq16bSi*q4iq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112775 /* svst1_scatter_s32index_u32 */, 112802 /* vq16bUi*q4iq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112818 /* svst1_scatter_s32offset_f32 */, 112719 /* vq16bf*q4iq4f */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112846 /* svst1_scatter_s32offset_s32 */, 112760 /* vq16bSi*q4iq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112874 /* svst1_scatter_s32offset_u32 */, 112802 /* vq16bUi*q4iq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112902 /* svst1_scatter_s64index_f64 */, 112929 /* vq16bd*q2Wiq2d */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112944 /* svst1_scatter_s64index_s64 */, 112971 /* vq16bSWi*q2Wiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{112989 /* svst1_scatter_s64index_u64 */, 113016 /* vq16bUWi*q2Wiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113035 /* svst1_scatter_s64offset_f64 */, 112929 /* vq16bd*q2Wiq2d */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113063 /* svst1_scatter_s64offset_s64 */, 112971 /* vq16bSWi*q2Wiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113091 /* svst1_scatter_s64offset_u64 */, 113016 /* vq16bUWi*q2Wiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113119 /* svst1_scatter_u32base_f32 */, 113145 /* vq16bq4Uiq4f */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113158 /* svst1_scatter_u32base_index_f32 */, 113190 /* vq16bq4UiWiq4f */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113205 /* svst1_scatter_u32base_index_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113252 /* svst1_scatter_u32base_index_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113300 /* svst1_scatter_u32base_offset_f32 */, 113190 /* vq16bq4UiWiq4f */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113333 /* svst1_scatter_u32base_offset_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113366 /* svst1_scatter_u32base_offset_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113399 /* svst1_scatter_u32base_s32 */, 113425 /* vq16bq4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113438 /* svst1_scatter_u32base_u32 */, 113464 /* vq16bq4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113478 /* svst1_scatter_u32index_f32 */, 113505 /* vq16bf*q4Uiq4f */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113520 /* svst1_scatter_u32index_s32 */, 113547 /* vq16bSi*q4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113563 /* svst1_scatter_u32index_u32 */, 113590 /* vq16bUi*q4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113607 /* svst1_scatter_u32offset_f32 */, 113505 /* vq16bf*q4Uiq4f */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113635 /* svst1_scatter_u32offset_s32 */, 113547 /* vq16bSi*q4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113663 /* svst1_scatter_u32offset_u32 */, 113590 /* vq16bUi*q4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113691 /* svst1_scatter_u64base_f64 */, 113717 /* vq16bq2UWiq2d */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113731 /* svst1_scatter_u64base_index_f64 */, 113763 /* vq16bq2UWiWiq2d */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113779 /* svst1_scatter_u64base_index_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113828 /* svst1_scatter_u64base_index_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113878 /* svst1_scatter_u64base_offset_f64 */, 113763 /* vq16bq2UWiWiq2d */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113911 /* svst1_scatter_u64base_offset_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113944 /* svst1_scatter_u64base_offset_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{113977 /* svst1_scatter_u64base_s64 */, 114003 /* vq16bq2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114018 /* svst1_scatter_u64base_u64 */, 114044 /* vq16bq2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114060 /* svst1_scatter_u64index_f64 */, 114087 /* vq16bd*q2UWiq2d */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114103 /* svst1_scatter_u64index_s64 */, 114130 /* vq16bSWi*q2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114149 /* svst1_scatter_u64index_u64 */, 114176 /* vq16bUWi*q2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114196 /* svst1_scatter_u64offset_f64 */, 114087 /* vq16bd*q2UWiq2d */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114224 /* svst1_scatter_u64offset_s64 */, 114130 /* vq16bSWi*q2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114252 /* svst1_scatter_u64offset_u64 */, 114176 /* vq16bUWi*q2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114280 /* svst1_u16 */, 114290 /* vq16bUs*q8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114303 /* svst1_u16_x2 */, 114316 /* vQaUs*q16Us */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114328 /* svst1_u16_x4 */, 114341 /* vQaUs*q32Us */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114353 /* svst1_u32 */, 114363 /* vq16bUi*q4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114376 /* svst1_u32_x2 */, 114389 /* vQaUi*q8Ui */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114400 /* svst1_u32_x4 */, 114413 /* vQaUi*q16Ui */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114425 /* svst1_u64 */, 114435 /* vq16bUWi*q2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114450 /* svst1_u64_x2 */, 114463 /* vQaUWi*q4UWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114476 /* svst1_u64_x4 */, 114489 /* vQaUWi*q8UWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114502 /* svst1_u8 */, 114511 /* vq16bUc*q16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114525 /* svst1_u8_x2 */, 114537 /* vQaUc*q32Uc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114549 /* svst1_u8_x4 */, 114561 /* vQaUc*q64Uc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114573 /* svst1_vnum_bf16 */, 114589 /* vq16by*Wiq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114602 /* svst1_vnum_bf16_x2 */, 114621 /* vQay*Wiq16y */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114633 /* svst1_vnum_bf16_x4 */, 114652 /* vQay*Wiq32y */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114664 /* svst1_vnum_f16 */, 114679 /* vq16bh*Wiq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114692 /* svst1_vnum_f16_x2 */, 114710 /* vQah*Wiq16h */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114722 /* svst1_vnum_f16_x4 */, 114740 /* vQah*Wiq32h */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114752 /* svst1_vnum_f32 */, 114767 /* vq16bf*Wiq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114780 /* svst1_vnum_f32_x2 */, 114798 /* vQaf*Wiq8f */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114809 /* svst1_vnum_f32_x4 */, 114827 /* vQaf*Wiq16f */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114839 /* svst1_vnum_f64 */, 114854 /* vq16bd*Wiq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114867 /* svst1_vnum_f64_x2 */, 114885 /* vQad*Wiq4d */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114896 /* svst1_vnum_f64_x4 */, 114914 /* vQad*Wiq8d */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114925 /* svst1_vnum_mf8 */, 114940 /* vq16bm*Wiq16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114954 /* svst1_vnum_mf8_x2 */, 114972 /* vQam*Wiq32m */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{114984 /* svst1_vnum_mf8_x4 */, 115002 /* vQam*Wiq64m */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115014 /* svst1_vnum_s16 */, 115029 /* vq16bSs*Wiq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115043 /* svst1_vnum_s16_x2 */, 115061 /* vQaSs*Wiq16s */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115074 /* svst1_vnum_s16_x4 */, 115092 /* vQaSs*Wiq32s */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115105 /* svst1_vnum_s32 */, 115120 /* vq16bSi*Wiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115134 /* svst1_vnum_s32_x2 */, 115152 /* vQaSi*Wiq8i */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115164 /* svst1_vnum_s32_x4 */, 115182 /* vQaSi*Wiq16i */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115195 /* svst1_vnum_s64 */, 115210 /* vq16bSWi*Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115226 /* svst1_vnum_s64_x2 */, 115244 /* vQaSWi*Wiq4Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115258 /* svst1_vnum_s64_x4 */, 115276 /* vQaSWi*Wiq8Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115290 /* svst1_vnum_s8 */, 115304 /* vq16bSc*Wiq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115320 /* svst1_vnum_s8_x2 */, 115337 /* vQaSc*Wiq32Sc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115351 /* svst1_vnum_s8_x4 */, 115368 /* vQaSc*Wiq64Sc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115382 /* svst1_vnum_u16 */, 115397 /* vq16bUs*Wiq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115412 /* svst1_vnum_u16_x2 */, 115430 /* vQaUs*Wiq16Us */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115444 /* svst1_vnum_u16_x4 */, 115462 /* vQaUs*Wiq32Us */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115476 /* svst1_vnum_u32 */, 115491 /* vq16bUi*Wiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115506 /* svst1_vnum_u32_x2 */, 115524 /* vQaUi*Wiq8Ui */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115537 /* svst1_vnum_u32_x4 */, 115555 /* vQaUi*Wiq16Ui */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115569 /* svst1_vnum_u64 */, 115584 /* vq16bUWi*Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115601 /* svst1_vnum_u64_x2 */, 115619 /* vQaUWi*Wiq4UWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115634 /* svst1_vnum_u64_x4 */, 115652 /* vQaUWi*Wiq8UWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115667 /* svst1_vnum_u8 */, 115681 /* vq16bUc*Wiq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115697 /* svst1_vnum_u8_x2 */, 115714 /* vQaUc*Wiq32Uc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115728 /* svst1_vnum_u8_x4 */, 115745 /* vQaUc*Wiq64Uc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115759 /* svst1b_s16 */, 115770 /* vq16bSc*q8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115782 /* svst1b_s32 */, 115793 /* vq16bSc*q4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115805 /* svst1b_s64 */, 115816 /* vq16bSc*q2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115829 /* svst1b_scatter_s32offset_s32 */, 115858 /* vq16bSc*q4iq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115873 /* svst1b_scatter_s32offset_u32 */, 115902 /* vq16bUc*q4iq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115918 /* svst1b_scatter_s64offset_s64 */, 115947 /* vq16bSc*q2Wiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{115964 /* svst1b_scatter_s64offset_u64 */, 115993 /* vq16bUc*q2Wiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116011 /* svst1b_scatter_u32base_offset_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116045 /* svst1b_scatter_u32base_offset_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116079 /* svst1b_scatter_u32base_s32 */, 113425 /* vq16bq4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116106 /* svst1b_scatter_u32base_u32 */, 113464 /* vq16bq4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116133 /* svst1b_scatter_u32offset_s32 */, 116162 /* vq16bSc*q4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116178 /* svst1b_scatter_u32offset_u32 */, 116207 /* vq16bUc*q4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116224 /* svst1b_scatter_u64base_offset_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116258 /* svst1b_scatter_u64base_offset_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116292 /* svst1b_scatter_u64base_s64 */, 114003 /* vq16bq2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116319 /* svst1b_scatter_u64base_u64 */, 114044 /* vq16bq2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116346 /* svst1b_scatter_u64offset_s64 */, 116375 /* vq16bSc*q2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116393 /* svst1b_scatter_u64offset_u64 */, 116422 /* vq16bUc*q2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116441 /* svst1b_u16 */, 116452 /* vq16bUc*q8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116465 /* svst1b_u32 */, 116476 /* vq16bUc*q4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116489 /* svst1b_u64 */, 116500 /* vq16bUc*q2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116514 /* svst1b_vnum_s16 */, 116530 /* vq16bSc*Wiq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116544 /* svst1b_vnum_s32 */, 116560 /* vq16bSc*Wiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116574 /* svst1b_vnum_s64 */, 116590 /* vq16bSc*Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116605 /* svst1b_vnum_u16 */, 116621 /* vq16bUc*Wiq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116636 /* svst1b_vnum_u32 */, 116652 /* vq16bUc*Wiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116667 /* svst1b_vnum_u64 */, 116683 /* vq16bUc*Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116699 /* svst1dq_f64 */, 112285 /* vq16bd*q2d */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116711 /* svst1dq_s64 */, 112557 /* vq16bSWi*q2Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116723 /* svst1dq_u64 */, 114435 /* vq16bUWi*q2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116735 /* svst1dq_vnum_f64 */, 114854 /* vq16bd*Wiq2d */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116752 /* svst1dq_vnum_s64 */, 115210 /* vq16bSWi*Wiq2Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116769 /* svst1dq_vnum_u64 */, 115584 /* vq16bUWi*Wiq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116786 /* svst1h_s32 */, 116797 /* vq16bSs*q4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116809 /* svst1h_s64 */, 116820 /* vq16bSs*q2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116833 /* svst1h_scatter_s32index_s32 */, 116861 /* vq16bSs*q4iq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116876 /* svst1h_scatter_s32index_u32 */, 116904 /* vq16bUs*q4iq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116920 /* svst1h_scatter_s32offset_s32 */, 116861 /* vq16bSs*q4iq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116949 /* svst1h_scatter_s32offset_u32 */, 116904 /* vq16bUs*q4iq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116978 /* svst1h_scatter_s64index_s64 */, 117006 /* vq16bSs*q2Wiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117023 /* svst1h_scatter_s64index_u64 */, 117051 /* vq16bUs*q2Wiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117069 /* svst1h_scatter_s64offset_s64 */, 117006 /* vq16bSs*q2Wiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117098 /* svst1h_scatter_s64offset_u64 */, 117051 /* vq16bUs*q2Wiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117127 /* svst1h_scatter_u32base_index_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117160 /* svst1h_scatter_u32base_index_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117193 /* svst1h_scatter_u32base_offset_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117227 /* svst1h_scatter_u32base_offset_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117261 /* svst1h_scatter_u32base_s32 */, 113425 /* vq16bq4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117288 /* svst1h_scatter_u32base_u32 */, 113464 /* vq16bq4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117315 /* svst1h_scatter_u32index_s32 */, 117343 /* vq16bSs*q4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117359 /* svst1h_scatter_u32index_u32 */, 117387 /* vq16bUs*q4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117404 /* svst1h_scatter_u32offset_s32 */, 117343 /* vq16bSs*q4Uiq4i */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117433 /* svst1h_scatter_u32offset_u32 */, 117387 /* vq16bUs*q4Uiq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117462 /* svst1h_scatter_u64base_index_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117495 /* svst1h_scatter_u64base_index_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117528 /* svst1h_scatter_u64base_offset_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117562 /* svst1h_scatter_u64base_offset_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117596 /* svst1h_scatter_u64base_s64 */, 114003 /* vq16bq2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117623 /* svst1h_scatter_u64base_u64 */, 114044 /* vq16bq2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117650 /* svst1h_scatter_u64index_s64 */, 117678 /* vq16bSs*q2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117696 /* svst1h_scatter_u64index_u64 */, 117724 /* vq16bUs*q2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117743 /* svst1h_scatter_u64offset_s64 */, 117678 /* vq16bSs*q2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117772 /* svst1h_scatter_u64offset_u64 */, 117724 /* vq16bUs*q2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117801 /* svst1h_u32 */, 117812 /* vq16bUs*q4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117825 /* svst1h_u64 */, 117836 /* vq16bUs*q2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117850 /* svst1h_vnum_s32 */, 117866 /* vq16bSs*Wiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117880 /* svst1h_vnum_s64 */, 117896 /* vq16bSs*Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117911 /* svst1h_vnum_u32 */, 117927 /* vq16bUs*Wiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117942 /* svst1h_vnum_u64 */, 117958 /* vq16bUs*Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117974 /* svst1q_scatter_s64index_bf16 */, 118003 /* vq16by*q2Wiq8y */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118018 /* svst1q_scatter_s64index_f16 */, 118046 /* vq16bh*q2Wiq8h */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118061 /* svst1q_scatter_s64index_f32 */, 118089 /* vq16bf*q2Wiq4f */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118104 /* svst1q_scatter_s64index_f64 */, 112929 /* vq16bd*q2Wiq2d */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118132 /* svst1q_scatter_s64index_s16 */, 118160 /* vq16bSs*q2Wiq8s */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118176 /* svst1q_scatter_s64index_s32 */, 118204 /* vq16bSi*q2Wiq4i */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118220 /* svst1q_scatter_s64index_s64 */, 112971 /* vq16bSWi*q2Wiq2Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118248 /* svst1q_scatter_s64index_u16 */, 118276 /* vq16bUs*q2Wiq8Us */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118293 /* svst1q_scatter_s64index_u32 */, 118321 /* vq16bUi*q2Wiq4Ui */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118338 /* svst1q_scatter_s64index_u64 */, 113016 /* vq16bUWi*q2Wiq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118366 /* svst1q_scatter_s64offset_bf16 */, 118003 /* vq16by*q2Wiq8y */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118396 /* svst1q_scatter_s64offset_f16 */, 118046 /* vq16bh*q2Wiq8h */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118425 /* svst1q_scatter_s64offset_f32 */, 118089 /* vq16bf*q2Wiq4f */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118454 /* svst1q_scatter_s64offset_f64 */, 112929 /* vq16bd*q2Wiq2d */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118483 /* svst1q_scatter_s64offset_mf8 */, 118512 /* vq16bm*q2Wiq16m */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118528 /* svst1q_scatter_s64offset_s16 */, 118160 /* vq16bSs*q2Wiq8s */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118557 /* svst1q_scatter_s64offset_s32 */, 118204 /* vq16bSi*q2Wiq4i */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118586 /* svst1q_scatter_s64offset_s64 */, 112971 /* vq16bSWi*q2Wiq2Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118615 /* svst1q_scatter_s64offset_s8 */, 118643 /* vq16bSc*q2Wiq16Sc */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118661 /* svst1q_scatter_s64offset_u16 */, 118276 /* vq16bUs*q2Wiq8Us */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118690 /* svst1q_scatter_s64offset_u32 */, 118321 /* vq16bUi*q2Wiq4Ui */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118719 /* svst1q_scatter_s64offset_u64 */, 113016 /* vq16bUWi*q2Wiq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118748 /* svst1q_scatter_s64offset_u8 */, 118776 /* vq16bUc*q2Wiq16Uc */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118794 /* svst1q_scatter_u64base_bf16 */, 118822 /* vq16bq2UWiq8y */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118836 /* svst1q_scatter_u64base_f16 */, 118863 /* vq16bq2UWiq8h */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118877 /* svst1q_scatter_u64base_f32 */, 118904 /* vq16bq2UWiq4f */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118918 /* svst1q_scatter_u64base_f64 */, 113717 /* vq16bq2UWiq2d */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118945 /* svst1q_scatter_u64base_index_bf16 */, 118979 /* vq16bq2UWiWiq8y */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{118995 /* svst1q_scatter_u64base_index_f16 */, 119028 /* vq16bq2UWiWiq8h */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119044 /* svst1q_scatter_u64base_index_f32 */, 119077 /* vq16bq2UWiWiq4f */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119093 /* svst1q_scatter_u64base_index_f64 */, 113763 /* vq16bq2UWiWiq2d */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119126 /* svst1q_scatter_u64base_index_s16 */, 119159 /* vq16bq2UWiWiq8s */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119175 /* svst1q_scatter_u64base_index_s32 */, 119208 /* vq16bq2UWiWiq4i */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119224 /* svst1q_scatter_u64base_index_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119257 /* svst1q_scatter_u64base_index_u16 */, 119290 /* vq16bq2UWiWiq8Us */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119307 /* svst1q_scatter_u64base_index_u32 */, 119340 /* vq16bq2UWiWiq4Ui */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119357 /* svst1q_scatter_u64base_index_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119390 /* svst1q_scatter_u64base_mf8 */, 119417 /* vq16bq2UWiq16m */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119432 /* svst1q_scatter_u64base_offset_bf16 */, 118979 /* vq16bq2UWiWiq8y */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119467 /* svst1q_scatter_u64base_offset_f16 */, 119028 /* vq16bq2UWiWiq8h */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119501 /* svst1q_scatter_u64base_offset_f32 */, 119077 /* vq16bq2UWiWiq4f */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119535 /* svst1q_scatter_u64base_offset_f64 */, 113763 /* vq16bq2UWiWiq2d */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119569 /* svst1q_scatter_u64base_offset_mf8 */, 119603 /* vq16bq2UWiWiq16m */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119620 /* svst1q_scatter_u64base_offset_s16 */, 119159 /* vq16bq2UWiWiq8s */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119654 /* svst1q_scatter_u64base_offset_s32 */, 119208 /* vq16bq2UWiWiq4i */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119688 /* svst1q_scatter_u64base_offset_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119722 /* svst1q_scatter_u64base_offset_s8 */, 119755 /* vq16bq2UWiWiq16Sc */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119773 /* svst1q_scatter_u64base_offset_u16 */, 119290 /* vq16bq2UWiWiq8Us */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119807 /* svst1q_scatter_u64base_offset_u32 */, 119340 /* vq16bq2UWiWiq4Ui */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119841 /* svst1q_scatter_u64base_offset_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119875 /* svst1q_scatter_u64base_offset_u8 */, 119908 /* vq16bq2UWiWiq16Uc */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119926 /* svst1q_scatter_u64base_s16 */, 119953 /* vq16bq2UWiq8s */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{119967 /* svst1q_scatter_u64base_s32 */, 119994 /* vq16bq2UWiq4i */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120008 /* svst1q_scatter_u64base_s64 */, 114003 /* vq16bq2UWiq2Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120035 /* svst1q_scatter_u64base_s8 */, 120061 /* vq16bq2UWiq16Sc */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120077 /* svst1q_scatter_u64base_u16 */, 120104 /* vq16bq2UWiq8Us */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120119 /* svst1q_scatter_u64base_u32 */, 120146 /* vq16bq2UWiq4Ui */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120161 /* svst1q_scatter_u64base_u64 */, 114044 /* vq16bq2UWiq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120188 /* svst1q_scatter_u64base_u8 */, 120214 /* vq16bq2UWiq16Uc */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120230 /* svst1q_scatter_u64index_bf16 */, 120259 /* vq16by*q2UWiq8y */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120275 /* svst1q_scatter_u64index_f16 */, 120303 /* vq16bh*q2UWiq8h */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120319 /* svst1q_scatter_u64index_f32 */, 120347 /* vq16bf*q2UWiq4f */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120363 /* svst1q_scatter_u64index_f64 */, 114087 /* vq16bd*q2UWiq2d */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120391 /* svst1q_scatter_u64index_s16 */, 120419 /* vq16bSs*q2UWiq8s */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120436 /* svst1q_scatter_u64index_s32 */, 120464 /* vq16bSi*q2UWiq4i */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120481 /* svst1q_scatter_u64index_s64 */, 114130 /* vq16bSWi*q2UWiq2Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120509 /* svst1q_scatter_u64index_u16 */, 120537 /* vq16bUs*q2UWiq8Us */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120555 /* svst1q_scatter_u64index_u32 */, 120583 /* vq16bUi*q2UWiq4Ui */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120601 /* svst1q_scatter_u64index_u64 */, 114176 /* vq16bUWi*q2UWiq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120629 /* svst1q_scatter_u64offset_bf16 */, 120259 /* vq16by*q2UWiq8y */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120659 /* svst1q_scatter_u64offset_f16 */, 120303 /* vq16bh*q2UWiq8h */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120688 /* svst1q_scatter_u64offset_f32 */, 120347 /* vq16bf*q2UWiq4f */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120717 /* svst1q_scatter_u64offset_f64 */, 114087 /* vq16bd*q2UWiq2d */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120746 /* svst1q_scatter_u64offset_mf8 */, 120775 /* vq16bm*q2UWiq16m */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120792 /* svst1q_scatter_u64offset_s16 */, 120419 /* vq16bSs*q2UWiq8s */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120821 /* svst1q_scatter_u64offset_s32 */, 120464 /* vq16bSi*q2UWiq4i */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120850 /* svst1q_scatter_u64offset_s64 */, 114130 /* vq16bSWi*q2UWiq2Wi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120879 /* svst1q_scatter_u64offset_s8 */, 120907 /* vq16bSc*q2UWiq16Sc */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120926 /* svst1q_scatter_u64offset_u16 */, 120537 /* vq16bUs*q2UWiq8Us */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120955 /* svst1q_scatter_u64offset_u32 */, 120583 /* vq16bUi*q2UWiq4Ui */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{120984 /* svst1q_scatter_u64offset_u64 */, 114176 /* vq16bUWi*q2UWiq2UWi */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121013 /* svst1q_scatter_u64offset_u8 */, 121041 /* vq16bUc*q2UWiq16Uc */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121060 /* svst1w_s64 */, 121071 /* vq16bSi*q2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121084 /* svst1w_scatter_s64index_s64 */, 121112 /* vq16bSi*q2Wiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121129 /* svst1w_scatter_s64index_u64 */, 121157 /* vq16bUi*q2Wiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121175 /* svst1w_scatter_s64offset_s64 */, 121112 /* vq16bSi*q2Wiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121204 /* svst1w_scatter_s64offset_u64 */, 121157 /* vq16bUi*q2Wiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121233 /* svst1w_scatter_u64base_index_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121266 /* svst1w_scatter_u64base_index_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121299 /* svst1w_scatter_u64base_offset_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121333 /* svst1w_scatter_u64base_offset_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121367 /* svst1w_scatter_u64base_s64 */, 114003 /* vq16bq2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121394 /* svst1w_scatter_u64base_u64 */, 114044 /* vq16bq2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121421 /* svst1w_scatter_u64index_s64 */, 121449 /* vq16bSi*q2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121467 /* svst1w_scatter_u64index_u64 */, 121495 /* vq16bUi*q2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121514 /* svst1w_scatter_u64offset_s64 */, 121449 /* vq16bSi*q2UWiq2Wi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121543 /* svst1w_scatter_u64offset_u64 */, 121495 /* vq16bUi*q2UWiq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121572 /* svst1w_u64 */, 121583 /* vq16bUi*q2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121597 /* svst1w_vnum_s64 */, 121613 /* vq16bSi*Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121628 /* svst1w_vnum_u64 */, 121644 /* vq16bUi*Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121660 /* svst1wq_f32 */, 112219 /* vq16bf*q4f */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121672 /* svst1wq_s32 */, 112488 /* vq16bSi*q4i */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121684 /* svst1wq_u32 */, 114363 /* vq16bUi*q4Ui */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121696 /* svst1wq_vnum_f32 */, 114767 /* vq16bf*Wiq4f */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121713 /* svst1wq_vnum_s32 */, 115120 /* vq16bSi*Wiq4i */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121730 /* svst1wq_vnum_u32 */, 115491 /* vq16bUi*Wiq4Ui */, 1 /* n */, 41701 /* sve,sve2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121747 /* svst2_bf16 */, 121758 /* vq16by*q16y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121770 /* svst2_f16 */, 121780 /* vq16bh*q16h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121792 /* svst2_f32 */, 121802 /* vq16bf*q8f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121813 /* svst2_f64 */, 121823 /* vq16bd*q4d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121834 /* svst2_mf8 */, 121844 /* vq16bm*q32m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121856 /* svst2_s16 */, 121866 /* vq16bSs*q16s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121879 /* svst2_s32 */, 121889 /* vq16bSi*q8i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121901 /* svst2_s64 */, 121911 /* vq16bSWi*q4Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121925 /* svst2_s8 */, 121934 /* vq16bSc*q32Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121948 /* svst2_u16 */, 121958 /* vq16bUs*q16Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121972 /* svst2_u32 */, 121982 /* vq16bUi*q8Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{121995 /* svst2_u64 */, 122005 /* vq16bUWi*q4UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122020 /* svst2_u8 */, 122029 /* vq16bUc*q32Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122043 /* svst2_vnum_bf16 */, 122059 /* vq16by*Wiq16y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122073 /* svst2_vnum_f16 */, 122088 /* vq16bh*Wiq16h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122102 /* svst2_vnum_f32 */, 122117 /* vq16bf*Wiq8f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122130 /* svst2_vnum_f64 */, 122145 /* vq16bd*Wiq4d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122158 /* svst2_vnum_mf8 */, 122173 /* vq16bm*Wiq32m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122187 /* svst2_vnum_s16 */, 122202 /* vq16bSs*Wiq16s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122217 /* svst2_vnum_s32 */, 122232 /* vq16bSi*Wiq8i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122246 /* svst2_vnum_s64 */, 122261 /* vq16bSWi*Wiq4Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122277 /* svst2_vnum_s8 */, 122291 /* vq16bSc*Wiq32Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122307 /* svst2_vnum_u16 */, 122322 /* vq16bUs*Wiq16Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122338 /* svst2_vnum_u32 */, 122353 /* vq16bUi*Wiq8Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122368 /* svst2_vnum_u64 */, 122383 /* vq16bUWi*Wiq4UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122400 /* svst2_vnum_u8 */, 122414 /* vq16bUc*Wiq32Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122430 /* svst2q_bf16 */, 122442 /* vq16byC*q16y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122455 /* svst2q_f16 */, 122466 /* vq16bhC*q16h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122479 /* svst2q_f32 */, 122490 /* vq16bfC*q8f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122502 /* svst2q_f64 */, 122513 /* vq16bdC*q4d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122525 /* svst2q_mf8 */, 122536 /* vq16bmC*q32m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122549 /* svst2q_s16 */, 122560 /* vq16bSsC*q16s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122574 /* svst2q_s32 */, 122585 /* vq16bSiC*q8i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122598 /* svst2q_s64 */, 122609 /* vq16bSWiC*q4Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122624 /* svst2q_s8 */, 122634 /* vq16bScC*q32Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122649 /* svst2q_u16 */, 122660 /* vq16bUsC*q16Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122675 /* svst2q_u32 */, 122686 /* vq16bUiC*q8Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122700 /* svst2q_u64 */, 122711 /* vq16bUWiC*q4UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122727 /* svst2q_u8 */, 122737 /* vq16bUcC*q32Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122752 /* svst2q_vnum_bf16 */, 122769 /* vq16byC*Wiq16y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122784 /* svst2q_vnum_f16 */, 122800 /* vq16bhC*Wiq16h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122815 /* svst2q_vnum_f32 */, 122831 /* vq16bfC*Wiq8f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122845 /* svst2q_vnum_f64 */, 122861 /* vq16bdC*Wiq4d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122875 /* svst2q_vnum_mf8 */, 122891 /* vq16bmC*Wiq32m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122906 /* svst2q_vnum_s16 */, 122922 /* vq16bSsC*Wiq16s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122938 /* svst2q_vnum_s32 */, 122954 /* vq16bSiC*Wiq8i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{122969 /* svst2q_vnum_s64 */, 122985 /* vq16bSWiC*Wiq4Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123002 /* svst2q_vnum_s8 */, 123017 /* vq16bScC*Wiq32Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123034 /* svst2q_vnum_u16 */, 123050 /* vq16bUsC*Wiq16Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123067 /* svst2q_vnum_u32 */, 123083 /* vq16bUiC*Wiq8Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123099 /* svst2q_vnum_u64 */, 123115 /* vq16bUWiC*Wiq4UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123133 /* svst2q_vnum_u8 */, 123148 /* vq16bUcC*Wiq32Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123165 /* svst3_bf16 */, 123176 /* vq16by*q24y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123188 /* svst3_f16 */, 123198 /* vq16bh*q24h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123210 /* svst3_f32 */, 123220 /* vq16bf*q12f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123232 /* svst3_f64 */, 123242 /* vq16bd*q6d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123253 /* svst3_mf8 */, 123263 /* vq16bm*q48m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123275 /* svst3_s16 */, 123285 /* vq16bSs*q24s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123298 /* svst3_s32 */, 123308 /* vq16bSi*q12i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123321 /* svst3_s64 */, 123331 /* vq16bSWi*q6Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123345 /* svst3_s8 */, 123354 /* vq16bSc*q48Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123368 /* svst3_u16 */, 123378 /* vq16bUs*q24Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123392 /* svst3_u32 */, 123402 /* vq16bUi*q12Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123416 /* svst3_u64 */, 123426 /* vq16bUWi*q6UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123441 /* svst3_u8 */, 123450 /* vq16bUc*q48Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123464 /* svst3_vnum_bf16 */, 123480 /* vq16by*Wiq24y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123494 /* svst3_vnum_f16 */, 123509 /* vq16bh*Wiq24h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123523 /* svst3_vnum_f32 */, 123538 /* vq16bf*Wiq12f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123552 /* svst3_vnum_f64 */, 123567 /* vq16bd*Wiq6d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123580 /* svst3_vnum_mf8 */, 123595 /* vq16bm*Wiq48m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123609 /* svst3_vnum_s16 */, 123624 /* vq16bSs*Wiq24s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123639 /* svst3_vnum_s32 */, 123654 /* vq16bSi*Wiq12i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123669 /* svst3_vnum_s64 */, 123684 /* vq16bSWi*Wiq6Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123700 /* svst3_vnum_s8 */, 123714 /* vq16bSc*Wiq48Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123730 /* svst3_vnum_u16 */, 123745 /* vq16bUs*Wiq24Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123761 /* svst3_vnum_u32 */, 123776 /* vq16bUi*Wiq12Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123792 /* svst3_vnum_u64 */, 123807 /* vq16bUWi*Wiq6UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123824 /* svst3_vnum_u8 */, 123838 /* vq16bUc*Wiq48Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123854 /* svst3q_bf16 */, 123866 /* vq16byC*q24y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123879 /* svst3q_f16 */, 123890 /* vq16bhC*q24h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123903 /* svst3q_f32 */, 123914 /* vq16bfC*q12f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123927 /* svst3q_f64 */, 123938 /* vq16bdC*q6d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123950 /* svst3q_mf8 */, 123961 /* vq16bmC*q48m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123974 /* svst3q_s16 */, 123985 /* vq16bSsC*q24s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{123999 /* svst3q_s32 */, 124010 /* vq16bSiC*q12i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124024 /* svst3q_s64 */, 124035 /* vq16bSWiC*q6Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124050 /* svst3q_s8 */, 124060 /* vq16bScC*q48Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124075 /* svst3q_u16 */, 124086 /* vq16bUsC*q24Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124101 /* svst3q_u32 */, 124112 /* vq16bUiC*q12Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124127 /* svst3q_u64 */, 124138 /* vq16bUWiC*q6UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124154 /* svst3q_u8 */, 124164 /* vq16bUcC*q48Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124179 /* svst3q_vnum_bf16 */, 124196 /* vq16byC*Wiq24y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124211 /* svst3q_vnum_f16 */, 124227 /* vq16bhC*Wiq24h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124242 /* svst3q_vnum_f32 */, 124258 /* vq16bfC*Wiq12f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124273 /* svst3q_vnum_f64 */, 124289 /* vq16bdC*Wiq6d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124303 /* svst3q_vnum_mf8 */, 124319 /* vq16bmC*Wiq48m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124334 /* svst3q_vnum_s16 */, 124350 /* vq16bSsC*Wiq24s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124366 /* svst3q_vnum_s32 */, 124382 /* vq16bSiC*Wiq12i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124398 /* svst3q_vnum_s64 */, 124414 /* vq16bSWiC*Wiq6Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124431 /* svst3q_vnum_s8 */, 124446 /* vq16bScC*Wiq48Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124463 /* svst3q_vnum_u16 */, 124479 /* vq16bUsC*Wiq24Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124496 /* svst3q_vnum_u32 */, 124512 /* vq16bUiC*Wiq12Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124529 /* svst3q_vnum_u64 */, 124545 /* vq16bUWiC*Wiq6UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124563 /* svst3q_vnum_u8 */, 124578 /* vq16bUcC*Wiq48Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124595 /* svst4_bf16 */, 124606 /* vq16by*q32y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124618 /* svst4_f16 */, 124628 /* vq16bh*q32h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124640 /* svst4_f32 */, 124650 /* vq16bf*q16f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124662 /* svst4_f64 */, 124672 /* vq16bd*q8d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124683 /* svst4_mf8 */, 124693 /* vq16bm*q64m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124705 /* svst4_s16 */, 124715 /* vq16bSs*q32s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124728 /* svst4_s32 */, 124738 /* vq16bSi*q16i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124751 /* svst4_s64 */, 124761 /* vq16bSWi*q8Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124775 /* svst4_s8 */, 124784 /* vq16bSc*q64Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124798 /* svst4_u16 */, 124808 /* vq16bUs*q32Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124822 /* svst4_u32 */, 124832 /* vq16bUi*q16Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124846 /* svst4_u64 */, 124856 /* vq16bUWi*q8UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124871 /* svst4_u8 */, 124880 /* vq16bUc*q64Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124894 /* svst4_vnum_bf16 */, 124910 /* vq16by*Wiq32y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124924 /* svst4_vnum_f16 */, 124939 /* vq16bh*Wiq32h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124953 /* svst4_vnum_f32 */, 124968 /* vq16bf*Wiq16f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{124982 /* svst4_vnum_f64 */, 124997 /* vq16bd*Wiq8d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125010 /* svst4_vnum_mf8 */, 125025 /* vq16bm*Wiq64m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125039 /* svst4_vnum_s16 */, 125054 /* vq16bSs*Wiq32s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125069 /* svst4_vnum_s32 */, 125084 /* vq16bSi*Wiq16i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125099 /* svst4_vnum_s64 */, 125114 /* vq16bSWi*Wiq8Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125130 /* svst4_vnum_s8 */, 125144 /* vq16bSc*Wiq64Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125160 /* svst4_vnum_u16 */, 125175 /* vq16bUs*Wiq32Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125191 /* svst4_vnum_u32 */, 125206 /* vq16bUi*Wiq16Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125222 /* svst4_vnum_u64 */, 125237 /* vq16bUWi*Wiq8UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125254 /* svst4_vnum_u8 */, 125268 /* vq16bUc*Wiq64Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125284 /* svst4q_bf16 */, 125296 /* vq16byC*q32y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125309 /* svst4q_f16 */, 125320 /* vq16bhC*q32h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125333 /* svst4q_f32 */, 125344 /* vq16bfC*q16f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125357 /* svst4q_f64 */, 125368 /* vq16bdC*q8d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125380 /* svst4q_mf8 */, 125391 /* vq16bmC*q64m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125404 /* svst4q_s16 */, 125415 /* vq16bSsC*q32s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125429 /* svst4q_s32 */, 125440 /* vq16bSiC*q16i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125454 /* svst4q_s64 */, 125465 /* vq16bSWiC*q8Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125480 /* svst4q_s8 */, 125490 /* vq16bScC*q64Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125505 /* svst4q_u16 */, 125516 /* vq16bUsC*q32Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125531 /* svst4q_u32 */, 125542 /* vq16bUiC*q16Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125557 /* svst4q_u64 */, 125568 /* vq16bUWiC*q8UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125584 /* svst4q_u8 */, 125594 /* vq16bUcC*q64Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125609 /* svst4q_vnum_bf16 */, 125626 /* vq16byC*Wiq32y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125641 /* svst4q_vnum_f16 */, 125657 /* vq16bhC*Wiq32h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125672 /* svst4q_vnum_f32 */, 125688 /* vq16bfC*Wiq16f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125703 /* svst4q_vnum_f64 */, 125719 /* vq16bdC*Wiq8d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125733 /* svst4q_vnum_mf8 */, 125749 /* vq16bmC*Wiq64m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125764 /* svst4q_vnum_s16 */, 125780 /* vq16bSsC*Wiq32s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125796 /* svst4q_vnum_s32 */, 125812 /* vq16bSiC*Wiq16i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125828 /* svst4q_vnum_s64 */, 125844 /* vq16bSWiC*Wiq8Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125861 /* svst4q_vnum_s8 */, 125876 /* vq16bScC*Wiq64Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125893 /* svst4q_vnum_u16 */, 125909 /* vq16bUsC*Wiq32Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125926 /* svst4q_vnum_u32 */, 125942 /* vq16bUiC*Wiq16Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125959 /* svst4q_vnum_u64 */, 125975 /* vq16bUWiC*Wiq8UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{125993 /* svst4q_vnum_u8 */, 126008 /* vq16bUcC*Wiq64Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126025 /* svstnt1_bf16 */, 112083 /* vq16by*q8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126038 /* svstnt1_bf16_x2 */, 112108 /* vQay*q16y */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126054 /* svstnt1_bf16_x4 */, 112132 /* vQay*q32y */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126070 /* svstnt1_f16 */, 112152 /* vq16bh*q8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126082 /* svstnt1_f16_x2 */, 112176 /* vQah*q16h */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126097 /* svstnt1_f16_x4 */, 112199 /* vQah*q32h */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126112 /* svstnt1_f32 */, 112219 /* vq16bf*q4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126124 /* svstnt1_f32_x2 */, 112243 /* vQaf*q8f */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126139 /* svstnt1_f32_x4 */, 112265 /* vQaf*q16f */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126154 /* svstnt1_f64 */, 112285 /* vq16bd*q2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126166 /* svstnt1_f64_x2 */, 112309 /* vQad*q4d */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126181 /* svstnt1_f64_x4 */, 112331 /* vQad*q8d */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126196 /* svstnt1_mf8 */, 112350 /* vq16bm*q16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126208 /* svstnt1_mf8_x2 */, 112375 /* vQam*q32m */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126223 /* svstnt1_mf8_x4 */, 112398 /* vQam*q64m */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126238 /* svstnt1_s16 */, 112418 /* vq16bSs*q8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126250 /* svstnt1_s16_x2 */, 112443 /* vQaSs*q16s */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126265 /* svstnt1_s16_x4 */, 112467 /* vQaSs*q32s */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126280 /* svstnt1_s32 */, 112488 /* vq16bSi*q4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126292 /* svstnt1_s32_x2 */, 112513 /* vQaSi*q8i */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126307 /* svstnt1_s32_x4 */, 112536 /* vQaSi*q16i */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126322 /* svstnt1_s64 */, 112557 /* vq16bSWi*q2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126334 /* svstnt1_s64_x2 */, 112584 /* vQaSWi*q4Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126349 /* svstnt1_s64_x4 */, 112609 /* vQaSWi*q8Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126364 /* svstnt1_s8 */, 112630 /* vq16bSc*q16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126375 /* svstnt1_s8_x2 */, 112656 /* vQaSc*q32Sc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126389 /* svstnt1_s8_x4 */, 112680 /* vQaSc*q64Sc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126403 /* svstnt1_scatter_s64index_f64 */, 112929 /* vq16bd*q2Wiq2d */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126432 /* svstnt1_scatter_s64index_s64 */, 112971 /* vq16bSWi*q2Wiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126461 /* svstnt1_scatter_s64index_u64 */, 113016 /* vq16bUWi*q2Wiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126490 /* svstnt1_scatter_s64offset_f64 */, 112929 /* vq16bd*q2Wiq2d */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126520 /* svstnt1_scatter_s64offset_s64 */, 112971 /* vq16bSWi*q2Wiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126550 /* svstnt1_scatter_s64offset_u64 */, 113016 /* vq16bUWi*q2Wiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126580 /* svstnt1_scatter_u32base_f32 */, 113145 /* vq16bq4Uiq4f */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126608 /* svstnt1_scatter_u32base_index_f32 */, 113190 /* vq16bq4UiWiq4f */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126642 /* svstnt1_scatter_u32base_index_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126676 /* svstnt1_scatter_u32base_index_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126710 /* svstnt1_scatter_u32base_offset_f32 */, 113190 /* vq16bq4UiWiq4f */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126745 /* svstnt1_scatter_u32base_offset_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126780 /* svstnt1_scatter_u32base_offset_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126815 /* svstnt1_scatter_u32base_s32 */, 113425 /* vq16bq4Uiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126843 /* svstnt1_scatter_u32base_u32 */, 113464 /* vq16bq4Uiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126871 /* svstnt1_scatter_u32offset_f32 */, 113505 /* vq16bf*q4Uiq4f */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126901 /* svstnt1_scatter_u32offset_s32 */, 113547 /* vq16bSi*q4Uiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126931 /* svstnt1_scatter_u32offset_u32 */, 113590 /* vq16bUi*q4Uiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126961 /* svstnt1_scatter_u64base_f64 */, 113717 /* vq16bq2UWiq2d */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{126989 /* svstnt1_scatter_u64base_index_f64 */, 113763 /* vq16bq2UWiWiq2d */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127023 /* svstnt1_scatter_u64base_index_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127057 /* svstnt1_scatter_u64base_index_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127091 /* svstnt1_scatter_u64base_offset_f64 */, 113763 /* vq16bq2UWiWiq2d */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127126 /* svstnt1_scatter_u64base_offset_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127161 /* svstnt1_scatter_u64base_offset_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127196 /* svstnt1_scatter_u64base_s64 */, 114003 /* vq16bq2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127224 /* svstnt1_scatter_u64base_u64 */, 114044 /* vq16bq2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127252 /* svstnt1_scatter_u64index_f64 */, 114087 /* vq16bd*q2UWiq2d */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127281 /* svstnt1_scatter_u64index_s64 */, 114130 /* vq16bSWi*q2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127310 /* svstnt1_scatter_u64index_u64 */, 114176 /* vq16bUWi*q2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127339 /* svstnt1_scatter_u64offset_f64 */, 114087 /* vq16bd*q2UWiq2d */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127369 /* svstnt1_scatter_u64offset_s64 */, 114130 /* vq16bSWi*q2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127399 /* svstnt1_scatter_u64offset_u64 */, 114176 /* vq16bUWi*q2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127429 /* svstnt1_u16 */, 114290 /* vq16bUs*q8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127441 /* svstnt1_u16_x2 */, 114316 /* vQaUs*q16Us */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127456 /* svstnt1_u16_x4 */, 114341 /* vQaUs*q32Us */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127471 /* svstnt1_u32 */, 114363 /* vq16bUi*q4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127483 /* svstnt1_u32_x2 */, 114389 /* vQaUi*q8Ui */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127498 /* svstnt1_u32_x4 */, 114413 /* vQaUi*q16Ui */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127513 /* svstnt1_u64 */, 114435 /* vq16bUWi*q2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127525 /* svstnt1_u64_x2 */, 114463 /* vQaUWi*q4UWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127540 /* svstnt1_u64_x4 */, 114489 /* vQaUWi*q8UWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127555 /* svstnt1_u8 */, 114511 /* vq16bUc*q16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127566 /* svstnt1_u8_x2 */, 114537 /* vQaUc*q32Uc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127580 /* svstnt1_u8_x4 */, 114561 /* vQaUc*q64Uc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127594 /* svstnt1_vnum_bf16 */, 114589 /* vq16by*Wiq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127612 /* svstnt1_vnum_bf16_x2 */, 114621 /* vQay*Wiq16y */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127633 /* svstnt1_vnum_bf16_x4 */, 114652 /* vQay*Wiq32y */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127654 /* svstnt1_vnum_f16 */, 114679 /* vq16bh*Wiq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127671 /* svstnt1_vnum_f16_x2 */, 114710 /* vQah*Wiq16h */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127691 /* svstnt1_vnum_f16_x4 */, 114740 /* vQah*Wiq32h */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127711 /* svstnt1_vnum_f32 */, 114767 /* vq16bf*Wiq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127728 /* svstnt1_vnum_f32_x2 */, 114798 /* vQaf*Wiq8f */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127748 /* svstnt1_vnum_f32_x4 */, 114827 /* vQaf*Wiq16f */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127768 /* svstnt1_vnum_f64 */, 114854 /* vq16bd*Wiq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127785 /* svstnt1_vnum_f64_x2 */, 114885 /* vQad*Wiq4d */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127805 /* svstnt1_vnum_f64_x4 */, 114914 /* vQad*Wiq8d */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127825 /* svstnt1_vnum_mf8 */, 114940 /* vq16bm*Wiq16m */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127842 /* svstnt1_vnum_mf8_x2 */, 114972 /* vQam*Wiq32m */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127862 /* svstnt1_vnum_mf8_x4 */, 115002 /* vQam*Wiq64m */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127882 /* svstnt1_vnum_s16 */, 115029 /* vq16bSs*Wiq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127899 /* svstnt1_vnum_s16_x2 */, 115061 /* vQaSs*Wiq16s */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127919 /* svstnt1_vnum_s16_x4 */, 115092 /* vQaSs*Wiq32s */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127939 /* svstnt1_vnum_s32 */, 115120 /* vq16bSi*Wiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127956 /* svstnt1_vnum_s32_x2 */, 115152 /* vQaSi*Wiq8i */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127976 /* svstnt1_vnum_s32_x4 */, 115182 /* vQaSi*Wiq16i */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{127996 /* svstnt1_vnum_s64 */, 115210 /* vq16bSWi*Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128013 /* svstnt1_vnum_s64_x2 */, 115244 /* vQaSWi*Wiq4Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128033 /* svstnt1_vnum_s64_x4 */, 115276 /* vQaSWi*Wiq8Wi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128053 /* svstnt1_vnum_s8 */, 115304 /* vq16bSc*Wiq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128069 /* svstnt1_vnum_s8_x2 */, 115337 /* vQaSc*Wiq32Sc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128088 /* svstnt1_vnum_s8_x4 */, 115368 /* vQaSc*Wiq64Sc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128107 /* svstnt1_vnum_u16 */, 115397 /* vq16bUs*Wiq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128124 /* svstnt1_vnum_u16_x2 */, 115430 /* vQaUs*Wiq16Us */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128144 /* svstnt1_vnum_u16_x4 */, 115462 /* vQaUs*Wiq32Us */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128164 /* svstnt1_vnum_u32 */, 115491 /* vq16bUi*Wiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128181 /* svstnt1_vnum_u32_x2 */, 115524 /* vQaUi*Wiq8Ui */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128201 /* svstnt1_vnum_u32_x4 */, 115555 /* vQaUi*Wiq16Ui */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128221 /* svstnt1_vnum_u64 */, 115584 /* vq16bUWi*Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128238 /* svstnt1_vnum_u64_x2 */, 115619 /* vQaUWi*Wiq4UWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128258 /* svstnt1_vnum_u64_x4 */, 115652 /* vQaUWi*Wiq8UWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128278 /* svstnt1_vnum_u8 */, 115681 /* vq16bUc*Wiq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128294 /* svstnt1_vnum_u8_x2 */, 115714 /* vQaUc*Wiq32Uc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128313 /* svstnt1_vnum_u8_x4 */, 115745 /* vQaUc*Wiq64Uc */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128332 /* svstnt1b_scatter_s64offset_s64 */, 115947 /* vq16bSc*q2Wiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128363 /* svstnt1b_scatter_s64offset_u64 */, 115993 /* vq16bUc*q2Wiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128394 /* svstnt1b_scatter_u32base_offset_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128430 /* svstnt1b_scatter_u32base_offset_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128466 /* svstnt1b_scatter_u32base_s32 */, 113425 /* vq16bq4Uiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128495 /* svstnt1b_scatter_u32base_u32 */, 113464 /* vq16bq4Uiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128524 /* svstnt1b_scatter_u32offset_s32 */, 116162 /* vq16bSc*q4Uiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128555 /* svstnt1b_scatter_u32offset_u32 */, 116207 /* vq16bUc*q4Uiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128586 /* svstnt1b_scatter_u64base_offset_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128622 /* svstnt1b_scatter_u64base_offset_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128658 /* svstnt1b_scatter_u64base_s64 */, 114003 /* vq16bq2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128687 /* svstnt1b_scatter_u64base_u64 */, 114044 /* vq16bq2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128716 /* svstnt1b_scatter_u64offset_s64 */, 116375 /* vq16bSc*q2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128747 /* svstnt1b_scatter_u64offset_u64 */, 116422 /* vq16bUc*q2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128778 /* svstnt1h_scatter_s64index_s64 */, 117006 /* vq16bSs*q2Wiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128808 /* svstnt1h_scatter_s64index_u64 */, 117051 /* vq16bUs*q2Wiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128838 /* svstnt1h_scatter_s64offset_s64 */, 117006 /* vq16bSs*q2Wiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128869 /* svstnt1h_scatter_s64offset_u64 */, 117051 /* vq16bUs*q2Wiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128900 /* svstnt1h_scatter_u32base_index_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128935 /* svstnt1h_scatter_u32base_index_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{128970 /* svstnt1h_scatter_u32base_offset_s32 */, 113237 /* vq16bq4UiWiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129006 /* svstnt1h_scatter_u32base_offset_u32 */, 113284 /* vq16bq4UiWiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129042 /* svstnt1h_scatter_u32base_s32 */, 113425 /* vq16bq4Uiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129071 /* svstnt1h_scatter_u32base_u32 */, 113464 /* vq16bq4Uiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129100 /* svstnt1h_scatter_u32offset_s32 */, 117343 /* vq16bSs*q4Uiq4i */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129131 /* svstnt1h_scatter_u32offset_u32 */, 117387 /* vq16bUs*q4Uiq4Ui */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129162 /* svstnt1h_scatter_u64base_index_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129197 /* svstnt1h_scatter_u64base_index_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129232 /* svstnt1h_scatter_u64base_offset_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129268 /* svstnt1h_scatter_u64base_offset_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129304 /* svstnt1h_scatter_u64base_s64 */, 114003 /* vq16bq2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129333 /* svstnt1h_scatter_u64base_u64 */, 114044 /* vq16bq2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129362 /* svstnt1h_scatter_u64index_s64 */, 117678 /* vq16bSs*q2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129392 /* svstnt1h_scatter_u64index_u64 */, 117724 /* vq16bUs*q2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129422 /* svstnt1h_scatter_u64offset_s64 */, 117678 /* vq16bSs*q2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129453 /* svstnt1h_scatter_u64offset_u64 */, 117724 /* vq16bUs*q2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129484 /* svstnt1w_scatter_s64index_s64 */, 121112 /* vq16bSi*q2Wiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129514 /* svstnt1w_scatter_s64index_u64 */, 121157 /* vq16bUi*q2Wiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129544 /* svstnt1w_scatter_s64offset_s64 */, 121112 /* vq16bSi*q2Wiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129575 /* svstnt1w_scatter_s64offset_u64 */, 121157 /* vq16bUi*q2Wiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129606 /* svstnt1w_scatter_u64base_index_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129641 /* svstnt1w_scatter_u64base_index_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129676 /* svstnt1w_scatter_u64base_offset_s64 */, 113811 /* vq16bq2UWiWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129712 /* svstnt1w_scatter_u64base_offset_u64 */, 113860 /* vq16bq2UWiWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129748 /* svstnt1w_scatter_u64base_s64 */, 114003 /* vq16bq2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129777 /* svstnt1w_scatter_u64base_u64 */, 114044 /* vq16bq2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129806 /* svstnt1w_scatter_u64index_s64 */, 121449 /* vq16bSi*q2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129836 /* svstnt1w_scatter_u64index_u64 */, 121495 /* vq16bUi*q2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129866 /* svstnt1w_scatter_u64offset_s64 */, 121449 /* vq16bSi*q2UWiq2Wi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129897 /* svstnt1w_scatter_u64offset_u64 */, 121495 /* vq16bUi*q2UWiq2UWi */, 1 /* n */, 35883 /* sve,sve2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129928 /* svsub_bf16_m */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129941 /* svsub_bf16_x */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129954 /* svsub_bf16_z */, 3880 /* q8yq16bq8yq8y */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129967 /* svsub_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129979 /* svsub_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{129991 /* svsub_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130003 /* svsub_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130015 /* svsub_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130027 /* svsub_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130039 /* svsub_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130051 /* svsub_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130063 /* svsub_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130075 /* svsub_n_bf16_m */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130090 /* svsub_n_bf16_x */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130105 /* svsub_n_bf16_z */, 4082 /* q8yq16bq8yy */, 1 /* n */, 3894 /* (sve,sve-b16b16)|(sme,sme2,sve-b16b16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130120 /* svsub_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130134 /* svsub_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130148 /* svsub_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130162 /* svsub_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130176 /* svsub_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130190 /* svsub_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130204 /* svsub_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130218 /* svsub_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130232 /* svsub_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130246 /* svsub_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130260 /* svsub_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130274 /* svsub_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130288 /* svsub_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130302 /* svsub_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130316 /* svsub_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130330 /* svsub_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130344 /* svsub_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130358 /* svsub_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130372 /* svsub_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130385 /* svsub_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130398 /* svsub_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130411 /* svsub_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130425 /* svsub_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130439 /* svsub_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130453 /* svsub_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130467 /* svsub_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130481 /* svsub_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130495 /* svsub_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130509 /* svsub_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130523 /* svsub_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130537 /* svsub_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130550 /* svsub_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130563 /* svsub_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130576 /* svsub_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130588 /* svsub_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130600 /* svsub_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130612 /* svsub_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130624 /* svsub_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130636 /* svsub_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130648 /* svsub_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130660 /* svsub_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130672 /* svsub_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130684 /* svsub_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130695 /* svsub_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130706 /* svsub_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130717 /* svsub_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130729 /* svsub_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130741 /* svsub_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130753 /* svsub_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130765 /* svsub_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130777 /* svsub_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130789 /* svsub_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130801 /* svsub_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130813 /* svsub_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130825 /* svsub_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130836 /* svsub_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130847 /* svsub_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130858 /* svsubhnb_n_s16 */, 5495 /* q16Scq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130873 /* svsubhnb_n_s32 */, 5520 /* q8sq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130888 /* svsubhnb_n_s64 */, 5543 /* q4iq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130903 /* svsubhnb_n_u16 */, 5568 /* q16Ucq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130918 /* svsubhnb_n_u32 */, 5595 /* q8Usq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130933 /* svsubhnb_n_u64 */, 5621 /* q4Uiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130948 /* svsubhnb_s16 */, 5647 /* q16Scq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130961 /* svsubhnb_s32 */, 5672 /* q8sq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130974 /* svsubhnb_s64 */, 5695 /* q4iq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{130987 /* svsubhnb_u16 */, 5720 /* q16Ucq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131000 /* svsubhnb_u32 */, 5747 /* q8Usq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131013 /* svsubhnb_u64 */, 5773 /* q4Uiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131026 /* svsubhnt_n_s16 */, 5803 /* q16Scq16Scq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131041 /* svsubhnt_n_s32 */, 5833 /* q8sq8sq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131056 /* svsubhnt_n_s64 */, 5859 /* q4iq4iq2WiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131071 /* svsubhnt_n_u16 */, 5887 /* q16Ucq16Ucq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131086 /* svsubhnt_n_u32 */, 5919 /* q8Usq8Usq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131101 /* svsubhnt_n_u64 */, 5949 /* q4Uiq4Uiq2UWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131116 /* svsubhnt_s16 */, 5979 /* q16Scq16Scq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131129 /* svsubhnt_s32 */, 6009 /* q8sq8sq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131142 /* svsubhnt_s64 */, 6035 /* q4iq4iq2Wiq2Wi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131155 /* svsubhnt_u16 */, 6063 /* q16Ucq16Ucq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131168 /* svsubhnt_u32 */, 6095 /* q8Usq8Usq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131181 /* svsubhnt_u64 */, 6125 /* q4Uiq4Uiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131194 /* svsublb_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131208 /* svsublb_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131222 /* svsublb_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131236 /* svsublb_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131250 /* svsublb_n_u32 */, 2258 /* q4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131264 /* svsublb_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131278 /* svsublb_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131290 /* svsublb_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131302 /* svsublb_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131314 /* svsublb_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131326 /* svsublb_u32 */, 2405 /* q4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131338 /* svsublb_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131350 /* svsublbt_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131365 /* svsublbt_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131380 /* svsublbt_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131395 /* svsublbt_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131408 /* svsublbt_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131421 /* svsublbt_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131434 /* svsublt_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131448 /* svsublt_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131462 /* svsublt_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131476 /* svsublt_n_u16 */, 2232 /* q8Usq16UcUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131490 /* svsublt_n_u32 */, 2258 /* q4Uiq8UsUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131504 /* svsublt_n_u64 */, 2283 /* q2UWiq4UiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131518 /* svsublt_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131530 /* svsublt_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131542 /* svsublt_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131554 /* svsublt_u16 */, 2378 /* q8Usq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131566 /* svsublt_u32 */, 2405 /* q4Uiq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131578 /* svsublt_u64 */, 2430 /* q2UWiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131590 /* svsubltb_n_s16 */, 2162 /* q8sq16ScSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131605 /* svsubltb_n_s32 */, 2187 /* q4iq8ss */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131620 /* svsubltb_n_s64 */, 2209 /* q2Wiq4ii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131635 /* svsubltb_s16 */, 2307 /* q8sq16Scq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131648 /* svsubltb_s32 */, 2333 /* q4iq8sq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131661 /* svsubltb_s64 */, 2355 /* q2Wiq4iq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131674 /* svsubr_f16_m */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131687 /* svsubr_f16_x */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131700 /* svsubr_f16_z */, 968 /* q8hq16bq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131713 /* svsubr_f32_m */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131726 /* svsubr_f32_x */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131739 /* svsubr_f32_z */, 1026 /* q4fq16bq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131752 /* svsubr_f64_m */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131765 /* svsubr_f64_x */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131778 /* svsubr_f64_z */, 1076 /* q2dq16bq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131791 /* svsubr_n_f16_m */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131806 /* svsubr_n_f16_x */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131821 /* svsubr_n_f16_z */, 1128 /* q8hq16bq8hh */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131836 /* svsubr_n_f32_m */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131851 /* svsubr_n_f32_x */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131866 /* svsubr_n_f32_z */, 1182 /* q4fq16bq4ff */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131881 /* svsubr_n_f64_m */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131896 /* svsubr_n_f64_x */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131911 /* svsubr_n_f64_z */, 1236 /* q2dq16bq2dd */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131926 /* svsubr_n_s16_m */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131941 /* svsubr_n_s16_x */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131956 /* svsubr_n_s16_z */, 1290 /* q8sq16bq8ss */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131971 /* svsubr_n_s32_m */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131986 /* svsubr_n_s32_x */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132001 /* svsubr_n_s32_z */, 1344 /* q4iq16bq4ii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132016 /* svsubr_n_s64_m */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132031 /* svsubr_n_s64_x */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132046 /* svsubr_n_s64_z */, 1398 /* q2Wiq16bq2WiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132061 /* svsubr_n_s8_m */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132075 /* svsubr_n_s8_x */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132089 /* svsubr_n_s8_z */, 1454 /* q16Scq16bq16ScSc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132103 /* svsubr_n_u16_m */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132118 /* svsubr_n_u16_x */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132133 /* svsubr_n_u16_z */, 1511 /* q8Usq16bq8UsUs */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132148 /* svsubr_n_u32_m */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132163 /* svsubr_n_u32_x */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132178 /* svsubr_n_u32_z */, 1568 /* q4Uiq16bq4UiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132193 /* svsubr_n_u64_m */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132208 /* svsubr_n_u64_x */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132223 /* svsubr_n_u64_z */, 1625 /* q2UWiq16bq2UWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132238 /* svsubr_n_u8_m */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132252 /* svsubr_n_u8_x */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132266 /* svsubr_n_u8_z */, 1684 /* q16Ucq16bq16UcUc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132280 /* svsubr_s16_m */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132293 /* svsubr_s16_x */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132306 /* svsubr_s16_z */, 1739 /* q8sq16bq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132319 /* svsubr_s32_m */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132332 /* svsubr_s32_x */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132345 /* svsubr_s32_z */, 1789 /* q4iq16bq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132358 /* svsubr_s64_m */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132371 /* svsubr_s64_x */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132384 /* svsubr_s64_z */, 1839 /* q2Wiq16bq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132397 /* svsubr_s8_m */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132409 /* svsubr_s8_x */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132421 /* svsubr_s8_z */, 1891 /* q16Scq16bq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132433 /* svsubr_u16_m */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132446 /* svsubr_u16_x */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132459 /* svsubr_u16_z */, 1945 /* q8Usq16bq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132472 /* svsubr_u32_m */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132485 /* svsubr_u32_x */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132498 /* svsubr_u32_z */, 1998 /* q4Uiq16bq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132511 /* svsubr_u64_m */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132524 /* svsubr_u64_x */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132537 /* svsubr_u64_z */, 2051 /* q2UWiq16bq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132550 /* svsubr_u8_m */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132562 /* svsubr_u8_x */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132574 /* svsubr_u8_z */, 2106 /* q16Ucq16bq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132586 /* svsubwb_n_s16 */, 7388 /* q8sq8sSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132600 /* svsubwb_n_s32 */, 7411 /* q4iq4is */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132614 /* svsubwb_n_s64 */, 7433 /* q2Wiq2Wii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132628 /* svsubwb_n_u16 */, 7457 /* q8Usq8UsUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132642 /* svsubwb_n_u32 */, 7482 /* q4Uiq4UiUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132656 /* svsubwb_n_u64 */, 7507 /* q2UWiq2UWiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132670 /* svsubwb_s16 */, 7532 /* q8sq8sq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132682 /* svsubwb_s32 */, 7556 /* q4iq4iq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132694 /* svsubwb_s64 */, 7578 /* q2Wiq2Wiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132706 /* svsubwb_u16 */, 7602 /* q8Usq8Usq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132718 /* svsubwb_u32 */, 7628 /* q4Uiq4Uiq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132730 /* svsubwb_u64 */, 7653 /* q2UWiq2UWiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132742 /* svsubwt_n_s16 */, 7388 /* q8sq8sSc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132756 /* svsubwt_n_s32 */, 7411 /* q4iq4is */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132770 /* svsubwt_n_s64 */, 7433 /* q2Wiq2Wii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132784 /* svsubwt_n_u16 */, 7457 /* q8Usq8UsUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132798 /* svsubwt_n_u32 */, 7482 /* q4Uiq4UiUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132812 /* svsubwt_n_u64 */, 7507 /* q2UWiq2UWiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132826 /* svsubwt_s16 */, 7532 /* q8sq8sq16Sc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132838 /* svsubwt_s32 */, 7556 /* q4iq4iq8s */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132850 /* svsubwt_s64 */, 7578 /* q2Wiq2Wiq4i */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132862 /* svsubwt_u16 */, 7602 /* q8Usq8Usq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132874 /* svsubwt_u32 */, 7628 /* q4Uiq4Uiq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132886 /* svsubwt_u64 */, 7653 /* q2UWiq2UWiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132898 /* svsudot_lane_s32 */, 132915 /* q4iq4iq16Scq16UcIUWi */, 1 /* n */, 132936 /* (sve,i8mm)|(sme,i8mm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132958 /* svsudot_n_s32 */, 132972 /* q4iq4iq16ScUc */, 1 /* n */, 132936 /* (sve,i8mm)|(sme,i8mm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132986 /* svsudot_s32 */, 132998 /* q4iq4iq16Scq16Uc */, 1 /* n */, 132936 /* (sve,i8mm)|(sme,i8mm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133015 /* svtbl2_bf16 */, 133027 /* q8yq16yq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133039 /* svtbl2_f16 */, 133050 /* q8hq16hq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133062 /* svtbl2_f32 */, 133073 /* q4fq8fq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133084 /* svtbl2_f64 */, 133095 /* q2dq4dq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133107 /* svtbl2_s16 */, 133118 /* q8sq16sq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133130 /* svtbl2_s32 */, 133141 /* q4iq8iq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133152 /* svtbl2_s64 */, 133163 /* q2Wiq4Wiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133177 /* svtbl2_s8 */, 133187 /* q16Scq32Scq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133203 /* svtbl2_u16 */, 133214 /* q8Usq16Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133228 /* svtbl2_u32 */, 133239 /* q4Uiq8Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133252 /* svtbl2_u64 */, 133263 /* q2UWiq4UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133279 /* svtbl2_u8 */, 133289 /* q16Ucq32Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133305 /* svtbl_bf16 */, 133316 /* q8yq8yq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133327 /* svtbl_f16 */, 133337 /* q8hq8hq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133348 /* svtbl_f32 */, 133358 /* q4fq4fq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133369 /* svtbl_f64 */, 133379 /* q2dq2dq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133391 /* svtbl_s16 */, 133401 /* q8sq8sq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133412 /* svtbl_s32 */, 133422 /* q4iq4iq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133433 /* svtbl_s64 */, 133443 /* q2Wiq2Wiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133457 /* svtbl_s8 */, 133466 /* q16Scq16Scq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133482 /* svtbl_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133492 /* svtbl_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133502 /* svtbl_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133512 /* svtbl_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133521 /* svtblq_bf16 */, 133316 /* q8yq8yq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133533 /* svtblq_f16 */, 133337 /* q8hq8hq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133544 /* svtblq_f32 */, 133358 /* q4fq4fq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133555 /* svtblq_f64 */, 133379 /* q2dq2dq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133566 /* svtblq_mf8 */, 133577 /* q16mq16mq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133591 /* svtblq_s16 */, 133401 /* q8sq8sq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133602 /* svtblq_s32 */, 133422 /* q4iq4iq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133613 /* svtblq_s64 */, 133443 /* q2Wiq2Wiq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133624 /* svtblq_s8 */, 133466 /* q16Scq16Scq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133634 /* svtblq_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133645 /* svtblq_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133656 /* svtblq_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133667 /* svtblq_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133677 /* svtbx_bf16 */, 133688 /* q8yq8yq8yq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133702 /* svtbx_f16 */, 133712 /* q8hq8hq8hq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133726 /* svtbx_f32 */, 133736 /* q4fq4fq4fq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133750 /* svtbx_f64 */, 133760 /* q2dq2dq2dq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133775 /* svtbx_s16 */, 133785 /* q8sq8sq8sq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133799 /* svtbx_s32 */, 133809 /* q4iq4iq4iq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133823 /* svtbx_s64 */, 133833 /* q2Wiq2Wiq2Wiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133851 /* svtbx_s8 */, 133860 /* q16Scq16Scq16Scq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133881 /* svtbx_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133891 /* svtbx_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133901 /* svtbx_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133911 /* svtbx_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133920 /* svtbxq_bf16 */, 133688 /* q8yq8yq8yq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133932 /* svtbxq_f16 */, 133712 /* q8hq8hq8hq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133943 /* svtbxq_f32 */, 133736 /* q4fq4fq4fq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133954 /* svtbxq_f64 */, 133760 /* q2dq2dq2dq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133965 /* svtbxq_mf8 */, 133976 /* q16mq16mq16mq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{133994 /* svtbxq_s16 */, 133785 /* q8sq8sq8sq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134005 /* svtbxq_s32 */, 133809 /* q4iq4iq4iq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134016 /* svtbxq_s64 */, 133833 /* q2Wiq2Wiq2Wiq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134027 /* svtbxq_s8 */, 133860 /* q16Scq16Scq16Scq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134037 /* svtbxq_u16 */, 353 /* q8Usq8Usq8Usq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134048 /* svtbxq_u32 */, 380 /* q4Uiq4Uiq4Uiq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134059 /* svtbxq_u64 */, 407 /* q2UWiq2UWiq2UWiq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134070 /* svtbxq_u8 */, 437 /* q16Ucq16Ucq16Ucq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134080 /* svtmad_f16 */, 33241 /* q8hq8hq8hIUWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134091 /* svtmad_f32 */, 33265 /* q4fq4fq4fIUWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134102 /* svtmad_f64 */, 33289 /* q2dq2dq2dIUWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134113 /* svtrn1_b16 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134124 /* svtrn1_b32 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134135 /* svtrn1_b64 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134146 /* svtrn1_b8 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134156 /* svtrn1_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134178 /* svtrn1_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134189 /* svtrn1_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134200 /* svtrn1_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134211 /* svtrn1_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134222 /* svtrn1_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134233 /* svtrn1_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134244 /* svtrn1_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134254 /* svtrn1_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134265 /* svtrn1_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134276 /* svtrn1_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134287 /* svtrn1_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134297 /* svtrn1q_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134310 /* svtrn1q_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134322 /* svtrn1q_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134334 /* svtrn1q_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134346 /* svtrn1q_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134358 /* svtrn1q_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134370 /* svtrn1q_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134382 /* svtrn1q_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134393 /* svtrn1q_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134405 /* svtrn1q_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134417 /* svtrn1q_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134429 /* svtrn1q_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134440 /* svtrn2_b16 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134451 /* svtrn2_b32 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134462 /* svtrn2_b64 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134473 /* svtrn2_b8 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134483 /* svtrn2_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134495 /* svtrn2_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134506 /* svtrn2_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134517 /* svtrn2_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134528 /* svtrn2_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134539 /* svtrn2_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134550 /* svtrn2_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134561 /* svtrn2_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134571 /* svtrn2_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134582 /* svtrn2_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134593 /* svtrn2_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134604 /* svtrn2_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134614 /* svtrn2q_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134627 /* svtrn2q_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134639 /* svtrn2q_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134651 /* svtrn2q_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134663 /* svtrn2q_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134675 /* svtrn2q_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134687 /* svtrn2q_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134699 /* svtrn2q_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134710 /* svtrn2q_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134722 /* svtrn2q_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134734 /* svtrn2q_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134746 /* svtrn2q_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134757 /* svtsmul_f16 */, 133337 /* q8hq8hq8Us */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134769 /* svtsmul_f32 */, 133358 /* q4fq4fq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134781 /* svtsmul_f64 */, 133379 /* q2dq2dq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134793 /* svtssel_f16 */, 133337 /* q8hq8hq8Us */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134805 /* svtssel_f32 */, 133358 /* q4fq4fq4Ui */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134817 /* svtssel_f64 */, 133379 /* q2dq2dq2UWi */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134829 /* svundef2_b */, 134840 /* q32b */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134845 /* svundef2_bf16 */, 134859 /* q16yv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134865 /* svundef2_f16 */, 134878 /* q16hv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134884 /* svundef2_f32 */, 134897 /* q8fv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134902 /* svundef2_f64 */, 134915 /* q4dv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134920 /* svundef2_mf8 */, 134933 /* q32mv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134939 /* svundef2_s16 */, 134952 /* q16sv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134958 /* svundef2_s32 */, 134971 /* q8iv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134976 /* svundef2_s64 */, 134989 /* q4Wiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134995 /* svundef2_s8 */, 135007 /* q32Scv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135014 /* svundef2_u16 */, 135027 /* q16Usv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135034 /* svundef2_u32 */, 135047 /* q8Uiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135053 /* svundef2_u64 */, 135066 /* q4UWiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135073 /* svundef2_u8 */, 135085 /* q32Ucv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135092 /* svundef3_bf16 */, 135106 /* q24yv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135112 /* svundef3_f16 */, 135125 /* q24hv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135131 /* svundef3_f32 */, 135144 /* q12fv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135150 /* svundef3_f64 */, 135163 /* q6dv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135168 /* svundef3_mf8 */, 135181 /* q48mv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135187 /* svundef3_s16 */, 135200 /* q24sv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135206 /* svundef3_s32 */, 135219 /* q12iv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135225 /* svundef3_s64 */, 135238 /* q6Wiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135244 /* svundef3_s8 */, 135256 /* q48Scv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135263 /* svundef3_u16 */, 135276 /* q24Usv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135283 /* svundef3_u32 */, 135296 /* q12Uiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135303 /* svundef3_u64 */, 135316 /* q6UWiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135323 /* svundef3_u8 */, 135335 /* q48Ucv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135342 /* svundef4_b */, 135353 /* q64b */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135358 /* svundef4_bf16 */, 135372 /* q32yv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135378 /* svundef4_f16 */, 135391 /* q32hv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135397 /* svundef4_f32 */, 135410 /* q16fv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135416 /* svundef4_f64 */, 135429 /* q8dv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135434 /* svundef4_mf8 */, 135447 /* q64mv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135453 /* svundef4_s16 */, 135466 /* q32sv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135472 /* svundef4_s32 */, 135485 /* q16iv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135491 /* svundef4_s64 */, 135504 /* q8Wiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135510 /* svundef4_s8 */, 135522 /* q64Scv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135529 /* svundef4_u16 */, 135542 /* q32Usv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135549 /* svundef4_u32 */, 135562 /* q16Uiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135569 /* svundef4_u64 */, 135582 /* q8UWiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135589 /* svundef4_u8 */, 135601 /* q64Ucv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135608 /* svundef_bf16 */, 135621 /* q8yv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135626 /* svundef_f16 */, 135638 /* q8hv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135643 /* svundef_f32 */, 135655 /* q4fv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135660 /* svundef_f64 */, 135672 /* q2dv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135677 /* svundef_mf8 */, 135689 /* q16mv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135695 /* svundef_s16 */, 135707 /* q8sv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135712 /* svundef_s32 */, 135724 /* q4iv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135729 /* svundef_s64 */, 135741 /* q2Wiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135747 /* svundef_s8 */, 135758 /* q16Scv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135765 /* svundef_u16 */, 135777 /* q8Usv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135783 /* svundef_u32 */, 135795 /* q4Uiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135801 /* svundef_u64 */, 135813 /* q2UWiv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135820 /* svundef_u8 */, 135831 /* q16Ucv */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135838 /* svunpk_s16_s8_x2 */, 135855 /* q16sq16Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135865 /* svunpk_s16_s8_x4 */, 135882 /* q32sq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135892 /* svunpk_s32_s16_x2 */, 135910 /* q8iq8s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135917 /* svunpk_s32_s16_x4 */, 135935 /* q16iq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135944 /* svunpk_s64_s32_x2 */, 135962 /* q4Wiq4i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135970 /* svunpk_s64_s32_x4 */, 135988 /* q8Wiq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{135996 /* svunpk_u16_u8_x2 */, 136013 /* q16Usq16Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136024 /* svunpk_u16_u8_x4 */, 136041 /* q32Usq32Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136052 /* svunpk_u32_u16_x2 */, 136070 /* q8Uiq8Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136079 /* svunpk_u32_u16_x4 */, 136097 /* q16Uiq16Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136108 /* svunpk_u64_u32_x2 */, 136126 /* q4UWiq4Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136136 /* svunpk_u64_u32_x4 */, 136154 /* q8UWiq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136164 /* svunpkhi_b */, 101488 /* q16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136175 /* svunpkhi_s16 */, 80330 /* q8sq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136188 /* svunpkhi_s32 */, 80351 /* q4iq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136201 /* svunpkhi_s64 */, 80370 /* q2Wiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136214 /* svunpkhi_u16 */, 80390 /* q8Usq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136227 /* svunpkhi_u32 */, 80412 /* q4Uiq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136240 /* svunpkhi_u64 */, 80433 /* q2UWiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136253 /* svunpklo_b */, 101488 /* q16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136264 /* svunpklo_s16 */, 80330 /* q8sq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136277 /* svunpklo_s32 */, 80351 /* q4iq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136290 /* svunpklo_s64 */, 80370 /* q2Wiq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136303 /* svunpklo_u16 */, 80390 /* q8Usq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136316 /* svunpklo_u32 */, 80412 /* q4Uiq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136329 /* svunpklo_u64 */, 80433 /* q2UWiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136342 /* svuqadd_n_s16_m */, 10303 /* q8sq16bq8sUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136358 /* svuqadd_n_s16_x */, 10303 /* q8sq16bq8sUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136374 /* svuqadd_n_s16_z */, 10303 /* q8sq16bq8sUs */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136390 /* svuqadd_n_s32_m */, 10358 /* q4iq16bq4iUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136406 /* svuqadd_n_s32_x */, 10358 /* q4iq16bq4iUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136422 /* svuqadd_n_s32_z */, 10358 /* q4iq16bq4iUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136438 /* svuqadd_n_s64_m */, 10413 /* q2Wiq16bq2WiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136454 /* svuqadd_n_s64_x */, 10413 /* q2Wiq16bq2WiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136470 /* svuqadd_n_s64_z */, 10413 /* q2Wiq16bq2WiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136486 /* svuqadd_n_s8_m */, 10470 /* q16Scq16bq16ScUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136501 /* svuqadd_n_s8_x */, 10470 /* q16Scq16bq16ScUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136516 /* svuqadd_n_s8_z */, 10470 /* q16Scq16bq16ScUc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136531 /* svuqadd_s16_m */, 10525 /* q8sq16bq8sq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136545 /* svuqadd_s16_x */, 10525 /* q8sq16bq8sq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136559 /* svuqadd_s16_z */, 10525 /* q8sq16bq8sq8Us */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136573 /* svuqadd_s32_m */, 10576 /* q4iq16bq4iq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136587 /* svuqadd_s32_x */, 10576 /* q4iq16bq4iq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136601 /* svuqadd_s32_z */, 10576 /* q4iq16bq4iq4Ui */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136615 /* svuqadd_s64_m */, 10627 /* q2Wiq16bq2Wiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136629 /* svuqadd_s64_x */, 10627 /* q2Wiq16bq2Wiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136643 /* svuqadd_s64_z */, 10627 /* q2Wiq16bq2Wiq2UWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136657 /* svuqadd_s8_m */, 10680 /* q16Scq16bq16Scq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136670 /* svuqadd_s8_x */, 10680 /* q16Scq16bq16Scq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136683 /* svuqadd_s8_z */, 10680 /* q16Scq16bq16Scq16Uc */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136696 /* svusdot_lane_s32 */, 136713 /* q4iq4iq16Ucq16ScIUWi */, 1 /* n */, 132936 /* (sve,i8mm)|(sme,i8mm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136734 /* svusdot_n_s32 */, 136748 /* q4iq4iq16UcSc */, 1 /* n */, 132936 /* (sve,i8mm)|(sme,i8mm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136762 /* svusdot_s32 */, 136774 /* q4iq4iq16Ucq16Sc */, 1 /* n */, 132936 /* (sve,i8mm)|(sme,i8mm) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136791 /* svusmmla_s32 */, 136774 /* q4iq4iq16Ucq16Sc */, 1 /* n */, 80288 /* sve,i8mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136804 /* svuzp1_b16 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136815 /* svuzp1_b32 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136826 /* svuzp1_b64 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136837 /* svuzp1_b8 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136847 /* svuzp1_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136859 /* svuzp1_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136870 /* svuzp1_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136881 /* svuzp1_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136892 /* svuzp1_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136903 /* svuzp1_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136914 /* svuzp1_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136925 /* svuzp1_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136935 /* svuzp1_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136946 /* svuzp1_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136957 /* svuzp1_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136968 /* svuzp1_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136978 /* svuzp1q_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{136991 /* svuzp1q_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137003 /* svuzp1q_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137015 /* svuzp1q_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137027 /* svuzp1q_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137039 /* svuzp1q_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137051 /* svuzp1q_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137063 /* svuzp1q_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137074 /* svuzp1q_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137086 /* svuzp1q_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137098 /* svuzp1q_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137110 /* svuzp1q_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137121 /* svuzp2_b16 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137132 /* svuzp2_b32 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137143 /* svuzp2_b64 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137154 /* svuzp2_b8 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137164 /* svuzp2_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137176 /* svuzp2_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137187 /* svuzp2_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137198 /* svuzp2_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137209 /* svuzp2_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137220 /* svuzp2_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137231 /* svuzp2_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137242 /* svuzp2_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137252 /* svuzp2_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137263 /* svuzp2_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137274 /* svuzp2_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137285 /* svuzp2_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137295 /* svuzp2q_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137308 /* svuzp2q_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137320 /* svuzp2q_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137332 /* svuzp2q_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137344 /* svuzp2q_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137356 /* svuzp2q_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137368 /* svuzp2q_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137380 /* svuzp2q_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137391 /* svuzp2q_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137403 /* svuzp2q_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137415 /* svuzp2q_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137427 /* svuzp2q_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137438 /* svuzp_bf16_x2 */, 137452 /* q16yq16y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137461 /* svuzp_bf16_x4 */, 137475 /* q32yq32y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137484 /* svuzp_f16_x2 */, 137497 /* q16hq16h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137506 /* svuzp_f16_x4 */, 137519 /* q32hq32h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137528 /* svuzp_f32_x2 */, 104242 /* q8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137541 /* svuzp_f32_x4 */, 104264 /* q16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137554 /* svuzp_f64_x2 */, 137567 /* q4dq4d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137574 /* svuzp_f64_x4 */, 137587 /* q8dq8d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137594 /* svuzp_mf8_x2 */, 137607 /* q32mq32m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137616 /* svuzp_mf8_x4 */, 137629 /* q64mq64m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137638 /* svuzp_s16_x2 */, 137651 /* q16sq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137660 /* svuzp_s16_x4 */, 137673 /* q32sq32s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137682 /* svuzp_s32_x2 */, 137695 /* q8iq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137702 /* svuzp_s32_x4 */, 137715 /* q16iq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137724 /* svuzp_s64_x2 */, 137737 /* q4Wiq4Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137746 /* svuzp_s64_x4 */, 137759 /* q8Wiq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137768 /* svuzp_s8_x2 */, 137780 /* q32Scq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137791 /* svuzp_s8_x4 */, 137803 /* q64Scq64Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137814 /* svuzp_u16_x2 */, 137827 /* q16Usq16Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137838 /* svuzp_u16_x4 */, 137851 /* q32Usq32Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137862 /* svuzp_u32_x2 */, 137875 /* q8Uiq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137884 /* svuzp_u32_x4 */, 137897 /* q16Uiq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137908 /* svuzp_u64_x2 */, 137921 /* q4UWiq4UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137932 /* svuzp_u64_x4 */, 137945 /* q8UWiq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137956 /* svuzp_u8_x2 */, 137968 /* q32Ucq32Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{137979 /* svuzp_u8_x4 */, 137991 /* q64Ucq64Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138002 /* svuzpq1_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138015 /* svuzpq1_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138027 /* svuzpq1_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138039 /* svuzpq1_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138051 /* svuzpq1_mf8 */, 138063 /* q16mq16mq16m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138076 /* svuzpq1_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138088 /* svuzpq1_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138100 /* svuzpq1_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138112 /* svuzpq1_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138123 /* svuzpq1_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138135 /* svuzpq1_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138147 /* svuzpq1_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138159 /* svuzpq1_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138170 /* svuzpq2_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138183 /* svuzpq2_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138195 /* svuzpq2_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138207 /* svuzpq2_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138219 /* svuzpq2_mf8 */, 138063 /* q16mq16mq16m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138231 /* svuzpq2_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138243 /* svuzpq2_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138255 /* svuzpq2_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138267 /* svuzpq2_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138278 /* svuzpq2_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138290 /* svuzpq2_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138302 /* svuzpq2_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138314 /* svuzpq2_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138325 /* svuzpq_bf16_x2 */, 137452 /* q16yq16y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138340 /* svuzpq_bf16_x4 */, 137475 /* q32yq32y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138355 /* svuzpq_f16_x2 */, 137497 /* q16hq16h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138369 /* svuzpq_f16_x4 */, 137519 /* q32hq32h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138383 /* svuzpq_f32_x2 */, 104242 /* q8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138397 /* svuzpq_f32_x4 */, 104264 /* q16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138411 /* svuzpq_f64_x2 */, 137567 /* q4dq4d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138425 /* svuzpq_f64_x4 */, 137587 /* q8dq8d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138439 /* svuzpq_mf8_x2 */, 137607 /* q32mq32m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138453 /* svuzpq_mf8_x4 */, 137629 /* q64mq64m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138467 /* svuzpq_s16_x2 */, 137651 /* q16sq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138481 /* svuzpq_s16_x4 */, 137673 /* q32sq32s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138495 /* svuzpq_s32_x2 */, 137695 /* q8iq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138509 /* svuzpq_s32_x4 */, 137715 /* q16iq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138523 /* svuzpq_s64_x2 */, 137737 /* q4Wiq4Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138537 /* svuzpq_s64_x4 */, 137759 /* q8Wiq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138551 /* svuzpq_s8_x2 */, 137780 /* q32Scq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138564 /* svuzpq_s8_x4 */, 137803 /* q64Scq64Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138577 /* svuzpq_u16_x2 */, 137827 /* q16Usq16Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138591 /* svuzpq_u16_x4 */, 137851 /* q32Usq32Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138605 /* svuzpq_u32_x2 */, 137875 /* q8Uiq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138619 /* svuzpq_u32_x4 */, 137897 /* q16Uiq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138633 /* svuzpq_u64_x2 */, 137921 /* q4UWiq4UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138647 /* svuzpq_u64_x4 */, 137945 /* q8UWiq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138661 /* svuzpq_u8_x2 */, 137968 /* q32Ucq32Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138674 /* svuzpq_u8_x4 */, 137991 /* q64Ucq64Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138687 /* svwhilege_b16_s32 */, 138705 /* q16bii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138712 /* svwhilege_b16_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138739 /* svwhilege_b16_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138769 /* svwhilege_b16_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138796 /* svwhilege_b16_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138825 /* svwhilege_b16_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138857 /* svwhilege_b32_s32 */, 138705 /* q16bii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138875 /* svwhilege_b32_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138893 /* svwhilege_b32_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138914 /* svwhilege_b32_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138932 /* svwhilege_b32_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138950 /* svwhilege_b32_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138971 /* svwhilege_b64_s32 */, 138705 /* q16bii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{138989 /* svwhilege_b64_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139007 /* svwhilege_b64_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139028 /* svwhilege_b64_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139046 /* svwhilege_b64_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139064 /* svwhilege_b64_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139085 /* svwhilege_b8_s32 */, 138705 /* q16bii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139102 /* svwhilege_b8_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139119 /* svwhilege_b8_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139139 /* svwhilege_b8_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139156 /* svwhilege_b8_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139173 /* svwhilege_b8_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139193 /* svwhilege_c16_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139222 /* svwhilege_c16_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139253 /* svwhilege_c32_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139271 /* svwhilege_c32_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139289 /* svwhilege_c64_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139307 /* svwhilege_c64_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139325 /* svwhilege_c8_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139342 /* svwhilege_c8_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139359 /* svwhilegt_b16_s32 */, 138705 /* q16bii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139377 /* svwhilegt_b16_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139395 /* svwhilegt_b16_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139416 /* svwhilegt_b16_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139434 /* svwhilegt_b16_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139452 /* svwhilegt_b16_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139473 /* svwhilegt_b32_s32 */, 138705 /* q16bii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139491 /* svwhilegt_b32_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139509 /* svwhilegt_b32_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139530 /* svwhilegt_b32_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139548 /* svwhilegt_b32_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139566 /* svwhilegt_b32_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139587 /* svwhilegt_b64_s32 */, 138705 /* q16bii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139605 /* svwhilegt_b64_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139623 /* svwhilegt_b64_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139644 /* svwhilegt_b64_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139662 /* svwhilegt_b64_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139680 /* svwhilegt_b64_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139701 /* svwhilegt_b8_s32 */, 138705 /* q16bii */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139718 /* svwhilegt_b8_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139735 /* svwhilegt_b8_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139755 /* svwhilegt_b8_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139772 /* svwhilegt_b8_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139789 /* svwhilegt_b8_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139809 /* svwhilegt_c16_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139827 /* svwhilegt_c16_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139845 /* svwhilegt_c32_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139863 /* svwhilegt_c32_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139881 /* svwhilegt_c64_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139899 /* svwhilegt_c64_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139917 /* svwhilegt_c8_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139934 /* svwhilegt_c8_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139951 /* svwhilele_b16_s32 */, 138705 /* q16bii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139969 /* svwhilele_b16_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{139987 /* svwhilele_b16_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140008 /* svwhilele_b16_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140026 /* svwhilele_b16_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140044 /* svwhilele_b16_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140065 /* svwhilele_b32_s32 */, 138705 /* q16bii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140083 /* svwhilele_b32_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140101 /* svwhilele_b32_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140122 /* svwhilele_b32_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140140 /* svwhilele_b32_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140158 /* svwhilele_b32_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140179 /* svwhilele_b64_s32 */, 138705 /* q16bii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140197 /* svwhilele_b64_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140215 /* svwhilele_b64_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140236 /* svwhilele_b64_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140254 /* svwhilele_b64_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140272 /* svwhilele_b64_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140293 /* svwhilele_b8_s32 */, 138705 /* q16bii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140310 /* svwhilele_b8_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140327 /* svwhilele_b8_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140347 /* svwhilele_b8_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140364 /* svwhilele_b8_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140381 /* svwhilele_b8_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140401 /* svwhilele_c16_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140419 /* svwhilele_c16_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140437 /* svwhilele_c32_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140455 /* svwhilele_c32_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140473 /* svwhilele_c64_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140491 /* svwhilele_c64_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140509 /* svwhilele_c8_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140526 /* svwhilele_c8_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140543 /* svwhilelt_b16_s32 */, 138705 /* q16bii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140561 /* svwhilelt_b16_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140579 /* svwhilelt_b16_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140600 /* svwhilelt_b16_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140618 /* svwhilelt_b16_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140636 /* svwhilelt_b16_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140657 /* svwhilelt_b32_s32 */, 138705 /* q16bii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140675 /* svwhilelt_b32_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140693 /* svwhilelt_b32_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140714 /* svwhilelt_b32_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140732 /* svwhilelt_b32_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140750 /* svwhilelt_b32_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140771 /* svwhilelt_b64_s32 */, 138705 /* q16bii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140789 /* svwhilelt_b64_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140807 /* svwhilelt_b64_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140828 /* svwhilelt_b64_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140846 /* svwhilelt_b64_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140864 /* svwhilelt_b64_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140885 /* svwhilelt_b8_s32 */, 138705 /* q16bii */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140902 /* svwhilelt_b8_s64 */, 138730 /* q16bWiWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140919 /* svwhilelt_b8_s64_x2 */, 138760 /* q32bWiWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140939 /* svwhilelt_b8_u32 */, 138787 /* q16bUiUi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140956 /* svwhilelt_b8_u64 */, 138814 /* q16bUWiUWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140973 /* svwhilelt_b8_u64_x2 */, 138846 /* q32bUWiUWi */, 1 /* n */, 12074 /* (sve,(sve2p1|sme2))|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{140993 /* svwhilelt_c16_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141011 /* svwhilelt_c16_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141029 /* svwhilelt_c32_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141047 /* svwhilelt_c32_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141065 /* svwhilelt_c64_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141083 /* svwhilelt_c64_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141101 /* svwhilelt_c8_s64 */, 139211 /* QaWiWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141118 /* svwhilelt_c8_u64 */, 139240 /* QaUWiUWiIUWi */, 1 /* n */, 21687 /* (sve,sve2p1)|(sme,(sve2p1|sme2)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141135 /* svwhilerw_bf16 */, 141150 /* q16byC*yC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141161 /* svwhilerw_f16 */, 141175 /* q16bhC*hC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141186 /* svwhilerw_f32 */, 141200 /* q16bfC*fC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141211 /* svwhilerw_f64 */, 141225 /* q16bdC*dC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141236 /* svwhilerw_s16 */, 141250 /* q16bSsC*SsC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141263 /* svwhilerw_s32 */, 141277 /* q16bSiC*SiC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141290 /* svwhilerw_s64 */, 141304 /* q16bSWiC*SWiC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141319 /* svwhilerw_s8 */, 141332 /* q16bScC*ScC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141345 /* svwhilerw_u16 */, 141359 /* q16bUsC*UsC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141372 /* svwhilerw_u32 */, 141386 /* q16bUiC*UiC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141399 /* svwhilerw_u64 */, 141413 /* q16bUWiC*UWiC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141428 /* svwhilerw_u8 */, 141441 /* q16bUcC*UcC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141454 /* svwhilewr_bf16 */, 141150 /* q16byC*yC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141469 /* svwhilewr_f16 */, 141175 /* q16bhC*hC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141483 /* svwhilewr_f32 */, 141200 /* q16bfC*fC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141497 /* svwhilewr_f64 */, 141225 /* q16bdC*dC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141511 /* svwhilewr_s16 */, 141250 /* q16bSsC*SsC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141525 /* svwhilewr_s32 */, 141277 /* q16bSiC*SiC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141539 /* svwhilewr_s64 */, 141304 /* q16bSWiC*SWiC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141553 /* svwhilewr_s8 */, 141332 /* q16bScC*ScC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141566 /* svwhilewr_u16 */, 141359 /* q16bUsC*UsC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141580 /* svwhilewr_u32 */, 141386 /* q16bUiC*UiC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141594 /* svwhilewr_u64 */, 141413 /* q16bUWiC*UWiC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141608 /* svwhilewr_u8 */, 141441 /* q16bUcC*UcC* */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141621 /* svwrffr */, 141629 /* vq16b */, 1 /* n */, 5434 /* sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141635 /* svxar_n_s16 */, 13758 /* q8sq8sq8sIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141647 /* svxar_n_s32 */, 13783 /* q4iq4iq4iIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141659 /* svxar_n_s64 */, 13808 /* q2Wiq2Wiq2WiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141671 /* svxar_n_s8 */, 13835 /* q16Scq16Scq16ScIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141682 /* svxar_n_u16 */, 13866 /* q8Usq8Usq8UsIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141694 /* svxar_n_u32 */, 13894 /* q4Uiq4Uiq4UiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141706 /* svxar_n_u64 */, 13922 /* q2UWiq2UWiq2UWiIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141718 /* svxar_n_u8 */, 13952 /* q16Ucq16Ucq16UcIUWi */, 1 /* n */, 26 /* (sve,(sve2|sme))|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141729 /* svzip1_b16 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141740 /* svzip1_b32 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141751 /* svzip1_b64 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141762 /* svzip1_b8 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141772 /* svzip1_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141784 /* svzip1_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141795 /* svzip1_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141806 /* svzip1_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141817 /* svzip1_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141828 /* svzip1_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141839 /* svzip1_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141850 /* svzip1_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141860 /* svzip1_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141871 /* svzip1_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141882 /* svzip1_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141893 /* svzip1_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141903 /* svzip1q_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141916 /* svzip1q_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141928 /* svzip1q_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141940 /* svzip1q_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141952 /* svzip1q_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141964 /* svzip1q_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141976 /* svzip1q_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141988 /* svzip1q_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{141999 /* svzip1q_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142011 /* svzip1q_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142023 /* svzip1q_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142035 /* svzip1q_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142046 /* svzip2_b16 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142057 /* svzip2_b32 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142068 /* svzip2_b64 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142079 /* svzip2_b8 */, 12926 /* q16bq16bq16b */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142089 /* svzip2_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142101 /* svzip2_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142112 /* svzip2_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142123 /* svzip2_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142134 /* svzip2_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142145 /* svzip2_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142156 /* svzip2_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142167 /* svzip2_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142177 /* svzip2_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142188 /* svzip2_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142199 /* svzip2_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142210 /* svzip2_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 982 /* sve|sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142220 /* svzip2q_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142233 /* svzip2q_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142245 /* svzip2q_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142257 /* svzip2q_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142269 /* svzip2q_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142281 /* svzip2q_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142293 /* svzip2q_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142305 /* svzip2q_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142316 /* svzip2q_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142328 /* svzip2q_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142340 /* svzip2q_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142352 /* svzip2q_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 43846 /* sve,f64mm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142363 /* svzip_bf16_x2 */, 137452 /* q16yq16y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142377 /* svzip_bf16_x4 */, 137475 /* q32yq32y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142391 /* svzip_f16_x2 */, 137497 /* q16hq16h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142404 /* svzip_f16_x4 */, 137519 /* q32hq32h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142417 /* svzip_f32_x2 */, 104242 /* q8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142430 /* svzip_f32_x4 */, 104264 /* q16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142443 /* svzip_f64_x2 */, 137567 /* q4dq4d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142456 /* svzip_f64_x4 */, 137587 /* q8dq8d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142469 /* svzip_mf8_x2 */, 137607 /* q32mq32m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142482 /* svzip_mf8_x4 */, 137629 /* q64mq64m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142495 /* svzip_s16_x2 */, 137651 /* q16sq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142508 /* svzip_s16_x4 */, 137673 /* q32sq32s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142521 /* svzip_s32_x2 */, 137695 /* q8iq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142534 /* svzip_s32_x4 */, 137715 /* q16iq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142547 /* svzip_s64_x2 */, 137737 /* q4Wiq4Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142560 /* svzip_s64_x4 */, 137759 /* q8Wiq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142573 /* svzip_s8_x2 */, 137780 /* q32Scq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142585 /* svzip_s8_x4 */, 137803 /* q64Scq64Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142597 /* svzip_u16_x2 */, 137827 /* q16Usq16Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142610 /* svzip_u16_x4 */, 137851 /* q32Usq32Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142623 /* svzip_u32_x2 */, 137875 /* q8Uiq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142636 /* svzip_u32_x4 */, 137897 /* q16Uiq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142649 /* svzip_u64_x2 */, 137921 /* q4UWiq4UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142662 /* svzip_u64_x4 */, 137945 /* q8UWiq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142675 /* svzip_u8_x2 */, 137968 /* q32Ucq32Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142687 /* svzip_u8_x4 */, 137991 /* q64Ucq64Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142699 /* svzipq1_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142712 /* svzipq1_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142724 /* svzipq1_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142736 /* svzipq1_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142748 /* svzipq1_mf8 */, 138063 /* q16mq16mq16m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142760 /* svzipq1_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142772 /* svzipq1_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142784 /* svzipq1_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142796 /* svzipq1_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142807 /* svzipq1_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142819 /* svzipq1_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142831 /* svzipq1_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142843 /* svzipq1_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142854 /* svzipq2_bf16 */, 134168 /* q8yq8yq8y */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142867 /* svzipq2_f16 */, 101608 /* q8hq8hq8h */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142879 /* svzipq2_f32 */, 101630 /* q4fq4fq4f */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142891 /* svzipq2_f64 */, 101652 /* q2dq2dq2d */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142903 /* svzipq2_mf8 */, 138063 /* q16mq16mq16m */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142915 /* svzipq2_s16 */, 90241 /* q8sq8sq8s */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142927 /* svzipq2_s32 */, 90301 /* q4iq4iq4i */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142939 /* svzipq2_s64 */, 90361 /* q2Wiq2Wiq2Wi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142951 /* svzipq2_s8 */, 90423 /* q16Scq16Scq16Sc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142962 /* svzipq2_u16 */, 11717 /* q8Usq8Usq8Us */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142974 /* svzipq2_u32 */, 7886 /* q4Uiq4Uiq4Ui */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142986 /* svzipq2_u64 */, 7964 /* q2UWiq2UWiq2UWi */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{142998 /* svzipq2_u8 */, 8393 /* q16Ucq16Ucq16Uc */, 1 /* n */, 6845 /* (sve,(sve2p1|sme2p1))|(sme,(sve2p1|sme2p1)) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143009 /* svzipq_bf16_x2 */, 137452 /* q16yq16y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143024 /* svzipq_bf16_x4 */, 137475 /* q32yq32y */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143039 /* svzipq_f16_x2 */, 137497 /* q16hq16h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143053 /* svzipq_f16_x4 */, 137519 /* q32hq32h */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143067 /* svzipq_f32_x2 */, 104242 /* q8fq8f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143081 /* svzipq_f32_x4 */, 104264 /* q16fq16f */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143095 /* svzipq_f64_x2 */, 137567 /* q4dq4d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143109 /* svzipq_f64_x4 */, 137587 /* q8dq8d */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143123 /* svzipq_mf8_x2 */, 137607 /* q32mq32m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143137 /* svzipq_mf8_x4 */, 137629 /* q64mq64m */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143151 /* svzipq_s16_x2 */, 137651 /* q16sq16s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143165 /* svzipq_s16_x4 */, 137673 /* q32sq32s */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143179 /* svzipq_s32_x2 */, 137695 /* q8iq8i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143193 /* svzipq_s32_x4 */, 137715 /* q16iq16i */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143207 /* svzipq_s64_x2 */, 137737 /* q4Wiq4Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143221 /* svzipq_s64_x4 */, 137759 /* q8Wiq8Wi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143235 /* svzipq_s8_x2 */, 137780 /* q32Scq32Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143248 /* svzipq_s8_x4 */, 137803 /* q64Scq64Sc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143261 /* svzipq_u16_x2 */, 137827 /* q16Usq16Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143275 /* svzipq_u16_x4 */, 137851 /* q32Usq32Us */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143289 /* svzipq_u32_x2 */, 137875 /* q8Uiq8Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143303 /* svzipq_u32_x4 */, 137897 /* q16Uiq16Ui */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143317 /* svzipq_u64_x2 */, 137921 /* q4UWiq4UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143331 /* svzipq_u64_x4 */, 137945 /* q8UWiq8UWi */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143345 /* svzipq_u8_x2 */, 137968 /* q32Ucq32Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143358 /* svzipq_u8_x4 */, 137991 /* q64Ucq64Uc */, 1 /* n */, 4753 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143379 /* reinterpret_s8_s8 */, 101983 /* q16Scq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143397 /* reinterpret_s8_u8 */, 143415 /* q16Scq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143426 /* reinterpret_s8_mf8 */, 143445 /* q16Scq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143455 /* reinterpret_s8_s16 */, 100340 /* q16Scq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143474 /* reinterpret_s8_u16 */, 143493 /* q16Scq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143503 /* reinterpret_s8_s32 */, 143522 /* q16Scq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143531 /* reinterpret_s8_u32 */, 143550 /* q16Scq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143560 /* reinterpret_s8_s64 */, 143579 /* q16Scq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143589 /* reinterpret_s8_u64 */, 143608 /* q16Scq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143619 /* reinterpret_s8_f16 */, 143638 /* q16Scq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143647 /* reinterpret_s8_bf16 */, 143667 /* q16Scq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143676 /* reinterpret_s8_f32 */, 143695 /* q16Scq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143704 /* reinterpret_s8_f64 */, 143723 /* q16Scq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143732 /* reinterpret_u8_s8 */, 143750 /* q16Ucq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143761 /* reinterpret_u8_u8 */, 8578 /* q16Ucq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143779 /* reinterpret_u8_mf8 */, 143798 /* q16Ucq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143808 /* reinterpret_u8_s16 */, 100615 /* q16Ucq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143827 /* reinterpret_u8_u16 */, 100400 /* q16Ucq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143846 /* reinterpret_u8_s32 */, 143865 /* q16Ucq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143874 /* reinterpret_u8_u32 */, 143893 /* q16Ucq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143903 /* reinterpret_u8_s64 */, 143922 /* q16Ucq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143932 /* reinterpret_u8_u64 */, 143951 /* q16Ucq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143962 /* reinterpret_u8_f16 */, 143981 /* q16Ucq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{143990 /* reinterpret_u8_bf16 */, 144010 /* q16Ucq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144019 /* reinterpret_u8_f32 */, 144038 /* q16Ucq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144047 /* reinterpret_u8_f64 */, 144066 /* q16Ucq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144075 /* reinterpret_mf8_s8 */, 144094 /* q16mq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144104 /* reinterpret_mf8_u8 */, 144123 /* q16mq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144133 /* reinterpret_mf8_mf8 */, 144153 /* q16mq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144162 /* reinterpret_mf8_s16 */, 144182 /* q16mq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144190 /* reinterpret_mf8_u16 */, 144210 /* q16mq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144219 /* reinterpret_mf8_s32 */, 144239 /* q16mq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144247 /* reinterpret_mf8_u32 */, 144267 /* q16mq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144276 /* reinterpret_mf8_s64 */, 144296 /* q16mq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144305 /* reinterpret_mf8_u64 */, 144325 /* q16mq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144335 /* reinterpret_mf8_f16 */, 144355 /* q16mq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144363 /* reinterpret_mf8_bf16 */, 144384 /* q16mq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144392 /* reinterpret_mf8_f32 */, 144412 /* q16mq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144420 /* reinterpret_mf8_f64 */, 144440 /* q16mq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144448 /* reinterpret_s16_s8 */, 80330 /* q8sq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144467 /* reinterpret_s16_u8 */, 144486 /* q8sq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144495 /* reinterpret_s16_mf8 */, 144515 /* q8sq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144523 /* reinterpret_s16_s16 */, 101931 /* q8sq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144543 /* reinterpret_s16_u16 */, 144563 /* q8sq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144571 /* reinterpret_s16_s32 */, 100361 /* q8sq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144591 /* reinterpret_s16_u32 */, 144611 /* q8sq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144619 /* reinterpret_s16_s64 */, 144639 /* q8sq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144647 /* reinterpret_s16_u64 */, 144667 /* q8sq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144676 /* reinterpret_s16_f16 */, 144696 /* q8sq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144703 /* reinterpret_s16_bf16 */, 144724 /* q8sq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144731 /* reinterpret_s16_f32 */, 144751 /* q8sq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144758 /* reinterpret_s16_f64 */, 144778 /* q8sq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144785 /* reinterpret_u16_s8 */, 144804 /* q8Usq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144814 /* reinterpret_u16_u8 */, 80390 /* q8Usq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144833 /* reinterpret_u16_mf8 */, 144853 /* q8Usq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144862 /* reinterpret_u16_s16 */, 144882 /* q8Usq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144890 /* reinterpret_u16_u16 */, 102004 /* q8Usq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144910 /* reinterpret_u16_s32 */, 100637 /* q8Usq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144930 /* reinterpret_u16_u32 */, 100422 /* q8Usq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144950 /* reinterpret_u16_s64 */, 144970 /* q8Usq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{144979 /* reinterpret_u16_u64 */, 144999 /* q8Usq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145009 /* reinterpret_u16_f16 */, 145029 /* q8Usq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145037 /* reinterpret_u16_bf16 */, 145058 /* q8Usq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145066 /* reinterpret_u16_f32 */, 145086 /* q8Usq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145094 /* reinterpret_u16_f64 */, 145114 /* q8Usq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145122 /* reinterpret_s32_s8 */, 145141 /* q4iq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145150 /* reinterpret_s32_u8 */, 145169 /* q4iq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145178 /* reinterpret_s32_mf8 */, 145198 /* q4iq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145206 /* reinterpret_s32_s16 */, 80351 /* q4iq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145226 /* reinterpret_s32_u16 */, 145246 /* q4iq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145254 /* reinterpret_s32_s32 */, 101948 /* q4iq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145274 /* reinterpret_s32_u32 */, 145294 /* q4iq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145302 /* reinterpret_s32_s64 */, 100380 /* q4iq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145322 /* reinterpret_s32_u64 */, 145342 /* q4iq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145351 /* reinterpret_s32_f16 */, 145371 /* q4iq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145378 /* reinterpret_s32_bf16 */, 145399 /* q4iq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145406 /* reinterpret_s32_f32 */, 145426 /* q4iq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145433 /* reinterpret_s32_f64 */, 145453 /* q4iq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145460 /* reinterpret_u32_s8 */, 145479 /* q4Uiq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145489 /* reinterpret_u32_u8 */, 145508 /* q4Uiq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145518 /* reinterpret_u32_mf8 */, 145538 /* q4Uiq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145547 /* reinterpret_u32_s16 */, 145567 /* q4Uiq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145575 /* reinterpret_u32_u16 */, 80412 /* q4Uiq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145595 /* reinterpret_u32_s32 */, 145615 /* q4Uiq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145623 /* reinterpret_u32_u32 */, 102023 /* q4Uiq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145643 /* reinterpret_u32_s64 */, 100658 /* q4Uiq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145663 /* reinterpret_u32_u64 */, 100443 /* q4Uiq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145683 /* reinterpret_u32_f16 */, 145703 /* q4Uiq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145711 /* reinterpret_u32_bf16 */, 145732 /* q4Uiq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145740 /* reinterpret_u32_f32 */, 145760 /* q4Uiq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145768 /* reinterpret_u32_f64 */, 145788 /* q4Uiq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145796 /* reinterpret_s64_s8 */, 145815 /* q2Wiq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145825 /* reinterpret_s64_u8 */, 145844 /* q2Wiq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145854 /* reinterpret_s64_mf8 */, 145874 /* q2Wiq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145883 /* reinterpret_s64_s16 */, 145903 /* q2Wiq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145911 /* reinterpret_s64_u16 */, 145931 /* q2Wiq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145940 /* reinterpret_s64_s32 */, 80370 /* q2Wiq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145960 /* reinterpret_s64_u32 */, 145980 /* q2Wiq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{145989 /* reinterpret_s64_s64 */, 101965 /* q2Wiq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146009 /* reinterpret_s64_u64 */, 146029 /* q2Wiq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146039 /* reinterpret_s64_f16 */, 146059 /* q2Wiq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146067 /* reinterpret_s64_bf16 */, 146088 /* q2Wiq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146096 /* reinterpret_s64_f32 */, 146116 /* q2Wiq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146124 /* reinterpret_s64_f64 */, 146144 /* q2Wiq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146152 /* reinterpret_u64_s8 */, 146171 /* q2UWiq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146182 /* reinterpret_u64_u8 */, 146201 /* q2UWiq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146212 /* reinterpret_u64_mf8 */, 146232 /* q2UWiq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146242 /* reinterpret_u64_s16 */, 146262 /* q2UWiq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146271 /* reinterpret_u64_u16 */, 146291 /* q2UWiq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146301 /* reinterpret_u64_s32 */, 146321 /* q2UWiq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146330 /* reinterpret_u64_u32 */, 80433 /* q2UWiq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146350 /* reinterpret_u64_s64 */, 146370 /* q2UWiq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146380 /* reinterpret_u64_u64 */, 102042 /* q2UWiq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146400 /* reinterpret_u64_f16 */, 146420 /* q2UWiq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146429 /* reinterpret_u64_bf16 */, 146450 /* q2UWiq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146459 /* reinterpret_u64_f32 */, 146479 /* q2UWiq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146488 /* reinterpret_u64_f64 */, 146508 /* q2UWiq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146517 /* reinterpret_f16_s8 */, 146536 /* q8hq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146545 /* reinterpret_f16_u8 */, 146564 /* q8hq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146573 /* reinterpret_f16_mf8 */, 146593 /* q8hq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146601 /* reinterpret_f16_s16 */, 146621 /* q8hq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146628 /* reinterpret_f16_u16 */, 32965 /* q8hq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146648 /* reinterpret_f16_s32 */, 146668 /* q8hq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146675 /* reinterpret_f16_u32 */, 146695 /* q8hq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146703 /* reinterpret_f16_s64 */, 146723 /* q8hq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146731 /* reinterpret_f16_u64 */, 146751 /* q8hq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146760 /* reinterpret_f16_f16 */, 101509 /* q8hq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146780 /* reinterpret_f16_bf16 */, 146801 /* q8hq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146808 /* reinterpret_f16_f32 */, 146828 /* q8hq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146835 /* reinterpret_f16_f64 */, 146855 /* q8hq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146862 /* reinterpret_bf16_s8 */, 146882 /* q8yq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146891 /* reinterpret_bf16_u8 */, 146911 /* q8yq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146920 /* reinterpret_bf16_mf8 */, 146941 /* q8yq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146949 /* reinterpret_bf16_s16 */, 146970 /* q8yq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{146977 /* reinterpret_bf16_u16 */, 146998 /* q8yq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147006 /* reinterpret_bf16_s32 */, 147027 /* q8yq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147034 /* reinterpret_bf16_u32 */, 147055 /* q8yq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147063 /* reinterpret_bf16_s64 */, 147084 /* q8yq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147092 /* reinterpret_bf16_u64 */, 147113 /* q8yq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147122 /* reinterpret_bf16_f16 */, 147143 /* q8yq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147150 /* reinterpret_bf16_bf16 */, 101884 /* q8yq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147172 /* reinterpret_bf16_f32 */, 147193 /* q8yq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147200 /* reinterpret_bf16_f64 */, 147221 /* q8yq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147228 /* reinterpret_f32_s8 */, 147247 /* q4fq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147256 /* reinterpret_f32_u8 */, 147275 /* q4fq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147284 /* reinterpret_f32_mf8 */, 147304 /* q4fq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147312 /* reinterpret_f32_s16 */, 147332 /* q4fq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147339 /* reinterpret_f32_u16 */, 147359 /* q4fq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147367 /* reinterpret_f32_s32 */, 147387 /* q4fq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147394 /* reinterpret_f32_u32 */, 33010 /* q4fq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147414 /* reinterpret_f32_s64 */, 147434 /* q4fq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147442 /* reinterpret_f32_u64 */, 147462 /* q4fq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147471 /* reinterpret_f32_f16 */, 147491 /* q4fq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147498 /* reinterpret_f32_bf16 */, 147519 /* q4fq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147526 /* reinterpret_f32_f32 */, 101528 /* q4fq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147546 /* reinterpret_f32_f64 */, 147566 /* q4fq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147573 /* reinterpret_f64_s8 */, 147592 /* q2dq16Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147601 /* reinterpret_f64_u8 */, 147620 /* q2dq16Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147629 /* reinterpret_f64_mf8 */, 147649 /* q2dq16m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147657 /* reinterpret_f64_s16 */, 147677 /* q2dq8s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147684 /* reinterpret_f64_u16 */, 147704 /* q2dq8Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147712 /* reinterpret_f64_s32 */, 147732 /* q2dq4i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147739 /* reinterpret_f64_u32 */, 147759 /* q2dq4Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147767 /* reinterpret_f64_s64 */, 147787 /* q2dq2Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147795 /* reinterpret_f64_u64 */, 33029 /* q2dq2UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147815 /* reinterpret_f64_f16 */, 147835 /* q2dq8h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147842 /* reinterpret_f64_bf16 */, 147863 /* q2dq8y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147870 /* reinterpret_f64_f32 */, 147890 /* q2dq4f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147897 /* reinterpret_f64_f64 */, 101547 /* q2dq2d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147917 /* reinterpret_s8_s8_x2 */, 137780 /* q32Scq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147938 /* reinterpret_s8_u8_x2 */, 147959 /* q32Scq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{147970 /* reinterpret_s8_mf8_x2 */, 147992 /* q32Scq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148002 /* reinterpret_s8_s16_x2 */, 148024 /* q32Scq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148034 /* reinterpret_s8_u16_x2 */, 148056 /* q32Scq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148067 /* reinterpret_s8_s32_x2 */, 148089 /* q32Scq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148098 /* reinterpret_s8_u32_x2 */, 148120 /* q32Scq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148130 /* reinterpret_s8_s64_x2 */, 148152 /* q32Scq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148162 /* reinterpret_s8_u64_x2 */, 148184 /* q32Scq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148195 /* reinterpret_s8_f16_x2 */, 148217 /* q32Scq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148227 /* reinterpret_s8_bf16_x2 */, 148250 /* q32Scq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148260 /* reinterpret_s8_f32_x2 */, 148282 /* q32Scq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148291 /* reinterpret_s8_f64_x2 */, 148313 /* q32Scq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148322 /* reinterpret_u8_s8_x2 */, 148343 /* q32Ucq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148354 /* reinterpret_u8_u8_x2 */, 137968 /* q32Ucq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148375 /* reinterpret_u8_mf8_x2 */, 148397 /* q32Ucq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148407 /* reinterpret_u8_s16_x2 */, 148429 /* q32Ucq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148439 /* reinterpret_u8_u16_x2 */, 148461 /* q32Ucq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148472 /* reinterpret_u8_s32_x2 */, 148494 /* q32Ucq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148503 /* reinterpret_u8_u32_x2 */, 148525 /* q32Ucq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148535 /* reinterpret_u8_s64_x2 */, 148557 /* q32Ucq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148567 /* reinterpret_u8_u64_x2 */, 148589 /* q32Ucq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148600 /* reinterpret_u8_f16_x2 */, 148622 /* q32Ucq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148632 /* reinterpret_u8_bf16_x2 */, 148655 /* q32Ucq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148665 /* reinterpret_u8_f32_x2 */, 148687 /* q32Ucq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148696 /* reinterpret_u8_f64_x2 */, 148718 /* q32Ucq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148727 /* reinterpret_mf8_s8_x2 */, 148749 /* q32mq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148759 /* reinterpret_mf8_u8_x2 */, 148781 /* q32mq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148791 /* reinterpret_mf8_mf8_x2 */, 137607 /* q32mq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148814 /* reinterpret_mf8_s16_x2 */, 148837 /* q32mq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148846 /* reinterpret_mf8_u16_x2 */, 148869 /* q32mq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148879 /* reinterpret_mf8_s32_x2 */, 148902 /* q32mq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148910 /* reinterpret_mf8_u32_x2 */, 148933 /* q32mq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148942 /* reinterpret_mf8_s64_x2 */, 148965 /* q32mq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{148974 /* reinterpret_mf8_u64_x2 */, 148997 /* q32mq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149007 /* reinterpret_mf8_f16_x2 */, 149030 /* q32mq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149039 /* reinterpret_mf8_bf16_x2 */, 149063 /* q32mq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149072 /* reinterpret_mf8_f32_x2 */, 149095 /* q32mq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149103 /* reinterpret_mf8_f64_x2 */, 149126 /* q32mq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149134 /* reinterpret_s16_s8_x2 */, 149156 /* q16sq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149166 /* reinterpret_s16_u8_x2 */, 149188 /* q16sq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149198 /* reinterpret_s16_mf8_x2 */, 149221 /* q16sq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149230 /* reinterpret_s16_s16_x2 */, 137651 /* q16sq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149253 /* reinterpret_s16_u16_x2 */, 149276 /* q16sq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149286 /* reinterpret_s16_s32_x2 */, 149309 /* q16sq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149317 /* reinterpret_s16_u32_x2 */, 149340 /* q16sq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149349 /* reinterpret_s16_s64_x2 */, 149372 /* q16sq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149381 /* reinterpret_s16_u64_x2 */, 149404 /* q16sq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149414 /* reinterpret_s16_f16_x2 */, 149437 /* q16sq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149446 /* reinterpret_s16_bf16_x2 */, 149470 /* q16sq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149479 /* reinterpret_s16_f32_x2 */, 149502 /* q16sq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149510 /* reinterpret_s16_f64_x2 */, 149533 /* q16sq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149541 /* reinterpret_u16_s8_x2 */, 149563 /* q16Usq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149574 /* reinterpret_u16_u8_x2 */, 149596 /* q16Usq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149607 /* reinterpret_u16_mf8_x2 */, 149630 /* q16Usq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149640 /* reinterpret_u16_s16_x2 */, 149663 /* q16Usq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149673 /* reinterpret_u16_u16_x2 */, 137827 /* q16Usq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149696 /* reinterpret_u16_s32_x2 */, 149719 /* q16Usq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149728 /* reinterpret_u16_u32_x2 */, 149751 /* q16Usq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149761 /* reinterpret_u16_s64_x2 */, 149784 /* q16Usq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149794 /* reinterpret_u16_u64_x2 */, 149817 /* q16Usq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149828 /* reinterpret_u16_f16_x2 */, 149851 /* q16Usq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149861 /* reinterpret_u16_bf16_x2 */, 149885 /* q16Usq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149895 /* reinterpret_u16_f32_x2 */, 149918 /* q16Usq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149927 /* reinterpret_u16_f64_x2 */, 149950 /* q16Usq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149959 /* reinterpret_s32_s8_x2 */, 149981 /* q8iq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{149990 /* reinterpret_s32_u8_x2 */, 150012 /* q8iq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150021 /* reinterpret_s32_mf8_x2 */, 150044 /* q8iq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150052 /* reinterpret_s32_s16_x2 */, 150075 /* q8iq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150083 /* reinterpret_s32_u16_x2 */, 150106 /* q8iq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150115 /* reinterpret_s32_s32_x2 */, 137695 /* q8iq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150138 /* reinterpret_s32_u32_x2 */, 150161 /* q8iq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150169 /* reinterpret_s32_s64_x2 */, 150192 /* q8iq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150200 /* reinterpret_s32_u64_x2 */, 150223 /* q8iq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150232 /* reinterpret_s32_f16_x2 */, 150255 /* q8iq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150263 /* reinterpret_s32_bf16_x2 */, 150287 /* q8iq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150295 /* reinterpret_s32_f32_x2 */, 25609 /* q8iq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150318 /* reinterpret_s32_f64_x2 */, 150341 /* q8iq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150348 /* reinterpret_u32_s8_x2 */, 150370 /* q8Uiq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150380 /* reinterpret_u32_u8_x2 */, 150402 /* q8Uiq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150412 /* reinterpret_u32_mf8_x2 */, 150435 /* q8Uiq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150444 /* reinterpret_u32_s16_x2 */, 150467 /* q8Uiq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150476 /* reinterpret_u32_u16_x2 */, 150499 /* q8Uiq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150509 /* reinterpret_u32_s32_x2 */, 150532 /* q8Uiq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150540 /* reinterpret_u32_u32_x2 */, 137875 /* q8Uiq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150563 /* reinterpret_u32_s64_x2 */, 150586 /* q8Uiq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150595 /* reinterpret_u32_u64_x2 */, 150618 /* q8Uiq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150628 /* reinterpret_u32_f16_x2 */, 150651 /* q8Uiq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150660 /* reinterpret_u32_bf16_x2 */, 150684 /* q8Uiq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150693 /* reinterpret_u32_f32_x2 */, 26132 /* q8Uiq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150716 /* reinterpret_u32_f64_x2 */, 150739 /* q8Uiq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150747 /* reinterpret_s64_s8_x2 */, 150769 /* q4Wiq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150779 /* reinterpret_s64_u8_x2 */, 150801 /* q4Wiq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150811 /* reinterpret_s64_mf8_x2 */, 150834 /* q4Wiq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150843 /* reinterpret_s64_s16_x2 */, 150866 /* q4Wiq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150875 /* reinterpret_s64_u16_x2 */, 150898 /* q4Wiq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150908 /* reinterpret_s64_s32_x2 */, 150931 /* q4Wiq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150939 /* reinterpret_s64_u32_x2 */, 150962 /* q4Wiq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150971 /* reinterpret_s64_s64_x2 */, 137737 /* q4Wiq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150994 /* reinterpret_s64_u64_x2 */, 151017 /* q4Wiq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151027 /* reinterpret_s64_f16_x2 */, 151050 /* q4Wiq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151059 /* reinterpret_s64_bf16_x2 */, 151083 /* q4Wiq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151092 /* reinterpret_s64_f32_x2 */, 151115 /* q4Wiq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151123 /* reinterpret_s64_f64_x2 */, 151146 /* q4Wiq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151154 /* reinterpret_u64_s8_x2 */, 151176 /* q4UWiq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151187 /* reinterpret_u64_u8_x2 */, 151209 /* q4UWiq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151220 /* reinterpret_u64_mf8_x2 */, 151243 /* q4UWiq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151253 /* reinterpret_u64_s16_x2 */, 151276 /* q4UWiq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151286 /* reinterpret_u64_u16_x2 */, 151309 /* q4UWiq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151320 /* reinterpret_u64_s32_x2 */, 151343 /* q4UWiq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151352 /* reinterpret_u64_u32_x2 */, 151375 /* q4UWiq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151385 /* reinterpret_u64_s64_x2 */, 151408 /* q4UWiq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151418 /* reinterpret_u64_u64_x2 */, 137921 /* q4UWiq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151441 /* reinterpret_u64_f16_x2 */, 151464 /* q4UWiq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151474 /* reinterpret_u64_bf16_x2 */, 151498 /* q4UWiq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151508 /* reinterpret_u64_f32_x2 */, 151531 /* q4UWiq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151540 /* reinterpret_u64_f64_x2 */, 151563 /* q4UWiq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151572 /* reinterpret_f16_s8_x2 */, 151594 /* q16hq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151604 /* reinterpret_f16_u8_x2 */, 151626 /* q16hq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151636 /* reinterpret_f16_mf8_x2 */, 151659 /* q16hq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151668 /* reinterpret_f16_s16_x2 */, 151691 /* q16hq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151700 /* reinterpret_f16_u16_x2 */, 151723 /* q16hq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151733 /* reinterpret_f16_s32_x2 */, 151756 /* q16hq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151764 /* reinterpret_f16_u32_x2 */, 151787 /* q16hq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151796 /* reinterpret_f16_s64_x2 */, 151819 /* q16hq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151828 /* reinterpret_f16_u64_x2 */, 151851 /* q16hq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151861 /* reinterpret_f16_f16_x2 */, 137497 /* q16hq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151884 /* reinterpret_f16_bf16_x2 */, 151908 /* q16hq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151917 /* reinterpret_f16_f32_x2 */, 151940 /* q16hq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151948 /* reinterpret_f16_f64_x2 */, 151971 /* q16hq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{151979 /* reinterpret_bf16_s8_x2 */, 152002 /* q16yq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152012 /* reinterpret_bf16_u8_x2 */, 152035 /* q16yq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152045 /* reinterpret_bf16_mf8_x2 */, 152069 /* q16yq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152078 /* reinterpret_bf16_s16_x2 */, 152102 /* q16yq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152111 /* reinterpret_bf16_u16_x2 */, 152135 /* q16yq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152145 /* reinterpret_bf16_s32_x2 */, 152169 /* q16yq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152177 /* reinterpret_bf16_u32_x2 */, 152201 /* q16yq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152210 /* reinterpret_bf16_s64_x2 */, 152234 /* q16yq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152243 /* reinterpret_bf16_u64_x2 */, 152267 /* q16yq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152277 /* reinterpret_bf16_f16_x2 */, 152301 /* q16yq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152310 /* reinterpret_bf16_bf16_x2 */, 137452 /* q16yq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152335 /* reinterpret_bf16_f32_x2 */, 152359 /* q16yq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152367 /* reinterpret_bf16_f64_x2 */, 152391 /* q16yq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152399 /* reinterpret_f32_s8_x2 */, 152421 /* q8fq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152430 /* reinterpret_f32_u8_x2 */, 152452 /* q8fq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152461 /* reinterpret_f32_mf8_x2 */, 152484 /* q8fq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152492 /* reinterpret_f32_s16_x2 */, 152515 /* q8fq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152523 /* reinterpret_f32_u16_x2 */, 152546 /* q8fq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152555 /* reinterpret_f32_s32_x2 */, 24515 /* q8fq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152578 /* reinterpret_f32_u32_x2 */, 24715 /* q8fq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152601 /* reinterpret_f32_s64_x2 */, 152624 /* q8fq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152632 /* reinterpret_f32_u64_x2 */, 152655 /* q8fq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152664 /* reinterpret_f32_f16_x2 */, 152687 /* q8fq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152695 /* reinterpret_f32_bf16_x2 */, 152719 /* q8fq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152727 /* reinterpret_f32_f32_x2 */, 104242 /* q8fq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152750 /* reinterpret_f32_f64_x2 */, 152773 /* q8fq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152780 /* reinterpret_f64_s8_x2 */, 152802 /* q4dq32Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152811 /* reinterpret_f64_u8_x2 */, 152833 /* q4dq32Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152842 /* reinterpret_f64_mf8_x2 */, 152865 /* q4dq32m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152873 /* reinterpret_f64_s16_x2 */, 152896 /* q4dq16s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152904 /* reinterpret_f64_u16_x2 */, 152927 /* q4dq16Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152936 /* reinterpret_f64_s32_x2 */, 152959 /* q4dq8i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152966 /* reinterpret_f64_u32_x2 */, 152989 /* q4dq8Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{152997 /* reinterpret_f64_s64_x2 */, 153020 /* q4dq4Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153028 /* reinterpret_f64_u64_x2 */, 153051 /* q4dq4UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153060 /* reinterpret_f64_f16_x2 */, 153083 /* q4dq16h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153091 /* reinterpret_f64_bf16_x2 */, 153115 /* q4dq16y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153123 /* reinterpret_f64_f32_x2 */, 153146 /* q4dq8f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153153 /* reinterpret_f64_f64_x2 */, 137567 /* q4dq4d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153176 /* reinterpret_s8_s8_x3 */, 153197 /* q48Scq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153208 /* reinterpret_s8_u8_x3 */, 153229 /* q48Scq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153240 /* reinterpret_s8_mf8_x3 */, 153262 /* q48Scq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153272 /* reinterpret_s8_s16_x3 */, 153294 /* q48Scq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153304 /* reinterpret_s8_u16_x3 */, 153326 /* q48Scq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153337 /* reinterpret_s8_s32_x3 */, 153359 /* q48Scq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153369 /* reinterpret_s8_u32_x3 */, 153391 /* q48Scq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153402 /* reinterpret_s8_s64_x3 */, 153424 /* q48Scq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153434 /* reinterpret_s8_u64_x3 */, 153456 /* q48Scq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153467 /* reinterpret_s8_f16_x3 */, 153489 /* q48Scq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153499 /* reinterpret_s8_bf16_x3 */, 153522 /* q48Scq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153532 /* reinterpret_s8_f32_x3 */, 153554 /* q48Scq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153564 /* reinterpret_s8_f64_x3 */, 153586 /* q48Scq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153595 /* reinterpret_u8_s8_x3 */, 153616 /* q48Ucq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153627 /* reinterpret_u8_u8_x3 */, 153648 /* q48Ucq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153659 /* reinterpret_u8_mf8_x3 */, 153681 /* q48Ucq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153691 /* reinterpret_u8_s16_x3 */, 153713 /* q48Ucq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153723 /* reinterpret_u8_u16_x3 */, 153745 /* q48Ucq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153756 /* reinterpret_u8_s32_x3 */, 153778 /* q48Ucq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153788 /* reinterpret_u8_u32_x3 */, 153810 /* q48Ucq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153821 /* reinterpret_u8_s64_x3 */, 153843 /* q48Ucq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153853 /* reinterpret_u8_u64_x3 */, 153875 /* q48Ucq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153886 /* reinterpret_u8_f16_x3 */, 153908 /* q48Ucq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153918 /* reinterpret_u8_bf16_x3 */, 153941 /* q48Ucq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153951 /* reinterpret_u8_f32_x3 */, 153973 /* q48Ucq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{153983 /* reinterpret_u8_f64_x3 */, 154005 /* q48Ucq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154014 /* reinterpret_mf8_s8_x3 */, 154036 /* q48mq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154046 /* reinterpret_mf8_u8_x3 */, 154068 /* q48mq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154078 /* reinterpret_mf8_mf8_x3 */, 154101 /* q48mq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154110 /* reinterpret_mf8_s16_x3 */, 154133 /* q48mq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154142 /* reinterpret_mf8_u16_x3 */, 154165 /* q48mq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154175 /* reinterpret_mf8_s32_x3 */, 154198 /* q48mq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154207 /* reinterpret_mf8_u32_x3 */, 154230 /* q48mq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154240 /* reinterpret_mf8_s64_x3 */, 154263 /* q48mq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154272 /* reinterpret_mf8_u64_x3 */, 154295 /* q48mq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154305 /* reinterpret_mf8_f16_x3 */, 154328 /* q48mq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154337 /* reinterpret_mf8_bf16_x3 */, 154361 /* q48mq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154370 /* reinterpret_mf8_f32_x3 */, 154393 /* q48mq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154402 /* reinterpret_mf8_f64_x3 */, 154425 /* q48mq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154433 /* reinterpret_s16_s8_x3 */, 154455 /* q24sq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154465 /* reinterpret_s16_u8_x3 */, 154487 /* q24sq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154497 /* reinterpret_s16_mf8_x3 */, 154520 /* q24sq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154529 /* reinterpret_s16_s16_x3 */, 154552 /* q24sq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154561 /* reinterpret_s16_u16_x3 */, 154584 /* q24sq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154594 /* reinterpret_s16_s32_x3 */, 154617 /* q24sq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154626 /* reinterpret_s16_u32_x3 */, 154649 /* q24sq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154659 /* reinterpret_s16_s64_x3 */, 154682 /* q24sq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154691 /* reinterpret_s16_u64_x3 */, 154714 /* q24sq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154724 /* reinterpret_s16_f16_x3 */, 154747 /* q24sq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154756 /* reinterpret_s16_bf16_x3 */, 154780 /* q24sq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154789 /* reinterpret_s16_f32_x3 */, 154812 /* q24sq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154821 /* reinterpret_s16_f64_x3 */, 154844 /* q24sq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154852 /* reinterpret_u16_s8_x3 */, 154874 /* q24Usq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154885 /* reinterpret_u16_u8_x3 */, 154907 /* q24Usq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154918 /* reinterpret_u16_mf8_x3 */, 154941 /* q24Usq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154951 /* reinterpret_u16_s16_x3 */, 154974 /* q24Usq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154984 /* reinterpret_u16_u16_x3 */, 155007 /* q24Usq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155018 /* reinterpret_u16_s32_x3 */, 155041 /* q24Usq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155051 /* reinterpret_u16_u32_x3 */, 155074 /* q24Usq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155085 /* reinterpret_u16_s64_x3 */, 155108 /* q24Usq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155118 /* reinterpret_u16_u64_x3 */, 155141 /* q24Usq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155152 /* reinterpret_u16_f16_x3 */, 155175 /* q24Usq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155185 /* reinterpret_u16_bf16_x3 */, 155209 /* q24Usq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155219 /* reinterpret_u16_f32_x3 */, 155242 /* q24Usq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155252 /* reinterpret_u16_f64_x3 */, 155275 /* q24Usq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155284 /* reinterpret_s32_s8_x3 */, 155306 /* q12iq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155316 /* reinterpret_s32_u8_x3 */, 155338 /* q12iq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155348 /* reinterpret_s32_mf8_x3 */, 155371 /* q12iq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155380 /* reinterpret_s32_s16_x3 */, 155403 /* q12iq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155412 /* reinterpret_s32_u16_x3 */, 155435 /* q12iq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155445 /* reinterpret_s32_s32_x3 */, 155468 /* q12iq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155477 /* reinterpret_s32_u32_x3 */, 155500 /* q12iq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155510 /* reinterpret_s32_s64_x3 */, 155533 /* q12iq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155542 /* reinterpret_s32_u64_x3 */, 155565 /* q12iq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155575 /* reinterpret_s32_f16_x3 */, 155598 /* q12iq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155607 /* reinterpret_s32_bf16_x3 */, 155631 /* q12iq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155640 /* reinterpret_s32_f32_x3 */, 155663 /* q12iq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155672 /* reinterpret_s32_f64_x3 */, 155695 /* q12iq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155703 /* reinterpret_u32_s8_x3 */, 155725 /* q12Uiq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155736 /* reinterpret_u32_u8_x3 */, 155758 /* q12Uiq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155769 /* reinterpret_u32_mf8_x3 */, 155792 /* q12Uiq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155802 /* reinterpret_u32_s16_x3 */, 155825 /* q12Uiq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155835 /* reinterpret_u32_u16_x3 */, 155858 /* q12Uiq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155869 /* reinterpret_u32_s32_x3 */, 155892 /* q12Uiq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155902 /* reinterpret_u32_u32_x3 */, 155925 /* q12Uiq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155936 /* reinterpret_u32_s64_x3 */, 155959 /* q12Uiq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{155969 /* reinterpret_u32_u64_x3 */, 155992 /* q12Uiq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156003 /* reinterpret_u32_f16_x3 */, 156026 /* q12Uiq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156036 /* reinterpret_u32_bf16_x3 */, 156060 /* q12Uiq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156070 /* reinterpret_u32_f32_x3 */, 156093 /* q12Uiq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156103 /* reinterpret_u32_f64_x3 */, 156126 /* q12Uiq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156135 /* reinterpret_s64_s8_x3 */, 156157 /* q6Wiq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156167 /* reinterpret_s64_u8_x3 */, 156189 /* q6Wiq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156199 /* reinterpret_s64_mf8_x3 */, 156222 /* q6Wiq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156231 /* reinterpret_s64_s16_x3 */, 156254 /* q6Wiq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156263 /* reinterpret_s64_u16_x3 */, 156286 /* q6Wiq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156296 /* reinterpret_s64_s32_x3 */, 156319 /* q6Wiq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156328 /* reinterpret_s64_u32_x3 */, 156351 /* q6Wiq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156361 /* reinterpret_s64_s64_x3 */, 156384 /* q6Wiq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156393 /* reinterpret_s64_u64_x3 */, 156416 /* q6Wiq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156426 /* reinterpret_s64_f16_x3 */, 156449 /* q6Wiq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156458 /* reinterpret_s64_bf16_x3 */, 156482 /* q6Wiq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156491 /* reinterpret_s64_f32_x3 */, 156514 /* q6Wiq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156523 /* reinterpret_s64_f64_x3 */, 156546 /* q6Wiq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156554 /* reinterpret_u64_s8_x3 */, 156576 /* q6UWiq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156587 /* reinterpret_u64_u8_x3 */, 156609 /* q6UWiq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156620 /* reinterpret_u64_mf8_x3 */, 156643 /* q6UWiq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156653 /* reinterpret_u64_s16_x3 */, 156676 /* q6UWiq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156686 /* reinterpret_u64_u16_x3 */, 156709 /* q6UWiq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156720 /* reinterpret_u64_s32_x3 */, 156743 /* q6UWiq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156753 /* reinterpret_u64_u32_x3 */, 156776 /* q6UWiq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156787 /* reinterpret_u64_s64_x3 */, 156810 /* q6UWiq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156820 /* reinterpret_u64_u64_x3 */, 156843 /* q6UWiq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156854 /* reinterpret_u64_f16_x3 */, 156877 /* q6UWiq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156887 /* reinterpret_u64_bf16_x3 */, 156911 /* q6UWiq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156921 /* reinterpret_u64_f32_x3 */, 156944 /* q6UWiq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156954 /* reinterpret_u64_f64_x3 */, 156977 /* q6UWiq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{156986 /* reinterpret_f16_s8_x3 */, 157008 /* q24hq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157018 /* reinterpret_f16_u8_x3 */, 157040 /* q24hq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157050 /* reinterpret_f16_mf8_x3 */, 157073 /* q24hq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157082 /* reinterpret_f16_s16_x3 */, 157105 /* q24hq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157114 /* reinterpret_f16_u16_x3 */, 157137 /* q24hq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157147 /* reinterpret_f16_s32_x3 */, 157170 /* q24hq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157179 /* reinterpret_f16_u32_x3 */, 157202 /* q24hq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157212 /* reinterpret_f16_s64_x3 */, 157235 /* q24hq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157244 /* reinterpret_f16_u64_x3 */, 157267 /* q24hq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157277 /* reinterpret_f16_f16_x3 */, 157300 /* q24hq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157309 /* reinterpret_f16_bf16_x3 */, 157333 /* q24hq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157342 /* reinterpret_f16_f32_x3 */, 157365 /* q24hq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157374 /* reinterpret_f16_f64_x3 */, 157397 /* q24hq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157405 /* reinterpret_bf16_s8_x3 */, 157428 /* q24yq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157438 /* reinterpret_bf16_u8_x3 */, 157461 /* q24yq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157471 /* reinterpret_bf16_mf8_x3 */, 157495 /* q24yq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157504 /* reinterpret_bf16_s16_x3 */, 157528 /* q24yq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157537 /* reinterpret_bf16_u16_x3 */, 157561 /* q24yq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157571 /* reinterpret_bf16_s32_x3 */, 157595 /* q24yq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157604 /* reinterpret_bf16_u32_x3 */, 157628 /* q24yq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157638 /* reinterpret_bf16_s64_x3 */, 157662 /* q24yq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157671 /* reinterpret_bf16_u64_x3 */, 157695 /* q24yq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157705 /* reinterpret_bf16_f16_x3 */, 157729 /* q24yq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157738 /* reinterpret_bf16_bf16_x3 */, 157763 /* q24yq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157772 /* reinterpret_bf16_f32_x3 */, 157796 /* q24yq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157805 /* reinterpret_bf16_f64_x3 */, 157829 /* q24yq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157837 /* reinterpret_f32_s8_x3 */, 157859 /* q12fq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157869 /* reinterpret_f32_u8_x3 */, 157891 /* q12fq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157901 /* reinterpret_f32_mf8_x3 */, 157924 /* q12fq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157933 /* reinterpret_f32_s16_x3 */, 157956 /* q12fq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157965 /* reinterpret_f32_u16_x3 */, 157988 /* q12fq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{157998 /* reinterpret_f32_s32_x3 */, 158021 /* q12fq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158030 /* reinterpret_f32_u32_x3 */, 158053 /* q12fq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158063 /* reinterpret_f32_s64_x3 */, 158086 /* q12fq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158095 /* reinterpret_f32_u64_x3 */, 158118 /* q12fq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158128 /* reinterpret_f32_f16_x3 */, 158151 /* q12fq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158160 /* reinterpret_f32_bf16_x3 */, 158184 /* q12fq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158193 /* reinterpret_f32_f32_x3 */, 158216 /* q12fq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158225 /* reinterpret_f32_f64_x3 */, 158248 /* q12fq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158256 /* reinterpret_f64_s8_x3 */, 158278 /* q6dq48Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158287 /* reinterpret_f64_u8_x3 */, 158309 /* q6dq48Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158318 /* reinterpret_f64_mf8_x3 */, 158341 /* q6dq48m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158349 /* reinterpret_f64_s16_x3 */, 158372 /* q6dq24s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158380 /* reinterpret_f64_u16_x3 */, 158403 /* q6dq24Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158412 /* reinterpret_f64_s32_x3 */, 158435 /* q6dq12i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158443 /* reinterpret_f64_u32_x3 */, 158466 /* q6dq12Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158475 /* reinterpret_f64_s64_x3 */, 158498 /* q6dq6Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158506 /* reinterpret_f64_u64_x3 */, 158529 /* q6dq6UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158538 /* reinterpret_f64_f16_x3 */, 158561 /* q6dq24h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158569 /* reinterpret_f64_bf16_x3 */, 158593 /* q6dq24y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158601 /* reinterpret_f64_f32_x3 */, 158624 /* q6dq12f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158632 /* reinterpret_f64_f64_x3 */, 158655 /* q6dq6d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158662 /* reinterpret_s8_s8_x4 */, 137803 /* q64Scq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158683 /* reinterpret_s8_u8_x4 */, 158704 /* q64Scq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158715 /* reinterpret_s8_mf8_x4 */, 158737 /* q64Scq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158747 /* reinterpret_s8_s16_x4 */, 158769 /* q64Scq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158779 /* reinterpret_s8_u16_x4 */, 158801 /* q64Scq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158812 /* reinterpret_s8_s32_x4 */, 158834 /* q64Scq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158844 /* reinterpret_s8_u32_x4 */, 158866 /* q64Scq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158877 /* reinterpret_s8_s64_x4 */, 158899 /* q64Scq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158909 /* reinterpret_s8_u64_x4 */, 158931 /* q64Scq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158942 /* reinterpret_s8_f16_x4 */, 158964 /* q64Scq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{158974 /* reinterpret_s8_bf16_x4 */, 158997 /* q64Scq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159007 /* reinterpret_s8_f32_x4 */, 159029 /* q64Scq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159039 /* reinterpret_s8_f64_x4 */, 159061 /* q64Scq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159070 /* reinterpret_u8_s8_x4 */, 159091 /* q64Ucq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159102 /* reinterpret_u8_u8_x4 */, 137991 /* q64Ucq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159123 /* reinterpret_u8_mf8_x4 */, 159145 /* q64Ucq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159155 /* reinterpret_u8_s16_x4 */, 159177 /* q64Ucq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159187 /* reinterpret_u8_u16_x4 */, 159209 /* q64Ucq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159220 /* reinterpret_u8_s32_x4 */, 159242 /* q64Ucq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159252 /* reinterpret_u8_u32_x4 */, 159274 /* q64Ucq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159285 /* reinterpret_u8_s64_x4 */, 159307 /* q64Ucq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159317 /* reinterpret_u8_u64_x4 */, 159339 /* q64Ucq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159350 /* reinterpret_u8_f16_x4 */, 159372 /* q64Ucq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159382 /* reinterpret_u8_bf16_x4 */, 159405 /* q64Ucq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159415 /* reinterpret_u8_f32_x4 */, 159437 /* q64Ucq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159447 /* reinterpret_u8_f64_x4 */, 159469 /* q64Ucq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159478 /* reinterpret_mf8_s8_x4 */, 159500 /* q64mq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159510 /* reinterpret_mf8_u8_x4 */, 159532 /* q64mq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159542 /* reinterpret_mf8_mf8_x4 */, 137629 /* q64mq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159565 /* reinterpret_mf8_s16_x4 */, 159588 /* q64mq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159597 /* reinterpret_mf8_u16_x4 */, 159620 /* q64mq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159630 /* reinterpret_mf8_s32_x4 */, 159653 /* q64mq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159662 /* reinterpret_mf8_u32_x4 */, 159685 /* q64mq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159695 /* reinterpret_mf8_s64_x4 */, 159718 /* q64mq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159727 /* reinterpret_mf8_u64_x4 */, 159750 /* q64mq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159760 /* reinterpret_mf8_f16_x4 */, 159783 /* q64mq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159792 /* reinterpret_mf8_bf16_x4 */, 159816 /* q64mq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159825 /* reinterpret_mf8_f32_x4 */, 159848 /* q64mq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159857 /* reinterpret_mf8_f64_x4 */, 159880 /* q64mq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159888 /* reinterpret_s16_s8_x4 */, 159910 /* q32sq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159920 /* reinterpret_s16_u8_x4 */, 159942 /* q32sq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159952 /* reinterpret_s16_mf8_x4 */, 159975 /* q32sq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{159984 /* reinterpret_s16_s16_x4 */, 137673 /* q32sq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160007 /* reinterpret_s16_u16_x4 */, 160030 /* q32sq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160040 /* reinterpret_s16_s32_x4 */, 160063 /* q32sq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160072 /* reinterpret_s16_u32_x4 */, 160095 /* q32sq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160105 /* reinterpret_s16_s64_x4 */, 160128 /* q32sq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160137 /* reinterpret_s16_u64_x4 */, 160160 /* q32sq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160170 /* reinterpret_s16_f16_x4 */, 160193 /* q32sq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160202 /* reinterpret_s16_bf16_x4 */, 160226 /* q32sq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160235 /* reinterpret_s16_f32_x4 */, 160258 /* q32sq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160267 /* reinterpret_s16_f64_x4 */, 160290 /* q32sq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160298 /* reinterpret_u16_s8_x4 */, 160320 /* q32Usq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160331 /* reinterpret_u16_u8_x4 */, 160353 /* q32Usq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160364 /* reinterpret_u16_mf8_x4 */, 160387 /* q32Usq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160397 /* reinterpret_u16_s16_x4 */, 160420 /* q32Usq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160430 /* reinterpret_u16_u16_x4 */, 137851 /* q32Usq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160453 /* reinterpret_u16_s32_x4 */, 160476 /* q32Usq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160486 /* reinterpret_u16_u32_x4 */, 160509 /* q32Usq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160520 /* reinterpret_u16_s64_x4 */, 160543 /* q32Usq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160553 /* reinterpret_u16_u64_x4 */, 160576 /* q32Usq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160587 /* reinterpret_u16_f16_x4 */, 160610 /* q32Usq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160620 /* reinterpret_u16_bf16_x4 */, 160644 /* q32Usq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160654 /* reinterpret_u16_f32_x4 */, 160677 /* q32Usq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160687 /* reinterpret_u16_f64_x4 */, 160710 /* q32Usq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160719 /* reinterpret_s32_s8_x4 */, 160741 /* q16iq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160751 /* reinterpret_s32_u8_x4 */, 160773 /* q16iq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160783 /* reinterpret_s32_mf8_x4 */, 160806 /* q16iq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160815 /* reinterpret_s32_s16_x4 */, 160838 /* q16iq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160847 /* reinterpret_s32_u16_x4 */, 160870 /* q16iq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160880 /* reinterpret_s32_s32_x4 */, 137715 /* q16iq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160903 /* reinterpret_s32_u32_x4 */, 160926 /* q16iq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160936 /* reinterpret_s32_s64_x4 */, 160959 /* q16iq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{160968 /* reinterpret_s32_u64_x4 */, 160991 /* q16iq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161001 /* reinterpret_s32_f16_x4 */, 161024 /* q16iq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161033 /* reinterpret_s32_bf16_x4 */, 161057 /* q16iq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161066 /* reinterpret_s32_f32_x4 */, 25633 /* q16iq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161089 /* reinterpret_s32_f64_x4 */, 161112 /* q16iq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161120 /* reinterpret_u32_s8_x4 */, 161142 /* q16Uiq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161153 /* reinterpret_u32_u8_x4 */, 161175 /* q16Uiq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161186 /* reinterpret_u32_mf8_x4 */, 161209 /* q16Uiq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161219 /* reinterpret_u32_s16_x4 */, 161242 /* q16Uiq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161252 /* reinterpret_u32_u16_x4 */, 161275 /* q16Uiq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161286 /* reinterpret_u32_s32_x4 */, 161309 /* q16Uiq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161319 /* reinterpret_u32_u32_x4 */, 137897 /* q16Uiq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161342 /* reinterpret_u32_s64_x4 */, 161365 /* q16Uiq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161375 /* reinterpret_u32_u64_x4 */, 161398 /* q16Uiq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161409 /* reinterpret_u32_f16_x4 */, 161432 /* q16Uiq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161442 /* reinterpret_u32_bf16_x4 */, 161466 /* q16Uiq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161476 /* reinterpret_u32_f32_x4 */, 26157 /* q16Uiq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161499 /* reinterpret_u32_f64_x4 */, 161522 /* q16Uiq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161531 /* reinterpret_s64_s8_x4 */, 161553 /* q8Wiq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161563 /* reinterpret_s64_u8_x4 */, 161585 /* q8Wiq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161595 /* reinterpret_s64_mf8_x4 */, 161618 /* q8Wiq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161627 /* reinterpret_s64_s16_x4 */, 161650 /* q8Wiq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161659 /* reinterpret_s64_u16_x4 */, 161682 /* q8Wiq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161692 /* reinterpret_s64_s32_x4 */, 161715 /* q8Wiq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161724 /* reinterpret_s64_u32_x4 */, 161747 /* q8Wiq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161757 /* reinterpret_s64_s64_x4 */, 137759 /* q8Wiq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161780 /* reinterpret_s64_u64_x4 */, 161803 /* q8Wiq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161813 /* reinterpret_s64_f16_x4 */, 161836 /* q8Wiq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161845 /* reinterpret_s64_bf16_x4 */, 161869 /* q8Wiq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161878 /* reinterpret_s64_f32_x4 */, 161901 /* q8Wiq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161910 /* reinterpret_s64_f64_x4 */, 161933 /* q8Wiq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161941 /* reinterpret_u64_s8_x4 */, 161963 /* q8UWiq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{161974 /* reinterpret_u64_u8_x4 */, 161996 /* q8UWiq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162007 /* reinterpret_u64_mf8_x4 */, 162030 /* q8UWiq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162040 /* reinterpret_u64_s16_x4 */, 162063 /* q8UWiq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162073 /* reinterpret_u64_u16_x4 */, 162096 /* q8UWiq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162107 /* reinterpret_u64_s32_x4 */, 162130 /* q8UWiq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162140 /* reinterpret_u64_u32_x4 */, 162163 /* q8UWiq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162174 /* reinterpret_u64_s64_x4 */, 162197 /* q8UWiq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162207 /* reinterpret_u64_u64_x4 */, 137945 /* q8UWiq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162230 /* reinterpret_u64_f16_x4 */, 162253 /* q8UWiq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162263 /* reinterpret_u64_bf16_x4 */, 162287 /* q8UWiq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162297 /* reinterpret_u64_f32_x4 */, 162320 /* q8UWiq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162330 /* reinterpret_u64_f64_x4 */, 162353 /* q8UWiq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162362 /* reinterpret_f16_s8_x4 */, 162384 /* q32hq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162394 /* reinterpret_f16_u8_x4 */, 162416 /* q32hq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162426 /* reinterpret_f16_mf8_x4 */, 162449 /* q32hq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162458 /* reinterpret_f16_s16_x4 */, 162481 /* q32hq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162490 /* reinterpret_f16_u16_x4 */, 162513 /* q32hq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162523 /* reinterpret_f16_s32_x4 */, 162546 /* q32hq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162555 /* reinterpret_f16_u32_x4 */, 162578 /* q32hq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162588 /* reinterpret_f16_s64_x4 */, 162611 /* q32hq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162620 /* reinterpret_f16_u64_x4 */, 162643 /* q32hq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162653 /* reinterpret_f16_f16_x4 */, 137519 /* q32hq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162676 /* reinterpret_f16_bf16_x4 */, 162700 /* q32hq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162709 /* reinterpret_f16_f32_x4 */, 162732 /* q32hq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162741 /* reinterpret_f16_f64_x4 */, 162764 /* q32hq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162772 /* reinterpret_bf16_s8_x4 */, 162795 /* q32yq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162805 /* reinterpret_bf16_u8_x4 */, 162828 /* q32yq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162838 /* reinterpret_bf16_mf8_x4 */, 162862 /* q32yq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162871 /* reinterpret_bf16_s16_x4 */, 162895 /* q32yq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162904 /* reinterpret_bf16_u16_x4 */, 162928 /* q32yq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162938 /* reinterpret_bf16_s32_x4 */, 162962 /* q32yq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{162971 /* reinterpret_bf16_u32_x4 */, 162995 /* q32yq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163005 /* reinterpret_bf16_s64_x4 */, 163029 /* q32yq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163038 /* reinterpret_bf16_u64_x4 */, 163062 /* q32yq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163072 /* reinterpret_bf16_f16_x4 */, 163096 /* q32yq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163105 /* reinterpret_bf16_bf16_x4 */, 137475 /* q32yq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163130 /* reinterpret_bf16_f32_x4 */, 163154 /* q32yq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163163 /* reinterpret_bf16_f64_x4 */, 163187 /* q32yq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163195 /* reinterpret_f32_s8_x4 */, 163217 /* q16fq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163227 /* reinterpret_f32_u8_x4 */, 163249 /* q16fq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163259 /* reinterpret_f32_mf8_x4 */, 163282 /* q16fq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163291 /* reinterpret_f32_s16_x4 */, 163314 /* q16fq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163323 /* reinterpret_f32_u16_x4 */, 163346 /* q16fq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163356 /* reinterpret_f32_s32_x4 */, 24539 /* q16fq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163379 /* reinterpret_f32_u32_x4 */, 24740 /* q16fq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163402 /* reinterpret_f32_s64_x4 */, 163425 /* q16fq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163434 /* reinterpret_f32_u64_x4 */, 163457 /* q16fq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163467 /* reinterpret_f32_f16_x4 */, 163490 /* q16fq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163499 /* reinterpret_f32_bf16_x4 */, 163523 /* q16fq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163532 /* reinterpret_f32_f32_x4 */, 104264 /* q16fq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163555 /* reinterpret_f32_f64_x4 */, 163578 /* q16fq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163586 /* reinterpret_f64_s8_x4 */, 163608 /* q8dq64Sc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163617 /* reinterpret_f64_u8_x4 */, 163639 /* q8dq64Uc */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163648 /* reinterpret_f64_mf8_x4 */, 163671 /* q8dq64m */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163679 /* reinterpret_f64_s16_x4 */, 163702 /* q8dq32s */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163710 /* reinterpret_f64_u16_x4 */, 163733 /* q8dq32Us */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163742 /* reinterpret_f64_s32_x4 */, 163765 /* q8dq16i */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163773 /* reinterpret_f64_u32_x4 */, 163796 /* q8dq16Ui */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163805 /* reinterpret_f64_s64_x4 */, 163828 /* q8dq8Wi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163836 /* reinterpret_f64_u64_x4 */, 163859 /* q8dq8UWi */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163868 /* reinterpret_f64_f16_x4 */, 163891 /* q8dq32h */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163899 /* reinterpret_f64_bf16_x4 */, 163923 /* q8dq32y */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163931 /* reinterpret_f64_f32_x4 */, 163954 /* q8dq16f */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{163962 /* reinterpret_f64_f64_x4 */, 137587 /* q8dq8d */, 1 /* n */, 143371 /* sme|sve */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
#endif // GET_SVE_BUILTIN_INFOS

