#ifdef DECL_SIGNATURE_TABLE
PrototypeDescriptor(7, 0, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(2, 19, 8),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(2, 19, 0),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 19, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 19, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 19, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 19, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 12, 136),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 12, 128),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 11, 136),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 11, 128),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 10, 136),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 10, 128),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 136),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 3, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 3, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 2, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 2, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 64),
PrototypeDescriptor(2, 0, 16),
PrototypeDescriptor(2, 1, 64),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 64),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 32),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 136),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 128),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 32),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 25, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 24, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 23, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 22, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 21, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 20, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 19, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 15, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 14, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 12, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 11, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 10, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 32),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 25, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 24, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 23, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 22, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 21, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 20, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 19, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 15, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 14, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 12, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 11, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 10, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 32),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 25, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 25, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 24, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 24, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 23, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 23, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 19, 8),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(2, 19, 0),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 1, 136),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 136),
PrototypeDescriptor(2, 1, 16),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(4, 0, 0),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 0, 136),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 136),
PrototypeDescriptor(2, 0, 16),
PrototypeDescriptor(2, 1, 64),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 16),
PrototypeDescriptor(2, 1, 32),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 16),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 64),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 32),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 32, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 31, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 30, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 29, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 11),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 2, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 32),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 16),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 16),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(4, 0, 1),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 3),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 9),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(5, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 8, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 7, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 6, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(3, 0, 0),
PrototypeDescriptor(1, 0, 1),
PrototypeDescriptor(2, 5, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 35, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 35, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(2, 34, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(2, 34, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 19, 8),
PrototypeDescriptor(2, 19, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 19, 0),
PrototypeDescriptor(2, 19, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 1, 128),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 128),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 128),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 128),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(4, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 1, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(4, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 9, 8),
PrototypeDescriptor(2, 9, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 9, 8),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(4, 0, 4),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(4, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(4, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(4, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 9, 8),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 9, 0),
PrototypeDescriptor(2, 9, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 9, 0),
PrototypeDescriptor(2, 9, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 9, 0),
PrototypeDescriptor(1, 0, 8),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(4, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(2, 4, 0),
PrototypeDescriptor(2, 36, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 36, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 19, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 17, 8),
PrototypeDescriptor(2, 19, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 17, 0),
PrototypeDescriptor(2, 18, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 18, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 1, 32),
PrototypeDescriptor(2, 1, 32),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 1, 32),
PrototypeDescriptor(2, 1, 32),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(2, 1, 0),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(1, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(6, 0, 0),
PrototypeDescriptor(2, 37, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 37, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 38, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 38, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 39, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 39, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 40, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 0, 8),
PrototypeDescriptor(2, 40, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 0, 0),
PrototypeDescriptor(2, 19, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 16, 8),
PrototypeDescriptor(2, 19, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
PrototypeDescriptor(2, 16, 0),
#endif
#ifdef DECL_INTRINSIC_RECORDS
{/*Name=*/"vadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_rtz_xu_f_v", /*OverloadedName=*/"vfcvt_rtz_xu", /*RequiredExtensions=*/"", /*PrototypeIndex=*/552, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwaddu_vv", /*OverloadedName=*/"vwaddu_vv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1065, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwaddu_wv", /*OverloadedName=*/"vwaddu_wv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1064, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/555, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/621, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/558, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/555, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/570, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/561, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vse8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/180, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/156, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/824, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/664, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/792, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/696, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vneg_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnot_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnot_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/65, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmmv_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfneg_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwcvtu_x_x_v", /*OverloadedName=*/"vwcvtu_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/486, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vncvt_x_x_w", /*OverloadedName=*/"vncvt_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/555, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/555, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/555, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/180, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/156, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/180, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/180, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/180, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/156, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/180, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/180, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/156, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vse64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/180, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/555, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/555, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/555, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlse64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/618, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/824, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/664, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/824, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/824, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/824, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/664, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/824, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/824, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/664, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsse64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/824, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/567, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/564, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/570, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/567, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/564, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/561, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/570, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/567, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/564, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/561, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/570, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/567, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/564, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/561, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/570, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/567, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/564, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/561, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/570, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/567, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/564, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/561, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/570, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/567, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/564, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/561, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/570, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/567, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/564, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/561, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/633, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/630, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/627, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/624, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/760, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/728, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/792, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/760, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/728, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/696, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/792, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/760, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/728, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/696, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/792, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/760, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/728, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/696, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/792, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/760, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/728, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/696, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/792, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/760, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/728, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/696, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/792, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/760, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/728, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/696, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/792, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/760, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/728, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/696, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/952, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/920, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/888, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"64bit", /*PrototypeIndex=*/856, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vle16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/621, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/558, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/621, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/621, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/621, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/558, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/621, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/621, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/558, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vle64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/621, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/402, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsseg2e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/177, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/150, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlseg2e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/426, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/405, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/402, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vssseg2e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/820, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/660, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/417, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/788, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlseg3e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/363, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/327, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/291, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/255, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/219, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/183, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/402, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/363, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/327, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/291, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/255, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/219, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/183, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/402, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/363, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/327, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/291, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/255, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/219, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/183, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/402, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/363, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/327, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/291, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/255, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/219, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/183, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/384, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/366, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/348, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/330, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/312, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/294, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/276, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/258, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/240, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/222, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/204, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e8ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/186, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/426, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/405, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/384, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/366, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/348, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/330, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/312, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/294, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/276, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/258, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/240, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/222, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/204, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/186, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/426, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/405, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/384, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/366, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/348, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/330, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/312, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/294, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/276, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/258, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/240, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/222, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/204, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/186, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/426, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/405, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/384, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/366, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/348, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/330, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/312, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/294, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/276, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/258, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/240, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/222, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/204, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/186, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/426, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/384, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/348, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/312, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/276, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/240, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/204, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/426, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/384, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/348, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/312, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/276, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/240, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e32ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/204, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/426, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/384, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/348, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/312, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/276, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/240, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e64ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/204, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg2e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/426, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg3e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/384, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg4e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/348, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg5e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/312, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg6e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/276, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg7e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/240, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlseg8e16ff_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/204, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/363, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/327, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/291, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/255, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/219, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/183, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/402, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/363, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/327, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/291, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/255, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/219, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/183, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/402, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/363, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/327, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/291, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/255, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/219, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/183, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/402, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/363, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/327, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/291, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/255, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/219, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/183, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/423, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/381, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/345, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/309, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/273, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/237, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlsseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/201, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/378, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/342, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/306, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/270, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/234, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/198, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/414, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/375, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/339, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/303, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/267, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/231, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/195, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/411, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/372, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/336, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/300, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/264, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/228, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/192, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/408, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/369, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/333, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/297, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/261, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/225, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/189, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/417, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/378, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/342, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/306, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/270, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/234, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/198, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/414, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/375, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/339, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/303, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/267, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/231, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/195, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/411, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/372, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/336, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/300, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/264, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/228, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/192, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/408, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/369, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/333, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/297, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/261, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/225, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/189, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/417, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/378, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/342, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/306, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/270, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/234, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/198, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/414, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/375, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/339, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/303, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/267, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/231, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/195, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/411, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/372, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/336, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/300, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/264, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/228, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/192, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/408, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/369, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/333, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/297, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/261, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/225, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/189, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/417, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/378, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/342, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/306, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/270, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/234, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/198, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/414, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/375, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/339, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/303, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/267, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/231, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/195, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/411, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/372, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/336, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/300, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/264, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/228, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/192, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/408, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/369, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/333, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/297, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/261, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/225, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/189, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vluxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/417, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/378, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/342, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/306, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/270, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/234, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/198, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/414, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/375, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/339, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/303, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/267, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/231, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/195, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/411, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/372, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/336, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/300, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/264, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/228, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/192, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/408, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/369, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/333, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/297, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/261, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/225, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/189, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/417, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/378, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/342, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/306, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/270, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/234, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/198, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/414, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/375, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/339, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/303, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/267, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/231, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/195, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/411, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/372, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/336, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/300, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/264, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/228, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/192, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/408, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/369, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/333, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/297, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/261, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/225, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/189, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/417, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/378, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/342, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/306, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/270, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/234, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/198, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/414, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/375, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/339, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/303, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/267, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/231, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/195, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/411, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/372, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/336, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/300, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/264, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/228, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/192, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/408, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/369, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/333, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/297, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/261, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/225, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/189, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/417, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/378, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/342, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/306, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/270, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/234, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/198, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/414, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/375, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/339, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/303, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/267, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/231, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/195, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/411, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/372, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/336, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/300, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/264, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/228, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/192, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/408, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/369, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/333, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/297, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/261, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/225, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/189, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/438, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/396, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/360, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/324, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/288, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/252, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/216, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/435, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/393, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/357, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/321, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/285, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/249, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/213, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/432, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/390, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/354, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/318, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/282, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/246, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/210, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/429, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/387, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/351, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/315, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/279, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/243, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vloxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/207, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsseg3e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/174, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/147, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/171, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/144, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/168, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/141, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/165, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/138, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/162, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/135, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/159, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/132, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/177, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/150, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/174, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/147, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/171, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/144, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/168, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/141, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/165, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/138, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/162, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/135, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/159, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/132, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/177, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/150, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/174, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/147, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/171, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/144, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/168, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/141, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/165, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/138, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/162, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/135, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/159, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/132, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/177, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/150, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/174, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/147, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/171, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/144, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/168, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/141, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/165, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/138, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/162, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/135, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/159, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/132, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/177, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/174, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/171, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/168, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/165, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/162, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/159, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/177, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/174, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/171, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/168, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/165, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/162, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/159, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/177, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/174, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/171, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/168, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/165, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/162, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/159, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/177, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/174, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/171, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/168, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/165, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/162, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/159, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/816, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/656, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/812, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/652, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/808, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/648, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/804, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/644, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/800, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/640, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/796, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/636, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/820, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/660, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/816, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/656, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/812, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/652, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/808, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/648, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/804, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/644, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/800, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/640, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/796, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/636, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/820, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/660, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/816, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/656, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/812, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/652, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/808, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/648, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/804, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/644, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/800, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/640, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/796, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/636, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/820, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/660, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/816, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/656, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/812, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/652, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/808, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/648, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/804, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/644, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/800, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/640, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/796, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/636, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/820, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/816, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/812, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/808, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/804, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/800, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/796, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/820, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/816, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/812, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/808, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/804, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/800, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/796, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/820, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/816, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/812, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/808, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/804, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/800, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/796, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg2e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/820, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg3e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/816, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg4e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/812, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg5e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/808, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg6e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/804, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg7e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/800, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vssseg8e16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/796, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/784, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/780, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/776, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/772, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/768, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/764, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/756, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/752, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/748, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/744, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/740, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/736, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/732, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/724, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/720, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/716, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/712, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/708, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/704, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/700, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/692, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/688, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/684, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/680, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/676, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/672, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/668, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/788, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/784, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/780, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/776, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/772, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/768, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/764, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/756, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/752, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/748, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/744, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/740, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/736, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/732, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/724, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/720, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/716, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/712, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/708, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/704, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/700, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/692, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/688, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/684, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/680, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/676, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/672, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/668, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/788, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/784, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/780, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/776, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/772, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/768, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/764, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/756, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/752, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/748, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/744, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/740, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/736, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/732, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/724, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/720, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/716, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/712, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/708, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/704, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/700, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/692, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/688, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/684, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/680, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/676, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/672, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/668, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/788, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/784, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/780, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/776, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/772, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/768, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/764, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/756, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/752, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/748, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/744, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/740, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/736, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/732, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/724, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/720, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/716, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/712, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/708, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/704, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/700, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/692, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/688, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/684, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/680, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/676, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/672, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/668, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsuxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/788, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/784, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/780, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/776, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/772, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/768, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/764, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/756, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/752, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/748, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/744, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/740, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/736, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/732, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/724, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/720, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/716, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/712, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/708, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/704, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/700, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/692, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/688, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/684, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/680, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/676, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/672, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/668, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/788, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/784, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/780, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/776, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/772, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/768, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/764, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/756, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/752, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/748, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/744, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/740, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/736, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/732, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/724, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/720, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/716, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/712, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/708, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/704, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/700, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/692, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/688, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/684, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/680, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/676, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/672, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/668, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/2, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/788, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/784, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/780, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/776, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/772, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/768, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/764, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/756, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/752, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/748, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/744, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/740, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/736, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/732, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/724, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/720, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/716, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/712, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/708, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/704, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/700, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/692, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/688, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/684, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/680, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/676, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/672, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/668, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/788, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/784, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/780, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/776, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/772, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/768, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/764, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/756, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/752, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/748, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/744, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/740, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/736, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/732, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/724, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/720, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/716, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/712, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/708, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/704, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/700, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/692, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/688, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/684, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/680, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/676, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/672, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/668, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/128, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/948, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/944, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/940, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/936, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/932, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/928, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei8_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/924, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/916, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/912, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/908, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/904, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/900, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/896, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei16_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/892, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/884, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/880, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/876, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/872, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/868, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/864, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei32_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/860, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg2ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/852, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg3ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/848, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg4ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/844, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg5ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/840, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg6ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/836, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg7ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/832, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsoxseg8ei64_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/828, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/1, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vadd_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vadd_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsub_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsub_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrsub_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrsub_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwaddu_vx", /*OverloadedName=*/"vwaddu_vx", /*RequiredExtensions=*/"", /*PrototypeIndex=*/489, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsubu_vv", /*OverloadedName=*/"vwsubu_vv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1065, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsubu_vx", /*OverloadedName=*/"vwsubu_vx", /*RequiredExtensions=*/"", /*PrototypeIndex=*/489, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwadd_vv", /*OverloadedName=*/"vwadd_vv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwadd_vx", /*OverloadedName=*/"vwadd_vx", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsub_vv", /*OverloadedName=*/"vwsub_vv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsub_vx", /*OverloadedName=*/"vwsub_vx", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwaddu_wx", /*OverloadedName=*/"vwaddu_wx", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1068, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsubu_wv", /*OverloadedName=*/"vwsubu_wv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1064, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsubu_wx", /*OverloadedName=*/"vwsubu_wx", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1068, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwadd_wv", /*OverloadedName=*/"vwadd_wv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwadd_wx", /*OverloadedName=*/"vwadd_wx", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsub_wv", /*OverloadedName=*/"vwsub_wv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsub_wx", /*OverloadedName=*/"vwsub_wx", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwcvt_x_x_v", /*OverloadedName=*/"vwcvt_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/495, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vadc_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1184, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vadc_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1192, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vadc_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1132, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vadc_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1140, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsbc_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1184, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsbc_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1192, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsbc_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1132, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsbc_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1140, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmadc_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmadc_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmadc_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmadc_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmadc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmadc_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmadc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmadc_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmsbc_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmsbc_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmsbc_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmsbc_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmsbc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmsbc_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmsbc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmsbc_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vand_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vand_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vand_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vand_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vxor_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vxor_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vxor_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vxor_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vor_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vor_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vor_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vor_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsll_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1176, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsll_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1156, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsll_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsll_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1108, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsrl_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsrl_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1108, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsra_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1176, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsra_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1156, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnsrl_wv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1104, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnsrl_wx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1100, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnsra_wv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1152, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnsra_wx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1148, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vncvt_x_x_w", /*OverloadedName=*/"vncvt_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1067, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmseq_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmseq_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmseq_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmseq_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsne_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsne_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsne_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsne_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsltu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsltu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmslt_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmslt_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsleu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsleu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsle_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsle_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsgtu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsgtu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsgt_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsgt_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsgeu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1044, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsgeu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1048, /*SuffixIndex=*/1046, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsge_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsge_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vminu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vminu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmin_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmin_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmaxu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmaxu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmax_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmax_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmul_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmul_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmulh_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmulh_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmulhu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmulhu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmulhsu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1176, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmulhsu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/615, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vdivu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vdivu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vdiv_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vdiv_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vremu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vremu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrem_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrem_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmul_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmulu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1065, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmulu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/489, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmulsu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1077, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmulsu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/495, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmacc_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1197, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmacc_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1144, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnmsac_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1197, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnmsac_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1144, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmadd_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1197, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmadd_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1144, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnmsub_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1197, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnmsub_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1144, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmaccu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1064, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmaccu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1068, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmacc_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmaccsu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1076, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmaccsu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1088, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwmaccus_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1080, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmerge_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1184, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmerge_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1192, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmerge_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1132, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmerge_vxm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1140, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmv_v_v", /*OverloadedName=*/"vmv_v", /*RequiredExtensions=*/"", /*PrototypeIndex=*/65, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmv_v_v", /*OverloadedName=*/"vmv_v", /*RequiredExtensions=*/"", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmv_v_x", /*OverloadedName=*/"vmv_v", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1057, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmv_v_x", /*OverloadedName=*/"vmv_v", /*RequiredExtensions=*/"", /*PrototypeIndex=*/490, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsaddu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsaddu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsadd_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vssubu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vssubu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vssub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vssub_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaaddu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1120, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaaddu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaadd_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vasubu_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1120, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vasubu_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vasub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vasub_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsmul_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vssrl_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1120, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vssrl_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1108, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vssra_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1176, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vssra_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1156, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnclipu_wv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1104, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnclipu_wx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1100, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnclip_wv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1152, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vnclip_wx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1148, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wv", /*OverloadedName=*/"vfwadd_wv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wf", /*OverloadedName=*/"vfwadd_wf", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wv", /*OverloadedName=*/"vfwadd_wv", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wf", /*OverloadedName=*/"vfwadd_wf", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wv", /*OverloadedName=*/"vfwadd_wv", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1072, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wf", /*OverloadedName=*/"vfwadd_wf", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1084, /*SuffixIndex=*/130, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wv", /*OverloadedName=*/"vfwsub_wv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wf", /*OverloadedName=*/"vfwsub_wf", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wv", /*OverloadedName=*/"vfwsub_wv", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wf", /*OverloadedName=*/"vfwsub_wf", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wv", /*OverloadedName=*/"vfwsub_wv", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1072, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wf", /*OverloadedName=*/"vfwsub_wf", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1084, /*SuffixIndex=*/130, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfdiv_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfdiv_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfdiv_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1247, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfdiv_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrdiv_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrdiv_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wv", /*OverloadedName=*/"vfwadd_wv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wf", /*OverloadedName=*/"vfwadd_wf", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wv", /*OverloadedName=*/"vfwadd_wv", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wf", /*OverloadedName=*/"vfwadd_wf", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wv", /*OverloadedName=*/"vfwadd_wv", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1072, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_wf", /*OverloadedName=*/"vfwadd_wf", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1084, /*SuffixIndex=*/130, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wv", /*OverloadedName=*/"vfwsub_wv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wf", /*OverloadedName=*/"vfwsub_wf", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wv", /*OverloadedName=*/"vfwsub_wv", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1072, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wf", /*OverloadedName=*/"vfwsub_wf", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1084, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wv", /*OverloadedName=*/"vfwsub_wv", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1072, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_wf", /*OverloadedName=*/"vfwsub_wf", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1084, /*SuffixIndex=*/130, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfdiv_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfdiv_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfdiv_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfdiv_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrdiv_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrdiv_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vv", /*OverloadedName=*/"vfwadd_vv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vf", /*OverloadedName=*/"vfwadd_vf", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vv", /*OverloadedName=*/"vfwadd_vv", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vf", /*OverloadedName=*/"vfwadd_vf", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vv", /*OverloadedName=*/"vfwadd_vv", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1237, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vf", /*OverloadedName=*/"vfwadd_vf", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1092, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vv", /*OverloadedName=*/"vfwsub_vv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vf", /*OverloadedName=*/"vfwsub_vf", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vv", /*OverloadedName=*/"vfwsub_vv", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vf", /*OverloadedName=*/"vfwsub_vf", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vv", /*OverloadedName=*/"vfwsub_vv", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1237, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vf", /*OverloadedName=*/"vfwsub_vf", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1092, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1237, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1092, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vv", /*OverloadedName=*/"vfwadd_vv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vf", /*OverloadedName=*/"vfwadd_vf", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vv", /*OverloadedName=*/"vfwadd_vv", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vf", /*OverloadedName=*/"vfwadd_vf", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vv", /*OverloadedName=*/"vfwadd_vv", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1237, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwadd_vf", /*OverloadedName=*/"vfwadd_vf", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1092, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vv", /*OverloadedName=*/"vfwsub_vv", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vf", /*OverloadedName=*/"vfwsub_vf", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vv", /*OverloadedName=*/"vfwsub_vv", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vf", /*OverloadedName=*/"vfwsub_vf", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vv", /*OverloadedName=*/"vfwsub_vv", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1237, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwsub_vf", /*OverloadedName=*/"vfwsub_vf", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1092, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1237, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1092, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1237, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmul_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1092, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1246, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmadd_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1202, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfnmsub_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1251, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1236, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1241, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1236, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1241, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1236, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1241, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1236, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1241, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmaccbf16_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfwma", /*PrototypeIndex=*/1226, /*SuffixIndex=*/58, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmaccbf16_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfwma", /*PrototypeIndex=*/1231, /*SuffixIndex=*/58, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1236, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1241, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1236, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmacc_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1241, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1236, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1241, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1236, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1241, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1236, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwnmsac_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1241, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmaccbf16_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfwma", /*PrototypeIndex=*/1226, /*SuffixIndex=*/58, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwmaccbf16_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfwma", /*PrototypeIndex=*/1231, /*SuffixIndex=*/58, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsqrt_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1228, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsqrt_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1228, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrec7_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1228, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrec7_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1228, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrec7_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1228, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsqrt_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsqrt_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrec7_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrec7_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrec7_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrsqrt7_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrsqrt7_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfrsqrt7_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmin_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmin_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmin_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmin_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmin_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmin_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmax_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmax_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmax_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmax_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmax_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfmax_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnj_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnj_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnj_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnj_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnj_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnj_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjn_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjn_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjn_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjn_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjn_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjn_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjx_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjx_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjx_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjx_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjx_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfsgnjx_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfneg_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfneg_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfabs_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfabs_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfabs_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmfeq_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfeq_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfeq_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfeq_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfeq_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfeq_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfne_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfne_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfne_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfne_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfne_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfne_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmflt_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmflt_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmflt_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmflt_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmflt_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmflt_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfle_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfle_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfle_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfle_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfle_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfle_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfgt_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfgt_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfgt_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfgt_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfgt_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfgt_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfge_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfge_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfge_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfge_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfge_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1052, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmfge_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1056, /*SuffixIndex=*/1043, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfclass_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/552, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfclass_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/552, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfclass_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/552, /*SuffixIndex=*/1078, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmerge_vvm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1184, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/240, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmerge_vfm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1192, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmerge_vfm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1192, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmerge_vfm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1192, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_v_f", /*OverloadedName=*/"vfmv_v", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1057, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_v_f", /*OverloadedName=*/"vfmv_v", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1057, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_v_f", /*OverloadedName=*/"vfmv_v", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1057, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfcvt_rtz_xu_f_v", /*OverloadedName=*/"vfcvt_rtz_xu", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/552, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_rtz_x_f_v", /*OverloadedName=*/"vfcvt_rtz_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/507, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_rtz_x_f_v", /*OverloadedName=*/"vfcvt_rtz_x", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/507, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_rtz_xu_f_v", /*OverloadedName=*/"vfwcvt_rtz_xu", /*RequiredExtensions=*/"", /*PrototypeIndex=*/492, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_rtz_xu_f_v", /*OverloadedName=*/"vfwcvt_rtz_xu", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/492, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_rtz_x_f_v", /*OverloadedName=*/"vfwcvt_rtz_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/483, /*SuffixIndex=*/483, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_rtz_x_f_v", /*OverloadedName=*/"vfwcvt_rtz_x", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/483, /*SuffixIndex=*/483, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_xu_v", /*OverloadedName=*/"vfwcvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/58, /*SuffixIndex=*/58, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/6, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_x_v", /*OverloadedName=*/"vfwcvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1227, /*SuffixIndex=*/58, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/6, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_xu_v", /*OverloadedName=*/"vfwcvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/58, /*SuffixIndex=*/58, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_x_v", /*OverloadedName=*/"vfwcvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1227, /*SuffixIndex=*/58, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_xu_v", /*OverloadedName=*/"vfwcvt_f_bf16", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/54, /*SuffixIndex=*/52, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_x_v", /*OverloadedName=*/"vfwcvt_f_bf16", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/56, /*SuffixIndex=*/52, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_f_v", /*OverloadedName=*/"vfwcvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/495, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_f_v", /*OverloadedName=*/"vfwcvt_f", /*RequiredExtensions=*/"zvfhmin", /*PrototypeIndex=*/495, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_f_v", /*OverloadedName=*/"vfwcvt_f", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/495, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_rtz_xu_f_w", /*OverloadedName=*/"vfncvt_rtz_xu", /*RequiredExtensions=*/"", /*PrototypeIndex=*/534, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/6, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_rtz_xu_f_w", /*OverloadedName=*/"vfncvt_rtz_xu", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/534, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_rtz_xu_f_w", /*OverloadedName=*/"vfncvt_rtz_xu", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/55, /*SuffixIndex=*/54, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_rtz_x_f_w", /*OverloadedName=*/"vfncvt_rtz_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/504, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/6, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_rtz_x_f_w", /*OverloadedName=*/"vfncvt_rtz_x", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/504, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_rtz_x_f_w", /*OverloadedName=*/"vfncvt_rtz_x", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/53, /*SuffixIndex=*/52, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_rod_f_f_w", /*OverloadedName=*/"vfncvt_rod_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_rod_f_f_w", /*OverloadedName=*/"vfncvt_rod_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_rod_f_f_w", /*OverloadedName=*/"vfncvt_rod_f_bf16", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_f_v", /*OverloadedName=*/"vfwcvt_f_f8e4m3_bf16", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/495, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/256, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_f_f_v", /*OverloadedName=*/"vfwcvt_f_f8e5m2_bf16", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/495, /*SuffixIndex=*/606, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/512, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/1,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_x_f_v", /*OverloadedName=*/"vfcvt_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/507, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_x_f_v", /*OverloadedName=*/"vfcvt_x", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/507, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_xu_f_v", /*OverloadedName=*/"vfcvt_xu", /*RequiredExtensions=*/"", /*PrototypeIndex=*/552, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_xu_f_v", /*OverloadedName=*/"vfcvt_xu", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/552, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_f_x_v", /*OverloadedName=*/"vfcvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/609, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_f_xu_v", /*OverloadedName=*/"vfcvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1177, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_f_x_v", /*OverloadedName=*/"vfcvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/609, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_f_xu_v", /*OverloadedName=*/"vfcvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1177, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_x_f_v", /*OverloadedName=*/"vfwcvt_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/483, /*SuffixIndex=*/483, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_x_f_v", /*OverloadedName=*/"vfwcvt_x", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/483, /*SuffixIndex=*/483, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_xu_f_v", /*OverloadedName=*/"vfwcvt_xu", /*RequiredExtensions=*/"", /*PrototypeIndex=*/492, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_xu_f_v", /*OverloadedName=*/"vfwcvt_xu", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/492, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_x_f_w", /*OverloadedName=*/"vfncvt_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/504, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/6, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_x_f_w", /*OverloadedName=*/"vfncvt_x", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/504, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_x_f_w", /*OverloadedName=*/"vfncvt_x", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/501, /*SuffixIndex=*/52, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_xu_f_w", /*OverloadedName=*/"vfncvt_xu", /*RequiredExtensions=*/"", /*PrototypeIndex=*/534, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/6, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_xu_f_w", /*OverloadedName=*/"vfncvt_xu", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/534, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_xu_f_w", /*OverloadedName=*/"vfncvt_xu", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/531, /*SuffixIndex=*/54, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_x_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/600, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_xu_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/603, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_x_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/600, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_xu_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/603, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"zvfhmin", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f_bf16", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvtbf16_f_f_w", /*OverloadedName=*/"vfncvtbf16_f", /*RequiredExtensions=*/"zvfbfmin", /*PrototypeIndex=*/597, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f_bf16_f8e4m3", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/606, /*SuffixIndex=*/495, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/256, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f_bf16_f8e5m2", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/606, /*SuffixIndex=*/495, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/512, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/1,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_sat_f_f_w", /*OverloadedName=*/"vfncvt_sat_f_bf16_f8e4m3", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/606, /*SuffixIndex=*/495, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/256, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_sat_f_f_w", /*OverloadedName=*/"vfncvt_sat_f_bf16_f8e5m2", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/606, /*SuffixIndex=*/495, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/512, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/1,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_q", /*OverloadedName=*/"vfncvt_f_f8e4m3", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/594, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/256, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_q", /*OverloadedName=*/"vfncvt_f_f8e5m2", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/594, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/512, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/1,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_sat_f_f_q", /*OverloadedName=*/"vfncvt_sat_f_f8e4m3", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/594, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/256, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_sat_f_f_q", /*OverloadedName=*/"vfncvt_sat_f_f8e5m2", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/594, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/512, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/1,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_x_f_v", /*OverloadedName=*/"vfcvt_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/507, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_x_f_v", /*OverloadedName=*/"vfcvt_x", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/507, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_xu_f_v", /*OverloadedName=*/"vfcvt_xu", /*RequiredExtensions=*/"", /*PrototypeIndex=*/552, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_xu_f_v", /*OverloadedName=*/"vfcvt_xu", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/552, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_f_x_v", /*OverloadedName=*/"vfcvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/609, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_f_xu_v", /*OverloadedName=*/"vfcvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1078, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_f_x_v", /*OverloadedName=*/"vfcvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/609, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfcvt_f_xu_v", /*OverloadedName=*/"vfcvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1078, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_x_f_v", /*OverloadedName=*/"vfwcvt_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/483, /*SuffixIndex=*/483, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_x_f_v", /*OverloadedName=*/"vfwcvt_x", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/483, /*SuffixIndex=*/483, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_xu_f_v", /*OverloadedName=*/"vfwcvt_xu", /*RequiredExtensions=*/"", /*PrototypeIndex=*/492, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfwcvt_xu_f_v", /*OverloadedName=*/"vfwcvt_xu", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/492, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_x_f_w", /*OverloadedName=*/"vfncvt_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/504, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/6, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_x_f_w", /*OverloadedName=*/"vfncvt_x", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/504, /*SuffixIndex=*/53, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_x_f_w", /*OverloadedName=*/"vfncvt_x", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/53, /*SuffixIndex=*/52, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_xu_f_w", /*OverloadedName=*/"vfncvt_xu", /*RequiredExtensions=*/"", /*PrototypeIndex=*/534, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/6, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_xu_f_w", /*OverloadedName=*/"vfncvt_xu", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/534, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_xu_f_w", /*OverloadedName=*/"vfncvt_xu", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/55, /*SuffixIndex=*/54, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_x_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/600, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_xu_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/603, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_x_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/600, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_xu_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/603, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f", /*RequiredExtensions=*/"zvfhmin", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f_bf16", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/606, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvtbf16_f_f_w", /*OverloadedName=*/"vfncvtbf16_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/57, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f_bf16_f8e4m3", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/606, /*SuffixIndex=*/495, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/256, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_w", /*OverloadedName=*/"vfncvt_f_bf16_f8e5m2", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/606, /*SuffixIndex=*/495, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/512, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/1,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_sat_f_f_w", /*OverloadedName=*/"vfncvt_sat_f_bf16_f8e4m3", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/606, /*SuffixIndex=*/495, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/256, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_sat_f_f_w", /*OverloadedName=*/"vfncvt_sat_f_bf16_f8e5m2", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/606, /*SuffixIndex=*/495, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/512, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/1,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_q", /*OverloadedName=*/"vfncvt_f_f8e4m3", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/594, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/256, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_f_f_q", /*OverloadedName=*/"vfncvt_f_f8e5m2", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/594, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/512, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/1,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_sat_f_f_q", /*OverloadedName=*/"vfncvt_sat_f_f8e4m3", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/594, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/256, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfncvt_sat_f_f_q", /*OverloadedName=*/"vfncvt_sat_f_f8e5m2", /*RequiredExtensions=*/"zvfofp8min", /*PrototypeIndex=*/594, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/512, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/1,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vredsum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredsum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/498, /*SuffixIndex=*/59, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredmaxu_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/498, /*SuffixIndex=*/59, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredmax_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredminu_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/498, /*SuffixIndex=*/59, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredmin_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredand_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredand_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/498, /*SuffixIndex=*/59, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredor_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredor_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/498, /*SuffixIndex=*/59, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredxor_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vredxor_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/498, /*SuffixIndex=*/59, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vwredsum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1060, /*SuffixIndex=*/1061, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vwredsumu_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/480, /*SuffixIndex=*/481, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredmax_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredmax_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredmin_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredmin_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredusum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredusum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredosum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredosum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfwredusum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1060, /*SuffixIndex=*/1061, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfwredusum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1060, /*SuffixIndex=*/1061, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfwredosum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1060, /*SuffixIndex=*/1061, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfwredosum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1060, /*SuffixIndex=*/1061, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/1, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredusum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredusum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredosum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfredosum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1096, /*SuffixIndex=*/1097, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfwredusum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1060, /*SuffixIndex=*/1061, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfwredusum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1060, /*SuffixIndex=*/1061, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfwredosum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1060, /*SuffixIndex=*/1061, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/64, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vfwredosum_vs", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1060, /*SuffixIndex=*/1061, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/0, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmnot_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"viota_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1046, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vid_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/47, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vid_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/45, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmv_x_s", /*OverloadedName=*/"vmv_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1233, /*SuffixIndex=*/1057, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmv_x_s", /*OverloadedName=*/"vmv_x", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1070, /*SuffixIndex=*/490, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmv_s_x", /*OverloadedName=*/"vmv_s", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1057, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmv_s_x", /*OverloadedName=*/"vmv_s", /*RequiredExtensions=*/"", /*PrototypeIndex=*/490, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_f_s", /*OverloadedName=*/"vfmv_f", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1233, /*SuffixIndex=*/1057, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_f_s", /*OverloadedName=*/"vfmv_f", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1233, /*SuffixIndex=*/1057, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_f_s", /*OverloadedName=*/"vfmv_f", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1233, /*SuffixIndex=*/1057, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_s_f", /*OverloadedName=*/"vfmv_s", /*RequiredExtensions=*/"", /*PrototypeIndex=*/1057, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_s_x", /*OverloadedName=*/"vfmv_s", /*RequiredExtensions=*/"", /*PrototypeIndex=*/490, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_s_f", /*OverloadedName=*/"vfmv_s", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1057, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_s_x", /*OverloadedName=*/"vfmv_s", /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/490, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfmv_s_f", /*OverloadedName=*/"vfmv_s", /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1057, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vslideup_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1180, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vslideup_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1128, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vslidedown_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1156, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vslidedown_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1108, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vslide1up_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vslide1up_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfslide1up_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfslide1up_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfslide1up_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vslide1down_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vslide1down_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfslide1down_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/192, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfslide1down_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfh", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/32, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vfslide1down_vf", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvfbfa", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrgather_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1176, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrgather_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1156, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrgatherei16_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/612, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrgather_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrgather_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1108, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrgatherei16_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/549, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vcompress_vm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1053, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcompress_vm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1045, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/179, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/152, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/176, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/149, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/173, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/146, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/170, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/143, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/167, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/140, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/164, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/137, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/161, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/134, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/456, /*SuffixIndex=*/102, /*OverloadedSuffixIndex=*/103, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/453, /*SuffixIndex=*/70, /*OverloadedSuffixIndex=*/71, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/450, /*SuffixIndex=*/100, /*OverloadedSuffixIndex=*/101, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/447, /*SuffixIndex=*/68, /*OverloadedSuffixIndex=*/69, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/444, /*SuffixIndex=*/98, /*OverloadedSuffixIndex=*/99, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/441, /*SuffixIndex=*/66, /*OverloadedSuffixIndex=*/67, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/591, /*SuffixIndex=*/420, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/528, /*SuffixIndex=*/399, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/588, /*SuffixIndex=*/1008, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/525, /*SuffixIndex=*/1000, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/585, /*SuffixIndex=*/1201, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/522, /*SuffixIndex=*/1196, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/582, /*SuffixIndex=*/1262, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/519, /*SuffixIndex=*/1256, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/579, /*SuffixIndex=*/1275, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/516, /*SuffixIndex=*/1268, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/576, /*SuffixIndex=*/1290, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/513, /*SuffixIndex=*/1282, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/573, /*SuffixIndex=*/1307, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vget_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/510, /*SuffixIndex=*/1298, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1040, /*SuffixIndex=*/114, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1036, /*SuffixIndex=*/82, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1032, /*SuffixIndex=*/112, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1028, /*SuffixIndex=*/80, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1024, /*SuffixIndex=*/110, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1020, /*SuffixIndex=*/78, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1016, /*SuffixIndex=*/422, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1012, /*SuffixIndex=*/401, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1004, /*SuffixIndex=*/588, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/996, /*SuffixIndex=*/525, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/992, /*SuffixIndex=*/585, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/988, /*SuffixIndex=*/522, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/984, /*SuffixIndex=*/582, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/980, /*SuffixIndex=*/519, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/976, /*SuffixIndex=*/579, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/972, /*SuffixIndex=*/516, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/968, /*SuffixIndex=*/576, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/964, /*SuffixIndex=*/513, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/960, /*SuffixIndex=*/573, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vset_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/956, /*SuffixIndex=*/510, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/474, /*SuffixIndex=*/26, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/471, /*SuffixIndex=*/20, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1221, /*SuffixIndex=*/24, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/468, /*SuffixIndex=*/14, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1216, /*SuffixIndex=*/18, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/465, /*SuffixIndex=*/10, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1325, /*SuffixIndex=*/22, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/9, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1211, /*SuffixIndex=*/12, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/462, /*SuffixIndex=*/6, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1316, /*SuffixIndex=*/16, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/9, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1206, /*SuffixIndex=*/8, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/459, /*SuffixIndex=*/4, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/1, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/420, /*SuffixIndex=*/179, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/399, /*SuffixIndex=*/152, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/2, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1008, /*SuffixIndex=*/176, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1000, /*SuffixIndex=*/149, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/3, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1201, /*SuffixIndex=*/173, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1196, /*SuffixIndex=*/146, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/5, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/4, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1262, /*SuffixIndex=*/170, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/6, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1256, /*SuffixIndex=*/143, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/6, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/5, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1275, /*SuffixIndex=*/167, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/7, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1268, /*SuffixIndex=*/140, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/7, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/6, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1290, /*SuffixIndex=*/164, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/8, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1282, /*SuffixIndex=*/137, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/8, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/7, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1307, /*SuffixIndex=*/161, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/9, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vcreate_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1298, /*SuffixIndex=*/134, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/9, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/8, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vandn_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vandn_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vbrev8_v", /*OverloadedName=*/"vbrev8", /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vbrev8_v", /*OverloadedName=*/"vbrev8", /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/65, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrev8_v", /*OverloadedName=*/"vrev8", /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrev8_v", /*OverloadedName=*/"vrev8", /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/65, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrol_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vrol_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/1108, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vror_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vror_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvkb", /*PrototypeIndex=*/1108, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vbrev_v", /*OverloadedName=*/"vbrev", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vbrev_v", /*OverloadedName=*/"vbrev", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/65, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclz_v", /*OverloadedName=*/"vclz", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclz_v", /*OverloadedName=*/"vclz", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/65, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vctz_v", /*OverloadedName=*/"vctz", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vctz_v", /*OverloadedName=*/"vctz", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/65, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vcpop_v", /*OverloadedName=*/"vcpop", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/421, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vcpop_v", /*OverloadedName=*/"vcpop", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/65, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsll_vv", /*OverloadedName=*/"vwsll", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/1065, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vwsll_vx", /*OverloadedName=*/"vwsll", /*RequiredExtensions=*/"zvbb", /*PrototypeIndex=*/486, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvbc", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclmul_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvbc", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclmul_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvbc", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclmul_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvbc", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclmulh_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvbc", /*PrototypeIndex=*/1009, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclmulh_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvbc", /*PrototypeIndex=*/1188, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclmulh_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvbc", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vclmulh_vx", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvbc", /*PrototypeIndex=*/1136, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/8, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vghsh_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvkg", /*PrototypeIndex=*/1197, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vgmul_vv", /*OverloadedName=*/"vgmul", /*RequiredExtensions=*/"zvkg", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdf_vv", /*OverloadedName=*/"vaesdf_vv", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdm_vv", /*OverloadedName=*/"vaesdm_vv", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesef_vv", /*OverloadedName=*/"vaesef_vv", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesem_vv", /*OverloadedName=*/"vaesem_vv", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaeskf1_vi", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/1125, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaeskf2_vi", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/1124, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsha2ch_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1197, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/12, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsha2cl_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1197, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/12, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsha2ms_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1197, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/12, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsm4k_vi", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvksed", /*PrototypeIndex=*/1125, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsm4r_vv", /*OverloadedName=*/"vsm4r_vv", /*RequiredExtensions=*/"zvksed", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsm3c_vi", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvksh", /*PrototypeIndex=*/1124, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsm3me_vv", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"zvksh", /*PrototypeIndex=*/1001, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vqdot_vv", /*OverloadedName=*/"vqdot", /*RequiredExtensions=*/"zvqdotq", /*PrototypeIndex=*/1168, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/124, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vqdot_vx", /*OverloadedName=*/"vqdot", /*RequiredExtensions=*/"zvqdotq", /*PrototypeIndex=*/1172, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/124, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vqdotu_vv", /*OverloadedName=*/"vqdotu", /*RequiredExtensions=*/"zvqdotq", /*PrototypeIndex=*/1112, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/124, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vqdotu_vx", /*OverloadedName=*/"vqdotu", /*RequiredExtensions=*/"zvqdotq", /*PrototypeIndex=*/1116, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/124, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vqdotsu_vv", /*OverloadedName=*/"vqdotsu", /*RequiredExtensions=*/"zvqdotq", /*PrototypeIndex=*/1164, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/124, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vqdotsu_vx", /*OverloadedName=*/"vqdotsu", /*RequiredExtensions=*/"zvqdotq", /*PrototypeIndex=*/1172, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/124, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vqdotus_vx", /*OverloadedName=*/"vqdotus", /*RequiredExtensions=*/"zvqdotq", /*PrototypeIndex=*/1160, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/4, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/124, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdf_vs", /*OverloadedName=*/"vaesdf_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/546, /*SuffixIndex=*/548, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdf_vs", /*OverloadedName=*/"vaesdf_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/543, /*SuffixIndex=*/545, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdf_vs", /*OverloadedName=*/"vaesdf_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/540, /*SuffixIndex=*/542, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdf_vs", /*OverloadedName=*/"vaesdf_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/537, /*SuffixIndex=*/539, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdm_vs", /*OverloadedName=*/"vaesdm_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/546, /*SuffixIndex=*/548, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdm_vs", /*OverloadedName=*/"vaesdm_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/543, /*SuffixIndex=*/545, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdm_vs", /*OverloadedName=*/"vaesdm_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/540, /*SuffixIndex=*/542, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesdm_vs", /*OverloadedName=*/"vaesdm_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/537, /*SuffixIndex=*/539, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesef_vs", /*OverloadedName=*/"vaesef_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/546, /*SuffixIndex=*/548, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesef_vs", /*OverloadedName=*/"vaesef_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/543, /*SuffixIndex=*/545, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesef_vs", /*OverloadedName=*/"vaesef_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/540, /*SuffixIndex=*/542, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesef_vs", /*OverloadedName=*/"vaesef_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/537, /*SuffixIndex=*/539, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesem_vs", /*OverloadedName=*/"vaesem_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/546, /*SuffixIndex=*/548, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesem_vs", /*OverloadedName=*/"vaesem_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/543, /*SuffixIndex=*/545, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesem_vs", /*OverloadedName=*/"vaesem_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/540, /*SuffixIndex=*/542, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesem_vs", /*OverloadedName=*/"vaesem_vs", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/537, /*SuffixIndex=*/539, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesz_vs", /*OverloadedName=*/"vaesz", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/546, /*SuffixIndex=*/548, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesz_vs", /*OverloadedName=*/"vaesz", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/543, /*SuffixIndex=*/545, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesz_vs", /*OverloadedName=*/"vaesz", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/540, /*SuffixIndex=*/542, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vaesz_vs", /*OverloadedName=*/"vaesz", /*RequiredExtensions=*/"zvkned", /*PrototypeIndex=*/537, /*SuffixIndex=*/539, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vcpop_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/2, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfirst_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/0, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vfwcvtbf16_f_f_v", /*OverloadedName=*/"vfwcvtbf16_f", /*RequiredExtensions=*/"zvfbfmin", /*PrototypeIndex=*/1227, /*SuffixIndex=*/58, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/16, /*Log2LMULMask=*/62, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vlenb", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/2, /*SuffixIndex=*/0, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/0, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/8, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlm_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/42, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/117, /*SuffixIndex=*/116, /*OverloadedSuffixIndex=*/22, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/119, /*SuffixIndex=*/118, /*OverloadedSuffixIndex=*/119, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/121, /*SuffixIndex=*/120, /*OverloadedSuffixIndex=*/121, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/115, /*SuffixIndex=*/114, /*OverloadedSuffixIndex=*/12, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/113, /*SuffixIndex=*/112, /*OverloadedSuffixIndex=*/6, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/111, /*SuffixIndex=*/110, /*OverloadedSuffixIndex=*/7, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/85, /*SuffixIndex=*/84, /*OverloadedSuffixIndex=*/16, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/87, /*SuffixIndex=*/86, /*OverloadedSuffixIndex=*/87, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/89, /*SuffixIndex=*/88, /*OverloadedSuffixIndex=*/89, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/83, /*SuffixIndex=*/82, /*OverloadedSuffixIndex=*/8, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/81, /*SuffixIndex=*/80, /*OverloadedSuffixIndex=*/4, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_ext_v", /*OverloadedName=*/"vlmul_ext", /*RequiredExtensions=*/"", /*PrototypeIndex=*/79, /*SuffixIndex=*/78, /*OverloadedSuffixIndex=*/5, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/105, /*SuffixIndex=*/104, /*OverloadedSuffixIndex=*/105, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/107, /*SuffixIndex=*/106, /*OverloadedSuffixIndex=*/107, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/109, /*SuffixIndex=*/108, /*OverloadedSuffixIndex=*/109, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/103, /*SuffixIndex=*/102, /*OverloadedSuffixIndex=*/103, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/101, /*SuffixIndex=*/100, /*OverloadedSuffixIndex=*/101, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/99, /*SuffixIndex=*/98, /*OverloadedSuffixIndex=*/99, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/73, /*SuffixIndex=*/72, /*OverloadedSuffixIndex=*/73, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/75, /*SuffixIndex=*/74, /*OverloadedSuffixIndex=*/75, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/77, /*SuffixIndex=*/76, /*OverloadedSuffixIndex=*/77, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/71, /*SuffixIndex=*/70, /*OverloadedSuffixIndex=*/71, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/69, /*SuffixIndex=*/68, /*OverloadedSuffixIndex=*/69, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vlmul_trunc_v", /*OverloadedName=*/"vlmul_trunc", /*RequiredExtensions=*/"", /*PrototypeIndex=*/67, /*SuffixIndex=*/66, /*OverloadedSuffixIndex=*/67, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vmand_mm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmandn_mm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmclr_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmnand_mm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmnor_mm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmor_mm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmorn_mm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmsbf_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmset_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmsif_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmsof_m", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/0, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPassthruOperand, },
{/*Name=*/"vmxnor_mm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vmxor_mm", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/477, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/39, /*SuffixIndex=*/38, /*OverloadedSuffixIndex=*/39, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/37, /*SuffixIndex=*/36, /*OverloadedSuffixIndex=*/37, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/35, /*SuffixIndex=*/34, /*OverloadedSuffixIndex=*/35, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/33, /*SuffixIndex=*/32, /*OverloadedSuffixIndex=*/33, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/31, /*SuffixIndex=*/30, /*OverloadedSuffixIndex=*/31, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/29, /*SuffixIndex=*/28, /*OverloadedSuffixIndex=*/29, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/609, /*SuffixIndex=*/507, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1078, /*SuffixIndex=*/552, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/97, /*SuffixIndex=*/128, /*OverloadedSuffixIndex=*/64, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/14, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/64, /*SuffixIndex=*/96, /*OverloadedSuffixIndex=*/64, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/14, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1171, /*SuffixIndex=*/1165, /*OverloadedSuffixIndex=*/1166, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/127, /*SuffixIndex=*/126, /*OverloadedSuffixIndex=*/127, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/125, /*SuffixIndex=*/124, /*OverloadedSuffixIndex=*/125, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/123, /*SuffixIndex=*/122, /*OverloadedSuffixIndex=*/123, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/507, /*SuffixIndex=*/609, /*OverloadedSuffixIndex=*/53, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/128, /*SuffixIndex=*/97, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/14, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1078, /*SuffixIndex=*/552, /*OverloadedSuffixIndex=*/47, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/38, /*SuffixIndex=*/39, /*OverloadedSuffixIndex=*/1, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/36, /*SuffixIndex=*/37, /*OverloadedSuffixIndex=*/1, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/34, /*SuffixIndex=*/35, /*OverloadedSuffixIndex=*/1, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/32, /*SuffixIndex=*/33, /*OverloadedSuffixIndex=*/1, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/30, /*SuffixIndex=*/31, /*OverloadedSuffixIndex=*/1, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/28, /*SuffixIndex=*/29, /*OverloadedSuffixIndex=*/1, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/1115, /*SuffixIndex=*/1113, /*OverloadedSuffixIndex=*/1114, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/95, /*SuffixIndex=*/94, /*OverloadedSuffixIndex=*/95, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/93, /*SuffixIndex=*/92, /*OverloadedSuffixIndex=*/93, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/91, /*SuffixIndex=*/90, /*OverloadedSuffixIndex=*/91, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/552, /*SuffixIndex=*/1078, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/16, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/96, /*SuffixIndex=*/64, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/14, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/552, /*SuffixIndex=*/1078, /*OverloadedSuffixIndex=*/45, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/62, /*SuffixIndex=*/61, /*OverloadedSuffixIndex=*/62, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/41, /*SuffixIndex=*/40, /*OverloadedSuffixIndex=*/41, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/61, /*SuffixIndex=*/62, /*OverloadedSuffixIndex=*/1, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vreinterpret_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/40, /*SuffixIndex=*/41, /*OverloadedSuffixIndex=*/1, /*PrototypeLength=*/2, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/1, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsext_vf2", /*OverloadedName=*/"vsext_vf2", /*RequiredExtensions=*/"", /*PrototypeIndex=*/495, /*SuffixIndex=*/131, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsext_vf4", /*OverloadedName=*/"vsext_vf4", /*RequiredExtensions=*/"", /*PrototypeIndex=*/50, /*SuffixIndex=*/50, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/3, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsext_vf8", /*OverloadedName=*/"vsext_vf8", /*RequiredExtensions=*/"", /*PrototypeIndex=*/46, /*SuffixIndex=*/46, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/15, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsm_v", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/153, /*SuffixIndex=*/1, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/0, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vsm4r_vs", /*OverloadedName=*/"vsm4r_vs", /*RequiredExtensions=*/"zvksed", /*PrototypeIndex=*/546, /*SuffixIndex=*/548, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsm4r_vs", /*OverloadedName=*/"vsm4r_vs", /*RequiredExtensions=*/"zvksed", /*PrototypeIndex=*/543, /*SuffixIndex=*/545, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsm4r_vs", /*OverloadedName=*/"vsm4r_vs", /*RequiredExtensions=*/"zvksed", /*PrototypeIndex=*/540, /*SuffixIndex=*/542, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vsm4r_vs", /*OverloadedName=*/"vsm4r_vs", /*RequiredExtensions=*/"zvksed", /*PrototypeIndex=*/537, /*SuffixIndex=*/539, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/3, /*SuffixLength=*/2, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/4, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPolicyOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/47, /*SuffixIndex=*/47, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/255, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vundefined", /*OverloadedName=*/nullptr, /*RequiredExtensions=*/"", /*PrototypeIndex=*/45, /*SuffixIndex=*/45, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/1, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/15, /*Log2LMULMask=*/127, /*NF=*/1, /*HasMasked=*/0, /*HasVL=*/0, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/SchemeNone, /*MaskedPolicyScheme=*/SchemeNone, },
{/*Name=*/"vzext_vf2", /*OverloadedName=*/"vzext_vf2", /*RequiredExtensions=*/"", /*PrototypeIndex=*/486, /*SuffixIndex=*/486, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/7, /*Log2LMULMask=*/63, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vzext_vf4", /*OverloadedName=*/"vzext_vf4", /*RequiredExtensions=*/"", /*PrototypeIndex=*/48, /*SuffixIndex=*/48, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/3, /*Log2LMULMask=*/31, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
{/*Name=*/"vzext_vf8", /*OverloadedName=*/"vzext_vf8", /*RequiredExtensions=*/"", /*PrototypeIndex=*/44, /*SuffixIndex=*/44, /*OverloadedSuffixIndex=*/0, /*PrototypeLength=*/2, /*SuffixLength=*/1, /*OverloadedSuffixSize=*/0, /*TypeRangeMask=*/1, /*Log2LMULMask=*/15, /*NF=*/1, /*HasMasked=*/1, /*HasVL=*/1, /*HasMaskedOffOperand=*/1, /*HasTailPolicy=*/1, /*HasMaskPolicy=*/1, /*HasFRMRoundModeOp=*/0, /*AltFmt=*/0,/*IsTuple=*/0, /*UnMaskedPolicyScheme=*/HasPassthruOperand, /*MaskedPolicyScheme=*/HasPolicyOperand, },
#endif
