#ifdef GET_SME_BUILTIN_ENUMERATORS
  BI__builtin_sme___arm_in_streaming_mode,
  BI__builtin_sme_svadd_write_single_za32_s32_vg1x2,
  BI__builtin_sme_svadd_write_single_za32_s32_vg1x4,
  BI__builtin_sme_svadd_write_single_za32_u32_vg1x2,
  BI__builtin_sme_svadd_write_single_za32_u32_vg1x4,
  BI__builtin_sme_svadd_write_single_za64_s64_vg1x2,
  BI__builtin_sme_svadd_write_single_za64_s64_vg1x4,
  BI__builtin_sme_svadd_write_single_za64_u64_vg1x2,
  BI__builtin_sme_svadd_write_single_za64_u64_vg1x4,
  BI__builtin_sme_svadd_write_za32_s32_vg1x2,
  BI__builtin_sme_svadd_write_za32_s32_vg1x4,
  BI__builtin_sme_svadd_write_za32_u32_vg1x2,
  BI__builtin_sme_svadd_write_za32_u32_vg1x4,
  BI__builtin_sme_svadd_write_za64_s64_vg1x2,
  BI__builtin_sme_svadd_write_za64_s64_vg1x4,
  BI__builtin_sme_svadd_write_za64_u64_vg1x2,
  BI__builtin_sme_svadd_write_za64_u64_vg1x4,
  BI__builtin_sme_svadd_za16_bf16_vg1x2,
  BI__builtin_sme_svadd_za16_bf16_vg1x4,
  BI__builtin_sme_svadd_za16_f16_vg1x2,
  BI__builtin_sme_svadd_za16_f16_vg1x4,
  BI__builtin_sme_svadd_za32_f32_vg1x2,
  BI__builtin_sme_svadd_za32_f32_vg1x4,
  BI__builtin_sme_svadd_za32_s32_vg1x2,
  BI__builtin_sme_svadd_za32_s32_vg1x4,
  BI__builtin_sme_svadd_za32_u32_vg1x2,
  BI__builtin_sme_svadd_za32_u32_vg1x4,
  BI__builtin_sme_svadd_za64_f64_vg1x2,
  BI__builtin_sme_svadd_za64_f64_vg1x4,
  BI__builtin_sme_svadd_za64_s64_vg1x2,
  BI__builtin_sme_svadd_za64_s64_vg1x4,
  BI__builtin_sme_svadd_za64_u64_vg1x2,
  BI__builtin_sme_svadd_za64_u64_vg1x4,
  BI__builtin_sme_svaddha_za32_s32_m,
  BI__builtin_sme_svaddha_za32_u32_m,
  BI__builtin_sme_svaddha_za64_s64_m,
  BI__builtin_sme_svaddha_za64_u64_m,
  BI__builtin_sme_svaddva_za32_s32_m,
  BI__builtin_sme_svaddva_za32_u32_m,
  BI__builtin_sme_svaddva_za64_s64_m,
  BI__builtin_sme_svaddva_za64_u64_m,
  BI__builtin_sme_svbmopa_za32_s32_m,
  BI__builtin_sme_svbmopa_za32_u32_m,
  BI__builtin_sme_svbmops_za32_s32_m,
  BI__builtin_sme_svbmops_za32_u32_m,
  BI__builtin_sme_svcntsb,
  BI__builtin_sme_svcntsd,
  BI__builtin_sme_svcntsh,
  BI__builtin_sme_svcntsw,
  BI__builtin_sme_svdot_lane_za16_mf8_vg1x2_fpm,
  BI__builtin_sme_svdot_lane_za16_mf8_vg1x4_fpm,
  BI__builtin_sme_svdot_lane_za32_bf16_vg1x2,
  BI__builtin_sme_svdot_lane_za32_bf16_vg1x4,
  BI__builtin_sme_svdot_lane_za32_f16_vg1x2,
  BI__builtin_sme_svdot_lane_za32_f16_vg1x4,
  BI__builtin_sme_svdot_lane_za32_mf8_vg1x2_fpm,
  BI__builtin_sme_svdot_lane_za32_mf8_vg1x4_fpm,
  BI__builtin_sme_svdot_lane_za32_s16_vg1x2,
  BI__builtin_sme_svdot_lane_za32_s16_vg1x4,
  BI__builtin_sme_svdot_lane_za32_s8_vg1x2,
  BI__builtin_sme_svdot_lane_za32_s8_vg1x4,
  BI__builtin_sme_svdot_lane_za32_u16_vg1x2,
  BI__builtin_sme_svdot_lane_za32_u16_vg1x4,
  BI__builtin_sme_svdot_lane_za32_u8_vg1x2,
  BI__builtin_sme_svdot_lane_za32_u8_vg1x4,
  BI__builtin_sme_svdot_lane_za64_s16_vg1x2,
  BI__builtin_sme_svdot_lane_za64_s16_vg1x4,
  BI__builtin_sme_svdot_lane_za64_u16_vg1x2,
  BI__builtin_sme_svdot_lane_za64_u16_vg1x4,
  BI__builtin_sme_svdot_single_za16_mf8_vg1x2_fpm,
  BI__builtin_sme_svdot_single_za16_mf8_vg1x4_fpm,
  BI__builtin_sme_svdot_single_za32_bf16_vg1x2,
  BI__builtin_sme_svdot_single_za32_bf16_vg1x4,
  BI__builtin_sme_svdot_single_za32_f16_vg1x2,
  BI__builtin_sme_svdot_single_za32_f16_vg1x4,
  BI__builtin_sme_svdot_single_za32_mf8_vg1x2_fpm,
  BI__builtin_sme_svdot_single_za32_mf8_vg1x4_fpm,
  BI__builtin_sme_svdot_single_za32_s16_vg1x2,
  BI__builtin_sme_svdot_single_za32_s16_vg1x4,
  BI__builtin_sme_svdot_single_za32_s8_vg1x2,
  BI__builtin_sme_svdot_single_za32_s8_vg1x4,
  BI__builtin_sme_svdot_single_za32_u16_vg1x2,
  BI__builtin_sme_svdot_single_za32_u16_vg1x4,
  BI__builtin_sme_svdot_single_za32_u8_vg1x2,
  BI__builtin_sme_svdot_single_za32_u8_vg1x4,
  BI__builtin_sme_svdot_single_za64_s16_vg1x2,
  BI__builtin_sme_svdot_single_za64_s16_vg1x4,
  BI__builtin_sme_svdot_single_za64_u16_vg1x2,
  BI__builtin_sme_svdot_single_za64_u16_vg1x4,
  BI__builtin_sme_svdot_za16_mf8_vg1x2_fpm,
  BI__builtin_sme_svdot_za16_mf8_vg1x4_fpm,
  BI__builtin_sme_svdot_za32_bf16_vg1x2,
  BI__builtin_sme_svdot_za32_bf16_vg1x4,
  BI__builtin_sme_svdot_za32_f16_vg1x2,
  BI__builtin_sme_svdot_za32_f16_vg1x4,
  BI__builtin_sme_svdot_za32_mf8_vg1x2_fpm,
  BI__builtin_sme_svdot_za32_mf8_vg1x4_fpm,
  BI__builtin_sme_svdot_za32_s16_vg1x2,
  BI__builtin_sme_svdot_za32_s16_vg1x4,
  BI__builtin_sme_svdot_za32_s8_vg1x2,
  BI__builtin_sme_svdot_za32_s8_vg1x4,
  BI__builtin_sme_svdot_za32_u16_vg1x2,
  BI__builtin_sme_svdot_za32_u16_vg1x4,
  BI__builtin_sme_svdot_za32_u8_vg1x2,
  BI__builtin_sme_svdot_za32_u8_vg1x4,
  BI__builtin_sme_svdot_za64_s16_vg1x2,
  BI__builtin_sme_svdot_za64_s16_vg1x4,
  BI__builtin_sme_svdot_za64_u16_vg1x2,
  BI__builtin_sme_svdot_za64_u16_vg1x4,
  BI__builtin_sme_svld1_hor_vnum_za128,
  BI__builtin_sme_svld1_hor_vnum_za16,
  BI__builtin_sme_svld1_hor_vnum_za32,
  BI__builtin_sme_svld1_hor_vnum_za64,
  BI__builtin_sme_svld1_hor_vnum_za8,
  BI__builtin_sme_svld1_hor_za128,
  BI__builtin_sme_svld1_hor_za16,
  BI__builtin_sme_svld1_hor_za32,
  BI__builtin_sme_svld1_hor_za64,
  BI__builtin_sme_svld1_hor_za8,
  BI__builtin_sme_svld1_ver_vnum_za128,
  BI__builtin_sme_svld1_ver_vnum_za16,
  BI__builtin_sme_svld1_ver_vnum_za32,
  BI__builtin_sme_svld1_ver_vnum_za64,
  BI__builtin_sme_svld1_ver_vnum_za8,
  BI__builtin_sme_svld1_ver_za128,
  BI__builtin_sme_svld1_ver_za16,
  BI__builtin_sme_svld1_ver_za32,
  BI__builtin_sme_svld1_ver_za64,
  BI__builtin_sme_svld1_ver_za8,
  BI__builtin_sme_svldr_vnum_za,
  BI__builtin_sme_svldr_za,
  BI__builtin_sme_svldr_zt,
  BI__builtin_sme_svluti2_lane_zt_bf16,
  BI__builtin_sme_svluti2_lane_zt_bf16_x2,
  BI__builtin_sme_svluti2_lane_zt_bf16_x4,
  BI__builtin_sme_svluti2_lane_zt_f16,
  BI__builtin_sme_svluti2_lane_zt_f16_x2,
  BI__builtin_sme_svluti2_lane_zt_f16_x4,
  BI__builtin_sme_svluti2_lane_zt_f32,
  BI__builtin_sme_svluti2_lane_zt_f32_x2,
  BI__builtin_sme_svluti2_lane_zt_f32_x4,
  BI__builtin_sme_svluti2_lane_zt_mf8,
  BI__builtin_sme_svluti2_lane_zt_mf8_x2,
  BI__builtin_sme_svluti2_lane_zt_mf8_x4,
  BI__builtin_sme_svluti2_lane_zt_s16,
  BI__builtin_sme_svluti2_lane_zt_s16_x2,
  BI__builtin_sme_svluti2_lane_zt_s16_x4,
  BI__builtin_sme_svluti2_lane_zt_s32,
  BI__builtin_sme_svluti2_lane_zt_s32_x2,
  BI__builtin_sme_svluti2_lane_zt_s32_x4,
  BI__builtin_sme_svluti2_lane_zt_s8,
  BI__builtin_sme_svluti2_lane_zt_s8_x2,
  BI__builtin_sme_svluti2_lane_zt_s8_x4,
  BI__builtin_sme_svluti2_lane_zt_u16,
  BI__builtin_sme_svluti2_lane_zt_u16_x2,
  BI__builtin_sme_svluti2_lane_zt_u16_x4,
  BI__builtin_sme_svluti2_lane_zt_u32,
  BI__builtin_sme_svluti2_lane_zt_u32_x2,
  BI__builtin_sme_svluti2_lane_zt_u32_x4,
  BI__builtin_sme_svluti2_lane_zt_u8,
  BI__builtin_sme_svluti2_lane_zt_u8_x2,
  BI__builtin_sme_svluti2_lane_zt_u8_x4,
  BI__builtin_sme_svluti4_lane_zt_bf16,
  BI__builtin_sme_svluti4_lane_zt_bf16_x2,
  BI__builtin_sme_svluti4_lane_zt_bf16_x4,
  BI__builtin_sme_svluti4_lane_zt_f16,
  BI__builtin_sme_svluti4_lane_zt_f16_x2,
  BI__builtin_sme_svluti4_lane_zt_f16_x4,
  BI__builtin_sme_svluti4_lane_zt_f32,
  BI__builtin_sme_svluti4_lane_zt_f32_x2,
  BI__builtin_sme_svluti4_lane_zt_f32_x4,
  BI__builtin_sme_svluti4_lane_zt_mf8,
  BI__builtin_sme_svluti4_lane_zt_mf8_x2,
  BI__builtin_sme_svluti4_lane_zt_s16,
  BI__builtin_sme_svluti4_lane_zt_s16_x2,
  BI__builtin_sme_svluti4_lane_zt_s16_x4,
  BI__builtin_sme_svluti4_lane_zt_s32,
  BI__builtin_sme_svluti4_lane_zt_s32_x2,
  BI__builtin_sme_svluti4_lane_zt_s32_x4,
  BI__builtin_sme_svluti4_lane_zt_s8,
  BI__builtin_sme_svluti4_lane_zt_s8_x2,
  BI__builtin_sme_svluti4_lane_zt_u16,
  BI__builtin_sme_svluti4_lane_zt_u16_x2,
  BI__builtin_sme_svluti4_lane_zt_u16_x4,
  BI__builtin_sme_svluti4_lane_zt_u32,
  BI__builtin_sme_svluti4_lane_zt_u32_x2,
  BI__builtin_sme_svluti4_lane_zt_u32_x4,
  BI__builtin_sme_svluti4_lane_zt_u8,
  BI__builtin_sme_svluti4_lane_zt_u8_x2,
  BI__builtin_sme_svluti4_zt_s8_x4,
  BI__builtin_sme_svluti4_zt_u8_x4,
  BI__builtin_sme_svmla_lane_za16_bf16_vg1x2,
  BI__builtin_sme_svmla_lane_za16_bf16_vg1x4,
  BI__builtin_sme_svmla_lane_za16_f16_vg1x2,
  BI__builtin_sme_svmla_lane_za16_f16_vg1x4,
  BI__builtin_sme_svmla_lane_za16_mf8_vg2x1_fpm,
  BI__builtin_sme_svmla_lane_za16_mf8_vg2x2_fpm,
  BI__builtin_sme_svmla_lane_za16_mf8_vg2x4_fpm,
  BI__builtin_sme_svmla_lane_za32_bf16_vg2x1,
  BI__builtin_sme_svmla_lane_za32_bf16_vg2x2,
  BI__builtin_sme_svmla_lane_za32_bf16_vg2x4,
  BI__builtin_sme_svmla_lane_za32_f16_vg2x1,
  BI__builtin_sme_svmla_lane_za32_f16_vg2x2,
  BI__builtin_sme_svmla_lane_za32_f16_vg2x4,
  BI__builtin_sme_svmla_lane_za32_f32_vg1x2,
  BI__builtin_sme_svmla_lane_za32_f32_vg1x4,
  BI__builtin_sme_svmla_lane_za32_mf8_vg4x1_fpm,
  BI__builtin_sme_svmla_lane_za32_mf8_vg4x2_fpm,
  BI__builtin_sme_svmla_lane_za32_mf8_vg4x4_fpm,
  BI__builtin_sme_svmla_lane_za32_s16_vg2x1,
  BI__builtin_sme_svmla_lane_za32_s16_vg2x2,
  BI__builtin_sme_svmla_lane_za32_s16_vg2x4,
  BI__builtin_sme_svmla_lane_za32_s8_vg4x1,
  BI__builtin_sme_svmla_lane_za32_s8_vg4x2,
  BI__builtin_sme_svmla_lane_za32_s8_vg4x4,
  BI__builtin_sme_svmla_lane_za32_u16_vg2x1,
  BI__builtin_sme_svmla_lane_za32_u16_vg2x2,
  BI__builtin_sme_svmla_lane_za32_u16_vg2x4,
  BI__builtin_sme_svmla_lane_za32_u8_vg4x1,
  BI__builtin_sme_svmla_lane_za32_u8_vg4x2,
  BI__builtin_sme_svmla_lane_za32_u8_vg4x4,
  BI__builtin_sme_svmla_lane_za64_f64_vg1x2,
  BI__builtin_sme_svmla_lane_za64_f64_vg1x4,
  BI__builtin_sme_svmla_lane_za64_s16_vg4x1,
  BI__builtin_sme_svmla_lane_za64_s16_vg4x2,
  BI__builtin_sme_svmla_lane_za64_s16_vg4x4,
  BI__builtin_sme_svmla_lane_za64_u16_vg4x1,
  BI__builtin_sme_svmla_lane_za64_u16_vg4x2,
  BI__builtin_sme_svmla_lane_za64_u16_vg4x4,
  BI__builtin_sme_svmla_single_za16_bf16_vg1x2,
  BI__builtin_sme_svmla_single_za16_bf16_vg1x4,
  BI__builtin_sme_svmla_single_za16_f16_vg1x2,
  BI__builtin_sme_svmla_single_za16_f16_vg1x4,
  BI__builtin_sme_svmla_single_za16_mf8_vg2x2_fpm,
  BI__builtin_sme_svmla_single_za16_mf8_vg2x4_fpm,
  BI__builtin_sme_svmla_single_za32_bf16_vg2x2,
  BI__builtin_sme_svmla_single_za32_bf16_vg2x4,
  BI__builtin_sme_svmla_single_za32_f16_vg2x2,
  BI__builtin_sme_svmla_single_za32_f16_vg2x4,
  BI__builtin_sme_svmla_single_za32_f32_vg1x2,
  BI__builtin_sme_svmla_single_za32_f32_vg1x4,
  BI__builtin_sme_svmla_single_za32_mf8_vg4x2_fpm,
  BI__builtin_sme_svmla_single_za32_mf8_vg4x4_fpm,
  BI__builtin_sme_svmla_single_za32_s16_vg2x2,
  BI__builtin_sme_svmla_single_za32_s16_vg2x4,
  BI__builtin_sme_svmla_single_za32_s8_vg4x2,
  BI__builtin_sme_svmla_single_za32_s8_vg4x4,
  BI__builtin_sme_svmla_single_za32_u16_vg2x2,
  BI__builtin_sme_svmla_single_za32_u16_vg2x4,
  BI__builtin_sme_svmla_single_za32_u8_vg4x2,
  BI__builtin_sme_svmla_single_za32_u8_vg4x4,
  BI__builtin_sme_svmla_single_za64_f64_vg1x2,
  BI__builtin_sme_svmla_single_za64_f64_vg1x4,
  BI__builtin_sme_svmla_single_za64_s16_vg4x2,
  BI__builtin_sme_svmla_single_za64_s16_vg4x4,
  BI__builtin_sme_svmla_single_za64_u16_vg4x2,
  BI__builtin_sme_svmla_single_za64_u16_vg4x4,
  BI__builtin_sme_svmla_za16_bf16_vg1x2,
  BI__builtin_sme_svmla_za16_bf16_vg1x4,
  BI__builtin_sme_svmla_za16_f16_vg1x2,
  BI__builtin_sme_svmla_za16_f16_vg1x4,
  BI__builtin_sme_svmla_za16_mf8_vg2x1_fpm,
  BI__builtin_sme_svmla_za16_mf8_vg2x2_fpm,
  BI__builtin_sme_svmla_za16_mf8_vg2x4_fpm,
  BI__builtin_sme_svmla_za32_bf16_vg2x1,
  BI__builtin_sme_svmla_za32_bf16_vg2x2,
  BI__builtin_sme_svmla_za32_bf16_vg2x4,
  BI__builtin_sme_svmla_za32_f16_vg2x1,
  BI__builtin_sme_svmla_za32_f16_vg2x2,
  BI__builtin_sme_svmla_za32_f16_vg2x4,
  BI__builtin_sme_svmla_za32_f32_vg1x2,
  BI__builtin_sme_svmla_za32_f32_vg1x4,
  BI__builtin_sme_svmla_za32_mf8_vg4x1_fpm,
  BI__builtin_sme_svmla_za32_mf8_vg4x2_fpm,
  BI__builtin_sme_svmla_za32_mf8_vg4x4_fpm,
  BI__builtin_sme_svmla_za32_s16_vg2x1,
  BI__builtin_sme_svmla_za32_s16_vg2x2,
  BI__builtin_sme_svmla_za32_s16_vg2x4,
  BI__builtin_sme_svmla_za32_s8_vg4x1,
  BI__builtin_sme_svmla_za32_s8_vg4x2,
  BI__builtin_sme_svmla_za32_s8_vg4x4,
  BI__builtin_sme_svmla_za32_u16_vg2x1,
  BI__builtin_sme_svmla_za32_u16_vg2x2,
  BI__builtin_sme_svmla_za32_u16_vg2x4,
  BI__builtin_sme_svmla_za32_u8_vg4x1,
  BI__builtin_sme_svmla_za32_u8_vg4x2,
  BI__builtin_sme_svmla_za32_u8_vg4x4,
  BI__builtin_sme_svmla_za64_f64_vg1x2,
  BI__builtin_sme_svmla_za64_f64_vg1x4,
  BI__builtin_sme_svmla_za64_s16_vg4x1,
  BI__builtin_sme_svmla_za64_s16_vg4x2,
  BI__builtin_sme_svmla_za64_s16_vg4x4,
  BI__builtin_sme_svmla_za64_u16_vg4x1,
  BI__builtin_sme_svmla_za64_u16_vg4x2,
  BI__builtin_sme_svmla_za64_u16_vg4x4,
  BI__builtin_sme_svmls_lane_za16_bf16_vg1x2,
  BI__builtin_sme_svmls_lane_za16_bf16_vg1x4,
  BI__builtin_sme_svmls_lane_za16_f16_vg1x2,
  BI__builtin_sme_svmls_lane_za16_f16_vg1x4,
  BI__builtin_sme_svmls_lane_za32_bf16_vg2x1,
  BI__builtin_sme_svmls_lane_za32_bf16_vg2x2,
  BI__builtin_sme_svmls_lane_za32_bf16_vg2x4,
  BI__builtin_sme_svmls_lane_za32_f16_vg2x1,
  BI__builtin_sme_svmls_lane_za32_f16_vg2x2,
  BI__builtin_sme_svmls_lane_za32_f16_vg2x4,
  BI__builtin_sme_svmls_lane_za32_f32_vg1x2,
  BI__builtin_sme_svmls_lane_za32_f32_vg1x4,
  BI__builtin_sme_svmls_lane_za32_s16_vg2x1,
  BI__builtin_sme_svmls_lane_za32_s16_vg2x2,
  BI__builtin_sme_svmls_lane_za32_s16_vg2x4,
  BI__builtin_sme_svmls_lane_za32_s8_vg4x1,
  BI__builtin_sme_svmls_lane_za32_s8_vg4x2,
  BI__builtin_sme_svmls_lane_za32_s8_vg4x4,
  BI__builtin_sme_svmls_lane_za32_u16_vg2x1,
  BI__builtin_sme_svmls_lane_za32_u16_vg2x2,
  BI__builtin_sme_svmls_lane_za32_u16_vg2x4,
  BI__builtin_sme_svmls_lane_za32_u8_vg4x1,
  BI__builtin_sme_svmls_lane_za32_u8_vg4x2,
  BI__builtin_sme_svmls_lane_za32_u8_vg4x4,
  BI__builtin_sme_svmls_lane_za64_f64_vg1x2,
  BI__builtin_sme_svmls_lane_za64_f64_vg1x4,
  BI__builtin_sme_svmls_lane_za64_s16_vg4x1,
  BI__builtin_sme_svmls_lane_za64_s16_vg4x2,
  BI__builtin_sme_svmls_lane_za64_s16_vg4x4,
  BI__builtin_sme_svmls_lane_za64_u16_vg4x1,
  BI__builtin_sme_svmls_lane_za64_u16_vg4x2,
  BI__builtin_sme_svmls_lane_za64_u16_vg4x4,
  BI__builtin_sme_svmls_single_za16_bf16_vg1x2,
  BI__builtin_sme_svmls_single_za16_bf16_vg1x4,
  BI__builtin_sme_svmls_single_za16_f16_vg1x2,
  BI__builtin_sme_svmls_single_za16_f16_vg1x4,
  BI__builtin_sme_svmls_single_za32_bf16_vg2x2,
  BI__builtin_sme_svmls_single_za32_bf16_vg2x4,
  BI__builtin_sme_svmls_single_za32_f16_vg2x2,
  BI__builtin_sme_svmls_single_za32_f16_vg2x4,
  BI__builtin_sme_svmls_single_za32_f32_vg1x2,
  BI__builtin_sme_svmls_single_za32_f32_vg1x4,
  BI__builtin_sme_svmls_single_za32_s16_vg2x2,
  BI__builtin_sme_svmls_single_za32_s16_vg2x4,
  BI__builtin_sme_svmls_single_za32_s8_vg4x2,
  BI__builtin_sme_svmls_single_za32_s8_vg4x4,
  BI__builtin_sme_svmls_single_za32_u16_vg2x2,
  BI__builtin_sme_svmls_single_za32_u16_vg2x4,
  BI__builtin_sme_svmls_single_za32_u8_vg4x2,
  BI__builtin_sme_svmls_single_za32_u8_vg4x4,
  BI__builtin_sme_svmls_single_za64_f64_vg1x2,
  BI__builtin_sme_svmls_single_za64_f64_vg1x4,
  BI__builtin_sme_svmls_single_za64_s16_vg4x2,
  BI__builtin_sme_svmls_single_za64_s16_vg4x4,
  BI__builtin_sme_svmls_single_za64_u16_vg4x2,
  BI__builtin_sme_svmls_single_za64_u16_vg4x4,
  BI__builtin_sme_svmls_za16_bf16_vg1x2,
  BI__builtin_sme_svmls_za16_bf16_vg1x4,
  BI__builtin_sme_svmls_za16_f16_vg1x2,
  BI__builtin_sme_svmls_za16_f16_vg1x4,
  BI__builtin_sme_svmls_za32_bf16_vg2x1,
  BI__builtin_sme_svmls_za32_bf16_vg2x2,
  BI__builtin_sme_svmls_za32_bf16_vg2x4,
  BI__builtin_sme_svmls_za32_f16_vg2x1,
  BI__builtin_sme_svmls_za32_f16_vg2x2,
  BI__builtin_sme_svmls_za32_f16_vg2x4,
  BI__builtin_sme_svmls_za32_f32_vg1x2,
  BI__builtin_sme_svmls_za32_f32_vg1x4,
  BI__builtin_sme_svmls_za32_s16_vg2x1,
  BI__builtin_sme_svmls_za32_s16_vg2x2,
  BI__builtin_sme_svmls_za32_s16_vg2x4,
  BI__builtin_sme_svmls_za32_s8_vg4x1,
  BI__builtin_sme_svmls_za32_s8_vg4x2,
  BI__builtin_sme_svmls_za32_s8_vg4x4,
  BI__builtin_sme_svmls_za32_u16_vg2x1,
  BI__builtin_sme_svmls_za32_u16_vg2x2,
  BI__builtin_sme_svmls_za32_u16_vg2x4,
  BI__builtin_sme_svmls_za32_u8_vg4x1,
  BI__builtin_sme_svmls_za32_u8_vg4x2,
  BI__builtin_sme_svmls_za32_u8_vg4x4,
  BI__builtin_sme_svmls_za64_f64_vg1x2,
  BI__builtin_sme_svmls_za64_f64_vg1x4,
  BI__builtin_sme_svmls_za64_s16_vg4x1,
  BI__builtin_sme_svmls_za64_s16_vg4x2,
  BI__builtin_sme_svmls_za64_s16_vg4x4,
  BI__builtin_sme_svmls_za64_u16_vg4x1,
  BI__builtin_sme_svmls_za64_u16_vg4x2,
  BI__builtin_sme_svmls_za64_u16_vg4x4,
  BI__builtin_sme_svmop4a_1x1_za16_bf16_bf16,
  BI__builtin_sme_svmop4a_1x1_za16_f16_f16,
  BI__builtin_sme_svmop4a_1x1_za16_mf8_mf8_fpm,
  BI__builtin_sme_svmop4a_1x1_za32_bf16_bf16,
  BI__builtin_sme_svmop4a_1x1_za32_f16_f16,
  BI__builtin_sme_svmop4a_1x1_za32_f32_f32,
  BI__builtin_sme_svmop4a_1x1_za32_mf8_mf8_fpm,
  BI__builtin_sme_svmop4a_1x1_za32_s16_s16,
  BI__builtin_sme_svmop4a_1x1_za32_s8_s8,
  BI__builtin_sme_svmop4a_1x1_za32_s8_u8,
  BI__builtin_sme_svmop4a_1x1_za32_u16_u16,
  BI__builtin_sme_svmop4a_1x1_za32_u8_s8,
  BI__builtin_sme_svmop4a_1x1_za32_u8_u8,
  BI__builtin_sme_svmop4a_1x1_za64_f64_f64,
  BI__builtin_sme_svmop4a_1x1_za64_s16_s16,
  BI__builtin_sme_svmop4a_1x1_za64_s16_u16,
  BI__builtin_sme_svmop4a_1x1_za64_u16_s16,
  BI__builtin_sme_svmop4a_1x1_za64_u16_u16,
  BI__builtin_sme_svmop4a_1x2_za16_bf16_bf16,
  BI__builtin_sme_svmop4a_1x2_za16_f16_f16,
  BI__builtin_sme_svmop4a_1x2_za16_mf8_mf8_fpm,
  BI__builtin_sme_svmop4a_1x2_za32_bf16_bf16,
  BI__builtin_sme_svmop4a_1x2_za32_f16_f16,
  BI__builtin_sme_svmop4a_1x2_za32_f32_f32,
  BI__builtin_sme_svmop4a_1x2_za32_mf8_mf8_fpm,
  BI__builtin_sme_svmop4a_1x2_za32_s16_s16,
  BI__builtin_sme_svmop4a_1x2_za32_s8_s8,
  BI__builtin_sme_svmop4a_1x2_za32_s8_u8,
  BI__builtin_sme_svmop4a_1x2_za32_u16_u16,
  BI__builtin_sme_svmop4a_1x2_za32_u8_s8,
  BI__builtin_sme_svmop4a_1x2_za32_u8_u8,
  BI__builtin_sme_svmop4a_1x2_za64_f64_f64,
  BI__builtin_sme_svmop4a_1x2_za64_s16_s16,
  BI__builtin_sme_svmop4a_1x2_za64_s16_u16,
  BI__builtin_sme_svmop4a_1x2_za64_u16_s16,
  BI__builtin_sme_svmop4a_1x2_za64_u16_u16,
  BI__builtin_sme_svmop4a_2x1_za16_bf16_bf16,
  BI__builtin_sme_svmop4a_2x1_za16_f16_f16,
  BI__builtin_sme_svmop4a_2x1_za16_mf8_mf8_fpm,
  BI__builtin_sme_svmop4a_2x1_za32_bf16_bf16,
  BI__builtin_sme_svmop4a_2x1_za32_f16_f16,
  BI__builtin_sme_svmop4a_2x1_za32_f32_f32,
  BI__builtin_sme_svmop4a_2x1_za32_mf8_mf8_fpm,
  BI__builtin_sme_svmop4a_2x1_za32_s16_s16,
  BI__builtin_sme_svmop4a_2x1_za32_s8_s8,
  BI__builtin_sme_svmop4a_2x1_za32_s8_u8,
  BI__builtin_sme_svmop4a_2x1_za32_u16_u16,
  BI__builtin_sme_svmop4a_2x1_za32_u8_s8,
  BI__builtin_sme_svmop4a_2x1_za32_u8_u8,
  BI__builtin_sme_svmop4a_2x1_za64_f64_f64,
  BI__builtin_sme_svmop4a_2x1_za64_s16_s16,
  BI__builtin_sme_svmop4a_2x1_za64_s16_u16,
  BI__builtin_sme_svmop4a_2x1_za64_u16_s16,
  BI__builtin_sme_svmop4a_2x1_za64_u16_u16,
  BI__builtin_sme_svmop4a_2x2_za16_bf16_bf16,
  BI__builtin_sme_svmop4a_2x2_za16_f16_f16,
  BI__builtin_sme_svmop4a_2x2_za16_mf8_mf8_fpm,
  BI__builtin_sme_svmop4a_2x2_za32_bf16_bf16,
  BI__builtin_sme_svmop4a_2x2_za32_f16_f16,
  BI__builtin_sme_svmop4a_2x2_za32_f32_f32,
  BI__builtin_sme_svmop4a_2x2_za32_mf8_mf8_fpm,
  BI__builtin_sme_svmop4a_2x2_za32_s16_s16,
  BI__builtin_sme_svmop4a_2x2_za32_s8_s8,
  BI__builtin_sme_svmop4a_2x2_za32_s8_u8,
  BI__builtin_sme_svmop4a_2x2_za32_u16_u16,
  BI__builtin_sme_svmop4a_2x2_za32_u8_s8,
  BI__builtin_sme_svmop4a_2x2_za32_u8_u8,
  BI__builtin_sme_svmop4a_2x2_za64_f64_f64,
  BI__builtin_sme_svmop4a_2x2_za64_s16_s16,
  BI__builtin_sme_svmop4a_2x2_za64_s16_u16,
  BI__builtin_sme_svmop4a_2x2_za64_u16_s16,
  BI__builtin_sme_svmop4a_2x2_za64_u16_u16,
  BI__builtin_sme_svmop4s_1x1_za16_bf16_bf16,
  BI__builtin_sme_svmop4s_1x1_za16_f16_f16,
  BI__builtin_sme_svmop4s_1x1_za32_bf16_bf16,
  BI__builtin_sme_svmop4s_1x1_za32_f16_f16,
  BI__builtin_sme_svmop4s_1x1_za32_f32_f32,
  BI__builtin_sme_svmop4s_1x1_za32_s16_s16,
  BI__builtin_sme_svmop4s_1x1_za32_s8_s8,
  BI__builtin_sme_svmop4s_1x1_za32_s8_u8,
  BI__builtin_sme_svmop4s_1x1_za32_u16_u16,
  BI__builtin_sme_svmop4s_1x1_za32_u8_s8,
  BI__builtin_sme_svmop4s_1x1_za32_u8_u8,
  BI__builtin_sme_svmop4s_1x1_za64_f64_f64,
  BI__builtin_sme_svmop4s_1x1_za64_s16_s16,
  BI__builtin_sme_svmop4s_1x1_za64_s16_u16,
  BI__builtin_sme_svmop4s_1x1_za64_u16_s16,
  BI__builtin_sme_svmop4s_1x1_za64_u16_u16,
  BI__builtin_sme_svmop4s_1x2_za16_bf16_bf16,
  BI__builtin_sme_svmop4s_1x2_za16_f16_f16,
  BI__builtin_sme_svmop4s_1x2_za32_bf16_bf16,
  BI__builtin_sme_svmop4s_1x2_za32_f16_f16,
  BI__builtin_sme_svmop4s_1x2_za32_f32_f32,
  BI__builtin_sme_svmop4s_1x2_za32_s16_s16,
  BI__builtin_sme_svmop4s_1x2_za32_s8_s8,
  BI__builtin_sme_svmop4s_1x2_za32_s8_u8,
  BI__builtin_sme_svmop4s_1x2_za32_u16_u16,
  BI__builtin_sme_svmop4s_1x2_za32_u8_s8,
  BI__builtin_sme_svmop4s_1x2_za32_u8_u8,
  BI__builtin_sme_svmop4s_1x2_za64_f64_f64,
  BI__builtin_sme_svmop4s_1x2_za64_s16_s16,
  BI__builtin_sme_svmop4s_1x2_za64_s16_u16,
  BI__builtin_sme_svmop4s_1x2_za64_u16_s16,
  BI__builtin_sme_svmop4s_1x2_za64_u16_u16,
  BI__builtin_sme_svmop4s_2x1_za16_bf16_bf16,
  BI__builtin_sme_svmop4s_2x1_za16_f16_f16,
  BI__builtin_sme_svmop4s_2x1_za32_bf16_bf16,
  BI__builtin_sme_svmop4s_2x1_za32_f16_f16,
  BI__builtin_sme_svmop4s_2x1_za32_f32_f32,
  BI__builtin_sme_svmop4s_2x1_za32_s16_s16,
  BI__builtin_sme_svmop4s_2x1_za32_s8_s8,
  BI__builtin_sme_svmop4s_2x1_za32_s8_u8,
  BI__builtin_sme_svmop4s_2x1_za32_u16_u16,
  BI__builtin_sme_svmop4s_2x1_za32_u8_s8,
  BI__builtin_sme_svmop4s_2x1_za32_u8_u8,
  BI__builtin_sme_svmop4s_2x1_za64_f64_f64,
  BI__builtin_sme_svmop4s_2x1_za64_s16_s16,
  BI__builtin_sme_svmop4s_2x1_za64_s16_u16,
  BI__builtin_sme_svmop4s_2x1_za64_u16_s16,
  BI__builtin_sme_svmop4s_2x1_za64_u16_u16,
  BI__builtin_sme_svmop4s_2x2_za16_bf16_bf16,
  BI__builtin_sme_svmop4s_2x2_za16_f16_f16,
  BI__builtin_sme_svmop4s_2x2_za32_bf16_bf16,
  BI__builtin_sme_svmop4s_2x2_za32_f16_f16,
  BI__builtin_sme_svmop4s_2x2_za32_f32_f32,
  BI__builtin_sme_svmop4s_2x2_za32_s16_s16,
  BI__builtin_sme_svmop4s_2x2_za32_s8_s8,
  BI__builtin_sme_svmop4s_2x2_za32_s8_u8,
  BI__builtin_sme_svmop4s_2x2_za32_u16_u16,
  BI__builtin_sme_svmop4s_2x2_za32_u8_s8,
  BI__builtin_sme_svmop4s_2x2_za32_u8_u8,
  BI__builtin_sme_svmop4s_2x2_za64_f64_f64,
  BI__builtin_sme_svmop4s_2x2_za64_s16_s16,
  BI__builtin_sme_svmop4s_2x2_za64_s16_u16,
  BI__builtin_sme_svmop4s_2x2_za64_u16_s16,
  BI__builtin_sme_svmop4s_2x2_za64_u16_u16,
  BI__builtin_sme_svmopa_za16_bf16_m,
  BI__builtin_sme_svmopa_za16_f16_m,
  BI__builtin_sme_svmopa_za16_mf8_m_fpm,
  BI__builtin_sme_svmopa_za32_bf16_m,
  BI__builtin_sme_svmopa_za32_f16_m,
  BI__builtin_sme_svmopa_za32_f32_m,
  BI__builtin_sme_svmopa_za32_mf8_m_fpm,
  BI__builtin_sme_svmopa_za32_s16_m,
  BI__builtin_sme_svmopa_za32_s8_m,
  BI__builtin_sme_svmopa_za32_u16_m,
  BI__builtin_sme_svmopa_za32_u8_m,
  BI__builtin_sme_svmopa_za64_f64_m,
  BI__builtin_sme_svmopa_za64_s16_m,
  BI__builtin_sme_svmopa_za64_u16_m,
  BI__builtin_sme_svmops_za16_bf16_m,
  BI__builtin_sme_svmops_za16_f16_m,
  BI__builtin_sme_svmops_za32_bf16_m,
  BI__builtin_sme_svmops_za32_f16_m,
  BI__builtin_sme_svmops_za32_f32_m,
  BI__builtin_sme_svmops_za32_s16_m,
  BI__builtin_sme_svmops_za32_s8_m,
  BI__builtin_sme_svmops_za32_u16_m,
  BI__builtin_sme_svmops_za32_u8_m,
  BI__builtin_sme_svmops_za64_f64_m,
  BI__builtin_sme_svmops_za64_s16_m,
  BI__builtin_sme_svmops_za64_u16_m,
  BI__builtin_sme_svread_hor_za128_bf16_m,
  BI__builtin_sme_svread_hor_za128_f16_m,
  BI__builtin_sme_svread_hor_za128_f32_m,
  BI__builtin_sme_svread_hor_za128_f64_m,
  BI__builtin_sme_svread_hor_za128_mf8_m,
  BI__builtin_sme_svread_hor_za128_s16_m,
  BI__builtin_sme_svread_hor_za128_s32_m,
  BI__builtin_sme_svread_hor_za128_s64_m,
  BI__builtin_sme_svread_hor_za128_s8_m,
  BI__builtin_sme_svread_hor_za128_u16_m,
  BI__builtin_sme_svread_hor_za128_u32_m,
  BI__builtin_sme_svread_hor_za128_u64_m,
  BI__builtin_sme_svread_hor_za128_u8_m,
  BI__builtin_sme_svread_hor_za16_bf16_m,
  BI__builtin_sme_svread_hor_za16_bf16_vg2,
  BI__builtin_sme_svread_hor_za16_bf16_vg4,
  BI__builtin_sme_svread_hor_za16_f16_m,
  BI__builtin_sme_svread_hor_za16_f16_vg2,
  BI__builtin_sme_svread_hor_za16_f16_vg4,
  BI__builtin_sme_svread_hor_za16_s16_m,
  BI__builtin_sme_svread_hor_za16_s16_vg2,
  BI__builtin_sme_svread_hor_za16_s16_vg4,
  BI__builtin_sme_svread_hor_za16_u16_m,
  BI__builtin_sme_svread_hor_za16_u16_vg2,
  BI__builtin_sme_svread_hor_za16_u16_vg4,
  BI__builtin_sme_svread_hor_za32_f32_m,
  BI__builtin_sme_svread_hor_za32_f32_vg2,
  BI__builtin_sme_svread_hor_za32_f32_vg4,
  BI__builtin_sme_svread_hor_za32_s32_m,
  BI__builtin_sme_svread_hor_za32_s32_vg2,
  BI__builtin_sme_svread_hor_za32_s32_vg4,
  BI__builtin_sme_svread_hor_za32_u32_m,
  BI__builtin_sme_svread_hor_za32_u32_vg2,
  BI__builtin_sme_svread_hor_za32_u32_vg4,
  BI__builtin_sme_svread_hor_za64_f64_m,
  BI__builtin_sme_svread_hor_za64_f64_vg2,
  BI__builtin_sme_svread_hor_za64_f64_vg4,
  BI__builtin_sme_svread_hor_za64_s64_m,
  BI__builtin_sme_svread_hor_za64_s64_vg2,
  BI__builtin_sme_svread_hor_za64_s64_vg4,
  BI__builtin_sme_svread_hor_za64_u64_m,
  BI__builtin_sme_svread_hor_za64_u64_vg2,
  BI__builtin_sme_svread_hor_za64_u64_vg4,
  BI__builtin_sme_svread_hor_za8_mf8_m,
  BI__builtin_sme_svread_hor_za8_mf8_vg2,
  BI__builtin_sme_svread_hor_za8_mf8_vg4,
  BI__builtin_sme_svread_hor_za8_s8_m,
  BI__builtin_sme_svread_hor_za8_s8_vg2,
  BI__builtin_sme_svread_hor_za8_s8_vg4,
  BI__builtin_sme_svread_hor_za8_u8_m,
  BI__builtin_sme_svread_hor_za8_u8_vg2,
  BI__builtin_sme_svread_hor_za8_u8_vg4,
  BI__builtin_sme_svread_ver_za128_bf16_m,
  BI__builtin_sme_svread_ver_za128_f16_m,
  BI__builtin_sme_svread_ver_za128_f32_m,
  BI__builtin_sme_svread_ver_za128_f64_m,
  BI__builtin_sme_svread_ver_za128_mf8_m,
  BI__builtin_sme_svread_ver_za128_s16_m,
  BI__builtin_sme_svread_ver_za128_s32_m,
  BI__builtin_sme_svread_ver_za128_s64_m,
  BI__builtin_sme_svread_ver_za128_s8_m,
  BI__builtin_sme_svread_ver_za128_u16_m,
  BI__builtin_sme_svread_ver_za128_u32_m,
  BI__builtin_sme_svread_ver_za128_u64_m,
  BI__builtin_sme_svread_ver_za128_u8_m,
  BI__builtin_sme_svread_ver_za16_bf16_m,
  BI__builtin_sme_svread_ver_za16_bf16_vg2,
  BI__builtin_sme_svread_ver_za16_bf16_vg4,
  BI__builtin_sme_svread_ver_za16_f16_m,
  BI__builtin_sme_svread_ver_za16_f16_vg2,
  BI__builtin_sme_svread_ver_za16_f16_vg4,
  BI__builtin_sme_svread_ver_za16_s16_m,
  BI__builtin_sme_svread_ver_za16_s16_vg2,
  BI__builtin_sme_svread_ver_za16_s16_vg4,
  BI__builtin_sme_svread_ver_za16_u16_m,
  BI__builtin_sme_svread_ver_za16_u16_vg2,
  BI__builtin_sme_svread_ver_za16_u16_vg4,
  BI__builtin_sme_svread_ver_za32_f32_m,
  BI__builtin_sme_svread_ver_za32_f32_vg2,
  BI__builtin_sme_svread_ver_za32_f32_vg4,
  BI__builtin_sme_svread_ver_za32_s32_m,
  BI__builtin_sme_svread_ver_za32_s32_vg2,
  BI__builtin_sme_svread_ver_za32_s32_vg4,
  BI__builtin_sme_svread_ver_za32_u32_m,
  BI__builtin_sme_svread_ver_za32_u32_vg2,
  BI__builtin_sme_svread_ver_za32_u32_vg4,
  BI__builtin_sme_svread_ver_za64_f64_m,
  BI__builtin_sme_svread_ver_za64_f64_vg2,
  BI__builtin_sme_svread_ver_za64_f64_vg4,
  BI__builtin_sme_svread_ver_za64_s64_m,
  BI__builtin_sme_svread_ver_za64_s64_vg2,
  BI__builtin_sme_svread_ver_za64_s64_vg4,
  BI__builtin_sme_svread_ver_za64_u64_m,
  BI__builtin_sme_svread_ver_za64_u64_vg2,
  BI__builtin_sme_svread_ver_za64_u64_vg4,
  BI__builtin_sme_svread_ver_za8_mf8_m,
  BI__builtin_sme_svread_ver_za8_mf8_vg2,
  BI__builtin_sme_svread_ver_za8_mf8_vg4,
  BI__builtin_sme_svread_ver_za8_s8_m,
  BI__builtin_sme_svread_ver_za8_s8_vg2,
  BI__builtin_sme_svread_ver_za8_s8_vg4,
  BI__builtin_sme_svread_ver_za8_u8_m,
  BI__builtin_sme_svread_ver_za8_u8_vg2,
  BI__builtin_sme_svread_ver_za8_u8_vg4,
  BI__builtin_sme_svread_za16_bf16_vg1x2,
  BI__builtin_sme_svread_za16_bf16_vg1x4,
  BI__builtin_sme_svread_za16_f16_vg1x2,
  BI__builtin_sme_svread_za16_f16_vg1x4,
  BI__builtin_sme_svread_za16_s16_vg1x2,
  BI__builtin_sme_svread_za16_s16_vg1x4,
  BI__builtin_sme_svread_za16_u16_vg1x2,
  BI__builtin_sme_svread_za16_u16_vg1x4,
  BI__builtin_sme_svread_za32_f32_vg1x2,
  BI__builtin_sme_svread_za32_f32_vg1x4,
  BI__builtin_sme_svread_za32_s32_vg1x2,
  BI__builtin_sme_svread_za32_s32_vg1x4,
  BI__builtin_sme_svread_za32_u32_vg1x2,
  BI__builtin_sme_svread_za32_u32_vg1x4,
  BI__builtin_sme_svread_za64_f64_vg1x2,
  BI__builtin_sme_svread_za64_f64_vg1x4,
  BI__builtin_sme_svread_za64_s64_vg1x2,
  BI__builtin_sme_svread_za64_s64_vg1x4,
  BI__builtin_sme_svread_za64_u64_vg1x2,
  BI__builtin_sme_svread_za64_u64_vg1x4,
  BI__builtin_sme_svread_za8_mf8_vg1x2,
  BI__builtin_sme_svread_za8_mf8_vg1x4,
  BI__builtin_sme_svread_za8_s8_vg1x2,
  BI__builtin_sme_svread_za8_s8_vg1x4,
  BI__builtin_sme_svread_za8_u8_vg1x2,
  BI__builtin_sme_svread_za8_u8_vg1x4,
  BI__builtin_sme_svreadz_hor_za128_bf16,
  BI__builtin_sme_svreadz_hor_za128_f16,
  BI__builtin_sme_svreadz_hor_za128_f32,
  BI__builtin_sme_svreadz_hor_za128_f64,
  BI__builtin_sme_svreadz_hor_za128_mf8,
  BI__builtin_sme_svreadz_hor_za128_s16,
  BI__builtin_sme_svreadz_hor_za128_s32,
  BI__builtin_sme_svreadz_hor_za128_s64,
  BI__builtin_sme_svreadz_hor_za128_s8,
  BI__builtin_sme_svreadz_hor_za128_u16,
  BI__builtin_sme_svreadz_hor_za128_u32,
  BI__builtin_sme_svreadz_hor_za128_u64,
  BI__builtin_sme_svreadz_hor_za128_u8,
  BI__builtin_sme_svreadz_hor_za16_bf16,
  BI__builtin_sme_svreadz_hor_za16_bf16_vg2,
  BI__builtin_sme_svreadz_hor_za16_bf16_vg4,
  BI__builtin_sme_svreadz_hor_za16_f16,
  BI__builtin_sme_svreadz_hor_za16_f16_vg2,
  BI__builtin_sme_svreadz_hor_za16_f16_vg4,
  BI__builtin_sme_svreadz_hor_za16_s16,
  BI__builtin_sme_svreadz_hor_za16_s16_vg2,
  BI__builtin_sme_svreadz_hor_za16_s16_vg4,
  BI__builtin_sme_svreadz_hor_za16_u16,
  BI__builtin_sme_svreadz_hor_za16_u16_vg2,
  BI__builtin_sme_svreadz_hor_za16_u16_vg4,
  BI__builtin_sme_svreadz_hor_za32_f32,
  BI__builtin_sme_svreadz_hor_za32_f32_vg2,
  BI__builtin_sme_svreadz_hor_za32_f32_vg4,
  BI__builtin_sme_svreadz_hor_za32_s32,
  BI__builtin_sme_svreadz_hor_za32_s32_vg2,
  BI__builtin_sme_svreadz_hor_za32_s32_vg4,
  BI__builtin_sme_svreadz_hor_za32_u32,
  BI__builtin_sme_svreadz_hor_za32_u32_vg2,
  BI__builtin_sme_svreadz_hor_za32_u32_vg4,
  BI__builtin_sme_svreadz_hor_za64_f64,
  BI__builtin_sme_svreadz_hor_za64_f64_vg2,
  BI__builtin_sme_svreadz_hor_za64_f64_vg4,
  BI__builtin_sme_svreadz_hor_za64_s64,
  BI__builtin_sme_svreadz_hor_za64_s64_vg2,
  BI__builtin_sme_svreadz_hor_za64_s64_vg4,
  BI__builtin_sme_svreadz_hor_za64_u64,
  BI__builtin_sme_svreadz_hor_za64_u64_vg2,
  BI__builtin_sme_svreadz_hor_za64_u64_vg4,
  BI__builtin_sme_svreadz_hor_za8_mf8,
  BI__builtin_sme_svreadz_hor_za8_mf8_vg2,
  BI__builtin_sme_svreadz_hor_za8_mf8_vg4,
  BI__builtin_sme_svreadz_hor_za8_s8,
  BI__builtin_sme_svreadz_hor_za8_s8_vg2,
  BI__builtin_sme_svreadz_hor_za8_s8_vg4,
  BI__builtin_sme_svreadz_hor_za8_u8,
  BI__builtin_sme_svreadz_hor_za8_u8_vg2,
  BI__builtin_sme_svreadz_hor_za8_u8_vg4,
  BI__builtin_sme_svreadz_ver_za128_bf16,
  BI__builtin_sme_svreadz_ver_za128_f16,
  BI__builtin_sme_svreadz_ver_za128_f32,
  BI__builtin_sme_svreadz_ver_za128_f64,
  BI__builtin_sme_svreadz_ver_za128_mf8,
  BI__builtin_sme_svreadz_ver_za128_s16,
  BI__builtin_sme_svreadz_ver_za128_s32,
  BI__builtin_sme_svreadz_ver_za128_s64,
  BI__builtin_sme_svreadz_ver_za128_s8,
  BI__builtin_sme_svreadz_ver_za128_u16,
  BI__builtin_sme_svreadz_ver_za128_u32,
  BI__builtin_sme_svreadz_ver_za128_u64,
  BI__builtin_sme_svreadz_ver_za128_u8,
  BI__builtin_sme_svreadz_ver_za16_bf16,
  BI__builtin_sme_svreadz_ver_za16_bf16_vg2,
  BI__builtin_sme_svreadz_ver_za16_bf16_vg4,
  BI__builtin_sme_svreadz_ver_za16_f16,
  BI__builtin_sme_svreadz_ver_za16_f16_vg2,
  BI__builtin_sme_svreadz_ver_za16_f16_vg4,
  BI__builtin_sme_svreadz_ver_za16_s16,
  BI__builtin_sme_svreadz_ver_za16_s16_vg2,
  BI__builtin_sme_svreadz_ver_za16_s16_vg4,
  BI__builtin_sme_svreadz_ver_za16_u16,
  BI__builtin_sme_svreadz_ver_za16_u16_vg2,
  BI__builtin_sme_svreadz_ver_za16_u16_vg4,
  BI__builtin_sme_svreadz_ver_za32_f32,
  BI__builtin_sme_svreadz_ver_za32_f32_vg2,
  BI__builtin_sme_svreadz_ver_za32_f32_vg4,
  BI__builtin_sme_svreadz_ver_za32_s32,
  BI__builtin_sme_svreadz_ver_za32_s32_vg2,
  BI__builtin_sme_svreadz_ver_za32_s32_vg4,
  BI__builtin_sme_svreadz_ver_za32_u32,
  BI__builtin_sme_svreadz_ver_za32_u32_vg2,
  BI__builtin_sme_svreadz_ver_za32_u32_vg4,
  BI__builtin_sme_svreadz_ver_za64_f64,
  BI__builtin_sme_svreadz_ver_za64_f64_vg2,
  BI__builtin_sme_svreadz_ver_za64_f64_vg4,
  BI__builtin_sme_svreadz_ver_za64_s64,
  BI__builtin_sme_svreadz_ver_za64_s64_vg2,
  BI__builtin_sme_svreadz_ver_za64_s64_vg4,
  BI__builtin_sme_svreadz_ver_za64_u64,
  BI__builtin_sme_svreadz_ver_za64_u64_vg2,
  BI__builtin_sme_svreadz_ver_za64_u64_vg4,
  BI__builtin_sme_svreadz_ver_za8_mf8,
  BI__builtin_sme_svreadz_ver_za8_mf8_vg2,
  BI__builtin_sme_svreadz_ver_za8_mf8_vg4,
  BI__builtin_sme_svreadz_ver_za8_s8,
  BI__builtin_sme_svreadz_ver_za8_s8_vg2,
  BI__builtin_sme_svreadz_ver_za8_s8_vg4,
  BI__builtin_sme_svreadz_ver_za8_u8,
  BI__builtin_sme_svreadz_ver_za8_u8_vg2,
  BI__builtin_sme_svreadz_ver_za8_u8_vg4,
  BI__builtin_sme_svreadz_za16_bf16_vg1x2,
  BI__builtin_sme_svreadz_za16_bf16_vg1x4,
  BI__builtin_sme_svreadz_za16_f16_vg1x2,
  BI__builtin_sme_svreadz_za16_f16_vg1x4,
  BI__builtin_sme_svreadz_za16_s16_vg1x2,
  BI__builtin_sme_svreadz_za16_s16_vg1x4,
  BI__builtin_sme_svreadz_za16_u16_vg1x2,
  BI__builtin_sme_svreadz_za16_u16_vg1x4,
  BI__builtin_sme_svreadz_za32_f32_vg1x2,
  BI__builtin_sme_svreadz_za32_f32_vg1x4,
  BI__builtin_sme_svreadz_za32_s32_vg1x2,
  BI__builtin_sme_svreadz_za32_s32_vg1x4,
  BI__builtin_sme_svreadz_za32_u32_vg1x2,
  BI__builtin_sme_svreadz_za32_u32_vg1x4,
  BI__builtin_sme_svreadz_za64_f64_vg1x2,
  BI__builtin_sme_svreadz_za64_f64_vg1x4,
  BI__builtin_sme_svreadz_za64_s64_vg1x2,
  BI__builtin_sme_svreadz_za64_s64_vg1x4,
  BI__builtin_sme_svreadz_za64_u64_vg1x2,
  BI__builtin_sme_svreadz_za64_u64_vg1x4,
  BI__builtin_sme_svreadz_za8_mf8_vg1x2,
  BI__builtin_sme_svreadz_za8_mf8_vg1x4,
  BI__builtin_sme_svreadz_za8_s8_vg1x2,
  BI__builtin_sme_svreadz_za8_s8_vg1x4,
  BI__builtin_sme_svreadz_za8_u8_vg1x2,
  BI__builtin_sme_svreadz_za8_u8_vg1x4,
  BI__builtin_sme_svst1_hor_vnum_za128,
  BI__builtin_sme_svst1_hor_vnum_za16,
  BI__builtin_sme_svst1_hor_vnum_za32,
  BI__builtin_sme_svst1_hor_vnum_za64,
  BI__builtin_sme_svst1_hor_vnum_za8,
  BI__builtin_sme_svst1_hor_za128,
  BI__builtin_sme_svst1_hor_za16,
  BI__builtin_sme_svst1_hor_za32,
  BI__builtin_sme_svst1_hor_za64,
  BI__builtin_sme_svst1_hor_za8,
  BI__builtin_sme_svst1_ver_vnum_za128,
  BI__builtin_sme_svst1_ver_vnum_za16,
  BI__builtin_sme_svst1_ver_vnum_za32,
  BI__builtin_sme_svst1_ver_vnum_za64,
  BI__builtin_sme_svst1_ver_vnum_za8,
  BI__builtin_sme_svst1_ver_za128,
  BI__builtin_sme_svst1_ver_za16,
  BI__builtin_sme_svst1_ver_za32,
  BI__builtin_sme_svst1_ver_za64,
  BI__builtin_sme_svst1_ver_za8,
  BI__builtin_sme_svstr_vnum_za,
  BI__builtin_sme_svstr_za,
  BI__builtin_sme_svstr_zt,
  BI__builtin_sme_svsub_write_single_za32_s32_vg1x2,
  BI__builtin_sme_svsub_write_single_za32_s32_vg1x4,
  BI__builtin_sme_svsub_write_single_za32_u32_vg1x2,
  BI__builtin_sme_svsub_write_single_za32_u32_vg1x4,
  BI__builtin_sme_svsub_write_single_za64_s64_vg1x2,
  BI__builtin_sme_svsub_write_single_za64_s64_vg1x4,
  BI__builtin_sme_svsub_write_single_za64_u64_vg1x2,
  BI__builtin_sme_svsub_write_single_za64_u64_vg1x4,
  BI__builtin_sme_svsub_write_za32_s32_vg1x2,
  BI__builtin_sme_svsub_write_za32_s32_vg1x4,
  BI__builtin_sme_svsub_write_za32_u32_vg1x2,
  BI__builtin_sme_svsub_write_za32_u32_vg1x4,
  BI__builtin_sme_svsub_write_za64_s64_vg1x2,
  BI__builtin_sme_svsub_write_za64_s64_vg1x4,
  BI__builtin_sme_svsub_write_za64_u64_vg1x2,
  BI__builtin_sme_svsub_write_za64_u64_vg1x4,
  BI__builtin_sme_svsub_za16_bf16_vg1x2,
  BI__builtin_sme_svsub_za16_bf16_vg1x4,
  BI__builtin_sme_svsub_za16_f16_vg1x2,
  BI__builtin_sme_svsub_za16_f16_vg1x4,
  BI__builtin_sme_svsub_za32_f32_vg1x2,
  BI__builtin_sme_svsub_za32_f32_vg1x4,
  BI__builtin_sme_svsub_za32_s32_vg1x2,
  BI__builtin_sme_svsub_za32_s32_vg1x4,
  BI__builtin_sme_svsub_za32_u32_vg1x2,
  BI__builtin_sme_svsub_za32_u32_vg1x4,
  BI__builtin_sme_svsub_za64_f64_vg1x2,
  BI__builtin_sme_svsub_za64_f64_vg1x4,
  BI__builtin_sme_svsub_za64_s64_vg1x2,
  BI__builtin_sme_svsub_za64_s64_vg1x4,
  BI__builtin_sme_svsub_za64_u64_vg1x2,
  BI__builtin_sme_svsub_za64_u64_vg1x4,
  BI__builtin_sme_svsudot_lane_za32_s8_vg1x2,
  BI__builtin_sme_svsudot_lane_za32_s8_vg1x4,
  BI__builtin_sme_svsudot_single_za32_s8_vg1x2,
  BI__builtin_sme_svsudot_single_za32_s8_vg1x4,
  BI__builtin_sme_svsudot_za32_s8_vg1x2,
  BI__builtin_sme_svsudot_za32_s8_vg1x4,
  BI__builtin_sme_svsumla_lane_za32_s8_vg4x1,
  BI__builtin_sme_svsumla_lane_za32_s8_vg4x2,
  BI__builtin_sme_svsumla_lane_za32_s8_vg4x4,
  BI__builtin_sme_svsumla_single_za32_s8_vg4x2,
  BI__builtin_sme_svsumla_single_za32_s8_vg4x4,
  BI__builtin_sme_svsumla_za32_s8_vg4x1,
  BI__builtin_sme_svsumla_za32_s8_vg4x2,
  BI__builtin_sme_svsumla_za32_s8_vg4x4,
  BI__builtin_sme_svsumopa_za32_s8_m,
  BI__builtin_sme_svsumopa_za64_s16_m,
  BI__builtin_sme_svsumops_za32_s8_m,
  BI__builtin_sme_svsumops_za64_s16_m,
  BI__builtin_sme_svsuvdot_lane_za32_s8_vg1x4,
  BI__builtin_sme_svtmopa_lane_za16_bf16_bf16,
  BI__builtin_sme_svtmopa_lane_za16_f16_f16,
  BI__builtin_sme_svtmopa_lane_za16_mf8_mf8_fpm,
  BI__builtin_sme_svtmopa_lane_za32_bf16_bf16,
  BI__builtin_sme_svtmopa_lane_za32_f16_f16,
  BI__builtin_sme_svtmopa_lane_za32_f32_f32,
  BI__builtin_sme_svtmopa_lane_za32_mf8_mf8_fpm,
  BI__builtin_sme_svtmopa_lane_za32_s16_s16,
  BI__builtin_sme_svtmopa_lane_za32_s8_s8,
  BI__builtin_sme_svtmopa_lane_za32_s8_u8,
  BI__builtin_sme_svtmopa_lane_za32_u16_u16,
  BI__builtin_sme_svtmopa_lane_za32_u8_s8,
  BI__builtin_sme_svtmopa_lane_za32_u8_u8,
  BI__builtin_sme_svusdot_lane_za32_u8_vg1x2,
  BI__builtin_sme_svusdot_lane_za32_u8_vg1x4,
  BI__builtin_sme_svusdot_single_za32_u8_vg1x2,
  BI__builtin_sme_svusdot_single_za32_u8_vg1x4,
  BI__builtin_sme_svusdot_za32_u8_vg1x2,
  BI__builtin_sme_svusdot_za32_u8_vg1x4,
  BI__builtin_sme_svusmla_lane_za32_u8_vg4x1,
  BI__builtin_sme_svusmla_lane_za32_u8_vg4x2,
  BI__builtin_sme_svusmla_lane_za32_u8_vg4x4,
  BI__builtin_sme_svusmla_single_za32_u8_vg4x2,
  BI__builtin_sme_svusmla_single_za32_u8_vg4x4,
  BI__builtin_sme_svusmla_za32_u8_vg4x1,
  BI__builtin_sme_svusmla_za32_u8_vg4x2,
  BI__builtin_sme_svusmla_za32_u8_vg4x4,
  BI__builtin_sme_svusmopa_za32_u8_m,
  BI__builtin_sme_svusmopa_za64_u16_m,
  BI__builtin_sme_svusmops_za32_u8_m,
  BI__builtin_sme_svusmops_za64_u16_m,
  BI__builtin_sme_svusvdot_lane_za32_u8_vg1x4,
  BI__builtin_sme_svvdot_lane_za16_mf8_vg1x2_fpm,
  BI__builtin_sme_svvdot_lane_za32_bf16_vg1x2,
  BI__builtin_sme_svvdot_lane_za32_f16_vg1x2,
  BI__builtin_sme_svvdot_lane_za32_s16_vg1x2,
  BI__builtin_sme_svvdot_lane_za32_s8_vg1x4,
  BI__builtin_sme_svvdot_lane_za32_u16_vg1x2,
  BI__builtin_sme_svvdot_lane_za32_u8_vg1x4,
  BI__builtin_sme_svvdot_lane_za64_s16_vg1x4,
  BI__builtin_sme_svvdot_lane_za64_u16_vg1x4,
  BI__builtin_sme_svvdotb_lane_za32_mf8_vg1x4_fpm,
  BI__builtin_sme_svvdott_lane_za32_mf8_vg1x4_fpm,
  BI__builtin_sme_svwrite_hor_za128_bf16_m,
  BI__builtin_sme_svwrite_hor_za128_f16_m,
  BI__builtin_sme_svwrite_hor_za128_f32_m,
  BI__builtin_sme_svwrite_hor_za128_f64_m,
  BI__builtin_sme_svwrite_hor_za128_mf8_m,
  BI__builtin_sme_svwrite_hor_za128_s16_m,
  BI__builtin_sme_svwrite_hor_za128_s32_m,
  BI__builtin_sme_svwrite_hor_za128_s64_m,
  BI__builtin_sme_svwrite_hor_za128_s8_m,
  BI__builtin_sme_svwrite_hor_za128_u16_m,
  BI__builtin_sme_svwrite_hor_za128_u32_m,
  BI__builtin_sme_svwrite_hor_za128_u64_m,
  BI__builtin_sme_svwrite_hor_za128_u8_m,
  BI__builtin_sme_svwrite_hor_za16_bf16_m,
  BI__builtin_sme_svwrite_hor_za16_bf16_vg2,
  BI__builtin_sme_svwrite_hor_za16_bf16_vg4,
  BI__builtin_sme_svwrite_hor_za16_f16_m,
  BI__builtin_sme_svwrite_hor_za16_f16_vg2,
  BI__builtin_sme_svwrite_hor_za16_f16_vg4,
  BI__builtin_sme_svwrite_hor_za16_s16_m,
  BI__builtin_sme_svwrite_hor_za16_s16_vg2,
  BI__builtin_sme_svwrite_hor_za16_s16_vg4,
  BI__builtin_sme_svwrite_hor_za16_u16_m,
  BI__builtin_sme_svwrite_hor_za16_u16_vg2,
  BI__builtin_sme_svwrite_hor_za16_u16_vg4,
  BI__builtin_sme_svwrite_hor_za32_f32_m,
  BI__builtin_sme_svwrite_hor_za32_f32_vg2,
  BI__builtin_sme_svwrite_hor_za32_f32_vg4,
  BI__builtin_sme_svwrite_hor_za32_s32_m,
  BI__builtin_sme_svwrite_hor_za32_s32_vg2,
  BI__builtin_sme_svwrite_hor_za32_s32_vg4,
  BI__builtin_sme_svwrite_hor_za32_u32_m,
  BI__builtin_sme_svwrite_hor_za32_u32_vg2,
  BI__builtin_sme_svwrite_hor_za32_u32_vg4,
  BI__builtin_sme_svwrite_hor_za64_f64_m,
  BI__builtin_sme_svwrite_hor_za64_f64_vg2,
  BI__builtin_sme_svwrite_hor_za64_f64_vg4,
  BI__builtin_sme_svwrite_hor_za64_s64_m,
  BI__builtin_sme_svwrite_hor_za64_s64_vg2,
  BI__builtin_sme_svwrite_hor_za64_s64_vg4,
  BI__builtin_sme_svwrite_hor_za64_u64_m,
  BI__builtin_sme_svwrite_hor_za64_u64_vg2,
  BI__builtin_sme_svwrite_hor_za64_u64_vg4,
  BI__builtin_sme_svwrite_hor_za8_mf8_m,
  BI__builtin_sme_svwrite_hor_za8_mf8_vg2,
  BI__builtin_sme_svwrite_hor_za8_mf8_vg4,
  BI__builtin_sme_svwrite_hor_za8_s8_m,
  BI__builtin_sme_svwrite_hor_za8_s8_vg2,
  BI__builtin_sme_svwrite_hor_za8_s8_vg4,
  BI__builtin_sme_svwrite_hor_za8_u8_m,
  BI__builtin_sme_svwrite_hor_za8_u8_vg2,
  BI__builtin_sme_svwrite_hor_za8_u8_vg4,
  BI__builtin_sme_svwrite_lane_zt_bf16,
  BI__builtin_sme_svwrite_lane_zt_f16,
  BI__builtin_sme_svwrite_lane_zt_f32,
  BI__builtin_sme_svwrite_lane_zt_f64,
  BI__builtin_sme_svwrite_lane_zt_s16,
  BI__builtin_sme_svwrite_lane_zt_s32,
  BI__builtin_sme_svwrite_lane_zt_s64,
  BI__builtin_sme_svwrite_lane_zt_s8,
  BI__builtin_sme_svwrite_lane_zt_u16,
  BI__builtin_sme_svwrite_lane_zt_u32,
  BI__builtin_sme_svwrite_lane_zt_u64,
  BI__builtin_sme_svwrite_lane_zt_u8,
  BI__builtin_sme_svwrite_ver_za128_bf16_m,
  BI__builtin_sme_svwrite_ver_za128_f16_m,
  BI__builtin_sme_svwrite_ver_za128_f32_m,
  BI__builtin_sme_svwrite_ver_za128_f64_m,
  BI__builtin_sme_svwrite_ver_za128_mf8_m,
  BI__builtin_sme_svwrite_ver_za128_s16_m,
  BI__builtin_sme_svwrite_ver_za128_s32_m,
  BI__builtin_sme_svwrite_ver_za128_s64_m,
  BI__builtin_sme_svwrite_ver_za128_s8_m,
  BI__builtin_sme_svwrite_ver_za128_u16_m,
  BI__builtin_sme_svwrite_ver_za128_u32_m,
  BI__builtin_sme_svwrite_ver_za128_u64_m,
  BI__builtin_sme_svwrite_ver_za128_u8_m,
  BI__builtin_sme_svwrite_ver_za16_bf16_m,
  BI__builtin_sme_svwrite_ver_za16_bf16_vg2,
  BI__builtin_sme_svwrite_ver_za16_bf16_vg4,
  BI__builtin_sme_svwrite_ver_za16_f16_m,
  BI__builtin_sme_svwrite_ver_za16_f16_vg2,
  BI__builtin_sme_svwrite_ver_za16_f16_vg4,
  BI__builtin_sme_svwrite_ver_za16_s16_m,
  BI__builtin_sme_svwrite_ver_za16_s16_vg2,
  BI__builtin_sme_svwrite_ver_za16_s16_vg4,
  BI__builtin_sme_svwrite_ver_za16_u16_m,
  BI__builtin_sme_svwrite_ver_za16_u16_vg2,
  BI__builtin_sme_svwrite_ver_za16_u16_vg4,
  BI__builtin_sme_svwrite_ver_za32_f32_m,
  BI__builtin_sme_svwrite_ver_za32_f32_vg2,
  BI__builtin_sme_svwrite_ver_za32_f32_vg4,
  BI__builtin_sme_svwrite_ver_za32_s32_m,
  BI__builtin_sme_svwrite_ver_za32_s32_vg2,
  BI__builtin_sme_svwrite_ver_za32_s32_vg4,
  BI__builtin_sme_svwrite_ver_za32_u32_m,
  BI__builtin_sme_svwrite_ver_za32_u32_vg2,
  BI__builtin_sme_svwrite_ver_za32_u32_vg4,
  BI__builtin_sme_svwrite_ver_za64_f64_m,
  BI__builtin_sme_svwrite_ver_za64_f64_vg2,
  BI__builtin_sme_svwrite_ver_za64_f64_vg4,
  BI__builtin_sme_svwrite_ver_za64_s64_m,
  BI__builtin_sme_svwrite_ver_za64_s64_vg2,
  BI__builtin_sme_svwrite_ver_za64_s64_vg4,
  BI__builtin_sme_svwrite_ver_za64_u64_m,
  BI__builtin_sme_svwrite_ver_za64_u64_vg2,
  BI__builtin_sme_svwrite_ver_za64_u64_vg4,
  BI__builtin_sme_svwrite_ver_za8_mf8_m,
  BI__builtin_sme_svwrite_ver_za8_mf8_vg2,
  BI__builtin_sme_svwrite_ver_za8_mf8_vg4,
  BI__builtin_sme_svwrite_ver_za8_s8_m,
  BI__builtin_sme_svwrite_ver_za8_s8_vg2,
  BI__builtin_sme_svwrite_ver_za8_s8_vg4,
  BI__builtin_sme_svwrite_ver_za8_u8_m,
  BI__builtin_sme_svwrite_ver_za8_u8_vg2,
  BI__builtin_sme_svwrite_ver_za8_u8_vg4,
  BI__builtin_sme_svwrite_za16_bf16_vg1x2,
  BI__builtin_sme_svwrite_za16_bf16_vg1x4,
  BI__builtin_sme_svwrite_za16_f16_vg1x2,
  BI__builtin_sme_svwrite_za16_f16_vg1x4,
  BI__builtin_sme_svwrite_za16_s16_vg1x2,
  BI__builtin_sme_svwrite_za16_s16_vg1x4,
  BI__builtin_sme_svwrite_za16_u16_vg1x2,
  BI__builtin_sme_svwrite_za16_u16_vg1x4,
  BI__builtin_sme_svwrite_za32_f32_vg1x2,
  BI__builtin_sme_svwrite_za32_f32_vg1x4,
  BI__builtin_sme_svwrite_za32_s32_vg1x2,
  BI__builtin_sme_svwrite_za32_s32_vg1x4,
  BI__builtin_sme_svwrite_za32_u32_vg1x2,
  BI__builtin_sme_svwrite_za32_u32_vg1x4,
  BI__builtin_sme_svwrite_za64_f64_vg1x2,
  BI__builtin_sme_svwrite_za64_f64_vg1x4,
  BI__builtin_sme_svwrite_za64_s64_vg1x2,
  BI__builtin_sme_svwrite_za64_s64_vg1x4,
  BI__builtin_sme_svwrite_za64_u64_vg1x2,
  BI__builtin_sme_svwrite_za64_u64_vg1x4,
  BI__builtin_sme_svwrite_za8_mf8_vg1x2,
  BI__builtin_sme_svwrite_za8_mf8_vg1x4,
  BI__builtin_sme_svwrite_za8_s8_vg1x2,
  BI__builtin_sme_svwrite_za8_s8_vg1x4,
  BI__builtin_sme_svwrite_za8_u8_vg1x2,
  BI__builtin_sme_svwrite_za8_u8_vg1x4,
  BI__builtin_sme_svwrite_zt_bf16,
  BI__builtin_sme_svwrite_zt_f16,
  BI__builtin_sme_svwrite_zt_f32,
  BI__builtin_sme_svwrite_zt_f64,
  BI__builtin_sme_svwrite_zt_s16,
  BI__builtin_sme_svwrite_zt_s32,
  BI__builtin_sme_svwrite_zt_s64,
  BI__builtin_sme_svwrite_zt_s8,
  BI__builtin_sme_svwrite_zt_u16,
  BI__builtin_sme_svwrite_zt_u32,
  BI__builtin_sme_svwrite_zt_u64,
  BI__builtin_sme_svwrite_zt_u8,
  BI__builtin_sme_svzero_mask_za,
  BI__builtin_sme_svzero_za,
  BI__builtin_sme_svzero_za64_vg1x2,
  BI__builtin_sme_svzero_za64_vg1x4,
  BI__builtin_sme_svzero_za64_vg2x1,
  BI__builtin_sme_svzero_za64_vg2x2,
  BI__builtin_sme_svzero_za64_vg2x4,
  BI__builtin_sme_svzero_za64_vg4x1,
  BI__builtin_sme_svzero_za64_vg4x2,
  BI__builtin_sme_svzero_za64_vg4x4,
  BI__builtin_sme_svzero_zt,
#endif // GET_SME_BUILTIN_ENUMERATORS

#ifdef GET_SME_BUILTIN_STR_TABLE

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinStringsStorage[] =
  "\0"
  "n\0"
  "__arm_in_streaming_mode\0"
  "bv\0"
  "sme\0"
  "svadd_write_single_za32_s32_vg1x2\0"
  "vUiq8iq4i\0"
  "sme,sme2\0"
  "svadd_write_single_za32_s32_vg1x4\0"
  "vUiq16iq4i\0"
  "svadd_write_single_za32_u32_vg1x2\0"
  "vUiq8Uiq4Ui\0"
  "svadd_write_single_za32_u32_vg1x4\0"
  "vUiq16Uiq4Ui\0"
  "svadd_write_single_za64_s64_vg1x2\0"
  "vUiq4Wiq2Wi\0"
  "sme,sme2,sme-i16i64\0"
  "svadd_write_single_za64_s64_vg1x4\0"
  "vUiq8Wiq2Wi\0"
  "svadd_write_single_za64_u64_vg1x2\0"
  "vUiq4UWiq2UWi\0"
  "svadd_write_single_za64_u64_vg1x4\0"
  "vUiq8UWiq2UWi\0"
  "svadd_write_za32_s32_vg1x2\0"
  "vUiq8iq8i\0"
  "svadd_write_za32_s32_vg1x4\0"
  "vUiq16iq16i\0"
  "svadd_write_za32_u32_vg1x2\0"
  "vUiq8Uiq8Ui\0"
  "svadd_write_za32_u32_vg1x4\0"
  "vUiq16Uiq16Ui\0"
  "svadd_write_za64_s64_vg1x2\0"
  "vUiq4Wiq4Wi\0"
  "svadd_write_za64_s64_vg1x4\0"
  "vUiq8Wiq8Wi\0"
  "svadd_write_za64_u64_vg1x2\0"
  "vUiq4UWiq4UWi\0"
  "svadd_write_za64_u64_vg1x4\0"
  "vUiq8UWiq8UWi\0"
  "svadd_za16_bf16_vg1x2\0"
  "vUiq16y\0"
  "sme,sme-b16b16\0"
  "svadd_za16_bf16_vg1x4\0"
  "vUiq32y\0"
  "svadd_za16_f16_vg1x2\0"
  "vUiq16h\0"
  "sme,(sme-f16f16|sme-f8f16)\0"
  "svadd_za16_f16_vg1x4\0"
  "vUiq32h\0"
  "svadd_za32_f32_vg1x2\0"
  "vUiq8f\0"
  "svadd_za32_f32_vg1x4\0"
  "vUiq16f\0"
  "svadd_za32_s32_vg1x2\0"
  "vUiq8i\0"
  "svadd_za32_s32_vg1x4\0"
  "vUiq16i\0"
  "svadd_za32_u32_vg1x2\0"
  "vUiq8Ui\0"
  "svadd_za32_u32_vg1x4\0"
  "vUiq16Ui\0"
  "svadd_za64_f64_vg1x2\0"
  "vUiq4d\0"
  "sme,sme2,sme-f64f64\0"
  "svadd_za64_f64_vg1x4\0"
  "vUiq8d\0"
  "svadd_za64_s64_vg1x2\0"
  "vUiq4Wi\0"
  "svadd_za64_s64_vg1x4\0"
  "vUiq8Wi\0"
  "svadd_za64_u64_vg1x2\0"
  "vUiq4UWi\0"
  "svadd_za64_u64_vg1x4\0"
  "vUiq8UWi\0"
  "svaddha_za32_s32_m\0"
  "vIUWiq16bq16bq4i\0"
  "svaddha_za32_u32_m\0"
  "vIUWiq16bq16bq4Ui\0"
  "svaddha_za64_s64_m\0"
  "vIUWiq16bq16bq2Wi\0"
  "sme,sme-i16i64\0"
  "svaddha_za64_u64_m\0"
  "vIUWiq16bq16bq2UWi\0"
  "svaddva_za32_s32_m\0"
  "svaddva_za32_u32_m\0"
  "svaddva_za64_s64_m\0"
  "svaddva_za64_u64_m\0"
  "svbmopa_za32_s32_m\0"
  "vIUWiq16bq16bq4iq4i\0"
  "svbmopa_za32_u32_m\0"
  "vIUWiq16bq16bq4Uiq4Ui\0"
  "svbmops_za32_s32_m\0"
  "svbmops_za32_u32_m\0"
  "svcntsb\0"
  "UWiv\0"
  "svcntsd\0"
  "svcntsh\0"
  "svcntsw\0"
  "svdot_lane_za16_mf8_vg1x2_fpm\0"
  "vUiq32mq16mIUWiUWi\0"
  "sme,sme-f8f16\0"
  "svdot_lane_za16_mf8_vg1x4_fpm\0"
  "vUiq64mq16mIUWiUWi\0"
  "svdot_lane_za32_bf16_vg1x2\0"
  "vUiq16yq8yIUWi\0"
  "svdot_lane_za32_bf16_vg1x4\0"
  "vUiq32yq8yIUWi\0"
  "svdot_lane_za32_f16_vg1x2\0"
  "vUiq16hq8hIUWi\0"
  "svdot_lane_za32_f16_vg1x4\0"
  "vUiq32hq8hIUWi\0"
  "svdot_lane_za32_mf8_vg1x2_fpm\0"
  "sme,sme-f8f32\0"
  "svdot_lane_za32_mf8_vg1x4_fpm\0"
  "svdot_lane_za32_s16_vg1x2\0"
  "vUiq16sq8sIUWi\0"
  "svdot_lane_za32_s16_vg1x4\0"
  "vUiq32sq8sIUWi\0"
  "svdot_lane_za32_s8_vg1x2\0"
  "vUiq32Scq16ScIUWi\0"
  "svdot_lane_za32_s8_vg1x4\0"
  "vUiq64Scq16ScIUWi\0"
  "svdot_lane_za32_u16_vg1x2\0"
  "vUiq16Usq8UsIUWi\0"
  "svdot_lane_za32_u16_vg1x4\0"
  "vUiq32Usq8UsIUWi\0"
  "svdot_lane_za32_u8_vg1x2\0"
  "vUiq32Ucq16UcIUWi\0"
  "svdot_lane_za32_u8_vg1x4\0"
  "vUiq64Ucq16UcIUWi\0"
  "svdot_lane_za64_s16_vg1x2\0"
  "svdot_lane_za64_s16_vg1x4\0"
  "svdot_lane_za64_u16_vg1x2\0"
  "svdot_lane_za64_u16_vg1x4\0"
  "svdot_single_za16_mf8_vg1x2_fpm\0"
  "vUiq32mq16mUWi\0"
  "svdot_single_za16_mf8_vg1x4_fpm\0"
  "vUiq64mq16mUWi\0"
  "svdot_single_za32_bf16_vg1x2\0"
  "vUiq16yq8y\0"
  "svdot_single_za32_bf16_vg1x4\0"
  "vUiq32yq8y\0"
  "svdot_single_za32_f16_vg1x2\0"
  "vUiq16hq8h\0"
  "svdot_single_za32_f16_vg1x4\0"
  "vUiq32hq8h\0"
  "svdot_single_za32_mf8_vg1x2_fpm\0"
  "svdot_single_za32_mf8_vg1x4_fpm\0"
  "svdot_single_za32_s16_vg1x2\0"
  "vUiq16sq8s\0"
  "svdot_single_za32_s16_vg1x4\0"
  "vUiq32sq8s\0"
  "svdot_single_za32_s8_vg1x2\0"
  "vUiq32Scq16Sc\0"
  "svdot_single_za32_s8_vg1x4\0"
  "vUiq64Scq16Sc\0"
  "svdot_single_za32_u16_vg1x2\0"
  "vUiq16Usq8Us\0"
  "svdot_single_za32_u16_vg1x4\0"
  "vUiq32Usq8Us\0"
  "svdot_single_za32_u8_vg1x2\0"
  "vUiq32Ucq16Uc\0"
  "svdot_single_za32_u8_vg1x4\0"
  "vUiq64Ucq16Uc\0"
  "svdot_single_za64_s16_vg1x2\0"
  "svdot_single_za64_s16_vg1x4\0"
  "svdot_single_za64_u16_vg1x2\0"
  "svdot_single_za64_u16_vg1x4\0"
  "svdot_za16_mf8_vg1x2_fpm\0"
  "vUiq32mq32mUWi\0"
  "svdot_za16_mf8_vg1x4_fpm\0"
  "vUiq64mq64mUWi\0"
  "svdot_za32_bf16_vg1x2\0"
  "vUiq16yq16y\0"
  "svdot_za32_bf16_vg1x4\0"
  "vUiq32yq32y\0"
  "svdot_za32_f16_vg1x2\0"
  "vUiq16hq16h\0"
  "svdot_za32_f16_vg1x4\0"
  "vUiq32hq32h\0"
  "svdot_za32_mf8_vg1x2_fpm\0"
  "svdot_za32_mf8_vg1x4_fpm\0"
  "svdot_za32_s16_vg1x2\0"
  "vUiq16sq16s\0"
  "svdot_za32_s16_vg1x4\0"
  "vUiq32sq32s\0"
  "svdot_za32_s8_vg1x2\0"
  "vUiq32Scq32Sc\0"
  "svdot_za32_s8_vg1x4\0"
  "vUiq64Scq64Sc\0"
  "svdot_za32_u16_vg1x2\0"
  "vUiq16Usq16Us\0"
  "svdot_za32_u16_vg1x4\0"
  "vUiq32Usq32Us\0"
  "svdot_za32_u8_vg1x2\0"
  "vUiq32Ucq32Uc\0"
  "svdot_za32_u8_vg1x4\0"
  "vUiq64Ucq64Uc\0"
  "svdot_za64_s16_vg1x2\0"
  "svdot_za64_s16_vg1x4\0"
  "svdot_za64_u16_vg1x2\0"
  "svdot_za64_u16_vg1x4\0"
  "svld1_hor_vnum_za128\0"
  "vIUWiUiq16bvC*Wi\0"
  "svld1_hor_vnum_za16\0"
  "svld1_hor_vnum_za32\0"
  "svld1_hor_vnum_za64\0"
  "svld1_hor_vnum_za8\0"
  "svld1_hor_za128\0"
  "vIUWiUiq16bvC*\0"
  "svld1_hor_za16\0"
  "svld1_hor_za32\0"
  "svld1_hor_za64\0"
  "svld1_hor_za8\0"
  "svld1_ver_vnum_za128\0"
  "svld1_ver_vnum_za16\0"
  "svld1_ver_vnum_za32\0"
  "svld1_ver_vnum_za64\0"
  "svld1_ver_vnum_za8\0"
  "svld1_ver_za128\0"
  "svld1_ver_za16\0"
  "svld1_ver_za32\0"
  "svld1_ver_za64\0"
  "svld1_ver_za8\0"
  "svldr_vnum_za\0"
  "vUivC*Wi\0"
  "svldr_za\0"
  "vUivC*\0"
  "svldr_zt\0"
  "vIUWivC*\0"
  "svluti2_lane_zt_bf16\0"
  "q8yIUWiq16UcIUWi\0"
  "svluti2_lane_zt_bf16_x2\0"
  "q16yIUWiq16UcIUWi\0"
  "svluti2_lane_zt_bf16_x4\0"
  "q32yIUWiq16UcIUWi\0"
  "svluti2_lane_zt_f16\0"
  "q8hIUWiq16UcIUWi\0"
  "svluti2_lane_zt_f16_x2\0"
  "q16hIUWiq16UcIUWi\0"
  "svluti2_lane_zt_f16_x4\0"
  "q32hIUWiq16UcIUWi\0"
  "svluti2_lane_zt_f32\0"
  "q4fIUWiq16UcIUWi\0"
  "svluti2_lane_zt_f32_x2\0"
  "q8fIUWiq16UcIUWi\0"
  "svluti2_lane_zt_f32_x4\0"
  "q16fIUWiq16UcIUWi\0"
  "svluti2_lane_zt_mf8\0"
  "q16mIUWiq16UcIUWi\0"
  "svluti2_lane_zt_mf8_x2\0"
  "q32mIUWiq16UcIUWi\0"
  "svluti2_lane_zt_mf8_x4\0"
  "q64mIUWiq16UcIUWi\0"
  "svluti2_lane_zt_s16\0"
  "q8sIUWiq16UcIUWi\0"
  "svluti2_lane_zt_s16_x2\0"
  "q16sIUWiq16UcIUWi\0"
  "svluti2_lane_zt_s16_x4\0"
  "q32sIUWiq16UcIUWi\0"
  "svluti2_lane_zt_s32\0"
  "q4iIUWiq16UcIUWi\0"
  "svluti2_lane_zt_s32_x2\0"
  "q8iIUWiq16UcIUWi\0"
  "svluti2_lane_zt_s32_x4\0"
  "q16iIUWiq16UcIUWi\0"
  "svluti2_lane_zt_s8\0"
  "q16ScIUWiq16UcIUWi\0"
  "svluti2_lane_zt_s8_x2\0"
  "q32ScIUWiq16UcIUWi\0"
  "svluti2_lane_zt_s8_x4\0"
  "q64ScIUWiq16UcIUWi\0"
  "svluti2_lane_zt_u16\0"
  "q8UsIUWiq16UcIUWi\0"
  "svluti2_lane_zt_u16_x2\0"
  "q16UsIUWiq16UcIUWi\0"
  "svluti2_lane_zt_u16_x4\0"
  "q32UsIUWiq16UcIUWi\0"
  "svluti2_lane_zt_u32\0"
  "q4UiIUWiq16UcIUWi\0"
  "svluti2_lane_zt_u32_x2\0"
  "q8UiIUWiq16UcIUWi\0"
  "svluti2_lane_zt_u32_x4\0"
  "q16UiIUWiq16UcIUWi\0"
  "svluti2_lane_zt_u8\0"
  "q16UcIUWiq16UcIUWi\0"
  "svluti2_lane_zt_u8_x2\0"
  "q32UcIUWiq16UcIUWi\0"
  "svluti2_lane_zt_u8_x4\0"
  "q64UcIUWiq16UcIUWi\0"
  "svluti4_lane_zt_bf16\0"
  "svluti4_lane_zt_bf16_x2\0"
  "svluti4_lane_zt_bf16_x4\0"
  "svluti4_lane_zt_f16\0"
  "svluti4_lane_zt_f16_x2\0"
  "svluti4_lane_zt_f16_x4\0"
  "svluti4_lane_zt_f32\0"
  "svluti4_lane_zt_f32_x2\0"
  "svluti4_lane_zt_f32_x4\0"
  "svluti4_lane_zt_mf8\0"
  "svluti4_lane_zt_mf8_x2\0"
  "svluti4_lane_zt_s16\0"
  "svluti4_lane_zt_s16_x2\0"
  "svluti4_lane_zt_s16_x4\0"
  "svluti4_lane_zt_s32\0"
  "svluti4_lane_zt_s32_x2\0"
  "svluti4_lane_zt_s32_x4\0"
  "svluti4_lane_zt_s8\0"
  "svluti4_lane_zt_s8_x2\0"
  "svluti4_lane_zt_u16\0"
  "svluti4_lane_zt_u16_x2\0"
  "svluti4_lane_zt_u16_x4\0"
  "svluti4_lane_zt_u32\0"
  "svluti4_lane_zt_u32_x2\0"
  "svluti4_lane_zt_u32_x4\0"
  "svluti4_lane_zt_u8\0"
  "svluti4_lane_zt_u8_x2\0"
  "svluti4_zt_s8_x4\0"
  "q64ScIUWiq32Uc\0"
  "sme,sme-lutv2\0"
  "svluti4_zt_u8_x4\0"
  "q64UcIUWiq32Uc\0"
  "svmla_lane_za16_bf16_vg1x2\0"
  "svmla_lane_za16_bf16_vg1x4\0"
  "svmla_lane_za16_f16_vg1x2\0"
  "sme,sme-f16f16\0"
  "svmla_lane_za16_f16_vg1x4\0"
  "svmla_lane_za16_mf8_vg2x1_fpm\0"
  "vUiq16mq16mIUWiUWi\0"
  "svmla_lane_za16_mf8_vg2x2_fpm\0"
  "svmla_lane_za16_mf8_vg2x4_fpm\0"
  "svmla_lane_za32_bf16_vg2x1\0"
  "vUiq8yq8yIUWi\0"
  "svmla_lane_za32_bf16_vg2x2\0"
  "svmla_lane_za32_bf16_vg2x4\0"
  "svmla_lane_za32_f16_vg2x1\0"
  "vUiq8hq8hIUWi\0"
  "svmla_lane_za32_f16_vg2x2\0"
  "svmla_lane_za32_f16_vg2x4\0"
  "svmla_lane_za32_f32_vg1x2\0"
  "vUiq8fq4fIUWi\0"
  "svmla_lane_za32_f32_vg1x4\0"
  "vUiq16fq4fIUWi\0"
  "svmla_lane_za32_mf8_vg4x1_fpm\0"
  "svmla_lane_za32_mf8_vg4x2_fpm\0"
  "svmla_lane_za32_mf8_vg4x4_fpm\0"
  "svmla_lane_za32_s16_vg2x1\0"
  "vUiq8sq8sIUWi\0"
  "svmla_lane_za32_s16_vg2x2\0"
  "svmla_lane_za32_s16_vg2x4\0"
  "svmla_lane_za32_s8_vg4x1\0"
  "vUiq16Scq16ScIUWi\0"
  "svmla_lane_za32_s8_vg4x2\0"
  "svmla_lane_za32_s8_vg4x4\0"
  "svmla_lane_za32_u16_vg2x1\0"
  "vUiq8Usq8UsIUWi\0"
  "svmla_lane_za32_u16_vg2x2\0"
  "svmla_lane_za32_u16_vg2x4\0"
  "svmla_lane_za32_u8_vg4x1\0"
  "vUiq16Ucq16UcIUWi\0"
  "svmla_lane_za32_u8_vg4x2\0"
  "svmla_lane_za32_u8_vg4x4\0"
  "svmla_lane_za64_f64_vg1x2\0"
  "vUiq4dq2dIUWi\0"
  "svmla_lane_za64_f64_vg1x4\0"
  "vUiq8dq2dIUWi\0"
  "svmla_lane_za64_s16_vg4x1\0"
  "svmla_lane_za64_s16_vg4x2\0"
  "svmla_lane_za64_s16_vg4x4\0"
  "svmla_lane_za64_u16_vg4x1\0"
  "svmla_lane_za64_u16_vg4x2\0"
  "svmla_lane_za64_u16_vg4x4\0"
  "svmla_single_za16_bf16_vg1x2\0"
  "svmla_single_za16_bf16_vg1x4\0"
  "svmla_single_za16_f16_vg1x2\0"
  "svmla_single_za16_f16_vg1x4\0"
  "svmla_single_za16_mf8_vg2x2_fpm\0"
  "svmla_single_za16_mf8_vg2x4_fpm\0"
  "svmla_single_za32_bf16_vg2x2\0"
  "svmla_single_za32_bf16_vg2x4\0"
  "svmla_single_za32_f16_vg2x2\0"
  "svmla_single_za32_f16_vg2x4\0"
  "svmla_single_za32_f32_vg1x2\0"
  "vUiq8fq4f\0"
  "svmla_single_za32_f32_vg1x4\0"
  "vUiq16fq4f\0"
  "svmla_single_za32_mf8_vg4x2_fpm\0"
  "svmla_single_za32_mf8_vg4x4_fpm\0"
  "svmla_single_za32_s16_vg2x2\0"
  "svmla_single_za32_s16_vg2x4\0"
  "svmla_single_za32_s8_vg4x2\0"
  "svmla_single_za32_s8_vg4x4\0"
  "svmla_single_za32_u16_vg2x2\0"
  "svmla_single_za32_u16_vg2x4\0"
  "svmla_single_za32_u8_vg4x2\0"
  "svmla_single_za32_u8_vg4x4\0"
  "svmla_single_za64_f64_vg1x2\0"
  "vUiq4dq2d\0"
  "svmla_single_za64_f64_vg1x4\0"
  "vUiq8dq2d\0"
  "svmla_single_za64_s16_vg4x2\0"
  "svmla_single_za64_s16_vg4x4\0"
  "svmla_single_za64_u16_vg4x2\0"
  "svmla_single_za64_u16_vg4x4\0"
  "svmla_za16_bf16_vg1x2\0"
  "svmla_za16_bf16_vg1x4\0"
  "svmla_za16_f16_vg1x2\0"
  "svmla_za16_f16_vg1x4\0"
  "svmla_za16_mf8_vg2x1_fpm\0"
  "vUiq16mq16mUWi\0"
  "svmla_za16_mf8_vg2x2_fpm\0"
  "svmla_za16_mf8_vg2x4_fpm\0"
  "svmla_za32_bf16_vg2x1\0"
  "vUiq8yq8y\0"
  "svmla_za32_bf16_vg2x2\0"
  "svmla_za32_bf16_vg2x4\0"
  "svmla_za32_f16_vg2x1\0"
  "vUiq8hq8h\0"
  "svmla_za32_f16_vg2x2\0"
  "svmla_za32_f16_vg2x4\0"
  "svmla_za32_f32_vg1x2\0"
  "vUiq8fq8f\0"
  "svmla_za32_f32_vg1x4\0"
  "vUiq16fq16f\0"
  "svmla_za32_mf8_vg4x1_fpm\0"
  "svmla_za32_mf8_vg4x2_fpm\0"
  "svmla_za32_mf8_vg4x4_fpm\0"
  "svmla_za32_s16_vg2x1\0"
  "vUiq8sq8s\0"
  "svmla_za32_s16_vg2x2\0"
  "svmla_za32_s16_vg2x4\0"
  "svmla_za32_s8_vg4x1\0"
  "vUiq16Scq16Sc\0"
  "svmla_za32_s8_vg4x2\0"
  "svmla_za32_s8_vg4x4\0"
  "svmla_za32_u16_vg2x1\0"
  "vUiq8Usq8Us\0"
  "svmla_za32_u16_vg2x2\0"
  "svmla_za32_u16_vg2x4\0"
  "svmla_za32_u8_vg4x1\0"
  "vUiq16Ucq16Uc\0"
  "svmla_za32_u8_vg4x2\0"
  "svmla_za32_u8_vg4x4\0"
  "svmla_za64_f64_vg1x2\0"
  "vUiq4dq4d\0"
  "svmla_za64_f64_vg1x4\0"
  "vUiq8dq8d\0"
  "svmla_za64_s16_vg4x1\0"
  "svmla_za64_s16_vg4x2\0"
  "svmla_za64_s16_vg4x4\0"
  "svmla_za64_u16_vg4x1\0"
  "svmla_za64_u16_vg4x2\0"
  "svmla_za64_u16_vg4x4\0"
  "svmls_lane_za16_bf16_vg1x2\0"
  "svmls_lane_za16_bf16_vg1x4\0"
  "svmls_lane_za16_f16_vg1x2\0"
  "svmls_lane_za16_f16_vg1x4\0"
  "svmls_lane_za32_bf16_vg2x1\0"
  "svmls_lane_za32_bf16_vg2x2\0"
  "svmls_lane_za32_bf16_vg2x4\0"
  "svmls_lane_za32_f16_vg2x1\0"
  "svmls_lane_za32_f16_vg2x2\0"
  "svmls_lane_za32_f16_vg2x4\0"
  "svmls_lane_za32_f32_vg1x2\0"
  "svmls_lane_za32_f32_vg1x4\0"
  "svmls_lane_za32_s16_vg2x1\0"
  "svmls_lane_za32_s16_vg2x2\0"
  "svmls_lane_za32_s16_vg2x4\0"
  "svmls_lane_za32_s8_vg4x1\0"
  "svmls_lane_za32_s8_vg4x2\0"
  "svmls_lane_za32_s8_vg4x4\0"
  "svmls_lane_za32_u16_vg2x1\0"
  "svmls_lane_za32_u16_vg2x2\0"
  "svmls_lane_za32_u16_vg2x4\0"
  "svmls_lane_za32_u8_vg4x1\0"
  "svmls_lane_za32_u8_vg4x2\0"
  "svmls_lane_za32_u8_vg4x4\0"
  "svmls_lane_za64_f64_vg1x2\0"
  "svmls_lane_za64_f64_vg1x4\0"
  "svmls_lane_za64_s16_vg4x1\0"
  "svmls_lane_za64_s16_vg4x2\0"
  "svmls_lane_za64_s16_vg4x4\0"
  "svmls_lane_za64_u16_vg4x1\0"
  "svmls_lane_za64_u16_vg4x2\0"
  "svmls_lane_za64_u16_vg4x4\0"
  "svmls_single_za16_bf16_vg1x2\0"
  "svmls_single_za16_bf16_vg1x4\0"
  "svmls_single_za16_f16_vg1x2\0"
  "svmls_single_za16_f16_vg1x4\0"
  "svmls_single_za32_bf16_vg2x2\0"
  "svmls_single_za32_bf16_vg2x4\0"
  "svmls_single_za32_f16_vg2x2\0"
  "svmls_single_za32_f16_vg2x4\0"
  "svmls_single_za32_f32_vg1x2\0"
  "svmls_single_za32_f32_vg1x4\0"
  "svmls_single_za32_s16_vg2x2\0"
  "svmls_single_za32_s16_vg2x4\0"
  "svmls_single_za32_s8_vg4x2\0"
  "svmls_single_za32_s8_vg4x4\0"
  "svmls_single_za32_u16_vg2x2\0"
  "svmls_single_za32_u16_vg2x4\0"
  "svmls_single_za32_u8_vg4x2\0"
  "svmls_single_za32_u8_vg4x4\0"
  "svmls_single_za64_f64_vg1x2\0"
  "svmls_single_za64_f64_vg1x4\0"
  "svmls_single_za64_s16_vg4x2\0"
  "svmls_single_za64_s16_vg4x4\0"
  "svmls_single_za64_u16_vg4x2\0"
  "svmls_single_za64_u16_vg4x4\0"
  "svmls_za16_bf16_vg1x2\0"
  "svmls_za16_bf16_vg1x4\0"
  "svmls_za16_f16_vg1x2\0"
  "svmls_za16_f16_vg1x4\0"
  "svmls_za32_bf16_vg2x1\0"
  "svmls_za32_bf16_vg2x2\0"
  "svmls_za32_bf16_vg2x4\0"
  "svmls_za32_f16_vg2x1\0"
  "svmls_za32_f16_vg2x2\0"
  "svmls_za32_f16_vg2x4\0"
  "svmls_za32_f32_vg1x2\0"
  "svmls_za32_f32_vg1x4\0"
  "svmls_za32_s16_vg2x1\0"
  "svmls_za32_s16_vg2x2\0"
  "svmls_za32_s16_vg2x4\0"
  "svmls_za32_s8_vg4x1\0"
  "svmls_za32_s8_vg4x2\0"
  "svmls_za32_s8_vg4x4\0"
  "svmls_za32_u16_vg2x1\0"
  "svmls_za32_u16_vg2x2\0"
  "svmls_za32_u16_vg2x4\0"
  "svmls_za32_u8_vg4x1\0"
  "svmls_za32_u8_vg4x2\0"
  "svmls_za32_u8_vg4x4\0"
  "svmls_za64_f64_vg1x2\0"
  "svmls_za64_f64_vg1x4\0"
  "svmls_za64_s16_vg4x1\0"
  "svmls_za64_s16_vg4x2\0"
  "svmls_za64_s16_vg4x4\0"
  "svmls_za64_u16_vg4x1\0"
  "svmls_za64_u16_vg4x2\0"
  "svmls_za64_u16_vg4x4\0"
  "svmop4a_1x1_za16_bf16_bf16\0"
  "vIUWiq8yq8y\0"
  "sme,sme2,sme-mop4,sme-b16b16\0"
  "svmop4a_1x1_za16_f16_f16\0"
  "vIUWiq8hq8h\0"
  "sme,sme2,sme-mop4,sme-f16f16\0"
  "svmop4a_1x1_za16_mf8_mf8_fpm\0"
  "vIUWiq16mq16mUWi\0"
  "sme,sme2,sme-mop4,sme-f8f16\0"
  "svmop4a_1x1_za32_bf16_bf16\0"
  "sme,sme2,sme-mop4\0"
  "svmop4a_1x1_za32_f16_f16\0"
  "svmop4a_1x1_za32_f32_f32\0"
  "vIUWiq4fq4f\0"
  "svmop4a_1x1_za32_mf8_mf8_fpm\0"
  "sme,sme2,sme-mop4,sme-f8f32\0"
  "svmop4a_1x1_za32_s16_s16\0"
  "vIUWiq8sq8s\0"
  "svmop4a_1x1_za32_s8_s8\0"
  "vIUWiq16Scq16Sc\0"
  "svmop4a_1x1_za32_s8_u8\0"
  "vIUWiq16Scq16Uc\0"
  "svmop4a_1x1_za32_u16_u16\0"
  "vIUWiq8Usq8Us\0"
  "svmop4a_1x1_za32_u8_s8\0"
  "vIUWiq16Ucq16Sc\0"
  "svmop4a_1x1_za32_u8_u8\0"
  "vIUWiq16Ucq16Uc\0"
  "svmop4a_1x1_za64_f64_f64\0"
  "vIUWiq2dq2d\0"
  "sme,sme2,sme-mop4,sme-f64f64\0"
  "svmop4a_1x1_za64_s16_s16\0"
  "sme,sme2,sme-mop4,sme-i16i64\0"
  "svmop4a_1x1_za64_s16_u16\0"
  "vIUWiq8sq8Us\0"
  "svmop4a_1x1_za64_u16_s16\0"
  "vIUWiq8Usq8s\0"
  "svmop4a_1x1_za64_u16_u16\0"
  "svmop4a_1x2_za16_bf16_bf16\0"
  "vIUWiq8yq16y\0"
  "svmop4a_1x2_za16_f16_f16\0"
  "vIUWiq8hq16h\0"
  "svmop4a_1x2_za16_mf8_mf8_fpm\0"
  "vIUWiq16mq32mUWi\0"
  "svmop4a_1x2_za32_bf16_bf16\0"
  "svmop4a_1x2_za32_f16_f16\0"
  "svmop4a_1x2_za32_f32_f32\0"
  "vIUWiq4fq8f\0"
  "svmop4a_1x2_za32_mf8_mf8_fpm\0"
  "svmop4a_1x2_za32_s16_s16\0"
  "vIUWiq8sq16s\0"
  "svmop4a_1x2_za32_s8_s8\0"
  "vIUWiq16Scq32Sc\0"
  "svmop4a_1x2_za32_s8_u8\0"
  "vIUWiq16Scq32Uc\0"
  "svmop4a_1x2_za32_u16_u16\0"
  "vIUWiq8Usq16Us\0"
  "svmop4a_1x2_za32_u8_s8\0"
  "vIUWiq16Ucq32Sc\0"
  "svmop4a_1x2_za32_u8_u8\0"
  "vIUWiq16Ucq32Uc\0"
  "svmop4a_1x2_za64_f64_f64\0"
  "vIUWiq2dq4d\0"
  "svmop4a_1x2_za64_s16_s16\0"
  "svmop4a_1x2_za64_s16_u16\0"
  "vIUWiq8sq16Us\0"
  "svmop4a_1x2_za64_u16_s16\0"
  "vIUWiq8Usq16s\0"
  "svmop4a_1x2_za64_u16_u16\0"
  "svmop4a_2x1_za16_bf16_bf16\0"
  "vIUWiq16yq8y\0"
  "svmop4a_2x1_za16_f16_f16\0"
  "vIUWiq16hq8h\0"
  "svmop4a_2x1_za16_mf8_mf8_fpm\0"
  "vIUWiq32mq16mUWi\0"
  "svmop4a_2x1_za32_bf16_bf16\0"
  "svmop4a_2x1_za32_f16_f16\0"
  "svmop4a_2x1_za32_f32_f32\0"
  "vIUWiq8fq4f\0"
  "svmop4a_2x1_za32_mf8_mf8_fpm\0"
  "svmop4a_2x1_za32_s16_s16\0"
  "vIUWiq16sq8s\0"
  "svmop4a_2x1_za32_s8_s8\0"
  "vIUWiq32Scq16Sc\0"
  "svmop4a_2x1_za32_s8_u8\0"
  "vIUWiq32Scq16Uc\0"
  "svmop4a_2x1_za32_u16_u16\0"
  "vIUWiq16Usq8Us\0"
  "svmop4a_2x1_za32_u8_s8\0"
  "vIUWiq32Ucq16Sc\0"
  "svmop4a_2x1_za32_u8_u8\0"
  "vIUWiq32Ucq16Uc\0"
  "svmop4a_2x1_za64_f64_f64\0"
  "vIUWiq4dq2d\0"
  "svmop4a_2x1_za64_s16_s16\0"
  "svmop4a_2x1_za64_s16_u16\0"
  "vIUWiq16sq8Us\0"
  "svmop4a_2x1_za64_u16_s16\0"
  "vIUWiq16Usq8s\0"
  "svmop4a_2x1_za64_u16_u16\0"
  "svmop4a_2x2_za16_bf16_bf16\0"
  "vIUWiq16yq16y\0"
  "svmop4a_2x2_za16_f16_f16\0"
  "vIUWiq16hq16h\0"
  "svmop4a_2x2_za16_mf8_mf8_fpm\0"
  "vIUWiq32mq32mUWi\0"
  "svmop4a_2x2_za32_bf16_bf16\0"
  "svmop4a_2x2_za32_f16_f16\0"
  "svmop4a_2x2_za32_f32_f32\0"
  "vIUWiq8fq8f\0"
  "svmop4a_2x2_za32_mf8_mf8_fpm\0"
  "svmop4a_2x2_za32_s16_s16\0"
  "vIUWiq16sq16s\0"
  "svmop4a_2x2_za32_s8_s8\0"
  "vIUWiq32Scq32Sc\0"
  "svmop4a_2x2_za32_s8_u8\0"
  "vIUWiq32Scq32Uc\0"
  "svmop4a_2x2_za32_u16_u16\0"
  "vIUWiq16Usq16Us\0"
  "svmop4a_2x2_za32_u8_s8\0"
  "vIUWiq32Ucq32Sc\0"
  "svmop4a_2x2_za32_u8_u8\0"
  "vIUWiq32Ucq32Uc\0"
  "svmop4a_2x2_za64_f64_f64\0"
  "vIUWiq4dq4d\0"
  "svmop4a_2x2_za64_s16_s16\0"
  "svmop4a_2x2_za64_s16_u16\0"
  "vIUWiq16sq16Us\0"
  "svmop4a_2x2_za64_u16_s16\0"
  "vIUWiq16Usq16s\0"
  "svmop4a_2x2_za64_u16_u16\0"
  "svmop4s_1x1_za16_bf16_bf16\0"
  "svmop4s_1x1_za16_f16_f16\0"
  "svmop4s_1x1_za32_bf16_bf16\0"
  "svmop4s_1x1_za32_f16_f16\0"
  "svmop4s_1x1_za32_f32_f32\0"
  "svmop4s_1x1_za32_s16_s16\0"
  "svmop4s_1x1_za32_s8_s8\0"
  "svmop4s_1x1_za32_s8_u8\0"
  "svmop4s_1x1_za32_u16_u16\0"
  "svmop4s_1x1_za32_u8_s8\0"
  "svmop4s_1x1_za32_u8_u8\0"
  "svmop4s_1x1_za64_f64_f64\0"
  "svmop4s_1x1_za64_s16_s16\0"
  "svmop4s_1x1_za64_s16_u16\0"
  "svmop4s_1x1_za64_u16_s16\0"
  "svmop4s_1x1_za64_u16_u16\0"
  "svmop4s_1x2_za16_bf16_bf16\0"
  "svmop4s_1x2_za16_f16_f16\0"
  "svmop4s_1x2_za32_bf16_bf16\0"
  "svmop4s_1x2_za32_f16_f16\0"
  "svmop4s_1x2_za32_f32_f32\0"
  "svmop4s_1x2_za32_s16_s16\0"
  "svmop4s_1x2_za32_s8_s8\0"
  "svmop4s_1x2_za32_s8_u8\0"
  "svmop4s_1x2_za32_u16_u16\0"
  "svmop4s_1x2_za32_u8_s8\0"
  "svmop4s_1x2_za32_u8_u8\0"
  "svmop4s_1x2_za64_f64_f64\0"
  "svmop4s_1x2_za64_s16_s16\0"
  "svmop4s_1x2_za64_s16_u16\0"
  "svmop4s_1x2_za64_u16_s16\0"
  "svmop4s_1x2_za64_u16_u16\0"
  "svmop4s_2x1_za16_bf16_bf16\0"
  "svmop4s_2x1_za16_f16_f16\0"
  "svmop4s_2x1_za32_bf16_bf16\0"
  "svmop4s_2x1_za32_f16_f16\0"
  "svmop4s_2x1_za32_f32_f32\0"
  "svmop4s_2x1_za32_s16_s16\0"
  "svmop4s_2x1_za32_s8_s8\0"
  "svmop4s_2x1_za32_s8_u8\0"
  "svmop4s_2x1_za32_u16_u16\0"
  "svmop4s_2x1_za32_u8_s8\0"
  "svmop4s_2x1_za32_u8_u8\0"
  "svmop4s_2x1_za64_f64_f64\0"
  "svmop4s_2x1_za64_s16_s16\0"
  "svmop4s_2x1_za64_s16_u16\0"
  "svmop4s_2x1_za64_u16_s16\0"
  "svmop4s_2x1_za64_u16_u16\0"
  "svmop4s_2x2_za16_bf16_bf16\0"
  "svmop4s_2x2_za16_f16_f16\0"
  "svmop4s_2x2_za32_bf16_bf16\0"
  "svmop4s_2x2_za32_f16_f16\0"
  "svmop4s_2x2_za32_f32_f32\0"
  "svmop4s_2x2_za32_s16_s16\0"
  "svmop4s_2x2_za32_s8_s8\0"
  "svmop4s_2x2_za32_s8_u8\0"
  "svmop4s_2x2_za32_u16_u16\0"
  "svmop4s_2x2_za32_u8_s8\0"
  "svmop4s_2x2_za32_u8_u8\0"
  "svmop4s_2x2_za64_f64_f64\0"
  "svmop4s_2x2_za64_s16_s16\0"
  "svmop4s_2x2_za64_s16_u16\0"
  "svmop4s_2x2_za64_u16_s16\0"
  "svmop4s_2x2_za64_u16_u16\0"
  "svmopa_za16_bf16_m\0"
  "vIUWiq16bq16bq8yq8y\0"
  "svmopa_za16_f16_m\0"
  "vIUWiq16bq16bq8hq8h\0"
  "svmopa_za16_mf8_m_fpm\0"
  "vIUWiq16bq16bq16mq16mUWi\0"
  "svmopa_za32_bf16_m\0"
  "svmopa_za32_f16_m\0"
  "svmopa_za32_f32_m\0"
  "vIUWiq16bq16bq4fq4f\0"
  "svmopa_za32_mf8_m_fpm\0"
  "svmopa_za32_s16_m\0"
  "vIUWiq16bq16bq8sq8s\0"
  "svmopa_za32_s8_m\0"
  "vIUWiq16bq16bq16Scq16Sc\0"
  "svmopa_za32_u16_m\0"
  "vIUWiq16bq16bq8Usq8Us\0"
  "svmopa_za32_u8_m\0"
  "vIUWiq16bq16bq16Ucq16Uc\0"
  "svmopa_za64_f64_m\0"
  "vIUWiq16bq16bq2dq2d\0"
  "sme,sme-f64f64\0"
  "svmopa_za64_s16_m\0"
  "svmopa_za64_u16_m\0"
  "svmops_za16_bf16_m\0"
  "svmops_za16_f16_m\0"
  "svmops_za32_bf16_m\0"
  "svmops_za32_f16_m\0"
  "svmops_za32_f32_m\0"
  "svmops_za32_s16_m\0"
  "svmops_za32_s8_m\0"
  "svmops_za32_u16_m\0"
  "svmops_za32_u8_m\0"
  "svmops_za64_f64_m\0"
  "svmops_za64_s16_m\0"
  "svmops_za64_u16_m\0"
  "svread_hor_za128_bf16_m\0"
  "q8yq8yq16bIUWiUi\0"
  "svread_hor_za128_f16_m\0"
  "q8hq8hq16bIUWiUi\0"
  "svread_hor_za128_f32_m\0"
  "q4fq4fq16bIUWiUi\0"
  "svread_hor_za128_f64_m\0"
  "q2dq2dq16bIUWiUi\0"
  "svread_hor_za128_mf8_m\0"
  "q16mq16mq16bIUWiUi\0"
  "svread_hor_za128_s16_m\0"
  "q8sq8sq16bIUWiUi\0"
  "svread_hor_za128_s32_m\0"
  "q4iq4iq16bIUWiUi\0"
  "svread_hor_za128_s64_m\0"
  "q2Wiq2Wiq16bIUWiUi\0"
  "svread_hor_za128_s8_m\0"
  "q16Scq16Scq16bIUWiUi\0"
  "svread_hor_za128_u16_m\0"
  "q8Usq8Usq16bIUWiUi\0"
  "svread_hor_za128_u32_m\0"
  "q4Uiq4Uiq16bIUWiUi\0"
  "svread_hor_za128_u64_m\0"
  "q2UWiq2UWiq16bIUWiUi\0"
  "svread_hor_za128_u8_m\0"
  "q16Ucq16Ucq16bIUWiUi\0"
  "svread_hor_za16_bf16_m\0"
  "svread_hor_za16_bf16_vg2\0"
  "q16yIUWiUi\0"
  "svread_hor_za16_bf16_vg4\0"
  "q32yIUWiUi\0"
  "svread_hor_za16_f16_m\0"
  "svread_hor_za16_f16_vg2\0"
  "q16hIUWiUi\0"
  "svread_hor_za16_f16_vg4\0"
  "q32hIUWiUi\0"
  "svread_hor_za16_s16_m\0"
  "svread_hor_za16_s16_vg2\0"
  "q16sIUWiUi\0"
  "svread_hor_za16_s16_vg4\0"
  "q32sIUWiUi\0"
  "svread_hor_za16_u16_m\0"
  "svread_hor_za16_u16_vg2\0"
  "q16UsIUWiUi\0"
  "svread_hor_za16_u16_vg4\0"
  "q32UsIUWiUi\0"
  "svread_hor_za32_f32_m\0"
  "svread_hor_za32_f32_vg2\0"
  "q8fIUWiUi\0"
  "svread_hor_za32_f32_vg4\0"
  "q16fIUWiUi\0"
  "svread_hor_za32_s32_m\0"
  "svread_hor_za32_s32_vg2\0"
  "q8iIUWiUi\0"
  "svread_hor_za32_s32_vg4\0"
  "q16iIUWiUi\0"
  "svread_hor_za32_u32_m\0"
  "svread_hor_za32_u32_vg2\0"
  "q8UiIUWiUi\0"
  "svread_hor_za32_u32_vg4\0"
  "q16UiIUWiUi\0"
  "svread_hor_za64_f64_m\0"
  "svread_hor_za64_f64_vg2\0"
  "q4dIUWiUi\0"
  "svread_hor_za64_f64_vg4\0"
  "q8dIUWiUi\0"
  "svread_hor_za64_s64_m\0"
  "svread_hor_za64_s64_vg2\0"
  "q4WiIUWiUi\0"
  "svread_hor_za64_s64_vg4\0"
  "q8WiIUWiUi\0"
  "svread_hor_za64_u64_m\0"
  "svread_hor_za64_u64_vg2\0"
  "q4UWiIUWiUi\0"
  "svread_hor_za64_u64_vg4\0"
  "q8UWiIUWiUi\0"
  "svread_hor_za8_mf8_m\0"
  "svread_hor_za8_mf8_vg2\0"
  "q32mIUWiUi\0"
  "svread_hor_za8_mf8_vg4\0"
  "q64mIUWiUi\0"
  "svread_hor_za8_s8_m\0"
  "svread_hor_za8_s8_vg2\0"
  "q32ScIUWiUi\0"
  "svread_hor_za8_s8_vg4\0"
  "q64ScIUWiUi\0"
  "svread_hor_za8_u8_m\0"
  "svread_hor_za8_u8_vg2\0"
  "q32UcIUWiUi\0"
  "svread_hor_za8_u8_vg4\0"
  "q64UcIUWiUi\0"
  "svread_ver_za128_bf16_m\0"
  "svread_ver_za128_f16_m\0"
  "svread_ver_za128_f32_m\0"
  "svread_ver_za128_f64_m\0"
  "svread_ver_za128_mf8_m\0"
  "svread_ver_za128_s16_m\0"
  "svread_ver_za128_s32_m\0"
  "svread_ver_za128_s64_m\0"
  "svread_ver_za128_s8_m\0"
  "svread_ver_za128_u16_m\0"
  "svread_ver_za128_u32_m\0"
  "svread_ver_za128_u64_m\0"
  "svread_ver_za128_u8_m\0"
  "svread_ver_za16_bf16_m\0"
  "svread_ver_za16_bf16_vg2\0"
  "svread_ver_za16_bf16_vg4\0"
  "svread_ver_za16_f16_m\0"
  "svread_ver_za16_f16_vg2\0"
  "svread_ver_za16_f16_vg4\0"
  "svread_ver_za16_s16_m\0"
  "svread_ver_za16_s16_vg2\0"
  "svread_ver_za16_s16_vg4\0"
  "svread_ver_za16_u16_m\0"
  "svread_ver_za16_u16_vg2\0"
  "svread_ver_za16_u16_vg4\0"
  "svread_ver_za32_f32_m\0"
  "svread_ver_za32_f32_vg2\0"
  "svread_ver_za32_f32_vg4\0"
  "svread_ver_za32_s32_m\0"
  "svread_ver_za32_s32_vg2\0"
  "svread_ver_za32_s32_vg4\0"
  "svread_ver_za32_u32_m\0"
  "svread_ver_za32_u32_vg2\0"
  "svread_ver_za32_u32_vg4\0"
  "svread_ver_za64_f64_m\0"
  "svread_ver_za64_f64_vg2\0"
  "svread_ver_za64_f64_vg4\0"
  "svread_ver_za64_s64_m\0"
  "svread_ver_za64_s64_vg2\0"
  "svread_ver_za64_s64_vg4\0"
  "svread_ver_za64_u64_m\0"
  "svread_ver_za64_u64_vg2\0"
  "svread_ver_za64_u64_vg4\0"
  "svread_ver_za8_mf8_m\0"
  "svread_ver_za8_mf8_vg2\0"
  "svread_ver_za8_mf8_vg4\0"
  "svread_ver_za8_s8_m\0"
  "svread_ver_za8_s8_vg2\0"
  "svread_ver_za8_s8_vg4\0"
  "svread_ver_za8_u8_m\0"
  "svread_ver_za8_u8_vg2\0"
  "svread_ver_za8_u8_vg4\0"
  "svread_za16_bf16_vg1x2\0"
  "q16yUi\0"
  "svread_za16_bf16_vg1x4\0"
  "q32yUi\0"
  "svread_za16_f16_vg1x2\0"
  "q16hUi\0"
  "svread_za16_f16_vg1x4\0"
  "q32hUi\0"
  "svread_za16_s16_vg1x2\0"
  "q16sUi\0"
  "svread_za16_s16_vg1x4\0"
  "q32sUi\0"
  "svread_za16_u16_vg1x2\0"
  "q16UsUi\0"
  "svread_za16_u16_vg1x4\0"
  "q32UsUi\0"
  "svread_za32_f32_vg1x2\0"
  "q8fUi\0"
  "svread_za32_f32_vg1x4\0"
  "q16fUi\0"
  "svread_za32_s32_vg1x2\0"
  "q8iUi\0"
  "svread_za32_s32_vg1x4\0"
  "q16iUi\0"
  "svread_za32_u32_vg1x2\0"
  "q8UiUi\0"
  "svread_za32_u32_vg1x4\0"
  "q16UiUi\0"
  "svread_za64_f64_vg1x2\0"
  "q4dUi\0"
  "svread_za64_f64_vg1x4\0"
  "q8dUi\0"
  "svread_za64_s64_vg1x2\0"
  "q4WiUi\0"
  "svread_za64_s64_vg1x4\0"
  "q8WiUi\0"
  "svread_za64_u64_vg1x2\0"
  "q4UWiUi\0"
  "svread_za64_u64_vg1x4\0"
  "q8UWiUi\0"
  "svread_za8_mf8_vg1x2\0"
  "q32mUi\0"
  "svread_za8_mf8_vg1x4\0"
  "q64mUi\0"
  "svread_za8_s8_vg1x2\0"
  "q32ScUi\0"
  "svread_za8_s8_vg1x4\0"
  "q64ScUi\0"
  "svread_za8_u8_vg1x2\0"
  "q32UcUi\0"
  "svread_za8_u8_vg1x4\0"
  "q64UcUi\0"
  "svreadz_hor_za128_bf16\0"
  "q8yIUWiUi\0"
  "sme,sme2p1\0"
  "svreadz_hor_za128_f16\0"
  "q8hIUWiUi\0"
  "svreadz_hor_za128_f32\0"
  "q4fIUWiUi\0"
  "svreadz_hor_za128_f64\0"
  "q2dIUWiUi\0"
  "svreadz_hor_za128_mf8\0"
  "q16mIUWiUi\0"
  "svreadz_hor_za128_s16\0"
  "q8sIUWiUi\0"
  "svreadz_hor_za128_s32\0"
  "q4iIUWiUi\0"
  "svreadz_hor_za128_s64\0"
  "q2WiIUWiUi\0"
  "svreadz_hor_za128_s8\0"
  "q16ScIUWiUi\0"
  "svreadz_hor_za128_u16\0"
  "q8UsIUWiUi\0"
  "svreadz_hor_za128_u32\0"
  "q4UiIUWiUi\0"
  "svreadz_hor_za128_u64\0"
  "q2UWiIUWiUi\0"
  "svreadz_hor_za128_u8\0"
  "q16UcIUWiUi\0"
  "svreadz_hor_za16_bf16\0"
  "svreadz_hor_za16_bf16_vg2\0"
  "svreadz_hor_za16_bf16_vg4\0"
  "svreadz_hor_za16_f16\0"
  "svreadz_hor_za16_f16_vg2\0"
  "svreadz_hor_za16_f16_vg4\0"
  "svreadz_hor_za16_s16\0"
  "svreadz_hor_za16_s16_vg2\0"
  "svreadz_hor_za16_s16_vg4\0"
  "svreadz_hor_za16_u16\0"
  "svreadz_hor_za16_u16_vg2\0"
  "svreadz_hor_za16_u16_vg4\0"
  "svreadz_hor_za32_f32\0"
  "svreadz_hor_za32_f32_vg2\0"
  "svreadz_hor_za32_f32_vg4\0"
  "svreadz_hor_za32_s32\0"
  "svreadz_hor_za32_s32_vg2\0"
  "svreadz_hor_za32_s32_vg4\0"
  "svreadz_hor_za32_u32\0"
  "svreadz_hor_za32_u32_vg2\0"
  "svreadz_hor_za32_u32_vg4\0"
  "svreadz_hor_za64_f64\0"
  "svreadz_hor_za64_f64_vg2\0"
  "svreadz_hor_za64_f64_vg4\0"
  "svreadz_hor_za64_s64\0"
  "svreadz_hor_za64_s64_vg2\0"
  "svreadz_hor_za64_s64_vg4\0"
  "svreadz_hor_za64_u64\0"
  "svreadz_hor_za64_u64_vg2\0"
  "svreadz_hor_za64_u64_vg4\0"
  "svreadz_hor_za8_mf8\0"
  "svreadz_hor_za8_mf8_vg2\0"
  "svreadz_hor_za8_mf8_vg4\0"
  "svreadz_hor_za8_s8\0"
  "svreadz_hor_za8_s8_vg2\0"
  "svreadz_hor_za8_s8_vg4\0"
  "svreadz_hor_za8_u8\0"
  "svreadz_hor_za8_u8_vg2\0"
  "svreadz_hor_za8_u8_vg4\0"
  "svreadz_ver_za128_bf16\0"
  "svreadz_ver_za128_f16\0"
  "svreadz_ver_za128_f32\0"
  "svreadz_ver_za128_f64\0"
  "svreadz_ver_za128_mf8\0"
  "svreadz_ver_za128_s16\0"
  "svreadz_ver_za128_s32\0"
  "svreadz_ver_za128_s64\0"
  "svreadz_ver_za128_s8\0"
  "svreadz_ver_za128_u16\0"
  "svreadz_ver_za128_u32\0"
  "svreadz_ver_za128_u64\0"
  "svreadz_ver_za128_u8\0"
  "svreadz_ver_za16_bf16\0"
  "svreadz_ver_za16_bf16_vg2\0"
  "svreadz_ver_za16_bf16_vg4\0"
  "svreadz_ver_za16_f16\0"
  "svreadz_ver_za16_f16_vg2\0"
  "svreadz_ver_za16_f16_vg4\0"
  "svreadz_ver_za16_s16\0"
  "svreadz_ver_za16_s16_vg2\0"
  "svreadz_ver_za16_s16_vg4\0"
  "svreadz_ver_za16_u16\0"
  "svreadz_ver_za16_u16_vg2\0"
  "svreadz_ver_za16_u16_vg4\0"
  "svreadz_ver_za32_f32\0"
  "svreadz_ver_za32_f32_vg2\0"
  "svreadz_ver_za32_f32_vg4\0"
  "svreadz_ver_za32_s32\0"
  "svreadz_ver_za32_s32_vg2\0"
  "svreadz_ver_za32_s32_vg4\0"
  "svreadz_ver_za32_u32\0"
  "svreadz_ver_za32_u32_vg2\0"
  "svreadz_ver_za32_u32_vg4\0"
  "svreadz_ver_za64_f64\0"
  "svreadz_ver_za64_f64_vg2\0"
  "svreadz_ver_za64_f64_vg4\0"
  "svreadz_ver_za64_s64\0"
  "svreadz_ver_za64_s64_vg2\0"
  "svreadz_ver_za64_s64_vg4\0"
  "svreadz_ver_za64_u64\0"
  "svreadz_ver_za64_u64_vg2\0"
  "svreadz_ver_za64_u64_vg4\0"
  "svreadz_ver_za8_mf8\0"
  "svreadz_ver_za8_mf8_vg2\0"
  "svreadz_ver_za8_mf8_vg4\0"
  "svreadz_ver_za8_s8\0"
  "svreadz_ver_za8_s8_vg2\0"
  "svreadz_ver_za8_s8_vg4\0"
  "svreadz_ver_za8_u8\0"
  "svreadz_ver_za8_u8_vg2\0"
  "svreadz_ver_za8_u8_vg4\0"
  "svreadz_za16_bf16_vg1x2\0"
  "svreadz_za16_bf16_vg1x4\0"
  "svreadz_za16_f16_vg1x2\0"
  "svreadz_za16_f16_vg1x4\0"
  "svreadz_za16_s16_vg1x2\0"
  "svreadz_za16_s16_vg1x4\0"
  "svreadz_za16_u16_vg1x2\0"
  "svreadz_za16_u16_vg1x4\0"
  "svreadz_za32_f32_vg1x2\0"
  "svreadz_za32_f32_vg1x4\0"
  "svreadz_za32_s32_vg1x2\0"
  "svreadz_za32_s32_vg1x4\0"
  "svreadz_za32_u32_vg1x2\0"
  "svreadz_za32_u32_vg1x4\0"
  "svreadz_za64_f64_vg1x2\0"
  "svreadz_za64_f64_vg1x4\0"
  "svreadz_za64_s64_vg1x2\0"
  "svreadz_za64_s64_vg1x4\0"
  "svreadz_za64_u64_vg1x2\0"
  "svreadz_za64_u64_vg1x4\0"
  "svreadz_za8_mf8_vg1x2\0"
  "svreadz_za8_mf8_vg1x4\0"
  "svreadz_za8_s8_vg1x2\0"
  "svreadz_za8_s8_vg1x4\0"
  "svreadz_za8_u8_vg1x2\0"
  "svreadz_za8_u8_vg1x4\0"
  "svst1_hor_vnum_za128\0"
  "vIUWiUiq16bv*Wi\0"
  "svst1_hor_vnum_za16\0"
  "svst1_hor_vnum_za32\0"
  "svst1_hor_vnum_za64\0"
  "svst1_hor_vnum_za8\0"
  "svst1_hor_za128\0"
  "vIUWiUiq16bv*\0"
  "svst1_hor_za16\0"
  "svst1_hor_za32\0"
  "svst1_hor_za64\0"
  "svst1_hor_za8\0"
  "svst1_ver_vnum_za128\0"
  "svst1_ver_vnum_za16\0"
  "svst1_ver_vnum_za32\0"
  "svst1_ver_vnum_za64\0"
  "svst1_ver_vnum_za8\0"
  "svst1_ver_za128\0"
  "svst1_ver_za16\0"
  "svst1_ver_za32\0"
  "svst1_ver_za64\0"
  "svst1_ver_za8\0"
  "svstr_vnum_za\0"
  "vUiv*Wi\0"
  "svstr_za\0"
  "vUiv*\0"
  "svstr_zt\0"
  "vIUWiv*\0"
  "svsub_write_single_za32_s32_vg1x2\0"
  "svsub_write_single_za32_s32_vg1x4\0"
  "svsub_write_single_za32_u32_vg1x2\0"
  "svsub_write_single_za32_u32_vg1x4\0"
  "svsub_write_single_za64_s64_vg1x2\0"
  "svsub_write_single_za64_s64_vg1x4\0"
  "svsub_write_single_za64_u64_vg1x2\0"
  "svsub_write_single_za64_u64_vg1x4\0"
  "svsub_write_za32_s32_vg1x2\0"
  "svsub_write_za32_s32_vg1x4\0"
  "svsub_write_za32_u32_vg1x2\0"
  "svsub_write_za32_u32_vg1x4\0"
  "svsub_write_za64_s64_vg1x2\0"
  "svsub_write_za64_s64_vg1x4\0"
  "svsub_write_za64_u64_vg1x2\0"
  "svsub_write_za64_u64_vg1x4\0"
  "svsub_za16_bf16_vg1x2\0"
  "svsub_za16_bf16_vg1x4\0"
  "svsub_za16_f16_vg1x2\0"
  "svsub_za16_f16_vg1x4\0"
  "svsub_za32_f32_vg1x2\0"
  "svsub_za32_f32_vg1x4\0"
  "svsub_za32_s32_vg1x2\0"
  "svsub_za32_s32_vg1x4\0"
  "svsub_za32_u32_vg1x2\0"
  "svsub_za32_u32_vg1x4\0"
  "svsub_za64_f64_vg1x2\0"
  "svsub_za64_f64_vg1x4\0"
  "svsub_za64_s64_vg1x2\0"
  "svsub_za64_s64_vg1x4\0"
  "svsub_za64_u64_vg1x2\0"
  "svsub_za64_u64_vg1x4\0"
  "svsudot_lane_za32_s8_vg1x2\0"
  "vUiq32Scq16UcIUWi\0"
  "svsudot_lane_za32_s8_vg1x4\0"
  "vUiq64Scq16UcIUWi\0"
  "svsudot_single_za32_s8_vg1x2\0"
  "vUiq32Scq16Uc\0"
  "svsudot_single_za32_s8_vg1x4\0"
  "vUiq64Scq16Uc\0"
  "svsudot_za32_s8_vg1x2\0"
  "vUiq32Scq32Uc\0"
  "svsudot_za32_s8_vg1x4\0"
  "vUiq64Scq64Uc\0"
  "svsumla_lane_za32_s8_vg4x1\0"
  "vUiq16Scq16UcIUWi\0"
  "svsumla_lane_za32_s8_vg4x2\0"
  "svsumla_lane_za32_s8_vg4x4\0"
  "svsumla_single_za32_s8_vg4x2\0"
  "svsumla_single_za32_s8_vg4x4\0"
  "svsumla_za32_s8_vg4x1\0"
  "vUiq16Scq16Uc\0"
  "svsumla_za32_s8_vg4x2\0"
  "svsumla_za32_s8_vg4x4\0"
  "svsumopa_za32_s8_m\0"
  "vIUWiq16bq16bq16Scq16Uc\0"
  "svsumopa_za64_s16_m\0"
  "vIUWiq16bq16bq8sq8Us\0"
  "svsumops_za32_s8_m\0"
  "svsumops_za64_s16_m\0"
  "svsuvdot_lane_za32_s8_vg1x4\0"
  "svtmopa_lane_za16_bf16_bf16\0"
  "vIUWiq16yq8yq16UcIUWi\0"
  "sme,sme2,sme-tmop,sme-b16b16\0"
  "svtmopa_lane_za16_f16_f16\0"
  "vIUWiq16hq8hq16UcIUWi\0"
  "sme,sme2,sme-tmop,sme-f16f16\0"
  "svtmopa_lane_za16_mf8_mf8_fpm\0"
  "vIUWiq32mq16mq16UcIUWiUWi\0"
  "sme,sme2,sme-tmop,sme-f8f16\0"
  "svtmopa_lane_za32_bf16_bf16\0"
  "sme,sme2,sme-tmop\0"
  "svtmopa_lane_za32_f16_f16\0"
  "svtmopa_lane_za32_f32_f32\0"
  "vIUWiq8fq4fq16UcIUWi\0"
  "svtmopa_lane_za32_mf8_mf8_fpm\0"
  "sme,sme2,sme-tmop,sme-f8f32\0"
  "svtmopa_lane_za32_s16_s16\0"
  "vIUWiq16sq8sq16UcIUWi\0"
  "svtmopa_lane_za32_s8_s8\0"
  "vIUWiq32Scq16Scq16UcIUWi\0"
  "svtmopa_lane_za32_s8_u8\0"
  "vIUWiq32Scq16Ucq16UcIUWi\0"
  "svtmopa_lane_za32_u16_u16\0"
  "vIUWiq16Usq8Usq16UcIUWi\0"
  "svtmopa_lane_za32_u8_s8\0"
  "vIUWiq32Ucq16Scq16UcIUWi\0"
  "svtmopa_lane_za32_u8_u8\0"
  "vIUWiq32Ucq16Ucq16UcIUWi\0"
  "svusdot_lane_za32_u8_vg1x2\0"
  "vUiq32Ucq16ScIUWi\0"
  "svusdot_lane_za32_u8_vg1x4\0"
  "vUiq64Ucq16ScIUWi\0"
  "svusdot_single_za32_u8_vg1x2\0"
  "vUiq32Ucq16Sc\0"
  "svusdot_single_za32_u8_vg1x4\0"
  "vUiq64Ucq16Sc\0"
  "svusdot_za32_u8_vg1x2\0"
  "vUiq32Ucq32Sc\0"
  "svusdot_za32_u8_vg1x4\0"
  "vUiq64Ucq64Sc\0"
  "svusmla_lane_za32_u8_vg4x1\0"
  "vUiq16Ucq16ScIUWi\0"
  "svusmla_lane_za32_u8_vg4x2\0"
  "svusmla_lane_za32_u8_vg4x4\0"
  "svusmla_single_za32_u8_vg4x2\0"
  "svusmla_single_za32_u8_vg4x4\0"
  "svusmla_za32_u8_vg4x1\0"
  "vUiq16Ucq16Sc\0"
  "svusmla_za32_u8_vg4x2\0"
  "svusmla_za32_u8_vg4x4\0"
  "svusmopa_za32_u8_m\0"
  "vIUWiq16bq16bq16Ucq16Sc\0"
  "svusmopa_za64_u16_m\0"
  "vIUWiq16bq16bq8Usq8s\0"
  "svusmops_za32_u8_m\0"
  "svusmops_za64_u16_m\0"
  "svusvdot_lane_za32_u8_vg1x4\0"
  "svvdot_lane_za16_mf8_vg1x2_fpm\0"
  "svvdot_lane_za32_bf16_vg1x2\0"
  "svvdot_lane_za32_f16_vg1x2\0"
  "svvdot_lane_za32_s16_vg1x2\0"
  "svvdot_lane_za32_s8_vg1x4\0"
  "svvdot_lane_za32_u16_vg1x2\0"
  "svvdot_lane_za32_u8_vg1x4\0"
  "svvdot_lane_za64_s16_vg1x4\0"
  "svvdot_lane_za64_u16_vg1x4\0"
  "svvdotb_lane_za32_mf8_vg1x4_fpm\0"
  "svvdott_lane_za32_mf8_vg1x4_fpm\0"
  "svwrite_hor_za128_bf16_m\0"
  "vIUWiUiq16bq8y\0"
  "svwrite_hor_za128_f16_m\0"
  "vIUWiUiq16bq8h\0"
  "svwrite_hor_za128_f32_m\0"
  "vIUWiUiq16bq4f\0"
  "svwrite_hor_za128_f64_m\0"
  "vIUWiUiq16bq2d\0"
  "svwrite_hor_za128_mf8_m\0"
  "vIUWiUiq16bq16m\0"
  "svwrite_hor_za128_s16_m\0"
  "vIUWiUiq16bq8s\0"
  "svwrite_hor_za128_s32_m\0"
  "vIUWiUiq16bq4i\0"
  "svwrite_hor_za128_s64_m\0"
  "vIUWiUiq16bq2Wi\0"
  "svwrite_hor_za128_s8_m\0"
  "vIUWiUiq16bq16Sc\0"
  "svwrite_hor_za128_u16_m\0"
  "vIUWiUiq16bq8Us\0"
  "svwrite_hor_za128_u32_m\0"
  "vIUWiUiq16bq4Ui\0"
  "svwrite_hor_za128_u64_m\0"
  "vIUWiUiq16bq2UWi\0"
  "svwrite_hor_za128_u8_m\0"
  "vIUWiUiq16bq16Uc\0"
  "svwrite_hor_za16_bf16_m\0"
  "svwrite_hor_za16_bf16_vg2\0"
  "vIUWiUiq16y\0"
  "svwrite_hor_za16_bf16_vg4\0"
  "vIUWiUiq32y\0"
  "svwrite_hor_za16_f16_m\0"
  "svwrite_hor_za16_f16_vg2\0"
  "vIUWiUiq16h\0"
  "svwrite_hor_za16_f16_vg4\0"
  "vIUWiUiq32h\0"
  "svwrite_hor_za16_s16_m\0"
  "svwrite_hor_za16_s16_vg2\0"
  "vIUWiUiq16s\0"
  "svwrite_hor_za16_s16_vg4\0"
  "vIUWiUiq32s\0"
  "svwrite_hor_za16_u16_m\0"
  "svwrite_hor_za16_u16_vg2\0"
  "vIUWiUiq16Us\0"
  "svwrite_hor_za16_u16_vg4\0"
  "vIUWiUiq32Us\0"
  "svwrite_hor_za32_f32_m\0"
  "svwrite_hor_za32_f32_vg2\0"
  "vIUWiUiq8f\0"
  "svwrite_hor_za32_f32_vg4\0"
  "vIUWiUiq16f\0"
  "svwrite_hor_za32_s32_m\0"
  "svwrite_hor_za32_s32_vg2\0"
  "vIUWiUiq8i\0"
  "svwrite_hor_za32_s32_vg4\0"
  "vIUWiUiq16i\0"
  "svwrite_hor_za32_u32_m\0"
  "svwrite_hor_za32_u32_vg2\0"
  "vIUWiUiq8Ui\0"
  "svwrite_hor_za32_u32_vg4\0"
  "vIUWiUiq16Ui\0"
  "svwrite_hor_za64_f64_m\0"
  "svwrite_hor_za64_f64_vg2\0"
  "vIUWiUiq4d\0"
  "svwrite_hor_za64_f64_vg4\0"
  "vIUWiUiq8d\0"
  "svwrite_hor_za64_s64_m\0"
  "svwrite_hor_za64_s64_vg2\0"
  "vIUWiUiq4Wi\0"
  "svwrite_hor_za64_s64_vg4\0"
  "vIUWiUiq8Wi\0"
  "svwrite_hor_za64_u64_m\0"
  "svwrite_hor_za64_u64_vg2\0"
  "vIUWiUiq4UWi\0"
  "svwrite_hor_za64_u64_vg4\0"
  "vIUWiUiq8UWi\0"
  "svwrite_hor_za8_mf8_m\0"
  "svwrite_hor_za8_mf8_vg2\0"
  "vIUWiUiq32m\0"
  "svwrite_hor_za8_mf8_vg4\0"
  "vIUWiUiq64m\0"
  "svwrite_hor_za8_s8_m\0"
  "svwrite_hor_za8_s8_vg2\0"
  "vIUWiUiq32Sc\0"
  "svwrite_hor_za8_s8_vg4\0"
  "vIUWiUiq64Sc\0"
  "svwrite_hor_za8_u8_m\0"
  "svwrite_hor_za8_u8_vg2\0"
  "vIUWiUiq32Uc\0"
  "svwrite_hor_za8_u8_vg4\0"
  "vIUWiUiq64Uc\0"
  "svwrite_lane_zt_bf16\0"
  "vIUWiq8yIUWi\0"
  "svwrite_lane_zt_f16\0"
  "vIUWiq8hIUWi\0"
  "svwrite_lane_zt_f32\0"
  "vIUWiq4fIUWi\0"
  "svwrite_lane_zt_f64\0"
  "vIUWiq2dIUWi\0"
  "svwrite_lane_zt_s16\0"
  "vIUWiq8sIUWi\0"
  "svwrite_lane_zt_s32\0"
  "vIUWiq4iIUWi\0"
  "svwrite_lane_zt_s64\0"
  "vIUWiq2WiIUWi\0"
  "svwrite_lane_zt_s8\0"
  "vIUWiq16ScIUWi\0"
  "svwrite_lane_zt_u16\0"
  "vIUWiq8UsIUWi\0"
  "svwrite_lane_zt_u32\0"
  "vIUWiq4UiIUWi\0"
  "svwrite_lane_zt_u64\0"
  "vIUWiq2UWiIUWi\0"
  "svwrite_lane_zt_u8\0"
  "vIUWiq16UcIUWi\0"
  "svwrite_ver_za128_bf16_m\0"
  "svwrite_ver_za128_f16_m\0"
  "svwrite_ver_za128_f32_m\0"
  "svwrite_ver_za128_f64_m\0"
  "svwrite_ver_za128_mf8_m\0"
  "svwrite_ver_za128_s16_m\0"
  "svwrite_ver_za128_s32_m\0"
  "svwrite_ver_za128_s64_m\0"
  "svwrite_ver_za128_s8_m\0"
  "svwrite_ver_za128_u16_m\0"
  "svwrite_ver_za128_u32_m\0"
  "svwrite_ver_za128_u64_m\0"
  "svwrite_ver_za128_u8_m\0"
  "svwrite_ver_za16_bf16_m\0"
  "svwrite_ver_za16_bf16_vg2\0"
  "svwrite_ver_za16_bf16_vg4\0"
  "svwrite_ver_za16_f16_m\0"
  "svwrite_ver_za16_f16_vg2\0"
  "svwrite_ver_za16_f16_vg4\0"
  "svwrite_ver_za16_s16_m\0"
  "svwrite_ver_za16_s16_vg2\0"
  "svwrite_ver_za16_s16_vg4\0"
  "svwrite_ver_za16_u16_m\0"
  "svwrite_ver_za16_u16_vg2\0"
  "svwrite_ver_za16_u16_vg4\0"
  "svwrite_ver_za32_f32_m\0"
  "svwrite_ver_za32_f32_vg2\0"
  "svwrite_ver_za32_f32_vg4\0"
  "svwrite_ver_za32_s32_m\0"
  "svwrite_ver_za32_s32_vg2\0"
  "svwrite_ver_za32_s32_vg4\0"
  "svwrite_ver_za32_u32_m\0"
  "svwrite_ver_za32_u32_vg2\0"
  "svwrite_ver_za32_u32_vg4\0"
  "svwrite_ver_za64_f64_m\0"
  "svwrite_ver_za64_f64_vg2\0"
  "svwrite_ver_za64_f64_vg4\0"
  "svwrite_ver_za64_s64_m\0"
  "svwrite_ver_za64_s64_vg2\0"
  "svwrite_ver_za64_s64_vg4\0"
  "svwrite_ver_za64_u64_m\0"
  "svwrite_ver_za64_u64_vg2\0"
  "svwrite_ver_za64_u64_vg4\0"
  "svwrite_ver_za8_mf8_m\0"
  "svwrite_ver_za8_mf8_vg2\0"
  "svwrite_ver_za8_mf8_vg4\0"
  "svwrite_ver_za8_s8_m\0"
  "svwrite_ver_za8_s8_vg2\0"
  "svwrite_ver_za8_s8_vg4\0"
  "svwrite_ver_za8_u8_m\0"
  "svwrite_ver_za8_u8_vg2\0"
  "svwrite_ver_za8_u8_vg4\0"
  "svwrite_za16_bf16_vg1x2\0"
  "svwrite_za16_bf16_vg1x4\0"
  "svwrite_za16_f16_vg1x2\0"
  "svwrite_za16_f16_vg1x4\0"
  "svwrite_za16_s16_vg1x2\0"
  "vUiq16s\0"
  "svwrite_za16_s16_vg1x4\0"
  "vUiq32s\0"
  "svwrite_za16_u16_vg1x2\0"
  "vUiq16Us\0"
  "svwrite_za16_u16_vg1x4\0"
  "vUiq32Us\0"
  "svwrite_za32_f32_vg1x2\0"
  "svwrite_za32_f32_vg1x4\0"
  "svwrite_za32_s32_vg1x2\0"
  "svwrite_za32_s32_vg1x4\0"
  "svwrite_za32_u32_vg1x2\0"
  "svwrite_za32_u32_vg1x4\0"
  "svwrite_za64_f64_vg1x2\0"
  "svwrite_za64_f64_vg1x4\0"
  "svwrite_za64_s64_vg1x2\0"
  "svwrite_za64_s64_vg1x4\0"
  "svwrite_za64_u64_vg1x2\0"
  "svwrite_za64_u64_vg1x4\0"
  "svwrite_za8_mf8_vg1x2\0"
  "vUiq32m\0"
  "svwrite_za8_mf8_vg1x4\0"
  "vUiq64m\0"
  "svwrite_za8_s8_vg1x2\0"
  "vUiq32Sc\0"
  "svwrite_za8_s8_vg1x4\0"
  "vUiq64Sc\0"
  "svwrite_za8_u8_vg1x2\0"
  "vUiq32Uc\0"
  "svwrite_za8_u8_vg1x4\0"
  "vUiq64Uc\0"
  "svwrite_zt_bf16\0"
  "vIUWiq8y\0"
  "svwrite_zt_f16\0"
  "vIUWiq8h\0"
  "svwrite_zt_f32\0"
  "vIUWiq4f\0"
  "svwrite_zt_f64\0"
  "vIUWiq2d\0"
  "svwrite_zt_s16\0"
  "vIUWiq8s\0"
  "svwrite_zt_s32\0"
  "vIUWiq4i\0"
  "svwrite_zt_s64\0"
  "vIUWiq2Wi\0"
  "svwrite_zt_s8\0"
  "vIUWiq16Sc\0"
  "svwrite_zt_u16\0"
  "vIUWiq8Us\0"
  "svwrite_zt_u32\0"
  "vIUWiq4Ui\0"
  "svwrite_zt_u64\0"
  "vIUWiq2UWi\0"
  "svwrite_zt_u8\0"
  "vIUWiq16Uc\0"
  "svzero_mask_za\0"
  "vIUWi\0"
  "svzero_za\0"
  "vv\0"
  "svzero_za64_vg1x2\0"
  "vUi\0"
  "svzero_za64_vg1x4\0"
  "svzero_za64_vg2x1\0"
  "svzero_za64_vg2x2\0"
  "svzero_za64_vg2x4\0"
  "svzero_za64_vg4x1\0"
  "svzero_za64_vg4x2\0"
  "svzero_za64_vg4x4\0"
  "svzero_zt\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinStrings = BuiltinStringsStorage;
#endif // GET_SME_BUILTIN_STR_TABLE

#ifdef GET_SME_BUILTIN_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{3 /* __arm_in_streaming_mode */, 27 /* bv */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34 /* svadd_write_single_za32_s32_vg1x2 */, 68 /* vUiq8iq4i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{87 /* svadd_write_single_za32_s32_vg1x4 */, 121 /* vUiq16iq4i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{132 /* svadd_write_single_za32_u32_vg1x2 */, 166 /* vUiq8Uiq4Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{178 /* svadd_write_single_za32_u32_vg1x4 */, 212 /* vUiq16Uiq4Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{225 /* svadd_write_single_za64_s64_vg1x2 */, 259 /* vUiq4Wiq2Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{291 /* svadd_write_single_za64_s64_vg1x4 */, 325 /* vUiq8Wiq2Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{337 /* svadd_write_single_za64_u64_vg1x2 */, 371 /* vUiq4UWiq2UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{385 /* svadd_write_single_za64_u64_vg1x4 */, 419 /* vUiq8UWiq2UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{433 /* svadd_write_za32_s32_vg1x2 */, 460 /* vUiq8iq8i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{470 /* svadd_write_za32_s32_vg1x4 */, 497 /* vUiq16iq16i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{509 /* svadd_write_za32_u32_vg1x2 */, 536 /* vUiq8Uiq8Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{548 /* svadd_write_za32_u32_vg1x4 */, 575 /* vUiq16Uiq16Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{589 /* svadd_write_za64_s64_vg1x2 */, 616 /* vUiq4Wiq4Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{628 /* svadd_write_za64_s64_vg1x4 */, 655 /* vUiq8Wiq8Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{667 /* svadd_write_za64_u64_vg1x2 */, 694 /* vUiq4UWiq4UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{708 /* svadd_write_za64_u64_vg1x4 */, 735 /* vUiq8UWiq8UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{749 /* svadd_za16_bf16_vg1x2 */, 771 /* vUiq16y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{794 /* svadd_za16_bf16_vg1x4 */, 816 /* vUiq32y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{824 /* svadd_za16_f16_vg1x2 */, 845 /* vUiq16h */, 1 /* n */, 853 /* sme,(sme-f16f16|sme-f8f16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{880 /* svadd_za16_f16_vg1x4 */, 901 /* vUiq32h */, 1 /* n */, 853 /* sme,(sme-f16f16|sme-f8f16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{909 /* svadd_za32_f32_vg1x2 */, 930 /* vUiq8f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{937 /* svadd_za32_f32_vg1x4 */, 958 /* vUiq16f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{966 /* svadd_za32_s32_vg1x2 */, 987 /* vUiq8i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{994 /* svadd_za32_s32_vg1x4 */, 1015 /* vUiq16i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1023 /* svadd_za32_u32_vg1x2 */, 1044 /* vUiq8Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1052 /* svadd_za32_u32_vg1x4 */, 1073 /* vUiq16Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1082 /* svadd_za64_f64_vg1x2 */, 1103 /* vUiq4d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1130 /* svadd_za64_f64_vg1x4 */, 1151 /* vUiq8d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1158 /* svadd_za64_s64_vg1x2 */, 1179 /* vUiq4Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1187 /* svadd_za64_s64_vg1x4 */, 1208 /* vUiq8Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1216 /* svadd_za64_u64_vg1x2 */, 1237 /* vUiq4UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1246 /* svadd_za64_u64_vg1x4 */, 1267 /* vUiq8UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1276 /* svaddha_za32_s32_m */, 1295 /* vIUWiq16bq16bq4i */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1312 /* svaddha_za32_u32_m */, 1331 /* vIUWiq16bq16bq4Ui */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1349 /* svaddha_za64_s64_m */, 1368 /* vIUWiq16bq16bq2Wi */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1401 /* svaddha_za64_u64_m */, 1420 /* vIUWiq16bq16bq2UWi */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1439 /* svaddva_za32_s32_m */, 1295 /* vIUWiq16bq16bq4i */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1458 /* svaddva_za32_u32_m */, 1331 /* vIUWiq16bq16bq4Ui */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1477 /* svaddva_za64_s64_m */, 1368 /* vIUWiq16bq16bq2Wi */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1496 /* svaddva_za64_u64_m */, 1420 /* vIUWiq16bq16bq2UWi */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1515 /* svbmopa_za32_s32_m */, 1534 /* vIUWiq16bq16bq4iq4i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1554 /* svbmopa_za32_u32_m */, 1573 /* vIUWiq16bq16bq4Uiq4Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1595 /* svbmops_za32_s32_m */, 1534 /* vIUWiq16bq16bq4iq4i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1614 /* svbmops_za32_u32_m */, 1573 /* vIUWiq16bq16bq4Uiq4Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1633 /* svcntsb */, 1641 /* UWiv */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1646 /* svcntsd */, 1641 /* UWiv */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1654 /* svcntsh */, 1641 /* UWiv */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1662 /* svcntsw */, 1641 /* UWiv */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1670 /* svdot_lane_za16_mf8_vg1x2_fpm */, 1700 /* vUiq32mq16mIUWiUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1733 /* svdot_lane_za16_mf8_vg1x4_fpm */, 1763 /* vUiq64mq16mIUWiUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1782 /* svdot_lane_za32_bf16_vg1x2 */, 1809 /* vUiq16yq8yIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1824 /* svdot_lane_za32_bf16_vg1x4 */, 1851 /* vUiq32yq8yIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1866 /* svdot_lane_za32_f16_vg1x2 */, 1892 /* vUiq16hq8hIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1907 /* svdot_lane_za32_f16_vg1x4 */, 1933 /* vUiq32hq8hIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1948 /* svdot_lane_za32_mf8_vg1x2_fpm */, 1700 /* vUiq32mq16mIUWiUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1992 /* svdot_lane_za32_mf8_vg1x4_fpm */, 1763 /* vUiq64mq16mIUWiUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2022 /* svdot_lane_za32_s16_vg1x2 */, 2048 /* vUiq16sq8sIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2063 /* svdot_lane_za32_s16_vg1x4 */, 2089 /* vUiq32sq8sIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2104 /* svdot_lane_za32_s8_vg1x2 */, 2129 /* vUiq32Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2147 /* svdot_lane_za32_s8_vg1x4 */, 2172 /* vUiq64Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2190 /* svdot_lane_za32_u16_vg1x2 */, 2216 /* vUiq16Usq8UsIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2233 /* svdot_lane_za32_u16_vg1x4 */, 2259 /* vUiq32Usq8UsIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2276 /* svdot_lane_za32_u8_vg1x2 */, 2301 /* vUiq32Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2319 /* svdot_lane_za32_u8_vg1x4 */, 2344 /* vUiq64Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2362 /* svdot_lane_za64_s16_vg1x2 */, 2048 /* vUiq16sq8sIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2388 /* svdot_lane_za64_s16_vg1x4 */, 2089 /* vUiq32sq8sIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2414 /* svdot_lane_za64_u16_vg1x2 */, 2216 /* vUiq16Usq8UsIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2440 /* svdot_lane_za64_u16_vg1x4 */, 2259 /* vUiq32Usq8UsIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2466 /* svdot_single_za16_mf8_vg1x2_fpm */, 2498 /* vUiq32mq16mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2513 /* svdot_single_za16_mf8_vg1x4_fpm */, 2545 /* vUiq64mq16mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2560 /* svdot_single_za32_bf16_vg1x2 */, 2589 /* vUiq16yq8y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2600 /* svdot_single_za32_bf16_vg1x4 */, 2629 /* vUiq32yq8y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2640 /* svdot_single_za32_f16_vg1x2 */, 2668 /* vUiq16hq8h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2679 /* svdot_single_za32_f16_vg1x4 */, 2707 /* vUiq32hq8h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2718 /* svdot_single_za32_mf8_vg1x2_fpm */, 2498 /* vUiq32mq16mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2750 /* svdot_single_za32_mf8_vg1x4_fpm */, 2545 /* vUiq64mq16mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2782 /* svdot_single_za32_s16_vg1x2 */, 2810 /* vUiq16sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2821 /* svdot_single_za32_s16_vg1x4 */, 2849 /* vUiq32sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2860 /* svdot_single_za32_s8_vg1x2 */, 2887 /* vUiq32Scq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2901 /* svdot_single_za32_s8_vg1x4 */, 2928 /* vUiq64Scq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2942 /* svdot_single_za32_u16_vg1x2 */, 2970 /* vUiq16Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2983 /* svdot_single_za32_u16_vg1x4 */, 3011 /* vUiq32Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3024 /* svdot_single_za32_u8_vg1x2 */, 3051 /* vUiq32Ucq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3065 /* svdot_single_za32_u8_vg1x4 */, 3092 /* vUiq64Ucq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3106 /* svdot_single_za64_s16_vg1x2 */, 2810 /* vUiq16sq8s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3134 /* svdot_single_za64_s16_vg1x4 */, 2849 /* vUiq32sq8s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3162 /* svdot_single_za64_u16_vg1x2 */, 2970 /* vUiq16Usq8Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3190 /* svdot_single_za64_u16_vg1x4 */, 3011 /* vUiq32Usq8Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3218 /* svdot_za16_mf8_vg1x2_fpm */, 3243 /* vUiq32mq32mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3258 /* svdot_za16_mf8_vg1x4_fpm */, 3283 /* vUiq64mq64mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3298 /* svdot_za32_bf16_vg1x2 */, 3320 /* vUiq16yq16y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3332 /* svdot_za32_bf16_vg1x4 */, 3354 /* vUiq32yq32y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3366 /* svdot_za32_f16_vg1x2 */, 3387 /* vUiq16hq16h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3399 /* svdot_za32_f16_vg1x4 */, 3420 /* vUiq32hq32h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3432 /* svdot_za32_mf8_vg1x2_fpm */, 3243 /* vUiq32mq32mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3457 /* svdot_za32_mf8_vg1x4_fpm */, 3283 /* vUiq64mq64mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3482 /* svdot_za32_s16_vg1x2 */, 3503 /* vUiq16sq16s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3515 /* svdot_za32_s16_vg1x4 */, 3536 /* vUiq32sq32s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3548 /* svdot_za32_s8_vg1x2 */, 3568 /* vUiq32Scq32Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3582 /* svdot_za32_s8_vg1x4 */, 3602 /* vUiq64Scq64Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3616 /* svdot_za32_u16_vg1x2 */, 3637 /* vUiq16Usq16Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3651 /* svdot_za32_u16_vg1x4 */, 3672 /* vUiq32Usq32Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3686 /* svdot_za32_u8_vg1x2 */, 3706 /* vUiq32Ucq32Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3720 /* svdot_za32_u8_vg1x4 */, 3740 /* vUiq64Ucq64Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3754 /* svdot_za64_s16_vg1x2 */, 3503 /* vUiq16sq16s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3775 /* svdot_za64_s16_vg1x4 */, 3536 /* vUiq32sq32s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3796 /* svdot_za64_u16_vg1x2 */, 3637 /* vUiq16Usq16Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3817 /* svdot_za64_u16_vg1x4 */, 3672 /* vUiq32Usq32Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3838 /* svld1_hor_vnum_za128 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3876 /* svld1_hor_vnum_za16 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3896 /* svld1_hor_vnum_za32 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3916 /* svld1_hor_vnum_za64 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3936 /* svld1_hor_vnum_za8 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3955 /* svld1_hor_za128 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3986 /* svld1_hor_za16 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4001 /* svld1_hor_za32 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4016 /* svld1_hor_za64 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4031 /* svld1_hor_za8 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4045 /* svld1_ver_vnum_za128 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4066 /* svld1_ver_vnum_za16 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4086 /* svld1_ver_vnum_za32 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4106 /* svld1_ver_vnum_za64 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4126 /* svld1_ver_vnum_za8 */, 3859 /* vIUWiUiq16bvC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4145 /* svld1_ver_za128 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4161 /* svld1_ver_za16 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4176 /* svld1_ver_za32 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4191 /* svld1_ver_za64 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4206 /* svld1_ver_za8 */, 3971 /* vIUWiUiq16bvC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4220 /* svldr_vnum_za */, 4234 /* vUivC*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4243 /* svldr_za */, 4252 /* vUivC* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4259 /* svldr_zt */, 4268 /* vIUWivC* */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4277 /* svluti2_lane_zt_bf16 */, 4298 /* q8yIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4315 /* svluti2_lane_zt_bf16_x2 */, 4339 /* q16yIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4357 /* svluti2_lane_zt_bf16_x4 */, 4381 /* q32yIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4399 /* svluti2_lane_zt_f16 */, 4419 /* q8hIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4436 /* svluti2_lane_zt_f16_x2 */, 4459 /* q16hIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4477 /* svluti2_lane_zt_f16_x4 */, 4500 /* q32hIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4518 /* svluti2_lane_zt_f32 */, 4538 /* q4fIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4555 /* svluti2_lane_zt_f32_x2 */, 4578 /* q8fIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4595 /* svluti2_lane_zt_f32_x4 */, 4618 /* q16fIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4636 /* svluti2_lane_zt_mf8 */, 4656 /* q16mIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4674 /* svluti2_lane_zt_mf8_x2 */, 4697 /* q32mIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4715 /* svluti2_lane_zt_mf8_x4 */, 4738 /* q64mIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4756 /* svluti2_lane_zt_s16 */, 4776 /* q8sIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4793 /* svluti2_lane_zt_s16_x2 */, 4816 /* q16sIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4834 /* svluti2_lane_zt_s16_x4 */, 4857 /* q32sIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4875 /* svluti2_lane_zt_s32 */, 4895 /* q4iIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4912 /* svluti2_lane_zt_s32_x2 */, 4935 /* q8iIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4952 /* svluti2_lane_zt_s32_x4 */, 4975 /* q16iIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4993 /* svluti2_lane_zt_s8 */, 5012 /* q16ScIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5031 /* svluti2_lane_zt_s8_x2 */, 5053 /* q32ScIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5072 /* svluti2_lane_zt_s8_x4 */, 5094 /* q64ScIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5113 /* svluti2_lane_zt_u16 */, 5133 /* q8UsIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5151 /* svluti2_lane_zt_u16_x2 */, 5174 /* q16UsIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5193 /* svluti2_lane_zt_u16_x4 */, 5216 /* q32UsIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5235 /* svluti2_lane_zt_u32 */, 5255 /* q4UiIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5273 /* svluti2_lane_zt_u32_x2 */, 5296 /* q8UiIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5314 /* svluti2_lane_zt_u32_x4 */, 5337 /* q16UiIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5356 /* svluti2_lane_zt_u8 */, 5375 /* q16UcIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5394 /* svluti2_lane_zt_u8_x2 */, 5416 /* q32UcIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5435 /* svluti2_lane_zt_u8_x4 */, 5457 /* q64UcIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5476 /* svluti4_lane_zt_bf16 */, 4298 /* q8yIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5497 /* svluti4_lane_zt_bf16_x2 */, 4339 /* q16yIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5521 /* svluti4_lane_zt_bf16_x4 */, 4381 /* q32yIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5545 /* svluti4_lane_zt_f16 */, 4419 /* q8hIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5565 /* svluti4_lane_zt_f16_x2 */, 4459 /* q16hIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5588 /* svluti4_lane_zt_f16_x4 */, 4500 /* q32hIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5611 /* svluti4_lane_zt_f32 */, 4538 /* q4fIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5631 /* svluti4_lane_zt_f32_x2 */, 4578 /* q8fIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5654 /* svluti4_lane_zt_f32_x4 */, 4618 /* q16fIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5677 /* svluti4_lane_zt_mf8 */, 4656 /* q16mIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5697 /* svluti4_lane_zt_mf8_x2 */, 4697 /* q32mIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5720 /* svluti4_lane_zt_s16 */, 4776 /* q8sIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5740 /* svluti4_lane_zt_s16_x2 */, 4816 /* q16sIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5763 /* svluti4_lane_zt_s16_x4 */, 4857 /* q32sIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5786 /* svluti4_lane_zt_s32 */, 4895 /* q4iIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5806 /* svluti4_lane_zt_s32_x2 */, 4935 /* q8iIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5829 /* svluti4_lane_zt_s32_x4 */, 4975 /* q16iIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5852 /* svluti4_lane_zt_s8 */, 5012 /* q16ScIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5871 /* svluti4_lane_zt_s8_x2 */, 5053 /* q32ScIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5893 /* svluti4_lane_zt_u16 */, 5133 /* q8UsIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5913 /* svluti4_lane_zt_u16_x2 */, 5174 /* q16UsIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5936 /* svluti4_lane_zt_u16_x4 */, 5216 /* q32UsIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5959 /* svluti4_lane_zt_u32 */, 5255 /* q4UiIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5979 /* svluti4_lane_zt_u32_x2 */, 5296 /* q8UiIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6002 /* svluti4_lane_zt_u32_x4 */, 5337 /* q16UiIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6025 /* svluti4_lane_zt_u8 */, 5375 /* q16UcIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6044 /* svluti4_lane_zt_u8_x2 */, 5416 /* q32UcIUWiq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6066 /* svluti4_zt_s8_x4 */, 6083 /* q64ScIUWiq32Uc */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6112 /* svluti4_zt_u8_x4 */, 6129 /* q64UcIUWiq32Uc */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6144 /* svmla_lane_za16_bf16_vg1x2 */, 1809 /* vUiq16yq8yIUWi */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6171 /* svmla_lane_za16_bf16_vg1x4 */, 1851 /* vUiq32yq8yIUWi */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6198 /* svmla_lane_za16_f16_vg1x2 */, 1892 /* vUiq16hq8hIUWi */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6239 /* svmla_lane_za16_f16_vg1x4 */, 1933 /* vUiq32hq8hIUWi */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6265 /* svmla_lane_za16_mf8_vg2x1_fpm */, 6295 /* vUiq16mq16mIUWiUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6314 /* svmla_lane_za16_mf8_vg2x2_fpm */, 1700 /* vUiq32mq16mIUWiUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6344 /* svmla_lane_za16_mf8_vg2x4_fpm */, 1763 /* vUiq64mq16mIUWiUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6374 /* svmla_lane_za32_bf16_vg2x1 */, 6401 /* vUiq8yq8yIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6415 /* svmla_lane_za32_bf16_vg2x2 */, 1809 /* vUiq16yq8yIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6442 /* svmla_lane_za32_bf16_vg2x4 */, 1851 /* vUiq32yq8yIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6469 /* svmla_lane_za32_f16_vg2x1 */, 6495 /* vUiq8hq8hIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6509 /* svmla_lane_za32_f16_vg2x2 */, 1892 /* vUiq16hq8hIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6535 /* svmla_lane_za32_f16_vg2x4 */, 1933 /* vUiq32hq8hIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6561 /* svmla_lane_za32_f32_vg1x2 */, 6587 /* vUiq8fq4fIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6601 /* svmla_lane_za32_f32_vg1x4 */, 6627 /* vUiq16fq4fIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6642 /* svmla_lane_za32_mf8_vg4x1_fpm */, 6295 /* vUiq16mq16mIUWiUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6672 /* svmla_lane_za32_mf8_vg4x2_fpm */, 1700 /* vUiq32mq16mIUWiUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6702 /* svmla_lane_za32_mf8_vg4x4_fpm */, 1763 /* vUiq64mq16mIUWiUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6732 /* svmla_lane_za32_s16_vg2x1 */, 6758 /* vUiq8sq8sIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6772 /* svmla_lane_za32_s16_vg2x2 */, 2048 /* vUiq16sq8sIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6798 /* svmla_lane_za32_s16_vg2x4 */, 2089 /* vUiq32sq8sIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6824 /* svmla_lane_za32_s8_vg4x1 */, 6849 /* vUiq16Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6867 /* svmla_lane_za32_s8_vg4x2 */, 2129 /* vUiq32Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6892 /* svmla_lane_za32_s8_vg4x4 */, 2172 /* vUiq64Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6917 /* svmla_lane_za32_u16_vg2x1 */, 6943 /* vUiq8Usq8UsIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6959 /* svmla_lane_za32_u16_vg2x2 */, 2216 /* vUiq16Usq8UsIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6985 /* svmla_lane_za32_u16_vg2x4 */, 2259 /* vUiq32Usq8UsIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7011 /* svmla_lane_za32_u8_vg4x1 */, 7036 /* vUiq16Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7054 /* svmla_lane_za32_u8_vg4x2 */, 2301 /* vUiq32Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7079 /* svmla_lane_za32_u8_vg4x4 */, 2344 /* vUiq64Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7104 /* svmla_lane_za64_f64_vg1x2 */, 7130 /* vUiq4dq2dIUWi */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7144 /* svmla_lane_za64_f64_vg1x4 */, 7170 /* vUiq8dq2dIUWi */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7184 /* svmla_lane_za64_s16_vg4x1 */, 6758 /* vUiq8sq8sIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7210 /* svmla_lane_za64_s16_vg4x2 */, 2048 /* vUiq16sq8sIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7236 /* svmla_lane_za64_s16_vg4x4 */, 2089 /* vUiq32sq8sIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7262 /* svmla_lane_za64_u16_vg4x1 */, 6943 /* vUiq8Usq8UsIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7288 /* svmla_lane_za64_u16_vg4x2 */, 2216 /* vUiq16Usq8UsIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7314 /* svmla_lane_za64_u16_vg4x4 */, 2259 /* vUiq32Usq8UsIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7340 /* svmla_single_za16_bf16_vg1x2 */, 2589 /* vUiq16yq8y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7369 /* svmla_single_za16_bf16_vg1x4 */, 2629 /* vUiq32yq8y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7398 /* svmla_single_za16_f16_vg1x2 */, 2668 /* vUiq16hq8h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7426 /* svmla_single_za16_f16_vg1x4 */, 2707 /* vUiq32hq8h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7454 /* svmla_single_za16_mf8_vg2x2_fpm */, 2498 /* vUiq32mq16mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7486 /* svmla_single_za16_mf8_vg2x4_fpm */, 2545 /* vUiq64mq16mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7518 /* svmla_single_za32_bf16_vg2x2 */, 2589 /* vUiq16yq8y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7547 /* svmla_single_za32_bf16_vg2x4 */, 2629 /* vUiq32yq8y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7576 /* svmla_single_za32_f16_vg2x2 */, 2668 /* vUiq16hq8h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7604 /* svmla_single_za32_f16_vg2x4 */, 2707 /* vUiq32hq8h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7632 /* svmla_single_za32_f32_vg1x2 */, 7660 /* vUiq8fq4f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7670 /* svmla_single_za32_f32_vg1x4 */, 7698 /* vUiq16fq4f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7709 /* svmla_single_za32_mf8_vg4x2_fpm */, 2498 /* vUiq32mq16mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7741 /* svmla_single_za32_mf8_vg4x4_fpm */, 2545 /* vUiq64mq16mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7773 /* svmla_single_za32_s16_vg2x2 */, 2810 /* vUiq16sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7801 /* svmla_single_za32_s16_vg2x4 */, 2849 /* vUiq32sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7829 /* svmla_single_za32_s8_vg4x2 */, 2887 /* vUiq32Scq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7856 /* svmla_single_za32_s8_vg4x4 */, 2928 /* vUiq64Scq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7883 /* svmla_single_za32_u16_vg2x2 */, 2970 /* vUiq16Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7911 /* svmla_single_za32_u16_vg2x4 */, 3011 /* vUiq32Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7939 /* svmla_single_za32_u8_vg4x2 */, 3051 /* vUiq32Ucq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7966 /* svmla_single_za32_u8_vg4x4 */, 3092 /* vUiq64Ucq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7993 /* svmla_single_za64_f64_vg1x2 */, 8021 /* vUiq4dq2d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8031 /* svmla_single_za64_f64_vg1x4 */, 8059 /* vUiq8dq2d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8069 /* svmla_single_za64_s16_vg4x2 */, 2810 /* vUiq16sq8s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8097 /* svmla_single_za64_s16_vg4x4 */, 2849 /* vUiq32sq8s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8125 /* svmla_single_za64_u16_vg4x2 */, 2970 /* vUiq16Usq8Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8153 /* svmla_single_za64_u16_vg4x4 */, 3011 /* vUiq32Usq8Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8181 /* svmla_za16_bf16_vg1x2 */, 3320 /* vUiq16yq16y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8203 /* svmla_za16_bf16_vg1x4 */, 3354 /* vUiq32yq32y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8225 /* svmla_za16_f16_vg1x2 */, 3387 /* vUiq16hq16h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8246 /* svmla_za16_f16_vg1x4 */, 3420 /* vUiq32hq32h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8267 /* svmla_za16_mf8_vg2x1_fpm */, 8292 /* vUiq16mq16mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8307 /* svmla_za16_mf8_vg2x2_fpm */, 3243 /* vUiq32mq32mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8332 /* svmla_za16_mf8_vg2x4_fpm */, 3283 /* vUiq64mq64mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8357 /* svmla_za32_bf16_vg2x1 */, 8379 /* vUiq8yq8y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8389 /* svmla_za32_bf16_vg2x2 */, 3320 /* vUiq16yq16y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8411 /* svmla_za32_bf16_vg2x4 */, 3354 /* vUiq32yq32y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8433 /* svmla_za32_f16_vg2x1 */, 8454 /* vUiq8hq8h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8464 /* svmla_za32_f16_vg2x2 */, 3387 /* vUiq16hq16h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8485 /* svmla_za32_f16_vg2x4 */, 3420 /* vUiq32hq32h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8506 /* svmla_za32_f32_vg1x2 */, 8527 /* vUiq8fq8f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8537 /* svmla_za32_f32_vg1x4 */, 8558 /* vUiq16fq16f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8570 /* svmla_za32_mf8_vg4x1_fpm */, 8292 /* vUiq16mq16mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8595 /* svmla_za32_mf8_vg4x2_fpm */, 3243 /* vUiq32mq32mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8620 /* svmla_za32_mf8_vg4x4_fpm */, 3283 /* vUiq64mq64mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8645 /* svmla_za32_s16_vg2x1 */, 8666 /* vUiq8sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8676 /* svmla_za32_s16_vg2x2 */, 3503 /* vUiq16sq16s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8697 /* svmla_za32_s16_vg2x4 */, 3536 /* vUiq32sq32s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8718 /* svmla_za32_s8_vg4x1 */, 8738 /* vUiq16Scq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8752 /* svmla_za32_s8_vg4x2 */, 3568 /* vUiq32Scq32Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8772 /* svmla_za32_s8_vg4x4 */, 3602 /* vUiq64Scq64Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8792 /* svmla_za32_u16_vg2x1 */, 8813 /* vUiq8Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8825 /* svmla_za32_u16_vg2x2 */, 3637 /* vUiq16Usq16Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8846 /* svmla_za32_u16_vg2x4 */, 3672 /* vUiq32Usq32Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8867 /* svmla_za32_u8_vg4x1 */, 8887 /* vUiq16Ucq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8901 /* svmla_za32_u8_vg4x2 */, 3706 /* vUiq32Ucq32Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8921 /* svmla_za32_u8_vg4x4 */, 3740 /* vUiq64Ucq64Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8941 /* svmla_za64_f64_vg1x2 */, 8962 /* vUiq4dq4d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8972 /* svmla_za64_f64_vg1x4 */, 8993 /* vUiq8dq8d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9003 /* svmla_za64_s16_vg4x1 */, 8666 /* vUiq8sq8s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9024 /* svmla_za64_s16_vg4x2 */, 3503 /* vUiq16sq16s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9045 /* svmla_za64_s16_vg4x4 */, 3536 /* vUiq32sq32s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9066 /* svmla_za64_u16_vg4x1 */, 8813 /* vUiq8Usq8Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9087 /* svmla_za64_u16_vg4x2 */, 3637 /* vUiq16Usq16Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9108 /* svmla_za64_u16_vg4x4 */, 3672 /* vUiq32Usq32Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9129 /* svmls_lane_za16_bf16_vg1x2 */, 1809 /* vUiq16yq8yIUWi */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9156 /* svmls_lane_za16_bf16_vg1x4 */, 1851 /* vUiq32yq8yIUWi */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9183 /* svmls_lane_za16_f16_vg1x2 */, 1892 /* vUiq16hq8hIUWi */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9209 /* svmls_lane_za16_f16_vg1x4 */, 1933 /* vUiq32hq8hIUWi */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9235 /* svmls_lane_za32_bf16_vg2x1 */, 6401 /* vUiq8yq8yIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9262 /* svmls_lane_za32_bf16_vg2x2 */, 1809 /* vUiq16yq8yIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9289 /* svmls_lane_za32_bf16_vg2x4 */, 1851 /* vUiq32yq8yIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9316 /* svmls_lane_za32_f16_vg2x1 */, 6495 /* vUiq8hq8hIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9342 /* svmls_lane_za32_f16_vg2x2 */, 1892 /* vUiq16hq8hIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9368 /* svmls_lane_za32_f16_vg2x4 */, 1933 /* vUiq32hq8hIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9394 /* svmls_lane_za32_f32_vg1x2 */, 6587 /* vUiq8fq4fIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9420 /* svmls_lane_za32_f32_vg1x4 */, 6627 /* vUiq16fq4fIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9446 /* svmls_lane_za32_s16_vg2x1 */, 6758 /* vUiq8sq8sIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9472 /* svmls_lane_za32_s16_vg2x2 */, 2048 /* vUiq16sq8sIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9498 /* svmls_lane_za32_s16_vg2x4 */, 2089 /* vUiq32sq8sIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9524 /* svmls_lane_za32_s8_vg4x1 */, 6849 /* vUiq16Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9549 /* svmls_lane_za32_s8_vg4x2 */, 2129 /* vUiq32Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9574 /* svmls_lane_za32_s8_vg4x4 */, 2172 /* vUiq64Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9599 /* svmls_lane_za32_u16_vg2x1 */, 6943 /* vUiq8Usq8UsIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9625 /* svmls_lane_za32_u16_vg2x2 */, 2216 /* vUiq16Usq8UsIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9651 /* svmls_lane_za32_u16_vg2x4 */, 2259 /* vUiq32Usq8UsIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9677 /* svmls_lane_za32_u8_vg4x1 */, 7036 /* vUiq16Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9702 /* svmls_lane_za32_u8_vg4x2 */, 2301 /* vUiq32Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9727 /* svmls_lane_za32_u8_vg4x4 */, 2344 /* vUiq64Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9752 /* svmls_lane_za64_f64_vg1x2 */, 7130 /* vUiq4dq2dIUWi */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9778 /* svmls_lane_za64_f64_vg1x4 */, 7170 /* vUiq8dq2dIUWi */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9804 /* svmls_lane_za64_s16_vg4x1 */, 6758 /* vUiq8sq8sIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9830 /* svmls_lane_za64_s16_vg4x2 */, 2048 /* vUiq16sq8sIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9856 /* svmls_lane_za64_s16_vg4x4 */, 2089 /* vUiq32sq8sIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9882 /* svmls_lane_za64_u16_vg4x1 */, 6943 /* vUiq8Usq8UsIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9908 /* svmls_lane_za64_u16_vg4x2 */, 2216 /* vUiq16Usq8UsIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9934 /* svmls_lane_za64_u16_vg4x4 */, 2259 /* vUiq32Usq8UsIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9960 /* svmls_single_za16_bf16_vg1x2 */, 2589 /* vUiq16yq8y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9989 /* svmls_single_za16_bf16_vg1x4 */, 2629 /* vUiq32yq8y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10018 /* svmls_single_za16_f16_vg1x2 */, 2668 /* vUiq16hq8h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10046 /* svmls_single_za16_f16_vg1x4 */, 2707 /* vUiq32hq8h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10074 /* svmls_single_za32_bf16_vg2x2 */, 2589 /* vUiq16yq8y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10103 /* svmls_single_za32_bf16_vg2x4 */, 2629 /* vUiq32yq8y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10132 /* svmls_single_za32_f16_vg2x2 */, 2668 /* vUiq16hq8h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10160 /* svmls_single_za32_f16_vg2x4 */, 2707 /* vUiq32hq8h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10188 /* svmls_single_za32_f32_vg1x2 */, 7660 /* vUiq8fq4f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10216 /* svmls_single_za32_f32_vg1x4 */, 7698 /* vUiq16fq4f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10244 /* svmls_single_za32_s16_vg2x2 */, 2810 /* vUiq16sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10272 /* svmls_single_za32_s16_vg2x4 */, 2849 /* vUiq32sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10300 /* svmls_single_za32_s8_vg4x2 */, 2887 /* vUiq32Scq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10327 /* svmls_single_za32_s8_vg4x4 */, 2928 /* vUiq64Scq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10354 /* svmls_single_za32_u16_vg2x2 */, 2970 /* vUiq16Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10382 /* svmls_single_za32_u16_vg2x4 */, 3011 /* vUiq32Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10410 /* svmls_single_za32_u8_vg4x2 */, 3051 /* vUiq32Ucq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10437 /* svmls_single_za32_u8_vg4x4 */, 3092 /* vUiq64Ucq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10464 /* svmls_single_za64_f64_vg1x2 */, 8021 /* vUiq4dq2d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10492 /* svmls_single_za64_f64_vg1x4 */, 8059 /* vUiq8dq2d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10520 /* svmls_single_za64_s16_vg4x2 */, 2810 /* vUiq16sq8s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10548 /* svmls_single_za64_s16_vg4x4 */, 2849 /* vUiq32sq8s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10576 /* svmls_single_za64_u16_vg4x2 */, 2970 /* vUiq16Usq8Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10604 /* svmls_single_za64_u16_vg4x4 */, 3011 /* vUiq32Usq8Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10632 /* svmls_za16_bf16_vg1x2 */, 3320 /* vUiq16yq16y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10654 /* svmls_za16_bf16_vg1x4 */, 3354 /* vUiq32yq32y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10676 /* svmls_za16_f16_vg1x2 */, 3387 /* vUiq16hq16h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10697 /* svmls_za16_f16_vg1x4 */, 3420 /* vUiq32hq32h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10718 /* svmls_za32_bf16_vg2x1 */, 8379 /* vUiq8yq8y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10740 /* svmls_za32_bf16_vg2x2 */, 3320 /* vUiq16yq16y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10762 /* svmls_za32_bf16_vg2x4 */, 3354 /* vUiq32yq32y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10784 /* svmls_za32_f16_vg2x1 */, 8454 /* vUiq8hq8h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10805 /* svmls_za32_f16_vg2x2 */, 3387 /* vUiq16hq16h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10826 /* svmls_za32_f16_vg2x4 */, 3420 /* vUiq32hq32h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10847 /* svmls_za32_f32_vg1x2 */, 8527 /* vUiq8fq8f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10868 /* svmls_za32_f32_vg1x4 */, 8558 /* vUiq16fq16f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10889 /* svmls_za32_s16_vg2x1 */, 8666 /* vUiq8sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10910 /* svmls_za32_s16_vg2x2 */, 3503 /* vUiq16sq16s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10931 /* svmls_za32_s16_vg2x4 */, 3536 /* vUiq32sq32s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10952 /* svmls_za32_s8_vg4x1 */, 8738 /* vUiq16Scq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10972 /* svmls_za32_s8_vg4x2 */, 3568 /* vUiq32Scq32Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10992 /* svmls_za32_s8_vg4x4 */, 3602 /* vUiq64Scq64Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11012 /* svmls_za32_u16_vg2x1 */, 8813 /* vUiq8Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11033 /* svmls_za32_u16_vg2x2 */, 3637 /* vUiq16Usq16Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11054 /* svmls_za32_u16_vg2x4 */, 3672 /* vUiq32Usq32Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11075 /* svmls_za32_u8_vg4x1 */, 8887 /* vUiq16Ucq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11095 /* svmls_za32_u8_vg4x2 */, 3706 /* vUiq32Ucq32Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11115 /* svmls_za32_u8_vg4x4 */, 3740 /* vUiq64Ucq64Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11135 /* svmls_za64_f64_vg1x2 */, 8962 /* vUiq4dq4d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11156 /* svmls_za64_f64_vg1x4 */, 8993 /* vUiq8dq8d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11177 /* svmls_za64_s16_vg4x1 */, 8666 /* vUiq8sq8s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11198 /* svmls_za64_s16_vg4x2 */, 3503 /* vUiq16sq16s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11219 /* svmls_za64_s16_vg4x4 */, 3536 /* vUiq32sq32s */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11240 /* svmls_za64_u16_vg4x1 */, 8813 /* vUiq8Usq8Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11261 /* svmls_za64_u16_vg4x2 */, 3637 /* vUiq16Usq16Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11282 /* svmls_za64_u16_vg4x4 */, 3672 /* vUiq32Usq32Us */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11303 /* svmop4a_1x1_za16_bf16_bf16 */, 11330 /* vIUWiq8yq8y */, 1 /* n */, 11342 /* sme,sme2,sme-mop4,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11371 /* svmop4a_1x1_za16_f16_f16 */, 11396 /* vIUWiq8hq8h */, 1 /* n */, 11408 /* sme,sme2,sme-mop4,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11437 /* svmop4a_1x1_za16_mf8_mf8_fpm */, 11466 /* vIUWiq16mq16mUWi */, 1 /* n */, 11483 /* sme,sme2,sme-mop4,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11511 /* svmop4a_1x1_za32_bf16_bf16 */, 11330 /* vIUWiq8yq8y */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11556 /* svmop4a_1x1_za32_f16_f16 */, 11396 /* vIUWiq8hq8h */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11581 /* svmop4a_1x1_za32_f32_f32 */, 11606 /* vIUWiq4fq4f */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11618 /* svmop4a_1x1_za32_mf8_mf8_fpm */, 11466 /* vIUWiq16mq16mUWi */, 1 /* n */, 11647 /* sme,sme2,sme-mop4,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11675 /* svmop4a_1x1_za32_s16_s16 */, 11700 /* vIUWiq8sq8s */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11712 /* svmop4a_1x1_za32_s8_s8 */, 11735 /* vIUWiq16Scq16Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11751 /* svmop4a_1x1_za32_s8_u8 */, 11774 /* vIUWiq16Scq16Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11790 /* svmop4a_1x1_za32_u16_u16 */, 11815 /* vIUWiq8Usq8Us */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11829 /* svmop4a_1x1_za32_u8_s8 */, 11852 /* vIUWiq16Ucq16Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11868 /* svmop4a_1x1_za32_u8_u8 */, 11891 /* vIUWiq16Ucq16Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11907 /* svmop4a_1x1_za64_f64_f64 */, 11932 /* vIUWiq2dq2d */, 1 /* n */, 11944 /* sme,sme2,sme-mop4,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11973 /* svmop4a_1x1_za64_s16_s16 */, 11700 /* vIUWiq8sq8s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12027 /* svmop4a_1x1_za64_s16_u16 */, 12052 /* vIUWiq8sq8Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12065 /* svmop4a_1x1_za64_u16_s16 */, 12090 /* vIUWiq8Usq8s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12103 /* svmop4a_1x1_za64_u16_u16 */, 11815 /* vIUWiq8Usq8Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12128 /* svmop4a_1x2_za16_bf16_bf16 */, 12155 /* vIUWiq8yq16y */, 1 /* n */, 11342 /* sme,sme2,sme-mop4,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12168 /* svmop4a_1x2_za16_f16_f16 */, 12193 /* vIUWiq8hq16h */, 1 /* n */, 11408 /* sme,sme2,sme-mop4,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12206 /* svmop4a_1x2_za16_mf8_mf8_fpm */, 12235 /* vIUWiq16mq32mUWi */, 1 /* n */, 11483 /* sme,sme2,sme-mop4,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12252 /* svmop4a_1x2_za32_bf16_bf16 */, 12155 /* vIUWiq8yq16y */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12279 /* svmop4a_1x2_za32_f16_f16 */, 12193 /* vIUWiq8hq16h */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12304 /* svmop4a_1x2_za32_f32_f32 */, 12329 /* vIUWiq4fq8f */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12341 /* svmop4a_1x2_za32_mf8_mf8_fpm */, 12235 /* vIUWiq16mq32mUWi */, 1 /* n */, 11647 /* sme,sme2,sme-mop4,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12370 /* svmop4a_1x2_za32_s16_s16 */, 12395 /* vIUWiq8sq16s */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12408 /* svmop4a_1x2_za32_s8_s8 */, 12431 /* vIUWiq16Scq32Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12447 /* svmop4a_1x2_za32_s8_u8 */, 12470 /* vIUWiq16Scq32Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12486 /* svmop4a_1x2_za32_u16_u16 */, 12511 /* vIUWiq8Usq16Us */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12526 /* svmop4a_1x2_za32_u8_s8 */, 12549 /* vIUWiq16Ucq32Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12565 /* svmop4a_1x2_za32_u8_u8 */, 12588 /* vIUWiq16Ucq32Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12604 /* svmop4a_1x2_za64_f64_f64 */, 12629 /* vIUWiq2dq4d */, 1 /* n */, 11944 /* sme,sme2,sme-mop4,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12641 /* svmop4a_1x2_za64_s16_s16 */, 12395 /* vIUWiq8sq16s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12666 /* svmop4a_1x2_za64_s16_u16 */, 12691 /* vIUWiq8sq16Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12705 /* svmop4a_1x2_za64_u16_s16 */, 12730 /* vIUWiq8Usq16s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12744 /* svmop4a_1x2_za64_u16_u16 */, 12511 /* vIUWiq8Usq16Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12769 /* svmop4a_2x1_za16_bf16_bf16 */, 12796 /* vIUWiq16yq8y */, 1 /* n */, 11342 /* sme,sme2,sme-mop4,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12809 /* svmop4a_2x1_za16_f16_f16 */, 12834 /* vIUWiq16hq8h */, 1 /* n */, 11408 /* sme,sme2,sme-mop4,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12847 /* svmop4a_2x1_za16_mf8_mf8_fpm */, 12876 /* vIUWiq32mq16mUWi */, 1 /* n */, 11483 /* sme,sme2,sme-mop4,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12893 /* svmop4a_2x1_za32_bf16_bf16 */, 12796 /* vIUWiq16yq8y */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12920 /* svmop4a_2x1_za32_f16_f16 */, 12834 /* vIUWiq16hq8h */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12945 /* svmop4a_2x1_za32_f32_f32 */, 12970 /* vIUWiq8fq4f */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12982 /* svmop4a_2x1_za32_mf8_mf8_fpm */, 12876 /* vIUWiq32mq16mUWi */, 1 /* n */, 11647 /* sme,sme2,sme-mop4,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13011 /* svmop4a_2x1_za32_s16_s16 */, 13036 /* vIUWiq16sq8s */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13049 /* svmop4a_2x1_za32_s8_s8 */, 13072 /* vIUWiq32Scq16Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13088 /* svmop4a_2x1_za32_s8_u8 */, 13111 /* vIUWiq32Scq16Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13127 /* svmop4a_2x1_za32_u16_u16 */, 13152 /* vIUWiq16Usq8Us */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13167 /* svmop4a_2x1_za32_u8_s8 */, 13190 /* vIUWiq32Ucq16Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13206 /* svmop4a_2x1_za32_u8_u8 */, 13229 /* vIUWiq32Ucq16Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13245 /* svmop4a_2x1_za64_f64_f64 */, 13270 /* vIUWiq4dq2d */, 1 /* n */, 11944 /* sme,sme2,sme-mop4,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13282 /* svmop4a_2x1_za64_s16_s16 */, 13036 /* vIUWiq16sq8s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13307 /* svmop4a_2x1_za64_s16_u16 */, 13332 /* vIUWiq16sq8Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13346 /* svmop4a_2x1_za64_u16_s16 */, 13371 /* vIUWiq16Usq8s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13385 /* svmop4a_2x1_za64_u16_u16 */, 13152 /* vIUWiq16Usq8Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13410 /* svmop4a_2x2_za16_bf16_bf16 */, 13437 /* vIUWiq16yq16y */, 1 /* n */, 11342 /* sme,sme2,sme-mop4,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13451 /* svmop4a_2x2_za16_f16_f16 */, 13476 /* vIUWiq16hq16h */, 1 /* n */, 11408 /* sme,sme2,sme-mop4,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13490 /* svmop4a_2x2_za16_mf8_mf8_fpm */, 13519 /* vIUWiq32mq32mUWi */, 1 /* n */, 11483 /* sme,sme2,sme-mop4,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13536 /* svmop4a_2x2_za32_bf16_bf16 */, 13437 /* vIUWiq16yq16y */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13563 /* svmop4a_2x2_za32_f16_f16 */, 13476 /* vIUWiq16hq16h */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13588 /* svmop4a_2x2_za32_f32_f32 */, 13613 /* vIUWiq8fq8f */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13625 /* svmop4a_2x2_za32_mf8_mf8_fpm */, 13519 /* vIUWiq32mq32mUWi */, 1 /* n */, 11647 /* sme,sme2,sme-mop4,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13654 /* svmop4a_2x2_za32_s16_s16 */, 13679 /* vIUWiq16sq16s */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13693 /* svmop4a_2x2_za32_s8_s8 */, 13716 /* vIUWiq32Scq32Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13732 /* svmop4a_2x2_za32_s8_u8 */, 13755 /* vIUWiq32Scq32Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13771 /* svmop4a_2x2_za32_u16_u16 */, 13796 /* vIUWiq16Usq16Us */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13812 /* svmop4a_2x2_za32_u8_s8 */, 13835 /* vIUWiq32Ucq32Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13851 /* svmop4a_2x2_za32_u8_u8 */, 13874 /* vIUWiq32Ucq32Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13890 /* svmop4a_2x2_za64_f64_f64 */, 13915 /* vIUWiq4dq4d */, 1 /* n */, 11944 /* sme,sme2,sme-mop4,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13927 /* svmop4a_2x2_za64_s16_s16 */, 13679 /* vIUWiq16sq16s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13952 /* svmop4a_2x2_za64_s16_u16 */, 13977 /* vIUWiq16sq16Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13992 /* svmop4a_2x2_za64_u16_s16 */, 14017 /* vIUWiq16Usq16s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14032 /* svmop4a_2x2_za64_u16_u16 */, 13796 /* vIUWiq16Usq16Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14057 /* svmop4s_1x1_za16_bf16_bf16 */, 11330 /* vIUWiq8yq8y */, 1 /* n */, 11342 /* sme,sme2,sme-mop4,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14084 /* svmop4s_1x1_za16_f16_f16 */, 11396 /* vIUWiq8hq8h */, 1 /* n */, 11408 /* sme,sme2,sme-mop4,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14109 /* svmop4s_1x1_za32_bf16_bf16 */, 11330 /* vIUWiq8yq8y */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14136 /* svmop4s_1x1_za32_f16_f16 */, 11396 /* vIUWiq8hq8h */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14161 /* svmop4s_1x1_za32_f32_f32 */, 11606 /* vIUWiq4fq4f */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14186 /* svmop4s_1x1_za32_s16_s16 */, 11700 /* vIUWiq8sq8s */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14211 /* svmop4s_1x1_za32_s8_s8 */, 11735 /* vIUWiq16Scq16Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14234 /* svmop4s_1x1_za32_s8_u8 */, 11774 /* vIUWiq16Scq16Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14257 /* svmop4s_1x1_za32_u16_u16 */, 11815 /* vIUWiq8Usq8Us */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14282 /* svmop4s_1x1_za32_u8_s8 */, 11852 /* vIUWiq16Ucq16Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14305 /* svmop4s_1x1_za32_u8_u8 */, 11891 /* vIUWiq16Ucq16Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14328 /* svmop4s_1x1_za64_f64_f64 */, 11932 /* vIUWiq2dq2d */, 1 /* n */, 11944 /* sme,sme2,sme-mop4,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14353 /* svmop4s_1x1_za64_s16_s16 */, 11700 /* vIUWiq8sq8s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14378 /* svmop4s_1x1_za64_s16_u16 */, 12052 /* vIUWiq8sq8Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14403 /* svmop4s_1x1_za64_u16_s16 */, 12090 /* vIUWiq8Usq8s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14428 /* svmop4s_1x1_za64_u16_u16 */, 11815 /* vIUWiq8Usq8Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14453 /* svmop4s_1x2_za16_bf16_bf16 */, 12155 /* vIUWiq8yq16y */, 1 /* n */, 11342 /* sme,sme2,sme-mop4,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14480 /* svmop4s_1x2_za16_f16_f16 */, 12193 /* vIUWiq8hq16h */, 1 /* n */, 11408 /* sme,sme2,sme-mop4,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14505 /* svmop4s_1x2_za32_bf16_bf16 */, 12155 /* vIUWiq8yq16y */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14532 /* svmop4s_1x2_za32_f16_f16 */, 12193 /* vIUWiq8hq16h */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14557 /* svmop4s_1x2_za32_f32_f32 */, 12329 /* vIUWiq4fq8f */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14582 /* svmop4s_1x2_za32_s16_s16 */, 12395 /* vIUWiq8sq16s */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14607 /* svmop4s_1x2_za32_s8_s8 */, 12431 /* vIUWiq16Scq32Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14630 /* svmop4s_1x2_za32_s8_u8 */, 12470 /* vIUWiq16Scq32Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14653 /* svmop4s_1x2_za32_u16_u16 */, 12511 /* vIUWiq8Usq16Us */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14678 /* svmop4s_1x2_za32_u8_s8 */, 12549 /* vIUWiq16Ucq32Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14701 /* svmop4s_1x2_za32_u8_u8 */, 12588 /* vIUWiq16Ucq32Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14724 /* svmop4s_1x2_za64_f64_f64 */, 12629 /* vIUWiq2dq4d */, 1 /* n */, 11944 /* sme,sme2,sme-mop4,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14749 /* svmop4s_1x2_za64_s16_s16 */, 12395 /* vIUWiq8sq16s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14774 /* svmop4s_1x2_za64_s16_u16 */, 12691 /* vIUWiq8sq16Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14799 /* svmop4s_1x2_za64_u16_s16 */, 12730 /* vIUWiq8Usq16s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14824 /* svmop4s_1x2_za64_u16_u16 */, 12511 /* vIUWiq8Usq16Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14849 /* svmop4s_2x1_za16_bf16_bf16 */, 12796 /* vIUWiq16yq8y */, 1 /* n */, 11342 /* sme,sme2,sme-mop4,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14876 /* svmop4s_2x1_za16_f16_f16 */, 12834 /* vIUWiq16hq8h */, 1 /* n */, 11408 /* sme,sme2,sme-mop4,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14901 /* svmop4s_2x1_za32_bf16_bf16 */, 12796 /* vIUWiq16yq8y */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14928 /* svmop4s_2x1_za32_f16_f16 */, 12834 /* vIUWiq16hq8h */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14953 /* svmop4s_2x1_za32_f32_f32 */, 12970 /* vIUWiq8fq4f */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14978 /* svmop4s_2x1_za32_s16_s16 */, 13036 /* vIUWiq16sq8s */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15003 /* svmop4s_2x1_za32_s8_s8 */, 13072 /* vIUWiq32Scq16Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15026 /* svmop4s_2x1_za32_s8_u8 */, 13111 /* vIUWiq32Scq16Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15049 /* svmop4s_2x1_za32_u16_u16 */, 13152 /* vIUWiq16Usq8Us */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15074 /* svmop4s_2x1_za32_u8_s8 */, 13190 /* vIUWiq32Ucq16Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15097 /* svmop4s_2x1_za32_u8_u8 */, 13229 /* vIUWiq32Ucq16Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15120 /* svmop4s_2x1_za64_f64_f64 */, 13270 /* vIUWiq4dq2d */, 1 /* n */, 11944 /* sme,sme2,sme-mop4,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15145 /* svmop4s_2x1_za64_s16_s16 */, 13036 /* vIUWiq16sq8s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15170 /* svmop4s_2x1_za64_s16_u16 */, 13332 /* vIUWiq16sq8Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15195 /* svmop4s_2x1_za64_u16_s16 */, 13371 /* vIUWiq16Usq8s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15220 /* svmop4s_2x1_za64_u16_u16 */, 13152 /* vIUWiq16Usq8Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15245 /* svmop4s_2x2_za16_bf16_bf16 */, 13437 /* vIUWiq16yq16y */, 1 /* n */, 11342 /* sme,sme2,sme-mop4,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15272 /* svmop4s_2x2_za16_f16_f16 */, 13476 /* vIUWiq16hq16h */, 1 /* n */, 11408 /* sme,sme2,sme-mop4,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15297 /* svmop4s_2x2_za32_bf16_bf16 */, 13437 /* vIUWiq16yq16y */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15324 /* svmop4s_2x2_za32_f16_f16 */, 13476 /* vIUWiq16hq16h */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15349 /* svmop4s_2x2_za32_f32_f32 */, 13613 /* vIUWiq8fq8f */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15374 /* svmop4s_2x2_za32_s16_s16 */, 13679 /* vIUWiq16sq16s */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15399 /* svmop4s_2x2_za32_s8_s8 */, 13716 /* vIUWiq32Scq32Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15422 /* svmop4s_2x2_za32_s8_u8 */, 13755 /* vIUWiq32Scq32Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15445 /* svmop4s_2x2_za32_u16_u16 */, 13796 /* vIUWiq16Usq16Us */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15470 /* svmop4s_2x2_za32_u8_s8 */, 13835 /* vIUWiq32Ucq32Sc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15493 /* svmop4s_2x2_za32_u8_u8 */, 13874 /* vIUWiq32Ucq32Uc */, 1 /* n */, 11538 /* sme,sme2,sme-mop4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15516 /* svmop4s_2x2_za64_f64_f64 */, 13915 /* vIUWiq4dq4d */, 1 /* n */, 11944 /* sme,sme2,sme-mop4,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15541 /* svmop4s_2x2_za64_s16_s16 */, 13679 /* vIUWiq16sq16s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15566 /* svmop4s_2x2_za64_s16_u16 */, 13977 /* vIUWiq16sq16Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15591 /* svmop4s_2x2_za64_u16_s16 */, 14017 /* vIUWiq16Usq16s */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15616 /* svmop4s_2x2_za64_u16_u16 */, 13796 /* vIUWiq16Usq16Us */, 1 /* n */, 11998 /* sme,sme2,sme-mop4,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15641 /* svmopa_za16_bf16_m */, 15660 /* vIUWiq16bq16bq8yq8y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15680 /* svmopa_za16_f16_m */, 15698 /* vIUWiq16bq16bq8hq8h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15718 /* svmopa_za16_mf8_m_fpm */, 15740 /* vIUWiq16bq16bq16mq16mUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15765 /* svmopa_za32_bf16_m */, 15660 /* vIUWiq16bq16bq8yq8y */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15784 /* svmopa_za32_f16_m */, 15698 /* vIUWiq16bq16bq8hq8h */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15802 /* svmopa_za32_f32_m */, 15820 /* vIUWiq16bq16bq4fq4f */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15840 /* svmopa_za32_mf8_m_fpm */, 15740 /* vIUWiq16bq16bq16mq16mUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15862 /* svmopa_za32_s16_m */, 15880 /* vIUWiq16bq16bq8sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15900 /* svmopa_za32_s8_m */, 15917 /* vIUWiq16bq16bq16Scq16Sc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15941 /* svmopa_za32_u16_m */, 15959 /* vIUWiq16bq16bq8Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15981 /* svmopa_za32_u8_m */, 15998 /* vIUWiq16bq16bq16Ucq16Uc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16022 /* svmopa_za64_f64_m */, 16040 /* vIUWiq16bq16bq2dq2d */, 1 /* n */, 16060 /* sme,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16075 /* svmopa_za64_s16_m */, 15880 /* vIUWiq16bq16bq8sq8s */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16093 /* svmopa_za64_u16_m */, 15959 /* vIUWiq16bq16bq8Usq8Us */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16111 /* svmops_za16_bf16_m */, 15660 /* vIUWiq16bq16bq8yq8y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16130 /* svmops_za16_f16_m */, 15698 /* vIUWiq16bq16bq8hq8h */, 1 /* n */, 6224 /* sme,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16148 /* svmops_za32_bf16_m */, 15660 /* vIUWiq16bq16bq8yq8y */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16167 /* svmops_za32_f16_m */, 15698 /* vIUWiq16bq16bq8hq8h */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16185 /* svmops_za32_f32_m */, 15820 /* vIUWiq16bq16bq4fq4f */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16203 /* svmops_za32_s16_m */, 15880 /* vIUWiq16bq16bq8sq8s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16221 /* svmops_za32_s8_m */, 15917 /* vIUWiq16bq16bq16Scq16Sc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16238 /* svmops_za32_u16_m */, 15959 /* vIUWiq16bq16bq8Usq8Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16256 /* svmops_za32_u8_m */, 15998 /* vIUWiq16bq16bq16Ucq16Uc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16273 /* svmops_za64_f64_m */, 16040 /* vIUWiq16bq16bq2dq2d */, 1 /* n */, 16060 /* sme,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16291 /* svmops_za64_s16_m */, 15880 /* vIUWiq16bq16bq8sq8s */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16309 /* svmops_za64_u16_m */, 15959 /* vIUWiq16bq16bq8Usq8Us */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16327 /* svread_hor_za128_bf16_m */, 16351 /* q8yq8yq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16368 /* svread_hor_za128_f16_m */, 16391 /* q8hq8hq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16408 /* svread_hor_za128_f32_m */, 16431 /* q4fq4fq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16448 /* svread_hor_za128_f64_m */, 16471 /* q2dq2dq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16488 /* svread_hor_za128_mf8_m */, 16511 /* q16mq16mq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16530 /* svread_hor_za128_s16_m */, 16553 /* q8sq8sq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16570 /* svread_hor_za128_s32_m */, 16593 /* q4iq4iq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16610 /* svread_hor_za128_s64_m */, 16633 /* q2Wiq2Wiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16652 /* svread_hor_za128_s8_m */, 16674 /* q16Scq16Scq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16695 /* svread_hor_za128_u16_m */, 16718 /* q8Usq8Usq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16737 /* svread_hor_za128_u32_m */, 16760 /* q4Uiq4Uiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16779 /* svread_hor_za128_u64_m */, 16802 /* q2UWiq2UWiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16823 /* svread_hor_za128_u8_m */, 16845 /* q16Ucq16Ucq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16866 /* svread_hor_za16_bf16_m */, 16351 /* q8yq8yq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16889 /* svread_hor_za16_bf16_vg2 */, 16914 /* q16yIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16925 /* svread_hor_za16_bf16_vg4 */, 16950 /* q32yIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16961 /* svread_hor_za16_f16_m */, 16391 /* q8hq8hq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16983 /* svread_hor_za16_f16_vg2 */, 17007 /* q16hIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17018 /* svread_hor_za16_f16_vg4 */, 17042 /* q32hIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17053 /* svread_hor_za16_s16_m */, 16553 /* q8sq8sq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17075 /* svread_hor_za16_s16_vg2 */, 17099 /* q16sIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17110 /* svread_hor_za16_s16_vg4 */, 17134 /* q32sIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17145 /* svread_hor_za16_u16_m */, 16718 /* q8Usq8Usq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17167 /* svread_hor_za16_u16_vg2 */, 17191 /* q16UsIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17203 /* svread_hor_za16_u16_vg4 */, 17227 /* q32UsIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17239 /* svread_hor_za32_f32_m */, 16431 /* q4fq4fq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17261 /* svread_hor_za32_f32_vg2 */, 17285 /* q8fIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17295 /* svread_hor_za32_f32_vg4 */, 17319 /* q16fIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17330 /* svread_hor_za32_s32_m */, 16593 /* q4iq4iq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17352 /* svread_hor_za32_s32_vg2 */, 17376 /* q8iIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17386 /* svread_hor_za32_s32_vg4 */, 17410 /* q16iIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17421 /* svread_hor_za32_u32_m */, 16760 /* q4Uiq4Uiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17443 /* svread_hor_za32_u32_vg2 */, 17467 /* q8UiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17478 /* svread_hor_za32_u32_vg4 */, 17502 /* q16UiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17514 /* svread_hor_za64_f64_m */, 16471 /* q2dq2dq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17536 /* svread_hor_za64_f64_vg2 */, 17560 /* q4dIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17570 /* svread_hor_za64_f64_vg4 */, 17594 /* q8dIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17604 /* svread_hor_za64_s64_m */, 16633 /* q2Wiq2Wiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17626 /* svread_hor_za64_s64_vg2 */, 17650 /* q4WiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17661 /* svread_hor_za64_s64_vg4 */, 17685 /* q8WiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17696 /* svread_hor_za64_u64_m */, 16802 /* q2UWiq2UWiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17718 /* svread_hor_za64_u64_vg2 */, 17742 /* q4UWiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17754 /* svread_hor_za64_u64_vg4 */, 17778 /* q8UWiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17790 /* svread_hor_za8_mf8_m */, 16511 /* q16mq16mq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17811 /* svread_hor_za8_mf8_vg2 */, 17834 /* q32mIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17845 /* svread_hor_za8_mf8_vg4 */, 17868 /* q64mIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17879 /* svread_hor_za8_s8_m */, 16674 /* q16Scq16Scq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17899 /* svread_hor_za8_s8_vg2 */, 17921 /* q32ScIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17933 /* svread_hor_za8_s8_vg4 */, 17955 /* q64ScIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17967 /* svread_hor_za8_u8_m */, 16845 /* q16Ucq16Ucq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17987 /* svread_hor_za8_u8_vg2 */, 18009 /* q32UcIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18021 /* svread_hor_za8_u8_vg4 */, 18043 /* q64UcIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18055 /* svread_ver_za128_bf16_m */, 16351 /* q8yq8yq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18079 /* svread_ver_za128_f16_m */, 16391 /* q8hq8hq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18102 /* svread_ver_za128_f32_m */, 16431 /* q4fq4fq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18125 /* svread_ver_za128_f64_m */, 16471 /* q2dq2dq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18148 /* svread_ver_za128_mf8_m */, 16511 /* q16mq16mq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18171 /* svread_ver_za128_s16_m */, 16553 /* q8sq8sq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18194 /* svread_ver_za128_s32_m */, 16593 /* q4iq4iq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18217 /* svread_ver_za128_s64_m */, 16633 /* q2Wiq2Wiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18240 /* svread_ver_za128_s8_m */, 16674 /* q16Scq16Scq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18262 /* svread_ver_za128_u16_m */, 16718 /* q8Usq8Usq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18285 /* svread_ver_za128_u32_m */, 16760 /* q4Uiq4Uiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18308 /* svread_ver_za128_u64_m */, 16802 /* q2UWiq2UWiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18331 /* svread_ver_za128_u8_m */, 16845 /* q16Ucq16Ucq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18353 /* svread_ver_za16_bf16_m */, 16351 /* q8yq8yq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18376 /* svread_ver_za16_bf16_vg2 */, 16914 /* q16yIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18401 /* svread_ver_za16_bf16_vg4 */, 16950 /* q32yIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18426 /* svread_ver_za16_f16_m */, 16391 /* q8hq8hq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18448 /* svread_ver_za16_f16_vg2 */, 17007 /* q16hIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18472 /* svread_ver_za16_f16_vg4 */, 17042 /* q32hIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18496 /* svread_ver_za16_s16_m */, 16553 /* q8sq8sq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18518 /* svread_ver_za16_s16_vg2 */, 17099 /* q16sIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18542 /* svread_ver_za16_s16_vg4 */, 17134 /* q32sIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18566 /* svread_ver_za16_u16_m */, 16718 /* q8Usq8Usq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18588 /* svread_ver_za16_u16_vg2 */, 17191 /* q16UsIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18612 /* svread_ver_za16_u16_vg4 */, 17227 /* q32UsIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18636 /* svread_ver_za32_f32_m */, 16431 /* q4fq4fq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18658 /* svread_ver_za32_f32_vg2 */, 17285 /* q8fIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18682 /* svread_ver_za32_f32_vg4 */, 17319 /* q16fIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18706 /* svread_ver_za32_s32_m */, 16593 /* q4iq4iq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18728 /* svread_ver_za32_s32_vg2 */, 17376 /* q8iIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18752 /* svread_ver_za32_s32_vg4 */, 17410 /* q16iIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18776 /* svread_ver_za32_u32_m */, 16760 /* q4Uiq4Uiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18798 /* svread_ver_za32_u32_vg2 */, 17467 /* q8UiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18822 /* svread_ver_za32_u32_vg4 */, 17502 /* q16UiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18846 /* svread_ver_za64_f64_m */, 16471 /* q2dq2dq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18868 /* svread_ver_za64_f64_vg2 */, 17560 /* q4dIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18892 /* svread_ver_za64_f64_vg4 */, 17594 /* q8dIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18916 /* svread_ver_za64_s64_m */, 16633 /* q2Wiq2Wiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18938 /* svread_ver_za64_s64_vg2 */, 17650 /* q4WiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18962 /* svread_ver_za64_s64_vg4 */, 17685 /* q8WiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18986 /* svread_ver_za64_u64_m */, 16802 /* q2UWiq2UWiq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19008 /* svread_ver_za64_u64_vg2 */, 17742 /* q4UWiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19032 /* svread_ver_za64_u64_vg4 */, 17778 /* q8UWiIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19056 /* svread_ver_za8_mf8_m */, 16511 /* q16mq16mq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19077 /* svread_ver_za8_mf8_vg2 */, 17834 /* q32mIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19100 /* svread_ver_za8_mf8_vg4 */, 17868 /* q64mIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19123 /* svread_ver_za8_s8_m */, 16674 /* q16Scq16Scq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19143 /* svread_ver_za8_s8_vg2 */, 17921 /* q32ScIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19165 /* svread_ver_za8_s8_vg4 */, 17955 /* q64ScIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19187 /* svread_ver_za8_u8_m */, 16845 /* q16Ucq16Ucq16bIUWiUi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19207 /* svread_ver_za8_u8_vg2 */, 18009 /* q32UcIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19229 /* svread_ver_za8_u8_vg4 */, 18043 /* q64UcIUWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19251 /* svread_za16_bf16_vg1x2 */, 19274 /* q16yUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19281 /* svread_za16_bf16_vg1x4 */, 19304 /* q32yUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19311 /* svread_za16_f16_vg1x2 */, 19333 /* q16hUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19340 /* svread_za16_f16_vg1x4 */, 19362 /* q32hUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19369 /* svread_za16_s16_vg1x2 */, 19391 /* q16sUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19398 /* svread_za16_s16_vg1x4 */, 19420 /* q32sUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19427 /* svread_za16_u16_vg1x2 */, 19449 /* q16UsUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19457 /* svread_za16_u16_vg1x4 */, 19479 /* q32UsUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19487 /* svread_za32_f32_vg1x2 */, 19509 /* q8fUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19515 /* svread_za32_f32_vg1x4 */, 19537 /* q16fUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19544 /* svread_za32_s32_vg1x2 */, 19566 /* q8iUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19572 /* svread_za32_s32_vg1x4 */, 19594 /* q16iUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19601 /* svread_za32_u32_vg1x2 */, 19623 /* q8UiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19630 /* svread_za32_u32_vg1x4 */, 19652 /* q16UiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19660 /* svread_za64_f64_vg1x2 */, 19682 /* q4dUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19688 /* svread_za64_f64_vg1x4 */, 19710 /* q8dUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19716 /* svread_za64_s64_vg1x2 */, 19738 /* q4WiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19745 /* svread_za64_s64_vg1x4 */, 19767 /* q8WiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19774 /* svread_za64_u64_vg1x2 */, 19796 /* q4UWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19804 /* svread_za64_u64_vg1x4 */, 19826 /* q8UWiUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19834 /* svread_za8_mf8_vg1x2 */, 19855 /* q32mUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19862 /* svread_za8_mf8_vg1x4 */, 19883 /* q64mUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19890 /* svread_za8_s8_vg1x2 */, 19910 /* q32ScUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19918 /* svread_za8_s8_vg1x4 */, 19938 /* q64ScUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19946 /* svread_za8_u8_vg1x2 */, 19966 /* q32UcUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19974 /* svread_za8_u8_vg1x4 */, 19994 /* q64UcUi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20002 /* svreadz_hor_za128_bf16 */, 20025 /* q8yIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20046 /* svreadz_hor_za128_f16 */, 20068 /* q8hIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20078 /* svreadz_hor_za128_f32 */, 20100 /* q4fIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20110 /* svreadz_hor_za128_f64 */, 20132 /* q2dIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20142 /* svreadz_hor_za128_mf8 */, 20164 /* q16mIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20175 /* svreadz_hor_za128_s16 */, 20197 /* q8sIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20207 /* svreadz_hor_za128_s32 */, 20229 /* q4iIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20239 /* svreadz_hor_za128_s64 */, 20261 /* q2WiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20272 /* svreadz_hor_za128_s8 */, 20293 /* q16ScIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20305 /* svreadz_hor_za128_u16 */, 20327 /* q8UsIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20338 /* svreadz_hor_za128_u32 */, 20360 /* q4UiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20371 /* svreadz_hor_za128_u64 */, 20393 /* q2UWiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20405 /* svreadz_hor_za128_u8 */, 20426 /* q16UcIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20438 /* svreadz_hor_za16_bf16 */, 20025 /* q8yIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20460 /* svreadz_hor_za16_bf16_vg2 */, 16914 /* q16yIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20486 /* svreadz_hor_za16_bf16_vg4 */, 16950 /* q32yIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20512 /* svreadz_hor_za16_f16 */, 20068 /* q8hIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20533 /* svreadz_hor_za16_f16_vg2 */, 17007 /* q16hIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20558 /* svreadz_hor_za16_f16_vg4 */, 17042 /* q32hIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20583 /* svreadz_hor_za16_s16 */, 20197 /* q8sIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20604 /* svreadz_hor_za16_s16_vg2 */, 17099 /* q16sIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20629 /* svreadz_hor_za16_s16_vg4 */, 17134 /* q32sIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20654 /* svreadz_hor_za16_u16 */, 20327 /* q8UsIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20675 /* svreadz_hor_za16_u16_vg2 */, 17191 /* q16UsIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20700 /* svreadz_hor_za16_u16_vg4 */, 17227 /* q32UsIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20725 /* svreadz_hor_za32_f32 */, 20100 /* q4fIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20746 /* svreadz_hor_za32_f32_vg2 */, 17285 /* q8fIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20771 /* svreadz_hor_za32_f32_vg4 */, 17319 /* q16fIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20796 /* svreadz_hor_za32_s32 */, 20229 /* q4iIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20817 /* svreadz_hor_za32_s32_vg2 */, 17376 /* q8iIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20842 /* svreadz_hor_za32_s32_vg4 */, 17410 /* q16iIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20867 /* svreadz_hor_za32_u32 */, 20360 /* q4UiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20888 /* svreadz_hor_za32_u32_vg2 */, 17467 /* q8UiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20913 /* svreadz_hor_za32_u32_vg4 */, 17502 /* q16UiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20938 /* svreadz_hor_za64_f64 */, 20132 /* q2dIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20959 /* svreadz_hor_za64_f64_vg2 */, 17560 /* q4dIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20984 /* svreadz_hor_za64_f64_vg4 */, 17594 /* q8dIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21009 /* svreadz_hor_za64_s64 */, 20261 /* q2WiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21030 /* svreadz_hor_za64_s64_vg2 */, 17650 /* q4WiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21055 /* svreadz_hor_za64_s64_vg4 */, 17685 /* q8WiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21080 /* svreadz_hor_za64_u64 */, 20393 /* q2UWiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21101 /* svreadz_hor_za64_u64_vg2 */, 17742 /* q4UWiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21126 /* svreadz_hor_za64_u64_vg4 */, 17778 /* q8UWiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21151 /* svreadz_hor_za8_mf8 */, 20164 /* q16mIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21171 /* svreadz_hor_za8_mf8_vg2 */, 17834 /* q32mIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21195 /* svreadz_hor_za8_mf8_vg4 */, 17868 /* q64mIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21219 /* svreadz_hor_za8_s8 */, 20293 /* q16ScIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21238 /* svreadz_hor_za8_s8_vg2 */, 17921 /* q32ScIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21261 /* svreadz_hor_za8_s8_vg4 */, 17955 /* q64ScIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21284 /* svreadz_hor_za8_u8 */, 20426 /* q16UcIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21303 /* svreadz_hor_za8_u8_vg2 */, 18009 /* q32UcIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21326 /* svreadz_hor_za8_u8_vg4 */, 18043 /* q64UcIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21349 /* svreadz_ver_za128_bf16 */, 20025 /* q8yIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21372 /* svreadz_ver_za128_f16 */, 20068 /* q8hIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21394 /* svreadz_ver_za128_f32 */, 20100 /* q4fIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21416 /* svreadz_ver_za128_f64 */, 20132 /* q2dIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21438 /* svreadz_ver_za128_mf8 */, 20164 /* q16mIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21460 /* svreadz_ver_za128_s16 */, 20197 /* q8sIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21482 /* svreadz_ver_za128_s32 */, 20229 /* q4iIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21504 /* svreadz_ver_za128_s64 */, 20261 /* q2WiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21526 /* svreadz_ver_za128_s8 */, 20293 /* q16ScIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21547 /* svreadz_ver_za128_u16 */, 20327 /* q8UsIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21569 /* svreadz_ver_za128_u32 */, 20360 /* q4UiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21591 /* svreadz_ver_za128_u64 */, 20393 /* q2UWiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21613 /* svreadz_ver_za128_u8 */, 20426 /* q16UcIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21634 /* svreadz_ver_za16_bf16 */, 20025 /* q8yIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21656 /* svreadz_ver_za16_bf16_vg2 */, 16914 /* q16yIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21682 /* svreadz_ver_za16_bf16_vg4 */, 16950 /* q32yIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21708 /* svreadz_ver_za16_f16 */, 20068 /* q8hIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21729 /* svreadz_ver_za16_f16_vg2 */, 17007 /* q16hIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21754 /* svreadz_ver_za16_f16_vg4 */, 17042 /* q32hIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21779 /* svreadz_ver_za16_s16 */, 20197 /* q8sIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21800 /* svreadz_ver_za16_s16_vg2 */, 17099 /* q16sIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21825 /* svreadz_ver_za16_s16_vg4 */, 17134 /* q32sIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21850 /* svreadz_ver_za16_u16 */, 20327 /* q8UsIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21871 /* svreadz_ver_za16_u16_vg2 */, 17191 /* q16UsIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21896 /* svreadz_ver_za16_u16_vg4 */, 17227 /* q32UsIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21921 /* svreadz_ver_za32_f32 */, 20100 /* q4fIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21942 /* svreadz_ver_za32_f32_vg2 */, 17285 /* q8fIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21967 /* svreadz_ver_za32_f32_vg4 */, 17319 /* q16fIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21992 /* svreadz_ver_za32_s32 */, 20229 /* q4iIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22013 /* svreadz_ver_za32_s32_vg2 */, 17376 /* q8iIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22038 /* svreadz_ver_za32_s32_vg4 */, 17410 /* q16iIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22063 /* svreadz_ver_za32_u32 */, 20360 /* q4UiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22084 /* svreadz_ver_za32_u32_vg2 */, 17467 /* q8UiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22109 /* svreadz_ver_za32_u32_vg4 */, 17502 /* q16UiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22134 /* svreadz_ver_za64_f64 */, 20132 /* q2dIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22155 /* svreadz_ver_za64_f64_vg2 */, 17560 /* q4dIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22180 /* svreadz_ver_za64_f64_vg4 */, 17594 /* q8dIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22205 /* svreadz_ver_za64_s64 */, 20261 /* q2WiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22226 /* svreadz_ver_za64_s64_vg2 */, 17650 /* q4WiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22251 /* svreadz_ver_za64_s64_vg4 */, 17685 /* q8WiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22276 /* svreadz_ver_za64_u64 */, 20393 /* q2UWiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22297 /* svreadz_ver_za64_u64_vg2 */, 17742 /* q4UWiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22322 /* svreadz_ver_za64_u64_vg4 */, 17778 /* q8UWiIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22347 /* svreadz_ver_za8_mf8 */, 20164 /* q16mIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22367 /* svreadz_ver_za8_mf8_vg2 */, 17834 /* q32mIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22391 /* svreadz_ver_za8_mf8_vg4 */, 17868 /* q64mIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22415 /* svreadz_ver_za8_s8 */, 20293 /* q16ScIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22434 /* svreadz_ver_za8_s8_vg2 */, 17921 /* q32ScIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22457 /* svreadz_ver_za8_s8_vg4 */, 17955 /* q64ScIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22480 /* svreadz_ver_za8_u8 */, 20426 /* q16UcIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22499 /* svreadz_ver_za8_u8_vg2 */, 18009 /* q32UcIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22522 /* svreadz_ver_za8_u8_vg4 */, 18043 /* q64UcIUWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22545 /* svreadz_za16_bf16_vg1x2 */, 19274 /* q16yUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22569 /* svreadz_za16_bf16_vg1x4 */, 19304 /* q32yUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22593 /* svreadz_za16_f16_vg1x2 */, 19333 /* q16hUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22616 /* svreadz_za16_f16_vg1x4 */, 19362 /* q32hUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22639 /* svreadz_za16_s16_vg1x2 */, 19391 /* q16sUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22662 /* svreadz_za16_s16_vg1x4 */, 19420 /* q32sUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22685 /* svreadz_za16_u16_vg1x2 */, 19449 /* q16UsUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22708 /* svreadz_za16_u16_vg1x4 */, 19479 /* q32UsUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22731 /* svreadz_za32_f32_vg1x2 */, 19509 /* q8fUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22754 /* svreadz_za32_f32_vg1x4 */, 19537 /* q16fUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22777 /* svreadz_za32_s32_vg1x2 */, 19566 /* q8iUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22800 /* svreadz_za32_s32_vg1x4 */, 19594 /* q16iUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22823 /* svreadz_za32_u32_vg1x2 */, 19623 /* q8UiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22846 /* svreadz_za32_u32_vg1x4 */, 19652 /* q16UiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22869 /* svreadz_za64_f64_vg1x2 */, 19682 /* q4dUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22892 /* svreadz_za64_f64_vg1x4 */, 19710 /* q8dUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22915 /* svreadz_za64_s64_vg1x2 */, 19738 /* q4WiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22938 /* svreadz_za64_s64_vg1x4 */, 19767 /* q8WiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22961 /* svreadz_za64_u64_vg1x2 */, 19796 /* q4UWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22984 /* svreadz_za64_u64_vg1x4 */, 19826 /* q8UWiUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23007 /* svreadz_za8_mf8_vg1x2 */, 19855 /* q32mUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23029 /* svreadz_za8_mf8_vg1x4 */, 19883 /* q64mUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23051 /* svreadz_za8_s8_vg1x2 */, 19910 /* q32ScUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23072 /* svreadz_za8_s8_vg1x4 */, 19938 /* q64ScUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23093 /* svreadz_za8_u8_vg1x2 */, 19966 /* q32UcUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23114 /* svreadz_za8_u8_vg1x4 */, 19994 /* q64UcUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23135 /* svst1_hor_vnum_za128 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23172 /* svst1_hor_vnum_za16 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23192 /* svst1_hor_vnum_za32 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23212 /* svst1_hor_vnum_za64 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23232 /* svst1_hor_vnum_za8 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23251 /* svst1_hor_za128 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23281 /* svst1_hor_za16 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23296 /* svst1_hor_za32 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23311 /* svst1_hor_za64 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23326 /* svst1_hor_za8 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23340 /* svst1_ver_vnum_za128 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23361 /* svst1_ver_vnum_za16 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23381 /* svst1_ver_vnum_za32 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23401 /* svst1_ver_vnum_za64 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23421 /* svst1_ver_vnum_za8 */, 23156 /* vIUWiUiq16bv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23440 /* svst1_ver_za128 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23456 /* svst1_ver_za16 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23471 /* svst1_ver_za32 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23486 /* svst1_ver_za64 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23501 /* svst1_ver_za8 */, 23267 /* vIUWiUiq16bv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23515 /* svstr_vnum_za */, 23529 /* vUiv*Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23537 /* svstr_za */, 23546 /* vUiv* */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23552 /* svstr_zt */, 23561 /* vIUWiv* */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23569 /* svsub_write_single_za32_s32_vg1x2 */, 68 /* vUiq8iq4i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23603 /* svsub_write_single_za32_s32_vg1x4 */, 121 /* vUiq16iq4i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23637 /* svsub_write_single_za32_u32_vg1x2 */, 166 /* vUiq8Uiq4Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23671 /* svsub_write_single_za32_u32_vg1x4 */, 212 /* vUiq16Uiq4Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23705 /* svsub_write_single_za64_s64_vg1x2 */, 259 /* vUiq4Wiq2Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23739 /* svsub_write_single_za64_s64_vg1x4 */, 325 /* vUiq8Wiq2Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23773 /* svsub_write_single_za64_u64_vg1x2 */, 371 /* vUiq4UWiq2UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23807 /* svsub_write_single_za64_u64_vg1x4 */, 419 /* vUiq8UWiq2UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23841 /* svsub_write_za32_s32_vg1x2 */, 460 /* vUiq8iq8i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23868 /* svsub_write_za32_s32_vg1x4 */, 497 /* vUiq16iq16i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23895 /* svsub_write_za32_u32_vg1x2 */, 536 /* vUiq8Uiq8Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23922 /* svsub_write_za32_u32_vg1x4 */, 575 /* vUiq16Uiq16Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23949 /* svsub_write_za64_s64_vg1x2 */, 616 /* vUiq4Wiq4Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23976 /* svsub_write_za64_s64_vg1x4 */, 655 /* vUiq8Wiq8Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24003 /* svsub_write_za64_u64_vg1x2 */, 694 /* vUiq4UWiq4UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24030 /* svsub_write_za64_u64_vg1x4 */, 735 /* vUiq8UWiq8UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24057 /* svsub_za16_bf16_vg1x2 */, 771 /* vUiq16y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24079 /* svsub_za16_bf16_vg1x4 */, 816 /* vUiq32y */, 1 /* n */, 779 /* sme,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24101 /* svsub_za16_f16_vg1x2 */, 845 /* vUiq16h */, 1 /* n */, 853 /* sme,(sme-f16f16|sme-f8f16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24122 /* svsub_za16_f16_vg1x4 */, 901 /* vUiq32h */, 1 /* n */, 853 /* sme,(sme-f16f16|sme-f8f16) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24143 /* svsub_za32_f32_vg1x2 */, 930 /* vUiq8f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24164 /* svsub_za32_f32_vg1x4 */, 958 /* vUiq16f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24185 /* svsub_za32_s32_vg1x2 */, 987 /* vUiq8i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24206 /* svsub_za32_s32_vg1x4 */, 1015 /* vUiq16i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24227 /* svsub_za32_u32_vg1x2 */, 1044 /* vUiq8Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24248 /* svsub_za32_u32_vg1x4 */, 1073 /* vUiq16Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24269 /* svsub_za64_f64_vg1x2 */, 1103 /* vUiq4d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24290 /* svsub_za64_f64_vg1x4 */, 1151 /* vUiq8d */, 1 /* n */, 1110 /* sme,sme2,sme-f64f64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24311 /* svsub_za64_s64_vg1x2 */, 1179 /* vUiq4Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24332 /* svsub_za64_s64_vg1x4 */, 1208 /* vUiq8Wi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24353 /* svsub_za64_u64_vg1x2 */, 1237 /* vUiq4UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24374 /* svsub_za64_u64_vg1x4 */, 1267 /* vUiq8UWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24395 /* svsudot_lane_za32_s8_vg1x2 */, 24422 /* vUiq32Scq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24440 /* svsudot_lane_za32_s8_vg1x4 */, 24467 /* vUiq64Scq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24485 /* svsudot_single_za32_s8_vg1x2 */, 24514 /* vUiq32Scq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24528 /* svsudot_single_za32_s8_vg1x4 */, 24557 /* vUiq64Scq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24571 /* svsudot_za32_s8_vg1x2 */, 24593 /* vUiq32Scq32Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24607 /* svsudot_za32_s8_vg1x4 */, 24629 /* vUiq64Scq64Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24643 /* svsumla_lane_za32_s8_vg4x1 */, 24670 /* vUiq16Scq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24688 /* svsumla_lane_za32_s8_vg4x2 */, 24422 /* vUiq32Scq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24715 /* svsumla_lane_za32_s8_vg4x4 */, 24467 /* vUiq64Scq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24742 /* svsumla_single_za32_s8_vg4x2 */, 24514 /* vUiq32Scq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24771 /* svsumla_single_za32_s8_vg4x4 */, 24557 /* vUiq64Scq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24800 /* svsumla_za32_s8_vg4x1 */, 24822 /* vUiq16Scq16Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24836 /* svsumla_za32_s8_vg4x2 */, 24593 /* vUiq32Scq32Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24858 /* svsumla_za32_s8_vg4x4 */, 24629 /* vUiq64Scq64Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24880 /* svsumopa_za32_s8_m */, 24899 /* vIUWiq16bq16bq16Scq16Uc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24923 /* svsumopa_za64_s16_m */, 24943 /* vIUWiq16bq16bq8sq8Us */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24964 /* svsumops_za32_s8_m */, 24899 /* vIUWiq16bq16bq16Scq16Uc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24983 /* svsumops_za64_s16_m */, 24943 /* vIUWiq16bq16bq8sq8Us */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25003 /* svsuvdot_lane_za32_s8_vg1x4 */, 2172 /* vUiq64Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25031 /* svtmopa_lane_za16_bf16_bf16 */, 25059 /* vIUWiq16yq8yq16UcIUWi */, 1 /* n */, 25081 /* sme,sme2,sme-tmop,sme-b16b16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25110 /* svtmopa_lane_za16_f16_f16 */, 25136 /* vIUWiq16hq8hq16UcIUWi */, 1 /* n */, 25158 /* sme,sme2,sme-tmop,sme-f16f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25187 /* svtmopa_lane_za16_mf8_mf8_fpm */, 25217 /* vIUWiq32mq16mq16UcIUWiUWi */, 1 /* n */, 25243 /* sme,sme2,sme-tmop,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25271 /* svtmopa_lane_za32_bf16_bf16 */, 25059 /* vIUWiq16yq8yq16UcIUWi */, 1 /* n */, 25299 /* sme,sme2,sme-tmop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25317 /* svtmopa_lane_za32_f16_f16 */, 25136 /* vIUWiq16hq8hq16UcIUWi */, 1 /* n */, 25299 /* sme,sme2,sme-tmop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25343 /* svtmopa_lane_za32_f32_f32 */, 25369 /* vIUWiq8fq4fq16UcIUWi */, 1 /* n */, 25299 /* sme,sme2,sme-tmop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25390 /* svtmopa_lane_za32_mf8_mf8_fpm */, 25217 /* vIUWiq32mq16mq16UcIUWiUWi */, 1 /* n */, 25420 /* sme,sme2,sme-tmop,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25448 /* svtmopa_lane_za32_s16_s16 */, 25474 /* vIUWiq16sq8sq16UcIUWi */, 1 /* n */, 25299 /* sme,sme2,sme-tmop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25496 /* svtmopa_lane_za32_s8_s8 */, 25520 /* vIUWiq32Scq16Scq16UcIUWi */, 1 /* n */, 25299 /* sme,sme2,sme-tmop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25545 /* svtmopa_lane_za32_s8_u8 */, 25569 /* vIUWiq32Scq16Ucq16UcIUWi */, 1 /* n */, 25299 /* sme,sme2,sme-tmop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25594 /* svtmopa_lane_za32_u16_u16 */, 25620 /* vIUWiq16Usq8Usq16UcIUWi */, 1 /* n */, 25299 /* sme,sme2,sme-tmop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25644 /* svtmopa_lane_za32_u8_s8 */, 25668 /* vIUWiq32Ucq16Scq16UcIUWi */, 1 /* n */, 25299 /* sme,sme2,sme-tmop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25693 /* svtmopa_lane_za32_u8_u8 */, 25717 /* vIUWiq32Ucq16Ucq16UcIUWi */, 1 /* n */, 25299 /* sme,sme2,sme-tmop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25742 /* svusdot_lane_za32_u8_vg1x2 */, 25769 /* vUiq32Ucq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25787 /* svusdot_lane_za32_u8_vg1x4 */, 25814 /* vUiq64Ucq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25832 /* svusdot_single_za32_u8_vg1x2 */, 25861 /* vUiq32Ucq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25875 /* svusdot_single_za32_u8_vg1x4 */, 25904 /* vUiq64Ucq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25918 /* svusdot_za32_u8_vg1x2 */, 25940 /* vUiq32Ucq32Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25954 /* svusdot_za32_u8_vg1x4 */, 25976 /* vUiq64Ucq64Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25990 /* svusmla_lane_za32_u8_vg4x1 */, 26017 /* vUiq16Ucq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26035 /* svusmla_lane_za32_u8_vg4x2 */, 25769 /* vUiq32Ucq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26062 /* svusmla_lane_za32_u8_vg4x4 */, 25814 /* vUiq64Ucq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26089 /* svusmla_single_za32_u8_vg4x2 */, 25861 /* vUiq32Ucq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26118 /* svusmla_single_za32_u8_vg4x4 */, 25904 /* vUiq64Ucq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26147 /* svusmla_za32_u8_vg4x1 */, 26169 /* vUiq16Ucq16Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26183 /* svusmla_za32_u8_vg4x2 */, 25940 /* vUiq32Ucq32Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26205 /* svusmla_za32_u8_vg4x4 */, 25976 /* vUiq64Ucq64Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26227 /* svusmopa_za32_u8_m */, 26246 /* vIUWiq16bq16bq16Ucq16Sc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26270 /* svusmopa_za64_u16_m */, 26290 /* vIUWiq16bq16bq8Usq8s */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26311 /* svusmops_za32_u8_m */, 26246 /* vIUWiq16bq16bq16Ucq16Sc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26330 /* svusmops_za64_u16_m */, 26290 /* vIUWiq16bq16bq8Usq8s */, 1 /* n */, 1386 /* sme,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26350 /* svusvdot_lane_za32_u8_vg1x4 */, 2344 /* vUiq64Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26378 /* svvdot_lane_za16_mf8_vg1x2_fpm */, 1700 /* vUiq32mq16mIUWiUWi */, 1 /* n */, 1719 /* sme,sme-f8f16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26409 /* svvdot_lane_za32_bf16_vg1x2 */, 1809 /* vUiq16yq8yIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26437 /* svvdot_lane_za32_f16_vg1x2 */, 1892 /* vUiq16hq8hIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26464 /* svvdot_lane_za32_s16_vg1x2 */, 2048 /* vUiq16sq8sIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26491 /* svvdot_lane_za32_s8_vg1x4 */, 2172 /* vUiq64Scq16ScIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26517 /* svvdot_lane_za32_u16_vg1x2 */, 2216 /* vUiq16Usq8UsIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26544 /* svvdot_lane_za32_u8_vg1x4 */, 2344 /* vUiq64Ucq16UcIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26570 /* svvdot_lane_za64_s16_vg1x4 */, 2089 /* vUiq32sq8sIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26597 /* svvdot_lane_za64_u16_vg1x4 */, 2259 /* vUiq32Usq8UsIUWi */, 1 /* n */, 271 /* sme,sme2,sme-i16i64 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26624 /* svvdotb_lane_za32_mf8_vg1x4_fpm */, 1700 /* vUiq32mq16mIUWiUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26656 /* svvdott_lane_za32_mf8_vg1x4_fpm */, 1700 /* vUiq32mq16mIUWiUWi */, 1 /* n */, 1978 /* sme,sme-f8f32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26688 /* svwrite_hor_za128_bf16_m */, 26713 /* vIUWiUiq16bq8y */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26728 /* svwrite_hor_za128_f16_m */, 26752 /* vIUWiUiq16bq8h */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26767 /* svwrite_hor_za128_f32_m */, 26791 /* vIUWiUiq16bq4f */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26806 /* svwrite_hor_za128_f64_m */, 26830 /* vIUWiUiq16bq2d */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26845 /* svwrite_hor_za128_mf8_m */, 26869 /* vIUWiUiq16bq16m */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26885 /* svwrite_hor_za128_s16_m */, 26909 /* vIUWiUiq16bq8s */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26924 /* svwrite_hor_za128_s32_m */, 26948 /* vIUWiUiq16bq4i */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26963 /* svwrite_hor_za128_s64_m */, 26987 /* vIUWiUiq16bq2Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27003 /* svwrite_hor_za128_s8_m */, 27026 /* vIUWiUiq16bq16Sc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27043 /* svwrite_hor_za128_u16_m */, 27067 /* vIUWiUiq16bq8Us */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27083 /* svwrite_hor_za128_u32_m */, 27107 /* vIUWiUiq16bq4Ui */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27123 /* svwrite_hor_za128_u64_m */, 27147 /* vIUWiUiq16bq2UWi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27164 /* svwrite_hor_za128_u8_m */, 27187 /* vIUWiUiq16bq16Uc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27204 /* svwrite_hor_za16_bf16_m */, 26713 /* vIUWiUiq16bq8y */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27228 /* svwrite_hor_za16_bf16_vg2 */, 27254 /* vIUWiUiq16y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27266 /* svwrite_hor_za16_bf16_vg4 */, 27292 /* vIUWiUiq32y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27304 /* svwrite_hor_za16_f16_m */, 26752 /* vIUWiUiq16bq8h */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27327 /* svwrite_hor_za16_f16_vg2 */, 27352 /* vIUWiUiq16h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27364 /* svwrite_hor_za16_f16_vg4 */, 27389 /* vIUWiUiq32h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27401 /* svwrite_hor_za16_s16_m */, 26909 /* vIUWiUiq16bq8s */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27424 /* svwrite_hor_za16_s16_vg2 */, 27449 /* vIUWiUiq16s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27461 /* svwrite_hor_za16_s16_vg4 */, 27486 /* vIUWiUiq32s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27498 /* svwrite_hor_za16_u16_m */, 27067 /* vIUWiUiq16bq8Us */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27521 /* svwrite_hor_za16_u16_vg2 */, 27546 /* vIUWiUiq16Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27559 /* svwrite_hor_za16_u16_vg4 */, 27584 /* vIUWiUiq32Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27597 /* svwrite_hor_za32_f32_m */, 26791 /* vIUWiUiq16bq4f */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27620 /* svwrite_hor_za32_f32_vg2 */, 27645 /* vIUWiUiq8f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27656 /* svwrite_hor_za32_f32_vg4 */, 27681 /* vIUWiUiq16f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27693 /* svwrite_hor_za32_s32_m */, 26948 /* vIUWiUiq16bq4i */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27716 /* svwrite_hor_za32_s32_vg2 */, 27741 /* vIUWiUiq8i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27752 /* svwrite_hor_za32_s32_vg4 */, 27777 /* vIUWiUiq16i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27789 /* svwrite_hor_za32_u32_m */, 27107 /* vIUWiUiq16bq4Ui */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27812 /* svwrite_hor_za32_u32_vg2 */, 27837 /* vIUWiUiq8Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27849 /* svwrite_hor_za32_u32_vg4 */, 27874 /* vIUWiUiq16Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27887 /* svwrite_hor_za64_f64_m */, 26830 /* vIUWiUiq16bq2d */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27910 /* svwrite_hor_za64_f64_vg2 */, 27935 /* vIUWiUiq4d */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27946 /* svwrite_hor_za64_f64_vg4 */, 27971 /* vIUWiUiq8d */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27982 /* svwrite_hor_za64_s64_m */, 26987 /* vIUWiUiq16bq2Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28005 /* svwrite_hor_za64_s64_vg2 */, 28030 /* vIUWiUiq4Wi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28042 /* svwrite_hor_za64_s64_vg4 */, 28067 /* vIUWiUiq8Wi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28079 /* svwrite_hor_za64_u64_m */, 27147 /* vIUWiUiq16bq2UWi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28102 /* svwrite_hor_za64_u64_vg2 */, 28127 /* vIUWiUiq4UWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28140 /* svwrite_hor_za64_u64_vg4 */, 28165 /* vIUWiUiq8UWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28178 /* svwrite_hor_za8_mf8_m */, 26869 /* vIUWiUiq16bq16m */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28200 /* svwrite_hor_za8_mf8_vg2 */, 28224 /* vIUWiUiq32m */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28236 /* svwrite_hor_za8_mf8_vg4 */, 28260 /* vIUWiUiq64m */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28272 /* svwrite_hor_za8_s8_m */, 27026 /* vIUWiUiq16bq16Sc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28293 /* svwrite_hor_za8_s8_vg2 */, 28316 /* vIUWiUiq32Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28329 /* svwrite_hor_za8_s8_vg4 */, 28352 /* vIUWiUiq64Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28365 /* svwrite_hor_za8_u8_m */, 27187 /* vIUWiUiq16bq16Uc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28386 /* svwrite_hor_za8_u8_vg2 */, 28409 /* vIUWiUiq32Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28422 /* svwrite_hor_za8_u8_vg4 */, 28445 /* vIUWiUiq64Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28458 /* svwrite_lane_zt_bf16 */, 28479 /* vIUWiq8yIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28492 /* svwrite_lane_zt_f16 */, 28512 /* vIUWiq8hIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28525 /* svwrite_lane_zt_f32 */, 28545 /* vIUWiq4fIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28558 /* svwrite_lane_zt_f64 */, 28578 /* vIUWiq2dIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28591 /* svwrite_lane_zt_s16 */, 28611 /* vIUWiq8sIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28624 /* svwrite_lane_zt_s32 */, 28644 /* vIUWiq4iIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28657 /* svwrite_lane_zt_s64 */, 28677 /* vIUWiq2WiIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28691 /* svwrite_lane_zt_s8 */, 28710 /* vIUWiq16ScIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28725 /* svwrite_lane_zt_u16 */, 28745 /* vIUWiq8UsIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28759 /* svwrite_lane_zt_u32 */, 28779 /* vIUWiq4UiIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28793 /* svwrite_lane_zt_u64 */, 28813 /* vIUWiq2UWiIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28828 /* svwrite_lane_zt_u8 */, 28847 /* vIUWiq16UcIUWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28862 /* svwrite_ver_za128_bf16_m */, 26713 /* vIUWiUiq16bq8y */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28887 /* svwrite_ver_za128_f16_m */, 26752 /* vIUWiUiq16bq8h */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28911 /* svwrite_ver_za128_f32_m */, 26791 /* vIUWiUiq16bq4f */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28935 /* svwrite_ver_za128_f64_m */, 26830 /* vIUWiUiq16bq2d */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28959 /* svwrite_ver_za128_mf8_m */, 26869 /* vIUWiUiq16bq16m */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28983 /* svwrite_ver_za128_s16_m */, 26909 /* vIUWiUiq16bq8s */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29007 /* svwrite_ver_za128_s32_m */, 26948 /* vIUWiUiq16bq4i */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29031 /* svwrite_ver_za128_s64_m */, 26987 /* vIUWiUiq16bq2Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29055 /* svwrite_ver_za128_s8_m */, 27026 /* vIUWiUiq16bq16Sc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29078 /* svwrite_ver_za128_u16_m */, 27067 /* vIUWiUiq16bq8Us */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29102 /* svwrite_ver_za128_u32_m */, 27107 /* vIUWiUiq16bq4Ui */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29126 /* svwrite_ver_za128_u64_m */, 27147 /* vIUWiUiq16bq2UWi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29150 /* svwrite_ver_za128_u8_m */, 27187 /* vIUWiUiq16bq16Uc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29173 /* svwrite_ver_za16_bf16_m */, 26713 /* vIUWiUiq16bq8y */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29197 /* svwrite_ver_za16_bf16_vg2 */, 27254 /* vIUWiUiq16y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29223 /* svwrite_ver_za16_bf16_vg4 */, 27292 /* vIUWiUiq32y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29249 /* svwrite_ver_za16_f16_m */, 26752 /* vIUWiUiq16bq8h */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29272 /* svwrite_ver_za16_f16_vg2 */, 27352 /* vIUWiUiq16h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29297 /* svwrite_ver_za16_f16_vg4 */, 27389 /* vIUWiUiq32h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29322 /* svwrite_ver_za16_s16_m */, 26909 /* vIUWiUiq16bq8s */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29345 /* svwrite_ver_za16_s16_vg2 */, 27449 /* vIUWiUiq16s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29370 /* svwrite_ver_za16_s16_vg4 */, 27486 /* vIUWiUiq32s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29395 /* svwrite_ver_za16_u16_m */, 27067 /* vIUWiUiq16bq8Us */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29418 /* svwrite_ver_za16_u16_vg2 */, 27546 /* vIUWiUiq16Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29443 /* svwrite_ver_za16_u16_vg4 */, 27584 /* vIUWiUiq32Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29468 /* svwrite_ver_za32_f32_m */, 26791 /* vIUWiUiq16bq4f */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29491 /* svwrite_ver_za32_f32_vg2 */, 27645 /* vIUWiUiq8f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29516 /* svwrite_ver_za32_f32_vg4 */, 27681 /* vIUWiUiq16f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29541 /* svwrite_ver_za32_s32_m */, 26948 /* vIUWiUiq16bq4i */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29564 /* svwrite_ver_za32_s32_vg2 */, 27741 /* vIUWiUiq8i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29589 /* svwrite_ver_za32_s32_vg4 */, 27777 /* vIUWiUiq16i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29614 /* svwrite_ver_za32_u32_m */, 27107 /* vIUWiUiq16bq4Ui */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29637 /* svwrite_ver_za32_u32_vg2 */, 27837 /* vIUWiUiq8Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29662 /* svwrite_ver_za32_u32_vg4 */, 27874 /* vIUWiUiq16Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29687 /* svwrite_ver_za64_f64_m */, 26830 /* vIUWiUiq16bq2d */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29710 /* svwrite_ver_za64_f64_vg2 */, 27935 /* vIUWiUiq4d */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29735 /* svwrite_ver_za64_f64_vg4 */, 27971 /* vIUWiUiq8d */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29760 /* svwrite_ver_za64_s64_m */, 26987 /* vIUWiUiq16bq2Wi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29783 /* svwrite_ver_za64_s64_vg2 */, 28030 /* vIUWiUiq4Wi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29808 /* svwrite_ver_za64_s64_vg4 */, 28067 /* vIUWiUiq8Wi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29833 /* svwrite_ver_za64_u64_m */, 27147 /* vIUWiUiq16bq2UWi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29856 /* svwrite_ver_za64_u64_vg2 */, 28127 /* vIUWiUiq4UWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29881 /* svwrite_ver_za64_u64_vg4 */, 28165 /* vIUWiUiq8UWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29906 /* svwrite_ver_za8_mf8_m */, 26869 /* vIUWiUiq16bq16m */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29928 /* svwrite_ver_za8_mf8_vg2 */, 28224 /* vIUWiUiq32m */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29952 /* svwrite_ver_za8_mf8_vg4 */, 28260 /* vIUWiUiq64m */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29976 /* svwrite_ver_za8_s8_m */, 27026 /* vIUWiUiq16bq16Sc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29997 /* svwrite_ver_za8_s8_vg2 */, 28316 /* vIUWiUiq32Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30020 /* svwrite_ver_za8_s8_vg4 */, 28352 /* vIUWiUiq64Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30043 /* svwrite_ver_za8_u8_m */, 27187 /* vIUWiUiq16bq16Uc */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30064 /* svwrite_ver_za8_u8_vg2 */, 28409 /* vIUWiUiq32Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30087 /* svwrite_ver_za8_u8_vg4 */, 28445 /* vIUWiUiq64Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30110 /* svwrite_za16_bf16_vg1x2 */, 771 /* vUiq16y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30134 /* svwrite_za16_bf16_vg1x4 */, 816 /* vUiq32y */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30158 /* svwrite_za16_f16_vg1x2 */, 845 /* vUiq16h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30181 /* svwrite_za16_f16_vg1x4 */, 901 /* vUiq32h */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30204 /* svwrite_za16_s16_vg1x2 */, 30227 /* vUiq16s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30235 /* svwrite_za16_s16_vg1x4 */, 30258 /* vUiq32s */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30266 /* svwrite_za16_u16_vg1x2 */, 30289 /* vUiq16Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30298 /* svwrite_za16_u16_vg1x4 */, 30321 /* vUiq32Us */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30330 /* svwrite_za32_f32_vg1x2 */, 930 /* vUiq8f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30353 /* svwrite_za32_f32_vg1x4 */, 958 /* vUiq16f */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30376 /* svwrite_za32_s32_vg1x2 */, 987 /* vUiq8i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30399 /* svwrite_za32_s32_vg1x4 */, 1015 /* vUiq16i */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30422 /* svwrite_za32_u32_vg1x2 */, 1044 /* vUiq8Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30445 /* svwrite_za32_u32_vg1x4 */, 1073 /* vUiq16Ui */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30468 /* svwrite_za64_f64_vg1x2 */, 1103 /* vUiq4d */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30491 /* svwrite_za64_f64_vg1x4 */, 1151 /* vUiq8d */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30514 /* svwrite_za64_s64_vg1x2 */, 1179 /* vUiq4Wi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30537 /* svwrite_za64_s64_vg1x4 */, 1208 /* vUiq8Wi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30560 /* svwrite_za64_u64_vg1x2 */, 1237 /* vUiq4UWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30583 /* svwrite_za64_u64_vg1x4 */, 1267 /* vUiq8UWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30606 /* svwrite_za8_mf8_vg1x2 */, 30628 /* vUiq32m */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30636 /* svwrite_za8_mf8_vg1x4 */, 30658 /* vUiq64m */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30666 /* svwrite_za8_s8_vg1x2 */, 30687 /* vUiq32Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30696 /* svwrite_za8_s8_vg1x4 */, 30717 /* vUiq64Sc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30726 /* svwrite_za8_u8_vg1x2 */, 30747 /* vUiq32Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30756 /* svwrite_za8_u8_vg1x4 */, 30777 /* vUiq64Uc */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30786 /* svwrite_zt_bf16 */, 30802 /* vIUWiq8y */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30811 /* svwrite_zt_f16 */, 30826 /* vIUWiq8h */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30835 /* svwrite_zt_f32 */, 30850 /* vIUWiq4f */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30859 /* svwrite_zt_f64 */, 30874 /* vIUWiq2d */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30883 /* svwrite_zt_s16 */, 30898 /* vIUWiq8s */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30907 /* svwrite_zt_s32 */, 30922 /* vIUWiq4i */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30931 /* svwrite_zt_s64 */, 30946 /* vIUWiq2Wi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30956 /* svwrite_zt_s8 */, 30970 /* vIUWiq16Sc */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30981 /* svwrite_zt_u16 */, 30996 /* vIUWiq8Us */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31006 /* svwrite_zt_u32 */, 31021 /* vIUWiq4Ui */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31031 /* svwrite_zt_u64 */, 31046 /* vIUWiq2UWi */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31057 /* svwrite_zt_u8 */, 31071 /* vIUWiq16Uc */, 1 /* n */, 6098 /* sme,sme-lutv2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31082 /* svzero_mask_za */, 31097 /* vIUWi */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31103 /* svzero_za */, 31113 /* vv */, 1 /* n */, 30 /* sme */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31116 /* svzero_za64_vg1x2 */, 31134 /* vUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31138 /* svzero_za64_vg1x4 */, 31134 /* vUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31156 /* svzero_za64_vg2x1 */, 31134 /* vUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31174 /* svzero_za64_vg2x2 */, 31134 /* vUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31192 /* svzero_za64_vg2x4 */, 31134 /* vUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31210 /* svzero_za64_vg4x1 */, 31134 /* vUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31228 /* svzero_za64_vg4x2 */, 31134 /* vUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31246 /* svzero_za64_vg4x4 */, 31134 /* vUi */, 1 /* n */, 20035 /* sme,sme2p1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31264 /* svzero_zt */, 31097 /* vIUWi */, 1 /* n */, 78 /* sme,sme2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
#endif // GET_SME_BUILTIN_INFOS

