/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* List of builtins that Clang recognizes                                     *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/


#ifdef GET_BUILTIN_ENUMERATORS
    BI_AddressOfReturnAddress,
    BI_BitScanForward,
    BI_BitScanReverse,
    BI_InterlockedAnd64,
    BI_InterlockedDecrement64,
    BI_InterlockedExchange64,
    BI_InterlockedExchangeAdd64,
    BI_InterlockedExchangeSub64,
    BI_InterlockedIncrement64,
    BI_InterlockedOr64,
    BI_InterlockedXor64,
    BI_ReadBarrier,
    BI_ReadWriteBarrier,
    BI_WriteBarrier,
    BI__cpuid,
    BI__cpuidex,
    BI__emul,
    BI__emulu,
    BI__int2c,
    BI__rdtsc,
    BI__readfsbyte,
    BI__readfsdword,
    BI__readfsqword,
    BI__readfsword,
    BI__readgsbyte,
    BI__readgsdword,
    BI__readgsqword,
    BI__readgsword,
    BI__stosb,
    BI__ud2,
    BI_m_prefetch,
    BI_m_prefetchw,
    BI_mm_clflush,
    BI_mm_getcsr,
    BI_mm_lfence,
    BI_mm_mfence,
    BI_mm_pause,
    BI_mm_prefetch,
    BI_mm_setcsr,
    BI_mm_sfence,
    BI_xgetbv,
    BI_xsetbv,
    BI__builtin_ia32_aadd32,
    BI__builtin_ia32_aand32,
    BI__builtin_ia32_addcarryx_u32,
    BI__builtin_ia32_addpd512,
    BI__builtin_ia32_addph512,
    BI__builtin_ia32_addps512,
    BI__builtin_ia32_addsd_round_mask,
    BI__builtin_ia32_addsh_round_mask,
    BI__builtin_ia32_addss_round_mask,
    BI__builtin_ia32_addsubpd,
    BI__builtin_ia32_addsubpd256,
    BI__builtin_ia32_addsubps,
    BI__builtin_ia32_addsubps256,
    BI__builtin_ia32_aesdec128,
    BI__builtin_ia32_aesdec128kl_u8,
    BI__builtin_ia32_aesdec256,
    BI__builtin_ia32_aesdec256kl_u8,
    BI__builtin_ia32_aesdec512,
    BI__builtin_ia32_aesdeclast128,
    BI__builtin_ia32_aesdeclast256,
    BI__builtin_ia32_aesdeclast512,
    BI__builtin_ia32_aesdecwide128kl_u8,
    BI__builtin_ia32_aesdecwide256kl_u8,
    BI__builtin_ia32_aesenc128,
    BI__builtin_ia32_aesenc128kl_u8,
    BI__builtin_ia32_aesenc256,
    BI__builtin_ia32_aesenc256kl_u8,
    BI__builtin_ia32_aesenc512,
    BI__builtin_ia32_aesenclast128,
    BI__builtin_ia32_aesenclast256,
    BI__builtin_ia32_aesenclast512,
    BI__builtin_ia32_aesencwide128kl_u8,
    BI__builtin_ia32_aesencwide256kl_u8,
    BI__builtin_ia32_aesimc128,
    BI__builtin_ia32_aeskeygenassist128,
    BI__builtin_ia32_alignd128,
    BI__builtin_ia32_alignd256,
    BI__builtin_ia32_alignd512,
    BI__builtin_ia32_alignq128,
    BI__builtin_ia32_alignq256,
    BI__builtin_ia32_alignq512,
    BI__builtin_ia32_aor32,
    BI__builtin_ia32_axor32,
    BI__builtin_ia32_bextr_u32,
    BI__builtin_ia32_bextri_u32,
    BI__builtin_ia32_blendpd,
    BI__builtin_ia32_blendpd256,
    BI__builtin_ia32_blendps,
    BI__builtin_ia32_blendps256,
    BI__builtin_ia32_blendvpd,
    BI__builtin_ia32_blendvpd256,
    BI__builtin_ia32_blendvps,
    BI__builtin_ia32_blendvps256,
    BI__builtin_ia32_bzhi_si,
    BI__builtin_ia32_cldemote,
    BI__builtin_ia32_clflush,
    BI__builtin_ia32_clflushopt,
    BI__builtin_ia32_clrssbsy,
    BI__builtin_ia32_clwb,
    BI__builtin_ia32_clzero,
    BI__builtin_ia32_cmpb128_mask,
    BI__builtin_ia32_cmpb256_mask,
    BI__builtin_ia32_cmpb512_mask,
    BI__builtin_ia32_cmpd128_mask,
    BI__builtin_ia32_cmpd256_mask,
    BI__builtin_ia32_cmpd512_mask,
    BI__builtin_ia32_cmpeqpd,
    BI__builtin_ia32_cmpeqps,
    BI__builtin_ia32_cmpeqsd,
    BI__builtin_ia32_cmpeqss,
    BI__builtin_ia32_cmplepd,
    BI__builtin_ia32_cmpleps,
    BI__builtin_ia32_cmplesd,
    BI__builtin_ia32_cmpless,
    BI__builtin_ia32_cmpltpd,
    BI__builtin_ia32_cmpltps,
    BI__builtin_ia32_cmpltsd,
    BI__builtin_ia32_cmpltss,
    BI__builtin_ia32_cmpneqpd,
    BI__builtin_ia32_cmpneqps,
    BI__builtin_ia32_cmpneqsd,
    BI__builtin_ia32_cmpneqss,
    BI__builtin_ia32_cmpnlepd,
    BI__builtin_ia32_cmpnleps,
    BI__builtin_ia32_cmpnlesd,
    BI__builtin_ia32_cmpnless,
    BI__builtin_ia32_cmpnltpd,
    BI__builtin_ia32_cmpnltps,
    BI__builtin_ia32_cmpnltsd,
    BI__builtin_ia32_cmpnltss,
    BI__builtin_ia32_cmpordpd,
    BI__builtin_ia32_cmpordps,
    BI__builtin_ia32_cmpordsd,
    BI__builtin_ia32_cmpordss,
    BI__builtin_ia32_cmppd,
    BI__builtin_ia32_cmppd128_mask,
    BI__builtin_ia32_cmppd256,
    BI__builtin_ia32_cmppd256_mask,
    BI__builtin_ia32_cmppd512_mask,
    BI__builtin_ia32_cmpph128_mask,
    BI__builtin_ia32_cmpph256_mask,
    BI__builtin_ia32_cmpph512_mask,
    BI__builtin_ia32_cmpps,
    BI__builtin_ia32_cmpps128_mask,
    BI__builtin_ia32_cmpps256,
    BI__builtin_ia32_cmpps256_mask,
    BI__builtin_ia32_cmpps512_mask,
    BI__builtin_ia32_cmpq128_mask,
    BI__builtin_ia32_cmpq256_mask,
    BI__builtin_ia32_cmpq512_mask,
    BI__builtin_ia32_cmpsd,
    BI__builtin_ia32_cmpsd_mask,
    BI__builtin_ia32_cmpsh_mask,
    BI__builtin_ia32_cmpss,
    BI__builtin_ia32_cmpss_mask,
    BI__builtin_ia32_cmpunordpd,
    BI__builtin_ia32_cmpunordps,
    BI__builtin_ia32_cmpunordsd,
    BI__builtin_ia32_cmpunordss,
    BI__builtin_ia32_cmpw128_mask,
    BI__builtin_ia32_cmpw256_mask,
    BI__builtin_ia32_cmpw512_mask,
    BI__builtin_ia32_comieq,
    BI__builtin_ia32_comige,
    BI__builtin_ia32_comigt,
    BI__builtin_ia32_comile,
    BI__builtin_ia32_comilt,
    BI__builtin_ia32_comineq,
    BI__builtin_ia32_comisdeq,
    BI__builtin_ia32_comisdge,
    BI__builtin_ia32_comisdgt,
    BI__builtin_ia32_comisdle,
    BI__builtin_ia32_comisdlt,
    BI__builtin_ia32_comisdneq,
    BI__builtin_ia32_compressdf128_mask,
    BI__builtin_ia32_compressdf256_mask,
    BI__builtin_ia32_compressdf512_mask,
    BI__builtin_ia32_compressdi128_mask,
    BI__builtin_ia32_compressdi256_mask,
    BI__builtin_ia32_compressdi512_mask,
    BI__builtin_ia32_compresshi128_mask,
    BI__builtin_ia32_compresshi256_mask,
    BI__builtin_ia32_compresshi512_mask,
    BI__builtin_ia32_compressqi128_mask,
    BI__builtin_ia32_compressqi256_mask,
    BI__builtin_ia32_compressqi512_mask,
    BI__builtin_ia32_compresssf128_mask,
    BI__builtin_ia32_compresssf256_mask,
    BI__builtin_ia32_compresssf512_mask,
    BI__builtin_ia32_compresssi128_mask,
    BI__builtin_ia32_compresssi256_mask,
    BI__builtin_ia32_compresssi512_mask,
    BI__builtin_ia32_compressstoredf128_mask,
    BI__builtin_ia32_compressstoredf256_mask,
    BI__builtin_ia32_compressstoredf512_mask,
    BI__builtin_ia32_compressstoredi128_mask,
    BI__builtin_ia32_compressstoredi256_mask,
    BI__builtin_ia32_compressstoredi512_mask,
    BI__builtin_ia32_compressstorehi128_mask,
    BI__builtin_ia32_compressstorehi256_mask,
    BI__builtin_ia32_compressstorehi512_mask,
    BI__builtin_ia32_compressstoreqi128_mask,
    BI__builtin_ia32_compressstoreqi256_mask,
    BI__builtin_ia32_compressstoreqi512_mask,
    BI__builtin_ia32_compressstoresf128_mask,
    BI__builtin_ia32_compressstoresf256_mask,
    BI__builtin_ia32_compressstoresf512_mask,
    BI__builtin_ia32_compressstoresi128_mask,
    BI__builtin_ia32_compressstoresi256_mask,
    BI__builtin_ia32_compressstoresi512_mask,
    BI__builtin_ia32_crc32hi,
    BI__builtin_ia32_crc32qi,
    BI__builtin_ia32_crc32si,
    BI__builtin_ia32_cvtb2mask128,
    BI__builtin_ia32_cvtb2mask256,
    BI__builtin_ia32_cvtb2mask512,
    BI__builtin_ia32_cvtd2mask128,
    BI__builtin_ia32_cvtd2mask256,
    BI__builtin_ia32_cvtd2mask512,
    BI__builtin_ia32_cvtdq2ps512_mask,
    BI__builtin_ia32_cvtmask2b128,
    BI__builtin_ia32_cvtmask2b256,
    BI__builtin_ia32_cvtmask2b512,
    BI__builtin_ia32_cvtmask2d128,
    BI__builtin_ia32_cvtmask2d256,
    BI__builtin_ia32_cvtmask2d512,
    BI__builtin_ia32_cvtmask2q128,
    BI__builtin_ia32_cvtmask2q256,
    BI__builtin_ia32_cvtmask2q512,
    BI__builtin_ia32_cvtmask2w128,
    BI__builtin_ia32_cvtmask2w256,
    BI__builtin_ia32_cvtmask2w512,
    BI__builtin_ia32_cvtne2ps2bf16_128,
    BI__builtin_ia32_cvtne2ps2bf16_256,
    BI__builtin_ia32_cvtne2ps2bf16_512,
    BI__builtin_ia32_cvtneps2bf16_128_mask,
    BI__builtin_ia32_cvtneps2bf16_256_mask,
    BI__builtin_ia32_cvtneps2bf16_512_mask,
    BI__builtin_ia32_cvtpd2dq,
    BI__builtin_ia32_cvtpd2dq128_mask,
    BI__builtin_ia32_cvtpd2dq256,
    BI__builtin_ia32_cvtpd2dq512_mask,
    BI__builtin_ia32_cvtpd2ps,
    BI__builtin_ia32_cvtpd2ps256,
    BI__builtin_ia32_cvtpd2ps512_mask,
    BI__builtin_ia32_cvtpd2ps_mask,
    BI__builtin_ia32_cvtpd2qq128_mask,
    BI__builtin_ia32_cvtpd2qq256_mask,
    BI__builtin_ia32_cvtpd2qq512_mask,
    BI__builtin_ia32_cvtpd2udq128_mask,
    BI__builtin_ia32_cvtpd2udq256_mask,
    BI__builtin_ia32_cvtpd2udq512_mask,
    BI__builtin_ia32_cvtpd2uqq128_mask,
    BI__builtin_ia32_cvtpd2uqq256_mask,
    BI__builtin_ia32_cvtpd2uqq512_mask,
    BI__builtin_ia32_cvtps2dq,
    BI__builtin_ia32_cvtps2dq256,
    BI__builtin_ia32_cvtps2dq512_mask,
    BI__builtin_ia32_cvtps2pd512_mask,
    BI__builtin_ia32_cvtps2qq128_mask,
    BI__builtin_ia32_cvtps2qq256_mask,
    BI__builtin_ia32_cvtps2qq512_mask,
    BI__builtin_ia32_cvtps2udq128_mask,
    BI__builtin_ia32_cvtps2udq256_mask,
    BI__builtin_ia32_cvtps2udq512_mask,
    BI__builtin_ia32_cvtps2uqq128_mask,
    BI__builtin_ia32_cvtps2uqq256_mask,
    BI__builtin_ia32_cvtps2uqq512_mask,
    BI__builtin_ia32_cvtq2mask128,
    BI__builtin_ia32_cvtq2mask256,
    BI__builtin_ia32_cvtq2mask512,
    BI__builtin_ia32_cvtqq2pd512_mask,
    BI__builtin_ia32_cvtqq2ps128_mask,
    BI__builtin_ia32_cvtqq2ps512_mask,
    BI__builtin_ia32_cvtsd2si,
    BI__builtin_ia32_cvtsd2ss,
    BI__builtin_ia32_cvtsd2ss_round_mask,
    BI__builtin_ia32_cvtsi2ss32,
    BI__builtin_ia32_cvtss2sd_round_mask,
    BI__builtin_ia32_cvtss2si,
    BI__builtin_ia32_cvttpd2dq,
    BI__builtin_ia32_cvttpd2dq128_mask,
    BI__builtin_ia32_cvttpd2dq256,
    BI__builtin_ia32_cvttpd2dq512_mask,
    BI__builtin_ia32_cvttpd2qq128_mask,
    BI__builtin_ia32_cvttpd2qq256_mask,
    BI__builtin_ia32_cvttpd2qq512_mask,
    BI__builtin_ia32_cvttpd2udq128_mask,
    BI__builtin_ia32_cvttpd2udq256_mask,
    BI__builtin_ia32_cvttpd2udq512_mask,
    BI__builtin_ia32_cvttpd2uqq128_mask,
    BI__builtin_ia32_cvttpd2uqq256_mask,
    BI__builtin_ia32_cvttpd2uqq512_mask,
    BI__builtin_ia32_cvttps2dq,
    BI__builtin_ia32_cvttps2dq256,
    BI__builtin_ia32_cvttps2dq512_mask,
    BI__builtin_ia32_cvttps2qq128_mask,
    BI__builtin_ia32_cvttps2qq256_mask,
    BI__builtin_ia32_cvttps2qq512_mask,
    BI__builtin_ia32_cvttps2udq128_mask,
    BI__builtin_ia32_cvttps2udq256_mask,
    BI__builtin_ia32_cvttps2udq512_mask,
    BI__builtin_ia32_cvttps2uqq128_mask,
    BI__builtin_ia32_cvttps2uqq256_mask,
    BI__builtin_ia32_cvttps2uqq512_mask,
    BI__builtin_ia32_cvttsd2si,
    BI__builtin_ia32_cvttss2si,
    BI__builtin_ia32_cvtudq2ps512_mask,
    BI__builtin_ia32_cvtuqq2pd512_mask,
    BI__builtin_ia32_cvtuqq2ps128_mask,
    BI__builtin_ia32_cvtuqq2ps512_mask,
    BI__builtin_ia32_cvtusi2ss32,
    BI__builtin_ia32_cvtw2mask128,
    BI__builtin_ia32_cvtw2mask256,
    BI__builtin_ia32_cvtw2mask512,
    BI__builtin_ia32_dbpsadbw128,
    BI__builtin_ia32_dbpsadbw256,
    BI__builtin_ia32_dbpsadbw512,
    BI__builtin_ia32_directstore_u32,
    BI__builtin_ia32_divpd512,
    BI__builtin_ia32_divph512,
    BI__builtin_ia32_divps512,
    BI__builtin_ia32_divsd_round_mask,
    BI__builtin_ia32_divsh_round_mask,
    BI__builtin_ia32_divss_round_mask,
    BI__builtin_ia32_dpbf16ps_128,
    BI__builtin_ia32_dpbf16ps_256,
    BI__builtin_ia32_dpbf16ps_512,
    BI__builtin_ia32_dppd,
    BI__builtin_ia32_dpps,
    BI__builtin_ia32_dpps256,
    BI__builtin_ia32_emms,
    BI__builtin_ia32_encodekey128_u32,
    BI__builtin_ia32_encodekey256_u32,
    BI__builtin_ia32_enqcmd,
    BI__builtin_ia32_enqcmds,
    BI__builtin_ia32_expanddf128_mask,
    BI__builtin_ia32_expanddf256_mask,
    BI__builtin_ia32_expanddf512_mask,
    BI__builtin_ia32_expanddi128_mask,
    BI__builtin_ia32_expanddi256_mask,
    BI__builtin_ia32_expanddi512_mask,
    BI__builtin_ia32_expandhi128_mask,
    BI__builtin_ia32_expandhi256_mask,
    BI__builtin_ia32_expandhi512_mask,
    BI__builtin_ia32_expandloaddf128_mask,
    BI__builtin_ia32_expandloaddf256_mask,
    BI__builtin_ia32_expandloaddf512_mask,
    BI__builtin_ia32_expandloaddi128_mask,
    BI__builtin_ia32_expandloaddi256_mask,
    BI__builtin_ia32_expandloaddi512_mask,
    BI__builtin_ia32_expandloadhi128_mask,
    BI__builtin_ia32_expandloadhi256_mask,
    BI__builtin_ia32_expandloadhi512_mask,
    BI__builtin_ia32_expandloadqi128_mask,
    BI__builtin_ia32_expandloadqi256_mask,
    BI__builtin_ia32_expandloadqi512_mask,
    BI__builtin_ia32_expandloadsf128_mask,
    BI__builtin_ia32_expandloadsf256_mask,
    BI__builtin_ia32_expandloadsf512_mask,
    BI__builtin_ia32_expandloadsi128_mask,
    BI__builtin_ia32_expandloadsi256_mask,
    BI__builtin_ia32_expandloadsi512_mask,
    BI__builtin_ia32_expandqi128_mask,
    BI__builtin_ia32_expandqi256_mask,
    BI__builtin_ia32_expandqi512_mask,
    BI__builtin_ia32_expandsf128_mask,
    BI__builtin_ia32_expandsf256_mask,
    BI__builtin_ia32_expandsf512_mask,
    BI__builtin_ia32_expandsi128_mask,
    BI__builtin_ia32_expandsi256_mask,
    BI__builtin_ia32_expandsi512_mask,
    BI__builtin_ia32_extract128i256,
    BI__builtin_ia32_extractf32x4_256_mask,
    BI__builtin_ia32_extractf32x4_mask,
    BI__builtin_ia32_extractf32x8_mask,
    BI__builtin_ia32_extractf64x2_256_mask,
    BI__builtin_ia32_extractf64x2_512_mask,
    BI__builtin_ia32_extractf64x4_mask,
    BI__builtin_ia32_extracti32x4_256_mask,
    BI__builtin_ia32_extracti32x4_mask,
    BI__builtin_ia32_extracti32x8_mask,
    BI__builtin_ia32_extracti64x2_256_mask,
    BI__builtin_ia32_extracti64x2_512_mask,
    BI__builtin_ia32_extracti64x4_mask,
    BI__builtin_ia32_extrq,
    BI__builtin_ia32_extrqi,
    BI__builtin_ia32_fixupimmpd128_mask,
    BI__builtin_ia32_fixupimmpd128_maskz,
    BI__builtin_ia32_fixupimmpd256_mask,
    BI__builtin_ia32_fixupimmpd256_maskz,
    BI__builtin_ia32_fixupimmpd512_mask,
    BI__builtin_ia32_fixupimmpd512_maskz,
    BI__builtin_ia32_fixupimmps128_mask,
    BI__builtin_ia32_fixupimmps128_maskz,
    BI__builtin_ia32_fixupimmps256_mask,
    BI__builtin_ia32_fixupimmps256_maskz,
    BI__builtin_ia32_fixupimmps512_mask,
    BI__builtin_ia32_fixupimmps512_maskz,
    BI__builtin_ia32_fixupimmsd_mask,
    BI__builtin_ia32_fixupimmsd_maskz,
    BI__builtin_ia32_fixupimmss_mask,
    BI__builtin_ia32_fixupimmss_maskz,
    BI__builtin_ia32_fpclasspd128_mask,
    BI__builtin_ia32_fpclasspd256_mask,
    BI__builtin_ia32_fpclasspd512_mask,
    BI__builtin_ia32_fpclassph128_mask,
    BI__builtin_ia32_fpclassph256_mask,
    BI__builtin_ia32_fpclassph512_mask,
    BI__builtin_ia32_fpclassps128_mask,
    BI__builtin_ia32_fpclassps256_mask,
    BI__builtin_ia32_fpclassps512_mask,
    BI__builtin_ia32_fpclasssd_mask,
    BI__builtin_ia32_fpclasssh_mask,
    BI__builtin_ia32_fpclassss_mask,
    BI__builtin_ia32_fxrstor,
    BI__builtin_ia32_fxsave,
    BI__builtin_ia32_gather3div2df,
    BI__builtin_ia32_gather3div2di,
    BI__builtin_ia32_gather3div4df,
    BI__builtin_ia32_gather3div4di,
    BI__builtin_ia32_gather3div4sf,
    BI__builtin_ia32_gather3div4si,
    BI__builtin_ia32_gather3div8sf,
    BI__builtin_ia32_gather3div8si,
    BI__builtin_ia32_gather3siv2df,
    BI__builtin_ia32_gather3siv2di,
    BI__builtin_ia32_gather3siv4df,
    BI__builtin_ia32_gather3siv4di,
    BI__builtin_ia32_gather3siv4sf,
    BI__builtin_ia32_gather3siv4si,
    BI__builtin_ia32_gather3siv8sf,
    BI__builtin_ia32_gather3siv8si,
    BI__builtin_ia32_gatherd_d,
    BI__builtin_ia32_gatherd_d256,
    BI__builtin_ia32_gatherd_pd,
    BI__builtin_ia32_gatherd_pd256,
    BI__builtin_ia32_gatherd_ps,
    BI__builtin_ia32_gatherd_ps256,
    BI__builtin_ia32_gatherd_q,
    BI__builtin_ia32_gatherd_q256,
    BI__builtin_ia32_gatherdiv8df,
    BI__builtin_ia32_gatherdiv8di,
    BI__builtin_ia32_gatherdiv16sf,
    BI__builtin_ia32_gatherdiv16si,
    BI__builtin_ia32_gatherq_d,
    BI__builtin_ia32_gatherq_d256,
    BI__builtin_ia32_gatherq_pd,
    BI__builtin_ia32_gatherq_pd256,
    BI__builtin_ia32_gatherq_ps,
    BI__builtin_ia32_gatherq_ps256,
    BI__builtin_ia32_gatherq_q,
    BI__builtin_ia32_gatherq_q256,
    BI__builtin_ia32_gathersiv8df,
    BI__builtin_ia32_gathersiv8di,
    BI__builtin_ia32_gathersiv16sf,
    BI__builtin_ia32_gathersiv16si,
    BI__builtin_ia32_getexppd128_mask,
    BI__builtin_ia32_getexppd256_mask,
    BI__builtin_ia32_getexppd512_mask,
    BI__builtin_ia32_getexpph128_mask,
    BI__builtin_ia32_getexpph256_mask,
    BI__builtin_ia32_getexpph512_mask,
    BI__builtin_ia32_getexpps128_mask,
    BI__builtin_ia32_getexpps256_mask,
    BI__builtin_ia32_getexpps512_mask,
    BI__builtin_ia32_getexpsd128_round_mask,
    BI__builtin_ia32_getexpsh128_round_mask,
    BI__builtin_ia32_getexpss128_round_mask,
    BI__builtin_ia32_getmantpd128_mask,
    BI__builtin_ia32_getmantpd256_mask,
    BI__builtin_ia32_getmantpd512_mask,
    BI__builtin_ia32_getmantph128_mask,
    BI__builtin_ia32_getmantph256_mask,
    BI__builtin_ia32_getmantph512_mask,
    BI__builtin_ia32_getmantps128_mask,
    BI__builtin_ia32_getmantps256_mask,
    BI__builtin_ia32_getmantps512_mask,
    BI__builtin_ia32_getmantsd_round_mask,
    BI__builtin_ia32_getmantsh_round_mask,
    BI__builtin_ia32_getmantss_round_mask,
    BI__builtin_ia32_haddpd,
    BI__builtin_ia32_haddpd256,
    BI__builtin_ia32_haddps,
    BI__builtin_ia32_haddps256,
    BI__builtin_ia32_hsubpd,
    BI__builtin_ia32_hsubpd256,
    BI__builtin_ia32_hsubps,
    BI__builtin_ia32_hsubps256,
    BI__builtin_ia32_incsspd,
    BI__builtin_ia32_insert128i256,
    BI__builtin_ia32_insertf32x4,
    BI__builtin_ia32_insertf32x4_256,
    BI__builtin_ia32_insertf32x8,
    BI__builtin_ia32_insertf64x2_256,
    BI__builtin_ia32_insertf64x2_512,
    BI__builtin_ia32_insertf64x4,
    BI__builtin_ia32_inserti32x4,
    BI__builtin_ia32_inserti32x4_256,
    BI__builtin_ia32_inserti32x8,
    BI__builtin_ia32_inserti64x2_256,
    BI__builtin_ia32_inserti64x2_512,
    BI__builtin_ia32_inserti64x4,
    BI__builtin_ia32_insertps128,
    BI__builtin_ia32_insertq,
    BI__builtin_ia32_insertqi,
    BI__builtin_ia32_invpcid,
    BI__builtin_ia32_kadddi,
    BI__builtin_ia32_kaddhi,
    BI__builtin_ia32_kaddqi,
    BI__builtin_ia32_kaddsi,
    BI__builtin_ia32_kanddi,
    BI__builtin_ia32_kandhi,
    BI__builtin_ia32_kandndi,
    BI__builtin_ia32_kandnhi,
    BI__builtin_ia32_kandnqi,
    BI__builtin_ia32_kandnsi,
    BI__builtin_ia32_kandqi,
    BI__builtin_ia32_kandsi,
    BI__builtin_ia32_kmovb,
    BI__builtin_ia32_kmovd,
    BI__builtin_ia32_kmovq,
    BI__builtin_ia32_kmovw,
    BI__builtin_ia32_knotdi,
    BI__builtin_ia32_knothi,
    BI__builtin_ia32_knotqi,
    BI__builtin_ia32_knotsi,
    BI__builtin_ia32_kordi,
    BI__builtin_ia32_korhi,
    BI__builtin_ia32_korqi,
    BI__builtin_ia32_korsi,
    BI__builtin_ia32_kortestcdi,
    BI__builtin_ia32_kortestchi,
    BI__builtin_ia32_kortestcqi,
    BI__builtin_ia32_kortestcsi,
    BI__builtin_ia32_kortestzdi,
    BI__builtin_ia32_kortestzhi,
    BI__builtin_ia32_kortestzqi,
    BI__builtin_ia32_kortestzsi,
    BI__builtin_ia32_kshiftlidi,
    BI__builtin_ia32_kshiftlihi,
    BI__builtin_ia32_kshiftliqi,
    BI__builtin_ia32_kshiftlisi,
    BI__builtin_ia32_kshiftridi,
    BI__builtin_ia32_kshiftrihi,
    BI__builtin_ia32_kshiftriqi,
    BI__builtin_ia32_kshiftrisi,
    BI__builtin_ia32_ktestcdi,
    BI__builtin_ia32_ktestchi,
    BI__builtin_ia32_ktestcqi,
    BI__builtin_ia32_ktestcsi,
    BI__builtin_ia32_ktestzdi,
    BI__builtin_ia32_ktestzhi,
    BI__builtin_ia32_ktestzqi,
    BI__builtin_ia32_ktestzsi,
    BI__builtin_ia32_kunpckdi,
    BI__builtin_ia32_kunpckhi,
    BI__builtin_ia32_kunpcksi,
    BI__builtin_ia32_kxnordi,
    BI__builtin_ia32_kxnorhi,
    BI__builtin_ia32_kxnorqi,
    BI__builtin_ia32_kxnorsi,
    BI__builtin_ia32_kxordi,
    BI__builtin_ia32_kxorhi,
    BI__builtin_ia32_kxorqi,
    BI__builtin_ia32_kxorsi,
    BI__builtin_ia32_lddqu,
    BI__builtin_ia32_lddqu256,
    BI__builtin_ia32_ldmxcsr,
    BI__builtin_ia32_lfence,
    BI__builtin_ia32_llwpcb,
    BI__builtin_ia32_loadapd128_mask,
    BI__builtin_ia32_loadapd256_mask,
    BI__builtin_ia32_loadapd512_mask,
    BI__builtin_ia32_loadaps128_mask,
    BI__builtin_ia32_loadaps256_mask,
    BI__builtin_ia32_loadaps512_mask,
    BI__builtin_ia32_loaddqudi128_mask,
    BI__builtin_ia32_loaddqudi256_mask,
    BI__builtin_ia32_loaddqudi512_mask,
    BI__builtin_ia32_loaddquhi128_mask,
    BI__builtin_ia32_loaddquhi256_mask,
    BI__builtin_ia32_loaddquhi512_mask,
    BI__builtin_ia32_loaddquqi128_mask,
    BI__builtin_ia32_loaddquqi256_mask,
    BI__builtin_ia32_loaddquqi512_mask,
    BI__builtin_ia32_loaddqusi128_mask,
    BI__builtin_ia32_loaddqusi256_mask,
    BI__builtin_ia32_loaddqusi512_mask,
    BI__builtin_ia32_loadiwkey,
    BI__builtin_ia32_loadsbf16128_mask,
    BI__builtin_ia32_loadsd128_mask,
    BI__builtin_ia32_loadsh128_mask,
    BI__builtin_ia32_loadss128_mask,
    BI__builtin_ia32_loadupd128_mask,
    BI__builtin_ia32_loadupd256_mask,
    BI__builtin_ia32_loadupd512_mask,
    BI__builtin_ia32_loadups128_mask,
    BI__builtin_ia32_loadups256_mask,
    BI__builtin_ia32_loadups512_mask,
    BI__builtin_ia32_lwpins32,
    BI__builtin_ia32_lwpval32,
    BI__builtin_ia32_lzcnt_u16,
    BI__builtin_ia32_lzcnt_u32,
    BI__builtin_ia32_maskloadd,
    BI__builtin_ia32_maskloadd256,
    BI__builtin_ia32_maskloadpd,
    BI__builtin_ia32_maskloadpd256,
    BI__builtin_ia32_maskloadps,
    BI__builtin_ia32_maskloadps256,
    BI__builtin_ia32_maskloadq,
    BI__builtin_ia32_maskloadq256,
    BI__builtin_ia32_maskmovdqu,
    BI__builtin_ia32_maskstored,
    BI__builtin_ia32_maskstored256,
    BI__builtin_ia32_maskstorepd,
    BI__builtin_ia32_maskstorepd256,
    BI__builtin_ia32_maskstoreps,
    BI__builtin_ia32_maskstoreps256,
    BI__builtin_ia32_maskstoreq,
    BI__builtin_ia32_maskstoreq256,
    BI__builtin_ia32_maxpd,
    BI__builtin_ia32_maxpd256,
    BI__builtin_ia32_maxpd512,
    BI__builtin_ia32_maxph128,
    BI__builtin_ia32_maxph256,
    BI__builtin_ia32_maxph512,
    BI__builtin_ia32_maxps,
    BI__builtin_ia32_maxps256,
    BI__builtin_ia32_maxps512,
    BI__builtin_ia32_maxsd,
    BI__builtin_ia32_maxsd_round_mask,
    BI__builtin_ia32_maxsh_round_mask,
    BI__builtin_ia32_maxss,
    BI__builtin_ia32_maxss_round_mask,
    BI__builtin_ia32_mfence,
    BI__builtin_ia32_minpd,
    BI__builtin_ia32_minpd256,
    BI__builtin_ia32_minpd512,
    BI__builtin_ia32_minph128,
    BI__builtin_ia32_minph256,
    BI__builtin_ia32_minph512,
    BI__builtin_ia32_minps,
    BI__builtin_ia32_minps256,
    BI__builtin_ia32_minps512,
    BI__builtin_ia32_minsd,
    BI__builtin_ia32_minsd_round_mask,
    BI__builtin_ia32_minsh_round_mask,
    BI__builtin_ia32_minss,
    BI__builtin_ia32_minss_round_mask,
    BI__builtin_ia32_monitor,
    BI__builtin_ia32_monitorx,
    BI__builtin_ia32_movdir64b,
    BI__builtin_ia32_movdqa32load128_mask,
    BI__builtin_ia32_movdqa32load256_mask,
    BI__builtin_ia32_movdqa32load512_mask,
    BI__builtin_ia32_movdqa32store128_mask,
    BI__builtin_ia32_movdqa32store256_mask,
    BI__builtin_ia32_movdqa32store512_mask,
    BI__builtin_ia32_movdqa64load128_mask,
    BI__builtin_ia32_movdqa64load256_mask,
    BI__builtin_ia32_movdqa64load512_mask,
    BI__builtin_ia32_movdqa64store128_mask,
    BI__builtin_ia32_movdqa64store256_mask,
    BI__builtin_ia32_movdqa64store512_mask,
    BI__builtin_ia32_movmskpd,
    BI__builtin_ia32_movmskpd256,
    BI__builtin_ia32_movmskps,
    BI__builtin_ia32_movmskps256,
    BI__builtin_ia32_movnti,
    BI__builtin_ia32_movntsd,
    BI__builtin_ia32_movntss,
    BI__builtin_ia32_mpsadbw128,
    BI__builtin_ia32_mpsadbw256,
    BI__builtin_ia32_mpsadbw512,
    BI__builtin_ia32_mulpd512,
    BI__builtin_ia32_mulph512,
    BI__builtin_ia32_mulps512,
    BI__builtin_ia32_mulsd_round_mask,
    BI__builtin_ia32_mulsh_round_mask,
    BI__builtin_ia32_mulss_round_mask,
    BI__builtin_ia32_mwait,
    BI__builtin_ia32_mwaitx,
    BI__builtin_ia32_packssdw128,
    BI__builtin_ia32_packssdw256,
    BI__builtin_ia32_packssdw512,
    BI__builtin_ia32_packsswb128,
    BI__builtin_ia32_packsswb256,
    BI__builtin_ia32_packsswb512,
    BI__builtin_ia32_packusdw128,
    BI__builtin_ia32_packusdw256,
    BI__builtin_ia32_packusdw512,
    BI__builtin_ia32_packuswb128,
    BI__builtin_ia32_packuswb256,
    BI__builtin_ia32_packuswb512,
    BI__builtin_ia32_palignr128,
    BI__builtin_ia32_palignr256,
    BI__builtin_ia32_palignr512,
    BI__builtin_ia32_pause,
    BI__builtin_ia32_pavgb128,
    BI__builtin_ia32_pavgb256,
    BI__builtin_ia32_pavgb512,
    BI__builtin_ia32_pavgw128,
    BI__builtin_ia32_pavgw256,
    BI__builtin_ia32_pavgw512,
    BI__builtin_ia32_pblendd128,
    BI__builtin_ia32_pblendd256,
    BI__builtin_ia32_pblendvb128,
    BI__builtin_ia32_pblendvb256,
    BI__builtin_ia32_pblendw128,
    BI__builtin_ia32_pblendw256,
    BI__builtin_ia32_pclmulqdq128,
    BI__builtin_ia32_pclmulqdq256,
    BI__builtin_ia32_pclmulqdq512,
    BI__builtin_ia32_pcmpestri128,
    BI__builtin_ia32_pcmpestria128,
    BI__builtin_ia32_pcmpestric128,
    BI__builtin_ia32_pcmpestrio128,
    BI__builtin_ia32_pcmpestris128,
    BI__builtin_ia32_pcmpestriz128,
    BI__builtin_ia32_pcmpestrm128,
    BI__builtin_ia32_pcmpistri128,
    BI__builtin_ia32_pcmpistria128,
    BI__builtin_ia32_pcmpistric128,
    BI__builtin_ia32_pcmpistrio128,
    BI__builtin_ia32_pcmpistris128,
    BI__builtin_ia32_pcmpistriz128,
    BI__builtin_ia32_pcmpistrm128,
    BI__builtin_ia32_pdep_si,
    BI__builtin_ia32_permdf256,
    BI__builtin_ia32_permdf512,
    BI__builtin_ia32_permdi256,
    BI__builtin_ia32_permdi512,
    BI__builtin_ia32_permti256,
    BI__builtin_ia32_permvardf256,
    BI__builtin_ia32_permvardf512,
    BI__builtin_ia32_permvardi256,
    BI__builtin_ia32_permvardi512,
    BI__builtin_ia32_permvarhi128,
    BI__builtin_ia32_permvarhi256,
    BI__builtin_ia32_permvarhi512,
    BI__builtin_ia32_permvarqi128,
    BI__builtin_ia32_permvarqi256,
    BI__builtin_ia32_permvarqi512,
    BI__builtin_ia32_permvarsf256,
    BI__builtin_ia32_permvarsf512,
    BI__builtin_ia32_permvarsi256,
    BI__builtin_ia32_permvarsi512,
    BI__builtin_ia32_pext_si,
    BI__builtin_ia32_phaddd128,
    BI__builtin_ia32_phaddd256,
    BI__builtin_ia32_phaddsw128,
    BI__builtin_ia32_phaddsw256,
    BI__builtin_ia32_phaddw128,
    BI__builtin_ia32_phaddw256,
    BI__builtin_ia32_phminposuw128,
    BI__builtin_ia32_phsubd128,
    BI__builtin_ia32_phsubd256,
    BI__builtin_ia32_phsubsw128,
    BI__builtin_ia32_phsubsw256,
    BI__builtin_ia32_phsubw128,
    BI__builtin_ia32_phsubw256,
    BI__builtin_ia32_pmaddubsw128,
    BI__builtin_ia32_pmaddubsw256,
    BI__builtin_ia32_pmaddubsw512,
    BI__builtin_ia32_pmaddwd128,
    BI__builtin_ia32_pmaddwd256,
    BI__builtin_ia32_pmaddwd512,
    BI__builtin_ia32_pmovdb128_mask,
    BI__builtin_ia32_pmovdb128mem_mask,
    BI__builtin_ia32_pmovdb256_mask,
    BI__builtin_ia32_pmovdb256mem_mask,
    BI__builtin_ia32_pmovdb512_mask,
    BI__builtin_ia32_pmovdb512mem_mask,
    BI__builtin_ia32_pmovdw128_mask,
    BI__builtin_ia32_pmovdw128mem_mask,
    BI__builtin_ia32_pmovdw256_mask,
    BI__builtin_ia32_pmovdw256mem_mask,
    BI__builtin_ia32_pmovdw512_mask,
    BI__builtin_ia32_pmovdw512mem_mask,
    BI__builtin_ia32_pmovmskb128,
    BI__builtin_ia32_pmovmskb256,
    BI__builtin_ia32_pmovqb128_mask,
    BI__builtin_ia32_pmovqb128mem_mask,
    BI__builtin_ia32_pmovqb256_mask,
    BI__builtin_ia32_pmovqb256mem_mask,
    BI__builtin_ia32_pmovqb512_mask,
    BI__builtin_ia32_pmovqb512mem_mask,
    BI__builtin_ia32_pmovqd128_mask,
    BI__builtin_ia32_pmovqd128mem_mask,
    BI__builtin_ia32_pmovqd256mem_mask,
    BI__builtin_ia32_pmovqd512_mask,
    BI__builtin_ia32_pmovqd512mem_mask,
    BI__builtin_ia32_pmovqw128_mask,
    BI__builtin_ia32_pmovqw128mem_mask,
    BI__builtin_ia32_pmovqw256_mask,
    BI__builtin_ia32_pmovqw256mem_mask,
    BI__builtin_ia32_pmovqw512_mask,
    BI__builtin_ia32_pmovqw512mem_mask,
    BI__builtin_ia32_pmovsdb128_mask,
    BI__builtin_ia32_pmovsdb128mem_mask,
    BI__builtin_ia32_pmovsdb256_mask,
    BI__builtin_ia32_pmovsdb256mem_mask,
    BI__builtin_ia32_pmovsdb512_mask,
    BI__builtin_ia32_pmovsdb512mem_mask,
    BI__builtin_ia32_pmovsdw128_mask,
    BI__builtin_ia32_pmovsdw128mem_mask,
    BI__builtin_ia32_pmovsdw256_mask,
    BI__builtin_ia32_pmovsdw256mem_mask,
    BI__builtin_ia32_pmovsdw512_mask,
    BI__builtin_ia32_pmovsdw512mem_mask,
    BI__builtin_ia32_pmovsqb128_mask,
    BI__builtin_ia32_pmovsqb128mem_mask,
    BI__builtin_ia32_pmovsqb256_mask,
    BI__builtin_ia32_pmovsqb256mem_mask,
    BI__builtin_ia32_pmovsqb512_mask,
    BI__builtin_ia32_pmovsqb512mem_mask,
    BI__builtin_ia32_pmovsqd128_mask,
    BI__builtin_ia32_pmovsqd128mem_mask,
    BI__builtin_ia32_pmovsqd256_mask,
    BI__builtin_ia32_pmovsqd256mem_mask,
    BI__builtin_ia32_pmovsqd512_mask,
    BI__builtin_ia32_pmovsqd512mem_mask,
    BI__builtin_ia32_pmovsqw128_mask,
    BI__builtin_ia32_pmovsqw128mem_mask,
    BI__builtin_ia32_pmovsqw256_mask,
    BI__builtin_ia32_pmovsqw256mem_mask,
    BI__builtin_ia32_pmovsqw512_mask,
    BI__builtin_ia32_pmovsqw512mem_mask,
    BI__builtin_ia32_pmovswb128_mask,
    BI__builtin_ia32_pmovswb128mem_mask,
    BI__builtin_ia32_pmovswb256_mask,
    BI__builtin_ia32_pmovswb256mem_mask,
    BI__builtin_ia32_pmovswb512_mask,
    BI__builtin_ia32_pmovswb512mem_mask,
    BI__builtin_ia32_pmovusdb128_mask,
    BI__builtin_ia32_pmovusdb128mem_mask,
    BI__builtin_ia32_pmovusdb256_mask,
    BI__builtin_ia32_pmovusdb256mem_mask,
    BI__builtin_ia32_pmovusdb512_mask,
    BI__builtin_ia32_pmovusdb512mem_mask,
    BI__builtin_ia32_pmovusdw128_mask,
    BI__builtin_ia32_pmovusdw128mem_mask,
    BI__builtin_ia32_pmovusdw256_mask,
    BI__builtin_ia32_pmovusdw256mem_mask,
    BI__builtin_ia32_pmovusdw512_mask,
    BI__builtin_ia32_pmovusdw512mem_mask,
    BI__builtin_ia32_pmovusqb128_mask,
    BI__builtin_ia32_pmovusqb128mem_mask,
    BI__builtin_ia32_pmovusqb256_mask,
    BI__builtin_ia32_pmovusqb256mem_mask,
    BI__builtin_ia32_pmovusqb512_mask,
    BI__builtin_ia32_pmovusqb512mem_mask,
    BI__builtin_ia32_pmovusqd128_mask,
    BI__builtin_ia32_pmovusqd128mem_mask,
    BI__builtin_ia32_pmovusqd256_mask,
    BI__builtin_ia32_pmovusqd256mem_mask,
    BI__builtin_ia32_pmovusqd512_mask,
    BI__builtin_ia32_pmovusqd512mem_mask,
    BI__builtin_ia32_pmovusqw128_mask,
    BI__builtin_ia32_pmovusqw128mem_mask,
    BI__builtin_ia32_pmovusqw256_mask,
    BI__builtin_ia32_pmovusqw256mem_mask,
    BI__builtin_ia32_pmovusqw512_mask,
    BI__builtin_ia32_pmovusqw512mem_mask,
    BI__builtin_ia32_pmovuswb128_mask,
    BI__builtin_ia32_pmovuswb128mem_mask,
    BI__builtin_ia32_pmovuswb256_mask,
    BI__builtin_ia32_pmovuswb256mem_mask,
    BI__builtin_ia32_pmovuswb512_mask,
    BI__builtin_ia32_pmovuswb512mem_mask,
    BI__builtin_ia32_pmovwb128_mask,
    BI__builtin_ia32_pmovwb128mem_mask,
    BI__builtin_ia32_pmovwb256mem_mask,
    BI__builtin_ia32_pmovwb512_mask,
    BI__builtin_ia32_pmovwb512mem_mask,
    BI__builtin_ia32_pmuldq128,
    BI__builtin_ia32_pmuldq256,
    BI__builtin_ia32_pmuldq512,
    BI__builtin_ia32_pmulhrsw128,
    BI__builtin_ia32_pmulhrsw256,
    BI__builtin_ia32_pmulhrsw512,
    BI__builtin_ia32_pmulhuw128,
    BI__builtin_ia32_pmulhuw256,
    BI__builtin_ia32_pmulhuw512,
    BI__builtin_ia32_pmulhw128,
    BI__builtin_ia32_pmulhw256,
    BI__builtin_ia32_pmulhw512,
    BI__builtin_ia32_pmuludq128,
    BI__builtin_ia32_pmuludq256,
    BI__builtin_ia32_pmuludq512,
    BI__builtin_ia32_prefetchrs,
    BI__builtin_ia32_prold128,
    BI__builtin_ia32_prold256,
    BI__builtin_ia32_prold512,
    BI__builtin_ia32_prolq128,
    BI__builtin_ia32_prolq256,
    BI__builtin_ia32_prolq512,
    BI__builtin_ia32_prord128,
    BI__builtin_ia32_prord256,
    BI__builtin_ia32_prord512,
    BI__builtin_ia32_prorq128,
    BI__builtin_ia32_prorq256,
    BI__builtin_ia32_prorq512,
    BI__builtin_ia32_psadbw128,
    BI__builtin_ia32_psadbw256,
    BI__builtin_ia32_psadbw512,
    BI__builtin_ia32_pshufb128,
    BI__builtin_ia32_pshufb256,
    BI__builtin_ia32_pshufb512,
    BI__builtin_ia32_pshufd,
    BI__builtin_ia32_pshufd256,
    BI__builtin_ia32_pshufd512,
    BI__builtin_ia32_pshufhw,
    BI__builtin_ia32_pshufhw256,
    BI__builtin_ia32_pshufhw512,
    BI__builtin_ia32_pshuflw,
    BI__builtin_ia32_pshuflw256,
    BI__builtin_ia32_pshuflw512,
    BI__builtin_ia32_psignb128,
    BI__builtin_ia32_psignb256,
    BI__builtin_ia32_psignd128,
    BI__builtin_ia32_psignd256,
    BI__builtin_ia32_psignw128,
    BI__builtin_ia32_psignw256,
    BI__builtin_ia32_pslld128,
    BI__builtin_ia32_pslld256,
    BI__builtin_ia32_pslld512,
    BI__builtin_ia32_pslldi128,
    BI__builtin_ia32_pslldi256,
    BI__builtin_ia32_pslldi512,
    BI__builtin_ia32_pslldqi128_byteshift,
    BI__builtin_ia32_pslldqi256_byteshift,
    BI__builtin_ia32_pslldqi512_byteshift,
    BI__builtin_ia32_psllq128,
    BI__builtin_ia32_psllq256,
    BI__builtin_ia32_psllq512,
    BI__builtin_ia32_psllqi128,
    BI__builtin_ia32_psllqi256,
    BI__builtin_ia32_psllqi512,
    BI__builtin_ia32_psllv2di,
    BI__builtin_ia32_psllv4di,
    BI__builtin_ia32_psllv4si,
    BI__builtin_ia32_psllv8di,
    BI__builtin_ia32_psllv8hi,
    BI__builtin_ia32_psllv8si,
    BI__builtin_ia32_psllv16hi,
    BI__builtin_ia32_psllv16si,
    BI__builtin_ia32_psllv32hi,
    BI__builtin_ia32_psllw128,
    BI__builtin_ia32_psllw256,
    BI__builtin_ia32_psllw512,
    BI__builtin_ia32_psllwi128,
    BI__builtin_ia32_psllwi256,
    BI__builtin_ia32_psllwi512,
    BI__builtin_ia32_psrad128,
    BI__builtin_ia32_psrad256,
    BI__builtin_ia32_psrad512,
    BI__builtin_ia32_psradi128,
    BI__builtin_ia32_psradi256,
    BI__builtin_ia32_psradi512,
    BI__builtin_ia32_psraq128,
    BI__builtin_ia32_psraq256,
    BI__builtin_ia32_psraq512,
    BI__builtin_ia32_psraqi128,
    BI__builtin_ia32_psraqi256,
    BI__builtin_ia32_psraqi512,
    BI__builtin_ia32_psrav4si,
    BI__builtin_ia32_psrav8di,
    BI__builtin_ia32_psrav8hi,
    BI__builtin_ia32_psrav8si,
    BI__builtin_ia32_psrav16hi,
    BI__builtin_ia32_psrav16si,
    BI__builtin_ia32_psrav32hi,
    BI__builtin_ia32_psravq128,
    BI__builtin_ia32_psravq256,
    BI__builtin_ia32_psraw128,
    BI__builtin_ia32_psraw256,
    BI__builtin_ia32_psraw512,
    BI__builtin_ia32_psrawi128,
    BI__builtin_ia32_psrawi256,
    BI__builtin_ia32_psrawi512,
    BI__builtin_ia32_psrld128,
    BI__builtin_ia32_psrld256,
    BI__builtin_ia32_psrld512,
    BI__builtin_ia32_psrldi128,
    BI__builtin_ia32_psrldi256,
    BI__builtin_ia32_psrldi512,
    BI__builtin_ia32_psrldqi128_byteshift,
    BI__builtin_ia32_psrldqi256_byteshift,
    BI__builtin_ia32_psrldqi512_byteshift,
    BI__builtin_ia32_psrlq128,
    BI__builtin_ia32_psrlq256,
    BI__builtin_ia32_psrlq512,
    BI__builtin_ia32_psrlqi128,
    BI__builtin_ia32_psrlqi256,
    BI__builtin_ia32_psrlqi512,
    BI__builtin_ia32_psrlv2di,
    BI__builtin_ia32_psrlv4di,
    BI__builtin_ia32_psrlv4si,
    BI__builtin_ia32_psrlv8di,
    BI__builtin_ia32_psrlv8hi,
    BI__builtin_ia32_psrlv8si,
    BI__builtin_ia32_psrlv16hi,
    BI__builtin_ia32_psrlv16si,
    BI__builtin_ia32_psrlv32hi,
    BI__builtin_ia32_psrlw128,
    BI__builtin_ia32_psrlw256,
    BI__builtin_ia32_psrlw512,
    BI__builtin_ia32_psrlwi128,
    BI__builtin_ia32_psrlwi256,
    BI__builtin_ia32_psrlwi512,
    BI__builtin_ia32_pternlogd128_mask,
    BI__builtin_ia32_pternlogd128_maskz,
    BI__builtin_ia32_pternlogd256_mask,
    BI__builtin_ia32_pternlogd256_maskz,
    BI__builtin_ia32_pternlogd512_mask,
    BI__builtin_ia32_pternlogd512_maskz,
    BI__builtin_ia32_pternlogq128_mask,
    BI__builtin_ia32_pternlogq128_maskz,
    BI__builtin_ia32_pternlogq256_mask,
    BI__builtin_ia32_pternlogq256_maskz,
    BI__builtin_ia32_pternlogq512_mask,
    BI__builtin_ia32_pternlogq512_maskz,
    BI__builtin_ia32_ptestc128,
    BI__builtin_ia32_ptestc256,
    BI__builtin_ia32_ptestnzc128,
    BI__builtin_ia32_ptestnzc256,
    BI__builtin_ia32_ptestz128,
    BI__builtin_ia32_ptestz256,
    BI__builtin_ia32_ptwrite32,
    BI__builtin_ia32_rangepd128_mask,
    BI__builtin_ia32_rangepd256_mask,
    BI__builtin_ia32_rangepd512_mask,
    BI__builtin_ia32_rangeps128_mask,
    BI__builtin_ia32_rangeps256_mask,
    BI__builtin_ia32_rangeps512_mask,
    BI__builtin_ia32_rangesd128_round_mask,
    BI__builtin_ia32_rangess128_round_mask,
    BI__builtin_ia32_rcp14pd128_mask,
    BI__builtin_ia32_rcp14pd256_mask,
    BI__builtin_ia32_rcp14pd512_mask,
    BI__builtin_ia32_rcp14ps128_mask,
    BI__builtin_ia32_rcp14ps256_mask,
    BI__builtin_ia32_rcp14ps512_mask,
    BI__builtin_ia32_rcp14sd_mask,
    BI__builtin_ia32_rcp14ss_mask,
    BI__builtin_ia32_rcpph128_mask,
    BI__builtin_ia32_rcpph256_mask,
    BI__builtin_ia32_rcpph512_mask,
    BI__builtin_ia32_rcpps,
    BI__builtin_ia32_rcpps256,
    BI__builtin_ia32_rcpsh_mask,
    BI__builtin_ia32_rcpss,
    BI__builtin_ia32_rdpid,
    BI__builtin_ia32_rdpkru,
    BI__builtin_ia32_rdpmc,
    BI__builtin_ia32_rdpru,
    BI__builtin_ia32_rdrand16_step,
    BI__builtin_ia32_rdrand32_step,
    BI__builtin_ia32_rdseed16_step,
    BI__builtin_ia32_rdseed32_step,
    BI__builtin_ia32_rdsspd,
    BI__builtin_ia32_rdtsc,
    BI__builtin_ia32_rdtscp,
    BI__builtin_ia32_readeflags_u32,
    BI__builtin_ia32_reduce_fadd_pd512,
    BI__builtin_ia32_reduce_fadd_ph128,
    BI__builtin_ia32_reduce_fadd_ph256,
    BI__builtin_ia32_reduce_fadd_ph512,
    BI__builtin_ia32_reduce_fadd_ps512,
    BI__builtin_ia32_reduce_fmax_pd512,
    BI__builtin_ia32_reduce_fmax_ph128,
    BI__builtin_ia32_reduce_fmax_ph256,
    BI__builtin_ia32_reduce_fmax_ph512,
    BI__builtin_ia32_reduce_fmax_ps512,
    BI__builtin_ia32_reduce_fmin_pd512,
    BI__builtin_ia32_reduce_fmin_ph128,
    BI__builtin_ia32_reduce_fmin_ph256,
    BI__builtin_ia32_reduce_fmin_ph512,
    BI__builtin_ia32_reduce_fmin_ps512,
    BI__builtin_ia32_reduce_fmul_pd512,
    BI__builtin_ia32_reduce_fmul_ph128,
    BI__builtin_ia32_reduce_fmul_ph256,
    BI__builtin_ia32_reduce_fmul_ph512,
    BI__builtin_ia32_reduce_fmul_ps512,
    BI__builtin_ia32_reducepd128_mask,
    BI__builtin_ia32_reducepd256_mask,
    BI__builtin_ia32_reducepd512_mask,
    BI__builtin_ia32_reduceph128_mask,
    BI__builtin_ia32_reduceph256_mask,
    BI__builtin_ia32_reduceph512_mask,
    BI__builtin_ia32_reduceps128_mask,
    BI__builtin_ia32_reduceps256_mask,
    BI__builtin_ia32_reduceps512_mask,
    BI__builtin_ia32_reducesd_mask,
    BI__builtin_ia32_reducesh_mask,
    BI__builtin_ia32_reducess_mask,
    BI__builtin_ia32_rndscalepd_128_mask,
    BI__builtin_ia32_rndscalepd_256_mask,
    BI__builtin_ia32_rndscalepd_mask,
    BI__builtin_ia32_rndscaleph_128_mask,
    BI__builtin_ia32_rndscaleph_256_mask,
    BI__builtin_ia32_rndscaleph_mask,
    BI__builtin_ia32_rndscaleps_128_mask,
    BI__builtin_ia32_rndscaleps_256_mask,
    BI__builtin_ia32_rndscaleps_mask,
    BI__builtin_ia32_rndscalesd_round_mask,
    BI__builtin_ia32_rndscalesh_round_mask,
    BI__builtin_ia32_rndscaless_round_mask,
    BI__builtin_ia32_roundpd,
    BI__builtin_ia32_roundpd256,
    BI__builtin_ia32_roundps,
    BI__builtin_ia32_roundps256,
    BI__builtin_ia32_roundsd,
    BI__builtin_ia32_roundss,
    BI__builtin_ia32_rsqrt14pd128_mask,
    BI__builtin_ia32_rsqrt14pd256_mask,
    BI__builtin_ia32_rsqrt14pd512_mask,
    BI__builtin_ia32_rsqrt14ps128_mask,
    BI__builtin_ia32_rsqrt14ps256_mask,
    BI__builtin_ia32_rsqrt14ps512_mask,
    BI__builtin_ia32_rsqrt14sd_mask,
    BI__builtin_ia32_rsqrt14ss_mask,
    BI__builtin_ia32_rsqrtph128_mask,
    BI__builtin_ia32_rsqrtph256_mask,
    BI__builtin_ia32_rsqrtph512_mask,
    BI__builtin_ia32_rsqrtps,
    BI__builtin_ia32_rsqrtps256,
    BI__builtin_ia32_rsqrtsh_mask,
    BI__builtin_ia32_rsqrtss,
    BI__builtin_ia32_rstorssp,
    BI__builtin_ia32_saveprevssp,
    BI__builtin_ia32_scalefpd128_mask,
    BI__builtin_ia32_scalefpd256_mask,
    BI__builtin_ia32_scalefpd512_mask,
    BI__builtin_ia32_scalefph128_mask,
    BI__builtin_ia32_scalefph256_mask,
    BI__builtin_ia32_scalefph512_mask,
    BI__builtin_ia32_scalefps128_mask,
    BI__builtin_ia32_scalefps256_mask,
    BI__builtin_ia32_scalefps512_mask,
    BI__builtin_ia32_scalefsd_round_mask,
    BI__builtin_ia32_scalefsh_round_mask,
    BI__builtin_ia32_scalefss_round_mask,
    BI__builtin_ia32_scatterdiv2df,
    BI__builtin_ia32_scatterdiv2di,
    BI__builtin_ia32_scatterdiv4df,
    BI__builtin_ia32_scatterdiv4di,
    BI__builtin_ia32_scatterdiv4sf,
    BI__builtin_ia32_scatterdiv4si,
    BI__builtin_ia32_scatterdiv8df,
    BI__builtin_ia32_scatterdiv8di,
    BI__builtin_ia32_scatterdiv8sf,
    BI__builtin_ia32_scatterdiv8si,
    BI__builtin_ia32_scatterdiv16sf,
    BI__builtin_ia32_scatterdiv16si,
    BI__builtin_ia32_scattersiv2df,
    BI__builtin_ia32_scattersiv2di,
    BI__builtin_ia32_scattersiv4df,
    BI__builtin_ia32_scattersiv4di,
    BI__builtin_ia32_scattersiv4sf,
    BI__builtin_ia32_scattersiv4si,
    BI__builtin_ia32_scattersiv8df,
    BI__builtin_ia32_scattersiv8di,
    BI__builtin_ia32_scattersiv8sf,
    BI__builtin_ia32_scattersiv8si,
    BI__builtin_ia32_scattersiv16sf,
    BI__builtin_ia32_scattersiv16si,
    BI__builtin_ia32_selectb_128,
    BI__builtin_ia32_selectb_256,
    BI__builtin_ia32_selectb_512,
    BI__builtin_ia32_selectd_128,
    BI__builtin_ia32_selectd_256,
    BI__builtin_ia32_selectd_512,
    BI__builtin_ia32_selectpbf_128,
    BI__builtin_ia32_selectpbf_256,
    BI__builtin_ia32_selectpbf_512,
    BI__builtin_ia32_selectpd_128,
    BI__builtin_ia32_selectpd_256,
    BI__builtin_ia32_selectpd_512,
    BI__builtin_ia32_selectph_128,
    BI__builtin_ia32_selectph_256,
    BI__builtin_ia32_selectph_512,
    BI__builtin_ia32_selectps_128,
    BI__builtin_ia32_selectps_256,
    BI__builtin_ia32_selectps_512,
    BI__builtin_ia32_selectq_128,
    BI__builtin_ia32_selectq_256,
    BI__builtin_ia32_selectq_512,
    BI__builtin_ia32_selectsbf_128,
    BI__builtin_ia32_selectsd_128,
    BI__builtin_ia32_selectsh_128,
    BI__builtin_ia32_selectss_128,
    BI__builtin_ia32_selectw_128,
    BI__builtin_ia32_selectw_256,
    BI__builtin_ia32_selectw_512,
    BI__builtin_ia32_serialize,
    BI__builtin_ia32_setssbsy,
    BI__builtin_ia32_sfence,
    BI__builtin_ia32_sha1msg1,
    BI__builtin_ia32_sha1msg2,
    BI__builtin_ia32_sha1nexte,
    BI__builtin_ia32_sha1rnds4,
    BI__builtin_ia32_sha256msg1,
    BI__builtin_ia32_sha256msg2,
    BI__builtin_ia32_sha256rnds2,
    BI__builtin_ia32_shuf_f32x4,
    BI__builtin_ia32_shuf_f32x4_256,
    BI__builtin_ia32_shuf_f64x2,
    BI__builtin_ia32_shuf_f64x2_256,
    BI__builtin_ia32_shuf_i32x4,
    BI__builtin_ia32_shuf_i32x4_256,
    BI__builtin_ia32_shuf_i64x2,
    BI__builtin_ia32_shuf_i64x2_256,
    BI__builtin_ia32_shufpd,
    BI__builtin_ia32_shufpd256,
    BI__builtin_ia32_shufpd512,
    BI__builtin_ia32_shufps,
    BI__builtin_ia32_shufps256,
    BI__builtin_ia32_shufps512,
    BI__builtin_ia32_slwpcb,
    BI__builtin_ia32_sqrtpd512,
    BI__builtin_ia32_sqrtph512,
    BI__builtin_ia32_sqrtps512,
    BI__builtin_ia32_sqrtsd_round_mask,
    BI__builtin_ia32_sqrtsh_round_mask,
    BI__builtin_ia32_sqrtss_round_mask,
    BI__builtin_ia32_stmxcsr,
    BI__builtin_ia32_storeapd128_mask,
    BI__builtin_ia32_storeapd256_mask,
    BI__builtin_ia32_storeapd512_mask,
    BI__builtin_ia32_storeaps128_mask,
    BI__builtin_ia32_storeaps256_mask,
    BI__builtin_ia32_storeaps512_mask,
    BI__builtin_ia32_storedqudi128_mask,
    BI__builtin_ia32_storedqudi256_mask,
    BI__builtin_ia32_storedqudi512_mask,
    BI__builtin_ia32_storedquhi128_mask,
    BI__builtin_ia32_storedquhi256_mask,
    BI__builtin_ia32_storedquhi512_mask,
    BI__builtin_ia32_storedquqi128_mask,
    BI__builtin_ia32_storedquqi256_mask,
    BI__builtin_ia32_storedquqi512_mask,
    BI__builtin_ia32_storedqusi128_mask,
    BI__builtin_ia32_storedqusi256_mask,
    BI__builtin_ia32_storedqusi512_mask,
    BI__builtin_ia32_storesbf16128_mask,
    BI__builtin_ia32_storesd128_mask,
    BI__builtin_ia32_storesh128_mask,
    BI__builtin_ia32_storess128_mask,
    BI__builtin_ia32_storeupd128_mask,
    BI__builtin_ia32_storeupd256_mask,
    BI__builtin_ia32_storeupd512_mask,
    BI__builtin_ia32_storeups128_mask,
    BI__builtin_ia32_storeups256_mask,
    BI__builtin_ia32_storeups512_mask,
    BI__builtin_ia32_subborrow_u32,
    BI__builtin_ia32_subpd512,
    BI__builtin_ia32_subph512,
    BI__builtin_ia32_subps512,
    BI__builtin_ia32_subsd_round_mask,
    BI__builtin_ia32_subsh_round_mask,
    BI__builtin_ia32_subss_round_mask,
    BI__builtin_ia32_tpause,
    BI__builtin_ia32_tzcnt_u16,
    BI__builtin_ia32_tzcnt_u32,
    BI__builtin_ia32_ucmpb128_mask,
    BI__builtin_ia32_ucmpb256_mask,
    BI__builtin_ia32_ucmpb512_mask,
    BI__builtin_ia32_ucmpd128_mask,
    BI__builtin_ia32_ucmpd256_mask,
    BI__builtin_ia32_ucmpd512_mask,
    BI__builtin_ia32_ucmpq128_mask,
    BI__builtin_ia32_ucmpq256_mask,
    BI__builtin_ia32_ucmpq512_mask,
    BI__builtin_ia32_ucmpw128_mask,
    BI__builtin_ia32_ucmpw256_mask,
    BI__builtin_ia32_ucmpw512_mask,
    BI__builtin_ia32_ucomieq,
    BI__builtin_ia32_ucomige,
    BI__builtin_ia32_ucomigt,
    BI__builtin_ia32_ucomile,
    BI__builtin_ia32_ucomilt,
    BI__builtin_ia32_ucomineq,
    BI__builtin_ia32_ucomisdeq,
    BI__builtin_ia32_ucomisdge,
    BI__builtin_ia32_ucomisdgt,
    BI__builtin_ia32_ucomisdle,
    BI__builtin_ia32_ucomisdlt,
    BI__builtin_ia32_ucomisdneq,
    BI__builtin_ia32_umonitor,
    BI__builtin_ia32_umwait,
    BI__builtin_ia32_undef128,
    BI__builtin_ia32_undef256,
    BI__builtin_ia32_undef512,
    BI__builtin_ia32_vaddbf16128,
    BI__builtin_ia32_vaddbf16256,
    BI__builtin_ia32_vaddbf16512,
    BI__builtin_ia32_vbcstnebf162ps128,
    BI__builtin_ia32_vbcstnebf162ps256,
    BI__builtin_ia32_vbcstnesh2ps128,
    BI__builtin_ia32_vbcstnesh2ps256,
    BI__builtin_ia32_vcmpbf16128_mask,
    BI__builtin_ia32_vcmpbf16256_mask,
    BI__builtin_ia32_vcmpbf16512_mask,
    BI__builtin_ia32_vcomisbf16eq,
    BI__builtin_ia32_vcomisbf16ge,
    BI__builtin_ia32_vcomisbf16gt,
    BI__builtin_ia32_vcomisbf16le,
    BI__builtin_ia32_vcomisbf16lt,
    BI__builtin_ia32_vcomisbf16neq,
    BI__builtin_ia32_vcomisd,
    BI__builtin_ia32_vcomish,
    BI__builtin_ia32_vcomiss,
    BI__builtin_ia32_vcvt2ph2bf8_128,
    BI__builtin_ia32_vcvt2ph2bf8_256,
    BI__builtin_ia32_vcvt2ph2bf8_512,
    BI__builtin_ia32_vcvt2ph2bf8s_128,
    BI__builtin_ia32_vcvt2ph2bf8s_256,
    BI__builtin_ia32_vcvt2ph2bf8s_512,
    BI__builtin_ia32_vcvt2ph2hf8_128,
    BI__builtin_ia32_vcvt2ph2hf8_256,
    BI__builtin_ia32_vcvt2ph2hf8_512,
    BI__builtin_ia32_vcvt2ph2hf8s_128,
    BI__builtin_ia32_vcvt2ph2hf8s_256,
    BI__builtin_ia32_vcvt2ph2hf8s_512,
    BI__builtin_ia32_vcvt2ps2phx128_mask,
    BI__builtin_ia32_vcvt2ps2phx256_mask,
    BI__builtin_ia32_vcvt2ps2phx512_mask,
    BI__builtin_ia32_vcvtbf162ibs128,
    BI__builtin_ia32_vcvtbf162ibs256,
    BI__builtin_ia32_vcvtbf162ibs512,
    BI__builtin_ia32_vcvtbf162iubs128,
    BI__builtin_ia32_vcvtbf162iubs256,
    BI__builtin_ia32_vcvtbf162iubs512,
    BI__builtin_ia32_vcvtbiasph2bf8_128_mask,
    BI__builtin_ia32_vcvtbiasph2bf8_256_mask,
    BI__builtin_ia32_vcvtbiasph2bf8_512_mask,
    BI__builtin_ia32_vcvtbiasph2bf8s_128_mask,
    BI__builtin_ia32_vcvtbiasph2bf8s_256_mask,
    BI__builtin_ia32_vcvtbiasph2bf8s_512_mask,
    BI__builtin_ia32_vcvtbiasph2hf8_128_mask,
    BI__builtin_ia32_vcvtbiasph2hf8_256_mask,
    BI__builtin_ia32_vcvtbiasph2hf8_512_mask,
    BI__builtin_ia32_vcvtbiasph2hf8s_128_mask,
    BI__builtin_ia32_vcvtbiasph2hf8s_256_mask,
    BI__builtin_ia32_vcvtbiasph2hf8s_512_mask,
    BI__builtin_ia32_vcvtdq2ph128_mask,
    BI__builtin_ia32_vcvtdq2ph256_mask,
    BI__builtin_ia32_vcvtdq2ph512_mask,
    BI__builtin_ia32_vcvthf8_2ph128_mask,
    BI__builtin_ia32_vcvthf8_2ph256_mask,
    BI__builtin_ia32_vcvthf8_2ph512_mask,
    BI__builtin_ia32_vcvtneebf162ps128,
    BI__builtin_ia32_vcvtneebf162ps256,
    BI__builtin_ia32_vcvtneeph2ps128,
    BI__builtin_ia32_vcvtneeph2ps256,
    BI__builtin_ia32_vcvtneobf162ps128,
    BI__builtin_ia32_vcvtneobf162ps256,
    BI__builtin_ia32_vcvtneoph2ps128,
    BI__builtin_ia32_vcvtneoph2ps256,
    BI__builtin_ia32_vcvtneps2bf16128,
    BI__builtin_ia32_vcvtneps2bf16256,
    BI__builtin_ia32_vcvtpd2ph128_mask,
    BI__builtin_ia32_vcvtpd2ph256_mask,
    BI__builtin_ia32_vcvtpd2ph512_mask,
    BI__builtin_ia32_vcvtph2bf8_128_mask,
    BI__builtin_ia32_vcvtph2bf8_256_mask,
    BI__builtin_ia32_vcvtph2bf8_512_mask,
    BI__builtin_ia32_vcvtph2bf8s_128_mask,
    BI__builtin_ia32_vcvtph2bf8s_256_mask,
    BI__builtin_ia32_vcvtph2bf8s_512_mask,
    BI__builtin_ia32_vcvtph2dq128_mask,
    BI__builtin_ia32_vcvtph2dq256_mask,
    BI__builtin_ia32_vcvtph2dq512_mask,
    BI__builtin_ia32_vcvtph2hf8_128_mask,
    BI__builtin_ia32_vcvtph2hf8_256_mask,
    BI__builtin_ia32_vcvtph2hf8_512_mask,
    BI__builtin_ia32_vcvtph2hf8s_128_mask,
    BI__builtin_ia32_vcvtph2hf8s_256_mask,
    BI__builtin_ia32_vcvtph2hf8s_512_mask,
    BI__builtin_ia32_vcvtph2ibs128_mask,
    BI__builtin_ia32_vcvtph2ibs256_mask,
    BI__builtin_ia32_vcvtph2ibs512_mask,
    BI__builtin_ia32_vcvtph2iubs128_mask,
    BI__builtin_ia32_vcvtph2iubs256_mask,
    BI__builtin_ia32_vcvtph2iubs512_mask,
    BI__builtin_ia32_vcvtph2pd128_mask,
    BI__builtin_ia32_vcvtph2pd256_mask,
    BI__builtin_ia32_vcvtph2pd512_mask,
    BI__builtin_ia32_vcvtph2ps256_mask,
    BI__builtin_ia32_vcvtph2ps512_mask,
    BI__builtin_ia32_vcvtph2ps_mask,
    BI__builtin_ia32_vcvtph2psx128_mask,
    BI__builtin_ia32_vcvtph2psx256_mask,
    BI__builtin_ia32_vcvtph2psx512_mask,
    BI__builtin_ia32_vcvtph2qq128_mask,
    BI__builtin_ia32_vcvtph2qq256_mask,
    BI__builtin_ia32_vcvtph2qq512_mask,
    BI__builtin_ia32_vcvtph2udq128_mask,
    BI__builtin_ia32_vcvtph2udq256_mask,
    BI__builtin_ia32_vcvtph2udq512_mask,
    BI__builtin_ia32_vcvtph2uqq128_mask,
    BI__builtin_ia32_vcvtph2uqq256_mask,
    BI__builtin_ia32_vcvtph2uqq512_mask,
    BI__builtin_ia32_vcvtph2uw128_mask,
    BI__builtin_ia32_vcvtph2uw256_mask,
    BI__builtin_ia32_vcvtph2uw512_mask,
    BI__builtin_ia32_vcvtph2w128_mask,
    BI__builtin_ia32_vcvtph2w256_mask,
    BI__builtin_ia32_vcvtph2w512_mask,
    BI__builtin_ia32_vcvtps2ibs128_mask,
    BI__builtin_ia32_vcvtps2ibs256_mask,
    BI__builtin_ia32_vcvtps2ibs512_mask,
    BI__builtin_ia32_vcvtps2iubs128_mask,
    BI__builtin_ia32_vcvtps2iubs256_mask,
    BI__builtin_ia32_vcvtps2iubs512_mask,
    BI__builtin_ia32_vcvtps2ph,
    BI__builtin_ia32_vcvtps2ph256,
    BI__builtin_ia32_vcvtps2ph256_mask,
    BI__builtin_ia32_vcvtps2ph512_mask,
    BI__builtin_ia32_vcvtps2ph_mask,
    BI__builtin_ia32_vcvtps2phx128_mask,
    BI__builtin_ia32_vcvtps2phx256_mask,
    BI__builtin_ia32_vcvtps2phx512_mask,
    BI__builtin_ia32_vcvtqq2ph128_mask,
    BI__builtin_ia32_vcvtqq2ph256_mask,
    BI__builtin_ia32_vcvtqq2ph512_mask,
    BI__builtin_ia32_vcvtsd2sh_round_mask,
    BI__builtin_ia32_vcvtsd2si32,
    BI__builtin_ia32_vcvtsd2usi32,
    BI__builtin_ia32_vcvtsh2sd_round_mask,
    BI__builtin_ia32_vcvtsh2si32,
    BI__builtin_ia32_vcvtsh2ss_round_mask,
    BI__builtin_ia32_vcvtsh2usi32,
    BI__builtin_ia32_vcvtsi2sh,
    BI__builtin_ia32_vcvtss2sh_round_mask,
    BI__builtin_ia32_vcvtss2si32,
    BI__builtin_ia32_vcvtss2usi32,
    BI__builtin_ia32_vcvttbf162ibs128,
    BI__builtin_ia32_vcvttbf162ibs256,
    BI__builtin_ia32_vcvttbf162ibs512,
    BI__builtin_ia32_vcvttbf162iubs128,
    BI__builtin_ia32_vcvttbf162iubs256,
    BI__builtin_ia32_vcvttbf162iubs512,
    BI__builtin_ia32_vcvttpd2dqs128_mask,
    BI__builtin_ia32_vcvttpd2dqs256_mask,
    BI__builtin_ia32_vcvttpd2dqs512_round_mask,
    BI__builtin_ia32_vcvttpd2qqs128_mask,
    BI__builtin_ia32_vcvttpd2qqs256_mask,
    BI__builtin_ia32_vcvttpd2qqs512_round_mask,
    BI__builtin_ia32_vcvttpd2udqs128_mask,
    BI__builtin_ia32_vcvttpd2udqs256_mask,
    BI__builtin_ia32_vcvttpd2udqs512_round_mask,
    BI__builtin_ia32_vcvttpd2uqqs128_mask,
    BI__builtin_ia32_vcvttpd2uqqs256_mask,
    BI__builtin_ia32_vcvttpd2uqqs512_round_mask,
    BI__builtin_ia32_vcvttph2dq128_mask,
    BI__builtin_ia32_vcvttph2dq256_mask,
    BI__builtin_ia32_vcvttph2dq512_mask,
    BI__builtin_ia32_vcvttph2ibs128_mask,
    BI__builtin_ia32_vcvttph2ibs256_mask,
    BI__builtin_ia32_vcvttph2ibs512_mask,
    BI__builtin_ia32_vcvttph2iubs128_mask,
    BI__builtin_ia32_vcvttph2iubs256_mask,
    BI__builtin_ia32_vcvttph2iubs512_mask,
    BI__builtin_ia32_vcvttph2qq128_mask,
    BI__builtin_ia32_vcvttph2qq256_mask,
    BI__builtin_ia32_vcvttph2qq512_mask,
    BI__builtin_ia32_vcvttph2udq128_mask,
    BI__builtin_ia32_vcvttph2udq256_mask,
    BI__builtin_ia32_vcvttph2udq512_mask,
    BI__builtin_ia32_vcvttph2uqq128_mask,
    BI__builtin_ia32_vcvttph2uqq256_mask,
    BI__builtin_ia32_vcvttph2uqq512_mask,
    BI__builtin_ia32_vcvttph2uw128_mask,
    BI__builtin_ia32_vcvttph2uw256_mask,
    BI__builtin_ia32_vcvttph2uw512_mask,
    BI__builtin_ia32_vcvttph2w128_mask,
    BI__builtin_ia32_vcvttph2w256_mask,
    BI__builtin_ia32_vcvttph2w512_mask,
    BI__builtin_ia32_vcvttps2dqs128_mask,
    BI__builtin_ia32_vcvttps2dqs256_mask,
    BI__builtin_ia32_vcvttps2dqs512_round_mask,
    BI__builtin_ia32_vcvttps2ibs128_mask,
    BI__builtin_ia32_vcvttps2ibs256_mask,
    BI__builtin_ia32_vcvttps2ibs512_mask,
    BI__builtin_ia32_vcvttps2iubs128_mask,
    BI__builtin_ia32_vcvttps2iubs256_mask,
    BI__builtin_ia32_vcvttps2iubs512_mask,
    BI__builtin_ia32_vcvttps2qqs128_mask,
    BI__builtin_ia32_vcvttps2qqs256_mask,
    BI__builtin_ia32_vcvttps2qqs512_round_mask,
    BI__builtin_ia32_vcvttps2udqs128_mask,
    BI__builtin_ia32_vcvttps2udqs256_mask,
    BI__builtin_ia32_vcvttps2udqs512_round_mask,
    BI__builtin_ia32_vcvttps2uqqs128_mask,
    BI__builtin_ia32_vcvttps2uqqs256_mask,
    BI__builtin_ia32_vcvttps2uqqs512_round_mask,
    BI__builtin_ia32_vcvttsd2si32,
    BI__builtin_ia32_vcvttsd2sis32,
    BI__builtin_ia32_vcvttsd2usi32,
    BI__builtin_ia32_vcvttsd2usis32,
    BI__builtin_ia32_vcvttsh2si32,
    BI__builtin_ia32_vcvttsh2usi32,
    BI__builtin_ia32_vcvttss2si32,
    BI__builtin_ia32_vcvttss2sis32,
    BI__builtin_ia32_vcvttss2usi32,
    BI__builtin_ia32_vcvttss2usis32,
    BI__builtin_ia32_vcvtudq2ph128_mask,
    BI__builtin_ia32_vcvtudq2ph256_mask,
    BI__builtin_ia32_vcvtudq2ph512_mask,
    BI__builtin_ia32_vcvtuqq2ph128_mask,
    BI__builtin_ia32_vcvtuqq2ph256_mask,
    BI__builtin_ia32_vcvtuqq2ph512_mask,
    BI__builtin_ia32_vcvtusi2sh,
    BI__builtin_ia32_vcvtuw2ph128_mask,
    BI__builtin_ia32_vcvtuw2ph256_mask,
    BI__builtin_ia32_vcvtuw2ph512_mask,
    BI__builtin_ia32_vcvtw2ph128_mask,
    BI__builtin_ia32_vcvtw2ph256_mask,
    BI__builtin_ia32_vcvtw2ph512_mask,
    BI__builtin_ia32_vdivbf16128,
    BI__builtin_ia32_vdivbf16256,
    BI__builtin_ia32_vdivbf16512,
    BI__builtin_ia32_vdpphps128,
    BI__builtin_ia32_vdpphps256,
    BI__builtin_ia32_vdpphps512,
    BI__builtin_ia32_vec_ext_v2di,
    BI__builtin_ia32_vec_ext_v4hi,
    BI__builtin_ia32_vec_ext_v4sf,
    BI__builtin_ia32_vec_ext_v4si,
    BI__builtin_ia32_vec_ext_v8hi,
    BI__builtin_ia32_vec_ext_v8si,
    BI__builtin_ia32_vec_ext_v16hi,
    BI__builtin_ia32_vec_ext_v16qi,
    BI__builtin_ia32_vec_ext_v32qi,
    BI__builtin_ia32_vec_set_v4hi,
    BI__builtin_ia32_vec_set_v4si,
    BI__builtin_ia32_vec_set_v8hi,
    BI__builtin_ia32_vec_set_v8si,
    BI__builtin_ia32_vec_set_v16hi,
    BI__builtin_ia32_vec_set_v16qi,
    BI__builtin_ia32_vec_set_v32qi,
    BI__builtin_ia32_vextractf128_pd256,
    BI__builtin_ia32_vextractf128_ps256,
    BI__builtin_ia32_vextractf128_si256,
    BI__builtin_ia32_vfcmaddcph128_mask,
    BI__builtin_ia32_vfcmaddcph128_maskz,
    BI__builtin_ia32_vfcmaddcph256_mask,
    BI__builtin_ia32_vfcmaddcph256_maskz,
    BI__builtin_ia32_vfcmaddcph512_mask,
    BI__builtin_ia32_vfcmaddcph512_mask3,
    BI__builtin_ia32_vfcmaddcph512_maskz,
    BI__builtin_ia32_vfcmaddcsh_mask,
    BI__builtin_ia32_vfcmaddcsh_maskz,
    BI__builtin_ia32_vfcmaddcsh_round_mask,
    BI__builtin_ia32_vfcmaddcsh_round_mask3,
    BI__builtin_ia32_vfcmulcph128_mask,
    BI__builtin_ia32_vfcmulcph256_mask,
    BI__builtin_ia32_vfcmulcph512_mask,
    BI__builtin_ia32_vfcmulcsh_mask,
    BI__builtin_ia32_vfmaddcph128_mask,
    BI__builtin_ia32_vfmaddcph128_maskz,
    BI__builtin_ia32_vfmaddcph256_mask,
    BI__builtin_ia32_vfmaddcph256_maskz,
    BI__builtin_ia32_vfmaddcph512_mask,
    BI__builtin_ia32_vfmaddcph512_mask3,
    BI__builtin_ia32_vfmaddcph512_maskz,
    BI__builtin_ia32_vfmaddcsh_mask,
    BI__builtin_ia32_vfmaddcsh_maskz,
    BI__builtin_ia32_vfmaddcsh_round_mask,
    BI__builtin_ia32_vfmaddcsh_round_mask3,
    BI__builtin_ia32_vfmaddpd512_mask,
    BI__builtin_ia32_vfmaddpd512_mask3,
    BI__builtin_ia32_vfmaddpd512_maskz,
    BI__builtin_ia32_vfmaddph512_mask,
    BI__builtin_ia32_vfmaddph512_mask3,
    BI__builtin_ia32_vfmaddph512_maskz,
    BI__builtin_ia32_vfmaddps512_mask,
    BI__builtin_ia32_vfmaddps512_mask3,
    BI__builtin_ia32_vfmaddps512_maskz,
    BI__builtin_ia32_vfmaddsd3_mask,
    BI__builtin_ia32_vfmaddsd3_mask3,
    BI__builtin_ia32_vfmaddsd3_maskz,
    BI__builtin_ia32_vfmaddsh3_mask,
    BI__builtin_ia32_vfmaddsh3_mask3,
    BI__builtin_ia32_vfmaddsh3_maskz,
    BI__builtin_ia32_vfmaddss3_mask,
    BI__builtin_ia32_vfmaddss3_mask3,
    BI__builtin_ia32_vfmaddss3_maskz,
    BI__builtin_ia32_vfmaddsubpd,
    BI__builtin_ia32_vfmaddsubpd256,
    BI__builtin_ia32_vfmaddsubpd512_mask,
    BI__builtin_ia32_vfmaddsubpd512_mask3,
    BI__builtin_ia32_vfmaddsubpd512_maskz,
    BI__builtin_ia32_vfmaddsubph,
    BI__builtin_ia32_vfmaddsubph256,
    BI__builtin_ia32_vfmaddsubph512_mask,
    BI__builtin_ia32_vfmaddsubph512_mask3,
    BI__builtin_ia32_vfmaddsubph512_maskz,
    BI__builtin_ia32_vfmaddsubps,
    BI__builtin_ia32_vfmaddsubps256,
    BI__builtin_ia32_vfmaddsubps512_mask,
    BI__builtin_ia32_vfmaddsubps512_mask3,
    BI__builtin_ia32_vfmaddsubps512_maskz,
    BI__builtin_ia32_vfmsubaddpd512_mask3,
    BI__builtin_ia32_vfmsubaddph512_mask3,
    BI__builtin_ia32_vfmsubaddps512_mask3,
    BI__builtin_ia32_vfmsubpd512_mask3,
    BI__builtin_ia32_vfmsubph512_mask3,
    BI__builtin_ia32_vfmsubps512_mask3,
    BI__builtin_ia32_vfmsubsd3_mask3,
    BI__builtin_ia32_vfmsubsh3_mask3,
    BI__builtin_ia32_vfmsubss3_mask3,
    BI__builtin_ia32_vfmulcph128_mask,
    BI__builtin_ia32_vfmulcph256_mask,
    BI__builtin_ia32_vfmulcph512_mask,
    BI__builtin_ia32_vfmulcsh_mask,
    BI__builtin_ia32_vfpclassbf16128_mask,
    BI__builtin_ia32_vfpclassbf16256_mask,
    BI__builtin_ia32_vfpclassbf16512_mask,
    BI__builtin_ia32_vfrczpd,
    BI__builtin_ia32_vfrczpd256,
    BI__builtin_ia32_vfrczps,
    BI__builtin_ia32_vfrczps256,
    BI__builtin_ia32_vfrczsd,
    BI__builtin_ia32_vfrczss,
    BI__builtin_ia32_vgetexpbf16128_mask,
    BI__builtin_ia32_vgetexpbf16256_mask,
    BI__builtin_ia32_vgetexpbf16512_mask,
    BI__builtin_ia32_vgetmantbf16128_mask,
    BI__builtin_ia32_vgetmantbf16256_mask,
    BI__builtin_ia32_vgetmantbf16512_mask,
    BI__builtin_ia32_vgf2p8affineinvqb_v16qi,
    BI__builtin_ia32_vgf2p8affineinvqb_v32qi,
    BI__builtin_ia32_vgf2p8affineinvqb_v64qi,
    BI__builtin_ia32_vgf2p8affineqb_v16qi,
    BI__builtin_ia32_vgf2p8affineqb_v32qi,
    BI__builtin_ia32_vgf2p8affineqb_v64qi,
    BI__builtin_ia32_vgf2p8mulb_v16qi,
    BI__builtin_ia32_vgf2p8mulb_v32qi,
    BI__builtin_ia32_vgf2p8mulb_v64qi,
    BI__builtin_ia32_vinsertf128_pd256,
    BI__builtin_ia32_vinsertf128_ps256,
    BI__builtin_ia32_vinsertf128_si256,
    BI__builtin_ia32_vmaxbf16128,
    BI__builtin_ia32_vmaxbf16256,
    BI__builtin_ia32_vmaxbf16512,
    BI__builtin_ia32_vminbf16128,
    BI__builtin_ia32_vminbf16256,
    BI__builtin_ia32_vminbf16512,
    BI__builtin_ia32_vminmaxbf16128,
    BI__builtin_ia32_vminmaxbf16256,
    BI__builtin_ia32_vminmaxbf16512,
    BI__builtin_ia32_vminmaxpd128_mask,
    BI__builtin_ia32_vminmaxpd256_mask,
    BI__builtin_ia32_vminmaxpd512_round_mask,
    BI__builtin_ia32_vminmaxph128_mask,
    BI__builtin_ia32_vminmaxph256_mask,
    BI__builtin_ia32_vminmaxph512_round_mask,
    BI__builtin_ia32_vminmaxps128_mask,
    BI__builtin_ia32_vminmaxps256_mask,
    BI__builtin_ia32_vminmaxps512_round_mask,
    BI__builtin_ia32_vminmaxsd_round_mask,
    BI__builtin_ia32_vminmaxsh_round_mask,
    BI__builtin_ia32_vminmaxss_round_mask,
    BI__builtin_ia32_vmulbf16128,
    BI__builtin_ia32_vmulbf16256,
    BI__builtin_ia32_vmulbf16512,
    BI__builtin_ia32_vp2intersect_d_128,
    BI__builtin_ia32_vp2intersect_d_256,
    BI__builtin_ia32_vp2intersect_d_512,
    BI__builtin_ia32_vp2intersect_q_128,
    BI__builtin_ia32_vp2intersect_q_256,
    BI__builtin_ia32_vp2intersect_q_512,
    BI__builtin_ia32_vpcomb,
    BI__builtin_ia32_vpcomd,
    BI__builtin_ia32_vpcomq,
    BI__builtin_ia32_vpcomub,
    BI__builtin_ia32_vpcomud,
    BI__builtin_ia32_vpcomuq,
    BI__builtin_ia32_vpcomuw,
    BI__builtin_ia32_vpcomw,
    BI__builtin_ia32_vpconflictdi_128,
    BI__builtin_ia32_vpconflictdi_256,
    BI__builtin_ia32_vpconflictdi_512,
    BI__builtin_ia32_vpconflictsi_128,
    BI__builtin_ia32_vpconflictsi_256,
    BI__builtin_ia32_vpconflictsi_512,
    BI__builtin_ia32_vpdpbssd128,
    BI__builtin_ia32_vpdpbssd256,
    BI__builtin_ia32_vpdpbssd512,
    BI__builtin_ia32_vpdpbssds128,
    BI__builtin_ia32_vpdpbssds256,
    BI__builtin_ia32_vpdpbssds512,
    BI__builtin_ia32_vpdpbsud128,
    BI__builtin_ia32_vpdpbsud256,
    BI__builtin_ia32_vpdpbsud512,
    BI__builtin_ia32_vpdpbsuds128,
    BI__builtin_ia32_vpdpbsuds256,
    BI__builtin_ia32_vpdpbsuds512,
    BI__builtin_ia32_vpdpbusd128,
    BI__builtin_ia32_vpdpbusd256,
    BI__builtin_ia32_vpdpbusd512,
    BI__builtin_ia32_vpdpbusds128,
    BI__builtin_ia32_vpdpbusds256,
    BI__builtin_ia32_vpdpbusds512,
    BI__builtin_ia32_vpdpbuud128,
    BI__builtin_ia32_vpdpbuud256,
    BI__builtin_ia32_vpdpbuud512,
    BI__builtin_ia32_vpdpbuuds128,
    BI__builtin_ia32_vpdpbuuds256,
    BI__builtin_ia32_vpdpbuuds512,
    BI__builtin_ia32_vpdpwssd128,
    BI__builtin_ia32_vpdpwssd256,
    BI__builtin_ia32_vpdpwssd512,
    BI__builtin_ia32_vpdpwssds128,
    BI__builtin_ia32_vpdpwssds256,
    BI__builtin_ia32_vpdpwssds512,
    BI__builtin_ia32_vpdpwsud128,
    BI__builtin_ia32_vpdpwsud256,
    BI__builtin_ia32_vpdpwsud512,
    BI__builtin_ia32_vpdpwsuds128,
    BI__builtin_ia32_vpdpwsuds256,
    BI__builtin_ia32_vpdpwsuds512,
    BI__builtin_ia32_vpdpwusd128,
    BI__builtin_ia32_vpdpwusd256,
    BI__builtin_ia32_vpdpwusd512,
    BI__builtin_ia32_vpdpwusds128,
    BI__builtin_ia32_vpdpwusds256,
    BI__builtin_ia32_vpdpwusds512,
    BI__builtin_ia32_vpdpwuud128,
    BI__builtin_ia32_vpdpwuud256,
    BI__builtin_ia32_vpdpwuud512,
    BI__builtin_ia32_vpdpwuuds128,
    BI__builtin_ia32_vpdpwuuds256,
    BI__builtin_ia32_vpdpwuuds512,
    BI__builtin_ia32_vperm2f128_pd256,
    BI__builtin_ia32_vperm2f128_ps256,
    BI__builtin_ia32_vperm2f128_si256,
    BI__builtin_ia32_vpermi2vard128,
    BI__builtin_ia32_vpermi2vard256,
    BI__builtin_ia32_vpermi2vard512,
    BI__builtin_ia32_vpermi2varhi128,
    BI__builtin_ia32_vpermi2varhi256,
    BI__builtin_ia32_vpermi2varhi512,
    BI__builtin_ia32_vpermi2varpd128,
    BI__builtin_ia32_vpermi2varpd256,
    BI__builtin_ia32_vpermi2varpd512,
    BI__builtin_ia32_vpermi2varps128,
    BI__builtin_ia32_vpermi2varps256,
    BI__builtin_ia32_vpermi2varps512,
    BI__builtin_ia32_vpermi2varq128,
    BI__builtin_ia32_vpermi2varq256,
    BI__builtin_ia32_vpermi2varq512,
    BI__builtin_ia32_vpermi2varqi128,
    BI__builtin_ia32_vpermi2varqi256,
    BI__builtin_ia32_vpermi2varqi512,
    BI__builtin_ia32_vpermil2pd,
    BI__builtin_ia32_vpermil2pd256,
    BI__builtin_ia32_vpermil2ps,
    BI__builtin_ia32_vpermil2ps256,
    BI__builtin_ia32_vpermilpd,
    BI__builtin_ia32_vpermilpd256,
    BI__builtin_ia32_vpermilpd512,
    BI__builtin_ia32_vpermilps,
    BI__builtin_ia32_vpermilps256,
    BI__builtin_ia32_vpermilps512,
    BI__builtin_ia32_vpermilvarpd,
    BI__builtin_ia32_vpermilvarpd256,
    BI__builtin_ia32_vpermilvarpd512,
    BI__builtin_ia32_vpermilvarps,
    BI__builtin_ia32_vpermilvarps256,
    BI__builtin_ia32_vpermilvarps512,
    BI__builtin_ia32_vphaddbd,
    BI__builtin_ia32_vphaddbq,
    BI__builtin_ia32_vphaddbw,
    BI__builtin_ia32_vphadddq,
    BI__builtin_ia32_vphaddubd,
    BI__builtin_ia32_vphaddubq,
    BI__builtin_ia32_vphaddubw,
    BI__builtin_ia32_vphaddudq,
    BI__builtin_ia32_vphadduwd,
    BI__builtin_ia32_vphadduwq,
    BI__builtin_ia32_vphaddwd,
    BI__builtin_ia32_vphaddwq,
    BI__builtin_ia32_vphsubbw,
    BI__builtin_ia32_vphsubdq,
    BI__builtin_ia32_vphsubwd,
    BI__builtin_ia32_vpmacsdd,
    BI__builtin_ia32_vpmacsdqh,
    BI__builtin_ia32_vpmacsdql,
    BI__builtin_ia32_vpmacssdd,
    BI__builtin_ia32_vpmacssdqh,
    BI__builtin_ia32_vpmacssdql,
    BI__builtin_ia32_vpmacsswd,
    BI__builtin_ia32_vpmacssww,
    BI__builtin_ia32_vpmacswd,
    BI__builtin_ia32_vpmacsww,
    BI__builtin_ia32_vpmadcsswd,
    BI__builtin_ia32_vpmadcswd,
    BI__builtin_ia32_vpmadd52huq128,
    BI__builtin_ia32_vpmadd52huq256,
    BI__builtin_ia32_vpmadd52huq512,
    BI__builtin_ia32_vpmadd52luq128,
    BI__builtin_ia32_vpmadd52luq256,
    BI__builtin_ia32_vpmadd52luq512,
    BI__builtin_ia32_vpmultishiftqb128,
    BI__builtin_ia32_vpmultishiftqb256,
    BI__builtin_ia32_vpmultishiftqb512,
    BI__builtin_ia32_vpperm,
    BI__builtin_ia32_vprotbi,
    BI__builtin_ia32_vprotdi,
    BI__builtin_ia32_vprotqi,
    BI__builtin_ia32_vprotwi,
    BI__builtin_ia32_vpshab,
    BI__builtin_ia32_vpshad,
    BI__builtin_ia32_vpshaq,
    BI__builtin_ia32_vpshaw,
    BI__builtin_ia32_vpshlb,
    BI__builtin_ia32_vpshld,
    BI__builtin_ia32_vpshldd128,
    BI__builtin_ia32_vpshldd256,
    BI__builtin_ia32_vpshldd512,
    BI__builtin_ia32_vpshldq128,
    BI__builtin_ia32_vpshldq256,
    BI__builtin_ia32_vpshldq512,
    BI__builtin_ia32_vpshldw128,
    BI__builtin_ia32_vpshldw256,
    BI__builtin_ia32_vpshldw512,
    BI__builtin_ia32_vpshlq,
    BI__builtin_ia32_vpshlw,
    BI__builtin_ia32_vpshrdd128,
    BI__builtin_ia32_vpshrdd256,
    BI__builtin_ia32_vpshrdd512,
    BI__builtin_ia32_vpshrdq128,
    BI__builtin_ia32_vpshrdq256,
    BI__builtin_ia32_vpshrdq512,
    BI__builtin_ia32_vpshrdw128,
    BI__builtin_ia32_vpshrdw256,
    BI__builtin_ia32_vpshrdw512,
    BI__builtin_ia32_vpshufbitqmb128_mask,
    BI__builtin_ia32_vpshufbitqmb256_mask,
    BI__builtin_ia32_vpshufbitqmb512_mask,
    BI__builtin_ia32_vrcpbf16128_mask,
    BI__builtin_ia32_vrcpbf16256_mask,
    BI__builtin_ia32_vrcpbf16512_mask,
    BI__builtin_ia32_vreducebf16128_mask,
    BI__builtin_ia32_vreducebf16256_mask,
    BI__builtin_ia32_vreducebf16512_mask,
    BI__builtin_ia32_vrndscalebf16_128_mask,
    BI__builtin_ia32_vrndscalebf16_256_mask,
    BI__builtin_ia32_vrndscalebf16_mask,
    BI__builtin_ia32_vrsqrtbf16128_mask,
    BI__builtin_ia32_vrsqrtbf16256_mask,
    BI__builtin_ia32_vrsqrtbf16512_mask,
    BI__builtin_ia32_vscalefbf16128_mask,
    BI__builtin_ia32_vscalefbf16256_mask,
    BI__builtin_ia32_vscalefbf16512_mask,
    BI__builtin_ia32_vsha512msg1,
    BI__builtin_ia32_vsha512msg2,
    BI__builtin_ia32_vsha512rnds2,
    BI__builtin_ia32_vsm3msg1,
    BI__builtin_ia32_vsm3msg2,
    BI__builtin_ia32_vsm3rnds2,
    BI__builtin_ia32_vsm4key4128,
    BI__builtin_ia32_vsm4key4256,
    BI__builtin_ia32_vsm4key4512,
    BI__builtin_ia32_vsm4rnds4128,
    BI__builtin_ia32_vsm4rnds4256,
    BI__builtin_ia32_vsm4rnds4512,
    BI__builtin_ia32_vsubbf16128,
    BI__builtin_ia32_vsubbf16256,
    BI__builtin_ia32_vsubbf16512,
    BI__builtin_ia32_vtestcpd,
    BI__builtin_ia32_vtestcpd256,
    BI__builtin_ia32_vtestcps,
    BI__builtin_ia32_vtestcps256,
    BI__builtin_ia32_vtestnzcpd,
    BI__builtin_ia32_vtestnzcpd256,
    BI__builtin_ia32_vtestnzcps,
    BI__builtin_ia32_vtestnzcps256,
    BI__builtin_ia32_vtestzpd,
    BI__builtin_ia32_vtestzpd256,
    BI__builtin_ia32_vtestzps,
    BI__builtin_ia32_vtestzps256,
    BI__builtin_ia32_vzeroall,
    BI__builtin_ia32_vzeroupper,
    BI__builtin_ia32_wbinvd,
    BI__builtin_ia32_wbnoinvd,
    BI__builtin_ia32_writeeflags_u32,
    BI__builtin_ia32_wrpkru,
    BI__builtin_ia32_wrssd,
    BI__builtin_ia32_wrussd,
    BI__builtin_ia32_xabort,
    BI__builtin_ia32_xbegin,
    BI__builtin_ia32_xend,
    BI__builtin_ia32_xgetbv,
    BI__builtin_ia32_xresldtrk,
    BI__builtin_ia32_xrstor,
    BI__builtin_ia32_xrstors,
    BI__builtin_ia32_xsave,
    BI__builtin_ia32_xsavec,
    BI__builtin_ia32_xsaveopt,
    BI__builtin_ia32_xsaves,
    BI__builtin_ia32_xsetbv,
    BI__builtin_ia32_xsusldtrk,
    BI__builtin_ia32_xtest,

#endif // GET_BUILTIN_ENUMERATORS

#ifdef GET_BUILTIN_STR_TABLE

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinStringsStorage[] =
  "\0"
  "_AddressOfReturnAddress\0"
  "v*\0"
  "nh\0"
  "_BitScanForward\0"
  "UcUNi*UNi\0"
  "_BitScanReverse\0"
  "_InterlockedAnd64\0"
  "WiWiD*Wi\0"
  "_InterlockedDecrement64\0"
  "WiWiD*\0"
  "_InterlockedExchange64\0"
  "_InterlockedExchangeAdd64\0"
  "_InterlockedExchangeSub64\0"
  "_InterlockedIncrement64\0"
  "_InterlockedOr64\0"
  "_InterlockedXor64\0"
  "_ReadBarrier\0"
  "v\0"
  "_ReadWriteBarrier\0"
  "_WriteBarrier\0"
  "__cpuid\0"
  "vi*i\0"
  "__cpuidex\0"
  "vi*ii\0"
  "__emul\0"
  "LLiii\0"
  "nch\0"
  "__emulu\0"
  "ULLiUiUi\0"
  "__int2c\0"
  "nhr\0"
  "__rdtsc\0"
  "UOi\0"
  "__readfsbyte\0"
  "UcUNi\0"
  "__readfsdword\0"
  "UNiUNi\0"
  "__readfsqword\0"
  "ULLiUNi\0"
  "__readfsword\0"
  "UsUNi\0"
  "__readgsbyte\0"
  "__readgsdword\0"
  "__readgsqword\0"
  "__readgsword\0"
  "__stosb\0"
  "vUc*Ucz\0"
  "__ud2\0"
  "_m_prefetch\0"
  "vv*\0"
  "nc\0"
  "_m_prefetchw\0"
  "vvDC*\0"
  "_mm_clflush\0"
  "vvC*\0"
  "_mm_getcsr\0"
  "Ui\0"
  "_mm_lfence\0"
  "_mm_mfence\0"
  "_mm_pause\0"
  "_mm_prefetch\0"
  "vcC*i\0"
  "_mm_setcsr\0"
  "vUi\0"
  "_mm_sfence\0"
  "_xgetbv\0"
  "UWiUi\0"
  "_xsetbv\0"
  "vUiUWi\0"
  "aadd32\0"
  "vv*Si\0"
  "n\0"
  "raoint\0"
  "aand32\0"
  "addcarryx_u32\0"
  "UcUcUiUiUi*\0"
  "nE\0"
  "addpd512\0"
  "V8dV8dV8dIi\0"
  "ncV:512:\0"
  "avx512f\0"
  "addph512\0"
  "V32xV32xV32xIi\0"
  "avx512fp16\0"
  "addps512\0"
  "V16fV16fV16fIi\0"
  "addsd_round_mask\0"
  "V2dV2dV2dV2dUcIi\0"
  "ncV:128:\0"
  "addsh_round_mask\0"
  "V8xV8xV8xV8xUcIi\0"
  "addss_round_mask\0"
  "V4fV4fV4fV4fUcIi\0"
  "addsubpd\0"
  "V2dV2dV2d\0"
  "ncEV:128:\0"
  "sse3\0"
  "addsubpd256\0"
  "V4dV4dV4d\0"
  "ncEV:256:\0"
  "avx\0"
  "addsubps\0"
  "V4fV4fV4f\0"
  "addsubps256\0"
  "V8fV8fV8f\0"
  "aesdec128\0"
  "V2OiV2OiV2Oi\0"
  "aes\0"
  "aesdec128kl_u8\0"
  "UcV2Oi*V2OivC*\0"
  "nV:128:\0"
  "kl\0"
  "aesdec256\0"
  "V4OiV4OiV4Oi\0"
  "ncV:256:\0"
  "vaes\0"
  "aesdec256kl_u8\0"
  "aesdec512\0"
  "V8OiV8OiV8Oi\0"
  "avx512f,vaes\0"
  "aesdeclast128\0"
  "aesdeclast256\0"
  "aesdeclast512\0"
  "aesdecwide128kl_u8\0"
  "UcV2Oi*V2OiC*vC*\0"
  "kl,widekl\0"
  "aesdecwide256kl_u8\0"
  "aesenc128\0"
  "aesenc128kl_u8\0"
  "aesenc256\0"
  "aesenc256kl_u8\0"
  "aesenc512\0"
  "aesenclast128\0"
  "aesenclast256\0"
  "aesenclast512\0"
  "aesencwide128kl_u8\0"
  "aesencwide256kl_u8\0"
  "aesimc128\0"
  "V2OiV2Oi\0"
  "aeskeygenassist128\0"
  "V2OiV2OiIc\0"
  "alignd128\0"
  "V4iV4iV4iIi\0"
  "avx512vl\0"
  "alignd256\0"
  "V8iV8iV8iIi\0"
  "alignd512\0"
  "V16iV16iV16iIi\0"
  "ncEV:512:\0"
  "alignq128\0"
  "V2OiV2OiV2OiIi\0"
  "alignq256\0"
  "V4OiV4OiV4OiIi\0"
  "alignq512\0"
  "V8OiV8OiV8OiIi\0"
  "aor32\0"
  "axor32\0"
  "bextr_u32\0"
  "UiUiUi\0"
  "ncE\0"
  "bmi\0"
  "bextri_u32\0"
  "UiUiIUi\0"
  "tbm\0"
  "blendpd\0"
  "V2dV2dV2dIi\0"
  "sse4.1\0"
  "blendpd256\0"
  "V4dV4dV4dIi\0"
  "blendps\0"
  "V4fV4fV4fIi\0"
  "blendps256\0"
  "V8fV8fV8fIi\0"
  "blendvpd\0"
  "V2dV2dV2dV2d\0"
  "blendvpd256\0"
  "V4dV4dV4dV4d\0"
  "blendvps\0"
  "V4fV4fV4fV4f\0"
  "blendvps256\0"
  "V8fV8fV8fV8f\0"
  "bzhi_si\0"
  "bmi2\0"
  "cldemote\0"
  "clflush\0"
  "sse2\0"
  "clflushopt\0"
  "clrssbsy\0"
  "shstk\0"
  "clwb\0"
  "clzero\0"
  "cmpb128_mask\0"
  "UsV16cV16cIiUs\0"
  "avx512vl,avx512bw\0"
  "cmpb256_mask\0"
  "UiV32cV32cIiUi\0"
  "cmpb512_mask\0"
  "UOiV64cV64cIiUOi\0"
  "avx512bw\0"
  "cmpd128_mask\0"
  "UcV4iV4iIiUc\0"
  "cmpd256_mask\0"
  "UcV8iV8iIiUc\0"
  "cmpd512_mask\0"
  "UsV16iV16iIiUs\0"
  "cmpeqpd\0"
  "cnV:128:\0"
  "cmpeqps\0"
  "sse\0"
  "cmpeqsd\0"
  "cmpeqss\0"
  "cmplepd\0"
  "cmpleps\0"
  "cmplesd\0"
  "cmpless\0"
  "cmpltpd\0"
  "cmpltps\0"
  "cmpltsd\0"
  "cmpltss\0"
  "cmpneqpd\0"
  "cmpneqps\0"
  "cmpneqsd\0"
  "cmpneqss\0"
  "cmpnlepd\0"
  "cmpnleps\0"
  "cmpnlesd\0"
  "cmpnless\0"
  "cmpnltpd\0"
  "cmpnltps\0"
  "cmpnltsd\0"
  "cmpnltss\0"
  "cmpordpd\0"
  "cmpordps\0"
  "cmpordsd\0"
  "cmpordss\0"
  "cmppd\0"
  "V2dV2dV2dIc\0"
  "cmppd128_mask\0"
  "UcV2dV2dIiUc\0"
  "cmppd256\0"
  "V4dV4dV4dIc\0"
  "cmppd256_mask\0"
  "UcV4dV4dIiUc\0"
  "cmppd512_mask\0"
  "UcV8dV8dIiUcIi\0"
  "cmpph128_mask\0"
  "UcV8xV8xIiUc\0"
  "avx512fp16,avx512vl\0"
  "cmpph256_mask\0"
  "UsV16xV16xIiUs\0"
  "cmpph512_mask\0"
  "UiV32xV32xIiUiIi\0"
  "cmpps\0"
  "V4fV4fV4fIc\0"
  "cmpps128_mask\0"
  "UcV4fV4fIiUc\0"
  "cmpps256\0"
  "V8fV8fV8fIc\0"
  "cmpps256_mask\0"
  "UcV8fV8fIiUc\0"
  "cmpps512_mask\0"
  "UsV16fV16fIiUsIi\0"
  "cmpq128_mask\0"
  "UcV2OiV2OiIiUc\0"
  "cmpq256_mask\0"
  "UcV4OiV4OiIiUc\0"
  "cmpq512_mask\0"
  "UcV8OiV8OiIiUc\0"
  "cmpsd\0"
  "cmpsd_mask\0"
  "UcV2dV2dIiUcIi\0"
  "cmpsh_mask\0"
  "UcV8xV8xIiUcIi\0"
  "cmpss\0"
  "cmpss_mask\0"
  "UcV4fV4fIiUcIi\0"
  "cmpunordpd\0"
  "cmpunordps\0"
  "cmpunordsd\0"
  "cmpunordss\0"
  "cmpw128_mask\0"
  "UcV8sV8sIiUc\0"
  "cmpw256_mask\0"
  "UsV16sV16sIiUs\0"
  "cmpw512_mask\0"
  "UiV32sV32sIiUi\0"
  "comieq\0"
  "iV4fV4f\0"
  "comige\0"
  "comigt\0"
  "comile\0"
  "comilt\0"
  "comineq\0"
  "comisdeq\0"
  "iV2dV2d\0"
  "comisdge\0"
  "comisdgt\0"
  "comisdle\0"
  "comisdlt\0"
  "comisdneq\0"
  "compressdf128_mask\0"
  "V2dV2dV2dUc\0"
  "compressdf256_mask\0"
  "V4dV4dV4dUc\0"
  "compressdf512_mask\0"
  "V8dV8dV8dUc\0"
  "compressdi128_mask\0"
  "V2OiV2OiV2OiUc\0"
  "compressdi256_mask\0"
  "V4OiV4OiV4OiUc\0"
  "compressdi512_mask\0"
  "V8OiV8OiV8OiUc\0"
  "compresshi128_mask\0"
  "V8sV8sV8sUc\0"
  "avx512vl,avx512vbmi2\0"
  "compresshi256_mask\0"
  "V16sV16sV16sUs\0"
  "compresshi512_mask\0"
  "V32sV32sV32sUi\0"
  "avx512vbmi2\0"
  "compressqi128_mask\0"
  "V16cV16cV16cUs\0"
  "compressqi256_mask\0"
  "V32cV32cV32cUi\0"
  "compressqi512_mask\0"
  "V64cV64cV64cUOi\0"
  "compresssf128_mask\0"
  "V4fV4fV4fUc\0"
  "compresssf256_mask\0"
  "V8fV8fV8fUc\0"
  "compresssf512_mask\0"
  "V16fV16fV16fUs\0"
  "compresssi128_mask\0"
  "V4iV4iV4iUc\0"
  "compresssi256_mask\0"
  "V8iV8iV8iUc\0"
  "compresssi512_mask\0"
  "V16iV16iV16iUs\0"
  "compressstoredf128_mask\0"
  "vV2d*V2dUc\0"
  "compressstoredf256_mask\0"
  "vV4d*V4dUc\0"
  "nV:256:\0"
  "compressstoredf512_mask\0"
  "vV8d*V8dUc\0"
  "nV:512:\0"
  "compressstoredi128_mask\0"
  "vV2Oi*V2OiUc\0"
  "compressstoredi256_mask\0"
  "vV4Oi*V4OiUc\0"
  "compressstoredi512_mask\0"
  "vV8Oi*V8OiUc\0"
  "compressstorehi128_mask\0"
  "vV8s*V8sUc\0"
  "compressstorehi256_mask\0"
  "vV16s*V16sUs\0"
  "compressstorehi512_mask\0"
  "vV32s*V32sUi\0"
  "compressstoreqi128_mask\0"
  "vV16c*V16cUs\0"
  "compressstoreqi256_mask\0"
  "vV32c*V32cUi\0"
  "compressstoreqi512_mask\0"
  "vV64c*V64cUOi\0"
  "compressstoresf128_mask\0"
  "vV4f*V4fUc\0"
  "compressstoresf256_mask\0"
  "vV8f*V8fUc\0"
  "compressstoresf512_mask\0"
  "vV16f*V16fUs\0"
  "compressstoresi128_mask\0"
  "vV4i*V4iUc\0"
  "compressstoresi256_mask\0"
  "vV8i*V8iUc\0"
  "compressstoresi512_mask\0"
  "vV16i*V16iUs\0"
  "crc32hi\0"
  "UiUiUs\0"
  "crc32\0"
  "crc32qi\0"
  "UiUiUc\0"
  "crc32si\0"
  "cvtb2mask128\0"
  "UsV16c\0"
  "avx512bw,avx512vl\0"
  "cvtb2mask256\0"
  "UiV32c\0"
  "cvtb2mask512\0"
  "UOiV64c\0"
  "cvtd2mask128\0"
  "UcV4i\0"
  "avx512dq,avx512vl\0"
  "cvtd2mask256\0"
  "UcV8i\0"
  "cvtd2mask512\0"
  "UsV16i\0"
  "avx512dq\0"
  "cvtdq2ps512_mask\0"
  "V16fV16iV16fUsIi\0"
  "cvtmask2b128\0"
  "V16cUs\0"
  "cvtmask2b256\0"
  "V32cUi\0"
  "cvtmask2b512\0"
  "V64cUOi\0"
  "cvtmask2d128\0"
  "V4iUc\0"
  "cvtmask2d256\0"
  "V8iUc\0"
  "cvtmask2d512\0"
  "V16iUs\0"
  "cvtmask2q128\0"
  "V2OiUc\0"
  "cvtmask2q256\0"
  "V4OiUc\0"
  "cvtmask2q512\0"
  "V8OiUc\0"
  "cvtmask2w128\0"
  "V8sUc\0"
  "cvtmask2w256\0"
  "V16sUs\0"
  "cvtmask2w512\0"
  "V32sUi\0"
  "cvtne2ps2bf16_128\0"
  "V8yV4fV4f\0"
  "avx512bf16,avx512vl\0"
  "cvtne2ps2bf16_256\0"
  "V16yV8fV8f\0"
  "cvtne2ps2bf16_512\0"
  "V32yV16fV16f\0"
  "avx512bf16\0"
  "cvtneps2bf16_128_mask\0"
  "V8yV4fV8yUc\0"
  "cvtneps2bf16_256_mask\0"
  "V8yV8fV8yUc\0"
  "cvtneps2bf16_512_mask\0"
  "V16yV16fV16yUs\0"
  "cvtpd2dq\0"
  "V2OiV2d\0"
  "cvtpd2dq128_mask\0"
  "V4iV2dV4iUc\0"
  "cvtpd2dq256\0"
  "V4iV4d\0"
  "cvtpd2dq512_mask\0"
  "V8iV8dV8iUcIi\0"
  "cvtpd2ps\0"
  "V4fV2d\0"
  "cvtpd2ps256\0"
  "V4fV4d\0"
  "cvtpd2ps512_mask\0"
  "V8fV8dV8fUcIi\0"
  "cvtpd2ps_mask\0"
  "V4fV2dV4fUc\0"
  "cvtpd2qq128_mask\0"
  "V2OiV2dV2OiUc\0"
  "avx512vl,avx512dq\0"
  "cvtpd2qq256_mask\0"
  "V4OiV4dV4OiUc\0"
  "cvtpd2qq512_mask\0"
  "V8OiV8dV8OiUcIi\0"
  "cvtpd2udq128_mask\0"
  "cvtpd2udq256_mask\0"
  "V4iV4dV4iUc\0"
  "cvtpd2udq512_mask\0"
  "cvtpd2uqq128_mask\0"
  "cvtpd2uqq256_mask\0"
  "cvtpd2uqq512_mask\0"
  "cvtps2dq\0"
  "V4iV4f\0"
  "cvtps2dq256\0"
  "V8iV8f\0"
  "cvtps2dq512_mask\0"
  "V16iV16fV16iUsIi\0"
  "cvtps2pd512_mask\0"
  "V8dV8fV8dUcIi\0"
  "cvtps2qq128_mask\0"
  "V2OiV4fV2OiUc\0"
  "cvtps2qq256_mask\0"
  "V4OiV4fV4OiUc\0"
  "cvtps2qq512_mask\0"
  "V8OiV8fV8OiUcIi\0"
  "cvtps2udq128_mask\0"
  "V4iV4fV4iUc\0"
  "cvtps2udq256_mask\0"
  "V8iV8fV8iUc\0"
  "cvtps2udq512_mask\0"
  "cvtps2uqq128_mask\0"
  "cvtps2uqq256_mask\0"
  "cvtps2uqq512_mask\0"
  "cvtq2mask128\0"
  "UcV2Oi\0"
  "cvtq2mask256\0"
  "UcV4Oi\0"
  "cvtq2mask512\0"
  "UcV8Oi\0"
  "cvtqq2pd512_mask\0"
  "V8dV8OiV8dUcIi\0"
  "cvtqq2ps128_mask\0"
  "V4fV2OiV4fUc\0"
  "cvtqq2ps512_mask\0"
  "V8fV8OiV8fUcIi\0"
  "cvtsd2si\0"
  "iV2d\0"
  "cvtsd2ss\0"
  "V4fV4fV2d\0"
  "cvtsd2ss_round_mask\0"
  "V4fV4fV2dV4fUcIi\0"
  "cvtsi2ss32\0"
  "V4fV4fiIi\0"
  "cvtss2sd_round_mask\0"
  "V2dV2dV4fV2dUcIi\0"
  "cvtss2si\0"
  "iV4f\0"
  "cvttpd2dq\0"
  "V4iV2d\0"
  "cvttpd2dq128_mask\0"
  "cvttpd2dq256\0"
  "cvttpd2dq512_mask\0"
  "cvttpd2qq128_mask\0"
  "cvttpd2qq256_mask\0"
  "cvttpd2qq512_mask\0"
  "cvttpd2udq128_mask\0"
  "cvttpd2udq256_mask\0"
  "cvttpd2udq512_mask\0"
  "cvttpd2uqq128_mask\0"
  "cvttpd2uqq256_mask\0"
  "cvttpd2uqq512_mask\0"
  "cvttps2dq\0"
  "cvttps2dq256\0"
  "cvttps2dq512_mask\0"
  "cvttps2qq128_mask\0"
  "cvttps2qq256_mask\0"
  "cvttps2qq512_mask\0"
  "cvttps2udq128_mask\0"
  "cvttps2udq256_mask\0"
  "cvttps2udq512_mask\0"
  "cvttps2uqq128_mask\0"
  "cvttps2uqq256_mask\0"
  "cvttps2uqq512_mask\0"
  "cvttsd2si\0"
  "cvttss2si\0"
  "cvtudq2ps512_mask\0"
  "cvtuqq2pd512_mask\0"
  "cvtuqq2ps128_mask\0"
  "cvtuqq2ps512_mask\0"
  "cvtusi2ss32\0"
  "V4fV4fUiIi\0"
  "cvtw2mask128\0"
  "UcV8s\0"
  "cvtw2mask256\0"
  "UsV16s\0"
  "cvtw2mask512\0"
  "UiV32s\0"
  "dbpsadbw128\0"
  "V8sV16cV16cIi\0"
  "dbpsadbw256\0"
  "V16sV32cV32cIi\0"
  "dbpsadbw512\0"
  "V32sV64cV64cIi\0"
  "directstore_u32\0"
  "vUi*Ui\0"
  "movdiri\0"
  "divpd512\0"
  "divph512\0"
  "divps512\0"
  "divsd_round_mask\0"
  "divsh_round_mask\0"
  "divss_round_mask\0"
  "dpbf16ps_128\0"
  "V4fV4fV8yV8y\0"
  "dpbf16ps_256\0"
  "V8fV8fV16yV16y\0"
  "dpbf16ps_512\0"
  "V16fV16fV32yV32y\0"
  "dppd\0"
  "dpps\0"
  "dpps256\0"
  "emms\0"
  "mmx\0"
  "encodekey128_u32\0"
  "UiUiV2Oiv*\0"
  "encodekey256_u32\0"
  "UiUiV2OiV2Oiv*\0"
  "enqcmd\0"
  "Ucv*vC*\0"
  "enqcmds\0"
  "expanddf128_mask\0"
  "expanddf256_mask\0"
  "expanddf512_mask\0"
  "expanddi128_mask\0"
  "expanddi256_mask\0"
  "expanddi512_mask\0"
  "expandhi128_mask\0"
  "expandhi256_mask\0"
  "expandhi512_mask\0"
  "expandloaddf128_mask\0"
  "V2dV2dC*V2dUc\0"
  "expandloaddf256_mask\0"
  "V4dV4dC*V4dUc\0"
  "expandloaddf512_mask\0"
  "V8dV8dC*V8dUc\0"
  "expandloaddi128_mask\0"
  "V4iV2OiC*V2OiUc\0"
  "expandloaddi256_mask\0"
  "V4OiV4OiC*V4OiUc\0"
  "expandloaddi512_mask\0"
  "V8OiV8OiC*V8OiUc\0"
  "expandloadhi128_mask\0"
  "V8sV8sC*V8sUc\0"
  "expandloadhi256_mask\0"
  "V16sV16sC*V16sUs\0"
  "expandloadhi512_mask\0"
  "V32sV32sC*V32sUi\0"
  "expandloadqi128_mask\0"
  "V16cV16cC*V16cUs\0"
  "expandloadqi256_mask\0"
  "V32cV32cC*V32cUi\0"
  "expandloadqi512_mask\0"
  "V64cV64cC*V64cUOi\0"
  "expandloadsf128_mask\0"
  "V4fV4fC*V4fUc\0"
  "expandloadsf256_mask\0"
  "V8fV8fC*V8fUc\0"
  "expandloadsf512_mask\0"
  "V16fV16fC*V16fUs\0"
  "expandloadsi128_mask\0"
  "V4iV4iC*V4iUc\0"
  "expandloadsi256_mask\0"
  "V8iV8iC*V8iUc\0"
  "expandloadsi512_mask\0"
  "V16iV16iC*V16iUs\0"
  "expandqi128_mask\0"
  "expandqi256_mask\0"
  "expandqi512_mask\0"
  "expandsf128_mask\0"
  "expandsf256_mask\0"
  "expandsf512_mask\0"
  "expandsi128_mask\0"
  "expandsi256_mask\0"
  "expandsi512_mask\0"
  "extract128i256\0"
  "V2OiV4OiIi\0"
  "avx2\0"
  "extractf32x4_256_mask\0"
  "V4fV8fIiV4fUc\0"
  "extractf32x4_mask\0"
  "V4fV16fIiV4fUc\0"
  "extractf32x8_mask\0"
  "V8fV16fIiV8fUc\0"
  "extractf64x2_256_mask\0"
  "V2dV4dIiV2dUc\0"
  "extractf64x2_512_mask\0"
  "V2dV8dIiV2dUc\0"
  "extractf64x4_mask\0"
  "V4dV8dIiV4dUc\0"
  "extracti32x4_256_mask\0"
  "V4iV8iIiV4iUc\0"
  "extracti32x4_mask\0"
  "V4iV16iIiV4iUc\0"
  "extracti32x8_mask\0"
  "V8iV16iIiV8iUc\0"
  "extracti64x2_256_mask\0"
  "V2OiV4OiIiV2OiUc\0"
  "extracti64x2_512_mask\0"
  "V2OiV8OiIiV2OiUc\0"
  "extracti64x4_mask\0"
  "V4OiV8OiIiV4OiUc\0"
  "extrq\0"
  "V2OiV2OiV16c\0"
  "sse4a\0"
  "extrqi\0"
  "V2OiV2OiIcIc\0"
  "fixupimmpd128_mask\0"
  "V2dV2dV2dV2OiIiUc\0"
  "fixupimmpd128_maskz\0"
  "fixupimmpd256_mask\0"
  "V4dV4dV4dV4OiIiUc\0"
  "fixupimmpd256_maskz\0"
  "fixupimmpd512_mask\0"
  "V8dV8dV8dV8OiIiUcIi\0"
  "fixupimmpd512_maskz\0"
  "fixupimmps128_mask\0"
  "V4fV4fV4fV4iIiUc\0"
  "fixupimmps128_maskz\0"
  "fixupimmps256_mask\0"
  "V8fV8fV8fV8iIiUc\0"
  "fixupimmps256_maskz\0"
  "fixupimmps512_mask\0"
  "V16fV16fV16fV16iIiUsIi\0"
  "fixupimmps512_maskz\0"
  "fixupimmsd_mask\0"
  "V2dV2dV2dV2OiIiUcIi\0"
  "fixupimmsd_maskz\0"
  "fixupimmss_mask\0"
  "V4fV4fV4fV4iIiUcIi\0"
  "fixupimmss_maskz\0"
  "fpclasspd128_mask\0"
  "UcV2dIiUc\0"
  "fpclasspd256_mask\0"
  "UcV4dIiUc\0"
  "fpclasspd512_mask\0"
  "UcV8dIiUc\0"
  "fpclassph128_mask\0"
  "UcV8xIiUc\0"
  "fpclassph256_mask\0"
  "UsV16xIiUs\0"
  "fpclassph512_mask\0"
  "UiV32xIiUi\0"
  "fpclassps128_mask\0"
  "UcV4fIiUc\0"
  "fpclassps256_mask\0"
  "UcV8fIiUc\0"
  "fpclassps512_mask\0"
  "UsV16fIiUs\0"
  "fpclasssd_mask\0"
  "fpclasssh_mask\0"
  "fpclassss_mask\0"
  "fxrstor\0"
  "fxsr\0"
  "fxsave\0"
  "gather3div2df\0"
  "V2dV2dvC*V2OiUcIi\0"
  "gather3div2di\0"
  "V2OiV2OivC*V2OiUcIi\0"
  "gather3div4df\0"
  "V4dV4dvC*V4OiUcIi\0"
  "gather3div4di\0"
  "V4OiV4OivC*V4OiUcIi\0"
  "gather3div4sf\0"
  "V4fV4fvC*V2OiUcIi\0"
  "gather3div4si\0"
  "V4iV4ivC*V2OiUcIi\0"
  "gather3div8sf\0"
  "V4fV4fvC*V4OiUcIi\0"
  "gather3div8si\0"
  "V4iV4ivC*V4OiUcIi\0"
  "gather3siv2df\0"
  "V2dV2dvC*V4iUcIi\0"
  "gather3siv2di\0"
  "V2OiV2OivC*V4iUcIi\0"
  "gather3siv4df\0"
  "V4dV4dvC*V4iUcIi\0"
  "gather3siv4di\0"
  "V4OiV4OivC*V4iUcIi\0"
  "gather3siv4sf\0"
  "V4fV4fvC*V4iUcIi\0"
  "gather3siv4si\0"
  "V4iV4ivC*V4iUcIi\0"
  "gather3siv8sf\0"
  "V8fV8fvC*V8iUcIi\0"
  "gather3siv8si\0"
  "V8iV8ivC*V8iUcIi\0"
  "gatherd_d\0"
  "V4iV4iiC*V4iV4iIc\0"
  "gatherd_d256\0"
  "V8iV8iiC*V8iV8iIc\0"
  "gatherd_pd\0"
  "V2dV2ddC*V4iV2dIc\0"
  "gatherd_pd256\0"
  "V4dV4ddC*V4iV4dIc\0"
  "gatherd_ps\0"
  "V4fV4ffC*V4iV4fIc\0"
  "gatherd_ps256\0"
  "V8fV8ffC*V8iV8fIc\0"
  "gatherd_q\0"
  "V2OiV2OiOiC*V4iV2OiIc\0"
  "gatherd_q256\0"
  "V4OiV4OiOiC*V4iV4OiIc\0"
  "gatherdiv8df\0"
  "V8dV8dvC*V8OiUcIi\0"
  "gatherdiv8di\0"
  "V8OiV8OivC*V8OiUcIi\0"
  "gatherdiv16sf\0"
  "V8fV8fvC*V8OiUcIi\0"
  "gatherdiv16si\0"
  "V8iV8ivC*V8OiUcIi\0"
  "gatherq_d\0"
  "V4iV4iiC*V2OiV4iIc\0"
  "gatherq_d256\0"
  "V4iV4iiC*V4OiV4iIc\0"
  "gatherq_pd\0"
  "V2dV2ddC*V2OiV2dIc\0"
  "gatherq_pd256\0"
  "V4dV4ddC*V4OiV4dIc\0"
  "gatherq_ps\0"
  "V4fV4ffC*V2OiV4fIc\0"
  "gatherq_ps256\0"
  "V4fV4ffC*V4OiV4fIc\0"
  "gatherq_q\0"
  "V2OiV2OiOiC*V2OiV2OiIc\0"
  "gatherq_q256\0"
  "V4OiV4OiOiC*V4OiV4OiIc\0"
  "gathersiv8df\0"
  "V8dV8dvC*V8iUcIi\0"
  "gathersiv8di\0"
  "V8OiV8OivC*V8iUcIi\0"
  "gathersiv16sf\0"
  "V16fV16fvC*V16iUsIi\0"
  "gathersiv16si\0"
  "V16iV16ivC*V16iUsIi\0"
  "getexppd128_mask\0"
  "getexppd256_mask\0"
  "getexppd512_mask\0"
  "V8dV8dV8dUcIi\0"
  "getexpph128_mask\0"
  "V8xV8xV8xUc\0"
  "getexpph256_mask\0"
  "V16xV16xV16xUs\0"
  "getexpph512_mask\0"
  "V32xV32xV32xUiIi\0"
  "getexpps128_mask\0"
  "getexpps256_mask\0"
  "getexpps512_mask\0"
  "V16fV16fV16fUsIi\0"
  "getexpsd128_round_mask\0"
  "getexpsh128_round_mask\0"
  "getexpss128_round_mask\0"
  "getmantpd128_mask\0"
  "V2dV2dIiV2dUc\0"
  "getmantpd256_mask\0"
  "V4dV4dIiV4dUc\0"
  "getmantpd512_mask\0"
  "V8dV8dIiV8dUcIi\0"
  "getmantph128_mask\0"
  "V8xV8xIiV8xUc\0"
  "getmantph256_mask\0"
  "V16xV16xIiV16xUs\0"
  "getmantph512_mask\0"
  "V32xV32xIiV32xUiIi\0"
  "getmantps128_mask\0"
  "V4fV4fIiV4fUc\0"
  "getmantps256_mask\0"
  "V8fV8fIiV8fUc\0"
  "getmantps512_mask\0"
  "V16fV16fIiV16fUsIi\0"
  "getmantsd_round_mask\0"
  "V2dV2dV2dIiV2dUcIi\0"
  "getmantsh_round_mask\0"
  "V8xV8xV8xIiV8xUcIi\0"
  "getmantss_round_mask\0"
  "V4fV4fV4fIiV4fUcIi\0"
  "haddpd\0"
  "haddpd256\0"
  "haddps\0"
  "haddps256\0"
  "hsubpd\0"
  "hsubpd256\0"
  "hsubps\0"
  "hsubps256\0"
  "incsspd\0"
  "insert128i256\0"
  "V4OiV4OiV2OiIi\0"
  "insertf32x4\0"
  "V16fV16fV4fIi\0"
  "insertf32x4_256\0"
  "V8fV8fV4fIi\0"
  "insertf32x8\0"
  "V16fV16fV8fIi\0"
  "insertf64x2_256\0"
  "V4dV4dV2dIi\0"
  "insertf64x2_512\0"
  "V8dV8dV2dIi\0"
  "insertf64x4\0"
  "V8dV8dV4dIi\0"
  "inserti32x4\0"
  "V16iV16iV4iIi\0"
  "inserti32x4_256\0"
  "V8iV8iV4iIi\0"
  "inserti32x8\0"
  "V16iV16iV8iIi\0"
  "inserti64x2_256\0"
  "inserti64x2_512\0"
  "V8OiV8OiV2OiIi\0"
  "inserti64x4\0"
  "V8OiV8OiV4OiIi\0"
  "insertps128\0"
  "insertq\0"
  "insertqi\0"
  "V2OiV2OiV2OiIcIc\0"
  "invpcid\0"
  "vUiv*\0"
  "kadddi\0"
  "UOiUOiUOi\0"
  "kaddhi\0"
  "UsUsUs\0"
  "kaddqi\0"
  "UcUcUc\0"
  "kaddsi\0"
  "kanddi\0"
  "kandhi\0"
  "kandndi\0"
  "kandnhi\0"
  "kandnqi\0"
  "kandnsi\0"
  "kandqi\0"
  "kandsi\0"
  "kmovb\0"
  "UcUc\0"
  "kmovd\0"
  "UiUi\0"
  "kmovq\0"
  "UOiUOi\0"
  "kmovw\0"
  "UsUs\0"
  "knotdi\0"
  "knothi\0"
  "knotqi\0"
  "knotsi\0"
  "kordi\0"
  "korhi\0"
  "korqi\0"
  "korsi\0"
  "kortestcdi\0"
  "iUOiUOi\0"
  "kortestchi\0"
  "iUsUs\0"
  "kortestcqi\0"
  "iUcUc\0"
  "kortestcsi\0"
  "iUiUi\0"
  "kortestzdi\0"
  "kortestzhi\0"
  "kortestzqi\0"
  "kortestzsi\0"
  "kshiftlidi\0"
  "UOiUOiIUi\0"
  "kshiftlihi\0"
  "UsUsIUi\0"
  "kshiftliqi\0"
  "UcUcIUi\0"
  "kshiftlisi\0"
  "kshiftridi\0"
  "kshiftrihi\0"
  "kshiftriqi\0"
  "kshiftrisi\0"
  "ktestcdi\0"
  "ktestchi\0"
  "ktestcqi\0"
  "ktestcsi\0"
  "ktestzdi\0"
  "ktestzhi\0"
  "ktestzqi\0"
  "ktestzsi\0"
  "kunpckdi\0"
  "kunpckhi\0"
  "kunpcksi\0"
  "kxnordi\0"
  "kxnorhi\0"
  "kxnorqi\0"
  "kxnorsi\0"
  "kxordi\0"
  "kxorhi\0"
  "kxorqi\0"
  "kxorsi\0"
  "lddqu\0"
  "V16ccC*\0"
  "lddqu256\0"
  "V32ccC*\0"
  "ldmxcsr\0"
  "lfence\0"
  "llwpcb\0"
  "lwp\0"
  "loadapd128_mask\0"
  "loadapd256_mask\0"
  "loadapd512_mask\0"
  "loadaps128_mask\0"
  "loadaps256_mask\0"
  "loadaps512_mask\0"
  "loaddqudi128_mask\0"
  "V2OiV2OiC*V2OiUc\0"
  "loaddqudi256_mask\0"
  "loaddqudi512_mask\0"
  "V8OiOiC*V8OiUc\0"
  "loaddquhi128_mask\0"
  "loaddquhi256_mask\0"
  "loaddquhi512_mask\0"
  "loaddquqi128_mask\0"
  "loaddquqi256_mask\0"
  "loaddquqi512_mask\0"
  "loaddqusi128_mask\0"
  "loaddqusi256_mask\0"
  "loaddqusi512_mask\0"
  "V16iiC*V16iUs\0"
  "loadiwkey\0"
  "vV2OiV2OiV2OiUi\0"
  "loadsbf16128_mask\0"
  "V8yV8yC*V8yUc\0"
  "avx10.2\0"
  "loadsd128_mask\0"
  "loadsh128_mask\0"
  "V8xV8xC*V8xUc\0"
  "loadss128_mask\0"
  "loadupd128_mask\0"
  "loadupd256_mask\0"
  "loadupd512_mask\0"
  "V8ddC*V8dUc\0"
  "loadups128_mask\0"
  "loadups256_mask\0"
  "loadups512_mask\0"
  "V16ffC*V16fUs\0"
  "lwpins32\0"
  "UcUiUiIUi\0"
  "lwpval32\0"
  "vUiUiIUi\0"
  "lzcnt_u16\0"
  "lzcnt_u32\0"
  "maskloadd\0"
  "V4iV4iC*V4i\0"
  "maskloadd256\0"
  "V8iV8iC*V8i\0"
  "maskloadpd\0"
  "V2dV2dC*V2Oi\0"
  "maskloadpd256\0"
  "V4dV4dC*V4Oi\0"
  "maskloadps\0"
  "V4fV4fC*V4i\0"
  "maskloadps256\0"
  "V8fV8fC*V8i\0"
  "maskloadq\0"
  "V2OiV2OiC*V2Oi\0"
  "maskloadq256\0"
  "V4OiV4OiC*V4Oi\0"
  "maskmovdqu\0"
  "vV16cV16cc*\0"
  "maskstored\0"
  "vV4i*V4iV4i\0"
  "maskstored256\0"
  "vV8i*V8iV8i\0"
  "maskstorepd\0"
  "vV2d*V2OiV2d\0"
  "maskstorepd256\0"
  "vV4d*V4OiV4d\0"
  "maskstoreps\0"
  "vV4f*V4iV4f\0"
  "maskstoreps256\0"
  "vV8f*V8iV8f\0"
  "maskstoreq\0"
  "vV2Oi*V2OiV2Oi\0"
  "maskstoreq256\0"
  "vV4Oi*V4OiV4Oi\0"
  "maxpd\0"
  "maxpd256\0"
  "maxpd512\0"
  "maxph128\0"
  "V8xV8xV8x\0"
  "maxph256\0"
  "V16xV16xV16x\0"
  "maxph512\0"
  "maxps\0"
  "maxps256\0"
  "maxps512\0"
  "maxsd\0"
  "maxsd_round_mask\0"
  "maxsh_round_mask\0"
  "maxss\0"
  "maxss_round_mask\0"
  "mfence\0"
  "minpd\0"
  "minpd256\0"
  "minpd512\0"
  "minph128\0"
  "minph256\0"
  "minph512\0"
  "minps\0"
  "minps256\0"
  "minps512\0"
  "minsd\0"
  "minsd_round_mask\0"
  "minsh_round_mask\0"
  "minss\0"
  "minss_round_mask\0"
  "monitor\0"
  "vvC*UiUi\0"
  "monitorx\0"
  "mwaitx\0"
  "movdir64b\0"
  "vv*vC*\0"
  "movdqa32load128_mask\0"
  "movdqa32load256_mask\0"
  "movdqa32load512_mask\0"
  "movdqa32store128_mask\0"
  "movdqa32store256_mask\0"
  "movdqa32store512_mask\0"
  "movdqa64load128_mask\0"
  "movdqa64load256_mask\0"
  "movdqa64load512_mask\0"
  "movdqa64store128_mask\0"
  "movdqa64store256_mask\0"
  "movdqa64store512_mask\0"
  "movmskpd\0"
  "movmskpd256\0"
  "iV4d\0"
  "movmskps\0"
  "movmskps256\0"
  "iV8f\0"
  "movnti\0"
  "movntsd\0"
  "vd*V2d\0"
  "movntss\0"
  "vf*V4f\0"
  "mpsadbw128\0"
  "V16cV16cV16cIc\0"
  "mpsadbw256\0"
  "V32cV32cV32cIc\0"
  "mpsadbw512\0"
  "V32sV64cV64cIc\0"
  "mulpd512\0"
  "mulph512\0"
  "mulps512\0"
  "mulsd_round_mask\0"
  "mulsh_round_mask\0"
  "mulss_round_mask\0"
  "mwait\0"
  "vUiUi\0"
  "vUiUiUi\0"
  "packssdw128\0"
  "V8sV4iV4i\0"
  "packssdw256\0"
  "V16sV8iV8i\0"
  "packssdw512\0"
  "V32sV16iV16i\0"
  "packsswb128\0"
  "V16cV8sV8s\0"
  "packsswb256\0"
  "V32cV16sV16s\0"
  "packsswb512\0"
  "V64cV32sV32s\0"
  "packusdw128\0"
  "packusdw256\0"
  "packusdw512\0"
  "packuswb128\0"
  "packuswb256\0"
  "packuswb512\0"
  "palignr128\0"
  "V16cV16cV16cIi\0"
  "ssse3\0"
  "palignr256\0"
  "V32cV32cV32cIi\0"
  "palignr512\0"
  "V64cV64cV64cIi\0"
  "pause\0"
  "pavgb128\0"
  "V16UcV16UcV16Uc\0"
  "pavgb256\0"
  "V32UcV32UcV32Uc\0"
  "pavgb512\0"
  "V64UcV64UcV64Uc\0"
  "pavgw128\0"
  "V8UsV8UsV8Us\0"
  "pavgw256\0"
  "V16UsV16UsV16Us\0"
  "pavgw512\0"
  "V32UsV32UsV32Us\0"
  "pblendd128\0"
  "pblendd256\0"
  "pblendvb128\0"
  "V16cV16cV16cV16c\0"
  "pblendvb256\0"
  "V32cV32cV32cV32c\0"
  "pblendw128\0"
  "V8sV8sV8sIi\0"
  "pblendw256\0"
  "V16sV16sV16sIi\0"
  "pclmulqdq128\0"
  "V2OiV2OiV2OiIc\0"
  "pclmul\0"
  "pclmulqdq256\0"
  "V4OiV4OiV4OiIc\0"
  "vpclmulqdq\0"
  "pclmulqdq512\0"
  "V8OiV8OiV8OiIc\0"
  "avx512f,vpclmulqdq\0"
  "pcmpestri128\0"
  "iV16ciV16ciIc\0"
  "sse4.2\0"
  "pcmpestria128\0"
  "pcmpestric128\0"
  "pcmpestrio128\0"
  "pcmpestris128\0"
  "pcmpestriz128\0"
  "pcmpestrm128\0"
  "V16cV16ciV16ciIc\0"
  "pcmpistri128\0"
  "iV16cV16cIc\0"
  "pcmpistria128\0"
  "pcmpistric128\0"
  "pcmpistrio128\0"
  "pcmpistris128\0"
  "pcmpistriz128\0"
  "pcmpistrm128\0"
  "pdep_si\0"
  "permdf256\0"
  "V4dV4dIi\0"
  "permdf512\0"
  "V8dV8dIi\0"
  "permdi256\0"
  "V4OiV4OiIi\0"
  "permdi512\0"
  "V8OiV8OiIi\0"
  "permti256\0"
  "permvardf256\0"
  "V4dV4dV4Oi\0"
  "permvardf512\0"
  "V8dV8dV8Oi\0"
  "permvardi256\0"
  "permvardi512\0"
  "permvarhi128\0"
  "V8sV8sV8s\0"
  "permvarhi256\0"
  "V16sV16sV16s\0"
  "permvarhi512\0"
  "V32sV32sV32s\0"
  "permvarqi128\0"
  "V16cV16cV16c\0"
  "avx512vbmi,avx512vl\0"
  "permvarqi256\0"
  "V32cV32cV32c\0"
  "permvarqi512\0"
  "V64cV64cV64c\0"
  "avx512vbmi\0"
  "permvarsf256\0"
  "V8fV8fV8i\0"
  "permvarsf512\0"
  "V16fV16fV16i\0"
  "permvarsi256\0"
  "V8iV8iV8i\0"
  "permvarsi512\0"
  "V16iV16iV16i\0"
  "pext_si\0"
  "phaddd128\0"
  "V4iV4iV4i\0"
  "phaddd256\0"
  "phaddsw128\0"
  "phaddsw256\0"
  "phaddw128\0"
  "phaddw256\0"
  "phminposuw128\0"
  "V8sV8s\0"
  "phsubd128\0"
  "phsubd256\0"
  "phsubsw128\0"
  "phsubsw256\0"
  "phsubw128\0"
  "phsubw256\0"
  "pmaddubsw128\0"
  "V8sV16cV16c\0"
  "pmaddubsw256\0"
  "V16sV32cV32c\0"
  "pmaddubsw512\0"
  "V32sV64cV64c\0"
  "pmaddwd128\0"
  "V4iV8sV8s\0"
  "pmaddwd256\0"
  "V8iV16sV16s\0"
  "pmaddwd512\0"
  "V16iV32sV32s\0"
  "pmovdb128_mask\0"
  "V16cV4iV16cUc\0"
  "pmovdb128mem_mask\0"
  "vV16c*V4iUc\0"
  "pmovdb256_mask\0"
  "V16cV8iV16cUc\0"
  "pmovdb256mem_mask\0"
  "vV16c*V8iUc\0"
  "pmovdb512_mask\0"
  "V16cV16iV16cUs\0"
  "pmovdb512mem_mask\0"
  "vV16c*V16iUs\0"
  "pmovdw128_mask\0"
  "V8sV4iV8sUc\0"
  "pmovdw128mem_mask\0"
  "vV8s*V4iUc\0"
  "pmovdw256_mask\0"
  "V8sV8iV8sUc\0"
  "pmovdw256mem_mask\0"
  "vV8s*V8iUc\0"
  "pmovdw512_mask\0"
  "V16sV16iV16sUs\0"
  "pmovdw512mem_mask\0"
  "vV16s*V16iUs\0"
  "pmovmskb128\0"
  "iV16c\0"
  "pmovmskb256\0"
  "iV32c\0"
  "pmovqb128_mask\0"
  "V16cV2OiV16cUc\0"
  "pmovqb128mem_mask\0"
  "vV16c*V2OiUc\0"
  "pmovqb256_mask\0"
  "V16cV4OiV16cUc\0"
  "pmovqb256mem_mask\0"
  "vV16c*V4OiUc\0"
  "pmovqb512_mask\0"
  "V16cV8OiV16cUc\0"
  "pmovqb512mem_mask\0"
  "vV16c*V8OiUc\0"
  "pmovqd128_mask\0"
  "V4iV2OiV4iUc\0"
  "pmovqd128mem_mask\0"
  "vV4i*V2OiUc\0"
  "pmovqd256mem_mask\0"
  "vV4i*V4OiUc\0"
  "pmovqd512_mask\0"
  "V8iV8OiV8iUc\0"
  "pmovqd512mem_mask\0"
  "vV8i*V8OiUc\0"
  "pmovqw128_mask\0"
  "V8sV2OiV8sUc\0"
  "pmovqw128mem_mask\0"
  "vV8s*V2OiUc\0"
  "pmovqw256_mask\0"
  "V8sV4OiV8sUc\0"
  "pmovqw256mem_mask\0"
  "vV8s*V4OiUc\0"
  "pmovqw512_mask\0"
  "V8sV8OiV8sUc\0"
  "pmovqw512mem_mask\0"
  "vV8s*V8OiUc\0"
  "pmovsdb128_mask\0"
  "pmovsdb128mem_mask\0"
  "pmovsdb256_mask\0"
  "pmovsdb256mem_mask\0"
  "pmovsdb512_mask\0"
  "pmovsdb512mem_mask\0"
  "pmovsdw128_mask\0"
  "pmovsdw128mem_mask\0"
  "pmovsdw256_mask\0"
  "pmovsdw256mem_mask\0"
  "pmovsdw512_mask\0"
  "pmovsdw512mem_mask\0"
  "pmovsqb128_mask\0"
  "pmovsqb128mem_mask\0"
  "pmovsqb256_mask\0"
  "pmovsqb256mem_mask\0"
  "pmovsqb512_mask\0"
  "pmovsqb512mem_mask\0"
  "pmovsqd128_mask\0"
  "pmovsqd128mem_mask\0"
  "pmovsqd256_mask\0"
  "V4iV4OiV4iUc\0"
  "pmovsqd256mem_mask\0"
  "pmovsqd512_mask\0"
  "pmovsqd512mem_mask\0"
  "pmovsqw128_mask\0"
  "pmovsqw128mem_mask\0"
  "pmovsqw256_mask\0"
  "pmovsqw256mem_mask\0"
  "pmovsqw512_mask\0"
  "pmovsqw512mem_mask\0"
  "pmovswb128_mask\0"
  "V16cV8sV16cUc\0"
  "pmovswb128mem_mask\0"
  "vV16c*V8sUc\0"
  "pmovswb256_mask\0"
  "V16cV16sV16cUs\0"
  "pmovswb256mem_mask\0"
  "vV16c*V16sUs\0"
  "pmovswb512_mask\0"
  "V32cV32sV32cUi\0"
  "pmovswb512mem_mask\0"
  "vV32c*V32sUi\0"
  "pmovusdb128_mask\0"
  "pmovusdb128mem_mask\0"
  "pmovusdb256_mask\0"
  "pmovusdb256mem_mask\0"
  "pmovusdb512_mask\0"
  "pmovusdb512mem_mask\0"
  "pmovusdw128_mask\0"
  "pmovusdw128mem_mask\0"
  "pmovusdw256_mask\0"
  "pmovusdw256mem_mask\0"
  "pmovusdw512_mask\0"
  "pmovusdw512mem_mask\0"
  "pmovusqb128_mask\0"
  "pmovusqb128mem_mask\0"
  "pmovusqb256_mask\0"
  "pmovusqb256mem_mask\0"
  "pmovusqb512_mask\0"
  "pmovusqb512mem_mask\0"
  "pmovusqd128_mask\0"
  "pmovusqd128mem_mask\0"
  "pmovusqd256_mask\0"
  "pmovusqd256mem_mask\0"
  "pmovusqd512_mask\0"
  "pmovusqd512mem_mask\0"
  "pmovusqw128_mask\0"
  "pmovusqw128mem_mask\0"
  "pmovusqw256_mask\0"
  "pmovusqw256mem_mask\0"
  "pmovusqw512_mask\0"
  "pmovusqw512mem_mask\0"
  "pmovuswb128_mask\0"
  "pmovuswb128mem_mask\0"
  "pmovuswb256_mask\0"
  "pmovuswb256mem_mask\0"
  "pmovuswb512_mask\0"
  "pmovuswb512mem_mask\0"
  "pmovwb128_mask\0"
  "pmovwb128mem_mask\0"
  "pmovwb256mem_mask\0"
  "pmovwb512_mask\0"
  "pmovwb512mem_mask\0"
  "pmuldq128\0"
  "V2OiV4iV4i\0"
  "pmuldq256\0"
  "V4OiV8iV8i\0"
  "pmuldq512\0"
  "V8OiV16iV16i\0"
  "pmulhrsw128\0"
  "pmulhrsw256\0"
  "pmulhrsw512\0"
  "pmulhuw128\0"
  "pmulhuw256\0"
  "pmulhuw512\0"
  "pmulhw128\0"
  "pmulhw256\0"
  "pmulhw512\0"
  "pmuludq128\0"
  "pmuludq256\0"
  "pmuludq512\0"
  "prefetchrs\0"
  "movrs\0"
  "prold128\0"
  "V4iV4iIi\0"
  "prold256\0"
  "V8iV8iIi\0"
  "prold512\0"
  "V16iV16iIi\0"
  "prolq128\0"
  "V2OiV2OiIi\0"
  "prolq256\0"
  "prolq512\0"
  "prord128\0"
  "prord256\0"
  "prord512\0"
  "prorq128\0"
  "prorq256\0"
  "prorq512\0"
  "psadbw128\0"
  "V2OiV16cV16c\0"
  "psadbw256\0"
  "V4OiV32cV32c\0"
  "psadbw512\0"
  "V8OiV64cV64c\0"
  "pshufb128\0"
  "pshufb256\0"
  "pshufb512\0"
  "pshufd\0"
  "pshufd256\0"
  "pshufd512\0"
  "pshufhw\0"
  "V8sV8sIi\0"
  "pshufhw256\0"
  "V16sV16sIi\0"
  "pshufhw512\0"
  "V32sV32sIi\0"
  "pshuflw\0"
  "pshuflw256\0"
  "pshuflw512\0"
  "psignb128\0"
  "psignb256\0"
  "psignd128\0"
  "psignd256\0"
  "psignw128\0"
  "psignw256\0"
  "pslld128\0"
  "pslld256\0"
  "V8iV8iV4i\0"
  "pslld512\0"
  "V16iV16iV4i\0"
  "pslldi128\0"
  "V4iV4ii\0"
  "pslldi256\0"
  "V8iV8ii\0"
  "pslldi512\0"
  "V16iV16ii\0"
  "pslldqi128_byteshift\0"
  "V16cV16cIi\0"
  "pslldqi256_byteshift\0"
  "V32cV32cIi\0"
  "pslldqi512_byteshift\0"
  "V64cV64cIi\0"
  "psllq128\0"
  "psllq256\0"
  "V4OiV4OiV2Oi\0"
  "psllq512\0"
  "V8OiV8OiV2Oi\0"
  "psllqi128\0"
  "V2OiV2Oii\0"
  "psllqi256\0"
  "V4OiV4Oii\0"
  "psllqi512\0"
  "V8OiV8Oii\0"
  "psllv2di\0"
  "psllv4di\0"
  "psllv4si\0"
  "psllv8di\0"
  "psllv8hi\0"
  "psllv8si\0"
  "psllv16hi\0"
  "psllv16si\0"
  "psllv32hi\0"
  "psllw128\0"
  "psllw256\0"
  "V16sV16sV8s\0"
  "psllw512\0"
  "V32sV32sV8s\0"
  "psllwi128\0"
  "V8sV8si\0"
  "psllwi256\0"
  "V16sV16si\0"
  "psllwi512\0"
  "V32sV32si\0"
  "psrad128\0"
  "psrad256\0"
  "psrad512\0"
  "psradi128\0"
  "psradi256\0"
  "psradi512\0"
  "psraq128\0"
  "psraq256\0"
  "psraq512\0"
  "psraqi128\0"
  "psraqi256\0"
  "psraqi512\0"
  "psrav4si\0"
  "psrav8di\0"
  "psrav8hi\0"
  "psrav8si\0"
  "psrav16hi\0"
  "psrav16si\0"
  "psrav32hi\0"
  "psravq128\0"
  "psravq256\0"
  "psraw128\0"
  "psraw256\0"
  "psraw512\0"
  "psrawi128\0"
  "psrawi256\0"
  "psrawi512\0"
  "psrld128\0"
  "psrld256\0"
  "psrld512\0"
  "psrldi128\0"
  "psrldi256\0"
  "psrldi512\0"
  "psrldqi128_byteshift\0"
  "psrldqi256_byteshift\0"
  "psrldqi512_byteshift\0"
  "psrlq128\0"
  "psrlq256\0"
  "psrlq512\0"
  "psrlqi128\0"
  "psrlqi256\0"
  "psrlqi512\0"
  "psrlv2di\0"
  "psrlv4di\0"
  "psrlv4si\0"
  "psrlv8di\0"
  "psrlv8hi\0"
  "psrlv8si\0"
  "psrlv16hi\0"
  "psrlv16si\0"
  "psrlv32hi\0"
  "psrlw128\0"
  "psrlw256\0"
  "psrlw512\0"
  "psrlwi128\0"
  "psrlwi256\0"
  "psrlwi512\0"
  "pternlogd128_mask\0"
  "V4iV4iV4iV4iIiUc\0"
  "pternlogd128_maskz\0"
  "pternlogd256_mask\0"
  "V8iV8iV8iV8iIiUc\0"
  "pternlogd256_maskz\0"
  "pternlogd512_mask\0"
  "V16iV16iV16iV16iIiUs\0"
  "pternlogd512_maskz\0"
  "pternlogq128_mask\0"
  "V2OiV2OiV2OiV2OiIiUc\0"
  "pternlogq128_maskz\0"
  "pternlogq256_mask\0"
  "V4OiV4OiV4OiV4OiIiUc\0"
  "pternlogq256_maskz\0"
  "pternlogq512_mask\0"
  "V8OiV8OiV8OiV8OiIiUc\0"
  "pternlogq512_maskz\0"
  "ptestc128\0"
  "iV2OiV2Oi\0"
  "ptestc256\0"
  "iV4OiV4Oi\0"
  "ptestnzc128\0"
  "ptestnzc256\0"
  "ptestz128\0"
  "ptestz256\0"
  "ptwrite32\0"
  "ptwrite\0"
  "rangepd128_mask\0"
  "V2dV2dV2dIiV2dUc\0"
  "rangepd256_mask\0"
  "V4dV4dV4dIiV4dUc\0"
  "rangepd512_mask\0"
  "V8dV8dV8dIiV8dUcIi\0"
  "rangeps128_mask\0"
  "V4fV4fV4fIiV4fUc\0"
  "rangeps256_mask\0"
  "V8fV8fV8fIiV8fUc\0"
  "rangeps512_mask\0"
  "V16fV16fV16fIiV16fUsIi\0"
  "rangesd128_round_mask\0"
  "V2dV2dV2dV2dUcIiIi\0"
  "rangess128_round_mask\0"
  "V4fV4fV4fV4fUcIiIi\0"
  "rcp14pd128_mask\0"
  "rcp14pd256_mask\0"
  "rcp14pd512_mask\0"
  "rcp14ps128_mask\0"
  "rcp14ps256_mask\0"
  "rcp14ps512_mask\0"
  "rcp14sd_mask\0"
  "V2dV2dV2dV2dUc\0"
  "rcp14ss_mask\0"
  "V4fV4fV4fV4fUc\0"
  "rcpph128_mask\0"
  "rcpph256_mask\0"
  "rcpph512_mask\0"
  "V32xV32xV32xUi\0"
  "rcpps\0"
  "V4fV4f\0"
  "rcpps256\0"
  "V8fV8f\0"
  "rcpsh_mask\0"
  "V8xV8xV8xV8xUc\0"
  "rcpss\0"
  "rdpid\0"
  "rdpkru\0"
  "pku\0"
  "rdpmc\0"
  "UOii\0"
  "rdpru\0"
  "ULLii\0"
  "rdrand16_step\0"
  "UiUs*\0"
  "rdrnd\0"
  "rdrand32_step\0"
  "UiUi*\0"
  "rdseed16_step\0"
  "rdseed\0"
  "rdseed32_step\0"
  "rdsspd\0"
  "rdtsc\0"
  "rdtscp\0"
  "UOiUi*\0"
  "readeflags_u32\0"
  "reduce_fadd_pd512\0"
  "ddV8d\0"
  "reduce_fadd_ph128\0"
  "xxV8x\0"
  "reduce_fadd_ph256\0"
  "xxV16x\0"
  "reduce_fadd_ph512\0"
  "xxV32x\0"
  "reduce_fadd_ps512\0"
  "ffV16f\0"
  "reduce_fmax_pd512\0"
  "dV8d\0"
  "reduce_fmax_ph128\0"
  "xV8x\0"
  "reduce_fmax_ph256\0"
  "xV16x\0"
  "reduce_fmax_ph512\0"
  "xV32x\0"
  "reduce_fmax_ps512\0"
  "fV16f\0"
  "reduce_fmin_pd512\0"
  "reduce_fmin_ph128\0"
  "reduce_fmin_ph256\0"
  "reduce_fmin_ph512\0"
  "reduce_fmin_ps512\0"
  "reduce_fmul_pd512\0"
  "reduce_fmul_ph128\0"
  "reduce_fmul_ph256\0"
  "reduce_fmul_ph512\0"
  "reduce_fmul_ps512\0"
  "reducepd128_mask\0"
  "reducepd256_mask\0"
  "reducepd512_mask\0"
  "reduceph128_mask\0"
  "reduceph256_mask\0"
  "reduceph512_mask\0"
  "reduceps128_mask\0"
  "reduceps256_mask\0"
  "reduceps512_mask\0"
  "reducesd_mask\0"
  "reducesh_mask\0"
  "V8xV8xV8xV8xUcIiIi\0"
  "reducess_mask\0"
  "rndscalepd_128_mask\0"
  "rndscalepd_256_mask\0"
  "rndscalepd_mask\0"
  "rndscaleph_128_mask\0"
  "rndscaleph_256_mask\0"
  "rndscaleph_mask\0"
  "rndscaleps_128_mask\0"
  "rndscaleps_256_mask\0"
  "rndscaleps_mask\0"
  "rndscalesd_round_mask\0"
  "rndscalesh_round_mask\0"
  "rndscaless_round_mask\0"
  "roundpd\0"
  "V2dV2dIi\0"
  "roundpd256\0"
  "roundps\0"
  "V4fV4fIi\0"
  "roundps256\0"
  "V8fV8fIi\0"
  "roundsd\0"
  "roundss\0"
  "rsqrt14pd128_mask\0"
  "rsqrt14pd256_mask\0"
  "rsqrt14pd512_mask\0"
  "rsqrt14ps128_mask\0"
  "rsqrt14ps256_mask\0"
  "rsqrt14ps512_mask\0"
  "rsqrt14sd_mask\0"
  "rsqrt14ss_mask\0"
  "rsqrtph128_mask\0"
  "rsqrtph256_mask\0"
  "rsqrtph512_mask\0"
  "rsqrtps\0"
  "rsqrtps256\0"
  "rsqrtsh_mask\0"
  "rsqrtss\0"
  "rstorssp\0"
  "saveprevssp\0"
  "scalefpd128_mask\0"
  "scalefpd256_mask\0"
  "V4dV4dV4dV4dUc\0"
  "scalefpd512_mask\0"
  "V8dV8dV8dV8dUcIi\0"
  "scalefph128_mask\0"
  "scalefph256_mask\0"
  "V16xV16xV16xV16xUs\0"
  "scalefph512_mask\0"
  "V32xV32xV32xV32xUiIi\0"
  "scalefps128_mask\0"
  "scalefps256_mask\0"
  "V8fV8fV8fV8fUc\0"
  "scalefps512_mask\0"
  "V16fV16fV16fV16fUsIi\0"
  "scalefsd_round_mask\0"
  "scalefsh_round_mask\0"
  "scalefss_round_mask\0"
  "scatterdiv2df\0"
  "vv*UcV2OiV2dIi\0"
  "scatterdiv2di\0"
  "vv*UcV2OiV2OiIi\0"
  "scatterdiv4df\0"
  "vv*UcV4OiV4dIi\0"
  "scatterdiv4di\0"
  "vv*UcV4OiV4OiIi\0"
  "scatterdiv4sf\0"
  "vv*UcV2OiV4fIi\0"
  "scatterdiv4si\0"
  "vv*UcV2OiV4iIi\0"
  "scatterdiv8df\0"
  "vv*UcV8OiV8dIi\0"
  "scatterdiv8di\0"
  "vv*UcV8OiV8OiIi\0"
  "scatterdiv8sf\0"
  "vv*UcV4OiV4fIi\0"
  "scatterdiv8si\0"
  "vv*UcV4OiV4iIi\0"
  "scatterdiv16sf\0"
  "vv*UcV8OiV8fIi\0"
  "scatterdiv16si\0"
  "vv*UcV8OiV8iIi\0"
  "scattersiv2df\0"
  "vv*UcV4iV2dIi\0"
  "scattersiv2di\0"
  "vv*UcV4iV2OiIi\0"
  "scattersiv4df\0"
  "vv*UcV4iV4dIi\0"
  "scattersiv4di\0"
  "vv*UcV4iV4OiIi\0"
  "scattersiv4sf\0"
  "vv*UcV4iV4fIi\0"
  "scattersiv4si\0"
  "vv*UcV4iV4iIi\0"
  "scattersiv8df\0"
  "vv*UcV8iV8dIi\0"
  "scattersiv8di\0"
  "vv*UcV8iV8OiIi\0"
  "scattersiv8sf\0"
  "vv*UcV8iV8fIi\0"
  "scattersiv8si\0"
  "vv*UcV8iV8iIi\0"
  "scattersiv16sf\0"
  "vv*UsV16iV16fIi\0"
  "scattersiv16si\0"
  "vv*UsV16iV16iIi\0"
  "selectb_128\0"
  "V16cUsV16cV16c\0"
  "selectb_256\0"
  "V32cUiV32cV32c\0"
  "selectb_512\0"
  "V64cUOiV64cV64c\0"
  "selectd_128\0"
  "V4iUcV4iV4i\0"
  "selectd_256\0"
  "V8iUcV8iV8i\0"
  "selectd_512\0"
  "V16iUsV16iV16i\0"
  "selectpbf_128\0"
  "V8yUcV8yV8y\0"
  "selectpbf_256\0"
  "V16yUsV16yV16y\0"
  "selectpbf_512\0"
  "V32yUiV32yV32y\0"
  "selectpd_128\0"
  "V2dUcV2dV2d\0"
  "selectpd_256\0"
  "V4dUcV4dV4d\0"
  "selectpd_512\0"
  "V8dUcV8dV8d\0"
  "selectph_128\0"
  "V8xUcV8xV8x\0"
  "selectph_256\0"
  "V16xUsV16xV16x\0"
  "selectph_512\0"
  "V32xUiV32xV32x\0"
  "selectps_128\0"
  "V4fUcV4fV4f\0"
  "selectps_256\0"
  "V8fUcV8fV8f\0"
  "selectps_512\0"
  "V16fUsV16fV16f\0"
  "selectq_128\0"
  "V2OiUcV2OiV2Oi\0"
  "selectq_256\0"
  "V4OiUcV4OiV4Oi\0"
  "selectq_512\0"
  "V8OiUcV8OiV8Oi\0"
  "selectsbf_128\0"
  "selectsd_128\0"
  "selectsh_128\0"
  "selectss_128\0"
  "selectw_128\0"
  "V8sUcV8sV8s\0"
  "selectw_256\0"
  "V16sUsV16sV16s\0"
  "selectw_512\0"
  "V32sUiV32sV32s\0"
  "serialize\0"
  "setssbsy\0"
  "sfence\0"
  "sha1msg1\0"
  "sha\0"
  "sha1msg2\0"
  "sha1nexte\0"
  "sha1rnds4\0"
  "V4iV4iV4iIc\0"
  "sha256msg1\0"
  "sha256msg2\0"
  "sha256rnds2\0"
  "V4iV4iV4iV4i\0"
  "shuf_f32x4\0"
  "shuf_f32x4_256\0"
  "shuf_f64x2\0"
  "shuf_f64x2_256\0"
  "shuf_i32x4\0"
  "shuf_i32x4_256\0"
  "shuf_i64x2\0"
  "shuf_i64x2_256\0"
  "shufpd\0"
  "shufpd256\0"
  "shufpd512\0"
  "shufps\0"
  "shufps256\0"
  "shufps512\0"
  "slwpcb\0"
  "sqrtpd512\0"
  "sqrtph512\0"
  "V32xV32xIi\0"
  "sqrtps512\0"
  "V16fV16fIi\0"
  "sqrtsd_round_mask\0"
  "sqrtsh_round_mask\0"
  "sqrtss_round_mask\0"
  "stmxcsr\0"
  "storeapd128_mask\0"
  "storeapd256_mask\0"
  "storeapd512_mask\0"
  "storeaps128_mask\0"
  "storeaps256_mask\0"
  "storeaps512_mask\0"
  "storedqudi128_mask\0"
  "storedqudi256_mask\0"
  "storedqudi512_mask\0"
  "vOi*V8OiUc\0"
  "storedquhi128_mask\0"
  "storedquhi256_mask\0"
  "storedquhi512_mask\0"
  "storedquqi128_mask\0"
  "storedquqi256_mask\0"
  "storedquqi512_mask\0"
  "storedqusi128_mask\0"
  "storedqusi256_mask\0"
  "storedqusi512_mask\0"
  "vi*V16iUs\0"
  "storesbf16128_mask\0"
  "vV8y*V8yUc\0"
  "storesd128_mask\0"
  "storesh128_mask\0"
  "vV8x*V8xUc\0"
  "storess128_mask\0"
  "storeupd128_mask\0"
  "storeupd256_mask\0"
  "storeupd512_mask\0"
  "vd*V8dUc\0"
  "storeups128_mask\0"
  "storeups256_mask\0"
  "storeups512_mask\0"
  "vf*V16fUs\0"
  "subborrow_u32\0"
  "subpd512\0"
  "subph512\0"
  "subps512\0"
  "subsd_round_mask\0"
  "subsh_round_mask\0"
  "subss_round_mask\0"
  "tpause\0"
  "UcUiUiUi\0"
  "waitpkg\0"
  "tzcnt_u16\0"
  "tzcnt_u32\0"
  "ucmpb128_mask\0"
  "ucmpb256_mask\0"
  "ucmpb512_mask\0"
  "ucmpd128_mask\0"
  "ucmpd256_mask\0"
  "ucmpd512_mask\0"
  "ucmpq128_mask\0"
  "ucmpq256_mask\0"
  "ucmpq512_mask\0"
  "ucmpw128_mask\0"
  "ucmpw256_mask\0"
  "ucmpw512_mask\0"
  "ucomieq\0"
  "ucomige\0"
  "ucomigt\0"
  "ucomile\0"
  "ucomilt\0"
  "ucomineq\0"
  "ucomisdeq\0"
  "ucomisdge\0"
  "ucomisdgt\0"
  "ucomisdle\0"
  "ucomisdlt\0"
  "ucomisdneq\0"
  "umonitor\0"
  "umwait\0"
  "undef128\0"
  "V2d\0"
  "undef256\0"
  "V4d\0"
  "cnV:256:\0"
  "undef512\0"
  "V8d\0"
  "cnV:512:\0"
  "vaddbf16128\0"
  "V8yV8yV8y\0"
  "vaddbf16256\0"
  "V16yV16yV16y\0"
  "vaddbf16512\0"
  "V32yV32yV32y\0"
  "vbcstnebf162ps128\0"
  "V4fyC*\0"
  "avxneconvert\0"
  "vbcstnebf162ps256\0"
  "V8fyC*\0"
  "vbcstnesh2ps128\0"
  "V4fxC*\0"
  "vbcstnesh2ps256\0"
  "V8fxC*\0"
  "vcmpbf16128_mask\0"
  "UcV8yV8yIiUc\0"
  "vcmpbf16256_mask\0"
  "UsV16yV16yIiUs\0"
  "vcmpbf16512_mask\0"
  "UiV32yV32yIiUi\0"
  "vcomisbf16eq\0"
  "iV8yV8y\0"
  "vcomisbf16ge\0"
  "vcomisbf16gt\0"
  "vcomisbf16le\0"
  "vcomisbf16lt\0"
  "vcomisbf16neq\0"
  "vcomisd\0"
  "iV2dV2dIiIi\0"
  "vcomish\0"
  "iV8xV8xIiIi\0"
  "vcomiss\0"
  "iV4fV4fIiIi\0"
  "vcvt2ph2bf8_128\0"
  "V16cV8xV8x\0"
  "vcvt2ph2bf8_256\0"
  "V32cV16xV16x\0"
  "vcvt2ph2bf8_512\0"
  "V64cV32xV32x\0"
  "vcvt2ph2bf8s_128\0"
  "vcvt2ph2bf8s_256\0"
  "vcvt2ph2bf8s_512\0"
  "vcvt2ph2hf8_128\0"
  "vcvt2ph2hf8_256\0"
  "vcvt2ph2hf8_512\0"
  "vcvt2ph2hf8s_128\0"
  "vcvt2ph2hf8s_256\0"
  "vcvt2ph2hf8s_512\0"
  "vcvt2ps2phx128_mask\0"
  "V8xV4fV4fV8xUc\0"
  "vcvt2ps2phx256_mask\0"
  "V16xV8fV8fV16xUs\0"
  "vcvt2ps2phx512_mask\0"
  "V32xV16fV16fV32xUiIi\0"
  "vcvtbf162ibs128\0"
  "V8UsV8y\0"
  "vcvtbf162ibs256\0"
  "V16UsV16y\0"
  "vcvtbf162ibs512\0"
  "V32UsV32y\0"
  "vcvtbf162iubs128\0"
  "vcvtbf162iubs256\0"
  "vcvtbf162iubs512\0"
  "vcvtbiasph2bf8_128_mask\0"
  "V16cV16cV8xV16cUc\0"
  "vcvtbiasph2bf8_256_mask\0"
  "V16cV32cV16xV16cUs\0"
  "vcvtbiasph2bf8_512_mask\0"
  "V32cV64cV32xV32cUi\0"
  "vcvtbiasph2bf8s_128_mask\0"
  "vcvtbiasph2bf8s_256_mask\0"
  "vcvtbiasph2bf8s_512_mask\0"
  "vcvtbiasph2hf8_128_mask\0"
  "vcvtbiasph2hf8_256_mask\0"
  "vcvtbiasph2hf8_512_mask\0"
  "vcvtbiasph2hf8s_128_mask\0"
  "vcvtbiasph2hf8s_256_mask\0"
  "vcvtbiasph2hf8s_512_mask\0"
  "vcvtdq2ph128_mask\0"
  "V8xV4iV8xUc\0"
  "vcvtdq2ph256_mask\0"
  "V8xV8iV8xUc\0"
  "vcvtdq2ph512_mask\0"
  "V16xV16iV16xUsIi\0"
  "vcvthf8_2ph128_mask\0"
  "V8xV16cV8xUc\0"
  "vcvthf8_2ph256_mask\0"
  "V16xV16cV16xUs\0"
  "vcvthf8_2ph512_mask\0"
  "V32xV32cV32xUi\0"
  "vcvtneebf162ps128\0"
  "V4fV8yC*\0"
  "vcvtneebf162ps256\0"
  "V8fV16yC*\0"
  "vcvtneeph2ps128\0"
  "V4fV8xC*\0"
  "vcvtneeph2ps256\0"
  "V8fV16xC*\0"
  "vcvtneobf162ps128\0"
  "vcvtneobf162ps256\0"
  "vcvtneoph2ps128\0"
  "vcvtneoph2ps256\0"
  "vcvtneps2bf16128\0"
  "V8yV4f\0"
  "avx512bf16,avx512vl|avxneconvert\0"
  "vcvtneps2bf16256\0"
  "V8yV8f\0"
  "vcvtpd2ph128_mask\0"
  "V8xV2dV8xUc\0"
  "vcvtpd2ph256_mask\0"
  "V8xV4dV8xUc\0"
  "vcvtpd2ph512_mask\0"
  "V8xV8dV8xUcIi\0"
  "vcvtph2bf8_128_mask\0"
  "V16cV8xV16cUc\0"
  "vcvtph2bf8_256_mask\0"
  "V16cV16xV16cUs\0"
  "vcvtph2bf8_512_mask\0"
  "V32cV32xV32cUi\0"
  "vcvtph2bf8s_128_mask\0"
  "vcvtph2bf8s_256_mask\0"
  "vcvtph2bf8s_512_mask\0"
  "vcvtph2dq128_mask\0"
  "V4iV8xV4iUc\0"
  "vcvtph2dq256_mask\0"
  "V8iV8xV8iUc\0"
  "vcvtph2dq512_mask\0"
  "V16iV16xV16iUsIi\0"
  "vcvtph2hf8_128_mask\0"
  "vcvtph2hf8_256_mask\0"
  "vcvtph2hf8_512_mask\0"
  "vcvtph2hf8s_128_mask\0"
  "vcvtph2hf8s_256_mask\0"
  "vcvtph2hf8s_512_mask\0"
  "vcvtph2ibs128_mask\0"
  "V8UsV8xV8UsUc\0"
  "vcvtph2ibs256_mask\0"
  "V16UsV16xV16UsUs\0"
  "vcvtph2ibs512_mask\0"
  "V32UsV32xV32UsUiIi\0"
  "vcvtph2iubs128_mask\0"
  "vcvtph2iubs256_mask\0"
  "vcvtph2iubs512_mask\0"
  "vcvtph2pd128_mask\0"
  "V2dV8xV2dUc\0"
  "vcvtph2pd256_mask\0"
  "V4dV8xV4dUc\0"
  "vcvtph2pd512_mask\0"
  "V8dV8xV8dUcIi\0"
  "vcvtph2ps256_mask\0"
  "V8fV8sV8fUc\0"
  "vcvtph2ps512_mask\0"
  "V16fV16sV16fUsIi\0"
  "vcvtph2ps_mask\0"
  "V4fV8sV4fUc\0"
  "vcvtph2psx128_mask\0"
  "V4fV8xV4fUc\0"
  "vcvtph2psx256_mask\0"
  "V8fV8xV8fUc\0"
  "vcvtph2psx512_mask\0"
  "V16fV16xV16fUsIi\0"
  "vcvtph2qq128_mask\0"
  "V2OiV8xV2OiUc\0"
  "vcvtph2qq256_mask\0"
  "V4OiV8xV4OiUc\0"
  "vcvtph2qq512_mask\0"
  "V8OiV8xV8OiUcIi\0"
  "vcvtph2udq128_mask\0"
  "V4UiV8xV4UiUc\0"
  "vcvtph2udq256_mask\0"
  "V8UiV8xV8UiUc\0"
  "vcvtph2udq512_mask\0"
  "V16UiV16xV16UiUsIi\0"
  "vcvtph2uqq128_mask\0"
  "V2UOiV8xV2UOiUc\0"
  "vcvtph2uqq256_mask\0"
  "V4UOiV8xV4UOiUc\0"
  "vcvtph2uqq512_mask\0"
  "V8UOiV8xV8UOiUcIi\0"
  "vcvtph2uw128_mask\0"
  "vcvtph2uw256_mask\0"
  "vcvtph2uw512_mask\0"
  "vcvtph2w128_mask\0"
  "V8sV8xV8sUc\0"
  "vcvtph2w256_mask\0"
  "V16sV16xV16sUs\0"
  "vcvtph2w512_mask\0"
  "V32sV32xV32sUiIi\0"
  "vcvtps2ibs128_mask\0"
  "V4UiV4fV4UiUc\0"
  "vcvtps2ibs256_mask\0"
  "V8UiV8fV8UiUc\0"
  "vcvtps2ibs512_mask\0"
  "V16UiV16fV16UiUsIi\0"
  "vcvtps2iubs128_mask\0"
  "vcvtps2iubs256_mask\0"
  "vcvtps2iubs512_mask\0"
  "vcvtps2ph\0"
  "V8sV4fIi\0"
  "f16c\0"
  "vcvtps2ph256\0"
  "V8sV8fIi\0"
  "vcvtps2ph256_mask\0"
  "V8sV8fIiV8sUc\0"
  "vcvtps2ph512_mask\0"
  "V16sV16fIiV16sUs\0"
  "vcvtps2ph_mask\0"
  "V8sV4fIiV8sUc\0"
  "vcvtps2phx128_mask\0"
  "V8xV4fV8xUc\0"
  "vcvtps2phx256_mask\0"
  "V8xV8fV8xUc\0"
  "vcvtps2phx512_mask\0"
  "V16xV16fV16xUsIi\0"
  "vcvtqq2ph128_mask\0"
  "V8xV2OiV8xUc\0"
  "vcvtqq2ph256_mask\0"
  "V8xV4OiV8xUc\0"
  "vcvtqq2ph512_mask\0"
  "V8xV8OiV8xUcIi\0"
  "vcvtsd2sh_round_mask\0"
  "V8xV8xV2dV8xUcIi\0"
  "vcvtsd2si32\0"
  "iV2dIi\0"
  "vcvtsd2usi32\0"
  "UiV2dIi\0"
  "vcvtsh2sd_round_mask\0"
  "V2dV2dV8xV2dUcIi\0"
  "vcvtsh2si32\0"
  "iV8xIi\0"
  "vcvtsh2ss_round_mask\0"
  "V4fV4fV8xV4fUcIi\0"
  "vcvtsh2usi32\0"
  "UiV8xIi\0"
  "vcvtsi2sh\0"
  "V8xV8xiIi\0"
  "vcvtss2sh_round_mask\0"
  "V8xV8xV4fV8xUcIi\0"
  "vcvtss2si32\0"
  "iV4fIi\0"
  "vcvtss2usi32\0"
  "UiV4fIi\0"
  "vcvttbf162ibs128\0"
  "vcvttbf162ibs256\0"
  "vcvttbf162ibs512\0"
  "vcvttbf162iubs128\0"
  "vcvttbf162iubs256\0"
  "vcvttbf162iubs512\0"
  "vcvttpd2dqs128_mask\0"
  "vcvttpd2dqs256_mask\0"
  "vcvttpd2dqs512_round_mask\0"
  "vcvttpd2qqs128_mask\0"
  "vcvttpd2qqs256_mask\0"
  "vcvttpd2qqs512_round_mask\0"
  "vcvttpd2udqs128_mask\0"
  "vcvttpd2udqs256_mask\0"
  "vcvttpd2udqs512_round_mask\0"
  "vcvttpd2uqqs128_mask\0"
  "vcvttpd2uqqs256_mask\0"
  "vcvttpd2uqqs512_round_mask\0"
  "vcvttph2dq128_mask\0"
  "vcvttph2dq256_mask\0"
  "vcvttph2dq512_mask\0"
  "vcvttph2ibs128_mask\0"
  "vcvttph2ibs256_mask\0"
  "vcvttph2ibs512_mask\0"
  "vcvttph2iubs128_mask\0"
  "vcvttph2iubs256_mask\0"
  "vcvttph2iubs512_mask\0"
  "vcvttph2qq128_mask\0"
  "vcvttph2qq256_mask\0"
  "vcvttph2qq512_mask\0"
  "vcvttph2udq128_mask\0"
  "vcvttph2udq256_mask\0"
  "vcvttph2udq512_mask\0"
  "vcvttph2uqq128_mask\0"
  "vcvttph2uqq256_mask\0"
  "vcvttph2uqq512_mask\0"
  "vcvttph2uw128_mask\0"
  "vcvttph2uw256_mask\0"
  "vcvttph2uw512_mask\0"
  "vcvttph2w128_mask\0"
  "vcvttph2w256_mask\0"
  "vcvttph2w512_mask\0"
  "vcvttps2dqs128_mask\0"
  "vcvttps2dqs256_mask\0"
  "vcvttps2dqs512_round_mask\0"
  "vcvttps2ibs128_mask\0"
  "vcvttps2ibs256_mask\0"
  "vcvttps2ibs512_mask\0"
  "vcvttps2iubs128_mask\0"
  "vcvttps2iubs256_mask\0"
  "vcvttps2iubs512_mask\0"
  "vcvttps2qqs128_mask\0"
  "vcvttps2qqs256_mask\0"
  "vcvttps2qqs512_round_mask\0"
  "vcvttps2udqs128_mask\0"
  "vcvttps2udqs256_mask\0"
  "vcvttps2udqs512_round_mask\0"
  "vcvttps2uqqs128_mask\0"
  "vcvttps2uqqs256_mask\0"
  "vcvttps2uqqs512_round_mask\0"
  "vcvttsd2si32\0"
  "vcvttsd2sis32\0"
  "vcvttsd2usi32\0"
  "vcvttsd2usis32\0"
  "vcvttsh2si32\0"
  "vcvttsh2usi32\0"
  "vcvttss2si32\0"
  "vcvttss2sis32\0"
  "vcvttss2usi32\0"
  "vcvttss2usis32\0"
  "vcvtudq2ph128_mask\0"
  "V8xV4UiV8xUc\0"
  "vcvtudq2ph256_mask\0"
  "V8xV8UiV8xUc\0"
  "vcvtudq2ph512_mask\0"
  "V16xV16UiV16xUsIi\0"
  "vcvtuqq2ph128_mask\0"
  "V8xV2UOiV8xUc\0"
  "vcvtuqq2ph256_mask\0"
  "V8xV4UOiV8xUc\0"
  "vcvtuqq2ph512_mask\0"
  "V8xV8UOiV8xUcIi\0"
  "vcvtusi2sh\0"
  "V8xV8xUiIi\0"
  "vcvtuw2ph128_mask\0"
  "V8xV8UsV8xUc\0"
  "vcvtuw2ph256_mask\0"
  "V16xV16UsV16xUs\0"
  "vcvtuw2ph512_mask\0"
  "V32xV32UsV32xUiIi\0"
  "vcvtw2ph128_mask\0"
  "V8xV8sV8xUc\0"
  "vcvtw2ph256_mask\0"
  "V16xV16sV16xUs\0"
  "vcvtw2ph512_mask\0"
  "V32xV32sV32xUiIi\0"
  "vdivbf16128\0"
  "vdivbf16256\0"
  "vdivbf16512\0"
  "vdpphps128\0"
  "V4fV4fV8xV8x\0"
  "vdpphps256\0"
  "V8fV8fV16xV16x\0"
  "vdpphps512\0"
  "V16fV16fV32xV32x\0"
  "vec_ext_v2di\0"
  "OiV2OiIi\0"
  "vec_ext_v4hi\0"
  "sV4sIi\0"
  "ncEV:64:\0"
  "vec_ext_v4sf\0"
  "fV4fIi\0"
  "vec_ext_v4si\0"
  "iV4iIi\0"
  "vec_ext_v8hi\0"
  "sV8sIi\0"
  "vec_ext_v8si\0"
  "iV8iIi\0"
  "vec_ext_v16hi\0"
  "sV16sIi\0"
  "vec_ext_v16qi\0"
  "cV16cIi\0"
  "vec_ext_v32qi\0"
  "cV32cIi\0"
  "vec_set_v4hi\0"
  "V4sV4ssIi\0"
  "vec_set_v4si\0"
  "V4iV4iiIi\0"
  "vec_set_v8hi\0"
  "V8sV8ssIi\0"
  "vec_set_v8si\0"
  "V8iV8iiIi\0"
  "vec_set_v16hi\0"
  "V16sV16ssIi\0"
  "vec_set_v16qi\0"
  "V16cV16ccIi\0"
  "vec_set_v32qi\0"
  "V32cV32ccIi\0"
  "vextractf128_pd256\0"
  "V2dV4dIi\0"
  "vextractf128_ps256\0"
  "V4fV8fIi\0"
  "vextractf128_si256\0"
  "V4iV8iIi\0"
  "vfcmaddcph128_mask\0"
  "vfcmaddcph128_maskz\0"
  "vfcmaddcph256_mask\0"
  "vfcmaddcph256_maskz\0"
  "vfcmaddcph512_mask\0"
  "vfcmaddcph512_mask3\0"
  "vfcmaddcph512_maskz\0"
  "vfcmaddcsh_mask\0"
  "vfcmaddcsh_maskz\0"
  "vfcmaddcsh_round_mask\0"
  "vfcmaddcsh_round_mask3\0"
  "vfcmulcph128_mask\0"
  "vfcmulcph256_mask\0"
  "vfcmulcph512_mask\0"
  "vfcmulcsh_mask\0"
  "vfmaddcph128_mask\0"
  "vfmaddcph128_maskz\0"
  "vfmaddcph256_mask\0"
  "vfmaddcph256_maskz\0"
  "vfmaddcph512_mask\0"
  "vfmaddcph512_mask3\0"
  "vfmaddcph512_maskz\0"
  "vfmaddcsh_mask\0"
  "vfmaddcsh_maskz\0"
  "vfmaddcsh_round_mask\0"
  "vfmaddcsh_round_mask3\0"
  "vfmaddpd512_mask\0"
  "vfmaddpd512_mask3\0"
  "vfmaddpd512_maskz\0"
  "vfmaddph512_mask\0"
  "vfmaddph512_mask3\0"
  "vfmaddph512_maskz\0"
  "vfmaddps512_mask\0"
  "vfmaddps512_mask3\0"
  "vfmaddps512_maskz\0"
  "vfmaddsd3_mask\0"
  "vfmaddsd3_mask3\0"
  "vfmaddsd3_maskz\0"
  "vfmaddsh3_mask\0"
  "vfmaddsh3_mask3\0"
  "vfmaddsh3_maskz\0"
  "vfmaddss3_mask\0"
  "vfmaddss3_mask3\0"
  "vfmaddss3_maskz\0"
  "vfmaddsubpd\0"
  "fma|fma4\0"
  "vfmaddsubpd256\0"
  "vfmaddsubpd512_mask\0"
  "vfmaddsubpd512_mask3\0"
  "vfmaddsubpd512_maskz\0"
  "vfmaddsubph\0"
  "V8xV8xV8xV8x\0"
  "vfmaddsubph256\0"
  "V16xV16xV16xV16x\0"
  "vfmaddsubph512_mask\0"
  "vfmaddsubph512_mask3\0"
  "vfmaddsubph512_maskz\0"
  "vfmaddsubps\0"
  "vfmaddsubps256\0"
  "vfmaddsubps512_mask\0"
  "vfmaddsubps512_mask3\0"
  "vfmaddsubps512_maskz\0"
  "vfmsubaddpd512_mask3\0"
  "vfmsubaddph512_mask3\0"
  "vfmsubaddps512_mask3\0"
  "vfmsubpd512_mask3\0"
  "vfmsubph512_mask3\0"
  "vfmsubps512_mask3\0"
  "vfmsubsd3_mask3\0"
  "vfmsubsh3_mask3\0"
  "vfmsubss3_mask3\0"
  "vfmulcph128_mask\0"
  "vfmulcph256_mask\0"
  "vfmulcph512_mask\0"
  "vfmulcsh_mask\0"
  "vfpclassbf16128_mask\0"
  "UcV8yIiUc\0"
  "vfpclassbf16256_mask\0"
  "UsV16yIiUs\0"
  "vfpclassbf16512_mask\0"
  "UiV32yIiUi\0"
  "vfrczpd\0"
  "V2dV2d\0"
  "xop\0"
  "vfrczpd256\0"
  "V4dV4d\0"
  "vfrczps\0"
  "vfrczps256\0"
  "vfrczsd\0"
  "vfrczss\0"
  "vgetexpbf16128_mask\0"
  "V8yV8yV8yUc\0"
  "vgetexpbf16256_mask\0"
  "V16yV16yV16yUs\0"
  "vgetexpbf16512_mask\0"
  "V32yV32yV32yUi\0"
  "vgetmantbf16128_mask\0"
  "V8yV8yIiV8yUc\0"
  "vgetmantbf16256_mask\0"
  "V16yV16yIiV16yUs\0"
  "vgetmantbf16512_mask\0"
  "V32yV32yIiV32yUi\0"
  "vgf2p8affineinvqb_v16qi\0"
  "gfni\0"
  "vgf2p8affineinvqb_v32qi\0"
  "avx,gfni\0"
  "vgf2p8affineinvqb_v64qi\0"
  "V64cV64cV64cIc\0"
  "avx512f,gfni\0"
  "vgf2p8affineqb_v16qi\0"
  "vgf2p8affineqb_v32qi\0"
  "vgf2p8affineqb_v64qi\0"
  "vgf2p8mulb_v16qi\0"
  "vgf2p8mulb_v32qi\0"
  "vgf2p8mulb_v64qi\0"
  "vinsertf128_pd256\0"
  "vinsertf128_ps256\0"
  "vinsertf128_si256\0"
  "vmaxbf16128\0"
  "vmaxbf16256\0"
  "vmaxbf16512\0"
  "vminbf16128\0"
  "vminbf16256\0"
  "vminbf16512\0"
  "vminmaxbf16128\0"
  "V8yV8yV8yIi\0"
  "vminmaxbf16256\0"
  "V16yV16yV16yIi\0"
  "vminmaxbf16512\0"
  "V32yV32yV32yIi\0"
  "vminmaxpd128_mask\0"
  "vminmaxpd256_mask\0"
  "vminmaxpd512_round_mask\0"
  "vminmaxph128_mask\0"
  "V8xV8xV8xIiV8xUc\0"
  "vminmaxph256_mask\0"
  "V16xV16xV16xIiV16xUs\0"
  "vminmaxph512_round_mask\0"
  "V32xV32xV32xIiV32xUiIi\0"
  "vminmaxps128_mask\0"
  "vminmaxps256_mask\0"
  "vminmaxps512_round_mask\0"
  "vminmaxsd_round_mask\0"
  "vminmaxsh_round_mask\0"
  "vminmaxss_round_mask\0"
  "vmulbf16128\0"
  "vmulbf16256\0"
  "vmulbf16512\0"
  "vp2intersect_d_128\0"
  "vV4iV4iUc*Uc*\0"
  "avx512vp2intersect,avx512vl\0"
  "vp2intersect_d_256\0"
  "vV8iV8iUc*Uc*\0"
  "vp2intersect_d_512\0"
  "vV16iV16iUs*Us*\0"
  "avx512vp2intersect\0"
  "vp2intersect_q_128\0"
  "vV2OiV2OiUc*Uc*\0"
  "vp2intersect_q_256\0"
  "vV4OiV4OiUc*Uc*\0"
  "vp2intersect_q_512\0"
  "vV8OiV8OiUc*Uc*\0"
  "vpcomb\0"
  "vpcomd\0"
  "vpcomq\0"
  "vpcomub\0"
  "vpcomud\0"
  "vpcomuq\0"
  "vpcomuw\0"
  "V8sV8sV8sIc\0"
  "vpcomw\0"
  "vpconflictdi_128\0"
  "avx512cd,avx512vl\0"
  "vpconflictdi_256\0"
  "V4OiV4Oi\0"
  "vpconflictdi_512\0"
  "V8OiV8Oi\0"
  "avx512cd\0"
  "vpconflictsi_128\0"
  "V4iV4i\0"
  "vpconflictsi_256\0"
  "V8iV8i\0"
  "vpconflictsi_512\0"
  "V16iV16i\0"
  "vpdpbssd128\0"
  "V4iV4iV16cV16c\0"
  "avxvnniint8|avx10.2\0"
  "vpdpbssd256\0"
  "V8iV8iV32cV32c\0"
  "vpdpbssd512\0"
  "V16iV16iV64cV64c\0"
  "vpdpbssds128\0"
  "vpdpbssds256\0"
  "vpdpbssds512\0"
  "vpdpbsud128\0"
  "V4iV4iV16cV16Uc\0"
  "vpdpbsud256\0"
  "V8iV8iV32cV32Uc\0"
  "vpdpbsud512\0"
  "V16iV16iV64cV64Uc\0"
  "vpdpbsuds128\0"
  "vpdpbsuds256\0"
  "vpdpbsuds512\0"
  "vpdpbusd128\0"
  "V4iV4iV16UcV16c\0"
  "avx512vl,avx512vnni|avxvnni\0"
  "vpdpbusd256\0"
  "V8iV8iV32UcV32c\0"
  "vpdpbusd512\0"
  "V16iV16iV64UcV64c\0"
  "avx512vnni\0"
  "vpdpbusds128\0"
  "vpdpbusds256\0"
  "vpdpbusds512\0"
  "vpdpbuud128\0"
  "V4iV4iV16UcV16Uc\0"
  "vpdpbuud256\0"
  "V8iV8iV32UcV32Uc\0"
  "vpdpbuud512\0"
  "V16iV16iV64UcV64Uc\0"
  "vpdpbuuds128\0"
  "vpdpbuuds256\0"
  "vpdpbuuds512\0"
  "vpdpwssd128\0"
  "V4iV4iV8sV8s\0"
  "vpdpwssd256\0"
  "V8iV8iV16sV16s\0"
  "vpdpwssd512\0"
  "V16iV16iV32sV32s\0"
  "vpdpwssds128\0"
  "vpdpwssds256\0"
  "vpdpwssds512\0"
  "vpdpwsud128\0"
  "V4iV4iV8sV8Us\0"
  "avxvnniint16|avx10.2\0"
  "vpdpwsud256\0"
  "V8iV8iV16sV16Us\0"
  "vpdpwsud512\0"
  "V16iV16iV32sV32Us\0"
  "vpdpwsuds128\0"
  "vpdpwsuds256\0"
  "vpdpwsuds512\0"
  "vpdpwusd128\0"
  "V4iV4iV8UsV8s\0"
  "vpdpwusd256\0"
  "V8iV8iV16UsV16s\0"
  "vpdpwusd512\0"
  "V16iV16iV32UsV32s\0"
  "vpdpwusds128\0"
  "vpdpwusds256\0"
  "vpdpwusds512\0"
  "vpdpwuud128\0"
  "V4iV4iV8UsV8Us\0"
  "vpdpwuud256\0"
  "V8iV8iV16UsV16Us\0"
  "vpdpwuud512\0"
  "V16iV16iV32UsV32Us\0"
  "vpdpwuuds128\0"
  "vpdpwuuds256\0"
  "vpdpwuuds512\0"
  "vperm2f128_pd256\0"
  "vperm2f128_ps256\0"
  "vperm2f128_si256\0"
  "vpermi2vard128\0"
  "vpermi2vard256\0"
  "V8iV8iV8iV8i\0"
  "vpermi2vard512\0"
  "V16iV16iV16iV16i\0"
  "vpermi2varhi128\0"
  "V8sV8sV8sV8s\0"
  "vpermi2varhi256\0"
  "V16sV16sV16sV16s\0"
  "vpermi2varhi512\0"
  "V32sV32sV32sV32s\0"
  "vpermi2varpd128\0"
  "V2dV2dV2OiV2d\0"
  "vpermi2varpd256\0"
  "V4dV4dV4OiV4d\0"
  "vpermi2varpd512\0"
  "V8dV8dV8OiV8d\0"
  "vpermi2varps128\0"
  "V4fV4fV4iV4f\0"
  "vpermi2varps256\0"
  "V8fV8fV8iV8f\0"
  "vpermi2varps512\0"
  "V16fV16fV16iV16f\0"
  "vpermi2varq128\0"
  "V2OiV2OiV2OiV2Oi\0"
  "vpermi2varq256\0"
  "V4OiV4OiV4OiV4Oi\0"
  "vpermi2varq512\0"
  "V8OiV8OiV8OiV8Oi\0"
  "vpermi2varqi128\0"
  "vpermi2varqi256\0"
  "vpermi2varqi512\0"
  "V64cV64cV64cV64c\0"
  "vpermil2pd\0"
  "V2dV2dV2dV2OiIc\0"
  "vpermil2pd256\0"
  "V4dV4dV4dV4OiIc\0"
  "vpermil2ps\0"
  "V4fV4fV4fV4iIc\0"
  "vpermil2ps256\0"
  "V8fV8fV8fV8iIc\0"
  "vpermilpd\0"
  "vpermilpd256\0"
  "vpermilpd512\0"
  "vpermilps\0"
  "vpermilps256\0"
  "vpermilps512\0"
  "vpermilvarpd\0"
  "V2dV2dV2Oi\0"
  "vpermilvarpd256\0"
  "vpermilvarpd512\0"
  "vpermilvarps\0"
  "V4fV4fV4i\0"
  "vpermilvarps256\0"
  "vpermilvarps512\0"
  "vphaddbd\0"
  "V4iV16c\0"
  "vphaddbq\0"
  "V2OiV16c\0"
  "vphaddbw\0"
  "V8sV16c\0"
  "vphadddq\0"
  "V2OiV4i\0"
  "vphaddubd\0"
  "vphaddubq\0"
  "vphaddubw\0"
  "vphaddudq\0"
  "vphadduwd\0"
  "V4iV8s\0"
  "vphadduwq\0"
  "V2OiV8s\0"
  "vphaddwd\0"
  "vphaddwq\0"
  "vphsubbw\0"
  "vphsubdq\0"
  "vphsubwd\0"
  "vpmacsdd\0"
  "vpmacsdqh\0"
  "V2OiV4iV4iV2Oi\0"
  "vpmacsdql\0"
  "vpmacssdd\0"
  "vpmacssdqh\0"
  "vpmacssdql\0"
  "vpmacsswd\0"
  "V4iV8sV8sV4i\0"
  "vpmacssww\0"
  "vpmacswd\0"
  "vpmacsww\0"
  "vpmadcsswd\0"
  "vpmadcswd\0"
  "vpmadd52huq128\0"
  "avx512ifma,avx512vl|avxifma\0"
  "vpmadd52huq256\0"
  "vpmadd52huq512\0"
  "avx512ifma\0"
  "vpmadd52luq128\0"
  "vpmadd52luq256\0"
  "vpmadd52luq512\0"
  "vpmultishiftqb128\0"
  "vpmultishiftqb256\0"
  "vpmultishiftqb512\0"
  "vpperm\0"
  "vprotbi\0"
  "V16cV16cIc\0"
  "vprotdi\0"
  "V4iV4iIc\0"
  "vprotqi\0"
  "vprotwi\0"
  "V8sV8sIc\0"
  "vpshab\0"
  "vpshad\0"
  "vpshaq\0"
  "vpshaw\0"
  "vpshlb\0"
  "vpshld\0"
  "vpshldd128\0"
  "vpshldd256\0"
  "vpshldd512\0"
  "vpshldq128\0"
  "vpshldq256\0"
  "vpshldq512\0"
  "vpshldw128\0"
  "vpshldw256\0"
  "vpshldw512\0"
  "V32sV32sV32sIi\0"
  "vpshlq\0"
  "vpshlw\0"
  "vpshrdd128\0"
  "vpshrdd256\0"
  "vpshrdd512\0"
  "vpshrdq128\0"
  "vpshrdq256\0"
  "vpshrdq512\0"
  "vpshrdw128\0"
  "vpshrdw256\0"
  "vpshrdw512\0"
  "vpshufbitqmb128_mask\0"
  "UsV16cV16cUs\0"
  "avx512vl,avx512bitalg\0"
  "vpshufbitqmb256_mask\0"
  "UiV32cV32cUi\0"
  "vpshufbitqmb512_mask\0"
  "UOiV64cV64cUOi\0"
  "avx512bitalg\0"
  "vrcpbf16128_mask\0"
  "vrcpbf16256_mask\0"
  "vrcpbf16512_mask\0"
  "vreducebf16128_mask\0"
  "vreducebf16256_mask\0"
  "vreducebf16512_mask\0"
  "vrndscalebf16_128_mask\0"
  "vrndscalebf16_256_mask\0"
  "vrndscalebf16_mask\0"
  "vrsqrtbf16128_mask\0"
  "vrsqrtbf16256_mask\0"
  "vrsqrtbf16512_mask\0"
  "vscalefbf16128_mask\0"
  "V8yV8yV8yV8yUc\0"
  "vscalefbf16256_mask\0"
  "V16yV16yV16yV16yUs\0"
  "vscalefbf16512_mask\0"
  "V32yV32yV32yV32yUi\0"
  "vsha512msg1\0"
  "V4UOiV4UOiV2UOi\0"
  "sha512\0"
  "vsha512msg2\0"
  "V4UOiV4UOiV4UOi\0"
  "vsha512rnds2\0"
  "V4UOiV4UOiV4UOiV2UOi\0"
  "vsm3msg1\0"
  "V4UiV4UiV4UiV4Ui\0"
  "sm3\0"
  "vsm3msg2\0"
  "vsm3rnds2\0"
  "V4UiV4UiV4UiV4UiIUi\0"
  "vsm4key4128\0"
  "V4UiV4UiV4Ui\0"
  "sm4\0"
  "vsm4key4256\0"
  "V8UiV8UiV8Ui\0"
  "vsm4key4512\0"
  "V16UiV16UiV16Ui\0"
  "avx10.2,sm4\0"
  "vsm4rnds4128\0"
  "vsm4rnds4256\0"
  "vsm4rnds4512\0"
  "vsubbf16128\0"
  "vsubbf16256\0"
  "vsubbf16512\0"
  "vtestcpd\0"
  "vtestcpd256\0"
  "iV4dV4d\0"
  "vtestcps\0"
  "vtestcps256\0"
  "iV8fV8f\0"
  "vtestnzcpd\0"
  "vtestnzcpd256\0"
  "vtestnzcps\0"
  "vtestnzcps256\0"
  "vtestzpd\0"
  "vtestzpd256\0"
  "vtestzps\0"
  "vtestzps256\0"
  "vzeroall\0"
  "vzeroupper\0"
  "wbinvd\0"
  "wbnoinvd\0"
  "writeeflags_u32\0"
  "wrpkru\0"
  "wrssd\0"
  "wrussd\0"
  "xabort\0"
  "vIc\0"
  "rtm\0"
  "xbegin\0"
  "i\0"
  "xend\0"
  "xgetbv\0"
  "UOiUi\0"
  "xsave\0"
  "xresldtrk\0"
  "tsxldtrk\0"
  "xrstor\0"
  "vv*UOi\0"
  "xrstors\0"
  "xsaves\0"
  "xsavec\0"
  "xsaveopt\0"
  "xsetbv\0"
  "vUiUOi\0"
  "xsusldtrk\0"
  "xtest\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinStrings = BuiltinStringsStorage;

#endif // GET_BUILTIN_STR_TABLE

#ifdef GET_BUILTIN_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{1 /* _AddressOfReturnAddress */, 25 /* v* */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31 /* _BitScanForward */, 47 /* UcUNi*UNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{57 /* _BitScanReverse */, 47 /* UcUNi*UNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{73 /* _InterlockedAnd64 */, 91 /* WiWiD*Wi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{100 /* _InterlockedDecrement64 */, 124 /* WiWiD* */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{131 /* _InterlockedExchange64 */, 91 /* WiWiD*Wi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154 /* _InterlockedExchangeAdd64 */, 91 /* WiWiD*Wi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{180 /* _InterlockedExchangeSub64 */, 91 /* WiWiD*Wi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{206 /* _InterlockedIncrement64 */, 124 /* WiWiD* */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{230 /* _InterlockedOr64 */, 91 /* WiWiD*Wi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{247 /* _InterlockedXor64 */, 91 /* WiWiD*Wi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{265 /* _ReadBarrier */, 278 /* v */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{280 /* _ReadWriteBarrier */, 278 /* v */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{298 /* _WriteBarrier */, 278 /* v */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{312 /* __cpuid */, 320 /* vi*i */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{325 /* __cpuidex */, 335 /* vi*ii */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{341 /* __emul */, 348 /* LLiii */, 354 /* nch */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{358 /* __emulu */, 366 /* ULLiUiUi */, 354 /* nch */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{375 /* __int2c */, 278 /* v */, 383 /* nhr */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{387 /* __rdtsc */, 395 /* UOi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{399 /* __readfsbyte */, 412 /* UcUNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{418 /* __readfsdword */, 432 /* UNiUNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{439 /* __readfsqword */, 453 /* ULLiUNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{461 /* __readfsword */, 474 /* UsUNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{480 /* __readgsbyte */, 412 /* UcUNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{493 /* __readgsdword */, 432 /* UNiUNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{507 /* __readgsqword */, 453 /* ULLiUNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{521 /* __readgsword */, 474 /* UsUNi */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{534 /* __stosb */, 542 /* vUc*Ucz */, 28 /* nh */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{550 /* __ud2 */, 278 /* v */, 383 /* nhr */, 0}, HeaderDesc::INTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{556 /* _m_prefetch */, 568 /* vv* */, 572 /* nc */, 0}, HeaderDesc::MMINTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{575 /* _m_prefetchw */, 588 /* vvDC* */, 572 /* nc */, 0}, HeaderDesc::INTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{594 /* _mm_clflush */, 606 /* vvC* */, 28 /* nh */, 0}, HeaderDesc::EMMINTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{611 /* _mm_getcsr */, 622 /* Ui */, 28 /* nh */, 0}, HeaderDesc::XMMINTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{625 /* _mm_lfence */, 278 /* v */, 28 /* nh */, 0}, HeaderDesc::EMMINTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{636 /* _mm_mfence */, 278 /* v */, 28 /* nh */, 0}, HeaderDesc::EMMINTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{647 /* _mm_pause */, 278 /* v */, 28 /* nh */, 0}, HeaderDesc::EMMINTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{657 /* _mm_prefetch */, 670 /* vcC*i */, 572 /* nc */, 0}, HeaderDesc::XMMINTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{676 /* _mm_setcsr */, 687 /* vUi */, 28 /* nh */, 0}, HeaderDesc::XMMINTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{691 /* _mm_sfence */, 278 /* v */, 28 /* nh */, 0}, HeaderDesc::XMMINTRIN_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{702 /* _xgetbv */, 710 /* UWiUi */, 28 /* nh */, 0}, HeaderDesc::IMMINTRIN_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{716 /* _xsetbv */, 724 /* vUiUWi */, 28 /* nh */, 0}, HeaderDesc::IMMINTRIN_H, ALL_MS_LANGUAGES},

#endif // GET_BUILTIN_INFOS

#ifdef GET_BUILTIN_PREFIXED_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{731 /* aadd32 */, 738 /* vv*Si */, 744 /* n */, 746 /* raoint */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{753 /* aand32 */, 738 /* vv*Si */, 744 /* n */, 746 /* raoint */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{760 /* addcarryx_u32 */, 774 /* UcUcUiUiUi* */, 786 /* nE */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{789 /* addpd512 */, 798 /* V8dV8dV8dIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{827 /* addph512 */, 836 /* V32xV32xV32xIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{862 /* addps512 */, 871 /* V16fV16fV16fIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{886 /* addsd_round_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{929 /* addsh_round_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{963 /* addss_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{997 /* addsubpd */, 1006 /* V2dV2dV2d */, 1016 /* ncEV:128: */, 1026 /* sse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1031 /* addsubpd256 */, 1043 /* V4dV4dV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1067 /* addsubps */, 1076 /* V4fV4fV4f */, 1016 /* ncEV:128: */, 1026 /* sse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1086 /* addsubps256 */, 1098 /* V8fV8fV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1108 /* aesdec128 */, 1118 /* V2OiV2OiV2Oi */, 920 /* ncV:128: */, 1131 /* aes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1135 /* aesdec128kl_u8 */, 1150 /* UcV2Oi*V2OivC* */, 1165 /* nV:128: */, 1173 /* kl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1176 /* aesdec256 */, 1186 /* V4OiV4OiV4Oi */, 1199 /* ncV:256: */, 1208 /* vaes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1213 /* aesdec256kl_u8 */, 1150 /* UcV2Oi*V2OivC* */, 1165 /* nV:128: */, 1173 /* kl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1228 /* aesdec512 */, 1238 /* V8OiV8OiV8Oi */, 810 /* ncV:512: */, 1251 /* avx512f,vaes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1264 /* aesdeclast128 */, 1118 /* V2OiV2OiV2Oi */, 920 /* ncV:128: */, 1131 /* aes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1278 /* aesdeclast256 */, 1186 /* V4OiV4OiV4Oi */, 1199 /* ncV:256: */, 1208 /* vaes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1292 /* aesdeclast512 */, 1238 /* V8OiV8OiV8Oi */, 810 /* ncV:512: */, 1251 /* avx512f,vaes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1306 /* aesdecwide128kl_u8 */, 1325 /* UcV2Oi*V2OiC*vC* */, 1165 /* nV:128: */, 1342 /* kl,widekl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1352 /* aesdecwide256kl_u8 */, 1325 /* UcV2Oi*V2OiC*vC* */, 1165 /* nV:128: */, 1342 /* kl,widekl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1371 /* aesenc128 */, 1118 /* V2OiV2OiV2Oi */, 920 /* ncV:128: */, 1131 /* aes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1381 /* aesenc128kl_u8 */, 1150 /* UcV2Oi*V2OivC* */, 1165 /* nV:128: */, 1173 /* kl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1396 /* aesenc256 */, 1186 /* V4OiV4OiV4Oi */, 1199 /* ncV:256: */, 1208 /* vaes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1406 /* aesenc256kl_u8 */, 1150 /* UcV2Oi*V2OivC* */, 1165 /* nV:128: */, 1173 /* kl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1421 /* aesenc512 */, 1238 /* V8OiV8OiV8Oi */, 810 /* ncV:512: */, 1251 /* avx512f,vaes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1431 /* aesenclast128 */, 1118 /* V2OiV2OiV2Oi */, 920 /* ncV:128: */, 1131 /* aes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1445 /* aesenclast256 */, 1186 /* V4OiV4OiV4Oi */, 1199 /* ncV:256: */, 1208 /* vaes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1459 /* aesenclast512 */, 1238 /* V8OiV8OiV8Oi */, 810 /* ncV:512: */, 1251 /* avx512f,vaes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1473 /* aesencwide128kl_u8 */, 1325 /* UcV2Oi*V2OiC*vC* */, 1165 /* nV:128: */, 1342 /* kl,widekl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1492 /* aesencwide256kl_u8 */, 1325 /* UcV2Oi*V2OiC*vC* */, 1165 /* nV:128: */, 1342 /* kl,widekl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1511 /* aesimc128 */, 1521 /* V2OiV2Oi */, 920 /* ncV:128: */, 1131 /* aes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1530 /* aeskeygenassist128 */, 1549 /* V2OiV2OiIc */, 920 /* ncV:128: */, 1131 /* aes */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1560 /* alignd128 */, 1570 /* V4iV4iV4iIi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1591 /* alignd256 */, 1601 /* V8iV8iV8iIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1613 /* alignd512 */, 1623 /* V16iV16iV16iIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1648 /* alignq128 */, 1658 /* V2OiV2OiV2OiIi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1673 /* alignq256 */, 1683 /* V4OiV4OiV4OiIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1698 /* alignq512 */, 1708 /* V8OiV8OiV8OiIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1723 /* aor32 */, 738 /* vv*Si */, 744 /* n */, 746 /* raoint */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1729 /* axor32 */, 738 /* vv*Si */, 744 /* n */, 746 /* raoint */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1736 /* bextr_u32 */, 1746 /* UiUiUi */, 1753 /* ncE */, 1757 /* bmi */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1761 /* bextri_u32 */, 1772 /* UiUiIUi */, 1753 /* ncE */, 1780 /* tbm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1784 /* blendpd */, 1792 /* V2dV2dV2dIi */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1811 /* blendpd256 */, 1822 /* V4dV4dV4dIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1834 /* blendps */, 1842 /* V4fV4fV4fIi */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1854 /* blendps256 */, 1865 /* V8fV8fV8fIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1877 /* blendvpd */, 1886 /* V2dV2dV2dV2d */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1899 /* blendvpd256 */, 1911 /* V4dV4dV4dV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1924 /* blendvps */, 1933 /* V4fV4fV4fV4f */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1946 /* blendvps256 */, 1958 /* V8fV8fV8fV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1971 /* bzhi_si */, 1746 /* UiUiUi */, 1753 /* ncE */, 1979 /* bmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1984 /* cldemote */, 606 /* vvC* */, 744 /* n */, 1984 /* cldemote */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1993 /* clflush */, 606 /* vvC* */, 744 /* n */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2006 /* clflushopt */, 606 /* vvC* */, 744 /* n */, 2006 /* clflushopt */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2017 /* clrssbsy */, 568 /* vv* */, 744 /* n */, 2026 /* shstk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2032 /* clwb */, 606 /* vvC* */, 744 /* n */, 2032 /* clwb */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2037 /* clzero */, 568 /* vv* */, 744 /* n */, 2037 /* clzero */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2044 /* cmpb128_mask */, 2057 /* UsV16cV16cIiUs */, 1016 /* ncEV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2090 /* cmpb256_mask */, 2103 /* UiV32cV32cIiUi */, 1053 /* ncEV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2118 /* cmpb512_mask */, 2131 /* UOiV64cV64cIiUOi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2157 /* cmpd128_mask */, 2170 /* UcV4iV4iIiUc */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2183 /* cmpd256_mask */, 2196 /* UcV8iV8iIiUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2209 /* cmpd512_mask */, 2222 /* UsV16iV16iIiUs */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2237 /* cmpeqpd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2254 /* cmpeqps */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2266 /* cmpeqsd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2274 /* cmpeqss */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2282 /* cmplepd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2290 /* cmpleps */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2298 /* cmplesd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2306 /* cmpless */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2314 /* cmpltpd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2322 /* cmpltps */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2330 /* cmpltsd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2338 /* cmpltss */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2346 /* cmpneqpd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2355 /* cmpneqps */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2364 /* cmpneqsd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2373 /* cmpneqss */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2382 /* cmpnlepd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2391 /* cmpnleps */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2400 /* cmpnlesd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2409 /* cmpnless */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2418 /* cmpnltpd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2427 /* cmpnltps */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2436 /* cmpnltsd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2445 /* cmpnltss */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2454 /* cmpordpd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2463 /* cmpordps */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2472 /* cmpordsd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2481 /* cmpordss */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2490 /* cmppd */, 2496 /* V2dV2dV2dIc */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2508 /* cmppd128_mask */, 2522 /* UcV2dV2dIiUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2535 /* cmppd256 */, 2544 /* V4dV4dV4dIc */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2556 /* cmppd256_mask */, 2570 /* UcV4dV4dIiUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2583 /* cmppd512_mask */, 2597 /* UcV8dV8dIiUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2612 /* cmpph128_mask */, 2626 /* UcV8xV8xIiUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2659 /* cmpph256_mask */, 2673 /* UsV16xV16xIiUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2688 /* cmpph512_mask */, 2702 /* UiV32xV32xIiUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2719 /* cmpps */, 2725 /* V4fV4fV4fIc */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2737 /* cmpps128_mask */, 2751 /* UcV4fV4fIiUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2764 /* cmpps256 */, 2773 /* V8fV8fV8fIc */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2785 /* cmpps256_mask */, 2799 /* UcV8fV8fIiUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2812 /* cmpps512_mask */, 2826 /* UsV16fV16fIiUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2843 /* cmpq128_mask */, 2856 /* UcV2OiV2OiIiUc */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2871 /* cmpq256_mask */, 2884 /* UcV4OiV4OiIiUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2899 /* cmpq512_mask */, 2912 /* UcV8OiV8OiIiUc */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2927 /* cmpsd */, 2496 /* V2dV2dV2dIc */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2933 /* cmpsd_mask */, 2944 /* UcV2dV2dIiUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2959 /* cmpsh_mask */, 2970 /* UcV8xV8xIiUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2985 /* cmpss */, 2725 /* V4fV4fV4fIc */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2991 /* cmpss_mask */, 3002 /* UcV4fV4fIiUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3017 /* cmpunordpd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3028 /* cmpunordps */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3039 /* cmpunordsd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3050 /* cmpunordss */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3061 /* cmpw128_mask */, 3074 /* UcV8sV8sIiUc */, 1016 /* ncEV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3087 /* cmpw256_mask */, 3100 /* UsV16sV16sIiUs */, 1053 /* ncEV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3115 /* cmpw512_mask */, 3128 /* UiV32sV32sIiUi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3143 /* comieq */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3158 /* comige */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3165 /* comigt */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3172 /* comile */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3179 /* comilt */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3186 /* comineq */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3194 /* comisdeq */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3211 /* comisdge */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3220 /* comisdgt */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3229 /* comisdle */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3238 /* comisdlt */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3247 /* comisdneq */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3257 /* compressdf128_mask */, 3276 /* V2dV2dV2dUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3288 /* compressdf256_mask */, 3307 /* V4dV4dV4dUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3319 /* compressdf512_mask */, 3338 /* V8dV8dV8dUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3350 /* compressdi128_mask */, 3369 /* V2OiV2OiV2OiUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3384 /* compressdi256_mask */, 3403 /* V4OiV4OiV4OiUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3418 /* compressdi512_mask */, 3437 /* V8OiV8OiV8OiUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3452 /* compresshi128_mask */, 3471 /* V8sV8sV8sUc */, 920 /* ncV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3504 /* compresshi256_mask */, 3523 /* V16sV16sV16sUs */, 1199 /* ncV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3538 /* compresshi512_mask */, 3557 /* V32sV32sV32sUi */, 810 /* ncV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3584 /* compressqi128_mask */, 3603 /* V16cV16cV16cUs */, 920 /* ncV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3618 /* compressqi256_mask */, 3637 /* V32cV32cV32cUi */, 1199 /* ncV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3652 /* compressqi512_mask */, 3671 /* V64cV64cV64cUOi */, 810 /* ncV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3687 /* compresssf128_mask */, 3706 /* V4fV4fV4fUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3718 /* compresssf256_mask */, 3737 /* V8fV8fV8fUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3749 /* compresssf512_mask */, 3768 /* V16fV16fV16fUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3783 /* compresssi128_mask */, 3802 /* V4iV4iV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3814 /* compresssi256_mask */, 3833 /* V8iV8iV8iUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3845 /* compresssi512_mask */, 3864 /* V16iV16iV16iUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3879 /* compressstoredf128_mask */, 3903 /* vV2d*V2dUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3914 /* compressstoredf256_mask */, 3938 /* vV4d*V4dUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3957 /* compressstoredf512_mask */, 3981 /* vV8d*V8dUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4000 /* compressstoredi128_mask */, 4024 /* vV2Oi*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4037 /* compressstoredi256_mask */, 4061 /* vV4Oi*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4074 /* compressstoredi512_mask */, 4098 /* vV8Oi*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4111 /* compressstorehi128_mask */, 4135 /* vV8s*V8sUc */, 1165 /* nV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4146 /* compressstorehi256_mask */, 4170 /* vV16s*V16sUs */, 3949 /* nV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4183 /* compressstorehi512_mask */, 4207 /* vV32s*V32sUi */, 3992 /* nV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4220 /* compressstoreqi128_mask */, 4244 /* vV16c*V16cUs */, 1165 /* nV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4257 /* compressstoreqi256_mask */, 4281 /* vV32c*V32cUi */, 3949 /* nV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4294 /* compressstoreqi512_mask */, 4318 /* vV64c*V64cUOi */, 3992 /* nV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4332 /* compressstoresf128_mask */, 4356 /* vV4f*V4fUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4367 /* compressstoresf256_mask */, 4391 /* vV8f*V8fUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4402 /* compressstoresf512_mask */, 4426 /* vV16f*V16fUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4439 /* compressstoresi128_mask */, 4463 /* vV4i*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4474 /* compressstoresi256_mask */, 4498 /* vV8i*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4509 /* compressstoresi512_mask */, 4533 /* vV16i*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4546 /* crc32hi */, 4554 /* UiUiUs */, 572 /* nc */, 4561 /* crc32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4567 /* crc32qi */, 4575 /* UiUiUc */, 572 /* nc */, 4561 /* crc32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4582 /* crc32si */, 1746 /* UiUiUi */, 572 /* nc */, 4561 /* crc32 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4590 /* cvtb2mask128 */, 4603 /* UsV16c */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4628 /* cvtb2mask256 */, 4641 /* UiV32c */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4648 /* cvtb2mask512 */, 4661 /* UOiV64c */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4669 /* cvtd2mask128 */, 4682 /* UcV4i */, 1016 /* ncEV:128: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4706 /* cvtd2mask256 */, 4719 /* UcV8i */, 1053 /* ncEV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4725 /* cvtd2mask512 */, 4738 /* UsV16i */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4754 /* cvtdq2ps512_mask */, 4771 /* V16fV16iV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4788 /* cvtmask2b128 */, 4801 /* V16cUs */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4808 /* cvtmask2b256 */, 4821 /* V32cUi */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4828 /* cvtmask2b512 */, 4841 /* V64cUOi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4849 /* cvtmask2d128 */, 4862 /* V4iUc */, 1016 /* ncEV:128: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4868 /* cvtmask2d256 */, 4881 /* V8iUc */, 1053 /* ncEV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4887 /* cvtmask2d512 */, 4900 /* V16iUs */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4907 /* cvtmask2q128 */, 4920 /* V2OiUc */, 1016 /* ncEV:128: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4927 /* cvtmask2q256 */, 4940 /* V4OiUc */, 1053 /* ncEV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4947 /* cvtmask2q512 */, 4960 /* V8OiUc */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4967 /* cvtmask2w128 */, 4980 /* V8sUc */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4986 /* cvtmask2w256 */, 4999 /* V16sUs */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5006 /* cvtmask2w512 */, 5019 /* V32sUi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5026 /* cvtne2ps2bf16_128 */, 5044 /* V8yV4fV4f */, 920 /* ncV:128: */, 5054 /* avx512bf16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5074 /* cvtne2ps2bf16_256 */, 5092 /* V16yV8fV8f */, 1199 /* ncV:256: */, 5054 /* avx512bf16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5103 /* cvtne2ps2bf16_512 */, 5121 /* V32yV16fV16f */, 810 /* ncV:512: */, 5134 /* avx512bf16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5145 /* cvtneps2bf16_128_mask */, 5167 /* V8yV4fV8yUc */, 920 /* ncV:128: */, 5054 /* avx512bf16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5179 /* cvtneps2bf16_256_mask */, 5201 /* V8yV8fV8yUc */, 1199 /* ncV:256: */, 5054 /* avx512bf16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5213 /* cvtneps2bf16_512_mask */, 5235 /* V16yV16fV16yUs */, 810 /* ncV:512: */, 5134 /* avx512bf16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5250 /* cvtpd2dq */, 5259 /* V2OiV2d */, 920 /* ncV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5267 /* cvtpd2dq128_mask */, 5284 /* V4iV2dV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5296 /* cvtpd2dq256 */, 5308 /* V4iV4d */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5315 /* cvtpd2dq512_mask */, 5332 /* V8iV8dV8iUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5346 /* cvtpd2ps */, 5355 /* V4fV2d */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5362 /* cvtpd2ps256 */, 5374 /* V4fV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5381 /* cvtpd2ps512_mask */, 5398 /* V8fV8dV8fUcIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5412 /* cvtpd2ps_mask */, 5426 /* V4fV2dV4fUc */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5438 /* cvtpd2qq128_mask */, 5455 /* V2OiV2dV2OiUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5487 /* cvtpd2qq256_mask */, 5504 /* V4OiV4dV4OiUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5518 /* cvtpd2qq512_mask */, 5535 /* V8OiV8dV8OiUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5551 /* cvtpd2udq128_mask */, 5284 /* V4iV2dV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5569 /* cvtpd2udq256_mask */, 5587 /* V4iV4dV4iUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5599 /* cvtpd2udq512_mask */, 5332 /* V8iV8dV8iUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5617 /* cvtpd2uqq128_mask */, 5455 /* V2OiV2dV2OiUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5635 /* cvtpd2uqq256_mask */, 5504 /* V4OiV4dV4OiUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5653 /* cvtpd2uqq512_mask */, 5535 /* V8OiV8dV8OiUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5671 /* cvtps2dq */, 5680 /* V4iV4f */, 920 /* ncV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5687 /* cvtps2dq256 */, 5699 /* V8iV8f */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5706 /* cvtps2dq512_mask */, 5723 /* V16iV16fV16iUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5740 /* cvtps2pd512_mask */, 5757 /* V8dV8fV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5771 /* cvtps2qq128_mask */, 5788 /* V2OiV4fV2OiUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5802 /* cvtps2qq256_mask */, 5819 /* V4OiV4fV4OiUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5833 /* cvtps2qq512_mask */, 5850 /* V8OiV8fV8OiUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5866 /* cvtps2udq128_mask */, 5884 /* V4iV4fV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5896 /* cvtps2udq256_mask */, 5914 /* V8iV8fV8iUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5926 /* cvtps2udq512_mask */, 5723 /* V16iV16fV16iUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5944 /* cvtps2uqq128_mask */, 5788 /* V2OiV4fV2OiUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5962 /* cvtps2uqq256_mask */, 5819 /* V4OiV4fV4OiUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5980 /* cvtps2uqq512_mask */, 5850 /* V8OiV8fV8OiUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5998 /* cvtq2mask128 */, 6011 /* UcV2Oi */, 1016 /* ncEV:128: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6018 /* cvtq2mask256 */, 6031 /* UcV4Oi */, 1053 /* ncEV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6038 /* cvtq2mask512 */, 6051 /* UcV8Oi */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6058 /* cvtqq2pd512_mask */, 6075 /* V8dV8OiV8dUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6090 /* cvtqq2ps128_mask */, 6107 /* V4fV2OiV4fUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6120 /* cvtqq2ps512_mask */, 6137 /* V8fV8OiV8fUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6152 /* cvtsd2si */, 6161 /* iV2d */, 920 /* ncV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6166 /* cvtsd2ss */, 6175 /* V4fV4fV2d */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6185 /* cvtsd2ss_round_mask */, 6205 /* V4fV4fV2dV4fUcIi */, 1016 /* ncEV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6222 /* cvtsi2ss32 */, 6233 /* V4fV4fiIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6243 /* cvtss2sd_round_mask */, 6263 /* V2dV2dV4fV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6280 /* cvtss2si */, 6289 /* iV4f */, 920 /* ncV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6294 /* cvttpd2dq */, 6304 /* V4iV2d */, 920 /* ncV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6311 /* cvttpd2dq128_mask */, 5284 /* V4iV2dV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6329 /* cvttpd2dq256 */, 5308 /* V4iV4d */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6342 /* cvttpd2dq512_mask */, 5332 /* V8iV8dV8iUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6360 /* cvttpd2qq128_mask */, 5455 /* V2OiV2dV2OiUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6378 /* cvttpd2qq256_mask */, 5504 /* V4OiV4dV4OiUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6396 /* cvttpd2qq512_mask */, 5535 /* V8OiV8dV8OiUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6414 /* cvttpd2udq128_mask */, 5284 /* V4iV2dV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6433 /* cvttpd2udq256_mask */, 5587 /* V4iV4dV4iUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6452 /* cvttpd2udq512_mask */, 5332 /* V8iV8dV8iUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6471 /* cvttpd2uqq128_mask */, 5455 /* V2OiV2dV2OiUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6490 /* cvttpd2uqq256_mask */, 5504 /* V4OiV4dV4OiUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6509 /* cvttpd2uqq512_mask */, 5535 /* V8OiV8dV8OiUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6528 /* cvttps2dq */, 5680 /* V4iV4f */, 920 /* ncV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6538 /* cvttps2dq256 */, 5699 /* V8iV8f */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6551 /* cvttps2dq512_mask */, 5723 /* V16iV16fV16iUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6569 /* cvttps2qq128_mask */, 5788 /* V2OiV4fV2OiUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6587 /* cvttps2qq256_mask */, 5819 /* V4OiV4fV4OiUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6605 /* cvttps2qq512_mask */, 5850 /* V8OiV8fV8OiUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6623 /* cvttps2udq128_mask */, 5884 /* V4iV4fV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6642 /* cvttps2udq256_mask */, 5914 /* V8iV8fV8iUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6661 /* cvttps2udq512_mask */, 5723 /* V16iV16fV16iUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6680 /* cvttps2uqq128_mask */, 5788 /* V2OiV4fV2OiUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6699 /* cvttps2uqq256_mask */, 5819 /* V4OiV4fV4OiUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6718 /* cvttps2uqq512_mask */, 5850 /* V8OiV8fV8OiUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6737 /* cvttsd2si */, 6161 /* iV2d */, 920 /* ncV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6747 /* cvttss2si */, 6289 /* iV4f */, 920 /* ncV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6757 /* cvtudq2ps512_mask */, 4771 /* V16fV16iV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6775 /* cvtuqq2pd512_mask */, 6075 /* V8dV8OiV8dUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6793 /* cvtuqq2ps128_mask */, 6107 /* V4fV2OiV4fUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6811 /* cvtuqq2ps512_mask */, 6137 /* V8fV8OiV8fUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6829 /* cvtusi2ss32 */, 6841 /* V4fV4fUiIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6852 /* cvtw2mask128 */, 6865 /* UcV8s */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6871 /* cvtw2mask256 */, 6884 /* UsV16s */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6891 /* cvtw2mask512 */, 6904 /* UiV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6911 /* dbpsadbw128 */, 6923 /* V8sV16cV16cIi */, 920 /* ncV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6937 /* dbpsadbw256 */, 6949 /* V16sV32cV32cIi */, 1199 /* ncV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6964 /* dbpsadbw512 */, 6976 /* V32sV64cV64cIi */, 810 /* ncV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6991 /* directstore_u32 */, 7007 /* vUi*Ui */, 744 /* n */, 7014 /* movdiri */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7022 /* divpd512 */, 798 /* V8dV8dV8dIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7031 /* divph512 */, 836 /* V32xV32xV32xIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7040 /* divps512 */, 871 /* V16fV16fV16fIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7049 /* divsd_round_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7066 /* divsh_round_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7083 /* divss_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7100 /* dpbf16ps_128 */, 7113 /* V4fV4fV8yV8y */, 920 /* ncV:128: */, 5054 /* avx512bf16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7126 /* dpbf16ps_256 */, 7139 /* V8fV8fV16yV16y */, 1199 /* ncV:256: */, 5054 /* avx512bf16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7154 /* dpbf16ps_512 */, 7167 /* V16fV16fV32yV32y */, 810 /* ncV:512: */, 5134 /* avx512bf16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7184 /* dppd */, 2496 /* V2dV2dV2dIc */, 920 /* ncV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7189 /* dpps */, 2725 /* V4fV4fV4fIc */, 920 /* ncV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7194 /* dpps256 */, 2773 /* V8fV8fV8fIc */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7202 /* emms */, 278 /* v */, 744 /* n */, 7207 /* mmx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7211 /* encodekey128_u32 */, 7228 /* UiUiV2Oiv* */, 1165 /* nV:128: */, 1173 /* kl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7239 /* encodekey256_u32 */, 7256 /* UiUiV2OiV2Oiv* */, 1165 /* nV:128: */, 1173 /* kl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7271 /* enqcmd */, 7278 /* Ucv*vC* */, 744 /* n */, 7271 /* enqcmd */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7286 /* enqcmds */, 7278 /* Ucv*vC* */, 744 /* n */, 7271 /* enqcmd */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7294 /* expanddf128_mask */, 3276 /* V2dV2dV2dUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7311 /* expanddf256_mask */, 3307 /* V4dV4dV4dUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7328 /* expanddf512_mask */, 3338 /* V8dV8dV8dUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7345 /* expanddi128_mask */, 3369 /* V2OiV2OiV2OiUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7362 /* expanddi256_mask */, 3403 /* V4OiV4OiV4OiUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7379 /* expanddi512_mask */, 3437 /* V8OiV8OiV8OiUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7396 /* expandhi128_mask */, 3471 /* V8sV8sV8sUc */, 920 /* ncV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7413 /* expandhi256_mask */, 3523 /* V16sV16sV16sUs */, 1199 /* ncV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7430 /* expandhi512_mask */, 3557 /* V32sV32sV32sUi */, 810 /* ncV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7447 /* expandloaddf128_mask */, 7468 /* V2dV2dC*V2dUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7482 /* expandloaddf256_mask */, 7503 /* V4dV4dC*V4dUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7517 /* expandloaddf512_mask */, 7538 /* V8dV8dC*V8dUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7552 /* expandloaddi128_mask */, 7573 /* V4iV2OiC*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7589 /* expandloaddi256_mask */, 7610 /* V4OiV4OiC*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7627 /* expandloaddi512_mask */, 7648 /* V8OiV8OiC*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7665 /* expandloadhi128_mask */, 7686 /* V8sV8sC*V8sUc */, 1165 /* nV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7700 /* expandloadhi256_mask */, 7721 /* V16sV16sC*V16sUs */, 3949 /* nV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7738 /* expandloadhi512_mask */, 7759 /* V32sV32sC*V32sUi */, 3992 /* nV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7776 /* expandloadqi128_mask */, 7797 /* V16cV16cC*V16cUs */, 1165 /* nV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7814 /* expandloadqi256_mask */, 7835 /* V32cV32cC*V32cUi */, 3949 /* nV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7852 /* expandloadqi512_mask */, 7873 /* V64cV64cC*V64cUOi */, 3992 /* nV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7891 /* expandloadsf128_mask */, 7912 /* V4fV4fC*V4fUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7926 /* expandloadsf256_mask */, 7947 /* V8fV8fC*V8fUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7961 /* expandloadsf512_mask */, 7982 /* V16fV16fC*V16fUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7999 /* expandloadsi128_mask */, 8020 /* V4iV4iC*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8034 /* expandloadsi256_mask */, 8055 /* V8iV8iC*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8069 /* expandloadsi512_mask */, 8090 /* V16iV16iC*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8107 /* expandqi128_mask */, 3603 /* V16cV16cV16cUs */, 920 /* ncV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8124 /* expandqi256_mask */, 3637 /* V32cV32cV32cUi */, 1199 /* ncV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8141 /* expandqi512_mask */, 3671 /* V64cV64cV64cUOi */, 810 /* ncV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8158 /* expandsf128_mask */, 3706 /* V4fV4fV4fUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8175 /* expandsf256_mask */, 3737 /* V8fV8fV8fUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8192 /* expandsf512_mask */, 3768 /* V16fV16fV16fUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8209 /* expandsi128_mask */, 3802 /* V4iV4iV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8226 /* expandsi256_mask */, 3833 /* V8iV8iV8iUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8243 /* expandsi512_mask */, 3864 /* V16iV16iV16iUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8260 /* extract128i256 */, 8275 /* V2OiV4OiIi */, 1016 /* ncEV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8291 /* extractf32x4_256_mask */, 8313 /* V4fV8fIiV4fUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8327 /* extractf32x4_mask */, 8345 /* V4fV16fIiV4fUc */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8360 /* extractf32x8_mask */, 8378 /* V8fV16fIiV8fUc */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8393 /* extractf64x2_256_mask */, 8415 /* V2dV4dIiV2dUc */, 1053 /* ncEV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8429 /* extractf64x2_512_mask */, 8451 /* V2dV8dIiV2dUc */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8465 /* extractf64x4_mask */, 8483 /* V4dV8dIiV4dUc */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8497 /* extracti32x4_256_mask */, 8519 /* V4iV8iIiV4iUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8533 /* extracti32x4_mask */, 8551 /* V4iV16iIiV4iUc */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8566 /* extracti32x8_mask */, 8584 /* V8iV16iIiV8iUc */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8599 /* extracti64x2_256_mask */, 8621 /* V2OiV4OiIiV2OiUc */, 1053 /* ncEV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8638 /* extracti64x2_512_mask */, 8660 /* V2OiV8OiIiV2OiUc */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8677 /* extracti64x4_mask */, 8695 /* V4OiV8OiIiV4OiUc */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8712 /* extrq */, 8718 /* V2OiV2OiV16c */, 920 /* ncV:128: */, 8731 /* sse4a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8737 /* extrqi */, 8744 /* V2OiV2OiIcIc */, 920 /* ncV:128: */, 8731 /* sse4a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8757 /* fixupimmpd128_mask */, 8776 /* V2dV2dV2dV2OiIiUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8794 /* fixupimmpd128_maskz */, 8776 /* V2dV2dV2dV2OiIiUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8814 /* fixupimmpd256_mask */, 8833 /* V4dV4dV4dV4OiIiUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8851 /* fixupimmpd256_maskz */, 8833 /* V4dV4dV4dV4OiIiUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8871 /* fixupimmpd512_mask */, 8890 /* V8dV8dV8dV8OiIiUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8910 /* fixupimmpd512_maskz */, 8890 /* V8dV8dV8dV8OiIiUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8930 /* fixupimmps128_mask */, 8949 /* V4fV4fV4fV4iIiUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8966 /* fixupimmps128_maskz */, 8949 /* V4fV4fV4fV4iIiUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8986 /* fixupimmps256_mask */, 9005 /* V8fV8fV8fV8iIiUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9022 /* fixupimmps256_maskz */, 9005 /* V8fV8fV8fV8iIiUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9042 /* fixupimmps512_mask */, 9061 /* V16fV16fV16fV16iIiUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9084 /* fixupimmps512_maskz */, 9061 /* V16fV16fV16fV16iIiUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9104 /* fixupimmsd_mask */, 9120 /* V2dV2dV2dV2OiIiUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9140 /* fixupimmsd_maskz */, 9120 /* V2dV2dV2dV2OiIiUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9157 /* fixupimmss_mask */, 9173 /* V4fV4fV4fV4iIiUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9192 /* fixupimmss_maskz */, 9173 /* V4fV4fV4fV4iIiUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9209 /* fpclasspd128_mask */, 9227 /* UcV2dIiUc */, 920 /* ncV:128: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9237 /* fpclasspd256_mask */, 9255 /* UcV4dIiUc */, 1199 /* ncV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9265 /* fpclasspd512_mask */, 9283 /* UcV8dIiUc */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9293 /* fpclassph128_mask */, 9311 /* UcV8xIiUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9321 /* fpclassph256_mask */, 9339 /* UsV16xIiUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9350 /* fpclassph512_mask */, 9368 /* UiV32xIiUi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9379 /* fpclassps128_mask */, 9397 /* UcV4fIiUc */, 920 /* ncV:128: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9407 /* fpclassps256_mask */, 9425 /* UcV8fIiUc */, 1199 /* ncV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9435 /* fpclassps512_mask */, 9453 /* UsV16fIiUs */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9464 /* fpclasssd_mask */, 9227 /* UcV2dIiUc */, 920 /* ncV:128: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9479 /* fpclasssh_mask */, 9311 /* UcV8xIiUc */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9494 /* fpclassss_mask */, 9397 /* UcV4fIiUc */, 920 /* ncV:128: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9509 /* fxrstor */, 568 /* vv* */, 744 /* n */, 9517 /* fxsr */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9522 /* fxsave */, 568 /* vv* */, 744 /* n */, 9517 /* fxsr */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9529 /* gather3div2df */, 9543 /* V2dV2dvC*V2OiUcIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9561 /* gather3div2di */, 9575 /* V2OiV2OivC*V2OiUcIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9595 /* gather3div4df */, 9609 /* V4dV4dvC*V4OiUcIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9627 /* gather3div4di */, 9641 /* V4OiV4OivC*V4OiUcIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9661 /* gather3div4sf */, 9675 /* V4fV4fvC*V2OiUcIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9693 /* gather3div4si */, 9707 /* V4iV4ivC*V2OiUcIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9725 /* gather3div8sf */, 9739 /* V4fV4fvC*V4OiUcIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9757 /* gather3div8si */, 9771 /* V4iV4ivC*V4OiUcIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9789 /* gather3siv2df */, 9803 /* V2dV2dvC*V4iUcIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9820 /* gather3siv2di */, 9834 /* V2OiV2OivC*V4iUcIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9853 /* gather3siv4df */, 9867 /* V4dV4dvC*V4iUcIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9884 /* gather3siv4di */, 9898 /* V4OiV4OivC*V4iUcIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9917 /* gather3siv4sf */, 9931 /* V4fV4fvC*V4iUcIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9948 /* gather3siv4si */, 9962 /* V4iV4ivC*V4iUcIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9979 /* gather3siv8sf */, 9993 /* V8fV8fvC*V8iUcIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10010 /* gather3siv8si */, 10024 /* V8iV8ivC*V8iUcIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10041 /* gatherd_d */, 10051 /* V4iV4iiC*V4iV4iIc */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10069 /* gatherd_d256 */, 10082 /* V8iV8iiC*V8iV8iIc */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10100 /* gatherd_pd */, 10111 /* V2dV2ddC*V4iV2dIc */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10129 /* gatherd_pd256 */, 10143 /* V4dV4ddC*V4iV4dIc */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10161 /* gatherd_ps */, 10172 /* V4fV4ffC*V4iV4fIc */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10190 /* gatherd_ps256 */, 10204 /* V8fV8ffC*V8iV8fIc */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10222 /* gatherd_q */, 10232 /* V2OiV2OiOiC*V4iV2OiIc */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10254 /* gatherd_q256 */, 10267 /* V4OiV4OiOiC*V4iV4OiIc */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10289 /* gatherdiv8df */, 10302 /* V8dV8dvC*V8OiUcIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10320 /* gatherdiv8di */, 10333 /* V8OiV8OivC*V8OiUcIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10353 /* gatherdiv16sf */, 10367 /* V8fV8fvC*V8OiUcIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10385 /* gatherdiv16si */, 10399 /* V8iV8ivC*V8OiUcIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10417 /* gatherq_d */, 10427 /* V4iV4iiC*V2OiV4iIc */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10446 /* gatherq_d256 */, 10459 /* V4iV4iiC*V4OiV4iIc */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10478 /* gatherq_pd */, 10489 /* V2dV2ddC*V2OiV2dIc */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10508 /* gatherq_pd256 */, 10522 /* V4dV4ddC*V4OiV4dIc */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10541 /* gatherq_ps */, 10552 /* V4fV4ffC*V2OiV4fIc */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10571 /* gatherq_ps256 */, 10585 /* V4fV4ffC*V4OiV4fIc */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10604 /* gatherq_q */, 10614 /* V2OiV2OiOiC*V2OiV2OiIc */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10637 /* gatherq_q256 */, 10650 /* V4OiV4OiOiC*V4OiV4OiIc */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10673 /* gathersiv8df */, 10686 /* V8dV8dvC*V8iUcIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10703 /* gathersiv8di */, 10716 /* V8OiV8OivC*V8iUcIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10735 /* gathersiv16sf */, 10749 /* V16fV16fvC*V16iUsIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10769 /* gathersiv16si */, 10783 /* V16iV16ivC*V16iUsIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10803 /* getexppd128_mask */, 3276 /* V2dV2dV2dUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10820 /* getexppd256_mask */, 3307 /* V4dV4dV4dUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10837 /* getexppd512_mask */, 10854 /* V8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10868 /* getexpph128_mask */, 10885 /* V8xV8xV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10897 /* getexpph256_mask */, 10914 /* V16xV16xV16xUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10929 /* getexpph512_mask */, 10946 /* V32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10963 /* getexpps128_mask */, 3706 /* V4fV4fV4fUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10980 /* getexpps256_mask */, 3737 /* V8fV8fV8fUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10997 /* getexpps512_mask */, 11014 /* V16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11031 /* getexpsd128_round_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11054 /* getexpsh128_round_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11077 /* getexpss128_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11100 /* getmantpd128_mask */, 11118 /* V2dV2dIiV2dUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11132 /* getmantpd256_mask */, 11150 /* V4dV4dIiV4dUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11164 /* getmantpd512_mask */, 11182 /* V8dV8dIiV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11198 /* getmantph128_mask */, 11216 /* V8xV8xIiV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11230 /* getmantph256_mask */, 11248 /* V16xV16xIiV16xUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11265 /* getmantph512_mask */, 11283 /* V32xV32xIiV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11302 /* getmantps128_mask */, 11320 /* V4fV4fIiV4fUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11334 /* getmantps256_mask */, 11352 /* V8fV8fIiV8fUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11366 /* getmantps512_mask */, 11384 /* V16fV16fIiV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11403 /* getmantsd_round_mask */, 11424 /* V2dV2dV2dIiV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11443 /* getmantsh_round_mask */, 11464 /* V8xV8xV8xIiV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11483 /* getmantss_round_mask */, 11504 /* V4fV4fV4fIiV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11523 /* haddpd */, 1006 /* V2dV2dV2d */, 1016 /* ncEV:128: */, 1026 /* sse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11530 /* haddpd256 */, 1043 /* V4dV4dV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11540 /* haddps */, 1076 /* V4fV4fV4f */, 1016 /* ncEV:128: */, 1026 /* sse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11547 /* haddps256 */, 1098 /* V8fV8fV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11557 /* hsubpd */, 1006 /* V2dV2dV2d */, 1016 /* ncEV:128: */, 1026 /* sse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11564 /* hsubpd256 */, 1043 /* V4dV4dV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11574 /* hsubps */, 1076 /* V4fV4fV4f */, 1016 /* ncEV:128: */, 1026 /* sse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11581 /* hsubps256 */, 1098 /* V8fV8fV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11591 /* incsspd */, 687 /* vUi */, 744 /* n */, 2026 /* shstk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11599 /* insert128i256 */, 11613 /* V4OiV4OiV2OiIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11628 /* insertf32x4 */, 11640 /* V16fV16fV4fIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11654 /* insertf32x4_256 */, 11670 /* V8fV8fV4fIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11682 /* insertf32x8 */, 11694 /* V16fV16fV8fIi */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11708 /* insertf64x2_256 */, 11724 /* V4dV4dV2dIi */, 1053 /* ncEV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11736 /* insertf64x2_512 */, 11752 /* V8dV8dV2dIi */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11764 /* insertf64x4 */, 11776 /* V8dV8dV4dIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11788 /* inserti32x4 */, 11800 /* V16iV16iV4iIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11814 /* inserti32x4_256 */, 11830 /* V8iV8iV4iIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11842 /* inserti32x8 */, 11854 /* V16iV16iV8iIi */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11868 /* inserti64x2_256 */, 11613 /* V4OiV4OiV2OiIi */, 1053 /* ncEV:256: */, 4688 /* avx512dq,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11884 /* inserti64x2_512 */, 11900 /* V8OiV8OiV2OiIi */, 1638 /* ncEV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11915 /* inserti64x4 */, 11927 /* V8OiV8OiV4OiIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11942 /* insertps128 */, 2725 /* V4fV4fV4fIc */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11954 /* insertq */, 1118 /* V2OiV2OiV2Oi */, 920 /* ncV:128: */, 8731 /* sse4a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11962 /* insertqi */, 11971 /* V2OiV2OiV2OiIcIc */, 920 /* ncV:128: */, 8731 /* sse4a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11988 /* invpcid */, 11996 /* vUiv* */, 572 /* nc */, 11988 /* invpcid */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12002 /* kadddi */, 12009 /* UOiUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12019 /* kaddhi */, 12026 /* UsUsUs */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12033 /* kaddqi */, 12040 /* UcUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12047 /* kaddsi */, 1746 /* UiUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12054 /* kanddi */, 12009 /* UOiUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12061 /* kandhi */, 12026 /* UsUsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12068 /* kandndi */, 12009 /* UOiUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12076 /* kandnhi */, 12026 /* UsUsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12084 /* kandnqi */, 12040 /* UcUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12092 /* kandnsi */, 1746 /* UiUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12100 /* kandqi */, 12040 /* UcUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12107 /* kandsi */, 1746 /* UiUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12114 /* kmovb */, 12120 /* UcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12125 /* kmovd */, 12131 /* UiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12136 /* kmovq */, 12142 /* UOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12149 /* kmovw */, 12155 /* UsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12160 /* knotdi */, 12142 /* UOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12167 /* knothi */, 12155 /* UsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12174 /* knotqi */, 12120 /* UcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12181 /* knotsi */, 12131 /* UiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12188 /* kordi */, 12009 /* UOiUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12194 /* korhi */, 12026 /* UsUsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12200 /* korqi */, 12040 /* UcUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12206 /* korsi */, 1746 /* UiUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12212 /* kortestcdi */, 12223 /* iUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12231 /* kortestchi */, 12242 /* iUsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12248 /* kortestcqi */, 12259 /* iUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12265 /* kortestcsi */, 12276 /* iUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12282 /* kortestzdi */, 12223 /* iUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12293 /* kortestzhi */, 12242 /* iUsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12304 /* kortestzqi */, 12259 /* iUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12315 /* kortestzsi */, 12276 /* iUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12326 /* kshiftlidi */, 12337 /* UOiUOiIUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12347 /* kshiftlihi */, 12358 /* UsUsIUi */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12366 /* kshiftliqi */, 12377 /* UcUcIUi */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12385 /* kshiftlisi */, 1772 /* UiUiIUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12396 /* kshiftridi */, 12337 /* UOiUOiIUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12407 /* kshiftrihi */, 12358 /* UsUsIUi */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12418 /* kshiftriqi */, 12377 /* UcUcIUi */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12429 /* kshiftrisi */, 1772 /* UiUiIUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12440 /* ktestcdi */, 12223 /* iUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12449 /* ktestchi */, 12242 /* iUsUs */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12458 /* ktestcqi */, 12259 /* iUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12467 /* ktestcsi */, 12276 /* iUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12476 /* ktestzdi */, 12223 /* iUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12485 /* ktestzhi */, 12242 /* iUsUs */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12494 /* ktestzqi */, 12259 /* iUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12503 /* ktestzsi */, 12276 /* iUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12512 /* kunpckdi */, 12009 /* UOiUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12521 /* kunpckhi */, 12026 /* UsUsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12530 /* kunpcksi */, 1746 /* UiUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12539 /* kxnordi */, 12009 /* UOiUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12547 /* kxnorhi */, 12026 /* UsUsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12555 /* kxnorqi */, 12040 /* UcUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12563 /* kxnorsi */, 1746 /* UiUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12571 /* kxordi */, 12009 /* UOiUOiUOi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12578 /* kxorhi */, 12026 /* UsUsUs */, 1753 /* ncE */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12585 /* kxorqi */, 12040 /* UcUcUc */, 1753 /* ncE */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12592 /* kxorsi */, 1746 /* UiUiUi */, 1753 /* ncE */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12599 /* lddqu */, 12605 /* V16ccC* */, 1165 /* nV:128: */, 1026 /* sse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12613 /* lddqu256 */, 12622 /* V32ccC* */, 3949 /* nV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12630 /* ldmxcsr */, 687 /* vUi */, 744 /* n */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12638 /* lfence */, 278 /* v */, 744 /* n */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12645 /* llwpcb */, 568 /* vv* */, 744 /* n */, 12652 /* lwp */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12656 /* loadapd128_mask */, 7468 /* V2dV2dC*V2dUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12672 /* loadapd256_mask */, 7503 /* V4dV4dC*V4dUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12688 /* loadapd512_mask */, 7538 /* V8dV8dC*V8dUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12704 /* loadaps128_mask */, 7912 /* V4fV4fC*V4fUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12720 /* loadaps256_mask */, 7947 /* V8fV8fC*V8fUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12736 /* loadaps512_mask */, 7982 /* V16fV16fC*V16fUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12752 /* loaddqudi128_mask */, 12770 /* V2OiV2OiC*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12787 /* loaddqudi256_mask */, 7610 /* V4OiV4OiC*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12805 /* loaddqudi512_mask */, 12823 /* V8OiOiC*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12838 /* loaddquhi128_mask */, 7686 /* V8sV8sC*V8sUc */, 1165 /* nV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12856 /* loaddquhi256_mask */, 7721 /* V16sV16sC*V16sUs */, 3949 /* nV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12874 /* loaddquhi512_mask */, 7759 /* V32sV32sC*V32sUi */, 3992 /* nV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12892 /* loaddquqi128_mask */, 7797 /* V16cV16cC*V16cUs */, 1165 /* nV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12910 /* loaddquqi256_mask */, 7835 /* V32cV32cC*V32cUi */, 3949 /* nV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12928 /* loaddquqi512_mask */, 7873 /* V64cV64cC*V64cUOi */, 3992 /* nV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12946 /* loaddqusi128_mask */, 8020 /* V4iV4iC*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12964 /* loaddqusi256_mask */, 8055 /* V8iV8iC*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12982 /* loaddqusi512_mask */, 13000 /* V16iiC*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13014 /* loadiwkey */, 13024 /* vV2OiV2OiV2OiUi */, 1165 /* nV:128: */, 1173 /* kl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13040 /* loadsbf16128_mask */, 13058 /* V8yV8yC*V8yUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13080 /* loadsd128_mask */, 7468 /* V2dV2dC*V2dUc */, 1165 /* nV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13095 /* loadsh128_mask */, 13110 /* V8xV8xC*V8xUc */, 1165 /* nV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13124 /* loadss128_mask */, 7912 /* V4fV4fC*V4fUc */, 1165 /* nV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13139 /* loadupd128_mask */, 7468 /* V2dV2dC*V2dUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13155 /* loadupd256_mask */, 7503 /* V4dV4dC*V4dUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13171 /* loadupd512_mask */, 13187 /* V8ddC*V8dUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13199 /* loadups128_mask */, 7912 /* V4fV4fC*V4fUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13215 /* loadups256_mask */, 7947 /* V8fV8fC*V8fUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13231 /* loadups512_mask */, 13247 /* V16ffC*V16fUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13261 /* lwpins32 */, 13270 /* UcUiUiIUi */, 744 /* n */, 12652 /* lwp */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13280 /* lwpval32 */, 13289 /* vUiUiIUi */, 744 /* n */, 12652 /* lwp */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13298 /* lzcnt_u16 */, 12155 /* UsUs */, 1753 /* ncE */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13308 /* lzcnt_u32 */, 12131 /* UiUi */, 1753 /* ncE */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13318 /* maskloadd */, 13328 /* V4iV4iC*V4i */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13340 /* maskloadd256 */, 13353 /* V8iV8iC*V8i */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13365 /* maskloadpd */, 13376 /* V2dV2dC*V2Oi */, 1165 /* nV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13389 /* maskloadpd256 */, 13403 /* V4dV4dC*V4Oi */, 3949 /* nV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13416 /* maskloadps */, 13427 /* V4fV4fC*V4i */, 1165 /* nV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13439 /* maskloadps256 */, 13453 /* V8fV8fC*V8i */, 3949 /* nV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13465 /* maskloadq */, 13475 /* V2OiV2OiC*V2Oi */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13490 /* maskloadq256 */, 13503 /* V4OiV4OiC*V4Oi */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13518 /* maskmovdqu */, 13529 /* vV16cV16cc* */, 1165 /* nV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13541 /* maskstored */, 13552 /* vV4i*V4iV4i */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13564 /* maskstored256 */, 13578 /* vV8i*V8iV8i */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13590 /* maskstorepd */, 13602 /* vV2d*V2OiV2d */, 1165 /* nV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13615 /* maskstorepd256 */, 13630 /* vV4d*V4OiV4d */, 3949 /* nV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13643 /* maskstoreps */, 13655 /* vV4f*V4iV4f */, 1165 /* nV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13667 /* maskstoreps256 */, 13682 /* vV8f*V8iV8f */, 3949 /* nV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13694 /* maskstoreq */, 13705 /* vV2Oi*V2OiV2Oi */, 1165 /* nV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13720 /* maskstoreq256 */, 13734 /* vV4Oi*V4OiV4Oi */, 3949 /* nV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13749 /* maxpd */, 1006 /* V2dV2dV2d */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13755 /* maxpd256 */, 1043 /* V4dV4dV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13764 /* maxpd512 */, 798 /* V8dV8dV8dIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13773 /* maxph128 */, 13782 /* V8xV8xV8x */, 1016 /* ncEV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13792 /* maxph256 */, 13801 /* V16xV16xV16x */, 1053 /* ncEV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13814 /* maxph512 */, 836 /* V32xV32xV32xIi */, 1638 /* ncEV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13823 /* maxps */, 1076 /* V4fV4fV4f */, 1016 /* ncEV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13829 /* maxps256 */, 1098 /* V8fV8fV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13838 /* maxps512 */, 871 /* V16fV16fV16fIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13847 /* maxsd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13853 /* maxsd_round_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13870 /* maxsh_round_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13887 /* maxss */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13893 /* maxss_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13910 /* mfence */, 278 /* v */, 744 /* n */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13917 /* minpd */, 1006 /* V2dV2dV2d */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13923 /* minpd256 */, 1043 /* V4dV4dV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13932 /* minpd512 */, 798 /* V8dV8dV8dIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13941 /* minph128 */, 13782 /* V8xV8xV8x */, 1016 /* ncEV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13950 /* minph256 */, 13801 /* V16xV16xV16x */, 1053 /* ncEV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13959 /* minph512 */, 836 /* V32xV32xV32xIi */, 1638 /* ncEV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13968 /* minps */, 1076 /* V4fV4fV4f */, 1016 /* ncEV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13974 /* minps256 */, 1098 /* V8fV8fV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13983 /* minps512 */, 871 /* V16fV16fV16fIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13992 /* minsd */, 1006 /* V2dV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13998 /* minsd_round_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14015 /* minsh_round_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14032 /* minss */, 1076 /* V4fV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14038 /* minss_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14055 /* monitor */, 14063 /* vvC*UiUi */, 744 /* n */, 1026 /* sse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14072 /* monitorx */, 14063 /* vvC*UiUi */, 744 /* n */, 14081 /* mwaitx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14088 /* movdir64b */, 14098 /* vv*vC* */, 744 /* n */, 14088 /* movdir64b */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14105 /* movdqa32load128_mask */, 8020 /* V4iV4iC*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14126 /* movdqa32load256_mask */, 8055 /* V8iV8iC*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14147 /* movdqa32load512_mask */, 8090 /* V16iV16iC*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14168 /* movdqa32store128_mask */, 4463 /* vV4i*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14190 /* movdqa32store256_mask */, 4498 /* vV8i*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14212 /* movdqa32store512_mask */, 4533 /* vV16i*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14234 /* movdqa64load128_mask */, 12770 /* V2OiV2OiC*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14255 /* movdqa64load256_mask */, 7610 /* V4OiV4OiC*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14276 /* movdqa64load512_mask */, 7648 /* V8OiV8OiC*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14297 /* movdqa64store128_mask */, 4024 /* vV2Oi*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14319 /* movdqa64store256_mask */, 4061 /* vV4Oi*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14341 /* movdqa64store512_mask */, 4098 /* vV8Oi*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14363 /* movmskpd */, 6161 /* iV2d */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14372 /* movmskpd256 */, 14384 /* iV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14389 /* movmskps */, 6289 /* iV4f */, 1016 /* ncEV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14398 /* movmskps256 */, 14410 /* iV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14415 /* movnti */, 320 /* vi*i */, 744 /* n */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14422 /* movntsd */, 14430 /* vd*V2d */, 1165 /* nV:128: */, 8731 /* sse4a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14437 /* movntss */, 14445 /* vf*V4f */, 1165 /* nV:128: */, 8731 /* sse4a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14452 /* mpsadbw128 */, 14463 /* V16cV16cV16cIc */, 920 /* ncV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14478 /* mpsadbw256 */, 14489 /* V32cV32cV32cIc */, 1199 /* ncV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14504 /* mpsadbw512 */, 14515 /* V32sV64cV64cIc */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14530 /* mulpd512 */, 798 /* V8dV8dV8dIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14539 /* mulph512 */, 836 /* V32xV32xV32xIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14548 /* mulps512 */, 871 /* V16fV16fV16fIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14557 /* mulsd_round_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14574 /* mulsh_round_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14591 /* mulss_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14608 /* mwait */, 14614 /* vUiUi */, 744 /* n */, 1026 /* sse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14081 /* mwaitx */, 14620 /* vUiUiUi */, 744 /* n */, 14081 /* mwaitx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14628 /* packssdw128 */, 14640 /* V8sV4iV4i */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14650 /* packssdw256 */, 14662 /* V16sV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14673 /* packssdw512 */, 14685 /* V32sV16iV16i */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14698 /* packsswb128 */, 14710 /* V16cV8sV8s */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14721 /* packsswb256 */, 14733 /* V32cV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14746 /* packsswb512 */, 14758 /* V64cV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14771 /* packusdw128 */, 14640 /* V8sV4iV4i */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14783 /* packusdw256 */, 14662 /* V16sV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14795 /* packusdw512 */, 14685 /* V32sV16iV16i */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14807 /* packuswb128 */, 14710 /* V16cV8sV8s */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14819 /* packuswb256 */, 14733 /* V32cV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14831 /* packuswb512 */, 14758 /* V64cV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14843 /* palignr128 */, 14854 /* V16cV16cV16cIi */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14875 /* palignr256 */, 14886 /* V32cV32cV32cIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14901 /* palignr512 */, 14912 /* V64cV64cV64cIi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14927 /* pause */, 278 /* v */, 744 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14933 /* pavgb128 */, 14942 /* V16UcV16UcV16Uc */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14958 /* pavgb256 */, 14967 /* V32UcV32UcV32Uc */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14983 /* pavgb512 */, 14992 /* V64UcV64UcV64Uc */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15008 /* pavgw128 */, 15017 /* V8UsV8UsV8Us */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15030 /* pavgw256 */, 15039 /* V16UsV16UsV16Us */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15055 /* pavgw512 */, 15064 /* V32UsV32UsV32Us */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15080 /* pblendd128 */, 1570 /* V4iV4iV4iIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15091 /* pblendd256 */, 1601 /* V8iV8iV8iIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15102 /* pblendvb128 */, 15114 /* V16cV16cV16cV16c */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15131 /* pblendvb256 */, 15143 /* V32cV32cV32cV32c */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15160 /* pblendw128 */, 15171 /* V8sV8sV8sIi */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15183 /* pblendw256 */, 15194 /* V16sV16sV16sIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15209 /* pclmulqdq128 */, 15222 /* V2OiV2OiV2OiIc */, 1016 /* ncEV:128: */, 15237 /* pclmul */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15244 /* pclmulqdq256 */, 15257 /* V4OiV4OiV4OiIc */, 1053 /* ncEV:256: */, 15272 /* vpclmulqdq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15283 /* pclmulqdq512 */, 15296 /* V8OiV8OiV8OiIc */, 1638 /* ncEV:512: */, 15311 /* avx512f,vpclmulqdq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15330 /* pcmpestri128 */, 15343 /* iV16ciV16ciIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15364 /* pcmpestria128 */, 15343 /* iV16ciV16ciIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15378 /* pcmpestric128 */, 15343 /* iV16ciV16ciIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15392 /* pcmpestrio128 */, 15343 /* iV16ciV16ciIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15406 /* pcmpestris128 */, 15343 /* iV16ciV16ciIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15420 /* pcmpestriz128 */, 15343 /* iV16ciV16ciIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15434 /* pcmpestrm128 */, 15447 /* V16cV16ciV16ciIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15464 /* pcmpistri128 */, 15477 /* iV16cV16cIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15489 /* pcmpistria128 */, 15477 /* iV16cV16cIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15503 /* pcmpistric128 */, 15477 /* iV16cV16cIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15517 /* pcmpistrio128 */, 15477 /* iV16cV16cIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15531 /* pcmpistris128 */, 15477 /* iV16cV16cIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15545 /* pcmpistriz128 */, 15477 /* iV16cV16cIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15559 /* pcmpistrm128 */, 14463 /* V16cV16cV16cIc */, 920 /* ncV:128: */, 15357 /* sse4.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15572 /* pdep_si */, 1746 /* UiUiUi */, 1753 /* ncE */, 1979 /* bmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15580 /* permdf256 */, 15590 /* V4dV4dIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15599 /* permdf512 */, 15609 /* V8dV8dIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15618 /* permdi256 */, 15628 /* V4OiV4OiIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15639 /* permdi512 */, 15649 /* V8OiV8OiIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15660 /* permti256 */, 1683 /* V4OiV4OiV4OiIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15670 /* permvardf256 */, 15683 /* V4dV4dV4Oi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15694 /* permvardf512 */, 15707 /* V8dV8dV8Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15718 /* permvardi256 */, 1186 /* V4OiV4OiV4Oi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15731 /* permvardi512 */, 1238 /* V8OiV8OiV8Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15744 /* permvarhi128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15767 /* permvarhi256 */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15793 /* permvarhi512 */, 15806 /* V32sV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15819 /* permvarqi128 */, 15832 /* V16cV16cV16c */, 1016 /* ncEV:128: */, 15845 /* avx512vbmi,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15865 /* permvarqi256 */, 15878 /* V32cV32cV32c */, 1053 /* ncEV:256: */, 15845 /* avx512vbmi,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15891 /* permvarqi512 */, 15904 /* V64cV64cV64c */, 1638 /* ncEV:512: */, 15917 /* avx512vbmi */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15928 /* permvarsf256 */, 15941 /* V8fV8fV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15951 /* permvarsf512 */, 15964 /* V16fV16fV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15977 /* permvarsi256 */, 15990 /* V8iV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16000 /* permvarsi512 */, 16013 /* V16iV16iV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16026 /* pext_si */, 1746 /* UiUiUi */, 1753 /* ncE */, 1979 /* bmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16034 /* phaddd128 */, 16044 /* V4iV4iV4i */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16054 /* phaddd256 */, 15990 /* V8iV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16064 /* phaddsw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16075 /* phaddsw256 */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16086 /* phaddw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16096 /* phaddw256 */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16106 /* phminposuw128 */, 16120 /* V8sV8s */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16127 /* phsubd128 */, 16044 /* V4iV4iV4i */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16137 /* phsubd256 */, 15990 /* V8iV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16147 /* phsubsw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16158 /* phsubsw256 */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16169 /* phsubw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16179 /* phsubw256 */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16189 /* pmaddubsw128 */, 16202 /* V8sV16cV16c */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16214 /* pmaddubsw256 */, 16227 /* V16sV32cV32c */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16240 /* pmaddubsw512 */, 16253 /* V32sV64cV64c */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16266 /* pmaddwd128 */, 16277 /* V4iV8sV8s */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16287 /* pmaddwd256 */, 16298 /* V8iV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16310 /* pmaddwd512 */, 16321 /* V16iV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16334 /* pmovdb128_mask */, 16349 /* V16cV4iV16cUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16363 /* pmovdb128mem_mask */, 16381 /* vV16c*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16393 /* pmovdb256_mask */, 16408 /* V16cV8iV16cUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16422 /* pmovdb256mem_mask */, 16440 /* vV16c*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16452 /* pmovdb512_mask */, 16467 /* V16cV16iV16cUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16482 /* pmovdb512mem_mask */, 16500 /* vV16c*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16513 /* pmovdw128_mask */, 16528 /* V8sV4iV8sUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16540 /* pmovdw128mem_mask */, 16558 /* vV8s*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16569 /* pmovdw256_mask */, 16584 /* V8sV8iV8sUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16596 /* pmovdw256mem_mask */, 16614 /* vV8s*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16625 /* pmovdw512_mask */, 16640 /* V16sV16iV16sUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16655 /* pmovdw512mem_mask */, 16673 /* vV16s*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16686 /* pmovmskb128 */, 16698 /* iV16c */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16704 /* pmovmskb256 */, 16716 /* iV32c */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16722 /* pmovqb128_mask */, 16737 /* V16cV2OiV16cUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16752 /* pmovqb128mem_mask */, 16770 /* vV16c*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16783 /* pmovqb256_mask */, 16798 /* V16cV4OiV16cUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16813 /* pmovqb256mem_mask */, 16831 /* vV16c*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16844 /* pmovqb512_mask */, 16859 /* V16cV8OiV16cUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16874 /* pmovqb512mem_mask */, 16892 /* vV16c*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16905 /* pmovqd128_mask */, 16920 /* V4iV2OiV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16933 /* pmovqd128mem_mask */, 16951 /* vV4i*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16963 /* pmovqd256mem_mask */, 16981 /* vV4i*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16993 /* pmovqd512_mask */, 17008 /* V8iV8OiV8iUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17021 /* pmovqd512mem_mask */, 17039 /* vV8i*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17051 /* pmovqw128_mask */, 17066 /* V8sV2OiV8sUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17079 /* pmovqw128mem_mask */, 17097 /* vV8s*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17109 /* pmovqw256_mask */, 17124 /* V8sV4OiV8sUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17137 /* pmovqw256mem_mask */, 17155 /* vV8s*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17167 /* pmovqw512_mask */, 17182 /* V8sV8OiV8sUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17195 /* pmovqw512mem_mask */, 17213 /* vV8s*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17225 /* pmovsdb128_mask */, 16349 /* V16cV4iV16cUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17241 /* pmovsdb128mem_mask */, 16381 /* vV16c*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17260 /* pmovsdb256_mask */, 16408 /* V16cV8iV16cUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17276 /* pmovsdb256mem_mask */, 16440 /* vV16c*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17295 /* pmovsdb512_mask */, 16467 /* V16cV16iV16cUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17311 /* pmovsdb512mem_mask */, 16500 /* vV16c*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17330 /* pmovsdw128_mask */, 16528 /* V8sV4iV8sUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17346 /* pmovsdw128mem_mask */, 16558 /* vV8s*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17365 /* pmovsdw256_mask */, 16584 /* V8sV8iV8sUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17381 /* pmovsdw256mem_mask */, 16614 /* vV8s*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17400 /* pmovsdw512_mask */, 16640 /* V16sV16iV16sUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17416 /* pmovsdw512mem_mask */, 16673 /* vV16s*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17435 /* pmovsqb128_mask */, 16737 /* V16cV2OiV16cUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17451 /* pmovsqb128mem_mask */, 16770 /* vV16c*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17470 /* pmovsqb256_mask */, 16798 /* V16cV4OiV16cUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17486 /* pmovsqb256mem_mask */, 16831 /* vV16c*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17505 /* pmovsqb512_mask */, 16859 /* V16cV8OiV16cUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17521 /* pmovsqb512mem_mask */, 16892 /* vV16c*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17540 /* pmovsqd128_mask */, 16920 /* V4iV2OiV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17556 /* pmovsqd128mem_mask */, 16951 /* vV4i*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17575 /* pmovsqd256_mask */, 17591 /* V4iV4OiV4iUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17604 /* pmovsqd256mem_mask */, 16981 /* vV4i*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17623 /* pmovsqd512_mask */, 17008 /* V8iV8OiV8iUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17639 /* pmovsqd512mem_mask */, 17039 /* vV8i*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17658 /* pmovsqw128_mask */, 17066 /* V8sV2OiV8sUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17674 /* pmovsqw128mem_mask */, 17097 /* vV8s*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17693 /* pmovsqw256_mask */, 17124 /* V8sV4OiV8sUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17709 /* pmovsqw256mem_mask */, 17155 /* vV8s*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17728 /* pmovsqw512_mask */, 17182 /* V8sV8OiV8sUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17744 /* pmovsqw512mem_mask */, 17213 /* vV8s*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17763 /* pmovswb128_mask */, 17779 /* V16cV8sV16cUc */, 920 /* ncV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17793 /* pmovswb128mem_mask */, 17812 /* vV16c*V8sUc */, 1165 /* nV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17824 /* pmovswb256_mask */, 17840 /* V16cV16sV16cUs */, 1199 /* ncV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17855 /* pmovswb256mem_mask */, 17874 /* vV16c*V16sUs */, 3949 /* nV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17887 /* pmovswb512_mask */, 17903 /* V32cV32sV32cUi */, 810 /* ncV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17918 /* pmovswb512mem_mask */, 17937 /* vV32c*V32sUi */, 3992 /* nV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17950 /* pmovusdb128_mask */, 16349 /* V16cV4iV16cUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17967 /* pmovusdb128mem_mask */, 16381 /* vV16c*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17987 /* pmovusdb256_mask */, 16408 /* V16cV8iV16cUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18004 /* pmovusdb256mem_mask */, 16440 /* vV16c*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18024 /* pmovusdb512_mask */, 16467 /* V16cV16iV16cUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18041 /* pmovusdb512mem_mask */, 16500 /* vV16c*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18061 /* pmovusdw128_mask */, 16528 /* V8sV4iV8sUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18078 /* pmovusdw128mem_mask */, 16558 /* vV8s*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18098 /* pmovusdw256_mask */, 16584 /* V8sV8iV8sUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18115 /* pmovusdw256mem_mask */, 16614 /* vV8s*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18135 /* pmovusdw512_mask */, 16640 /* V16sV16iV16sUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18152 /* pmovusdw512mem_mask */, 16673 /* vV16s*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18172 /* pmovusqb128_mask */, 16737 /* V16cV2OiV16cUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18189 /* pmovusqb128mem_mask */, 16770 /* vV16c*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18209 /* pmovusqb256_mask */, 16798 /* V16cV4OiV16cUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18226 /* pmovusqb256mem_mask */, 16831 /* vV16c*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18246 /* pmovusqb512_mask */, 16859 /* V16cV8OiV16cUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18263 /* pmovusqb512mem_mask */, 16892 /* vV16c*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18283 /* pmovusqd128_mask */, 16920 /* V4iV2OiV4iUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18300 /* pmovusqd128mem_mask */, 16951 /* vV4i*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18320 /* pmovusqd256_mask */, 17591 /* V4iV4OiV4iUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18337 /* pmovusqd256mem_mask */, 16981 /* vV4i*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18357 /* pmovusqd512_mask */, 17008 /* V8iV8OiV8iUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18374 /* pmovusqd512mem_mask */, 17039 /* vV8i*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18394 /* pmovusqw128_mask */, 17066 /* V8sV2OiV8sUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18411 /* pmovusqw128mem_mask */, 17097 /* vV8s*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18431 /* pmovusqw256_mask */, 17124 /* V8sV4OiV8sUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18448 /* pmovusqw256mem_mask */, 17155 /* vV8s*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18468 /* pmovusqw512_mask */, 17182 /* V8sV8OiV8sUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18485 /* pmovusqw512mem_mask */, 17213 /* vV8s*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18505 /* pmovuswb128_mask */, 17779 /* V16cV8sV16cUc */, 920 /* ncV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18522 /* pmovuswb128mem_mask */, 17812 /* vV16c*V8sUc */, 1165 /* nV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18542 /* pmovuswb256_mask */, 17840 /* V16cV16sV16cUs */, 1199 /* ncV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18559 /* pmovuswb256mem_mask */, 17874 /* vV16c*V16sUs */, 3949 /* nV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18579 /* pmovuswb512_mask */, 17903 /* V32cV32sV32cUi */, 810 /* ncV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18596 /* pmovuswb512mem_mask */, 17937 /* vV32c*V32sUi */, 3992 /* nV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18616 /* pmovwb128_mask */, 17779 /* V16cV8sV16cUc */, 920 /* ncV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18631 /* pmovwb128mem_mask */, 17812 /* vV16c*V8sUc */, 1165 /* nV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18649 /* pmovwb256mem_mask */, 17874 /* vV16c*V16sUs */, 3949 /* nV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18667 /* pmovwb512_mask */, 17903 /* V32cV32sV32cUi */, 810 /* ncV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18682 /* pmovwb512mem_mask */, 17937 /* vV32c*V32sUi */, 3992 /* nV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18700 /* pmuldq128 */, 18710 /* V2OiV4iV4i */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18721 /* pmuldq256 */, 18731 /* V4OiV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18742 /* pmuldq512 */, 18752 /* V8OiV16iV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18765 /* pmulhrsw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18777 /* pmulhrsw256 */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18789 /* pmulhrsw512 */, 15806 /* V32sV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18801 /* pmulhuw128 */, 15017 /* V8UsV8UsV8Us */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18812 /* pmulhuw256 */, 15039 /* V16UsV16UsV16Us */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18823 /* pmulhuw512 */, 15064 /* V32UsV32UsV32Us */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18834 /* pmulhw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18844 /* pmulhw256 */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18854 /* pmulhw512 */, 15806 /* V32sV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18864 /* pmuludq128 */, 18710 /* V2OiV4iV4i */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18875 /* pmuludq256 */, 18731 /* V4OiV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18886 /* pmuludq512 */, 18752 /* V8OiV16iV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18897 /* prefetchrs */, 606 /* vvC* */, 572 /* nc */, 18908 /* movrs */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18914 /* prold128 */, 18923 /* V4iV4iIi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18932 /* prold256 */, 18941 /* V8iV8iIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18950 /* prold512 */, 18959 /* V16iV16iIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18970 /* prolq128 */, 18979 /* V2OiV2OiIi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18990 /* prolq256 */, 15628 /* V4OiV4OiIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18999 /* prolq512 */, 15649 /* V8OiV8OiIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19008 /* prord128 */, 18923 /* V4iV4iIi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19017 /* prord256 */, 18941 /* V8iV8iIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19026 /* prord512 */, 18959 /* V16iV16iIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19035 /* prorq128 */, 18979 /* V2OiV2OiIi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19044 /* prorq256 */, 15628 /* V4OiV4OiIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19053 /* prorq512 */, 15649 /* V8OiV8OiIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19062 /* psadbw128 */, 19072 /* V2OiV16cV16c */, 920 /* ncV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19085 /* psadbw256 */, 19095 /* V4OiV32cV32c */, 1199 /* ncV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19108 /* psadbw512 */, 19118 /* V8OiV64cV64c */, 810 /* ncV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19131 /* pshufb128 */, 15832 /* V16cV16cV16c */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19141 /* pshufb256 */, 15878 /* V32cV32cV32c */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19151 /* pshufb512 */, 15904 /* V64cV64cV64c */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19161 /* pshufd */, 18923 /* V4iV4iIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19168 /* pshufd256 */, 18941 /* V8iV8iIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19178 /* pshufd512 */, 18959 /* V16iV16iIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19188 /* pshufhw */, 19196 /* V8sV8sIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19205 /* pshufhw256 */, 19216 /* V16sV16sIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19227 /* pshufhw512 */, 19238 /* V32sV32sIi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19249 /* pshuflw */, 19196 /* V8sV8sIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19257 /* pshuflw256 */, 19216 /* V16sV16sIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19268 /* pshuflw512 */, 19238 /* V32sV32sIi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19279 /* psignb128 */, 15832 /* V16cV16cV16c */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19289 /* psignb256 */, 15878 /* V32cV32cV32c */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19299 /* psignd128 */, 16044 /* V4iV4iV4i */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19309 /* psignd256 */, 15990 /* V8iV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19319 /* psignw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 14869 /* ssse3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19329 /* psignw256 */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19339 /* pslld128 */, 16044 /* V4iV4iV4i */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19348 /* pslld256 */, 19357 /* V8iV8iV4i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19367 /* pslld512 */, 19376 /* V16iV16iV4i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19388 /* pslldi128 */, 19398 /* V4iV4ii */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19406 /* pslldi256 */, 19416 /* V8iV8ii */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19424 /* pslldi512 */, 19434 /* V16iV16ii */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19444 /* pslldqi128_byteshift */, 19465 /* V16cV16cIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19476 /* pslldqi256_byteshift */, 19497 /* V32cV32cIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19508 /* pslldqi512_byteshift */, 19529 /* V64cV64cIi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19540 /* psllq128 */, 1118 /* V2OiV2OiV2Oi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19549 /* psllq256 */, 19558 /* V4OiV4OiV2Oi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19571 /* psllq512 */, 19580 /* V8OiV8OiV2Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19593 /* psllqi128 */, 19603 /* V2OiV2Oii */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19613 /* psllqi256 */, 19623 /* V4OiV4Oii */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19633 /* psllqi512 */, 19643 /* V8OiV8Oii */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19653 /* psllv2di */, 1118 /* V2OiV2OiV2Oi */, 1016 /* ncEV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19662 /* psllv4di */, 1186 /* V4OiV4OiV4Oi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19671 /* psllv4si */, 16044 /* V4iV4iV4i */, 1016 /* ncEV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19680 /* psllv8di */, 1238 /* V8OiV8OiV8Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19689 /* psllv8hi */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19698 /* psllv8si */, 15990 /* V8iV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19707 /* psllv16hi */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19717 /* psllv16si */, 16013 /* V16iV16iV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19727 /* psllv32hi */, 15806 /* V32sV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19737 /* psllw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19746 /* psllw256 */, 19755 /* V16sV16sV8s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19767 /* psllw512 */, 19776 /* V32sV32sV8s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19788 /* psllwi128 */, 19798 /* V8sV8si */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19806 /* psllwi256 */, 19816 /* V16sV16si */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19826 /* psllwi512 */, 19836 /* V32sV32si */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19846 /* psrad128 */, 16044 /* V4iV4iV4i */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19855 /* psrad256 */, 19357 /* V8iV8iV4i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19864 /* psrad512 */, 19376 /* V16iV16iV4i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19873 /* psradi128 */, 19398 /* V4iV4ii */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19883 /* psradi256 */, 19416 /* V8iV8ii */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19893 /* psradi512 */, 19434 /* V16iV16ii */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19903 /* psraq128 */, 1118 /* V2OiV2OiV2Oi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19912 /* psraq256 */, 19558 /* V4OiV4OiV2Oi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19921 /* psraq512 */, 19580 /* V8OiV8OiV2Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19930 /* psraqi128 */, 19603 /* V2OiV2Oii */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19940 /* psraqi256 */, 19623 /* V4OiV4Oii */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19950 /* psraqi512 */, 19643 /* V8OiV8Oii */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19960 /* psrav4si */, 16044 /* V4iV4iV4i */, 1016 /* ncEV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19969 /* psrav8di */, 1238 /* V8OiV8OiV8Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19978 /* psrav8hi */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19987 /* psrav8si */, 15990 /* V8iV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19996 /* psrav16hi */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20006 /* psrav16si */, 16013 /* V16iV16iV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20016 /* psrav32hi */, 15806 /* V32sV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20026 /* psravq128 */, 1118 /* V2OiV2OiV2Oi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20036 /* psravq256 */, 1186 /* V4OiV4OiV4Oi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20046 /* psraw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20055 /* psraw256 */, 19755 /* V16sV16sV8s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20064 /* psraw512 */, 19776 /* V32sV32sV8s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20073 /* psrawi128 */, 19798 /* V8sV8si */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20083 /* psrawi256 */, 19816 /* V16sV16si */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20093 /* psrawi512 */, 19836 /* V32sV32si */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20103 /* psrld128 */, 16044 /* V4iV4iV4i */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20112 /* psrld256 */, 19357 /* V8iV8iV4i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20121 /* psrld512 */, 19376 /* V16iV16iV4i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20130 /* psrldi128 */, 19398 /* V4iV4ii */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20140 /* psrldi256 */, 19416 /* V8iV8ii */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20150 /* psrldi512 */, 19434 /* V16iV16ii */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20160 /* psrldqi128_byteshift */, 19465 /* V16cV16cIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20181 /* psrldqi256_byteshift */, 19497 /* V32cV32cIi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20202 /* psrldqi512_byteshift */, 19529 /* V64cV64cIi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20223 /* psrlq128 */, 1118 /* V2OiV2OiV2Oi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20232 /* psrlq256 */, 19558 /* V4OiV4OiV2Oi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20241 /* psrlq512 */, 19580 /* V8OiV8OiV2Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20250 /* psrlqi128 */, 19603 /* V2OiV2Oii */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20260 /* psrlqi256 */, 19623 /* V4OiV4Oii */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20270 /* psrlqi512 */, 19643 /* V8OiV8Oii */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20280 /* psrlv2di */, 1118 /* V2OiV2OiV2Oi */, 1016 /* ncEV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20289 /* psrlv4di */, 1186 /* V4OiV4OiV4Oi */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20298 /* psrlv4si */, 16044 /* V4iV4iV4i */, 1016 /* ncEV:128: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20307 /* psrlv8di */, 1238 /* V8OiV8OiV8Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20316 /* psrlv8hi */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20325 /* psrlv8si */, 15990 /* V8iV8iV8i */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20334 /* psrlv16hi */, 15780 /* V16sV16sV16s */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20344 /* psrlv16si */, 16013 /* V16iV16iV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20354 /* psrlv32hi */, 15806 /* V32sV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20364 /* psrlw128 */, 15757 /* V8sV8sV8s */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20373 /* psrlw256 */, 19755 /* V16sV16sV8s */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20382 /* psrlw512 */, 19776 /* V32sV32sV8s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20391 /* psrlwi128 */, 19798 /* V8sV8si */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20401 /* psrlwi256 */, 19816 /* V16sV16si */, 1053 /* ncEV:256: */, 8286 /* avx2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20411 /* psrlwi512 */, 19836 /* V32sV32si */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20421 /* pternlogd128_mask */, 20439 /* V4iV4iV4iV4iIiUc */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20456 /* pternlogd128_maskz */, 20439 /* V4iV4iV4iV4iIiUc */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20475 /* pternlogd256_mask */, 20493 /* V8iV8iV8iV8iIiUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20510 /* pternlogd256_maskz */, 20493 /* V8iV8iV8iV8iIiUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20529 /* pternlogd512_mask */, 20547 /* V16iV16iV16iV16iIiUs */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20568 /* pternlogd512_maskz */, 20547 /* V16iV16iV16iV16iIiUs */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20587 /* pternlogq128_mask */, 20605 /* V2OiV2OiV2OiV2OiIiUc */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20626 /* pternlogq128_maskz */, 20605 /* V2OiV2OiV2OiV2OiIiUc */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20645 /* pternlogq256_mask */, 20663 /* V4OiV4OiV4OiV4OiIiUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20684 /* pternlogq256_maskz */, 20663 /* V4OiV4OiV4OiV4OiIiUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20703 /* pternlogq512_mask */, 20721 /* V8OiV8OiV8OiV8OiIiUc */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20742 /* pternlogq512_maskz */, 20721 /* V8OiV8OiV8OiV8OiIiUc */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20761 /* ptestc128 */, 20771 /* iV2OiV2Oi */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20781 /* ptestc256 */, 20791 /* iV4OiV4Oi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20801 /* ptestnzc128 */, 20771 /* iV2OiV2Oi */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20813 /* ptestnzc256 */, 20791 /* iV4OiV4Oi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20825 /* ptestz128 */, 20771 /* iV2OiV2Oi */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20835 /* ptestz256 */, 20791 /* iV4OiV4Oi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20845 /* ptwrite32 */, 687 /* vUi */, 744 /* n */, 20855 /* ptwrite */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20863 /* rangepd128_mask */, 20879 /* V2dV2dV2dIiV2dUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20896 /* rangepd256_mask */, 20912 /* V4dV4dV4dIiV4dUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20929 /* rangepd512_mask */, 20945 /* V8dV8dV8dIiV8dUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20964 /* rangeps128_mask */, 20980 /* V4fV4fV4fIiV4fUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20997 /* rangeps256_mask */, 21013 /* V8fV8fV8fIiV8fUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21030 /* rangeps512_mask */, 21046 /* V16fV16fV16fIiV16fUsIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21069 /* rangesd128_round_mask */, 21091 /* V2dV2dV2dV2dUcIiIi */, 920 /* ncV:128: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21110 /* rangess128_round_mask */, 21132 /* V4fV4fV4fV4fUcIiIi */, 920 /* ncV:128: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21151 /* rcp14pd128_mask */, 3276 /* V2dV2dV2dUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21167 /* rcp14pd256_mask */, 3307 /* V4dV4dV4dUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21183 /* rcp14pd512_mask */, 3338 /* V8dV8dV8dUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21199 /* rcp14ps128_mask */, 3706 /* V4fV4fV4fUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21215 /* rcp14ps256_mask */, 3737 /* V8fV8fV8fUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21231 /* rcp14ps512_mask */, 3768 /* V16fV16fV16fUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21247 /* rcp14sd_mask */, 21260 /* V2dV2dV2dV2dUc */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21275 /* rcp14ss_mask */, 21288 /* V4fV4fV4fV4fUc */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21303 /* rcpph128_mask */, 10885 /* V8xV8xV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21317 /* rcpph256_mask */, 10914 /* V16xV16xV16xUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21331 /* rcpph512_mask */, 21345 /* V32xV32xV32xUi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21360 /* rcpps */, 21366 /* V4fV4f */, 920 /* ncV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21373 /* rcpps256 */, 21382 /* V8fV8f */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21389 /* rcpsh_mask */, 21400 /* V8xV8xV8xV8xUc */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21415 /* rcpss */, 21366 /* V4fV4f */, 920 /* ncV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21421 /* rdpid */, 622 /* Ui */, 744 /* n */, 21421 /* rdpid */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21427 /* rdpkru */, 622 /* Ui */, 744 /* n */, 21434 /* pku */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21438 /* rdpmc */, 21444 /* UOii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21449 /* rdpru */, 21455 /* ULLii */, 744 /* n */, 21449 /* rdpru */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21461 /* rdrand16_step */, 21475 /* UiUs* */, 744 /* n */, 21481 /* rdrnd */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21487 /* rdrand32_step */, 21501 /* UiUi* */, 744 /* n */, 21481 /* rdrnd */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21507 /* rdseed16_step */, 21475 /* UiUs* */, 744 /* n */, 21521 /* rdseed */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21528 /* rdseed32_step */, 21501 /* UiUi* */, 744 /* n */, 21521 /* rdseed */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21542 /* rdsspd */, 12131 /* UiUi */, 744 /* n */, 2026 /* shstk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21549 /* rdtsc */, 395 /* UOi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21555 /* rdtscp */, 21562 /* UOiUi* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21569 /* readeflags_u32 */, 622 /* Ui */, 744 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21584 /* reduce_fadd_pd512 */, 21602 /* ddV8d */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21608 /* reduce_fadd_ph128 */, 21626 /* xxV8x */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21632 /* reduce_fadd_ph256 */, 21650 /* xxV16x */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21657 /* reduce_fadd_ph512 */, 21675 /* xxV32x */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21682 /* reduce_fadd_ps512 */, 21700 /* ffV16f */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21707 /* reduce_fmax_pd512 */, 21725 /* dV8d */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21730 /* reduce_fmax_ph128 */, 21748 /* xV8x */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21753 /* reduce_fmax_ph256 */, 21771 /* xV16x */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21777 /* reduce_fmax_ph512 */, 21795 /* xV32x */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21801 /* reduce_fmax_ps512 */, 21819 /* fV16f */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21825 /* reduce_fmin_pd512 */, 21725 /* dV8d */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21843 /* reduce_fmin_ph128 */, 21748 /* xV8x */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21861 /* reduce_fmin_ph256 */, 21771 /* xV16x */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21879 /* reduce_fmin_ph512 */, 21795 /* xV32x */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21897 /* reduce_fmin_ps512 */, 21819 /* fV16f */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21915 /* reduce_fmul_pd512 */, 21602 /* ddV8d */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21933 /* reduce_fmul_ph128 */, 21626 /* xxV8x */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21951 /* reduce_fmul_ph256 */, 21650 /* xxV16x */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21969 /* reduce_fmul_ph512 */, 21675 /* xxV32x */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21987 /* reduce_fmul_ps512 */, 21700 /* ffV16f */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22005 /* reducepd128_mask */, 11118 /* V2dV2dIiV2dUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22022 /* reducepd256_mask */, 11150 /* V4dV4dIiV4dUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22039 /* reducepd512_mask */, 11182 /* V8dV8dIiV8dUcIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22056 /* reduceph128_mask */, 11216 /* V8xV8xIiV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22073 /* reduceph256_mask */, 11248 /* V16xV16xIiV16xUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22090 /* reduceph512_mask */, 11283 /* V32xV32xIiV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22107 /* reduceps128_mask */, 11320 /* V4fV4fIiV4fUc */, 920 /* ncV:128: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22124 /* reduceps256_mask */, 11352 /* V8fV8fIiV8fUc */, 1199 /* ncV:256: */, 5469 /* avx512vl,avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22141 /* reduceps512_mask */, 11384 /* V16fV16fIiV16fUsIi */, 810 /* ncV:512: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22158 /* reducesd_mask */, 21091 /* V2dV2dV2dV2dUcIiIi */, 920 /* ncV:128: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22172 /* reducesh_mask */, 22186 /* V8xV8xV8xV8xUcIiIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22205 /* reducess_mask */, 21132 /* V4fV4fV4fV4fUcIiIi */, 920 /* ncV:128: */, 4745 /* avx512dq */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22219 /* rndscalepd_128_mask */, 11118 /* V2dV2dIiV2dUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22239 /* rndscalepd_256_mask */, 11150 /* V4dV4dIiV4dUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22259 /* rndscalepd_mask */, 11182 /* V8dV8dIiV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22275 /* rndscaleph_128_mask */, 11216 /* V8xV8xIiV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22295 /* rndscaleph_256_mask */, 11248 /* V16xV16xIiV16xUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22315 /* rndscaleph_mask */, 11283 /* V32xV32xIiV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22331 /* rndscaleps_128_mask */, 11320 /* V4fV4fIiV4fUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22351 /* rndscaleps_256_mask */, 11352 /* V8fV8fIiV8fUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22371 /* rndscaleps_mask */, 11384 /* V16fV16fIiV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22387 /* rndscalesd_round_mask */, 21091 /* V2dV2dV2dV2dUcIiIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22409 /* rndscalesh_round_mask */, 22186 /* V8xV8xV8xV8xUcIiIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22431 /* rndscaless_round_mask */, 21132 /* V4fV4fV4fV4fUcIiIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22453 /* roundpd */, 22461 /* V2dV2dIi */, 920 /* ncV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22470 /* roundpd256 */, 15590 /* V4dV4dIi */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22481 /* roundps */, 22489 /* V4fV4fIi */, 920 /* ncV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22498 /* roundps256 */, 22509 /* V8fV8fIi */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22518 /* roundsd */, 1792 /* V2dV2dV2dIi */, 920 /* ncV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22526 /* roundss */, 1842 /* V4fV4fV4fIi */, 920 /* ncV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22534 /* rsqrt14pd128_mask */, 3276 /* V2dV2dV2dUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22552 /* rsqrt14pd256_mask */, 3307 /* V4dV4dV4dUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22570 /* rsqrt14pd512_mask */, 3338 /* V8dV8dV8dUc */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22588 /* rsqrt14ps128_mask */, 3706 /* V4fV4fV4fUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22606 /* rsqrt14ps256_mask */, 3737 /* V8fV8fV8fUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22624 /* rsqrt14ps512_mask */, 3768 /* V16fV16fV16fUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22642 /* rsqrt14sd_mask */, 21260 /* V2dV2dV2dV2dUc */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22657 /* rsqrt14ss_mask */, 21288 /* V4fV4fV4fV4fUc */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22672 /* rsqrtph128_mask */, 10885 /* V8xV8xV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22688 /* rsqrtph256_mask */, 10914 /* V16xV16xV16xUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22704 /* rsqrtph512_mask */, 21345 /* V32xV32xV32xUi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22720 /* rsqrtps */, 21366 /* V4fV4f */, 920 /* ncV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22728 /* rsqrtps256 */, 21382 /* V8fV8f */, 1199 /* ncV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22739 /* rsqrtsh_mask */, 21400 /* V8xV8xV8xV8xUc */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22752 /* rsqrtss */, 21366 /* V4fV4f */, 920 /* ncV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22760 /* rstorssp */, 568 /* vv* */, 744 /* n */, 2026 /* shstk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22769 /* saveprevssp */, 278 /* v */, 744 /* n */, 2026 /* shstk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22781 /* scalefpd128_mask */, 21260 /* V2dV2dV2dV2dUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22798 /* scalefpd256_mask */, 22815 /* V4dV4dV4dV4dUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22830 /* scalefpd512_mask */, 22847 /* V8dV8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22864 /* scalefph128_mask */, 21400 /* V8xV8xV8xV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22881 /* scalefph256_mask */, 22898 /* V16xV16xV16xV16xUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22917 /* scalefph512_mask */, 22934 /* V32xV32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22955 /* scalefps128_mask */, 21288 /* V4fV4fV4fV4fUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22972 /* scalefps256_mask */, 22989 /* V8fV8fV8fV8fUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23004 /* scalefps512_mask */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23042 /* scalefsd_round_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23062 /* scalefsh_round_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23082 /* scalefss_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23102 /* scatterdiv2df */, 23116 /* vv*UcV2OiV2dIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23131 /* scatterdiv2di */, 23145 /* vv*UcV2OiV2OiIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23161 /* scatterdiv4df */, 23175 /* vv*UcV4OiV4dIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23190 /* scatterdiv4di */, 23204 /* vv*UcV4OiV4OiIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23220 /* scatterdiv4sf */, 23234 /* vv*UcV2OiV4fIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23249 /* scatterdiv4si */, 23263 /* vv*UcV2OiV4iIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23278 /* scatterdiv8df */, 23292 /* vv*UcV8OiV8dIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23307 /* scatterdiv8di */, 23321 /* vv*UcV8OiV8OiIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23337 /* scatterdiv8sf */, 23351 /* vv*UcV4OiV4fIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23366 /* scatterdiv8si */, 23380 /* vv*UcV4OiV4iIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23395 /* scatterdiv16sf */, 23410 /* vv*UcV8OiV8fIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23425 /* scatterdiv16si */, 23440 /* vv*UcV8OiV8iIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23455 /* scattersiv2df */, 23469 /* vv*UcV4iV2dIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23483 /* scattersiv2di */, 23497 /* vv*UcV4iV2OiIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23512 /* scattersiv4df */, 23526 /* vv*UcV4iV4dIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23540 /* scattersiv4di */, 23554 /* vv*UcV4iV4OiIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23569 /* scattersiv4sf */, 23583 /* vv*UcV4iV4fIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23597 /* scattersiv4si */, 23611 /* vv*UcV4iV4iIi */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23625 /* scattersiv8df */, 23639 /* vv*UcV8iV8dIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23653 /* scattersiv8di */, 23667 /* vv*UcV8iV8OiIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23682 /* scattersiv8sf */, 23696 /* vv*UcV8iV8fIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23710 /* scattersiv8si */, 23724 /* vv*UcV8iV8iIi */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23738 /* scattersiv16sf */, 23753 /* vv*UsV16iV16fIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23769 /* scattersiv16si */, 23784 /* vv*UsV16iV16iIi */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23800 /* selectb_128 */, 23812 /* V16cUsV16cV16c */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23827 /* selectb_256 */, 23839 /* V32cUiV32cV32c */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23854 /* selectb_512 */, 23866 /* V64cUOiV64cV64c */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23882 /* selectd_128 */, 23894 /* V4iUcV4iV4i */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23906 /* selectd_256 */, 23918 /* V8iUcV8iV8i */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23930 /* selectd_512 */, 23942 /* V16iUsV16iV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23957 /* selectpbf_128 */, 23971 /* V8yUcV8yV8y */, 1016 /* ncEV:128: */, 5054 /* avx512bf16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23983 /* selectpbf_256 */, 23997 /* V16yUsV16yV16y */, 1053 /* ncEV:256: */, 5054 /* avx512bf16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24012 /* selectpbf_512 */, 24026 /* V32yUiV32yV32y */, 1638 /* ncEV:512: */, 5134 /* avx512bf16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24041 /* selectpd_128 */, 24054 /* V2dUcV2dV2d */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24066 /* selectpd_256 */, 24079 /* V4dUcV4dV4d */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24091 /* selectpd_512 */, 24104 /* V8dUcV8dV8d */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24116 /* selectph_128 */, 24129 /* V8xUcV8xV8x */, 1016 /* ncEV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24141 /* selectph_256 */, 24154 /* V16xUsV16xV16x */, 1053 /* ncEV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24169 /* selectph_512 */, 24182 /* V32xUiV32xV32x */, 1638 /* ncEV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24197 /* selectps_128 */, 24210 /* V4fUcV4fV4f */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24222 /* selectps_256 */, 24235 /* V8fUcV8fV8f */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24247 /* selectps_512 */, 24260 /* V16fUsV16fV16f */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24275 /* selectq_128 */, 24287 /* V2OiUcV2OiV2Oi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24302 /* selectq_256 */, 24314 /* V4OiUcV4OiV4Oi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24329 /* selectq_512 */, 24341 /* V8OiUcV8OiV8Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24356 /* selectsbf_128 */, 23971 /* V8yUcV8yV8y */, 1016 /* ncEV:128: */, 5134 /* avx512bf16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24370 /* selectsd_128 */, 24054 /* V2dUcV2dV2d */, 1016 /* ncEV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24383 /* selectsh_128 */, 24129 /* V8xUcV8xV8x */, 1016 /* ncEV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24396 /* selectss_128 */, 24210 /* V4fUcV4fV4f */, 1016 /* ncEV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24409 /* selectw_128 */, 24421 /* V8sUcV8sV8s */, 1016 /* ncEV:128: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24433 /* selectw_256 */, 24445 /* V16sUsV16sV16s */, 1053 /* ncEV:256: */, 4610 /* avx512bw,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24460 /* selectw_512 */, 24472 /* V32sUiV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24487 /* serialize */, 278 /* v */, 744 /* n */, 24487 /* serialize */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24497 /* setssbsy */, 278 /* v */, 744 /* n */, 2026 /* shstk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24506 /* sfence */, 278 /* v */, 744 /* n */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24513 /* sha1msg1 */, 16044 /* V4iV4iV4i */, 920 /* ncV:128: */, 24522 /* sha */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24526 /* sha1msg2 */, 16044 /* V4iV4iV4i */, 920 /* ncV:128: */, 24522 /* sha */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24535 /* sha1nexte */, 16044 /* V4iV4iV4i */, 920 /* ncV:128: */, 24522 /* sha */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24545 /* sha1rnds4 */, 24555 /* V4iV4iV4iIc */, 920 /* ncV:128: */, 24522 /* sha */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24567 /* sha256msg1 */, 16044 /* V4iV4iV4i */, 920 /* ncV:128: */, 24522 /* sha */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24578 /* sha256msg2 */, 16044 /* V4iV4iV4i */, 920 /* ncV:128: */, 24522 /* sha */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24589 /* sha256rnds2 */, 24601 /* V4iV4iV4iV4i */, 920 /* ncV:128: */, 24522 /* sha */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24614 /* shuf_f32x4 */, 871 /* V16fV16fV16fIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24625 /* shuf_f32x4_256 */, 1865 /* V8fV8fV8fIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24640 /* shuf_f64x2 */, 798 /* V8dV8dV8dIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24651 /* shuf_f64x2_256 */, 1822 /* V4dV4dV4dIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24666 /* shuf_i32x4 */, 1623 /* V16iV16iV16iIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24677 /* shuf_i32x4_256 */, 1601 /* V8iV8iV8iIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24692 /* shuf_i64x2 */, 1708 /* V8OiV8OiV8OiIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24703 /* shuf_i64x2_256 */, 1683 /* V4OiV4OiV4OiIi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24718 /* shufpd */, 1792 /* V2dV2dV2dIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24725 /* shufpd256 */, 1822 /* V4dV4dV4dIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24735 /* shufpd512 */, 798 /* V8dV8dV8dIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24745 /* shufps */, 1842 /* V4fV4fV4fIi */, 1016 /* ncEV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24752 /* shufps256 */, 1865 /* V8fV8fV8fIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24762 /* shufps512 */, 871 /* V16fV16fV16fIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24772 /* slwpcb */, 25 /* v* */, 744 /* n */, 12652 /* lwp */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24779 /* sqrtpd512 */, 15609 /* V8dV8dIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24789 /* sqrtph512 */, 24799 /* V32xV32xIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24810 /* sqrtps512 */, 24820 /* V16fV16fIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24831 /* sqrtsd_round_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24849 /* sqrtsh_round_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24867 /* sqrtss_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24885 /* stmxcsr */, 622 /* Ui */, 744 /* n */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24893 /* storeapd128_mask */, 3903 /* vV2d*V2dUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24910 /* storeapd256_mask */, 3938 /* vV4d*V4dUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24927 /* storeapd512_mask */, 3981 /* vV8d*V8dUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24944 /* storeaps128_mask */, 4356 /* vV4f*V4fUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24961 /* storeaps256_mask */, 4391 /* vV8f*V8fUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24978 /* storeaps512_mask */, 4426 /* vV16f*V16fUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24995 /* storedqudi128_mask */, 4024 /* vV2Oi*V2OiUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25014 /* storedqudi256_mask */, 4061 /* vV4Oi*V4OiUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25033 /* storedqudi512_mask */, 25052 /* vOi*V8OiUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25063 /* storedquhi128_mask */, 4135 /* vV8s*V8sUc */, 1165 /* nV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25082 /* storedquhi256_mask */, 4170 /* vV16s*V16sUs */, 3949 /* nV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25101 /* storedquhi512_mask */, 4207 /* vV32s*V32sUi */, 3992 /* nV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25120 /* storedquqi128_mask */, 4244 /* vV16c*V16cUs */, 1165 /* nV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25139 /* storedquqi256_mask */, 4281 /* vV32c*V32cUi */, 3949 /* nV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25158 /* storedquqi512_mask */, 4318 /* vV64c*V64cUOi */, 3992 /* nV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25177 /* storedqusi128_mask */, 4463 /* vV4i*V4iUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25196 /* storedqusi256_mask */, 4498 /* vV8i*V8iUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25215 /* storedqusi512_mask */, 25234 /* vi*V16iUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25244 /* storesbf16128_mask */, 25263 /* vV8y*V8yUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25274 /* storesd128_mask */, 3903 /* vV2d*V2dUc */, 1165 /* nV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25290 /* storesh128_mask */, 25306 /* vV8x*V8xUc */, 1165 /* nV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25317 /* storess128_mask */, 4356 /* vV4f*V4fUc */, 1165 /* nV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25333 /* storeupd128_mask */, 3903 /* vV2d*V2dUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25350 /* storeupd256_mask */, 3938 /* vV4d*V4dUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25367 /* storeupd512_mask */, 25384 /* vd*V8dUc */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25393 /* storeups128_mask */, 4356 /* vV4f*V4fUc */, 1165 /* nV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25410 /* storeups256_mask */, 4391 /* vV8f*V8fUc */, 3949 /* nV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25427 /* storeups512_mask */, 25444 /* vf*V16fUs */, 3992 /* nV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25454 /* subborrow_u32 */, 774 /* UcUcUiUiUi* */, 786 /* nE */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25468 /* subpd512 */, 798 /* V8dV8dV8dIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25477 /* subph512 */, 836 /* V32xV32xV32xIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25486 /* subps512 */, 871 /* V16fV16fV16fIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25495 /* subsd_round_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25512 /* subsh_round_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25529 /* subss_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25546 /* tpause */, 25553 /* UcUiUiUi */, 744 /* n */, 25562 /* waitpkg */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25570 /* tzcnt_u16 */, 12155 /* UsUs */, 1753 /* ncE */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25580 /* tzcnt_u32 */, 12131 /* UiUi */, 1753 /* ncE */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25590 /* ucmpb128_mask */, 2057 /* UsV16cV16cIiUs */, 1016 /* ncEV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25604 /* ucmpb256_mask */, 2103 /* UiV32cV32cIiUi */, 1053 /* ncEV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25618 /* ucmpb512_mask */, 2131 /* UOiV64cV64cIiUOi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25632 /* ucmpd128_mask */, 2170 /* UcV4iV4iIiUc */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25646 /* ucmpd256_mask */, 2196 /* UcV8iV8iIiUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25660 /* ucmpd512_mask */, 2222 /* UsV16iV16iIiUs */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25674 /* ucmpq128_mask */, 2856 /* UcV2OiV2OiIiUc */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25688 /* ucmpq256_mask */, 2884 /* UcV4OiV4OiIiUc */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25702 /* ucmpq512_mask */, 2912 /* UcV8OiV8OiIiUc */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25716 /* ucmpw128_mask */, 3074 /* UcV8sV8sIiUc */, 1016 /* ncEV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25730 /* ucmpw256_mask */, 3100 /* UsV16sV16sIiUs */, 1053 /* ncEV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25744 /* ucmpw512_mask */, 3128 /* UiV32sV32sIiUi */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25758 /* ucomieq */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25766 /* ucomige */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25774 /* ucomigt */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25782 /* ucomile */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25790 /* ucomilt */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25798 /* ucomineq */, 3150 /* iV4fV4f */, 2245 /* cnV:128: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25807 /* ucomisdeq */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25817 /* ucomisdge */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25827 /* ucomisdgt */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25837 /* ucomisdle */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25847 /* ucomisdlt */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25857 /* ucomisdneq */, 3203 /* iV2dV2d */, 2245 /* cnV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25868 /* umonitor */, 606 /* vvC* */, 744 /* n */, 25562 /* waitpkg */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25877 /* umwait */, 25553 /* UcUiUiUi */, 744 /* n */, 25562 /* waitpkg */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25884 /* undef128 */, 25893 /* V2d */, 2245 /* cnV:128: */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25897 /* undef256 */, 25906 /* V4d */, 25910 /* cnV:256: */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25919 /* undef512 */, 25928 /* V8d */, 25932 /* cnV:512: */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25941 /* vaddbf16128 */, 25953 /* V8yV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25963 /* vaddbf16256 */, 25975 /* V16yV16yV16y */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25988 /* vaddbf16512 */, 26000 /* V32yV32yV32y */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26013 /* vbcstnebf162ps128 */, 26031 /* V4fyC* */, 1165 /* nV:128: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26051 /* vbcstnebf162ps256 */, 26069 /* V8fyC* */, 3949 /* nV:256: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26076 /* vbcstnesh2ps128 */, 26092 /* V4fxC* */, 1165 /* nV:128: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26099 /* vbcstnesh2ps256 */, 26115 /* V8fxC* */, 3949 /* nV:256: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26122 /* vcmpbf16128_mask */, 26139 /* UcV8yV8yIiUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26152 /* vcmpbf16256_mask */, 26169 /* UsV16yV16yIiUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26184 /* vcmpbf16512_mask */, 26201 /* UiV32yV32yIiUi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26216 /* vcomisbf16eq */, 26229 /* iV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26237 /* vcomisbf16ge */, 26229 /* iV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26250 /* vcomisbf16gt */, 26229 /* iV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26263 /* vcomisbf16le */, 26229 /* iV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26276 /* vcomisbf16lt */, 26229 /* iV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26289 /* vcomisbf16neq */, 26229 /* iV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26303 /* vcomisd */, 26311 /* iV2dV2dIiIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26323 /* vcomish */, 26331 /* iV8xV8xIiIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26343 /* vcomiss */, 26351 /* iV4fV4fIiIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26363 /* vcvt2ph2bf8_128 */, 26379 /* V16cV8xV8x */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26390 /* vcvt2ph2bf8_256 */, 26406 /* V32cV16xV16x */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26419 /* vcvt2ph2bf8_512 */, 26435 /* V64cV32xV32x */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26448 /* vcvt2ph2bf8s_128 */, 26379 /* V16cV8xV8x */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26465 /* vcvt2ph2bf8s_256 */, 26406 /* V32cV16xV16x */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26482 /* vcvt2ph2bf8s_512 */, 26435 /* V64cV32xV32x */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26499 /* vcvt2ph2hf8_128 */, 26379 /* V16cV8xV8x */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26515 /* vcvt2ph2hf8_256 */, 26406 /* V32cV16xV16x */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26531 /* vcvt2ph2hf8_512 */, 26435 /* V64cV32xV32x */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26547 /* vcvt2ph2hf8s_128 */, 26379 /* V16cV8xV8x */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26564 /* vcvt2ph2hf8s_256 */, 26406 /* V32cV16xV16x */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26581 /* vcvt2ph2hf8s_512 */, 26435 /* V64cV32xV32x */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26598 /* vcvt2ps2phx128_mask */, 26618 /* V8xV4fV4fV8xUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26633 /* vcvt2ps2phx256_mask */, 26653 /* V16xV8fV8fV16xUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26670 /* vcvt2ps2phx512_mask */, 26690 /* V32xV16fV16fV32xUiIi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26711 /* vcvtbf162ibs128 */, 26727 /* V8UsV8y */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26735 /* vcvtbf162ibs256 */, 26751 /* V16UsV16y */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26761 /* vcvtbf162ibs512 */, 26777 /* V32UsV32y */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26787 /* vcvtbf162iubs128 */, 26727 /* V8UsV8y */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26804 /* vcvtbf162iubs256 */, 26751 /* V16UsV16y */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26821 /* vcvtbf162iubs512 */, 26777 /* V32UsV32y */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26838 /* vcvtbiasph2bf8_128_mask */, 26862 /* V16cV16cV8xV16cUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26880 /* vcvtbiasph2bf8_256_mask */, 26904 /* V16cV32cV16xV16cUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26923 /* vcvtbiasph2bf8_512_mask */, 26947 /* V32cV64cV32xV32cUi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26966 /* vcvtbiasph2bf8s_128_mask */, 26862 /* V16cV16cV8xV16cUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26991 /* vcvtbiasph2bf8s_256_mask */, 26904 /* V16cV32cV16xV16cUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27016 /* vcvtbiasph2bf8s_512_mask */, 26947 /* V32cV64cV32xV32cUi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27041 /* vcvtbiasph2hf8_128_mask */, 26862 /* V16cV16cV8xV16cUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27065 /* vcvtbiasph2hf8_256_mask */, 26904 /* V16cV32cV16xV16cUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27089 /* vcvtbiasph2hf8_512_mask */, 26947 /* V32cV64cV32xV32cUi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27113 /* vcvtbiasph2hf8s_128_mask */, 26862 /* V16cV16cV8xV16cUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27138 /* vcvtbiasph2hf8s_256_mask */, 26904 /* V16cV32cV16xV16cUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27163 /* vcvtbiasph2hf8s_512_mask */, 26947 /* V32cV64cV32xV32cUi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27188 /* vcvtdq2ph128_mask */, 27206 /* V8xV4iV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27218 /* vcvtdq2ph256_mask */, 27236 /* V8xV8iV8xUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27248 /* vcvtdq2ph512_mask */, 27266 /* V16xV16iV16xUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27283 /* vcvthf8_2ph128_mask */, 27303 /* V8xV16cV8xUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27316 /* vcvthf8_2ph256_mask */, 27336 /* V16xV16cV16xUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27351 /* vcvthf8_2ph512_mask */, 27371 /* V32xV32cV32xUi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27386 /* vcvtneebf162ps128 */, 27404 /* V4fV8yC* */, 1165 /* nV:128: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27413 /* vcvtneebf162ps256 */, 27431 /* V8fV16yC* */, 3949 /* nV:256: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27441 /* vcvtneeph2ps128 */, 27457 /* V4fV8xC* */, 1165 /* nV:128: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27466 /* vcvtneeph2ps256 */, 27482 /* V8fV16xC* */, 3949 /* nV:256: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27492 /* vcvtneobf162ps128 */, 27404 /* V4fV8yC* */, 1165 /* nV:128: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27510 /* vcvtneobf162ps256 */, 27431 /* V8fV16yC* */, 3949 /* nV:256: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27528 /* vcvtneoph2ps128 */, 27457 /* V4fV8xC* */, 1165 /* nV:128: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27544 /* vcvtneoph2ps256 */, 27482 /* V8fV16xC* */, 3949 /* nV:256: */, 26038 /* avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27560 /* vcvtneps2bf16128 */, 27577 /* V8yV4f */, 1165 /* nV:128: */, 27584 /* avx512bf16,avx512vl|avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27617 /* vcvtneps2bf16256 */, 27634 /* V8yV8f */, 3949 /* nV:256: */, 27584 /* avx512bf16,avx512vl|avxneconvert */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27641 /* vcvtpd2ph128_mask */, 27659 /* V8xV2dV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27671 /* vcvtpd2ph256_mask */, 27689 /* V8xV4dV8xUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27701 /* vcvtpd2ph512_mask */, 27719 /* V8xV8dV8xUcIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27733 /* vcvtph2bf8_128_mask */, 27753 /* V16cV8xV16cUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27767 /* vcvtph2bf8_256_mask */, 27787 /* V16cV16xV16cUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27802 /* vcvtph2bf8_512_mask */, 27822 /* V32cV32xV32cUi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27837 /* vcvtph2bf8s_128_mask */, 27753 /* V16cV8xV16cUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27858 /* vcvtph2bf8s_256_mask */, 27787 /* V16cV16xV16cUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27879 /* vcvtph2bf8s_512_mask */, 27822 /* V32cV32xV32cUi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27900 /* vcvtph2dq128_mask */, 27918 /* V4iV8xV4iUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27930 /* vcvtph2dq256_mask */, 27948 /* V8iV8xV8iUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27960 /* vcvtph2dq512_mask */, 27978 /* V16iV16xV16iUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27995 /* vcvtph2hf8_128_mask */, 27753 /* V16cV8xV16cUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28015 /* vcvtph2hf8_256_mask */, 27787 /* V16cV16xV16cUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28035 /* vcvtph2hf8_512_mask */, 27822 /* V32cV32xV32cUi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28055 /* vcvtph2hf8s_128_mask */, 27753 /* V16cV8xV16cUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28076 /* vcvtph2hf8s_256_mask */, 27787 /* V16cV16xV16cUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28097 /* vcvtph2hf8s_512_mask */, 27822 /* V32cV32xV32cUi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28118 /* vcvtph2ibs128_mask */, 28137 /* V8UsV8xV8UsUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28151 /* vcvtph2ibs256_mask */, 28170 /* V16UsV16xV16UsUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28187 /* vcvtph2ibs512_mask */, 28206 /* V32UsV32xV32UsUiIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28225 /* vcvtph2iubs128_mask */, 28137 /* V8UsV8xV8UsUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28245 /* vcvtph2iubs256_mask */, 28170 /* V16UsV16xV16UsUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28265 /* vcvtph2iubs512_mask */, 28206 /* V32UsV32xV32UsUiIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28285 /* vcvtph2pd128_mask */, 28303 /* V2dV8xV2dUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28315 /* vcvtph2pd256_mask */, 28333 /* V4dV8xV4dUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28345 /* vcvtph2pd512_mask */, 28363 /* V8dV8xV8dUcIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28377 /* vcvtph2ps256_mask */, 28395 /* V8fV8sV8fUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28407 /* vcvtph2ps512_mask */, 28425 /* V16fV16sV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28442 /* vcvtph2ps_mask */, 28457 /* V4fV8sV4fUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28469 /* vcvtph2psx128_mask */, 28488 /* V4fV8xV4fUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28500 /* vcvtph2psx256_mask */, 28519 /* V8fV8xV8fUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28531 /* vcvtph2psx512_mask */, 28550 /* V16fV16xV16fUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28567 /* vcvtph2qq128_mask */, 28585 /* V2OiV8xV2OiUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28599 /* vcvtph2qq256_mask */, 28617 /* V4OiV8xV4OiUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28631 /* vcvtph2qq512_mask */, 28649 /* V8OiV8xV8OiUcIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28665 /* vcvtph2udq128_mask */, 28684 /* V4UiV8xV4UiUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28698 /* vcvtph2udq256_mask */, 28717 /* V8UiV8xV8UiUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28731 /* vcvtph2udq512_mask */, 28750 /* V16UiV16xV16UiUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28769 /* vcvtph2uqq128_mask */, 28788 /* V2UOiV8xV2UOiUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28804 /* vcvtph2uqq256_mask */, 28823 /* V4UOiV8xV4UOiUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28839 /* vcvtph2uqq512_mask */, 28858 /* V8UOiV8xV8UOiUcIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28876 /* vcvtph2uw128_mask */, 28137 /* V8UsV8xV8UsUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28894 /* vcvtph2uw256_mask */, 28170 /* V16UsV16xV16UsUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28912 /* vcvtph2uw512_mask */, 28206 /* V32UsV32xV32UsUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28930 /* vcvtph2w128_mask */, 28947 /* V8sV8xV8sUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28959 /* vcvtph2w256_mask */, 28976 /* V16sV16xV16sUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28991 /* vcvtph2w512_mask */, 29008 /* V32sV32xV32sUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29025 /* vcvtps2ibs128_mask */, 29044 /* V4UiV4fV4UiUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29058 /* vcvtps2ibs256_mask */, 29077 /* V8UiV8fV8UiUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29091 /* vcvtps2ibs512_mask */, 29110 /* V16UiV16fV16UiUsIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29129 /* vcvtps2iubs128_mask */, 29044 /* V4UiV4fV4UiUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29149 /* vcvtps2iubs256_mask */, 29077 /* V8UiV8fV8UiUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29169 /* vcvtps2iubs512_mask */, 29110 /* V16UiV16fV16UiUsIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29189 /* vcvtps2ph */, 29199 /* V8sV4fIi */, 1016 /* ncEV:128: */, 29208 /* f16c */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29213 /* vcvtps2ph256 */, 29226 /* V8sV8fIi */, 1053 /* ncEV:256: */, 29208 /* f16c */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29235 /* vcvtps2ph256_mask */, 29253 /* V8sV8fIiV8sUc */, 1199 /* ncV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29267 /* vcvtps2ph512_mask */, 29285 /* V16sV16fIiV16sUs */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29302 /* vcvtps2ph_mask */, 29317 /* V8sV4fIiV8sUc */, 920 /* ncV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29331 /* vcvtps2phx128_mask */, 29350 /* V8xV4fV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29362 /* vcvtps2phx256_mask */, 29381 /* V8xV8fV8xUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29393 /* vcvtps2phx512_mask */, 29412 /* V16xV16fV16xUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29429 /* vcvtqq2ph128_mask */, 29447 /* V8xV2OiV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29460 /* vcvtqq2ph256_mask */, 29478 /* V8xV4OiV8xUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29491 /* vcvtqq2ph512_mask */, 29509 /* V8xV8OiV8xUcIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29524 /* vcvtsd2sh_round_mask */, 29545 /* V8xV8xV2dV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29562 /* vcvtsd2si32 */, 29574 /* iV2dIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29581 /* vcvtsd2usi32 */, 29594 /* UiV2dIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29602 /* vcvtsh2sd_round_mask */, 29623 /* V2dV2dV8xV2dUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29640 /* vcvtsh2si32 */, 29652 /* iV8xIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29659 /* vcvtsh2ss_round_mask */, 29680 /* V4fV4fV8xV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29697 /* vcvtsh2usi32 */, 29710 /* UiV8xIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29718 /* vcvtsi2sh */, 29728 /* V8xV8xiIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29738 /* vcvtss2sh_round_mask */, 29759 /* V8xV8xV4fV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29776 /* vcvtss2si32 */, 29788 /* iV4fIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29795 /* vcvtss2usi32 */, 29808 /* UiV4fIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29816 /* vcvttbf162ibs128 */, 26727 /* V8UsV8y */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29833 /* vcvttbf162ibs256 */, 26751 /* V16UsV16y */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29850 /* vcvttbf162ibs512 */, 26777 /* V32UsV32y */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29867 /* vcvttbf162iubs128 */, 26727 /* V8UsV8y */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29885 /* vcvttbf162iubs256 */, 26751 /* V16UsV16y */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29903 /* vcvttbf162iubs512 */, 26777 /* V32UsV32y */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29921 /* vcvttpd2dqs128_mask */, 5284 /* V4iV2dV4iUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29941 /* vcvttpd2dqs256_mask */, 5587 /* V4iV4dV4iUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29961 /* vcvttpd2dqs512_round_mask */, 5332 /* V8iV8dV8iUcIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29987 /* vcvttpd2qqs128_mask */, 5455 /* V2OiV2dV2OiUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30007 /* vcvttpd2qqs256_mask */, 5504 /* V4OiV4dV4OiUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30027 /* vcvttpd2qqs512_round_mask */, 5535 /* V8OiV8dV8OiUcIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30053 /* vcvttpd2udqs128_mask */, 5284 /* V4iV2dV4iUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30074 /* vcvttpd2udqs256_mask */, 5587 /* V4iV4dV4iUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30095 /* vcvttpd2udqs512_round_mask */, 5332 /* V8iV8dV8iUcIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30122 /* vcvttpd2uqqs128_mask */, 5455 /* V2OiV2dV2OiUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30143 /* vcvttpd2uqqs256_mask */, 5504 /* V4OiV4dV4OiUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30164 /* vcvttpd2uqqs512_round_mask */, 5535 /* V8OiV8dV8OiUcIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30191 /* vcvttph2dq128_mask */, 27918 /* V4iV8xV4iUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30210 /* vcvttph2dq256_mask */, 27948 /* V8iV8xV8iUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30229 /* vcvttph2dq512_mask */, 27978 /* V16iV16xV16iUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30248 /* vcvttph2ibs128_mask */, 28137 /* V8UsV8xV8UsUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30268 /* vcvttph2ibs256_mask */, 28170 /* V16UsV16xV16UsUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30288 /* vcvttph2ibs512_mask */, 28206 /* V32UsV32xV32UsUiIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30308 /* vcvttph2iubs128_mask */, 28137 /* V8UsV8xV8UsUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30329 /* vcvttph2iubs256_mask */, 28170 /* V16UsV16xV16UsUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30350 /* vcvttph2iubs512_mask */, 28206 /* V32UsV32xV32UsUiIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30371 /* vcvttph2qq128_mask */, 28585 /* V2OiV8xV2OiUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30390 /* vcvttph2qq256_mask */, 28617 /* V4OiV8xV4OiUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30409 /* vcvttph2qq512_mask */, 28649 /* V8OiV8xV8OiUcIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30428 /* vcvttph2udq128_mask */, 28684 /* V4UiV8xV4UiUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30448 /* vcvttph2udq256_mask */, 28717 /* V8UiV8xV8UiUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30468 /* vcvttph2udq512_mask */, 28750 /* V16UiV16xV16UiUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30488 /* vcvttph2uqq128_mask */, 28788 /* V2UOiV8xV2UOiUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30508 /* vcvttph2uqq256_mask */, 28823 /* V4UOiV8xV4UOiUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30528 /* vcvttph2uqq512_mask */, 28858 /* V8UOiV8xV8UOiUcIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30548 /* vcvttph2uw128_mask */, 28137 /* V8UsV8xV8UsUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30567 /* vcvttph2uw256_mask */, 28170 /* V16UsV16xV16UsUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30586 /* vcvttph2uw512_mask */, 28206 /* V32UsV32xV32UsUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30605 /* vcvttph2w128_mask */, 28947 /* V8sV8xV8sUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30623 /* vcvttph2w256_mask */, 28976 /* V16sV16xV16sUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30641 /* vcvttph2w512_mask */, 29008 /* V32sV32xV32sUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30659 /* vcvttps2dqs128_mask */, 5884 /* V4iV4fV4iUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30679 /* vcvttps2dqs256_mask */, 5914 /* V8iV8fV8iUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30699 /* vcvttps2dqs512_round_mask */, 5723 /* V16iV16fV16iUsIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30725 /* vcvttps2ibs128_mask */, 29044 /* V4UiV4fV4UiUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30745 /* vcvttps2ibs256_mask */, 29077 /* V8UiV8fV8UiUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30765 /* vcvttps2ibs512_mask */, 29110 /* V16UiV16fV16UiUsIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30785 /* vcvttps2iubs128_mask */, 29044 /* V4UiV4fV4UiUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30806 /* vcvttps2iubs256_mask */, 29077 /* V8UiV8fV8UiUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30827 /* vcvttps2iubs512_mask */, 29110 /* V16UiV16fV16UiUsIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30848 /* vcvttps2qqs128_mask */, 5788 /* V2OiV4fV2OiUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30868 /* vcvttps2qqs256_mask */, 5819 /* V4OiV4fV4OiUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30888 /* vcvttps2qqs512_round_mask */, 5850 /* V8OiV8fV8OiUcIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30914 /* vcvttps2udqs128_mask */, 5884 /* V4iV4fV4iUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30935 /* vcvttps2udqs256_mask */, 5914 /* V8iV8fV8iUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30956 /* vcvttps2udqs512_round_mask */, 5723 /* V16iV16fV16iUsIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30983 /* vcvttps2uqqs128_mask */, 5788 /* V2OiV4fV2OiUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31004 /* vcvttps2uqqs256_mask */, 5819 /* V4OiV4fV4OiUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31025 /* vcvttps2uqqs512_round_mask */, 5850 /* V8OiV8fV8OiUcIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31052 /* vcvttsd2si32 */, 29574 /* iV2dIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31065 /* vcvttsd2sis32 */, 29574 /* iV2dIi */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31079 /* vcvttsd2usi32 */, 29594 /* UiV2dIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31093 /* vcvttsd2usis32 */, 29594 /* UiV2dIi */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31108 /* vcvttsh2si32 */, 29652 /* iV8xIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31121 /* vcvttsh2usi32 */, 29710 /* UiV8xIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31135 /* vcvttss2si32 */, 29788 /* iV4fIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31148 /* vcvttss2sis32 */, 29788 /* iV4fIi */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31162 /* vcvttss2usi32 */, 29808 /* UiV4fIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31176 /* vcvttss2usis32 */, 29808 /* UiV4fIi */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31191 /* vcvtudq2ph128_mask */, 31210 /* V8xV4UiV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31223 /* vcvtudq2ph256_mask */, 31242 /* V8xV8UiV8xUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31255 /* vcvtudq2ph512_mask */, 31274 /* V16xV16UiV16xUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31292 /* vcvtuqq2ph128_mask */, 31311 /* V8xV2UOiV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31325 /* vcvtuqq2ph256_mask */, 31344 /* V8xV4UOiV8xUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31358 /* vcvtuqq2ph512_mask */, 31377 /* V8xV8UOiV8xUcIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31393 /* vcvtusi2sh */, 31404 /* V8xV8xUiIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31415 /* vcvtuw2ph128_mask */, 31433 /* V8xV8UsV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31446 /* vcvtuw2ph256_mask */, 31464 /* V16xV16UsV16xUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31480 /* vcvtuw2ph512_mask */, 31498 /* V32xV32UsV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31516 /* vcvtw2ph128_mask */, 31533 /* V8xV8sV8xUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31545 /* vcvtw2ph256_mask */, 31562 /* V16xV16sV16xUs */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31577 /* vcvtw2ph512_mask */, 31594 /* V32xV32sV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31611 /* vdivbf16128 */, 25953 /* V8yV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31623 /* vdivbf16256 */, 25975 /* V16yV16yV16y */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31635 /* vdivbf16512 */, 26000 /* V32yV32yV32y */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31647 /* vdpphps128 */, 31658 /* V4fV4fV8xV8x */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31671 /* vdpphps256 */, 31682 /* V8fV8fV16xV16x */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31697 /* vdpphps512 */, 31708 /* V16fV16fV32xV32x */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31725 /* vec_ext_v2di */, 31738 /* OiV2OiIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31747 /* vec_ext_v4hi */, 31760 /* sV4sIi */, 31767 /* ncEV:64: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31776 /* vec_ext_v4sf */, 31789 /* fV4fIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31796 /* vec_ext_v4si */, 31809 /* iV4iIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31816 /* vec_ext_v8hi */, 31829 /* sV8sIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31836 /* vec_ext_v8si */, 31849 /* iV8iIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31856 /* vec_ext_v16hi */, 31870 /* sV16sIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31878 /* vec_ext_v16qi */, 31892 /* cV16cIi */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31900 /* vec_ext_v32qi */, 31914 /* cV32cIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31922 /* vec_set_v4hi */, 31935 /* V4sV4ssIi */, 31767 /* ncEV:64: */, 2262 /* sse */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31945 /* vec_set_v4si */, 31958 /* V4iV4iiIi */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31968 /* vec_set_v8hi */, 31981 /* V8sV8ssIi */, 1016 /* ncEV:128: */, 2001 /* sse2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31991 /* vec_set_v8si */, 32004 /* V8iV8iiIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32014 /* vec_set_v16hi */, 32028 /* V16sV16ssIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32040 /* vec_set_v16qi */, 32054 /* V16cV16ccIi */, 1016 /* ncEV:128: */, 1804 /* sse4.1 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32066 /* vec_set_v32qi */, 32080 /* V32cV32ccIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32092 /* vextractf128_pd256 */, 32111 /* V2dV4dIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32120 /* vextractf128_ps256 */, 32139 /* V4fV8fIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32148 /* vextractf128_si256 */, 32167 /* V4iV8iIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32176 /* vfcmaddcph128_mask */, 21288 /* V4fV4fV4fV4fUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32195 /* vfcmaddcph128_maskz */, 21288 /* V4fV4fV4fV4fUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32215 /* vfcmaddcph256_mask */, 22989 /* V8fV8fV8fV8fUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32234 /* vfcmaddcph256_maskz */, 22989 /* V8fV8fV8fV8fUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32254 /* vfcmaddcph512_mask */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32273 /* vfcmaddcph512_mask3 */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32293 /* vfcmaddcph512_maskz */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32313 /* vfcmaddcsh_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32329 /* vfcmaddcsh_maskz */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32346 /* vfcmaddcsh_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32368 /* vfcmaddcsh_round_mask3 */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32391 /* vfcmulcph128_mask */, 21288 /* V4fV4fV4fV4fUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32409 /* vfcmulcph256_mask */, 22989 /* V8fV8fV8fV8fUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32427 /* vfcmulcph512_mask */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32445 /* vfcmulcsh_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32460 /* vfmaddcph128_mask */, 21288 /* V4fV4fV4fV4fUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32478 /* vfmaddcph128_maskz */, 21288 /* V4fV4fV4fV4fUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32497 /* vfmaddcph256_mask */, 22989 /* V8fV8fV8fV8fUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32515 /* vfmaddcph256_maskz */, 22989 /* V8fV8fV8fV8fUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32534 /* vfmaddcph512_mask */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32552 /* vfmaddcph512_mask3 */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32571 /* vfmaddcph512_maskz */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32590 /* vfmaddcsh_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32605 /* vfmaddcsh_maskz */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32621 /* vfmaddcsh_round_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32642 /* vfmaddcsh_round_mask3 */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32664 /* vfmaddpd512_mask */, 22847 /* V8dV8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32681 /* vfmaddpd512_mask3 */, 22847 /* V8dV8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32699 /* vfmaddpd512_maskz */, 22847 /* V8dV8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32717 /* vfmaddph512_mask */, 22934 /* V32xV32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32734 /* vfmaddph512_mask3 */, 22934 /* V32xV32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32752 /* vfmaddph512_maskz */, 22934 /* V32xV32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32770 /* vfmaddps512_mask */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32787 /* vfmaddps512_mask3 */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32805 /* vfmaddps512_maskz */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32823 /* vfmaddsd3_mask */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32838 /* vfmaddsd3_mask3 */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32854 /* vfmaddsd3_maskz */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32870 /* vfmaddsh3_mask */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32885 /* vfmaddsh3_mask3 */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32901 /* vfmaddsh3_maskz */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32917 /* vfmaddss3_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32932 /* vfmaddss3_mask3 */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32948 /* vfmaddss3_maskz */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32964 /* vfmaddsubpd */, 1886 /* V2dV2dV2dV2d */, 920 /* ncV:128: */, 32976 /* fma|fma4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32985 /* vfmaddsubpd256 */, 1911 /* V4dV4dV4dV4d */, 1199 /* ncV:256: */, 32976 /* fma|fma4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33000 /* vfmaddsubpd512_mask */, 22847 /* V8dV8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33020 /* vfmaddsubpd512_mask3 */, 22847 /* V8dV8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33041 /* vfmaddsubpd512_maskz */, 22847 /* V8dV8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33062 /* vfmaddsubph */, 33074 /* V8xV8xV8xV8x */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33087 /* vfmaddsubph256 */, 33102 /* V16xV16xV16xV16x */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33119 /* vfmaddsubph512_mask */, 22934 /* V32xV32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33139 /* vfmaddsubph512_mask3 */, 22934 /* V32xV32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33160 /* vfmaddsubph512_maskz */, 22934 /* V32xV32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33181 /* vfmaddsubps */, 1933 /* V4fV4fV4fV4f */, 920 /* ncV:128: */, 32976 /* fma|fma4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33193 /* vfmaddsubps256 */, 1958 /* V8fV8fV8fV8f */, 1199 /* ncV:256: */, 32976 /* fma|fma4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33208 /* vfmaddsubps512_mask */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33228 /* vfmaddsubps512_mask3 */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33249 /* vfmaddsubps512_maskz */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33270 /* vfmsubaddpd512_mask3 */, 22847 /* V8dV8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33291 /* vfmsubaddph512_mask3 */, 22934 /* V32xV32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33312 /* vfmsubaddps512_mask3 */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33333 /* vfmsubpd512_mask3 */, 22847 /* V8dV8dV8dV8dUcIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33351 /* vfmsubph512_mask3 */, 22934 /* V32xV32xV32xV32xUiIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33369 /* vfmsubps512_mask3 */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33387 /* vfmsubsd3_mask3 */, 903 /* V2dV2dV2dV2dUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33403 /* vfmsubsh3_mask3 */, 946 /* V8xV8xV8xV8xUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33419 /* vfmsubss3_mask3 */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33435 /* vfmulcph128_mask */, 21288 /* V4fV4fV4fV4fUc */, 920 /* ncV:128: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33452 /* vfmulcph256_mask */, 22989 /* V8fV8fV8fV8fUc */, 1199 /* ncV:256: */, 2639 /* avx512fp16,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33469 /* vfmulcph512_mask */, 23021 /* V16fV16fV16fV16fUsIi */, 810 /* ncV:512: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33486 /* vfmulcsh_mask */, 980 /* V4fV4fV4fV4fUcIi */, 920 /* ncV:128: */, 851 /* avx512fp16 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33500 /* vfpclassbf16128_mask */, 33521 /* UcV8yIiUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33531 /* vfpclassbf16256_mask */, 33552 /* UsV16yIiUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33563 /* vfpclassbf16512_mask */, 33584 /* UiV32yIiUi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33595 /* vfrczpd */, 33603 /* V2dV2d */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33614 /* vfrczpd256 */, 33625 /* V4dV4d */, 1199 /* ncV:256: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33632 /* vfrczps */, 21366 /* V4fV4f */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33640 /* vfrczps256 */, 21382 /* V8fV8f */, 1199 /* ncV:256: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33651 /* vfrczsd */, 33603 /* V2dV2d */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33659 /* vfrczss */, 21366 /* V4fV4f */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33667 /* vgetexpbf16128_mask */, 33687 /* V8yV8yV8yUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33699 /* vgetexpbf16256_mask */, 33719 /* V16yV16yV16yUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33734 /* vgetexpbf16512_mask */, 33754 /* V32yV32yV32yUi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33769 /* vgetmantbf16128_mask */, 33790 /* V8yV8yIiV8yUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33804 /* vgetmantbf16256_mask */, 33825 /* V16yV16yIiV16yUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33842 /* vgetmantbf16512_mask */, 33863 /* V32yV32yIiV32yUi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33880 /* vgf2p8affineinvqb_v16qi */, 14463 /* V16cV16cV16cIc */, 1016 /* ncEV:128: */, 33904 /* gfni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33909 /* vgf2p8affineinvqb_v32qi */, 14489 /* V32cV32cV32cIc */, 1053 /* ncEV:256: */, 33933 /* avx,gfni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33942 /* vgf2p8affineinvqb_v64qi */, 33966 /* V64cV64cV64cIc */, 1638 /* ncEV:512: */, 33981 /* avx512f,gfni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{33994 /* vgf2p8affineqb_v16qi */, 14463 /* V16cV16cV16cIc */, 1016 /* ncEV:128: */, 33904 /* gfni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34015 /* vgf2p8affineqb_v32qi */, 14489 /* V32cV32cV32cIc */, 1053 /* ncEV:256: */, 33933 /* avx,gfni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34036 /* vgf2p8affineqb_v64qi */, 33966 /* V64cV64cV64cIc */, 1638 /* ncEV:512: */, 33981 /* avx512f,gfni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34057 /* vgf2p8mulb_v16qi */, 15832 /* V16cV16cV16c */, 1016 /* ncEV:128: */, 33904 /* gfni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34074 /* vgf2p8mulb_v32qi */, 15878 /* V32cV32cV32c */, 1053 /* ncEV:256: */, 33933 /* avx,gfni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34091 /* vgf2p8mulb_v64qi */, 15904 /* V64cV64cV64c */, 1638 /* ncEV:512: */, 33981 /* avx512f,gfni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34108 /* vinsertf128_pd256 */, 11724 /* V4dV4dV2dIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34126 /* vinsertf128_ps256 */, 11670 /* V8fV8fV4fIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34144 /* vinsertf128_si256 */, 11830 /* V8iV8iV4iIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34162 /* vmaxbf16128 */, 25953 /* V8yV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34174 /* vmaxbf16256 */, 25975 /* V16yV16yV16y */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34186 /* vmaxbf16512 */, 26000 /* V32yV32yV32y */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34198 /* vminbf16128 */, 25953 /* V8yV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34210 /* vminbf16256 */, 25975 /* V16yV16yV16y */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34222 /* vminbf16512 */, 26000 /* V32yV32yV32y */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34234 /* vminmaxbf16128 */, 34249 /* V8yV8yV8yIi */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34261 /* vminmaxbf16256 */, 34276 /* V16yV16yV16yIi */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34291 /* vminmaxbf16512 */, 34306 /* V32yV32yV32yIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34321 /* vminmaxpd128_mask */, 20879 /* V2dV2dV2dIiV2dUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34339 /* vminmaxpd256_mask */, 20912 /* V4dV4dV4dIiV4dUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34357 /* vminmaxpd512_round_mask */, 20945 /* V8dV8dV8dIiV8dUcIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34381 /* vminmaxph128_mask */, 34399 /* V8xV8xV8xIiV8xUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34416 /* vminmaxph256_mask */, 34434 /* V16xV16xV16xIiV16xUs */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34455 /* vminmaxph512_round_mask */, 34479 /* V32xV32xV32xIiV32xUiIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34502 /* vminmaxps128_mask */, 20980 /* V4fV4fV4fIiV4fUc */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34520 /* vminmaxps256_mask */, 21013 /* V8fV8fV8fIiV8fUc */, 3949 /* nV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34538 /* vminmaxps512_round_mask */, 21046 /* V16fV16fV16fIiV16fUsIi */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34562 /* vminmaxsd_round_mask */, 11424 /* V2dV2dV2dIiV2dUcIi */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34583 /* vminmaxsh_round_mask */, 11464 /* V8xV8xV8xIiV8xUcIi */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34604 /* vminmaxss_round_mask */, 11504 /* V4fV4fV4fIiV4fUcIi */, 1165 /* nV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34625 /* vmulbf16128 */, 25953 /* V8yV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34637 /* vmulbf16256 */, 25975 /* V16yV16yV16y */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34649 /* vmulbf16512 */, 26000 /* V32yV32yV32y */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34661 /* vp2intersect_d_128 */, 34680 /* vV4iV4iUc*Uc* */, 1165 /* nV:128: */, 34694 /* avx512vp2intersect,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34722 /* vp2intersect_d_256 */, 34741 /* vV8iV8iUc*Uc* */, 3949 /* nV:256: */, 34694 /* avx512vp2intersect,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34755 /* vp2intersect_d_512 */, 34774 /* vV16iV16iUs*Us* */, 3992 /* nV:512: */, 34790 /* avx512vp2intersect */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34809 /* vp2intersect_q_128 */, 34828 /* vV2OiV2OiUc*Uc* */, 1165 /* nV:128: */, 34694 /* avx512vp2intersect,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34844 /* vp2intersect_q_256 */, 34863 /* vV4OiV4OiUc*Uc* */, 3949 /* nV:256: */, 34694 /* avx512vp2intersect,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34879 /* vp2intersect_q_512 */, 34898 /* vV8OiV8OiUc*Uc* */, 3992 /* nV:512: */, 34790 /* avx512vp2intersect */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34914 /* vpcomb */, 14463 /* V16cV16cV16cIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34921 /* vpcomd */, 24555 /* V4iV4iV4iIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34928 /* vpcomq */, 15222 /* V2OiV2OiV2OiIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34935 /* vpcomub */, 14463 /* V16cV16cV16cIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34943 /* vpcomud */, 24555 /* V4iV4iV4iIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34951 /* vpcomuq */, 15222 /* V2OiV2OiV2OiIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34959 /* vpcomuw */, 34967 /* V8sV8sV8sIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34979 /* vpcomw */, 34967 /* V8sV8sV8sIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{34986 /* vpconflictdi_128 */, 1521 /* V2OiV2Oi */, 1016 /* ncEV:128: */, 35003 /* avx512cd,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35021 /* vpconflictdi_256 */, 35038 /* V4OiV4Oi */, 1053 /* ncEV:256: */, 35003 /* avx512cd,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35047 /* vpconflictdi_512 */, 35064 /* V8OiV8Oi */, 1638 /* ncEV:512: */, 35073 /* avx512cd */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35082 /* vpconflictsi_128 */, 35099 /* V4iV4i */, 1016 /* ncEV:128: */, 35003 /* avx512cd,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35106 /* vpconflictsi_256 */, 35123 /* V8iV8i */, 1053 /* ncEV:256: */, 35003 /* avx512cd,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35130 /* vpconflictsi_512 */, 35147 /* V16iV16i */, 1638 /* ncEV:512: */, 35073 /* avx512cd */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35156 /* vpdpbssd128 */, 35168 /* V4iV4iV16cV16c */, 920 /* ncV:128: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35203 /* vpdpbssd256 */, 35215 /* V8iV8iV32cV32c */, 1199 /* ncV:256: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35230 /* vpdpbssd512 */, 35242 /* V16iV16iV64cV64c */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35259 /* vpdpbssds128 */, 35168 /* V4iV4iV16cV16c */, 920 /* ncV:128: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35272 /* vpdpbssds256 */, 35215 /* V8iV8iV32cV32c */, 1199 /* ncV:256: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35285 /* vpdpbssds512 */, 35242 /* V16iV16iV64cV64c */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35298 /* vpdpbsud128 */, 35310 /* V4iV4iV16cV16Uc */, 920 /* ncV:128: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35326 /* vpdpbsud256 */, 35338 /* V8iV8iV32cV32Uc */, 1199 /* ncV:256: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35354 /* vpdpbsud512 */, 35366 /* V16iV16iV64cV64Uc */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35384 /* vpdpbsuds128 */, 35310 /* V4iV4iV16cV16Uc */, 920 /* ncV:128: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35397 /* vpdpbsuds256 */, 35338 /* V8iV8iV32cV32Uc */, 1199 /* ncV:256: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35410 /* vpdpbsuds512 */, 35366 /* V16iV16iV64cV64Uc */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35423 /* vpdpbusd128 */, 35435 /* V4iV4iV16UcV16c */, 920 /* ncV:128: */, 35451 /* avx512vl,avx512vnni|avxvnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35479 /* vpdpbusd256 */, 35491 /* V8iV8iV32UcV32c */, 1199 /* ncV:256: */, 35451 /* avx512vl,avx512vnni|avxvnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35507 /* vpdpbusd512 */, 35519 /* V16iV16iV64UcV64c */, 810 /* ncV:512: */, 35537 /* avx512vnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35548 /* vpdpbusds128 */, 35435 /* V4iV4iV16UcV16c */, 920 /* ncV:128: */, 35451 /* avx512vl,avx512vnni|avxvnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35561 /* vpdpbusds256 */, 35491 /* V8iV8iV32UcV32c */, 1199 /* ncV:256: */, 35451 /* avx512vl,avx512vnni|avxvnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35574 /* vpdpbusds512 */, 35519 /* V16iV16iV64UcV64c */, 810 /* ncV:512: */, 35537 /* avx512vnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35587 /* vpdpbuud128 */, 35599 /* V4iV4iV16UcV16Uc */, 920 /* ncV:128: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35616 /* vpdpbuud256 */, 35628 /* V8iV8iV32UcV32Uc */, 1199 /* ncV:256: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35645 /* vpdpbuud512 */, 35657 /* V16iV16iV64UcV64Uc */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35676 /* vpdpbuuds128 */, 35599 /* V4iV4iV16UcV16Uc */, 920 /* ncV:128: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35689 /* vpdpbuuds256 */, 35628 /* V8iV8iV32UcV32Uc */, 1199 /* ncV:256: */, 35183 /* avxvnniint8|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35702 /* vpdpbuuds512 */, 35657 /* V16iV16iV64UcV64Uc */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35715 /* vpdpwssd128 */, 35727 /* V4iV4iV8sV8s */, 920 /* ncV:128: */, 35451 /* avx512vl,avx512vnni|avxvnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35740 /* vpdpwssd256 */, 35752 /* V8iV8iV16sV16s */, 1199 /* ncV:256: */, 35451 /* avx512vl,avx512vnni|avxvnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35767 /* vpdpwssd512 */, 35779 /* V16iV16iV32sV32s */, 810 /* ncV:512: */, 35537 /* avx512vnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35796 /* vpdpwssds128 */, 35727 /* V4iV4iV8sV8s */, 920 /* ncV:128: */, 35451 /* avx512vl,avx512vnni|avxvnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35809 /* vpdpwssds256 */, 35752 /* V8iV8iV16sV16s */, 1199 /* ncV:256: */, 35451 /* avx512vl,avx512vnni|avxvnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35822 /* vpdpwssds512 */, 35779 /* V16iV16iV32sV32s */, 810 /* ncV:512: */, 35537 /* avx512vnni */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35835 /* vpdpwsud128 */, 35847 /* V4iV4iV8sV8Us */, 1165 /* nV:128: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35882 /* vpdpwsud256 */, 35894 /* V8iV8iV16sV16Us */, 3949 /* nV:256: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35910 /* vpdpwsud512 */, 35922 /* V16iV16iV32sV32Us */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35940 /* vpdpwsuds128 */, 35847 /* V4iV4iV8sV8Us */, 1165 /* nV:128: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35953 /* vpdpwsuds256 */, 35894 /* V8iV8iV16sV16Us */, 3949 /* nV:256: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35966 /* vpdpwsuds512 */, 35922 /* V16iV16iV32sV32Us */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{35979 /* vpdpwusd128 */, 35991 /* V4iV4iV8UsV8s */, 1165 /* nV:128: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36005 /* vpdpwusd256 */, 36017 /* V8iV8iV16UsV16s */, 3949 /* nV:256: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36033 /* vpdpwusd512 */, 36045 /* V16iV16iV32UsV32s */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36063 /* vpdpwusds128 */, 35991 /* V4iV4iV8UsV8s */, 1165 /* nV:128: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36076 /* vpdpwusds256 */, 36017 /* V8iV8iV16UsV16s */, 3949 /* nV:256: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36089 /* vpdpwusds512 */, 36045 /* V16iV16iV32UsV32s */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36102 /* vpdpwuud128 */, 36114 /* V4iV4iV8UsV8Us */, 1165 /* nV:128: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36129 /* vpdpwuud256 */, 36141 /* V8iV8iV16UsV16Us */, 3949 /* nV:256: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36158 /* vpdpwuud512 */, 36170 /* V16iV16iV32UsV32Us */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36189 /* vpdpwuuds128 */, 36114 /* V4iV4iV8UsV8Us */, 1165 /* nV:128: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36202 /* vpdpwuuds256 */, 36141 /* V8iV8iV16UsV16Us */, 3949 /* nV:256: */, 35861 /* avxvnniint16|avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36215 /* vpdpwuuds512 */, 36170 /* V16iV16iV32UsV32Us */, 3992 /* nV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36228 /* vperm2f128_pd256 */, 1822 /* V4dV4dV4dIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36245 /* vperm2f128_ps256 */, 1865 /* V8fV8fV8fIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36262 /* vperm2f128_si256 */, 1601 /* V8iV8iV8iIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36279 /* vpermi2vard128 */, 24601 /* V4iV4iV4iV4i */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36294 /* vpermi2vard256 */, 36309 /* V8iV8iV8iV8i */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36322 /* vpermi2vard512 */, 36337 /* V16iV16iV16iV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36354 /* vpermi2varhi128 */, 36370 /* V8sV8sV8sV8s */, 1016 /* ncEV:128: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36383 /* vpermi2varhi256 */, 36399 /* V16sV16sV16sV16s */, 1053 /* ncEV:256: */, 2072 /* avx512vl,avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36416 /* vpermi2varhi512 */, 36432 /* V32sV32sV32sV32s */, 1638 /* ncEV:512: */, 2148 /* avx512bw */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36449 /* vpermi2varpd128 */, 36465 /* V2dV2dV2OiV2d */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36479 /* vpermi2varpd256 */, 36495 /* V4dV4dV4OiV4d */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36509 /* vpermi2varpd512 */, 36525 /* V8dV8dV8OiV8d */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36539 /* vpermi2varps128 */, 36555 /* V4fV4fV4iV4f */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36568 /* vpermi2varps256 */, 36584 /* V8fV8fV8iV8f */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36597 /* vpermi2varps512 */, 36613 /* V16fV16fV16iV16f */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36630 /* vpermi2varq128 */, 36645 /* V2OiV2OiV2OiV2Oi */, 1016 /* ncEV:128: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36662 /* vpermi2varq256 */, 36677 /* V4OiV4OiV4OiV4Oi */, 1053 /* ncEV:256: */, 1582 /* avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36694 /* vpermi2varq512 */, 36709 /* V8OiV8OiV8OiV8Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36726 /* vpermi2varqi128 */, 15114 /* V16cV16cV16cV16c */, 1016 /* ncEV:128: */, 15845 /* avx512vbmi,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36742 /* vpermi2varqi256 */, 15143 /* V32cV32cV32cV32c */, 1053 /* ncEV:256: */, 15845 /* avx512vbmi,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36758 /* vpermi2varqi512 */, 36774 /* V64cV64cV64cV64c */, 1638 /* ncEV:512: */, 15917 /* avx512vbmi */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36791 /* vpermil2pd */, 36802 /* V2dV2dV2dV2OiIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36818 /* vpermil2pd256 */, 36832 /* V4dV4dV4dV4OiIc */, 1199 /* ncV:256: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36848 /* vpermil2ps */, 36859 /* V4fV4fV4fV4iIc */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36874 /* vpermil2ps256 */, 36888 /* V8fV8fV8fV8iIc */, 1199 /* ncV:256: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36903 /* vpermilpd */, 22461 /* V2dV2dIi */, 1016 /* ncEV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36913 /* vpermilpd256 */, 15590 /* V4dV4dIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36926 /* vpermilpd512 */, 15609 /* V8dV8dIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36939 /* vpermilps */, 22489 /* V4fV4fIi */, 1016 /* ncEV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36949 /* vpermilps256 */, 22509 /* V8fV8fIi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36962 /* vpermilps512 */, 24820 /* V16fV16fIi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36975 /* vpermilvarpd */, 36988 /* V2dV2dV2Oi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{36999 /* vpermilvarpd256 */, 15683 /* V4dV4dV4Oi */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37015 /* vpermilvarpd512 */, 15707 /* V8dV8dV8Oi */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37031 /* vpermilvarps */, 37044 /* V4fV4fV4i */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37054 /* vpermilvarps256 */, 15941 /* V8fV8fV8i */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37070 /* vpermilvarps512 */, 15964 /* V16fV16fV16i */, 1638 /* ncEV:512: */, 819 /* avx512f */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37086 /* vphaddbd */, 37095 /* V4iV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37103 /* vphaddbq */, 37112 /* V2OiV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37121 /* vphaddbw */, 37130 /* V8sV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37138 /* vphadddq */, 37147 /* V2OiV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37155 /* vphaddubd */, 37095 /* V4iV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37165 /* vphaddubq */, 37112 /* V2OiV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37175 /* vphaddubw */, 37130 /* V8sV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37185 /* vphaddudq */, 37147 /* V2OiV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37195 /* vphadduwd */, 37205 /* V4iV8s */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37212 /* vphadduwq */, 37222 /* V2OiV8s */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37230 /* vphaddwd */, 37205 /* V4iV8s */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37239 /* vphaddwq */, 37222 /* V2OiV8s */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37248 /* vphsubbw */, 37130 /* V8sV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37257 /* vphsubdq */, 37147 /* V2OiV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37266 /* vphsubwd */, 37205 /* V4iV8s */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37275 /* vpmacsdd */, 24601 /* V4iV4iV4iV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37284 /* vpmacsdqh */, 37294 /* V2OiV4iV4iV2Oi */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37309 /* vpmacsdql */, 37294 /* V2OiV4iV4iV2Oi */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37319 /* vpmacssdd */, 24601 /* V4iV4iV4iV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37329 /* vpmacssdqh */, 37294 /* V2OiV4iV4iV2Oi */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37340 /* vpmacssdql */, 37294 /* V2OiV4iV4iV2Oi */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37351 /* vpmacsswd */, 37361 /* V4iV8sV8sV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37374 /* vpmacssww */, 36370 /* V8sV8sV8sV8s */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37384 /* vpmacswd */, 37361 /* V4iV8sV8sV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37393 /* vpmacsww */, 36370 /* V8sV8sV8sV8s */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37402 /* vpmadcsswd */, 37361 /* V4iV8sV8sV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37413 /* vpmadcswd */, 37361 /* V4iV8sV8sV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37423 /* vpmadd52huq128 */, 36645 /* V2OiV2OiV2OiV2Oi */, 1016 /* ncEV:128: */, 37438 /* avx512ifma,avx512vl|avxifma */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37466 /* vpmadd52huq256 */, 36677 /* V4OiV4OiV4OiV4Oi */, 1053 /* ncEV:256: */, 37438 /* avx512ifma,avx512vl|avxifma */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37481 /* vpmadd52huq512 */, 36709 /* V8OiV8OiV8OiV8Oi */, 1638 /* ncEV:512: */, 37496 /* avx512ifma */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37507 /* vpmadd52luq128 */, 36645 /* V2OiV2OiV2OiV2Oi */, 1016 /* ncEV:128: */, 37438 /* avx512ifma,avx512vl|avxifma */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37522 /* vpmadd52luq256 */, 36677 /* V4OiV4OiV4OiV4Oi */, 1053 /* ncEV:256: */, 37438 /* avx512ifma,avx512vl|avxifma */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37537 /* vpmadd52luq512 */, 36709 /* V8OiV8OiV8OiV8Oi */, 1638 /* ncEV:512: */, 37496 /* avx512ifma */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37552 /* vpmultishiftqb128 */, 15832 /* V16cV16cV16c */, 1016 /* ncEV:128: */, 15845 /* avx512vbmi,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37570 /* vpmultishiftqb256 */, 15878 /* V32cV32cV32c */, 1053 /* ncEV:256: */, 15845 /* avx512vbmi,avx512vl */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37588 /* vpmultishiftqb512 */, 15904 /* V64cV64cV64c */, 1638 /* ncEV:512: */, 15917 /* avx512vbmi */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37606 /* vpperm */, 15114 /* V16cV16cV16cV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37613 /* vprotbi */, 37621 /* V16cV16cIc */, 1016 /* ncEV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37632 /* vprotdi */, 37640 /* V4iV4iIc */, 1016 /* ncEV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37649 /* vprotqi */, 1549 /* V2OiV2OiIc */, 1016 /* ncEV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37657 /* vprotwi */, 37665 /* V8sV8sIc */, 1016 /* ncEV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37674 /* vpshab */, 15832 /* V16cV16cV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37681 /* vpshad */, 16044 /* V4iV4iV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37688 /* vpshaq */, 1118 /* V2OiV2OiV2Oi */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37695 /* vpshaw */, 15757 /* V8sV8sV8s */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37702 /* vpshlb */, 15832 /* V16cV16cV16c */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37709 /* vpshld */, 16044 /* V4iV4iV4i */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37716 /* vpshldd128 */, 1570 /* V4iV4iV4iIi */, 1016 /* ncEV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37727 /* vpshldd256 */, 1601 /* V8iV8iV8iIi */, 1053 /* ncEV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37738 /* vpshldd512 */, 1623 /* V16iV16iV16iIi */, 1638 /* ncEV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37749 /* vpshldq128 */, 1658 /* V2OiV2OiV2OiIi */, 1016 /* ncEV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37760 /* vpshldq256 */, 1683 /* V4OiV4OiV4OiIi */, 1053 /* ncEV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37771 /* vpshldq512 */, 1708 /* V8OiV8OiV8OiIi */, 1638 /* ncEV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37782 /* vpshldw128 */, 15171 /* V8sV8sV8sIi */, 1016 /* ncEV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37793 /* vpshldw256 */, 15194 /* V16sV16sV16sIi */, 1053 /* ncEV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37804 /* vpshldw512 */, 37815 /* V32sV32sV32sIi */, 1638 /* ncEV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37830 /* vpshlq */, 1118 /* V2OiV2OiV2Oi */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37837 /* vpshlw */, 15757 /* V8sV8sV8s */, 920 /* ncV:128: */, 33610 /* xop */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37844 /* vpshrdd128 */, 1570 /* V4iV4iV4iIi */, 1016 /* ncEV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37855 /* vpshrdd256 */, 1601 /* V8iV8iV8iIi */, 1053 /* ncEV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37866 /* vpshrdd512 */, 1623 /* V16iV16iV16iIi */, 1638 /* ncEV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37877 /* vpshrdq128 */, 1658 /* V2OiV2OiV2OiIi */, 1016 /* ncEV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37888 /* vpshrdq256 */, 1683 /* V4OiV4OiV4OiIi */, 1053 /* ncEV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37899 /* vpshrdq512 */, 1708 /* V8OiV8OiV8OiIi */, 1638 /* ncEV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37910 /* vpshrdw128 */, 15171 /* V8sV8sV8sIi */, 1016 /* ncEV:128: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37921 /* vpshrdw256 */, 15194 /* V16sV16sV16sIi */, 1053 /* ncEV:256: */, 3483 /* avx512vl,avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37932 /* vpshrdw512 */, 37815 /* V32sV32sV32sIi */, 1638 /* ncEV:512: */, 3572 /* avx512vbmi2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37943 /* vpshufbitqmb128_mask */, 37964 /* UsV16cV16cUs */, 1016 /* ncEV:128: */, 37977 /* avx512vl,avx512bitalg */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{37999 /* vpshufbitqmb256_mask */, 38020 /* UiV32cV32cUi */, 1053 /* ncEV:256: */, 37977 /* avx512vl,avx512bitalg */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38033 /* vpshufbitqmb512_mask */, 38054 /* UOiV64cV64cUOi */, 1638 /* ncEV:512: */, 38069 /* avx512bitalg */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38082 /* vrcpbf16128_mask */, 33687 /* V8yV8yV8yUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38099 /* vrcpbf16256_mask */, 33719 /* V16yV16yV16yUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38116 /* vrcpbf16512_mask */, 33754 /* V32yV32yV32yUi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38133 /* vreducebf16128_mask */, 33790 /* V8yV8yIiV8yUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38153 /* vreducebf16256_mask */, 33825 /* V16yV16yIiV16yUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38173 /* vreducebf16512_mask */, 33863 /* V32yV32yIiV32yUi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38193 /* vrndscalebf16_128_mask */, 33790 /* V8yV8yIiV8yUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38216 /* vrndscalebf16_256_mask */, 33825 /* V16yV16yIiV16yUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38239 /* vrndscalebf16_mask */, 33863 /* V32yV32yIiV32yUi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38258 /* vrsqrtbf16128_mask */, 33687 /* V8yV8yV8yUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38277 /* vrsqrtbf16256_mask */, 33719 /* V16yV16yV16yUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38296 /* vrsqrtbf16512_mask */, 33754 /* V32yV32yV32yUi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38315 /* vscalefbf16128_mask */, 38335 /* V8yV8yV8yV8yUc */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38350 /* vscalefbf16256_mask */, 38370 /* V16yV16yV16yV16yUs */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38389 /* vscalefbf16512_mask */, 38409 /* V32yV32yV32yV32yUi */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38428 /* vsha512msg1 */, 38440 /* V4UOiV4UOiV2UOi */, 3949 /* nV:256: */, 38456 /* sha512 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38463 /* vsha512msg2 */, 38475 /* V4UOiV4UOiV4UOi */, 3949 /* nV:256: */, 38456 /* sha512 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38491 /* vsha512rnds2 */, 38504 /* V4UOiV4UOiV4UOiV2UOi */, 3949 /* nV:256: */, 38456 /* sha512 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38525 /* vsm3msg1 */, 38534 /* V4UiV4UiV4UiV4Ui */, 1165 /* nV:128: */, 38551 /* sm3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38555 /* vsm3msg2 */, 38534 /* V4UiV4UiV4UiV4Ui */, 1165 /* nV:128: */, 38551 /* sm3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38564 /* vsm3rnds2 */, 38574 /* V4UiV4UiV4UiV4UiIUi */, 1165 /* nV:128: */, 38551 /* sm3 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38594 /* vsm4key4128 */, 38606 /* V4UiV4UiV4Ui */, 1165 /* nV:128: */, 38619 /* sm4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38623 /* vsm4key4256 */, 38635 /* V8UiV8UiV8Ui */, 3949 /* nV:256: */, 38619 /* sm4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38648 /* vsm4key4512 */, 38660 /* V16UiV16UiV16Ui */, 3992 /* nV:512: */, 38676 /* avx10.2,sm4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38688 /* vsm4rnds4128 */, 38606 /* V4UiV4UiV4Ui */, 1165 /* nV:128: */, 38619 /* sm4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38701 /* vsm4rnds4256 */, 38635 /* V8UiV8UiV8Ui */, 3949 /* nV:256: */, 38619 /* sm4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38714 /* vsm4rnds4512 */, 38660 /* V16UiV16UiV16Ui */, 3992 /* nV:512: */, 38676 /* avx10.2,sm4 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38727 /* vsubbf16128 */, 25953 /* V8yV8yV8y */, 920 /* ncV:128: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38739 /* vsubbf16256 */, 25975 /* V16yV16yV16y */, 1199 /* ncV:256: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38751 /* vsubbf16512 */, 26000 /* V32yV32yV32y */, 810 /* ncV:512: */, 13072 /* avx10.2 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38763 /* vtestcpd */, 3203 /* iV2dV2d */, 1016 /* ncEV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38772 /* vtestcpd256 */, 38784 /* iV4dV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38792 /* vtestcps */, 3150 /* iV4fV4f */, 1016 /* ncEV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38801 /* vtestcps256 */, 38813 /* iV8fV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38821 /* vtestnzcpd */, 3203 /* iV2dV2d */, 1016 /* ncEV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38832 /* vtestnzcpd256 */, 38784 /* iV4dV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38846 /* vtestnzcps */, 3150 /* iV4fV4f */, 1016 /* ncEV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38857 /* vtestnzcps256 */, 38813 /* iV8fV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38871 /* vtestzpd */, 3203 /* iV2dV2d */, 1016 /* ncEV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38880 /* vtestzpd256 */, 38784 /* iV4dV4d */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38892 /* vtestzps */, 3150 /* iV4fV4f */, 1016 /* ncEV:128: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38901 /* vtestzps256 */, 38813 /* iV8fV8f */, 1053 /* ncEV:256: */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38913 /* vzeroall */, 278 /* v */, 744 /* n */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38922 /* vzeroupper */, 278 /* v */, 744 /* n */, 1063 /* avx */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38933 /* wbinvd */, 278 /* v */, 744 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38940 /* wbnoinvd */, 278 /* v */, 744 /* n */, 38940 /* wbnoinvd */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38949 /* writeeflags_u32 */, 687 /* vUi */, 744 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38965 /* wrpkru */, 687 /* vUi */, 744 /* n */, 21434 /* pku */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38972 /* wrssd */, 11996 /* vUiv* */, 744 /* n */, 2026 /* shstk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38978 /* wrussd */, 11996 /* vUiv* */, 744 /* n */, 2026 /* shstk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{38985 /* xabort */, 38992 /* vIc */, 744 /* n */, 38996 /* rtm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39000 /* xbegin */, 39007 /* i */, 744 /* n */, 38996 /* rtm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39009 /* xend */, 278 /* v */, 744 /* n */, 38996 /* rtm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39014 /* xgetbv */, 39021 /* UOiUi */, 744 /* n */, 39027 /* xsave */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39033 /* xresldtrk */, 278 /* v */, 744 /* n */, 39043 /* tsxldtrk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39052 /* xrstor */, 39059 /* vv*UOi */, 744 /* n */, 39027 /* xsave */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39066 /* xrstors */, 39059 /* vv*UOi */, 744 /* n */, 39074 /* xsaves */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39027 /* xsave */, 39059 /* vv*UOi */, 744 /* n */, 39027 /* xsave */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39081 /* xsavec */, 39059 /* vv*UOi */, 744 /* n */, 39081 /* xsavec */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39088 /* xsaveopt */, 39059 /* vv*UOi */, 744 /* n */, 39088 /* xsaveopt */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39074 /* xsaves */, 39059 /* vv*UOi */, 744 /* n */, 39074 /* xsaves */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39097 /* xsetbv */, 39104 /* vUiUOi */, 744 /* n */, 39027 /* xsave */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39111 /* xsusldtrk */, 278 /* v */, 744 /* n */, 39043 /* tsxldtrk */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{39121 /* xtest */, 39007 /* i */, 744 /* n */, 38996 /* rtm */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},

#endif // GET_BUILTIN_PREFIXED_INFOS

#ifdef ATOMIC_BUILTIN

#endif // ATOMIC_BUILTIN
#undef ATOMIC_BUILTIN
