/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* List of builtins that Clang recognizes                                     *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/


#ifdef GET_BUILTIN_ENUMERATORS
    BI__builtin_SI_to_SXTHI_asrh,
    BI__builtin_brev_ldb,
    BI__builtin_brev_ldd,
    BI__builtin_brev_ldh,
    BI__builtin_brev_ldub,
    BI__builtin_brev_lduh,
    BI__builtin_brev_ldw,
    BI__builtin_brev_stb,
    BI__builtin_brev_std,
    BI__builtin_brev_sth,
    BI__builtin_brev_sthhi,
    BI__builtin_brev_stw,
    BI__builtin_circ_ldb,
    BI__builtin_circ_ldd,
    BI__builtin_circ_ldh,
    BI__builtin_circ_ldub,
    BI__builtin_circ_lduh,
    BI__builtin_circ_ldw,
    BI__builtin_circ_stb,
    BI__builtin_circ_std,
    BI__builtin_circ_sth,
    BI__builtin_circ_sthhi,
    BI__builtin_circ_stw,
    BI__builtin_HEXAGON_A2_abs,
    BI__builtin_HEXAGON_A2_absp,
    BI__builtin_HEXAGON_A2_abssat,
    BI__builtin_HEXAGON_A2_add,
    BI__builtin_HEXAGON_A2_addh_h16_hh,
    BI__builtin_HEXAGON_A2_addh_h16_hl,
    BI__builtin_HEXAGON_A2_addh_h16_lh,
    BI__builtin_HEXAGON_A2_addh_h16_ll,
    BI__builtin_HEXAGON_A2_addh_h16_sat_hh,
    BI__builtin_HEXAGON_A2_addh_h16_sat_hl,
    BI__builtin_HEXAGON_A2_addh_h16_sat_lh,
    BI__builtin_HEXAGON_A2_addh_h16_sat_ll,
    BI__builtin_HEXAGON_A2_addh_l16_hl,
    BI__builtin_HEXAGON_A2_addh_l16_ll,
    BI__builtin_HEXAGON_A2_addh_l16_sat_hl,
    BI__builtin_HEXAGON_A2_addh_l16_sat_ll,
    BI__builtin_HEXAGON_A2_addi,
    BI__builtin_HEXAGON_A2_addp,
    BI__builtin_HEXAGON_A2_addpsat,
    BI__builtin_HEXAGON_A2_addsat,
    BI__builtin_HEXAGON_A2_addsp,
    BI__builtin_HEXAGON_A2_and,
    BI__builtin_HEXAGON_A2_andir,
    BI__builtin_HEXAGON_A2_andp,
    BI__builtin_HEXAGON_A2_aslh,
    BI__builtin_HEXAGON_A2_asrh,
    BI__builtin_HEXAGON_A2_combine_hh,
    BI__builtin_HEXAGON_A2_combine_hl,
    BI__builtin_HEXAGON_A2_combine_lh,
    BI__builtin_HEXAGON_A2_combine_ll,
    BI__builtin_HEXAGON_A2_combineii,
    BI__builtin_HEXAGON_A2_combinew,
    BI__builtin_HEXAGON_A2_max,
    BI__builtin_HEXAGON_A2_maxp,
    BI__builtin_HEXAGON_A2_maxu,
    BI__builtin_HEXAGON_A2_maxup,
    BI__builtin_HEXAGON_A2_min,
    BI__builtin_HEXAGON_A2_minp,
    BI__builtin_HEXAGON_A2_minu,
    BI__builtin_HEXAGON_A2_minup,
    BI__builtin_HEXAGON_A2_neg,
    BI__builtin_HEXAGON_A2_negp,
    BI__builtin_HEXAGON_A2_negsat,
    BI__builtin_HEXAGON_A2_not,
    BI__builtin_HEXAGON_A2_notp,
    BI__builtin_HEXAGON_A2_or,
    BI__builtin_HEXAGON_A2_orir,
    BI__builtin_HEXAGON_A2_orp,
    BI__builtin_HEXAGON_A2_roundsat,
    BI__builtin_HEXAGON_A2_sat,
    BI__builtin_HEXAGON_A2_satb,
    BI__builtin_HEXAGON_A2_sath,
    BI__builtin_HEXAGON_A2_satub,
    BI__builtin_HEXAGON_A2_satuh,
    BI__builtin_HEXAGON_A2_sub,
    BI__builtin_HEXAGON_A2_subh_h16_hh,
    BI__builtin_HEXAGON_A2_subh_h16_hl,
    BI__builtin_HEXAGON_A2_subh_h16_lh,
    BI__builtin_HEXAGON_A2_subh_h16_ll,
    BI__builtin_HEXAGON_A2_subh_h16_sat_hh,
    BI__builtin_HEXAGON_A2_subh_h16_sat_hl,
    BI__builtin_HEXAGON_A2_subh_h16_sat_lh,
    BI__builtin_HEXAGON_A2_subh_h16_sat_ll,
    BI__builtin_HEXAGON_A2_subh_l16_hl,
    BI__builtin_HEXAGON_A2_subh_l16_ll,
    BI__builtin_HEXAGON_A2_subh_l16_sat_hl,
    BI__builtin_HEXAGON_A2_subh_l16_sat_ll,
    BI__builtin_HEXAGON_A2_subp,
    BI__builtin_HEXAGON_A2_subri,
    BI__builtin_HEXAGON_A2_subsat,
    BI__builtin_HEXAGON_A2_svaddh,
    BI__builtin_HEXAGON_A2_svaddhs,
    BI__builtin_HEXAGON_A2_svadduhs,
    BI__builtin_HEXAGON_A2_svavgh,
    BI__builtin_HEXAGON_A2_svavghs,
    BI__builtin_HEXAGON_A2_svnavgh,
    BI__builtin_HEXAGON_A2_svsubh,
    BI__builtin_HEXAGON_A2_svsubhs,
    BI__builtin_HEXAGON_A2_svsubuhs,
    BI__builtin_HEXAGON_A2_swiz,
    BI__builtin_HEXAGON_A2_sxtb,
    BI__builtin_HEXAGON_A2_sxth,
    BI__builtin_HEXAGON_A2_sxtw,
    BI__builtin_HEXAGON_A2_tfr,
    BI__builtin_HEXAGON_A2_tfrih,
    BI__builtin_HEXAGON_A2_tfril,
    BI__builtin_HEXAGON_A2_tfrp,
    BI__builtin_HEXAGON_A2_tfrpi,
    BI__builtin_HEXAGON_A2_tfrsi,
    BI__builtin_HEXAGON_A2_vabsh,
    BI__builtin_HEXAGON_A2_vabshsat,
    BI__builtin_HEXAGON_A2_vabsw,
    BI__builtin_HEXAGON_A2_vabswsat,
    BI__builtin_HEXAGON_A2_vaddb_map,
    BI__builtin_HEXAGON_A2_vaddh,
    BI__builtin_HEXAGON_A2_vaddhs,
    BI__builtin_HEXAGON_A2_vaddub,
    BI__builtin_HEXAGON_A2_vaddubs,
    BI__builtin_HEXAGON_A2_vadduhs,
    BI__builtin_HEXAGON_A2_vaddw,
    BI__builtin_HEXAGON_A2_vaddws,
    BI__builtin_HEXAGON_A2_vavgh,
    BI__builtin_HEXAGON_A2_vavghcr,
    BI__builtin_HEXAGON_A2_vavghr,
    BI__builtin_HEXAGON_A2_vavgub,
    BI__builtin_HEXAGON_A2_vavgubr,
    BI__builtin_HEXAGON_A2_vavguh,
    BI__builtin_HEXAGON_A2_vavguhr,
    BI__builtin_HEXAGON_A2_vavguw,
    BI__builtin_HEXAGON_A2_vavguwr,
    BI__builtin_HEXAGON_A2_vavgw,
    BI__builtin_HEXAGON_A2_vavgwcr,
    BI__builtin_HEXAGON_A2_vavgwr,
    BI__builtin_HEXAGON_A2_vcmpbeq,
    BI__builtin_HEXAGON_A2_vcmpbgtu,
    BI__builtin_HEXAGON_A2_vcmpheq,
    BI__builtin_HEXAGON_A2_vcmphgt,
    BI__builtin_HEXAGON_A2_vcmphgtu,
    BI__builtin_HEXAGON_A2_vcmpweq,
    BI__builtin_HEXAGON_A2_vcmpwgt,
    BI__builtin_HEXAGON_A2_vcmpwgtu,
    BI__builtin_HEXAGON_A2_vconj,
    BI__builtin_HEXAGON_A2_vmaxb,
    BI__builtin_HEXAGON_A2_vmaxh,
    BI__builtin_HEXAGON_A2_vmaxub,
    BI__builtin_HEXAGON_A2_vmaxuh,
    BI__builtin_HEXAGON_A2_vmaxuw,
    BI__builtin_HEXAGON_A2_vmaxw,
    BI__builtin_HEXAGON_A2_vminb,
    BI__builtin_HEXAGON_A2_vminh,
    BI__builtin_HEXAGON_A2_vminub,
    BI__builtin_HEXAGON_A2_vminuh,
    BI__builtin_HEXAGON_A2_vminuw,
    BI__builtin_HEXAGON_A2_vminw,
    BI__builtin_HEXAGON_A2_vnavgh,
    BI__builtin_HEXAGON_A2_vnavghcr,
    BI__builtin_HEXAGON_A2_vnavghr,
    BI__builtin_HEXAGON_A2_vnavgw,
    BI__builtin_HEXAGON_A2_vnavgwcr,
    BI__builtin_HEXAGON_A2_vnavgwr,
    BI__builtin_HEXAGON_A2_vraddub,
    BI__builtin_HEXAGON_A2_vraddub_acc,
    BI__builtin_HEXAGON_A2_vrsadub,
    BI__builtin_HEXAGON_A2_vrsadub_acc,
    BI__builtin_HEXAGON_A2_vsubb_map,
    BI__builtin_HEXAGON_A2_vsubh,
    BI__builtin_HEXAGON_A2_vsubhs,
    BI__builtin_HEXAGON_A2_vsubub,
    BI__builtin_HEXAGON_A2_vsububs,
    BI__builtin_HEXAGON_A2_vsubuhs,
    BI__builtin_HEXAGON_A2_vsubw,
    BI__builtin_HEXAGON_A2_vsubws,
    BI__builtin_HEXAGON_A2_xor,
    BI__builtin_HEXAGON_A2_xorp,
    BI__builtin_HEXAGON_A2_zxtb,
    BI__builtin_HEXAGON_A2_zxth,
    BI__builtin_HEXAGON_A4_andn,
    BI__builtin_HEXAGON_A4_andnp,
    BI__builtin_HEXAGON_A4_bitsplit,
    BI__builtin_HEXAGON_A4_bitspliti,
    BI__builtin_HEXAGON_A4_boundscheck,
    BI__builtin_HEXAGON_A4_cmpbeq,
    BI__builtin_HEXAGON_A4_cmpbeqi,
    BI__builtin_HEXAGON_A4_cmpbgt,
    BI__builtin_HEXAGON_A4_cmpbgti,
    BI__builtin_HEXAGON_A4_cmpbgtu,
    BI__builtin_HEXAGON_A4_cmpbgtui,
    BI__builtin_HEXAGON_A4_cmpheq,
    BI__builtin_HEXAGON_A4_cmpheqi,
    BI__builtin_HEXAGON_A4_cmphgt,
    BI__builtin_HEXAGON_A4_cmphgti,
    BI__builtin_HEXAGON_A4_cmphgtu,
    BI__builtin_HEXAGON_A4_cmphgtui,
    BI__builtin_HEXAGON_A4_combineir,
    BI__builtin_HEXAGON_A4_combineri,
    BI__builtin_HEXAGON_A4_cround_ri,
    BI__builtin_HEXAGON_A4_cround_rr,
    BI__builtin_HEXAGON_A4_modwrapu,
    BI__builtin_HEXAGON_A4_orn,
    BI__builtin_HEXAGON_A4_ornp,
    BI__builtin_HEXAGON_A4_rcmpeq,
    BI__builtin_HEXAGON_A4_rcmpeqi,
    BI__builtin_HEXAGON_A4_rcmpneq,
    BI__builtin_HEXAGON_A4_rcmpneqi,
    BI__builtin_HEXAGON_A4_round_ri,
    BI__builtin_HEXAGON_A4_round_ri_sat,
    BI__builtin_HEXAGON_A4_round_rr,
    BI__builtin_HEXAGON_A4_round_rr_sat,
    BI__builtin_HEXAGON_A4_tlbmatch,
    BI__builtin_HEXAGON_A4_vcmpbeq_any,
    BI__builtin_HEXAGON_A4_vcmpbeqi,
    BI__builtin_HEXAGON_A4_vcmpbgt,
    BI__builtin_HEXAGON_A4_vcmpbgti,
    BI__builtin_HEXAGON_A4_vcmpbgtui,
    BI__builtin_HEXAGON_A4_vcmpheqi,
    BI__builtin_HEXAGON_A4_vcmphgti,
    BI__builtin_HEXAGON_A4_vcmphgtui,
    BI__builtin_HEXAGON_A4_vcmpweqi,
    BI__builtin_HEXAGON_A4_vcmpwgti,
    BI__builtin_HEXAGON_A4_vcmpwgtui,
    BI__builtin_HEXAGON_A4_vrmaxh,
    BI__builtin_HEXAGON_A4_vrmaxuh,
    BI__builtin_HEXAGON_A4_vrmaxuw,
    BI__builtin_HEXAGON_A4_vrmaxw,
    BI__builtin_HEXAGON_A4_vrminh,
    BI__builtin_HEXAGON_A4_vrminuh,
    BI__builtin_HEXAGON_A4_vrminuw,
    BI__builtin_HEXAGON_A4_vrminw,
    BI__builtin_HEXAGON_A5_vaddhubs,
    BI__builtin_HEXAGON_A6_vcmpbeq_notany,
    BI__builtin_HEXAGON_A6_vminub_RdP,
    BI__builtin_HEXAGON_A7_clip,
    BI__builtin_HEXAGON_A7_croundd_ri,
    BI__builtin_HEXAGON_A7_croundd_rr,
    BI__builtin_HEXAGON_A7_vclip,
    BI__builtin_HEXAGON_C2_all8,
    BI__builtin_HEXAGON_C2_and,
    BI__builtin_HEXAGON_C2_andn,
    BI__builtin_HEXAGON_C2_any8,
    BI__builtin_HEXAGON_C2_bitsclr,
    BI__builtin_HEXAGON_C2_bitsclri,
    BI__builtin_HEXAGON_C2_bitsset,
    BI__builtin_HEXAGON_C2_cmpeq,
    BI__builtin_HEXAGON_C2_cmpeqi,
    BI__builtin_HEXAGON_C2_cmpeqp,
    BI__builtin_HEXAGON_C2_cmpgei,
    BI__builtin_HEXAGON_C2_cmpgeui,
    BI__builtin_HEXAGON_C2_cmpgt,
    BI__builtin_HEXAGON_C2_cmpgti,
    BI__builtin_HEXAGON_C2_cmpgtp,
    BI__builtin_HEXAGON_C2_cmpgtu,
    BI__builtin_HEXAGON_C2_cmpgtui,
    BI__builtin_HEXAGON_C2_cmpgtup,
    BI__builtin_HEXAGON_C2_cmplt,
    BI__builtin_HEXAGON_C2_cmpltu,
    BI__builtin_HEXAGON_C2_mask,
    BI__builtin_HEXAGON_C2_mux,
    BI__builtin_HEXAGON_C2_muxii,
    BI__builtin_HEXAGON_C2_muxir,
    BI__builtin_HEXAGON_C2_muxri,
    BI__builtin_HEXAGON_C2_not,
    BI__builtin_HEXAGON_C2_or,
    BI__builtin_HEXAGON_C2_orn,
    BI__builtin_HEXAGON_C2_pxfer_map,
    BI__builtin_HEXAGON_C2_tfrpr,
    BI__builtin_HEXAGON_C2_tfrrp,
    BI__builtin_HEXAGON_C2_vitpack,
    BI__builtin_HEXAGON_C2_vmux,
    BI__builtin_HEXAGON_C2_xor,
    BI__builtin_HEXAGON_C4_and_and,
    BI__builtin_HEXAGON_C4_and_andn,
    BI__builtin_HEXAGON_C4_and_or,
    BI__builtin_HEXAGON_C4_and_orn,
    BI__builtin_HEXAGON_C4_cmplte,
    BI__builtin_HEXAGON_C4_cmpltei,
    BI__builtin_HEXAGON_C4_cmplteu,
    BI__builtin_HEXAGON_C4_cmplteui,
    BI__builtin_HEXAGON_C4_cmpneq,
    BI__builtin_HEXAGON_C4_cmpneqi,
    BI__builtin_HEXAGON_C4_fastcorner9,
    BI__builtin_HEXAGON_C4_fastcorner9_not,
    BI__builtin_HEXAGON_C4_nbitsclr,
    BI__builtin_HEXAGON_C4_nbitsclri,
    BI__builtin_HEXAGON_C4_nbitsset,
    BI__builtin_HEXAGON_C4_or_and,
    BI__builtin_HEXAGON_C4_or_andn,
    BI__builtin_HEXAGON_C4_or_or,
    BI__builtin_HEXAGON_C4_or_orn,
    BI__builtin_HEXAGON_F2_conv_d2df,
    BI__builtin_HEXAGON_F2_conv_d2sf,
    BI__builtin_HEXAGON_F2_conv_df2d,
    BI__builtin_HEXAGON_F2_conv_df2d_chop,
    BI__builtin_HEXAGON_F2_conv_df2sf,
    BI__builtin_HEXAGON_F2_conv_df2ud,
    BI__builtin_HEXAGON_F2_conv_df2ud_chop,
    BI__builtin_HEXAGON_F2_conv_df2uw,
    BI__builtin_HEXAGON_F2_conv_df2uw_chop,
    BI__builtin_HEXAGON_F2_conv_df2w,
    BI__builtin_HEXAGON_F2_conv_df2w_chop,
    BI__builtin_HEXAGON_F2_conv_sf2d,
    BI__builtin_HEXAGON_F2_conv_sf2d_chop,
    BI__builtin_HEXAGON_F2_conv_sf2df,
    BI__builtin_HEXAGON_F2_conv_sf2ud,
    BI__builtin_HEXAGON_F2_conv_sf2ud_chop,
    BI__builtin_HEXAGON_F2_conv_sf2uw,
    BI__builtin_HEXAGON_F2_conv_sf2uw_chop,
    BI__builtin_HEXAGON_F2_conv_sf2w,
    BI__builtin_HEXAGON_F2_conv_sf2w_chop,
    BI__builtin_HEXAGON_F2_conv_ud2df,
    BI__builtin_HEXAGON_F2_conv_ud2sf,
    BI__builtin_HEXAGON_F2_conv_uw2df,
    BI__builtin_HEXAGON_F2_conv_uw2sf,
    BI__builtin_HEXAGON_F2_conv_w2df,
    BI__builtin_HEXAGON_F2_conv_w2sf,
    BI__builtin_HEXAGON_F2_dfadd,
    BI__builtin_HEXAGON_F2_dfclass,
    BI__builtin_HEXAGON_F2_dfcmpeq,
    BI__builtin_HEXAGON_F2_dfcmpge,
    BI__builtin_HEXAGON_F2_dfcmpgt,
    BI__builtin_HEXAGON_F2_dfcmpuo,
    BI__builtin_HEXAGON_F2_dfimm_n,
    BI__builtin_HEXAGON_F2_dfimm_p,
    BI__builtin_HEXAGON_F2_dfmax,
    BI__builtin_HEXAGON_F2_dfmin,
    BI__builtin_HEXAGON_F2_dfmpyfix,
    BI__builtin_HEXAGON_F2_dfmpyhh,
    BI__builtin_HEXAGON_F2_dfmpylh,
    BI__builtin_HEXAGON_F2_dfmpyll,
    BI__builtin_HEXAGON_F2_dfsub,
    BI__builtin_HEXAGON_F2_sfadd,
    BI__builtin_HEXAGON_F2_sfclass,
    BI__builtin_HEXAGON_F2_sfcmpeq,
    BI__builtin_HEXAGON_F2_sfcmpge,
    BI__builtin_HEXAGON_F2_sfcmpgt,
    BI__builtin_HEXAGON_F2_sfcmpuo,
    BI__builtin_HEXAGON_F2_sffixupd,
    BI__builtin_HEXAGON_F2_sffixupn,
    BI__builtin_HEXAGON_F2_sffixupr,
    BI__builtin_HEXAGON_F2_sffma,
    BI__builtin_HEXAGON_F2_sffma_lib,
    BI__builtin_HEXAGON_F2_sffma_sc,
    BI__builtin_HEXAGON_F2_sffms,
    BI__builtin_HEXAGON_F2_sffms_lib,
    BI__builtin_HEXAGON_F2_sfimm_n,
    BI__builtin_HEXAGON_F2_sfimm_p,
    BI__builtin_HEXAGON_F2_sfmax,
    BI__builtin_HEXAGON_F2_sfmin,
    BI__builtin_HEXAGON_F2_sfmpy,
    BI__builtin_HEXAGON_F2_sfsub,
    BI__builtin_HEXAGON_L2_loadrb_pci,
    BI__builtin_HEXAGON_L2_loadrb_pcr,
    BI__builtin_HEXAGON_L2_loadrd_pci,
    BI__builtin_HEXAGON_L2_loadrd_pcr,
    BI__builtin_HEXAGON_L2_loadrh_pci,
    BI__builtin_HEXAGON_L2_loadrh_pcr,
    BI__builtin_HEXAGON_L2_loadri_pci,
    BI__builtin_HEXAGON_L2_loadri_pcr,
    BI__builtin_HEXAGON_L2_loadrub_pci,
    BI__builtin_HEXAGON_L2_loadrub_pcr,
    BI__builtin_HEXAGON_L2_loadruh_pci,
    BI__builtin_HEXAGON_L2_loadruh_pcr,
    BI__builtin_HEXAGON_M2_acci,
    BI__builtin_HEXAGON_M2_accii,
    BI__builtin_HEXAGON_M2_cmaci_s0,
    BI__builtin_HEXAGON_M2_cmacr_s0,
    BI__builtin_HEXAGON_M2_cmacs_s0,
    BI__builtin_HEXAGON_M2_cmacs_s1,
    BI__builtin_HEXAGON_M2_cmacsc_s0,
    BI__builtin_HEXAGON_M2_cmacsc_s1,
    BI__builtin_HEXAGON_M2_cmpyi_s0,
    BI__builtin_HEXAGON_M2_cmpyr_s0,
    BI__builtin_HEXAGON_M2_cmpyrs_s0,
    BI__builtin_HEXAGON_M2_cmpyrs_s1,
    BI__builtin_HEXAGON_M2_cmpyrsc_s0,
    BI__builtin_HEXAGON_M2_cmpyrsc_s1,
    BI__builtin_HEXAGON_M2_cmpys_s0,
    BI__builtin_HEXAGON_M2_cmpys_s1,
    BI__builtin_HEXAGON_M2_cmpysc_s0,
    BI__builtin_HEXAGON_M2_cmpysc_s1,
    BI__builtin_HEXAGON_M2_cnacs_s0,
    BI__builtin_HEXAGON_M2_cnacs_s1,
    BI__builtin_HEXAGON_M2_cnacsc_s0,
    BI__builtin_HEXAGON_M2_cnacsc_s1,
    BI__builtin_HEXAGON_M2_dpmpyss_acc_s0,
    BI__builtin_HEXAGON_M2_dpmpyss_nac_s0,
    BI__builtin_HEXAGON_M2_dpmpyss_rnd_s0,
    BI__builtin_HEXAGON_M2_dpmpyss_s0,
    BI__builtin_HEXAGON_M2_dpmpyuu_acc_s0,
    BI__builtin_HEXAGON_M2_dpmpyuu_nac_s0,
    BI__builtin_HEXAGON_M2_dpmpyuu_s0,
    BI__builtin_HEXAGON_M2_hmmpyh_rs1,
    BI__builtin_HEXAGON_M2_hmmpyh_s1,
    BI__builtin_HEXAGON_M2_hmmpyl_rs1,
    BI__builtin_HEXAGON_M2_hmmpyl_s1,
    BI__builtin_HEXAGON_M2_maci,
    BI__builtin_HEXAGON_M2_macsin,
    BI__builtin_HEXAGON_M2_macsip,
    BI__builtin_HEXAGON_M2_mmachs_rs0,
    BI__builtin_HEXAGON_M2_mmachs_rs1,
    BI__builtin_HEXAGON_M2_mmachs_s0,
    BI__builtin_HEXAGON_M2_mmachs_s1,
    BI__builtin_HEXAGON_M2_mmacls_rs0,
    BI__builtin_HEXAGON_M2_mmacls_rs1,
    BI__builtin_HEXAGON_M2_mmacls_s0,
    BI__builtin_HEXAGON_M2_mmacls_s1,
    BI__builtin_HEXAGON_M2_mmacuhs_rs0,
    BI__builtin_HEXAGON_M2_mmacuhs_rs1,
    BI__builtin_HEXAGON_M2_mmacuhs_s0,
    BI__builtin_HEXAGON_M2_mmacuhs_s1,
    BI__builtin_HEXAGON_M2_mmaculs_rs0,
    BI__builtin_HEXAGON_M2_mmaculs_rs1,
    BI__builtin_HEXAGON_M2_mmaculs_s0,
    BI__builtin_HEXAGON_M2_mmaculs_s1,
    BI__builtin_HEXAGON_M2_mmpyh_rs0,
    BI__builtin_HEXAGON_M2_mmpyh_rs1,
    BI__builtin_HEXAGON_M2_mmpyh_s0,
    BI__builtin_HEXAGON_M2_mmpyh_s1,
    BI__builtin_HEXAGON_M2_mmpyl_rs0,
    BI__builtin_HEXAGON_M2_mmpyl_rs1,
    BI__builtin_HEXAGON_M2_mmpyl_s0,
    BI__builtin_HEXAGON_M2_mmpyl_s1,
    BI__builtin_HEXAGON_M2_mmpyuh_rs0,
    BI__builtin_HEXAGON_M2_mmpyuh_rs1,
    BI__builtin_HEXAGON_M2_mmpyuh_s0,
    BI__builtin_HEXAGON_M2_mmpyuh_s1,
    BI__builtin_HEXAGON_M2_mmpyul_rs0,
    BI__builtin_HEXAGON_M2_mmpyul_rs1,
    BI__builtin_HEXAGON_M2_mmpyul_s0,
    BI__builtin_HEXAGON_M2_mmpyul_s1,
    BI__builtin_HEXAGON_M2_mnaci,
    BI__builtin_HEXAGON_M2_mpy_acc_hh_s0,
    BI__builtin_HEXAGON_M2_mpy_acc_hh_s1,
    BI__builtin_HEXAGON_M2_mpy_acc_hl_s0,
    BI__builtin_HEXAGON_M2_mpy_acc_hl_s1,
    BI__builtin_HEXAGON_M2_mpy_acc_lh_s0,
    BI__builtin_HEXAGON_M2_mpy_acc_lh_s1,
    BI__builtin_HEXAGON_M2_mpy_acc_ll_s0,
    BI__builtin_HEXAGON_M2_mpy_acc_ll_s1,
    BI__builtin_HEXAGON_M2_mpy_acc_sat_hh_s0,
    BI__builtin_HEXAGON_M2_mpy_acc_sat_hh_s1,
    BI__builtin_HEXAGON_M2_mpy_acc_sat_hl_s0,
    BI__builtin_HEXAGON_M2_mpy_acc_sat_hl_s1,
    BI__builtin_HEXAGON_M2_mpy_acc_sat_lh_s0,
    BI__builtin_HEXAGON_M2_mpy_acc_sat_lh_s1,
    BI__builtin_HEXAGON_M2_mpy_acc_sat_ll_s0,
    BI__builtin_HEXAGON_M2_mpy_acc_sat_ll_s1,
    BI__builtin_HEXAGON_M2_mpy_hh_s0,
    BI__builtin_HEXAGON_M2_mpy_hh_s1,
    BI__builtin_HEXAGON_M2_mpy_hl_s0,
    BI__builtin_HEXAGON_M2_mpy_hl_s1,
    BI__builtin_HEXAGON_M2_mpy_lh_s0,
    BI__builtin_HEXAGON_M2_mpy_lh_s1,
    BI__builtin_HEXAGON_M2_mpy_ll_s0,
    BI__builtin_HEXAGON_M2_mpy_ll_s1,
    BI__builtin_HEXAGON_M2_mpy_nac_hh_s0,
    BI__builtin_HEXAGON_M2_mpy_nac_hh_s1,
    BI__builtin_HEXAGON_M2_mpy_nac_hl_s0,
    BI__builtin_HEXAGON_M2_mpy_nac_hl_s1,
    BI__builtin_HEXAGON_M2_mpy_nac_lh_s0,
    BI__builtin_HEXAGON_M2_mpy_nac_lh_s1,
    BI__builtin_HEXAGON_M2_mpy_nac_ll_s0,
    BI__builtin_HEXAGON_M2_mpy_nac_ll_s1,
    BI__builtin_HEXAGON_M2_mpy_nac_sat_hh_s0,
    BI__builtin_HEXAGON_M2_mpy_nac_sat_hh_s1,
    BI__builtin_HEXAGON_M2_mpy_nac_sat_hl_s0,
    BI__builtin_HEXAGON_M2_mpy_nac_sat_hl_s1,
    BI__builtin_HEXAGON_M2_mpy_nac_sat_lh_s0,
    BI__builtin_HEXAGON_M2_mpy_nac_sat_lh_s1,
    BI__builtin_HEXAGON_M2_mpy_nac_sat_ll_s0,
    BI__builtin_HEXAGON_M2_mpy_nac_sat_ll_s1,
    BI__builtin_HEXAGON_M2_mpy_rnd_hh_s0,
    BI__builtin_HEXAGON_M2_mpy_rnd_hh_s1,
    BI__builtin_HEXAGON_M2_mpy_rnd_hl_s0,
    BI__builtin_HEXAGON_M2_mpy_rnd_hl_s1,
    BI__builtin_HEXAGON_M2_mpy_rnd_lh_s0,
    BI__builtin_HEXAGON_M2_mpy_rnd_lh_s1,
    BI__builtin_HEXAGON_M2_mpy_rnd_ll_s0,
    BI__builtin_HEXAGON_M2_mpy_rnd_ll_s1,
    BI__builtin_HEXAGON_M2_mpy_sat_hh_s0,
    BI__builtin_HEXAGON_M2_mpy_sat_hh_s1,
    BI__builtin_HEXAGON_M2_mpy_sat_hl_s0,
    BI__builtin_HEXAGON_M2_mpy_sat_hl_s1,
    BI__builtin_HEXAGON_M2_mpy_sat_lh_s0,
    BI__builtin_HEXAGON_M2_mpy_sat_lh_s1,
    BI__builtin_HEXAGON_M2_mpy_sat_ll_s0,
    BI__builtin_HEXAGON_M2_mpy_sat_ll_s1,
    BI__builtin_HEXAGON_M2_mpy_sat_rnd_hh_s0,
    BI__builtin_HEXAGON_M2_mpy_sat_rnd_hh_s1,
    BI__builtin_HEXAGON_M2_mpy_sat_rnd_hl_s0,
    BI__builtin_HEXAGON_M2_mpy_sat_rnd_hl_s1,
    BI__builtin_HEXAGON_M2_mpy_sat_rnd_lh_s0,
    BI__builtin_HEXAGON_M2_mpy_sat_rnd_lh_s1,
    BI__builtin_HEXAGON_M2_mpy_sat_rnd_ll_s0,
    BI__builtin_HEXAGON_M2_mpy_sat_rnd_ll_s1,
    BI__builtin_HEXAGON_M2_mpy_up,
    BI__builtin_HEXAGON_M2_mpy_up_s1,
    BI__builtin_HEXAGON_M2_mpy_up_s1_sat,
    BI__builtin_HEXAGON_M2_mpyd_acc_hh_s0,
    BI__builtin_HEXAGON_M2_mpyd_acc_hh_s1,
    BI__builtin_HEXAGON_M2_mpyd_acc_hl_s0,
    BI__builtin_HEXAGON_M2_mpyd_acc_hl_s1,
    BI__builtin_HEXAGON_M2_mpyd_acc_lh_s0,
    BI__builtin_HEXAGON_M2_mpyd_acc_lh_s1,
    BI__builtin_HEXAGON_M2_mpyd_acc_ll_s0,
    BI__builtin_HEXAGON_M2_mpyd_acc_ll_s1,
    BI__builtin_HEXAGON_M2_mpyd_hh_s0,
    BI__builtin_HEXAGON_M2_mpyd_hh_s1,
    BI__builtin_HEXAGON_M2_mpyd_hl_s0,
    BI__builtin_HEXAGON_M2_mpyd_hl_s1,
    BI__builtin_HEXAGON_M2_mpyd_lh_s0,
    BI__builtin_HEXAGON_M2_mpyd_lh_s1,
    BI__builtin_HEXAGON_M2_mpyd_ll_s0,
    BI__builtin_HEXAGON_M2_mpyd_ll_s1,
    BI__builtin_HEXAGON_M2_mpyd_nac_hh_s0,
    BI__builtin_HEXAGON_M2_mpyd_nac_hh_s1,
    BI__builtin_HEXAGON_M2_mpyd_nac_hl_s0,
    BI__builtin_HEXAGON_M2_mpyd_nac_hl_s1,
    BI__builtin_HEXAGON_M2_mpyd_nac_lh_s0,
    BI__builtin_HEXAGON_M2_mpyd_nac_lh_s1,
    BI__builtin_HEXAGON_M2_mpyd_nac_ll_s0,
    BI__builtin_HEXAGON_M2_mpyd_nac_ll_s1,
    BI__builtin_HEXAGON_M2_mpyd_rnd_hh_s0,
    BI__builtin_HEXAGON_M2_mpyd_rnd_hh_s1,
    BI__builtin_HEXAGON_M2_mpyd_rnd_hl_s0,
    BI__builtin_HEXAGON_M2_mpyd_rnd_hl_s1,
    BI__builtin_HEXAGON_M2_mpyd_rnd_lh_s0,
    BI__builtin_HEXAGON_M2_mpyd_rnd_lh_s1,
    BI__builtin_HEXAGON_M2_mpyd_rnd_ll_s0,
    BI__builtin_HEXAGON_M2_mpyd_rnd_ll_s1,
    BI__builtin_HEXAGON_M2_mpyi,
    BI__builtin_HEXAGON_M2_mpysmi,
    BI__builtin_HEXAGON_M2_mpysu_up,
    BI__builtin_HEXAGON_M2_mpyu_acc_hh_s0,
    BI__builtin_HEXAGON_M2_mpyu_acc_hh_s1,
    BI__builtin_HEXAGON_M2_mpyu_acc_hl_s0,
    BI__builtin_HEXAGON_M2_mpyu_acc_hl_s1,
    BI__builtin_HEXAGON_M2_mpyu_acc_lh_s0,
    BI__builtin_HEXAGON_M2_mpyu_acc_lh_s1,
    BI__builtin_HEXAGON_M2_mpyu_acc_ll_s0,
    BI__builtin_HEXAGON_M2_mpyu_acc_ll_s1,
    BI__builtin_HEXAGON_M2_mpyu_hh_s0,
    BI__builtin_HEXAGON_M2_mpyu_hh_s1,
    BI__builtin_HEXAGON_M2_mpyu_hl_s0,
    BI__builtin_HEXAGON_M2_mpyu_hl_s1,
    BI__builtin_HEXAGON_M2_mpyu_lh_s0,
    BI__builtin_HEXAGON_M2_mpyu_lh_s1,
    BI__builtin_HEXAGON_M2_mpyu_ll_s0,
    BI__builtin_HEXAGON_M2_mpyu_ll_s1,
    BI__builtin_HEXAGON_M2_mpyu_nac_hh_s0,
    BI__builtin_HEXAGON_M2_mpyu_nac_hh_s1,
    BI__builtin_HEXAGON_M2_mpyu_nac_hl_s0,
    BI__builtin_HEXAGON_M2_mpyu_nac_hl_s1,
    BI__builtin_HEXAGON_M2_mpyu_nac_lh_s0,
    BI__builtin_HEXAGON_M2_mpyu_nac_lh_s1,
    BI__builtin_HEXAGON_M2_mpyu_nac_ll_s0,
    BI__builtin_HEXAGON_M2_mpyu_nac_ll_s1,
    BI__builtin_HEXAGON_M2_mpyu_up,
    BI__builtin_HEXAGON_M2_mpyud_acc_hh_s0,
    BI__builtin_HEXAGON_M2_mpyud_acc_hh_s1,
    BI__builtin_HEXAGON_M2_mpyud_acc_hl_s0,
    BI__builtin_HEXAGON_M2_mpyud_acc_hl_s1,
    BI__builtin_HEXAGON_M2_mpyud_acc_lh_s0,
    BI__builtin_HEXAGON_M2_mpyud_acc_lh_s1,
    BI__builtin_HEXAGON_M2_mpyud_acc_ll_s0,
    BI__builtin_HEXAGON_M2_mpyud_acc_ll_s1,
    BI__builtin_HEXAGON_M2_mpyud_hh_s0,
    BI__builtin_HEXAGON_M2_mpyud_hh_s1,
    BI__builtin_HEXAGON_M2_mpyud_hl_s0,
    BI__builtin_HEXAGON_M2_mpyud_hl_s1,
    BI__builtin_HEXAGON_M2_mpyud_lh_s0,
    BI__builtin_HEXAGON_M2_mpyud_lh_s1,
    BI__builtin_HEXAGON_M2_mpyud_ll_s0,
    BI__builtin_HEXAGON_M2_mpyud_ll_s1,
    BI__builtin_HEXAGON_M2_mpyud_nac_hh_s0,
    BI__builtin_HEXAGON_M2_mpyud_nac_hh_s1,
    BI__builtin_HEXAGON_M2_mpyud_nac_hl_s0,
    BI__builtin_HEXAGON_M2_mpyud_nac_hl_s1,
    BI__builtin_HEXAGON_M2_mpyud_nac_lh_s0,
    BI__builtin_HEXAGON_M2_mpyud_nac_lh_s1,
    BI__builtin_HEXAGON_M2_mpyud_nac_ll_s0,
    BI__builtin_HEXAGON_M2_mpyud_nac_ll_s1,
    BI__builtin_HEXAGON_M2_mpyui,
    BI__builtin_HEXAGON_M2_nacci,
    BI__builtin_HEXAGON_M2_naccii,
    BI__builtin_HEXAGON_M2_subacc,
    BI__builtin_HEXAGON_M2_vabsdiffh,
    BI__builtin_HEXAGON_M2_vabsdiffw,
    BI__builtin_HEXAGON_M2_vcmac_s0_sat_i,
    BI__builtin_HEXAGON_M2_vcmac_s0_sat_r,
    BI__builtin_HEXAGON_M2_vcmpy_s0_sat_i,
    BI__builtin_HEXAGON_M2_vcmpy_s0_sat_r,
    BI__builtin_HEXAGON_M2_vcmpy_s1_sat_i,
    BI__builtin_HEXAGON_M2_vcmpy_s1_sat_r,
    BI__builtin_HEXAGON_M2_vdmacs_s0,
    BI__builtin_HEXAGON_M2_vdmacs_s1,
    BI__builtin_HEXAGON_M2_vdmpyrs_s0,
    BI__builtin_HEXAGON_M2_vdmpyrs_s1,
    BI__builtin_HEXAGON_M2_vdmpys_s0,
    BI__builtin_HEXAGON_M2_vdmpys_s1,
    BI__builtin_HEXAGON_M2_vmac2,
    BI__builtin_HEXAGON_M2_vmac2es,
    BI__builtin_HEXAGON_M2_vmac2es_s0,
    BI__builtin_HEXAGON_M2_vmac2es_s1,
    BI__builtin_HEXAGON_M2_vmac2s_s0,
    BI__builtin_HEXAGON_M2_vmac2s_s1,
    BI__builtin_HEXAGON_M2_vmac2su_s0,
    BI__builtin_HEXAGON_M2_vmac2su_s1,
    BI__builtin_HEXAGON_M2_vmpy2es_s0,
    BI__builtin_HEXAGON_M2_vmpy2es_s1,
    BI__builtin_HEXAGON_M2_vmpy2s_s0,
    BI__builtin_HEXAGON_M2_vmpy2s_s0pack,
    BI__builtin_HEXAGON_M2_vmpy2s_s1,
    BI__builtin_HEXAGON_M2_vmpy2s_s1pack,
    BI__builtin_HEXAGON_M2_vmpy2su_s0,
    BI__builtin_HEXAGON_M2_vmpy2su_s1,
    BI__builtin_HEXAGON_M2_vraddh,
    BI__builtin_HEXAGON_M2_vradduh,
    BI__builtin_HEXAGON_M2_vrcmaci_s0,
    BI__builtin_HEXAGON_M2_vrcmaci_s0c,
    BI__builtin_HEXAGON_M2_vrcmacr_s0,
    BI__builtin_HEXAGON_M2_vrcmacr_s0c,
    BI__builtin_HEXAGON_M2_vrcmpyi_s0,
    BI__builtin_HEXAGON_M2_vrcmpyi_s0c,
    BI__builtin_HEXAGON_M2_vrcmpyr_s0,
    BI__builtin_HEXAGON_M2_vrcmpyr_s0c,
    BI__builtin_HEXAGON_M2_vrcmpys_acc_s1,
    BI__builtin_HEXAGON_M2_vrcmpys_s1,
    BI__builtin_HEXAGON_M2_vrcmpys_s1rp,
    BI__builtin_HEXAGON_M2_vrmac_s0,
    BI__builtin_HEXAGON_M2_vrmpy_s0,
    BI__builtin_HEXAGON_M2_xor_xacc,
    BI__builtin_HEXAGON_M4_and_and,
    BI__builtin_HEXAGON_M4_and_andn,
    BI__builtin_HEXAGON_M4_and_or,
    BI__builtin_HEXAGON_M4_and_xor,
    BI__builtin_HEXAGON_M4_cmpyi_wh,
    BI__builtin_HEXAGON_M4_cmpyi_whc,
    BI__builtin_HEXAGON_M4_cmpyr_wh,
    BI__builtin_HEXAGON_M4_cmpyr_whc,
    BI__builtin_HEXAGON_M4_mac_up_s1_sat,
    BI__builtin_HEXAGON_M4_mpyri_addi,
    BI__builtin_HEXAGON_M4_mpyri_addr,
    BI__builtin_HEXAGON_M4_mpyri_addr_u2,
    BI__builtin_HEXAGON_M4_mpyrr_addi,
    BI__builtin_HEXAGON_M4_mpyrr_addr,
    BI__builtin_HEXAGON_M4_nac_up_s1_sat,
    BI__builtin_HEXAGON_M4_or_and,
    BI__builtin_HEXAGON_M4_or_andn,
    BI__builtin_HEXAGON_M4_or_or,
    BI__builtin_HEXAGON_M4_or_xor,
    BI__builtin_HEXAGON_M4_pmpyw,
    BI__builtin_HEXAGON_M4_pmpyw_acc,
    BI__builtin_HEXAGON_M4_vpmpyh,
    BI__builtin_HEXAGON_M4_vpmpyh_acc,
    BI__builtin_HEXAGON_M4_vrmpyeh_acc_s0,
    BI__builtin_HEXAGON_M4_vrmpyeh_acc_s1,
    BI__builtin_HEXAGON_M4_vrmpyeh_s0,
    BI__builtin_HEXAGON_M4_vrmpyeh_s1,
    BI__builtin_HEXAGON_M4_vrmpyoh_acc_s0,
    BI__builtin_HEXAGON_M4_vrmpyoh_acc_s1,
    BI__builtin_HEXAGON_M4_vrmpyoh_s0,
    BI__builtin_HEXAGON_M4_vrmpyoh_s1,
    BI__builtin_HEXAGON_M4_xor_and,
    BI__builtin_HEXAGON_M4_xor_andn,
    BI__builtin_HEXAGON_M4_xor_or,
    BI__builtin_HEXAGON_M4_xor_xacc,
    BI__builtin_HEXAGON_M5_vdmacbsu,
    BI__builtin_HEXAGON_M5_vdmpybsu,
    BI__builtin_HEXAGON_M5_vmacbsu,
    BI__builtin_HEXAGON_M5_vmacbuu,
    BI__builtin_HEXAGON_M5_vmpybsu,
    BI__builtin_HEXAGON_M5_vmpybuu,
    BI__builtin_HEXAGON_M5_vrmacbsu,
    BI__builtin_HEXAGON_M5_vrmacbuu,
    BI__builtin_HEXAGON_M5_vrmpybsu,
    BI__builtin_HEXAGON_M5_vrmpybuu,
    BI__builtin_HEXAGON_M6_vabsdiffb,
    BI__builtin_HEXAGON_M6_vabsdiffub,
    BI__builtin_HEXAGON_M7_dcmpyiw,
    BI__builtin_HEXAGON_M7_dcmpyiw_acc,
    BI__builtin_HEXAGON_M7_dcmpyiwc,
    BI__builtin_HEXAGON_M7_dcmpyiwc_acc,
    BI__builtin_HEXAGON_M7_dcmpyrw,
    BI__builtin_HEXAGON_M7_dcmpyrw_acc,
    BI__builtin_HEXAGON_M7_dcmpyrwc,
    BI__builtin_HEXAGON_M7_dcmpyrwc_acc,
    BI__builtin_HEXAGON_M7_vdmpy,
    BI__builtin_HEXAGON_M7_vdmpy_acc,
    BI__builtin_HEXAGON_M7_wcmpyiw,
    BI__builtin_HEXAGON_M7_wcmpyiw_rnd,
    BI__builtin_HEXAGON_M7_wcmpyiwc,
    BI__builtin_HEXAGON_M7_wcmpyiwc_rnd,
    BI__builtin_HEXAGON_M7_wcmpyrw,
    BI__builtin_HEXAGON_M7_wcmpyrw_rnd,
    BI__builtin_HEXAGON_M7_wcmpyrwc,
    BI__builtin_HEXAGON_M7_wcmpyrwc_rnd,
    BI__builtin_HEXAGON_S2_addasl_rrri,
    BI__builtin_HEXAGON_S2_asl_i_p,
    BI__builtin_HEXAGON_S2_asl_i_p_acc,
    BI__builtin_HEXAGON_S2_asl_i_p_and,
    BI__builtin_HEXAGON_S2_asl_i_p_nac,
    BI__builtin_HEXAGON_S2_asl_i_p_or,
    BI__builtin_HEXAGON_S2_asl_i_p_xacc,
    BI__builtin_HEXAGON_S2_asl_i_r,
    BI__builtin_HEXAGON_S2_asl_i_r_acc,
    BI__builtin_HEXAGON_S2_asl_i_r_and,
    BI__builtin_HEXAGON_S2_asl_i_r_nac,
    BI__builtin_HEXAGON_S2_asl_i_r_or,
    BI__builtin_HEXAGON_S2_asl_i_r_sat,
    BI__builtin_HEXAGON_S2_asl_i_r_xacc,
    BI__builtin_HEXAGON_S2_asl_i_vh,
    BI__builtin_HEXAGON_S2_asl_i_vw,
    BI__builtin_HEXAGON_S2_asl_r_p,
    BI__builtin_HEXAGON_S2_asl_r_p_acc,
    BI__builtin_HEXAGON_S2_asl_r_p_and,
    BI__builtin_HEXAGON_S2_asl_r_p_nac,
    BI__builtin_HEXAGON_S2_asl_r_p_or,
    BI__builtin_HEXAGON_S2_asl_r_p_xor,
    BI__builtin_HEXAGON_S2_asl_r_r,
    BI__builtin_HEXAGON_S2_asl_r_r_acc,
    BI__builtin_HEXAGON_S2_asl_r_r_and,
    BI__builtin_HEXAGON_S2_asl_r_r_nac,
    BI__builtin_HEXAGON_S2_asl_r_r_or,
    BI__builtin_HEXAGON_S2_asl_r_r_sat,
    BI__builtin_HEXAGON_S2_asl_r_vh,
    BI__builtin_HEXAGON_S2_asl_r_vw,
    BI__builtin_HEXAGON_S2_asr_i_p,
    BI__builtin_HEXAGON_S2_asr_i_p_acc,
    BI__builtin_HEXAGON_S2_asr_i_p_and,
    BI__builtin_HEXAGON_S2_asr_i_p_nac,
    BI__builtin_HEXAGON_S2_asr_i_p_or,
    BI__builtin_HEXAGON_S2_asr_i_p_rnd,
    BI__builtin_HEXAGON_S2_asr_i_p_rnd_goodsyntax,
    BI__builtin_HEXAGON_S2_asr_i_r,
    BI__builtin_HEXAGON_S2_asr_i_r_acc,
    BI__builtin_HEXAGON_S2_asr_i_r_and,
    BI__builtin_HEXAGON_S2_asr_i_r_nac,
    BI__builtin_HEXAGON_S2_asr_i_r_or,
    BI__builtin_HEXAGON_S2_asr_i_r_rnd,
    BI__builtin_HEXAGON_S2_asr_i_r_rnd_goodsyntax,
    BI__builtin_HEXAGON_S2_asr_i_svw_trun,
    BI__builtin_HEXAGON_S2_asr_i_vh,
    BI__builtin_HEXAGON_S2_asr_i_vw,
    BI__builtin_HEXAGON_S2_asr_r_p,
    BI__builtin_HEXAGON_S2_asr_r_p_acc,
    BI__builtin_HEXAGON_S2_asr_r_p_and,
    BI__builtin_HEXAGON_S2_asr_r_p_nac,
    BI__builtin_HEXAGON_S2_asr_r_p_or,
    BI__builtin_HEXAGON_S2_asr_r_p_xor,
    BI__builtin_HEXAGON_S2_asr_r_r,
    BI__builtin_HEXAGON_S2_asr_r_r_acc,
    BI__builtin_HEXAGON_S2_asr_r_r_and,
    BI__builtin_HEXAGON_S2_asr_r_r_nac,
    BI__builtin_HEXAGON_S2_asr_r_r_or,
    BI__builtin_HEXAGON_S2_asr_r_r_sat,
    BI__builtin_HEXAGON_S2_asr_r_svw_trun,
    BI__builtin_HEXAGON_S2_asr_r_vh,
    BI__builtin_HEXAGON_S2_asr_r_vw,
    BI__builtin_HEXAGON_S2_brev,
    BI__builtin_HEXAGON_S2_brevp,
    BI__builtin_HEXAGON_S2_cl0,
    BI__builtin_HEXAGON_S2_cl0p,
    BI__builtin_HEXAGON_S2_cl1,
    BI__builtin_HEXAGON_S2_cl1p,
    BI__builtin_HEXAGON_S2_clb,
    BI__builtin_HEXAGON_S2_clbnorm,
    BI__builtin_HEXAGON_S2_clbp,
    BI__builtin_HEXAGON_S2_clrbit_i,
    BI__builtin_HEXAGON_S2_clrbit_r,
    BI__builtin_HEXAGON_S2_ct0,
    BI__builtin_HEXAGON_S2_ct0p,
    BI__builtin_HEXAGON_S2_ct1,
    BI__builtin_HEXAGON_S2_ct1p,
    BI__builtin_HEXAGON_S2_deinterleave,
    BI__builtin_HEXAGON_S2_extractu,
    BI__builtin_HEXAGON_S2_extractu_rp,
    BI__builtin_HEXAGON_S2_extractup,
    BI__builtin_HEXAGON_S2_extractup_rp,
    BI__builtin_HEXAGON_S2_insert,
    BI__builtin_HEXAGON_S2_insert_rp,
    BI__builtin_HEXAGON_S2_insertp,
    BI__builtin_HEXAGON_S2_insertp_rp,
    BI__builtin_HEXAGON_S2_interleave,
    BI__builtin_HEXAGON_S2_lfsp,
    BI__builtin_HEXAGON_S2_lsl_r_p,
    BI__builtin_HEXAGON_S2_lsl_r_p_acc,
    BI__builtin_HEXAGON_S2_lsl_r_p_and,
    BI__builtin_HEXAGON_S2_lsl_r_p_nac,
    BI__builtin_HEXAGON_S2_lsl_r_p_or,
    BI__builtin_HEXAGON_S2_lsl_r_p_xor,
    BI__builtin_HEXAGON_S2_lsl_r_r,
    BI__builtin_HEXAGON_S2_lsl_r_r_acc,
    BI__builtin_HEXAGON_S2_lsl_r_r_and,
    BI__builtin_HEXAGON_S2_lsl_r_r_nac,
    BI__builtin_HEXAGON_S2_lsl_r_r_or,
    BI__builtin_HEXAGON_S2_lsl_r_vh,
    BI__builtin_HEXAGON_S2_lsl_r_vw,
    BI__builtin_HEXAGON_S2_lsr_i_p,
    BI__builtin_HEXAGON_S2_lsr_i_p_acc,
    BI__builtin_HEXAGON_S2_lsr_i_p_and,
    BI__builtin_HEXAGON_S2_lsr_i_p_nac,
    BI__builtin_HEXAGON_S2_lsr_i_p_or,
    BI__builtin_HEXAGON_S2_lsr_i_p_xacc,
    BI__builtin_HEXAGON_S2_lsr_i_r,
    BI__builtin_HEXAGON_S2_lsr_i_r_acc,
    BI__builtin_HEXAGON_S2_lsr_i_r_and,
    BI__builtin_HEXAGON_S2_lsr_i_r_nac,
    BI__builtin_HEXAGON_S2_lsr_i_r_or,
    BI__builtin_HEXAGON_S2_lsr_i_r_xacc,
    BI__builtin_HEXAGON_S2_lsr_i_vh,
    BI__builtin_HEXAGON_S2_lsr_i_vw,
    BI__builtin_HEXAGON_S2_lsr_r_p,
    BI__builtin_HEXAGON_S2_lsr_r_p_acc,
    BI__builtin_HEXAGON_S2_lsr_r_p_and,
    BI__builtin_HEXAGON_S2_lsr_r_p_nac,
    BI__builtin_HEXAGON_S2_lsr_r_p_or,
    BI__builtin_HEXAGON_S2_lsr_r_p_xor,
    BI__builtin_HEXAGON_S2_lsr_r_r,
    BI__builtin_HEXAGON_S2_lsr_r_r_acc,
    BI__builtin_HEXAGON_S2_lsr_r_r_and,
    BI__builtin_HEXAGON_S2_lsr_r_r_nac,
    BI__builtin_HEXAGON_S2_lsr_r_r_or,
    BI__builtin_HEXAGON_S2_lsr_r_vh,
    BI__builtin_HEXAGON_S2_lsr_r_vw,
    BI__builtin_HEXAGON_S2_mask,
    BI__builtin_HEXAGON_S2_packhl,
    BI__builtin_HEXAGON_S2_parityp,
    BI__builtin_HEXAGON_S2_setbit_i,
    BI__builtin_HEXAGON_S2_setbit_r,
    BI__builtin_HEXAGON_S2_shuffeb,
    BI__builtin_HEXAGON_S2_shuffeh,
    BI__builtin_HEXAGON_S2_shuffob,
    BI__builtin_HEXAGON_S2_shuffoh,
    BI__builtin_HEXAGON_S2_storerb_pci,
    BI__builtin_HEXAGON_S2_storerb_pcr,
    BI__builtin_HEXAGON_S2_storerd_pci,
    BI__builtin_HEXAGON_S2_storerd_pcr,
    BI__builtin_HEXAGON_S2_storerf_pci,
    BI__builtin_HEXAGON_S2_storerf_pcr,
    BI__builtin_HEXAGON_S2_storerh_pci,
    BI__builtin_HEXAGON_S2_storerh_pcr,
    BI__builtin_HEXAGON_S2_storeri_pci,
    BI__builtin_HEXAGON_S2_storeri_pcr,
    BI__builtin_HEXAGON_S2_svsathb,
    BI__builtin_HEXAGON_S2_svsathub,
    BI__builtin_HEXAGON_S2_tableidxb_goodsyntax,
    BI__builtin_HEXAGON_S2_tableidxd_goodsyntax,
    BI__builtin_HEXAGON_S2_tableidxh_goodsyntax,
    BI__builtin_HEXAGON_S2_tableidxw_goodsyntax,
    BI__builtin_HEXAGON_S2_togglebit_i,
    BI__builtin_HEXAGON_S2_togglebit_r,
    BI__builtin_HEXAGON_S2_tstbit_i,
    BI__builtin_HEXAGON_S2_tstbit_r,
    BI__builtin_HEXAGON_S2_valignib,
    BI__builtin_HEXAGON_S2_valignrb,
    BI__builtin_HEXAGON_S2_vcnegh,
    BI__builtin_HEXAGON_S2_vcrotate,
    BI__builtin_HEXAGON_S2_vrcnegh,
    BI__builtin_HEXAGON_S2_vrndpackwh,
    BI__builtin_HEXAGON_S2_vrndpackwhs,
    BI__builtin_HEXAGON_S2_vsathb,
    BI__builtin_HEXAGON_S2_vsathb_nopack,
    BI__builtin_HEXAGON_S2_vsathub,
    BI__builtin_HEXAGON_S2_vsathub_nopack,
    BI__builtin_HEXAGON_S2_vsatwh,
    BI__builtin_HEXAGON_S2_vsatwh_nopack,
    BI__builtin_HEXAGON_S2_vsatwuh,
    BI__builtin_HEXAGON_S2_vsatwuh_nopack,
    BI__builtin_HEXAGON_S2_vsplatrb,
    BI__builtin_HEXAGON_S2_vsplatrh,
    BI__builtin_HEXAGON_S2_vspliceib,
    BI__builtin_HEXAGON_S2_vsplicerb,
    BI__builtin_HEXAGON_S2_vsxtbh,
    BI__builtin_HEXAGON_S2_vsxthw,
    BI__builtin_HEXAGON_S2_vtrunehb,
    BI__builtin_HEXAGON_S2_vtrunewh,
    BI__builtin_HEXAGON_S2_vtrunohb,
    BI__builtin_HEXAGON_S2_vtrunowh,
    BI__builtin_HEXAGON_S2_vzxtbh,
    BI__builtin_HEXAGON_S2_vzxthw,
    BI__builtin_HEXAGON_S4_addaddi,
    BI__builtin_HEXAGON_S4_addi_asl_ri,
    BI__builtin_HEXAGON_S4_addi_lsr_ri,
    BI__builtin_HEXAGON_S4_andi_asl_ri,
    BI__builtin_HEXAGON_S4_andi_lsr_ri,
    BI__builtin_HEXAGON_S4_clbaddi,
    BI__builtin_HEXAGON_S4_clbpaddi,
    BI__builtin_HEXAGON_S4_clbpnorm,
    BI__builtin_HEXAGON_S4_extract,
    BI__builtin_HEXAGON_S4_extract_rp,
    BI__builtin_HEXAGON_S4_extractp,
    BI__builtin_HEXAGON_S4_extractp_rp,
    BI__builtin_HEXAGON_S4_lsli,
    BI__builtin_HEXAGON_S4_ntstbit_i,
    BI__builtin_HEXAGON_S4_ntstbit_r,
    BI__builtin_HEXAGON_S4_or_andi,
    BI__builtin_HEXAGON_S4_or_andix,
    BI__builtin_HEXAGON_S4_or_ori,
    BI__builtin_HEXAGON_S4_ori_asl_ri,
    BI__builtin_HEXAGON_S4_ori_lsr_ri,
    BI__builtin_HEXAGON_S4_parity,
    BI__builtin_HEXAGON_S4_subaddi,
    BI__builtin_HEXAGON_S4_subi_asl_ri,
    BI__builtin_HEXAGON_S4_subi_lsr_ri,
    BI__builtin_HEXAGON_S4_vrcrotate,
    BI__builtin_HEXAGON_S4_vrcrotate_acc,
    BI__builtin_HEXAGON_S4_vxaddsubh,
    BI__builtin_HEXAGON_S4_vxaddsubhr,
    BI__builtin_HEXAGON_S4_vxaddsubw,
    BI__builtin_HEXAGON_S4_vxsubaddh,
    BI__builtin_HEXAGON_S4_vxsubaddhr,
    BI__builtin_HEXAGON_S4_vxsubaddw,
    BI__builtin_HEXAGON_S5_asrhub_rnd_sat_goodsyntax,
    BI__builtin_HEXAGON_S5_asrhub_sat,
    BI__builtin_HEXAGON_S5_popcountp,
    BI__builtin_HEXAGON_S5_vasrhrnd_goodsyntax,
    BI__builtin_HEXAGON_S6_rol_i_p,
    BI__builtin_HEXAGON_S6_rol_i_p_acc,
    BI__builtin_HEXAGON_S6_rol_i_p_and,
    BI__builtin_HEXAGON_S6_rol_i_p_nac,
    BI__builtin_HEXAGON_S6_rol_i_p_or,
    BI__builtin_HEXAGON_S6_rol_i_p_xacc,
    BI__builtin_HEXAGON_S6_rol_i_r,
    BI__builtin_HEXAGON_S6_rol_i_r_acc,
    BI__builtin_HEXAGON_S6_rol_i_r_and,
    BI__builtin_HEXAGON_S6_rol_i_r_nac,
    BI__builtin_HEXAGON_S6_rol_i_r_or,
    BI__builtin_HEXAGON_S6_rol_i_r_xacc,
    BI__builtin_HEXAGON_S6_vsplatrbp,
    BI__builtin_HEXAGON_S6_vtrunehb_ppp,
    BI__builtin_HEXAGON_S6_vtrunohb_ppp,
    BI__builtin_HEXAGON_V6_extractw,
    BI__builtin_HEXAGON_V6_extractw_128B,
    BI__builtin_HEXAGON_V6_get_qfext,
    BI__builtin_HEXAGON_V6_get_qfext_128B,
    BI__builtin_HEXAGON_V6_get_qfext_oracc,
    BI__builtin_HEXAGON_V6_get_qfext_oracc_128B,
    BI__builtin_HEXAGON_V6_hi,
    BI__builtin_HEXAGON_V6_hi_128B,
    BI__builtin_HEXAGON_V6_lo,
    BI__builtin_HEXAGON_V6_lo_128B,
    BI__builtin_HEXAGON_V6_lvsplatb,
    BI__builtin_HEXAGON_V6_lvsplatb_128B,
    BI__builtin_HEXAGON_V6_lvsplath,
    BI__builtin_HEXAGON_V6_lvsplath_128B,
    BI__builtin_HEXAGON_V6_lvsplatw,
    BI__builtin_HEXAGON_V6_lvsplatw_128B,
    BI__builtin_HEXAGON_V6_pred_and,
    BI__builtin_HEXAGON_V6_pred_and_128B,
    BI__builtin_HEXAGON_V6_pred_and_n,
    BI__builtin_HEXAGON_V6_pred_and_n_128B,
    BI__builtin_HEXAGON_V6_pred_not,
    BI__builtin_HEXAGON_V6_pred_not_128B,
    BI__builtin_HEXAGON_V6_pred_or,
    BI__builtin_HEXAGON_V6_pred_or_128B,
    BI__builtin_HEXAGON_V6_pred_or_n,
    BI__builtin_HEXAGON_V6_pred_or_n_128B,
    BI__builtin_HEXAGON_V6_pred_scalar2,
    BI__builtin_HEXAGON_V6_pred_scalar2_128B,
    BI__builtin_HEXAGON_V6_pred_scalar2v2,
    BI__builtin_HEXAGON_V6_pred_scalar2v2_128B,
    BI__builtin_HEXAGON_V6_pred_xor,
    BI__builtin_HEXAGON_V6_pred_xor_128B,
    BI__builtin_HEXAGON_V6_set_qfext,
    BI__builtin_HEXAGON_V6_set_qfext_128B,
    BI__builtin_HEXAGON_V6_shuffeqh,
    BI__builtin_HEXAGON_V6_shuffeqh_128B,
    BI__builtin_HEXAGON_V6_shuffeqw,
    BI__builtin_HEXAGON_V6_shuffeqw_128B,
    BI__builtin_HEXAGON_V6_v6mpyhubs10,
    BI__builtin_HEXAGON_V6_v6mpyhubs10_128B,
    BI__builtin_HEXAGON_V6_v6mpyhubs10_vxx,
    BI__builtin_HEXAGON_V6_v6mpyhubs10_vxx_128B,
    BI__builtin_HEXAGON_V6_v6mpyvubs10,
    BI__builtin_HEXAGON_V6_v6mpyvubs10_128B,
    BI__builtin_HEXAGON_V6_v6mpyvubs10_vxx,
    BI__builtin_HEXAGON_V6_v6mpyvubs10_vxx_128B,
    BI__builtin_HEXAGON_V6_vS32b_nqpred_ai,
    BI__builtin_HEXAGON_V6_vS32b_nqpred_ai_128B,
    BI__builtin_HEXAGON_V6_vS32b_nt_nqpred_ai,
    BI__builtin_HEXAGON_V6_vS32b_nt_nqpred_ai_128B,
    BI__builtin_HEXAGON_V6_vS32b_nt_qpred_ai,
    BI__builtin_HEXAGON_V6_vS32b_nt_qpred_ai_128B,
    BI__builtin_HEXAGON_V6_vS32b_qpred_ai,
    BI__builtin_HEXAGON_V6_vS32b_qpred_ai_128B,
    BI__builtin_HEXAGON_V6_vabs_f8,
    BI__builtin_HEXAGON_V6_vabs_f8_128B,
    BI__builtin_HEXAGON_V6_vabs_hf,
    BI__builtin_HEXAGON_V6_vabs_hf_128B,
    BI__builtin_HEXAGON_V6_vabs_qf16_hf,
    BI__builtin_HEXAGON_V6_vabs_qf16_hf_128B,
    BI__builtin_HEXAGON_V6_vabs_qf16_qf16,
    BI__builtin_HEXAGON_V6_vabs_qf16_qf16_128B,
    BI__builtin_HEXAGON_V6_vabs_qf32_qf32,
    BI__builtin_HEXAGON_V6_vabs_qf32_qf32_128B,
    BI__builtin_HEXAGON_V6_vabs_qf32_sf,
    BI__builtin_HEXAGON_V6_vabs_qf32_sf_128B,
    BI__builtin_HEXAGON_V6_vabs_sf,
    BI__builtin_HEXAGON_V6_vabs_sf_128B,
    BI__builtin_HEXAGON_V6_vabsb,
    BI__builtin_HEXAGON_V6_vabsb_128B,
    BI__builtin_HEXAGON_V6_vabsb_sat,
    BI__builtin_HEXAGON_V6_vabsb_sat_128B,
    BI__builtin_HEXAGON_V6_vabsdiffh,
    BI__builtin_HEXAGON_V6_vabsdiffh_128B,
    BI__builtin_HEXAGON_V6_vabsdiffub,
    BI__builtin_HEXAGON_V6_vabsdiffub_128B,
    BI__builtin_HEXAGON_V6_vabsdiffuh,
    BI__builtin_HEXAGON_V6_vabsdiffuh_128B,
    BI__builtin_HEXAGON_V6_vabsdiffw,
    BI__builtin_HEXAGON_V6_vabsdiffw_128B,
    BI__builtin_HEXAGON_V6_vabsh,
    BI__builtin_HEXAGON_V6_vabsh_128B,
    BI__builtin_HEXAGON_V6_vabsh_sat,
    BI__builtin_HEXAGON_V6_vabsh_sat_128B,
    BI__builtin_HEXAGON_V6_vabsw,
    BI__builtin_HEXAGON_V6_vabsw_128B,
    BI__builtin_HEXAGON_V6_vabsw_sat,
    BI__builtin_HEXAGON_V6_vabsw_sat_128B,
    BI__builtin_HEXAGON_V6_vadd_hf,
    BI__builtin_HEXAGON_V6_vadd_hf_128B,
    BI__builtin_HEXAGON_V6_vadd_hf_f8,
    BI__builtin_HEXAGON_V6_vadd_hf_f8_128B,
    BI__builtin_HEXAGON_V6_vadd_hf_hf,
    BI__builtin_HEXAGON_V6_vadd_hf_hf_128B,
    BI__builtin_HEXAGON_V6_vadd_qf16,
    BI__builtin_HEXAGON_V6_vadd_qf16_128B,
    BI__builtin_HEXAGON_V6_vadd_qf16_mix,
    BI__builtin_HEXAGON_V6_vadd_qf16_mix_128B,
    BI__builtin_HEXAGON_V6_vadd_qf32,
    BI__builtin_HEXAGON_V6_vadd_qf32_128B,
    BI__builtin_HEXAGON_V6_vadd_qf32_mix,
    BI__builtin_HEXAGON_V6_vadd_qf32_mix_128B,
    BI__builtin_HEXAGON_V6_vadd_sf,
    BI__builtin_HEXAGON_V6_vadd_sf_128B,
    BI__builtin_HEXAGON_V6_vadd_sf_bf,
    BI__builtin_HEXAGON_V6_vadd_sf_bf_128B,
    BI__builtin_HEXAGON_V6_vadd_sf_hf,
    BI__builtin_HEXAGON_V6_vadd_sf_hf_128B,
    BI__builtin_HEXAGON_V6_vadd_sf_sf,
    BI__builtin_HEXAGON_V6_vadd_sf_sf_128B,
    BI__builtin_HEXAGON_V6_vaddb,
    BI__builtin_HEXAGON_V6_vaddb_128B,
    BI__builtin_HEXAGON_V6_vaddb_dv,
    BI__builtin_HEXAGON_V6_vaddb_dv_128B,
    BI__builtin_HEXAGON_V6_vaddbnq,
    BI__builtin_HEXAGON_V6_vaddbnq_128B,
    BI__builtin_HEXAGON_V6_vaddbq,
    BI__builtin_HEXAGON_V6_vaddbq_128B,
    BI__builtin_HEXAGON_V6_vaddbsat,
    BI__builtin_HEXAGON_V6_vaddbsat_128B,
    BI__builtin_HEXAGON_V6_vaddbsat_dv,
    BI__builtin_HEXAGON_V6_vaddbsat_dv_128B,
    BI__builtin_HEXAGON_V6_vaddcarry,
    BI__builtin_HEXAGON_V6_vaddcarry_128B,
    BI__builtin_HEXAGON_V6_vaddcarryo,
    BI__builtin_HEXAGON_V6_vaddcarryo_128B,
    BI__builtin_HEXAGON_V6_vaddcarrysat,
    BI__builtin_HEXAGON_V6_vaddcarrysat_128B,
    BI__builtin_HEXAGON_V6_vaddclbh,
    BI__builtin_HEXAGON_V6_vaddclbh_128B,
    BI__builtin_HEXAGON_V6_vaddclbw,
    BI__builtin_HEXAGON_V6_vaddclbw_128B,
    BI__builtin_HEXAGON_V6_vaddh,
    BI__builtin_HEXAGON_V6_vaddh_128B,
    BI__builtin_HEXAGON_V6_vaddh_dv,
    BI__builtin_HEXAGON_V6_vaddh_dv_128B,
    BI__builtin_HEXAGON_V6_vaddhnq,
    BI__builtin_HEXAGON_V6_vaddhnq_128B,
    BI__builtin_HEXAGON_V6_vaddhq,
    BI__builtin_HEXAGON_V6_vaddhq_128B,
    BI__builtin_HEXAGON_V6_vaddhsat,
    BI__builtin_HEXAGON_V6_vaddhsat_128B,
    BI__builtin_HEXAGON_V6_vaddhsat_dv,
    BI__builtin_HEXAGON_V6_vaddhsat_dv_128B,
    BI__builtin_HEXAGON_V6_vaddhw,
    BI__builtin_HEXAGON_V6_vaddhw_128B,
    BI__builtin_HEXAGON_V6_vaddhw_acc,
    BI__builtin_HEXAGON_V6_vaddhw_acc_128B,
    BI__builtin_HEXAGON_V6_vaddubh,
    BI__builtin_HEXAGON_V6_vaddubh_128B,
    BI__builtin_HEXAGON_V6_vaddubh_acc,
    BI__builtin_HEXAGON_V6_vaddubh_acc_128B,
    BI__builtin_HEXAGON_V6_vaddubsat,
    BI__builtin_HEXAGON_V6_vaddubsat_128B,
    BI__builtin_HEXAGON_V6_vaddubsat_dv,
    BI__builtin_HEXAGON_V6_vaddubsat_dv_128B,
    BI__builtin_HEXAGON_V6_vaddububb_sat,
    BI__builtin_HEXAGON_V6_vaddububb_sat_128B,
    BI__builtin_HEXAGON_V6_vadduhsat,
    BI__builtin_HEXAGON_V6_vadduhsat_128B,
    BI__builtin_HEXAGON_V6_vadduhsat_dv,
    BI__builtin_HEXAGON_V6_vadduhsat_dv_128B,
    BI__builtin_HEXAGON_V6_vadduhw,
    BI__builtin_HEXAGON_V6_vadduhw_128B,
    BI__builtin_HEXAGON_V6_vadduhw_acc,
    BI__builtin_HEXAGON_V6_vadduhw_acc_128B,
    BI__builtin_HEXAGON_V6_vadduwsat,
    BI__builtin_HEXAGON_V6_vadduwsat_128B,
    BI__builtin_HEXAGON_V6_vadduwsat_dv,
    BI__builtin_HEXAGON_V6_vadduwsat_dv_128B,
    BI__builtin_HEXAGON_V6_vaddw,
    BI__builtin_HEXAGON_V6_vaddw_128B,
    BI__builtin_HEXAGON_V6_vaddw_dv,
    BI__builtin_HEXAGON_V6_vaddw_dv_128B,
    BI__builtin_HEXAGON_V6_vaddwnq,
    BI__builtin_HEXAGON_V6_vaddwnq_128B,
    BI__builtin_HEXAGON_V6_vaddwq,
    BI__builtin_HEXAGON_V6_vaddwq_128B,
    BI__builtin_HEXAGON_V6_vaddwsat,
    BI__builtin_HEXAGON_V6_vaddwsat_128B,
    BI__builtin_HEXAGON_V6_vaddwsat_dv,
    BI__builtin_HEXAGON_V6_vaddwsat_dv_128B,
    BI__builtin_HEXAGON_V6_valign4,
    BI__builtin_HEXAGON_V6_valign4_128B,
    BI__builtin_HEXAGON_V6_valignb,
    BI__builtin_HEXAGON_V6_valignb_128B,
    BI__builtin_HEXAGON_V6_valignbi,
    BI__builtin_HEXAGON_V6_valignbi_128B,
    BI__builtin_HEXAGON_V6_vand,
    BI__builtin_HEXAGON_V6_vand_128B,
    BI__builtin_HEXAGON_V6_vandnqrt,
    BI__builtin_HEXAGON_V6_vandnqrt_128B,
    BI__builtin_HEXAGON_V6_vandnqrt_acc,
    BI__builtin_HEXAGON_V6_vandnqrt_acc_128B,
    BI__builtin_HEXAGON_V6_vandqrt,
    BI__builtin_HEXAGON_V6_vandqrt_128B,
    BI__builtin_HEXAGON_V6_vandqrt_acc,
    BI__builtin_HEXAGON_V6_vandqrt_acc_128B,
    BI__builtin_HEXAGON_V6_vandvnqv,
    BI__builtin_HEXAGON_V6_vandvnqv_128B,
    BI__builtin_HEXAGON_V6_vandvqv,
    BI__builtin_HEXAGON_V6_vandvqv_128B,
    BI__builtin_HEXAGON_V6_vandvrt,
    BI__builtin_HEXAGON_V6_vandvrt_128B,
    BI__builtin_HEXAGON_V6_vandvrt_acc,
    BI__builtin_HEXAGON_V6_vandvrt_acc_128B,
    BI__builtin_HEXAGON_V6_vaslh,
    BI__builtin_HEXAGON_V6_vaslh_128B,
    BI__builtin_HEXAGON_V6_vaslh_acc,
    BI__builtin_HEXAGON_V6_vaslh_acc_128B,
    BI__builtin_HEXAGON_V6_vaslhv,
    BI__builtin_HEXAGON_V6_vaslhv_128B,
    BI__builtin_HEXAGON_V6_vaslw,
    BI__builtin_HEXAGON_V6_vaslw_128B,
    BI__builtin_HEXAGON_V6_vaslw_acc,
    BI__builtin_HEXAGON_V6_vaslw_acc_128B,
    BI__builtin_HEXAGON_V6_vaslwv,
    BI__builtin_HEXAGON_V6_vaslwv_128B,
    BI__builtin_HEXAGON_V6_vasr_into,
    BI__builtin_HEXAGON_V6_vasr_into_128B,
    BI__builtin_HEXAGON_V6_vasrh,
    BI__builtin_HEXAGON_V6_vasrh_128B,
    BI__builtin_HEXAGON_V6_vasrh_acc,
    BI__builtin_HEXAGON_V6_vasrh_acc_128B,
    BI__builtin_HEXAGON_V6_vasrhbrndsat,
    BI__builtin_HEXAGON_V6_vasrhbrndsat_128B,
    BI__builtin_HEXAGON_V6_vasrhbsat,
    BI__builtin_HEXAGON_V6_vasrhbsat_128B,
    BI__builtin_HEXAGON_V6_vasrhubrndsat,
    BI__builtin_HEXAGON_V6_vasrhubrndsat_128B,
    BI__builtin_HEXAGON_V6_vasrhubsat,
    BI__builtin_HEXAGON_V6_vasrhubsat_128B,
    BI__builtin_HEXAGON_V6_vasrhv,
    BI__builtin_HEXAGON_V6_vasrhv_128B,
    BI__builtin_HEXAGON_V6_vasruhubrndsat,
    BI__builtin_HEXAGON_V6_vasruhubrndsat_128B,
    BI__builtin_HEXAGON_V6_vasruhubsat,
    BI__builtin_HEXAGON_V6_vasruhubsat_128B,
    BI__builtin_HEXAGON_V6_vasruwuhrndsat,
    BI__builtin_HEXAGON_V6_vasruwuhrndsat_128B,
    BI__builtin_HEXAGON_V6_vasruwuhsat,
    BI__builtin_HEXAGON_V6_vasruwuhsat_128B,
    BI__builtin_HEXAGON_V6_vasrvuhubrndsat,
    BI__builtin_HEXAGON_V6_vasrvuhubrndsat_128B,
    BI__builtin_HEXAGON_V6_vasrvuhubsat,
    BI__builtin_HEXAGON_V6_vasrvuhubsat_128B,
    BI__builtin_HEXAGON_V6_vasrvwuhrndsat,
    BI__builtin_HEXAGON_V6_vasrvwuhrndsat_128B,
    BI__builtin_HEXAGON_V6_vasrvwuhsat,
    BI__builtin_HEXAGON_V6_vasrvwuhsat_128B,
    BI__builtin_HEXAGON_V6_vasrw,
    BI__builtin_HEXAGON_V6_vasrw_128B,
    BI__builtin_HEXAGON_V6_vasrw_acc,
    BI__builtin_HEXAGON_V6_vasrw_acc_128B,
    BI__builtin_HEXAGON_V6_vasrwh,
    BI__builtin_HEXAGON_V6_vasrwh_128B,
    BI__builtin_HEXAGON_V6_vasrwhrndsat,
    BI__builtin_HEXAGON_V6_vasrwhrndsat_128B,
    BI__builtin_HEXAGON_V6_vasrwhsat,
    BI__builtin_HEXAGON_V6_vasrwhsat_128B,
    BI__builtin_HEXAGON_V6_vasrwuhrndsat,
    BI__builtin_HEXAGON_V6_vasrwuhrndsat_128B,
    BI__builtin_HEXAGON_V6_vasrwuhsat,
    BI__builtin_HEXAGON_V6_vasrwuhsat_128B,
    BI__builtin_HEXAGON_V6_vasrwv,
    BI__builtin_HEXAGON_V6_vasrwv_128B,
    BI__builtin_HEXAGON_V6_vassign,
    BI__builtin_HEXAGON_V6_vassign_128B,
    BI__builtin_HEXAGON_V6_vassign_fp,
    BI__builtin_HEXAGON_V6_vassign_fp_128B,
    BI__builtin_HEXAGON_V6_vassignp,
    BI__builtin_HEXAGON_V6_vassignp_128B,
    BI__builtin_HEXAGON_V6_vavgb,
    BI__builtin_HEXAGON_V6_vavgb_128B,
    BI__builtin_HEXAGON_V6_vavgbrnd,
    BI__builtin_HEXAGON_V6_vavgbrnd_128B,
    BI__builtin_HEXAGON_V6_vavgh,
    BI__builtin_HEXAGON_V6_vavgh_128B,
    BI__builtin_HEXAGON_V6_vavghrnd,
    BI__builtin_HEXAGON_V6_vavghrnd_128B,
    BI__builtin_HEXAGON_V6_vavgub,
    BI__builtin_HEXAGON_V6_vavgub_128B,
    BI__builtin_HEXAGON_V6_vavgubrnd,
    BI__builtin_HEXAGON_V6_vavgubrnd_128B,
    BI__builtin_HEXAGON_V6_vavguh,
    BI__builtin_HEXAGON_V6_vavguh_128B,
    BI__builtin_HEXAGON_V6_vavguhrnd,
    BI__builtin_HEXAGON_V6_vavguhrnd_128B,
    BI__builtin_HEXAGON_V6_vavguw,
    BI__builtin_HEXAGON_V6_vavguw_128B,
    BI__builtin_HEXAGON_V6_vavguwrnd,
    BI__builtin_HEXAGON_V6_vavguwrnd_128B,
    BI__builtin_HEXAGON_V6_vavgw,
    BI__builtin_HEXAGON_V6_vavgw_128B,
    BI__builtin_HEXAGON_V6_vavgwrnd,
    BI__builtin_HEXAGON_V6_vavgwrnd_128B,
    BI__builtin_HEXAGON_V6_vcl0h,
    BI__builtin_HEXAGON_V6_vcl0h_128B,
    BI__builtin_HEXAGON_V6_vcl0w,
    BI__builtin_HEXAGON_V6_vcl0w_128B,
    BI__builtin_HEXAGON_V6_vcombine,
    BI__builtin_HEXAGON_V6_vcombine_128B,
    BI__builtin_HEXAGON_V6_vconv_bf_qf32,
    BI__builtin_HEXAGON_V6_vconv_bf_qf32_128B,
    BI__builtin_HEXAGON_V6_vconv_f8_qf16,
    BI__builtin_HEXAGON_V6_vconv_f8_qf16_128B,
    BI__builtin_HEXAGON_V6_vconv_h_hf,
    BI__builtin_HEXAGON_V6_vconv_h_hf_128B,
    BI__builtin_HEXAGON_V6_vconv_h_hf_rnd,
    BI__builtin_HEXAGON_V6_vconv_h_hf_rnd_128B,
    BI__builtin_HEXAGON_V6_vconv_hf_h,
    BI__builtin_HEXAGON_V6_vconv_hf_h_128B,
    BI__builtin_HEXAGON_V6_vconv_hf_qf16,
    BI__builtin_HEXAGON_V6_vconv_hf_qf16_128B,
    BI__builtin_HEXAGON_V6_vconv_hf_qf32,
    BI__builtin_HEXAGON_V6_vconv_hf_qf32_128B,
    BI__builtin_HEXAGON_V6_vconv_qf16_f8,
    BI__builtin_HEXAGON_V6_vconv_qf16_f8_128B,
    BI__builtin_HEXAGON_V6_vconv_qf16_hf,
    BI__builtin_HEXAGON_V6_vconv_qf16_hf_128B,
    BI__builtin_HEXAGON_V6_vconv_qf16_qf16,
    BI__builtin_HEXAGON_V6_vconv_qf16_qf16_128B,
    BI__builtin_HEXAGON_V6_vconv_qf32_qf32,
    BI__builtin_HEXAGON_V6_vconv_qf32_qf32_128B,
    BI__builtin_HEXAGON_V6_vconv_qf32_sf,
    BI__builtin_HEXAGON_V6_vconv_qf32_sf_128B,
    BI__builtin_HEXAGON_V6_vconv_sf_qf32,
    BI__builtin_HEXAGON_V6_vconv_sf_qf32_128B,
    BI__builtin_HEXAGON_V6_vconv_sf_w,
    BI__builtin_HEXAGON_V6_vconv_sf_w_128B,
    BI__builtin_HEXAGON_V6_vconv_w_sf,
    BI__builtin_HEXAGON_V6_vconv_w_sf_128B,
    BI__builtin_HEXAGON_V6_vcvt2_b_hf,
    BI__builtin_HEXAGON_V6_vcvt2_b_hf_128B,
    BI__builtin_HEXAGON_V6_vcvt2_hf_b,
    BI__builtin_HEXAGON_V6_vcvt2_hf_b_128B,
    BI__builtin_HEXAGON_V6_vcvt2_hf_ub,
    BI__builtin_HEXAGON_V6_vcvt2_hf_ub_128B,
    BI__builtin_HEXAGON_V6_vcvt2_ub_hf,
    BI__builtin_HEXAGON_V6_vcvt2_ub_hf_128B,
    BI__builtin_HEXAGON_V6_vcvt_b_hf,
    BI__builtin_HEXAGON_V6_vcvt_b_hf_128B,
    BI__builtin_HEXAGON_V6_vcvt_bf_sf,
    BI__builtin_HEXAGON_V6_vcvt_bf_sf_128B,
    BI__builtin_HEXAGON_V6_vcvt_f8_hf,
    BI__builtin_HEXAGON_V6_vcvt_f8_hf_128B,
    BI__builtin_HEXAGON_V6_vcvt_h_hf,
    BI__builtin_HEXAGON_V6_vcvt_h_hf_128B,
    BI__builtin_HEXAGON_V6_vcvt_hf_b,
    BI__builtin_HEXAGON_V6_vcvt_hf_b_128B,
    BI__builtin_HEXAGON_V6_vcvt_hf_f8,
    BI__builtin_HEXAGON_V6_vcvt_hf_f8_128B,
    BI__builtin_HEXAGON_V6_vcvt_hf_h,
    BI__builtin_HEXAGON_V6_vcvt_hf_h_128B,
    BI__builtin_HEXAGON_V6_vcvt_hf_sf,
    BI__builtin_HEXAGON_V6_vcvt_hf_sf_128B,
    BI__builtin_HEXAGON_V6_vcvt_hf_ub,
    BI__builtin_HEXAGON_V6_vcvt_hf_ub_128B,
    BI__builtin_HEXAGON_V6_vcvt_hf_uh,
    BI__builtin_HEXAGON_V6_vcvt_hf_uh_128B,
    BI__builtin_HEXAGON_V6_vcvt_sf_hf,
    BI__builtin_HEXAGON_V6_vcvt_sf_hf_128B,
    BI__builtin_HEXAGON_V6_vcvt_ub_hf,
    BI__builtin_HEXAGON_V6_vcvt_ub_hf_128B,
    BI__builtin_HEXAGON_V6_vcvt_uh_hf,
    BI__builtin_HEXAGON_V6_vcvt_uh_hf_128B,
    BI__builtin_HEXAGON_V6_vd0,
    BI__builtin_HEXAGON_V6_vd0_128B,
    BI__builtin_HEXAGON_V6_vdd0,
    BI__builtin_HEXAGON_V6_vdd0_128B,
    BI__builtin_HEXAGON_V6_vdealb,
    BI__builtin_HEXAGON_V6_vdealb4w,
    BI__builtin_HEXAGON_V6_vdealb4w_128B,
    BI__builtin_HEXAGON_V6_vdealb_128B,
    BI__builtin_HEXAGON_V6_vdealh,
    BI__builtin_HEXAGON_V6_vdealh_128B,
    BI__builtin_HEXAGON_V6_vdealvdd,
    BI__builtin_HEXAGON_V6_vdealvdd_128B,
    BI__builtin_HEXAGON_V6_vdelta,
    BI__builtin_HEXAGON_V6_vdelta_128B,
    BI__builtin_HEXAGON_V6_vdmpy_sf_hf,
    BI__builtin_HEXAGON_V6_vdmpy_sf_hf_128B,
    BI__builtin_HEXAGON_V6_vdmpy_sf_hf_acc,
    BI__builtin_HEXAGON_V6_vdmpy_sf_hf_acc_128B,
    BI__builtin_HEXAGON_V6_vdmpybus,
    BI__builtin_HEXAGON_V6_vdmpybus_128B,
    BI__builtin_HEXAGON_V6_vdmpybus_acc,
    BI__builtin_HEXAGON_V6_vdmpybus_acc_128B,
    BI__builtin_HEXAGON_V6_vdmpybus_dv,
    BI__builtin_HEXAGON_V6_vdmpybus_dv_128B,
    BI__builtin_HEXAGON_V6_vdmpybus_dv_acc,
    BI__builtin_HEXAGON_V6_vdmpybus_dv_acc_128B,
    BI__builtin_HEXAGON_V6_vdmpyhb,
    BI__builtin_HEXAGON_V6_vdmpyhb_128B,
    BI__builtin_HEXAGON_V6_vdmpyhb_acc,
    BI__builtin_HEXAGON_V6_vdmpyhb_acc_128B,
    BI__builtin_HEXAGON_V6_vdmpyhb_dv,
    BI__builtin_HEXAGON_V6_vdmpyhb_dv_128B,
    BI__builtin_HEXAGON_V6_vdmpyhb_dv_acc,
    BI__builtin_HEXAGON_V6_vdmpyhb_dv_acc_128B,
    BI__builtin_HEXAGON_V6_vdmpyhisat,
    BI__builtin_HEXAGON_V6_vdmpyhisat_128B,
    BI__builtin_HEXAGON_V6_vdmpyhisat_acc,
    BI__builtin_HEXAGON_V6_vdmpyhisat_acc_128B,
    BI__builtin_HEXAGON_V6_vdmpyhsat,
    BI__builtin_HEXAGON_V6_vdmpyhsat_128B,
    BI__builtin_HEXAGON_V6_vdmpyhsat_acc,
    BI__builtin_HEXAGON_V6_vdmpyhsat_acc_128B,
    BI__builtin_HEXAGON_V6_vdmpyhsuisat,
    BI__builtin_HEXAGON_V6_vdmpyhsuisat_128B,
    BI__builtin_HEXAGON_V6_vdmpyhsuisat_acc,
    BI__builtin_HEXAGON_V6_vdmpyhsuisat_acc_128B,
    BI__builtin_HEXAGON_V6_vdmpyhsusat,
    BI__builtin_HEXAGON_V6_vdmpyhsusat_128B,
    BI__builtin_HEXAGON_V6_vdmpyhsusat_acc,
    BI__builtin_HEXAGON_V6_vdmpyhsusat_acc_128B,
    BI__builtin_HEXAGON_V6_vdmpyhvsat,
    BI__builtin_HEXAGON_V6_vdmpyhvsat_128B,
    BI__builtin_HEXAGON_V6_vdmpyhvsat_acc,
    BI__builtin_HEXAGON_V6_vdmpyhvsat_acc_128B,
    BI__builtin_HEXAGON_V6_vdsaduh,
    BI__builtin_HEXAGON_V6_vdsaduh_128B,
    BI__builtin_HEXAGON_V6_vdsaduh_acc,
    BI__builtin_HEXAGON_V6_vdsaduh_acc_128B,
    BI__builtin_HEXAGON_V6_veqb,
    BI__builtin_HEXAGON_V6_veqb_128B,
    BI__builtin_HEXAGON_V6_veqb_and,
    BI__builtin_HEXAGON_V6_veqb_and_128B,
    BI__builtin_HEXAGON_V6_veqb_or,
    BI__builtin_HEXAGON_V6_veqb_or_128B,
    BI__builtin_HEXAGON_V6_veqb_xor,
    BI__builtin_HEXAGON_V6_veqb_xor_128B,
    BI__builtin_HEXAGON_V6_veqh,
    BI__builtin_HEXAGON_V6_veqh_128B,
    BI__builtin_HEXAGON_V6_veqh_and,
    BI__builtin_HEXAGON_V6_veqh_and_128B,
    BI__builtin_HEXAGON_V6_veqh_or,
    BI__builtin_HEXAGON_V6_veqh_or_128B,
    BI__builtin_HEXAGON_V6_veqh_xor,
    BI__builtin_HEXAGON_V6_veqh_xor_128B,
    BI__builtin_HEXAGON_V6_veqhf,
    BI__builtin_HEXAGON_V6_veqhf_128B,
    BI__builtin_HEXAGON_V6_veqhf_and,
    BI__builtin_HEXAGON_V6_veqhf_and_128B,
    BI__builtin_HEXAGON_V6_veqhf_or,
    BI__builtin_HEXAGON_V6_veqhf_or_128B,
    BI__builtin_HEXAGON_V6_veqhf_xor,
    BI__builtin_HEXAGON_V6_veqhf_xor_128B,
    BI__builtin_HEXAGON_V6_veqsf,
    BI__builtin_HEXAGON_V6_veqsf_128B,
    BI__builtin_HEXAGON_V6_veqsf_and,
    BI__builtin_HEXAGON_V6_veqsf_and_128B,
    BI__builtin_HEXAGON_V6_veqsf_or,
    BI__builtin_HEXAGON_V6_veqsf_or_128B,
    BI__builtin_HEXAGON_V6_veqsf_xor,
    BI__builtin_HEXAGON_V6_veqsf_xor_128B,
    BI__builtin_HEXAGON_V6_veqw,
    BI__builtin_HEXAGON_V6_veqw_128B,
    BI__builtin_HEXAGON_V6_veqw_and,
    BI__builtin_HEXAGON_V6_veqw_and_128B,
    BI__builtin_HEXAGON_V6_veqw_or,
    BI__builtin_HEXAGON_V6_veqw_or_128B,
    BI__builtin_HEXAGON_V6_veqw_xor,
    BI__builtin_HEXAGON_V6_veqw_xor_128B,
    BI__builtin_HEXAGON_V6_vfmax_f8,
    BI__builtin_HEXAGON_V6_vfmax_f8_128B,
    BI__builtin_HEXAGON_V6_vfmax_hf,
    BI__builtin_HEXAGON_V6_vfmax_hf_128B,
    BI__builtin_HEXAGON_V6_vfmax_sf,
    BI__builtin_HEXAGON_V6_vfmax_sf_128B,
    BI__builtin_HEXAGON_V6_vfmin_f8,
    BI__builtin_HEXAGON_V6_vfmin_f8_128B,
    BI__builtin_HEXAGON_V6_vfmin_hf,
    BI__builtin_HEXAGON_V6_vfmin_hf_128B,
    BI__builtin_HEXAGON_V6_vfmin_sf,
    BI__builtin_HEXAGON_V6_vfmin_sf_128B,
    BI__builtin_HEXAGON_V6_vfneg_f8,
    BI__builtin_HEXAGON_V6_vfneg_f8_128B,
    BI__builtin_HEXAGON_V6_vfneg_hf,
    BI__builtin_HEXAGON_V6_vfneg_hf_128B,
    BI__builtin_HEXAGON_V6_vfneg_sf,
    BI__builtin_HEXAGON_V6_vfneg_sf_128B,
    BI__builtin_HEXAGON_V6_vgathermh,
    BI__builtin_HEXAGON_V6_vgathermh_128B,
    BI__builtin_HEXAGON_V6_vgathermhq,
    BI__builtin_HEXAGON_V6_vgathermhq_128B,
    BI__builtin_HEXAGON_V6_vgathermhw,
    BI__builtin_HEXAGON_V6_vgathermhw_128B,
    BI__builtin_HEXAGON_V6_vgathermhwq,
    BI__builtin_HEXAGON_V6_vgathermhwq_128B,
    BI__builtin_HEXAGON_V6_vgathermw,
    BI__builtin_HEXAGON_V6_vgathermw_128B,
    BI__builtin_HEXAGON_V6_vgathermwq,
    BI__builtin_HEXAGON_V6_vgathermwq_128B,
    BI__builtin_HEXAGON_V6_vgtb,
    BI__builtin_HEXAGON_V6_vgtb_128B,
    BI__builtin_HEXAGON_V6_vgtb_and,
    BI__builtin_HEXAGON_V6_vgtb_and_128B,
    BI__builtin_HEXAGON_V6_vgtb_or,
    BI__builtin_HEXAGON_V6_vgtb_or_128B,
    BI__builtin_HEXAGON_V6_vgtb_xor,
    BI__builtin_HEXAGON_V6_vgtb_xor_128B,
    BI__builtin_HEXAGON_V6_vgtbf,
    BI__builtin_HEXAGON_V6_vgtbf_128B,
    BI__builtin_HEXAGON_V6_vgtbf_and,
    BI__builtin_HEXAGON_V6_vgtbf_and_128B,
    BI__builtin_HEXAGON_V6_vgtbf_or,
    BI__builtin_HEXAGON_V6_vgtbf_or_128B,
    BI__builtin_HEXAGON_V6_vgtbf_xor,
    BI__builtin_HEXAGON_V6_vgtbf_xor_128B,
    BI__builtin_HEXAGON_V6_vgth,
    BI__builtin_HEXAGON_V6_vgth_128B,
    BI__builtin_HEXAGON_V6_vgth_and,
    BI__builtin_HEXAGON_V6_vgth_and_128B,
    BI__builtin_HEXAGON_V6_vgth_or,
    BI__builtin_HEXAGON_V6_vgth_or_128B,
    BI__builtin_HEXAGON_V6_vgth_xor,
    BI__builtin_HEXAGON_V6_vgth_xor_128B,
    BI__builtin_HEXAGON_V6_vgthf,
    BI__builtin_HEXAGON_V6_vgthf_128B,
    BI__builtin_HEXAGON_V6_vgthf_and,
    BI__builtin_HEXAGON_V6_vgthf_and_128B,
    BI__builtin_HEXAGON_V6_vgthf_or,
    BI__builtin_HEXAGON_V6_vgthf_or_128B,
    BI__builtin_HEXAGON_V6_vgthf_xor,
    BI__builtin_HEXAGON_V6_vgthf_xor_128B,
    BI__builtin_HEXAGON_V6_vgtsf,
    BI__builtin_HEXAGON_V6_vgtsf_128B,
    BI__builtin_HEXAGON_V6_vgtsf_and,
    BI__builtin_HEXAGON_V6_vgtsf_and_128B,
    BI__builtin_HEXAGON_V6_vgtsf_or,
    BI__builtin_HEXAGON_V6_vgtsf_or_128B,
    BI__builtin_HEXAGON_V6_vgtsf_xor,
    BI__builtin_HEXAGON_V6_vgtsf_xor_128B,
    BI__builtin_HEXAGON_V6_vgtub,
    BI__builtin_HEXAGON_V6_vgtub_128B,
    BI__builtin_HEXAGON_V6_vgtub_and,
    BI__builtin_HEXAGON_V6_vgtub_and_128B,
    BI__builtin_HEXAGON_V6_vgtub_or,
    BI__builtin_HEXAGON_V6_vgtub_or_128B,
    BI__builtin_HEXAGON_V6_vgtub_xor,
    BI__builtin_HEXAGON_V6_vgtub_xor_128B,
    BI__builtin_HEXAGON_V6_vgtuh,
    BI__builtin_HEXAGON_V6_vgtuh_128B,
    BI__builtin_HEXAGON_V6_vgtuh_and,
    BI__builtin_HEXAGON_V6_vgtuh_and_128B,
    BI__builtin_HEXAGON_V6_vgtuh_or,
    BI__builtin_HEXAGON_V6_vgtuh_or_128B,
    BI__builtin_HEXAGON_V6_vgtuh_xor,
    BI__builtin_HEXAGON_V6_vgtuh_xor_128B,
    BI__builtin_HEXAGON_V6_vgtuw,
    BI__builtin_HEXAGON_V6_vgtuw_128B,
    BI__builtin_HEXAGON_V6_vgtuw_and,
    BI__builtin_HEXAGON_V6_vgtuw_and_128B,
    BI__builtin_HEXAGON_V6_vgtuw_or,
    BI__builtin_HEXAGON_V6_vgtuw_or_128B,
    BI__builtin_HEXAGON_V6_vgtuw_xor,
    BI__builtin_HEXAGON_V6_vgtuw_xor_128B,
    BI__builtin_HEXAGON_V6_vgtw,
    BI__builtin_HEXAGON_V6_vgtw_128B,
    BI__builtin_HEXAGON_V6_vgtw_and,
    BI__builtin_HEXAGON_V6_vgtw_and_128B,
    BI__builtin_HEXAGON_V6_vgtw_or,
    BI__builtin_HEXAGON_V6_vgtw_or_128B,
    BI__builtin_HEXAGON_V6_vgtw_xor,
    BI__builtin_HEXAGON_V6_vgtw_xor_128B,
    BI__builtin_HEXAGON_V6_vilog2_hf,
    BI__builtin_HEXAGON_V6_vilog2_hf_128B,
    BI__builtin_HEXAGON_V6_vilog2_qf16,
    BI__builtin_HEXAGON_V6_vilog2_qf16_128B,
    BI__builtin_HEXAGON_V6_vilog2_qf32,
    BI__builtin_HEXAGON_V6_vilog2_qf32_128B,
    BI__builtin_HEXAGON_V6_vilog2_sf,
    BI__builtin_HEXAGON_V6_vilog2_sf_128B,
    BI__builtin_HEXAGON_V6_vinsertwr,
    BI__builtin_HEXAGON_V6_vinsertwr_128B,
    BI__builtin_HEXAGON_V6_vlalignb,
    BI__builtin_HEXAGON_V6_vlalignb_128B,
    BI__builtin_HEXAGON_V6_vlalignbi,
    BI__builtin_HEXAGON_V6_vlalignbi_128B,
    BI__builtin_HEXAGON_V6_vlsrb,
    BI__builtin_HEXAGON_V6_vlsrb_128B,
    BI__builtin_HEXAGON_V6_vlsrh,
    BI__builtin_HEXAGON_V6_vlsrh_128B,
    BI__builtin_HEXAGON_V6_vlsrhv,
    BI__builtin_HEXAGON_V6_vlsrhv_128B,
    BI__builtin_HEXAGON_V6_vlsrw,
    BI__builtin_HEXAGON_V6_vlsrw_128B,
    BI__builtin_HEXAGON_V6_vlsrwv,
    BI__builtin_HEXAGON_V6_vlsrwv_128B,
    BI__builtin_HEXAGON_V6_vlut4,
    BI__builtin_HEXAGON_V6_vlut4_128B,
    BI__builtin_HEXAGON_V6_vlutvvb,
    BI__builtin_HEXAGON_V6_vlutvvb_128B,
    BI__builtin_HEXAGON_V6_vlutvvb_nm,
    BI__builtin_HEXAGON_V6_vlutvvb_nm_128B,
    BI__builtin_HEXAGON_V6_vlutvvb_oracc,
    BI__builtin_HEXAGON_V6_vlutvvb_oracc_128B,
    BI__builtin_HEXAGON_V6_vlutvvb_oracci,
    BI__builtin_HEXAGON_V6_vlutvvb_oracci_128B,
    BI__builtin_HEXAGON_V6_vlutvvbi,
    BI__builtin_HEXAGON_V6_vlutvvbi_128B,
    BI__builtin_HEXAGON_V6_vlutvwh,
    BI__builtin_HEXAGON_V6_vlutvwh_128B,
    BI__builtin_HEXAGON_V6_vlutvwh_nm,
    BI__builtin_HEXAGON_V6_vlutvwh_nm_128B,
    BI__builtin_HEXAGON_V6_vlutvwh_oracc,
    BI__builtin_HEXAGON_V6_vlutvwh_oracc_128B,
    BI__builtin_HEXAGON_V6_vlutvwh_oracci,
    BI__builtin_HEXAGON_V6_vlutvwh_oracci_128B,
    BI__builtin_HEXAGON_V6_vlutvwhi,
    BI__builtin_HEXAGON_V6_vlutvwhi_128B,
    BI__builtin_HEXAGON_V6_vmaskedstorenq,
    BI__builtin_HEXAGON_V6_vmaskedstorenq_128B,
    BI__builtin_HEXAGON_V6_vmaskedstorentnq,
    BI__builtin_HEXAGON_V6_vmaskedstorentnq_128B,
    BI__builtin_HEXAGON_V6_vmaskedstorentq,
    BI__builtin_HEXAGON_V6_vmaskedstorentq_128B,
    BI__builtin_HEXAGON_V6_vmaskedstoreq,
    BI__builtin_HEXAGON_V6_vmaskedstoreq_128B,
    BI__builtin_HEXAGON_V6_vmax_bf,
    BI__builtin_HEXAGON_V6_vmax_bf_128B,
    BI__builtin_HEXAGON_V6_vmax_hf,
    BI__builtin_HEXAGON_V6_vmax_hf_128B,
    BI__builtin_HEXAGON_V6_vmax_sf,
    BI__builtin_HEXAGON_V6_vmax_sf_128B,
    BI__builtin_HEXAGON_V6_vmaxb,
    BI__builtin_HEXAGON_V6_vmaxb_128B,
    BI__builtin_HEXAGON_V6_vmaxh,
    BI__builtin_HEXAGON_V6_vmaxh_128B,
    BI__builtin_HEXAGON_V6_vmaxub,
    BI__builtin_HEXAGON_V6_vmaxub_128B,
    BI__builtin_HEXAGON_V6_vmaxuh,
    BI__builtin_HEXAGON_V6_vmaxuh_128B,
    BI__builtin_HEXAGON_V6_vmaxw,
    BI__builtin_HEXAGON_V6_vmaxw_128B,
    BI__builtin_HEXAGON_V6_vmerge_qf,
    BI__builtin_HEXAGON_V6_vmerge_qf_128B,
    BI__builtin_HEXAGON_V6_vmin_bf,
    BI__builtin_HEXAGON_V6_vmin_bf_128B,
    BI__builtin_HEXAGON_V6_vmin_hf,
    BI__builtin_HEXAGON_V6_vmin_hf_128B,
    BI__builtin_HEXAGON_V6_vmin_sf,
    BI__builtin_HEXAGON_V6_vmin_sf_128B,
    BI__builtin_HEXAGON_V6_vminb,
    BI__builtin_HEXAGON_V6_vminb_128B,
    BI__builtin_HEXAGON_V6_vminh,
    BI__builtin_HEXAGON_V6_vminh_128B,
    BI__builtin_HEXAGON_V6_vminub,
    BI__builtin_HEXAGON_V6_vminub_128B,
    BI__builtin_HEXAGON_V6_vminuh,
    BI__builtin_HEXAGON_V6_vminuh_128B,
    BI__builtin_HEXAGON_V6_vminw,
    BI__builtin_HEXAGON_V6_vminw_128B,
    BI__builtin_HEXAGON_V6_vmpabus,
    BI__builtin_HEXAGON_V6_vmpabus_128B,
    BI__builtin_HEXAGON_V6_vmpabus_acc,
    BI__builtin_HEXAGON_V6_vmpabus_acc_128B,
    BI__builtin_HEXAGON_V6_vmpabusv,
    BI__builtin_HEXAGON_V6_vmpabusv_128B,
    BI__builtin_HEXAGON_V6_vmpabuu,
    BI__builtin_HEXAGON_V6_vmpabuu_128B,
    BI__builtin_HEXAGON_V6_vmpabuu_acc,
    BI__builtin_HEXAGON_V6_vmpabuu_acc_128B,
    BI__builtin_HEXAGON_V6_vmpabuuv,
    BI__builtin_HEXAGON_V6_vmpabuuv_128B,
    BI__builtin_HEXAGON_V6_vmpahb,
    BI__builtin_HEXAGON_V6_vmpahb_128B,
    BI__builtin_HEXAGON_V6_vmpahb_acc,
    BI__builtin_HEXAGON_V6_vmpahb_acc_128B,
    BI__builtin_HEXAGON_V6_vmpahhsat,
    BI__builtin_HEXAGON_V6_vmpahhsat_128B,
    BI__builtin_HEXAGON_V6_vmpauhb,
    BI__builtin_HEXAGON_V6_vmpauhb_128B,
    BI__builtin_HEXAGON_V6_vmpauhb_acc,
    BI__builtin_HEXAGON_V6_vmpauhb_acc_128B,
    BI__builtin_HEXAGON_V6_vmpauhuhsat,
    BI__builtin_HEXAGON_V6_vmpauhuhsat_128B,
    BI__builtin_HEXAGON_V6_vmpsuhuhsat,
    BI__builtin_HEXAGON_V6_vmpsuhuhsat_128B,
    BI__builtin_HEXAGON_V6_vmpy_hf_f8,
    BI__builtin_HEXAGON_V6_vmpy_hf_f8_128B,
    BI__builtin_HEXAGON_V6_vmpy_hf_f8_acc,
    BI__builtin_HEXAGON_V6_vmpy_hf_f8_acc_128B,
    BI__builtin_HEXAGON_V6_vmpy_hf_hf,
    BI__builtin_HEXAGON_V6_vmpy_hf_hf_128B,
    BI__builtin_HEXAGON_V6_vmpy_hf_hf_acc,
    BI__builtin_HEXAGON_V6_vmpy_hf_hf_acc_128B,
    BI__builtin_HEXAGON_V6_vmpy_qf16,
    BI__builtin_HEXAGON_V6_vmpy_qf16_128B,
    BI__builtin_HEXAGON_V6_vmpy_qf16_hf,
    BI__builtin_HEXAGON_V6_vmpy_qf16_hf_128B,
    BI__builtin_HEXAGON_V6_vmpy_qf16_mix_hf,
    BI__builtin_HEXAGON_V6_vmpy_qf16_mix_hf_128B,
    BI__builtin_HEXAGON_V6_vmpy_qf32,
    BI__builtin_HEXAGON_V6_vmpy_qf32_128B,
    BI__builtin_HEXAGON_V6_vmpy_qf32_hf,
    BI__builtin_HEXAGON_V6_vmpy_qf32_hf_128B,
    BI__builtin_HEXAGON_V6_vmpy_qf32_mix_hf,
    BI__builtin_HEXAGON_V6_vmpy_qf32_mix_hf_128B,
    BI__builtin_HEXAGON_V6_vmpy_qf32_qf16,
    BI__builtin_HEXAGON_V6_vmpy_qf32_qf16_128B,
    BI__builtin_HEXAGON_V6_vmpy_qf32_sf,
    BI__builtin_HEXAGON_V6_vmpy_qf32_sf_128B,
    BI__builtin_HEXAGON_V6_vmpy_rt_hf,
    BI__builtin_HEXAGON_V6_vmpy_rt_hf_128B,
    BI__builtin_HEXAGON_V6_vmpy_rt_qf16,
    BI__builtin_HEXAGON_V6_vmpy_rt_qf16_128B,
    BI__builtin_HEXAGON_V6_vmpy_rt_sf,
    BI__builtin_HEXAGON_V6_vmpy_rt_sf_128B,
    BI__builtin_HEXAGON_V6_vmpy_sf_bf,
    BI__builtin_HEXAGON_V6_vmpy_sf_bf_128B,
    BI__builtin_HEXAGON_V6_vmpy_sf_bf_acc,
    BI__builtin_HEXAGON_V6_vmpy_sf_bf_acc_128B,
    BI__builtin_HEXAGON_V6_vmpy_sf_hf,
    BI__builtin_HEXAGON_V6_vmpy_sf_hf_128B,
    BI__builtin_HEXAGON_V6_vmpy_sf_hf_acc,
    BI__builtin_HEXAGON_V6_vmpy_sf_hf_acc_128B,
    BI__builtin_HEXAGON_V6_vmpy_sf_sf,
    BI__builtin_HEXAGON_V6_vmpy_sf_sf_128B,
    BI__builtin_HEXAGON_V6_vmpybus,
    BI__builtin_HEXAGON_V6_vmpybus_128B,
    BI__builtin_HEXAGON_V6_vmpybus_acc,
    BI__builtin_HEXAGON_V6_vmpybus_acc_128B,
    BI__builtin_HEXAGON_V6_vmpybusv,
    BI__builtin_HEXAGON_V6_vmpybusv_128B,
    BI__builtin_HEXAGON_V6_vmpybusv_acc,
    BI__builtin_HEXAGON_V6_vmpybusv_acc_128B,
    BI__builtin_HEXAGON_V6_vmpybv,
    BI__builtin_HEXAGON_V6_vmpybv_128B,
    BI__builtin_HEXAGON_V6_vmpybv_acc,
    BI__builtin_HEXAGON_V6_vmpybv_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyewuh,
    BI__builtin_HEXAGON_V6_vmpyewuh_64,
    BI__builtin_HEXAGON_V6_vmpyewuh_64_128B,
    BI__builtin_HEXAGON_V6_vmpyewuh_128B,
    BI__builtin_HEXAGON_V6_vmpyh,
    BI__builtin_HEXAGON_V6_vmpyh_128B,
    BI__builtin_HEXAGON_V6_vmpyh_acc,
    BI__builtin_HEXAGON_V6_vmpyh_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyhsat_acc,
    BI__builtin_HEXAGON_V6_vmpyhsat_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyhsrs,
    BI__builtin_HEXAGON_V6_vmpyhsrs_128B,
    BI__builtin_HEXAGON_V6_vmpyhss,
    BI__builtin_HEXAGON_V6_vmpyhss_128B,
    BI__builtin_HEXAGON_V6_vmpyhus,
    BI__builtin_HEXAGON_V6_vmpyhus_128B,
    BI__builtin_HEXAGON_V6_vmpyhus_acc,
    BI__builtin_HEXAGON_V6_vmpyhus_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyhv,
    BI__builtin_HEXAGON_V6_vmpyhv_128B,
    BI__builtin_HEXAGON_V6_vmpyhv_acc,
    BI__builtin_HEXAGON_V6_vmpyhv_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyhvsrs,
    BI__builtin_HEXAGON_V6_vmpyhvsrs_128B,
    BI__builtin_HEXAGON_V6_vmpyieoh,
    BI__builtin_HEXAGON_V6_vmpyieoh_128B,
    BI__builtin_HEXAGON_V6_vmpyiewh_acc,
    BI__builtin_HEXAGON_V6_vmpyiewh_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyiewuh,
    BI__builtin_HEXAGON_V6_vmpyiewuh_128B,
    BI__builtin_HEXAGON_V6_vmpyiewuh_acc,
    BI__builtin_HEXAGON_V6_vmpyiewuh_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyih,
    BI__builtin_HEXAGON_V6_vmpyih_128B,
    BI__builtin_HEXAGON_V6_vmpyih_acc,
    BI__builtin_HEXAGON_V6_vmpyih_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyihb,
    BI__builtin_HEXAGON_V6_vmpyihb_128B,
    BI__builtin_HEXAGON_V6_vmpyihb_acc,
    BI__builtin_HEXAGON_V6_vmpyihb_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyiowh,
    BI__builtin_HEXAGON_V6_vmpyiowh_128B,
    BI__builtin_HEXAGON_V6_vmpyiwb,
    BI__builtin_HEXAGON_V6_vmpyiwb_128B,
    BI__builtin_HEXAGON_V6_vmpyiwb_acc,
    BI__builtin_HEXAGON_V6_vmpyiwb_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyiwh,
    BI__builtin_HEXAGON_V6_vmpyiwh_128B,
    BI__builtin_HEXAGON_V6_vmpyiwh_acc,
    BI__builtin_HEXAGON_V6_vmpyiwh_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyiwub,
    BI__builtin_HEXAGON_V6_vmpyiwub_128B,
    BI__builtin_HEXAGON_V6_vmpyiwub_acc,
    BI__builtin_HEXAGON_V6_vmpyiwub_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyowh,
    BI__builtin_HEXAGON_V6_vmpyowh_64_acc,
    BI__builtin_HEXAGON_V6_vmpyowh_64_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyowh_128B,
    BI__builtin_HEXAGON_V6_vmpyowh_rnd,
    BI__builtin_HEXAGON_V6_vmpyowh_rnd_128B,
    BI__builtin_HEXAGON_V6_vmpyowh_rnd_sacc,
    BI__builtin_HEXAGON_V6_vmpyowh_rnd_sacc_128B,
    BI__builtin_HEXAGON_V6_vmpyowh_sacc,
    BI__builtin_HEXAGON_V6_vmpyowh_sacc_128B,
    BI__builtin_HEXAGON_V6_vmpyub,
    BI__builtin_HEXAGON_V6_vmpyub_128B,
    BI__builtin_HEXAGON_V6_vmpyub_acc,
    BI__builtin_HEXAGON_V6_vmpyub_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyubv,
    BI__builtin_HEXAGON_V6_vmpyubv_128B,
    BI__builtin_HEXAGON_V6_vmpyubv_acc,
    BI__builtin_HEXAGON_V6_vmpyubv_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyuh,
    BI__builtin_HEXAGON_V6_vmpyuh_128B,
    BI__builtin_HEXAGON_V6_vmpyuh_acc,
    BI__builtin_HEXAGON_V6_vmpyuh_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyuhe,
    BI__builtin_HEXAGON_V6_vmpyuhe_128B,
    BI__builtin_HEXAGON_V6_vmpyuhe_acc,
    BI__builtin_HEXAGON_V6_vmpyuhe_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyuhv,
    BI__builtin_HEXAGON_V6_vmpyuhv_128B,
    BI__builtin_HEXAGON_V6_vmpyuhv_acc,
    BI__builtin_HEXAGON_V6_vmpyuhv_acc_128B,
    BI__builtin_HEXAGON_V6_vmpyuhvs,
    BI__builtin_HEXAGON_V6_vmpyuhvs_128B,
    BI__builtin_HEXAGON_V6_vmux,
    BI__builtin_HEXAGON_V6_vmux_128B,
    BI__builtin_HEXAGON_V6_vnavgb,
    BI__builtin_HEXAGON_V6_vnavgb_128B,
    BI__builtin_HEXAGON_V6_vnavgh,
    BI__builtin_HEXAGON_V6_vnavgh_128B,
    BI__builtin_HEXAGON_V6_vnavgub,
    BI__builtin_HEXAGON_V6_vnavgub_128B,
    BI__builtin_HEXAGON_V6_vnavgw,
    BI__builtin_HEXAGON_V6_vnavgw_128B,
    BI__builtin_HEXAGON_V6_vneg_qf16_hf,
    BI__builtin_HEXAGON_V6_vneg_qf16_hf_128B,
    BI__builtin_HEXAGON_V6_vneg_qf16_qf16,
    BI__builtin_HEXAGON_V6_vneg_qf16_qf16_128B,
    BI__builtin_HEXAGON_V6_vneg_qf32_qf32,
    BI__builtin_HEXAGON_V6_vneg_qf32_qf32_128B,
    BI__builtin_HEXAGON_V6_vneg_qf32_sf,
    BI__builtin_HEXAGON_V6_vneg_qf32_sf_128B,
    BI__builtin_HEXAGON_V6_vnormamth,
    BI__builtin_HEXAGON_V6_vnormamth_128B,
    BI__builtin_HEXAGON_V6_vnormamtw,
    BI__builtin_HEXAGON_V6_vnormamtw_128B,
    BI__builtin_HEXAGON_V6_vnot,
    BI__builtin_HEXAGON_V6_vnot_128B,
    BI__builtin_HEXAGON_V6_vor,
    BI__builtin_HEXAGON_V6_vor_128B,
    BI__builtin_HEXAGON_V6_vpackeb,
    BI__builtin_HEXAGON_V6_vpackeb_128B,
    BI__builtin_HEXAGON_V6_vpackeh,
    BI__builtin_HEXAGON_V6_vpackeh_128B,
    BI__builtin_HEXAGON_V6_vpackhb_sat,
    BI__builtin_HEXAGON_V6_vpackhb_sat_128B,
    BI__builtin_HEXAGON_V6_vpackhub_sat,
    BI__builtin_HEXAGON_V6_vpackhub_sat_128B,
    BI__builtin_HEXAGON_V6_vpackob,
    BI__builtin_HEXAGON_V6_vpackob_128B,
    BI__builtin_HEXAGON_V6_vpackoh,
    BI__builtin_HEXAGON_V6_vpackoh_128B,
    BI__builtin_HEXAGON_V6_vpackwh_sat,
    BI__builtin_HEXAGON_V6_vpackwh_sat_128B,
    BI__builtin_HEXAGON_V6_vpackwuh_sat,
    BI__builtin_HEXAGON_V6_vpackwuh_sat_128B,
    BI__builtin_HEXAGON_V6_vpopcounth,
    BI__builtin_HEXAGON_V6_vpopcounth_128B,
    BI__builtin_HEXAGON_V6_vprefixqb,
    BI__builtin_HEXAGON_V6_vprefixqb_128B,
    BI__builtin_HEXAGON_V6_vprefixqh,
    BI__builtin_HEXAGON_V6_vprefixqh_128B,
    BI__builtin_HEXAGON_V6_vprefixqw,
    BI__builtin_HEXAGON_V6_vprefixqw_128B,
    BI__builtin_HEXAGON_V6_vrdelta,
    BI__builtin_HEXAGON_V6_vrdelta_128B,
    BI__builtin_HEXAGON_V6_vrmpybub_rtt,
    BI__builtin_HEXAGON_V6_vrmpybub_rtt_128B,
    BI__builtin_HEXAGON_V6_vrmpybub_rtt_acc,
    BI__builtin_HEXAGON_V6_vrmpybub_rtt_acc_128B,
    BI__builtin_HEXAGON_V6_vrmpybus,
    BI__builtin_HEXAGON_V6_vrmpybus_128B,
    BI__builtin_HEXAGON_V6_vrmpybus_acc,
    BI__builtin_HEXAGON_V6_vrmpybus_acc_128B,
    BI__builtin_HEXAGON_V6_vrmpybusi,
    BI__builtin_HEXAGON_V6_vrmpybusi_128B,
    BI__builtin_HEXAGON_V6_vrmpybusi_acc,
    BI__builtin_HEXAGON_V6_vrmpybusi_acc_128B,
    BI__builtin_HEXAGON_V6_vrmpybusv,
    BI__builtin_HEXAGON_V6_vrmpybusv_128B,
    BI__builtin_HEXAGON_V6_vrmpybusv_acc,
    BI__builtin_HEXAGON_V6_vrmpybusv_acc_128B,
    BI__builtin_HEXAGON_V6_vrmpybv,
    BI__builtin_HEXAGON_V6_vrmpybv_128B,
    BI__builtin_HEXAGON_V6_vrmpybv_acc,
    BI__builtin_HEXAGON_V6_vrmpybv_acc_128B,
    BI__builtin_HEXAGON_V6_vrmpyub,
    BI__builtin_HEXAGON_V6_vrmpyub_128B,
    BI__builtin_HEXAGON_V6_vrmpyub_acc,
    BI__builtin_HEXAGON_V6_vrmpyub_acc_128B,
    BI__builtin_HEXAGON_V6_vrmpyub_rtt,
    BI__builtin_HEXAGON_V6_vrmpyub_rtt_128B,
    BI__builtin_HEXAGON_V6_vrmpyub_rtt_acc,
    BI__builtin_HEXAGON_V6_vrmpyub_rtt_acc_128B,
    BI__builtin_HEXAGON_V6_vrmpyubi,
    BI__builtin_HEXAGON_V6_vrmpyubi_128B,
    BI__builtin_HEXAGON_V6_vrmpyubi_acc,
    BI__builtin_HEXAGON_V6_vrmpyubi_acc_128B,
    BI__builtin_HEXAGON_V6_vrmpyubv,
    BI__builtin_HEXAGON_V6_vrmpyubv_128B,
    BI__builtin_HEXAGON_V6_vrmpyubv_acc,
    BI__builtin_HEXAGON_V6_vrmpyubv_acc_128B,
    BI__builtin_HEXAGON_V6_vror,
    BI__builtin_HEXAGON_V6_vror_128B,
    BI__builtin_HEXAGON_V6_vrotr,
    BI__builtin_HEXAGON_V6_vrotr_128B,
    BI__builtin_HEXAGON_V6_vroundhb,
    BI__builtin_HEXAGON_V6_vroundhb_128B,
    BI__builtin_HEXAGON_V6_vroundhub,
    BI__builtin_HEXAGON_V6_vroundhub_128B,
    BI__builtin_HEXAGON_V6_vrounduhub,
    BI__builtin_HEXAGON_V6_vrounduhub_128B,
    BI__builtin_HEXAGON_V6_vrounduwuh,
    BI__builtin_HEXAGON_V6_vrounduwuh_128B,
    BI__builtin_HEXAGON_V6_vroundwh,
    BI__builtin_HEXAGON_V6_vroundwh_128B,
    BI__builtin_HEXAGON_V6_vroundwuh,
    BI__builtin_HEXAGON_V6_vroundwuh_128B,
    BI__builtin_HEXAGON_V6_vrsadubi,
    BI__builtin_HEXAGON_V6_vrsadubi_128B,
    BI__builtin_HEXAGON_V6_vrsadubi_acc,
    BI__builtin_HEXAGON_V6_vrsadubi_acc_128B,
    BI__builtin_HEXAGON_V6_vsatdw,
    BI__builtin_HEXAGON_V6_vsatdw_128B,
    BI__builtin_HEXAGON_V6_vsathub,
    BI__builtin_HEXAGON_V6_vsathub_128B,
    BI__builtin_HEXAGON_V6_vsatuwuh,
    BI__builtin_HEXAGON_V6_vsatuwuh_128B,
    BI__builtin_HEXAGON_V6_vsatwh,
    BI__builtin_HEXAGON_V6_vsatwh_128B,
    BI__builtin_HEXAGON_V6_vsb,
    BI__builtin_HEXAGON_V6_vsb_128B,
    BI__builtin_HEXAGON_V6_vscattermh,
    BI__builtin_HEXAGON_V6_vscattermh_128B,
    BI__builtin_HEXAGON_V6_vscattermh_add,
    BI__builtin_HEXAGON_V6_vscattermh_add_128B,
    BI__builtin_HEXAGON_V6_vscattermhq,
    BI__builtin_HEXAGON_V6_vscattermhq_128B,
    BI__builtin_HEXAGON_V6_vscattermhw,
    BI__builtin_HEXAGON_V6_vscattermhw_128B,
    BI__builtin_HEXAGON_V6_vscattermhw_add,
    BI__builtin_HEXAGON_V6_vscattermhw_add_128B,
    BI__builtin_HEXAGON_V6_vscattermhwq,
    BI__builtin_HEXAGON_V6_vscattermhwq_128B,
    BI__builtin_HEXAGON_V6_vscattermw,
    BI__builtin_HEXAGON_V6_vscattermw_128B,
    BI__builtin_HEXAGON_V6_vscattermw_add,
    BI__builtin_HEXAGON_V6_vscattermw_add_128B,
    BI__builtin_HEXAGON_V6_vscattermwq,
    BI__builtin_HEXAGON_V6_vscattermwq_128B,
    BI__builtin_HEXAGON_V6_vsh,
    BI__builtin_HEXAGON_V6_vsh_128B,
    BI__builtin_HEXAGON_V6_vshufeh,
    BI__builtin_HEXAGON_V6_vshufeh_128B,
    BI__builtin_HEXAGON_V6_vshuffb,
    BI__builtin_HEXAGON_V6_vshuffb_128B,
    BI__builtin_HEXAGON_V6_vshuffeb,
    BI__builtin_HEXAGON_V6_vshuffeb_128B,
    BI__builtin_HEXAGON_V6_vshuffh,
    BI__builtin_HEXAGON_V6_vshuffh_128B,
    BI__builtin_HEXAGON_V6_vshuffob,
    BI__builtin_HEXAGON_V6_vshuffob_128B,
    BI__builtin_HEXAGON_V6_vshuffvdd,
    BI__builtin_HEXAGON_V6_vshuffvdd_128B,
    BI__builtin_HEXAGON_V6_vshufoeb,
    BI__builtin_HEXAGON_V6_vshufoeb_128B,
    BI__builtin_HEXAGON_V6_vshufoeh,
    BI__builtin_HEXAGON_V6_vshufoeh_128B,
    BI__builtin_HEXAGON_V6_vshufoh,
    BI__builtin_HEXAGON_V6_vshufoh_128B,
    BI__builtin_HEXAGON_V6_vsub_hf,
    BI__builtin_HEXAGON_V6_vsub_hf_128B,
    BI__builtin_HEXAGON_V6_vsub_hf_f8,
    BI__builtin_HEXAGON_V6_vsub_hf_f8_128B,
    BI__builtin_HEXAGON_V6_vsub_hf_hf,
    BI__builtin_HEXAGON_V6_vsub_hf_hf_128B,
    BI__builtin_HEXAGON_V6_vsub_hf_mix,
    BI__builtin_HEXAGON_V6_vsub_hf_mix_128B,
    BI__builtin_HEXAGON_V6_vsub_qf16,
    BI__builtin_HEXAGON_V6_vsub_qf16_128B,
    BI__builtin_HEXAGON_V6_vsub_qf16_mix,
    BI__builtin_HEXAGON_V6_vsub_qf16_mix_128B,
    BI__builtin_HEXAGON_V6_vsub_qf32,
    BI__builtin_HEXAGON_V6_vsub_qf32_128B,
    BI__builtin_HEXAGON_V6_vsub_qf32_mix,
    BI__builtin_HEXAGON_V6_vsub_qf32_mix_128B,
    BI__builtin_HEXAGON_V6_vsub_sf,
    BI__builtin_HEXAGON_V6_vsub_sf_128B,
    BI__builtin_HEXAGON_V6_vsub_sf_bf,
    BI__builtin_HEXAGON_V6_vsub_sf_bf_128B,
    BI__builtin_HEXAGON_V6_vsub_sf_hf,
    BI__builtin_HEXAGON_V6_vsub_sf_hf_128B,
    BI__builtin_HEXAGON_V6_vsub_sf_mix,
    BI__builtin_HEXAGON_V6_vsub_sf_mix_128B,
    BI__builtin_HEXAGON_V6_vsub_sf_sf,
    BI__builtin_HEXAGON_V6_vsub_sf_sf_128B,
    BI__builtin_HEXAGON_V6_vsubb,
    BI__builtin_HEXAGON_V6_vsubb_128B,
    BI__builtin_HEXAGON_V6_vsubb_dv,
    BI__builtin_HEXAGON_V6_vsubb_dv_128B,
    BI__builtin_HEXAGON_V6_vsubbnq,
    BI__builtin_HEXAGON_V6_vsubbnq_128B,
    BI__builtin_HEXAGON_V6_vsubbq,
    BI__builtin_HEXAGON_V6_vsubbq_128B,
    BI__builtin_HEXAGON_V6_vsubbsat,
    BI__builtin_HEXAGON_V6_vsubbsat_128B,
    BI__builtin_HEXAGON_V6_vsubbsat_dv,
    BI__builtin_HEXAGON_V6_vsubbsat_dv_128B,
    BI__builtin_HEXAGON_V6_vsubcarry,
    BI__builtin_HEXAGON_V6_vsubcarry_128B,
    BI__builtin_HEXAGON_V6_vsubcarryo,
    BI__builtin_HEXAGON_V6_vsubcarryo_128B,
    BI__builtin_HEXAGON_V6_vsubh,
    BI__builtin_HEXAGON_V6_vsubh_128B,
    BI__builtin_HEXAGON_V6_vsubh_dv,
    BI__builtin_HEXAGON_V6_vsubh_dv_128B,
    BI__builtin_HEXAGON_V6_vsubhnq,
    BI__builtin_HEXAGON_V6_vsubhnq_128B,
    BI__builtin_HEXAGON_V6_vsubhq,
    BI__builtin_HEXAGON_V6_vsubhq_128B,
    BI__builtin_HEXAGON_V6_vsubhsat,
    BI__builtin_HEXAGON_V6_vsubhsat_128B,
    BI__builtin_HEXAGON_V6_vsubhsat_dv,
    BI__builtin_HEXAGON_V6_vsubhsat_dv_128B,
    BI__builtin_HEXAGON_V6_vsubhw,
    BI__builtin_HEXAGON_V6_vsubhw_128B,
    BI__builtin_HEXAGON_V6_vsububh,
    BI__builtin_HEXAGON_V6_vsububh_128B,
    BI__builtin_HEXAGON_V6_vsububsat,
    BI__builtin_HEXAGON_V6_vsububsat_128B,
    BI__builtin_HEXAGON_V6_vsububsat_dv,
    BI__builtin_HEXAGON_V6_vsububsat_dv_128B,
    BI__builtin_HEXAGON_V6_vsubububb_sat,
    BI__builtin_HEXAGON_V6_vsubububb_sat_128B,
    BI__builtin_HEXAGON_V6_vsubuhsat,
    BI__builtin_HEXAGON_V6_vsubuhsat_128B,
    BI__builtin_HEXAGON_V6_vsubuhsat_dv,
    BI__builtin_HEXAGON_V6_vsubuhsat_dv_128B,
    BI__builtin_HEXAGON_V6_vsubuhw,
    BI__builtin_HEXAGON_V6_vsubuhw_128B,
    BI__builtin_HEXAGON_V6_vsubuwsat,
    BI__builtin_HEXAGON_V6_vsubuwsat_128B,
    BI__builtin_HEXAGON_V6_vsubuwsat_dv,
    BI__builtin_HEXAGON_V6_vsubuwsat_dv_128B,
    BI__builtin_HEXAGON_V6_vsubw,
    BI__builtin_HEXAGON_V6_vsubw_128B,
    BI__builtin_HEXAGON_V6_vsubw_dv,
    BI__builtin_HEXAGON_V6_vsubw_dv_128B,
    BI__builtin_HEXAGON_V6_vsubwnq,
    BI__builtin_HEXAGON_V6_vsubwnq_128B,
    BI__builtin_HEXAGON_V6_vsubwq,
    BI__builtin_HEXAGON_V6_vsubwq_128B,
    BI__builtin_HEXAGON_V6_vsubwsat,
    BI__builtin_HEXAGON_V6_vsubwsat_128B,
    BI__builtin_HEXAGON_V6_vsubwsat_dv,
    BI__builtin_HEXAGON_V6_vsubwsat_dv_128B,
    BI__builtin_HEXAGON_V6_vswap,
    BI__builtin_HEXAGON_V6_vswap_128B,
    BI__builtin_HEXAGON_V6_vtmpyb,
    BI__builtin_HEXAGON_V6_vtmpyb_128B,
    BI__builtin_HEXAGON_V6_vtmpyb_acc,
    BI__builtin_HEXAGON_V6_vtmpyb_acc_128B,
    BI__builtin_HEXAGON_V6_vtmpybus,
    BI__builtin_HEXAGON_V6_vtmpybus_128B,
    BI__builtin_HEXAGON_V6_vtmpybus_acc,
    BI__builtin_HEXAGON_V6_vtmpybus_acc_128B,
    BI__builtin_HEXAGON_V6_vtmpyhb,
    BI__builtin_HEXAGON_V6_vtmpyhb_128B,
    BI__builtin_HEXAGON_V6_vtmpyhb_acc,
    BI__builtin_HEXAGON_V6_vtmpyhb_acc_128B,
    BI__builtin_HEXAGON_V6_vunpackb,
    BI__builtin_HEXAGON_V6_vunpackb_128B,
    BI__builtin_HEXAGON_V6_vunpackh,
    BI__builtin_HEXAGON_V6_vunpackh_128B,
    BI__builtin_HEXAGON_V6_vunpackob,
    BI__builtin_HEXAGON_V6_vunpackob_128B,
    BI__builtin_HEXAGON_V6_vunpackoh,
    BI__builtin_HEXAGON_V6_vunpackoh_128B,
    BI__builtin_HEXAGON_V6_vunpackub,
    BI__builtin_HEXAGON_V6_vunpackub_128B,
    BI__builtin_HEXAGON_V6_vunpackuh,
    BI__builtin_HEXAGON_V6_vunpackuh_128B,
    BI__builtin_HEXAGON_V6_vxor,
    BI__builtin_HEXAGON_V6_vxor_128B,
    BI__builtin_HEXAGON_V6_vzb,
    BI__builtin_HEXAGON_V6_vzb_128B,
    BI__builtin_HEXAGON_V6_vzh,
    BI__builtin_HEXAGON_V6_vzh_128B,
    BI__builtin_HEXAGON_Y2_dccleana,
    BI__builtin_HEXAGON_Y2_dccleaninva,
    BI__builtin_HEXAGON_Y2_dcfetch,
    BI__builtin_HEXAGON_Y2_dcinva,
    BI__builtin_HEXAGON_Y2_dczeroa,
    BI__builtin_HEXAGON_Y4_l2fetch,
    BI__builtin_HEXAGON_Y5_l2fetch,
    BI__builtin_HEXAGON_Y6_dmlink,
    BI__builtin_HEXAGON_Y6_dmpause,
    BI__builtin_HEXAGON_Y6_dmpoll,
    BI__builtin_HEXAGON_Y6_dmresume,
    BI__builtin_HEXAGON_Y6_dmstart,
    BI__builtin_HEXAGON_Y6_dmwait,
    BI__builtin_HEXAGON_prefetch,

#endif // GET_BUILTIN_ENUMERATORS

#ifdef GET_BUILTIN_STR_TABLE

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinStringsStorage[] =
  "\0"
  "__builtin_SI_to_SXTHI_asrh\0"
  "ii\0"
  "v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81\0"
  "__builtin_brev_ldb\0"
  "v*Sc*CSc*iC\0"
  "__builtin_brev_ldd\0"
  "v*LLi*CLLi*iC\0"
  "__builtin_brev_ldh\0"
  "v*s*Cs*iC\0"
  "__builtin_brev_ldub\0"
  "v*Uc*CUc*iC\0"
  "__builtin_brev_lduh\0"
  "v*Us*CUs*iC\0"
  "__builtin_brev_ldw\0"
  "v*i*Ci*iC\0"
  "__builtin_brev_stb\0"
  "c*Cc*iiC\0"
  "__builtin_brev_std\0"
  "LLi*CLLi*LLiiC\0"
  "__builtin_brev_sth\0"
  "s*Cs*iiC\0"
  "__builtin_brev_sthhi\0"
  "__builtin_brev_stw\0"
  "i*Ci*iiC\0"
  "__builtin_circ_ldb\0"
  "c*c*c*iIi\0"
  "__builtin_circ_ldd\0"
  "LLi*LLi*LLi*iIi\0"
  "__builtin_circ_ldh\0"
  "s*s*s*iIi\0"
  "__builtin_circ_ldub\0"
  "Uc*Uc*Uc*iIi\0"
  "__builtin_circ_lduh\0"
  "Us*Us*Us*iIi\0"
  "__builtin_circ_ldw\0"
  "i*i*i*iIi\0"
  "__builtin_circ_stb\0"
  "c*c*iiIi\0"
  "__builtin_circ_std\0"
  "LLi*LLi*LLiiIi\0"
  "__builtin_circ_sth\0"
  "s*s*iiIi\0"
  "__builtin_circ_sthhi\0"
  "__builtin_circ_stw\0"
  "i*i*iiIi\0"
  "A2_abs\0"
  "A2_absp\0"
  "LLiLLi\0"
  "A2_abssat\0"
  "A2_add\0"
  "iii\0"
  "A2_addh_h16_hh\0"
  "A2_addh_h16_hl\0"
  "A2_addh_h16_lh\0"
  "A2_addh_h16_ll\0"
  "A2_addh_h16_sat_hh\0"
  "A2_addh_h16_sat_hl\0"
  "A2_addh_h16_sat_lh\0"
  "A2_addh_h16_sat_ll\0"
  "A2_addh_l16_hl\0"
  "A2_addh_l16_ll\0"
  "A2_addh_l16_sat_hl\0"
  "A2_addh_l16_sat_ll\0"
  "A2_addi\0"
  "iiIi\0"
  "A2_addp\0"
  "LLiLLiLLi\0"
  "A2_addpsat\0"
  "A2_addsat\0"
  "A2_addsp\0"
  "LLiiLLi\0"
  "A2_and\0"
  "A2_andir\0"
  "A2_andp\0"
  "A2_aslh\0"
  "A2_asrh\0"
  "A2_combine_hh\0"
  "A2_combine_hl\0"
  "A2_combine_lh\0"
  "A2_combine_ll\0"
  "A2_combineii\0"
  "LLiIiIi\0"
  "A2_combinew\0"
  "LLiii\0"
  "A2_max\0"
  "A2_maxp\0"
  "A2_maxu\0"
  "Uiii\0"
  "A2_maxup\0"
  "ULLiLLiLLi\0"
  "A2_min\0"
  "A2_minp\0"
  "A2_minu\0"
  "A2_minup\0"
  "A2_neg\0"
  "A2_negp\0"
  "A2_negsat\0"
  "A2_not\0"
  "A2_notp\0"
  "A2_or\0"
  "A2_orir\0"
  "A2_orp\0"
  "A2_roundsat\0"
  "iLLi\0"
  "A2_sat\0"
  "A2_satb\0"
  "A2_sath\0"
  "A2_satub\0"
  "A2_satuh\0"
  "A2_sub\0"
  "A2_subh_h16_hh\0"
  "A2_subh_h16_hl\0"
  "A2_subh_h16_lh\0"
  "A2_subh_h16_ll\0"
  "A2_subh_h16_sat_hh\0"
  "A2_subh_h16_sat_hl\0"
  "A2_subh_h16_sat_lh\0"
  "A2_subh_h16_sat_ll\0"
  "A2_subh_l16_hl\0"
  "A2_subh_l16_ll\0"
  "A2_subh_l16_sat_hl\0"
  "A2_subh_l16_sat_ll\0"
  "A2_subp\0"
  "A2_subri\0"
  "iIii\0"
  "A2_subsat\0"
  "A2_svaddh\0"
  "A2_svaddhs\0"
  "A2_svadduhs\0"
  "A2_svavgh\0"
  "A2_svavghs\0"
  "A2_svnavgh\0"
  "A2_svsubh\0"
  "A2_svsubhs\0"
  "A2_svsubuhs\0"
  "A2_swiz\0"
  "A2_sxtb\0"
  "A2_sxth\0"
  "A2_sxtw\0"
  "LLii\0"
  "A2_tfr\0"
  "A2_tfrih\0"
  "iiUIi\0"
  "A2_tfril\0"
  "A2_tfrp\0"
  "A2_tfrpi\0"
  "LLiIi\0"
  "A2_tfrsi\0"
  "iIi\0"
  "A2_vabsh\0"
  "A2_vabshsat\0"
  "A2_vabsw\0"
  "A2_vabswsat\0"
  "A2_vaddb_map\0"
  "A2_vaddh\0"
  "A2_vaddhs\0"
  "A2_vaddub\0"
  "A2_vaddubs\0"
  "A2_vadduhs\0"
  "A2_vaddw\0"
  "A2_vaddws\0"
  "A2_vavgh\0"
  "A2_vavghcr\0"
  "A2_vavghr\0"
  "A2_vavgub\0"
  "A2_vavgubr\0"
  "A2_vavguh\0"
  "A2_vavguhr\0"
  "A2_vavguw\0"
  "A2_vavguwr\0"
  "A2_vavgw\0"
  "A2_vavgwcr\0"
  "A2_vavgwr\0"
  "A2_vcmpbeq\0"
  "iLLiLLi\0"
  "A2_vcmpbgtu\0"
  "A2_vcmpheq\0"
  "A2_vcmphgt\0"
  "A2_vcmphgtu\0"
  "A2_vcmpweq\0"
  "A2_vcmpwgt\0"
  "A2_vcmpwgtu\0"
  "A2_vconj\0"
  "A2_vmaxb\0"
  "A2_vmaxh\0"
  "A2_vmaxub\0"
  "A2_vmaxuh\0"
  "A2_vmaxuw\0"
  "A2_vmaxw\0"
  "A2_vminb\0"
  "A2_vminh\0"
  "A2_vminub\0"
  "A2_vminuh\0"
  "A2_vminuw\0"
  "A2_vminw\0"
  "A2_vnavgh\0"
  "A2_vnavghcr\0"
  "A2_vnavghr\0"
  "A2_vnavgw\0"
  "A2_vnavgwcr\0"
  "A2_vnavgwr\0"
  "A2_vraddub\0"
  "A2_vraddub_acc\0"
  "LLiLLiLLiLLi\0"
  "A2_vrsadub\0"
  "A2_vrsadub_acc\0"
  "A2_vsubb_map\0"
  "A2_vsubh\0"
  "A2_vsubhs\0"
  "A2_vsubub\0"
  "A2_vsububs\0"
  "A2_vsubuhs\0"
  "A2_vsubw\0"
  "A2_vsubws\0"
  "A2_xor\0"
  "A2_xorp\0"
  "A2_zxtb\0"
  "A2_zxth\0"
  "A4_andn\0"
  "A4_andnp\0"
  "A4_bitsplit\0"
  "A4_bitspliti\0"
  "LLiiUIi\0"
  "A4_boundscheck\0"
  "iiLLi\0"
  "A4_cmpbeq\0"
  "A4_cmpbeqi\0"
  "A4_cmpbgt\0"
  "A4_cmpbgti\0"
  "A4_cmpbgtu\0"
  "A4_cmpbgtui\0"
  "A4_cmpheq\0"
  "A4_cmpheqi\0"
  "A4_cmphgt\0"
  "A4_cmphgti\0"
  "A4_cmphgtu\0"
  "A4_cmphgtui\0"
  "A4_combineir\0"
  "LLiIii\0"
  "A4_combineri\0"
  "LLiiIi\0"
  "A4_cround_ri\0"
  "A4_cround_rr\0"
  "A4_modwrapu\0"
  "A4_orn\0"
  "A4_ornp\0"
  "A4_rcmpeq\0"
  "A4_rcmpeqi\0"
  "A4_rcmpneq\0"
  "A4_rcmpneqi\0"
  "A4_round_ri\0"
  "A4_round_ri_sat\0"
  "A4_round_rr\0"
  "A4_round_rr_sat\0"
  "A4_tlbmatch\0"
  "iLLii\0"
  "A4_vcmpbeq_any\0"
  "A4_vcmpbeqi\0"
  "iLLiUIi\0"
  "A4_vcmpbgt\0"
  "A4_vcmpbgti\0"
  "iLLiIi\0"
  "A4_vcmpbgtui\0"
  "A4_vcmpheqi\0"
  "A4_vcmphgti\0"
  "A4_vcmphgtui\0"
  "A4_vcmpweqi\0"
  "A4_vcmpwgti\0"
  "A4_vcmpwgtui\0"
  "A4_vrmaxh\0"
  "LLiLLiLLii\0"
  "A4_vrmaxuh\0"
  "A4_vrmaxuw\0"
  "A4_vrmaxw\0"
  "A4_vrminh\0"
  "A4_vrminuh\0"
  "A4_vrminuw\0"
  "A4_vrminw\0"
  "A5_vaddhubs\0"
  "A6_vcmpbeq_notany\0"
  "v65|v66|v67|v68|v69|v71|v73|v75|v79|v81\0"
  "A6_vminub_RdP\0"
  "v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81\0"
  "A7_clip\0"
  "audio\0"
  "A7_croundd_ri\0"
  "LLiLLiUIi\0"
  "A7_croundd_rr\0"
  "LLiLLii\0"
  "A7_vclip\0"
  "C2_all8\0"
  "C2_and\0"
  "C2_andn\0"
  "C2_any8\0"
  "C2_bitsclr\0"
  "C2_bitsclri\0"
  "C2_bitsset\0"
  "C2_cmpeq\0"
  "C2_cmpeqi\0"
  "C2_cmpeqp\0"
  "C2_cmpgei\0"
  "C2_cmpgeui\0"
  "C2_cmpgt\0"
  "C2_cmpgti\0"
  "C2_cmpgtp\0"
  "C2_cmpgtu\0"
  "C2_cmpgtui\0"
  "C2_cmpgtup\0"
  "C2_cmplt\0"
  "C2_cmpltu\0"
  "C2_mask\0"
  "C2_mux\0"
  "iiii\0"
  "C2_muxii\0"
  "iiIiIi\0"
  "C2_muxir\0"
  "iiiIi\0"
  "C2_muxri\0"
  "iiIii\0"
  "C2_not\0"
  "C2_or\0"
  "C2_orn\0"
  "C2_pxfer_map\0"
  "C2_tfrpr\0"
  "C2_tfrrp\0"
  "C2_vitpack\0"
  "C2_vmux\0"
  "LLiiLLiLLi\0"
  "C2_xor\0"
  "C4_and_and\0"
  "C4_and_andn\0"
  "C4_and_or\0"
  "C4_and_orn\0"
  "C4_cmplte\0"
  "C4_cmpltei\0"
  "C4_cmplteu\0"
  "C4_cmplteui\0"
  "C4_cmpneq\0"
  "C4_cmpneqi\0"
  "C4_fastcorner9\0"
  "C4_fastcorner9_not\0"
  "C4_nbitsclr\0"
  "C4_nbitsclri\0"
  "C4_nbitsset\0"
  "C4_or_and\0"
  "C4_or_andn\0"
  "C4_or_or\0"
  "C4_or_orn\0"
  "F2_conv_d2df\0"
  "dLLi\0"
  "F2_conv_d2sf\0"
  "fLLi\0"
  "F2_conv_df2d\0"
  "LLid\0"
  "F2_conv_df2d_chop\0"
  "F2_conv_df2sf\0"
  "fd\0"
  "F2_conv_df2ud\0"
  "F2_conv_df2ud_chop\0"
  "F2_conv_df2uw\0"
  "id\0"
  "F2_conv_df2uw_chop\0"
  "F2_conv_df2w\0"
  "F2_conv_df2w_chop\0"
  "F2_conv_sf2d\0"
  "LLif\0"
  "F2_conv_sf2d_chop\0"
  "F2_conv_sf2df\0"
  "df\0"
  "F2_conv_sf2ud\0"
  "F2_conv_sf2ud_chop\0"
  "F2_conv_sf2uw\0"
  "if\0"
  "F2_conv_sf2uw_chop\0"
  "F2_conv_sf2w\0"
  "F2_conv_sf2w_chop\0"
  "F2_conv_ud2df\0"
  "F2_conv_ud2sf\0"
  "F2_conv_uw2df\0"
  "di\0"
  "F2_conv_uw2sf\0"
  "fi\0"
  "F2_conv_w2df\0"
  "F2_conv_w2sf\0"
  "F2_dfadd\0"
  "ddd\0"
  "v66|v67|v68|v69|v71|v73|v75|v79|v81\0"
  "F2_dfclass\0"
  "idUIi\0"
  "F2_dfcmpeq\0"
  "idd\0"
  "F2_dfcmpge\0"
  "F2_dfcmpgt\0"
  "F2_dfcmpuo\0"
  "F2_dfimm_n\0"
  "dUIi\0"
  "F2_dfimm_p\0"
  "F2_dfmax\0"
  "v67|v68|v69|v71|v73|v75|v79|v81\0"
  "F2_dfmin\0"
  "F2_dfmpyfix\0"
  "F2_dfmpyhh\0"
  "dddd\0"
  "F2_dfmpylh\0"
  "F2_dfmpyll\0"
  "F2_dfsub\0"
  "F2_sfadd\0"
  "fff\0"
  "F2_sfclass\0"
  "ifUIi\0"
  "F2_sfcmpeq\0"
  "iff\0"
  "F2_sfcmpge\0"
  "F2_sfcmpgt\0"
  "F2_sfcmpuo\0"
  "F2_sffixupd\0"
  "F2_sffixupn\0"
  "F2_sffixupr\0"
  "ff\0"
  "F2_sffma\0"
  "ffff\0"
  "F2_sffma_lib\0"
  "F2_sffma_sc\0"
  "ffffi\0"
  "F2_sffms\0"
  "F2_sffms_lib\0"
  "F2_sfimm_n\0"
  "fUIi\0"
  "F2_sfimm_p\0"
  "F2_sfmax\0"
  "F2_sfmin\0"
  "F2_sfmpy\0"
  "F2_sfsub\0"
  "L2_loadrb_pci\0"
  "iv*IiivC*\0"
  "L2_loadrb_pcr\0"
  "iv*ivC*\0"
  "L2_loadrd_pci\0"
  "LLiv*IiivC*\0"
  "L2_loadrd_pcr\0"
  "LLiv*ivC*\0"
  "L2_loadrh_pci\0"
  "L2_loadrh_pcr\0"
  "L2_loadri_pci\0"
  "L2_loadri_pcr\0"
  "L2_loadrub_pci\0"
  "L2_loadrub_pcr\0"
  "L2_loadruh_pci\0"
  "L2_loadruh_pcr\0"
  "M2_acci\0"
  "M2_accii\0"
  "M2_cmaci_s0\0"
  "LLiLLiii\0"
  "M2_cmacr_s0\0"
  "M2_cmacs_s0\0"
  "M2_cmacs_s1\0"
  "M2_cmacsc_s0\0"
  "M2_cmacsc_s1\0"
  "M2_cmpyi_s0\0"
  "M2_cmpyr_s0\0"
  "M2_cmpyrs_s0\0"
  "M2_cmpyrs_s1\0"
  "M2_cmpyrsc_s0\0"
  "M2_cmpyrsc_s1\0"
  "M2_cmpys_s0\0"
  "M2_cmpys_s1\0"
  "M2_cmpysc_s0\0"
  "M2_cmpysc_s1\0"
  "M2_cnacs_s0\0"
  "M2_cnacs_s1\0"
  "M2_cnacsc_s0\0"
  "M2_cnacsc_s1\0"
  "M2_dpmpyss_acc_s0\0"
  "M2_dpmpyss_nac_s0\0"
  "M2_dpmpyss_rnd_s0\0"
  "M2_dpmpyss_s0\0"
  "M2_dpmpyuu_acc_s0\0"
  "M2_dpmpyuu_nac_s0\0"
  "M2_dpmpyuu_s0\0"
  "ULLiii\0"
  "M2_hmmpyh_rs1\0"
  "M2_hmmpyh_s1\0"
  "M2_hmmpyl_rs1\0"
  "M2_hmmpyl_s1\0"
  "M2_maci\0"
  "M2_macsin\0"
  "iiiUIi\0"
  "M2_macsip\0"
  "M2_mmachs_rs0\0"
  "M2_mmachs_rs1\0"
  "M2_mmachs_s0\0"
  "M2_mmachs_s1\0"
  "M2_mmacls_rs0\0"
  "M2_mmacls_rs1\0"
  "M2_mmacls_s0\0"
  "M2_mmacls_s1\0"
  "M2_mmacuhs_rs0\0"
  "M2_mmacuhs_rs1\0"
  "M2_mmacuhs_s0\0"
  "M2_mmacuhs_s1\0"
  "M2_mmaculs_rs0\0"
  "M2_mmaculs_rs1\0"
  "M2_mmaculs_s0\0"
  "M2_mmaculs_s1\0"
  "M2_mmpyh_rs0\0"
  "M2_mmpyh_rs1\0"
  "M2_mmpyh_s0\0"
  "M2_mmpyh_s1\0"
  "M2_mmpyl_rs0\0"
  "M2_mmpyl_rs1\0"
  "M2_mmpyl_s0\0"
  "M2_mmpyl_s1\0"
  "M2_mmpyuh_rs0\0"
  "M2_mmpyuh_rs1\0"
  "M2_mmpyuh_s0\0"
  "M2_mmpyuh_s1\0"
  "M2_mmpyul_rs0\0"
  "M2_mmpyul_rs1\0"
  "M2_mmpyul_s0\0"
  "M2_mmpyul_s1\0"
  "M2_mnaci\0"
  "M2_mpy_acc_hh_s0\0"
  "M2_mpy_acc_hh_s1\0"
  "M2_mpy_acc_hl_s0\0"
  "M2_mpy_acc_hl_s1\0"
  "M2_mpy_acc_lh_s0\0"
  "M2_mpy_acc_lh_s1\0"
  "M2_mpy_acc_ll_s0\0"
  "M2_mpy_acc_ll_s1\0"
  "M2_mpy_acc_sat_hh_s0\0"
  "M2_mpy_acc_sat_hh_s1\0"
  "M2_mpy_acc_sat_hl_s0\0"
  "M2_mpy_acc_sat_hl_s1\0"
  "M2_mpy_acc_sat_lh_s0\0"
  "M2_mpy_acc_sat_lh_s1\0"
  "M2_mpy_acc_sat_ll_s0\0"
  "M2_mpy_acc_sat_ll_s1\0"
  "M2_mpy_hh_s0\0"
  "M2_mpy_hh_s1\0"
  "M2_mpy_hl_s0\0"
  "M2_mpy_hl_s1\0"
  "M2_mpy_lh_s0\0"
  "M2_mpy_lh_s1\0"
  "M2_mpy_ll_s0\0"
  "M2_mpy_ll_s1\0"
  "M2_mpy_nac_hh_s0\0"
  "M2_mpy_nac_hh_s1\0"
  "M2_mpy_nac_hl_s0\0"
  "M2_mpy_nac_hl_s1\0"
  "M2_mpy_nac_lh_s0\0"
  "M2_mpy_nac_lh_s1\0"
  "M2_mpy_nac_ll_s0\0"
  "M2_mpy_nac_ll_s1\0"
  "M2_mpy_nac_sat_hh_s0\0"
  "M2_mpy_nac_sat_hh_s1\0"
  "M2_mpy_nac_sat_hl_s0\0"
  "M2_mpy_nac_sat_hl_s1\0"
  "M2_mpy_nac_sat_lh_s0\0"
  "M2_mpy_nac_sat_lh_s1\0"
  "M2_mpy_nac_sat_ll_s0\0"
  "M2_mpy_nac_sat_ll_s1\0"
  "M2_mpy_rnd_hh_s0\0"
  "M2_mpy_rnd_hh_s1\0"
  "M2_mpy_rnd_hl_s0\0"
  "M2_mpy_rnd_hl_s1\0"
  "M2_mpy_rnd_lh_s0\0"
  "M2_mpy_rnd_lh_s1\0"
  "M2_mpy_rnd_ll_s0\0"
  "M2_mpy_rnd_ll_s1\0"
  "M2_mpy_sat_hh_s0\0"
  "M2_mpy_sat_hh_s1\0"
  "M2_mpy_sat_hl_s0\0"
  "M2_mpy_sat_hl_s1\0"
  "M2_mpy_sat_lh_s0\0"
  "M2_mpy_sat_lh_s1\0"
  "M2_mpy_sat_ll_s0\0"
  "M2_mpy_sat_ll_s1\0"
  "M2_mpy_sat_rnd_hh_s0\0"
  "M2_mpy_sat_rnd_hh_s1\0"
  "M2_mpy_sat_rnd_hl_s0\0"
  "M2_mpy_sat_rnd_hl_s1\0"
  "M2_mpy_sat_rnd_lh_s0\0"
  "M2_mpy_sat_rnd_lh_s1\0"
  "M2_mpy_sat_rnd_ll_s0\0"
  "M2_mpy_sat_rnd_ll_s1\0"
  "M2_mpy_up\0"
  "M2_mpy_up_s1\0"
  "M2_mpy_up_s1_sat\0"
  "M2_mpyd_acc_hh_s0\0"
  "M2_mpyd_acc_hh_s1\0"
  "M2_mpyd_acc_hl_s0\0"
  "M2_mpyd_acc_hl_s1\0"
  "M2_mpyd_acc_lh_s0\0"
  "M2_mpyd_acc_lh_s1\0"
  "M2_mpyd_acc_ll_s0\0"
  "M2_mpyd_acc_ll_s1\0"
  "M2_mpyd_hh_s0\0"
  "M2_mpyd_hh_s1\0"
  "M2_mpyd_hl_s0\0"
  "M2_mpyd_hl_s1\0"
  "M2_mpyd_lh_s0\0"
  "M2_mpyd_lh_s1\0"
  "M2_mpyd_ll_s0\0"
  "M2_mpyd_ll_s1\0"
  "M2_mpyd_nac_hh_s0\0"
  "M2_mpyd_nac_hh_s1\0"
  "M2_mpyd_nac_hl_s0\0"
  "M2_mpyd_nac_hl_s1\0"
  "M2_mpyd_nac_lh_s0\0"
  "M2_mpyd_nac_lh_s1\0"
  "M2_mpyd_nac_ll_s0\0"
  "M2_mpyd_nac_ll_s1\0"
  "M2_mpyd_rnd_hh_s0\0"
  "M2_mpyd_rnd_hh_s1\0"
  "M2_mpyd_rnd_hl_s0\0"
  "M2_mpyd_rnd_hl_s1\0"
  "M2_mpyd_rnd_lh_s0\0"
  "M2_mpyd_rnd_lh_s1\0"
  "M2_mpyd_rnd_ll_s0\0"
  "M2_mpyd_rnd_ll_s1\0"
  "M2_mpyi\0"
  "M2_mpysmi\0"
  "M2_mpysu_up\0"
  "M2_mpyu_acc_hh_s0\0"
  "M2_mpyu_acc_hh_s1\0"
  "M2_mpyu_acc_hl_s0\0"
  "M2_mpyu_acc_hl_s1\0"
  "M2_mpyu_acc_lh_s0\0"
  "M2_mpyu_acc_lh_s1\0"
  "M2_mpyu_acc_ll_s0\0"
  "M2_mpyu_acc_ll_s1\0"
  "M2_mpyu_hh_s0\0"
  "M2_mpyu_hh_s1\0"
  "M2_mpyu_hl_s0\0"
  "M2_mpyu_hl_s1\0"
  "M2_mpyu_lh_s0\0"
  "M2_mpyu_lh_s1\0"
  "M2_mpyu_ll_s0\0"
  "M2_mpyu_ll_s1\0"
  "M2_mpyu_nac_hh_s0\0"
  "M2_mpyu_nac_hh_s1\0"
  "M2_mpyu_nac_hl_s0\0"
  "M2_mpyu_nac_hl_s1\0"
  "M2_mpyu_nac_lh_s0\0"
  "M2_mpyu_nac_lh_s1\0"
  "M2_mpyu_nac_ll_s0\0"
  "M2_mpyu_nac_ll_s1\0"
  "M2_mpyu_up\0"
  "M2_mpyud_acc_hh_s0\0"
  "M2_mpyud_acc_hh_s1\0"
  "M2_mpyud_acc_hl_s0\0"
  "M2_mpyud_acc_hl_s1\0"
  "M2_mpyud_acc_lh_s0\0"
  "M2_mpyud_acc_lh_s1\0"
  "M2_mpyud_acc_ll_s0\0"
  "M2_mpyud_acc_ll_s1\0"
  "M2_mpyud_hh_s0\0"
  "M2_mpyud_hh_s1\0"
  "M2_mpyud_hl_s0\0"
  "M2_mpyud_hl_s1\0"
  "M2_mpyud_lh_s0\0"
  "M2_mpyud_lh_s1\0"
  "M2_mpyud_ll_s0\0"
  "M2_mpyud_ll_s1\0"
  "M2_mpyud_nac_hh_s0\0"
  "M2_mpyud_nac_hh_s1\0"
  "M2_mpyud_nac_hl_s0\0"
  "M2_mpyud_nac_hl_s1\0"
  "M2_mpyud_nac_lh_s0\0"
  "M2_mpyud_nac_lh_s1\0"
  "M2_mpyud_nac_ll_s0\0"
  "M2_mpyud_nac_ll_s1\0"
  "M2_mpyui\0"
  "M2_nacci\0"
  "M2_naccii\0"
  "M2_subacc\0"
  "M2_vabsdiffh\0"
  "M2_vabsdiffw\0"
  "M2_vcmac_s0_sat_i\0"
  "M2_vcmac_s0_sat_r\0"
  "M2_vcmpy_s0_sat_i\0"
  "M2_vcmpy_s0_sat_r\0"
  "M2_vcmpy_s1_sat_i\0"
  "M2_vcmpy_s1_sat_r\0"
  "M2_vdmacs_s0\0"
  "M2_vdmacs_s1\0"
  "M2_vdmpyrs_s0\0"
  "M2_vdmpyrs_s1\0"
  "M2_vdmpys_s0\0"
  "M2_vdmpys_s1\0"
  "M2_vmac2\0"
  "M2_vmac2es\0"
  "M2_vmac2es_s0\0"
  "M2_vmac2es_s1\0"
  "M2_vmac2s_s0\0"
  "M2_vmac2s_s1\0"
  "M2_vmac2su_s0\0"
  "M2_vmac2su_s1\0"
  "M2_vmpy2es_s0\0"
  "M2_vmpy2es_s1\0"
  "M2_vmpy2s_s0\0"
  "M2_vmpy2s_s0pack\0"
  "M2_vmpy2s_s1\0"
  "M2_vmpy2s_s1pack\0"
  "M2_vmpy2su_s0\0"
  "M2_vmpy2su_s1\0"
  "M2_vraddh\0"
  "M2_vradduh\0"
  "M2_vrcmaci_s0\0"
  "M2_vrcmaci_s0c\0"
  "M2_vrcmacr_s0\0"
  "M2_vrcmacr_s0c\0"
  "M2_vrcmpyi_s0\0"
  "M2_vrcmpyi_s0c\0"
  "M2_vrcmpyr_s0\0"
  "M2_vrcmpyr_s0c\0"
  "M2_vrcmpys_acc_s1\0"
  "M2_vrcmpys_s1\0"
  "M2_vrcmpys_s1rp\0"
  "M2_vrmac_s0\0"
  "M2_vrmpy_s0\0"
  "M2_xor_xacc\0"
  "M4_and_and\0"
  "M4_and_andn\0"
  "M4_and_or\0"
  "M4_and_xor\0"
  "M4_cmpyi_wh\0"
  "M4_cmpyi_whc\0"
  "M4_cmpyr_wh\0"
  "M4_cmpyr_whc\0"
  "M4_mac_up_s1_sat\0"
  "M4_mpyri_addi\0"
  "iUIiiUIi\0"
  "M4_mpyri_addr\0"
  "M4_mpyri_addr_u2\0"
  "iiUIii\0"
  "M4_mpyrr_addi\0"
  "iUIiii\0"
  "M4_mpyrr_addr\0"
  "M4_nac_up_s1_sat\0"
  "M4_or_and\0"
  "M4_or_andn\0"
  "M4_or_or\0"
  "M4_or_xor\0"
  "M4_pmpyw\0"
  "M4_pmpyw_acc\0"
  "M4_vpmpyh\0"
  "M4_vpmpyh_acc\0"
  "M4_vrmpyeh_acc_s0\0"
  "M4_vrmpyeh_acc_s1\0"
  "M4_vrmpyeh_s0\0"
  "M4_vrmpyeh_s1\0"
  "M4_vrmpyoh_acc_s0\0"
  "M4_vrmpyoh_acc_s1\0"
  "M4_vrmpyoh_s0\0"
  "M4_vrmpyoh_s1\0"
  "M4_xor_and\0"
  "M4_xor_andn\0"
  "M4_xor_or\0"
  "M4_xor_xacc\0"
  "M5_vdmacbsu\0"
  "M5_vdmpybsu\0"
  "M5_vmacbsu\0"
  "M5_vmacbuu\0"
  "M5_vmpybsu\0"
  "M5_vmpybuu\0"
  "M5_vrmacbsu\0"
  "M5_vrmacbuu\0"
  "M5_vrmpybsu\0"
  "M5_vrmpybuu\0"
  "M6_vabsdiffb\0"
  "M6_vabsdiffub\0"
  "M7_dcmpyiw\0"
  "M7_dcmpyiw_acc\0"
  "M7_dcmpyiwc\0"
  "M7_dcmpyiwc_acc\0"
  "M7_dcmpyrw\0"
  "M7_dcmpyrw_acc\0"
  "M7_dcmpyrwc\0"
  "M7_dcmpyrwc_acc\0"
  "M7_vdmpy\0"
  "M7_vdmpy_acc\0"
  "M7_wcmpyiw\0"
  "M7_wcmpyiw_rnd\0"
  "M7_wcmpyiwc\0"
  "M7_wcmpyiwc_rnd\0"
  "M7_wcmpyrw\0"
  "M7_wcmpyrw_rnd\0"
  "M7_wcmpyrwc\0"
  "M7_wcmpyrwc_rnd\0"
  "S2_addasl_rrri\0"
  "S2_asl_i_p\0"
  "S2_asl_i_p_acc\0"
  "LLiLLiLLiUIi\0"
  "S2_asl_i_p_and\0"
  "S2_asl_i_p_nac\0"
  "S2_asl_i_p_or\0"
  "S2_asl_i_p_xacc\0"
  "S2_asl_i_r\0"
  "S2_asl_i_r_acc\0"
  "S2_asl_i_r_and\0"
  "S2_asl_i_r_nac\0"
  "S2_asl_i_r_or\0"
  "S2_asl_i_r_sat\0"
  "S2_asl_i_r_xacc\0"
  "S2_asl_i_vh\0"
  "S2_asl_i_vw\0"
  "S2_asl_r_p\0"
  "S2_asl_r_p_acc\0"
  "S2_asl_r_p_and\0"
  "S2_asl_r_p_nac\0"
  "S2_asl_r_p_or\0"
  "S2_asl_r_p_xor\0"
  "S2_asl_r_r\0"
  "S2_asl_r_r_acc\0"
  "S2_asl_r_r_and\0"
  "S2_asl_r_r_nac\0"
  "S2_asl_r_r_or\0"
  "S2_asl_r_r_sat\0"
  "S2_asl_r_vh\0"
  "S2_asl_r_vw\0"
  "S2_asr_i_p\0"
  "S2_asr_i_p_acc\0"
  "S2_asr_i_p_and\0"
  "S2_asr_i_p_nac\0"
  "S2_asr_i_p_or\0"
  "S2_asr_i_p_rnd\0"
  "S2_asr_i_p_rnd_goodsyntax\0"
  "S2_asr_i_r\0"
  "S2_asr_i_r_acc\0"
  "S2_asr_i_r_and\0"
  "S2_asr_i_r_nac\0"
  "S2_asr_i_r_or\0"
  "S2_asr_i_r_rnd\0"
  "S2_asr_i_r_rnd_goodsyntax\0"
  "S2_asr_i_svw_trun\0"
  "S2_asr_i_vh\0"
  "S2_asr_i_vw\0"
  "S2_asr_r_p\0"
  "S2_asr_r_p_acc\0"
  "S2_asr_r_p_and\0"
  "S2_asr_r_p_nac\0"
  "S2_asr_r_p_or\0"
  "S2_asr_r_p_xor\0"
  "S2_asr_r_r\0"
  "S2_asr_r_r_acc\0"
  "S2_asr_r_r_and\0"
  "S2_asr_r_r_nac\0"
  "S2_asr_r_r_or\0"
  "S2_asr_r_r_sat\0"
  "S2_asr_r_svw_trun\0"
  "S2_asr_r_vh\0"
  "S2_asr_r_vw\0"
  "S2_brev\0"
  "S2_brevp\0"
  "S2_cl0\0"
  "S2_cl0p\0"
  "S2_cl1\0"
  "S2_cl1p\0"
  "S2_clb\0"
  "S2_clbnorm\0"
  "S2_clbp\0"
  "S2_clrbit_i\0"
  "S2_clrbit_r\0"
  "S2_ct0\0"
  "S2_ct0p\0"
  "S2_ct1\0"
  "S2_ct1p\0"
  "S2_deinterleave\0"
  "S2_extractu\0"
  "iiUIiUIi\0"
  "S2_extractu_rp\0"
  "S2_extractup\0"
  "LLiLLiUIiUIi\0"
  "S2_extractup_rp\0"
  "S2_insert\0"
  "iiiUIiUIi\0"
  "S2_insert_rp\0"
  "iiiLLi\0"
  "S2_insertp\0"
  "LLiLLiLLiUIiUIi\0"
  "S2_insertp_rp\0"
  "S2_interleave\0"
  "S2_lfsp\0"
  "S2_lsl_r_p\0"
  "S2_lsl_r_p_acc\0"
  "S2_lsl_r_p_and\0"
  "S2_lsl_r_p_nac\0"
  "S2_lsl_r_p_or\0"
  "S2_lsl_r_p_xor\0"
  "S2_lsl_r_r\0"
  "S2_lsl_r_r_acc\0"
  "S2_lsl_r_r_and\0"
  "S2_lsl_r_r_nac\0"
  "S2_lsl_r_r_or\0"
  "S2_lsl_r_vh\0"
  "S2_lsl_r_vw\0"
  "S2_lsr_i_p\0"
  "S2_lsr_i_p_acc\0"
  "S2_lsr_i_p_and\0"
  "S2_lsr_i_p_nac\0"
  "S2_lsr_i_p_or\0"
  "S2_lsr_i_p_xacc\0"
  "S2_lsr_i_r\0"
  "S2_lsr_i_r_acc\0"
  "S2_lsr_i_r_and\0"
  "S2_lsr_i_r_nac\0"
  "S2_lsr_i_r_or\0"
  "S2_lsr_i_r_xacc\0"
  "S2_lsr_i_vh\0"
  "S2_lsr_i_vw\0"
  "S2_lsr_r_p\0"
  "S2_lsr_r_p_acc\0"
  "S2_lsr_r_p_and\0"
  "S2_lsr_r_p_nac\0"
  "S2_lsr_r_p_or\0"
  "S2_lsr_r_p_xor\0"
  "S2_lsr_r_r\0"
  "S2_lsr_r_r_acc\0"
  "S2_lsr_r_r_and\0"
  "S2_lsr_r_r_nac\0"
  "S2_lsr_r_r_or\0"
  "S2_lsr_r_vh\0"
  "S2_lsr_r_vw\0"
  "S2_mask\0"
  "iUIiUIi\0"
  "S2_packhl\0"
  "S2_parityp\0"
  "S2_setbit_i\0"
  "S2_setbit_r\0"
  "S2_shuffeb\0"
  "S2_shuffeh\0"
  "S2_shuffob\0"
  "S2_shuffoh\0"
  "S2_storerb_pci\0"
  "vv*IiiivC*\0"
  "S2_storerb_pcr\0"
  "vv*iivC*\0"
  "S2_storerd_pci\0"
  "vv*IiiLLivC*\0"
  "S2_storerd_pcr\0"
  "vv*iLLivC*\0"
  "S2_storerf_pci\0"
  "S2_storerf_pcr\0"
  "S2_storerh_pci\0"
  "S2_storerh_pcr\0"
  "S2_storeri_pci\0"
  "S2_storeri_pcr\0"
  "S2_svsathb\0"
  "S2_svsathub\0"
  "S2_tableidxb_goodsyntax\0"
  "S2_tableidxd_goodsyntax\0"
  "S2_tableidxh_goodsyntax\0"
  "S2_tableidxw_goodsyntax\0"
  "S2_togglebit_i\0"
  "S2_togglebit_r\0"
  "S2_tstbit_i\0"
  "S2_tstbit_r\0"
  "S2_valignib\0"
  "S2_valignrb\0"
  "S2_vcnegh\0"
  "S2_vcrotate\0"
  "S2_vrcnegh\0"
  "S2_vrndpackwh\0"
  "S2_vrndpackwhs\0"
  "S2_vsathb\0"
  "S2_vsathb_nopack\0"
  "S2_vsathub\0"
  "S2_vsathub_nopack\0"
  "S2_vsatwh\0"
  "S2_vsatwh_nopack\0"
  "S2_vsatwuh\0"
  "S2_vsatwuh_nopack\0"
  "S2_vsplatrb\0"
  "S2_vsplatrh\0"
  "S2_vspliceib\0"
  "S2_vsplicerb\0"
  "S2_vsxtbh\0"
  "S2_vsxthw\0"
  "S2_vtrunehb\0"
  "S2_vtrunewh\0"
  "S2_vtrunohb\0"
  "S2_vtrunowh\0"
  "S2_vzxtbh\0"
  "S2_vzxthw\0"
  "S4_addaddi\0"
  "S4_addi_asl_ri\0"
  "S4_addi_lsr_ri\0"
  "S4_andi_asl_ri\0"
  "S4_andi_lsr_ri\0"
  "S4_clbaddi\0"
  "S4_clbpaddi\0"
  "S4_clbpnorm\0"
  "S4_extract\0"
  "S4_extract_rp\0"
  "S4_extractp\0"
  "S4_extractp_rp\0"
  "S4_lsli\0"
  "S4_ntstbit_i\0"
  "S4_ntstbit_r\0"
  "S4_or_andi\0"
  "S4_or_andix\0"
  "S4_or_ori\0"
  "S4_ori_asl_ri\0"
  "S4_ori_lsr_ri\0"
  "S4_parity\0"
  "S4_subaddi\0"
  "S4_subi_asl_ri\0"
  "S4_subi_lsr_ri\0"
  "S4_vrcrotate\0"
  "LLiLLiiUIi\0"
  "S4_vrcrotate_acc\0"
  "LLiLLiLLiiUIi\0"
  "S4_vxaddsubh\0"
  "S4_vxaddsubhr\0"
  "S4_vxaddsubw\0"
  "S4_vxsubaddh\0"
  "S4_vxsubaddhr\0"
  "S4_vxsubaddw\0"
  "S5_asrhub_rnd_sat_goodsyntax\0"
  "S5_asrhub_sat\0"
  "S5_popcountp\0"
  "S5_vasrhrnd_goodsyntax\0"
  "S6_rol_i_p\0"
  "v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81\0"
  "S6_rol_i_p_acc\0"
  "S6_rol_i_p_and\0"
  "S6_rol_i_p_nac\0"
  "S6_rol_i_p_or\0"
  "S6_rol_i_p_xacc\0"
  "S6_rol_i_r\0"
  "S6_rol_i_r_acc\0"
  "S6_rol_i_r_and\0"
  "S6_rol_i_r_nac\0"
  "S6_rol_i_r_or\0"
  "S6_rol_i_r_xacc\0"
  "S6_vsplatrbp\0"
  "S6_vtrunehb_ppp\0"
  "S6_vtrunohb_ppp\0"
  "V6_extractw\0"
  "iV16ii\0"
  "hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81\0"
  "V6_extractw_128B\0"
  "iV32ii\0"
  "V6_get_qfext\0"
  "V16iV16ii\0"
  "hvxv79|hvxv81\0"
  "V6_get_qfext_128B\0"
  "V32iV32ii\0"
  "V6_get_qfext_oracc\0"
  "V16iV16iV16ii\0"
  "V6_get_qfext_oracc_128B\0"
  "V32iV32iV32ii\0"
  "V6_hi\0"
  "V16iV32i\0"
  "V6_hi_128B\0"
  "V32iV64i\0"
  "V6_lo\0"
  "V6_lo_128B\0"
  "V6_lvsplatb\0"
  "V16ii\0"
  "hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81\0"
  "V6_lvsplatb_128B\0"
  "V32ii\0"
  "V6_lvsplath\0"
  "V6_lvsplath_128B\0"
  "V6_lvsplatw\0"
  "V6_lvsplatw_128B\0"
  "V6_pred_and\0"
  "V64bV64bV64b\0"
  "V6_pred_and_128B\0"
  "V128bV128bV128b\0"
  "V6_pred_and_n\0"
  "V6_pred_and_n_128B\0"
  "V6_pred_not\0"
  "V64bV64b\0"
  "V6_pred_not_128B\0"
  "V128bV128b\0"
  "V6_pred_or\0"
  "V6_pred_or_128B\0"
  "V6_pred_or_n\0"
  "V6_pred_or_n_128B\0"
  "V6_pred_scalar2\0"
  "V64bi\0"
  "V6_pred_scalar2_128B\0"
  "V128bi\0"
  "V6_pred_scalar2v2\0"
  "V6_pred_scalar2v2_128B\0"
  "V6_pred_xor\0"
  "V6_pred_xor_128B\0"
  "V6_set_qfext\0"
  "V6_set_qfext_128B\0"
  "V6_shuffeqh\0"
  "V6_shuffeqh_128B\0"
  "V6_shuffeqw\0"
  "V6_shuffeqw_128B\0"
  "V6_v6mpyhubs10\0"
  "V32iV32iV32iUIi\0"
  "hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81\0"
  "V6_v6mpyhubs10_128B\0"
  "V64iV64iV64iUIi\0"
  "V6_v6mpyhubs10_vxx\0"
  "V32iV32iV32iV32iUIi\0"
  "V6_v6mpyhubs10_vxx_128B\0"
  "V64iV64iV64iV64iUIi\0"
  "V6_v6mpyvubs10\0"
  "V6_v6mpyvubs10_128B\0"
  "V6_v6mpyvubs10_vxx\0"
  "V6_v6mpyvubs10_vxx_128B\0"
  "V6_vS32b_nqpred_ai\0"
  "vV64bv*V16i\0"
  "V6_vS32b_nqpred_ai_128B\0"
  "vV128bv*V32i\0"
  "V6_vS32b_nt_nqpred_ai\0"
  "V6_vS32b_nt_nqpred_ai_128B\0"
  "V6_vS32b_nt_qpred_ai\0"
  "V6_vS32b_nt_qpred_ai_128B\0"
  "V6_vS32b_qpred_ai\0"
  "V6_vS32b_qpred_ai_128B\0"
  "V6_vabs_f8\0"
  "V16iV16i\0"
  "V6_vabs_f8_128B\0"
  "V32iV32i\0"
  "V6_vabs_hf\0"
  "V6_vabs_hf_128B\0"
  "V6_vabs_qf16_hf\0"
  "hvxv81\0"
  "V6_vabs_qf16_hf_128B\0"
  "V6_vabs_qf16_qf16\0"
  "V6_vabs_qf16_qf16_128B\0"
  "V6_vabs_qf32_qf32\0"
  "V6_vabs_qf32_qf32_128B\0"
  "V6_vabs_qf32_sf\0"
  "V6_vabs_qf32_sf_128B\0"
  "V6_vabs_sf\0"
  "V6_vabs_sf_128B\0"
  "V6_vabsb\0"
  "hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81\0"
  "V6_vabsb_128B\0"
  "V6_vabsb_sat\0"
  "V6_vabsb_sat_128B\0"
  "V6_vabsdiffh\0"
  "V16iV16iV16i\0"
  "V6_vabsdiffh_128B\0"
  "V32iV32iV32i\0"
  "V6_vabsdiffub\0"
  "V6_vabsdiffub_128B\0"
  "V6_vabsdiffuh\0"
  "V6_vabsdiffuh_128B\0"
  "V6_vabsdiffw\0"
  "V6_vabsdiffw_128B\0"
  "V6_vabsh\0"
  "V6_vabsh_128B\0"
  "V6_vabsh_sat\0"
  "V6_vabsh_sat_128B\0"
  "V6_vabsw\0"
  "V6_vabsw_128B\0"
  "V6_vabsw_sat\0"
  "V6_vabsw_sat_128B\0"
  "V6_vadd_hf\0"
  "V6_vadd_hf_128B\0"
  "V6_vadd_hf_f8\0"
  "V32iV16iV16i\0"
  "V6_vadd_hf_f8_128B\0"
  "V64iV32iV32i\0"
  "V6_vadd_hf_hf\0"
  "V6_vadd_hf_hf_128B\0"
  "V6_vadd_qf16\0"
  "V6_vadd_qf16_128B\0"
  "V6_vadd_qf16_mix\0"
  "V6_vadd_qf16_mix_128B\0"
  "V6_vadd_qf32\0"
  "V6_vadd_qf32_128B\0"
  "V6_vadd_qf32_mix\0"
  "V6_vadd_qf32_mix_128B\0"
  "V6_vadd_sf\0"
  "V6_vadd_sf_128B\0"
  "V6_vadd_sf_bf\0"
  "hvxv73|hvxv75|hvxv79|hvxv81\0"
  "V6_vadd_sf_bf_128B\0"
  "V6_vadd_sf_hf\0"
  "V6_vadd_sf_hf_128B\0"
  "V6_vadd_sf_sf\0"
  "V6_vadd_sf_sf_128B\0"
  "V6_vaddb\0"
  "V6_vaddb_128B\0"
  "V6_vaddb_dv\0"
  "V6_vaddb_dv_128B\0"
  "V64iV64iV64i\0"
  "V6_vaddbnq\0"
  "V16iV64bV16iV16i\0"
  "V6_vaddbnq_128B\0"
  "V32iV128bV32iV32i\0"
  "V6_vaddbq\0"
  "V6_vaddbq_128B\0"
  "V6_vaddbsat\0"
  "V6_vaddbsat_128B\0"
  "V6_vaddbsat_dv\0"
  "V6_vaddbsat_dv_128B\0"
  "V6_vaddcarry\0"
  "V16iV16iV16iv*\0"
  "V6_vaddcarry_128B\0"
  "V32iV32iV32iv*\0"
  "V6_vaddcarryo\0"
  "hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81\0"
  "V6_vaddcarryo_128B\0"
  "V6_vaddcarrysat\0"
  "V16iV16iV16iV64b\0"
  "V6_vaddcarrysat_128B\0"
  "V32iV32iV32iV128b\0"
  "V6_vaddclbh\0"
  "V6_vaddclbh_128B\0"
  "V6_vaddclbw\0"
  "V6_vaddclbw_128B\0"
  "V6_vaddh\0"
  "V6_vaddh_128B\0"
  "V6_vaddh_dv\0"
  "V6_vaddh_dv_128B\0"
  "V6_vaddhnq\0"
  "V6_vaddhnq_128B\0"
  "V6_vaddhq\0"
  "V6_vaddhq_128B\0"
  "V6_vaddhsat\0"
  "V6_vaddhsat_128B\0"
  "V6_vaddhsat_dv\0"
  "V6_vaddhsat_dv_128B\0"
  "V6_vaddhw\0"
  "V6_vaddhw_128B\0"
  "V6_vaddhw_acc\0"
  "V32iV32iV16iV16i\0"
  "V6_vaddhw_acc_128B\0"
  "V64iV64iV32iV32i\0"
  "V6_vaddubh\0"
  "V6_vaddubh_128B\0"
  "V6_vaddubh_acc\0"
  "V6_vaddubh_acc_128B\0"
  "V6_vaddubsat\0"
  "V6_vaddubsat_128B\0"
  "V6_vaddubsat_dv\0"
  "V6_vaddubsat_dv_128B\0"
  "V6_vaddububb_sat\0"
  "V6_vaddububb_sat_128B\0"
  "V6_vadduhsat\0"
  "V6_vadduhsat_128B\0"
  "V6_vadduhsat_dv\0"
  "V6_vadduhsat_dv_128B\0"
  "V6_vadduhw\0"
  "V6_vadduhw_128B\0"
  "V6_vadduhw_acc\0"
  "V6_vadduhw_acc_128B\0"
  "V6_vadduwsat\0"
  "V6_vadduwsat_128B\0"
  "V6_vadduwsat_dv\0"
  "V6_vadduwsat_dv_128B\0"
  "V6_vaddw\0"
  "V6_vaddw_128B\0"
  "V6_vaddw_dv\0"
  "V6_vaddw_dv_128B\0"
  "V6_vaddwnq\0"
  "V6_vaddwnq_128B\0"
  "V6_vaddwq\0"
  "V6_vaddwq_128B\0"
  "V6_vaddwsat\0"
  "V6_vaddwsat_128B\0"
  "V6_vaddwsat_dv\0"
  "V6_vaddwsat_dv_128B\0"
  "V6_valign4\0"
  "V6_valign4_128B\0"
  "V6_valignb\0"
  "V6_valignb_128B\0"
  "V6_valignbi\0"
  "V16iV16iV16iUIi\0"
  "V6_valignbi_128B\0"
  "V6_vand\0"
  "V6_vand_128B\0"
  "V6_vandnqrt\0"
  "V16iV64bi\0"
  "V6_vandnqrt_128B\0"
  "V32iV128bi\0"
  "V6_vandnqrt_acc\0"
  "V16iV16iV64bi\0"
  "V6_vandnqrt_acc_128B\0"
  "V32iV32iV128bi\0"
  "V6_vandqrt\0"
  "V6_vandqrt_128B\0"
  "V6_vandqrt_acc\0"
  "V6_vandqrt_acc_128B\0"
  "V6_vandvnqv\0"
  "V16iV64bV16i\0"
  "V6_vandvnqv_128B\0"
  "V32iV128bV32i\0"
  "V6_vandvqv\0"
  "V6_vandvqv_128B\0"
  "V6_vandvrt\0"
  "V64bV16ii\0"
  "V6_vandvrt_128B\0"
  "V128bV32ii\0"
  "V6_vandvrt_acc\0"
  "V64bV64bV16ii\0"
  "V6_vandvrt_acc_128B\0"
  "V128bV128bV32ii\0"
  "V6_vaslh\0"
  "V6_vaslh_128B\0"
  "V6_vaslh_acc\0"
  "V6_vaslh_acc_128B\0"
  "V6_vaslhv\0"
  "V6_vaslhv_128B\0"
  "V6_vaslw\0"
  "V6_vaslw_128B\0"
  "V6_vaslw_acc\0"
  "V6_vaslw_acc_128B\0"
  "V6_vaslwv\0"
  "V6_vaslwv_128B\0"
  "V6_vasr_into\0"
  "V6_vasr_into_128B\0"
  "V6_vasrh\0"
  "V6_vasrh_128B\0"
  "V6_vasrh_acc\0"
  "V6_vasrh_acc_128B\0"
  "V6_vasrhbrndsat\0"
  "V6_vasrhbrndsat_128B\0"
  "V6_vasrhbsat\0"
  "V6_vasrhbsat_128B\0"
  "V6_vasrhubrndsat\0"
  "V6_vasrhubrndsat_128B\0"
  "V6_vasrhubsat\0"
  "V6_vasrhubsat_128B\0"
  "V6_vasrhv\0"
  "V6_vasrhv_128B\0"
  "V6_vasruhubrndsat\0"
  "V6_vasruhubrndsat_128B\0"
  "V6_vasruhubsat\0"
  "V6_vasruhubsat_128B\0"
  "V6_vasruwuhrndsat\0"
  "V6_vasruwuhrndsat_128B\0"
  "V6_vasruwuhsat\0"
  "V6_vasruwuhsat_128B\0"
  "V6_vasrvuhubrndsat\0"
  "V16iV32iV16i\0"
  "hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81\0"
  "V6_vasrvuhubrndsat_128B\0"
  "V32iV64iV32i\0"
  "V6_vasrvuhubsat\0"
  "V6_vasrvuhubsat_128B\0"
  "V6_vasrvwuhrndsat\0"
  "V6_vasrvwuhrndsat_128B\0"
  "V6_vasrvwuhsat\0"
  "V6_vasrvwuhsat_128B\0"
  "V6_vasrw\0"
  "V6_vasrw_128B\0"
  "V6_vasrw_acc\0"
  "V6_vasrw_acc_128B\0"
  "V6_vasrwh\0"
  "V6_vasrwh_128B\0"
  "V6_vasrwhrndsat\0"
  "V6_vasrwhrndsat_128B\0"
  "V6_vasrwhsat\0"
  "V6_vasrwhsat_128B\0"
  "V6_vasrwuhrndsat\0"
  "V6_vasrwuhrndsat_128B\0"
  "V6_vasrwuhsat\0"
  "V6_vasrwuhsat_128B\0"
  "V6_vasrwv\0"
  "V6_vasrwv_128B\0"
  "V6_vassign\0"
  "V6_vassign_128B\0"
  "V6_vassign_fp\0"
  "V6_vassign_fp_128B\0"
  "V6_vassignp\0"
  "V6_vassignp_128B\0"
  "V64iV64i\0"
  "V6_vavgb\0"
  "V6_vavgb_128B\0"
  "V6_vavgbrnd\0"
  "V6_vavgbrnd_128B\0"
  "V6_vavgh\0"
  "V6_vavgh_128B\0"
  "V6_vavghrnd\0"
  "V6_vavghrnd_128B\0"
  "V6_vavgub\0"
  "V6_vavgub_128B\0"
  "V6_vavgubrnd\0"
  "V6_vavgubrnd_128B\0"
  "V6_vavguh\0"
  "V6_vavguh_128B\0"
  "V6_vavguhrnd\0"
  "V6_vavguhrnd_128B\0"
  "V6_vavguw\0"
  "V6_vavguw_128B\0"
  "V6_vavguwrnd\0"
  "V6_vavguwrnd_128B\0"
  "V6_vavgw\0"
  "V6_vavgw_128B\0"
  "V6_vavgwrnd\0"
  "V6_vavgwrnd_128B\0"
  "V6_vcl0h\0"
  "V6_vcl0h_128B\0"
  "V6_vcl0w\0"
  "V6_vcl0w_128B\0"
  "V6_vcombine\0"
  "V6_vcombine_128B\0"
  "V6_vconv_bf_qf32\0"
  "V6_vconv_bf_qf32_128B\0"
  "V6_vconv_f8_qf16\0"
  "V6_vconv_f8_qf16_128B\0"
  "V6_vconv_h_hf\0"
  "V6_vconv_h_hf_128B\0"
  "V6_vconv_h_hf_rnd\0"
  "V6_vconv_h_hf_rnd_128B\0"
  "V6_vconv_hf_h\0"
  "V6_vconv_hf_h_128B\0"
  "V6_vconv_hf_qf16\0"
  "V6_vconv_hf_qf16_128B\0"
  "V6_vconv_hf_qf32\0"
  "V6_vconv_hf_qf32_128B\0"
  "V6_vconv_qf16_f8\0"
  "V32iV16i\0"
  "V6_vconv_qf16_f8_128B\0"
  "V64iV32i\0"
  "V6_vconv_qf16_hf\0"
  "V6_vconv_qf16_hf_128B\0"
  "V6_vconv_qf16_qf16\0"
  "V6_vconv_qf16_qf16_128B\0"
  "V6_vconv_qf32_qf32\0"
  "V6_vconv_qf32_qf32_128B\0"
  "V6_vconv_qf32_sf\0"
  "V6_vconv_qf32_sf_128B\0"
  "V6_vconv_sf_qf32\0"
  "V6_vconv_sf_qf32_128B\0"
  "V6_vconv_sf_w\0"
  "V6_vconv_sf_w_128B\0"
  "V6_vconv_w_sf\0"
  "V6_vconv_w_sf_128B\0"
  "V6_vcvt2_b_hf\0"
  "V6_vcvt2_b_hf_128B\0"
  "V6_vcvt2_hf_b\0"
  "V6_vcvt2_hf_b_128B\0"
  "V6_vcvt2_hf_ub\0"
  "V6_vcvt2_hf_ub_128B\0"
  "V6_vcvt2_ub_hf\0"
  "V6_vcvt2_ub_hf_128B\0"
  "V6_vcvt_b_hf\0"
  "V6_vcvt_b_hf_128B\0"
  "V6_vcvt_bf_sf\0"
  "V6_vcvt_bf_sf_128B\0"
  "V6_vcvt_f8_hf\0"
  "V6_vcvt_f8_hf_128B\0"
  "V6_vcvt_h_hf\0"
  "V6_vcvt_h_hf_128B\0"
  "V6_vcvt_hf_b\0"
  "V6_vcvt_hf_b_128B\0"
  "V6_vcvt_hf_f8\0"
  "V6_vcvt_hf_f8_128B\0"
  "V6_vcvt_hf_h\0"
  "V6_vcvt_hf_h_128B\0"
  "V6_vcvt_hf_sf\0"
  "V6_vcvt_hf_sf_128B\0"
  "V6_vcvt_hf_ub\0"
  "V6_vcvt_hf_ub_128B\0"
  "V6_vcvt_hf_uh\0"
  "V6_vcvt_hf_uh_128B\0"
  "V6_vcvt_sf_hf\0"
  "V6_vcvt_sf_hf_128B\0"
  "V6_vcvt_ub_hf\0"
  "V6_vcvt_ub_hf_128B\0"
  "V6_vcvt_uh_hf\0"
  "V6_vcvt_uh_hf_128B\0"
  "V6_vd0\0"
  "V16i\0"
  "V6_vd0_128B\0"
  "V32i\0"
  "V6_vdd0\0"
  "V6_vdd0_128B\0"
  "V64i\0"
  "V6_vdealb\0"
  "V6_vdealb4w\0"
  "V6_vdealb4w_128B\0"
  "V6_vdealb_128B\0"
  "V6_vdealh\0"
  "V6_vdealh_128B\0"
  "V6_vdealvdd\0"
  "V32iV16iV16ii\0"
  "V6_vdealvdd_128B\0"
  "V64iV32iV32ii\0"
  "V6_vdelta\0"
  "V6_vdelta_128B\0"
  "V6_vdmpy_sf_hf\0"
  "V6_vdmpy_sf_hf_128B\0"
  "V6_vdmpy_sf_hf_acc\0"
  "V16iV16iV16iV16i\0"
  "V6_vdmpy_sf_hf_acc_128B\0"
  "V32iV32iV32iV32i\0"
  "V6_vdmpybus\0"
  "V6_vdmpybus_128B\0"
  "V6_vdmpybus_acc\0"
  "V6_vdmpybus_acc_128B\0"
  "V6_vdmpybus_dv\0"
  "V6_vdmpybus_dv_128B\0"
  "V64iV64ii\0"
  "V6_vdmpybus_dv_acc\0"
  "V6_vdmpybus_dv_acc_128B\0"
  "V64iV64iV64ii\0"
  "V6_vdmpyhb\0"
  "V6_vdmpyhb_128B\0"
  "V6_vdmpyhb_acc\0"
  "V6_vdmpyhb_acc_128B\0"
  "V6_vdmpyhb_dv\0"
  "V6_vdmpyhb_dv_128B\0"
  "V6_vdmpyhb_dv_acc\0"
  "V6_vdmpyhb_dv_acc_128B\0"
  "V6_vdmpyhisat\0"
  "V16iV32ii\0"
  "V6_vdmpyhisat_128B\0"
  "V32iV64ii\0"
  "V6_vdmpyhisat_acc\0"
  "V16iV16iV32ii\0"
  "V6_vdmpyhisat_acc_128B\0"
  "V32iV32iV64ii\0"
  "V6_vdmpyhsat\0"
  "V6_vdmpyhsat_128B\0"
  "V6_vdmpyhsat_acc\0"
  "V6_vdmpyhsat_acc_128B\0"
  "V6_vdmpyhsuisat\0"
  "V6_vdmpyhsuisat_128B\0"
  "V6_vdmpyhsuisat_acc\0"
  "V6_vdmpyhsuisat_acc_128B\0"
  "V6_vdmpyhsusat\0"
  "V6_vdmpyhsusat_128B\0"
  "V6_vdmpyhsusat_acc\0"
  "V6_vdmpyhsusat_acc_128B\0"
  "V6_vdmpyhvsat\0"
  "V6_vdmpyhvsat_128B\0"
  "V6_vdmpyhvsat_acc\0"
  "V6_vdmpyhvsat_acc_128B\0"
  "V6_vdsaduh\0"
  "V6_vdsaduh_128B\0"
  "V6_vdsaduh_acc\0"
  "V6_vdsaduh_acc_128B\0"
  "V6_veqb\0"
  "V64bV16iV16i\0"
  "V6_veqb_128B\0"
  "V128bV32iV32i\0"
  "V6_veqb_and\0"
  "V64bV64bV16iV16i\0"
  "V6_veqb_and_128B\0"
  "V128bV128bV32iV32i\0"
  "V6_veqb_or\0"
  "V6_veqb_or_128B\0"
  "V6_veqb_xor\0"
  "V6_veqb_xor_128B\0"
  "V6_veqh\0"
  "V6_veqh_128B\0"
  "V6_veqh_and\0"
  "V6_veqh_and_128B\0"
  "V6_veqh_or\0"
  "V6_veqh_or_128B\0"
  "V6_veqh_xor\0"
  "V6_veqh_xor_128B\0"
  "V6_veqhf\0"
  "V6_veqhf_128B\0"
  "V6_veqhf_and\0"
  "V6_veqhf_and_128B\0"
  "V6_veqhf_or\0"
  "V6_veqhf_or_128B\0"
  "V6_veqhf_xor\0"
  "V6_veqhf_xor_128B\0"
  "V6_veqsf\0"
  "V6_veqsf_128B\0"
  "V6_veqsf_and\0"
  "V6_veqsf_and_128B\0"
  "V6_veqsf_or\0"
  "V6_veqsf_or_128B\0"
  "V6_veqsf_xor\0"
  "V6_veqsf_xor_128B\0"
  "V6_veqw\0"
  "V6_veqw_128B\0"
  "V6_veqw_and\0"
  "V6_veqw_and_128B\0"
  "V6_veqw_or\0"
  "V6_veqw_or_128B\0"
  "V6_veqw_xor\0"
  "V6_veqw_xor_128B\0"
  "V6_vfmax_f8\0"
  "V6_vfmax_f8_128B\0"
  "V6_vfmax_hf\0"
  "V6_vfmax_hf_128B\0"
  "V6_vfmax_sf\0"
  "V6_vfmax_sf_128B\0"
  "V6_vfmin_f8\0"
  "V6_vfmin_f8_128B\0"
  "V6_vfmin_hf\0"
  "V6_vfmin_hf_128B\0"
  "V6_vfmin_sf\0"
  "V6_vfmin_sf_128B\0"
  "V6_vfneg_f8\0"
  "V6_vfneg_f8_128B\0"
  "V6_vfneg_hf\0"
  "V6_vfneg_hf_128B\0"
  "V6_vfneg_sf\0"
  "V6_vfneg_sf_128B\0"
  "V6_vgathermh\0"
  "vv*iiV16i\0"
  "V6_vgathermh_128B\0"
  "vv*iiV32i\0"
  "V6_vgathermhq\0"
  "vv*V64biiV16i\0"
  "V6_vgathermhq_128B\0"
  "vv*V128biiV32i\0"
  "V6_vgathermhw\0"
  "V6_vgathermhw_128B\0"
  "vv*iiV64i\0"
  "V6_vgathermhwq\0"
  "vv*V64biiV32i\0"
  "V6_vgathermhwq_128B\0"
  "vv*V128biiV64i\0"
  "V6_vgathermw\0"
  "V6_vgathermw_128B\0"
  "V6_vgathermwq\0"
  "V6_vgathermwq_128B\0"
  "V6_vgtb\0"
  "V6_vgtb_128B\0"
  "V6_vgtb_and\0"
  "V6_vgtb_and_128B\0"
  "V6_vgtb_or\0"
  "V6_vgtb_or_128B\0"
  "V6_vgtb_xor\0"
  "V6_vgtb_xor_128B\0"
  "V6_vgtbf\0"
  "V6_vgtbf_128B\0"
  "V6_vgtbf_and\0"
  "V6_vgtbf_and_128B\0"
  "V6_vgtbf_or\0"
  "V6_vgtbf_or_128B\0"
  "V6_vgtbf_xor\0"
  "V6_vgtbf_xor_128B\0"
  "V6_vgth\0"
  "V6_vgth_128B\0"
  "V6_vgth_and\0"
  "V6_vgth_and_128B\0"
  "V6_vgth_or\0"
  "V6_vgth_or_128B\0"
  "V6_vgth_xor\0"
  "V6_vgth_xor_128B\0"
  "V6_vgthf\0"
  "V6_vgthf_128B\0"
  "V6_vgthf_and\0"
  "V6_vgthf_and_128B\0"
  "V6_vgthf_or\0"
  "V6_vgthf_or_128B\0"
  "V6_vgthf_xor\0"
  "V6_vgthf_xor_128B\0"
  "V6_vgtsf\0"
  "V6_vgtsf_128B\0"
  "V6_vgtsf_and\0"
  "V6_vgtsf_and_128B\0"
  "V6_vgtsf_or\0"
  "V6_vgtsf_or_128B\0"
  "V6_vgtsf_xor\0"
  "V6_vgtsf_xor_128B\0"
  "V6_vgtub\0"
  "V6_vgtub_128B\0"
  "V6_vgtub_and\0"
  "V6_vgtub_and_128B\0"
  "V6_vgtub_or\0"
  "V6_vgtub_or_128B\0"
  "V6_vgtub_xor\0"
  "V6_vgtub_xor_128B\0"
  "V6_vgtuh\0"
  "V6_vgtuh_128B\0"
  "V6_vgtuh_and\0"
  "V6_vgtuh_and_128B\0"
  "V6_vgtuh_or\0"
  "V6_vgtuh_or_128B\0"
  "V6_vgtuh_xor\0"
  "V6_vgtuh_xor_128B\0"
  "V6_vgtuw\0"
  "V6_vgtuw_128B\0"
  "V6_vgtuw_and\0"
  "V6_vgtuw_and_128B\0"
  "V6_vgtuw_or\0"
  "V6_vgtuw_or_128B\0"
  "V6_vgtuw_xor\0"
  "V6_vgtuw_xor_128B\0"
  "V6_vgtw\0"
  "V6_vgtw_128B\0"
  "V6_vgtw_and\0"
  "V6_vgtw_and_128B\0"
  "V6_vgtw_or\0"
  "V6_vgtw_or_128B\0"
  "V6_vgtw_xor\0"
  "V6_vgtw_xor_128B\0"
  "V6_vilog2_hf\0"
  "V6_vilog2_hf_128B\0"
  "V6_vilog2_qf16\0"
  "V6_vilog2_qf16_128B\0"
  "V6_vilog2_qf32\0"
  "V6_vilog2_qf32_128B\0"
  "V6_vilog2_sf\0"
  "V6_vilog2_sf_128B\0"
  "V6_vinsertwr\0"
  "V6_vinsertwr_128B\0"
  "V6_vlalignb\0"
  "V6_vlalignb_128B\0"
  "V6_vlalignbi\0"
  "V6_vlalignbi_128B\0"
  "V6_vlsrb\0"
  "V6_vlsrb_128B\0"
  "V6_vlsrh\0"
  "V6_vlsrh_128B\0"
  "V6_vlsrhv\0"
  "V6_vlsrhv_128B\0"
  "V6_vlsrw\0"
  "V6_vlsrw_128B\0"
  "V6_vlsrwv\0"
  "V6_vlsrwv_128B\0"
  "V6_vlut4\0"
  "V16iV16iLLi\0"
  "V6_vlut4_128B\0"
  "V32iV32iLLi\0"
  "V6_vlutvvb\0"
  "V6_vlutvvb_128B\0"
  "V6_vlutvvb_nm\0"
  "V6_vlutvvb_nm_128B\0"
  "V6_vlutvvb_oracc\0"
  "V16iV16iV16iV16ii\0"
  "V6_vlutvvb_oracc_128B\0"
  "V32iV32iV32iV32ii\0"
  "V6_vlutvvb_oracci\0"
  "V16iV16iV16iV16iUIi\0"
  "V6_vlutvvb_oracci_128B\0"
  "V6_vlutvvbi\0"
  "V6_vlutvvbi_128B\0"
  "V6_vlutvwh\0"
  "V6_vlutvwh_128B\0"
  "V6_vlutvwh_nm\0"
  "V6_vlutvwh_nm_128B\0"
  "V6_vlutvwh_oracc\0"
  "V32iV32iV16iV16ii\0"
  "V6_vlutvwh_oracc_128B\0"
  "V64iV64iV32iV32ii\0"
  "V6_vlutvwh_oracci\0"
  "V32iV32iV16iV16iUIi\0"
  "V6_vlutvwh_oracci_128B\0"
  "V64iV64iV32iV32iUIi\0"
  "V6_vlutvwhi\0"
  "V32iV16iV16iUIi\0"
  "V6_vlutvwhi_128B\0"
  "V64iV32iV32iUIi\0"
  "V6_vmaskedstorenq\0"
  "V6_vmaskedstorenq_128B\0"
  "V6_vmaskedstorentnq\0"
  "V6_vmaskedstorentnq_128B\0"
  "V6_vmaskedstorentq\0"
  "V6_vmaskedstorentq_128B\0"
  "V6_vmaskedstoreq\0"
  "V6_vmaskedstoreq_128B\0"
  "V6_vmax_bf\0"
  "V6_vmax_bf_128B\0"
  "V6_vmax_hf\0"
  "V6_vmax_hf_128B\0"
  "V6_vmax_sf\0"
  "V6_vmax_sf_128B\0"
  "V6_vmaxb\0"
  "V6_vmaxb_128B\0"
  "V6_vmaxh\0"
  "V6_vmaxh_128B\0"
  "V6_vmaxub\0"
  "V6_vmaxub_128B\0"
  "V6_vmaxuh\0"
  "V6_vmaxuh_128B\0"
  "V6_vmaxw\0"
  "V6_vmaxw_128B\0"
  "V6_vmerge_qf\0"
  "V6_vmerge_qf_128B\0"
  "V6_vmin_bf\0"
  "V6_vmin_bf_128B\0"
  "V6_vmin_hf\0"
  "V6_vmin_hf_128B\0"
  "V6_vmin_sf\0"
  "V6_vmin_sf_128B\0"
  "V6_vminb\0"
  "V6_vminb_128B\0"
  "V6_vminh\0"
  "V6_vminh_128B\0"
  "V6_vminub\0"
  "V6_vminub_128B\0"
  "V6_vminuh\0"
  "V6_vminuh_128B\0"
  "V6_vminw\0"
  "V6_vminw_128B\0"
  "V6_vmpabus\0"
  "V6_vmpabus_128B\0"
  "V6_vmpabus_acc\0"
  "V6_vmpabus_acc_128B\0"
  "V6_vmpabusv\0"
  "V6_vmpabusv_128B\0"
  "V6_vmpabuu\0"
  "V6_vmpabuu_128B\0"
  "V6_vmpabuu_acc\0"
  "V6_vmpabuu_acc_128B\0"
  "V6_vmpabuuv\0"
  "V6_vmpabuuv_128B\0"
  "V6_vmpahb\0"
  "V6_vmpahb_128B\0"
  "V6_vmpahb_acc\0"
  "V6_vmpahb_acc_128B\0"
  "V6_vmpahhsat\0"
  "V16iV16iV16iLLi\0"
  "V6_vmpahhsat_128B\0"
  "V32iV32iV32iLLi\0"
  "V6_vmpauhb\0"
  "V6_vmpauhb_128B\0"
  "V6_vmpauhb_acc\0"
  "V6_vmpauhb_acc_128B\0"
  "V6_vmpauhuhsat\0"
  "V6_vmpauhuhsat_128B\0"
  "V6_vmpsuhuhsat\0"
  "V6_vmpsuhuhsat_128B\0"
  "V6_vmpy_hf_f8\0"
  "V6_vmpy_hf_f8_128B\0"
  "V6_vmpy_hf_f8_acc\0"
  "V6_vmpy_hf_f8_acc_128B\0"
  "V6_vmpy_hf_hf\0"
  "V6_vmpy_hf_hf_128B\0"
  "V6_vmpy_hf_hf_acc\0"
  "V6_vmpy_hf_hf_acc_128B\0"
  "V6_vmpy_qf16\0"
  "V6_vmpy_qf16_128B\0"
  "V6_vmpy_qf16_hf\0"
  "V6_vmpy_qf16_hf_128B\0"
  "V6_vmpy_qf16_mix_hf\0"
  "V6_vmpy_qf16_mix_hf_128B\0"
  "V6_vmpy_qf32\0"
  "V6_vmpy_qf32_128B\0"
  "V6_vmpy_qf32_hf\0"
  "V6_vmpy_qf32_hf_128B\0"
  "V6_vmpy_qf32_mix_hf\0"
  "V6_vmpy_qf32_mix_hf_128B\0"
  "V6_vmpy_qf32_qf16\0"
  "V6_vmpy_qf32_qf16_128B\0"
  "V6_vmpy_qf32_sf\0"
  "V6_vmpy_qf32_sf_128B\0"
  "V6_vmpy_rt_hf\0"
  "V6_vmpy_rt_hf_128B\0"
  "V6_vmpy_rt_qf16\0"
  "V6_vmpy_rt_qf16_128B\0"
  "V6_vmpy_rt_sf\0"
  "V6_vmpy_rt_sf_128B\0"
  "V6_vmpy_sf_bf\0"
  "V6_vmpy_sf_bf_128B\0"
  "V6_vmpy_sf_bf_acc\0"
  "V6_vmpy_sf_bf_acc_128B\0"
  "V6_vmpy_sf_hf\0"
  "V6_vmpy_sf_hf_128B\0"
  "V6_vmpy_sf_hf_acc\0"
  "V6_vmpy_sf_hf_acc_128B\0"
  "V6_vmpy_sf_sf\0"
  "V6_vmpy_sf_sf_128B\0"
  "V6_vmpybus\0"
  "V32iV16ii\0"
  "V6_vmpybus_128B\0"
  "V64iV32ii\0"
  "V6_vmpybus_acc\0"
  "V32iV32iV16ii\0"
  "V6_vmpybus_acc_128B\0"
  "V64iV64iV32ii\0"
  "V6_vmpybusv\0"
  "V6_vmpybusv_128B\0"
  "V6_vmpybusv_acc\0"
  "V6_vmpybusv_acc_128B\0"
  "V6_vmpybv\0"
  "V6_vmpybv_128B\0"
  "V6_vmpybv_acc\0"
  "V6_vmpybv_acc_128B\0"
  "V6_vmpyewuh\0"
  "V6_vmpyewuh_64\0"
  "V6_vmpyewuh_64_128B\0"
  "V6_vmpyewuh_128B\0"
  "V6_vmpyh\0"
  "V6_vmpyh_128B\0"
  "V6_vmpyh_acc\0"
  "V6_vmpyh_acc_128B\0"
  "V6_vmpyhsat_acc\0"
  "V6_vmpyhsat_acc_128B\0"
  "V6_vmpyhsrs\0"
  "V6_vmpyhsrs_128B\0"
  "V6_vmpyhss\0"
  "V6_vmpyhss_128B\0"
  "V6_vmpyhus\0"
  "V6_vmpyhus_128B\0"
  "V6_vmpyhus_acc\0"
  "V6_vmpyhus_acc_128B\0"
  "V6_vmpyhv\0"
  "V6_vmpyhv_128B\0"
  "V6_vmpyhv_acc\0"
  "V6_vmpyhv_acc_128B\0"
  "V6_vmpyhvsrs\0"
  "V6_vmpyhvsrs_128B\0"
  "V6_vmpyieoh\0"
  "V6_vmpyieoh_128B\0"
  "V6_vmpyiewh_acc\0"
  "V6_vmpyiewh_acc_128B\0"
  "V6_vmpyiewuh\0"
  "V6_vmpyiewuh_128B\0"
  "V6_vmpyiewuh_acc\0"
  "V6_vmpyiewuh_acc_128B\0"
  "V6_vmpyih\0"
  "V6_vmpyih_128B\0"
  "V6_vmpyih_acc\0"
  "V6_vmpyih_acc_128B\0"
  "V6_vmpyihb\0"
  "V6_vmpyihb_128B\0"
  "V6_vmpyihb_acc\0"
  "V6_vmpyihb_acc_128B\0"
  "V6_vmpyiowh\0"
  "V6_vmpyiowh_128B\0"
  "V6_vmpyiwb\0"
  "V6_vmpyiwb_128B\0"
  "V6_vmpyiwb_acc\0"
  "V6_vmpyiwb_acc_128B\0"
  "V6_vmpyiwh\0"
  "V6_vmpyiwh_128B\0"
  "V6_vmpyiwh_acc\0"
  "V6_vmpyiwh_acc_128B\0"
  "V6_vmpyiwub\0"
  "V6_vmpyiwub_128B\0"
  "V6_vmpyiwub_acc\0"
  "V6_vmpyiwub_acc_128B\0"
  "V6_vmpyowh\0"
  "V6_vmpyowh_64_acc\0"
  "V6_vmpyowh_64_acc_128B\0"
  "V6_vmpyowh_128B\0"
  "V6_vmpyowh_rnd\0"
  "V6_vmpyowh_rnd_128B\0"
  "V6_vmpyowh_rnd_sacc\0"
  "V6_vmpyowh_rnd_sacc_128B\0"
  "V6_vmpyowh_sacc\0"
  "V6_vmpyowh_sacc_128B\0"
  "V6_vmpyub\0"
  "V6_vmpyub_128B\0"
  "V6_vmpyub_acc\0"
  "V6_vmpyub_acc_128B\0"
  "V6_vmpyubv\0"
  "V6_vmpyubv_128B\0"
  "V6_vmpyubv_acc\0"
  "V6_vmpyubv_acc_128B\0"
  "V6_vmpyuh\0"
  "V6_vmpyuh_128B\0"
  "V6_vmpyuh_acc\0"
  "V6_vmpyuh_acc_128B\0"
  "V6_vmpyuhe\0"
  "V6_vmpyuhe_128B\0"
  "V6_vmpyuhe_acc\0"
  "V6_vmpyuhe_acc_128B\0"
  "V6_vmpyuhv\0"
  "V6_vmpyuhv_128B\0"
  "V6_vmpyuhv_acc\0"
  "V6_vmpyuhv_acc_128B\0"
  "V6_vmpyuhvs\0"
  "V6_vmpyuhvs_128B\0"
  "V6_vmux\0"
  "V6_vmux_128B\0"
  "V6_vnavgb\0"
  "V6_vnavgb_128B\0"
  "V6_vnavgh\0"
  "V6_vnavgh_128B\0"
  "V6_vnavgub\0"
  "V6_vnavgub_128B\0"
  "V6_vnavgw\0"
  "V6_vnavgw_128B\0"
  "V6_vneg_qf16_hf\0"
  "V6_vneg_qf16_hf_128B\0"
  "V6_vneg_qf16_qf16\0"
  "V6_vneg_qf16_qf16_128B\0"
  "V6_vneg_qf32_qf32\0"
  "V6_vneg_qf32_qf32_128B\0"
  "V6_vneg_qf32_sf\0"
  "V6_vneg_qf32_sf_128B\0"
  "V6_vnormamth\0"
  "V6_vnormamth_128B\0"
  "V6_vnormamtw\0"
  "V6_vnormamtw_128B\0"
  "V6_vnot\0"
  "V6_vnot_128B\0"
  "V6_vor\0"
  "V6_vor_128B\0"
  "V6_vpackeb\0"
  "V6_vpackeb_128B\0"
  "V6_vpackeh\0"
  "V6_vpackeh_128B\0"
  "V6_vpackhb_sat\0"
  "V6_vpackhb_sat_128B\0"
  "V6_vpackhub_sat\0"
  "V6_vpackhub_sat_128B\0"
  "V6_vpackob\0"
  "V6_vpackob_128B\0"
  "V6_vpackoh\0"
  "V6_vpackoh_128B\0"
  "V6_vpackwh_sat\0"
  "V6_vpackwh_sat_128B\0"
  "V6_vpackwuh_sat\0"
  "V6_vpackwuh_sat_128B\0"
  "V6_vpopcounth\0"
  "V6_vpopcounth_128B\0"
  "V6_vprefixqb\0"
  "V16iV64b\0"
  "V6_vprefixqb_128B\0"
  "V32iV128b\0"
  "V6_vprefixqh\0"
  "V6_vprefixqh_128B\0"
  "V6_vprefixqw\0"
  "V6_vprefixqw_128B\0"
  "V6_vrdelta\0"
  "V6_vrdelta_128B\0"
  "V6_vrmpybub_rtt\0"
  "V32iV16iLLi\0"
  "hvxv65\0"
  "V6_vrmpybub_rtt_128B\0"
  "V64iV32iLLi\0"
  "V6_vrmpybub_rtt_acc\0"
  "V32iV32iV16iLLi\0"
  "V6_vrmpybub_rtt_acc_128B\0"
  "V64iV64iV32iLLi\0"
  "V6_vrmpybus\0"
  "V6_vrmpybus_128B\0"
  "V6_vrmpybus_acc\0"
  "V6_vrmpybus_acc_128B\0"
  "V6_vrmpybusi\0"
  "V32iV32iiUIi\0"
  "V6_vrmpybusi_128B\0"
  "V64iV64iiUIi\0"
  "V6_vrmpybusi_acc\0"
  "V32iV32iV32iiUIi\0"
  "V6_vrmpybusi_acc_128B\0"
  "V64iV64iV64iiUIi\0"
  "V6_vrmpybusv\0"
  "V6_vrmpybusv_128B\0"
  "V6_vrmpybusv_acc\0"
  "V6_vrmpybusv_acc_128B\0"
  "V6_vrmpybv\0"
  "V6_vrmpybv_128B\0"
  "V6_vrmpybv_acc\0"
  "V6_vrmpybv_acc_128B\0"
  "V6_vrmpyub\0"
  "V6_vrmpyub_128B\0"
  "V6_vrmpyub_acc\0"
  "V6_vrmpyub_acc_128B\0"
  "V6_vrmpyub_rtt\0"
  "V6_vrmpyub_rtt_128B\0"
  "V6_vrmpyub_rtt_acc\0"
  "V6_vrmpyub_rtt_acc_128B\0"
  "V6_vrmpyubi\0"
  "V6_vrmpyubi_128B\0"
  "V6_vrmpyubi_acc\0"
  "V6_vrmpyubi_acc_128B\0"
  "V6_vrmpyubv\0"
  "V6_vrmpyubv_128B\0"
  "V6_vrmpyubv_acc\0"
  "V6_vrmpyubv_acc_128B\0"
  "V6_vror\0"
  "V6_vror_128B\0"
  "V6_vrotr\0"
  "V6_vrotr_128B\0"
  "V6_vroundhb\0"
  "V6_vroundhb_128B\0"
  "V6_vroundhub\0"
  "V6_vroundhub_128B\0"
  "V6_vrounduhub\0"
  "V6_vrounduhub_128B\0"
  "V6_vrounduwuh\0"
  "V6_vrounduwuh_128B\0"
  "V6_vroundwh\0"
  "V6_vroundwh_128B\0"
  "V6_vroundwuh\0"
  "V6_vroundwuh_128B\0"
  "V6_vrsadubi\0"
  "V6_vrsadubi_128B\0"
  "V6_vrsadubi_acc\0"
  "V6_vrsadubi_acc_128B\0"
  "V6_vsatdw\0"
  "V6_vsatdw_128B\0"
  "V6_vsathub\0"
  "V6_vsathub_128B\0"
  "V6_vsatuwuh\0"
  "V6_vsatuwuh_128B\0"
  "V6_vsatwh\0"
  "V6_vsatwh_128B\0"
  "V6_vsb\0"
  "V6_vsb_128B\0"
  "V6_vscattermh\0"
  "viiV16iV16i\0"
  "V6_vscattermh_128B\0"
  "viiV32iV32i\0"
  "V6_vscattermh_add\0"
  "V6_vscattermh_add_128B\0"
  "V6_vscattermhq\0"
  "vV64biiV16iV16i\0"
  "V6_vscattermhq_128B\0"
  "vV128biiV32iV32i\0"
  "V6_vscattermhw\0"
  "viiV32iV16i\0"
  "V6_vscattermhw_128B\0"
  "viiV64iV32i\0"
  "V6_vscattermhw_add\0"
  "V6_vscattermhw_add_128B\0"
  "V6_vscattermhwq\0"
  "vV64biiV32iV16i\0"
  "V6_vscattermhwq_128B\0"
  "vV128biiV64iV32i\0"
  "V6_vscattermw\0"
  "V6_vscattermw_128B\0"
  "V6_vscattermw_add\0"
  "V6_vscattermw_add_128B\0"
  "V6_vscattermwq\0"
  "V6_vscattermwq_128B\0"
  "V6_vsh\0"
  "V6_vsh_128B\0"
  "V6_vshufeh\0"
  "V6_vshufeh_128B\0"
  "V6_vshuffb\0"
  "V6_vshuffb_128B\0"
  "V6_vshuffeb\0"
  "V6_vshuffeb_128B\0"
  "V6_vshuffh\0"
  "V6_vshuffh_128B\0"
  "V6_vshuffob\0"
  "V6_vshuffob_128B\0"
  "V6_vshuffvdd\0"
  "V6_vshuffvdd_128B\0"
  "V6_vshufoeb\0"
  "V6_vshufoeb_128B\0"
  "V6_vshufoeh\0"
  "V6_vshufoeh_128B\0"
  "V6_vshufoh\0"
  "V6_vshufoh_128B\0"
  "V6_vsub_hf\0"
  "V6_vsub_hf_128B\0"
  "V6_vsub_hf_f8\0"
  "V6_vsub_hf_f8_128B\0"
  "V6_vsub_hf_hf\0"
  "V6_vsub_hf_hf_128B\0"
  "V6_vsub_hf_mix\0"
  "V6_vsub_hf_mix_128B\0"
  "V6_vsub_qf16\0"
  "V6_vsub_qf16_128B\0"
  "V6_vsub_qf16_mix\0"
  "V6_vsub_qf16_mix_128B\0"
  "V6_vsub_qf32\0"
  "V6_vsub_qf32_128B\0"
  "V6_vsub_qf32_mix\0"
  "V6_vsub_qf32_mix_128B\0"
  "V6_vsub_sf\0"
  "V6_vsub_sf_128B\0"
  "V6_vsub_sf_bf\0"
  "V6_vsub_sf_bf_128B\0"
  "V6_vsub_sf_hf\0"
  "V6_vsub_sf_hf_128B\0"
  "V6_vsub_sf_mix\0"
  "V6_vsub_sf_mix_128B\0"
  "V6_vsub_sf_sf\0"
  "V6_vsub_sf_sf_128B\0"
  "V6_vsubb\0"
  "V6_vsubb_128B\0"
  "V6_vsubb_dv\0"
  "V6_vsubb_dv_128B\0"
  "V6_vsubbnq\0"
  "V6_vsubbnq_128B\0"
  "V6_vsubbq\0"
  "V6_vsubbq_128B\0"
  "V6_vsubbsat\0"
  "V6_vsubbsat_128B\0"
  "V6_vsubbsat_dv\0"
  "V6_vsubbsat_dv_128B\0"
  "V6_vsubcarry\0"
  "V6_vsubcarry_128B\0"
  "V6_vsubcarryo\0"
  "V6_vsubcarryo_128B\0"
  "V6_vsubh\0"
  "V6_vsubh_128B\0"
  "V6_vsubh_dv\0"
  "V6_vsubh_dv_128B\0"
  "V6_vsubhnq\0"
  "V6_vsubhnq_128B\0"
  "V6_vsubhq\0"
  "V6_vsubhq_128B\0"
  "V6_vsubhsat\0"
  "V6_vsubhsat_128B\0"
  "V6_vsubhsat_dv\0"
  "V6_vsubhsat_dv_128B\0"
  "V6_vsubhw\0"
  "V6_vsubhw_128B\0"
  "V6_vsububh\0"
  "V6_vsububh_128B\0"
  "V6_vsububsat\0"
  "V6_vsububsat_128B\0"
  "V6_vsububsat_dv\0"
  "V6_vsububsat_dv_128B\0"
  "V6_vsubububb_sat\0"
  "V6_vsubububb_sat_128B\0"
  "V6_vsubuhsat\0"
  "V6_vsubuhsat_128B\0"
  "V6_vsubuhsat_dv\0"
  "V6_vsubuhsat_dv_128B\0"
  "V6_vsubuhw\0"
  "V6_vsubuhw_128B\0"
  "V6_vsubuwsat\0"
  "V6_vsubuwsat_128B\0"
  "V6_vsubuwsat_dv\0"
  "V6_vsubuwsat_dv_128B\0"
  "V6_vsubw\0"
  "V6_vsubw_128B\0"
  "V6_vsubw_dv\0"
  "V6_vsubw_dv_128B\0"
  "V6_vsubwnq\0"
  "V6_vsubwnq_128B\0"
  "V6_vsubwq\0"
  "V6_vsubwq_128B\0"
  "V6_vsubwsat\0"
  "V6_vsubwsat_128B\0"
  "V6_vsubwsat_dv\0"
  "V6_vsubwsat_dv_128B\0"
  "V6_vswap\0"
  "V32iV64bV16iV16i\0"
  "V6_vswap_128B\0"
  "V64iV128bV32iV32i\0"
  "V6_vtmpyb\0"
  "V6_vtmpyb_128B\0"
  "V6_vtmpyb_acc\0"
  "V6_vtmpyb_acc_128B\0"
  "V6_vtmpybus\0"
  "V6_vtmpybus_128B\0"
  "V6_vtmpybus_acc\0"
  "V6_vtmpybus_acc_128B\0"
  "V6_vtmpyhb\0"
  "V6_vtmpyhb_128B\0"
  "V6_vtmpyhb_acc\0"
  "V6_vtmpyhb_acc_128B\0"
  "V6_vunpackb\0"
  "V6_vunpackb_128B\0"
  "V6_vunpackh\0"
  "V6_vunpackh_128B\0"
  "V6_vunpackob\0"
  "V32iV32iV16i\0"
  "V6_vunpackob_128B\0"
  "V64iV64iV32i\0"
  "V6_vunpackoh\0"
  "V6_vunpackoh_128B\0"
  "V6_vunpackub\0"
  "V6_vunpackub_128B\0"
  "V6_vunpackuh\0"
  "V6_vunpackuh_128B\0"
  "V6_vxor\0"
  "V6_vxor_128B\0"
  "V6_vzb\0"
  "V6_vzb_128B\0"
  "V6_vzh\0"
  "V6_vzh_128B\0"
  "Y2_dccleana\0"
  "vv*\0"
  "Y2_dccleaninva\0"
  "Y2_dcfetch\0"
  "Y2_dcinva\0"
  "Y2_dczeroa\0"
  "Y4_l2fetch\0"
  "vv*i\0"
  "Y5_l2fetch\0"
  "vv*LLi\0"
  "Y6_dmlink\0"
  "vv*v*\0"
  "v68|v69|v71|v73|v75|v79|v81\0"
  "Y6_dmpause\0"
  "i\0"
  "Y6_dmpoll\0"
  "Y6_dmresume\0"
  "Y6_dmstart\0"
  "Y6_dmwait\0"
  "prefetch\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinStrings = BuiltinStringsStorage;

#endif // GET_BUILTIN_STR_TABLE

#ifdef GET_BUILTIN_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{1 /* __builtin_SI_to_SXTHI_asrh */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{86 /* __builtin_brev_ldb */, 105 /* v*Sc*CSc*iC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{117 /* __builtin_brev_ldd */, 136 /* v*LLi*CLLi*iC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{150 /* __builtin_brev_ldh */, 169 /* v*s*Cs*iC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{179 /* __builtin_brev_ldub */, 199 /* v*Uc*CUc*iC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{211 /* __builtin_brev_lduh */, 231 /* v*Us*CUs*iC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{243 /* __builtin_brev_ldw */, 262 /* v*i*Ci*iC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{272 /* __builtin_brev_stb */, 291 /* c*Cc*iiC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{300 /* __builtin_brev_std */, 319 /* LLi*CLLi*LLiiC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{334 /* __builtin_brev_sth */, 353 /* s*Cs*iiC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{362 /* __builtin_brev_sthhi */, 353 /* s*Cs*iiC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{383 /* __builtin_brev_stw */, 402 /* i*Ci*iiC */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{411 /* __builtin_circ_ldb */, 430 /* c*c*c*iIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{440 /* __builtin_circ_ldd */, 459 /* LLi*LLi*LLi*iIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{475 /* __builtin_circ_ldh */, 494 /* s*s*s*iIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{504 /* __builtin_circ_ldub */, 524 /* Uc*Uc*Uc*iIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{537 /* __builtin_circ_lduh */, 557 /* Us*Us*Us*iIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{570 /* __builtin_circ_ldw */, 589 /* i*i*i*iIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{599 /* __builtin_circ_stb */, 618 /* c*c*iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{627 /* __builtin_circ_std */, 646 /* LLi*LLi*LLiiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{661 /* __builtin_circ_sth */, 680 /* s*s*iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{689 /* __builtin_circ_sthhi */, 680 /* s*s*iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{710 /* __builtin_circ_stw */, 729 /* i*i*iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},

#endif // GET_BUILTIN_INFOS

#ifdef GET_BUILTIN_PREFIXED_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{738 /* A2_abs */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{745 /* A2_absp */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{760 /* A2_abssat */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{770 /* A2_add */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{781 /* A2_addh_h16_hh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{796 /* A2_addh_h16_hl */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{811 /* A2_addh_h16_lh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{826 /* A2_addh_h16_ll */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{841 /* A2_addh_h16_sat_hh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{860 /* A2_addh_h16_sat_hl */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{879 /* A2_addh_h16_sat_lh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{898 /* A2_addh_h16_sat_ll */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{917 /* A2_addh_l16_hl */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{932 /* A2_addh_l16_ll */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{947 /* A2_addh_l16_sat_hl */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{966 /* A2_addh_l16_sat_ll */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{985 /* A2_addi */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{998 /* A2_addp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1016 /* A2_addpsat */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1027 /* A2_addsat */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1037 /* A2_addsp */, 1046 /* LLiiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1054 /* A2_and */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1061 /* A2_andir */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1070 /* A2_andp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1078 /* A2_aslh */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1086 /* A2_asrh */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1094 /* A2_combine_hh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1108 /* A2_combine_hl */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1122 /* A2_combine_lh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1136 /* A2_combine_ll */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1150 /* A2_combineii */, 1163 /* LLiIiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1171 /* A2_combinew */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1189 /* A2_max */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1196 /* A2_maxp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1204 /* A2_maxu */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1217 /* A2_maxup */, 1226 /* ULLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1237 /* A2_min */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1244 /* A2_minp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1252 /* A2_minu */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1260 /* A2_minup */, 1226 /* ULLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1269 /* A2_neg */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1276 /* A2_negp */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1284 /* A2_negsat */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1294 /* A2_not */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1301 /* A2_notp */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1309 /* A2_or */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1315 /* A2_orir */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1323 /* A2_orp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1330 /* A2_roundsat */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1347 /* A2_sat */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1354 /* A2_satb */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1362 /* A2_sath */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1370 /* A2_satub */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1379 /* A2_satuh */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1388 /* A2_sub */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1395 /* A2_subh_h16_hh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1410 /* A2_subh_h16_hl */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1425 /* A2_subh_h16_lh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1440 /* A2_subh_h16_ll */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1455 /* A2_subh_h16_sat_hh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1474 /* A2_subh_h16_sat_hl */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1493 /* A2_subh_h16_sat_lh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1512 /* A2_subh_h16_sat_ll */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1531 /* A2_subh_l16_hl */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1546 /* A2_subh_l16_ll */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1561 /* A2_subh_l16_sat_hl */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1580 /* A2_subh_l16_sat_ll */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1599 /* A2_subp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1607 /* A2_subri */, 1616 /* iIii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1621 /* A2_subsat */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1631 /* A2_svaddh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1641 /* A2_svaddhs */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1652 /* A2_svadduhs */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1664 /* A2_svavgh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1674 /* A2_svavghs */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1685 /* A2_svnavgh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1696 /* A2_svsubh */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1706 /* A2_svsubhs */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1717 /* A2_svsubuhs */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1729 /* A2_swiz */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1737 /* A2_sxtb */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1745 /* A2_sxth */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1753 /* A2_sxtw */, 1761 /* LLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1766 /* A2_tfr */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1773 /* A2_tfrih */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1788 /* A2_tfril */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1797 /* A2_tfrp */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1805 /* A2_tfrpi */, 1814 /* LLiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1820 /* A2_tfrsi */, 1829 /* iIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1833 /* A2_vabsh */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1842 /* A2_vabshsat */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1854 /* A2_vabsw */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1863 /* A2_vabswsat */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1875 /* A2_vaddb_map */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1888 /* A2_vaddh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1897 /* A2_vaddhs */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1907 /* A2_vaddub */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1917 /* A2_vaddubs */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1928 /* A2_vadduhs */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1939 /* A2_vaddw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1948 /* A2_vaddws */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1958 /* A2_vavgh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1967 /* A2_vavghcr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1978 /* A2_vavghr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1988 /* A2_vavgub */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1998 /* A2_vavgubr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2009 /* A2_vavguh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2019 /* A2_vavguhr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2030 /* A2_vavguw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2040 /* A2_vavguwr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2051 /* A2_vavgw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2060 /* A2_vavgwcr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2071 /* A2_vavgwr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2081 /* A2_vcmpbeq */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2100 /* A2_vcmpbgtu */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2112 /* A2_vcmpheq */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2123 /* A2_vcmphgt */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2134 /* A2_vcmphgtu */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2146 /* A2_vcmpweq */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2157 /* A2_vcmpwgt */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2168 /* A2_vcmpwgtu */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2180 /* A2_vconj */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2189 /* A2_vmaxb */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2198 /* A2_vmaxh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2207 /* A2_vmaxub */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2217 /* A2_vmaxuh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2227 /* A2_vmaxuw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2237 /* A2_vmaxw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2246 /* A2_vminb */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2255 /* A2_vminh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2264 /* A2_vminub */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2274 /* A2_vminuh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2284 /* A2_vminuw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2294 /* A2_vminw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2303 /* A2_vnavgh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2313 /* A2_vnavghcr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2325 /* A2_vnavghr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2336 /* A2_vnavgw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2346 /* A2_vnavgwcr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2358 /* A2_vnavgwr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2369 /* A2_vraddub */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2380 /* A2_vraddub_acc */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2408 /* A2_vrsadub */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2419 /* A2_vrsadub_acc */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2434 /* A2_vsubb_map */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2447 /* A2_vsubh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2456 /* A2_vsubhs */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2466 /* A2_vsubub */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2476 /* A2_vsububs */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2487 /* A2_vsubuhs */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2498 /* A2_vsubw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2507 /* A2_vsubws */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2517 /* A2_xor */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2524 /* A2_xorp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2532 /* A2_zxtb */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2540 /* A2_zxth */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2548 /* A4_andn */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2556 /* A4_andnp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2565 /* A4_bitsplit */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2577 /* A4_bitspliti */, 2590 /* LLiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2598 /* A4_boundscheck */, 2613 /* iiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2619 /* A4_cmpbeq */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2629 /* A4_cmpbeqi */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2640 /* A4_cmpbgt */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2650 /* A4_cmpbgti */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2661 /* A4_cmpbgtu */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2672 /* A4_cmpbgtui */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2684 /* A4_cmpheq */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2694 /* A4_cmpheqi */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2705 /* A4_cmphgt */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2715 /* A4_cmphgti */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2726 /* A4_cmphgtu */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2737 /* A4_cmphgtui */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2749 /* A4_combineir */, 2762 /* LLiIii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2769 /* A4_combineri */, 2782 /* LLiiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2789 /* A4_cround_ri */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2802 /* A4_cround_rr */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2815 /* A4_modwrapu */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2827 /* A4_orn */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2834 /* A4_ornp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2842 /* A4_rcmpeq */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2852 /* A4_rcmpeqi */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2863 /* A4_rcmpneq */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2874 /* A4_rcmpneqi */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2886 /* A4_round_ri */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2898 /* A4_round_ri_sat */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2914 /* A4_round_rr */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2926 /* A4_round_rr_sat */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2942 /* A4_tlbmatch */, 2954 /* iLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2960 /* A4_vcmpbeq_any */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2975 /* A4_vcmpbeqi */, 2987 /* iLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2995 /* A4_vcmpbgt */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3006 /* A4_vcmpbgti */, 3018 /* iLLiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3025 /* A4_vcmpbgtui */, 2987 /* iLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3038 /* A4_vcmpheqi */, 3018 /* iLLiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3050 /* A4_vcmphgti */, 3018 /* iLLiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3062 /* A4_vcmphgtui */, 2987 /* iLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3075 /* A4_vcmpweqi */, 3018 /* iLLiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3087 /* A4_vcmpwgti */, 3018 /* iLLiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3099 /* A4_vcmpwgtui */, 2987 /* iLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3112 /* A4_vrmaxh */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3133 /* A4_vrmaxuh */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3144 /* A4_vrmaxuw */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3155 /* A4_vrmaxw */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3165 /* A4_vrminh */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3175 /* A4_vrminuh */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3186 /* A4_vrminuw */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3197 /* A4_vrminw */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3207 /* A5_vaddhubs */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3219 /* A6_vcmpbeq_notany */, 2092 /* iLLiLLi */, 0 /*  */, 3237 /* v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3277 /* A6_vminub_RdP */, 1006 /* LLiLLiLLi */, 0 /*  */, 3291 /* v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3335 /* A7_clip */, 1782 /* iiUIi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3349 /* A7_croundd_ri */, 3363 /* LLiLLiUIi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3373 /* A7_croundd_rr */, 3387 /* LLiLLii */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3395 /* A7_vclip */, 3363 /* LLiLLiUIi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3404 /* C2_all8 */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3412 /* C2_and */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3419 /* C2_andn */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3427 /* C2_any8 */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3435 /* C2_bitsclr */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3446 /* C2_bitsclri */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3458 /* C2_bitsset */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3469 /* C2_cmpeq */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3478 /* C2_cmpeqi */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3488 /* C2_cmpeqp */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3498 /* C2_cmpgei */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3508 /* C2_cmpgeui */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3519 /* C2_cmpgt */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3528 /* C2_cmpgti */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3538 /* C2_cmpgtp */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3548 /* C2_cmpgtu */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3558 /* C2_cmpgtui */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3569 /* C2_cmpgtup */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3580 /* C2_cmplt */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3589 /* C2_cmpltu */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3599 /* C2_mask */, 1761 /* LLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3607 /* C2_mux */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3619 /* C2_muxii */, 3628 /* iiIiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3635 /* C2_muxir */, 3644 /* iiiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3650 /* C2_muxri */, 3659 /* iiIii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3665 /* C2_not */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3672 /* C2_or */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3678 /* C2_orn */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3685 /* C2_pxfer_map */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3698 /* C2_tfrpr */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3707 /* C2_tfrrp */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3716 /* C2_vitpack */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3727 /* C2_vmux */, 3735 /* LLiiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3746 /* C2_xor */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3753 /* C4_and_and */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3764 /* C4_and_andn */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3776 /* C4_and_or */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3786 /* C4_and_orn */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3797 /* C4_cmplte */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3807 /* C4_cmpltei */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3818 /* C4_cmplteu */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3829 /* C4_cmplteui */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3841 /* C4_cmpneq */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3851 /* C4_cmpneqi */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3862 /* C4_fastcorner9 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3877 /* C4_fastcorner9_not */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3896 /* C4_nbitsclr */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3908 /* C4_nbitsclri */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3921 /* C4_nbitsset */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3933 /* C4_or_and */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3943 /* C4_or_andn */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3954 /* C4_or_or */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3963 /* C4_or_orn */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3973 /* F2_conv_d2df */, 3986 /* dLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3991 /* F2_conv_d2sf */, 4004 /* fLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4009 /* F2_conv_df2d */, 4022 /* LLid */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4027 /* F2_conv_df2d_chop */, 4022 /* LLid */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4045 /* F2_conv_df2sf */, 4059 /* fd */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4062 /* F2_conv_df2ud */, 4022 /* LLid */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4076 /* F2_conv_df2ud_chop */, 4022 /* LLid */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4095 /* F2_conv_df2uw */, 4109 /* id */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4112 /* F2_conv_df2uw_chop */, 4109 /* id */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4131 /* F2_conv_df2w */, 4109 /* id */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4144 /* F2_conv_df2w_chop */, 4109 /* id */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4162 /* F2_conv_sf2d */, 4175 /* LLif */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4180 /* F2_conv_sf2d_chop */, 4175 /* LLif */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4198 /* F2_conv_sf2df */, 4212 /* df */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4215 /* F2_conv_sf2ud */, 4175 /* LLif */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4229 /* F2_conv_sf2ud_chop */, 4175 /* LLif */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4248 /* F2_conv_sf2uw */, 4262 /* if */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4265 /* F2_conv_sf2uw_chop */, 4262 /* if */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4284 /* F2_conv_sf2w */, 4262 /* if */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4297 /* F2_conv_sf2w_chop */, 4262 /* if */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4315 /* F2_conv_ud2df */, 3986 /* dLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4329 /* F2_conv_ud2sf */, 4004 /* fLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4343 /* F2_conv_uw2df */, 4357 /* di */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4360 /* F2_conv_uw2sf */, 4374 /* fi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4377 /* F2_conv_w2df */, 4357 /* di */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4390 /* F2_conv_w2sf */, 4374 /* fi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4403 /* F2_dfadd */, 4412 /* ddd */, 0 /*  */, 4416 /* v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4452 /* F2_dfclass */, 4463 /* idUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4469 /* F2_dfcmpeq */, 4480 /* idd */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4484 /* F2_dfcmpge */, 4480 /* idd */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4495 /* F2_dfcmpgt */, 4480 /* idd */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4506 /* F2_dfcmpuo */, 4480 /* idd */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4517 /* F2_dfimm_n */, 4528 /* dUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4533 /* F2_dfimm_p */, 4528 /* dUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4544 /* F2_dfmax */, 4412 /* ddd */, 0 /*  */, 4553 /* v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4585 /* F2_dfmin */, 4412 /* ddd */, 0 /*  */, 4553 /* v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4594 /* F2_dfmpyfix */, 4412 /* ddd */, 0 /*  */, 4553 /* v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4606 /* F2_dfmpyhh */, 4617 /* dddd */, 0 /*  */, 4553 /* v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4622 /* F2_dfmpylh */, 4617 /* dddd */, 0 /*  */, 4553 /* v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4633 /* F2_dfmpyll */, 4412 /* ddd */, 0 /*  */, 4553 /* v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4644 /* F2_dfsub */, 4412 /* ddd */, 0 /*  */, 4416 /* v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4653 /* F2_sfadd */, 4662 /* fff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4666 /* F2_sfclass */, 4677 /* ifUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4683 /* F2_sfcmpeq */, 4694 /* iff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4698 /* F2_sfcmpge */, 4694 /* iff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4709 /* F2_sfcmpgt */, 4694 /* iff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4720 /* F2_sfcmpuo */, 4694 /* iff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4731 /* F2_sffixupd */, 4662 /* fff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4743 /* F2_sffixupn */, 4662 /* fff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4755 /* F2_sffixupr */, 4767 /* ff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4770 /* F2_sffma */, 4779 /* ffff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4784 /* F2_sffma_lib */, 4779 /* ffff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4797 /* F2_sffma_sc */, 4809 /* ffffi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4815 /* F2_sffms */, 4779 /* ffff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4824 /* F2_sffms_lib */, 4779 /* ffff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4837 /* F2_sfimm_n */, 4848 /* fUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4853 /* F2_sfimm_p */, 4848 /* fUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4864 /* F2_sfmax */, 4662 /* fff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4873 /* F2_sfmin */, 4662 /* fff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4882 /* F2_sfmpy */, 4662 /* fff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4891 /* F2_sfsub */, 4662 /* fff */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4900 /* L2_loadrb_pci */, 4914 /* iv*IiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4924 /* L2_loadrb_pcr */, 4938 /* iv*ivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4946 /* L2_loadrd_pci */, 4960 /* LLiv*IiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4972 /* L2_loadrd_pcr */, 4986 /* LLiv*ivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4996 /* L2_loadrh_pci */, 4914 /* iv*IiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5010 /* L2_loadrh_pcr */, 4938 /* iv*ivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5024 /* L2_loadri_pci */, 4914 /* iv*IiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5038 /* L2_loadri_pcr */, 4938 /* iv*ivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5052 /* L2_loadrub_pci */, 4914 /* iv*IiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5067 /* L2_loadrub_pcr */, 4938 /* iv*ivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5082 /* L2_loadruh_pci */, 4914 /* iv*IiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5097 /* L2_loadruh_pcr */, 4938 /* iv*ivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5112 /* M2_acci */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5120 /* M2_accii */, 3644 /* iiiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5129 /* M2_cmaci_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5150 /* M2_cmacr_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5162 /* M2_cmacs_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5174 /* M2_cmacs_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5186 /* M2_cmacsc_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5199 /* M2_cmacsc_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5212 /* M2_cmpyi_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5224 /* M2_cmpyr_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5236 /* M2_cmpyrs_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5249 /* M2_cmpyrs_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5262 /* M2_cmpyrsc_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5276 /* M2_cmpyrsc_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5290 /* M2_cmpys_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5302 /* M2_cmpys_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5314 /* M2_cmpysc_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5327 /* M2_cmpysc_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5340 /* M2_cnacs_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5352 /* M2_cnacs_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5364 /* M2_cnacsc_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5377 /* M2_cnacsc_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5390 /* M2_dpmpyss_acc_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5408 /* M2_dpmpyss_nac_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5426 /* M2_dpmpyss_rnd_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5444 /* M2_dpmpyss_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5458 /* M2_dpmpyuu_acc_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5476 /* M2_dpmpyuu_nac_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5494 /* M2_dpmpyuu_s0 */, 5508 /* ULLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5515 /* M2_hmmpyh_rs1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5529 /* M2_hmmpyh_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5542 /* M2_hmmpyl_rs1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5556 /* M2_hmmpyl_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5569 /* M2_maci */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5577 /* M2_macsin */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5594 /* M2_macsip */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5604 /* M2_mmachs_rs0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5618 /* M2_mmachs_rs1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5632 /* M2_mmachs_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5645 /* M2_mmachs_s1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5658 /* M2_mmacls_rs0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5672 /* M2_mmacls_rs1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5686 /* M2_mmacls_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5699 /* M2_mmacls_s1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5712 /* M2_mmacuhs_rs0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5727 /* M2_mmacuhs_rs1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5742 /* M2_mmacuhs_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5756 /* M2_mmacuhs_s1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5770 /* M2_mmaculs_rs0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5785 /* M2_mmaculs_rs1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5800 /* M2_mmaculs_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5814 /* M2_mmaculs_s1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5828 /* M2_mmpyh_rs0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5841 /* M2_mmpyh_rs1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5854 /* M2_mmpyh_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5866 /* M2_mmpyh_s1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5878 /* M2_mmpyl_rs0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5891 /* M2_mmpyl_rs1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5904 /* M2_mmpyl_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5916 /* M2_mmpyl_s1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5928 /* M2_mmpyuh_rs0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5942 /* M2_mmpyuh_rs1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5956 /* M2_mmpyuh_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5969 /* M2_mmpyuh_s1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5982 /* M2_mmpyul_rs0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5996 /* M2_mmpyul_rs1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6010 /* M2_mmpyul_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6023 /* M2_mmpyul_s1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6036 /* M2_mnaci */, 3614 /* iiii */, 0 /*  */, 4416 /* v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6045 /* M2_mpy_acc_hh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6062 /* M2_mpy_acc_hh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6079 /* M2_mpy_acc_hl_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6096 /* M2_mpy_acc_hl_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6113 /* M2_mpy_acc_lh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6130 /* M2_mpy_acc_lh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6147 /* M2_mpy_acc_ll_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6164 /* M2_mpy_acc_ll_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6181 /* M2_mpy_acc_sat_hh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6202 /* M2_mpy_acc_sat_hh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6223 /* M2_mpy_acc_sat_hl_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6244 /* M2_mpy_acc_sat_hl_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6265 /* M2_mpy_acc_sat_lh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6286 /* M2_mpy_acc_sat_lh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6307 /* M2_mpy_acc_sat_ll_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6328 /* M2_mpy_acc_sat_ll_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6349 /* M2_mpy_hh_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6362 /* M2_mpy_hh_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6375 /* M2_mpy_hl_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6388 /* M2_mpy_hl_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6401 /* M2_mpy_lh_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6414 /* M2_mpy_lh_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6427 /* M2_mpy_ll_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6440 /* M2_mpy_ll_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6453 /* M2_mpy_nac_hh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6470 /* M2_mpy_nac_hh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6487 /* M2_mpy_nac_hl_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6504 /* M2_mpy_nac_hl_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6521 /* M2_mpy_nac_lh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6538 /* M2_mpy_nac_lh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6555 /* M2_mpy_nac_ll_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6572 /* M2_mpy_nac_ll_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6589 /* M2_mpy_nac_sat_hh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6610 /* M2_mpy_nac_sat_hh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6631 /* M2_mpy_nac_sat_hl_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6652 /* M2_mpy_nac_sat_hl_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6673 /* M2_mpy_nac_sat_lh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6694 /* M2_mpy_nac_sat_lh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6715 /* M2_mpy_nac_sat_ll_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6736 /* M2_mpy_nac_sat_ll_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6757 /* M2_mpy_rnd_hh_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6774 /* M2_mpy_rnd_hh_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6791 /* M2_mpy_rnd_hl_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6808 /* M2_mpy_rnd_hl_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6825 /* M2_mpy_rnd_lh_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6842 /* M2_mpy_rnd_lh_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6859 /* M2_mpy_rnd_ll_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6876 /* M2_mpy_rnd_ll_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6893 /* M2_mpy_sat_hh_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6910 /* M2_mpy_sat_hh_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6927 /* M2_mpy_sat_hl_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6944 /* M2_mpy_sat_hl_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6961 /* M2_mpy_sat_lh_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6978 /* M2_mpy_sat_lh_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6995 /* M2_mpy_sat_ll_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7012 /* M2_mpy_sat_ll_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7029 /* M2_mpy_sat_rnd_hh_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7050 /* M2_mpy_sat_rnd_hh_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7071 /* M2_mpy_sat_rnd_hl_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7092 /* M2_mpy_sat_rnd_hl_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7113 /* M2_mpy_sat_rnd_lh_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7134 /* M2_mpy_sat_rnd_lh_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7155 /* M2_mpy_sat_rnd_ll_s0 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7176 /* M2_mpy_sat_rnd_ll_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7197 /* M2_mpy_up */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7207 /* M2_mpy_up_s1 */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7220 /* M2_mpy_up_s1_sat */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7237 /* M2_mpyd_acc_hh_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7255 /* M2_mpyd_acc_hh_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7273 /* M2_mpyd_acc_hl_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7291 /* M2_mpyd_acc_hl_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7309 /* M2_mpyd_acc_lh_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7327 /* M2_mpyd_acc_lh_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7345 /* M2_mpyd_acc_ll_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7363 /* M2_mpyd_acc_ll_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7381 /* M2_mpyd_hh_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7395 /* M2_mpyd_hh_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7409 /* M2_mpyd_hl_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7423 /* M2_mpyd_hl_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7437 /* M2_mpyd_lh_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7451 /* M2_mpyd_lh_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7465 /* M2_mpyd_ll_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7479 /* M2_mpyd_ll_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7493 /* M2_mpyd_nac_hh_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7511 /* M2_mpyd_nac_hh_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7529 /* M2_mpyd_nac_hl_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7547 /* M2_mpyd_nac_hl_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7565 /* M2_mpyd_nac_lh_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7583 /* M2_mpyd_nac_lh_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7601 /* M2_mpyd_nac_ll_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7619 /* M2_mpyd_nac_ll_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7637 /* M2_mpyd_rnd_hh_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7655 /* M2_mpyd_rnd_hh_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7673 /* M2_mpyd_rnd_hl_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7691 /* M2_mpyd_rnd_hl_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7709 /* M2_mpyd_rnd_lh_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7727 /* M2_mpyd_rnd_lh_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7745 /* M2_mpyd_rnd_ll_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7763 /* M2_mpyd_rnd_ll_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7781 /* M2_mpyi */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7789 /* M2_mpysmi */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7799 /* M2_mpysu_up */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7811 /* M2_mpyu_acc_hh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7829 /* M2_mpyu_acc_hh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7847 /* M2_mpyu_acc_hl_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7865 /* M2_mpyu_acc_hl_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7883 /* M2_mpyu_acc_lh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7901 /* M2_mpyu_acc_lh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7919 /* M2_mpyu_acc_ll_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7937 /* M2_mpyu_acc_ll_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7955 /* M2_mpyu_hh_s0 */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7969 /* M2_mpyu_hh_s1 */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7983 /* M2_mpyu_hl_s0 */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7997 /* M2_mpyu_hl_s1 */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8011 /* M2_mpyu_lh_s0 */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8025 /* M2_mpyu_lh_s1 */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8039 /* M2_mpyu_ll_s0 */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8053 /* M2_mpyu_ll_s1 */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8067 /* M2_mpyu_nac_hh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8085 /* M2_mpyu_nac_hh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8103 /* M2_mpyu_nac_hl_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8121 /* M2_mpyu_nac_hl_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8139 /* M2_mpyu_nac_lh_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8157 /* M2_mpyu_nac_lh_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8175 /* M2_mpyu_nac_ll_s0 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8193 /* M2_mpyu_nac_ll_s1 */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8211 /* M2_mpyu_up */, 1212 /* Uiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8222 /* M2_mpyud_acc_hh_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8241 /* M2_mpyud_acc_hh_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8260 /* M2_mpyud_acc_hl_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8279 /* M2_mpyud_acc_hl_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8298 /* M2_mpyud_acc_lh_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8317 /* M2_mpyud_acc_lh_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8336 /* M2_mpyud_acc_ll_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8355 /* M2_mpyud_acc_ll_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8374 /* M2_mpyud_hh_s0 */, 5508 /* ULLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8389 /* M2_mpyud_hh_s1 */, 5508 /* ULLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8404 /* M2_mpyud_hl_s0 */, 5508 /* ULLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8419 /* M2_mpyud_hl_s1 */, 5508 /* ULLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8434 /* M2_mpyud_lh_s0 */, 5508 /* ULLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8449 /* M2_mpyud_lh_s1 */, 5508 /* ULLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8464 /* M2_mpyud_ll_s0 */, 5508 /* ULLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8479 /* M2_mpyud_ll_s1 */, 5508 /* ULLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8494 /* M2_mpyud_nac_hh_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8513 /* M2_mpyud_nac_hh_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8532 /* M2_mpyud_nac_hl_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8551 /* M2_mpyud_nac_hl_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8570 /* M2_mpyud_nac_lh_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8589 /* M2_mpyud_nac_lh_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8608 /* M2_mpyud_nac_ll_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8627 /* M2_mpyud_nac_ll_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8646 /* M2_mpyui */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8655 /* M2_nacci */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8664 /* M2_naccii */, 3644 /* iiiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8674 /* M2_subacc */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8684 /* M2_vabsdiffh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8697 /* M2_vabsdiffw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8710 /* M2_vcmac_s0_sat_i */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8728 /* M2_vcmac_s0_sat_r */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8746 /* M2_vcmpy_s0_sat_i */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8764 /* M2_vcmpy_s0_sat_r */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8782 /* M2_vcmpy_s1_sat_i */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8800 /* M2_vcmpy_s1_sat_r */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8818 /* M2_vdmacs_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8831 /* M2_vdmacs_s1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8844 /* M2_vdmpyrs_s0 */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8858 /* M2_vdmpyrs_s1 */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8872 /* M2_vdmpys_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8885 /* M2_vdmpys_s1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8898 /* M2_vmac2 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8907 /* M2_vmac2es */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8918 /* M2_vmac2es_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8932 /* M2_vmac2es_s1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8946 /* M2_vmac2s_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8959 /* M2_vmac2s_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8972 /* M2_vmac2su_s0 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8986 /* M2_vmac2su_s1 */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9000 /* M2_vmpy2es_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9014 /* M2_vmpy2es_s1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9028 /* M2_vmpy2s_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9041 /* M2_vmpy2s_s0pack */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9058 /* M2_vmpy2s_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9071 /* M2_vmpy2s_s1pack */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9088 /* M2_vmpy2su_s0 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9102 /* M2_vmpy2su_s1 */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9116 /* M2_vraddh */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9126 /* M2_vradduh */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9137 /* M2_vrcmaci_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9151 /* M2_vrcmaci_s0c */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9166 /* M2_vrcmacr_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9180 /* M2_vrcmacr_s0c */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9195 /* M2_vrcmpyi_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9209 /* M2_vrcmpyi_s0c */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9224 /* M2_vrcmpyr_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9238 /* M2_vrcmpyr_s0c */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9253 /* M2_vrcmpys_acc_s1 */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9271 /* M2_vrcmpys_s1 */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9285 /* M2_vrcmpys_s1rp */, 2954 /* iLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9301 /* M2_vrmac_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9313 /* M2_vrmpy_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9325 /* M2_xor_xacc */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9337 /* M4_and_and */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9348 /* M4_and_andn */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9360 /* M4_and_or */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9370 /* M4_and_xor */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9381 /* M4_cmpyi_wh */, 2954 /* iLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9393 /* M4_cmpyi_whc */, 2954 /* iLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9406 /* M4_cmpyr_wh */, 2954 /* iLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9418 /* M4_cmpyr_whc */, 2954 /* iLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9431 /* M4_mac_up_s1_sat */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9448 /* M4_mpyri_addi */, 9462 /* iUIiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9471 /* M4_mpyri_addr */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9485 /* M4_mpyri_addr_u2 */, 9502 /* iiUIii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9509 /* M4_mpyrr_addi */, 9523 /* iUIiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9530 /* M4_mpyrr_addr */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9544 /* M4_nac_up_s1_sat */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9561 /* M4_or_and */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9571 /* M4_or_andn */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9582 /* M4_or_or */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9591 /* M4_or_xor */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9601 /* M4_pmpyw */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9610 /* M4_pmpyw_acc */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9623 /* M4_vpmpyh */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9633 /* M4_vpmpyh_acc */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9647 /* M4_vrmpyeh_acc_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9665 /* M4_vrmpyeh_acc_s1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9683 /* M4_vrmpyeh_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9697 /* M4_vrmpyeh_s1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9711 /* M4_vrmpyoh_acc_s0 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9729 /* M4_vrmpyoh_acc_s1 */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9747 /* M4_vrmpyoh_s0 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9761 /* M4_vrmpyoh_s1 */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9775 /* M4_xor_and */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9786 /* M4_xor_andn */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9798 /* M4_xor_or */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9808 /* M4_xor_xacc */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9820 /* M5_vdmacbsu */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9832 /* M5_vdmpybsu */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9844 /* M5_vmacbsu */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9855 /* M5_vmacbuu */, 5141 /* LLiLLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9866 /* M5_vmpybsu */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9877 /* M5_vmpybuu */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9888 /* M5_vrmacbsu */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9900 /* M5_vrmacbuu */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9912 /* M5_vrmpybsu */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9924 /* M5_vrmpybuu */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9936 /* M6_vabsdiffb */, 1006 /* LLiLLiLLi */, 0 /*  */, 3291 /* v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9949 /* M6_vabsdiffub */, 1006 /* LLiLLiLLi */, 0 /*  */, 3291 /* v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9963 /* M7_dcmpyiw */, 1006 /* LLiLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9974 /* M7_dcmpyiw_acc */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9989 /* M7_dcmpyiwc */, 1006 /* LLiLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10001 /* M7_dcmpyiwc_acc */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10017 /* M7_dcmpyrw */, 1006 /* LLiLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10028 /* M7_dcmpyrw_acc */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10043 /* M7_dcmpyrwc */, 1006 /* LLiLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10055 /* M7_dcmpyrwc_acc */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10071 /* M7_vdmpy */, 1006 /* LLiLLiLLi */, 0 /*  */, 4553 /* v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10080 /* M7_vdmpy_acc */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 4553 /* v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10093 /* M7_wcmpyiw */, 2092 /* iLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10104 /* M7_wcmpyiw_rnd */, 2092 /* iLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10119 /* M7_wcmpyiwc */, 2092 /* iLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10131 /* M7_wcmpyiwc_rnd */, 2092 /* iLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10147 /* M7_wcmpyrw */, 2092 /* iLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10158 /* M7_wcmpyrw_rnd */, 2092 /* iLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10173 /* M7_wcmpyrwc */, 2092 /* iLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10185 /* M7_wcmpyrwc_rnd */, 2092 /* iLLiLLi */, 0 /*  */, 3343 /* audio */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10201 /* S2_addasl_rrri */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10216 /* S2_asl_i_p */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10227 /* S2_asl_i_p_acc */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10255 /* S2_asl_i_p_and */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10270 /* S2_asl_i_p_nac */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10285 /* S2_asl_i_p_or */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10299 /* S2_asl_i_p_xacc */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10315 /* S2_asl_i_r */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10326 /* S2_asl_i_r_acc */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10341 /* S2_asl_i_r_and */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10356 /* S2_asl_i_r_nac */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10371 /* S2_asl_i_r_or */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10385 /* S2_asl_i_r_sat */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10400 /* S2_asl_i_r_xacc */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10416 /* S2_asl_i_vh */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10428 /* S2_asl_i_vw */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10440 /* S2_asl_r_p */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10451 /* S2_asl_r_p_acc */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10466 /* S2_asl_r_p_and */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10481 /* S2_asl_r_p_nac */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10496 /* S2_asl_r_p_or */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10510 /* S2_asl_r_p_xor */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10525 /* S2_asl_r_r */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10536 /* S2_asl_r_r_acc */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10551 /* S2_asl_r_r_and */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10566 /* S2_asl_r_r_nac */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10581 /* S2_asl_r_r_or */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10595 /* S2_asl_r_r_sat */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10610 /* S2_asl_r_vh */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10622 /* S2_asl_r_vw */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10634 /* S2_asr_i_p */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10645 /* S2_asr_i_p_acc */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10660 /* S2_asr_i_p_and */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10675 /* S2_asr_i_p_nac */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10690 /* S2_asr_i_p_or */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10704 /* S2_asr_i_p_rnd */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10719 /* S2_asr_i_p_rnd_goodsyntax */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10745 /* S2_asr_i_r */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10756 /* S2_asr_i_r_acc */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10771 /* S2_asr_i_r_and */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10786 /* S2_asr_i_r_nac */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10801 /* S2_asr_i_r_or */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10815 /* S2_asr_i_r_rnd */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10830 /* S2_asr_i_r_rnd_goodsyntax */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10856 /* S2_asr_i_svw_trun */, 2987 /* iLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10874 /* S2_asr_i_vh */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10886 /* S2_asr_i_vw */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10898 /* S2_asr_r_p */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10909 /* S2_asr_r_p_acc */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10924 /* S2_asr_r_p_and */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10939 /* S2_asr_r_p_nac */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10954 /* S2_asr_r_p_or */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10968 /* S2_asr_r_p_xor */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10983 /* S2_asr_r_r */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10994 /* S2_asr_r_r_acc */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11009 /* S2_asr_r_r_and */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11024 /* S2_asr_r_r_nac */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11039 /* S2_asr_r_r_or */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11053 /* S2_asr_r_r_sat */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11068 /* S2_asr_r_svw_trun */, 2954 /* iLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11086 /* S2_asr_r_vh */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11098 /* S2_asr_r_vw */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11110 /* S2_brev */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11118 /* S2_brevp */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11127 /* S2_cl0 */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11134 /* S2_cl0p */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11142 /* S2_cl1 */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11149 /* S2_cl1p */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11157 /* S2_clb */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11164 /* S2_clbnorm */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11175 /* S2_clbp */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11183 /* S2_clrbit_i */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11195 /* S2_clrbit_r */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11207 /* S2_ct0 */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11214 /* S2_ct0p */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11222 /* S2_ct1 */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11229 /* S2_ct1p */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11237 /* S2_deinterleave */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11253 /* S2_extractu */, 11265 /* iiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11274 /* S2_extractu_rp */, 2613 /* iiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11289 /* S2_extractup */, 11302 /* LLiLLiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11315 /* S2_extractup_rp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11331 /* S2_insert */, 11341 /* iiiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11351 /* S2_insert_rp */, 11364 /* iiiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11371 /* S2_insertp */, 11382 /* LLiLLiLLiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11398 /* S2_insertp_rp */, 2395 /* LLiLLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11412 /* S2_interleave */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11426 /* S2_lfsp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11434 /* S2_lsl_r_p */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11445 /* S2_lsl_r_p_acc */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11460 /* S2_lsl_r_p_and */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11475 /* S2_lsl_r_p_nac */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11490 /* S2_lsl_r_p_or */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11504 /* S2_lsl_r_p_xor */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11519 /* S2_lsl_r_r */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11530 /* S2_lsl_r_r_acc */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11545 /* S2_lsl_r_r_and */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11560 /* S2_lsl_r_r_nac */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11575 /* S2_lsl_r_r_or */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11589 /* S2_lsl_r_vh */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11601 /* S2_lsl_r_vw */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11613 /* S2_lsr_i_p */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11624 /* S2_lsr_i_p_acc */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11639 /* S2_lsr_i_p_and */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11654 /* S2_lsr_i_p_nac */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11669 /* S2_lsr_i_p_or */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11683 /* S2_lsr_i_p_xacc */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11699 /* S2_lsr_i_r */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11710 /* S2_lsr_i_r_acc */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11725 /* S2_lsr_i_r_and */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11740 /* S2_lsr_i_r_nac */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11755 /* S2_lsr_i_r_or */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11769 /* S2_lsr_i_r_xacc */, 5587 /* iiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11785 /* S2_lsr_i_vh */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11797 /* S2_lsr_i_vw */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11809 /* S2_lsr_r_p */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11820 /* S2_lsr_r_p_acc */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11835 /* S2_lsr_r_p_and */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11850 /* S2_lsr_r_p_nac */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11865 /* S2_lsr_r_p_or */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11879 /* S2_lsr_r_p_xor */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11894 /* S2_lsr_r_r */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11905 /* S2_lsr_r_r_acc */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11920 /* S2_lsr_r_r_and */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11935 /* S2_lsr_r_r_nac */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11950 /* S2_lsr_r_r_or */, 3614 /* iiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11964 /* S2_lsr_r_vh */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11976 /* S2_lsr_r_vw */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11988 /* S2_mask */, 11996 /* iUIiUIi */, 0 /*  */, 4416 /* v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12004 /* S2_packhl */, 1183 /* LLiii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12014 /* S2_parityp */, 2092 /* iLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12025 /* S2_setbit_i */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12037 /* S2_setbit_r */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12049 /* S2_shuffeb */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12060 /* S2_shuffeh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12071 /* S2_shuffob */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12082 /* S2_shuffoh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12093 /* S2_storerb_pci */, 12108 /* vv*IiiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12119 /* S2_storerb_pcr */, 12134 /* vv*iivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12143 /* S2_storerd_pci */, 12158 /* vv*IiiLLivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12171 /* S2_storerd_pcr */, 12186 /* vv*iLLivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12197 /* S2_storerf_pci */, 12108 /* vv*IiiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12212 /* S2_storerf_pcr */, 12134 /* vv*iivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12227 /* S2_storerh_pci */, 12108 /* vv*IiiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12242 /* S2_storerh_pcr */, 12134 /* vv*iivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12257 /* S2_storeri_pci */, 12108 /* vv*IiiivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12272 /* S2_storeri_pcr */, 12134 /* vv*iivC* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12287 /* S2_svsathb */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12298 /* S2_svsathub */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12310 /* S2_tableidxb_goodsyntax */, 11341 /* iiiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12334 /* S2_tableidxd_goodsyntax */, 11341 /* iiiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12358 /* S2_tableidxh_goodsyntax */, 11341 /* iiiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12382 /* S2_tableidxw_goodsyntax */, 11341 /* iiiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12406 /* S2_togglebit_i */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12421 /* S2_togglebit_r */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12436 /* S2_tstbit_i */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12448 /* S2_tstbit_r */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12460 /* S2_valignib */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12472 /* S2_valignrb */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12484 /* S2_vcnegh */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12494 /* S2_vcrotate */, 3387 /* LLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12506 /* S2_vrcnegh */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12517 /* S2_vrndpackwh */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12531 /* S2_vrndpackwhs */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12546 /* S2_vsathb */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12556 /* S2_vsathb_nopack */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12573 /* S2_vsathub */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12584 /* S2_vsathub_nopack */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12602 /* S2_vsatwh */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12612 /* S2_vsatwh_nopack */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12629 /* S2_vsatwuh */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12640 /* S2_vsatwuh_nopack */, 753 /* LLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12658 /* S2_vsplatrb */, 28 /* ii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12670 /* S2_vsplatrh */, 1761 /* LLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12682 /* S2_vspliceib */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12695 /* S2_vsplicerb */, 3122 /* LLiLLiLLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12708 /* S2_vsxtbh */, 1761 /* LLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12718 /* S2_vsxthw */, 1761 /* LLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12728 /* S2_vtrunehb */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12740 /* S2_vtrunewh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12752 /* S2_vtrunohb */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12764 /* S2_vtrunowh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12776 /* S2_vzxtbh */, 1761 /* LLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12786 /* S2_vzxthw */, 1761 /* LLii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12796 /* S4_addaddi */, 3644 /* iiiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12807 /* S4_addi_asl_ri */, 9462 /* iUIiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12822 /* S4_addi_lsr_ri */, 9462 /* iUIiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12837 /* S4_andi_asl_ri */, 9462 /* iUIiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12852 /* S4_andi_lsr_ri */, 9462 /* iUIiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12867 /* S4_clbaddi */, 993 /* iiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12878 /* S4_clbpaddi */, 3018 /* iLLiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12890 /* S4_clbpnorm */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12902 /* S4_extract */, 11265 /* iiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12913 /* S4_extract_rp */, 2613 /* iiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12927 /* S4_extractp */, 11302 /* LLiLLiUIiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12939 /* S4_extractp_rp */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12954 /* S4_lsli */, 1616 /* iIii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12962 /* S4_ntstbit_i */, 1782 /* iiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12975 /* S4_ntstbit_r */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12988 /* S4_or_andi */, 3644 /* iiiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12999 /* S4_or_andix */, 3644 /* iiiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13011 /* S4_or_ori */, 3644 /* iiiIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13021 /* S4_ori_asl_ri */, 9462 /* iUIiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13035 /* S4_ori_lsr_ri */, 9462 /* iUIiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13049 /* S4_parity */, 777 /* iii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13059 /* S4_subaddi */, 3659 /* iiIii */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13070 /* S4_subi_asl_ri */, 9462 /* iUIiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13085 /* S4_subi_lsr_ri */, 9462 /* iUIiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13100 /* S4_vrcrotate */, 13113 /* LLiLLiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13124 /* S4_vrcrotate_acc */, 13141 /* LLiLLiLLiiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13155 /* S4_vxaddsubh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13168 /* S4_vxaddsubhr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13182 /* S4_vxaddsubw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13195 /* S4_vxsubaddh */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13208 /* S4_vxsubaddhr */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13222 /* S4_vxsubaddw */, 1006 /* LLiLLiLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13235 /* S5_asrhub_rnd_sat_goodsyntax */, 2987 /* iLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13264 /* S5_asrhub_sat */, 2987 /* iLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13278 /* S5_popcountp */, 1342 /* iLLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13291 /* S5_vasrhrnd_goodsyntax */, 3363 /* LLiLLiUIi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13314 /* S6_rol_i_p */, 3363 /* LLiLLiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13373 /* S6_rol_i_p_acc */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13388 /* S6_rol_i_p_and */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13403 /* S6_rol_i_p_nac */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13418 /* S6_rol_i_p_or */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13432 /* S6_rol_i_p_xacc */, 10242 /* LLiLLiLLiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13448 /* S6_rol_i_r */, 1782 /* iiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13459 /* S6_rol_i_r_acc */, 5587 /* iiiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13474 /* S6_rol_i_r_and */, 5587 /* iiiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13489 /* S6_rol_i_r_nac */, 5587 /* iiiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13504 /* S6_rol_i_r_or */, 5587 /* iiiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13518 /* S6_rol_i_r_xacc */, 5587 /* iiiUIi */, 0 /*  */, 13325 /* v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13534 /* S6_vsplatrbp */, 1761 /* LLii */, 0 /*  */, 3291 /* v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13547 /* S6_vtrunehb_ppp */, 1006 /* LLiLLiLLi */, 0 /*  */, 3291 /* v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13563 /* S6_vtrunohb_ppp */, 1006 /* LLiLLiLLi */, 0 /*  */, 3291 /* v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13579 /* V6_extractw */, 13591 /* iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13682 /* V6_extractw_128B */, 13699 /* iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13706 /* V6_get_qfext */, 13719 /* V16iV16ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13743 /* V6_get_qfext_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13771 /* V6_get_qfext_oracc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13804 /* V6_get_qfext_oracc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13842 /* V6_hi */, 13848 /* V16iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13857 /* V6_hi_128B */, 13868 /* V32iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13877 /* V6_lo */, 13848 /* V16iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13883 /* V6_lo_128B */, 13868 /* V32iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13894 /* V6_lvsplatb */, 13906 /* V16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13989 /* V6_lvsplatb_128B */, 14006 /* V32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14012 /* V6_lvsplath */, 13906 /* V16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14024 /* V6_lvsplath_128B */, 14006 /* V32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14041 /* V6_lvsplatw */, 13906 /* V16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14053 /* V6_lvsplatw_128B */, 14006 /* V32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14070 /* V6_pred_and */, 14082 /* V64bV64bV64b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14095 /* V6_pred_and_128B */, 14112 /* V128bV128bV128b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14128 /* V6_pred_and_n */, 14082 /* V64bV64bV64b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14142 /* V6_pred_and_n_128B */, 14112 /* V128bV128bV128b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14161 /* V6_pred_not */, 14173 /* V64bV64b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14182 /* V6_pred_not_128B */, 14199 /* V128bV128b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14210 /* V6_pred_or */, 14082 /* V64bV64bV64b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14221 /* V6_pred_or_128B */, 14112 /* V128bV128bV128b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14237 /* V6_pred_or_n */, 14082 /* V64bV64bV64b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14250 /* V6_pred_or_n_128B */, 14112 /* V128bV128bV128b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14268 /* V6_pred_scalar2 */, 14284 /* V64bi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14290 /* V6_pred_scalar2_128B */, 14311 /* V128bi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14318 /* V6_pred_scalar2v2 */, 14284 /* V64bi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14336 /* V6_pred_scalar2v2_128B */, 14311 /* V128bi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14359 /* V6_pred_xor */, 14082 /* V64bV64bV64b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14371 /* V6_pred_xor_128B */, 14112 /* V128bV128bV128b */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14388 /* V6_set_qfext */, 13719 /* V16iV16ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14401 /* V6_set_qfext_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14419 /* V6_shuffeqh */, 14082 /* V64bV64bV64b */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14431 /* V6_shuffeqh_128B */, 14112 /* V128bV128bV128b */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14448 /* V6_shuffeqw */, 14082 /* V64bV64bV64b */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14460 /* V6_shuffeqw_128B */, 14112 /* V128bV128bV128b */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14477 /* V6_v6mpyhubs10 */, 14492 /* V32iV32iV32iUIi */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14557 /* V6_v6mpyhubs10_128B */, 14577 /* V64iV64iV64iUIi */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14593 /* V6_v6mpyhubs10_vxx */, 14612 /* V32iV32iV32iV32iUIi */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14632 /* V6_v6mpyhubs10_vxx_128B */, 14656 /* V64iV64iV64iV64iUIi */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14676 /* V6_v6mpyvubs10 */, 14492 /* V32iV32iV32iUIi */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14691 /* V6_v6mpyvubs10_128B */, 14577 /* V64iV64iV64iUIi */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14711 /* V6_v6mpyvubs10_vxx */, 14612 /* V32iV32iV32iV32iUIi */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14730 /* V6_v6mpyvubs10_vxx_128B */, 14656 /* V64iV64iV64iV64iUIi */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14754 /* V6_vS32b_nqpred_ai */, 14773 /* vV64bv*V16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14785 /* V6_vS32b_nqpred_ai_128B */, 14809 /* vV128bv*V32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14822 /* V6_vS32b_nt_nqpred_ai */, 14773 /* vV64bv*V16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14844 /* V6_vS32b_nt_nqpred_ai_128B */, 14809 /* vV128bv*V32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14871 /* V6_vS32b_nt_qpred_ai */, 14773 /* vV64bv*V16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14892 /* V6_vS32b_nt_qpred_ai_128B */, 14809 /* vV128bv*V32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14918 /* V6_vS32b_qpred_ai */, 14773 /* vV64bv*V16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14936 /* V6_vS32b_qpred_ai_128B */, 14809 /* vV128bv*V32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14959 /* V6_vabs_f8 */, 14970 /* V16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14979 /* V6_vabs_f8_128B */, 14995 /* V32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15004 /* V6_vabs_hf */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15015 /* V6_vabs_hf_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15031 /* V6_vabs_qf16_hf */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15054 /* V6_vabs_qf16_hf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15075 /* V6_vabs_qf16_qf16 */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15093 /* V6_vabs_qf16_qf16_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15116 /* V6_vabs_qf32_qf32 */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15134 /* V6_vabs_qf32_qf32_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15157 /* V6_vabs_qf32_sf */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15173 /* V6_vabs_qf32_sf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15194 /* V6_vabs_sf */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15205 /* V6_vabs_sf_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15221 /* V6_vabsb */, 14970 /* V16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15300 /* V6_vabsb_128B */, 14995 /* V32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15314 /* V6_vabsb_sat */, 14970 /* V16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15327 /* V6_vabsb_sat_128B */, 14995 /* V32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15345 /* V6_vabsdiffh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15371 /* V6_vabsdiffh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15402 /* V6_vabsdiffub */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15416 /* V6_vabsdiffub_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15435 /* V6_vabsdiffuh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15449 /* V6_vabsdiffuh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15468 /* V6_vabsdiffw */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15481 /* V6_vabsdiffw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15499 /* V6_vabsh */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15508 /* V6_vabsh_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15522 /* V6_vabsh_sat */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15535 /* V6_vabsh_sat_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15553 /* V6_vabsw */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15562 /* V6_vabsw_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15576 /* V6_vabsw_sat */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15589 /* V6_vabsw_sat_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15607 /* V6_vadd_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15618 /* V6_vadd_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15634 /* V6_vadd_hf_f8 */, 15648 /* V32iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15661 /* V6_vadd_hf_f8_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15693 /* V6_vadd_hf_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15707 /* V6_vadd_hf_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15726 /* V6_vadd_qf16 */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15739 /* V6_vadd_qf16_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15757 /* V6_vadd_qf16_mix */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15774 /* V6_vadd_qf16_mix_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15796 /* V6_vadd_qf32 */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15809 /* V6_vadd_qf32_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15827 /* V6_vadd_qf32_mix */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15844 /* V6_vadd_qf32_mix_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15866 /* V6_vadd_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15877 /* V6_vadd_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15893 /* V6_vadd_sf_bf */, 15648 /* V32iV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15935 /* V6_vadd_sf_bf_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15954 /* V6_vadd_sf_hf */, 15648 /* V32iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15968 /* V6_vadd_sf_hf_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15987 /* V6_vadd_sf_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16001 /* V6_vadd_sf_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16020 /* V6_vaddb */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16029 /* V6_vaddb_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16043 /* V6_vaddb_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16055 /* V6_vaddb_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16085 /* V6_vaddbnq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16113 /* V6_vaddbnq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16147 /* V6_vaddbq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16157 /* V6_vaddbq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16172 /* V6_vaddbsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16184 /* V6_vaddbsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16201 /* V6_vaddbsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16216 /* V6_vaddbsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16236 /* V6_vaddcarry */, 16249 /* V16iV16iV16iv* */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16264 /* V6_vaddcarry_128B */, 16282 /* V32iV32iV32iv* */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16297 /* V6_vaddcarryo */, 16249 /* V16iV16iV16iv* */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16374 /* V6_vaddcarryo_128B */, 16282 /* V32iV32iV32iv* */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16393 /* V6_vaddcarrysat */, 16409 /* V16iV16iV16iV64b */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16426 /* V6_vaddcarrysat_128B */, 16447 /* V32iV32iV32iV128b */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16465 /* V6_vaddclbh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16477 /* V6_vaddclbh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16494 /* V6_vaddclbw */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16506 /* V6_vaddclbw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16523 /* V6_vaddh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16532 /* V6_vaddh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16546 /* V6_vaddh_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16558 /* V6_vaddh_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16575 /* V6_vaddhnq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16586 /* V6_vaddhnq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16602 /* V6_vaddhq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16612 /* V6_vaddhq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16627 /* V6_vaddhsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16639 /* V6_vaddhsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16656 /* V6_vaddhsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16671 /* V6_vaddhsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16691 /* V6_vaddhw */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16701 /* V6_vaddhw_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16716 /* V6_vaddhw_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16747 /* V6_vaddhw_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16783 /* V6_vaddubh */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16794 /* V6_vaddubh_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16810 /* V6_vaddubh_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16825 /* V6_vaddubh_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16845 /* V6_vaddubsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16858 /* V6_vaddubsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16876 /* V6_vaddubsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16892 /* V6_vaddubsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16913 /* V6_vaddububb_sat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16930 /* V6_vaddububb_sat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16952 /* V6_vadduhsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16965 /* V6_vadduhsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16983 /* V6_vadduhsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16999 /* V6_vadduhsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17020 /* V6_vadduhw */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17031 /* V6_vadduhw_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17047 /* V6_vadduhw_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17062 /* V6_vadduhw_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17082 /* V6_vadduwsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17095 /* V6_vadduwsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17113 /* V6_vadduwsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17129 /* V6_vadduwsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17150 /* V6_vaddw */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17159 /* V6_vaddw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17173 /* V6_vaddw_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17185 /* V6_vaddw_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17202 /* V6_vaddwnq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17213 /* V6_vaddwnq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17229 /* V6_vaddwq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17239 /* V6_vaddwq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17254 /* V6_vaddwsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17266 /* V6_vaddwsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17283 /* V6_vaddwsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17298 /* V6_vaddwsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17318 /* V6_valign4 */, 13790 /* V16iV16iV16ii */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17329 /* V6_valign4_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17345 /* V6_valignb */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17356 /* V6_valignb_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17372 /* V6_valignbi */, 17384 /* V16iV16iV16iUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17400 /* V6_valignbi_128B */, 14492 /* V32iV32iV32iUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17417 /* V6_vand */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17425 /* V6_vand_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17438 /* V6_vandnqrt */, 17450 /* V16iV64bi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17460 /* V6_vandnqrt_128B */, 17477 /* V32iV128bi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17488 /* V6_vandnqrt_acc */, 17504 /* V16iV16iV64bi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17518 /* V6_vandnqrt_acc_128B */, 17539 /* V32iV32iV128bi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17554 /* V6_vandqrt */, 17450 /* V16iV64bi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17565 /* V6_vandqrt_128B */, 17477 /* V32iV128bi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17581 /* V6_vandqrt_acc */, 17504 /* V16iV16iV64bi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17596 /* V6_vandqrt_acc_128B */, 17539 /* V32iV32iV128bi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17616 /* V6_vandvnqv */, 17628 /* V16iV64bV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17641 /* V6_vandvnqv_128B */, 17658 /* V32iV128bV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17672 /* V6_vandvqv */, 17628 /* V16iV64bV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17683 /* V6_vandvqv_128B */, 17658 /* V32iV128bV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17699 /* V6_vandvrt */, 17710 /* V64bV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17720 /* V6_vandvrt_128B */, 17736 /* V128bV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17747 /* V6_vandvrt_acc */, 17762 /* V64bV64bV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17776 /* V6_vandvrt_acc_128B */, 17796 /* V128bV128bV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17812 /* V6_vaslh */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17821 /* V6_vaslh_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17835 /* V6_vaslh_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17848 /* V6_vaslh_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17866 /* V6_vaslhv */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17876 /* V6_vaslhv_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17891 /* V6_vaslw */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17900 /* V6_vaslw_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17914 /* V6_vaslw_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17927 /* V6_vaslw_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17945 /* V6_vaslwv */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17955 /* V6_vaslwv_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17970 /* V6_vasr_into */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17983 /* V6_vasr_into_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18001 /* V6_vasrh */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18010 /* V6_vasrh_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18024 /* V6_vasrh_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18037 /* V6_vasrh_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18055 /* V6_vasrhbrndsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18071 /* V6_vasrhbrndsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18092 /* V6_vasrhbsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18105 /* V6_vasrhbsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18123 /* V6_vasrhubrndsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18140 /* V6_vasrhubrndsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18162 /* V6_vasrhubsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18176 /* V6_vasrhubsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18195 /* V6_vasrhv */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18205 /* V6_vasrhv_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18220 /* V6_vasruhubrndsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18238 /* V6_vasruhubrndsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18261 /* V6_vasruhubsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18276 /* V6_vasruhubsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18296 /* V6_vasruwuhrndsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18314 /* V6_vasruwuhrndsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18337 /* V6_vasruwuhsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18352 /* V6_vasruwuhsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18372 /* V6_vasrvuhubrndsat */, 18391 /* V16iV32iV16i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18446 /* V6_vasrvuhubrndsat_128B */, 18470 /* V32iV64iV32i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18483 /* V6_vasrvuhubsat */, 18391 /* V16iV32iV16i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18499 /* V6_vasrvuhubsat_128B */, 18470 /* V32iV64iV32i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18520 /* V6_vasrvwuhrndsat */, 18391 /* V16iV32iV16i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18538 /* V6_vasrvwuhrndsat_128B */, 18470 /* V32iV64iV32i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18561 /* V6_vasrvwuhsat */, 18391 /* V16iV32iV16i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18576 /* V6_vasrvwuhsat_128B */, 18470 /* V32iV64iV32i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18596 /* V6_vasrw */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18605 /* V6_vasrw_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18619 /* V6_vasrw_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18632 /* V6_vasrw_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18650 /* V6_vasrwh */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18660 /* V6_vasrwh_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18675 /* V6_vasrwhrndsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18691 /* V6_vasrwhrndsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18712 /* V6_vasrwhsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18725 /* V6_vasrwhsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18743 /* V6_vasrwuhrndsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18760 /* V6_vasrwuhrndsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18782 /* V6_vasrwuhsat */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18796 /* V6_vasrwuhsat_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18815 /* V6_vasrwv */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18825 /* V6_vasrwv_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18840 /* V6_vassign */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18851 /* V6_vassign_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18867 /* V6_vassign_fp */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18881 /* V6_vassign_fp_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18900 /* V6_vassignp */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18912 /* V6_vassignp_128B */, 18929 /* V64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18938 /* V6_vavgb */, 15358 /* V16iV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18947 /* V6_vavgb_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18961 /* V6_vavgbrnd */, 15358 /* V16iV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18973 /* V6_vavgbrnd_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18990 /* V6_vavgh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18999 /* V6_vavgh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19013 /* V6_vavghrnd */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19025 /* V6_vavghrnd_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19042 /* V6_vavgub */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19052 /* V6_vavgub_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19067 /* V6_vavgubrnd */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19080 /* V6_vavgubrnd_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19098 /* V6_vavguh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19108 /* V6_vavguh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19123 /* V6_vavguhrnd */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19136 /* V6_vavguhrnd_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19154 /* V6_vavguw */, 15358 /* V16iV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19164 /* V6_vavguw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19179 /* V6_vavguwrnd */, 15358 /* V16iV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19192 /* V6_vavguwrnd_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19210 /* V6_vavgw */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19219 /* V6_vavgw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19233 /* V6_vavgwrnd */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19245 /* V6_vavgwrnd_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19262 /* V6_vcl0h */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19271 /* V6_vcl0h_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19285 /* V6_vcl0w */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19294 /* V6_vcl0w_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19308 /* V6_vcombine */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19320 /* V6_vcombine_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19337 /* V6_vconv_bf_qf32 */, 13848 /* V16iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19354 /* V6_vconv_bf_qf32_128B */, 13868 /* V32iV64i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19376 /* V6_vconv_f8_qf16 */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19393 /* V6_vconv_f8_qf16_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19415 /* V6_vconv_h_hf */, 14970 /* V16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19429 /* V6_vconv_h_hf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19448 /* V6_vconv_h_hf_rnd */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19466 /* V6_vconv_h_hf_rnd_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19489 /* V6_vconv_hf_h */, 14970 /* V16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19503 /* V6_vconv_hf_h_128B */, 14995 /* V32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19522 /* V6_vconv_hf_qf16 */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19539 /* V6_vconv_hf_qf16_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19561 /* V6_vconv_hf_qf32 */, 13848 /* V16iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19578 /* V6_vconv_hf_qf32_128B */, 13868 /* V32iV64i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19600 /* V6_vconv_qf16_f8 */, 19617 /* V32iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19626 /* V6_vconv_qf16_f8_128B */, 19648 /* V64iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19657 /* V6_vconv_qf16_hf */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19674 /* V6_vconv_qf16_hf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19696 /* V6_vconv_qf16_qf16 */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19715 /* V6_vconv_qf16_qf16_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19739 /* V6_vconv_qf32_qf32 */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19758 /* V6_vconv_qf32_qf32_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19782 /* V6_vconv_qf32_sf */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19799 /* V6_vconv_qf32_sf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19821 /* V6_vconv_sf_qf32 */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19838 /* V6_vconv_sf_qf32_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19860 /* V6_vconv_sf_w */, 14970 /* V16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19874 /* V6_vconv_sf_w_128B */, 14995 /* V32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19893 /* V6_vconv_w_sf */, 14970 /* V16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19907 /* V6_vconv_w_sf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19926 /* V6_vcvt2_b_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19940 /* V6_vcvt2_b_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19959 /* V6_vcvt2_hf_b */, 19617 /* V32iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19973 /* V6_vcvt2_hf_b_128B */, 19648 /* V64iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19992 /* V6_vcvt2_hf_ub */, 19617 /* V32iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20007 /* V6_vcvt2_hf_ub_128B */, 19648 /* V64iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20027 /* V6_vcvt2_ub_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20042 /* V6_vcvt2_ub_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20062 /* V6_vcvt_b_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20075 /* V6_vcvt_b_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20093 /* V6_vcvt_bf_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20107 /* V6_vcvt_bf_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20126 /* V6_vcvt_f8_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20140 /* V6_vcvt_f8_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20159 /* V6_vcvt_h_hf */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20172 /* V6_vcvt_h_hf_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20190 /* V6_vcvt_hf_b */, 19617 /* V32iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20203 /* V6_vcvt_hf_b_128B */, 19648 /* V64iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20221 /* V6_vcvt_hf_f8 */, 19617 /* V32iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20235 /* V6_vcvt_hf_f8_128B */, 19648 /* V64iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20254 /* V6_vcvt_hf_h */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20267 /* V6_vcvt_hf_h_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20285 /* V6_vcvt_hf_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20299 /* V6_vcvt_hf_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20318 /* V6_vcvt_hf_ub */, 19617 /* V32iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20332 /* V6_vcvt_hf_ub_128B */, 19648 /* V64iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20351 /* V6_vcvt_hf_uh */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20365 /* V6_vcvt_hf_uh_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20384 /* V6_vcvt_sf_hf */, 19617 /* V32iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20398 /* V6_vcvt_sf_hf_128B */, 19648 /* V64iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20417 /* V6_vcvt_ub_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20431 /* V6_vcvt_ub_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20450 /* V6_vcvt_uh_hf */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20464 /* V6_vcvt_uh_hf_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20483 /* V6_vd0 */, 20490 /* V16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20495 /* V6_vd0_128B */, 20507 /* V32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20512 /* V6_vdd0 */, 20507 /* V32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20520 /* V6_vdd0_128B */, 20533 /* V64i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20538 /* V6_vdealb */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20548 /* V6_vdealb4w */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20560 /* V6_vdealb4w_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20577 /* V6_vdealb_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20592 /* V6_vdealh */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20602 /* V6_vdealh_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20617 /* V6_vdealvdd */, 20629 /* V32iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20643 /* V6_vdealvdd_128B */, 20660 /* V64iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20674 /* V6_vdelta */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20684 /* V6_vdelta_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20699 /* V6_vdmpy_sf_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20714 /* V6_vdmpy_sf_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20734 /* V6_vdmpy_sf_hf_acc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20770 /* V6_vdmpy_sf_hf_acc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20811 /* V6_vdmpybus */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20823 /* V6_vdmpybus_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20840 /* V6_vdmpybus_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20856 /* V6_vdmpybus_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20877 /* V6_vdmpybus_dv */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20892 /* V6_vdmpybus_dv_128B */, 20912 /* V64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20922 /* V6_vdmpybus_dv_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20941 /* V6_vdmpybus_dv_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20979 /* V6_vdmpyhb */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20990 /* V6_vdmpyhb_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21006 /* V6_vdmpyhb_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21021 /* V6_vdmpyhb_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21041 /* V6_vdmpyhb_dv */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21055 /* V6_vdmpyhb_dv_128B */, 20912 /* V64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21074 /* V6_vdmpyhb_dv_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21092 /* V6_vdmpyhb_dv_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21115 /* V6_vdmpyhisat */, 21129 /* V16iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21139 /* V6_vdmpyhisat_128B */, 21158 /* V32iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21168 /* V6_vdmpyhisat_acc */, 21186 /* V16iV16iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21200 /* V6_vdmpyhisat_acc_128B */, 21223 /* V32iV32iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21237 /* V6_vdmpyhsat */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21250 /* V6_vdmpyhsat_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21268 /* V6_vdmpyhsat_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21285 /* V6_vdmpyhsat_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21307 /* V6_vdmpyhsuisat */, 21129 /* V16iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21323 /* V6_vdmpyhsuisat_128B */, 21158 /* V32iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21344 /* V6_vdmpyhsuisat_acc */, 21186 /* V16iV16iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21364 /* V6_vdmpyhsuisat_acc_128B */, 21223 /* V32iV32iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21389 /* V6_vdmpyhsusat */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21404 /* V6_vdmpyhsusat_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21424 /* V6_vdmpyhsusat_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21443 /* V6_vdmpyhsusat_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21467 /* V6_vdmpyhvsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21481 /* V6_vdmpyhvsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21500 /* V6_vdmpyhvsat_acc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21518 /* V6_vdmpyhvsat_acc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21541 /* V6_vdsaduh */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21552 /* V6_vdsaduh_128B */, 20912 /* V64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21568 /* V6_vdsaduh_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21583 /* V6_vdsaduh_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21603 /* V6_veqb */, 21611 /* V64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21624 /* V6_veqb_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21651 /* V6_veqb_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21680 /* V6_veqb_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21716 /* V6_veqb_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21727 /* V6_veqb_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21743 /* V6_veqb_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21755 /* V6_veqb_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21772 /* V6_veqh */, 21611 /* V64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21780 /* V6_veqh_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21793 /* V6_veqh_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21805 /* V6_veqh_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21822 /* V6_veqh_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21833 /* V6_veqh_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21849 /* V6_veqh_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21861 /* V6_veqh_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21878 /* V6_veqhf */, 21611 /* V64bV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21887 /* V6_veqhf_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21901 /* V6_veqhf_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21914 /* V6_veqhf_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21932 /* V6_veqhf_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21944 /* V6_veqhf_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21961 /* V6_veqhf_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21974 /* V6_veqhf_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21992 /* V6_veqsf */, 21611 /* V64bV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22001 /* V6_veqsf_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22015 /* V6_veqsf_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22028 /* V6_veqsf_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22046 /* V6_veqsf_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22058 /* V6_veqsf_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22075 /* V6_veqsf_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22088 /* V6_veqsf_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22106 /* V6_veqw */, 21611 /* V64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22114 /* V6_veqw_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22127 /* V6_veqw_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22139 /* V6_veqw_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22156 /* V6_veqw_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22167 /* V6_veqw_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22183 /* V6_veqw_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22195 /* V6_veqw_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22212 /* V6_vfmax_f8 */, 15358 /* V16iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22224 /* V6_vfmax_f8_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22241 /* V6_vfmax_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22253 /* V6_vfmax_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22270 /* V6_vfmax_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22282 /* V6_vfmax_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22299 /* V6_vfmin_f8 */, 15358 /* V16iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22311 /* V6_vfmin_f8_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22328 /* V6_vfmin_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22340 /* V6_vfmin_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22357 /* V6_vfmin_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22369 /* V6_vfmin_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22386 /* V6_vfneg_f8 */, 14970 /* V16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22398 /* V6_vfneg_f8_128B */, 14995 /* V32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22415 /* V6_vfneg_hf */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22427 /* V6_vfneg_hf_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22444 /* V6_vfneg_sf */, 14970 /* V16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22456 /* V6_vfneg_sf_128B */, 14995 /* V32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22473 /* V6_vgathermh */, 22486 /* vv*iiV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22496 /* V6_vgathermh_128B */, 22514 /* vv*iiV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22524 /* V6_vgathermhq */, 22538 /* vv*V64biiV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22552 /* V6_vgathermhq_128B */, 22571 /* vv*V128biiV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22586 /* V6_vgathermhw */, 22514 /* vv*iiV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22600 /* V6_vgathermhw_128B */, 22619 /* vv*iiV64i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22629 /* V6_vgathermhwq */, 22644 /* vv*V64biiV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22658 /* V6_vgathermhwq_128B */, 22678 /* vv*V128biiV64i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22693 /* V6_vgathermw */, 22486 /* vv*iiV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22706 /* V6_vgathermw_128B */, 22514 /* vv*iiV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22724 /* V6_vgathermwq */, 22538 /* vv*V64biiV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22738 /* V6_vgathermwq_128B */, 22571 /* vv*V128biiV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22757 /* V6_vgtb */, 21611 /* V64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22765 /* V6_vgtb_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22778 /* V6_vgtb_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22790 /* V6_vgtb_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22807 /* V6_vgtb_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22818 /* V6_vgtb_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22834 /* V6_vgtb_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22846 /* V6_vgtb_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22863 /* V6_vgtbf */, 21611 /* V64bV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22872 /* V6_vgtbf_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22886 /* V6_vgtbf_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22899 /* V6_vgtbf_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22917 /* V6_vgtbf_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22929 /* V6_vgtbf_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22946 /* V6_vgtbf_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22959 /* V6_vgtbf_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22977 /* V6_vgth */, 21611 /* V64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22985 /* V6_vgth_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22998 /* V6_vgth_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23010 /* V6_vgth_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23027 /* V6_vgth_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23038 /* V6_vgth_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23054 /* V6_vgth_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23066 /* V6_vgth_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23083 /* V6_vgthf */, 21611 /* V64bV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23092 /* V6_vgthf_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23106 /* V6_vgthf_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23119 /* V6_vgthf_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23137 /* V6_vgthf_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23149 /* V6_vgthf_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23166 /* V6_vgthf_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23179 /* V6_vgthf_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23197 /* V6_vgtsf */, 21611 /* V64bV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23206 /* V6_vgtsf_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23220 /* V6_vgtsf_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23233 /* V6_vgtsf_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23251 /* V6_vgtsf_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23263 /* V6_vgtsf_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23280 /* V6_vgtsf_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23293 /* V6_vgtsf_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23311 /* V6_vgtub */, 21611 /* V64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23320 /* V6_vgtub_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23334 /* V6_vgtub_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23347 /* V6_vgtub_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23365 /* V6_vgtub_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23377 /* V6_vgtub_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23394 /* V6_vgtub_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23407 /* V6_vgtub_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23425 /* V6_vgtuh */, 21611 /* V64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23434 /* V6_vgtuh_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23448 /* V6_vgtuh_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23461 /* V6_vgtuh_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23479 /* V6_vgtuh_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23491 /* V6_vgtuh_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23508 /* V6_vgtuh_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23521 /* V6_vgtuh_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23539 /* V6_vgtuw */, 21611 /* V64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23548 /* V6_vgtuw_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23562 /* V6_vgtuw_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23575 /* V6_vgtuw_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23593 /* V6_vgtuw_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23605 /* V6_vgtuw_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23622 /* V6_vgtuw_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23635 /* V6_vgtuw_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23653 /* V6_vgtw */, 21611 /* V64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23661 /* V6_vgtw_128B */, 21637 /* V128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23674 /* V6_vgtw_and */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23686 /* V6_vgtw_and_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23703 /* V6_vgtw_or */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23714 /* V6_vgtw_or_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23730 /* V6_vgtw_xor */, 21663 /* V64bV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23742 /* V6_vgtw_xor_128B */, 21697 /* V128bV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23759 /* V6_vilog2_hf */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23772 /* V6_vilog2_hf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23790 /* V6_vilog2_qf16 */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23805 /* V6_vilog2_qf16_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23825 /* V6_vilog2_qf32 */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23840 /* V6_vilog2_qf32_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23860 /* V6_vilog2_sf */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23873 /* V6_vilog2_sf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23891 /* V6_vinsertwr */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23904 /* V6_vinsertwr_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23922 /* V6_vlalignb */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23934 /* V6_vlalignb_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23951 /* V6_vlalignbi */, 17384 /* V16iV16iV16iUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23964 /* V6_vlalignbi_128B */, 14492 /* V32iV32iV32iUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23982 /* V6_vlsrb */, 13719 /* V16iV16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23991 /* V6_vlsrb_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24005 /* V6_vlsrh */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24014 /* V6_vlsrh_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24028 /* V6_vlsrhv */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24038 /* V6_vlsrhv_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24053 /* V6_vlsrw */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24062 /* V6_vlsrw_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24076 /* V6_vlsrwv */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24086 /* V6_vlsrwv_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24101 /* V6_vlut4 */, 24110 /* V16iV16iLLi */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24122 /* V6_vlut4_128B */, 24136 /* V32iV32iLLi */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24148 /* V6_vlutvvb */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24159 /* V6_vlutvvb_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24175 /* V6_vlutvvb_nm */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24189 /* V6_vlutvvb_nm_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24208 /* V6_vlutvvb_oracc */, 24225 /* V16iV16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24243 /* V6_vlutvvb_oracc_128B */, 24265 /* V32iV32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24283 /* V6_vlutvvb_oracci */, 24301 /* V16iV16iV16iV16iUIi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24321 /* V6_vlutvvb_oracci_128B */, 14612 /* V32iV32iV32iV32iUIi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24344 /* V6_vlutvvbi */, 17384 /* V16iV16iV16iUIi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24356 /* V6_vlutvvbi_128B */, 14492 /* V32iV32iV32iUIi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24373 /* V6_vlutvwh */, 20629 /* V32iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24384 /* V6_vlutvwh_128B */, 20660 /* V64iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24400 /* V6_vlutvwh_nm */, 20629 /* V32iV16iV16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24414 /* V6_vlutvwh_nm_128B */, 20660 /* V64iV32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24433 /* V6_vlutvwh_oracc */, 24450 /* V32iV32iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24468 /* V6_vlutvwh_oracc_128B */, 24490 /* V64iV64iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24508 /* V6_vlutvwh_oracci */, 24526 /* V32iV32iV16iV16iUIi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24546 /* V6_vlutvwh_oracci_128B */, 24569 /* V64iV64iV32iV32iUIi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24589 /* V6_vlutvwhi */, 24601 /* V32iV16iV16iUIi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24617 /* V6_vlutvwhi_128B */, 24634 /* V64iV32iV32iUIi */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24650 /* V6_vmaskedstorenq */, 14773 /* vV64bv*V16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24668 /* V6_vmaskedstorenq_128B */, 14809 /* vV128bv*V32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24691 /* V6_vmaskedstorentnq */, 14773 /* vV64bv*V16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24711 /* V6_vmaskedstorentnq_128B */, 14809 /* vV128bv*V32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24736 /* V6_vmaskedstorentq */, 14773 /* vV64bv*V16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24755 /* V6_vmaskedstorentq_128B */, 14809 /* vV128bv*V32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24779 /* V6_vmaskedstoreq */, 14773 /* vV64bv*V16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24796 /* V6_vmaskedstoreq_128B */, 14809 /* vV128bv*V32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24818 /* V6_vmax_bf */, 15358 /* V16iV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24829 /* V6_vmax_bf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24845 /* V6_vmax_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24856 /* V6_vmax_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24872 /* V6_vmax_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24883 /* V6_vmax_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24899 /* V6_vmaxb */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24908 /* V6_vmaxb_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24922 /* V6_vmaxh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24931 /* V6_vmaxh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24945 /* V6_vmaxub */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24955 /* V6_vmaxub_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24970 /* V6_vmaxuh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24980 /* V6_vmaxuh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24995 /* V6_vmaxw */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25004 /* V6_vmaxw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25018 /* V6_vmerge_qf */, 15358 /* V16iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25031 /* V6_vmerge_qf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25049 /* V6_vmin_bf */, 15358 /* V16iV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25060 /* V6_vmin_bf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25076 /* V6_vmin_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25087 /* V6_vmin_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25103 /* V6_vmin_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25114 /* V6_vmin_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25130 /* V6_vminb */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25139 /* V6_vminb_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25153 /* V6_vminh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25162 /* V6_vminh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25176 /* V6_vminub */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25186 /* V6_vminub_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25201 /* V6_vminuh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25211 /* V6_vminuh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25226 /* V6_vminw */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25235 /* V6_vminw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25249 /* V6_vmpabus */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25260 /* V6_vmpabus_128B */, 20912 /* V64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25276 /* V6_vmpabus_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25291 /* V6_vmpabus_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25311 /* V6_vmpabusv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25323 /* V6_vmpabusv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25340 /* V6_vmpabuu */, 13761 /* V32iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25351 /* V6_vmpabuu_128B */, 20912 /* V64iV64ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25367 /* V6_vmpabuu_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25382 /* V6_vmpabuu_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25402 /* V6_vmpabuuv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25414 /* V6_vmpabuuv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25431 /* V6_vmpahb */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25441 /* V6_vmpahb_128B */, 20912 /* V64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25456 /* V6_vmpahb_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25470 /* V6_vmpahb_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25489 /* V6_vmpahhsat */, 25502 /* V16iV16iV16iLLi */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25518 /* V6_vmpahhsat_128B */, 25536 /* V32iV32iV32iLLi */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25552 /* V6_vmpauhb */, 13761 /* V32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25563 /* V6_vmpauhb_128B */, 20912 /* V64iV64ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25579 /* V6_vmpauhb_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25594 /* V6_vmpauhb_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25614 /* V6_vmpauhuhsat */, 25502 /* V16iV16iV16iLLi */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25629 /* V6_vmpauhuhsat_128B */, 25536 /* V32iV32iV32iLLi */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25649 /* V6_vmpsuhuhsat */, 25502 /* V16iV16iV16iLLi */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25664 /* V6_vmpsuhuhsat_128B */, 25536 /* V32iV32iV32iLLi */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25684 /* V6_vmpy_hf_f8 */, 15648 /* V32iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25698 /* V6_vmpy_hf_f8_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25717 /* V6_vmpy_hf_f8_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25735 /* V6_vmpy_hf_f8_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25758 /* V6_vmpy_hf_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25772 /* V6_vmpy_hf_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25791 /* V6_vmpy_hf_hf_acc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25809 /* V6_vmpy_hf_hf_acc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25832 /* V6_vmpy_qf16 */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25845 /* V6_vmpy_qf16_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25863 /* V6_vmpy_qf16_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25879 /* V6_vmpy_qf16_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25900 /* V6_vmpy_qf16_mix_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25920 /* V6_vmpy_qf16_mix_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25945 /* V6_vmpy_qf32 */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25958 /* V6_vmpy_qf32_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25976 /* V6_vmpy_qf32_hf */, 15648 /* V32iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25992 /* V6_vmpy_qf32_hf_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26013 /* V6_vmpy_qf32_mix_hf */, 15648 /* V32iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26033 /* V6_vmpy_qf32_mix_hf_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26058 /* V6_vmpy_qf32_qf16 */, 15648 /* V32iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26076 /* V6_vmpy_qf32_qf16_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26099 /* V6_vmpy_qf32_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26115 /* V6_vmpy_qf32_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26136 /* V6_vmpy_rt_hf */, 13719 /* V16iV16ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26150 /* V6_vmpy_rt_hf_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26169 /* V6_vmpy_rt_qf16 */, 13719 /* V16iV16ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26185 /* V6_vmpy_rt_qf16_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26206 /* V6_vmpy_rt_sf */, 13719 /* V16iV16ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26220 /* V6_vmpy_rt_sf_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26239 /* V6_vmpy_sf_bf */, 15648 /* V32iV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26253 /* V6_vmpy_sf_bf_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26272 /* V6_vmpy_sf_bf_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26290 /* V6_vmpy_sf_bf_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26313 /* V6_vmpy_sf_hf */, 15648 /* V32iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26327 /* V6_vmpy_sf_hf_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26346 /* V6_vmpy_sf_hf_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26364 /* V6_vmpy_sf_hf_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26387 /* V6_vmpy_sf_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26401 /* V6_vmpy_sf_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26420 /* V6_vmpybus */, 26431 /* V32iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26441 /* V6_vmpybus_128B */, 26457 /* V64iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26467 /* V6_vmpybus_acc */, 26482 /* V32iV32iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26496 /* V6_vmpybus_acc_128B */, 26516 /* V64iV64iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26530 /* V6_vmpybusv */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26542 /* V6_vmpybusv_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26559 /* V6_vmpybusv_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26575 /* V6_vmpybusv_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26596 /* V6_vmpybv */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26606 /* V6_vmpybv_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26621 /* V6_vmpybv_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26635 /* V6_vmpybv_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26654 /* V6_vmpyewuh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26666 /* V6_vmpyewuh_64 */, 15648 /* V32iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26681 /* V6_vmpyewuh_64_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26701 /* V6_vmpyewuh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26718 /* V6_vmpyh */, 26431 /* V32iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26727 /* V6_vmpyh_128B */, 26457 /* V64iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26741 /* V6_vmpyh_acc */, 26482 /* V32iV32iV16ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26754 /* V6_vmpyh_acc_128B */, 26516 /* V64iV64iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26772 /* V6_vmpyhsat_acc */, 26482 /* V32iV32iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26788 /* V6_vmpyhsat_acc_128B */, 26516 /* V64iV64iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26809 /* V6_vmpyhsrs */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26821 /* V6_vmpyhsrs_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26838 /* V6_vmpyhss */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26849 /* V6_vmpyhss_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26865 /* V6_vmpyhus */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26876 /* V6_vmpyhus_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26892 /* V6_vmpyhus_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26907 /* V6_vmpyhus_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26927 /* V6_vmpyhv */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26937 /* V6_vmpyhv_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26952 /* V6_vmpyhv_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26966 /* V6_vmpyhv_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26985 /* V6_vmpyhvsrs */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26998 /* V6_vmpyhvsrs_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27016 /* V6_vmpyieoh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27028 /* V6_vmpyieoh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27045 /* V6_vmpyiewh_acc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27061 /* V6_vmpyiewh_acc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27082 /* V6_vmpyiewuh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27095 /* V6_vmpyiewuh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27113 /* V6_vmpyiewuh_acc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27130 /* V6_vmpyiewuh_acc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27152 /* V6_vmpyih */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27162 /* V6_vmpyih_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27177 /* V6_vmpyih_acc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27191 /* V6_vmpyih_acc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27210 /* V6_vmpyihb */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27221 /* V6_vmpyihb_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27237 /* V6_vmpyihb_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27252 /* V6_vmpyihb_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27272 /* V6_vmpyiowh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27284 /* V6_vmpyiowh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27301 /* V6_vmpyiwb */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27312 /* V6_vmpyiwb_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27328 /* V6_vmpyiwb_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27343 /* V6_vmpyiwb_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27363 /* V6_vmpyiwh */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27374 /* V6_vmpyiwh_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27390 /* V6_vmpyiwh_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27405 /* V6_vmpyiwh_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27425 /* V6_vmpyiwub */, 13719 /* V16iV16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27437 /* V6_vmpyiwub_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27454 /* V6_vmpyiwub_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27470 /* V6_vmpyiwub_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27491 /* V6_vmpyowh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27502 /* V6_vmpyowh_64_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27520 /* V6_vmpyowh_64_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27543 /* V6_vmpyowh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27559 /* V6_vmpyowh_rnd */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27574 /* V6_vmpyowh_rnd_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27594 /* V6_vmpyowh_rnd_sacc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27614 /* V6_vmpyowh_rnd_sacc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27639 /* V6_vmpyowh_sacc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27655 /* V6_vmpyowh_sacc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27676 /* V6_vmpyub */, 26431 /* V32iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27686 /* V6_vmpyub_128B */, 26457 /* V64iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27701 /* V6_vmpyub_acc */, 26482 /* V32iV32iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27715 /* V6_vmpyub_acc_128B */, 26516 /* V64iV64iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27734 /* V6_vmpyubv */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27745 /* V6_vmpyubv_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27761 /* V6_vmpyubv_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27776 /* V6_vmpyubv_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27796 /* V6_vmpyuh */, 26431 /* V32iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27806 /* V6_vmpyuh_128B */, 26457 /* V64iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27821 /* V6_vmpyuh_acc */, 26482 /* V32iV32iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27835 /* V6_vmpyuh_acc_128B */, 26516 /* V64iV64iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27854 /* V6_vmpyuhe */, 13719 /* V16iV16ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27865 /* V6_vmpyuhe_128B */, 13761 /* V32iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27881 /* V6_vmpyuhe_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27896 /* V6_vmpyuhe_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27916 /* V6_vmpyuhv */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27927 /* V6_vmpyuhv_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27943 /* V6_vmpyuhv_acc */, 16730 /* V32iV32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27958 /* V6_vmpyuhv_acc_128B */, 16766 /* V64iV64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27978 /* V6_vmpyuhvs */, 15358 /* V16iV16iV16i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27990 /* V6_vmpyuhvs_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 18404 /* hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28007 /* V6_vmux */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28015 /* V6_vmux_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28028 /* V6_vnavgb */, 15358 /* V16iV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28038 /* V6_vnavgb_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28053 /* V6_vnavgh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28063 /* V6_vnavgh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28078 /* V6_vnavgub */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28089 /* V6_vnavgub_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28105 /* V6_vnavgw */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28115 /* V6_vnavgw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28130 /* V6_vneg_qf16_hf */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28146 /* V6_vneg_qf16_hf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28167 /* V6_vneg_qf16_qf16 */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28185 /* V6_vneg_qf16_qf16_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28208 /* V6_vneg_qf32_qf32 */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28226 /* V6_vneg_qf32_qf32_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28249 /* V6_vneg_qf32_sf */, 14970 /* V16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28265 /* V6_vneg_qf32_sf_128B */, 14995 /* V32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28286 /* V6_vnormamth */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28299 /* V6_vnormamth_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28317 /* V6_vnormamtw */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28330 /* V6_vnormamtw_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28348 /* V6_vnot */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28356 /* V6_vnot_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28369 /* V6_vor */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28376 /* V6_vor_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28388 /* V6_vpackeb */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28399 /* V6_vpackeb_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28415 /* V6_vpackeh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28426 /* V6_vpackeh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28442 /* V6_vpackhb_sat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28457 /* V6_vpackhb_sat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28477 /* V6_vpackhub_sat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28493 /* V6_vpackhub_sat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28514 /* V6_vpackob */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28525 /* V6_vpackob_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28541 /* V6_vpackoh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28552 /* V6_vpackoh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28568 /* V6_vpackwh_sat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28583 /* V6_vpackwh_sat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28603 /* V6_vpackwuh_sat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28619 /* V6_vpackwuh_sat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28640 /* V6_vpopcounth */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28654 /* V6_vpopcounth_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28673 /* V6_vprefixqb */, 28686 /* V16iV64b */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28695 /* V6_vprefixqb_128B */, 28713 /* V32iV128b */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28723 /* V6_vprefixqh */, 28686 /* V16iV64b */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28736 /* V6_vprefixqh_128B */, 28713 /* V32iV128b */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28754 /* V6_vprefixqw */, 28686 /* V16iV64b */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28767 /* V6_vprefixqw_128B */, 28713 /* V32iV128b */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28785 /* V6_vrdelta */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28796 /* V6_vrdelta_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28812 /* V6_vrmpybub_rtt */, 28828 /* V32iV16iLLi */, 0 /*  */, 28840 /* hvxv65 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28847 /* V6_vrmpybub_rtt_128B */, 28868 /* V64iV32iLLi */, 0 /*  */, 28840 /* hvxv65 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28880 /* V6_vrmpybub_rtt_acc */, 28900 /* V32iV32iV16iLLi */, 0 /*  */, 28840 /* hvxv65 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28916 /* V6_vrmpybub_rtt_acc_128B */, 28941 /* V64iV64iV32iLLi */, 0 /*  */, 28840 /* hvxv65 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28957 /* V6_vrmpybus */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28969 /* V6_vrmpybus_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28986 /* V6_vrmpybus_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29002 /* V6_vrmpybus_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29023 /* V6_vrmpybusi */, 29036 /* V32iV32iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29049 /* V6_vrmpybusi_128B */, 29067 /* V64iV64iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29080 /* V6_vrmpybusi_acc */, 29097 /* V32iV32iV32iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29114 /* V6_vrmpybusi_acc_128B */, 29136 /* V64iV64iV64iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29153 /* V6_vrmpybusv */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29166 /* V6_vrmpybusv_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29184 /* V6_vrmpybusv_acc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29201 /* V6_vrmpybusv_acc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29223 /* V6_vrmpybv */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29234 /* V6_vrmpybv_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29250 /* V6_vrmpybv_acc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29265 /* V6_vrmpybv_acc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29285 /* V6_vrmpyub */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29296 /* V6_vrmpyub_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29312 /* V6_vrmpyub_acc */, 13790 /* V16iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29327 /* V6_vrmpyub_acc_128B */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29347 /* V6_vrmpyub_rtt */, 28828 /* V32iV16iLLi */, 0 /*  */, 28840 /* hvxv65 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29362 /* V6_vrmpyub_rtt_128B */, 28868 /* V64iV32iLLi */, 0 /*  */, 28840 /* hvxv65 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29382 /* V6_vrmpyub_rtt_acc */, 28900 /* V32iV32iV16iLLi */, 0 /*  */, 28840 /* hvxv65 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29401 /* V6_vrmpyub_rtt_acc_128B */, 28941 /* V64iV64iV32iLLi */, 0 /*  */, 28840 /* hvxv65 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29425 /* V6_vrmpyubi */, 29036 /* V32iV32iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29437 /* V6_vrmpyubi_128B */, 29067 /* V64iV64iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29454 /* V6_vrmpyubi_acc */, 29097 /* V32iV32iV32iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29470 /* V6_vrmpyubi_acc_128B */, 29136 /* V64iV64iV64iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29491 /* V6_vrmpyubv */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29503 /* V6_vrmpyubv_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29520 /* V6_vrmpyubv_acc */, 20753 /* V16iV16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29536 /* V6_vrmpyubv_acc_128B */, 20794 /* V32iV32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29557 /* V6_vror */, 13719 /* V16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29565 /* V6_vror_128B */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29578 /* V6_vrotr */, 15358 /* V16iV16iV16i */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29587 /* V6_vrotr_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29601 /* V6_vroundhb */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29613 /* V6_vroundhb_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29630 /* V6_vroundhub */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29643 /* V6_vroundhub_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29661 /* V6_vrounduhub */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29675 /* V6_vrounduhub_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29694 /* V6_vrounduwuh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29708 /* V6_vrounduwuh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29727 /* V6_vroundwh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29739 /* V6_vroundwh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29756 /* V6_vroundwuh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29769 /* V6_vroundwuh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29787 /* V6_vrsadubi */, 29036 /* V32iV32iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29799 /* V6_vrsadubi_128B */, 29067 /* V64iV64iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29816 /* V6_vrsadubi_acc */, 29097 /* V32iV32iV32iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29832 /* V6_vrsadubi_acc_128B */, 29136 /* V64iV64iV64iiUIi */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29853 /* V6_vsatdw */, 15358 /* V16iV16iV16i */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29863 /* V6_vsatdw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29878 /* V6_vsathub */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29889 /* V6_vsathub_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29905 /* V6_vsatuwuh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29917 /* V6_vsatuwuh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29934 /* V6_vsatwh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29944 /* V6_vsatwh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29959 /* V6_vsb */, 19617 /* V32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29966 /* V6_vsb_128B */, 19648 /* V64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29978 /* V6_vscattermh */, 29992 /* viiV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30004 /* V6_vscattermh_128B */, 30023 /* viiV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30035 /* V6_vscattermh_add */, 29992 /* viiV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30053 /* V6_vscattermh_add_128B */, 30023 /* viiV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30076 /* V6_vscattermhq */, 30091 /* vV64biiV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30107 /* V6_vscattermhq_128B */, 30127 /* vV128biiV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30144 /* V6_vscattermhw */, 30159 /* viiV32iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30171 /* V6_vscattermhw_128B */, 30191 /* viiV64iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30203 /* V6_vscattermhw_add */, 30159 /* viiV32iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30222 /* V6_vscattermhw_add_128B */, 30191 /* viiV64iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30246 /* V6_vscattermhwq */, 30262 /* vV64biiV32iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30278 /* V6_vscattermhwq_128B */, 30299 /* vV128biiV64iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30316 /* V6_vscattermw */, 29992 /* viiV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30330 /* V6_vscattermw_128B */, 30023 /* viiV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30349 /* V6_vscattermw_add */, 29992 /* viiV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30367 /* V6_vscattermw_add_128B */, 30023 /* viiV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30390 /* V6_vscattermwq */, 30091 /* vV64biiV16iV16i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30405 /* V6_vscattermwq_128B */, 30127 /* vV128biiV32iV32i */, 0 /*  */, 15230 /* hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30425 /* V6_vsh */, 19617 /* V32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30432 /* V6_vsh_128B */, 19648 /* V64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30444 /* V6_vshufeh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30455 /* V6_vshufeh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30471 /* V6_vshuffb */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30482 /* V6_vshuffb_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30498 /* V6_vshuffeb */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30510 /* V6_vshuffeb_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30527 /* V6_vshuffh */, 14970 /* V16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30538 /* V6_vshuffh_128B */, 14995 /* V32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30554 /* V6_vshuffob */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30566 /* V6_vshuffob_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30583 /* V6_vshuffvdd */, 20629 /* V32iV16iV16ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30596 /* V6_vshuffvdd_128B */, 20660 /* V64iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30614 /* V6_vshufoeb */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30626 /* V6_vshufoeb_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30643 /* V6_vshufoeh */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30655 /* V6_vshufoeh_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30672 /* V6_vshufoh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30683 /* V6_vshufoh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30699 /* V6_vsub_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30710 /* V6_vsub_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30726 /* V6_vsub_hf_f8 */, 15648 /* V32iV16iV16i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30740 /* V6_vsub_hf_f8_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13729 /* hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30759 /* V6_vsub_hf_hf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30773 /* V6_vsub_hf_hf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30792 /* V6_vsub_hf_mix */, 15358 /* V16iV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30807 /* V6_vsub_hf_mix_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30827 /* V6_vsub_qf16 */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30840 /* V6_vsub_qf16_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30858 /* V6_vsub_qf16_mix */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30875 /* V6_vsub_qf16_mix_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30897 /* V6_vsub_qf32 */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30910 /* V6_vsub_qf32_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30928 /* V6_vsub_qf32_mix */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30945 /* V6_vsub_qf32_mix_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30967 /* V6_vsub_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30978 /* V6_vsub_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30994 /* V6_vsub_sf_bf */, 15648 /* V32iV16iV16i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31008 /* V6_vsub_sf_bf_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 15907 /* hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31027 /* V6_vsub_sf_hf */, 15648 /* V32iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31041 /* V6_vsub_sf_hf_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31060 /* V6_vsub_sf_mix */, 15358 /* V16iV16iV16i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31075 /* V6_vsub_sf_mix_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 15047 /* hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31095 /* V6_vsub_sf_sf */, 15358 /* V16iV16iV16i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31109 /* V6_vsub_sf_sf_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 14508 /* hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31128 /* V6_vsubb */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31137 /* V6_vsubb_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31151 /* V6_vsubb_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31163 /* V6_vsubb_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31180 /* V6_vsubbnq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31191 /* V6_vsubbnq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31207 /* V6_vsubbq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31217 /* V6_vsubbq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31232 /* V6_vsubbsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31244 /* V6_vsubbsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31261 /* V6_vsubbsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31276 /* V6_vsubbsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31296 /* V6_vsubcarry */, 16249 /* V16iV16iV16iv* */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31309 /* V6_vsubcarry_128B */, 16282 /* V32iV32iV32iv* */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31327 /* V6_vsubcarryo */, 16249 /* V16iV16iV16iv* */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31341 /* V6_vsubcarryo_128B */, 16282 /* V32iV32iV32iv* */, 0 /*  */, 16311 /* hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31360 /* V6_vsubh */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31369 /* V6_vsubh_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31383 /* V6_vsubh_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31395 /* V6_vsubh_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31412 /* V6_vsubhnq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31423 /* V6_vsubhnq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31439 /* V6_vsubhq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31449 /* V6_vsubhq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31464 /* V6_vsubhsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31476 /* V6_vsubhsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31493 /* V6_vsubhsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31508 /* V6_vsubhsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31528 /* V6_vsubhw */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31538 /* V6_vsubhw_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31553 /* V6_vsububh */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31564 /* V6_vsububh_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31580 /* V6_vsububsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31593 /* V6_vsububsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31611 /* V6_vsububsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31627 /* V6_vsububsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31648 /* V6_vsubububb_sat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31665 /* V6_vsubububb_sat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31687 /* V6_vsubuhsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31700 /* V6_vsubuhsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31718 /* V6_vsubuhsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31734 /* V6_vsubuhsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31755 /* V6_vsubuhw */, 15648 /* V32iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31766 /* V6_vsubuhw_128B */, 15680 /* V64iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31782 /* V6_vsubuwsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31795 /* V6_vsubuwsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31813 /* V6_vsubuwsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31829 /* V6_vsubuwsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13912 /* hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31850 /* V6_vsubw */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31859 /* V6_vsubw_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31873 /* V6_vsubw_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31885 /* V6_vsubw_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31902 /* V6_vsubwnq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31913 /* V6_vsubwnq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31929 /* V6_vsubwq */, 16096 /* V16iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31939 /* V6_vsubwq_128B */, 16129 /* V32iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31954 /* V6_vsubwsat */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31966 /* V6_vsubwsat_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31983 /* V6_vsubwsat_dv */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{31998 /* V6_vsubwsat_dv_128B */, 16072 /* V64iV64iV64i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32018 /* V6_vswap */, 32027 /* V32iV64bV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32044 /* V6_vswap_128B */, 32058 /* V64iV128bV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32076 /* V6_vtmpyb */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32086 /* V6_vtmpyb_128B */, 20912 /* V64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32101 /* V6_vtmpyb_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32115 /* V6_vtmpyb_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32134 /* V6_vtmpybus */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32146 /* V6_vtmpybus_128B */, 20912 /* V64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32163 /* V6_vtmpybus_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32179 /* V6_vtmpybus_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32200 /* V6_vtmpyhb */, 13761 /* V32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32211 /* V6_vtmpyhb_128B */, 20912 /* V64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32227 /* V6_vtmpyhb_acc */, 13828 /* V32iV32iV32ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32242 /* V6_vtmpyhb_acc_128B */, 20965 /* V64iV64iV64ii */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32262 /* V6_vunpackb */, 19617 /* V32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32274 /* V6_vunpackb_128B */, 19648 /* V64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32291 /* V6_vunpackh */, 19617 /* V32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32303 /* V6_vunpackh_128B */, 19648 /* V64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32320 /* V6_vunpackob */, 32333 /* V32iV32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32346 /* V6_vunpackob_128B */, 32364 /* V64iV64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32377 /* V6_vunpackoh */, 32333 /* V32iV32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32390 /* V6_vunpackoh_128B */, 32364 /* V64iV64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32408 /* V6_vunpackub */, 19617 /* V32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32421 /* V6_vunpackub_128B */, 19648 /* V64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32439 /* V6_vunpackuh */, 19617 /* V32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32452 /* V6_vunpackuh_128B */, 19648 /* V64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32470 /* V6_vxor */, 15358 /* V16iV16iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32478 /* V6_vxor_128B */, 15389 /* V32iV32iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32491 /* V6_vzb */, 19617 /* V32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32498 /* V6_vzb_128B */, 19648 /* V64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32510 /* V6_vzh */, 19617 /* V32iV16i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32517 /* V6_vzh_128B */, 19648 /* V64iV32i */, 0 /*  */, 13598 /* hvxv60|hvxv62|hvxv65|hvxv66|hvxv67|hvxv68|hvxv69|hvxv71|hvxv73|hvxv75|hvxv79|hvxv81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32529 /* Y2_dccleana */, 32541 /* vv* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32545 /* Y2_dccleaninva */, 32541 /* vv* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32560 /* Y2_dcfetch */, 32541 /* vv* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32571 /* Y2_dcinva */, 32541 /* vv* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32581 /* Y2_dczeroa */, 32541 /* vv* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32592 /* Y4_l2fetch */, 32603 /* vv*i */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32608 /* Y5_l2fetch */, 32619 /* vv*LLi */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32626 /* Y6_dmlink */, 32636 /* vv*v* */, 0 /*  */, 32642 /* v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32670 /* Y6_dmpause */, 32681 /* i */, 0 /*  */, 32642 /* v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32683 /* Y6_dmpoll */, 32681 /* i */, 0 /*  */, 32642 /* v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32693 /* Y6_dmresume */, 32541 /* vv* */, 0 /*  */, 32642 /* v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32705 /* Y6_dmstart */, 32541 /* vv* */, 0 /*  */, 32642 /* v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32716 /* Y6_dmwait */, 32681 /* i */, 0 /*  */, 32642 /* v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{32726 /* prefetch */, 32541 /* vv* */, 0 /*  */, 31 /* v5|v55|v60|v62|v65|v66|v67|v68|v69|v71|v73|v75|v79|v81 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},

#endif // GET_BUILTIN_PREFIXED_INFOS

#ifdef ATOMIC_BUILTIN

#endif // ATOMIC_BUILTIN
#undef ATOMIC_BUILTIN
