/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* List of builtins that Clang recognizes                                     *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/


#ifdef GET_BUILTIN_ENUMERATORS
    BI__bmma_m8n8k128_ld_a_b1,
    BI__bmma_m8n8k128_ld_b_b1,
    BI__bmma_m8n8k128_ld_c,
    BI__bmma_m8n8k128_mma_and_popc_b1,
    BI__bmma_m8n8k128_mma_xor_popc_b1,
    BI__bmma_m8n8k128_st_c_i32,
    BI__builtin_ptx_get_image_channel_data_typei_,
    BI__builtin_ptx_get_image_channel_orderi_,
    BI__builtin_ptx_get_image_depthi_,
    BI__builtin_ptx_get_image_heighti_,
    BI__builtin_ptx_get_image_widthi_,
    BI__builtin_ptx_read_image2Dff_,
    BI__builtin_ptx_read_image2Dfi_,
    BI__builtin_ptx_read_image2Dif_,
    BI__builtin_ptx_read_image2Dii_,
    BI__builtin_ptx_read_image3Dff_,
    BI__builtin_ptx_read_image3Dfi_,
    BI__builtin_ptx_read_image3Dif_,
    BI__builtin_ptx_read_image3Dii_,
    BI__builtin_ptx_write_image2Df_,
    BI__builtin_ptx_write_image2Di_,
    BI__builtin_ptx_write_image2Dui_,
    BI__dmma_m8n8k4_ld_a,
    BI__dmma_m8n8k4_ld_b,
    BI__dmma_m8n8k4_ld_c,
    BI__dmma_m8n8k4_mma_f64,
    BI__dmma_m8n8k4_st_c_f64,
    BI__hmma_m8n32k16_ld_a,
    BI__hmma_m8n32k16_ld_b,
    BI__hmma_m8n32k16_ld_c_f16,
    BI__hmma_m8n32k16_ld_c_f32,
    BI__hmma_m8n32k16_mma_f16f16,
    BI__hmma_m8n32k16_mma_f16f32,
    BI__hmma_m8n32k16_mma_f32f16,
    BI__hmma_m8n32k16_mma_f32f32,
    BI__hmma_m8n32k16_st_c_f16,
    BI__hmma_m8n32k16_st_c_f32,
    BI__hmma_m16n16k16_ld_a,
    BI__hmma_m16n16k16_ld_b,
    BI__hmma_m16n16k16_ld_c_f16,
    BI__hmma_m16n16k16_ld_c_f32,
    BI__hmma_m16n16k16_mma_f16f16,
    BI__hmma_m16n16k16_mma_f16f32,
    BI__hmma_m16n16k16_mma_f32f16,
    BI__hmma_m16n16k16_mma_f32f32,
    BI__hmma_m16n16k16_st_c_f16,
    BI__hmma_m16n16k16_st_c_f32,
    BI__hmma_m32n8k16_ld_a,
    BI__hmma_m32n8k16_ld_b,
    BI__hmma_m32n8k16_ld_c_f16,
    BI__hmma_m32n8k16_ld_c_f32,
    BI__hmma_m32n8k16_mma_f16f16,
    BI__hmma_m32n8k16_mma_f16f32,
    BI__hmma_m32n8k16_mma_f32f16,
    BI__hmma_m32n8k16_mma_f32f32,
    BI__hmma_m32n8k16_st_c_f16,
    BI__hmma_m32n8k16_st_c_f32,
    BI__imma_m8n8k32_ld_a_s4,
    BI__imma_m8n8k32_ld_a_u4,
    BI__imma_m8n8k32_ld_b_s4,
    BI__imma_m8n8k32_ld_b_u4,
    BI__imma_m8n8k32_ld_c,
    BI__imma_m8n8k32_mma_s4,
    BI__imma_m8n8k32_mma_u4,
    BI__imma_m8n8k32_st_c_i32,
    BI__imma_m8n32k16_ld_a_s8,
    BI__imma_m8n32k16_ld_a_u8,
    BI__imma_m8n32k16_ld_b_s8,
    BI__imma_m8n32k16_ld_b_u8,
    BI__imma_m8n32k16_ld_c,
    BI__imma_m8n32k16_mma_s8,
    BI__imma_m8n32k16_mma_u8,
    BI__imma_m8n32k16_st_c_i32,
    BI__imma_m16n16k16_ld_a_s8,
    BI__imma_m16n16k16_ld_a_u8,
    BI__imma_m16n16k16_ld_b_s8,
    BI__imma_m16n16k16_ld_b_u8,
    BI__imma_m16n16k16_ld_c,
    BI__imma_m16n16k16_mma_s8,
    BI__imma_m16n16k16_mma_u8,
    BI__imma_m16n16k16_st_c_i32,
    BI__imma_m32n8k16_ld_a_s8,
    BI__imma_m32n8k16_ld_a_u8,
    BI__imma_m32n8k16_ld_b_s8,
    BI__imma_m32n8k16_ld_b_u8,
    BI__imma_m32n8k16_ld_c,
    BI__imma_m32n8k16_mma_s8,
    BI__imma_m32n8k16_mma_u8,
    BI__imma_m32n8k16_st_c_i32,
    BI__mma_bf16_m8n32k16_ld_a,
    BI__mma_bf16_m8n32k16_ld_b,
    BI__mma_bf16_m8n32k16_mma_f32,
    BI__mma_bf16_m16n16k16_ld_a,
    BI__mma_bf16_m16n16k16_ld_b,
    BI__mma_bf16_m16n16k16_mma_f32,
    BI__mma_bf16_m32n8k16_ld_a,
    BI__mma_bf16_m32n8k16_ld_b,
    BI__mma_bf16_m32n8k16_mma_f32,
    BI__mma_m16n16k8_st_c_f32,
    BI__mma_tf32_m16n16k8_ld_a,
    BI__mma_tf32_m16n16k8_ld_b,
    BI__mma_tf32_m16n16k8_ld_c,
    BI__mma_tf32_m16n16k8_mma_f32,
    BI__nvvm_abs_bf16,
    BI__nvvm_abs_bf16x2,
    BI__nvvm_activemask,
    BI__nvvm_add_rm_d,
    BI__nvvm_add_rm_f,
    BI__nvvm_add_rm_ftz_f,
    BI__nvvm_add_rm_ftz_sat_f,
    BI__nvvm_add_rm_sat_f,
    BI__nvvm_add_rn_d,
    BI__nvvm_add_rn_f,
    BI__nvvm_add_rn_ftz_f,
    BI__nvvm_add_rn_ftz_sat_f,
    BI__nvvm_add_rn_sat_f,
    BI__nvvm_add_rp_d,
    BI__nvvm_add_rp_f,
    BI__nvvm_add_rp_ftz_f,
    BI__nvvm_add_rp_ftz_sat_f,
    BI__nvvm_add_rp_sat_f,
    BI__nvvm_add_rz_d,
    BI__nvvm_add_rz_f,
    BI__nvvm_add_rz_ftz_f,
    BI__nvvm_add_rz_ftz_sat_f,
    BI__nvvm_add_rz_sat_f,
    BI__nvvm_atom_add_gen_d,
    BI__nvvm_atom_add_gen_f,
    BI__nvvm_atom_add_gen_i,
    BI__nvvm_atom_add_gen_l,
    BI__nvvm_atom_add_gen_ll,
    BI__nvvm_atom_and_gen_i,
    BI__nvvm_atom_and_gen_l,
    BI__nvvm_atom_and_gen_ll,
    BI__nvvm_atom_cas_gen_i,
    BI__nvvm_atom_cas_gen_l,
    BI__nvvm_atom_cas_gen_ll,
    BI__nvvm_atom_cas_gen_us,
    BI__nvvm_atom_cta_add_gen_d,
    BI__nvvm_atom_cta_add_gen_f,
    BI__nvvm_atom_cta_add_gen_i,
    BI__nvvm_atom_cta_add_gen_l,
    BI__nvvm_atom_cta_add_gen_ll,
    BI__nvvm_atom_cta_and_gen_i,
    BI__nvvm_atom_cta_and_gen_l,
    BI__nvvm_atom_cta_and_gen_ll,
    BI__nvvm_atom_cta_cas_gen_i,
    BI__nvvm_atom_cta_cas_gen_l,
    BI__nvvm_atom_cta_cas_gen_ll,
    BI__nvvm_atom_cta_cas_gen_us,
    BI__nvvm_atom_cta_dec_gen_ui,
    BI__nvvm_atom_cta_inc_gen_ui,
    BI__nvvm_atom_cta_max_gen_i,
    BI__nvvm_atom_cta_max_gen_l,
    BI__nvvm_atom_cta_max_gen_ll,
    BI__nvvm_atom_cta_max_gen_ui,
    BI__nvvm_atom_cta_max_gen_ul,
    BI__nvvm_atom_cta_max_gen_ull,
    BI__nvvm_atom_cta_min_gen_i,
    BI__nvvm_atom_cta_min_gen_l,
    BI__nvvm_atom_cta_min_gen_ll,
    BI__nvvm_atom_cta_min_gen_ui,
    BI__nvvm_atom_cta_min_gen_ul,
    BI__nvvm_atom_cta_min_gen_ull,
    BI__nvvm_atom_cta_or_gen_i,
    BI__nvvm_atom_cta_or_gen_l,
    BI__nvvm_atom_cta_or_gen_ll,
    BI__nvvm_atom_cta_xchg_gen_i,
    BI__nvvm_atom_cta_xchg_gen_l,
    BI__nvvm_atom_cta_xchg_gen_ll,
    BI__nvvm_atom_cta_xor_gen_i,
    BI__nvvm_atom_cta_xor_gen_l,
    BI__nvvm_atom_cta_xor_gen_ll,
    BI__nvvm_atom_dec_gen_ui,
    BI__nvvm_atom_inc_gen_ui,
    BI__nvvm_atom_max_gen_i,
    BI__nvvm_atom_max_gen_l,
    BI__nvvm_atom_max_gen_ll,
    BI__nvvm_atom_max_gen_ui,
    BI__nvvm_atom_max_gen_ul,
    BI__nvvm_atom_max_gen_ull,
    BI__nvvm_atom_min_gen_i,
    BI__nvvm_atom_min_gen_l,
    BI__nvvm_atom_min_gen_ll,
    BI__nvvm_atom_min_gen_ui,
    BI__nvvm_atom_min_gen_ul,
    BI__nvvm_atom_min_gen_ull,
    BI__nvvm_atom_or_gen_i,
    BI__nvvm_atom_or_gen_l,
    BI__nvvm_atom_or_gen_ll,
    BI__nvvm_atom_sub_gen_i,
    BI__nvvm_atom_sub_gen_l,
    BI__nvvm_atom_sub_gen_ll,
    BI__nvvm_atom_sys_add_gen_d,
    BI__nvvm_atom_sys_add_gen_f,
    BI__nvvm_atom_sys_add_gen_i,
    BI__nvvm_atom_sys_add_gen_l,
    BI__nvvm_atom_sys_add_gen_ll,
    BI__nvvm_atom_sys_and_gen_i,
    BI__nvvm_atom_sys_and_gen_l,
    BI__nvvm_atom_sys_and_gen_ll,
    BI__nvvm_atom_sys_cas_gen_i,
    BI__nvvm_atom_sys_cas_gen_l,
    BI__nvvm_atom_sys_cas_gen_ll,
    BI__nvvm_atom_sys_cas_gen_us,
    BI__nvvm_atom_sys_dec_gen_ui,
    BI__nvvm_atom_sys_inc_gen_ui,
    BI__nvvm_atom_sys_max_gen_i,
    BI__nvvm_atom_sys_max_gen_l,
    BI__nvvm_atom_sys_max_gen_ll,
    BI__nvvm_atom_sys_max_gen_ui,
    BI__nvvm_atom_sys_max_gen_ul,
    BI__nvvm_atom_sys_max_gen_ull,
    BI__nvvm_atom_sys_min_gen_i,
    BI__nvvm_atom_sys_min_gen_l,
    BI__nvvm_atom_sys_min_gen_ll,
    BI__nvvm_atom_sys_min_gen_ui,
    BI__nvvm_atom_sys_min_gen_ul,
    BI__nvvm_atom_sys_min_gen_ull,
    BI__nvvm_atom_sys_or_gen_i,
    BI__nvvm_atom_sys_or_gen_l,
    BI__nvvm_atom_sys_or_gen_ll,
    BI__nvvm_atom_sys_xchg_gen_i,
    BI__nvvm_atom_sys_xchg_gen_l,
    BI__nvvm_atom_sys_xchg_gen_ll,
    BI__nvvm_atom_sys_xor_gen_i,
    BI__nvvm_atom_sys_xor_gen_l,
    BI__nvvm_atom_sys_xor_gen_ll,
    BI__nvvm_atom_xchg_gen_i,
    BI__nvvm_atom_xchg_gen_l,
    BI__nvvm_atom_xchg_gen_ll,
    BI__nvvm_atom_xor_gen_i,
    BI__nvvm_atom_xor_gen_l,
    BI__nvvm_atom_xor_gen_ll,
    BI__nvvm_bar0_and,
    BI__nvvm_bar0_or,
    BI__nvvm_bar0_popc,
    BI__nvvm_bar_sync,
    BI__nvvm_bar_warp_sync,
    BI__nvvm_barrier_cluster_arrive,
    BI__nvvm_barrier_cluster_arrive_relaxed,
    BI__nvvm_barrier_cluster_wait,
    BI__nvvm_barrier_sync,
    BI__nvvm_barrier_sync_cnt,
    BI__nvvm_bf16x2_to_ue8m0x2_rp,
    BI__nvvm_bf16x2_to_ue8m0x2_rp_satfinite,
    BI__nvvm_bf16x2_to_ue8m0x2_rz,
    BI__nvvm_bf16x2_to_ue8m0x2_rz_satfinite,
    BI__nvvm_ceil_d,
    BI__nvvm_ceil_f,
    BI__nvvm_ceil_ftz_f,
    BI__nvvm_compiler_error,
    BI__nvvm_compiler_warn,
    BI__nvvm_cos_approx_f,
    BI__nvvm_cos_approx_ftz_f,
    BI__nvvm_cp_async_ca_shared_global_4,
    BI__nvvm_cp_async_ca_shared_global_8,
    BI__nvvm_cp_async_ca_shared_global_16,
    BI__nvvm_cp_async_cg_shared_global_16,
    BI__nvvm_cp_async_commit_group,
    BI__nvvm_cp_async_mbarrier_arrive,
    BI__nvvm_cp_async_mbarrier_arrive_noinc,
    BI__nvvm_cp_async_mbarrier_arrive_noinc_shared,
    BI__nvvm_cp_async_mbarrier_arrive_shared,
    BI__nvvm_cp_async_wait_all,
    BI__nvvm_cp_async_wait_group,
    BI__nvvm_d2f_rm,
    BI__nvvm_d2f_rm_ftz,
    BI__nvvm_d2f_rn,
    BI__nvvm_d2f_rn_ftz,
    BI__nvvm_d2f_rp,
    BI__nvvm_d2f_rp_ftz,
    BI__nvvm_d2f_rz,
    BI__nvvm_d2f_rz_ftz,
    BI__nvvm_d2i_hi,
    BI__nvvm_d2i_lo,
    BI__nvvm_d2i_rm,
    BI__nvvm_d2i_rn,
    BI__nvvm_d2i_rp,
    BI__nvvm_d2i_rz,
    BI__nvvm_d2ll_rm,
    BI__nvvm_d2ll_rn,
    BI__nvvm_d2ll_rp,
    BI__nvvm_d2ll_rz,
    BI__nvvm_d2ui_rm,
    BI__nvvm_d2ui_rn,
    BI__nvvm_d2ui_rp,
    BI__nvvm_d2ui_rz,
    BI__nvvm_d2ull_rm,
    BI__nvvm_d2ull_rn,
    BI__nvvm_d2ull_rp,
    BI__nvvm_d2ull_rz,
    BI__nvvm_div_approx_f,
    BI__nvvm_div_approx_ftz_f,
    BI__nvvm_div_rm_d,
    BI__nvvm_div_rm_f,
    BI__nvvm_div_rm_ftz_f,
    BI__nvvm_div_rn_d,
    BI__nvvm_div_rn_f,
    BI__nvvm_div_rn_ftz_f,
    BI__nvvm_div_rp_d,
    BI__nvvm_div_rp_f,
    BI__nvvm_div_rp_ftz_f,
    BI__nvvm_div_rz_d,
    BI__nvvm_div_rz_f,
    BI__nvvm_div_rz_ftz_f,
    BI__nvvm_e2m1x2_to_f16x2_rn,
    BI__nvvm_e2m1x2_to_f16x2_rn_relu,
    BI__nvvm_e2m3x2_to_f16x2_rn,
    BI__nvvm_e2m3x2_to_f16x2_rn_relu,
    BI__nvvm_e3m2x2_to_f16x2_rn,
    BI__nvvm_e3m2x2_to_f16x2_rn_relu,
    BI__nvvm_e4m3x2_to_f16x2_rn,
    BI__nvvm_e4m3x2_to_f16x2_rn_relu,
    BI__nvvm_e5m2x2_to_f16x2_rn,
    BI__nvvm_e5m2x2_to_f16x2_rn_relu,
    BI__nvvm_ex2_approx_d,
    BI__nvvm_ex2_approx_f,
    BI__nvvm_ex2_approx_f16,
    BI__nvvm_ex2_approx_f16x2,
    BI__nvvm_ex2_approx_ftz_f,
    BI__nvvm_exit,
    BI__nvvm_f2bf16_rn,
    BI__nvvm_f2bf16_rn_relu,
    BI__nvvm_f2bf16_rn_relu_satfinite,
    BI__nvvm_f2bf16_rn_satfinite,
    BI__nvvm_f2bf16_rz,
    BI__nvvm_f2bf16_rz_relu,
    BI__nvvm_f2bf16_rz_relu_satfinite,
    BI__nvvm_f2bf16_rz_satfinite,
    BI__nvvm_f2f16_rn,
    BI__nvvm_f2f16_rn_relu,
    BI__nvvm_f2f16_rn_relu_satfinite,
    BI__nvvm_f2f16_rn_satfinite,
    BI__nvvm_f2f16_rz,
    BI__nvvm_f2f16_rz_relu,
    BI__nvvm_f2f16_rz_relu_satfinite,
    BI__nvvm_f2f16_rz_satfinite,
    BI__nvvm_f2h_rn,
    BI__nvvm_f2h_rn_ftz,
    BI__nvvm_f2i_rm,
    BI__nvvm_f2i_rm_ftz,
    BI__nvvm_f2i_rn,
    BI__nvvm_f2i_rn_ftz,
    BI__nvvm_f2i_rp,
    BI__nvvm_f2i_rp_ftz,
    BI__nvvm_f2i_rz,
    BI__nvvm_f2i_rz_ftz,
    BI__nvvm_f2ll_rm,
    BI__nvvm_f2ll_rm_ftz,
    BI__nvvm_f2ll_rn,
    BI__nvvm_f2ll_rn_ftz,
    BI__nvvm_f2ll_rp,
    BI__nvvm_f2ll_rp_ftz,
    BI__nvvm_f2ll_rz,
    BI__nvvm_f2ll_rz_ftz,
    BI__nvvm_f2tf32_rn,
    BI__nvvm_f2tf32_rn_relu,
    BI__nvvm_f2tf32_rn_relu_satfinite,
    BI__nvvm_f2tf32_rn_satfinite,
    BI__nvvm_f2tf32_rna,
    BI__nvvm_f2tf32_rna_satfinite,
    BI__nvvm_f2tf32_rz,
    BI__nvvm_f2tf32_rz_relu,
    BI__nvvm_f2tf32_rz_relu_satfinite,
    BI__nvvm_f2tf32_rz_satfinite,
    BI__nvvm_f2ui_rm,
    BI__nvvm_f2ui_rm_ftz,
    BI__nvvm_f2ui_rn,
    BI__nvvm_f2ui_rn_ftz,
    BI__nvvm_f2ui_rp,
    BI__nvvm_f2ui_rp_ftz,
    BI__nvvm_f2ui_rz,
    BI__nvvm_f2ui_rz_ftz,
    BI__nvvm_f2ull_rm,
    BI__nvvm_f2ull_rm_ftz,
    BI__nvvm_f2ull_rn,
    BI__nvvm_f2ull_rn_ftz,
    BI__nvvm_f2ull_rp,
    BI__nvvm_f2ull_rp_ftz,
    BI__nvvm_f2ull_rz,
    BI__nvvm_f2ull_rz_ftz,
    BI__nvvm_f16x2_to_e4m3x2_rn,
    BI__nvvm_f16x2_to_e4m3x2_rn_relu,
    BI__nvvm_f16x2_to_e5m2x2_rn,
    BI__nvvm_f16x2_to_e5m2x2_rn_relu,
    BI__nvvm_f32x4_to_e2m1x4_rs_relu_satfinite,
    BI__nvvm_f32x4_to_e2m1x4_rs_satfinite,
    BI__nvvm_f32x4_to_e2m3x4_rs_relu_satfinite,
    BI__nvvm_f32x4_to_e2m3x4_rs_satfinite,
    BI__nvvm_f32x4_to_e3m2x4_rs_relu_satfinite,
    BI__nvvm_f32x4_to_e3m2x4_rs_satfinite,
    BI__nvvm_f32x4_to_e4m3x4_rs_relu_satfinite,
    BI__nvvm_f32x4_to_e4m3x4_rs_satfinite,
    BI__nvvm_f32x4_to_e5m2x4_rs_relu_satfinite,
    BI__nvvm_f32x4_to_e5m2x4_rs_satfinite,
    BI__nvvm_fabs_d,
    BI__nvvm_fabs_f,
    BI__nvvm_fabs_f16,
    BI__nvvm_fabs_f16x2,
    BI__nvvm_fabs_ftz_f,
    BI__nvvm_fabs_ftz_f16,
    BI__nvvm_fabs_ftz_f16x2,
    BI__nvvm_fence_sc_cluster,
    BI__nvvm_ff2bf16x2_rn,
    BI__nvvm_ff2bf16x2_rn_relu,
    BI__nvvm_ff2bf16x2_rn_relu_satfinite,
    BI__nvvm_ff2bf16x2_rn_satfinite,
    BI__nvvm_ff2bf16x2_rs,
    BI__nvvm_ff2bf16x2_rs_relu,
    BI__nvvm_ff2bf16x2_rs_relu_satfinite,
    BI__nvvm_ff2bf16x2_rs_satfinite,
    BI__nvvm_ff2bf16x2_rz,
    BI__nvvm_ff2bf16x2_rz_relu,
    BI__nvvm_ff2bf16x2_rz_relu_satfinite,
    BI__nvvm_ff2bf16x2_rz_satfinite,
    BI__nvvm_ff2f16x2_rn,
    BI__nvvm_ff2f16x2_rn_relu,
    BI__nvvm_ff2f16x2_rn_relu_satfinite,
    BI__nvvm_ff2f16x2_rn_satfinite,
    BI__nvvm_ff2f16x2_rs,
    BI__nvvm_ff2f16x2_rs_relu,
    BI__nvvm_ff2f16x2_rs_relu_satfinite,
    BI__nvvm_ff2f16x2_rs_satfinite,
    BI__nvvm_ff2f16x2_rz,
    BI__nvvm_ff2f16x2_rz_relu,
    BI__nvvm_ff2f16x2_rz_relu_satfinite,
    BI__nvvm_ff2f16x2_rz_satfinite,
    BI__nvvm_ff_to_e2m1x2_rn_relu_satfinite,
    BI__nvvm_ff_to_e2m1x2_rn_satfinite,
    BI__nvvm_ff_to_e2m3x2_rn_relu_satfinite,
    BI__nvvm_ff_to_e2m3x2_rn_satfinite,
    BI__nvvm_ff_to_e3m2x2_rn_relu_satfinite,
    BI__nvvm_ff_to_e3m2x2_rn_satfinite,
    BI__nvvm_ff_to_e4m3x2_rn,
    BI__nvvm_ff_to_e4m3x2_rn_relu,
    BI__nvvm_ff_to_e5m2x2_rn,
    BI__nvvm_ff_to_e5m2x2_rn_relu,
    BI__nvvm_ff_to_ue8m0x2_rp,
    BI__nvvm_ff_to_ue8m0x2_rp_satfinite,
    BI__nvvm_ff_to_ue8m0x2_rz,
    BI__nvvm_ff_to_ue8m0x2_rz_satfinite,
    BI__nvvm_floor_d,
    BI__nvvm_floor_f,
    BI__nvvm_floor_ftz_f,
    BI__nvvm_fma_rm_d,
    BI__nvvm_fma_rm_f,
    BI__nvvm_fma_rm_ftz_f,
    BI__nvvm_fma_rm_ftz_sat_f,
    BI__nvvm_fma_rm_sat_f,
    BI__nvvm_fma_rn_bf16,
    BI__nvvm_fma_rn_bf16x2,
    BI__nvvm_fma_rn_d,
    BI__nvvm_fma_rn_f,
    BI__nvvm_fma_rn_f16,
    BI__nvvm_fma_rn_f16x2,
    BI__nvvm_fma_rn_ftz_f,
    BI__nvvm_fma_rn_ftz_f16,
    BI__nvvm_fma_rn_ftz_f16x2,
    BI__nvvm_fma_rn_ftz_relu_f16,
    BI__nvvm_fma_rn_ftz_relu_f16x2,
    BI__nvvm_fma_rn_ftz_sat_f,
    BI__nvvm_fma_rn_ftz_sat_f16,
    BI__nvvm_fma_rn_ftz_sat_f16x2,
    BI__nvvm_fma_rn_relu_bf16,
    BI__nvvm_fma_rn_relu_bf16x2,
    BI__nvvm_fma_rn_relu_f16,
    BI__nvvm_fma_rn_relu_f16x2,
    BI__nvvm_fma_rn_sat_f,
    BI__nvvm_fma_rn_sat_f16,
    BI__nvvm_fma_rn_sat_f16x2,
    BI__nvvm_fma_rp_d,
    BI__nvvm_fma_rp_f,
    BI__nvvm_fma_rp_ftz_f,
    BI__nvvm_fma_rp_ftz_sat_f,
    BI__nvvm_fma_rp_sat_f,
    BI__nvvm_fma_rz_d,
    BI__nvvm_fma_rz_f,
    BI__nvvm_fma_rz_ftz_f,
    BI__nvvm_fma_rz_ftz_sat_f,
    BI__nvvm_fma_rz_sat_f,
    BI__nvvm_fmax_bf16,
    BI__nvvm_fmax_bf16x2,
    BI__nvvm_fmax_d,
    BI__nvvm_fmax_f,
    BI__nvvm_fmax_f16,
    BI__nvvm_fmax_f16x2,
    BI__nvvm_fmax_ftz_bf16,
    BI__nvvm_fmax_ftz_bf16x2,
    BI__nvvm_fmax_ftz_f,
    BI__nvvm_fmax_ftz_f16,
    BI__nvvm_fmax_ftz_f16x2,
    BI__nvvm_fmax_ftz_nan_bf16,
    BI__nvvm_fmax_ftz_nan_bf16x2,
    BI__nvvm_fmax_ftz_nan_f,
    BI__nvvm_fmax_ftz_nan_f16,
    BI__nvvm_fmax_ftz_nan_f16x2,
    BI__nvvm_fmax_ftz_nan_xorsign_abs_f,
    BI__nvvm_fmax_ftz_nan_xorsign_abs_f16,
    BI__nvvm_fmax_ftz_nan_xorsign_abs_f16x2,
    BI__nvvm_fmax_ftz_xorsign_abs_f,
    BI__nvvm_fmax_ftz_xorsign_abs_f16,
    BI__nvvm_fmax_ftz_xorsign_abs_f16x2,
    BI__nvvm_fmax_nan_bf16,
    BI__nvvm_fmax_nan_bf16x2,
    BI__nvvm_fmax_nan_f,
    BI__nvvm_fmax_nan_f16,
    BI__nvvm_fmax_nan_f16x2,
    BI__nvvm_fmax_nan_xorsign_abs_bf16,
    BI__nvvm_fmax_nan_xorsign_abs_bf16x2,
    BI__nvvm_fmax_nan_xorsign_abs_f,
    BI__nvvm_fmax_nan_xorsign_abs_f16,
    BI__nvvm_fmax_nan_xorsign_abs_f16x2,
    BI__nvvm_fmax_xorsign_abs_bf16,
    BI__nvvm_fmax_xorsign_abs_bf16x2,
    BI__nvvm_fmax_xorsign_abs_f,
    BI__nvvm_fmax_xorsign_abs_f16,
    BI__nvvm_fmax_xorsign_abs_f16x2,
    BI__nvvm_fmin_bf16,
    BI__nvvm_fmin_bf16x2,
    BI__nvvm_fmin_d,
    BI__nvvm_fmin_f,
    BI__nvvm_fmin_f16,
    BI__nvvm_fmin_f16x2,
    BI__nvvm_fmin_ftz_bf16,
    BI__nvvm_fmin_ftz_bf16x2,
    BI__nvvm_fmin_ftz_f,
    BI__nvvm_fmin_ftz_f16,
    BI__nvvm_fmin_ftz_f16x2,
    BI__nvvm_fmin_ftz_nan_bf16,
    BI__nvvm_fmin_ftz_nan_bf16x2,
    BI__nvvm_fmin_ftz_nan_f,
    BI__nvvm_fmin_ftz_nan_f16,
    BI__nvvm_fmin_ftz_nan_f16x2,
    BI__nvvm_fmin_ftz_nan_xorsign_abs_f,
    BI__nvvm_fmin_ftz_nan_xorsign_abs_f16,
    BI__nvvm_fmin_ftz_nan_xorsign_abs_f16x2,
    BI__nvvm_fmin_ftz_xorsign_abs_f,
    BI__nvvm_fmin_ftz_xorsign_abs_f16,
    BI__nvvm_fmin_ftz_xorsign_abs_f16x2,
    BI__nvvm_fmin_nan_bf16,
    BI__nvvm_fmin_nan_bf16x2,
    BI__nvvm_fmin_nan_f,
    BI__nvvm_fmin_nan_f16,
    BI__nvvm_fmin_nan_f16x2,
    BI__nvvm_fmin_nan_xorsign_abs_bf16,
    BI__nvvm_fmin_nan_xorsign_abs_bf16x2,
    BI__nvvm_fmin_nan_xorsign_abs_f,
    BI__nvvm_fmin_nan_xorsign_abs_f16,
    BI__nvvm_fmin_nan_xorsign_abs_f16x2,
    BI__nvvm_fmin_xorsign_abs_bf16,
    BI__nvvm_fmin_xorsign_abs_bf16x2,
    BI__nvvm_fmin_xorsign_abs_f,
    BI__nvvm_fmin_xorsign_abs_f16,
    BI__nvvm_fmin_xorsign_abs_f16x2,
    BI__nvvm_fns,
    BI__nvvm_getctarank,
    BI__nvvm_getctarank_shared_cluster,
    BI__nvvm_i2d_rm,
    BI__nvvm_i2d_rn,
    BI__nvvm_i2d_rp,
    BI__nvvm_i2d_rz,
    BI__nvvm_i2f_rm,
    BI__nvvm_i2f_rn,
    BI__nvvm_i2f_rp,
    BI__nvvm_i2f_rz,
    BI__nvvm_is_explicit_cluster,
    BI__nvvm_isspacep_const,
    BI__nvvm_isspacep_global,
    BI__nvvm_isspacep_local,
    BI__nvvm_isspacep_shared,
    BI__nvvm_isspacep_shared_cluster,
    BI__nvvm_ldg_c,
    BI__nvvm_ldg_c2,
    BI__nvvm_ldg_c4,
    BI__nvvm_ldg_d,
    BI__nvvm_ldg_d2,
    BI__nvvm_ldg_f,
    BI__nvvm_ldg_f2,
    BI__nvvm_ldg_f4,
    BI__nvvm_ldg_h,
    BI__nvvm_ldg_h2,
    BI__nvvm_ldg_i,
    BI__nvvm_ldg_i2,
    BI__nvvm_ldg_i4,
    BI__nvvm_ldg_l,
    BI__nvvm_ldg_l2,
    BI__nvvm_ldg_ll,
    BI__nvvm_ldg_ll2,
    BI__nvvm_ldg_s,
    BI__nvvm_ldg_s2,
    BI__nvvm_ldg_s4,
    BI__nvvm_ldg_sc,
    BI__nvvm_ldg_sc2,
    BI__nvvm_ldg_sc4,
    BI__nvvm_ldg_uc,
    BI__nvvm_ldg_uc2,
    BI__nvvm_ldg_uc4,
    BI__nvvm_ldg_ui,
    BI__nvvm_ldg_ui2,
    BI__nvvm_ldg_ui4,
    BI__nvvm_ldg_ul,
    BI__nvvm_ldg_ul2,
    BI__nvvm_ldg_ull,
    BI__nvvm_ldg_ull2,
    BI__nvvm_ldg_us,
    BI__nvvm_ldg_us2,
    BI__nvvm_ldg_us4,
    BI__nvvm_ldu_c,
    BI__nvvm_ldu_c2,
    BI__nvvm_ldu_c4,
    BI__nvvm_ldu_d,
    BI__nvvm_ldu_d2,
    BI__nvvm_ldu_f,
    BI__nvvm_ldu_f2,
    BI__nvvm_ldu_f4,
    BI__nvvm_ldu_h,
    BI__nvvm_ldu_h2,
    BI__nvvm_ldu_i,
    BI__nvvm_ldu_i2,
    BI__nvvm_ldu_i4,
    BI__nvvm_ldu_l,
    BI__nvvm_ldu_l2,
    BI__nvvm_ldu_ll,
    BI__nvvm_ldu_ll2,
    BI__nvvm_ldu_s,
    BI__nvvm_ldu_s2,
    BI__nvvm_ldu_s4,
    BI__nvvm_ldu_sc,
    BI__nvvm_ldu_sc2,
    BI__nvvm_ldu_sc4,
    BI__nvvm_ldu_uc,
    BI__nvvm_ldu_uc2,
    BI__nvvm_ldu_uc4,
    BI__nvvm_ldu_ui,
    BI__nvvm_ldu_ui2,
    BI__nvvm_ldu_ui4,
    BI__nvvm_ldu_ul,
    BI__nvvm_ldu_ul2,
    BI__nvvm_ldu_ull,
    BI__nvvm_ldu_ull2,
    BI__nvvm_ldu_us,
    BI__nvvm_ldu_us2,
    BI__nvvm_ldu_us4,
    BI__nvvm_lg2_approx_d,
    BI__nvvm_lg2_approx_f,
    BI__nvvm_lg2_approx_ftz_f,
    BI__nvvm_ll2d_rm,
    BI__nvvm_ll2d_rn,
    BI__nvvm_ll2d_rp,
    BI__nvvm_ll2d_rz,
    BI__nvvm_ll2f_rm,
    BI__nvvm_ll2f_rn,
    BI__nvvm_ll2f_rp,
    BI__nvvm_ll2f_rz,
    BI__nvvm_lohi_i2d,
    BI__nvvm_mapa,
    BI__nvvm_mapa_shared_cluster,
    BI__nvvm_match_all_sync_i32p,
    BI__nvvm_match_all_sync_i64p,
    BI__nvvm_match_any_sync_i32,
    BI__nvvm_match_any_sync_i64,
    BI__nvvm_mbarrier_arrive,
    BI__nvvm_mbarrier_arrive_drop,
    BI__nvvm_mbarrier_arrive_drop_noComplete,
    BI__nvvm_mbarrier_arrive_drop_noComplete_shared,
    BI__nvvm_mbarrier_arrive_drop_shared,
    BI__nvvm_mbarrier_arrive_noComplete,
    BI__nvvm_mbarrier_arrive_noComplete_shared,
    BI__nvvm_mbarrier_arrive_shared,
    BI__nvvm_mbarrier_init,
    BI__nvvm_mbarrier_init_shared,
    BI__nvvm_mbarrier_inval,
    BI__nvvm_mbarrier_inval_shared,
    BI__nvvm_mbarrier_pending_count,
    BI__nvvm_mbarrier_test_wait,
    BI__nvvm_mbarrier_test_wait_shared,
    BI__nvvm_membar_cta,
    BI__nvvm_membar_gl,
    BI__nvvm_membar_sys,
    BI__nvvm_memcpy,
    BI__nvvm_memset,
    BI__nvvm_mul24_i,
    BI__nvvm_mul24_ui,
    BI__nvvm_mul_rm_d,
    BI__nvvm_mul_rm_f,
    BI__nvvm_mul_rm_ftz_f,
    BI__nvvm_mul_rn_d,
    BI__nvvm_mul_rn_f,
    BI__nvvm_mul_rn_ftz_f,
    BI__nvvm_mul_rp_d,
    BI__nvvm_mul_rp_f,
    BI__nvvm_mul_rp_ftz_f,
    BI__nvvm_mul_rz_d,
    BI__nvvm_mul_rz_f,
    BI__nvvm_mul_rz_ftz_f,
    BI__nvvm_mulhi_i,
    BI__nvvm_mulhi_ll,
    BI__nvvm_mulhi_ui,
    BI__nvvm_mulhi_ull,
    BI__nvvm_nanosleep,
    BI__nvvm_neg_bf16,
    BI__nvvm_neg_bf16x2,
    BI__nvvm_pm_event_mask,
    BI__nvvm_prmt,
    BI__nvvm_rcp_approx_ftz_d,
    BI__nvvm_rcp_approx_ftz_f,
    BI__nvvm_rcp_rm_d,
    BI__nvvm_rcp_rm_f,
    BI__nvvm_rcp_rm_ftz_f,
    BI__nvvm_rcp_rn_d,
    BI__nvvm_rcp_rn_f,
    BI__nvvm_rcp_rn_ftz_f,
    BI__nvvm_rcp_rp_d,
    BI__nvvm_rcp_rp_f,
    BI__nvvm_rcp_rp_ftz_f,
    BI__nvvm_rcp_rz_d,
    BI__nvvm_rcp_rz_f,
    BI__nvvm_rcp_rz_ftz_f,
    BI__nvvm_read_ptx_sreg_clock,
    BI__nvvm_read_ptx_sreg_clock64,
    BI__nvvm_read_ptx_sreg_cluster_ctaid_w,
    BI__nvvm_read_ptx_sreg_cluster_ctaid_x,
    BI__nvvm_read_ptx_sreg_cluster_ctaid_y,
    BI__nvvm_read_ptx_sreg_cluster_ctaid_z,
    BI__nvvm_read_ptx_sreg_cluster_ctarank,
    BI__nvvm_read_ptx_sreg_cluster_nctaid_w,
    BI__nvvm_read_ptx_sreg_cluster_nctaid_x,
    BI__nvvm_read_ptx_sreg_cluster_nctaid_y,
    BI__nvvm_read_ptx_sreg_cluster_nctaid_z,
    BI__nvvm_read_ptx_sreg_cluster_nctarank,
    BI__nvvm_read_ptx_sreg_clusterid_w,
    BI__nvvm_read_ptx_sreg_clusterid_x,
    BI__nvvm_read_ptx_sreg_clusterid_y,
    BI__nvvm_read_ptx_sreg_clusterid_z,
    BI__nvvm_read_ptx_sreg_ctaid_w,
    BI__nvvm_read_ptx_sreg_ctaid_x,
    BI__nvvm_read_ptx_sreg_ctaid_y,
    BI__nvvm_read_ptx_sreg_ctaid_z,
    BI__nvvm_read_ptx_sreg_globaltimer,
    BI__nvvm_read_ptx_sreg_gridid,
    BI__nvvm_read_ptx_sreg_laneid,
    BI__nvvm_read_ptx_sreg_lanemask_eq,
    BI__nvvm_read_ptx_sreg_lanemask_ge,
    BI__nvvm_read_ptx_sreg_lanemask_gt,
    BI__nvvm_read_ptx_sreg_lanemask_le,
    BI__nvvm_read_ptx_sreg_lanemask_lt,
    BI__nvvm_read_ptx_sreg_nclusterid_w,
    BI__nvvm_read_ptx_sreg_nclusterid_x,
    BI__nvvm_read_ptx_sreg_nclusterid_y,
    BI__nvvm_read_ptx_sreg_nclusterid_z,
    BI__nvvm_read_ptx_sreg_nctaid_w,
    BI__nvvm_read_ptx_sreg_nctaid_x,
    BI__nvvm_read_ptx_sreg_nctaid_y,
    BI__nvvm_read_ptx_sreg_nctaid_z,
    BI__nvvm_read_ptx_sreg_nsmid,
    BI__nvvm_read_ptx_sreg_ntid_w,
    BI__nvvm_read_ptx_sreg_ntid_x,
    BI__nvvm_read_ptx_sreg_ntid_y,
    BI__nvvm_read_ptx_sreg_ntid_z,
    BI__nvvm_read_ptx_sreg_nwarpid,
    BI__nvvm_read_ptx_sreg_pm0,
    BI__nvvm_read_ptx_sreg_pm1,
    BI__nvvm_read_ptx_sreg_pm2,
    BI__nvvm_read_ptx_sreg_pm3,
    BI__nvvm_read_ptx_sreg_smid,
    BI__nvvm_read_ptx_sreg_tid_w,
    BI__nvvm_read_ptx_sreg_tid_x,
    BI__nvvm_read_ptx_sreg_tid_y,
    BI__nvvm_read_ptx_sreg_tid_z,
    BI__nvvm_read_ptx_sreg_warpid,
    BI__nvvm_read_ptx_sreg_warpsize,
    BI__nvvm_redux_sync_add,
    BI__nvvm_redux_sync_and,
    BI__nvvm_redux_sync_fmax,
    BI__nvvm_redux_sync_fmax_NaN,
    BI__nvvm_redux_sync_fmax_abs,
    BI__nvvm_redux_sync_fmax_abs_NaN,
    BI__nvvm_redux_sync_fmin,
    BI__nvvm_redux_sync_fmin_NaN,
    BI__nvvm_redux_sync_fmin_abs,
    BI__nvvm_redux_sync_fmin_abs_NaN,
    BI__nvvm_redux_sync_max,
    BI__nvvm_redux_sync_min,
    BI__nvvm_redux_sync_or,
    BI__nvvm_redux_sync_umax,
    BI__nvvm_redux_sync_umin,
    BI__nvvm_redux_sync_xor,
    BI__nvvm_reflect,
    BI__nvvm_round_d,
    BI__nvvm_round_f,
    BI__nvvm_round_ftz_f,
    BI__nvvm_rsqrt_approx_d,
    BI__nvvm_rsqrt_approx_f,
    BI__nvvm_rsqrt_approx_ftz_f,
    BI__nvvm_sad_i,
    BI__nvvm_sad_ui,
    BI__nvvm_saturate_d,
    BI__nvvm_saturate_f,
    BI__nvvm_saturate_ftz_f,
    BI__nvvm_shfl_bfly_f32,
    BI__nvvm_shfl_bfly_i32,
    BI__nvvm_shfl_down_f32,
    BI__nvvm_shfl_down_i32,
    BI__nvvm_shfl_idx_f32,
    BI__nvvm_shfl_idx_i32,
    BI__nvvm_shfl_sync_bfly_f32,
    BI__nvvm_shfl_sync_bfly_i32,
    BI__nvvm_shfl_sync_down_f32,
    BI__nvvm_shfl_sync_down_i32,
    BI__nvvm_shfl_sync_idx_f32,
    BI__nvvm_shfl_sync_idx_i32,
    BI__nvvm_shfl_sync_up_f32,
    BI__nvvm_shfl_sync_up_i32,
    BI__nvvm_shfl_up_f32,
    BI__nvvm_shfl_up_i32,
    BI__nvvm_sin_approx_f,
    BI__nvvm_sin_approx_ftz_f,
    BI__nvvm_sqrt_approx_f,
    BI__nvvm_sqrt_approx_ftz_f,
    BI__nvvm_sqrt_rm_d,
    BI__nvvm_sqrt_rm_f,
    BI__nvvm_sqrt_rm_ftz_f,
    BI__nvvm_sqrt_rn_d,
    BI__nvvm_sqrt_rn_f,
    BI__nvvm_sqrt_rn_ftz_f,
    BI__nvvm_sqrt_rp_d,
    BI__nvvm_sqrt_rp_f,
    BI__nvvm_sqrt_rp_ftz_f,
    BI__nvvm_sqrt_rz_d,
    BI__nvvm_sqrt_rz_f,
    BI__nvvm_sqrt_rz_ftz_f,
    BI__nvvm_trunc_d,
    BI__nvvm_trunc_f,
    BI__nvvm_trunc_ftz_f,
    BI__nvvm_ue8m0x2_to_bf16x2,
    BI__nvvm_ui2d_rm,
    BI__nvvm_ui2d_rn,
    BI__nvvm_ui2d_rp,
    BI__nvvm_ui2d_rz,
    BI__nvvm_ui2f_rm,
    BI__nvvm_ui2f_rn,
    BI__nvvm_ui2f_rp,
    BI__nvvm_ui2f_rz,
    BI__nvvm_ull2d_rm,
    BI__nvvm_ull2d_rn,
    BI__nvvm_ull2d_rp,
    BI__nvvm_ull2d_rz,
    BI__nvvm_ull2f_rm,
    BI__nvvm_ull2f_rn,
    BI__nvvm_ull2f_rp,
    BI__nvvm_ull2f_rz,
    BI__nvvm_vote_all,
    BI__nvvm_vote_all_sync,
    BI__nvvm_vote_any,
    BI__nvvm_vote_any_sync,
    BI__nvvm_vote_ballot,
    BI__nvvm_vote_ballot_sync,
    BI__nvvm_vote_uni,
    BI__nvvm_vote_uni_sync,
    BI__syncthreads,

#endif // GET_BUILTIN_ENUMERATORS

#ifdef GET_BUILTIN_STR_TABLE

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinStringsStorage[] =
  "\0"
  "__bmma_m8n8k128_ld_a_b1\0"
  "vi*iC*UiIi\0"
  "(sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__bmma_m8n8k128_ld_b_b1\0"
  "__bmma_m8n8k128_ld_c\0"
  "__bmma_m8n8k128_mma_and_popc_b1\0"
  "vi*iC*iC*iC*Ii\0"
  "(sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__bmma_m8n8k128_mma_xor_popc_b1\0"
  "__bmma_m8n8k128_st_c_i32\0"
  "__builtin_ptx_get_image_channel_data_typei_\0"
  "ii\0"
  "__builtin_ptx_get_image_channel_orderi_\0"
  "__builtin_ptx_get_image_depthi_\0"
  "__builtin_ptx_get_image_heighti_\0"
  "__builtin_ptx_get_image_widthi_\0"
  "__builtin_ptx_read_image2Dff_\0"
  "V4fiiff\0"
  "__builtin_ptx_read_image2Dfi_\0"
  "V4fiiii\0"
  "__builtin_ptx_read_image2Dif_\0"
  "V4iiiff\0"
  "__builtin_ptx_read_image2Dii_\0"
  "V4iiiii\0"
  "__builtin_ptx_read_image3Dff_\0"
  "V4fiiffff\0"
  "__builtin_ptx_read_image3Dfi_\0"
  "V4fiiiiii\0"
  "__builtin_ptx_read_image3Dif_\0"
  "V4iiiffff\0"
  "__builtin_ptx_read_image3Dii_\0"
  "V4iiiiiii\0"
  "__builtin_ptx_write_image2Df_\0"
  "viiiffff\0"
  "__builtin_ptx_write_image2Di_\0"
  "viiiiiii\0"
  "__builtin_ptx_write_image2Dui_\0"
  "viiiUiUiUiUi\0"
  "__dmma_m8n8k4_ld_a\0"
  "vd*dC*UiIi\0"
  "(sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__dmma_m8n8k4_ld_b\0"
  "__dmma_m8n8k4_ld_c\0"
  "__dmma_m8n8k4_mma_f64\0"
  "vd*dC*dC*dC*IiIi\0"
  "__dmma_m8n8k4_st_c_f64\0"
  "__hmma_m8n32k16_ld_a\0"
  "(sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__hmma_m8n32k16_ld_b\0"
  "__hmma_m8n32k16_ld_c_f16\0"
  "__hmma_m8n32k16_ld_c_f32\0"
  "vf*fC*UiIi\0"
  "__hmma_m8n32k16_mma_f16f16\0"
  "vi*iC*iC*iC*IiIi\0"
  "__hmma_m8n32k16_mma_f16f32\0"
  "vi*iC*iC*fC*IiIi\0"
  "__hmma_m8n32k16_mma_f32f16\0"
  "vf*iC*iC*iC*IiIi\0"
  "__hmma_m8n32k16_mma_f32f32\0"
  "vf*iC*iC*fC*IiIi\0"
  "__hmma_m8n32k16_st_c_f16\0"
  "__hmma_m8n32k16_st_c_f32\0"
  "__hmma_m16n16k16_ld_a\0"
  "(sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__hmma_m16n16k16_ld_b\0"
  "__hmma_m16n16k16_ld_c_f16\0"
  "__hmma_m16n16k16_ld_c_f32\0"
  "__hmma_m16n16k16_mma_f16f16\0"
  "__hmma_m16n16k16_mma_f16f32\0"
  "__hmma_m16n16k16_mma_f32f16\0"
  "__hmma_m16n16k16_mma_f32f32\0"
  "__hmma_m16n16k16_st_c_f16\0"
  "__hmma_m16n16k16_st_c_f32\0"
  "__hmma_m32n8k16_ld_a\0"
  "__hmma_m32n8k16_ld_b\0"
  "__hmma_m32n8k16_ld_c_f16\0"
  "__hmma_m32n8k16_ld_c_f32\0"
  "__hmma_m32n8k16_mma_f16f16\0"
  "__hmma_m32n8k16_mma_f16f32\0"
  "__hmma_m32n8k16_mma_f32f16\0"
  "__hmma_m32n8k16_mma_f32f32\0"
  "__hmma_m32n8k16_st_c_f16\0"
  "__hmma_m32n8k16_st_c_f32\0"
  "__imma_m8n8k32_ld_a_s4\0"
  "__imma_m8n8k32_ld_a_u4\0"
  "__imma_m8n8k32_ld_b_s4\0"
  "__imma_m8n8k32_ld_b_u4\0"
  "__imma_m8n8k32_ld_c\0"
  "__imma_m8n8k32_mma_s4\0"
  "__imma_m8n8k32_mma_u4\0"
  "__imma_m8n8k32_st_c_i32\0"
  "__imma_m8n32k16_ld_a_s8\0"
  "(sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__imma_m8n32k16_ld_a_u8\0"
  "__imma_m8n32k16_ld_b_s8\0"
  "__imma_m8n32k16_ld_b_u8\0"
  "__imma_m8n32k16_ld_c\0"
  "__imma_m8n32k16_mma_s8\0"
  "__imma_m8n32k16_mma_u8\0"
  "__imma_m8n32k16_st_c_i32\0"
  "__imma_m16n16k16_ld_a_s8\0"
  "__imma_m16n16k16_ld_a_u8\0"
  "__imma_m16n16k16_ld_b_s8\0"
  "__imma_m16n16k16_ld_b_u8\0"
  "__imma_m16n16k16_ld_c\0"
  "__imma_m16n16k16_mma_s8\0"
  "__imma_m16n16k16_mma_u8\0"
  "__imma_m16n16k16_st_c_i32\0"
  "__imma_m32n8k16_ld_a_s8\0"
  "__imma_m32n8k16_ld_a_u8\0"
  "__imma_m32n8k16_ld_b_s8\0"
  "__imma_m32n8k16_ld_b_u8\0"
  "__imma_m32n8k16_ld_c\0"
  "__imma_m32n8k16_mma_s8\0"
  "__imma_m32n8k16_mma_u8\0"
  "__imma_m32n8k16_st_c_i32\0"
  "__mma_bf16_m8n32k16_ld_a\0"
  "__mma_bf16_m8n32k16_ld_b\0"
  "__mma_bf16_m8n32k16_mma_f32\0"
  "__mma_bf16_m16n16k16_ld_a\0"
  "__mma_bf16_m16n16k16_ld_b\0"
  "__mma_bf16_m16n16k16_mma_f32\0"
  "__mma_bf16_m32n8k16_ld_a\0"
  "__mma_bf16_m32n8k16_ld_b\0"
  "__mma_bf16_m32n8k16_mma_f32\0"
  "__mma_m16n16k8_st_c_f32\0"
  "__mma_tf32_m16n16k8_ld_a\0"
  "__mma_tf32_m16n16k8_ld_b\0"
  "__mma_tf32_m16n16k8_ld_c\0"
  "__mma_tf32_m16n16k8_mma_f32\0"
  "__nvvm_abs_bf16\0"
  "yy\0"
  "__nvvm_abs_bf16x2\0"
  "V2yV2y\0"
  "__nvvm_activemask\0"
  "Ui\0"
  "n\0"
  "ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90\0"
  "__nvvm_add_rm_d\0"
  "ddd\0"
  "__nvvm_add_rm_f\0"
  "fff\0"
  "__nvvm_add_rm_ftz_f\0"
  "__nvvm_add_rm_ftz_sat_f\0"
  "__nvvm_add_rm_sat_f\0"
  "__nvvm_add_rn_d\0"
  "__nvvm_add_rn_f\0"
  "__nvvm_add_rn_ftz_f\0"
  "__nvvm_add_rn_ftz_sat_f\0"
  "__nvvm_add_rn_sat_f\0"
  "__nvvm_add_rp_d\0"
  "__nvvm_add_rp_f\0"
  "__nvvm_add_rp_ftz_f\0"
  "__nvvm_add_rp_ftz_sat_f\0"
  "__nvvm_add_rp_sat_f\0"
  "__nvvm_add_rz_d\0"
  "__nvvm_add_rz_f\0"
  "__nvvm_add_rz_ftz_f\0"
  "__nvvm_add_rz_ftz_sat_f\0"
  "__nvvm_add_rz_sat_f\0"
  "__nvvm_atom_add_gen_d\0"
  "ddD*d\0"
  "sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a\0"
  "__nvvm_atom_add_gen_f\0"
  "ffD*f\0"
  "__nvvm_atom_add_gen_i\0"
  "iiD*i\0"
  "__nvvm_atom_add_gen_l\0"
  "LiLiD*Li\0"
  "__nvvm_atom_add_gen_ll\0"
  "LLiLLiD*LLi\0"
  "__nvvm_atom_and_gen_i\0"
  "__nvvm_atom_and_gen_l\0"
  "__nvvm_atom_and_gen_ll\0"
  "__nvvm_atom_cas_gen_i\0"
  "iiD*ii\0"
  "__nvvm_atom_cas_gen_l\0"
  "LiLiD*LiLi\0"
  "__nvvm_atom_cas_gen_ll\0"
  "LLiLLiD*LLiLLi\0"
  "__nvvm_atom_cas_gen_us\0"
  "UsUsD*UsUs\0"
  "sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a\0"
  "__nvvm_atom_cta_add_gen_d\0"
  "__nvvm_atom_cta_add_gen_f\0"
  "__nvvm_atom_cta_add_gen_i\0"
  "__nvvm_atom_cta_add_gen_l\0"
  "__nvvm_atom_cta_add_gen_ll\0"
  "__nvvm_atom_cta_and_gen_i\0"
  "__nvvm_atom_cta_and_gen_l\0"
  "__nvvm_atom_cta_and_gen_ll\0"
  "__nvvm_atom_cta_cas_gen_i\0"
  "__nvvm_atom_cta_cas_gen_l\0"
  "__nvvm_atom_cta_cas_gen_ll\0"
  "__nvvm_atom_cta_cas_gen_us\0"
  "__nvvm_atom_cta_dec_gen_ui\0"
  "UiUiD*Ui\0"
  "__nvvm_atom_cta_inc_gen_ui\0"
  "__nvvm_atom_cta_max_gen_i\0"
  "__nvvm_atom_cta_max_gen_l\0"
  "__nvvm_atom_cta_max_gen_ll\0"
  "__nvvm_atom_cta_max_gen_ui\0"
  "__nvvm_atom_cta_max_gen_ul\0"
  "ULiULiD*ULi\0"
  "__nvvm_atom_cta_max_gen_ull\0"
  "ULLiULLiD*ULLi\0"
  "__nvvm_atom_cta_min_gen_i\0"
  "__nvvm_atom_cta_min_gen_l\0"
  "__nvvm_atom_cta_min_gen_ll\0"
  "__nvvm_atom_cta_min_gen_ui\0"
  "__nvvm_atom_cta_min_gen_ul\0"
  "__nvvm_atom_cta_min_gen_ull\0"
  "__nvvm_atom_cta_or_gen_i\0"
  "__nvvm_atom_cta_or_gen_l\0"
  "__nvvm_atom_cta_or_gen_ll\0"
  "__nvvm_atom_cta_xchg_gen_i\0"
  "__nvvm_atom_cta_xchg_gen_l\0"
  "__nvvm_atom_cta_xchg_gen_ll\0"
  "__nvvm_atom_cta_xor_gen_i\0"
  "__nvvm_atom_cta_xor_gen_l\0"
  "__nvvm_atom_cta_xor_gen_ll\0"
  "__nvvm_atom_dec_gen_ui\0"
  "__nvvm_atom_inc_gen_ui\0"
  "__nvvm_atom_max_gen_i\0"
  "__nvvm_atom_max_gen_l\0"
  "__nvvm_atom_max_gen_ll\0"
  "__nvvm_atom_max_gen_ui\0"
  "__nvvm_atom_max_gen_ul\0"
  "__nvvm_atom_max_gen_ull\0"
  "__nvvm_atom_min_gen_i\0"
  "__nvvm_atom_min_gen_l\0"
  "__nvvm_atom_min_gen_ll\0"
  "__nvvm_atom_min_gen_ui\0"
  "__nvvm_atom_min_gen_ul\0"
  "__nvvm_atom_min_gen_ull\0"
  "__nvvm_atom_or_gen_i\0"
  "__nvvm_atom_or_gen_l\0"
  "__nvvm_atom_or_gen_ll\0"
  "__nvvm_atom_sub_gen_i\0"
  "__nvvm_atom_sub_gen_l\0"
  "__nvvm_atom_sub_gen_ll\0"
  "__nvvm_atom_sys_add_gen_d\0"
  "__nvvm_atom_sys_add_gen_f\0"
  "__nvvm_atom_sys_add_gen_i\0"
  "__nvvm_atom_sys_add_gen_l\0"
  "__nvvm_atom_sys_add_gen_ll\0"
  "__nvvm_atom_sys_and_gen_i\0"
  "__nvvm_atom_sys_and_gen_l\0"
  "__nvvm_atom_sys_and_gen_ll\0"
  "__nvvm_atom_sys_cas_gen_i\0"
  "__nvvm_atom_sys_cas_gen_l\0"
  "__nvvm_atom_sys_cas_gen_ll\0"
  "__nvvm_atom_sys_cas_gen_us\0"
  "__nvvm_atom_sys_dec_gen_ui\0"
  "__nvvm_atom_sys_inc_gen_ui\0"
  "__nvvm_atom_sys_max_gen_i\0"
  "__nvvm_atom_sys_max_gen_l\0"
  "__nvvm_atom_sys_max_gen_ll\0"
  "__nvvm_atom_sys_max_gen_ui\0"
  "__nvvm_atom_sys_max_gen_ul\0"
  "__nvvm_atom_sys_max_gen_ull\0"
  "__nvvm_atom_sys_min_gen_i\0"
  "__nvvm_atom_sys_min_gen_l\0"
  "__nvvm_atom_sys_min_gen_ll\0"
  "__nvvm_atom_sys_min_gen_ui\0"
  "__nvvm_atom_sys_min_gen_ul\0"
  "__nvvm_atom_sys_min_gen_ull\0"
  "__nvvm_atom_sys_or_gen_i\0"
  "__nvvm_atom_sys_or_gen_l\0"
  "__nvvm_atom_sys_or_gen_ll\0"
  "__nvvm_atom_sys_xchg_gen_i\0"
  "__nvvm_atom_sys_xchg_gen_l\0"
  "__nvvm_atom_sys_xchg_gen_ll\0"
  "__nvvm_atom_sys_xor_gen_i\0"
  "__nvvm_atom_sys_xor_gen_l\0"
  "__nvvm_atom_sys_xor_gen_ll\0"
  "__nvvm_atom_xchg_gen_i\0"
  "__nvvm_atom_xchg_gen_l\0"
  "__nvvm_atom_xchg_gen_ll\0"
  "__nvvm_atom_xor_gen_i\0"
  "__nvvm_atom_xor_gen_l\0"
  "__nvvm_atom_xor_gen_ll\0"
  "__nvvm_bar0_and\0"
  "__nvvm_bar0_or\0"
  "__nvvm_bar0_popc\0"
  "__nvvm_bar_sync\0"
  "vi\0"
  "__nvvm_bar_warp_sync\0"
  "vUi\0"
  "ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90\0"
  "__nvvm_barrier_cluster_arrive\0"
  "v\0"
  "(sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_barrier_cluster_arrive_relaxed\0"
  "(sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_barrier_cluster_wait\0"
  "__nvvm_barrier_sync\0"
  "__nvvm_barrier_sync_cnt\0"
  "vUiUi\0"
  "__nvvm_bf16x2_to_ue8m0x2_rp\0"
  "sV2y\0"
  "(sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_bf16x2_to_ue8m0x2_rp_satfinite\0"
  "__nvvm_bf16x2_to_ue8m0x2_rz\0"
  "__nvvm_bf16x2_to_ue8m0x2_rz_satfinite\0"
  "__nvvm_ceil_d\0"
  "dd\0"
  "__nvvm_ceil_f\0"
  "ff\0"
  "__nvvm_ceil_ftz_f\0"
  "__nvvm_compiler_error\0"
  "vcC*4\0"
  "__nvvm_compiler_warn\0"
  "__nvvm_cos_approx_f\0"
  "__nvvm_cos_approx_ftz_f\0"
  "__nvvm_cp_async_ca_shared_global_4\0"
  "vv*3vC*1.\0"
  "__nvvm_cp_async_ca_shared_global_8\0"
  "__nvvm_cp_async_ca_shared_global_16\0"
  "__nvvm_cp_async_cg_shared_global_16\0"
  "__nvvm_cp_async_commit_group\0"
  "__nvvm_cp_async_mbarrier_arrive\0"
  "vWi*\0"
  "__nvvm_cp_async_mbarrier_arrive_noinc\0"
  "__nvvm_cp_async_mbarrier_arrive_noinc_shared\0"
  "vWi*3\0"
  "__nvvm_cp_async_mbarrier_arrive_shared\0"
  "__nvvm_cp_async_wait_all\0"
  "__nvvm_cp_async_wait_group\0"
  "vIi\0"
  "__nvvm_d2f_rm\0"
  "fd\0"
  "__nvvm_d2f_rm_ftz\0"
  "__nvvm_d2f_rn\0"
  "__nvvm_d2f_rn_ftz\0"
  "__nvvm_d2f_rp\0"
  "__nvvm_d2f_rp_ftz\0"
  "__nvvm_d2f_rz\0"
  "__nvvm_d2f_rz_ftz\0"
  "__nvvm_d2i_hi\0"
  "id\0"
  "__nvvm_d2i_lo\0"
  "__nvvm_d2i_rm\0"
  "__nvvm_d2i_rn\0"
  "__nvvm_d2i_rp\0"
  "__nvvm_d2i_rz\0"
  "__nvvm_d2ll_rm\0"
  "LLid\0"
  "__nvvm_d2ll_rn\0"
  "__nvvm_d2ll_rp\0"
  "__nvvm_d2ll_rz\0"
  "__nvvm_d2ui_rm\0"
  "Uid\0"
  "__nvvm_d2ui_rn\0"
  "__nvvm_d2ui_rp\0"
  "__nvvm_d2ui_rz\0"
  "__nvvm_d2ull_rm\0"
  "ULLid\0"
  "__nvvm_d2ull_rn\0"
  "__nvvm_d2ull_rp\0"
  "__nvvm_d2ull_rz\0"
  "__nvvm_div_approx_f\0"
  "__nvvm_div_approx_ftz_f\0"
  "__nvvm_div_rm_d\0"
  "__nvvm_div_rm_f\0"
  "__nvvm_div_rm_ftz_f\0"
  "__nvvm_div_rn_d\0"
  "__nvvm_div_rn_f\0"
  "__nvvm_div_rn_ftz_f\0"
  "__nvvm_div_rp_d\0"
  "__nvvm_div_rp_f\0"
  "__nvvm_div_rp_ftz_f\0"
  "__nvvm_div_rz_d\0"
  "__nvvm_div_rz_f\0"
  "__nvvm_div_rz_ftz_f\0"
  "__nvvm_e2m1x2_to_f16x2_rn\0"
  "V2hs\0"
  "__nvvm_e2m1x2_to_f16x2_rn_relu\0"
  "__nvvm_e2m3x2_to_f16x2_rn\0"
  "__nvvm_e2m3x2_to_f16x2_rn_relu\0"
  "__nvvm_e3m2x2_to_f16x2_rn\0"
  "__nvvm_e3m2x2_to_f16x2_rn_relu\0"
  "__nvvm_e4m3x2_to_f16x2_rn\0"
  "(sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_e4m3x2_to_f16x2_rn_relu\0"
  "__nvvm_e5m2x2_to_f16x2_rn\0"
  "__nvvm_e5m2x2_to_f16x2_rn_relu\0"
  "__nvvm_ex2_approx_d\0"
  "__nvvm_ex2_approx_f\0"
  "__nvvm_ex2_approx_f16\0"
  "hh\0"
  "(sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_ex2_approx_f16x2\0"
  "V2hV2h\0"
  "__nvvm_ex2_approx_ftz_f\0"
  "__nvvm_exit\0"
  "r\0"
  "__nvvm_f2bf16_rn\0"
  "yf\0"
  "__nvvm_f2bf16_rn_relu\0"
  "__nvvm_f2bf16_rn_relu_satfinite\0"
  "(sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_f2bf16_rn_satfinite\0"
  "__nvvm_f2bf16_rz\0"
  "__nvvm_f2bf16_rz_relu\0"
  "__nvvm_f2bf16_rz_relu_satfinite\0"
  "__nvvm_f2bf16_rz_satfinite\0"
  "__nvvm_f2f16_rn\0"
  "hf\0"
  "__nvvm_f2f16_rn_relu\0"
  "__nvvm_f2f16_rn_relu_satfinite\0"
  "__nvvm_f2f16_rn_satfinite\0"
  "__nvvm_f2f16_rz\0"
  "__nvvm_f2f16_rz_relu\0"
  "__nvvm_f2f16_rz_relu_satfinite\0"
  "__nvvm_f2f16_rz_satfinite\0"
  "__nvvm_f2h_rn\0"
  "Usf\0"
  "__nvvm_f2h_rn_ftz\0"
  "__nvvm_f2i_rm\0"
  "if\0"
  "__nvvm_f2i_rm_ftz\0"
  "__nvvm_f2i_rn\0"
  "__nvvm_f2i_rn_ftz\0"
  "__nvvm_f2i_rp\0"
  "__nvvm_f2i_rp_ftz\0"
  "__nvvm_f2i_rz\0"
  "__nvvm_f2i_rz_ftz\0"
  "__nvvm_f2ll_rm\0"
  "LLif\0"
  "__nvvm_f2ll_rm_ftz\0"
  "__nvvm_f2ll_rn\0"
  "__nvvm_f2ll_rn_ftz\0"
  "__nvvm_f2ll_rp\0"
  "__nvvm_f2ll_rp_ftz\0"
  "__nvvm_f2ll_rz\0"
  "__nvvm_f2ll_rz_ftz\0"
  "__nvvm_f2tf32_rn\0"
  "Zif\0"
  "__nvvm_f2tf32_rn_relu\0"
  "__nvvm_f2tf32_rn_relu_satfinite\0"
  "(sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_f2tf32_rn_satfinite\0"
  "__nvvm_f2tf32_rna\0"
  "__nvvm_f2tf32_rna_satfinite\0"
  "__nvvm_f2tf32_rz\0"
  "__nvvm_f2tf32_rz_relu\0"
  "__nvvm_f2tf32_rz_relu_satfinite\0"
  "__nvvm_f2tf32_rz_satfinite\0"
  "__nvvm_f2ui_rm\0"
  "Uif\0"
  "__nvvm_f2ui_rm_ftz\0"
  "__nvvm_f2ui_rn\0"
  "__nvvm_f2ui_rn_ftz\0"
  "__nvvm_f2ui_rp\0"
  "__nvvm_f2ui_rp_ftz\0"
  "__nvvm_f2ui_rz\0"
  "__nvvm_f2ui_rz_ftz\0"
  "__nvvm_f2ull_rm\0"
  "ULLif\0"
  "__nvvm_f2ull_rm_ftz\0"
  "__nvvm_f2ull_rn\0"
  "__nvvm_f2ull_rn_ftz\0"
  "__nvvm_f2ull_rp\0"
  "__nvvm_f2ull_rp_ftz\0"
  "__nvvm_f2ull_rz\0"
  "__nvvm_f2ull_rz_ftz\0"
  "__nvvm_f16x2_to_e4m3x2_rn\0"
  "sV2h\0"
  "__nvvm_f16x2_to_e4m3x2_rn_relu\0"
  "__nvvm_f16x2_to_e5m2x2_rn\0"
  "__nvvm_f16x2_to_e5m2x2_rn_relu\0"
  "__nvvm_f32x4_to_e2m1x4_rs_relu_satfinite\0"
  "sV4fUZi\0"
  "(sm_100a|sm_103a),(ptx87|ptx88|ptx90)\0"
  "__nvvm_f32x4_to_e2m1x4_rs_satfinite\0"
  "__nvvm_f32x4_to_e2m3x4_rs_relu_satfinite\0"
  "V4cV4fUZi\0"
  "__nvvm_f32x4_to_e2m3x4_rs_satfinite\0"
  "__nvvm_f32x4_to_e3m2x4_rs_relu_satfinite\0"
  "__nvvm_f32x4_to_e3m2x4_rs_satfinite\0"
  "__nvvm_f32x4_to_e4m3x4_rs_relu_satfinite\0"
  "__nvvm_f32x4_to_e4m3x4_rs_satfinite\0"
  "__nvvm_f32x4_to_e5m2x4_rs_relu_satfinite\0"
  "__nvvm_f32x4_to_e5m2x4_rs_satfinite\0"
  "__nvvm_fabs_d\0"
  "__nvvm_fabs_f\0"
  "__nvvm_fabs_f16\0"
  "(sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_fabs_f16x2\0"
  "__nvvm_fabs_ftz_f\0"
  "__nvvm_fabs_ftz_f16\0"
  "__nvvm_fabs_ftz_f16x2\0"
  "__nvvm_fence_sc_cluster\0"
  "__nvvm_ff2bf16x2_rn\0"
  "V2yff\0"
  "__nvvm_ff2bf16x2_rn_relu\0"
  "__nvvm_ff2bf16x2_rn_relu_satfinite\0"
  "__nvvm_ff2bf16x2_rn_satfinite\0"
  "__nvvm_ff2bf16x2_rs\0"
  "V2yffUZi\0"
  "__nvvm_ff2bf16x2_rs_relu\0"
  "__nvvm_ff2bf16x2_rs_relu_satfinite\0"
  "__nvvm_ff2bf16x2_rs_satfinite\0"
  "__nvvm_ff2bf16x2_rz\0"
  "__nvvm_ff2bf16x2_rz_relu\0"
  "__nvvm_ff2bf16x2_rz_relu_satfinite\0"
  "__nvvm_ff2bf16x2_rz_satfinite\0"
  "__nvvm_ff2f16x2_rn\0"
  "V2hff\0"
  "__nvvm_ff2f16x2_rn_relu\0"
  "__nvvm_ff2f16x2_rn_relu_satfinite\0"
  "__nvvm_ff2f16x2_rn_satfinite\0"
  "__nvvm_ff2f16x2_rs\0"
  "V2hffUZi\0"
  "__nvvm_ff2f16x2_rs_relu\0"
  "__nvvm_ff2f16x2_rs_relu_satfinite\0"
  "__nvvm_ff2f16x2_rs_satfinite\0"
  "__nvvm_ff2f16x2_rz\0"
  "__nvvm_ff2f16x2_rz_relu\0"
  "__nvvm_ff2f16x2_rz_relu_satfinite\0"
  "__nvvm_ff2f16x2_rz_satfinite\0"
  "__nvvm_ff_to_e2m1x2_rn_relu_satfinite\0"
  "sff\0"
  "__nvvm_ff_to_e2m1x2_rn_satfinite\0"
  "__nvvm_ff_to_e2m3x2_rn_relu_satfinite\0"
  "__nvvm_ff_to_e2m3x2_rn_satfinite\0"
  "__nvvm_ff_to_e3m2x2_rn_relu_satfinite\0"
  "__nvvm_ff_to_e3m2x2_rn_satfinite\0"
  "__nvvm_ff_to_e4m3x2_rn\0"
  "__nvvm_ff_to_e4m3x2_rn_relu\0"
  "__nvvm_ff_to_e5m2x2_rn\0"
  "__nvvm_ff_to_e5m2x2_rn_relu\0"
  "__nvvm_ff_to_ue8m0x2_rp\0"
  "__nvvm_ff_to_ue8m0x2_rp_satfinite\0"
  "__nvvm_ff_to_ue8m0x2_rz\0"
  "__nvvm_ff_to_ue8m0x2_rz_satfinite\0"
  "__nvvm_floor_d\0"
  "__nvvm_floor_f\0"
  "__nvvm_floor_ftz_f\0"
  "__nvvm_fma_rm_d\0"
  "dddd\0"
  "__nvvm_fma_rm_f\0"
  "ffff\0"
  "__nvvm_fma_rm_ftz_f\0"
  "__nvvm_fma_rm_ftz_sat_f\0"
  "__nvvm_fma_rm_sat_f\0"
  "__nvvm_fma_rn_bf16\0"
  "yyyy\0"
  "__nvvm_fma_rn_bf16x2\0"
  "V2yV2yV2yV2y\0"
  "__nvvm_fma_rn_d\0"
  "__nvvm_fma_rn_f\0"
  "__nvvm_fma_rn_f16\0"
  "hhhh\0"
  "(sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_fma_rn_f16x2\0"
  "V2hV2hV2hV2h\0"
  "__nvvm_fma_rn_ftz_f\0"
  "__nvvm_fma_rn_ftz_f16\0"
  "__nvvm_fma_rn_ftz_f16x2\0"
  "__nvvm_fma_rn_ftz_relu_f16\0"
  "__nvvm_fma_rn_ftz_relu_f16x2\0"
  "__nvvm_fma_rn_ftz_sat_f\0"
  "__nvvm_fma_rn_ftz_sat_f16\0"
  "__nvvm_fma_rn_ftz_sat_f16x2\0"
  "__nvvm_fma_rn_relu_bf16\0"
  "__nvvm_fma_rn_relu_bf16x2\0"
  "__nvvm_fma_rn_relu_f16\0"
  "__nvvm_fma_rn_relu_f16x2\0"
  "__nvvm_fma_rn_sat_f\0"
  "__nvvm_fma_rn_sat_f16\0"
  "__nvvm_fma_rn_sat_f16x2\0"
  "__nvvm_fma_rp_d\0"
  "__nvvm_fma_rp_f\0"
  "__nvvm_fma_rp_ftz_f\0"
  "__nvvm_fma_rp_ftz_sat_f\0"
  "__nvvm_fma_rp_sat_f\0"
  "__nvvm_fma_rz_d\0"
  "__nvvm_fma_rz_f\0"
  "__nvvm_fma_rz_ftz_f\0"
  "__nvvm_fma_rz_ftz_sat_f\0"
  "__nvvm_fma_rz_sat_f\0"
  "__nvvm_fmax_bf16\0"
  "yyy\0"
  "__nvvm_fmax_bf16x2\0"
  "V2yV2yV2y\0"
  "__nvvm_fmax_d\0"
  "__nvvm_fmax_f\0"
  "__nvvm_fmax_f16\0"
  "hhh\0"
  "__nvvm_fmax_f16x2\0"
  "V2hV2hV2h\0"
  "__nvvm_fmax_ftz_bf16\0"
  "__nvvm_fmax_ftz_bf16x2\0"
  "__nvvm_fmax_ftz_f\0"
  "__nvvm_fmax_ftz_f16\0"
  "__nvvm_fmax_ftz_f16x2\0"
  "__nvvm_fmax_ftz_nan_bf16\0"
  "__nvvm_fmax_ftz_nan_bf16x2\0"
  "__nvvm_fmax_ftz_nan_f\0"
  "__nvvm_fmax_ftz_nan_f16\0"
  "__nvvm_fmax_ftz_nan_f16x2\0"
  "__nvvm_fmax_ftz_nan_xorsign_abs_f\0"
  "(sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_fmax_ftz_nan_xorsign_abs_f16\0"
  "__nvvm_fmax_ftz_nan_xorsign_abs_f16x2\0"
  "__nvvm_fmax_ftz_xorsign_abs_f\0"
  "__nvvm_fmax_ftz_xorsign_abs_f16\0"
  "__nvvm_fmax_ftz_xorsign_abs_f16x2\0"
  "__nvvm_fmax_nan_bf16\0"
  "__nvvm_fmax_nan_bf16x2\0"
  "__nvvm_fmax_nan_f\0"
  "__nvvm_fmax_nan_f16\0"
  "__nvvm_fmax_nan_f16x2\0"
  "__nvvm_fmax_nan_xorsign_abs_bf16\0"
  "__nvvm_fmax_nan_xorsign_abs_bf16x2\0"
  "__nvvm_fmax_nan_xorsign_abs_f\0"
  "__nvvm_fmax_nan_xorsign_abs_f16\0"
  "__nvvm_fmax_nan_xorsign_abs_f16x2\0"
  "__nvvm_fmax_xorsign_abs_bf16\0"
  "__nvvm_fmax_xorsign_abs_bf16x2\0"
  "__nvvm_fmax_xorsign_abs_f\0"
  "__nvvm_fmax_xorsign_abs_f16\0"
  "__nvvm_fmax_xorsign_abs_f16x2\0"
  "__nvvm_fmin_bf16\0"
  "__nvvm_fmin_bf16x2\0"
  "__nvvm_fmin_d\0"
  "__nvvm_fmin_f\0"
  "__nvvm_fmin_f16\0"
  "__nvvm_fmin_f16x2\0"
  "__nvvm_fmin_ftz_bf16\0"
  "__nvvm_fmin_ftz_bf16x2\0"
  "__nvvm_fmin_ftz_f\0"
  "__nvvm_fmin_ftz_f16\0"
  "__nvvm_fmin_ftz_f16x2\0"
  "__nvvm_fmin_ftz_nan_bf16\0"
  "__nvvm_fmin_ftz_nan_bf16x2\0"
  "__nvvm_fmin_ftz_nan_f\0"
  "__nvvm_fmin_ftz_nan_f16\0"
  "__nvvm_fmin_ftz_nan_f16x2\0"
  "__nvvm_fmin_ftz_nan_xorsign_abs_f\0"
  "__nvvm_fmin_ftz_nan_xorsign_abs_f16\0"
  "__nvvm_fmin_ftz_nan_xorsign_abs_f16x2\0"
  "__nvvm_fmin_ftz_xorsign_abs_f\0"
  "__nvvm_fmin_ftz_xorsign_abs_f16\0"
  "__nvvm_fmin_ftz_xorsign_abs_f16x2\0"
  "__nvvm_fmin_nan_bf16\0"
  "__nvvm_fmin_nan_bf16x2\0"
  "__nvvm_fmin_nan_f\0"
  "__nvvm_fmin_nan_f16\0"
  "__nvvm_fmin_nan_f16x2\0"
  "__nvvm_fmin_nan_xorsign_abs_bf16\0"
  "__nvvm_fmin_nan_xorsign_abs_bf16x2\0"
  "__nvvm_fmin_nan_xorsign_abs_f\0"
  "__nvvm_fmin_nan_xorsign_abs_f16\0"
  "__nvvm_fmin_nan_xorsign_abs_f16x2\0"
  "__nvvm_fmin_xorsign_abs_bf16\0"
  "__nvvm_fmin_xorsign_abs_bf16x2\0"
  "__nvvm_fmin_xorsign_abs_f\0"
  "__nvvm_fmin_xorsign_abs_f16\0"
  "__nvvm_fmin_xorsign_abs_f16x2\0"
  "__nvvm_fns\0"
  "UiUiUii\0"
  "__nvvm_getctarank\0"
  "iv*\0"
  "__nvvm_getctarank_shared_cluster\0"
  "iv*3\0"
  "__nvvm_i2d_rm\0"
  "di\0"
  "__nvvm_i2d_rn\0"
  "__nvvm_i2d_rp\0"
  "__nvvm_i2d_rz\0"
  "__nvvm_i2f_rm\0"
  "fi\0"
  "__nvvm_i2f_rn\0"
  "__nvvm_i2f_rp\0"
  "__nvvm_i2f_rz\0"
  "__nvvm_is_explicit_cluster\0"
  "b\0"
  "nc\0"
  "__nvvm_isspacep_const\0"
  "bvC*\0"
  "__nvvm_isspacep_global\0"
  "__nvvm_isspacep_local\0"
  "__nvvm_isspacep_shared\0"
  "__nvvm_isspacep_shared_cluster\0"
  "__nvvm_ldg_c\0"
  "ccC*\0"
  "__nvvm_ldg_c2\0"
  "E2cE2cC*\0"
  "__nvvm_ldg_c4\0"
  "E4cE4cC*\0"
  "__nvvm_ldg_d\0"
  "ddC*\0"
  "__nvvm_ldg_d2\0"
  "E2dE2dC*\0"
  "__nvvm_ldg_f\0"
  "ffC*\0"
  "__nvvm_ldg_f2\0"
  "E2fE2fC*\0"
  "__nvvm_ldg_f4\0"
  "E4fE4fC*\0"
  "__nvvm_ldg_h\0"
  "hhC*\0"
  "__nvvm_ldg_h2\0"
  "E2hE2hC*\0"
  "__nvvm_ldg_i\0"
  "iiC*\0"
  "__nvvm_ldg_i2\0"
  "E2iE2iC*\0"
  "__nvvm_ldg_i4\0"
  "E4iE4iC*\0"
  "__nvvm_ldg_l\0"
  "LiLiC*\0"
  "__nvvm_ldg_l2\0"
  "E2LiE2LiC*\0"
  "__nvvm_ldg_ll\0"
  "LLiLLiC*\0"
  "__nvvm_ldg_ll2\0"
  "E2LLiE2LLiC*\0"
  "__nvvm_ldg_s\0"
  "ssC*\0"
  "__nvvm_ldg_s2\0"
  "E2sE2sC*\0"
  "__nvvm_ldg_s4\0"
  "E4sE4sC*\0"
  "__nvvm_ldg_sc\0"
  "ScScC*\0"
  "__nvvm_ldg_sc2\0"
  "E2ScE2ScC*\0"
  "__nvvm_ldg_sc4\0"
  "E4ScE4ScC*\0"
  "__nvvm_ldg_uc\0"
  "UcUcC*\0"
  "__nvvm_ldg_uc2\0"
  "E2UcE2UcC*\0"
  "__nvvm_ldg_uc4\0"
  "E4UcE4UcC*\0"
  "__nvvm_ldg_ui\0"
  "UiUiC*\0"
  "__nvvm_ldg_ui2\0"
  "E2UiE2UiC*\0"
  "__nvvm_ldg_ui4\0"
  "E4UiE4UiC*\0"
  "__nvvm_ldg_ul\0"
  "ULiULiC*\0"
  "__nvvm_ldg_ul2\0"
  "E2ULiE2ULiC*\0"
  "__nvvm_ldg_ull\0"
  "ULLiULLiC*\0"
  "__nvvm_ldg_ull2\0"
  "E2ULLiE2ULLiC*\0"
  "__nvvm_ldg_us\0"
  "UsUsC*\0"
  "__nvvm_ldg_us2\0"
  "E2UsE2UsC*\0"
  "__nvvm_ldg_us4\0"
  "E4UsE4UsC*\0"
  "__nvvm_ldu_c\0"
  "__nvvm_ldu_c2\0"
  "__nvvm_ldu_c4\0"
  "__nvvm_ldu_d\0"
  "__nvvm_ldu_d2\0"
  "__nvvm_ldu_f\0"
  "__nvvm_ldu_f2\0"
  "__nvvm_ldu_f4\0"
  "__nvvm_ldu_h\0"
  "__nvvm_ldu_h2\0"
  "__nvvm_ldu_i\0"
  "__nvvm_ldu_i2\0"
  "__nvvm_ldu_i4\0"
  "__nvvm_ldu_l\0"
  "__nvvm_ldu_l2\0"
  "__nvvm_ldu_ll\0"
  "__nvvm_ldu_ll2\0"
  "__nvvm_ldu_s\0"
  "__nvvm_ldu_s2\0"
  "__nvvm_ldu_s4\0"
  "__nvvm_ldu_sc\0"
  "__nvvm_ldu_sc2\0"
  "__nvvm_ldu_sc4\0"
  "__nvvm_ldu_uc\0"
  "__nvvm_ldu_uc2\0"
  "__nvvm_ldu_uc4\0"
  "__nvvm_ldu_ui\0"
  "__nvvm_ldu_ui2\0"
  "__nvvm_ldu_ui4\0"
  "__nvvm_ldu_ul\0"
  "__nvvm_ldu_ul2\0"
  "__nvvm_ldu_ull\0"
  "__nvvm_ldu_ull2\0"
  "__nvvm_ldu_us\0"
  "__nvvm_ldu_us2\0"
  "__nvvm_ldu_us4\0"
  "__nvvm_lg2_approx_d\0"
  "__nvvm_lg2_approx_f\0"
  "__nvvm_lg2_approx_ftz_f\0"
  "__nvvm_ll2d_rm\0"
  "dLLi\0"
  "__nvvm_ll2d_rn\0"
  "__nvvm_ll2d_rp\0"
  "__nvvm_ll2d_rz\0"
  "__nvvm_ll2f_rm\0"
  "fLLi\0"
  "__nvvm_ll2f_rn\0"
  "__nvvm_ll2f_rp\0"
  "__nvvm_ll2f_rz\0"
  "__nvvm_lohi_i2d\0"
  "dii\0"
  "__nvvm_mapa\0"
  "v*v*i\0"
  "__nvvm_mapa_shared_cluster\0"
  "v*3v*3i\0"
  "__nvvm_match_all_sync_i32p\0"
  "UiUiUii*\0"
  "__nvvm_match_all_sync_i64p\0"
  "UiUiWii*\0"
  "__nvvm_match_any_sync_i32\0"
  "UiUiUi\0"
  "__nvvm_match_any_sync_i64\0"
  "UiUiWi\0"
  "__nvvm_mbarrier_arrive\0"
  "WiWi*\0"
  "__nvvm_mbarrier_arrive_drop\0"
  "__nvvm_mbarrier_arrive_drop_noComplete\0"
  "WiWi*i\0"
  "__nvvm_mbarrier_arrive_drop_noComplete_shared\0"
  "WiWi*3i\0"
  "__nvvm_mbarrier_arrive_drop_shared\0"
  "WiWi*3\0"
  "__nvvm_mbarrier_arrive_noComplete\0"
  "__nvvm_mbarrier_arrive_noComplete_shared\0"
  "__nvvm_mbarrier_arrive_shared\0"
  "__nvvm_mbarrier_init\0"
  "vWi*i\0"
  "__nvvm_mbarrier_init_shared\0"
  "vWi*3i\0"
  "__nvvm_mbarrier_inval\0"
  "__nvvm_mbarrier_inval_shared\0"
  "__nvvm_mbarrier_pending_count\0"
  "iWi\0"
  "__nvvm_mbarrier_test_wait\0"
  "bWi*Wi\0"
  "__nvvm_mbarrier_test_wait_shared\0"
  "bWi*3Wi\0"
  "__nvvm_membar_cta\0"
  "__nvvm_membar_gl\0"
  "__nvvm_membar_sys\0"
  "__nvvm_memcpy\0"
  "vUc*Uc*zi\0"
  "__nvvm_memset\0"
  "vUc*Uczi\0"
  "__nvvm_mul24_i\0"
  "iii\0"
  "__nvvm_mul24_ui\0"
  "__nvvm_mul_rm_d\0"
  "__nvvm_mul_rm_f\0"
  "__nvvm_mul_rm_ftz_f\0"
  "__nvvm_mul_rn_d\0"
  "__nvvm_mul_rn_f\0"
  "__nvvm_mul_rn_ftz_f\0"
  "__nvvm_mul_rp_d\0"
  "__nvvm_mul_rp_f\0"
  "__nvvm_mul_rp_ftz_f\0"
  "__nvvm_mul_rz_d\0"
  "__nvvm_mul_rz_f\0"
  "__nvvm_mul_rz_ftz_f\0"
  "__nvvm_mulhi_i\0"
  "__nvvm_mulhi_ll\0"
  "LLiLLiLLi\0"
  "__nvvm_mulhi_ui\0"
  "__nvvm_mulhi_ull\0"
  "ULLiULLiULLi\0"
  "__nvvm_nanosleep\0"
  "(sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_neg_bf16\0"
  "__nvvm_neg_bf16x2\0"
  "__nvvm_pm_event_mask\0"
  "vIUs\0"
  "__nvvm_prmt\0"
  "UiUiUiUi\0"
  "__nvvm_rcp_approx_ftz_d\0"
  "__nvvm_rcp_approx_ftz_f\0"
  "__nvvm_rcp_rm_d\0"
  "__nvvm_rcp_rm_f\0"
  "__nvvm_rcp_rm_ftz_f\0"
  "__nvvm_rcp_rn_d\0"
  "__nvvm_rcp_rn_f\0"
  "__nvvm_rcp_rn_ftz_f\0"
  "__nvvm_rcp_rp_d\0"
  "__nvvm_rcp_rp_f\0"
  "__nvvm_rcp_rp_ftz_f\0"
  "__nvvm_rcp_rz_d\0"
  "__nvvm_rcp_rz_f\0"
  "__nvvm_rcp_rz_ftz_f\0"
  "__nvvm_read_ptx_sreg_clock\0"
  "i\0"
  "__nvvm_read_ptx_sreg_clock64\0"
  "LLi\0"
  "__nvvm_read_ptx_sreg_cluster_ctaid_w\0"
  "__nvvm_read_ptx_sreg_cluster_ctaid_x\0"
  "__nvvm_read_ptx_sreg_cluster_ctaid_y\0"
  "__nvvm_read_ptx_sreg_cluster_ctaid_z\0"
  "__nvvm_read_ptx_sreg_cluster_ctarank\0"
  "__nvvm_read_ptx_sreg_cluster_nctaid_w\0"
  "__nvvm_read_ptx_sreg_cluster_nctaid_x\0"
  "__nvvm_read_ptx_sreg_cluster_nctaid_y\0"
  "__nvvm_read_ptx_sreg_cluster_nctaid_z\0"
  "__nvvm_read_ptx_sreg_cluster_nctarank\0"
  "__nvvm_read_ptx_sreg_clusterid_w\0"
  "__nvvm_read_ptx_sreg_clusterid_x\0"
  "__nvvm_read_ptx_sreg_clusterid_y\0"
  "__nvvm_read_ptx_sreg_clusterid_z\0"
  "__nvvm_read_ptx_sreg_ctaid_w\0"
  "__nvvm_read_ptx_sreg_ctaid_x\0"
  "__nvvm_read_ptx_sreg_ctaid_y\0"
  "__nvvm_read_ptx_sreg_ctaid_z\0"
  "__nvvm_read_ptx_sreg_globaltimer\0"
  "__nvvm_read_ptx_sreg_gridid\0"
  "__nvvm_read_ptx_sreg_laneid\0"
  "__nvvm_read_ptx_sreg_lanemask_eq\0"
  "__nvvm_read_ptx_sreg_lanemask_ge\0"
  "__nvvm_read_ptx_sreg_lanemask_gt\0"
  "__nvvm_read_ptx_sreg_lanemask_le\0"
  "__nvvm_read_ptx_sreg_lanemask_lt\0"
  "__nvvm_read_ptx_sreg_nclusterid_w\0"
  "__nvvm_read_ptx_sreg_nclusterid_x\0"
  "__nvvm_read_ptx_sreg_nclusterid_y\0"
  "__nvvm_read_ptx_sreg_nclusterid_z\0"
  "__nvvm_read_ptx_sreg_nctaid_w\0"
  "__nvvm_read_ptx_sreg_nctaid_x\0"
  "__nvvm_read_ptx_sreg_nctaid_y\0"
  "__nvvm_read_ptx_sreg_nctaid_z\0"
  "__nvvm_read_ptx_sreg_nsmid\0"
  "__nvvm_read_ptx_sreg_ntid_w\0"
  "__nvvm_read_ptx_sreg_ntid_x\0"
  "__nvvm_read_ptx_sreg_ntid_y\0"
  "__nvvm_read_ptx_sreg_ntid_z\0"
  "__nvvm_read_ptx_sreg_nwarpid\0"
  "__nvvm_read_ptx_sreg_pm0\0"
  "__nvvm_read_ptx_sreg_pm1\0"
  "__nvvm_read_ptx_sreg_pm2\0"
  "__nvvm_read_ptx_sreg_pm3\0"
  "__nvvm_read_ptx_sreg_smid\0"
  "__nvvm_read_ptx_sreg_tid_w\0"
  "__nvvm_read_ptx_sreg_tid_x\0"
  "__nvvm_read_ptx_sreg_tid_y\0"
  "__nvvm_read_ptx_sreg_tid_z\0"
  "__nvvm_read_ptx_sreg_warpid\0"
  "__nvvm_read_ptx_sreg_warpsize\0"
  "__nvvm_redux_sync_add\0"
  "__nvvm_redux_sync_and\0"
  "__nvvm_redux_sync_fmax\0"
  "ffi\0"
  "(sm_100a),(ptx86|ptx87|ptx88|ptx90)\0"
  "__nvvm_redux_sync_fmax_NaN\0"
  "__nvvm_redux_sync_fmax_abs\0"
  "__nvvm_redux_sync_fmax_abs_NaN\0"
  "__nvvm_redux_sync_fmin\0"
  "__nvvm_redux_sync_fmin_NaN\0"
  "__nvvm_redux_sync_fmin_abs\0"
  "__nvvm_redux_sync_fmin_abs_NaN\0"
  "__nvvm_redux_sync_max\0"
  "__nvvm_redux_sync_min\0"
  "__nvvm_redux_sync_or\0"
  "__nvvm_redux_sync_umax\0"
  "UiUii\0"
  "__nvvm_redux_sync_umin\0"
  "__nvvm_redux_sync_xor\0"
  "__nvvm_reflect\0"
  "UicC*\0"
  "__nvvm_round_d\0"
  "__nvvm_round_f\0"
  "__nvvm_round_ftz_f\0"
  "__nvvm_rsqrt_approx_d\0"
  "__nvvm_rsqrt_approx_f\0"
  "__nvvm_rsqrt_approx_ftz_f\0"
  "__nvvm_sad_i\0"
  "iiii\0"
  "__nvvm_sad_ui\0"
  "__nvvm_saturate_d\0"
  "__nvvm_saturate_f\0"
  "__nvvm_saturate_ftz_f\0"
  "__nvvm_shfl_bfly_f32\0"
  "ffii\0"
  "__nvvm_shfl_bfly_i32\0"
  "__nvvm_shfl_down_f32\0"
  "__nvvm_shfl_down_i32\0"
  "__nvvm_shfl_idx_f32\0"
  "__nvvm_shfl_idx_i32\0"
  "__nvvm_shfl_sync_bfly_f32\0"
  "fUifii\0"
  "__nvvm_shfl_sync_bfly_i32\0"
  "iUiiii\0"
  "__nvvm_shfl_sync_down_f32\0"
  "__nvvm_shfl_sync_down_i32\0"
  "__nvvm_shfl_sync_idx_f32\0"
  "__nvvm_shfl_sync_idx_i32\0"
  "__nvvm_shfl_sync_up_f32\0"
  "__nvvm_shfl_sync_up_i32\0"
  "__nvvm_shfl_up_f32\0"
  "__nvvm_shfl_up_i32\0"
  "__nvvm_sin_approx_f\0"
  "__nvvm_sin_approx_ftz_f\0"
  "__nvvm_sqrt_approx_f\0"
  "__nvvm_sqrt_approx_ftz_f\0"
  "__nvvm_sqrt_rm_d\0"
  "__nvvm_sqrt_rm_f\0"
  "__nvvm_sqrt_rm_ftz_f\0"
  "__nvvm_sqrt_rn_d\0"
  "__nvvm_sqrt_rn_f\0"
  "__nvvm_sqrt_rn_ftz_f\0"
  "__nvvm_sqrt_rp_d\0"
  "__nvvm_sqrt_rp_f\0"
  "__nvvm_sqrt_rp_ftz_f\0"
  "__nvvm_sqrt_rz_d\0"
  "__nvvm_sqrt_rz_f\0"
  "__nvvm_sqrt_rz_ftz_f\0"
  "__nvvm_trunc_d\0"
  "__nvvm_trunc_f\0"
  "__nvvm_trunc_ftz_f\0"
  "__nvvm_ue8m0x2_to_bf16x2\0"
  "V2ys\0"
  "__nvvm_ui2d_rm\0"
  "dUi\0"
  "__nvvm_ui2d_rn\0"
  "__nvvm_ui2d_rp\0"
  "__nvvm_ui2d_rz\0"
  "__nvvm_ui2f_rm\0"
  "fUi\0"
  "__nvvm_ui2f_rn\0"
  "__nvvm_ui2f_rp\0"
  "__nvvm_ui2f_rz\0"
  "__nvvm_ull2d_rm\0"
  "dULLi\0"
  "__nvvm_ull2d_rn\0"
  "__nvvm_ull2d_rp\0"
  "__nvvm_ull2d_rz\0"
  "__nvvm_ull2f_rm\0"
  "fULLi\0"
  "__nvvm_ull2f_rn\0"
  "__nvvm_ull2f_rp\0"
  "__nvvm_ull2f_rz\0"
  "__nvvm_vote_all\0"
  "bb\0"
  "__nvvm_vote_all_sync\0"
  "bUib\0"
  "__nvvm_vote_any\0"
  "__nvvm_vote_any_sync\0"
  "__nvvm_vote_ballot\0"
  "Uib\0"
  "__nvvm_vote_ballot_sync\0"
  "UiUib\0"
  "__nvvm_vote_uni\0"
  "__nvvm_vote_uni_sync\0"
  "__syncthreads\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinStrings = BuiltinStringsStorage;

#endif // GET_BUILTIN_STR_TABLE

#ifdef GET_BUILTIN_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{1 /* __bmma_m8n8k128_ld_a_b1 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{311 /* __bmma_m8n8k128_ld_b_b1 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{335 /* __bmma_m8n8k128_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{356 /* __bmma_m8n8k128_mma_and_popc_b1 */, 388 /* vi*iC*iC*iC*Ii */, 0 /*  */, 403 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{648 /* __bmma_m8n8k128_mma_xor_popc_b1 */, 388 /* vi*iC*iC*iC*Ii */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{680 /* __bmma_m8n8k128_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{705 /* __builtin_ptx_get_image_channel_data_typei_ */, 749 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{752 /* __builtin_ptx_get_image_channel_orderi_ */, 749 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{792 /* __builtin_ptx_get_image_depthi_ */, 749 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{824 /* __builtin_ptx_get_image_heighti_ */, 749 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{857 /* __builtin_ptx_get_image_widthi_ */, 749 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{889 /* __builtin_ptx_read_image2Dff_ */, 919 /* V4fiiff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{927 /* __builtin_ptx_read_image2Dfi_ */, 957 /* V4fiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{965 /* __builtin_ptx_read_image2Dif_ */, 995 /* V4iiiff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1003 /* __builtin_ptx_read_image2Dii_ */, 1033 /* V4iiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1041 /* __builtin_ptx_read_image3Dff_ */, 1071 /* V4fiiffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1081 /* __builtin_ptx_read_image3Dfi_ */, 1111 /* V4fiiiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1121 /* __builtin_ptx_read_image3Dif_ */, 1151 /* V4iiiffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1161 /* __builtin_ptx_read_image3Dii_ */, 1191 /* V4iiiiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1201 /* __builtin_ptx_write_image2Df_ */, 1231 /* viiiffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1240 /* __builtin_ptx_write_image2Di_ */, 1270 /* viiiiiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1279 /* __builtin_ptx_write_image2Dui_ */, 1310 /* viiiUiUiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1323 /* __dmma_m8n8k4_ld_a */, 1342 /* vd*dC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1604 /* __dmma_m8n8k4_ld_b */, 1342 /* vd*dC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1623 /* __dmma_m8n8k4_ld_c */, 1342 /* vd*dC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1642 /* __dmma_m8n8k4_mma_f64 */, 1664 /* vd*dC*dC*dC*IiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1681 /* __dmma_m8n8k4_st_c_f64 */, 1342 /* vd*dC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1704 /* __hmma_m8n32k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2024 /* __hmma_m8n32k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2045 /* __hmma_m8n32k16_ld_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2070 /* __hmma_m8n32k16_ld_c_f32 */, 2095 /* vf*fC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2106 /* __hmma_m8n32k16_mma_f16f16 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2150 /* __hmma_m8n32k16_mma_f16f32 */, 2177 /* vi*iC*iC*fC*IiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2194 /* __hmma_m8n32k16_mma_f32f16 */, 2221 /* vf*iC*iC*iC*IiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2238 /* __hmma_m8n32k16_mma_f32f32 */, 2265 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2282 /* __hmma_m8n32k16_st_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2307 /* __hmma_m8n32k16_st_c_f32 */, 2095 /* vf*fC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2332 /* __hmma_m16n16k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2659 /* __hmma_m16n16k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2681 /* __hmma_m16n16k16_ld_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2707 /* __hmma_m16n16k16_ld_c_f32 */, 2095 /* vf*fC*UiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2733 /* __hmma_m16n16k16_mma_f16f16 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2761 /* __hmma_m16n16k16_mma_f16f32 */, 2177 /* vi*iC*iC*fC*IiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2789 /* __hmma_m16n16k16_mma_f32f16 */, 2221 /* vf*iC*iC*iC*IiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2817 /* __hmma_m16n16k16_mma_f32f32 */, 2265 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2845 /* __hmma_m16n16k16_st_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2871 /* __hmma_m16n16k16_st_c_f32 */, 2095 /* vf*fC*UiIi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2897 /* __hmma_m32n8k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2918 /* __hmma_m32n8k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2939 /* __hmma_m32n8k16_ld_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2964 /* __hmma_m32n8k16_ld_c_f32 */, 2095 /* vf*fC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2989 /* __hmma_m32n8k16_mma_f16f16 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3016 /* __hmma_m32n8k16_mma_f16f32 */, 2177 /* vi*iC*iC*fC*IiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3043 /* __hmma_m32n8k16_mma_f32f16 */, 2221 /* vf*iC*iC*iC*IiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3070 /* __hmma_m32n8k16_mma_f32f32 */, 2265 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3097 /* __hmma_m32n8k16_st_c_f16 */, 25 /* vi*iC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3122 /* __hmma_m32n8k16_st_c_f32 */, 2095 /* vf*fC*UiIi */, 0 /*  */, 1725 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3147 /* __imma_m8n8k32_ld_a_s4 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3170 /* __imma_m8n8k32_ld_a_u4 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3193 /* __imma_m8n8k32_ld_b_s4 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3216 /* __imma_m8n8k32_ld_b_u4 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3239 /* __imma_m8n8k32_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3259 /* __imma_m8n8k32_mma_s4 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3281 /* __imma_m8n8k32_mma_u4 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3303 /* __imma_m8n8k32_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 36 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3327 /* __imma_m8n32k16_ld_a_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3632 /* __imma_m8n32k16_ld_a_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3656 /* __imma_m8n32k16_ld_b_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3680 /* __imma_m8n32k16_ld_b_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3704 /* __imma_m8n32k16_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3725 /* __imma_m8n32k16_mma_s8 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3748 /* __imma_m8n32k16_mma_u8 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3771 /* __imma_m8n32k16_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3796 /* __imma_m16n16k16_ld_a_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3821 /* __imma_m16n16k16_ld_a_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3846 /* __imma_m16n16k16_ld_b_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3871 /* __imma_m16n16k16_ld_b_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3896 /* __imma_m16n16k16_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3918 /* __imma_m16n16k16_mma_s8 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3942 /* __imma_m16n16k16_mma_u8 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3966 /* __imma_m16n16k16_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3992 /* __imma_m32n8k16_ld_a_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4016 /* __imma_m32n8k16_ld_a_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4040 /* __imma_m32n8k16_ld_b_s8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4064 /* __imma_m32n8k16_ld_b_u8 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4088 /* __imma_m32n8k16_ld_c */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4109 /* __imma_m32n8k16_mma_s8 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4132 /* __imma_m32n8k16_mma_u8 */, 2133 /* vi*iC*iC*iC*IiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4155 /* __imma_m32n8k16_st_c_i32 */, 25 /* vi*iC*UiIi */, 0 /*  */, 3351 /* (sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4180 /* __mma_bf16_m8n32k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4205 /* __mma_bf16_m8n32k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4230 /* __mma_bf16_m8n32k16_mma_f32 */, 2265 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4258 /* __mma_bf16_m16n16k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4284 /* __mma_bf16_m16n16k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4310 /* __mma_bf16_m16n16k16_mma_f32 */, 2265 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4339 /* __mma_bf16_m32n8k16_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4364 /* __mma_bf16_m32n8k16_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4389 /* __mma_bf16_m32n8k16_mma_f32 */, 2265 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4417 /* __mma_m16n16k8_st_c_f32 */, 2095 /* vf*fC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4441 /* __mma_tf32_m16n16k8_ld_a */, 25 /* vi*iC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4466 /* __mma_tf32_m16n16k8_ld_b */, 25 /* vi*iC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4491 /* __mma_tf32_m16n16k8_ld_c */, 2095 /* vf*fC*UiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4516 /* __mma_tf32_m16n16k8_mma_f32 */, 2265 /* vf*iC*iC*fC*IiIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4544 /* __nvvm_abs_bf16 */, 4560 /* yy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4563 /* __nvvm_abs_bf16x2 */, 4581 /* V2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4588 /* __nvvm_activemask */, 4606 /* Ui */, 4609 /* n */, 4611 /* ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4749 /* __nvvm_add_rm_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4769 /* __nvvm_add_rm_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4789 /* __nvvm_add_rm_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4809 /* __nvvm_add_rm_ftz_sat_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4833 /* __nvvm_add_rm_sat_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4853 /* __nvvm_add_rn_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4869 /* __nvvm_add_rn_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4885 /* __nvvm_add_rn_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4905 /* __nvvm_add_rn_ftz_sat_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4929 /* __nvvm_add_rn_sat_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4949 /* __nvvm_add_rp_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4965 /* __nvvm_add_rp_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4981 /* __nvvm_add_rp_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5001 /* __nvvm_add_rp_ftz_sat_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5025 /* __nvvm_add_rp_sat_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5045 /* __nvvm_add_rz_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5061 /* __nvvm_add_rz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5077 /* __nvvm_add_rz_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5097 /* __nvvm_add_rz_ftz_sat_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5121 /* __nvvm_add_rz_sat_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5141 /* __nvvm_atom_add_gen_d */, 5163 /* ddD*d */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5338 /* __nvvm_atom_add_gen_f */, 5360 /* ffD*f */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5366 /* __nvvm_atom_add_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5394 /* __nvvm_atom_add_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5425 /* __nvvm_atom_add_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5460 /* __nvvm_atom_and_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5482 /* __nvvm_atom_and_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5504 /* __nvvm_atom_and_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5527 /* __nvvm_atom_cas_gen_i */, 5549 /* iiD*ii */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5556 /* __nvvm_atom_cas_gen_l */, 5578 /* LiLiD*LiLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5589 /* __nvvm_atom_cas_gen_ll */, 5612 /* LLiLLiD*LLiLLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5627 /* __nvvm_atom_cas_gen_us */, 5650 /* UsUsD*UsUs */, 4609 /* n */, 5661 /* sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5812 /* __nvvm_atom_cta_add_gen_d */, 5163 /* ddD*d */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5838 /* __nvvm_atom_cta_add_gen_f */, 5360 /* ffD*f */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5864 /* __nvvm_atom_cta_add_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5890 /* __nvvm_atom_cta_add_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5916 /* __nvvm_atom_cta_add_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5943 /* __nvvm_atom_cta_and_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5969 /* __nvvm_atom_cta_and_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5995 /* __nvvm_atom_cta_and_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6022 /* __nvvm_atom_cta_cas_gen_i */, 5549 /* iiD*ii */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6048 /* __nvvm_atom_cta_cas_gen_l */, 5578 /* LiLiD*LiLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6074 /* __nvvm_atom_cta_cas_gen_ll */, 5612 /* LLiLLiD*LLiLLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6101 /* __nvvm_atom_cta_cas_gen_us */, 5650 /* UsUsD*UsUs */, 4609 /* n */, 5661 /* sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6128 /* __nvvm_atom_cta_dec_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6164 /* __nvvm_atom_cta_inc_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6191 /* __nvvm_atom_cta_max_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6217 /* __nvvm_atom_cta_max_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6243 /* __nvvm_atom_cta_max_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6270 /* __nvvm_atom_cta_max_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6297 /* __nvvm_atom_cta_max_gen_ul */, 6324 /* ULiULiD*ULi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6336 /* __nvvm_atom_cta_max_gen_ull */, 6364 /* ULLiULLiD*ULLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6379 /* __nvvm_atom_cta_min_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6405 /* __nvvm_atom_cta_min_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6431 /* __nvvm_atom_cta_min_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6458 /* __nvvm_atom_cta_min_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6485 /* __nvvm_atom_cta_min_gen_ul */, 6324 /* ULiULiD*ULi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6512 /* __nvvm_atom_cta_min_gen_ull */, 6364 /* ULLiULLiD*ULLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6540 /* __nvvm_atom_cta_or_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6565 /* __nvvm_atom_cta_or_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6590 /* __nvvm_atom_cta_or_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6616 /* __nvvm_atom_cta_xchg_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6643 /* __nvvm_atom_cta_xchg_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6670 /* __nvvm_atom_cta_xchg_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6698 /* __nvvm_atom_cta_xor_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6724 /* __nvvm_atom_cta_xor_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6750 /* __nvvm_atom_cta_xor_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6777 /* __nvvm_atom_dec_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6800 /* __nvvm_atom_inc_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6823 /* __nvvm_atom_max_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6845 /* __nvvm_atom_max_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6867 /* __nvvm_atom_max_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6890 /* __nvvm_atom_max_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6913 /* __nvvm_atom_max_gen_ul */, 6324 /* ULiULiD*ULi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6936 /* __nvvm_atom_max_gen_ull */, 6364 /* ULLiULLiD*ULLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6960 /* __nvvm_atom_min_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6982 /* __nvvm_atom_min_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7004 /* __nvvm_atom_min_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7027 /* __nvvm_atom_min_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7050 /* __nvvm_atom_min_gen_ul */, 6324 /* ULiULiD*ULi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7073 /* __nvvm_atom_min_gen_ull */, 6364 /* ULLiULLiD*ULLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7097 /* __nvvm_atom_or_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7118 /* __nvvm_atom_or_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7139 /* __nvvm_atom_or_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7161 /* __nvvm_atom_sub_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7183 /* __nvvm_atom_sub_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7205 /* __nvvm_atom_sub_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7228 /* __nvvm_atom_sys_add_gen_d */, 5163 /* ddD*d */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7254 /* __nvvm_atom_sys_add_gen_f */, 5360 /* ffD*f */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7280 /* __nvvm_atom_sys_add_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7306 /* __nvvm_atom_sys_add_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7332 /* __nvvm_atom_sys_add_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7359 /* __nvvm_atom_sys_and_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7385 /* __nvvm_atom_sys_and_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7411 /* __nvvm_atom_sys_and_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7438 /* __nvvm_atom_sys_cas_gen_i */, 5549 /* iiD*ii */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7464 /* __nvvm_atom_sys_cas_gen_l */, 5578 /* LiLiD*LiLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7490 /* __nvvm_atom_sys_cas_gen_ll */, 5612 /* LLiLLiD*LLiLLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7517 /* __nvvm_atom_sys_cas_gen_us */, 5650 /* UsUsD*UsUs */, 4609 /* n */, 5661 /* sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7544 /* __nvvm_atom_sys_dec_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7571 /* __nvvm_atom_sys_inc_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7598 /* __nvvm_atom_sys_max_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7624 /* __nvvm_atom_sys_max_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7650 /* __nvvm_atom_sys_max_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7677 /* __nvvm_atom_sys_max_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7704 /* __nvvm_atom_sys_max_gen_ul */, 6324 /* ULiULiD*ULi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7731 /* __nvvm_atom_sys_max_gen_ull */, 6364 /* ULLiULLiD*ULLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7759 /* __nvvm_atom_sys_min_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7785 /* __nvvm_atom_sys_min_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7811 /* __nvvm_atom_sys_min_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7838 /* __nvvm_atom_sys_min_gen_ui */, 6155 /* UiUiD*Ui */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7865 /* __nvvm_atom_sys_min_gen_ul */, 6324 /* ULiULiD*ULi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7892 /* __nvvm_atom_sys_min_gen_ull */, 6364 /* ULLiULLiD*ULLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7920 /* __nvvm_atom_sys_or_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7945 /* __nvvm_atom_sys_or_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7970 /* __nvvm_atom_sys_or_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7996 /* __nvvm_atom_sys_xchg_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8023 /* __nvvm_atom_sys_xchg_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8050 /* __nvvm_atom_sys_xchg_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8078 /* __nvvm_atom_sys_xor_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8104 /* __nvvm_atom_sys_xor_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8130 /* __nvvm_atom_sys_xor_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 5169 /* sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8157 /* __nvvm_atom_xchg_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8180 /* __nvvm_atom_xchg_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8203 /* __nvvm_atom_xchg_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8227 /* __nvvm_atom_xor_gen_i */, 5388 /* iiD*i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8249 /* __nvvm_atom_xor_gen_l */, 5416 /* LiLiD*Li */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8271 /* __nvvm_atom_xor_gen_ll */, 5448 /* LLiLLiD*LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8294 /* __nvvm_bar0_and */, 749 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8310 /* __nvvm_bar0_or */, 749 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8325 /* __nvvm_bar0_popc */, 749 /* ii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8342 /* __nvvm_bar_sync */, 8358 /* vi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8361 /* __nvvm_bar_warp_sync */, 8382 /* vUi */, 4609 /* n */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8536 /* __nvvm_barrier_cluster_arrive */, 8566 /* v */, 4609 /* n */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8741 /* __nvvm_barrier_cluster_arrive_relaxed */, 8566 /* v */, 4609 /* n */, 8779 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8946 /* __nvvm_barrier_cluster_wait */, 8566 /* v */, 4609 /* n */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8974 /* __nvvm_barrier_sync */, 8382 /* vUi */, 4609 /* n */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8994 /* __nvvm_barrier_sync_cnt */, 9018 /* vUiUi */, 4609 /* n */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9024 /* __nvvm_bf16x2_to_ue8m0x2_rp */, 9052 /* sV2y */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9109 /* __nvvm_bf16x2_to_ue8m0x2_rp_satfinite */, 9052 /* sV2y */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9147 /* __nvvm_bf16x2_to_ue8m0x2_rz */, 9052 /* sV2y */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9175 /* __nvvm_bf16x2_to_ue8m0x2_rz_satfinite */, 9052 /* sV2y */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9213 /* __nvvm_ceil_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9230 /* __nvvm_ceil_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9247 /* __nvvm_ceil_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9265 /* __nvvm_compiler_error */, 9287 /* vcC*4 */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9293 /* __nvvm_compiler_warn */, 9287 /* vcC*4 */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9314 /* __nvvm_cos_approx_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9334 /* __nvvm_cos_approx_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9358 /* __nvvm_cp_async_ca_shared_global_4 */, 9393 /* vv*3vC*1. */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9403 /* __nvvm_cp_async_ca_shared_global_8 */, 9393 /* vv*3vC*1. */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9438 /* __nvvm_cp_async_ca_shared_global_16 */, 9393 /* vv*3vC*1. */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9474 /* __nvvm_cp_async_cg_shared_global_16 */, 9393 /* vv*3vC*1. */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9510 /* __nvvm_cp_async_commit_group */, 8566 /* v */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9539 /* __nvvm_cp_async_mbarrier_arrive */, 9571 /* vWi* */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9576 /* __nvvm_cp_async_mbarrier_arrive_noinc */, 9571 /* vWi* */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9614 /* __nvvm_cp_async_mbarrier_arrive_noinc_shared */, 9659 /* vWi*3 */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9665 /* __nvvm_cp_async_mbarrier_arrive_shared */, 9659 /* vWi*3 */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9704 /* __nvvm_cp_async_wait_all */, 8566 /* v */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9729 /* __nvvm_cp_async_wait_group */, 9756 /* vIi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9760 /* __nvvm_d2f_rm */, 9774 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9777 /* __nvvm_d2f_rm_ftz */, 9774 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9795 /* __nvvm_d2f_rn */, 9774 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9809 /* __nvvm_d2f_rn_ftz */, 9774 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9827 /* __nvvm_d2f_rp */, 9774 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9841 /* __nvvm_d2f_rp_ftz */, 9774 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9859 /* __nvvm_d2f_rz */, 9774 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9873 /* __nvvm_d2f_rz_ftz */, 9774 /* fd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9891 /* __nvvm_d2i_hi */, 9905 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9908 /* __nvvm_d2i_lo */, 9905 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9922 /* __nvvm_d2i_rm */, 9905 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9936 /* __nvvm_d2i_rn */, 9905 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9950 /* __nvvm_d2i_rp */, 9905 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9964 /* __nvvm_d2i_rz */, 9905 /* id */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9978 /* __nvvm_d2ll_rm */, 9993 /* LLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9998 /* __nvvm_d2ll_rn */, 9993 /* LLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10013 /* __nvvm_d2ll_rp */, 9993 /* LLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10028 /* __nvvm_d2ll_rz */, 9993 /* LLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10043 /* __nvvm_d2ui_rm */, 10058 /* Uid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10062 /* __nvvm_d2ui_rn */, 10058 /* Uid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10077 /* __nvvm_d2ui_rp */, 10058 /* Uid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10092 /* __nvvm_d2ui_rz */, 10058 /* Uid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10107 /* __nvvm_d2ull_rm */, 10123 /* ULLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10129 /* __nvvm_d2ull_rn */, 10123 /* ULLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10145 /* __nvvm_d2ull_rp */, 10123 /* ULLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10161 /* __nvvm_d2ull_rz */, 10123 /* ULLid */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10177 /* __nvvm_div_approx_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10197 /* __nvvm_div_approx_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10221 /* __nvvm_div_rm_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10237 /* __nvvm_div_rm_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10253 /* __nvvm_div_rm_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10273 /* __nvvm_div_rn_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10289 /* __nvvm_div_rn_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10305 /* __nvvm_div_rn_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10325 /* __nvvm_div_rp_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10341 /* __nvvm_div_rp_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10357 /* __nvvm_div_rp_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10377 /* __nvvm_div_rz_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10393 /* __nvvm_div_rz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10409 /* __nvvm_div_rz_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10429 /* __nvvm_e2m1x2_to_f16x2_rn */, 10455 /* V2hs */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10460 /* __nvvm_e2m1x2_to_f16x2_rn_relu */, 10455 /* V2hs */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10491 /* __nvvm_e2m3x2_to_f16x2_rn */, 10455 /* V2hs */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10517 /* __nvvm_e2m3x2_to_f16x2_rn_relu */, 10455 /* V2hs */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10548 /* __nvvm_e3m2x2_to_f16x2_rn */, 10455 /* V2hs */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10574 /* __nvvm_e3m2x2_to_f16x2_rn_relu */, 10455 /* V2hs */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10605 /* __nvvm_e4m3x2_to_f16x2_rn */, 10455 /* V2hs */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10798 /* __nvvm_e4m3x2_to_f16x2_rn_relu */, 10455 /* V2hs */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10829 /* __nvvm_e5m2x2_to_f16x2_rn */, 10455 /* V2hs */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10855 /* __nvvm_e5m2x2_to_f16x2_rn_relu */, 10455 /* V2hs */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10886 /* __nvvm_ex2_approx_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10906 /* __nvvm_ex2_approx_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10926 /* __nvvm_ex2_approx_f16 */, 10948 /* hh */, 0 /*  */, 10951 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11208 /* __nvvm_ex2_approx_f16x2 */, 11232 /* V2hV2h */, 0 /*  */, 10951 /* (sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11239 /* __nvvm_ex2_approx_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11263 /* __nvvm_exit */, 8566 /* v */, 11275 /* r */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11277 /* __nvvm_f2bf16_rn */, 11294 /* yf */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11297 /* __nvvm_f2bf16_rn_relu */, 11294 /* yf */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11319 /* __nvvm_f2bf16_rn_relu_satfinite */, 11294 /* yf */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11542 /* __nvvm_f2bf16_rn_satfinite */, 11294 /* yf */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11569 /* __nvvm_f2bf16_rz */, 11294 /* yf */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11586 /* __nvvm_f2bf16_rz_relu */, 11294 /* yf */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11608 /* __nvvm_f2bf16_rz_relu_satfinite */, 11294 /* yf */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11640 /* __nvvm_f2bf16_rz_satfinite */, 11294 /* yf */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11667 /* __nvvm_f2f16_rn */, 11683 /* hf */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11686 /* __nvvm_f2f16_rn_relu */, 11683 /* hf */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11707 /* __nvvm_f2f16_rn_relu_satfinite */, 11683 /* hf */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11738 /* __nvvm_f2f16_rn_satfinite */, 11683 /* hf */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11764 /* __nvvm_f2f16_rz */, 11683 /* hf */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11780 /* __nvvm_f2f16_rz_relu */, 11683 /* hf */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11801 /* __nvvm_f2f16_rz_relu_satfinite */, 11683 /* hf */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11832 /* __nvvm_f2f16_rz_satfinite */, 11683 /* hf */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11858 /* __nvvm_f2h_rn */, 11872 /* Usf */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11876 /* __nvvm_f2h_rn_ftz */, 11872 /* Usf */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11894 /* __nvvm_f2i_rm */, 11908 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11911 /* __nvvm_f2i_rm_ftz */, 11908 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11929 /* __nvvm_f2i_rn */, 11908 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11943 /* __nvvm_f2i_rn_ftz */, 11908 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11961 /* __nvvm_f2i_rp */, 11908 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11975 /* __nvvm_f2i_rp_ftz */, 11908 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11993 /* __nvvm_f2i_rz */, 11908 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12007 /* __nvvm_f2i_rz_ftz */, 11908 /* if */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12025 /* __nvvm_f2ll_rm */, 12040 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12045 /* __nvvm_f2ll_rm_ftz */, 12040 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12064 /* __nvvm_f2ll_rn */, 12040 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12079 /* __nvvm_f2ll_rn_ftz */, 12040 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12098 /* __nvvm_f2ll_rp */, 12040 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12113 /* __nvvm_f2ll_rp_ftz */, 12040 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12132 /* __nvvm_f2ll_rz */, 12040 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12147 /* __nvvm_f2ll_rz_ftz */, 12040 /* LLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12166 /* __nvvm_f2tf32_rn */, 12183 /* Zif */, 0 /*  */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12187 /* __nvvm_f2tf32_rn_relu */, 12183 /* Zif */, 0 /*  */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12209 /* __nvvm_f2tf32_rn_relu_satfinite */, 12183 /* Zif */, 0 /*  */, 12241 /* (sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12359 /* __nvvm_f2tf32_rn_satfinite */, 12183 /* Zif */, 0 /*  */, 12241 /* (sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12386 /* __nvvm_f2tf32_rna */, 12183 /* Zif */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12404 /* __nvvm_f2tf32_rna_satfinite */, 12183 /* Zif */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12432 /* __nvvm_f2tf32_rz */, 12183 /* Zif */, 0 /*  */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12449 /* __nvvm_f2tf32_rz_relu */, 12183 /* Zif */, 0 /*  */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12471 /* __nvvm_f2tf32_rz_relu_satfinite */, 12183 /* Zif */, 0 /*  */, 12241 /* (sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12503 /* __nvvm_f2tf32_rz_satfinite */, 12183 /* Zif */, 0 /*  */, 12241 /* (sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12530 /* __nvvm_f2ui_rm */, 12545 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12549 /* __nvvm_f2ui_rm_ftz */, 12545 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12568 /* __nvvm_f2ui_rn */, 12545 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12583 /* __nvvm_f2ui_rn_ftz */, 12545 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12602 /* __nvvm_f2ui_rp */, 12545 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12617 /* __nvvm_f2ui_rp_ftz */, 12545 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12636 /* __nvvm_f2ui_rz */, 12545 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12651 /* __nvvm_f2ui_rz_ftz */, 12545 /* Uif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12670 /* __nvvm_f2ull_rm */, 12686 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12692 /* __nvvm_f2ull_rm_ftz */, 12686 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12712 /* __nvvm_f2ull_rn */, 12686 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12728 /* __nvvm_f2ull_rn_ftz */, 12686 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12748 /* __nvvm_f2ull_rp */, 12686 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12764 /* __nvvm_f2ull_rp_ftz */, 12686 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12784 /* __nvvm_f2ull_rz */, 12686 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12800 /* __nvvm_f2ull_rz_ftz */, 12686 /* ULLif */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12820 /* __nvvm_f16x2_to_e4m3x2_rn */, 12846 /* sV2h */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12851 /* __nvvm_f16x2_to_e4m3x2_rn_relu */, 12846 /* sV2h */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12882 /* __nvvm_f16x2_to_e5m2x2_rn */, 12846 /* sV2h */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12908 /* __nvvm_f16x2_to_e5m2x2_rn_relu */, 12846 /* sV2h */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12939 /* __nvvm_f32x4_to_e2m1x4_rs_relu_satfinite */, 12980 /* sV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13026 /* __nvvm_f32x4_to_e2m1x4_rs_satfinite */, 12980 /* sV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13062 /* __nvvm_f32x4_to_e2m3x4_rs_relu_satfinite */, 13103 /* V4cV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13113 /* __nvvm_f32x4_to_e2m3x4_rs_satfinite */, 13103 /* V4cV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13149 /* __nvvm_f32x4_to_e3m2x4_rs_relu_satfinite */, 13103 /* V4cV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13190 /* __nvvm_f32x4_to_e3m2x4_rs_satfinite */, 13103 /* V4cV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13226 /* __nvvm_f32x4_to_e4m3x4_rs_relu_satfinite */, 13103 /* V4cV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13267 /* __nvvm_f32x4_to_e4m3x4_rs_satfinite */, 13103 /* V4cV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13303 /* __nvvm_f32x4_to_e5m2x4_rs_relu_satfinite */, 13103 /* V4cV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13344 /* __nvvm_f32x4_to_e5m2x4_rs_satfinite */, 13103 /* V4cV4fUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13380 /* __nvvm_fabs_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13394 /* __nvvm_fabs_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13408 /* __nvvm_fabs_f16 */, 10948 /* hh */, 0 /*  */, 13424 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13723 /* __nvvm_fabs_f16x2 */, 11232 /* V2hV2h */, 0 /*  */, 13424 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13741 /* __nvvm_fabs_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13759 /* __nvvm_fabs_ftz_f16 */, 10948 /* hh */, 0 /*  */, 13424 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13779 /* __nvvm_fabs_ftz_f16x2 */, 11232 /* V2hV2h */, 0 /*  */, 13424 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13801 /* __nvvm_fence_sc_cluster */, 8566 /* v */, 4609 /* n */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13825 /* __nvvm_ff2bf16x2_rn */, 13845 /* V2yff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13851 /* __nvvm_ff2bf16x2_rn_relu */, 13845 /* V2yff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13876 /* __nvvm_ff2bf16x2_rn_relu_satfinite */, 13845 /* V2yff */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13911 /* __nvvm_ff2bf16x2_rn_satfinite */, 13845 /* V2yff */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13941 /* __nvvm_ff2bf16x2_rs */, 13961 /* V2yffUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13970 /* __nvvm_ff2bf16x2_rs_relu */, 13961 /* V2yffUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{13995 /* __nvvm_ff2bf16x2_rs_relu_satfinite */, 13961 /* V2yffUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14030 /* __nvvm_ff2bf16x2_rs_satfinite */, 13961 /* V2yffUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14060 /* __nvvm_ff2bf16x2_rz */, 13845 /* V2yff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14080 /* __nvvm_ff2bf16x2_rz_relu */, 13845 /* V2yff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14105 /* __nvvm_ff2bf16x2_rz_relu_satfinite */, 13845 /* V2yff */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14140 /* __nvvm_ff2bf16x2_rz_satfinite */, 13845 /* V2yff */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14170 /* __nvvm_ff2f16x2_rn */, 14189 /* V2hff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14195 /* __nvvm_ff2f16x2_rn_relu */, 14189 /* V2hff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14219 /* __nvvm_ff2f16x2_rn_relu_satfinite */, 14189 /* V2hff */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14253 /* __nvvm_ff2f16x2_rn_satfinite */, 14189 /* V2hff */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14282 /* __nvvm_ff2f16x2_rs */, 14301 /* V2hffUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14310 /* __nvvm_ff2f16x2_rs_relu */, 14301 /* V2hffUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14334 /* __nvvm_ff2f16x2_rs_relu_satfinite */, 14301 /* V2hffUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14368 /* __nvvm_ff2f16x2_rs_satfinite */, 14301 /* V2hffUZi */, 0 /*  */, 12988 /* (sm_100a|sm_103a),(ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14397 /* __nvvm_ff2f16x2_rz */, 14189 /* V2hff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14416 /* __nvvm_ff2f16x2_rz_relu */, 14189 /* V2hff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14440 /* __nvvm_ff2f16x2_rz_relu_satfinite */, 14189 /* V2hff */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14474 /* __nvvm_ff2f16x2_rz_satfinite */, 14189 /* V2hff */, 0 /*  */, 11351 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14503 /* __nvvm_ff_to_e2m1x2_rn_relu_satfinite */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14545 /* __nvvm_ff_to_e2m1x2_rn_satfinite */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14578 /* __nvvm_ff_to_e2m3x2_rn_relu_satfinite */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14616 /* __nvvm_ff_to_e2m3x2_rn_satfinite */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14649 /* __nvvm_ff_to_e3m2x2_rn_relu_satfinite */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14687 /* __nvvm_ff_to_e3m2x2_rn_satfinite */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14720 /* __nvvm_ff_to_e4m3x2_rn */, 14541 /* sff */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14743 /* __nvvm_ff_to_e4m3x2_rn_relu */, 14541 /* sff */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14771 /* __nvvm_ff_to_e5m2x2_rn */, 14541 /* sff */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14794 /* __nvvm_ff_to_e5m2x2_rn_relu */, 14541 /* sff */, 0 /*  */, 10631 /* (sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14822 /* __nvvm_ff_to_ue8m0x2_rp */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14846 /* __nvvm_ff_to_ue8m0x2_rp_satfinite */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14880 /* __nvvm_ff_to_ue8m0x2_rz */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14904 /* __nvvm_ff_to_ue8m0x2_rz_satfinite */, 14541 /* sff */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14938 /* __nvvm_floor_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14953 /* __nvvm_floor_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14968 /* __nvvm_floor_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{14987 /* __nvvm_fma_rm_d */, 15003 /* dddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15008 /* __nvvm_fma_rm_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15029 /* __nvvm_fma_rm_ftz_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15049 /* __nvvm_fma_rm_ftz_sat_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15073 /* __nvvm_fma_rm_sat_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15093 /* __nvvm_fma_rn_bf16 */, 15112 /* yyyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15117 /* __nvvm_fma_rn_bf16x2 */, 15138 /* V2yV2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15151 /* __nvvm_fma_rn_d */, 15003 /* dddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15167 /* __nvvm_fma_rn_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15183 /* __nvvm_fma_rn_f16 */, 15201 /* hhhh */, 0 /*  */, 15206 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15541 /* __nvvm_fma_rn_f16x2 */, 15561 /* V2hV2hV2hV2h */, 0 /*  */, 15206 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15574 /* __nvvm_fma_rn_ftz_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15594 /* __nvvm_fma_rn_ftz_f16 */, 15201 /* hhhh */, 0 /*  */, 15206 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15616 /* __nvvm_fma_rn_ftz_f16x2 */, 15561 /* V2hV2hV2hV2h */, 0 /*  */, 15206 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15640 /* __nvvm_fma_rn_ftz_relu_f16 */, 15201 /* hhhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15667 /* __nvvm_fma_rn_ftz_relu_f16x2 */, 15561 /* V2hV2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15696 /* __nvvm_fma_rn_ftz_sat_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15720 /* __nvvm_fma_rn_ftz_sat_f16 */, 15201 /* hhhh */, 0 /*  */, 15206 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15746 /* __nvvm_fma_rn_ftz_sat_f16x2 */, 15561 /* V2hV2hV2hV2h */, 0 /*  */, 15206 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15774 /* __nvvm_fma_rn_relu_bf16 */, 15112 /* yyyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15798 /* __nvvm_fma_rn_relu_bf16x2 */, 15138 /* V2yV2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15824 /* __nvvm_fma_rn_relu_f16 */, 15201 /* hhhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15847 /* __nvvm_fma_rn_relu_f16x2 */, 15561 /* V2hV2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15872 /* __nvvm_fma_rn_sat_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15892 /* __nvvm_fma_rn_sat_f16 */, 15201 /* hhhh */, 0 /*  */, 15206 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15914 /* __nvvm_fma_rn_sat_f16x2 */, 15561 /* V2hV2hV2hV2h */, 0 /*  */, 15206 /* (sm_53|sm_60|sm_61|sm_62|sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx42|ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15938 /* __nvvm_fma_rp_d */, 15003 /* dddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15954 /* __nvvm_fma_rp_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15970 /* __nvvm_fma_rp_ftz_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15990 /* __nvvm_fma_rp_ftz_sat_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16014 /* __nvvm_fma_rp_sat_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16034 /* __nvvm_fma_rz_d */, 15003 /* dddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16050 /* __nvvm_fma_rz_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16066 /* __nvvm_fma_rz_ftz_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16086 /* __nvvm_fma_rz_ftz_sat_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16110 /* __nvvm_fma_rz_sat_f */, 15024 /* ffff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16130 /* __nvvm_fmax_bf16 */, 16147 /* yyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16151 /* __nvvm_fmax_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16180 /* __nvvm_fmax_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16194 /* __nvvm_fmax_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16208 /* __nvvm_fmax_f16 */, 16224 /* hhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16228 /* __nvvm_fmax_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16256 /* __nvvm_fmax_ftz_bf16 */, 16147 /* yyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16277 /* __nvvm_fmax_ftz_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16300 /* __nvvm_fmax_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16318 /* __nvvm_fmax_ftz_f16 */, 16224 /* hhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16338 /* __nvvm_fmax_ftz_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16360 /* __nvvm_fmax_ftz_nan_bf16 */, 16147 /* yyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16385 /* __nvvm_fmax_ftz_nan_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16412 /* __nvvm_fmax_ftz_nan_f */, 4785 /* fff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16434 /* __nvvm_fmax_ftz_nan_f16 */, 16224 /* hhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16458 /* __nvvm_fmax_ftz_nan_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16484 /* __nvvm_fmax_ftz_nan_xorsign_abs_f */, 4785 /* fff */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16751 /* __nvvm_fmax_ftz_nan_xorsign_abs_f16 */, 16224 /* hhh */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16787 /* __nvvm_fmax_ftz_nan_xorsign_abs_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16825 /* __nvvm_fmax_ftz_xorsign_abs_f */, 4785 /* fff */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16855 /* __nvvm_fmax_ftz_xorsign_abs_f16 */, 16224 /* hhh */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16887 /* __nvvm_fmax_ftz_xorsign_abs_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16921 /* __nvvm_fmax_nan_bf16 */, 16147 /* yyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16942 /* __nvvm_fmax_nan_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16965 /* __nvvm_fmax_nan_f */, 4785 /* fff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16983 /* __nvvm_fmax_nan_f16 */, 16224 /* hhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17003 /* __nvvm_fmax_nan_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17025 /* __nvvm_fmax_nan_xorsign_abs_bf16 */, 16147 /* yyy */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17058 /* __nvvm_fmax_nan_xorsign_abs_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17093 /* __nvvm_fmax_nan_xorsign_abs_f */, 4785 /* fff */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17123 /* __nvvm_fmax_nan_xorsign_abs_f16 */, 16224 /* hhh */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17155 /* __nvvm_fmax_nan_xorsign_abs_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17189 /* __nvvm_fmax_xorsign_abs_bf16 */, 16147 /* yyy */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17218 /* __nvvm_fmax_xorsign_abs_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17249 /* __nvvm_fmax_xorsign_abs_f */, 4785 /* fff */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17275 /* __nvvm_fmax_xorsign_abs_f16 */, 16224 /* hhh */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17303 /* __nvvm_fmax_xorsign_abs_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17333 /* __nvvm_fmin_bf16 */, 16147 /* yyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17350 /* __nvvm_fmin_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17369 /* __nvvm_fmin_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17383 /* __nvvm_fmin_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17397 /* __nvvm_fmin_f16 */, 16224 /* hhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17413 /* __nvvm_fmin_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17431 /* __nvvm_fmin_ftz_bf16 */, 16147 /* yyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17452 /* __nvvm_fmin_ftz_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17475 /* __nvvm_fmin_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17493 /* __nvvm_fmin_ftz_f16 */, 16224 /* hhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17513 /* __nvvm_fmin_ftz_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17535 /* __nvvm_fmin_ftz_nan_bf16 */, 16147 /* yyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17560 /* __nvvm_fmin_ftz_nan_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17587 /* __nvvm_fmin_ftz_nan_f */, 4785 /* fff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17609 /* __nvvm_fmin_ftz_nan_f16 */, 16224 /* hhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17633 /* __nvvm_fmin_ftz_nan_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17659 /* __nvvm_fmin_ftz_nan_xorsign_abs_f */, 4785 /* fff */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17693 /* __nvvm_fmin_ftz_nan_xorsign_abs_f16 */, 16224 /* hhh */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17729 /* __nvvm_fmin_ftz_nan_xorsign_abs_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17767 /* __nvvm_fmin_ftz_xorsign_abs_f */, 4785 /* fff */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17797 /* __nvvm_fmin_ftz_xorsign_abs_f16 */, 16224 /* hhh */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17829 /* __nvvm_fmin_ftz_xorsign_abs_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17863 /* __nvvm_fmin_nan_bf16 */, 16147 /* yyy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17884 /* __nvvm_fmin_nan_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17907 /* __nvvm_fmin_nan_f */, 4785 /* fff */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17925 /* __nvvm_fmin_nan_f16 */, 16224 /* hhh */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17945 /* __nvvm_fmin_nan_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17967 /* __nvvm_fmin_nan_xorsign_abs_bf16 */, 16147 /* yyy */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18000 /* __nvvm_fmin_nan_xorsign_abs_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18035 /* __nvvm_fmin_nan_xorsign_abs_f */, 4785 /* fff */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18065 /* __nvvm_fmin_nan_xorsign_abs_f16 */, 16224 /* hhh */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18097 /* __nvvm_fmin_nan_xorsign_abs_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18131 /* __nvvm_fmin_xorsign_abs_bf16 */, 16147 /* yyy */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18160 /* __nvvm_fmin_xorsign_abs_bf16x2 */, 16170 /* V2yV2yV2y */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18191 /* __nvvm_fmin_xorsign_abs_f */, 4785 /* fff */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18217 /* __nvvm_fmin_xorsign_abs_f16 */, 16224 /* hhh */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18245 /* __nvvm_fmin_xorsign_abs_f16x2 */, 16246 /* V2hV2hV2h */, 0 /*  */, 16518 /* (sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18275 /* __nvvm_fns */, 18286 /* UiUiUii */, 4609 /* n */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18294 /* __nvvm_getctarank */, 18312 /* iv* */, 0 /*  */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18316 /* __nvvm_getctarank_shared_cluster */, 18349 /* iv*3 */, 0 /*  */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18354 /* __nvvm_i2d_rm */, 18368 /* di */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18371 /* __nvvm_i2d_rn */, 18368 /* di */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18385 /* __nvvm_i2d_rp */, 18368 /* di */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18399 /* __nvvm_i2d_rz */, 18368 /* di */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18413 /* __nvvm_i2f_rm */, 18427 /* fi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18430 /* __nvvm_i2f_rn */, 18427 /* fi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18444 /* __nvvm_i2f_rp */, 18427 /* fi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18458 /* __nvvm_i2f_rz */, 18427 /* fi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18472 /* __nvvm_is_explicit_cluster */, 18499 /* b */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18504 /* __nvvm_isspacep_const */, 18526 /* bvC* */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18531 /* __nvvm_isspacep_global */, 18526 /* bvC* */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18554 /* __nvvm_isspacep_local */, 18526 /* bvC* */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18576 /* __nvvm_isspacep_shared */, 18526 /* bvC* */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18599 /* __nvvm_isspacep_shared_cluster */, 18526 /* bvC* */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18630 /* __nvvm_ldg_c */, 18643 /* ccC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18648 /* __nvvm_ldg_c2 */, 18662 /* E2cE2cC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18671 /* __nvvm_ldg_c4 */, 18685 /* E4cE4cC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18694 /* __nvvm_ldg_d */, 18707 /* ddC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18712 /* __nvvm_ldg_d2 */, 18726 /* E2dE2dC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18735 /* __nvvm_ldg_f */, 18748 /* ffC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18753 /* __nvvm_ldg_f2 */, 18767 /* E2fE2fC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18776 /* __nvvm_ldg_f4 */, 18790 /* E4fE4fC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18799 /* __nvvm_ldg_h */, 18812 /* hhC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18817 /* __nvvm_ldg_h2 */, 18831 /* E2hE2hC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18840 /* __nvvm_ldg_i */, 18853 /* iiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18858 /* __nvvm_ldg_i2 */, 18872 /* E2iE2iC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18881 /* __nvvm_ldg_i4 */, 18895 /* E4iE4iC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18904 /* __nvvm_ldg_l */, 18917 /* LiLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18924 /* __nvvm_ldg_l2 */, 18938 /* E2LiE2LiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18949 /* __nvvm_ldg_ll */, 18963 /* LLiLLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18972 /* __nvvm_ldg_ll2 */, 18987 /* E2LLiE2LLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19000 /* __nvvm_ldg_s */, 19013 /* ssC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19018 /* __nvvm_ldg_s2 */, 19032 /* E2sE2sC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19041 /* __nvvm_ldg_s4 */, 19055 /* E4sE4sC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19064 /* __nvvm_ldg_sc */, 19078 /* ScScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19085 /* __nvvm_ldg_sc2 */, 19100 /* E2ScE2ScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19111 /* __nvvm_ldg_sc4 */, 19126 /* E4ScE4ScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19137 /* __nvvm_ldg_uc */, 19151 /* UcUcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19158 /* __nvvm_ldg_uc2 */, 19173 /* E2UcE2UcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19184 /* __nvvm_ldg_uc4 */, 19199 /* E4UcE4UcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19210 /* __nvvm_ldg_ui */, 19224 /* UiUiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19231 /* __nvvm_ldg_ui2 */, 19246 /* E2UiE2UiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19257 /* __nvvm_ldg_ui4 */, 19272 /* E4UiE4UiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19283 /* __nvvm_ldg_ul */, 19297 /* ULiULiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19306 /* __nvvm_ldg_ul2 */, 19321 /* E2ULiE2ULiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19334 /* __nvvm_ldg_ull */, 19349 /* ULLiULLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19360 /* __nvvm_ldg_ull2 */, 19376 /* E2ULLiE2ULLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19391 /* __nvvm_ldg_us */, 19405 /* UsUsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19412 /* __nvvm_ldg_us2 */, 19427 /* E2UsE2UsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19438 /* __nvvm_ldg_us4 */, 19453 /* E4UsE4UsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19464 /* __nvvm_ldu_c */, 18643 /* ccC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19477 /* __nvvm_ldu_c2 */, 18662 /* E2cE2cC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19491 /* __nvvm_ldu_c4 */, 18685 /* E4cE4cC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19505 /* __nvvm_ldu_d */, 18707 /* ddC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19518 /* __nvvm_ldu_d2 */, 18726 /* E2dE2dC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19532 /* __nvvm_ldu_f */, 18748 /* ffC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19545 /* __nvvm_ldu_f2 */, 18767 /* E2fE2fC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19559 /* __nvvm_ldu_f4 */, 18790 /* E4fE4fC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19573 /* __nvvm_ldu_h */, 18812 /* hhC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19586 /* __nvvm_ldu_h2 */, 18831 /* E2hE2hC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19600 /* __nvvm_ldu_i */, 18853 /* iiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19613 /* __nvvm_ldu_i2 */, 18872 /* E2iE2iC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19627 /* __nvvm_ldu_i4 */, 18895 /* E4iE4iC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19641 /* __nvvm_ldu_l */, 18917 /* LiLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19654 /* __nvvm_ldu_l2 */, 18938 /* E2LiE2LiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19668 /* __nvvm_ldu_ll */, 18963 /* LLiLLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19682 /* __nvvm_ldu_ll2 */, 18987 /* E2LLiE2LLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19697 /* __nvvm_ldu_s */, 19013 /* ssC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19710 /* __nvvm_ldu_s2 */, 19032 /* E2sE2sC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19724 /* __nvvm_ldu_s4 */, 19055 /* E4sE4sC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19738 /* __nvvm_ldu_sc */, 19078 /* ScScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19752 /* __nvvm_ldu_sc2 */, 19100 /* E2ScE2ScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19767 /* __nvvm_ldu_sc4 */, 19126 /* E4ScE4ScC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19782 /* __nvvm_ldu_uc */, 19151 /* UcUcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19796 /* __nvvm_ldu_uc2 */, 19173 /* E2UcE2UcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19811 /* __nvvm_ldu_uc4 */, 19199 /* E4UcE4UcC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19826 /* __nvvm_ldu_ui */, 19224 /* UiUiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19840 /* __nvvm_ldu_ui2 */, 19246 /* E2UiE2UiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19855 /* __nvvm_ldu_ui4 */, 19272 /* E4UiE4UiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19870 /* __nvvm_ldu_ul */, 19297 /* ULiULiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19884 /* __nvvm_ldu_ul2 */, 19321 /* E2ULiE2ULiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19899 /* __nvvm_ldu_ull */, 19349 /* ULLiULLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19914 /* __nvvm_ldu_ull2 */, 19376 /* E2ULLiE2ULLiC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19930 /* __nvvm_ldu_us */, 19405 /* UsUsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19944 /* __nvvm_ldu_us2 */, 19427 /* E2UsE2UsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19959 /* __nvvm_ldu_us4 */, 19453 /* E4UsE4UsC* */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19974 /* __nvvm_lg2_approx_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19994 /* __nvvm_lg2_approx_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20014 /* __nvvm_lg2_approx_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20038 /* __nvvm_ll2d_rm */, 20053 /* dLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20058 /* __nvvm_ll2d_rn */, 20053 /* dLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20073 /* __nvvm_ll2d_rp */, 20053 /* dLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20088 /* __nvvm_ll2d_rz */, 20053 /* dLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20103 /* __nvvm_ll2f_rm */, 20118 /* fLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20123 /* __nvvm_ll2f_rn */, 20118 /* fLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20138 /* __nvvm_ll2f_rp */, 20118 /* fLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20153 /* __nvvm_ll2f_rz */, 20118 /* fLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20168 /* __nvvm_lohi_i2d */, 20184 /* dii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20188 /* __nvvm_mapa */, 20200 /* v*v*i */, 0 /*  */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20206 /* __nvvm_mapa_shared_cluster */, 20233 /* v*3v*3i */, 0 /*  */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20241 /* __nvvm_match_all_sync_i32p */, 20268 /* UiUiUii* */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20277 /* __nvvm_match_all_sync_i64p */, 20304 /* UiUiWii* */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20313 /* __nvvm_match_any_sync_i32 */, 20339 /* UiUiUi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20346 /* __nvvm_match_any_sync_i64 */, 20372 /* UiUiWi */, 0 /*  */, 2354 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20379 /* __nvvm_mbarrier_arrive */, 20402 /* WiWi* */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20408 /* __nvvm_mbarrier_arrive_drop */, 20402 /* WiWi* */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20436 /* __nvvm_mbarrier_arrive_drop_noComplete */, 20475 /* WiWi*i */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20482 /* __nvvm_mbarrier_arrive_drop_noComplete_shared */, 20528 /* WiWi*3i */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20536 /* __nvvm_mbarrier_arrive_drop_shared */, 20571 /* WiWi*3 */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20578 /* __nvvm_mbarrier_arrive_noComplete */, 20475 /* WiWi*i */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20612 /* __nvvm_mbarrier_arrive_noComplete_shared */, 20528 /* WiWi*3i */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20653 /* __nvvm_mbarrier_arrive_shared */, 20571 /* WiWi*3 */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20683 /* __nvvm_mbarrier_init */, 20704 /* vWi*i */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20710 /* __nvvm_mbarrier_init_shared */, 20738 /* vWi*3i */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20745 /* __nvvm_mbarrier_inval */, 9571 /* vWi* */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20767 /* __nvvm_mbarrier_inval_shared */, 9659 /* vWi*3 */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20796 /* __nvvm_mbarrier_pending_count */, 20826 /* iWi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20830 /* __nvvm_mbarrier_test_wait */, 20856 /* bWi*Wi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20863 /* __nvvm_mbarrier_test_wait_shared */, 20896 /* bWi*3Wi */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20904 /* __nvvm_membar_cta */, 8566 /* v */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20922 /* __nvvm_membar_gl */, 8566 /* v */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20939 /* __nvvm_membar_sys */, 8566 /* v */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20957 /* __nvvm_memcpy */, 20971 /* vUc*Uc*zi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20981 /* __nvvm_memset */, 20995 /* vUc*Uczi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21004 /* __nvvm_mul24_i */, 21019 /* iii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21023 /* __nvvm_mul24_ui */, 20339 /* UiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21039 /* __nvvm_mul_rm_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21055 /* __nvvm_mul_rm_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21071 /* __nvvm_mul_rm_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21091 /* __nvvm_mul_rn_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21107 /* __nvvm_mul_rn_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21123 /* __nvvm_mul_rn_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21143 /* __nvvm_mul_rp_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21159 /* __nvvm_mul_rp_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21175 /* __nvvm_mul_rp_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21195 /* __nvvm_mul_rz_d */, 4765 /* ddd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21211 /* __nvvm_mul_rz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21227 /* __nvvm_mul_rz_ftz_f */, 4785 /* fff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21247 /* __nvvm_mulhi_i */, 21019 /* iii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21262 /* __nvvm_mulhi_ll */, 21278 /* LLiLLiLLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21288 /* __nvvm_mulhi_ui */, 20339 /* UiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21304 /* __nvvm_mulhi_ull */, 21321 /* ULLiULLiULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21334 /* __nvvm_nanosleep */, 8382 /* vUi */, 4609 /* n */, 21351 /* (sm_70|sm_72|sm_75|sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21638 /* __nvvm_neg_bf16 */, 4560 /* yy */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21654 /* __nvvm_neg_bf16x2 */, 4581 /* V2yV2y */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21672 /* __nvvm_pm_event_mask */, 21693 /* vIUs */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21698 /* __nvvm_prmt */, 21710 /* UiUiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21719 /* __nvvm_rcp_approx_ftz_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21743 /* __nvvm_rcp_approx_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21767 /* __nvvm_rcp_rm_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21783 /* __nvvm_rcp_rm_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21799 /* __nvvm_rcp_rm_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21819 /* __nvvm_rcp_rn_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21835 /* __nvvm_rcp_rn_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21851 /* __nvvm_rcp_rn_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21871 /* __nvvm_rcp_rp_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21887 /* __nvvm_rcp_rp_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21903 /* __nvvm_rcp_rp_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21923 /* __nvvm_rcp_rz_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21939 /* __nvvm_rcp_rz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21955 /* __nvvm_rcp_rz_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21975 /* __nvvm_read_ptx_sreg_clock */, 22002 /* i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22004 /* __nvvm_read_ptx_sreg_clock64 */, 22033 /* LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22037 /* __nvvm_read_ptx_sreg_cluster_ctaid_w */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22074 /* __nvvm_read_ptx_sreg_cluster_ctaid_x */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22111 /* __nvvm_read_ptx_sreg_cluster_ctaid_y */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22148 /* __nvvm_read_ptx_sreg_cluster_ctaid_z */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22185 /* __nvvm_read_ptx_sreg_cluster_ctarank */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22222 /* __nvvm_read_ptx_sreg_cluster_nctaid_w */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22260 /* __nvvm_read_ptx_sreg_cluster_nctaid_x */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22298 /* __nvvm_read_ptx_sreg_cluster_nctaid_y */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22336 /* __nvvm_read_ptx_sreg_cluster_nctaid_z */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22374 /* __nvvm_read_ptx_sreg_cluster_nctarank */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22412 /* __nvvm_read_ptx_sreg_clusterid_w */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22445 /* __nvvm_read_ptx_sreg_clusterid_x */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22478 /* __nvvm_read_ptx_sreg_clusterid_y */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22511 /* __nvvm_read_ptx_sreg_clusterid_z */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22544 /* __nvvm_read_ptx_sreg_ctaid_w */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22573 /* __nvvm_read_ptx_sreg_ctaid_x */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22602 /* __nvvm_read_ptx_sreg_ctaid_y */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22631 /* __nvvm_read_ptx_sreg_ctaid_z */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22660 /* __nvvm_read_ptx_sreg_globaltimer */, 22033 /* LLi */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22693 /* __nvvm_read_ptx_sreg_gridid */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22721 /* __nvvm_read_ptx_sreg_laneid */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22749 /* __nvvm_read_ptx_sreg_lanemask_eq */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22782 /* __nvvm_read_ptx_sreg_lanemask_ge */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22815 /* __nvvm_read_ptx_sreg_lanemask_gt */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22848 /* __nvvm_read_ptx_sreg_lanemask_le */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22881 /* __nvvm_read_ptx_sreg_lanemask_lt */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22914 /* __nvvm_read_ptx_sreg_nclusterid_w */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22948 /* __nvvm_read_ptx_sreg_nclusterid_x */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22982 /* __nvvm_read_ptx_sreg_nclusterid_y */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23016 /* __nvvm_read_ptx_sreg_nclusterid_z */, 22002 /* i */, 18501 /* nc */, 8568 /* (sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23050 /* __nvvm_read_ptx_sreg_nctaid_w */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23080 /* __nvvm_read_ptx_sreg_nctaid_x */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23110 /* __nvvm_read_ptx_sreg_nctaid_y */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23140 /* __nvvm_read_ptx_sreg_nctaid_z */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23170 /* __nvvm_read_ptx_sreg_nsmid */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23197 /* __nvvm_read_ptx_sreg_ntid_w */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23225 /* __nvvm_read_ptx_sreg_ntid_x */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23253 /* __nvvm_read_ptx_sreg_ntid_y */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23281 /* __nvvm_read_ptx_sreg_ntid_z */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23309 /* __nvvm_read_ptx_sreg_nwarpid */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23338 /* __nvvm_read_ptx_sreg_pm0 */, 22002 /* i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23363 /* __nvvm_read_ptx_sreg_pm1 */, 22002 /* i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23388 /* __nvvm_read_ptx_sreg_pm2 */, 22002 /* i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23413 /* __nvvm_read_ptx_sreg_pm3 */, 22002 /* i */, 4609 /* n */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23438 /* __nvvm_read_ptx_sreg_smid */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23464 /* __nvvm_read_ptx_sreg_tid_w */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23491 /* __nvvm_read_ptx_sreg_tid_x */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23518 /* __nvvm_read_ptx_sreg_tid_y */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23545 /* __nvvm_read_ptx_sreg_tid_z */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23572 /* __nvvm_read_ptx_sreg_warpid */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23600 /* __nvvm_read_ptx_sreg_warpsize */, 22002 /* i */, 18501 /* nc */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23630 /* __nvvm_redux_sync_add */, 21019 /* iii */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23652 /* __nvvm_redux_sync_and */, 21019 /* iii */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23674 /* __nvvm_redux_sync_fmax */, 23697 /* ffi */, 0 /*  */, 23701 /* (sm_100a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23737 /* __nvvm_redux_sync_fmax_NaN */, 23697 /* ffi */, 0 /*  */, 23701 /* (sm_100a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23764 /* __nvvm_redux_sync_fmax_abs */, 23697 /* ffi */, 0 /*  */, 23701 /* (sm_100a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23791 /* __nvvm_redux_sync_fmax_abs_NaN */, 23697 /* ffi */, 0 /*  */, 23701 /* (sm_100a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23822 /* __nvvm_redux_sync_fmin */, 23697 /* ffi */, 0 /*  */, 23701 /* (sm_100a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23845 /* __nvvm_redux_sync_fmin_NaN */, 23697 /* ffi */, 0 /*  */, 23701 /* (sm_100a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23872 /* __nvvm_redux_sync_fmin_abs */, 23697 /* ffi */, 0 /*  */, 23701 /* (sm_100a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23899 /* __nvvm_redux_sync_fmin_abs_NaN */, 23697 /* ffi */, 0 /*  */, 23701 /* (sm_100a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23930 /* __nvvm_redux_sync_max */, 21019 /* iii */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23952 /* __nvvm_redux_sync_min */, 21019 /* iii */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23974 /* __nvvm_redux_sync_or */, 21019 /* iii */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23995 /* __nvvm_redux_sync_umax */, 24018 /* UiUii */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24024 /* __nvvm_redux_sync_umin */, 24018 /* UiUii */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24047 /* __nvvm_redux_sync_xor */, 21019 /* iii */, 0 /*  */, 1353 /* (sm_80|sm_86|sm_87|sm_88|sm_89|sm_90|sm_90a|sm_100|sm_100a|sm_101|sm_101a|sm_103|sm_103a|sm_110|sm_110a|sm_120|sm_120a|sm_121|sm_121a),(ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24069 /* __nvvm_reflect */, 24084 /* UicC* */, 11275 /* r */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24090 /* __nvvm_round_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24105 /* __nvvm_round_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24120 /* __nvvm_round_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24139 /* __nvvm_rsqrt_approx_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24161 /* __nvvm_rsqrt_approx_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24183 /* __nvvm_rsqrt_approx_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24209 /* __nvvm_sad_i */, 24222 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24227 /* __nvvm_sad_ui */, 21710 /* UiUiUiUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24241 /* __nvvm_saturate_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24259 /* __nvvm_saturate_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24277 /* __nvvm_saturate_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24299 /* __nvvm_shfl_bfly_f32 */, 24320 /* ffii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24325 /* __nvvm_shfl_bfly_i32 */, 24222 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24346 /* __nvvm_shfl_down_f32 */, 24320 /* ffii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24367 /* __nvvm_shfl_down_i32 */, 24222 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24388 /* __nvvm_shfl_idx_f32 */, 24320 /* ffii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24408 /* __nvvm_shfl_idx_i32 */, 24222 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24428 /* __nvvm_shfl_sync_bfly_f32 */, 24454 /* fUifii */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24461 /* __nvvm_shfl_sync_bfly_i32 */, 24487 /* iUiiii */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24494 /* __nvvm_shfl_sync_down_f32 */, 24454 /* fUifii */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24520 /* __nvvm_shfl_sync_down_i32 */, 24487 /* iUiiii */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24546 /* __nvvm_shfl_sync_idx_f32 */, 24454 /* fUifii */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24571 /* __nvvm_shfl_sync_idx_i32 */, 24487 /* iUiiii */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24596 /* __nvvm_shfl_sync_up_f32 */, 24454 /* fUifii */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24620 /* __nvvm_shfl_sync_up_i32 */, 24487 /* iUiiii */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24644 /* __nvvm_shfl_up_f32 */, 24320 /* ffii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24663 /* __nvvm_shfl_up_i32 */, 24222 /* iiii */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24682 /* __nvvm_sin_approx_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24702 /* __nvvm_sin_approx_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24726 /* __nvvm_sqrt_approx_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24747 /* __nvvm_sqrt_approx_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24772 /* __nvvm_sqrt_rm_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24789 /* __nvvm_sqrt_rm_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24806 /* __nvvm_sqrt_rm_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24827 /* __nvvm_sqrt_rn_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24844 /* __nvvm_sqrt_rn_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24861 /* __nvvm_sqrt_rn_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24882 /* __nvvm_sqrt_rp_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24899 /* __nvvm_sqrt_rp_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24916 /* __nvvm_sqrt_rp_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24937 /* __nvvm_sqrt_rz_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24954 /* __nvvm_sqrt_rz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24971 /* __nvvm_sqrt_rz_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24992 /* __nvvm_trunc_d */, 9227 /* dd */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25007 /* __nvvm_trunc_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25022 /* __nvvm_trunc_ftz_f */, 9244 /* ff */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25041 /* __nvvm_ue8m0x2_to_bf16x2 */, 25066 /* V2ys */, 0 /*  */, 9057 /* (sm_100a|sm_101a|sm_120a),(ptx86|ptx87|ptx88|ptx90) */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25071 /* __nvvm_ui2d_rm */, 25086 /* dUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25090 /* __nvvm_ui2d_rn */, 25086 /* dUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25105 /* __nvvm_ui2d_rp */, 25086 /* dUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25120 /* __nvvm_ui2d_rz */, 25086 /* dUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25135 /* __nvvm_ui2f_rm */, 25150 /* fUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25154 /* __nvvm_ui2f_rn */, 25150 /* fUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25169 /* __nvvm_ui2f_rp */, 25150 /* fUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25184 /* __nvvm_ui2f_rz */, 25150 /* fUi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25199 /* __nvvm_ull2d_rm */, 25215 /* dULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25221 /* __nvvm_ull2d_rn */, 25215 /* dULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25237 /* __nvvm_ull2d_rp */, 25215 /* dULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25253 /* __nvvm_ull2d_rz */, 25215 /* dULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25269 /* __nvvm_ull2f_rm */, 25285 /* fULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25291 /* __nvvm_ull2f_rn */, 25285 /* fULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25307 /* __nvvm_ull2f_rp */, 25285 /* fULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25323 /* __nvvm_ull2f_rz */, 25285 /* fULLi */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25339 /* __nvvm_vote_all */, 25355 /* bb */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25358 /* __nvvm_vote_all_sync */, 25379 /* bUib */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25384 /* __nvvm_vote_any */, 25355 /* bb */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25400 /* __nvvm_vote_any_sync */, 25379 /* bUib */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25421 /* __nvvm_vote_ballot */, 25440 /* Uib */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25444 /* __nvvm_vote_ballot_sync */, 25468 /* UiUib */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25474 /* __nvvm_vote_uni */, 25355 /* bb */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25490 /* __nvvm_vote_uni_sync */, 25379 /* bUib */, 0 /*  */, 8386 /* ptx60|ptx61|ptx62|ptx63|ptx64|ptx65|ptx70|ptx71|ptx72|ptx73|ptx74|ptx75|ptx76|ptx77|ptx78|ptx80|ptx81|ptx82|ptx83|ptx84|ptx85|ptx86|ptx87|ptx88|ptx90 */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25511 /* __syncthreads */, 8566 /* v */, 0 /*  */, 0 /*  */}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},

#endif // GET_BUILTIN_INFOS

#ifdef GET_BUILTIN_PREFIXED_INFOS

#endif // GET_BUILTIN_PREFIXED_INFOS

#ifdef ATOMIC_BUILTIN

#endif // ATOMIC_BUILTIN
#undef ATOMIC_BUILTIN
