/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* List of builtins that Clang recognizes                                     *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/


#ifdef GET_BUILTIN_ENUMERATORS
    BI__atomic_add_fetch,
    BI__atomic_and_fetch,
    BI__atomic_clear,
    BI__atomic_compare_exchange,
    BI__atomic_compare_exchange_n,
    BI__atomic_exchange,
    BI__atomic_exchange_n,
    BI__atomic_fetch_add,
    BI__atomic_fetch_and,
    BI__atomic_fetch_max,
    BI__atomic_fetch_min,
    BI__atomic_fetch_nand,
    BI__atomic_fetch_or,
    BI__atomic_fetch_sub,
    BI__atomic_fetch_udec,
    BI__atomic_fetch_uinc,
    BI__atomic_fetch_xor,
    BI__atomic_load,
    BI__atomic_load_n,
    BI__atomic_max_fetch,
    BI__atomic_min_fetch,
    BI__atomic_nand_fetch,
    BI__atomic_or_fetch,
    BI__atomic_store,
    BI__atomic_store_n,
    BI__atomic_sub_fetch,
    BI__atomic_test_and_set,
    BI__atomic_xor_fetch,
    BI__c11_atomic_compare_exchange_strong,
    BI__c11_atomic_compare_exchange_weak,
    BI__c11_atomic_exchange,
    BI__c11_atomic_fetch_add,
    BI__c11_atomic_fetch_and,
    BI__c11_atomic_fetch_max,
    BI__c11_atomic_fetch_min,
    BI__c11_atomic_fetch_nand,
    BI__c11_atomic_fetch_or,
    BI__c11_atomic_fetch_sub,
    BI__c11_atomic_fetch_xor,
    BI__c11_atomic_init,
    BI__c11_atomic_load,
    BI__c11_atomic_store,
    BI__hip_atomic_compare_exchange_strong,
    BI__hip_atomic_compare_exchange_weak,
    BI__hip_atomic_exchange,
    BI__hip_atomic_fetch_add,
    BI__hip_atomic_fetch_and,
    BI__hip_atomic_fetch_max,
    BI__hip_atomic_fetch_min,
    BI__hip_atomic_fetch_or,
    BI__hip_atomic_fetch_sub,
    BI__hip_atomic_fetch_xor,
    BI__hip_atomic_load,
    BI__hip_atomic_store,
    BI__opencl_atomic_compare_exchange_strong,
    BI__opencl_atomic_compare_exchange_weak,
    BI__opencl_atomic_exchange,
    BI__opencl_atomic_fetch_add,
    BI__opencl_atomic_fetch_and,
    BI__opencl_atomic_fetch_max,
    BI__opencl_atomic_fetch_min,
    BI__opencl_atomic_fetch_or,
    BI__opencl_atomic_fetch_sub,
    BI__opencl_atomic_fetch_xor,
    BI__opencl_atomic_init,
    BI__opencl_atomic_load,
    BI__opencl_atomic_store,
    BI__scoped_atomic_add_fetch,
    BI__scoped_atomic_and_fetch,
    BI__scoped_atomic_compare_exchange,
    BI__scoped_atomic_compare_exchange_n,
    BI__scoped_atomic_exchange,
    BI__scoped_atomic_exchange_n,
    BI__scoped_atomic_fetch_add,
    BI__scoped_atomic_fetch_and,
    BI__scoped_atomic_fetch_max,
    BI__scoped_atomic_fetch_min,
    BI__scoped_atomic_fetch_nand,
    BI__scoped_atomic_fetch_or,
    BI__scoped_atomic_fetch_sub,
    BI__scoped_atomic_fetch_xor,
    BI__scoped_atomic_load,
    BI__scoped_atomic_load_n,
    BI__scoped_atomic_max_fetch,
    BI__scoped_atomic_min_fetch,
    BI__scoped_atomic_nand_fetch,
    BI__scoped_atomic_or_fetch,
    BI__scoped_atomic_store,
    BI__scoped_atomic_store_n,
    BI__scoped_atomic_sub_fetch,
    BI__scoped_atomic_fetch_udec,
    BI__scoped_atomic_fetch_uinc,
    BI__scoped_atomic_xor_fetch,
    BI__abnormal_termination,
    BI_abnormal_termination,
    BI__builtin_abort,
    BIabort,
    BI__builtin_abs,
    BIabs,
    BI__builtin_labs,
    BIlabs,
    BI__builtin_llabs,
    BIllabs,
    BI__builtin_acosf,
    BIacosf,
    BI__builtin_acos,
    BIacos,
    BI__builtin_acosl,
    BIacosl,
    BI__builtin_acosf16,
    BI__builtin_acosf128,
    BI__builtin_acoshf,
    BIacoshf,
    BI__builtin_acosh,
    BIacosh,
    BI__builtin_acoshl,
    BIacoshl,
    BI__builtin_acoshf128,
    BI__builtin_add_overflow,
    BI__builtin_addcb,
    BI__builtin_addcs,
    BI__builtin_addc,
    BI__builtin_addcl,
    BI__builtin_addcll,
    BIaddressof,
    BI__builtin_align_down,
    BI__builtin_align_up,
    BIaligned_alloc,
    BI__builtin_alloca,
    BIalloca,
    BI_alloca,
    BI__builtin_alloca_uninitialized,
    BI__builtin_alloca_with_align,
    BI__builtin_alloca_with_align_uninitialized,
    BI__builtin_allow_runtime_check,
    BI__builtin_annotation,
    BI__arithmetic_fence,
    BIas_const,
    BI__builtin_asinf,
    BIasinf,
    BI__builtin_asin,
    BIasin,
    BI__builtin_asinl,
    BIasinl,
    BI__builtin_asinf16,
    BI__builtin_asinf128,
    BI__builtin_asinhf,
    BIasinhf,
    BI__builtin_asinh,
    BIasinh,
    BI__builtin_asinhl,
    BIasinhl,
    BI__builtin_asinhf128,
    BI__builtin_assume,
    BI__builtin_assume_separate_storage,
    BI__builtin_atanf,
    BIatanf,
    BI__builtin_atan,
    BIatan,
    BI__builtin_atanl,
    BIatanl,
    BI__builtin_atan2f,
    BIatan2f,
    BI__builtin_atan2,
    BIatan2,
    BI__builtin_atan2l,
    BIatan2l,
    BI__builtin_atan2f16,
    BI__builtin_atan2f128,
    BI__builtin_atanf16,
    BI__builtin_atanf128,
    BI__builtin_atanhf,
    BIatanhf,
    BI__builtin_atanh,
    BIatanh,
    BI__builtin_atanhl,
    BIatanhl,
    BI__builtin_atanhf128,
    BI__atomic_always_lock_free,
    BI__atomic_is_lock_free,
    BI__atomic_signal_fence,
    BI__atomic_thread_fence,
    BIbcmp,
    BI__builtin_bswap16,
    BI__builtin_bswap32,
    BI__builtin_bswap64,
    BI__builtin_bswapg,
    BI__builtin_bzero,
    BIbzero,
    BI__builtin_bcopy,
    BIbcopy,
    BI__builtin_bitreverse8,
    BI__builtin_bitreverse16,
    BI__builtin_bitreverse32,
    BI__builtin_bitreverse64,
    BI_bittest,
    BI_bittest64,
    BI_bittestandcomplement,
    BI_bittestandcomplement64,
    BI_bittestandreset,
    BI_bittestandreset64,
    BI_bittestandset,
    BI_bittestandset64,
    BI_Block_object_assign,
    BI_Block_object_dispose,
    BI__builtin_addressof,
    BI__builtin_allow_sanitize_check,
    BI__builtin_assume_aligned,
    BI__builtin_assume_dereferenceable,
    BI__builtin_bcmp,
    BI__builtin_c23_va_start,
    BI__builtin___CFStringMakeConstantString,
    BI__builtin_cpu_init,
    BI__builtin_cpu_is,
    BI__builtin_cpu_supports,
    BI__builtin_calloc,
    BI__builtin_char_memchr,
    BI__builtin_classify_type,
    BI__builtin_complex,
    BI__builtin_constant_p,
    BI__builtin_dump_struct,
    BI__builtin_flt_rounds,
    BI__builtin_free,
    BI__builtin_function_start,
    BI__builtin_longjmp,
    BI__builtin_malloc,
    BI__builtin_memcmp,
    BI__builtin_memcpy_inline,
    BI__builtin_mempcpy,
    BI__builtin_memset_inline,
    BI__builtin___NSStringMakeConstantString,
    BI__builtin_operator_delete,
    BI__builtin_operator_new,
    BI__builtin_preserve_access_index,
    BI__builtin_printf,
    BI__builtin_realloc,
    BI__builtin_return_address,
    BI__builtin_set_flt_rounds,
    BI__builtin_setjmp,
    BI__builtin_stdarg_start,
    BI__builtin_strcspn,
    BI__builtin_trivially_relocate,
    BI__builtin_va_start,
    BI__c11_atomic_is_lock_free,
    BI__c11_atomic_signal_fence,
    BI__c11_atomic_thread_fence,
    BI__builtin_cabsf,
    BIcabsf,
    BI__builtin_cabs,
    BIcabs,
    BI__builtin_cabsl,
    BIcabsl,
    BI__builtin_cacosf,
    BIcacosf,
    BI__builtin_cacos,
    BIcacos,
    BI__builtin_cacosl,
    BIcacosl,
    BI__builtin_cacoshf,
    BIcacoshf,
    BI__builtin_cacosh,
    BIcacosh,
    BI__builtin_cacoshl,
    BIcacoshl,
    BI__builtin_call_with_static_chain,
    BIcalloc,
    BI__builtin_canonicalizef,
    BI__builtin_canonicalize,
    BI__builtin_canonicalizel,
    BI__builtin_canonicalizef16,
    BI__builtin_cargf,
    BIcargf,
    BI__builtin_carg,
    BIcarg,
    BI__builtin_cargl,
    BIcargl,
    BI__builtin_casinf,
    BIcasinf,
    BI__builtin_casin,
    BIcasin,
    BI__builtin_casinl,
    BIcasinl,
    BI__builtin_casinhf,
    BIcasinhf,
    BI__builtin_casinh,
    BIcasinh,
    BI__builtin_casinhl,
    BIcasinhl,
    BI__builtin_catanf,
    BIcatanf,
    BI__builtin_catan,
    BIcatan,
    BI__builtin_catanl,
    BIcatanl,
    BI__builtin_catanhf,
    BIcatanhf,
    BI__builtin_catanh,
    BIcatanh,
    BI__builtin_catanhl,
    BIcatanhl,
    BI__builtin_cbrtf,
    BIcbrtf,
    BI__builtin_cbrt,
    BIcbrt,
    BI__builtin_cbrtl,
    BIcbrtl,
    BI__builtin_cbrtf128,
    BI__builtin_ccosf,
    BIccosf,
    BI__builtin_ccos,
    BIccos,
    BI__builtin_ccosl,
    BIccosl,
    BI__builtin_ccoshf,
    BIccoshf,
    BI__builtin_ccosh,
    BIccosh,
    BI__builtin_ccoshl,
    BIccoshl,
    BI__builtin_ceilf,
    BIceilf,
    BI__builtin_ceil,
    BIceil,
    BI__builtin_ceill,
    BIceill,
    BI__builtin_ceilf16,
    BI__builtin_ceilf128,
    BI__builtin_cexpf,
    BIcexpf,
    BI__builtin_cexp,
    BIcexp,
    BI__builtin_cexpl,
    BIcexpl,
    BI__builtin_cimagf,
    BIcimagf,
    BI__builtin_cimag,
    BIcimag,
    BI__builtin_cimagl,
    BIcimagl,
    BI__builtin___clear_cache,
    BI__builtin_clogf,
    BIclogf,
    BI__builtin_clog,
    BIclog,
    BI__builtin_clogl,
    BIclogl,
    BI__builtin_clrsb,
    BI__builtin_clrsbl,
    BI__builtin_clrsbll,
    BI__builtin_clzs,
    BI__builtin_clz,
    BI__builtin_clzl,
    BI__builtin_clzll,
    BI__builtin_clzg,
    BIcommit_read_pipe,
    BIcommit_write_pipe,
    BI__builtin_conjf,
    BIconjf,
    BI__builtin_conj,
    BIconj,
    BI__builtin_conjl,
    BIconjl,
    BI__builtin_convertvector,
    BI__builtin_copysignf,
    BIcopysignf,
    BI__builtin_copysign,
    BIcopysign,
    BI__builtin_copysignl,
    BIcopysignl,
    BI__builtin_copysignf16,
    BI__builtin_copysignf128,
    BI__builtin_coro_align,
    BI__builtin_coro_alloc,
    BI__builtin_coro_begin,
    BI__builtin_coro_destroy,
    BI__builtin_coro_done,
    BI__builtin_coro_end,
    BI__builtin_coro_frame,
    BI__builtin_coro_free,
    BI__builtin_coro_id,
    BI__builtin_coro_noop,
    BI__builtin_coro_promise,
    BI__builtin_coro_resume,
    BI__builtin_coro_size,
    BI__builtin_coro_suspend,
    BI__builtin_cosf,
    BIcosf,
    BI__builtin_cos,
    BIcos,
    BI__builtin_cosl,
    BIcosl,
    BI__builtin_cosf16,
    BI__builtin_cosf128,
    BI__builtin_coshf,
    BIcoshf,
    BI__builtin_cosh,
    BIcosh,
    BI__builtin_coshl,
    BIcoshl,
    BI__builtin_coshf16,
    BI__builtin_coshf128,
    BI__cospif,
    BI__cospi,
    BI__builtin_counted_by_ref,
    BI__builtin_cpowf,
    BIcpowf,
    BI__builtin_cpow,
    BIcpow,
    BI__builtin_cpowl,
    BIcpowl,
    BI__builtin_cprojf,
    BIcprojf,
    BI__builtin_cproj,
    BIcproj,
    BI__builtin_cprojl,
    BIcprojl,
    BI__builtin_crealf,
    BIcrealf,
    BI__builtin_creal,
    BIcreal,
    BI__builtin_creall,
    BIcreall,
    BI__builtin_csinf,
    BIcsinf,
    BI__builtin_csin,
    BIcsin,
    BI__builtin_csinl,
    BIcsinl,
    BI__builtin_csinhf,
    BIcsinhf,
    BI__builtin_csinh,
    BIcsinh,
    BI__builtin_csinhl,
    BIcsinhl,
    BI__builtin_csqrtf,
    BIcsqrtf,
    BI__builtin_csqrt,
    BIcsqrt,
    BI__builtin_csqrtl,
    BIcsqrtl,
    BI__builtin_ctanf,
    BIctanf,
    BI__builtin_ctan,
    BIctan,
    BI__builtin_ctanl,
    BIctanl,
    BI__builtin_ctanhf,
    BIctanhf,
    BI__builtin_ctanh,
    BIctanh,
    BI__builtin_ctanhl,
    BIctanhl,
    BI__builtin_ctzs,
    BI__builtin_ctz,
    BI__builtin_ctzl,
    BI__builtin_ctzll,
    BI__builtin_ctzg,
    BI__builtin_dwarf_sp_column,
    BI__builtin_dwarf_cfa,
    BI__debugbreak,
    BI__builtin_debugtrap,
    BI__builtin_dynamic_object_size,
    BI__builtin_eh_return,
    BI__builtin_eh_return_data_regno,
    BI__builtin_elementwise_acos,
    BI__builtin_elementwise_asin,
    BI__builtin_elementwise_atan,
    BI__builtin_elementwise_atan2,
    BI__builtin_elementwise_abs,
    BI__builtin_elementwise_add_sat,
    BI__builtin_elementwise_bitreverse,
    BI__builtin_elementwise_canonicalize,
    BI__builtin_elementwise_ceil,
    BI__builtin_elementwise_copysign,
    BI__builtin_elementwise_cos,
    BI__builtin_elementwise_cosh,
    BI__builtin_elementwise_clzg,
    BI__builtin_elementwise_ctzg,
    BI__builtin_elementwise_exp,
    BI__builtin_elementwise_exp2,
    BI__builtin_elementwise_exp10,
    BI__builtin_elementwise_floor,
    BI__builtin_elementwise_fma,
    BI__builtin_elementwise_fmod,
    BI__builtin_elementwise_fshl,
    BI__builtin_elementwise_fshr,
    BI__builtin_elementwise_ldexp,
    BI__builtin_elementwise_log,
    BI__builtin_elementwise_log2,
    BI__builtin_elementwise_log10,
    BI__builtin_elementwise_max,
    BI__builtin_elementwise_maxnum,
    BI__builtin_elementwise_maximum,
    BI__builtin_elementwise_maximumnum,
    BI__builtin_elementwise_min,
    BI__builtin_elementwise_minnum,
    BI__builtin_elementwise_minimum,
    BI__builtin_elementwise_minimumnum,
    BI__builtin_elementwise_nearbyint,
    BI__builtin_elementwise_popcount,
    BI__builtin_elementwise_pow,
    BI__builtin_elementwise_rint,
    BI__builtin_elementwise_round,
    BI__builtin_elementwise_roundeven,
    BI__builtin_elementwise_sin,
    BI__builtin_elementwise_sinh,
    BI__builtin_elementwise_sqrt,
    BI__builtin_elementwise_sub_sat,
    BI__builtin_elementwise_tan,
    BI__builtin_elementwise_tanh,
    BI__builtin_elementwise_trunc,
    BIenqueue_kernel,
    BI__builtin_erff,
    BIerff,
    BI__builtin_erf,
    BIerf,
    BI__builtin_erfl,
    BIerfl,
    BI__builtin_erff128,
    BI__builtin_erfcf,
    BIerfcf,
    BI__builtin_erfc,
    BIerfc,
    BI__builtin_erfcl,
    BIerfcl,
    BI__builtin_erfcf128,
    BI__exception_code,
    BI_exception_code,
    BI__exception_info,
    BI_exception_info,
    BIexit,
    BI_Exit,
    BI__builtin_expf,
    BIexpf,
    BI__builtin_exp,
    BIexp,
    BI__builtin_expl,
    BIexpl,
    BI__builtin_exp2f,
    BIexp2f,
    BI__builtin_exp2,
    BIexp2,
    BI__builtin_exp2l,
    BIexp2l,
    BI__builtin_exp2f16,
    BI__builtin_exp2f128,
    BI__builtin_exp10f,
    BI__builtin_exp10,
    BI__builtin_exp10l,
    BI__builtin_exp10f16,
    BI__builtin_exp10f128,
    BI__builtin_expf16,
    BI__builtin_expf128,
    BI__builtin_expect,
    BI__builtin_expect_with_probability,
    BI__builtin_expm1f,
    BIexpm1f,
    BI__builtin_expm1,
    BIexpm1,
    BI__builtin_expm1l,
    BIexpm1l,
    BI__builtin_expm1f128,
    BI__builtin_extend_pointer,
    BI__builtin_extract_return_addr,
    BI__builtin_ffs,
    BI__builtin_ffsl,
    BI__builtin_ffsll,
    BI__builtin_fpclassify,
    BI__builtin_fprintf,
    BIfprintf,
    BI__builtin___fprintf_chk,
    BI__builtin_fscanf,
    BIfscanf,
    BI__builtin_fabsf,
    BIfabsf,
    BI__builtin_fabs,
    BIfabs,
    BI__builtin_fabsl,
    BIfabsl,
    BI__builtin_fabsf16,
    BI__builtin_fabsf128,
    BI__fastfail,
    BI__builtin_fdimf,
    BIfdimf,
    BI__builtin_fdim,
    BIfdim,
    BI__builtin_fdiml,
    BIfdiml,
    BI__builtin_fdimf128,
    BIfinitef,
    BIfinite,
    BIfinitel,
    BI__builtin_floorf,
    BIfloorf,
    BI__builtin_floor,
    BIfloor,
    BI__builtin_floorl,
    BIfloorl,
    BI__builtin_floorf16,
    BI__builtin_floorf128,
    BI__builtin_fmaf,
    BIfmaf,
    BI__builtin_fma,
    BIfma,
    BI__builtin_fmal,
    BIfmal,
    BI__builtin_fmaf16,
    BI__builtin_fmaf128,
    BI__builtin_fmaxf,
    BIfmaxf,
    BI__builtin_fmax,
    BIfmax,
    BI__builtin_fmaxl,
    BIfmaxl,
    BI__builtin_fmaxf16,
    BI__builtin_fmaxf128,
    BI__builtin_fmaximum_numf,
    BIfmaximum_numf,
    BI__builtin_fmaximum_num,
    BIfmaximum_num,
    BI__builtin_fmaximum_numl,
    BIfmaximum_numl,
    BI__builtin_fmaximum_numf16,
    BI__builtin_fmaximum_numf128,
    BI__builtin_fminf,
    BIfminf,
    BI__builtin_fmin,
    BIfmin,
    BI__builtin_fminl,
    BIfminl,
    BI__builtin_fminf16,
    BI__builtin_fminf128,
    BI__builtin_fminimum_numf,
    BIfminimum_numf,
    BI__builtin_fminimum_num,
    BIfminimum_num,
    BI__builtin_fminimum_numl,
    BIfminimum_numl,
    BI__builtin_fminimum_numf16,
    BI__builtin_fminimum_numf128,
    BI__builtin_fmodf,
    BIfmodf,
    BI__builtin_fmod,
    BIfmod,
    BI__builtin_fmodl,
    BIfmodl,
    BI__builtin_fmodf16,
    BI__builtin_fmodf128,
    BIfopen,
    BIforward,
    BIforward_like,
    BI__builtin_frame_address,
    BIfread,
    BIfree,
    BI__builtin_frexpf,
    BIfrexpf,
    BI__builtin_frexp,
    BIfrexp,
    BI__builtin_frexpl,
    BIfrexpl,
    BI__builtin_frexpf16,
    BI__builtin_frexpf128,
    BI__builtin_frob_return_addr,
    BIfwrite,
    BI_exit,
    BI_longjmp,
    BIgetcontext,
    BI__builtin_get_device_side_mangled_name,
    BI__GetExceptionInfo,
    BIget_kernel_max_sub_group_size_for_ndrange,
    BIget_kernel_preferred_work_group_size_multiple,
    BIget_kernel_sub_group_count_for_ndrange,
    BIget_kernel_work_group_size,
    BIget_pipe_max_packets,
    BIget_pipe_num_packets,
    BI__builtin___get_unsafe_stack_bottom,
    BI__builtin___get_unsafe_stack_ptr,
    BI__builtin___get_unsafe_stack_start,
    BI__builtin___get_unsafe_stack_top,
    BI__builtin_get_vtable_pointer,
    BI__builtin_hlsl_adduint64,
    BI__builtin_hlsl_all,
    BI__builtin_hlsl_and,
    BI__builtin_hlsl_any,
    BI__builtin_hlsl_asdouble,
    BI__builtin_hlsl_buffer_update_counter,
    BI__builtin_hlsl_elementwise_clamp,
    BI__builtin_hlsl_elementwise_clip,
    BI__builtin_hlsl_crossf32,
    BI__builtin_hlsl_crossf16,
    BI__builtin_hlsl_elementwise_ddx_coarse,
    BI__builtin_hlsl_elementwise_ddx_fine,
    BI__builtin_hlsl_elementwise_ddy_coarse,
    BI__builtin_hlsl_elementwise_ddy_fine,
    BI__builtin_hlsl_elementwise_degrees,
    BI__builtin_hlsl_dot4add_i8packed,
    BI__builtin_hlsl_dot4add_u8packed,
    BI__builtin_hlsl_dot,
    BI__builtin_hlsl_elementwise_f16tof32,
    BI__builtin_hlsl_elementwise_firstbithigh,
    BI__builtin_hlsl_elementwise_firstbitlow,
    BI__builtin_hlsl_elementwise_frac,
    BI__builtin_get_spirv_spec_constant_bool,
    BI__builtin_get_spirv_spec_constant_char,
    BI__builtin_get_spirv_spec_constant_short,
    BI__builtin_get_spirv_spec_constant_int,
    BI__builtin_get_spirv_spec_constant_longlong,
    BI__builtin_get_spirv_spec_constant_ushort,
    BI__builtin_get_spirv_spec_constant_uint,
    BI__builtin_get_spirv_spec_constant_ulonglong,
    BI__builtin_get_spirv_spec_constant_half,
    BI__builtin_get_spirv_spec_constant_float,
    BI__builtin_get_spirv_spec_constant_double,
    BI__builtin_hlsl_group_memory_barrier_with_group_sync,
    BI__builtin_hlsl_elementwise_isinf,
    BI__builtin_hlsl_elementwise_isnan,
    BI__builtin_hlsl_lerp,
    BI__builtin_hlsl_mad,
    BI__builtin_hlsl_normalize,
    BI__builtin_hlsl_or,
    BI__builtin_hlsl_elementwise_rsqrt,
    BI__builtin_hlsl_elementwise_radians,
    BI__builtin_hlsl_elementwise_rcp,
    BI__builtin_hlsl_resource_counterhandlefromimplicitbinding,
    BI__builtin_hlsl_resource_getdimensions_x,
    BI__builtin_hlsl_resource_getpointer,
    BI__builtin_hlsl_resource_getstride,
    BI__builtin_hlsl_resource_handlefrombinding,
    BI__builtin_hlsl_resource_handlefromimplicitbinding,
    BI__builtin_hlsl_resource_load_with_status,
    BI__builtin_hlsl_resource_nonuniformindex,
    BI__builtin_hlsl_resource_uninitializedhandle,
    BI__builtin_hlsl_elementwise_saturate,
    BI__builtin_hlsl_select,
    BI__builtin_hlsl_elementwise_sign,
    BI__builtin_hlsl_elementwise_splitdouble,
    BI__builtin_hlsl_step,
    BI__builtin_hlsl_wave_active_all_true,
    BI__builtin_hlsl_wave_active_any_true,
    BI__builtin_hlsl_wave_active_ballot,
    BI__builtin_hlsl_wave_active_count_bits,
    BI__builtin_hlsl_wave_active_max,
    BI__builtin_hlsl_wave_active_min,
    BI__builtin_hlsl_wave_active_sum,
    BI__builtin_hlsl_wave_get_lane_count,
    BI__builtin_hlsl_wave_get_lane_index,
    BI__builtin_hlsl_wave_is_first_lane,
    BI__builtin_hlsl_wave_read_lane_at,
    BI__builtin_huge_valf,
    BI__builtin_huge_val,
    BI__builtin_huge_vall,
    BI__builtin_huge_valf128,
    BI__builtin_huge_valf16,
    BI__builtin_hypotf,
    BIhypotf,
    BI__builtin_hypot,
    BIhypot,
    BI__builtin_hypotl,
    BIhypotl,
    BI__builtin_hypotf128,
    BI__builtin_ilogbf128,
    BI__builtin_ilogbf,
    BIilogbf,
    BI__builtin_ilogb,
    BIilogb,
    BI__builtin_ilogbl,
    BIilogbl,
    BI__builtin_index,
    BIindex,
    BI__builtin_inff,
    BI__builtin_inf,
    BI__builtin_infl,
    BI__builtin_inff128,
    BI__builtin_inff16,
    BI__builtin_infer_alloc_token,
    BI__builtin_init_dwarf_reg_size_table,
    BI_InterlockedAnd8,
    BI_InterlockedAnd16,
    BI_InterlockedAnd,
    BI_interlockedbittestandreset,
    BI_interlockedbittestandreset64,
    BI_interlockedbittestandreset64_acq,
    BI_interlockedbittestandreset64_nf,
    BI_interlockedbittestandreset64_rel,
    BI_interlockedbittestandreset_acq,
    BI_interlockedbittestandreset_nf,
    BI_interlockedbittestandreset_rel,
    BI_interlockedbittestandset,
    BI_interlockedbittestandset64,
    BI_interlockedbittestandset64_acq,
    BI_interlockedbittestandset64_nf,
    BI_interlockedbittestandset64_rel,
    BI_interlockedbittestandset_acq,
    BI_interlockedbittestandset_nf,
    BI_interlockedbittestandset_rel,
    BI_InterlockedCompareExchange8,
    BI_InterlockedCompareExchange16,
    BI_InterlockedCompareExchange,
    BI_InterlockedCompareExchange64,
    BI_InterlockedCompareExchangePointer,
    BI_InterlockedCompareExchangePointer_nf,
    BI_InterlockedDecrement16,
    BI_InterlockedDecrement,
    BI_InterlockedExchange8,
    BI_InterlockedExchange16,
    BI_InterlockedExchange,
    BI_InterlockedExchangeAdd8,
    BI_InterlockedExchangeAdd16,
    BI_InterlockedExchangeAdd,
    BI_InterlockedExchangePointer,
    BI_InterlockedExchangeSub8,
    BI_InterlockedExchangeSub16,
    BI_InterlockedExchangeSub,
    BI_InterlockedIncrement16,
    BI_InterlockedIncrement,
    BI_InterlockedOr8,
    BI_InterlockedOr16,
    BI_InterlockedOr,
    BI_InterlockedXor8,
    BI_InterlockedXor16,
    BI_InterlockedXor,
    BI__builtin_invoke,
    BIisalnum,
    BI__builtin_is_aligned,
    BIisalpha,
    BIisblank,
    BIiscntrl,
    BI__builtin_is_constant_evaluated,
    BIisdigit,
    BI__builtin_isfpclass,
    BI__builtin_isfinite,
    BIisgraph,
    BI__builtin_isgreater,
    BI__builtin_isgreaterequal,
    BI__builtin_isinf,
    BI__builtin_isinf_sign,
    BI__builtin_isless,
    BI__builtin_islessequal,
    BI__builtin_islessgreater,
    BIislower,
    BI__builtin_isnan,
    BI__builtin_isnormal,
    BIisprint,
    BIispunct,
    BI__builtin_issignaling,
    BIisspace,
    BI__builtin_issubnormal,
    BI__builtin_isunordered,
    BIisupper,
    BI__builtin_is_within_lifetime,
    BIisxdigit,
    BI__builtin_iszero,
    BI__iso_volatile_load8,
    BI__iso_volatile_load16,
    BI__iso_volatile_load32,
    BI__iso_volatile_load64,
    BI__iso_volatile_store8,
    BI__iso_volatile_store16,
    BI__iso_volatile_store32,
    BI__iso_volatile_store64,
    BI__builtin_llrintf128,
    BI__builtin_llroundf128,
    BI__builtin_launder,
    BI__builtin_ldexpf,
    BIldexpf,
    BI__builtin_ldexp,
    BIldexp,
    BI__builtin_ldexpl,
    BIldexpl,
    BI__builtin_ldexpf16,
    BI__builtin_ldexpf128,
    BI__builtin_lgammaf,
    BIlgammaf,
    BI__builtin_lgamma,
    BIlgamma,
    BI__builtin_lgammal,
    BIlgammal,
    BI__builtin_lgammaf128,
    BI__builtin_llrintf,
    BIllrintf,
    BI__builtin_llrint,
    BIllrint,
    BI__builtin_llrintl,
    BIllrintl,
    BI__builtin_llroundf,
    BIllroundf,
    BI__builtin_llround,
    BIllround,
    BI__builtin_llroundl,
    BIllroundl,
    BI__builtin_load_halff,
    BI__builtin_load_half,
    BI__builtin_logf,
    BIlogf,
    BI__builtin_log,
    BIlog,
    BI__builtin_logl,
    BIlogl,
    BI__builtin_log1pf,
    BIlog1pf,
    BI__builtin_log1p,
    BIlog1p,
    BI__builtin_log1pl,
    BIlog1pl,
    BI__builtin_log1pf128,
    BI__builtin_log2f,
    BIlog2f,
    BI__builtin_log2,
    BIlog2,
    BI__builtin_log2l,
    BIlog2l,
    BI__builtin_log2f16,
    BI__builtin_log2f128,
    BI__builtin_log10f,
    BIlog10f,
    BI__builtin_log10,
    BIlog10,
    BI__builtin_log10l,
    BIlog10l,
    BI__builtin_log10f16,
    BI__builtin_log10f128,
    BI__builtin_logf16,
    BI__builtin_logf128,
    BI__builtin_logbf,
    BIlogbf,
    BI__builtin_logb,
    BIlogb,
    BI__builtin_logbl,
    BIlogbl,
    BI__builtin_logbf128,
    BIlongjmp,
    BI__builtin_lrintf,
    BIlrintf,
    BI__builtin_lrint,
    BIlrint,
    BI__builtin_lrintl,
    BIlrintl,
    BI__builtin_lrintf128,
    BI_lrotl,
    BI_lrotr,
    BI__builtin_lroundf,
    BIlroundf,
    BI__builtin_lround,
    BIlround,
    BI__builtin_lroundl,
    BIlroundl,
    BI__builtin_lroundf128,
    BI__annotation,
    BI__assume,
    BI_byteswap_ushort,
    BI_byteswap_ulong,
    BI_byteswap_uint64,
    BI__lzcnt16,
    BI__lzcnt,
    BI__lzcnt64,
    BI__popcnt16,
    BI__popcnt,
    BI__popcnt64,
    BI_ReturnAddress,
    BI__builtin_ms_va_copy,
    BI__builtin_ms_va_end,
    BI__builtin_ms_va_start,
    BI__va_start,
    BImalloc,
    BI__builtin_masked_compress_store,
    BI__builtin_masked_expand_load,
    BI__builtin_masked_gather,
    BI__builtin_masked_load,
    BI__builtin_masked_scatter,
    BI__builtin_masked_store,
    BI__builtin_matrix_column_major_load,
    BI__builtin_matrix_column_major_store,
    BI__builtin_matrix_transpose,
    BImemalign,
    BI__builtin_memchr,
    BImemchr,
    BImemcmp,
    BI__builtin_memcpy,
    BImemcpy,
    BI__builtin_memmove,
    BImemmove,
    BI__builtin_memset,
    BImemset,
    BImemccpy,
    BI__builtin___memccpy_chk,
    BI__builtin___memcpy_chk,
    BI__builtin___memmove_chk,
    BImempcpy,
    BI__builtin___mempcpy_chk,
    BI__builtin___memset_chk,
    BI__builtin_modff,
    BImodff,
    BI__builtin_modf,
    BImodf,
    BI__builtin_modfl,
    BImodfl,
    BI__builtin_modff128,
    BImove,
    BImove_if_noexcept,
    BI__builtin_mul_overflow,
    BINSLog,
    BINSLogv,
    BI__builtin_nanf,
    BInanf,
    BI__builtin_nan,
    BInan,
    BI__builtin_nanl,
    BInanl,
    BI__builtin_nanf16,
    BI__builtin_nanf128,
    BI__builtin_nansf,
    BI__builtin_nans,
    BI__builtin_nansl,
    BI__builtin_nansf16,
    BI__builtin_nansf128,
    BI__builtin_nearbyintf,
    BInearbyintf,
    BI__builtin_nearbyint,
    BInearbyint,
    BI__builtin_nearbyintl,
    BInearbyintl,
    BI__builtin_nearbyintf128,
    BI__builtin_nextafterf,
    BInextafterf,
    BI__builtin_nextafter,
    BInextafter,
    BI__builtin_nextafterl,
    BInextafterl,
    BI__builtin_nextafterf128,
    BI__builtin_nexttowardf,
    BInexttowardf,
    BI__builtin_nexttoward,
    BInexttoward,
    BI__builtin_nexttowardl,
    BInexttowardl,
    BI__builtin_nexttowardf128,
    BI__builtin_nondeterministic_value,
    BI__builtin_nontemporal_load,
    BI__builtin_nontemporal_store,
    BI__noop,
    BI__builtin_os_log_format,
    BI__builtin_os_log_format_buffer_size,
    BI__exp10f,
    BI__exp10,
    BI__finitef,
    BI__finite,
    BI__finitel,
    BIobjc_assign_global,
    BIobjc_assign_ivar,
    BIobjc_assign_strongCast,
    BIobjc_assign_weak,
    BIobjc_enumerationMutation,
    BIobjc_exception_extract,
    BIobjc_exception_match,
    BIobjc_exception_throw,
    BIobjc_exception_try_enter,
    BIobjc_exception_try_exit,
    BIobjc_getClass,
    BIobjc_getMetaClass,
    BI__builtin_objc_memmove_collectable,
    BIobjc_msgSend,
    BIobjc_msgSend_fp2ret,
    BIobjc_msgSend_fpret,
    BIobjc_msgSend_stret,
    BIobjc_msgSendSuper,
    BIobjc_msgSendSuper_stret,
    BIobjc_read_weak,
    BIobjc_sync_enter,
    BIobjc_sync_exit,
    BI__builtin_object_size,
    BI__builtin_parity,
    BI__builtin_parityl,
    BI__builtin_parityll,
    BI__builtin_popcount,
    BI__builtin_popcountl,
    BI__builtin_popcountll,
    BI__builtin_popcountg,
    BI__builtin_powf,
    BIpowf,
    BI__builtin_pow,
    BIpow,
    BI__builtin_powl,
    BIpowl,
    BI__builtin_powf16,
    BI__builtin_powf128,
    BI__builtin_powif,
    BI__builtin_powi,
    BI__builtin_powil,
    BI__builtin_prefetch,
    BIprintf,
    BI__builtin___printf_chk,
    BIpthread_create,
    BI__builtin_ptrauth_auth,
    BI__builtin_ptrauth_auth_and_resign,
    BI__builtin_ptrauth_blend_discriminator,
    BI__builtin_ptrauth_sign_constant,
    BI__builtin_ptrauth_sign_generic_data,
    BI__builtin_ptrauth_sign_unauthenticated,
    BI__builtin_ptrauth_string_discriminator,
    BI__builtin_ptrauth_strip,
    BI__builtin_readcyclecounter,
    BIread_pipe,
    BI__builtin_readsteadycounter,
    BIrealloc,
    BI__builtin_reduce_add,
    BI__builtin_reduce_and,
    BI__builtin_reduce_max,
    BI__builtin_reduce_maximum,
    BI__builtin_reduce_min,
    BI__builtin_reduce_minimum,
    BI__builtin_reduce_mul,
    BI__builtin_reduce_or,
    BI__builtin_reduce_xor,
    BI__builtin_remainderf,
    BIremainderf,
    BI__builtin_remainder,
    BIremainder,
    BI__builtin_remainderl,
    BIremainderl,
    BI__builtin_remainderf128,
    BI__builtin_remquof,
    BIremquof,
    BI__builtin_remquo,
    BIremquo,
    BI__builtin_remquol,
    BIremquol,
    BI__builtin_remquof128,
    BIreserve_read_pipe,
    BIreserve_write_pipe,
    BI__builtin_rindex,
    BIrindex,
    BI__builtin_rintf,
    BIrintf,
    BI__builtin_rint,
    BIrint,
    BI__builtin_rintl,
    BIrintl,
    BI__builtin_rintf16,
    BI__builtin_rintf128,
    BI__builtin_rotateleft8,
    BI__builtin_rotateleft16,
    BI__builtin_rotateleft32,
    BI__builtin_rotateleft64,
    BI__builtin_rotateright8,
    BI__builtin_rotateright16,
    BI__builtin_rotateright32,
    BI__builtin_rotateright64,
    BI_rotl,
    BI_rotl8,
    BI_rotl16,
    BI_rotl64,
    BI_rotr,
    BI_rotr8,
    BI_rotr16,
    BI_rotr64,
    BI__builtin_roundf,
    BIroundf,
    BI__builtin_round,
    BIround,
    BI__builtin_roundl,
    BIroundl,
    BI__builtin_roundevenf,
    BIroundevenf,
    BI__builtin_roundeven,
    BIroundeven,
    BI__builtin_roundevenl,
    BIroundevenl,
    BI__builtin_roundf16,
    BI__builtin_roundf128,
    BI__builtin_roundevenf16,
    BI__builtin_roundevenf128,
    BI__builtin___snprintf_chk,
    BI__builtin_sprintf,
    BIsprintf,
    BI__builtin___sprintf_chk,
    BI__builtin_sscanf,
    BIsscanf,
    BI__builtin_sadd_overflow,
    BI__builtin_saddl_overflow,
    BI__builtin_saddll_overflow,
    BIsavectx,
    BI__builtin_scalblnf,
    BIscalblnf,
    BI__builtin_scalbln,
    BIscalbln,
    BI__builtin_scalblnl,
    BIscalblnl,
    BI__builtin_scalbnf,
    BIscalbnf,
    BI__builtin_scalbn,
    BIscalbn,
    BI__builtin_scalbnl,
    BIscalbnl,
    BI__builtin_scanf,
    BIscanf,
    BI__builtin_scalblnf128,
    BI__builtin_scalbnf128,
    BI__scoped_atomic_thread_fence,
    BIsetjmp,
    BI_setjmp,
    BI_setjmpex,
    BI__builtin_shufflevector,
    BIsiglongjmp,
    BIsigsetjmp,
    BI__sigsetjmp,
    BI__builtin_signbit,
    BI__builtin_signbitf,
    BI__builtin_signbitl,
    BI__builtin_sinf,
    BIsinf,
    BI__builtin_sin,
    BIsin,
    BI__builtin_sinl,
    BIsinl,
    BI__builtin_sinf16,
    BI__builtin_sinf128,
    BI__builtin_sincosf,
    BIsincosf,
    BI__builtin_sincos,
    BIsincos,
    BI__builtin_sincosl,
    BIsincosl,
    BI__builtin_sincosf16,
    BI__builtin_sincosf128,
    BI__builtin_sincospif,
    BI__builtin_sincospi,
    BI__builtin_sincospil,
    BI__builtin_sinhf,
    BIsinhf,
    BI__builtin_sinh,
    BIsinh,
    BI__builtin_sinhl,
    BIsinhl,
    BI__builtin_sinhf16,
    BI__builtin_sinhf128,
    BI__sinpif,
    BI__sinpi,
    BI__builtin_smul_overflow,
    BI__builtin_smull_overflow,
    BI__builtin_smulll_overflow,
    BI__builtin_snprintf,
    BIsnprintf,
    BI__builtin_sqrtf,
    BIsqrtf,
    BI__builtin_sqrt,
    BIsqrt,
    BI__builtin_sqrtl,
    BIsqrtl,
    BI__builtin_sqrtf16,
    BI__builtin_sqrtf128,
    BI__builtin_ssub_overflow,
    BI__builtin_ssubl_overflow,
    BI__builtin_ssubll_overflow,
    BI__builtin_stack_address,
    BI__builtin_store_halff,
    BI__builtin_store_half,
    BI__builtin_stpcpy,
    BIstpcpy,
    BI__builtin___stpcpy_chk,
    BI__builtin_stpncpy,
    BIstpncpy,
    BI__builtin___stpncpy_chk,
    BI__builtin_strcasecmp,
    BIstrcasecmp,
    BI__builtin_strcat,
    BIstrcat,
    BI__builtin_strchr,
    BIstrchr,
    BI__builtin_strcmp,
    BIstrcmp,
    BI__builtin_strcpy,
    BIstrcpy,
    BI__builtin_strdup,
    BIstrdup,
    BIstrerror,
    BI__builtin_strlen,
    BIstrlen,
    BI__builtin_strncat,
    BIstrncat,
    BI__builtin_strncmp,
    BIstrncmp,
    BI__builtin_strncpy,
    BIstrncpy,
    BI__builtin_strspn,
    BIstrspn,
    BI__builtin_strstr,
    BIstrstr,
    BIstrtod,
    BIstrtof,
    BIstrtol,
    BIstrtoll,
    BIstrtold,
    BIstrtoul,
    BIstrtoull,
    BIstrtok,
    BIstrcspn,
    BI__builtin___strcat_chk,
    BI__builtin___strcpy_chk,
    BIstrlcat,
    BIstrlcpy,
    BI__builtin___strlcat_chk,
    BI__builtin___strlcpy_chk,
    BI__builtin_strncasecmp,
    BIstrncasecmp,
    BI__builtin_strndup,
    BIstrndup,
    BI__builtin___strncat_chk,
    BI__builtin___strncpy_chk,
    BI__builtin_strpbrk,
    BIstrpbrk,
    BI__builtin_strrchr,
    BIstrrchr,
    BIstrxfrm,
    BIsub_group_commit_read_pipe,
    BIsub_group_commit_write_pipe,
    BIsub_group_reserve_read_pipe,
    BIsub_group_reserve_write_pipe,
    BI__builtin_sub_overflow,
    BI__builtin_subcb,
    BI__builtin_subcs,
    BI__builtin_subc,
    BI__builtin_subcl,
    BI__builtin_subcll,
    BI__sync_lock_test_and_set_1,
    BI__sync_lock_test_and_set_2,
    BI__sync_lock_test_and_set_4,
    BI__sync_lock_test_and_set_8,
    BI__sync_lock_test_and_set_16,
    BI__sync_val_compare_and_swap_1,
    BI__sync_val_compare_and_swap_2,
    BI__sync_val_compare_and_swap_4,
    BI__sync_val_compare_and_swap_8,
    BI__sync_val_compare_and_swap_16,
    BI__sync_add_and_fetch,
    BI__sync_add_and_fetch_1,
    BI__sync_add_and_fetch_2,
    BI__sync_add_and_fetch_4,
    BI__sync_add_and_fetch_8,
    BI__sync_add_and_fetch_16,
    BI__sync_and_and_fetch,
    BI__sync_and_and_fetch_1,
    BI__sync_and_and_fetch_2,
    BI__sync_and_and_fetch_4,
    BI__sync_and_and_fetch_8,
    BI__sync_and_and_fetch_16,
    BI__sync_bool_compare_and_swap,
    BI__sync_bool_compare_and_swap_1,
    BI__sync_bool_compare_and_swap_2,
    BI__sync_bool_compare_and_swap_4,
    BI__sync_bool_compare_and_swap_8,
    BI__sync_bool_compare_and_swap_16,
    BI__sync_fetch_and_add,
    BI__sync_fetch_and_add_1,
    BI__sync_fetch_and_add_2,
    BI__sync_fetch_and_add_4,
    BI__sync_fetch_and_add_8,
    BI__sync_fetch_and_add_16,
    BI__sync_fetch_and_and,
    BI__sync_fetch_and_and_1,
    BI__sync_fetch_and_and_2,
    BI__sync_fetch_and_and_4,
    BI__sync_fetch_and_and_8,
    BI__sync_fetch_and_and_16,
    BI__sync_fetch_and_max,
    BI__sync_fetch_and_min,
    BI__sync_fetch_and_nand,
    BI__sync_fetch_and_nand_1,
    BI__sync_fetch_and_nand_2,
    BI__sync_fetch_and_nand_4,
    BI__sync_fetch_and_nand_8,
    BI__sync_fetch_and_nand_16,
    BI__sync_fetch_and_or,
    BI__sync_fetch_and_or_1,
    BI__sync_fetch_and_or_2,
    BI__sync_fetch_and_or_4,
    BI__sync_fetch_and_or_8,
    BI__sync_fetch_and_or_16,
    BI__sync_fetch_and_sub,
    BI__sync_fetch_and_sub_1,
    BI__sync_fetch_and_sub_2,
    BI__sync_fetch_and_sub_4,
    BI__sync_fetch_and_sub_8,
    BI__sync_fetch_and_sub_16,
    BI__sync_fetch_and_umax,
    BI__sync_fetch_and_umin,
    BI__sync_fetch_and_xor,
    BI__sync_fetch_and_xor_1,
    BI__sync_fetch_and_xor_2,
    BI__sync_fetch_and_xor_4,
    BI__sync_fetch_and_xor_8,
    BI__sync_fetch_and_xor_16,
    BI__sync_lock_release,
    BI__sync_lock_release_1,
    BI__sync_lock_release_2,
    BI__sync_lock_release_4,
    BI__sync_lock_release_8,
    BI__sync_lock_release_16,
    BI__sync_lock_test_and_set,
    BI__sync_nand_and_fetch,
    BI__sync_nand_and_fetch_1,
    BI__sync_nand_and_fetch_2,
    BI__sync_nand_and_fetch_4,
    BI__sync_nand_and_fetch_8,
    BI__sync_nand_and_fetch_16,
    BI__sync_or_and_fetch,
    BI__sync_or_and_fetch_1,
    BI__sync_or_and_fetch_2,
    BI__sync_or_and_fetch_4,
    BI__sync_or_and_fetch_8,
    BI__sync_or_and_fetch_16,
    BI__sync_sub_and_fetch,
    BI__sync_sub_and_fetch_1,
    BI__sync_sub_and_fetch_2,
    BI__sync_sub_and_fetch_4,
    BI__sync_sub_and_fetch_8,
    BI__sync_sub_and_fetch_16,
    BI__sync_swap,
    BI__sync_swap_1,
    BI__sync_swap_2,
    BI__sync_swap_4,
    BI__sync_swap_8,
    BI__sync_swap_16,
    BI__sync_synchronize,
    BI__sync_val_compare_and_swap,
    BI__sync_xor_and_fetch,
    BI__sync_xor_and_fetch_1,
    BI__sync_xor_and_fetch_2,
    BI__sync_xor_and_fetch_4,
    BI__sync_xor_and_fetch_8,
    BI__sync_xor_and_fetch_16,
    BI__builtin_tanf,
    BItanf,
    BI__builtin_tan,
    BItan,
    BI__builtin_tanl,
    BItanl,
    BI__builtin_tanf16,
    BI__builtin_tanf128,
    BI__builtin_tanhf,
    BItanhf,
    BI__builtin_tanh,
    BItanh,
    BI__builtin_tanhl,
    BItanhl,
    BI__builtin_tanhf16,
    BI__builtin_tanhf128,
    BI__tanpif,
    BI__tanpi,
    BI__builtin_tgammaf,
    BItgammaf,
    BI__builtin_tgamma,
    BItgamma,
    BI__builtin_tgammal,
    BItgammal,
    BI__builtin_tgammaf128,
    BI__builtin_thread_pointer,
    BIto_global,
    BIto_local,
    BItolower,
    BIto_private,
    BItoupper,
    BI__builtin_trap,
    BI__builtin_truncf,
    BItruncf,
    BI__builtin_trunc,
    BItrunc,
    BI__builtin_truncl,
    BItruncl,
    BI__builtin_truncf16,
    BI__builtin_truncf128,
    BI__builtin_uadd_overflow,
    BI__builtin_uaddl_overflow,
    BI__builtin_uaddll_overflow,
    BI__builtin_umul_overflow,
    BI__builtin_umull_overflow,
    BI__builtin_umulll_overflow,
    BI__builtin_unpredictable,
    BI__builtin_unreachable,
    BI__builtin_unwind_init,
    BI__builtin_usub_overflow,
    BI__builtin_usubl_overflow,
    BI__builtin_usubll_overflow,
    BI__builtin___vfprintf_chk,
    BI__builtin_vfscanf,
    BIvfscanf,
    BIvfork,
    BI__builtin_vprintf,
    BIvprintf,
    BI__builtin___vprintf_chk,
    BI__builtin___vsnprintf_chk,
    BI__builtin___vsprintf_chk,
    BI__builtin_vsscanf,
    BIvsscanf,
    BI__builtin_vscanf,
    BIvscanf,
    BI__builtin_va_copy,
    BIva_copy,
    BI__builtin_va_end,
    BIva_end,
    BIva_start,
    BI__builtin_verbose_trap,
    BI__builtin_vfprintf,
    BIvfprintf,
    BI__builtin_vsprintf,
    BIvsprintf,
    BI__builtin_vsnprintf,
    BIvsnprintf,
    BI__builtin_wmemchr,
    BIwmemchr,
    BI__builtin_wmemcmp,
    BIwmemcmp,
    BI__builtin_wmemcpy,
    BIwmemcpy,
    BI__builtin_wmemmove,
    BIwmemmove,
    BI__warn_memset_zero_len,
    BI__builtin_wcschr,
    BIwcschr,
    BI__builtin_wcscmp,
    BIwcscmp,
    BI__builtin_wcslen,
    BIwcslen,
    BI__builtin_wcsncmp,
    BIwcsncmp,
    BIwork_group_commit_read_pipe,
    BIwork_group_commit_write_pipe,
    BIwork_group_reserve_read_pipe,
    BIwork_group_reserve_write_pipe,
    BIwrite_pipe,
    BI__xray_customevent,
    BI__xray_typedevent,
    BI__addressof,

#endif // GET_BUILTIN_ENUMERATORS

#ifdef GET_BUILTIN_STR_TABLE

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinStringsStorage[] =
  "\0"
  "__atomic_add_fetch\0"
  "v.\0"
  "t\0"
  "__atomic_and_fetch\0"
  "__atomic_clear\0"
  "nt\0"
  "__atomic_compare_exchange\0"
  "__atomic_compare_exchange_n\0"
  "__atomic_exchange\0"
  "__atomic_exchange_n\0"
  "__atomic_fetch_add\0"
  "__atomic_fetch_and\0"
  "__atomic_fetch_max\0"
  "__atomic_fetch_min\0"
  "__atomic_fetch_nand\0"
  "__atomic_fetch_or\0"
  "__atomic_fetch_sub\0"
  "__atomic_fetch_udec\0"
  "__atomic_fetch_uinc\0"
  "__atomic_fetch_xor\0"
  "__atomic_load\0"
  "__atomic_load_n\0"
  "__atomic_max_fetch\0"
  "__atomic_min_fetch\0"
  "__atomic_nand_fetch\0"
  "__atomic_or_fetch\0"
  "__atomic_store\0"
  "__atomic_store_n\0"
  "__atomic_sub_fetch\0"
  "__atomic_test_and_set\0"
  "b.\0"
  "__atomic_xor_fetch\0"
  "__c11_atomic_compare_exchange_strong\0"
  "__c11_atomic_compare_exchange_weak\0"
  "__c11_atomic_exchange\0"
  "__c11_atomic_fetch_add\0"
  "__c11_atomic_fetch_and\0"
  "__c11_atomic_fetch_max\0"
  "__c11_atomic_fetch_min\0"
  "__c11_atomic_fetch_nand\0"
  "__c11_atomic_fetch_or\0"
  "__c11_atomic_fetch_sub\0"
  "__c11_atomic_fetch_xor\0"
  "__c11_atomic_init\0"
  "__c11_atomic_load\0"
  "__c11_atomic_store\0"
  "__hip_atomic_compare_exchange_strong\0"
  "__hip_atomic_compare_exchange_weak\0"
  "__hip_atomic_exchange\0"
  "__hip_atomic_fetch_add\0"
  "__hip_atomic_fetch_and\0"
  "__hip_atomic_fetch_max\0"
  "__hip_atomic_fetch_min\0"
  "__hip_atomic_fetch_or\0"
  "__hip_atomic_fetch_sub\0"
  "__hip_atomic_fetch_xor\0"
  "__hip_atomic_load\0"
  "__hip_atomic_store\0"
  "__opencl_atomic_compare_exchange_strong\0"
  "__opencl_atomic_compare_exchange_weak\0"
  "__opencl_atomic_exchange\0"
  "__opencl_atomic_fetch_add\0"
  "__opencl_atomic_fetch_and\0"
  "__opencl_atomic_fetch_max\0"
  "__opencl_atomic_fetch_min\0"
  "__opencl_atomic_fetch_or\0"
  "__opencl_atomic_fetch_sub\0"
  "__opencl_atomic_fetch_xor\0"
  "__opencl_atomic_init\0"
  "__opencl_atomic_load\0"
  "__opencl_atomic_store\0"
  "__scoped_atomic_add_fetch\0"
  "__scoped_atomic_and_fetch\0"
  "__scoped_atomic_compare_exchange\0"
  "__scoped_atomic_compare_exchange_n\0"
  "__scoped_atomic_exchange\0"
  "__scoped_atomic_exchange_n\0"
  "__scoped_atomic_fetch_add\0"
  "__scoped_atomic_fetch_and\0"
  "__scoped_atomic_fetch_max\0"
  "__scoped_atomic_fetch_min\0"
  "__scoped_atomic_fetch_nand\0"
  "__scoped_atomic_fetch_or\0"
  "__scoped_atomic_fetch_sub\0"
  "__scoped_atomic_fetch_xor\0"
  "__scoped_atomic_load\0"
  "__scoped_atomic_load_n\0"
  "__scoped_atomic_max_fetch\0"
  "__scoped_atomic_min_fetch\0"
  "__scoped_atomic_nand_fetch\0"
  "__scoped_atomic_or_fetch\0"
  "__scoped_atomic_store\0"
  "__scoped_atomic_store_n\0"
  "__scoped_atomic_sub_fetch\0"
  "__scoped_atomic_fetch_udec\0"
  "__scoped_atomic_fetch_uinc\0"
  "__scoped_atomic_xor_fetch\0"
  "__abnormal_termination\0"
  "i\0"
  "n\0"
  "_abnormal_termination\0"
  "__builtin_abort\0"
  "v\0"
  "Fnr\0"
  "abort\0"
  "fnr\0"
  "__builtin_abs\0"
  "ii\0"
  "FEnc\0"
  "abs\0"
  "fnc\0"
  "__builtin_labs\0"
  "LiLi\0"
  "labs\0"
  "__builtin_llabs\0"
  "LLiLLi\0"
  "llabs\0"
  "__builtin_acosf\0"
  "ff\0"
  "Fne\0"
  "acosf\0"
  "fne\0"
  "__builtin_acos\0"
  "dd\0"
  "acos\0"
  "__builtin_acosl\0"
  "LdLd\0"
  "acosl\0"
  "__builtin_acosf16\0"
  "hh\0"
  "__builtin_acosf128\0"
  "LLdLLd\0"
  "__builtin_acoshf\0"
  "acoshf\0"
  "__builtin_acosh\0"
  "acosh\0"
  "__builtin_acoshl\0"
  "acoshl\0"
  "__builtin_acoshf128\0"
  "__builtin_add_overflow\0"
  "ntE\0"
  "__builtin_addcb\0"
  "UcUcCUcCUcCUc*\0"
  "nE\0"
  "__builtin_addcs\0"
  "UsUsCUsCUsCUs*\0"
  "__builtin_addc\0"
  "UiUiCUiCUiCUi*\0"
  "__builtin_addcl\0"
  "ULiULiCULiCULiCULi*\0"
  "__builtin_addcll\0"
  "ULLiULLiCULLiCULLiCULLi*\0"
  "addressof\0"
  "v*v&\0"
  "fzncThE\0"
  "__builtin_align_down\0"
  "v*vC*z\0"
  "nctE\0"
  "__builtin_align_up\0"
  "aligned_alloc\0"
  "v*zz\0"
  "f\0"
  "__builtin_alloca\0"
  "v*z\0"
  "Fn\0"
  "alloca\0"
  "fn\0"
  "_alloca\0"
  "__builtin_alloca_uninitialized\0"
  "__builtin_alloca_with_align\0"
  "v*zIz\0"
  "__builtin_alloca_with_align_uninitialized\0"
  "__builtin_allow_runtime_check\0"
  "bcC*\0"
  "nUc\0"
  "__builtin_annotation\0"
  "__arithmetic_fence\0"
  "tE\0"
  "as_const\0"
  "v&v&\0"
  "__builtin_asinf\0"
  "asinf\0"
  "__builtin_asin\0"
  "asin\0"
  "__builtin_asinl\0"
  "asinl\0"
  "__builtin_asinf16\0"
  "__builtin_asinf128\0"
  "__builtin_asinhf\0"
  "asinhf\0"
  "__builtin_asinh\0"
  "asinh\0"
  "__builtin_asinhl\0"
  "asinhl\0"
  "__builtin_asinhf128\0"
  "__builtin_assume\0"
  "vb\0"
  "__builtin_assume_separate_storage\0"
  "vvCD*vCD*\0"
  "__builtin_atanf\0"
  "atanf\0"
  "__builtin_atan\0"
  "atan\0"
  "__builtin_atanl\0"
  "atanl\0"
  "__builtin_atan2f\0"
  "fff\0"
  "atan2f\0"
  "__builtin_atan2\0"
  "ddd\0"
  "atan2\0"
  "__builtin_atan2l\0"
  "LdLdLd\0"
  "atan2l\0"
  "__builtin_atan2f16\0"
  "hhh\0"
  "__builtin_atan2f128\0"
  "LLdLLdLLd\0"
  "__builtin_atanf16\0"
  "__builtin_atanf128\0"
  "__builtin_atanhf\0"
  "atanhf\0"
  "__builtin_atanh\0"
  "atanh\0"
  "__builtin_atanhl\0"
  "atanhl\0"
  "__builtin_atanhf128\0"
  "__atomic_always_lock_free\0"
  "bzvCD*\0"
  "__atomic_is_lock_free\0"
  "__atomic_signal_fence\0"
  "vi\0"
  "__atomic_thread_fence\0"
  "bcmp\0"
  "ivC*vC*z\0"
  "fE\0"
  "__builtin_bswap16\0"
  "UsUs\0"
  "ncE\0"
  "__builtin_bswap32\0"
  "UZiUZi\0"
  "__builtin_bswap64\0"
  "UWiUWi\0"
  "__builtin_bswapg\0"
  "i.\0"
  "ncEt\0"
  "__builtin_bzero\0"
  "vv*z\0"
  "bzero\0"
  "__builtin_bcopy\0"
  "vvC*v*z\0"
  "bcopy\0"
  "__builtin_bitreverse8\0"
  "UcUc\0"
  "__builtin_bitreverse16\0"
  "__builtin_bitreverse32\0"
  "__builtin_bitreverse64\0"
  "_bittest\0"
  "UcNiC*Ni\0"
  "_bittest64\0"
  "UcWiC*Wi\0"
  "_bittestandcomplement\0"
  "UcNi*Ni\0"
  "_bittestandcomplement64\0"
  "UcWi*Wi\0"
  "_bittestandreset\0"
  "_bittestandreset64\0"
  "_bittestandset\0"
  "_bittestandset64\0"
  "_Block_object_assign\0"
  "vv*vC*iC\0"
  "_Block_object_dispose\0"
  "vvC*iC\0"
  "__builtin_addressof\0"
  "__builtin_allow_sanitize_check\0"
  "nct\0"
  "__builtin_assume_aligned\0"
  "v*vC*z.\0"
  "__builtin_assume_dereferenceable\0"
  "vvC*z\0"
  "nc\0"
  "__builtin_bcmp\0"
  "FnE\0"
  "__builtin_c23_va_start\0"
  "vA.\0"
  "__builtin___CFStringMakeConstantString\0"
  "FC*cC*\0"
  "__builtin_cpu_init\0"
  "__builtin_cpu_is\0"
  "__builtin_cpu_supports\0"
  "__builtin_calloc\0"
  "__builtin_char_memchr\0"
  "c*cC*iz\0"
  "__builtin_classify_type\0"
  "nctuE\0"
  "__builtin_complex\0"
  "__builtin_constant_p\0"
  "__builtin_dump_struct\0"
  "__builtin_flt_rounds\0"
  "__builtin_free\0"
  "vv*\0"
  "__builtin_function_start\0"
  "__builtin_longjmp\0"
  "vv**i\0"
  "r\0"
  "__builtin_malloc\0"
  "__builtin_memcmp\0"
  "__builtin_memcpy_inline\0"
  "vv*vC*Iz\0"
  "__builtin_mempcpy\0"
  "v*v*vC*z\0"
  "__builtin_memset_inline\0"
  "vv*iIz\0"
  "__builtin___NSStringMakeConstantString\0"
  "__builtin_operator_delete\0"
  "__builtin_operator_new\0"
  "ctE\0"
  "__builtin_preserve_access_index\0"
  "__builtin_printf\0"
  "icC*R.\0"
  "np:0:FN:0:<0>\0"
  "__builtin_realloc\0"
  "v*v*z\0"
  "__builtin_return_address\0"
  "v*IUi\0"
  "__builtin_set_flt_rounds\0"
  "__builtin_setjmp\0"
  "iv**\0"
  "j\0"
  "__builtin_stdarg_start\0"
  "__builtin_strcspn\0"
  "zcC*cC*\0"
  "__builtin_trivially_relocate\0"
  "v*v*v*z\0"
  "Ftn\0"
  "__builtin_va_start\0"
  "__c11_atomic_is_lock_free\0"
  "bz\0"
  "__c11_atomic_signal_fence\0"
  "__c11_atomic_thread_fence\0"
  "__builtin_cabsf\0"
  "fXf\0"
  "cabsf\0"
  "__builtin_cabs\0"
  "dXd\0"
  "cabs\0"
  "__builtin_cabsl\0"
  "LdXLd\0"
  "cabsl\0"
  "__builtin_cacosf\0"
  "XfXf\0"
  "cacosf\0"
  "__builtin_cacos\0"
  "XdXd\0"
  "cacos\0"
  "__builtin_cacosl\0"
  "XLdXLd\0"
  "cacosl\0"
  "__builtin_cacoshf\0"
  "cacoshf\0"
  "__builtin_cacosh\0"
  "cacosh\0"
  "__builtin_cacoshl\0"
  "cacoshl\0"
  "__builtin_call_with_static_chain\0"
  "calloc\0"
  "__builtin_canonicalizef\0"
  "__builtin_canonicalize\0"
  "__builtin_canonicalizel\0"
  "__builtin_canonicalizef16\0"
  "__builtin_cargf\0"
  "cargf\0"
  "__builtin_carg\0"
  "carg\0"
  "__builtin_cargl\0"
  "cargl\0"
  "__builtin_casinf\0"
  "casinf\0"
  "__builtin_casin\0"
  "casin\0"
  "__builtin_casinl\0"
  "casinl\0"
  "__builtin_casinhf\0"
  "casinhf\0"
  "__builtin_casinh\0"
  "casinh\0"
  "__builtin_casinhl\0"
  "casinhl\0"
  "__builtin_catanf\0"
  "catanf\0"
  "__builtin_catan\0"
  "catan\0"
  "__builtin_catanl\0"
  "catanl\0"
  "__builtin_catanhf\0"
  "catanhf\0"
  "__builtin_catanh\0"
  "catanh\0"
  "__builtin_catanhl\0"
  "catanhl\0"
  "__builtin_cbrtf\0"
  "Fnc\0"
  "cbrtf\0"
  "__builtin_cbrt\0"
  "cbrt\0"
  "__builtin_cbrtl\0"
  "cbrtl\0"
  "__builtin_cbrtf128\0"
  "__builtin_ccosf\0"
  "ccosf\0"
  "__builtin_ccos\0"
  "ccos\0"
  "__builtin_ccosl\0"
  "ccosl\0"
  "__builtin_ccoshf\0"
  "ccoshf\0"
  "__builtin_ccosh\0"
  "ccosh\0"
  "__builtin_ccoshl\0"
  "ccoshl\0"
  "__builtin_ceilf\0"
  "ceilf\0"
  "__builtin_ceil\0"
  "ceil\0"
  "__builtin_ceill\0"
  "ceill\0"
  "__builtin_ceilf16\0"
  "__builtin_ceilf128\0"
  "__builtin_cexpf\0"
  "cexpf\0"
  "__builtin_cexp\0"
  "cexp\0"
  "__builtin_cexpl\0"
  "cexpl\0"
  "__builtin_cimagf\0"
  "cimagf\0"
  "__builtin_cimag\0"
  "cimag\0"
  "__builtin_cimagl\0"
  "cimagl\0"
  "__builtin___clear_cache\0"
  "vv*v*\0"
  "__builtin_clogf\0"
  "clogf\0"
  "__builtin_clog\0"
  "clog\0"
  "__builtin_clogl\0"
  "clogl\0"
  "__builtin_clrsb\0"
  "__builtin_clrsbl\0"
  "iLi\0"
  "__builtin_clrsbll\0"
  "iLLi\0"
  "__builtin_clzs\0"
  "iUs\0"
  "__builtin_clz\0"
  "iUi\0"
  "__builtin_clzl\0"
  "iULi\0"
  "__builtin_clzll\0"
  "iULLi\0"
  "__builtin_clzg\0"
  "commit_read_pipe\0"
  "tn\0"
  "commit_write_pipe\0"
  "__builtin_conjf\0"
  "conjf\0"
  "__builtin_conj\0"
  "conj\0"
  "__builtin_conjl\0"
  "conjl\0"
  "__builtin_convertvector\0"
  "__builtin_copysignf\0"
  "copysignf\0"
  "__builtin_copysign\0"
  "copysign\0"
  "__builtin_copysignl\0"
  "copysignl\0"
  "__builtin_copysignf16\0"
  "__builtin_copysignf128\0"
  "FncE\0"
  "__builtin_coro_align\0"
  "z\0"
  "__builtin_coro_alloc\0"
  "b\0"
  "__builtin_coro_begin\0"
  "v*v*\0"
  "__builtin_coro_destroy\0"
  "__builtin_coro_done\0"
  "bv*\0"
  "__builtin_coro_end\0"
  "bv*Ib\0"
  "__builtin_coro_frame\0"
  "v*\0"
  "__builtin_coro_free\0"
  "__builtin_coro_id\0"
  "v*Iiv*v*v*\0"
  "__builtin_coro_noop\0"
  "__builtin_coro_promise\0"
  "v*v*IiIb\0"
  "__builtin_coro_resume\0"
  "__builtin_coro_size\0"
  "__builtin_coro_suspend\0"
  "cIb\0"
  "__builtin_cosf\0"
  "cosf\0"
  "__builtin_cos\0"
  "cos\0"
  "__builtin_cosl\0"
  "cosl\0"
  "__builtin_cosf16\0"
  "__builtin_cosf128\0"
  "__builtin_coshf\0"
  "coshf\0"
  "__builtin_cosh\0"
  "cosh\0"
  "__builtin_coshl\0"
  "coshl\0"
  "__builtin_coshf16\0"
  "__builtin_coshf128\0"
  "__cospif\0"
  "__cospi\0"
  "__builtin_counted_by_ref\0"
  "__builtin_cpowf\0"
  "XfXfXf\0"
  "cpowf\0"
  "__builtin_cpow\0"
  "XdXdXd\0"
  "cpow\0"
  "__builtin_cpowl\0"
  "XLdXLdXLd\0"
  "cpowl\0"
  "__builtin_cprojf\0"
  "cprojf\0"
  "__builtin_cproj\0"
  "cproj\0"
  "__builtin_cprojl\0"
  "cprojl\0"
  "__builtin_crealf\0"
  "crealf\0"
  "__builtin_creal\0"
  "creal\0"
  "__builtin_creall\0"
  "creall\0"
  "__builtin_csinf\0"
  "csinf\0"
  "__builtin_csin\0"
  "csin\0"
  "__builtin_csinl\0"
  "csinl\0"
  "__builtin_csinhf\0"
  "csinhf\0"
  "__builtin_csinh\0"
  "csinh\0"
  "__builtin_csinhl\0"
  "csinhl\0"
  "__builtin_csqrtf\0"
  "csqrtf\0"
  "__builtin_csqrt\0"
  "csqrt\0"
  "__builtin_csqrtl\0"
  "csqrtl\0"
  "__builtin_ctanf\0"
  "ctanf\0"
  "__builtin_ctan\0"
  "ctan\0"
  "__builtin_ctanl\0"
  "ctanl\0"
  "__builtin_ctanhf\0"
  "ctanhf\0"
  "__builtin_ctanh\0"
  "ctanh\0"
  "__builtin_ctanhl\0"
  "ctanhl\0"
  "__builtin_ctzs\0"
  "__builtin_ctz\0"
  "__builtin_ctzl\0"
  "__builtin_ctzll\0"
  "__builtin_ctzg\0"
  "__builtin_dwarf_sp_column\0"
  "Ui\0"
  "__builtin_dwarf_cfa\0"
  "__debugbreak\0"
  "__builtin_debugtrap\0"
  "__builtin_dynamic_object_size\0"
  "zvC*i\0"
  "nuE\0"
  "__builtin_eh_return\0"
  "vzv*\0"
  "__builtin_eh_return_data_regno\0"
  "iIi\0"
  "__builtin_elementwise_acos\0"
  "__builtin_elementwise_asin\0"
  "__builtin_elementwise_atan\0"
  "__builtin_elementwise_atan2\0"
  "__builtin_elementwise_abs\0"
  "__builtin_elementwise_add_sat\0"
  "__builtin_elementwise_bitreverse\0"
  "__builtin_elementwise_canonicalize\0"
  "__builtin_elementwise_ceil\0"
  "__builtin_elementwise_copysign\0"
  "__builtin_elementwise_cos\0"
  "__builtin_elementwise_cosh\0"
  "__builtin_elementwise_clzg\0"
  "__builtin_elementwise_ctzg\0"
  "__builtin_elementwise_exp\0"
  "__builtin_elementwise_exp2\0"
  "__builtin_elementwise_exp10\0"
  "__builtin_elementwise_floor\0"
  "__builtin_elementwise_fma\0"
  "__builtin_elementwise_fmod\0"
  "__builtin_elementwise_fshl\0"
  "__builtin_elementwise_fshr\0"
  "__builtin_elementwise_ldexp\0"
  "__builtin_elementwise_log\0"
  "__builtin_elementwise_log2\0"
  "__builtin_elementwise_log10\0"
  "__builtin_elementwise_max\0"
  "__builtin_elementwise_maxnum\0"
  "__builtin_elementwise_maximum\0"
  "__builtin_elementwise_maximumnum\0"
  "__builtin_elementwise_min\0"
  "__builtin_elementwise_minnum\0"
  "__builtin_elementwise_minimum\0"
  "__builtin_elementwise_minimumnum\0"
  "__builtin_elementwise_nearbyint\0"
  "__builtin_elementwise_popcount\0"
  "__builtin_elementwise_pow\0"
  "__builtin_elementwise_rint\0"
  "__builtin_elementwise_round\0"
  "__builtin_elementwise_roundeven\0"
  "__builtin_elementwise_sin\0"
  "__builtin_elementwise_sinh\0"
  "__builtin_elementwise_sqrt\0"
  "__builtin_elementwise_sub_sat\0"
  "__builtin_elementwise_tan\0"
  "__builtin_elementwise_tanh\0"
  "__builtin_elementwise_trunc\0"
  "enqueue_kernel\0"
  "__builtin_erff\0"
  "erff\0"
  "__builtin_erf\0"
  "erf\0"
  "__builtin_erfl\0"
  "erfl\0"
  "__builtin_erff128\0"
  "__builtin_erfcf\0"
  "erfcf\0"
  "__builtin_erfc\0"
  "erfc\0"
  "__builtin_erfcl\0"
  "erfcl\0"
  "__builtin_erfcf128\0"
  "__exception_code\0"
  "UNi\0"
  "_exception_code\0"
  "__exception_info\0"
  "_exception_info\0"
  "exit\0"
  "fr\0"
  "_Exit\0"
  "__builtin_expf\0"
  "expf\0"
  "__builtin_exp\0"
  "exp\0"
  "__builtin_expl\0"
  "expl\0"
  "__builtin_exp2f\0"
  "exp2f\0"
  "__builtin_exp2\0"
  "exp2\0"
  "__builtin_exp2l\0"
  "exp2l\0"
  "__builtin_exp2f16\0"
  "__builtin_exp2f128\0"
  "__builtin_exp10f\0"
  "__builtin_exp10\0"
  "__builtin_exp10l\0"
  "__builtin_exp10f16\0"
  "__builtin_exp10f128\0"
  "__builtin_expf16\0"
  "__builtin_expf128\0"
  "__builtin_expect\0"
  "LiLiLi\0"
  "__builtin_expect_with_probability\0"
  "LiLiLid\0"
  "__builtin_expm1f\0"
  "expm1f\0"
  "__builtin_expm1\0"
  "expm1\0"
  "__builtin_expm1l\0"
  "expm1l\0"
  "__builtin_expm1f128\0"
  "__builtin_extend_pointer\0"
  "ULLiv*\0"
  "__builtin_extract_return_addr\0"
  "__builtin_ffs\0"
  "__builtin_ffsl\0"
  "__builtin_ffsll\0"
  "__builtin_fpclassify\0"
  "iiiiii.\0"
  "FnctE\0"
  "__builtin_fprintf\0"
  "iP*RcC*R.\0"
  "Fnp:1:N:0:<0,1>\0"
  "fprintf\0"
  "fnp:1:N:0:<0,1>\0"
  "__builtin___fprintf_chk\0"
  "iP*RicC*R.\0"
  "Fp:2:\0"
  "__builtin_fscanf\0"
  "Fs:1:N:0:<0,1>\0"
  "fscanf\0"
  "fs:1:N:0:<0,1>\0"
  "__builtin_fabsf\0"
  "fabsf\0"
  "__builtin_fabs\0"
  "fabs\0"
  "__builtin_fabsl\0"
  "fabsl\0"
  "__builtin_fabsf16\0"
  "__builtin_fabsf128\0"
  "__fastfail\0"
  "vUi\0"
  "nr\0"
  "__builtin_fdimf\0"
  "fdimf\0"
  "__builtin_fdim\0"
  "fdim\0"
  "__builtin_fdiml\0"
  "fdiml\0"
  "__builtin_fdimf128\0"
  "finitef\0"
  "if\0"
  "finite\0"
  "id\0"
  "finitel\0"
  "iLd\0"
  "__builtin_floorf\0"
  "floorf\0"
  "__builtin_floor\0"
  "floor\0"
  "__builtin_floorl\0"
  "floorl\0"
  "__builtin_floorf16\0"
  "__builtin_floorf128\0"
  "__builtin_fmaf\0"
  "ffff\0"
  "fmaf\0"
  "__builtin_fma\0"
  "dddd\0"
  "fma\0"
  "__builtin_fmal\0"
  "LdLdLdLd\0"
  "fmal\0"
  "__builtin_fmaf16\0"
  "hhhh\0"
  "__builtin_fmaf128\0"
  "LLdLLdLLdLLd\0"
  "__builtin_fmaxf\0"
  "fmaxf\0"
  "__builtin_fmax\0"
  "fmax\0"
  "__builtin_fmaxl\0"
  "fmaxl\0"
  "__builtin_fmaxf16\0"
  "__builtin_fmaxf128\0"
  "__builtin_fmaximum_numf\0"
  "fmaximum_numf\0"
  "__builtin_fmaximum_num\0"
  "fmaximum_num\0"
  "__builtin_fmaximum_numl\0"
  "fmaximum_numl\0"
  "__builtin_fmaximum_numf16\0"
  "__builtin_fmaximum_numf128\0"
  "__builtin_fminf\0"
  "fminf\0"
  "__builtin_fmin\0"
  "fmin\0"
  "__builtin_fminl\0"
  "fminl\0"
  "__builtin_fminf16\0"
  "__builtin_fminf128\0"
  "__builtin_fminimum_numf\0"
  "fminimum_numf\0"
  "__builtin_fminimum_num\0"
  "fminimum_num\0"
  "__builtin_fminimum_numl\0"
  "fminimum_numl\0"
  "__builtin_fminimum_numf16\0"
  "__builtin_fminimum_numf128\0"
  "__builtin_fmodf\0"
  "fmodf\0"
  "__builtin_fmod\0"
  "fmod\0"
  "__builtin_fmodl\0"
  "fmodl\0"
  "__builtin_fmodf16\0"
  "__builtin_fmodf128\0"
  "fopen\0"
  "P*cC*cC*\0"
  "forward\0"
  "forward_like\0"
  "__builtin_frame_address\0"
  "fread\0"
  "zv*zzP*\0"
  "free\0"
  "__builtin_frexpf\0"
  "ffi*\0"
  "frexpf\0"
  "__builtin_frexp\0"
  "ddi*\0"
  "frexp\0"
  "__builtin_frexpl\0"
  "LdLdi*\0"
  "frexpl\0"
  "__builtin_frexpf16\0"
  "hhi*\0"
  "__builtin_frexpf128\0"
  "LLdLLdi*\0"
  "__builtin_frob_return_addr\0"
  "fwrite\0"
  "zvC*zzP*\0"
  "_exit\0"
  "_longjmp\0"
  "vJi\0"
  "frT\0"
  "getcontext\0"
  "iK*\0"
  "fjT\0"
  "__builtin_get_device_side_mangled_name\0"
  "cC*.\0"
  "ncT\0"
  "__GetExceptionInfo\0"
  "v*.\0"
  "zntu\0"
  "get_kernel_max_sub_group_size_for_ndrange\0"
  "Ui.\0"
  "get_kernel_preferred_work_group_size_multiple\0"
  "get_kernel_sub_group_count_for_ndrange\0"
  "get_kernel_work_group_size\0"
  "get_pipe_max_packets\0"
  "get_pipe_num_packets\0"
  "__builtin___get_unsafe_stack_bottom\0"
  "nF\0"
  "__builtin___get_unsafe_stack_ptr\0"
  "__builtin___get_unsafe_stack_start\0"
  "__builtin___get_unsafe_stack_top\0"
  "__builtin_get_vtable_pointer\0"
  "tnc\0"
  "__builtin_hlsl_adduint64\0"
  "__builtin_hlsl_all\0"
  "__builtin_hlsl_and\0"
  "__builtin_hlsl_any\0"
  "__builtin_hlsl_asdouble\0"
  "__builtin_hlsl_buffer_update_counter\0"
  "UZiQri\0"
  "__builtin_hlsl_elementwise_clamp\0"
  "__builtin_hlsl_elementwise_clip\0"
  "__builtin_hlsl_crossf32\0"
  "E3fE3fE3f\0"
  "__builtin_hlsl_crossf16\0"
  "E3hE3hE3h\0"
  "__builtin_hlsl_elementwise_ddx_coarse\0"
  "__builtin_hlsl_elementwise_ddx_fine\0"
  "__builtin_hlsl_elementwise_ddy_coarse\0"
  "__builtin_hlsl_elementwise_ddy_fine\0"
  "__builtin_hlsl_elementwise_degrees\0"
  "__builtin_hlsl_dot4add_i8packed\0"
  "iUiUii\0"
  "__builtin_hlsl_dot4add_u8packed\0"
  "UiUiUiUi\0"
  "__builtin_hlsl_dot\0"
  "__builtin_hlsl_elementwise_f16tof32\0"
  "__builtin_hlsl_elementwise_firstbithigh\0"
  "__builtin_hlsl_elementwise_firstbitlow\0"
  "__builtin_hlsl_elementwise_frac\0"
  "__builtin_get_spirv_spec_constant_bool\0"
  "bUib\0"
  "ncU\0"
  "__builtin_get_spirv_spec_constant_char\0"
  "cUic\0"
  "__builtin_get_spirv_spec_constant_short\0"
  "sUis\0"
  "__builtin_get_spirv_spec_constant_int\0"
  "iUii\0"
  "__builtin_get_spirv_spec_constant_longlong\0"
  "LLiUiLLi\0"
  "__builtin_get_spirv_spec_constant_ushort\0"
  "UsUiUs\0"
  "__builtin_get_spirv_spec_constant_uint\0"
  "UiUiUi\0"
  "__builtin_get_spirv_spec_constant_ulonglong\0"
  "ULLiUiULLi\0"
  "__builtin_get_spirv_spec_constant_half\0"
  "hUih\0"
  "__builtin_get_spirv_spec_constant_float\0"
  "fUif\0"
  "__builtin_get_spirv_spec_constant_double\0"
  "dUid\0"
  "__builtin_hlsl_group_memory_barrier_with_group_sync\0"
  "__builtin_hlsl_elementwise_isinf\0"
  "__builtin_hlsl_elementwise_isnan\0"
  "__builtin_hlsl_lerp\0"
  "__builtin_hlsl_mad\0"
  "__builtin_hlsl_normalize\0"
  "__builtin_hlsl_or\0"
  "__builtin_hlsl_elementwise_rsqrt\0"
  "__builtin_hlsl_elementwise_radians\0"
  "__builtin_hlsl_elementwise_rcp\0"
  "__builtin_hlsl_resource_counterhandlefromimplicitbinding\0"
  "QrQrUZiUZi\0"
  "__builtin_hlsl_resource_getdimensions_x\0"
  "vQrUZi&\0"
  "__builtin_hlsl_resource_getpointer\0"
  "__builtin_hlsl_resource_getstride\0"
  "__builtin_hlsl_resource_handlefrombinding\0"
  "QrQrUZiUZiZiUZicC*\0"
  "__builtin_hlsl_resource_handlefromimplicitbinding\0"
  "__builtin_hlsl_resource_load_with_status\0"
  "__builtin_hlsl_resource_nonuniformindex\0"
  "__builtin_hlsl_resource_uninitializedhandle\0"
  "QrQr\0"
  "__builtin_hlsl_elementwise_saturate\0"
  "__builtin_hlsl_select\0"
  "__builtin_hlsl_elementwise_sign\0"
  "__builtin_hlsl_elementwise_splitdouble\0"
  "__builtin_hlsl_step\0"
  "__builtin_hlsl_wave_active_all_true\0"
  "bb\0"
  "__builtin_hlsl_wave_active_any_true\0"
  "__builtin_hlsl_wave_active_ballot\0"
  "E4Uib\0"
  "__builtin_hlsl_wave_active_count_bits\0"
  "Uib\0"
  "__builtin_hlsl_wave_active_max\0"
  "__builtin_hlsl_wave_active_min\0"
  "__builtin_hlsl_wave_active_sum\0"
  "__builtin_hlsl_wave_get_lane_count\0"
  "__builtin_hlsl_wave_get_lane_index\0"
  "__builtin_hlsl_wave_is_first_lane\0"
  "__builtin_hlsl_wave_read_lane_at\0"
  "__builtin_huge_valf\0"
  "__builtin_huge_val\0"
  "d\0"
  "__builtin_huge_vall\0"
  "Ld\0"
  "__builtin_huge_valf128\0"
  "LLd\0"
  "__builtin_huge_valf16\0"
  "x\0"
  "__builtin_hypotf\0"
  "hypotf\0"
  "__builtin_hypot\0"
  "hypot\0"
  "__builtin_hypotl\0"
  "hypotl\0"
  "__builtin_hypotf128\0"
  "__builtin_ilogbf128\0"
  "iLLd\0"
  "__builtin_ilogbf\0"
  "ilogbf\0"
  "__builtin_ilogb\0"
  "ilogb\0"
  "__builtin_ilogbl\0"
  "ilogbl\0"
  "__builtin_index\0"
  "c*cC*i\0"
  "index\0"
  "__builtin_inff\0"
  "__builtin_inf\0"
  "__builtin_infl\0"
  "__builtin_inff128\0"
  "__builtin_inff16\0"
  "__builtin_infer_alloc_token\0"
  "z.\0"
  "ncUtEu\0"
  "__builtin_init_dwarf_reg_size_table\0"
  "_InterlockedAnd8\0"
  "ccD*c\0"
  "_InterlockedAnd16\0"
  "ssD*s\0"
  "_InterlockedAnd\0"
  "NiNiD*Ni\0"
  "_interlockedbittestandreset\0"
  "UcNiD*Ni\0"
  "_interlockedbittestandreset64\0"
  "UcWiD*Wi\0"
  "_interlockedbittestandreset64_acq\0"
  "_interlockedbittestandreset64_nf\0"
  "_interlockedbittestandreset64_rel\0"
  "_interlockedbittestandreset_acq\0"
  "_interlockedbittestandreset_nf\0"
  "_interlockedbittestandreset_rel\0"
  "_interlockedbittestandset\0"
  "_interlockedbittestandset64\0"
  "_interlockedbittestandset64_acq\0"
  "_interlockedbittestandset64_nf\0"
  "_interlockedbittestandset64_rel\0"
  "_interlockedbittestandset_acq\0"
  "_interlockedbittestandset_nf\0"
  "_interlockedbittestandset_rel\0"
  "_InterlockedCompareExchange8\0"
  "ccD*cc\0"
  "_InterlockedCompareExchange16\0"
  "ssD*ss\0"
  "_InterlockedCompareExchange\0"
  "NiNiD*NiNi\0"
  "_InterlockedCompareExchange64\0"
  "LLiLLiD*LLiLLi\0"
  "_InterlockedCompareExchangePointer\0"
  "v*v*D*v*v*\0"
  "_InterlockedCompareExchangePointer_nf\0"
  "_InterlockedDecrement16\0"
  "ssD*\0"
  "_InterlockedDecrement\0"
  "NiNiD*\0"
  "_InterlockedExchange8\0"
  "_InterlockedExchange16\0"
  "_InterlockedExchange\0"
  "_InterlockedExchangeAdd8\0"
  "_InterlockedExchangeAdd16\0"
  "_InterlockedExchangeAdd\0"
  "_InterlockedExchangePointer\0"
  "v*v*D*v*\0"
  "_InterlockedExchangeSub8\0"
  "_InterlockedExchangeSub16\0"
  "_InterlockedExchangeSub\0"
  "_InterlockedIncrement16\0"
  "_InterlockedIncrement\0"
  "_InterlockedOr8\0"
  "_InterlockedOr16\0"
  "_InterlockedOr\0"
  "_InterlockedXor8\0"
  "_InterlockedXor16\0"
  "_InterlockedXor\0"
  "__builtin_invoke\0"
  "isalnum\0"
  "fnU\0"
  "__builtin_is_aligned\0"
  "bvC*z\0"
  "isalpha\0"
  "isblank\0"
  "iscntrl\0"
  "__builtin_is_constant_evaluated\0"
  "isdigit\0"
  "__builtin_isfpclass\0"
  "__builtin_isfinite\0"
  "isgraph\0"
  "__builtin_isgreater\0"
  "__builtin_isgreaterequal\0"
  "__builtin_isinf\0"
  "__builtin_isinf_sign\0"
  "__builtin_isless\0"
  "__builtin_islessequal\0"
  "__builtin_islessgreater\0"
  "islower\0"
  "__builtin_isnan\0"
  "__builtin_isnormal\0"
  "isprint\0"
  "ispunct\0"
  "__builtin_issignaling\0"
  "isspace\0"
  "__builtin_issubnormal\0"
  "__builtin_isunordered\0"
  "isupper\0"
  "__builtin_is_within_lifetime\0"
  "ntEG\0"
  "isxdigit\0"
  "__builtin_iszero\0"
  "__iso_volatile_load8\0"
  "ccCD*\0"
  "__iso_volatile_load16\0"
  "ssCD*\0"
  "__iso_volatile_load32\0"
  "iiCD*\0"
  "__iso_volatile_load64\0"
  "LLiLLiCD*\0"
  "__iso_volatile_store8\0"
  "vcD*c\0"
  "__iso_volatile_store16\0"
  "vsD*s\0"
  "__iso_volatile_store32\0"
  "viD*i\0"
  "__iso_volatile_store64\0"
  "vLLiD*LLi\0"
  "__builtin_llrintf128\0"
  "LLiLLd\0"
  "__builtin_llroundf128\0"
  "__builtin_launder\0"
  "__builtin_ldexpf\0"
  "ffi\0"
  "ldexpf\0"
  "__builtin_ldexp\0"
  "ddi\0"
  "ldexp\0"
  "__builtin_ldexpl\0"
  "LdLdi\0"
  "ldexpl\0"
  "__builtin_ldexpf16\0"
  "hhi\0"
  "__builtin_ldexpf128\0"
  "LLdLLdi\0"
  "__builtin_lgammaf\0"
  "lgammaf\0"
  "__builtin_lgamma\0"
  "lgamma\0"
  "__builtin_lgammal\0"
  "lgammal\0"
  "__builtin_lgammaf128\0"
  "__builtin_llrintf\0"
  "LLif\0"
  "llrintf\0"
  "__builtin_llrint\0"
  "LLid\0"
  "llrint\0"
  "__builtin_llrintl\0"
  "LLiLd\0"
  "llrintl\0"
  "__builtin_llroundf\0"
  "llroundf\0"
  "__builtin_llround\0"
  "llround\0"
  "__builtin_llroundl\0"
  "llroundl\0"
  "__builtin_load_halff\0"
  "fhC*\0"
  "__builtin_load_half\0"
  "dhC*\0"
  "__builtin_logf\0"
  "logf\0"
  "__builtin_log\0"
  "log\0"
  "__builtin_logl\0"
  "logl\0"
  "__builtin_log1pf\0"
  "log1pf\0"
  "__builtin_log1p\0"
  "log1p\0"
  "__builtin_log1pl\0"
  "log1pl\0"
  "__builtin_log1pf128\0"
  "__builtin_log2f\0"
  "log2f\0"
  "__builtin_log2\0"
  "log2\0"
  "__builtin_log2l\0"
  "log2l\0"
  "__builtin_log2f16\0"
  "__builtin_log2f128\0"
  "__builtin_log10f\0"
  "log10f\0"
  "__builtin_log10\0"
  "log10\0"
  "__builtin_log10l\0"
  "log10l\0"
  "__builtin_log10f16\0"
  "__builtin_log10f128\0"
  "__builtin_logf16\0"
  "__builtin_logf128\0"
  "__builtin_logbf\0"
  "logbf\0"
  "__builtin_logb\0"
  "logb\0"
  "__builtin_logbl\0"
  "logbl\0"
  "__builtin_logbf128\0"
  "longjmp\0"
  "__builtin_lrintf\0"
  "Lif\0"
  "lrintf\0"
  "__builtin_lrint\0"
  "Lid\0"
  "lrint\0"
  "__builtin_lrintl\0"
  "LiLd\0"
  "lrintl\0"
  "__builtin_lrintf128\0"
  "LiLLd\0"
  "_lrotl\0"
  "ULiULii\0"
  "_lrotr\0"
  "__builtin_lroundf\0"
  "lroundf\0"
  "__builtin_lround\0"
  "lround\0"
  "__builtin_lroundl\0"
  "lroundl\0"
  "__builtin_lroundf128\0"
  "__annotation\0"
  "wC*.\0"
  "__assume\0"
  "_byteswap_ushort\0"
  "_byteswap_ulong\0"
  "UNiUNi\0"
  "_byteswap_uint64\0"
  "ULLiULLi\0"
  "__lzcnt16\0"
  "__lzcnt\0"
  "UiUi\0"
  "__lzcnt64\0"
  "__popcnt16\0"
  "__popcnt\0"
  "__popcnt64\0"
  "_ReturnAddress\0"
  "__builtin_ms_va_copy\0"
  "vc*&c*&\0"
  "__builtin_ms_va_end\0"
  "vc*&\0"
  "__builtin_ms_va_start\0"
  "vc*&.\0"
  "__va_start\0"
  "vc**.\0"
  "malloc\0"
  "__builtin_masked_compress_store\0"
  "__builtin_masked_expand_load\0"
  "__builtin_masked_gather\0"
  "nUt\0"
  "__builtin_masked_load\0"
  "__builtin_masked_scatter\0"
  "__builtin_masked_store\0"
  "__builtin_matrix_column_major_load\0"
  "nFt\0"
  "__builtin_matrix_column_major_store\0"
  "__builtin_matrix_transpose\0"
  "memalign\0"
  "__builtin_memchr\0"
  "v*vC*iz\0"
  "memchr\0"
  "fnE\0"
  "memcmp\0"
  "__builtin_memcpy\0"
  "memcpy\0"
  "__builtin_memmove\0"
  "memmove\0"
  "__builtin_memset\0"
  "v*v*iz\0"
  "memset\0"
  "memccpy\0"
  "v*v*vC*iz\0"
  "__builtin___memccpy_chk\0"
  "v*v*vC*izz\0"
  "__builtin___memcpy_chk\0"
  "v*v*vC*zz\0"
  "__builtin___memmove_chk\0"
  "mempcpy\0"
  "__builtin___mempcpy_chk\0"
  "__builtin___memset_chk\0"
  "v*v*izz\0"
  "__builtin_modff\0"
  "fff*\0"
  "modff\0"
  "__builtin_modf\0"
  "ddd*\0"
  "modf\0"
  "__builtin_modfl\0"
  "LdLdLd*\0"
  "modfl\0"
  "__builtin_modff128\0"
  "LLdLLdLLd*\0"
  "move\0"
  "move_if_noexcept\0"
  "__builtin_mul_overflow\0"
  "NSLog\0"
  "vG.\0"
  "fp:0:\0"
  "NSLogv\0"
  "vGa\0"
  "fP:0:\0"
  "__builtin_nanf\0"
  "fcC*\0"
  "FEUn\0"
  "nanf\0"
  "fUn\0"
  "__builtin_nan\0"
  "dcC*\0"
  "nan\0"
  "__builtin_nanl\0"
  "LdcC*\0"
  "nanl\0"
  "__builtin_nanf16\0"
  "xcC*\0"
  "FnUE\0"
  "__builtin_nanf128\0"
  "LLdcC*\0"
  "__builtin_nansf\0"
  "__builtin_nans\0"
  "__builtin_nansl\0"
  "__builtin_nansf16\0"
  "__builtin_nansf128\0"
  "__builtin_nearbyintf\0"
  "nearbyintf\0"
  "__builtin_nearbyint\0"
  "nearbyint\0"
  "__builtin_nearbyintl\0"
  "nearbyintl\0"
  "__builtin_nearbyintf128\0"
  "__builtin_nextafterf\0"
  "nextafterf\0"
  "__builtin_nextafter\0"
  "nextafter\0"
  "__builtin_nextafterl\0"
  "nextafterl\0"
  "__builtin_nextafterf128\0"
  "__builtin_nexttowardf\0"
  "ffLd\0"
  "nexttowardf\0"
  "__builtin_nexttoward\0"
  "ddLd\0"
  "nexttoward\0"
  "__builtin_nexttowardl\0"
  "nexttowardl\0"
  "__builtin_nexttowardf128\0"
  "__builtin_nondeterministic_value\0"
  "__builtin_nontemporal_load\0"
  "__builtin_nontemporal_store\0"
  "__noop\0"
  "__builtin_os_log_format\0"
  "v*v*cC*.\0"
  "p:0:nt\0"
  "__builtin_os_log_format_buffer_size\0"
  "zcC*.\0"
  "p:0:nutE\0"
  "__exp10f\0"
  "__exp10\0"
  "__finitef\0"
  "__finite\0"
  "__finitel\0"
  "objc_assign_global\0"
  "GGG*\0"
  "objc_assign_ivar\0"
  "GGGY\0"
  "objc_assign_strongCast\0"
  "objc_assign_weak\0"
  "objc_enumerationMutation\0"
  "vG\0"
  "objc_exception_extract\0"
  "Gv*\0"
  "objc_exception_match\0"
  "iGG\0"
  "objc_exception_throw\0"
  "objc_exception_try_enter\0"
  "objc_exception_try_exit\0"
  "objc_getClass\0"
  "GcC*\0"
  "objc_getMetaClass\0"
  "__builtin_objc_memmove_collectable\0"
  "objc_msgSend\0"
  "GGH.\0"
  "objc_msgSend_fp2ret\0"
  "XLdGH.\0"
  "objc_msgSend_fpret\0"
  "LdGH.\0"
  "objc_msgSend_stret\0"
  "vGH.\0"
  "objc_msgSendSuper\0"
  "GM*H.\0"
  "objc_msgSendSuper_stret\0"
  "vM*H.\0"
  "objc_read_weak\0"
  "GG*\0"
  "objc_sync_enter\0"
  "iG\0"
  "objc_sync_exit\0"
  "__builtin_object_size\0"
  "__builtin_parity\0"
  "__builtin_parityl\0"
  "__builtin_parityll\0"
  "__builtin_popcount\0"
  "__builtin_popcountl\0"
  "__builtin_popcountll\0"
  "__builtin_popcountg\0"
  "__builtin_powf\0"
  "powf\0"
  "__builtin_pow\0"
  "pow\0"
  "__builtin_powl\0"
  "powl\0"
  "__builtin_powf16\0"
  "__builtin_powf128\0"
  "__builtin_powif\0"
  "__builtin_powi\0"
  "__builtin_powil\0"
  "__builtin_prefetch\0"
  "vvC*.\0"
  "printf\0"
  "icC*.\0"
  "fp:0:N:0:<0>\0"
  "__builtin___printf_chk\0"
  "iicC*R.\0"
  "Fp:1:\0"
  "pthread_create\0"
  "ffC<2,3>\0"
  "__builtin_ptrauth_auth\0"
  "v*v*iv*\0"
  "__builtin_ptrauth_auth_and_resign\0"
  "v*v*iv*iv*\0"
  "__builtin_ptrauth_blend_discriminator\0"
  "zv*i\0"
  "__builtin_ptrauth_sign_constant\0"
  "tncE\0"
  "__builtin_ptrauth_sign_generic_data\0"
  "zv*v*\0"
  "__builtin_ptrauth_sign_unauthenticated\0"
  "__builtin_ptrauth_string_discriminator\0"
  "zcC*\0"
  "__builtin_ptrauth_strip\0"
  "v*v*i\0"
  "__builtin_readcyclecounter\0"
  "ULLi\0"
  "read_pipe\0"
  "__builtin_readsteadycounter\0"
  "realloc\0"
  "__builtin_reduce_add\0"
  "__builtin_reduce_and\0"
  "__builtin_reduce_max\0"
  "__builtin_reduce_maximum\0"
  "__builtin_reduce_min\0"
  "__builtin_reduce_minimum\0"
  "__builtin_reduce_mul\0"
  "__builtin_reduce_or\0"
  "__builtin_reduce_xor\0"
  "__builtin_remainderf\0"
  "remainderf\0"
  "__builtin_remainder\0"
  "remainder\0"
  "__builtin_remainderl\0"
  "remainderl\0"
  "__builtin_remainderf128\0"
  "__builtin_remquof\0"
  "fffi*\0"
  "remquof\0"
  "__builtin_remquo\0"
  "dddi*\0"
  "remquo\0"
  "__builtin_remquol\0"
  "LdLdLdi*\0"
  "remquol\0"
  "__builtin_remquof128\0"
  "LLdLLdLLdi*\0"
  "reserve_read_pipe\0"
  "reserve_write_pipe\0"
  "__builtin_rindex\0"
  "rindex\0"
  "__builtin_rintf\0"
  "Fng\0"
  "rintf\0"
  "fng\0"
  "__builtin_rint\0"
  "rint\0"
  "__builtin_rintl\0"
  "rintl\0"
  "__builtin_rintf16\0"
  "__builtin_rintf128\0"
  "__builtin_rotateleft8\0"
  "UcUcUc\0"
  "__builtin_rotateleft16\0"
  "UsUsUs\0"
  "__builtin_rotateleft32\0"
  "UZiUZiUZi\0"
  "__builtin_rotateleft64\0"
  "UWiUWiUWi\0"
  "__builtin_rotateright8\0"
  "__builtin_rotateright16\0"
  "__builtin_rotateright32\0"
  "__builtin_rotateright64\0"
  "_rotl\0"
  "UiUii\0"
  "_rotl8\0"
  "_rotl16\0"
  "UsUsUc\0"
  "_rotl64\0"
  "UWiUWii\0"
  "_rotr\0"
  "_rotr8\0"
  "_rotr16\0"
  "_rotr64\0"
  "__builtin_roundf\0"
  "roundf\0"
  "__builtin_round\0"
  "round\0"
  "__builtin_roundl\0"
  "roundl\0"
  "__builtin_roundevenf\0"
  "roundevenf\0"
  "__builtin_roundeven\0"
  "roundeven\0"
  "__builtin_roundevenl\0"
  "roundevenl\0"
  "__builtin_roundf16\0"
  "__builtin_roundf128\0"
  "__builtin_roundevenf16\0"
  "__builtin_roundevenf128\0"
  "__builtin___snprintf_chk\0"
  "ic*RzizcC*R.\0"
  "Fp:4:\0"
  "__builtin_sprintf\0"
  "ic*RcC*R.\0"
  "sprintf\0"
  "__builtin___sprintf_chk\0"
  "ic*RizcC*R.\0"
  "Fp:3:\0"
  "__builtin_sscanf\0"
  "icC*RcC*R.\0"
  "sscanf\0"
  "__builtin_sadd_overflow\0"
  "biCiCi*\0"
  "__builtin_saddl_overflow\0"
  "bLiCLiCLi*\0"
  "__builtin_saddll_overflow\0"
  "bLLiCLLiCLLi*\0"
  "savectx\0"
  "iSJ\0"
  "__builtin_scalblnf\0"
  "ffLi\0"
  "scalblnf\0"
  "__builtin_scalbln\0"
  "ddLi\0"
  "scalbln\0"
  "__builtin_scalblnl\0"
  "LdLdLi\0"
  "scalblnl\0"
  "__builtin_scalbnf\0"
  "scalbnf\0"
  "__builtin_scalbn\0"
  "scalbn\0"
  "__builtin_scalbnl\0"
  "scalbnl\0"
  "__builtin_scanf\0"
  "Fs:0:N:0:<0>\0"
  "scanf\0"
  "fs:0:N:0:<0>\0"
  "__builtin_scalblnf128\0"
  "LLdLLdLi\0"
  "__builtin_scalbnf128\0"
  "__scoped_atomic_thread_fence\0"
  "vii\0"
  "setjmp\0"
  "iJ\0"
  "_setjmp\0"
  "_setjmpex\0"
  "fTj\0"
  "__builtin_shufflevector\0"
  "siglongjmp\0"
  "vSJi\0"
  "sigsetjmp\0"
  "iSJi\0"
  "__sigsetjmp\0"
  "__builtin_signbit\0"
  "__builtin_signbitf\0"
  "__builtin_signbitl\0"
  "__builtin_sinf\0"
  "sinf\0"
  "__builtin_sin\0"
  "sin\0"
  "__builtin_sinl\0"
  "sinl\0"
  "__builtin_sinf16\0"
  "__builtin_sinf128\0"
  "__builtin_sincosf\0"
  "vff*f*\0"
  "sincosf\0"
  "__builtin_sincos\0"
  "vdd*d*\0"
  "sincos\0"
  "__builtin_sincosl\0"
  "vLdLd*Ld*\0"
  "sincosl\0"
  "__builtin_sincosf16\0"
  "vhh*h*\0"
  "__builtin_sincosf128\0"
  "vLLdLLd*LLd*\0"
  "__builtin_sincospif\0"
  "__builtin_sincospi\0"
  "__builtin_sincospil\0"
  "__builtin_sinhf\0"
  "sinhf\0"
  "__builtin_sinh\0"
  "sinh\0"
  "__builtin_sinhl\0"
  "sinhl\0"
  "__builtin_sinhf16\0"
  "__builtin_sinhf128\0"
  "__sinpif\0"
  "__sinpi\0"
  "__builtin_smul_overflow\0"
  "__builtin_smull_overflow\0"
  "__builtin_smulll_overflow\0"
  "__builtin_snprintf\0"
  "ic*RzcC*R.\0"
  "Fnp:2:N:0:<2>\0"
  "snprintf\0"
  "fnp:2:N:0:<2>\0"
  "__builtin_sqrtf\0"
  "sqrtf\0"
  "__builtin_sqrt\0"
  "sqrt\0"
  "__builtin_sqrtl\0"
  "sqrtl\0"
  "__builtin_sqrtf16\0"
  "__builtin_sqrtf128\0"
  "__builtin_ssub_overflow\0"
  "__builtin_ssubl_overflow\0"
  "__builtin_ssubll_overflow\0"
  "__builtin_stack_address\0"
  "__builtin_store_halff\0"
  "vfh*\0"
  "__builtin_store_half\0"
  "vdh*\0"
  "__builtin_stpcpy\0"
  "c*c*cC*\0"
  "stpcpy\0"
  "__builtin___stpcpy_chk\0"
  "c*c*cC*z\0"
  "__builtin_stpncpy\0"
  "stpncpy\0"
  "__builtin___stpncpy_chk\0"
  "c*c*cC*zz\0"
  "__builtin_strcasecmp\0"
  "icC*cC*\0"
  "F\0"
  "strcasecmp\0"
  "__builtin_strcat\0"
  "strcat\0"
  "__builtin_strchr\0"
  "strchr\0"
  "__builtin_strcmp\0"
  "strcmp\0"
  "__builtin_strcpy\0"
  "strcpy\0"
  "__builtin_strdup\0"
  "c*cC*\0"
  "strdup\0"
  "strerror\0"
  "c*i\0"
  "__builtin_strlen\0"
  "strlen\0"
  "__builtin_strncat\0"
  "strncat\0"
  "__builtin_strncmp\0"
  "icC*cC*z\0"
  "strncmp\0"
  "__builtin_strncpy\0"
  "strncpy\0"
  "__builtin_strspn\0"
  "strspn\0"
  "__builtin_strstr\0"
  "c*cC*cC*\0"
  "strstr\0"
  "strtod\0"
  "dcC*c**\0"
  "strtof\0"
  "fcC*c**\0"
  "strtol\0"
  "LicC*c**i\0"
  "strtoll\0"
  "LLicC*c**i\0"
  "strtold\0"
  "LdcC*c**\0"
  "strtoul\0"
  "ULicC*c**i\0"
  "strtoull\0"
  "ULLicC*c**i\0"
  "strtok\0"
  "strcspn\0"
  "__builtin___strcat_chk\0"
  "__builtin___strcpy_chk\0"
  "strlcat\0"
  "zc*cC*z\0"
  "strlcpy\0"
  "__builtin___strlcat_chk\0"
  "zc*cC*zz\0"
  "__builtin___strlcpy_chk\0"
  "__builtin_strncasecmp\0"
  "strncasecmp\0"
  "__builtin_strndup\0"
  "c*cC*z\0"
  "strndup\0"
  "__builtin___strncat_chk\0"
  "__builtin___strncpy_chk\0"
  "__builtin_strpbrk\0"
  "strpbrk\0"
  "__builtin_strrchr\0"
  "strrchr\0"
  "strxfrm\0"
  "sub_group_commit_read_pipe\0"
  "sub_group_commit_write_pipe\0"
  "sub_group_reserve_read_pipe\0"
  "sub_group_reserve_write_pipe\0"
  "__builtin_sub_overflow\0"
  "__builtin_subcb\0"
  "__builtin_subcs\0"
  "__builtin_subc\0"
  "__builtin_subcl\0"
  "__builtin_subcll\0"
  "__sync_lock_test_and_set_1\0"
  "ccD*c.\0"
  "__sync_lock_test_and_set_2\0"
  "ssD*s.\0"
  "__sync_lock_test_and_set_4\0"
  "iiD*i.\0"
  "__sync_lock_test_and_set_8\0"
  "LLiLLiD*LLi.\0"
  "__sync_lock_test_and_set_16\0"
  "LLLiLLLiD*LLLi.\0"
  "__sync_val_compare_and_swap_1\0"
  "ccD*cc.\0"
  "__sync_val_compare_and_swap_2\0"
  "ssD*ss.\0"
  "__sync_val_compare_and_swap_4\0"
  "iiD*ii.\0"
  "__sync_val_compare_and_swap_8\0"
  "LLiLLiD*LLiLLi.\0"
  "__sync_val_compare_and_swap_16\0"
  "LLLiLLLiD*LLLiLLLi.\0"
  "__sync_add_and_fetch\0"
  "__sync_add_and_fetch_1\0"
  "__sync_add_and_fetch_2\0"
  "__sync_add_and_fetch_4\0"
  "__sync_add_and_fetch_8\0"
  "__sync_add_and_fetch_16\0"
  "__sync_and_and_fetch\0"
  "__sync_and_and_fetch_1\0"
  "__sync_and_and_fetch_2\0"
  "__sync_and_and_fetch_4\0"
  "__sync_and_and_fetch_8\0"
  "__sync_and_and_fetch_16\0"
  "__sync_bool_compare_and_swap\0"
  "__sync_bool_compare_and_swap_1\0"
  "bcD*cc.\0"
  "__sync_bool_compare_and_swap_2\0"
  "bsD*ss.\0"
  "__sync_bool_compare_and_swap_4\0"
  "biD*ii.\0"
  "__sync_bool_compare_and_swap_8\0"
  "bLLiD*LLiLLi.\0"
  "__sync_bool_compare_and_swap_16\0"
  "bLLLiD*LLLiLLLi.\0"
  "__sync_fetch_and_add\0"
  "__sync_fetch_and_add_1\0"
  "__sync_fetch_and_add_2\0"
  "__sync_fetch_and_add_4\0"
  "__sync_fetch_and_add_8\0"
  "__sync_fetch_and_add_16\0"
  "__sync_fetch_and_and\0"
  "__sync_fetch_and_and_1\0"
  "__sync_fetch_and_and_2\0"
  "__sync_fetch_and_and_4\0"
  "__sync_fetch_and_and_8\0"
  "__sync_fetch_and_and_16\0"
  "__sync_fetch_and_max\0"
  "iiD*i\0"
  "__sync_fetch_and_min\0"
  "__sync_fetch_and_nand\0"
  "__sync_fetch_and_nand_1\0"
  "__sync_fetch_and_nand_2\0"
  "__sync_fetch_and_nand_4\0"
  "__sync_fetch_and_nand_8\0"
  "__sync_fetch_and_nand_16\0"
  "__sync_fetch_and_or\0"
  "__sync_fetch_and_or_1\0"
  "__sync_fetch_and_or_2\0"
  "__sync_fetch_and_or_4\0"
  "__sync_fetch_and_or_8\0"
  "__sync_fetch_and_or_16\0"
  "__sync_fetch_and_sub\0"
  "__sync_fetch_and_sub_1\0"
  "__sync_fetch_and_sub_2\0"
  "__sync_fetch_and_sub_4\0"
  "__sync_fetch_and_sub_8\0"
  "__sync_fetch_and_sub_16\0"
  "__sync_fetch_and_umax\0"
  "UiUiD*Ui\0"
  "__sync_fetch_and_umin\0"
  "__sync_fetch_and_xor\0"
  "__sync_fetch_and_xor_1\0"
  "__sync_fetch_and_xor_2\0"
  "__sync_fetch_and_xor_4\0"
  "__sync_fetch_and_xor_8\0"
  "__sync_fetch_and_xor_16\0"
  "__sync_lock_release\0"
  "__sync_lock_release_1\0"
  "vcD*.\0"
  "__sync_lock_release_2\0"
  "vsD*.\0"
  "__sync_lock_release_4\0"
  "viD*.\0"
  "__sync_lock_release_8\0"
  "vLLiD*.\0"
  "__sync_lock_release_16\0"
  "vLLLiD*.\0"
  "__sync_lock_test_and_set\0"
  "__sync_nand_and_fetch\0"
  "__sync_nand_and_fetch_1\0"
  "__sync_nand_and_fetch_2\0"
  "__sync_nand_and_fetch_4\0"
  "__sync_nand_and_fetch_8\0"
  "__sync_nand_and_fetch_16\0"
  "__sync_or_and_fetch\0"
  "__sync_or_and_fetch_1\0"
  "__sync_or_and_fetch_2\0"
  "__sync_or_and_fetch_4\0"
  "__sync_or_and_fetch_8\0"
  "__sync_or_and_fetch_16\0"
  "__sync_sub_and_fetch\0"
  "__sync_sub_and_fetch_1\0"
  "__sync_sub_and_fetch_2\0"
  "__sync_sub_and_fetch_4\0"
  "__sync_sub_and_fetch_8\0"
  "__sync_sub_and_fetch_16\0"
  "__sync_swap\0"
  "__sync_swap_1\0"
  "__sync_swap_2\0"
  "__sync_swap_4\0"
  "__sync_swap_8\0"
  "__sync_swap_16\0"
  "__sync_synchronize\0"
  "__sync_val_compare_and_swap\0"
  "__sync_xor_and_fetch\0"
  "__sync_xor_and_fetch_1\0"
  "__sync_xor_and_fetch_2\0"
  "__sync_xor_and_fetch_4\0"
  "__sync_xor_and_fetch_8\0"
  "__sync_xor_and_fetch_16\0"
  "__builtin_tanf\0"
  "tanf\0"
  "__builtin_tan\0"
  "tan\0"
  "__builtin_tanl\0"
  "tanl\0"
  "__builtin_tanf16\0"
  "__builtin_tanf128\0"
  "__builtin_tanhf\0"
  "tanhf\0"
  "__builtin_tanh\0"
  "tanh\0"
  "__builtin_tanhl\0"
  "tanhl\0"
  "__builtin_tanhf16\0"
  "__builtin_tanhf128\0"
  "__tanpif\0"
  "__tanpi\0"
  "__builtin_tgammaf\0"
  "tgammaf\0"
  "__builtin_tgamma\0"
  "tgamma\0"
  "__builtin_tgammal\0"
  "tgammal\0"
  "__builtin_tgammaf128\0"
  "__builtin_thread_pointer\0"
  "to_global\0"
  "to_local\0"
  "tolower\0"
  "to_private\0"
  "toupper\0"
  "__builtin_trap\0"
  "__builtin_truncf\0"
  "truncf\0"
  "__builtin_trunc\0"
  "trunc\0"
  "__builtin_truncl\0"
  "truncl\0"
  "__builtin_truncf16\0"
  "__builtin_truncf128\0"
  "__builtin_uadd_overflow\0"
  "bUiCUiCUi*\0"
  "__builtin_uaddl_overflow\0"
  "bULiCULiCULi*\0"
  "__builtin_uaddll_overflow\0"
  "bULLiCULLiCULLi*\0"
  "__builtin_umul_overflow\0"
  "__builtin_umull_overflow\0"
  "__builtin_umulll_overflow\0"
  "__builtin_unpredictable\0"
  "__builtin_unreachable\0"
  "__builtin_unwind_init\0"
  "__builtin_usub_overflow\0"
  "__builtin_usubl_overflow\0"
  "__builtin_usubll_overflow\0"
  "__builtin___vfprintf_chk\0"
  "iP*RicC*Ra\0"
  "FP:2:\0"
  "__builtin_vfscanf\0"
  "iP*RcC*Ra\0"
  "FS:1:N:0:<0,1>\0"
  "vfscanf\0"
  "fS:1:N:0:<0,1>\0"
  "vfork\0"
  "p\0"
  "__builtin_vprintf\0"
  "icC*Ra\0"
  "FnP:0:N:0:<0>\0"
  "vprintf\0"
  "fnP:0:N:0:<0>\0"
  "__builtin___vprintf_chk\0"
  "iicC*Ra\0"
  "FP:1:\0"
  "__builtin___vsnprintf_chk\0"
  "ic*RzizcC*Ra\0"
  "FP:4:\0"
  "__builtin___vsprintf_chk\0"
  "ic*RizcC*Ra\0"
  "FP:3:\0"
  "__builtin_vsscanf\0"
  "icC*RcC*Ra\0"
  "vsscanf\0"
  "__builtin_vscanf\0"
  "FS:0:N:0:<0>\0"
  "vscanf\0"
  "fS:0:N:0:<0>\0"
  "__builtin_va_copy\0"
  "vAA\0"
  "va_copy\0"
  "__builtin_va_end\0"
  "vA\0"
  "va_end\0"
  "va_start\0"
  "__builtin_verbose_trap\0"
  "vcC*cC*\0"
  "__builtin_vfprintf\0"
  "FnP:1:N:0:<0,1>\0"
  "vfprintf\0"
  "fnP:1:N:0:<0,1>\0"
  "__builtin_vsprintf\0"
  "ic*RcC*Ra\0"
  "vsprintf\0"
  "__builtin_vsnprintf\0"
  "ic*RzcC*Ra\0"
  "FnP:2:N:0:<2>\0"
  "vsnprintf\0"
  "fnP:2:N:0:<2>\0"
  "__builtin_wmemchr\0"
  "w*wC*wz\0"
  "wmemchr\0"
  "__builtin_wmemcmp\0"
  "iwC*wC*z\0"
  "wmemcmp\0"
  "__builtin_wmemcpy\0"
  "w*w*wC*z\0"
  "wmemcpy\0"
  "__builtin_wmemmove\0"
  "wmemmove\0"
  "__warn_memset_zero_len\0"
  "nU\0"
  "__builtin_wcschr\0"
  "w*wC*w\0"
  "wcschr\0"
  "fnUE\0"
  "__builtin_wcscmp\0"
  "iwC*wC*\0"
  "wcscmp\0"
  "__builtin_wcslen\0"
  "zwC*\0"
  "wcslen\0"
  "__builtin_wcsncmp\0"
  "wcsncmp\0"
  "work_group_commit_read_pipe\0"
  "work_group_commit_write_pipe\0"
  "work_group_reserve_read_pipe\0"
  "work_group_reserve_write_pipe\0"
  "write_pipe\0"
  "__xray_customevent\0"
  "vcC*z\0"
  "__xray_typedevent\0"
  "vzcC*z\0"
  "__addressof\0"
  "zfncTE\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinStrings = BuiltinStringsStorage;

#endif // GET_BUILTIN_STR_TABLE

#ifdef GET_BUILTIN_INFOS
    Builtin::Info{Builtin::Info::StrOffsets{1 /* __atomic_add_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25 /* __atomic_and_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{44 /* __atomic_clear */, 20 /* v. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{62 /* __atomic_compare_exchange */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{88 /* __atomic_compare_exchange_n */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{116 /* __atomic_exchange */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{134 /* __atomic_exchange_n */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{154 /* __atomic_fetch_add */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{173 /* __atomic_fetch_and */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{192 /* __atomic_fetch_max */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{211 /* __atomic_fetch_min */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{230 /* __atomic_fetch_nand */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{250 /* __atomic_fetch_or */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{268 /* __atomic_fetch_sub */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{287 /* __atomic_fetch_udec */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{307 /* __atomic_fetch_uinc */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{327 /* __atomic_fetch_xor */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{346 /* __atomic_load */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{360 /* __atomic_load_n */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{376 /* __atomic_max_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{395 /* __atomic_min_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{414 /* __atomic_nand_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{434 /* __atomic_or_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{452 /* __atomic_store */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{467 /* __atomic_store_n */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{484 /* __atomic_sub_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{503 /* __atomic_test_and_set */, 525 /* b. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{528 /* __atomic_xor_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{547 /* __c11_atomic_compare_exchange_strong */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{584 /* __c11_atomic_compare_exchange_weak */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{619 /* __c11_atomic_exchange */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{641 /* __c11_atomic_fetch_add */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{664 /* __c11_atomic_fetch_and */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{687 /* __c11_atomic_fetch_max */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{710 /* __c11_atomic_fetch_min */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{733 /* __c11_atomic_fetch_nand */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{757 /* __c11_atomic_fetch_or */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{779 /* __c11_atomic_fetch_sub */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{802 /* __c11_atomic_fetch_xor */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{825 /* __c11_atomic_init */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{843 /* __c11_atomic_load */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{861 /* __c11_atomic_store */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{880 /* __hip_atomic_compare_exchange_strong */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{917 /* __hip_atomic_compare_exchange_weak */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{952 /* __hip_atomic_exchange */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{974 /* __hip_atomic_fetch_add */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{997 /* __hip_atomic_fetch_and */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1020 /* __hip_atomic_fetch_max */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1043 /* __hip_atomic_fetch_min */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1066 /* __hip_atomic_fetch_or */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1088 /* __hip_atomic_fetch_sub */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1111 /* __hip_atomic_fetch_xor */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1134 /* __hip_atomic_load */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1152 /* __hip_atomic_store */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1171 /* __opencl_atomic_compare_exchange_strong */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1211 /* __opencl_atomic_compare_exchange_weak */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1249 /* __opencl_atomic_exchange */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1274 /* __opencl_atomic_fetch_add */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1300 /* __opencl_atomic_fetch_and */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1326 /* __opencl_atomic_fetch_max */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1352 /* __opencl_atomic_fetch_min */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1378 /* __opencl_atomic_fetch_or */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1403 /* __opencl_atomic_fetch_sub */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1429 /* __opencl_atomic_fetch_xor */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1455 /* __opencl_atomic_init */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1476 /* __opencl_atomic_load */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1497 /* __opencl_atomic_store */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1519 /* __scoped_atomic_add_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1545 /* __scoped_atomic_and_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1571 /* __scoped_atomic_compare_exchange */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1604 /* __scoped_atomic_compare_exchange_n */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1639 /* __scoped_atomic_exchange */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1664 /* __scoped_atomic_exchange_n */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1691 /* __scoped_atomic_fetch_add */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1717 /* __scoped_atomic_fetch_and */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1743 /* __scoped_atomic_fetch_max */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1769 /* __scoped_atomic_fetch_min */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1795 /* __scoped_atomic_fetch_nand */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1822 /* __scoped_atomic_fetch_or */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1847 /* __scoped_atomic_fetch_sub */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1873 /* __scoped_atomic_fetch_xor */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1899 /* __scoped_atomic_load */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1920 /* __scoped_atomic_load_n */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1943 /* __scoped_atomic_max_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1969 /* __scoped_atomic_min_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{1995 /* __scoped_atomic_nand_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2022 /* __scoped_atomic_or_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2047 /* __scoped_atomic_store */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2069 /* __scoped_atomic_store_n */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2093 /* __scoped_atomic_sub_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2119 /* __scoped_atomic_fetch_udec */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2146 /* __scoped_atomic_fetch_uinc */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2173 /* __scoped_atomic_xor_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2199 /* __abnormal_termination */, 2222 /* i */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2226 /* _abnormal_termination */, 2222 /* i */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2248 /* __builtin_abort */, 2264 /* v */, 2266 /* Fnr */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2270 /* abort */, 2264 /* v */, 2276 /* fnr */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2280 /* __builtin_abs */, 2294 /* ii */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2302 /* abs */, 2294 /* ii */, 2306 /* fnc */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2310 /* __builtin_labs */, 2325 /* LiLi */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2330 /* labs */, 2325 /* LiLi */, 2306 /* fnc */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2335 /* __builtin_llabs */, 2351 /* LLiLLi */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2358 /* llabs */, 2351 /* LLiLLi */, 2306 /* fnc */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2364 /* __builtin_acosf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2387 /* acosf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2397 /* __builtin_acos */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2415 /* acos */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2420 /* __builtin_acosl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2441 /* acosl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2447 /* __builtin_acosf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2468 /* __builtin_acosf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2494 /* __builtin_acoshf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2511 /* acoshf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2518 /* __builtin_acosh */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2534 /* acosh */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2540 /* __builtin_acoshl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2557 /* acoshl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2564 /* __builtin_acoshf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2584 /* __builtin_add_overflow */, 525 /* b. */, 2607 /* ntE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2611 /* __builtin_addcb */, 2627 /* UcUcCUcCUcCUc* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2645 /* __builtin_addcs */, 2661 /* UsUsCUsCUsCUs* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2676 /* __builtin_addc */, 2691 /* UiUiCUiCUiCUi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2706 /* __builtin_addcl */, 2722 /* ULiULiCULiCULiCULi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2742 /* __builtin_addcll */, 2759 /* ULLiULLiCULLiCULLiCULLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2784 /* addressof */, 2794 /* v*v& */, 2799 /* fzncThE */, 0}, HeaderDesc::MEMORY, CXX_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{2807 /* __builtin_align_down */, 2828 /* v*vC*z */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2840 /* __builtin_align_up */, 2828 /* v*vC*z */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2859 /* aligned_alloc */, 2873 /* v*zz */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2880 /* __builtin_alloca */, 2897 /* v*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2904 /* alloca */, 2897 /* v*z */, 2911 /* fn */, 0}, HeaderDesc::STDLIB_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2914 /* _alloca */, 2897 /* v*z */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2922 /* __builtin_alloca_uninitialized */, 2897 /* v*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2953 /* __builtin_alloca_with_align */, 2981 /* v*zIz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{2987 /* __builtin_alloca_with_align_uninitialized */, 2981 /* v*zIz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3029 /* __builtin_allow_runtime_check */, 3059 /* bcC* */, 3064 /* nUc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3068 /* __builtin_annotation */, 20 /* v. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3089 /* __arithmetic_fence */, 20 /* v. */, 3108 /* tE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3111 /* as_const */, 3120 /* v&v& */, 2799 /* fzncThE */, 0}, HeaderDesc::UTILITY, CXX_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{3125 /* __builtin_asinf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3141 /* asinf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3147 /* __builtin_asin */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3162 /* asin */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3167 /* __builtin_asinl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3183 /* asinl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3189 /* __builtin_asinf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3207 /* __builtin_asinf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3226 /* __builtin_asinhf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3243 /* asinhf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3250 /* __builtin_asinh */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3266 /* asinh */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3272 /* __builtin_asinhl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3289 /* asinhl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3296 /* __builtin_asinhf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3316 /* __builtin_assume */, 3333 /* vb */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3336 /* __builtin_assume_separate_storage */, 3370 /* vvCD*vCD* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3380 /* __builtin_atanf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3396 /* atanf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3402 /* __builtin_atan */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3417 /* atan */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3422 /* __builtin_atanl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3438 /* atanl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3444 /* __builtin_atan2f */, 3461 /* fff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3465 /* atan2f */, 3461 /* fff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3472 /* __builtin_atan2 */, 3488 /* ddd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3492 /* atan2 */, 3488 /* ddd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3498 /* __builtin_atan2l */, 3515 /* LdLdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3522 /* atan2l */, 3515 /* LdLdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3529 /* __builtin_atan2f16 */, 3548 /* hhh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3552 /* __builtin_atan2f128 */, 3572 /* LLdLLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3582 /* __builtin_atanf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3600 /* __builtin_atanf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3619 /* __builtin_atanhf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3636 /* atanhf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3643 /* __builtin_atanh */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3659 /* atanh */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3665 /* __builtin_atanhl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3682 /* atanhl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3689 /* __builtin_atanhf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3709 /* __atomic_always_lock_free */, 3735 /* bzvCD* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3742 /* __atomic_is_lock_free */, 3735 /* bzvCD* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3764 /* __atomic_signal_fence */, 3786 /* vi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3789 /* __atomic_thread_fence */, 3786 /* vi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3811 /* bcmp */, 3816 /* ivC*vC*z */, 3825 /* fE */, 0}, HeaderDesc::STRINGS_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3828 /* __builtin_bswap16 */, 3846 /* UsUs */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3855 /* __builtin_bswap32 */, 3873 /* UZiUZi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3880 /* __builtin_bswap64 */, 3898 /* UWiUWi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3905 /* __builtin_bswapg */, 3922 /* i. */, 3925 /* ncEt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3930 /* __builtin_bzero */, 3946 /* vv*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3951 /* bzero */, 3946 /* vv*z */, 2911 /* fn */, 0}, HeaderDesc::STRINGS_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3957 /* __builtin_bcopy */, 3973 /* vvC*v*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3981 /* bcopy */, 3973 /* vvC*v*z */, 2911 /* fn */, 0}, HeaderDesc::STRINGS_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{3987 /* __builtin_bitreverse8 */, 4009 /* UcUc */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4014 /* __builtin_bitreverse16 */, 3846 /* UsUs */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4037 /* __builtin_bitreverse32 */, 3873 /* UZiUZi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4060 /* __builtin_bitreverse64 */, 3898 /* UWiUWi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4083 /* _bittest */, 4092 /* UcNiC*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4101 /* _bittest64 */, 4112 /* UcWiC*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4121 /* _bittestandcomplement */, 4143 /* UcNi*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4151 /* _bittestandcomplement64 */, 4175 /* UcWi*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4183 /* _bittestandreset */, 4143 /* UcNi*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4200 /* _bittestandreset64 */, 4175 /* UcWi*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4219 /* _bittestandset */, 4143 /* UcNi*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4234 /* _bittestandset64 */, 4175 /* UcWi*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4251 /* _Block_object_assign */, 4272 /* vv*vC*iC */, 2878 /* f */, 0}, HeaderDesc::BLOCKS_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4281 /* _Block_object_dispose */, 4303 /* vvC*iC */, 2878 /* f */, 0}, HeaderDesc::BLOCKS_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4310 /* __builtin_addressof */, 2794 /* v*v& */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4330 /* __builtin_allow_sanitize_check */, 3059 /* bcC* */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4365 /* __builtin_assume_aligned */, 4390 /* v*vC*z. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4398 /* __builtin_assume_dereferenceable */, 4431 /* vvC*z */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4440 /* __builtin_bcmp */, 3816 /* ivC*vC*z */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4459 /* __builtin_c23_va_start */, 4482 /* vA. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, C23_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{4486 /* __builtin___CFStringMakeConstantString */, 4525 /* FC*cC* */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4532 /* __builtin_cpu_init */, 2264 /* v */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4551 /* __builtin_cpu_is */, 3059 /* bcC* */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4568 /* __builtin_cpu_supports */, 3059 /* bcC* */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4591 /* __builtin_calloc */, 2873 /* v*zz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4608 /* __builtin_char_memchr */, 4630 /* c*cC*iz */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4638 /* __builtin_classify_type */, 3922 /* i. */, 4662 /* nctuE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4668 /* __builtin_complex */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4686 /* __builtin_constant_p */, 3922 /* i. */, 4662 /* nctuE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4707 /* __builtin_dump_struct */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4729 /* __builtin_flt_rounds */, 2222 /* i */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4750 /* __builtin_free */, 4765 /* vv* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4769 /* __builtin_function_start */, 2794 /* v*v& */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4794 /* __builtin_longjmp */, 4812 /* vv**i */, 4818 /* r */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4820 /* __builtin_malloc */, 2897 /* v*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4837 /* __builtin_memcmp */, 3816 /* ivC*vC*z */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4854 /* __builtin_memcpy_inline */, 4878 /* vv*vC*Iz */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4887 /* __builtin_mempcpy */, 4905 /* v*v*vC*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4914 /* __builtin_memset_inline */, 4938 /* vv*iIz */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4945 /* __builtin___NSStringMakeConstantString */, 4525 /* FC*cC* */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{4984 /* __builtin_operator_delete */, 4765 /* vv* */, 2607 /* ntE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5010 /* __builtin_operator_new */, 2897 /* v*z */, 5033 /* ctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5037 /* __builtin_preserve_access_index */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5069 /* __builtin_printf */, 5086 /* icC*R. */, 5093 /* np:0:FN:0:<0> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5107 /* __builtin_realloc */, 5125 /* v*v*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5131 /* __builtin_return_address */, 5156 /* v*IUi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5162 /* __builtin_set_flt_rounds */, 3786 /* vi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5187 /* __builtin_setjmp */, 5204 /* iv** */, 5209 /* j */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5211 /* __builtin_stdarg_start */, 4482 /* vA. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5234 /* __builtin_strcspn */, 5252 /* zcC*cC* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5260 /* __builtin_trivially_relocate */, 5289 /* v*v*v*z */, 5297 /* Ftn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5301 /* __builtin_va_start */, 4482 /* vA. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5320 /* __c11_atomic_is_lock_free */, 5346 /* bz */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5349 /* __c11_atomic_signal_fence */, 3786 /* vi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5375 /* __c11_atomic_thread_fence */, 3786 /* vi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5401 /* __builtin_cabsf */, 5417 /* fXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5421 /* cabsf */, 5417 /* fXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5427 /* __builtin_cabs */, 5442 /* dXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5446 /* cabs */, 5442 /* dXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5451 /* __builtin_cabsl */, 5467 /* LdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5473 /* cabsl */, 5467 /* LdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5479 /* __builtin_cacosf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5501 /* cacosf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5508 /* __builtin_cacos */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5529 /* cacos */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5535 /* __builtin_cacosl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5559 /* cacosl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5566 /* __builtin_cacoshf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5584 /* cacoshf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5592 /* __builtin_cacosh */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5609 /* cacosh */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5616 /* __builtin_cacoshl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5634 /* cacoshl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5642 /* __builtin_call_with_static_chain */, 20 /* v. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5675 /* calloc */, 2873 /* v*zz */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5682 /* __builtin_canonicalizef */, 2380 /* ff */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5706 /* __builtin_canonicalize */, 2412 /* dd */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5729 /* __builtin_canonicalizel */, 2436 /* LdLd */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5753 /* __builtin_canonicalizef16 */, 2465 /* hh */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5779 /* __builtin_cargf */, 5417 /* fXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5795 /* cargf */, 5417 /* fXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5801 /* __builtin_carg */, 5442 /* dXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5816 /* carg */, 5442 /* dXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5821 /* __builtin_cargl */, 5467 /* LdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5837 /* cargl */, 5467 /* LdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5843 /* __builtin_casinf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5860 /* casinf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5867 /* __builtin_casin */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5883 /* casin */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5889 /* __builtin_casinl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5906 /* casinl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5913 /* __builtin_casinhf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5931 /* casinhf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5939 /* __builtin_casinh */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5956 /* casinh */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5963 /* __builtin_casinhl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5981 /* casinhl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{5989 /* __builtin_catanf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6006 /* catanf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6013 /* __builtin_catan */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6029 /* catan */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6035 /* __builtin_catanl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6052 /* catanl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6059 /* __builtin_catanhf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6077 /* catanhf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6085 /* __builtin_catanh */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6102 /* catanh */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6109 /* __builtin_catanhl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6127 /* catanhl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6135 /* __builtin_cbrtf */, 2380 /* ff */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6155 /* cbrtf */, 2380 /* ff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6161 /* __builtin_cbrt */, 2412 /* dd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6176 /* cbrt */, 2412 /* dd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6181 /* __builtin_cbrtl */, 2436 /* LdLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6197 /* cbrtl */, 2436 /* LdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6203 /* __builtin_cbrtf128 */, 2487 /* LLdLLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6222 /* __builtin_ccosf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6238 /* ccosf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6244 /* __builtin_ccos */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6259 /* ccos */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6264 /* __builtin_ccosl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6280 /* ccosl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6286 /* __builtin_ccoshf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6303 /* ccoshf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6310 /* __builtin_ccosh */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6326 /* ccosh */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6332 /* __builtin_ccoshl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6349 /* ccoshl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6356 /* __builtin_ceilf */, 2380 /* ff */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6372 /* ceilf */, 2380 /* ff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6378 /* __builtin_ceil */, 2412 /* dd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6393 /* ceil */, 2412 /* dd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6398 /* __builtin_ceill */, 2436 /* LdLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6414 /* ceill */, 2436 /* LdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6420 /* __builtin_ceilf16 */, 2465 /* hh */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6438 /* __builtin_ceilf128 */, 2487 /* LLdLLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6457 /* __builtin_cexpf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6473 /* cexpf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6479 /* __builtin_cexp */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6494 /* cexp */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6499 /* __builtin_cexpl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6515 /* cexpl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6521 /* __builtin_cimagf */, 5417 /* fXf */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6538 /* cimagf */, 5417 /* fXf */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6545 /* __builtin_cimag */, 5442 /* dXd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6561 /* cimag */, 5442 /* dXd */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6567 /* __builtin_cimagl */, 5467 /* LdXLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6584 /* cimagl */, 5467 /* LdXLd */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6591 /* __builtin___clear_cache */, 6615 /* vv*v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6621 /* __builtin_clogf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6637 /* clogf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6643 /* __builtin_clog */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6658 /* clog */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6663 /* __builtin_clogl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6679 /* clogl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6685 /* __builtin_clrsb */, 2294 /* ii */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6701 /* __builtin_clrsbl */, 6718 /* iLi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6722 /* __builtin_clrsbll */, 6740 /* iLLi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6745 /* __builtin_clzs */, 6760 /* iUs */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6764 /* __builtin_clz */, 6778 /* iUi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6782 /* __builtin_clzl */, 6797 /* iULi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6802 /* __builtin_clzll */, 6818 /* iULLi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6824 /* __builtin_clzg */, 3922 /* i. */, 3925 /* ncEt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6839 /* commit_read_pipe */, 20 /* v. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{6859 /* commit_write_pipe */, 20 /* v. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{6877 /* __builtin_conjf */, 5496 /* XfXf */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6893 /* conjf */, 5496 /* XfXf */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6899 /* __builtin_conj */, 5524 /* XdXd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6914 /* conj */, 5524 /* XdXd */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6919 /* __builtin_conjl */, 5552 /* XLdXLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6935 /* conjl */, 5552 /* XLdXLd */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6941 /* __builtin_convertvector */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6965 /* __builtin_copysignf */, 3461 /* fff */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6985 /* copysignf */, 3461 /* fff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{6995 /* __builtin_copysign */, 3488 /* ddd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7014 /* copysign */, 3488 /* ddd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7023 /* __builtin_copysignl */, 3515 /* LdLdLd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7043 /* copysignl */, 3515 /* LdLdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7053 /* __builtin_copysignf16 */, 3548 /* hhh */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7075 /* __builtin_copysignf128 */, 3572 /* LLdLLdLLd */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7103 /* __builtin_coro_align */, 7124 /* z */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7126 /* __builtin_coro_alloc */, 7147 /* b */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7149 /* __builtin_coro_begin */, 7170 /* v*v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7175 /* __builtin_coro_destroy */, 4765 /* vv* */, 0 /*  */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7198 /* __builtin_coro_done */, 7218 /* bv* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7222 /* __builtin_coro_end */, 7241 /* bv*Ib */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7247 /* __builtin_coro_frame */, 7268 /* v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7271 /* __builtin_coro_free */, 7170 /* v*v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7291 /* __builtin_coro_id */, 7309 /* v*Iiv*v*v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7320 /* __builtin_coro_noop */, 7268 /* v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7340 /* __builtin_coro_promise */, 7363 /* v*v*IiIb */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7372 /* __builtin_coro_resume */, 4765 /* vv* */, 0 /*  */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7394 /* __builtin_coro_size */, 7124 /* z */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7414 /* __builtin_coro_suspend */, 7437 /* cIb */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, COR_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{7441 /* __builtin_cosf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7456 /* cosf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7461 /* __builtin_cos */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7475 /* cos */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7479 /* __builtin_cosl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7494 /* cosl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7499 /* __builtin_cosf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7516 /* __builtin_cosf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7534 /* __builtin_coshf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7550 /* coshf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7556 /* __builtin_cosh */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7571 /* cosh */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7576 /* __builtin_coshl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7592 /* coshl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7598 /* __builtin_coshf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7616 /* __builtin_coshf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7635 /* __cospif */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7644 /* __cospi */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7652 /* __builtin_counted_by_ref */, 3922 /* i. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7677 /* __builtin_cpowf */, 7693 /* XfXfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7700 /* cpowf */, 7693 /* XfXfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7706 /* __builtin_cpow */, 7721 /* XdXdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7728 /* cpow */, 7721 /* XdXdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7733 /* __builtin_cpowl */, 7749 /* XLdXLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7759 /* cpowl */, 7749 /* XLdXLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7765 /* __builtin_cprojf */, 5496 /* XfXf */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7782 /* cprojf */, 5496 /* XfXf */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7789 /* __builtin_cproj */, 5524 /* XdXd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7805 /* cproj */, 5524 /* XdXd */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7811 /* __builtin_cprojl */, 5552 /* XLdXLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7828 /* cprojl */, 5552 /* XLdXLd */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7835 /* __builtin_crealf */, 5417 /* fXf */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7852 /* crealf */, 5417 /* fXf */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7859 /* __builtin_creal */, 5442 /* dXd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7875 /* creal */, 5442 /* dXd */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7881 /* __builtin_creall */, 5467 /* LdXLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7898 /* creall */, 5467 /* LdXLd */, 2306 /* fnc */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7905 /* __builtin_csinf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7921 /* csinf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7927 /* __builtin_csin */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7942 /* csin */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7947 /* __builtin_csinl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7963 /* csinl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7969 /* __builtin_csinhf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7986 /* csinhf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{7993 /* __builtin_csinh */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8009 /* csinh */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8015 /* __builtin_csinhl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8032 /* csinhl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8039 /* __builtin_csqrtf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8056 /* csqrtf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8063 /* __builtin_csqrt */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8079 /* csqrt */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8085 /* __builtin_csqrtl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8102 /* csqrtl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8109 /* __builtin_ctanf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8125 /* ctanf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8131 /* __builtin_ctan */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8146 /* ctan */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8151 /* __builtin_ctanl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8167 /* ctanl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8173 /* __builtin_ctanhf */, 5496 /* XfXf */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8190 /* ctanhf */, 5496 /* XfXf */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8197 /* __builtin_ctanh */, 5524 /* XdXd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8213 /* ctanh */, 5524 /* XdXd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8219 /* __builtin_ctanhl */, 5552 /* XLdXLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8236 /* ctanhl */, 5552 /* XLdXLd */, 2393 /* fne */, 0}, HeaderDesc::COMPLEX_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8243 /* __builtin_ctzs */, 6760 /* iUs */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8258 /* __builtin_ctz */, 6778 /* iUi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8272 /* __builtin_ctzl */, 6797 /* iULi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8287 /* __builtin_ctzll */, 6818 /* iULLi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8303 /* __builtin_ctzg */, 3922 /* i. */, 3925 /* ncEt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8318 /* __builtin_dwarf_sp_column */, 8344 /* Ui */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8347 /* __builtin_dwarf_cfa */, 7268 /* v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8367 /* __debugbreak */, 2264 /* v */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8380 /* __builtin_debugtrap */, 2264 /* v */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8400 /* __builtin_dynamic_object_size */, 8430 /* zvC*i */, 8436 /* nuE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8440 /* __builtin_eh_return */, 8460 /* vzv* */, 4818 /* r */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8465 /* __builtin_eh_return_data_regno */, 8496 /* iIi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8500 /* __builtin_elementwise_acos */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8527 /* __builtin_elementwise_asin */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8554 /* __builtin_elementwise_atan */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8581 /* __builtin_elementwise_atan2 */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8609 /* __builtin_elementwise_abs */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8635 /* __builtin_elementwise_add_sat */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8665 /* __builtin_elementwise_bitreverse */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8698 /* __builtin_elementwise_canonicalize */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8733 /* __builtin_elementwise_ceil */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8760 /* __builtin_elementwise_copysign */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8791 /* __builtin_elementwise_cos */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8817 /* __builtin_elementwise_cosh */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8844 /* __builtin_elementwise_clzg */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8871 /* __builtin_elementwise_ctzg */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8898 /* __builtin_elementwise_exp */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8924 /* __builtin_elementwise_exp2 */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8951 /* __builtin_elementwise_exp10 */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{8979 /* __builtin_elementwise_floor */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9007 /* __builtin_elementwise_fma */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9033 /* __builtin_elementwise_fmod */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9060 /* __builtin_elementwise_fshl */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9087 /* __builtin_elementwise_fshr */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9114 /* __builtin_elementwise_ldexp */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9142 /* __builtin_elementwise_log */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9168 /* __builtin_elementwise_log2 */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9195 /* __builtin_elementwise_log10 */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9223 /* __builtin_elementwise_max */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9249 /* __builtin_elementwise_maxnum */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9278 /* __builtin_elementwise_maximum */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9308 /* __builtin_elementwise_maximumnum */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9341 /* __builtin_elementwise_min */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9367 /* __builtin_elementwise_minnum */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9396 /* __builtin_elementwise_minimum */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9426 /* __builtin_elementwise_minimumnum */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9459 /* __builtin_elementwise_nearbyint */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9491 /* __builtin_elementwise_popcount */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9522 /* __builtin_elementwise_pow */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9548 /* __builtin_elementwise_rint */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9575 /* __builtin_elementwise_round */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9603 /* __builtin_elementwise_roundeven */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9635 /* __builtin_elementwise_sin */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9661 /* __builtin_elementwise_sinh */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9688 /* __builtin_elementwise_sqrt */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9715 /* __builtin_elementwise_sub_sat */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9745 /* __builtin_elementwise_tan */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9771 /* __builtin_elementwise_tanh */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9798 /* __builtin_elementwise_trunc */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9826 /* enqueue_kernel */, 3922 /* i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_DSE},
    Builtin::Info{Builtin::Info::StrOffsets{9841 /* __builtin_erff */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9856 /* erff */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9861 /* __builtin_erf */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9875 /* erf */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9879 /* __builtin_erfl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9894 /* erfl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9899 /* __builtin_erff128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9917 /* __builtin_erfcf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9933 /* erfcf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9939 /* __builtin_erfc */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9954 /* erfc */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9959 /* __builtin_erfcl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9975 /* erfcl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{9981 /* __builtin_erfcf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10000 /* __exception_code */, 10017 /* UNi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10021 /* _exception_code */, 10017 /* UNi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10037 /* __exception_info */, 7268 /* v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10054 /* _exception_info */, 7268 /* v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10070 /* exit */, 3786 /* vi */, 10075 /* fr */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10078 /* _Exit */, 3786 /* vi */, 10075 /* fr */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10084 /* __builtin_expf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10099 /* expf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10104 /* __builtin_exp */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10118 /* exp */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10122 /* __builtin_expl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10137 /* expl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10142 /* __builtin_exp2f */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10158 /* exp2f */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10164 /* __builtin_exp2 */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10179 /* exp2 */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10184 /* __builtin_exp2l */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10200 /* exp2l */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10206 /* __builtin_exp2f16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10224 /* __builtin_exp2f128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10243 /* __builtin_exp10f */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10260 /* __builtin_exp10 */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10276 /* __builtin_exp10l */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10293 /* __builtin_exp10f16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10312 /* __builtin_exp10f128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10332 /* __builtin_expf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10349 /* __builtin_expf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10367 /* __builtin_expect */, 10384 /* LiLiLi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10391 /* __builtin_expect_with_probability */, 10425 /* LiLiLid */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10433 /* __builtin_expm1f */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10450 /* expm1f */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10457 /* __builtin_expm1 */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10473 /* expm1 */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10479 /* __builtin_expm1l */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10496 /* expm1l */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10503 /* __builtin_expm1f128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10523 /* __builtin_extend_pointer */, 10548 /* ULLiv* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10555 /* __builtin_extract_return_addr */, 7170 /* v*v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10585 /* __builtin_ffs */, 2294 /* ii */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10599 /* __builtin_ffsl */, 6718 /* iLi */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10614 /* __builtin_ffsll */, 6740 /* iLLi */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10630 /* __builtin_fpclassify */, 10651 /* iiiiii. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10665 /* __builtin_fprintf */, 10683 /* iP*RcC*R. */, 10693 /* Fnp:1:N:0:<0,1> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10709 /* fprintf */, 10683 /* iP*RcC*R. */, 10717 /* fnp:1:N:0:<0,1> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10733 /* __builtin___fprintf_chk */, 10757 /* iP*RicC*R. */, 10768 /* Fp:2: */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10774 /* __builtin_fscanf */, 10683 /* iP*RcC*R. */, 10791 /* Fs:1:N:0:<0,1> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10806 /* fscanf */, 10683 /* iP*RcC*R. */, 10813 /* fs:1:N:0:<0,1> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10828 /* __builtin_fabsf */, 2380 /* ff */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10844 /* fabsf */, 2380 /* ff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10850 /* __builtin_fabs */, 2412 /* dd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10865 /* fabs */, 2412 /* dd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10870 /* __builtin_fabsl */, 2436 /* LdLd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10886 /* fabsl */, 2436 /* LdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10892 /* __builtin_fabsf16 */, 2465 /* hh */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10910 /* __builtin_fabsf128 */, 2487 /* LLdLLd */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10929 /* __fastfail */, 10940 /* vUi */, 10944 /* nr */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10947 /* __builtin_fdimf */, 3461 /* fff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10963 /* fdimf */, 3461 /* fff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10969 /* __builtin_fdim */, 3488 /* ddd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10984 /* fdim */, 3488 /* ddd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{10989 /* __builtin_fdiml */, 3515 /* LdLdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11005 /* fdiml */, 3515 /* LdLdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11011 /* __builtin_fdimf128 */, 3572 /* LLdLLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11030 /* finitef */, 11038 /* if */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11041 /* finite */, 11048 /* id */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11051 /* finitel */, 11059 /* iLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11063 /* __builtin_floorf */, 2380 /* ff */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11080 /* floorf */, 2380 /* ff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11087 /* __builtin_floor */, 2412 /* dd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11103 /* floor */, 2412 /* dd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11109 /* __builtin_floorl */, 2436 /* LdLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11126 /* floorl */, 2436 /* LdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11133 /* __builtin_floorf16 */, 2465 /* hh */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11152 /* __builtin_floorf128 */, 2487 /* LLdLLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11172 /* __builtin_fmaf */, 11187 /* ffff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11192 /* fmaf */, 11187 /* ffff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11197 /* __builtin_fma */, 11211 /* dddd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11216 /* fma */, 11211 /* dddd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11220 /* __builtin_fmal */, 11235 /* LdLdLdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11244 /* fmal */, 11235 /* LdLdLdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11249 /* __builtin_fmaf16 */, 11266 /* hhhh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11271 /* __builtin_fmaf128 */, 11289 /* LLdLLdLLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11302 /* __builtin_fmaxf */, 3461 /* fff */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11318 /* fmaxf */, 3461 /* fff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11324 /* __builtin_fmax */, 3488 /* ddd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11339 /* fmax */, 3488 /* ddd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11344 /* __builtin_fmaxl */, 3515 /* LdLdLd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11360 /* fmaxl */, 3515 /* LdLdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11366 /* __builtin_fmaxf16 */, 3548 /* hhh */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11384 /* __builtin_fmaxf128 */, 3572 /* LLdLLdLLd */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11403 /* __builtin_fmaximum_numf */, 3461 /* fff */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11427 /* fmaximum_numf */, 3461 /* fff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11441 /* __builtin_fmaximum_num */, 3488 /* ddd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11464 /* fmaximum_num */, 3488 /* ddd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11477 /* __builtin_fmaximum_numl */, 3515 /* LdLdLd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11501 /* fmaximum_numl */, 3515 /* LdLdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11515 /* __builtin_fmaximum_numf16 */, 3548 /* hhh */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11541 /* __builtin_fmaximum_numf128 */, 3572 /* LLdLLdLLd */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11568 /* __builtin_fminf */, 3461 /* fff */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11584 /* fminf */, 3461 /* fff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11590 /* __builtin_fmin */, 3488 /* ddd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11605 /* fmin */, 3488 /* ddd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11610 /* __builtin_fminl */, 3515 /* LdLdLd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11626 /* fminl */, 3515 /* LdLdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11632 /* __builtin_fminf16 */, 3548 /* hhh */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11650 /* __builtin_fminf128 */, 3572 /* LLdLLdLLd */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11669 /* __builtin_fminimum_numf */, 3461 /* fff */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11693 /* fminimum_numf */, 3461 /* fff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11707 /* __builtin_fminimum_num */, 3488 /* ddd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11730 /* fminimum_num */, 3488 /* ddd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11743 /* __builtin_fminimum_numl */, 3515 /* LdLdLd */, 2297 /* FEnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11767 /* fminimum_numl */, 3515 /* LdLdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11781 /* __builtin_fminimum_numf16 */, 3548 /* hhh */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11807 /* __builtin_fminimum_numf128 */, 3572 /* LLdLLdLLd */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11834 /* __builtin_fmodf */, 3461 /* fff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11850 /* fmodf */, 3461 /* fff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11856 /* __builtin_fmod */, 3488 /* ddd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11871 /* fmod */, 3488 /* ddd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11876 /* __builtin_fmodl */, 3515 /* LdLdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11892 /* fmodl */, 3515 /* LdLdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11898 /* __builtin_fmodf16 */, 3548 /* hhh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11916 /* __builtin_fmodf128 */, 3572 /* LLdLLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11935 /* fopen */, 11941 /* P*cC*cC* */, 2878 /* f */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11950 /* forward */, 3120 /* v&v& */, 2799 /* fzncThE */, 0}, HeaderDesc::UTILITY, CXX_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{11958 /* forward_like */, 3120 /* v&v& */, 2799 /* fzncThE */, 0}, HeaderDesc::UTILITY, CXX_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{11971 /* __builtin_frame_address */, 5156 /* v*IUi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{11995 /* fread */, 12001 /* zv*zzP* */, 2878 /* f */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12009 /* free */, 4765 /* vv* */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12014 /* __builtin_frexpf */, 12031 /* ffi* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12036 /* frexpf */, 12031 /* ffi* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12043 /* __builtin_frexp */, 12059 /* ddi* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12064 /* frexp */, 12059 /* ddi* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12070 /* __builtin_frexpl */, 12087 /* LdLdi* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12094 /* frexpl */, 12087 /* LdLdi* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12101 /* __builtin_frexpf16 */, 12120 /* hhi* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12125 /* __builtin_frexpf128 */, 12145 /* LLdLLdi* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12154 /* __builtin_frob_return_addr */, 7170 /* v*v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12181 /* fwrite */, 12188 /* zvC*zzP* */, 2878 /* f */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12197 /* _exit */, 3786 /* vi */, 10075 /* fr */, 0}, HeaderDesc::UNISTD_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12203 /* _longjmp */, 12212 /* vJi */, 12216 /* frT */, 0}, HeaderDesc::SETJMP_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12220 /* getcontext */, 12231 /* iK* */, 12235 /* fjT */, 0}, HeaderDesc::SETJMP_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12239 /* __builtin_get_device_side_mangled_name */, 12278 /* cC*. */, 12283 /* ncT */, 0}, HeaderDesc::NO_HEADER, CUDA_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12287 /* __GetExceptionInfo */, 12306 /* v*. */, 12310 /* zntu */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12315 /* get_kernel_max_sub_group_size_for_ndrange */, 12357 /* Ui. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_DSE},
    Builtin::Info{Builtin::Info::StrOffsets{12361 /* get_kernel_preferred_work_group_size_multiple */, 12357 /* Ui. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_DSE},
    Builtin::Info{Builtin::Info::StrOffsets{12407 /* get_kernel_sub_group_count_for_ndrange */, 12357 /* Ui. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_DSE},
    Builtin::Info{Builtin::Info::StrOffsets{12446 /* get_kernel_work_group_size */, 12357 /* Ui. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_DSE},
    Builtin::Info{Builtin::Info::StrOffsets{12473 /* get_pipe_max_packets */, 12357 /* Ui. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{12494 /* get_pipe_num_packets */, 12357 /* Ui. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{12515 /* __builtin___get_unsafe_stack_bottom */, 7268 /* v* */, 12551 /* nF */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12554 /* __builtin___get_unsafe_stack_ptr */, 7268 /* v* */, 12551 /* nF */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12587 /* __builtin___get_unsafe_stack_start */, 7268 /* v* */, 12551 /* nF */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12622 /* __builtin___get_unsafe_stack_top */, 7268 /* v* */, 12551 /* nF */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{12655 /* __builtin_get_vtable_pointer */, 7170 /* v*v* */, 12684 /* tnc */, 0}, HeaderDesc::NO_HEADER, CXX_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12688 /* __builtin_hlsl_adduint64 */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12713 /* __builtin_hlsl_all */, 525 /* b. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12732 /* __builtin_hlsl_and */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12751 /* __builtin_hlsl_any */, 525 /* b. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12770 /* __builtin_hlsl_asdouble */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12794 /* __builtin_hlsl_buffer_update_counter */, 12831 /* UZiQri */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12838 /* __builtin_hlsl_elementwise_clamp */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12871 /* __builtin_hlsl_elementwise_clip */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12903 /* __builtin_hlsl_crossf32 */, 12927 /* E3fE3fE3f */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12937 /* __builtin_hlsl_crossf16 */, 12961 /* E3hE3hE3h */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{12971 /* __builtin_hlsl_elementwise_ddx_coarse */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13009 /* __builtin_hlsl_elementwise_ddx_fine */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13045 /* __builtin_hlsl_elementwise_ddy_coarse */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13083 /* __builtin_hlsl_elementwise_ddy_fine */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13119 /* __builtin_hlsl_elementwise_degrees */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13154 /* __builtin_hlsl_dot4add_i8packed */, 13186 /* iUiUii */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13193 /* __builtin_hlsl_dot4add_u8packed */, 13225 /* UiUiUiUi */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13234 /* __builtin_hlsl_dot */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13253 /* __builtin_hlsl_elementwise_f16tof32 */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13289 /* __builtin_hlsl_elementwise_firstbithigh */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13329 /* __builtin_hlsl_elementwise_firstbitlow */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13368 /* __builtin_hlsl_elementwise_frac */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13400 /* __builtin_get_spirv_spec_constant_bool */, 13439 /* bUib */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13448 /* __builtin_get_spirv_spec_constant_char */, 13487 /* cUic */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13492 /* __builtin_get_spirv_spec_constant_short */, 13532 /* sUis */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13537 /* __builtin_get_spirv_spec_constant_int */, 13575 /* iUii */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13580 /* __builtin_get_spirv_spec_constant_longlong */, 13623 /* LLiUiLLi */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13632 /* __builtin_get_spirv_spec_constant_ushort */, 13673 /* UsUiUs */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13680 /* __builtin_get_spirv_spec_constant_uint */, 13719 /* UiUiUi */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13726 /* __builtin_get_spirv_spec_constant_ulonglong */, 13770 /* ULLiUiULLi */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13781 /* __builtin_get_spirv_spec_constant_half */, 13820 /* hUih */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13825 /* __builtin_get_spirv_spec_constant_float */, 13865 /* fUif */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13870 /* __builtin_get_spirv_spec_constant_double */, 13911 /* dUid */, 13444 /* ncU */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13916 /* __builtin_hlsl_group_memory_barrier_with_group_sync */, 2264 /* v */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{13968 /* __builtin_hlsl_elementwise_isinf */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14001 /* __builtin_hlsl_elementwise_isnan */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14034 /* __builtin_hlsl_lerp */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14054 /* __builtin_hlsl_mad */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14073 /* __builtin_hlsl_normalize */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14098 /* __builtin_hlsl_or */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14116 /* __builtin_hlsl_elementwise_rsqrt */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14149 /* __builtin_hlsl_elementwise_radians */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14184 /* __builtin_hlsl_elementwise_rcp */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14215 /* __builtin_hlsl_resource_counterhandlefromimplicitbinding */, 14272 /* QrQrUZiUZi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14283 /* __builtin_hlsl_resource_getdimensions_x */, 14323 /* vQrUZi& */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14331 /* __builtin_hlsl_resource_getpointer */, 20 /* v. */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14366 /* __builtin_hlsl_resource_getstride */, 14323 /* vQrUZi& */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14400 /* __builtin_hlsl_resource_handlefrombinding */, 14442 /* QrQrUZiUZiZiUZicC* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14461 /* __builtin_hlsl_resource_handlefromimplicitbinding */, 14442 /* QrQrUZiUZiZiUZicC* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14511 /* __builtin_hlsl_resource_load_with_status */, 20 /* v. */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14552 /* __builtin_hlsl_resource_nonuniformindex */, 3873 /* UZiUZi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14592 /* __builtin_hlsl_resource_uninitializedhandle */, 14636 /* QrQr */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14641 /* __builtin_hlsl_elementwise_saturate */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14677 /* __builtin_hlsl_select */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14699 /* __builtin_hlsl_elementwise_sign */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14731 /* __builtin_hlsl_elementwise_splitdouble */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14770 /* __builtin_hlsl_step */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14790 /* __builtin_hlsl_wave_active_all_true */, 14826 /* bb */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14829 /* __builtin_hlsl_wave_active_any_true */, 14826 /* bb */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14865 /* __builtin_hlsl_wave_active_ballot */, 14899 /* E4Uib */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14905 /* __builtin_hlsl_wave_active_count_bits */, 14943 /* Uib */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14947 /* __builtin_hlsl_wave_active_max */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{14978 /* __builtin_hlsl_wave_active_min */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{15009 /* __builtin_hlsl_wave_active_sum */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{15040 /* __builtin_hlsl_wave_get_lane_count */, 8344 /* Ui */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{15075 /* __builtin_hlsl_wave_get_lane_index */, 8344 /* Ui */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{15110 /* __builtin_hlsl_wave_is_first_lane */, 7147 /* b */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{15144 /* __builtin_hlsl_wave_read_lane_at */, 20 /* v. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, HLSL_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{15177 /* __builtin_huge_valf */, 2878 /* f */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15197 /* __builtin_huge_val */, 15216 /* d */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15218 /* __builtin_huge_vall */, 15238 /* Ld */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15241 /* __builtin_huge_valf128 */, 15264 /* LLd */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15268 /* __builtin_huge_valf16 */, 15290 /* x */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15292 /* __builtin_hypotf */, 3461 /* fff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15309 /* hypotf */, 3461 /* fff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15316 /* __builtin_hypot */, 3488 /* ddd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15332 /* hypot */, 3488 /* ddd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15338 /* __builtin_hypotl */, 3515 /* LdLdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15355 /* hypotl */, 3515 /* LdLdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15362 /* __builtin_hypotf128 */, 3572 /* LLdLLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15382 /* __builtin_ilogbf128 */, 15402 /* iLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15407 /* __builtin_ilogbf */, 11038 /* if */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15424 /* ilogbf */, 11038 /* if */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15431 /* __builtin_ilogb */, 11048 /* id */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15447 /* ilogb */, 11048 /* id */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15453 /* __builtin_ilogbl */, 11059 /* iLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15470 /* ilogbl */, 11059 /* iLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15477 /* __builtin_index */, 15493 /* c*cC*i */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15500 /* index */, 15493 /* c*cC*i */, 2911 /* fn */, 0}, HeaderDesc::STRINGS_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15506 /* __builtin_inff */, 2878 /* f */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15521 /* __builtin_inf */, 15216 /* d */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15535 /* __builtin_infl */, 15238 /* Ld */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15550 /* __builtin_inff128 */, 15264 /* LLd */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15568 /* __builtin_inff16 */, 15290 /* x */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15585 /* __builtin_infer_alloc_token */, 15613 /* z. */, 15616 /* ncUtEu */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15623 /* __builtin_init_dwarf_reg_size_table */, 4765 /* vv* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15659 /* _InterlockedAnd8 */, 15676 /* ccD*c */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15682 /* _InterlockedAnd16 */, 15700 /* ssD*s */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15706 /* _InterlockedAnd */, 15722 /* NiNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15731 /* _interlockedbittestandreset */, 15759 /* UcNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15768 /* _interlockedbittestandreset64 */, 15798 /* UcWiD*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15807 /* _interlockedbittestandreset64_acq */, 15798 /* UcWiD*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15841 /* _interlockedbittestandreset64_nf */, 15798 /* UcWiD*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15874 /* _interlockedbittestandreset64_rel */, 15798 /* UcWiD*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15908 /* _interlockedbittestandreset_acq */, 15759 /* UcNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15940 /* _interlockedbittestandreset_nf */, 15759 /* UcNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{15971 /* _interlockedbittestandreset_rel */, 15759 /* UcNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16003 /* _interlockedbittestandset */, 15759 /* UcNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16029 /* _interlockedbittestandset64 */, 15798 /* UcWiD*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16057 /* _interlockedbittestandset64_acq */, 15798 /* UcWiD*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16089 /* _interlockedbittestandset64_nf */, 15798 /* UcWiD*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16120 /* _interlockedbittestandset64_rel */, 15798 /* UcWiD*Wi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16152 /* _interlockedbittestandset_acq */, 15759 /* UcNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16182 /* _interlockedbittestandset_nf */, 15759 /* UcNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16211 /* _interlockedbittestandset_rel */, 15759 /* UcNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16241 /* _InterlockedCompareExchange8 */, 16270 /* ccD*cc */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16277 /* _InterlockedCompareExchange16 */, 16307 /* ssD*ss */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16314 /* _InterlockedCompareExchange */, 16342 /* NiNiD*NiNi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16353 /* _InterlockedCompareExchange64 */, 16383 /* LLiLLiD*LLiLLi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16398 /* _InterlockedCompareExchangePointer */, 16433 /* v*v*D*v*v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16444 /* _InterlockedCompareExchangePointer_nf */, 16433 /* v*v*D*v*v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16482 /* _InterlockedDecrement16 */, 16506 /* ssD* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16511 /* _InterlockedDecrement */, 16533 /* NiNiD* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16540 /* _InterlockedExchange8 */, 15676 /* ccD*c */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16562 /* _InterlockedExchange16 */, 15700 /* ssD*s */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16585 /* _InterlockedExchange */, 15722 /* NiNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16606 /* _InterlockedExchangeAdd8 */, 15676 /* ccD*c */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16631 /* _InterlockedExchangeAdd16 */, 15700 /* ssD*s */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16657 /* _InterlockedExchangeAdd */, 15722 /* NiNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16681 /* _InterlockedExchangePointer */, 16709 /* v*v*D*v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16718 /* _InterlockedExchangeSub8 */, 15676 /* ccD*c */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16743 /* _InterlockedExchangeSub16 */, 15700 /* ssD*s */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16769 /* _InterlockedExchangeSub */, 15722 /* NiNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16793 /* _InterlockedIncrement16 */, 16506 /* ssD* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16817 /* _InterlockedIncrement */, 16533 /* NiNiD* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16839 /* _InterlockedOr8 */, 15676 /* ccD*c */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16855 /* _InterlockedOr16 */, 15700 /* ssD*s */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16872 /* _InterlockedOr */, 15722 /* NiNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16887 /* _InterlockedXor8 */, 15676 /* ccD*c */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16904 /* _InterlockedXor16 */, 15700 /* ssD*s */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16922 /* _InterlockedXor */, 15722 /* NiNiD*Ni */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16938 /* __builtin_invoke */, 20 /* v. */, 3108 /* tE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16955 /* isalnum */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16967 /* __builtin_is_aligned */, 16988 /* bvC*z */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{16994 /* isalpha */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17002 /* isblank */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17010 /* iscntrl */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17018 /* __builtin_is_constant_evaluated */, 7147 /* b */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, CXX_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{17050 /* isdigit */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17058 /* __builtin_isfpclass */, 3922 /* i. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17078 /* __builtin_isfinite */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17097 /* isgraph */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17105 /* __builtin_isgreater */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17125 /* __builtin_isgreaterequal */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17150 /* __builtin_isinf */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17166 /* __builtin_isinf_sign */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17187 /* __builtin_isless */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17204 /* __builtin_islessequal */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17226 /* __builtin_islessgreater */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17250 /* islower */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17258 /* __builtin_isnan */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17274 /* __builtin_isnormal */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17293 /* isprint */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17301 /* ispunct */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17309 /* __builtin_issignaling */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17331 /* isspace */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17339 /* __builtin_issubnormal */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17361 /* __builtin_isunordered */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17383 /* isupper */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17391 /* __builtin_is_within_lifetime */, 7218 /* bv* */, 17420 /* ntEG */, 0}, HeaderDesc::NO_HEADER, CXX_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{17425 /* isxdigit */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17434 /* __builtin_iszero */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17451 /* __iso_volatile_load8 */, 17472 /* ccCD* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17478 /* __iso_volatile_load16 */, 17500 /* ssCD* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17506 /* __iso_volatile_load32 */, 17528 /* iiCD* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17534 /* __iso_volatile_load64 */, 17556 /* LLiLLiCD* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17566 /* __iso_volatile_store8 */, 17588 /* vcD*c */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17594 /* __iso_volatile_store16 */, 17617 /* vsD*s */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17623 /* __iso_volatile_store32 */, 17646 /* viD*i */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17652 /* __iso_volatile_store64 */, 17675 /* vLLiD*LLi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17685 /* __builtin_llrintf128 */, 17706 /* LLiLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17713 /* __builtin_llroundf128 */, 17706 /* LLiLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17735 /* __builtin_launder */, 7170 /* v*v* */, 2607 /* ntE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17753 /* __builtin_ldexpf */, 17770 /* ffi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17774 /* ldexpf */, 17770 /* ffi */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17781 /* __builtin_ldexp */, 17797 /* ddi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17801 /* ldexp */, 17797 /* ddi */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17807 /* __builtin_ldexpl */, 17824 /* LdLdi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17830 /* ldexpl */, 17824 /* LdLdi */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17837 /* __builtin_ldexpf16 */, 17856 /* hhi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17860 /* __builtin_ldexpf128 */, 17880 /* LLdLLdi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17888 /* __builtin_lgammaf */, 2380 /* ff */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17906 /* lgammaf */, 2380 /* ff */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17914 /* __builtin_lgamma */, 2412 /* dd */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17931 /* lgamma */, 2412 /* dd */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17938 /* __builtin_lgammal */, 2436 /* LdLd */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17956 /* lgammal */, 2436 /* LdLd */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17964 /* __builtin_lgammaf128 */, 2487 /* LLdLLd */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{17985 /* __builtin_llrintf */, 18003 /* LLif */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18008 /* llrintf */, 18003 /* LLif */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18016 /* __builtin_llrint */, 18033 /* LLid */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18038 /* llrint */, 18033 /* LLid */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18045 /* __builtin_llrintl */, 18063 /* LLiLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18069 /* llrintl */, 18063 /* LLiLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18077 /* __builtin_llroundf */, 18003 /* LLif */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18096 /* llroundf */, 18003 /* LLif */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18105 /* __builtin_llround */, 18033 /* LLid */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18123 /* llround */, 18033 /* LLid */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18131 /* __builtin_llroundl */, 18063 /* LLiLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18150 /* llroundl */, 18063 /* LLiLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18159 /* __builtin_load_halff */, 18180 /* fhC* */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_OCL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18185 /* __builtin_load_half */, 18205 /* dhC* */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_OCL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18210 /* __builtin_logf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18225 /* logf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18230 /* __builtin_log */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18244 /* log */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18248 /* __builtin_logl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18263 /* logl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18268 /* __builtin_log1pf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18285 /* log1pf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18292 /* __builtin_log1p */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18308 /* log1p */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18314 /* __builtin_log1pl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18331 /* log1pl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18338 /* __builtin_log1pf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18358 /* __builtin_log2f */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18374 /* log2f */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18380 /* __builtin_log2 */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18395 /* log2 */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18400 /* __builtin_log2l */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18416 /* log2l */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18422 /* __builtin_log2f16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18440 /* __builtin_log2f128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18459 /* __builtin_log10f */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18476 /* log10f */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18483 /* __builtin_log10 */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18499 /* log10 */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18505 /* __builtin_log10l */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18522 /* log10l */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18529 /* __builtin_log10f16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18548 /* __builtin_log10f128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18568 /* __builtin_logf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18585 /* __builtin_logf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18603 /* __builtin_logbf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18619 /* logbf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18625 /* __builtin_logb */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18640 /* logb */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18645 /* __builtin_logbl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18661 /* logbl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18667 /* __builtin_logbf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18686 /* longjmp */, 12212 /* vJi */, 12216 /* frT */, 0}, HeaderDesc::SETJMP_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18694 /* __builtin_lrintf */, 18711 /* Lif */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18715 /* lrintf */, 18711 /* Lif */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18722 /* __builtin_lrint */, 18738 /* Lid */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18742 /* lrint */, 18738 /* Lid */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18748 /* __builtin_lrintl */, 18765 /* LiLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18770 /* lrintl */, 18765 /* LiLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18777 /* __builtin_lrintf128 */, 18797 /* LiLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18803 /* _lrotl */, 18810 /* ULiULii */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18818 /* _lrotr */, 18810 /* ULiULii */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18825 /* __builtin_lroundf */, 18711 /* Lif */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18843 /* lroundf */, 18711 /* Lif */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18851 /* __builtin_lround */, 18738 /* Lid */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18868 /* lround */, 18738 /* Lid */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18875 /* __builtin_lroundl */, 18765 /* LiLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18893 /* lroundl */, 18765 /* LiLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18901 /* __builtin_lroundf128 */, 18797 /* LiLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18922 /* __annotation */, 18935 /* wC*. */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18940 /* __assume */, 3333 /* vb */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18949 /* _byteswap_ushort */, 3846 /* UsUs */, 2306 /* fnc */, 0}, HeaderDesc::STDLIB_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18966 /* _byteswap_ulong */, 18982 /* UNiUNi */, 2306 /* fnc */, 0}, HeaderDesc::STDLIB_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{18989 /* _byteswap_uint64 */, 19006 /* ULLiULLi */, 2306 /* fnc */, 0}, HeaderDesc::STDLIB_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19015 /* __lzcnt16 */, 3846 /* UsUs */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19025 /* __lzcnt */, 19033 /* UiUi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19038 /* __lzcnt64 */, 3898 /* UWiUWi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19048 /* __popcnt16 */, 3846 /* UsUs */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19059 /* __popcnt */, 19033 /* UiUi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19068 /* __popcnt64 */, 3898 /* UWiUWi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19079 /* _ReturnAddress */, 7268 /* v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19094 /* __builtin_ms_va_copy */, 19115 /* vc*&c*& */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19123 /* __builtin_ms_va_end */, 19143 /* vc*& */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19148 /* __builtin_ms_va_start */, 19170 /* vc*&. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19176 /* __va_start */, 19187 /* vc**. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19193 /* malloc */, 2897 /* v*z */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19200 /* __builtin_masked_compress_store */, 20 /* v. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19232 /* __builtin_masked_expand_load */, 20 /* v. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19261 /* __builtin_masked_gather */, 20 /* v. */, 19285 /* nUt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19289 /* __builtin_masked_load */, 20 /* v. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19311 /* __builtin_masked_scatter */, 20 /* v. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19336 /* __builtin_masked_store */, 20 /* v. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19359 /* __builtin_matrix_column_major_load */, 20 /* v. */, 19394 /* nFt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19398 /* __builtin_matrix_column_major_store */, 20 /* v. */, 19394 /* nFt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19434 /* __builtin_matrix_transpose */, 20 /* v. */, 19394 /* nFt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19461 /* memalign */, 2873 /* v*zz */, 2878 /* f */, 0}, HeaderDesc::MALLOC_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19470 /* __builtin_memchr */, 19487 /* v*vC*iz */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19495 /* memchr */, 19487 /* v*vC*iz */, 19502 /* fnE */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19506 /* memcmp */, 3816 /* ivC*vC*z */, 3825 /* fE */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19513 /* __builtin_memcpy */, 4905 /* v*v*vC*z */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19530 /* memcpy */, 4905 /* v*v*vC*z */, 19502 /* fnE */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19537 /* __builtin_memmove */, 4905 /* v*v*vC*z */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19555 /* memmove */, 4905 /* v*v*vC*z */, 19502 /* fnE */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19563 /* __builtin_memset */, 19580 /* v*v*iz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19587 /* memset */, 19580 /* v*v*iz */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19594 /* memccpy */, 19602 /* v*v*vC*iz */, 2878 /* f */, 0}, HeaderDesc::STRING_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19612 /* __builtin___memccpy_chk */, 19636 /* v*v*vC*izz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19647 /* __builtin___memcpy_chk */, 19670 /* v*v*vC*zz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19680 /* __builtin___memmove_chk */, 19670 /* v*v*vC*zz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19704 /* mempcpy */, 4905 /* v*v*vC*z */, 2878 /* f */, 0}, HeaderDesc::STRING_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19712 /* __builtin___mempcpy_chk */, 19670 /* v*v*vC*zz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19736 /* __builtin___memset_chk */, 19759 /* v*v*izz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19767 /* __builtin_modff */, 19783 /* fff* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19788 /* modff */, 19783 /* fff* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19794 /* __builtin_modf */, 19809 /* ddd* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19814 /* modf */, 19809 /* ddd* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19819 /* __builtin_modfl */, 19835 /* LdLdLd* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19843 /* modfl */, 19835 /* LdLdLd* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19849 /* __builtin_modff128 */, 19868 /* LLdLLdLLd* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19879 /* move */, 3120 /* v&v& */, 2799 /* fzncThE */, 0}, HeaderDesc::UTILITY, CXX_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{19884 /* move_if_noexcept */, 3120 /* v&v& */, 2799 /* fzncThE */, 0}, HeaderDesc::UTILITY, CXX_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{19901 /* __builtin_mul_overflow */, 525 /* b. */, 2607 /* ntE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19924 /* NSLog */, 19930 /* vG. */, 19934 /* fp:0: */, 0}, HeaderDesc::FOUNDATION_NSOBJCRUNTIME_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{19940 /* NSLogv */, 19947 /* vGa */, 19951 /* fP:0: */, 0}, HeaderDesc::FOUNDATION_NSOBJCRUNTIME_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{19957 /* __builtin_nanf */, 19972 /* fcC* */, 19977 /* FEUn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19982 /* nanf */, 19972 /* fcC* */, 19987 /* fUn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{19991 /* __builtin_nan */, 20005 /* dcC* */, 19977 /* FEUn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20010 /* nan */, 20005 /* dcC* */, 19987 /* fUn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20014 /* __builtin_nanl */, 20029 /* LdcC* */, 19977 /* FEUn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20035 /* nanl */, 20029 /* LdcC* */, 19987 /* fUn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20040 /* __builtin_nanf16 */, 20057 /* xcC* */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20067 /* __builtin_nanf128 */, 20085 /* LLdcC* */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20092 /* __builtin_nansf */, 19972 /* fcC* */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20108 /* __builtin_nans */, 20005 /* dcC* */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20123 /* __builtin_nansl */, 20029 /* LdcC* */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20139 /* __builtin_nansf16 */, 20057 /* xcC* */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20157 /* __builtin_nansf128 */, 20085 /* LLdcC* */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20176 /* __builtin_nearbyintf */, 2380 /* ff */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20197 /* nearbyintf */, 2380 /* ff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20208 /* __builtin_nearbyint */, 2412 /* dd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20228 /* nearbyint */, 2412 /* dd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20238 /* __builtin_nearbyintl */, 2436 /* LdLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20259 /* nearbyintl */, 2436 /* LdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20270 /* __builtin_nearbyintf128 */, 2487 /* LLdLLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20294 /* __builtin_nextafterf */, 3461 /* fff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20315 /* nextafterf */, 3461 /* fff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20326 /* __builtin_nextafter */, 3488 /* ddd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20346 /* nextafter */, 3488 /* ddd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20356 /* __builtin_nextafterl */, 3515 /* LdLdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20377 /* nextafterl */, 3515 /* LdLdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20388 /* __builtin_nextafterf128 */, 3572 /* LLdLLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20412 /* __builtin_nexttowardf */, 20434 /* ffLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20439 /* nexttowardf */, 20434 /* ffLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20451 /* __builtin_nexttoward */, 20472 /* ddLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20477 /* nexttoward */, 20472 /* ddLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20488 /* __builtin_nexttowardl */, 3515 /* LdLdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20510 /* nexttowardl */, 3515 /* LdLdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20522 /* __builtin_nexttowardf128 */, 3572 /* LLdLLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20547 /* __builtin_nondeterministic_value */, 20 /* v. */, 59 /* nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20580 /* __builtin_nontemporal_load */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20607 /* __builtin_nontemporal_store */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20635 /* __noop */, 3922 /* i. */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20642 /* __builtin_os_log_format */, 20666 /* v*v*cC*. */, 20675 /* p:0:nt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20682 /* __builtin_os_log_format_buffer_size */, 20718 /* zcC*. */, 20724 /* p:0:nutE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20733 /* __exp10f */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20742 /* __exp10 */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20750 /* __finitef */, 11038 /* if */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20760 /* __finite */, 11048 /* id */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20769 /* __finitel */, 11059 /* iLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{20779 /* objc_assign_global */, 20798 /* GGG* */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_AUTO_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{20803 /* objc_assign_ivar */, 20820 /* GGGY */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_AUTO_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{20825 /* objc_assign_strongCast */, 20798 /* GGG* */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_AUTO_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{20848 /* objc_assign_weak */, 20798 /* GGG* */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_AUTO_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{20865 /* objc_enumerationMutation */, 20890 /* vG */, 2878 /* f */, 0}, HeaderDesc::OBJC_RUNTIME_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{20893 /* objc_exception_extract */, 20916 /* Gv* */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_EXCEPTION_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{20920 /* objc_exception_match */, 20941 /* iGG */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_EXCEPTION_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{20945 /* objc_exception_throw */, 20890 /* vG */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_EXCEPTION_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{20966 /* objc_exception_try_enter */, 4765 /* vv* */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_EXCEPTION_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{20991 /* objc_exception_try_exit */, 4765 /* vv* */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_EXCEPTION_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21015 /* objc_getClass */, 21029 /* GcC* */, 2878 /* f */, 0}, HeaderDesc::OBJC_RUNTIME_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21034 /* objc_getMetaClass */, 21029 /* GcC* */, 2878 /* f */, 0}, HeaderDesc::OBJC_RUNTIME_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21052 /* __builtin_objc_memmove_collectable */, 4905 /* v*v*vC*z */, 12551 /* nF */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21087 /* objc_msgSend */, 21100 /* GGH. */, 2878 /* f */, 0}, HeaderDesc::OBJC_MESSAGE_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21105 /* objc_msgSend_fp2ret */, 21125 /* XLdGH. */, 2878 /* f */, 0}, HeaderDesc::OBJC_MESSAGE_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21132 /* objc_msgSend_fpret */, 21151 /* LdGH. */, 2878 /* f */, 0}, HeaderDesc::OBJC_MESSAGE_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21157 /* objc_msgSend_stret */, 21176 /* vGH. */, 2878 /* f */, 0}, HeaderDesc::OBJC_MESSAGE_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21181 /* objc_msgSendSuper */, 21199 /* GM*H. */, 2878 /* f */, 0}, HeaderDesc::OBJC_MESSAGE_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21205 /* objc_msgSendSuper_stret */, 21229 /* vM*H. */, 2878 /* f */, 0}, HeaderDesc::OBJC_MESSAGE_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21235 /* objc_read_weak */, 21250 /* GG* */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_AUTO_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21254 /* objc_sync_enter */, 21270 /* iG */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_SYNC_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21273 /* objc_sync_exit */, 21270 /* iG */, 2878 /* f */, 0}, HeaderDesc::OBJC_OBJC_SYNC_H, OBJC_LANG},
    Builtin::Info{Builtin::Info::StrOffsets{21288 /* __builtin_object_size */, 8430 /* zvC*i */, 8436 /* nuE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21310 /* __builtin_parity */, 6778 /* iUi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21327 /* __builtin_parityl */, 6797 /* iULi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21345 /* __builtin_parityll */, 6818 /* iULLi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21364 /* __builtin_popcount */, 6778 /* iUi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21383 /* __builtin_popcountl */, 6797 /* iULi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21403 /* __builtin_popcountll */, 6818 /* iULLi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21424 /* __builtin_popcountg */, 3922 /* i. */, 3925 /* ncEt */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21444 /* __builtin_powf */, 3461 /* fff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21459 /* powf */, 3461 /* fff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21464 /* __builtin_pow */, 3488 /* ddd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21478 /* pow */, 3488 /* ddd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21482 /* __builtin_powl */, 3515 /* LdLdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21497 /* powl */, 3515 /* LdLdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21502 /* __builtin_powf16 */, 3548 /* hhh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21519 /* __builtin_powf128 */, 3572 /* LLdLLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21537 /* __builtin_powif */, 17770 /* ffi */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21553 /* __builtin_powi */, 17797 /* ddi */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21568 /* __builtin_powil */, 17824 /* LdLdi */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21584 /* __builtin_prefetch */, 21603 /* vvC*. */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21609 /* printf */, 21616 /* icC*. */, 21622 /* fp:0:N:0:<0> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21635 /* __builtin___printf_chk */, 21658 /* iicC*R. */, 21666 /* Fp:1: */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21672 /* pthread_create */, 0 /*  */, 21687 /* ffC<2,3> */, 0}, HeaderDesc::PTHREAD_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21696 /* __builtin_ptrauth_auth */, 21719 /* v*v*iv* */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21727 /* __builtin_ptrauth_auth_and_resign */, 21761 /* v*v*iv*iv* */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21772 /* __builtin_ptrauth_blend_discriminator */, 21810 /* zv*i */, 12684 /* tnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21815 /* __builtin_ptrauth_sign_constant */, 21719 /* v*v*iv* */, 21847 /* tncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21852 /* __builtin_ptrauth_sign_generic_data */, 21888 /* zv*v* */, 12684 /* tnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21894 /* __builtin_ptrauth_sign_unauthenticated */, 21719 /* v*v*iv* */, 12684 /* tnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21933 /* __builtin_ptrauth_string_discriminator */, 21972 /* zcC* */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{21977 /* __builtin_ptrauth_strip */, 22001 /* v*v*i */, 12684 /* tnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22007 /* __builtin_readcyclecounter */, 22034 /* ULLi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22039 /* read_pipe */, 3922 /* i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{22049 /* __builtin_readsteadycounter */, 22034 /* ULLi */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22077 /* realloc */, 5125 /* v*v*z */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22085 /* __builtin_reduce_add */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22106 /* __builtin_reduce_and */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22127 /* __builtin_reduce_max */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22148 /* __builtin_reduce_maximum */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22173 /* __builtin_reduce_min */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22194 /* __builtin_reduce_minimum */, 20 /* v. */, 4361 /* nct */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22219 /* __builtin_reduce_mul */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22240 /* __builtin_reduce_or */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22260 /* __builtin_reduce_xor */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22281 /* __builtin_remainderf */, 3461 /* fff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22302 /* remainderf */, 3461 /* fff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22313 /* __builtin_remainder */, 3488 /* ddd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22333 /* remainder */, 3488 /* ddd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22343 /* __builtin_remainderl */, 3515 /* LdLdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22364 /* remainderl */, 3515 /* LdLdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22375 /* __builtin_remainderf128 */, 3572 /* LLdLLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22399 /* __builtin_remquof */, 22417 /* fffi* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22423 /* remquof */, 22417 /* fffi* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22431 /* __builtin_remquo */, 22448 /* dddi* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22454 /* remquo */, 22448 /* dddi* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22461 /* __builtin_remquol */, 22479 /* LdLdLdi* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22488 /* remquol */, 22479 /* LdLdLdi* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22496 /* __builtin_remquof128 */, 22517 /* LLdLLdLLdi* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22529 /* reserve_read_pipe */, 3922 /* i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{22547 /* reserve_write_pipe */, 3922 /* i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{22566 /* __builtin_rindex */, 15493 /* c*cC*i */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22583 /* rindex */, 15493 /* c*cC*i */, 2911 /* fn */, 0}, HeaderDesc::STRINGS_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22590 /* __builtin_rintf */, 2380 /* ff */, 22606 /* Fng */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22610 /* rintf */, 2380 /* ff */, 22616 /* fng */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22620 /* __builtin_rint */, 2412 /* dd */, 22606 /* Fng */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22635 /* rint */, 2412 /* dd */, 22616 /* fng */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22640 /* __builtin_rintl */, 2436 /* LdLd */, 22606 /* Fng */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22656 /* rintl */, 2436 /* LdLd */, 22616 /* fng */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22662 /* __builtin_rintf16 */, 2465 /* hh */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22680 /* __builtin_rintf128 */, 2487 /* LLdLLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22699 /* __builtin_rotateleft8 */, 22721 /* UcUcUc */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22728 /* __builtin_rotateleft16 */, 22751 /* UsUsUs */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22758 /* __builtin_rotateleft32 */, 22781 /* UZiUZiUZi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22791 /* __builtin_rotateleft64 */, 22814 /* UWiUWiUWi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22824 /* __builtin_rotateright8 */, 22721 /* UcUcUc */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22847 /* __builtin_rotateright16 */, 22751 /* UsUsUs */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22871 /* __builtin_rotateright32 */, 22781 /* UZiUZiUZi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22895 /* __builtin_rotateright64 */, 22814 /* UWiUWiUWi */, 3851 /* ncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22919 /* _rotl */, 22925 /* UiUii */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22931 /* _rotl8 */, 22721 /* UcUcUc */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22938 /* _rotl16 */, 22946 /* UsUsUc */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22953 /* _rotl64 */, 22961 /* UWiUWii */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22969 /* _rotr */, 22925 /* UiUii */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22975 /* _rotr8 */, 22721 /* UcUcUc */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22982 /* _rotr16 */, 22946 /* UsUsUc */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22990 /* _rotr64 */, 22961 /* UWiUWii */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{22998 /* __builtin_roundf */, 2380 /* ff */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23015 /* roundf */, 2380 /* ff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23022 /* __builtin_round */, 2412 /* dd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23038 /* round */, 2412 /* dd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23044 /* __builtin_roundl */, 2436 /* LdLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23061 /* roundl */, 2436 /* LdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23068 /* __builtin_roundevenf */, 2380 /* ff */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23089 /* roundevenf */, 2380 /* ff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23100 /* __builtin_roundeven */, 2412 /* dd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23120 /* roundeven */, 2412 /* dd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23130 /* __builtin_roundevenl */, 2436 /* LdLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23151 /* roundevenl */, 2436 /* LdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23162 /* __builtin_roundf16 */, 2465 /* hh */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23181 /* __builtin_roundf128 */, 2487 /* LLdLLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23201 /* __builtin_roundevenf16 */, 2465 /* hh */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23224 /* __builtin_roundevenf128 */, 2487 /* LLdLLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23248 /* __builtin___snprintf_chk */, 23273 /* ic*RzizcC*R. */, 23286 /* Fp:4: */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23292 /* __builtin_sprintf */, 23310 /* ic*RcC*R. */, 10693 /* Fnp:1:N:0:<0,1> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23320 /* sprintf */, 23310 /* ic*RcC*R. */, 10717 /* fnp:1:N:0:<0,1> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23328 /* __builtin___sprintf_chk */, 23352 /* ic*RizcC*R. */, 23364 /* Fp:3: */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23370 /* __builtin_sscanf */, 23387 /* icC*RcC*R. */, 10791 /* Fs:1:N:0:<0,1> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23398 /* sscanf */, 23387 /* icC*RcC*R. */, 10813 /* fs:1:N:0:<0,1> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23405 /* __builtin_sadd_overflow */, 23429 /* biCiCi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23437 /* __builtin_saddl_overflow */, 23462 /* bLiCLiCLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23473 /* __builtin_saddll_overflow */, 23499 /* bLLiCLLiCLLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23513 /* savectx */, 23521 /* iSJ */, 12235 /* fjT */, 0}, HeaderDesc::SETJMP_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23525 /* __builtin_scalblnf */, 23544 /* ffLi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23549 /* scalblnf */, 23544 /* ffLi */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23558 /* __builtin_scalbln */, 23576 /* ddLi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23581 /* scalbln */, 23576 /* ddLi */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23589 /* __builtin_scalblnl */, 23608 /* LdLdLi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23615 /* scalblnl */, 23608 /* LdLdLi */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23624 /* __builtin_scalbnf */, 17770 /* ffi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23642 /* scalbnf */, 17770 /* ffi */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23650 /* __builtin_scalbn */, 17797 /* ddi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23667 /* scalbn */, 17797 /* ddi */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23674 /* __builtin_scalbnl */, 17824 /* LdLdi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23692 /* scalbnl */, 17824 /* LdLdi */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23700 /* __builtin_scanf */, 5086 /* icC*R. */, 23716 /* Fs:0:N:0:<0> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23729 /* scanf */, 5086 /* icC*R. */, 23735 /* fs:0:N:0:<0> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23748 /* __builtin_scalblnf128 */, 23770 /* LLdLLdLi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23779 /* __builtin_scalbnf128 */, 17880 /* LLdLLdi */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23800 /* __scoped_atomic_thread_fence */, 23829 /* vii */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23833 /* setjmp */, 23840 /* iJ */, 12235 /* fjT */, 0}, HeaderDesc::SETJMP_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23843 /* _setjmp */, 23840 /* iJ */, 12235 /* fjT */, 0}, HeaderDesc::SETJMP_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23851 /* _setjmpex */, 23840 /* iJ */, 23861 /* fTj */, 0}, HeaderDesc::SETJMPEX_H, ALL_MS_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23865 /* __builtin_shufflevector */, 20 /* v. */, 2835 /* nctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23889 /* siglongjmp */, 23900 /* vSJi */, 12216 /* frT */, 0}, HeaderDesc::SETJMP_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23905 /* sigsetjmp */, 23915 /* iSJi */, 12235 /* fjT */, 0}, HeaderDesc::SETJMP_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23920 /* __sigsetjmp */, 23915 /* iSJi */, 12235 /* fjT */, 0}, HeaderDesc::SETJMP_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23932 /* __builtin_signbit */, 3922 /* i. */, 10659 /* FnctE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23950 /* __builtin_signbitf */, 11038 /* if */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23969 /* __builtin_signbitl */, 11059 /* iLd */, 7098 /* FncE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{23988 /* __builtin_sinf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24003 /* sinf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24008 /* __builtin_sin */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24022 /* sin */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24026 /* __builtin_sinl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24041 /* sinl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24046 /* __builtin_sinf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24063 /* __builtin_sinf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24081 /* __builtin_sincosf */, 24099 /* vff*f* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24106 /* sincosf */, 24099 /* vff*f* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24114 /* __builtin_sincos */, 24131 /* vdd*d* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24138 /* sincos */, 24131 /* vdd*d* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24145 /* __builtin_sincosl */, 24163 /* vLdLd*Ld* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24173 /* sincosl */, 24163 /* vLdLd*Ld* */, 2911 /* fn */, 0}, HeaderDesc::MATH_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24181 /* __builtin_sincosf16 */, 24201 /* vhh*h* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24208 /* __builtin_sincosf128 */, 24229 /* vLLdLLd*LLd* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24242 /* __builtin_sincospif */, 24099 /* vff*f* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24262 /* __builtin_sincospi */, 24131 /* vdd*d* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24281 /* __builtin_sincospil */, 24163 /* vLdLd*Ld* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24301 /* __builtin_sinhf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24317 /* sinhf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24323 /* __builtin_sinh */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24338 /* sinh */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24343 /* __builtin_sinhl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24359 /* sinhl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24365 /* __builtin_sinhf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24383 /* __builtin_sinhf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24402 /* __sinpif */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24411 /* __sinpi */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24419 /* __builtin_smul_overflow */, 23429 /* biCiCi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24443 /* __builtin_smull_overflow */, 23462 /* bLiCLiCLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24468 /* __builtin_smulll_overflow */, 23499 /* bLLiCLLiCLLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24494 /* __builtin_snprintf */, 24513 /* ic*RzcC*R. */, 24524 /* Fnp:2:N:0:<2> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24538 /* snprintf */, 24513 /* ic*RzcC*R. */, 24547 /* fnp:2:N:0:<2> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24561 /* __builtin_sqrtf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24577 /* sqrtf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24583 /* __builtin_sqrt */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24598 /* sqrt */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24603 /* __builtin_sqrtl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24619 /* sqrtl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24625 /* __builtin_sqrtf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24643 /* __builtin_sqrtf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24662 /* __builtin_ssub_overflow */, 23429 /* biCiCi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24686 /* __builtin_ssubl_overflow */, 23462 /* bLiCLiCLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24711 /* __builtin_ssubll_overflow */, 23499 /* bLLiCLLiCLLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24737 /* __builtin_stack_address */, 7268 /* v* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24761 /* __builtin_store_halff */, 24783 /* vfh* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_OCL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24788 /* __builtin_store_half */, 24809 /* vdh* */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_OCL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24814 /* __builtin_stpcpy */, 24831 /* c*c*cC* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24839 /* stpcpy */, 24831 /* c*c*cC* */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24846 /* __builtin___stpcpy_chk */, 24869 /* c*c*cC*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24878 /* __builtin_stpncpy */, 24869 /* c*c*cC*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24896 /* stpncpy */, 24869 /* c*c*cC*z */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24904 /* __builtin___stpncpy_chk */, 24928 /* c*c*cC*zz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24938 /* __builtin_strcasecmp */, 24959 /* icC*cC* */, 24967 /* F */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24969 /* strcasecmp */, 24959 /* icC*cC* */, 2878 /* f */, 0}, HeaderDesc::STRINGS_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24980 /* __builtin_strcat */, 24831 /* c*c*cC* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{24997 /* strcat */, 24831 /* c*c*cC* */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25004 /* __builtin_strchr */, 15493 /* c*cC*i */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25021 /* strchr */, 15493 /* c*cC*i */, 19502 /* fnE */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25028 /* __builtin_strcmp */, 24959 /* icC*cC* */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25045 /* strcmp */, 24959 /* icC*cC* */, 19502 /* fnE */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25052 /* __builtin_strcpy */, 24831 /* c*c*cC* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25069 /* strcpy */, 24831 /* c*c*cC* */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25076 /* __builtin_strdup */, 25093 /* c*cC* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25099 /* strdup */, 25093 /* c*cC* */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25106 /* strerror */, 25115 /* c*i */, 2878 /* f */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25119 /* __builtin_strlen */, 21972 /* zcC* */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25136 /* strlen */, 21972 /* zcC* */, 19502 /* fnE */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25143 /* __builtin_strncat */, 24869 /* c*c*cC*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25161 /* strncat */, 24869 /* c*c*cC*z */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25169 /* __builtin_strncmp */, 25187 /* icC*cC*z */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25196 /* strncmp */, 25187 /* icC*cC*z */, 19502 /* fnE */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25204 /* __builtin_strncpy */, 24869 /* c*c*cC*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25222 /* strncpy */, 24869 /* c*c*cC*z */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25230 /* __builtin_strspn */, 5252 /* zcC*cC* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25247 /* strspn */, 5252 /* zcC*cC* */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25254 /* __builtin_strstr */, 25271 /* c*cC*cC* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25280 /* strstr */, 25271 /* c*cC*cC* */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25287 /* strtod */, 25294 /* dcC*c** */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25302 /* strtof */, 25309 /* fcC*c** */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25317 /* strtol */, 25324 /* LicC*c**i */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25334 /* strtoll */, 25342 /* LLicC*c**i */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25353 /* strtold */, 25361 /* LdcC*c** */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25370 /* strtoul */, 25378 /* ULicC*c**i */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25389 /* strtoull */, 25398 /* ULLicC*c**i */, 2878 /* f */, 0}, HeaderDesc::STDLIB_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25410 /* strtok */, 24831 /* c*c*cC* */, 2878 /* f */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25417 /* strcspn */, 5252 /* zcC*cC* */, 2878 /* f */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25425 /* __builtin___strcat_chk */, 24869 /* c*c*cC*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25448 /* __builtin___strcpy_chk */, 24869 /* c*c*cC*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25471 /* strlcat */, 25479 /* zc*cC*z */, 2878 /* f */, 0}, HeaderDesc::STRING_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25487 /* strlcpy */, 25479 /* zc*cC*z */, 2878 /* f */, 0}, HeaderDesc::STRING_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25495 /* __builtin___strlcat_chk */, 25519 /* zc*cC*zz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25528 /* __builtin___strlcpy_chk */, 25519 /* zc*cC*zz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25552 /* __builtin_strncasecmp */, 25187 /* icC*cC*z */, 24967 /* F */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25574 /* strncasecmp */, 25187 /* icC*cC*z */, 2878 /* f */, 0}, HeaderDesc::STRINGS_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25586 /* __builtin_strndup */, 25604 /* c*cC*z */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25611 /* strndup */, 25604 /* c*cC*z */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_GNU_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25619 /* __builtin___strncat_chk */, 24928 /* c*c*cC*zz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25643 /* __builtin___strncpy_chk */, 24928 /* c*c*cC*zz */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25667 /* __builtin_strpbrk */, 25271 /* c*cC*cC* */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25685 /* strpbrk */, 25271 /* c*cC*cC* */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25693 /* __builtin_strrchr */, 15493 /* c*cC*i */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25711 /* strrchr */, 15493 /* c*cC*i */, 2911 /* fn */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25719 /* strxfrm */, 25479 /* zc*cC*z */, 2878 /* f */, 0}, HeaderDesc::STRING_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25727 /* sub_group_commit_read_pipe */, 20 /* v. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{25754 /* sub_group_commit_write_pipe */, 20 /* v. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{25782 /* sub_group_reserve_read_pipe */, 3922 /* i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{25810 /* sub_group_reserve_write_pipe */, 3922 /* i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{25839 /* __builtin_sub_overflow */, 525 /* b. */, 2607 /* ntE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25862 /* __builtin_subcb */, 2627 /* UcUcCUcCUcCUc* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25878 /* __builtin_subcs */, 2661 /* UsUsCUsCUsCUs* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25894 /* __builtin_subc */, 2691 /* UiUiCUiCUiCUi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25909 /* __builtin_subcl */, 2722 /* ULiULiCULiCULiCULi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25925 /* __builtin_subcll */, 2759 /* ULLiULLiCULLiCULLiCULLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25942 /* __sync_lock_test_and_set_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{25976 /* __sync_lock_test_and_set_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26010 /* __sync_lock_test_and_set_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26044 /* __sync_lock_test_and_set_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26084 /* __sync_lock_test_and_set_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26128 /* __sync_val_compare_and_swap_1 */, 26158 /* ccD*cc. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26166 /* __sync_val_compare_and_swap_2 */, 26196 /* ssD*ss. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26204 /* __sync_val_compare_and_swap_4 */, 26234 /* iiD*ii. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26242 /* __sync_val_compare_and_swap_8 */, 26272 /* LLiLLiD*LLiLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26288 /* __sync_val_compare_and_swap_16 */, 26319 /* LLLiLLLiD*LLLiLLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26339 /* __sync_add_and_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26360 /* __sync_add_and_fetch_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26383 /* __sync_add_and_fetch_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26406 /* __sync_add_and_fetch_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26429 /* __sync_add_and_fetch_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26452 /* __sync_add_and_fetch_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26476 /* __sync_and_and_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26497 /* __sync_and_and_fetch_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26520 /* __sync_and_and_fetch_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26543 /* __sync_and_and_fetch_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26566 /* __sync_and_and_fetch_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26589 /* __sync_and_and_fetch_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26613 /* __sync_bool_compare_and_swap */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26642 /* __sync_bool_compare_and_swap_1 */, 26673 /* bcD*cc. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26681 /* __sync_bool_compare_and_swap_2 */, 26712 /* bsD*ss. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26720 /* __sync_bool_compare_and_swap_4 */, 26751 /* biD*ii. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26759 /* __sync_bool_compare_and_swap_8 */, 26790 /* bLLiD*LLiLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26804 /* __sync_bool_compare_and_swap_16 */, 26836 /* bLLLiD*LLLiLLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26853 /* __sync_fetch_and_add */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26874 /* __sync_fetch_and_add_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26897 /* __sync_fetch_and_add_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26920 /* __sync_fetch_and_add_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26943 /* __sync_fetch_and_add_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26966 /* __sync_fetch_and_add_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{26990 /* __sync_fetch_and_and */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27011 /* __sync_fetch_and_and_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27034 /* __sync_fetch_and_and_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27057 /* __sync_fetch_and_and_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27080 /* __sync_fetch_and_and_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27103 /* __sync_fetch_and_and_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27127 /* __sync_fetch_and_max */, 27148 /* iiD*i */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27154 /* __sync_fetch_and_min */, 27148 /* iiD*i */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27175 /* __sync_fetch_and_nand */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27197 /* __sync_fetch_and_nand_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27221 /* __sync_fetch_and_nand_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27245 /* __sync_fetch_and_nand_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27269 /* __sync_fetch_and_nand_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27293 /* __sync_fetch_and_nand_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27318 /* __sync_fetch_and_or */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27338 /* __sync_fetch_and_or_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27360 /* __sync_fetch_and_or_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27382 /* __sync_fetch_and_or_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27404 /* __sync_fetch_and_or_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27426 /* __sync_fetch_and_or_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27449 /* __sync_fetch_and_sub */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27470 /* __sync_fetch_and_sub_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27493 /* __sync_fetch_and_sub_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27516 /* __sync_fetch_and_sub_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27539 /* __sync_fetch_and_sub_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27562 /* __sync_fetch_and_sub_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27586 /* __sync_fetch_and_umax */, 27608 /* UiUiD*Ui */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27617 /* __sync_fetch_and_umin */, 27608 /* UiUiD*Ui */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27639 /* __sync_fetch_and_xor */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27660 /* __sync_fetch_and_xor_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27683 /* __sync_fetch_and_xor_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27706 /* __sync_fetch_and_xor_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27729 /* __sync_fetch_and_xor_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27752 /* __sync_fetch_and_xor_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27776 /* __sync_lock_release */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27796 /* __sync_lock_release_1 */, 27818 /* vcD*. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27824 /* __sync_lock_release_2 */, 27846 /* vsD*. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27852 /* __sync_lock_release_4 */, 27874 /* viD*. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27880 /* __sync_lock_release_8 */, 27902 /* vLLiD*. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27910 /* __sync_lock_release_16 */, 27933 /* vLLLiD*. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27942 /* __sync_lock_test_and_set */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27967 /* __sync_nand_and_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{27989 /* __sync_nand_and_fetch_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28013 /* __sync_nand_and_fetch_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28037 /* __sync_nand_and_fetch_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28061 /* __sync_nand_and_fetch_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28085 /* __sync_nand_and_fetch_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28110 /* __sync_or_and_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28130 /* __sync_or_and_fetch_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28152 /* __sync_or_and_fetch_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28174 /* __sync_or_and_fetch_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28196 /* __sync_or_and_fetch_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28218 /* __sync_or_and_fetch_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28241 /* __sync_sub_and_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28262 /* __sync_sub_and_fetch_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28285 /* __sync_sub_and_fetch_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28308 /* __sync_sub_and_fetch_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28331 /* __sync_sub_and_fetch_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28354 /* __sync_sub_and_fetch_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28378 /* __sync_swap */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28390 /* __sync_swap_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28404 /* __sync_swap_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28418 /* __sync_swap_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28432 /* __sync_swap_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28446 /* __sync_swap_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28461 /* __sync_synchronize */, 2264 /* v */, 2224 /* n */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28480 /* __sync_val_compare_and_swap */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28508 /* __sync_xor_and_fetch */, 20 /* v. */, 23 /* t */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28529 /* __sync_xor_and_fetch_1 */, 25969 /* ccD*c. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28552 /* __sync_xor_and_fetch_2 */, 26003 /* ssD*s. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28575 /* __sync_xor_and_fetch_4 */, 26037 /* iiD*i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28598 /* __sync_xor_and_fetch_8 */, 26071 /* LLiLLiD*LLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28621 /* __sync_xor_and_fetch_16 */, 26112 /* LLLiLLLiD*LLLi. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28645 /* __builtin_tanf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28660 /* tanf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28665 /* __builtin_tan */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28679 /* tan */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28683 /* __builtin_tanl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28698 /* tanl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28703 /* __builtin_tanf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28720 /* __builtin_tanf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28738 /* __builtin_tanhf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28754 /* tanhf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28760 /* __builtin_tanh */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28775 /* tanh */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28780 /* __builtin_tanhl */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28796 /* tanhl */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28802 /* __builtin_tanhf16 */, 2465 /* hh */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28820 /* __builtin_tanhf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28839 /* __tanpif */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28848 /* __tanpi */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28856 /* __builtin_tgammaf */, 2380 /* ff */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28874 /* tgammaf */, 2380 /* ff */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28882 /* __builtin_tgamma */, 2412 /* dd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28899 /* tgamma */, 2412 /* dd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28906 /* __builtin_tgammal */, 2436 /* LdLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28924 /* tgammal */, 2436 /* LdLd */, 2393 /* fne */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28932 /* __builtin_tgammaf128 */, 2487 /* LLdLLd */, 2383 /* Fne */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28953 /* __builtin_thread_pointer */, 7268 /* v* */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{28978 /* to_global */, 7170 /* v*v* */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_GAS},
    Builtin::Info{Builtin::Info::StrOffsets{28988 /* to_local */, 7170 /* v*v* */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_GAS},
    Builtin::Info{Builtin::Info::StrOffsets{28997 /* tolower */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29005 /* to_private */, 7170 /* v*v* */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_GAS},
    Builtin::Info{Builtin::Info::StrOffsets{29016 /* toupper */, 2294 /* ii */, 16963 /* fnU */, 0}, HeaderDesc::CTYPE_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29024 /* __builtin_trap */, 2264 /* v */, 10944 /* nr */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29039 /* __builtin_truncf */, 2380 /* ff */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29056 /* truncf */, 2380 /* ff */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29063 /* __builtin_trunc */, 2412 /* dd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29079 /* trunc */, 2412 /* dd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29085 /* __builtin_truncl */, 2436 /* LdLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29102 /* truncl */, 2436 /* LdLd */, 2306 /* fnc */, 0}, HeaderDesc::MATH_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29109 /* __builtin_truncf16 */, 2465 /* hh */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29128 /* __builtin_truncf128 */, 2487 /* LLdLLd */, 6151 /* Fnc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29148 /* __builtin_uadd_overflow */, 29172 /* bUiCUiCUi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29183 /* __builtin_uaddl_overflow */, 29208 /* bULiCULiCULi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29222 /* __builtin_uaddll_overflow */, 29248 /* bULLiCULLiCULLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29265 /* __builtin_umul_overflow */, 29172 /* bUiCUiCUi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29289 /* __builtin_umull_overflow */, 29208 /* bULiCULiCULi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29314 /* __builtin_umulll_overflow */, 29248 /* bULLiCULLiCULLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29340 /* __builtin_unpredictable */, 2325 /* LiLi */, 4437 /* nc */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29364 /* __builtin_unreachable */, 2264 /* v */, 10944 /* nr */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29386 /* __builtin_unwind_init */, 2264 /* v */, 0 /*  */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29408 /* __builtin_usub_overflow */, 29172 /* bUiCUiCUi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29432 /* __builtin_usubl_overflow */, 29208 /* bULiCULiCULi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29457 /* __builtin_usubll_overflow */, 29248 /* bULLiCULLiCULLi* */, 2642 /* nE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29483 /* __builtin___vfprintf_chk */, 29508 /* iP*RicC*Ra */, 29519 /* FP:2: */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29525 /* __builtin_vfscanf */, 29543 /* iP*RcC*Ra */, 29553 /* FS:1:N:0:<0,1> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29568 /* vfscanf */, 29543 /* iP*RcC*Ra */, 29576 /* fS:1:N:0:<0,1> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29591 /* vfork */, 29597 /* p */, 12235 /* fjT */, 0}, HeaderDesc::UNISTD_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29599 /* __builtin_vprintf */, 29617 /* icC*Ra */, 29624 /* FnP:0:N:0:<0> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29638 /* vprintf */, 29617 /* icC*Ra */, 29646 /* fnP:0:N:0:<0> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29660 /* __builtin___vprintf_chk */, 29684 /* iicC*Ra */, 29692 /* FP:1: */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29698 /* __builtin___vsnprintf_chk */, 29724 /* ic*RzizcC*Ra */, 29737 /* FP:4: */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29743 /* __builtin___vsprintf_chk */, 29768 /* ic*RizcC*Ra */, 29780 /* FP:3: */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29786 /* __builtin_vsscanf */, 29804 /* icC*RcC*Ra */, 29553 /* FS:1:N:0:<0,1> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29815 /* vsscanf */, 29804 /* icC*RcC*Ra */, 29576 /* fS:1:N:0:<0,1> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29823 /* __builtin_vscanf */, 29617 /* icC*Ra */, 29840 /* FS:0:N:0:<0> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29853 /* vscanf */, 29617 /* icC*Ra */, 29860 /* fS:0:N:0:<0> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29873 /* __builtin_va_copy */, 29891 /* vAA */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29895 /* va_copy */, 29891 /* vAA */, 2911 /* fn */, 0}, HeaderDesc::STDARG_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29903 /* __builtin_va_end */, 29920 /* vA */, 2901 /* Fn */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29923 /* va_end */, 29920 /* vA */, 2911 /* fn */, 0}, HeaderDesc::STDARG_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29930 /* va_start */, 4482 /* vA. */, 2911 /* fn */, 0}, HeaderDesc::STDARG_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29939 /* __builtin_verbose_trap */, 29962 /* vcC*cC* */, 10944 /* nr */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{29970 /* __builtin_vfprintf */, 29543 /* iP*RcC*Ra */, 29989 /* FnP:1:N:0:<0,1> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30005 /* vfprintf */, 29543 /* iP*RcC*Ra */, 30014 /* fnP:1:N:0:<0,1> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30030 /* __builtin_vsprintf */, 30049 /* ic*RcC*Ra */, 29989 /* FnP:1:N:0:<0,1> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30059 /* vsprintf */, 30049 /* ic*RcC*Ra */, 30014 /* fnP:1:N:0:<0,1> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30068 /* __builtin_vsnprintf */, 30088 /* ic*RzcC*Ra */, 30099 /* FnP:2:N:0:<2> */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30113 /* vsnprintf */, 30088 /* ic*RzcC*Ra */, 30123 /* fnP:2:N:0:<2> */, 0}, HeaderDesc::STDIO_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30137 /* __builtin_wmemchr */, 30155 /* w*wC*wz */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30163 /* wmemchr */, 30155 /* w*wC*wz */, 19502 /* fnE */, 0}, HeaderDesc::WCHAR_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30171 /* __builtin_wmemcmp */, 30189 /* iwC*wC*z */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30198 /* wmemcmp */, 30189 /* iwC*wC*z */, 19502 /* fnE */, 0}, HeaderDesc::WCHAR_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30206 /* __builtin_wmemcpy */, 30224 /* w*w*wC*z */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30233 /* wmemcpy */, 30224 /* w*w*wC*z */, 19502 /* fnE */, 0}, HeaderDesc::WCHAR_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30241 /* __builtin_wmemmove */, 30224 /* w*w*wC*z */, 4455 /* FnE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30260 /* wmemmove */, 30224 /* w*w*wC*z */, 19502 /* fnE */, 0}, HeaderDesc::WCHAR_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30269 /* __warn_memset_zero_len */, 2264 /* v */, 30292 /* nU */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30295 /* __builtin_wcschr */, 30312 /* w*wC*w */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30319 /* wcschr */, 30312 /* w*wC*w */, 30326 /* fnUE */, 0}, HeaderDesc::WCHAR_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30331 /* __builtin_wcscmp */, 30348 /* iwC*wC* */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30356 /* wcscmp */, 30348 /* iwC*wC* */, 30326 /* fnUE */, 0}, HeaderDesc::WCHAR_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30363 /* __builtin_wcslen */, 30380 /* zwC* */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30385 /* wcslen */, 30380 /* zwC* */, 30326 /* fnUE */, 0}, HeaderDesc::WCHAR_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30392 /* __builtin_wcsncmp */, 30189 /* iwC*wC*z */, 20062 /* FnUE */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30410 /* wcsncmp */, 30189 /* iwC*wC*z */, 30326 /* fnUE */, 0}, HeaderDesc::WCHAR_H, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30418 /* work_group_commit_read_pipe */, 20 /* v. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{30446 /* work_group_commit_write_pipe */, 20 /* v. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{30475 /* work_group_reserve_read_pipe */, 3922 /* i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{30504 /* work_group_reserve_write_pipe */, 3922 /* i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{30534 /* write_pipe */, 3922 /* i. */, 6856 /* tn */, 0}, HeaderDesc::NO_HEADER, OCL_PIPE},
    Builtin::Info{Builtin::Info::StrOffsets{30545 /* __xray_customevent */, 30564 /* vcC*z */, 0 /*  */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30570 /* __xray_typedevent */, 30588 /* vzcC*z */, 0 /*  */, 0}, HeaderDesc::NO_HEADER, ALL_LANGUAGES},
    Builtin::Info{Builtin::Info::StrOffsets{30595 /* __addressof */, 2794 /* v*v& */, 30607 /* zfncTE */, 0}, HeaderDesc::NO_HEADER, CXX_LANG},

#endif // GET_BUILTIN_INFOS

#ifdef GET_BUILTIN_PREFIXED_INFOS

#endif // GET_BUILTIN_PREFIXED_INFOS

#ifdef ATOMIC_BUILTIN
ATOMIC_BUILTIN(__atomic_add_fetch, "v.", "t")
ATOMIC_BUILTIN(__atomic_and_fetch, "v.", "t")
ATOMIC_BUILTIN(__atomic_clear, "v.", "nt")
ATOMIC_BUILTIN(__atomic_compare_exchange, "v.", "t")
ATOMIC_BUILTIN(__atomic_compare_exchange_n, "v.", "t")
ATOMIC_BUILTIN(__atomic_exchange, "v.", "t")
ATOMIC_BUILTIN(__atomic_exchange_n, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_add, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_and, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_max, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_min, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_nand, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_or, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_sub, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_udec, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_uinc, "v.", "t")
ATOMIC_BUILTIN(__atomic_fetch_xor, "v.", "t")
ATOMIC_BUILTIN(__atomic_load, "v.", "t")
ATOMIC_BUILTIN(__atomic_load_n, "v.", "t")
ATOMIC_BUILTIN(__atomic_max_fetch, "v.", "t")
ATOMIC_BUILTIN(__atomic_min_fetch, "v.", "t")
ATOMIC_BUILTIN(__atomic_nand_fetch, "v.", "t")
ATOMIC_BUILTIN(__atomic_or_fetch, "v.", "t")
ATOMIC_BUILTIN(__atomic_store, "v.", "t")
ATOMIC_BUILTIN(__atomic_store_n, "v.", "t")
ATOMIC_BUILTIN(__atomic_sub_fetch, "v.", "t")
ATOMIC_BUILTIN(__atomic_test_and_set, "b.", "nt")
ATOMIC_BUILTIN(__atomic_xor_fetch, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_compare_exchange_strong, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_compare_exchange_weak, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_exchange, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_fetch_add, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_fetch_and, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_fetch_max, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_fetch_min, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_fetch_nand, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_fetch_or, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_fetch_sub, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_fetch_xor, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_init, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_load, "v.", "t")
ATOMIC_BUILTIN(__c11_atomic_store, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_compare_exchange_strong, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_compare_exchange_weak, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_exchange, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_fetch_add, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_fetch_and, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_fetch_max, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_fetch_min, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_fetch_or, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_fetch_sub, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_fetch_xor, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_load, "v.", "t")
ATOMIC_BUILTIN(__hip_atomic_store, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_compare_exchange_strong, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_compare_exchange_weak, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_exchange, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_fetch_add, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_fetch_and, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_fetch_max, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_fetch_min, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_fetch_or, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_fetch_sub, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_fetch_xor, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_init, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_load, "v.", "t")
ATOMIC_BUILTIN(__opencl_atomic_store, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_add_fetch, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_and_fetch, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_compare_exchange, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_compare_exchange_n, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_exchange, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_exchange_n, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_add, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_and, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_max, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_min, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_nand, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_or, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_sub, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_xor, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_load, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_load_n, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_max_fetch, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_min_fetch, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_nand_fetch, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_or_fetch, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_store, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_store_n, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_sub_fetch, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_udec, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_fetch_uinc, "v.", "t")
ATOMIC_BUILTIN(__scoped_atomic_xor_fetch, "v.", "t")

#endif // ATOMIC_BUILTIN
#undef ATOMIC_BUILTIN
